systemheader:(RI    )#                                                  RI     1
####################################################################### RI     2
##                                                                    # RI     3
##                          *** RIGBY ***                               RI     4
##                                                                      RI     5
##                  WRITTEN BY                J.N. VARGHESE  1972 FEB.  RI     6
##                  EXTENSIVELY MODIFIED  BY  G.A. WILLIAMS  1979 MAY.  RI     7
## SEMI XTAL-ISED & REVIVED FROM THE DEAD BY  D.J. DU BOULAY 1993 OCT.  RI     8
##                                                                      RI     9
##   PREPROCESS WITH THE S.R. HALL RATMAC PREPROCESSOR AND LINK WITH    RI    10
##   APPROPRIATE  XTAL3.2 XMACROS AND XTAL3.2 SYSTEM ROUTINES (AA.R).   RI    11
##                                                                      RI    12
##                                                                      RI    13
##   PROGRAM USES TSL METHOD OF SCHOMAKER AND TRUEBLOOD, ACTA CRYST.,   RI    14
##   B24, 63 (1968).                                                    RI    15
##   ANOTHER USEFUL REFERENCE IS INT. TABLES, VOL.4, PP.320-336.        RI    16
##                                                                      RI    17
##   WORKS OK WITH UNIT WEIGHTS. IF SOME OTHER WEIGHTING SCHEME IS      RI    18
##   DESIRED, MODIFICATIONS MAY BE NECESSARY (MAY 1979 - SEEMS TO APPLY RI    19
##   WEIGHTS IN WRONG ORDER TO THE VARIOUS U(IJ). ALSO APPLIES WEIGHTS  RI    20
##   TO WRONG ATOMS UNLESS ATOM ORDER IN SEGMENT IS SAME AS ORDER OF    RI    21
##   THE WEIGHTS).                                                      RI    22
##                                                                      RI    23
##   H-ATOMS CAN BE INSERTED INTO THE MODEL, WITH 3 EXTRA COMPONENTS    RI    24
##   (T(11),L(22),L(33)) ADDED TO THE TSL MATRIX FOR EACH C-H BOND.     RI    25
##   (IE. ALLOWS FOR C-H BOND VIBRATION, WITH X DIRECTION ALONG THE     RI    26
##   C-H BOND, AND LIBRATION IN PLANE PERPENDICULAR TO THE BOND).       RI    27
##   H-ATOMS ARE INSERTED BY INPUTTING THEIR X,Y,Z COORDINATES.         RI    28
##   U(IJ) WILL BE COMPUTED FOR EACH H-ATOM, AND PRESUMABLY USE OF THE  RI    29
##   NEW L(22)/L(33) COMPONENTS GIVES BETTER C-H DISTANCES. HOWEVER,    RI    30
##   THIS ONLY CORRECTS FOR THERMAL MOTION EFFECTS. BOND-SHORTENING DUE RI    31
##   TO DISPLACEMENT OF THE CHARGE CENTROID FROM THE NUCLEUS WILL NOT   RI    32
##   BE CORRECTED. NEW U(IJ) ARE THEN COMPUTED FOR ALL THE OTHER        RI    33
##   ATOMS (ONLY J.N.V. KNOWS THE SIGNIFICANCE OF THESE NEW U(IJ)).     RI    34
##   N.B. IT IS NECESSARY TO HAVE ACCURATE THERMAL PARAMETERS FROM      RI    35
##   HIGH-ACCURACY DIFFRACTION DATA FOR RIGBY TO WORK PROPERLY.         RI    36
##                                                                      RI    37
##   NVAR IS NO. OF VARIABLES TO BE REFINED. (NVAR LE 50 AT PRESENT)    RI    38
##   NATOM IS NO. OF ATOMS (INCLUDING NH H-ATOMS).                      RI    39
##   NSEG IS NO. OF SEGMENTS, NJOINT IS NO. OF JOINTS.                  RI    40
##   NNMAX (DEFINED IN OVERLAY 1) SHOULD BE SET EQUAL TO MAXIMUM NO. OF RI    41
##   ATOMS (NATOM) AND CONSTRAINTS (AT PRESENT, NNMAX = 42).            RI    42
##   (NSEG+NH) AND (NJOINT+NH) BOTH LE 10 AT PRESENT.                   RI    43
##   DIMENSIONS OF VARIABLES........                                    RI    44
##   BB11...BB33(NNMAX), AT(NNMAX), WG(6*NNMAX), V(6*NNMAX), X(3,NNMAX) RI    45
##   XO(3,NNMAX), AN(6*NNMAX,50), TSL1(21*NJOINT), TSL2(21*NJOINT),     RI    46
##   NX(NSEG), Q(NJOINT,3,3), O(3,NJOINT), T1(NSEG,(NJOINT+NH)),        RI    47
##   A(3*NNMAX*6*(NJOINT+NH)).                                          RI    48
##   OVERLAY 1 ..... XO(3,NNMAX), B11...B33(NNMAX).                     RI    49
##   OVERLAY 2 ..... AAT(NNMAX), TT(3*NNMAX,6*(NJOINT+NH)),             RI    50
##   B11..B33(NNMAX)  OVERLAY 4 ..... TT(3*NNMAX,6*(NJOINT+NH))         RI    51
##                                                                      RI    52
##   INPUT CARDS........                                                RI    53
##   CARD 1. (16A5) ALPHANUMERIC TITLE.                                 RI    54
##                                                                      RI    55
##   CARD 2. (10X,6F10.5) CELL PARAMETERS - A, B, C (ANGSTROM), ALPHA,  RI    56
##   BETA, GAMMA (DEGREES).                                             RI    57
##                                                                      RI    58
##   CARD 3. (10X,5I3) RIGID BODY DIMENSIONS - NATOM, NSEG,NJOINT,NVAR. RI    59
##                                                                      RI    60
##   CARDS 4,5. (10X,7F10.5) CONTROL OPTIONS - CONTRL(1) ... CONTRL(14) RI    61
##   CONTRL(1) - CONVERSION FACTOR TO MAKE INPUT TEMP. FACTORS = U(IJ). RI    62
##   CONTRL(2) - ZERO IF U(IJ) TO HAVE UNIT WEIGHTS. OTHERWISE, PROGRAM RI    63
##   WILL EXPECT WEIGHTS INPUT VIA CARDS 7.                             RI    64
##   CONTRL(3) - PRINT-OUT SUPPRESSOR. (TO PREVENT A LARGE USELESS O/P) RI    65
##   CONTRL(4) - EIGENVALUE FILTERING THRESHOLD (EG. 0.00001).          RI    66
##   CONTRL(5) - CONVERGENCE THRESHOLD FOR THE JACOBI EIGENFUNCTION     RI    67
##   (EG. -8.0)                                                         RI    68
##   CONTRL(6) - NO. OF FILTERING CYCLES IN EXCESS OF ONE.              RI    69
##   CONTRL(7) - NOT USED.                                              RI    70
##   CONTRL(8) - LOWER LIMIT (GAMMA 1) OF CHI-SQUARED DISTRIBUTION.     RI    71
##   CONTRL(9) - UPPER LIMIT (GAMMA 2) OF CHI-SQUARED DISTRIBUTION.     RI    72
##   CONTRL(10) - GAMMA VALUE OF STUDENTS T-DISTRIBUTION.               RI    73
##   CONTRL,11) - PROBABILITY LEVEL OF CONFIDENCE INTERVAL.             RI    74
##                                                                      RI    75
##   CARD 6. (20X,6F10.8) ORIGIN OF MOLECULAR AXIAL SYSTEM. X, Y, AND   RI    76
##   Z (FRACTIONAL COORDINATES)                                         RI    77
##                                                                      RI    78
##   CARDS 7. A SERIES OF 2 (OR 3 IF CONTRL(2) NE 0) CARDS PER ATOM.    RI    79
##   CARD 7A.(7X,A5,1X,3F8.5) ATOM NAME, X,Y,Z (FRACTIONAL COORDS.)     RI    80
##   CARD 7B. (13X,6F8.6)  U(11), U(22), U(33), U(12), U(13), U(23).    RI    81
##   CARD 7C. (20X,6F10.5) WEIGHTS FOR U(IJ) IN ABOVE ORDER.            RI    82
##                                                                      RI    83
##   CARD 8. (10X,3I4) ATOM TRIPLET TO DETERMINE MOLECULAR AXES.        RI    84
##   ATOM A NUMBER, ATOM B NUMBER, ATOM C NUMBER.                       RI    85
##   X AXIS DEFINED IN DIRECTION AC, THE ATOMS ABC DEFINE THE XY PLANE, RI    86
##   AND AB X BC DEFINES THE Z AXIS.                                    RI    87
##                                                                      RI    88
##   CARD 9. (10X,4I5) ATOM TRIPLET (AS FOR CARD 8) TO DETERMINE THE    RI    89
##   JOINT AXES (ONE CARD PER JOINT). THE 4TH VARIABLE ON CARD 9 IS M,  RI    90
##   THE ATOM NUMBER USED AS THE JOINT ORIGIN (THIS IS USUALLY AN ATOM  RI    91
##   WHICH LIES IN THE BOND THROUGH WHICH THE JOINT IS ACTING).         RI    92
##   IF M=0, A FURTHER CARD IS REQUIRED (10X,3F10.5) WITH THE           RI    93
##   FRACTIONAL COORDINATES OF THE JOINT ORIGIN.                        RI    94
##                                                                      RI    95
##   CARD 10. (10X,10I5) NO. OF ATOMS IN EACH SEGMENT.                  RI    96
##   (NSEG VALUES REQD)                                                 RI    97
##                                                                      RI    98
##   CARD 11. (10X,12A5) ATOMS IN EACH SEGMENT.                         RI    99
##   (NSEG OF THESE CARDS ARE                                           RI   100
##   REQUIRED, IE. ONE CARD PER SEGMENT).                               RI   101
##   ATOM 1 OF SEGMENT (ALPHANUMERIC TITLE) ... ATOM 12 OF SEGMENT.     RI   102
##   IF THERE ARE MORE THAN 12 ATOMS IN THE SEGMENT, CONTINUE ON THE    RI   103
##   NEXT CARD USING THE SAME FORMAT. THE ATOM NAME USED MUST BE        RI   104
##   IDENTICAL WITH THAT USED ON CARD 7.                                RI   105
##                                                                      RI   106
##   CARD 12. (10X,21I3) T1 MATRIX ELEMENTS. (ONE CARD PER SEGMENT)     RI   107
##   JOINT 1 REPRESENTATION OF DESIGN MATRIX, JOINT 2 REPRESENTATION,   RI   108
##   ETC. SET TO ZERO IF BLOCK A(MN) OF DESIGN MATRIX IS ZERO.          RI   109
##   SET TO NO. OF ATOMS IN SEGMENT IF BLOCK A(MN) IS NON-ZERO.         RI   110
##                                                                      RI   111
##   CARD 13. (10X,21I3) TSL MATRIX STRING. (ONE CARD PER JOINT)        RI   112
##   VARIABLE INDEX (NUMERIC) FOR T(11) OF JOINT, VARIABLE INDEX FOR    RI   113
##   T(12), ETC. LAST ENTRY IN COLUMNS 71-73 IS VARIABLE INDEX FOR      RI   114
##   L(33) OF JOINT.                                                    RI   115
##   SET VARIABLE INDEX TO ZERO TO SUPPRESS REFINEMENT.                 RI   116
##   THE VARIABLE INDICES MUST BE INTEGERS AND BE MONOTONIC ASCENDING   RI   117
##   STARTING AT UNITY. IE. NO 2 VARIABLES CAN HAVE THE SAME INDEX      RI   118
##   WHETHER THEY ARE IN THE SAME OR DIFFERENT JOINTS.                  RI   119
##   THE ORDER OF THE TSL STRING IS ... T(11),T(12),T(13),T(22),T(23),  RI   120
##   T(33),S(11),S(12),S(13),S(21),S(22),S(23),S(31),S(32),S(33),L(11), RI   121
##   L(12),L(13),L(22),L(23),L(33).                                     RI   122
##   IN THE ABOVE, T(IJ) IS THE TRANSLATION TENSOR (SYMMETRIC), S(IJ)   RI   123
##   IS THE SCREW COUPLING TENSOR (NON-SYMMETRIC), AND L(IJ) IS THE     RI   124
##   LIBRATION TENSOR (SYMMETRIC).                                      RI   125
##                                                                      RI   126
##   CARDS 14. CONSTRAINT OPTION (3 CARDS PER CONSTRAINT).              RI   127
##   CARD 14A. (10X,I3,F10.5) NO. OF CONSTRAINT VARIABLES (NCV), AND    RI   128
##   COEFFICIENT OF CONSTRAINT (COEF).                                  RI   129
##   CARD 14B. (2X,25I3) 1ST VARIABLE INDEX ... 25TH VARIABLE INDEX.    RI   130
##   IF MORE THAN 25 VARIABLES IN CONSTRAINT, CONTINUE ON A 2ND CARD.   RI   131
##   CARD 14C. (2X,7F10.5) COEFFICIENT OF 1ST VARIABLE IN CONSTRAINT    RI   132
##   EQUATION ... COEFFICIENT OF 7TH VARIABLE.                          RI   133
##   IF MORE THAN 7 VARIABLES IN THE CONSTRAINT, CONTINUE ON A 2ND CARD RI   134
##   CARDS 14 MUST END WITH A BLANK CARD.                               RI   135
##                                                                      RI   136
##   CARDS 15. (10X,3I5) Y,M,N. INTEGER NO.S (POSITIONS IN ATOM-ARRAY)  RI   137
##   OF PARTICULAR ATOMS TO DEFINE BOND (N=0) OR ANGLE TO BE CALCULATED RI   138
##   END EACH SET OF THESE CARDS WITH A BLANK CARD. NEED AS MANY SETS   RI   139
##   OF THESE CARDS AS THERE ARE SEGMENTS.                              RI   140
##                                                                      RI   141
##   CARD 16. (10X,2I5) NH AND NHV (NO. OF H-ATOMS AND TSL MATRIX       RI   142
##   PARAMETERS ADDED TO THE SYSTEM). SEE PROGRAM SEG3 FOR FURTHER      RI   143
##   INPUT CARDS.  IF NH=0, ONE HAS THE OPTION OF STARTING AFRESH WITH  RI   144
##   A WHOLE NEW SET OF DATA (STARTING WITH CARD 1 - TITLE) OR ONE CAN  RI   145
##   EXIT FROM RIGBY BY PUNCHING FINIS IN THE FIRST 5 COLUMNS OF A      RI   146
##   FINAL DATA CARD. HAPPY DAYS.                                       RI   147
##                                                                      RI   148
##                                                                    # RI   149
####################################################################### RI   150
#                                                                       RI   151
include:(XMACRO)#                                                       RI   152
#                                                                       RI   153
macro:(dabs:,ifelse:(mxbtwd:,32,DABS($1),ABS($1)))#                     RI   154
macro:(db:,ifelse:(mxbtwd:,32,DBLE($1),$1))#                            RI   155
macro:(dp:,ifelse:(mxbtwd:,32,DOUBLE PRECISION ,REAL ))#                RI   156
macro:(dsqrt:,ifelse:(mxbtwd:,32,DSQRT($1),SQRT($1)))#                  RI   157
macro:(dcos:,ifelse:(mxbtwd:,32,DCOS($1),COS($1)))#                     RI   158
macro:(dsin:,ifelse:(mxbtwd:,32,DSIN($1),SIN($1)))#                     RI   159
macro:(dacos:,ifelse:(mxbtwd:,32,DACOS($1),ACOS($1)))#                  RI   160
macro:(sg:,ifelse:(mxbtwd:,32,SNGL($1),$1))#                            RI   161
#                                                                       RI   162
extrude:(RICOMN)#                                                       RI   163
#                                                                       RI   164
INTEGER FIN#                                                            RI   165
INTEGER CDR#                                                            RI   166
INTEGER LPT#                                                            RI   167
INTEGER LPT2#                                                           RI   168
INTEGER NSEG#                                                           RI   169
INTEGER NJOINT#                                                         RI   170
INTEGER NATOM#                                                          RI   171
INTEGER NVAR#                                                           RI   172
INTEGER ID#                                                             RI   173
INTEGER IKK#                                                            RI   174
INTEGER NNMAX#                                                          RI   175
INTEGER NN2#                                                            RI   176
INTEGER TSL1(210)#                                                      RI   177
INTEGER TSL2(210)#                                                      RI   178
INTEGER T1(10,10)#                                                      RI   179
INTEGER AT(42)#                        Atom labels as integers          RI   180
INTEGER NX(10)#                                                         RI   181
#                                                                       RI   182
dp: CONTRL(14)#                                                         RI   183
dp: X1,Y1,Z1#                                                           RI   184
dp:  TO(3,3)#                          Rollet matrix                    RI   185
dp:  AR(3,3)#                          R                                RI   186
dp:  V(252)#                           R                                RI   187
dp: WG(252)#                           Weights                          RI   188
dp: VAR(50)#                           Sorted eigenvalues               RI   189
dp: EIG(50)#                           Eigenvalues                      RI   190
dp: BB11(42),BB12(42),BB13(42),BB22(42),BB23(42),BB33(42)#              RI   191
dp:   X(3,42)#                         Fractional atom coords           RI   192
dp:  XO(3,42)#                         Fractional atom coords           RI   193
dp: SIG(3,42)#                         Sigmas on fractn atom coords     RI   194
dp:   O(3,10)#                                                          RI   195
dp: AN(252,50)#                                                         RI   196
dp:  A(50,50)#                                                          RI   197
dp: AA(50,50)#                                                          RI   198
dp:  Q(10,3,3)#                                                         RI   199
#                                                                       RI   200
COMMON/COMFRI/X1,Y1,Z1,TO,AR,VAR,CONTRL,WG,O,XO,$#                      RI   201
BB11,BB12,BB13,BB22,BB23,BB33,V,EIG,X,SIG,AN,A,AA,Q#                    RI   202
#                                                                       RI   203
COMMON/COMIRI/FIN,CDR,LPT,LPT2,NSEG,NJOINT,NATOM,NVAR,ID,IKK,$#         RI   204
NNMAX,NN2,TSL1,TSL2,T1,AT,NX#                                           RI   205
#                                                                       RI   206
endext:#                                                                RI   207
#                                                                       RI   208
systemheader:(RI00)#                   General routine                  RI00   1
# ################################################################## #  RI00   2
#                                                                    #  RI00   3
#                               XT00                                 #  RI00   4
#                                                                    #  RI00   5
# ################################################################## #  RI00   6
SUBROUTINE RI00#                                                        RI00   7
include:(AACOMN)#                                                       RI00   8
include:(RICOMN)#                                                       RI00   9
#                                                                       RI00  10
REPEAT#                                                                 RI00  11
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI00  12
CALL SEG0#                                                              RI00  13
CALL SEG1#                                                              RI00  14
CALL SEG2#                                                              RI00  15
CALL CORLIB#                                                            RI00  16
CALL SEG3#                                                              RI00  17
IF (FIN==yes:) BREAK#                                                   RI00  18
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI00  19
#                                                                       RI00  20
RETURN#                                                                 RI00  21
END#                                                                    RI00  22
#                                                                       RI00  23
systemheader:(RI10)#                                                    RI10   1
# ################################################################## #  RI10   2
#                                                                    #  RI10   3
#                               RI10                                 #  RI10   4
#                                                                    #  RI10   5
# ################################################################## #  RI10   6
SUBROUTINE SEG0#                                                        RI10   7
include:(AACOMN)#                                                       RI10   8
include:(RICOMN)#                                                       RI10   9
#                                                                       RI10  10
dp: B11(42),B12(42),B13(42),B22(42),B23(42),B33(42)# 
dp: C(3,3),F(3,3),G(3,3),P(3,3),X2(3,42)# 
dp: TOO(3,3),DIAG(3,3),E(3,3)#
INTEGER Q1,Q2,Q3#                                                       RI10  13
CHARACTER CRD1*80#                                                      RI10  14
dp: ALP,BET,GAM,CAS,SAS,CA,CB,CG,SA,SB,SG#                              RI10  15
dp: AD,BB,CC#                                                           RI10  16
dp: RAD,PISQ,const#   
#                                                                       RI10  18
#                                                                       RI10  19
CDR=ioinp:#                                                             RI10  20
LPT=ioprt:#                                                             RI10  21
LPT2=ioprt:#                                                            RI10  22
NNMAX=42#                                                               RI10  23
##                                                                      RI10  24
#***** READ CARD 1           TITLE#                                     RI10  25
#                                                                       RI10  26
FIN=no:#                                                                RI10  27
#                                                                       RI10  28
READ(CDR,1000) CRD1#                                                    RI10  29
WRITE(LPT,*)CRD1#                                                       RI10  30
IF(CRD1 =='FINIS')#                                                     RI10  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI10  32
FIN=yes:#                                                               RI10  33
RETURN#                                                                 RI10  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI10  35
1000 FORMAT(10X,10A80)#                                                 RI10  36
#1001 FORMAT(/,/,1H ,16A5)#                                           R RI10  37
1029 FORMAT(/,//)#                     Ri                               RI10  38
#      READ CARD 2           UNIT CELL#                                 RI10  39
READ(CDR,1003)AD,BB,CC,ALP,BET,GAM#                                     RI10  40
1003 FORMAT(10X,6F10.5)#                                                RI10  41
WRITE(LPT,1020) AD,BB,CC,ALP,BET,GAM#                                   RI10  42
1020 FORMAT(///,' CELL PARAMETERS',/,5X,'A',11X,'B',11X,'C',9X,$#       RI10  43
'ALPH',8X,'BETA',8X,'GAMMA',/,6F11.5,//)#                               RI10  44
#                                                                       RI10  45
#      READ CARD 3           DIMENSIONS OF RIGID BODY#                  RI10  46
#                                                                       RI10  47
READ(CDR,1004)NATOM,NSEG,NJOINT,NVAR#                                   RI10  48
1004 FORMAT(10X,4I3)#                                                   RI10  49
#                                                                       RI10  50
#      READ CARDS 4,5        CONTROL OPTIONS#                           RI10  51
#                                                                       RI10  52
READ(CDR,1026)(CONTRL(I),I=1,14)#                                       RI10  53
1026 FORMAT(10X,7F10.5)#                                                RI10  54
WRITE(LPT,1021) NATOM,NSEG,NJOINT,NVAR#                                 RI10  55
1021 FORMAT(' NOS.OF ATOMS =',I3,/,' NOS. OF SEGMENTS =',$#             RI10  56
I3,/,' NOS. OF JOINTS =',I3,/,' NO. OF VARIABLES TO BE REFINED =',$#    RI10  57
I3,///)#                                                                RI10  58
WRITE(LPT,1027)(CONTRL(I),I=1,14)#                                      RI10  59
1027 FORMAT(1X,' CONTROL OPTIONS',/,5X,2(/7F10.5),//)#                  RI10  60
##                                                                      RI10  61
#      TRANSFORM TO ORTHOGONAL AXES VIA TO MATRIX#                      RI10  62
##                                                                      RI10  63
RAD=3.14159/180.#                                                       RI10  64
ALP=ALP*RAD#                                                            RI10  65
BET=BET*RAD#                                                            RI10  66
GAM=GAM*RAD#                                                            RI10  67
CA=dcos:(ALP)#                                                          RI10  68
CB=dcos:(BET)#                                                          RI10  69
CG=dcos:(GAM)#                                                          RI10  70
SA=dsin:(ALP)#                                                          RI10  71
SB=dsin:(BET)#                                                          RI10  71
SG=dsin:(GAM)#                                                          RI10  72
CAS=(CB*CG-CA)/(SB*SG)#                                                 RI10  73
SAS=(dsin:(dacos:(CAS)))#                                               RI10  74
TO(1,1)=AD#                            Fractional to orthogonal         RI10  75
TO(1,2)=BB*CG#                         Rollett matrix.                  RI10  76
TO(1,3)=CC*CB#                                                          RI10  77
TO(2,2)=BB*SAS*SG#                                                      RI10  78
TO(3,2)=-BB*CAS*SG#                                                     RI10  79
TO(3,3)=CC*SB#                                                          RI10  80
TO(2,1)=0. ; TO(2,3)=0. ; TO(3,1)=0.#  
#
##  To convert Uij tensor from crystal coords in Angstrom 
##  to orthogonal coords in Angstroms
## From  C. Scheringer  Acta Cryst (1966) 20 P 316-319     and
## From  Cruikshank, Pilling bujosa Lovell and Truter Compting 
## Methods and the phase problem in X-ray crystal analysis P 32
##                    
TOO(1,1)=1.0#                   
TOO(1,2)=CG#                   
TOO(1,3)=CB#                                                          RI10  77
TOO(2,2)=SAS*SG#                                                      RI10  78
TOO(3,2)=-CAS*SG#                                                     RI10  79
TOO(3,3)=SB#                                                          RI10  80
TOO(2,1)=0. ; TOO(2,3)=0. ; TOO(3,1)=0.#  
const=dsqrt:(1.0+2.0*ca*cb*cg-ca*ca-cb*cb-cg*cg)#
DIAG(1,1)=sa/const#
DIAG(2,2)=sb/const#
DIAG(3,3)=sg/const#
DIAG(2,1)=0. ; DIAG(2,3)=0. ; DIAG(3,1)=0.#  
DIAG(1,2)=0. ; DIAG(3,2)=0. ; DIAG(1,3)=0.#  
CALL MATMUL(E,TOO,DIAG,0)#              TOO times DIAG
#
WRITE(LPT,1013)#                                                        RI10  84
WRITE(LPT,1006)((TO(I,J),J=1,3),I=1,3)#                                 RI10  85
1013 FORMAT(' MATRIX TRANSFORMING FROM CELL TO ',$#                     RI10  86
'ORTHOGONAL AXES  (ROLLETT MATRIX)')#                                   RI10  87
WRITE(LPT,1029)#                                                        RI10  88
#      READ CARD 6           ORIGIN OF MOLECULAR AXIAL SYSTEM#          RI10  89
READ(CDR,1008)X1,Y1,Z1#                                                 RI10  90
1008 FORMAT(20X,6F10.8)#                                                RI10  91
1007 FORMAT(' MOLECULAR AXIS ORIGIN (FRACT.COORDS.)',3F10.5,///)#       RI10  92
WRITE(LPT,1007)X1,Y1,Z1#                                                RI10  93
#                                                                       RI10  94
#  BETA(IJ) = 2*PISQ*U(IJ)*ASTAR(I)*ASTAR(J)#                           RI10  95
#  B(IJ) = 8*PISQ*U(IJ)#                                                RI10  96
#  B(ISOTROPIC) = 8*PISQ*U(ISOTROPIC)#                                  RI10  97
#  PROGRAM REQUIRES C(IJ) TO BE U(IJ). IF U(IJ) ARE INPUT,#             RI10  98
#  CONTRL(1)=1. IF B(IJ) ARE INPUT, CONTRL(1)=0.01267 (IE. 1/8*PISQ)    RI10  99
#                                                                       RI10 100
DO I=1,6*NNMAX ; WG(I)=1.0#                                             RI10 101
#                                                                       RI10 102
#      READ CARDS 7          ATOM PARAMETERS#                           RI10 103
#                                                                       RI10 104
DO I=1, NATOM#                                                          RI10 105
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI10 106
READ(CDR,1009)AT(I),(X(J,I),J=1,3),(SIG(J,I),J=1,3)#                    RI10 107
1009 FORMAT(4X,A5,3F9.5,9X,3F7.5)#     Ri10                             RI10 108
READ(CDR,4000)B11(I),B22(I),B33(I),B12(I),B13(I),B23(I)#                RI10 109
4000 FORMAT(9X,6F9.5)#                 R                                RI10 110
IF(CONTRL(2)<0.001)#                                                    RI10 111
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI10 112
#READ(CDR,4000)SUIJ,SUIJ,SUIJ,SUIJ,SUIJ,SUIJ#                           RI10 113
NEXT#                                                                   RI10 114
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI10 115
I11=6*(I-1)+1#                                                          RI10 116
I12=I11+1#                                                              RI10 117
I13=I11+2#                                                              RI10 118
I14=I11+3#                                                              RI10 119
I15=I11+4#                                                              RI10 120
I16=I11+5#                                                              RI10 121
READ(CDR,43)WG(I11),WG(I13),WG(I16),WG(I12),WG(I14),WG(I15)#            RI10 122
43   FORMAT(20X,6F10.5)#                                                RI10 123
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI10 124
#                                                                       RI10 125
#      READ CARD 8           MOLECULAR AXES#                            RI10 126
6    READ(CDR,2005)Q1,Q2,Q3#                                            RI10 127
WRITE(LPT,1022)#                                                        RI10 128
1022 FORMAT('  ATOMS',8X,'X',9X,'Y',9X,'Z',18X,'B11',7X,$#              RI10 129
'B12',7X,'B22',7X,'B13',7X,'B23',7X,'B33')#                             RI10 130
DO  I=1,NATOM#                                                          RI10 131
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI10 132
WRITE(LPT,1023)AT(I),(X(J,I),J=1,3),B11(I),B12(I),B22(I),B13(I),#       RI10 133
B23(I),B33(I)#                                                          RI10 134
I11=6*(I-1)+1#                                                          RI10 135
I12=I11+5#                                                              RI10 136
WRITE(LPT,1034)(WG(J),J=I11,I12)#                                       RI10 137
1034 FORMAT(42X,'WEIGHTS ',6F10.5)#                                     RI10 138
1023 FORMAT(2X,A5,3X,3F10.5,10X,6F10.5)#                                RI10 139
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI10 140
#                                                                       RI10 141
#                                                                       RI10 142
WRITE(LPT,1029)#                                                        RI10 143
##                                                                      RI10 144
#      CREATE AND INVERT ROTATION MATRIX FOR CELL CARTESIAN COORDS.#    RI10 145
##                                                                      RI10 146
2005 FORMAT(10X,3I4)#                                                   RI10 147
WRITE(LPT,3333)AT(Q1),AT(Q3),AT(Q1),AT(Q2),AT(Q3)#                      RI10 148
3333 FORMAT(/,' ROTATION MATRIX TO AN ORTHOGONAL AXIS SET ',$# Ri       RI10 149
'GIVEN BY ATOMS  ',2A5,' DEFINING THE X-AXIS, AND  ',3A5,$#             RI10 150
' DEFINING THE X,Y PLANE ',//)#                                         RI10 151
DO  KN=1,NATOM#                                                         RI10 152
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI10 153
X(1,KN)=X(1,KN)-X1#                                                     RI10 154
X(2,KN)=X(2,KN)-Y1#                                                     RI10 155
X(3,KN)=X(3,KN)-Z1#                                                     RI10 156
DO I=1,3#                                                               RI10 157
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI10 158
X2(I,KN)=0.#                                                            RI10 159
DO J=1,3 ; X2(I,KN)=X2(I,KN)+TO(I,J)*X(J,KN)#                           RI10 160
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI10 161
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI10 162
350   CONTINUE#                                                         RI10 163
DO  J=1,3#                                                              RI10 164
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI10 165
P(1,J)=X2(J,Q1)#                                                        RI10 166
P(2,J)=X2(J,Q2)#                                                        RI10 167
P(3,J)=X2(J,Q3)#                                                        RI10 168
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI10 169
CALL MATROT(P,AR)#                                                      RI10 170
309  WRITE(LPT,1006)((AR(I,J),J=1,3),I=1,3)#                            RI10 171
1006 FORMAT(3(1X,3F10.5,/),//)#                                         RI10 172
##                                                                      RI10 173
#      COORDINATES RELATIVE TO MOLECULAR AXES#                          RI10 174
##                                                                      RI10 175
WRITE(LPT,1011)#                                                        RI10 176
1011 FORMAT('  COORDINATES RELATIVE TO MOLECULAR AXIS',$#               RI10 177
/'  ATOM',8X,'X',9X,'Y',9X,'Z',8X,'U11',7X,'U12',7X,'U22',$#            RI10 178
7X,'U13',7X,'U23',7X,'U33')#                                            RI10 179
IF(CONTRL(2)>0.001)CALL WGLQ(WG,TO,AR,NATOM)#                           RI10 180
#
CALL MATMUL(F,AR,E,0)#                  AR times TOO times DIAG
#
DO KN=1,NATOM#                                                          RI10 181
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  RI10 182
DO I=1,3#                              
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  RI10 184
X(I,KN)=0.#                                                             RI10 185
DO  J=1,3; X(I,KN)=X(I,KN)+X2(J,KN)*AR(I,J)#                            RI10 186
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  RI10 187
C(1,1)=B11(KN)*CONTRL(1)#                                               RI10 188
C(2,2)=B22(KN)*CONTRL(1)#                                               RI10 189
C(3,3)=B33(KN)*CONTRL(1)#                                               RI10 190
C(1,2)=B12(KN)*CONTRL(1)#                                               RI10 191
C(1,3)=B13(KN)*CONTRL(1)#                                               RI10 192
C(2,3)=B23(KN)*CONTRL(1)#                                               RI10 193
C(2,1)=C(1,2)#                                                          RI10 194
C(3,1)=C(1,3)#                                                          RI10 195
C(3,2)=C(2,3)#                                                          RI10 196
#
CALL MATMUL(E,F,C,0)#                   AR mult TOO mult DIAG mult U
CALL MATMUL(G,F,E,1)#                   E times TRANSP(ar tOO DIAG)
#
BB11(KN)=G(1,1)#                                                        RI10 210
BB12(KN)=G(1,2)#                                                        RI10 211
BB13(KN)=G(1,3)#                                                        RI10 212
BB22(KN)=G(2,2)#                                                        RI10 213
BB23(KN)=G(2,3)#                                                        RI10 214
BB33(KN)=G(3,3)#                                                        RI10 215
WRITE(LPT,1012)AT(KN),(X(I,KN),I=1,3),BB11(KN),BB12(KN),BB22(KN),$#     RI10 216
BB13(KN),BB23(KN),BB33(KN)#                                             RI10 217
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  RI10 218
1012 FORMAT(1X,A5,3X,9F10.5)#                                           RI10 219
RETURN#                                                                 RI10 220
END#                                                                    RI10 221
systemheader:(RI13)#            
# ################################################################## #  RI13   2
#                                                                    #  RI13   3
#                               RI13                                 #  RI13   4
#                                                                    #  RI13   5
# ################################################################## #  RI13   6
SUBROUTINE WGLQ(W,X,Y,N)#                                               RI13   7
#                                                                       RI13   8
dp: W(252),X(3,3),Y(3,3),U(3,3),B(3,3)# Ri13                            RI13   9
INTEGER N#                                                              RI13  10
#                                                                       RI13  11
DO  K=1,N#                                                              RI13  12
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI13  13
I1=6*(K-1)+1#                                                           RI13  14
U(1,1)=W(I1)#                                                           RI13  15
U(1,2)=W(I1+1)#                                                         RI13  16
U(2,2)=W(I1+2)#                                                         RI13  17
U(1,3)=W(I1+3)#                                                         RI13  18
U(2,3)=W(I1+4)#                                                         RI13  19
U(3,3)=W(I1+5)#                                                         RI13  20
PISQ=19.7392088#                                                        RI13  21
DO  I=1,3#                                                              RI13  22
DO  J=I,3#                                                              RI13  23
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI13  24
IF(I==J)U(I,I)=U(I,I)/PISQ#                                             RI13  25
IF(I==J)NEXT#                                                           RI13  26
U(I,J)=U(I,J)/PISQ*0.5#                                                 RI13  27
U(J,I)=U(I,J)#                                                          RI13  28
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI13  29
CALL MATMUL(B,X,U,0)#                                                   RI13  30
CALL MATMUL(U,X,B,1)#                                                   RI13  31
CALL MATMUL(B,Y,U,0)#                                                   RI13  32
CALL MATMUL(U,Y,B,1)#                                                   RI13  33
DO  I=1,3#                                                              RI13  34
DO  J=1,3 ;  U(I,J)=1./(U(I,J)*U(I,J))#                                 RI13  35
W(I1)=U(1,1)#                                                           RI13  36
W(I1+1)=U(1,2)#                                                         RI13  37
W(I1+2)=U(2,2)#                                                         RI13  38
W(I1+3)=U(1,3)#                                                         RI13  39
W(I1+4)=U(2,3)#                                                         RI13  40
W(I1+5)=U(3,3)#                                                         RI13  41
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI13  42
#                                                                       RI13  43
RETURN#                                                                 RI13  44
END#                                                                    RI13  45
systemheader:(RI15)#                   Enter control lines & bdf        RI15   1
# ################################################################## #  RI15   2
#                                                                    #  RI15   3
#                               RI10                                 #  RI15   4
#                                                                    #  RI15   5
# ################################################################## #  RI15   6
SUBROUTINE MATMUL(A,B,C,N)#                                             RI15   7
#                                                                       RI15   8
dp: A(3,3),B(3,3),C(3,3)#              Ri15                             RI15   9
INTEGER N
#                                                                       RI15  10
DO  I=1,3#                                                              RI15  11
DO  J=1,3#                                                              RI15  12
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI15  13
A(I,J)=0.#                                                              RI15  14
DO  K=1,3#                                                              RI15  15
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI15  16
IF(N<=0) A(I,J)=A(I,J)+B(I,K)*C(K,J)#   B times C
ELSE     A(I,J)=A(I,J)+C(I,K)*B(J,K)#   C times Transpose B
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI15  19
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI15  20
#                                                                       RI15  21
RETURN#                                                                 RI15  22
END#                                                                    RI15  23
#                                                                       RI15  24
systemheader:(RI17)#                   Enter control lines & bdf        RI17   1
# ################################################################## #  RI17   2
#                                                                    #  RI17   3
#                               RI01                                 #  RI17   4
#                                                                    #  RI17   5
# ################################################################## #  RI17   6
SUBROUTINE MATROT(P,AR)#                                                RI17   7
##                                                                      RI17   8
#        CALCULATES THE ROTATION MATRIX, TRANSFORMING INTO A SYSTEM#    RI17   9
#        WHERE P3-P1 DEFINES THE X-AXIS, AND (P2-P1)X(P3-P2) DEFINES#   RI17  10
#        THE Z-AXIS#                                                    RI17  11
##                                                                      RI17  12
dp: P(3,3),AR(3,3),XP(3,3)#                                             RI17  13
#                                                                       RI17  14
#                                                                       RI17  15
DO J=1,3#                                                               RI17  16
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI17  17
XP(1,J)=P(3,J)-P(1,J)#                                                  RI17  18
XP(3,J)=P(2,J)-P(1,J)#                                                  RI17  19
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI17  20
XP1=  XP(3,2)*XP(1,3)-XP(3,3)*XP(1,2)#                                  RI17  21
XP2=-(XP(3,1)*XP(1,3)-XP(3,3)*XP(1,1))#                                 RI17  22
XP3=  XP(3,1)*XP(1,2)-XP(3,2)*XP(1,1)#                                  RI17  23
XP(3,1)=XP1#                                                            RI17  24
XP(3,2)=XP2#                                                            RI17  25
XP(3,3)=XP3#                                                            RI17  26
XP(2,1)=  XP(3,2)*XP(1,3)-XP(3,3)*XP(1,2)#                              RI17  27
XP(2,2)=-(XP(3,1)*XP(1,3)-XP(3,3)*XP(1,1))#                             RI17  28
XP(2,3)=  XP(3,1)*XP(1,2)-XP(3,2)*XP(1,1)#                              RI17  29
X2=dsqrt:(XP(1,1)*XP(1,1)+XP(1,2)*XP(1,2)+XP(1,3)*XP(1,3))#             RI17  30
Y2=dsqrt:(XP(2,1)*XP(2,1)+XP(2,2)*XP(2,2)+XP(2,3)*XP(2,3))#             RI17  31
Z2=dsqrt:(XP(3,1)*XP(3,1)+XP(3,2)*XP(3,2)+XP(3,3)*XP(3,3))#             RI17  32
DO I=1,3#                                                               RI17  33
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI17  34
AR(1,I)=XP(1,I)/X2#                                                     RI17  35
AR(2,I)=XP(2,I)/Y2#                                                     RI17  36
AR(3,I)=XP(3,I)/Z2#                                                     RI17  37
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI17  38
#                                                                       RI17  39
RETURN#                                                                 RI17  40
END#                                                                    RI17  41
#                                                                       RI17  42
systemheader:(RI20)#                   Enter control lines & bdf        RI20   1
# ################################################################## #  RI20   2
#                                                                    #  RI20   3
#                               RI20                                 #  RI20   4
#   SEE C.K. JOHNSON, PAPER 9 (AND APPENDIX) IN THERMAL NEUTRON         RI20   5
#   DIFFRACTION, ED. B.T.M. WILLIS, OUP, 1970. PHYSICS LIBRARY 548.83   RI20   6
#                                                                       RI20   7
#                                                                    #  RI20   8
# ################################################################## #  RI20   9
SUBROUTINE SEG1#                                                        RI20  10
include:(AACOMN)#                      R                                RI20  11
include:(RICOMN)#                      R                                RI20  12
#                                                                       RI20  13
dp: WT(50)#                            Ri20                             RI20  14
dp: RR(3,3),TT(126,60),R(3,3),P(3,3)#  Ri20                             RI20  15
dp: B11(42),B12(42),B13(42),B22(42),B23(42),B33(42)# Ri20               RI20  16
#                                                                       RI20  17
EQUIVALENCE (A(1,1),TT(1,1))#                                           RI20  18
INTEGER SN,SM,RS#                                                       RI20  19
INTEGER Q1,Q2,Q3,Q4,B(50),AAT(42)#                                      RI20  20
INTEGER L1,L2,L3#                                                       RI20  21
INTEGER M#                                                              RI20  22
#                                                                       RI20  23
#  CREATING THE DESIGN MATRIX TT(I,J)#                                  RI20  24
#  READ IN ROTATIONAL MATRIX FOR EACH JOINT - M IS ATOM NO. USED AS#    RI20  25
#  THE ORIGIN FOR AN ORTHOGONAL AXIS SET IN THE SEGMENT TO WHICH THE#   RI20  26
#  JOINT IS ATTACHED.    NX IS THE NO. OF ATOMS IN THE SEGMENT.#        RI20  27
#                                                                       RI20  28
DO  K=1,NJOINT#                                                         RI20  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI20  30
#      READ CARD 9           JOINT AXES#                                RI20  31
READ(CDR,1013)L1,L2,L3,M#                                               RI20  32
1013 FORMAT(10X,4I5)#                                                   RI20  33
DO  J=1,3#                                                              RI20  34
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI20  35
P(1,J)=X(J,L1)#                                                         RI20  36
P(2,J)=X(J,L2)#                                                         RI20  37
P(3,J)=X(J,L3)#                                                         RI20  38
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI20  39
CALL MATROT(P,R)#                                                       RI20  40
DO  I=1,3#                                                              RI20  41
DO  J=1,3 ;Q(K,I,J)=R(I,J)#                                             RI20  42
#                                                                       RI20  43
WRITE (LPT ,1014) K,((Q(K,I,J),J=1,3),I=1,3)#                           RI20  44
1014 FORMAT(/,' ROTATION MATRIX FOR JOINT ',I2,/,3(/,3F10.5 ))# Ri      RI20  45
IF(M==0)READ(CDR,1020)(O(I,K),I=1,3)#                                   RI20  46
IF(M==0) GO TO 35#                                                      RI20  47
1020 FORMAT(10X,3F10.5)#                                                RI20  48
DO  I=1,3; O(I,K)=X(I,M)#                                               RI20  49
35   WRITE (LPT,1016)(O(I,K),I=1,3)#                                    RI20  50
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI20  51
1016 FORMAT(/,' ORIGIN AT ',3F10.5,///)#                                RI20  52
1017 FORMAT(10X,10I5)#                                                  RI20  53
NN2=0#                                                                  RI20  54
#      READ CARD 10          NO. OF ATOMS IN SEGMENTS#                  RI20  55
READ(CDR,1017)(NX(I),I=1,NSEG)#                                         RI20  56
DO  K=1,NATOM#                                                          RI20  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI20  58
AAT(K)=0#                                                               RI20  59
B11(K)=0.0#                                                             RI20  60
B22(K)=0.0#                                                             RI20  61
B33(K)=0.0#                                                             RI20  62
B12(K)=0.0#                                                             RI20  63
B13(K)=0.0#                                                             RI20  64
B23(K)=0.0#                                                             RI20  65
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI20  66
DO K=1,NSEG#                           Loop over distinct segments      RI20  67
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI20  68
Q1=NX(K)#                              Number of atoms in segment k     RI20  69
#      READ CARDS 11         ATOMS IN SEGMENTS#                         RI20  70
#                                                                       RI20  71
READ(CDR,1018)(AAT(NN2+I),I=1,Q1)#     Read in atom labels              RI20  72
1018 FORMAT(10X,12A5)#                                                  RI20  73
WRITE(LPT,2)K#                         Write header for segment k       RI20  74
2 FORMAT(/,' ATOMS IN SEGMENT ',I2,/)#                                  RI20  75
#                                                                       RI20  76
WRITE(LPT,5)(AAT(NN2+I),I=1,Q1)#       Write out atom labels            RI20  77
5 FORMAT(/,1X,20A6,/)#                                                  RI20  78
NN2=NN2+NX(K)#                         Nn2= num of atoms in segments    RI20  79
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI20  80
#                                                                       RI20  81
#                                                                       RI20  82
DO  L=1,NATOM#                         Loop over all atoms              RI20  83
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI20  84
DO  K=1,NN2#                           Loop over atoms in segments      RI20  85
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI20  86
IF (AAT(K)!=AT(L)) NEXT#                                                RI20  87
B11(K)=BB11(L)#                                                         RI20  88
B12(K)=BB12(L)#                                                         RI20  89
B13(K)=BB13(L)#                                                         RI20  90
B22(K)=BB22(L)#                                                         RI20  91
B23(K)=BB23(L)#                                                         RI20  92
B33(K)=BB33(L)#                                                         RI20  93
DO  I=1,3 ; XO(I,K)=X(I,L)#                                             RI20  94
BREAK#                                                                  RI20  95
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI20  96
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI20  97
#                                                                       RI20  98
#   CONSTRUCT DESIGN MATRIX#                                            RI20  99
#   THE T1 MATRIX CONTAINS THE LOGIC TO DESCRIBE THE ENTIRE SEGMENTE #  RI20 100
#   BODY. EACH ELEMENT REPRESENTS A SEGMENT-JOINT BLOCK OF THE DESIG    RI20 101
#   MATRIX. A ZERO SIGNIFIES THE BLOCK IS NULL. A NON-ZERO NUMBER#      RI20 102
#   SIGNIFIES THE BLOCK CONTAINS THAT NO. OF ATOMS IN THE SEGMENT.#     RI20 103
#                                                                       RI20 104
#      READ CARDS 12         T1 MATRIX ELEMENTS#                        RI20 105
DO L=1,NSEG ; READ(CDR,555)(T1(L,M),M=1,NJOINT)#                        RI20 106
WRITE (LPT,2001)#                                                       RI20 107
2001 FORMAT(' BLOCK ELEMENTS OF TRANS. MATRIX')#                        RI20 108
DO L=1,NSEG ; WRITE(LPT,2002)(T1(L,M),M=1,NJOINT)#                      RI20 109
2002 FORMAT(5X,25I3,////)#                                              RI20 110
SN=0.#                                                                  RI20 111
Q2=3*NN2#                                                               RI20 112
Q3=6*NJOINT#                                                            RI20 113
DO L=1,Q2#                                                              RI20 114
DO M=1,Q3 ; TT(L,M)=0.#                                                 RI20 115
#                                                                       RI20 116
DO  L=1,NSEG#                                                           RI20 117
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI20 118
DO  M=1,NJOINT#                                                         RI20 119
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  RI20 120
IF(T1(L,M)==0) NEXT#                                                    RI20 121
SM=T1(L,M)#                                                             RI20 122
Q2=SN+1#                                                                RI20 123
Q3=SN+SM#                                                               RI20 124
#  THE FOLLOWING SECTION IMPLIES THAT IF A SEGMENT CONSISTS OF ONLY#    RI20 125
#  ONE ATOM, WITH THAT ATOM AT THE ORIGIN, THEN LIBRATION MATRIX = 0    RI20 126
DO  N=Q2,Q3#                                                            RI20 127
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  RI20 128
DO I=1,3#                                                               RI20 129
DO J=1,3 ; TT((N-1)*3+I,6*(M-1)+J)=Q(M,I,J)#                            RI20 130
#                                                                       RI20 131
DX=XO(1,N)-O(1,M)#                                                      RI20 132
DY=XO(2,N)-O(2,M)#                                                      RI20 133
DZ=XO(3,N)-O(3,M)#                                                      RI20 134
R(1,1)=0.#                                                              RI20 135
R(1,2)=-DZ#                                                             RI20 136
R(1,3)= DY#                                                             RI20 137
R(2,1)= DZ#                                                             RI20 138
R(2,2)=0.#                                                              RI20 139
R(2,3)=-DX#                                                             RI20 140
R(3,1)=-DY#                                                             RI20 141
R(3,2)= DX#                                                             RI20 142
R(3,3)= 0.#                                                             RI20 143
#                                                                       RI20 144
DO  I=1,3#                                                              RI20 145
DO  J=1,3#                                                              RI20 146
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 RI20 147
SS=0.#                                                                  RI20 148
DO IM=1,3#                                                              RI20 149
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 RI20 150
RR(I,J)=SS+R(I,IM)*Q(M,IM,J)#                                           RI20 151
SS=RR(I,J)#                                                             RI20 152
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 RI20 153
TT((N-1)*3+I,6*(M-1)+J+3)=RR(I,J)#                                      RI20 154
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 RI20 155
#                                                                       RI20 156
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  RI20 157
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  RI20 158
SN=SN+SM#                                                               RI20 159
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI20 160
2004 FORMAT(/)#                        Ri                               RI20 161
IF(CONTRL(3)==0.) GO TO 93#                                             RI20 162
WRITE(LPT,48)#                                                          RI20 163
48   FORMAT(/,'  ELEMENTS OF TRANSFORMATION  (DESIGN) MATRIX',/)#       RI20 164
Q2=3*NN2#                                                               RI20 165
Q3=6*NJOINT#                                                            RI20 166
DO I=1,Q2 ; WRITE(LPT,10)(TT(I,J),J=1,Q3)#                              RI20 167
10   FORMAT(2X,14F8.4,/,2X,14F8.4)#                                     RI20 168
WRITE(LPT,2004)#                                                        RI20 169
93   CONTINUE#                                                          RI20 170
#                                                                       RI20 171
#   THE TSL MATRIX IS READ IN FOR EACH JOINT#                           RI20 172
#   THERE ARE 21 ELEMENTS IN EACH JOINT.#                               RI20 173
#   IF AN ELEMENT IS ZERO IT IS IGNORED IN THE REFINEMENT.#             RI20 174
#   NO TWO ELEMENTS MAY HAVE THE SAME NUMBER.#                          RI20 175
#   MAXIMUM NO. OF ELEMENTS REFINEABLE IS 50.   THE REFINEABLE#         RI20 176
#   ELEMENTS MUST BE CONTINUOUS ASCENDING INTEGERS FROM 1 TO NVAR.#     RI20 177
#                                                                       RI20 178
IK=0#                                                                   RI20 179
IO=1#                                                                   RI20 180
DO  MA=1,NJOINT#                                                        RI20 181
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 RI20 182
#      READ CARDS 13         TSL MATRIX STRING#                         RI20 183
READ(CDR,555)(TSL1(IK+JJ),JJ=1,21)#                                     RI20 184
555  FORMAT(10X,21I3)#                                                  RI20 185
WRITE(LPT,3) MA,(TSL1(IK+JJ),JJ=1,21)#                                  RI20 186
3 FORMAT(/,' SEGMENT ',I2,' CONTRIBUTION TO THE TSL MATRIX',/,21I4)#    RI20 187
IK=IK+21#                                                               RI20 188
IA=6*(MA-1)#                                                            RI20 189
JA=6*(MA-1)#                                                            RI20 190
Q1=IA+1#                                                                RI20 191
Q2=IA+3#                                                                RI20 192
Q3=JA+3#                                                                RI20 193
DO  I=Q1,Q2#                                                            RI20 194
DO  J=I,Q3#                                                             RI20 195
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 RI20 196
TSL2(IO)=I*1000+J#                                                      RI20 197
IO=IO+1#                                                                RI20 198
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 RI20 199
JA=JA+3#                                                                RI20 200
Q4=JA+1#                                                                RI20 201
Q3=JA+3#                                                                RI20 202
DO  I=Q1,Q2#                                                            RI20 203
DO  J=Q4,Q3#                                                            RI20 204
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 RI20 205
TSL2(IO)=I*1000+J#                                                      RI20 206
IO=IO+1#                                                                RI20 207
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 RI20 208
IA=3+IA#                                                                RI20 209
Q1=IA+1#                                                                RI20 210
Q2=IA+3#                                                                RI20 211
DO  I=Q1,Q2#                                                            RI20 212
DO  J=I,Q3#                                                             RI20 213
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 RI20 214
TSL2(IO)=I*1000+J#                                                      RI20 215
IO=IO+1#                                                                RI20 216
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 RI20 217
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 RI20 218
DO  I=1,NJOINT#                                                         RI20 219
DO  J1=1,21#                                                            RI20 220
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 RI20 221
J=J1+(I-1)*21#                                                          RI20 222
IF(TSL1(J)==0) NEXT#                                                    RI20 223
B(TSL1(J))=TSL2(J)#                                                     RI20 224
WRITE(LPT,4) TSL1(J),TSL2(J)#                                           RI20 225
4 FORMAT('  B(',I2,') = TSL',I6)#                                       RI20 226
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 RI20 227
WRITE(LPT,15)#                                                          RI20 228
15 FORMAT(/,'END OF B(I) INSERTION ',///)# Ri                           RI20 229
##                                                                      RI20 230
#      SETTING UP OF MATRIX OF CONSTRAINTS#                             RI20 231
##                                                                      RI20 232
RS=0#                                                                   RI20 233
IQ=1#                                                                   RI20 234
DO  IM=1,NN2#                                                           RI20 235
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 RI20 236
V(IQ)  =B11(IM)#                                                        RI20 237
V(IQ+1)=B12(IM)#                                                        RI20 238
V(IQ+2)=B13(IM)#                                                        RI20 239
V(IQ+3)=B22(IM)#                                                        RI20 240
V(IQ+4)=B23(IM)#                                                        RI20 241
V(IQ+5)=B33(IM)#                                                        RI20 242
IQ=IQ+6#                                                                RI20 243
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 RI20 244
#     INITIALISATION OF V(I) ARRAY.#                                    RI20 245
IB=6*NN2+1#                                                             RI20 246
IC=6*NNMAX#                                                             RI20 247
DO IM=IB,IC;V(IM)=0.0#                                                  RI20 248
IF(CONTRL(3)!=0.)WRITE(LPT,13)#                                         RI20 249
13   FORMAT('  MATRIX OF NORMAL EQUATIONS ',/)#                         RI20 250
DO IM=1,NN2#                                                            RI20 251
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 RI20 252
MM=3*(IM-1)#                                                            RI20 253
Q1=MM+1#                                                                RI20 254
Q2=MM+3#                                                                RI20 255
DO  IA=Q1,Q2#                                                           RI20 256
DO  JA=IA,Q2#                                                           RI20 257
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 RI20 258
RS=RS+1#                                                                RI20 259
DO  K=1,NVAR#                                                           RI20 260
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 RI20 261
M=B(K)/1000#                                                            RI20 262
N=B(K)-M*1000#                                                          RI20 263
IF(N==M) AN(RS,K)=TT(IA,M)*TT(JA,M)#                                    RI20 264
ELSE     AN(RS,K)=TT(IA,M)*TT(JA,N)+TT(IA,N)*TT(JA,M)#                  RI20 265
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 RI20 266
IF(CONTRL(3)==0.) NEXT#                                                 RI20 267
WRITE(LPT,10)(AN(RS,II),II=1,NVAR)#                                     RI20 268
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 RI20 269
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 RI20 270
##                                                                      RI20 271
#      ADD EXTRA CONSTRAINT EQUATIONS DUE TO TRACE OF SCREW TENSOR#     RI20 272
#      AND SITE SYMMETRY#                                               RI20 273
#      NCV IS THE NOS OF VARABLES IN THE CONSTRAINT#                    RI20 274
#      COEFF IS THE CONST OF THE CONSTAINT EQ#                          RI20 275
#      WT(I) ARE THE COEFFICIENTS OF THE CONSTRAINT VARIABLES.#         RI20 276
WRITE(LPT,2005)#                                                        RI20 277
2005 FORMAT(//,' EQUATIONS OF CONSTRAINTS',/)#                          RI20 278
IKK=0#                                                                  RI20 279
#      READ CARDS 14         3 CARDS FOR EACH CONSTRAINT#               RI20 280
#                                                                       RI20 281
REPEAT#                                                                 RI20 282
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 RI20 283
READ(CDR,501)NCV,COEF,CWT#                                              RI20 284
IF(NCV==0) BREAK#                                                       RI20 285
READ(CDR,503)(B(I),I=1,NCV)#                                            RI20 286
READ(CDR,504)(WT(I),I=1,NCV)#                                           RI20 287
IKK=IKK+1#                                                              RI20 288
DO I=1,50 ; AN(RS+IKK,I)=0.#                                            RI20 289
DO I=1,NCV; AN(RS+IKK,B(I))=WT(I)#                                      RI20 290
V(RS+IKK)=COEF#                                                         RI20 291
WG(RS+IKK)=CWT#                                                         RI20 292
WRITE(LPT,10)(AN(RS+IKK,J),J=1,NVAR)#                                   RI20 293
WRITE(LPT,2006)V(RS+IKK)#                                               RI20 294
2006 FORMAT(/,'COEFF = ',F10.5,///)#                                    RI20 295
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 RI20 296
501 FORMAT(10X,I3,F10.5,E12.5)#                                         RI20 297
503 FORMAT(10X,25I3)#                                                   RI20 298
504 FORMAT(10X,7F10.5)#                                                 RI20 299
#                                                                       RI20 300
Q1=6*NN2+IKK#                                                           RI20 301
DO  I=1,NVAR#                                                           RI20 302
DO  K=1,NVAR#                                                           RI20 303
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 RI20 304
SS=0.#                                                                  RI20 305
DO  J=1,Q1#                                                             RI20 306
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 RI20 307
SQ=AN(J,I)*WG(J)*AN(J,K)#                                               RI20 308
SS=SQ+SS#                                                               RI20 309
A(I,K)=SS#                                                              RI20 310
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 RI20 311
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 RI20 312
DO  I=1,3#                                                              RI20 313
DO  J=1,NATOM#                                                          RI20 314
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 RI20 315
AFF=X(I,J)#                                                             RI20 316
BFF=XO(I,J)#                                                            RI20 317
XO(I,J)=AFF#                                                            RI20 318
X(I,J)=BFF#                                                             RI20 319
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 RI20 320
DO I=1,NN2 ; AT(I)=AAT(I)#                                              RI20 321
#                                                                       RI20 322
RETURN#                                                                 RI20 323
END#                                                                    RI20 324
#                                                                       RI20 325
systemheader:(RI30)#                   Refine                           RI30   1
# ################################################################## #  RI30   2
#                                                                    #  RI30   3
#                               RI30                                 #  RI30   4
#                                                                    #  RI30   5
# ################################################################## #  RI30   6
SUBROUTINE  SEG2#                                                       RI30   7
include:(AACOMN)#                                                       RI30   8
include:(RICOMN)#                                                       RI30   9
#                                                                       RI30  10
INTEGER Q1,Q2#                                                          RI30  11
dp: WGT#                               R                                RI30  12
#                                                                       RI30  13
dp: XSS(50),PA(50),PB(252),SIG1(252)#                                   RI30  14
#                                                                       RI30  15
# OFF IS THE CONVERGENCE THRESHOLD OF EIGEN FUNCTION EXPANSION#         RI30  16
# THRESH IS THE EIGENVALUE THRESHOLD FOR FILTERING OF PRINCIPAL COM #   RI30  17
#                                                                       RI30  18
#                                                                       RI30  19
THRESH=CONTRL(4)#                                                       RI30  20
OFF=db:(1.E-08)#                                                        RI30  21
IF(CONTRL(5)!=0.) OFF=10**(IFIX(sg:(CONTRL(5))))#                       RI30  22
##                                                                      RI30  23
IF(CONTRL(3)!=0.)#                                                      RI30  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI30  25
WRITE(LPT,6667)#                                                        RI30  26
6667 FORMAT(5X,' LEAST SQUARES MATRIX',/)#                              RI30  27
DO I=1,NVAR ; WRITE(LPT,6666)(A(I,J),J=1,NVAR)#                         RI30  28
6666 FORMAT(2X,10F10.4)#                                                RI30  29
WRITE(LPT,1)#                                                           RI30  30
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI30  31
##                                                                      RI30  32
#      INVERTING NORMAL MATRIX BY SIMILARITY TRANSFORMATIONS#           RI30  33
#      AND FILTERING OF PRINCIPAL COMPONENTS#                           RI30  34
##                                                                      RI30  35
CALL EIGN1M(NVAR,OFF)#                                                  RI30  36
WRITE(LPT,6661)(EIG(I),I=1,NVAR)#                                       RI30  37
6661 FORMAT(/,'EIGENVALUES OF LEAST-SQUARES MATRIX',/,$#                RI30  38
5(1X,10F11.4/))#                                                        RI30  39
#                                                                       RI30  40
IF(CONTRL(3)!=0)#                                                       RI30  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI30  42
WRITE(LPT,729)#                                                         RI30  43
729  FORMAT(5X,' EIGENVECTORS OF LEAST SQUARES MATRIX',/)#              RI30  44
DO I=1,NVAR; WRITE(LPT,6666)(A(I,J),J=1,NVAR)#                          RI30  45
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI30  46
#                                                                       RI30  47
IMAX=1+CONTRL(6)+0.1#                                                   RI30  48
MIN=0#                                                                  RI30  49
I=1#                                                                    RI30  50
730  IF(dabs:(EIG(I))>THRESH) MIN=I#                                    RI30  51
I=I+1#                                                                  RI30  52
IF((MIN==0)&(I<=NVAR)) GO TO 730#                                       RI30  53
IF(MIN!=1)#                                                             RI30  54
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI30  55
WRITE(LPT,6662)#                                                        RI30  56
6662 FORMAT(/,' MATRIX ILLCONDITIONED - NEEDS FILTERING OF ',$#         RI30  57
'PRINCIPLE COMPONENTS ')#                                               RI30  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI30  59
1    FORMAT(/)#                                                         RI30  60
REPEAT#                                                                 RI30  61
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI30  62
Q1=6*NN2+IKK#                                                           RI30  63
Q2=MIN-1#                                                               RI30  64
DO I=MIN,NVAR; XSS(I)=1./EIG(I)#                                        RI30  65
IF(MIN!=1) DO I=1,Q2 ; XSS(I)=0.0#                                      RI30  66
DO L=1,NVAR#                                                            RI30  67
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI30  68
PA(L)=0.#                                                               RI30  69
DO I=1,Q1; PA(L)=PA(L)+AN(I,L)*V(I)*WG(I)#                              RI30  70
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI30  71
DO M=1,NVAR#                                                            RI30  72
DO I=1,NVAR#                                                            RI30  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI30  74
AA(I,M)=0.#                                                             RI30  75
DO J=1,NVAR; AA(I,M)=AA(I,M)+A(I,J)*A(M,J)*XSS(J)#                      RI30  76
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI30  77
DO I=1,NVAR#                                                            RI30  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI30  79
VAR(I)=0.#                                                              RI30  80
DO J=1,NVAR ; VAR(I)=VAR(I)+AA(I,J)*PA(J)#                              RI30  81
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI30  82
INVAR=NVAR-MIN+1#                                                       RI30  83
WRITE(LPT,6665)INVAR#                                                   RI30  84
6665 FORMAT(5X,I3,' PRINCIPAL COMPONENTS FILTERED')#                    RI30  85
WGT=0.#                                                                 RI30  86
DO  I=1,Q1#                                                             RI30  87
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  RI30  88
PB(I)=0.#                                                               RI30  89
DO J=1,NVAR; PB(I)=PB(I)+AN(I,J)*VAR(J)#                                RI30  90
WGT=WGT+(PB(I)-V(I))*(PB(I)-V(I))*WG(I)#                                RI30  91
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  RI30  92
WGT=WGT/(6*NN2+IKK-NVAR)#                                               RI30  93
DO I=1,Q1#                                                              RI30  94
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  RI30  95
SIG1(I)=0.#                                                             RI30  96
DO J=1,NVAR#                                                            RI30  97
DO  K=1,NVAR;SIG1(I)=SIG1(I)+AN(I,J)*AA(J,K)*AN(I,K)#                   RI30  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  RI30  99
WRITE(LPT,1)#                                                           RI30 100
CALL OUTPT(PB,WGT,SIG1)#                                                RI30 101
WRITE(LPT,1)#                                                           RI30 102
CALL OUTVAR(WGT)#                                                       RI30 103
MIN=MIN+1#                                                              RI30 104
IMAX=IMAX-1#                                                            RI30 105
#IF(IMAX!=0)GO TO 734#                                                  RI30 106
IF(IMAX==0)BREAK#                                                       RI30 107
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI30 108
6    CONTINUE#                                                          RI30 109
#                                                                       RI30 110
RETURN#                                                                 RI30 111
END#                                                                    RI30 112
systemheader:(RI32)#                                                    RI32   1
# ################################################################## #  RI32   2
#                                                                    #  RI32   3
#                               RI32                                 #  RI32   4
#                                                                    #  RI32   5
# ################################################################## #  RI32   6
SUBROUTINE OUTPT(CAL,W,R)#                                              RI32   7
include:(AACOMN)#                                                       RI32   8
include:(RICOMN)#                                                       RI32   9
#                                                                       RI32  10
dp: W#                                 R                                RI32  11
dp: CAL(252),R(252)#                   Ri32                             RI32  12
dp: OBS(252)#                          Ri32                             RI32  13
dp: SUMOBS#                                                             RI32  14
dp: SUMCAL#                                                             RI32  15
dp: SDELTA#                                                             RI32  16
dp: DELTA#                                                              RI32  17
#                                                                       RI32  18
realdata:(FMT,[511052.,611052.,711052.,$#                               RI32  19
811052.,911052.,1011052.,1111052.])#                                    RI32  20
chardata:(RI321,"U11U12U13U22U23U33 ")#                                 RI32  21
chardata:(RI322," ATOM SEG    X         Y         Z          $#         RI32  22
UIJ OBS   UIJ CAL     SIGMA    WEIGHT        I1        I2$#             RI32  23
   DEL UIJ")#                                                           RI32  24
datastuff:#                                                             RI32  25
#                                                                       RI32  26
DO I=1,252;OBS(I)=V(I)#                                                 RI32  27
SUMOBS=0.#                                                              RI32  28
SUMCAL=0.#                                                              RI32  29
SDELTA=0.#                                                              RI32  30
WRITE(LPT,2) CONTRL(11)#                                                RI32  31
WRITE(LPT,1)#                                                           RI32  32
1    FORMAT(22X,'COORDS',69X,'CONFIDENCE INTERVAL',/)#                  RI32  33
2    FORMAT(' ALL CONFIDENCE INTERVALS (I1,I2) SET AT PROBABILITY',$#   RI32  34
' OF  ',F10.5,/,' ASSUMING SUM OF SQUARES OF DIFFERENCE OF OBS AND',$#  RI32  35
'CAL VALUES IS DISTRIBUTED AS CHI SQUARED')#                            RI32  36
3 FORMAT(/,A5,I2,3F10.5)#              Ri                               RI32  37
LINRM=2;     writeline:(1,RI322,NRI322,1,3)# Set pagination             RI32  38
ISEG=0#                                                                 RI32  39
IGUN=1#                                                                 RI32  40
IIJJ=NN2+IKK#                                                           RI32  41
DO I=1,IIJJ#                                                            RI32  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI32  43
I1=6*(I-1)+1#                                                           RI32  44
IF(I<=NN2)GO TO 9#                                                      RI32  45
IT=I-NN2#                                                               RI32  46
WRITE(LPT,4)IT#                                                         RI32  47
4   FORMAT(5X,'CONSTRAINTS',I2)#                                        RI32  48
L=L+1#                                                                  RI32  49
IF(R(L)<0.000000001)R(L)=0.000000001#                                   RI32  50
SID=CONTRL(10)*dsqrt:(W*R(L))#                                          RI32  51
R(L)=dsqrt:(W*R(L))#                                                    RI32  52
CON1=CAL(L)-SID#                                                        RI32  53
CON2=CAL(L)+SID#                                                        RI32  54
DELTA=OBS(L)-CAL(L)#                                                    RI32  55
BUFOT(1)=OBS(L)#                                                        RI32  56
BUFOT(2)=CAL(L)#                                                        RI32  57
BUFOT(3)=R(L)#                                                          RI32  58
BUFOT(4)=WG(L)#                                                         RI32  59
BUFOT(5)=CON1#                                                          RI32  60
BUFOT(6)=CON2#                                                          RI32  61
BUFOT(7)=DELTA#                                                         RI32  62
ncodefld:(BUFOT,1,CHROT,FMT,7)#                                         RI32  63
writeline:(0,RI322,NRI322,1,3)#                                         RI32  64
#WRITE(LPT,FMT)OBS(L),CAL(L),R(L),WG(L),CON1,CON2,DELTA#                RI32  65
NEXT#                                                                   RI32  66
9    CONTINUE#                                                          RI32  67
IF(I>=IGUN)ISEG=ISEG+1#                                                 RI32  68
WRITE(LPT,3)AT(I),ISEG,(X(J,I),J=1,3)#                                  RI32  69
IF(I>=IGUN) IGUN=IGUN+NX(ISEG)#                                         RI32  70
DO  K=1,6#                                                              RI32  71
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI32  72
IF(K==1) L=I1#                                                          RI32  73
IF(K==2) L=I1+1#                                                        RI32  74
IF(K==3) L=I1+2#                                                        RI32  75
IF(K==4) L=I1+3#                                                        RI32  76
IF(K==5) L=I1+4#                                                        RI32  77
IF(K==6) L=I1+5#                                                        RI32  78
movechar:(RI321,1+3*(K-1),CHROT,39,3,0)#                                RI32  79
#                                                                       RI32  80
SID=CONTRL(10)*dsqrt:(W*R(L))#                                          RI32  81
R(L)=dsqrt:(W*R(L))#                                                    RI32  82
CON1=CAL(L)-SID#                                                        RI32  83
CON2=CAL(L)+SID#                                                        RI32  84
SUMOBS=SUMOBS+dabs:(OBS(L))#                                            RI32  85
SUMCAL=SUMCAL+dabs:(CAL(L))#                                            RI32  86
DELTA=OBS(L)-CAL(L)#                                                    RI32  87
SDELTA=SDELTA+dabs:(DELTA)#                                             RI32  88
BUFOT(1)=OBS(L)#                                                        RI32  89
BUFOT(2)=CAL(L)#                                                        RI32  90
BUFOT(3)=R(L)#                                                          RI32  91
BUFOT(4)=WG(L)#                                                         RI32  92
BUFOT(5)=CON1#                                                          RI32  93
BUFOT(6)=CON2#                                                          RI32  94
BUFOT(7)=DELTA#                                                         RI32  95
ncodefld:(BUFOT,1,CHROT,FMT,7)#                                         RI32  96
writeline:(0,XT322,NXT322,1,3)#                                         RI32  97
#WRITE(LPT,FMT)OBS(L),CAL(L),R(L),WG(L),CON1,CON2,DELTA#                RI32  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI32  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI32 100
RFAC=SDELTA/SUMOBS#                                                     RI32 101
WRITE(LPT,30)SUMOBS,SUMCAL,SDELTA,RFAC#                                 RI32 102
30   FORMAT(10X,'SUM OF UIJOBS = ',E15.5,/,10X,'SUM OF UIJCAL = ',$#    RI32 103
E15.5,/,10X,'SUM OF MOD OF DIFFERENCE = ',E15.5,/,10X,$#                RI32 104
'OVERALL R-FACTOR = ',F10.5)#                                           RI32 105
#                                                                       RI32 106
RETURN#                                                                 RI32 107
END#                                                                    RI32 108
#                                                                       RI32 109
systemheader:(RI34)#                                                    RI34   1
# ################################################################## #  RI34   2
#                                                                    #  RI34   3
#                               RI34                                 #  RI34   4
#                                                                    #  RI34   5
# ################################################################## #  RI34   6
SUBROUTINE OUTVAR(WGT)#                                                 RI34   7
include:(AACOMN)#                                                       RI34   8
include:(RICOMN)#                                                       RI34   9
#                                                                       RI34  10
dp: COVAR(50)#                         Ri34                             RI34  11
dp: SIG2#                              R                                RI34  12
dp: WGT#                               R                                RI34  13
dp: ARGU#                                                               RI34  14
dp: SIGMA#                                                              RI34  15
dp: SIN#                                                                RI34  16
dp: CON1,CON2#                                                          RI34  17
dp: VARS#                                                               RI34  18
#                                                                       RI34  19
#                                                                       RI34  20
JOINT=1#                                                                RI34  21
M=1#                                                                    RI34  22
WRITE(LPT,1)#                                                           RI34  23
1  FORMAT(5X,'ELEMENTS OF TSL MATRIX REFINED',//,57X,'CONFIDENCE ',$#   RI34  24
'INTERVAL',/,2X,'ELEMENT  TSL INDEX  JOINT  CALCULATED VALUE SIGMA',$#  RI34  25
9X,'I1',8X,'I2',/)#                                                     RI34  26
DO  I=1,NVAR#                                                           RI34  27
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI34  28
SIG2=dsqrt:(WGT)#                                                       RI34  29
SIGMA=SIG2*dsqrt:(AA(I,I))#                                             RI34  30
SIN=CONTRL(10)*SIGMA#                                                   RI34  31
CON1=VAR(I)-SIN#                                                        RI34  32
CON2=VAR(I)+SIN#                                                        RI34  33
IOI=0#                                                                  RI34  34
IJ1=21*(JOINT-1)+1#                                                     RI34  35
IJ2=21*JOINT#                                                           RI34  36
DO J=IJ1,IJ2; IF(TSL1(J)==I) IOI=1#                                     RI34  37
IF(IOI==0)JOINT=JOINT+1#                                                RI34  38
DO L=M,210#                                                             RI34  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI34  40
IF(TSL1(L)==0)NEXT#                                                     RI34  41
WRITE(LPT,2)TSL1(L),TSL2(L),JOINT,VAR(I),SIGMA,CON1,CON2#               RI34  42
2 FORMAT(3X,'B(',I2,')',5X,I5,5X,I2,6X,F10.5,3X,3F10.5)#                RI34  43
BREAK#                                                                  RI34  44
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI34  45
M=L+1#                                                                  RI34  46
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI34  47
CON1=CONTRL(8)*SIG2#                                                    RI34  48
CON2=CONTRL(9)*SIG2#                                                    RI34  49
ARGU=2*(6*NN2+IKK-NVAR)#                                                RI34  50
VARS=SIG2/dsqrt:(ARGU)#                                                 RI34  51
WRITE(LPT,3) SIG2 ,VARS,CON1,CON2#                                      RI34  52
3     FORMAT(//,'  GOODNESS OF FIT INDEX (SIGMA)',10X,E15.5,/,$#        RI34  53
'  VARIANCE OF SIGMA',21X,E15.5,/,'  CONFIDENCE INTERVAL ','(',$#       RI34  54
E15.5,',',E15.5,')')#                                                   RI34  55
I2=0#                                                                   RI34  56
WRITE(LPT,102)#                                                         RI34  57
N1=NVAR/10#                                                             RI34  58
N2=NVAR-N1*10#                                                          RI34  59
K1=0#                                                                   RI34  60
L1=1#                                                                   RI34  61
K=1#                                                                    RI34  62
I2=0#                                                                   RI34  63
I3=10#                                                                  RI34  64
#                                                                       RI34  65
#                                                                       RI34  66
#                                                                       RI34  67
REPEAT#                                                                 RI34  68
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI34  69
L3=(K-1)*10#                                                            RI34  70
I1=L3+1#                                                                RI34  71
IJ1=I2+I3#                                                              RI34  72
WRITE(LPT,100)(J,J=I1,IJ1)#                                             RI34  73
DO L2=I1,NVAR#                                                          RI34  74
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI34  75
I2=L1#                                                                  RI34  76
DO I=I1,I2#                                                             RI34  77
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI34  78
K1=K1+1#                                                                RI34  79
IF (AA(I,I)<=0.\ AA(L2,L2)<=0.) COVAR(K1)=0.0000#                       RI34  80
ELSE COVAR(K1)=AA(I,L2)/(dsqrt:(AA(I,I))*dsqrt:(AA(L2,L2)))#            RI34  81
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI34  82
K1=0#                                                                   RI34  83
IJ1=(K-1)*10#                                                           RI34  84
IJ2=I2-IJ1#                                                             RI34  85
IJ1=I1-IJ1#                                                             RI34  86
WRITE(LPT,101)L2,(COVAR(J),J=IJ1,IJ2)#                                  RI34  87
L1=L1+1#                                                                RI34  88
IF(L1>K*10) L1=L1-1#                                                    RI34  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI34  90
L1=K*10+1#                                                              RI34  91
K=K+1#                                                                  RI34  92
#                                                                       RI34  93
IF     (K-N1-1 >0) BREAK#                                               RI34  94
ELSEIF (K-N1-1==0)#                                                     RI34  95
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI34  96
IF(N2==0)  BREAK#                                                       RI34  97
I3=N2#                                                                  RI34  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI34  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI34 100
100  FORMAT(//,5X,10I10)#                                               RI34 101
101  FORMAT(2X,'VAR(',I2,') ',10F10.7)#                                 RI34 102
102  FORMAT(/,/'   MATRIX OF CORRELATIONS',//)# Ri                      RI34 103
#                                                                       RI34 104
RETURN#                                                                 RI34 105
END#                                                                    RI34 106
systemheader:(RI36)#                   Enter control lines & bdf        RI36   1
# ################################################################## #  RI36   2
#                                                                    #  RI36   3
#                               RI10                                 #  RI36   4
#                                                                    #  RI36   5
# ################################################################## #  RI36   6
SUBROUTINE EIGN1M(NN,RHO)#                                              RI36   7
include:(AACOMN)#                                                       RI36   8
include:(RICOMN)#                                                       RI36   9
#                                                                       RI36  10
#  THIS ROUTINE IS A VARIANT OF EIGN WHICH DIAGONALIZES A MATRIX#       RI36  11
#  WITHIN ITSELF.#                                                      RI36  12
#  THE INPUT ARRAY, A, IS A SYMMETRIC ARRAY.#                           RI36  13
#  IF IMAT=0,  A IS A SYMMETRIC N X N MATRIX IN A (140,140) MATRIX#     RI36  14
#  IF IMAT =1, A IS A LINEAR TRIANGULAR MATRIX  IN THE FIRST HALF OF    RI36  15
#  A ( I.E. B(I,1) ).#                                                  RI36  16
#  IF IMAT=2, A IS IN THE SECOND HALF OF A.#                            RI36  17
#  THE ARRAY A IS TOTALLY DESTROYED (ALL 20000 WORDS)#                  RI36  18
#  RHO= UPPER LIMIT FOR OFF-DIAGONAL ELEMENT#                           RI36  19
#  NN = SIZE OF MATRIX#                                                 RI36  20
#  EIG = RETURNED EIGENVALUES IN ALGEBRAIC ASCENDING ORDER#             RI36  21
#  VEC = RETURNED EIGENVECTORS IN COLUMNS#                              RI36  22
#                                                                       RI36  23
INTEGER NN#                                                             RI36  24
INTEGER N#                                                              RI36  25
#                                                                       RI36  26
dp: A2#                                R                                RI36  27
dp: VEC(50,50)#                        Ri36                             RI36  28
dp: GAMMA(50),BETA(50),BETASQ(50),W(50)# Ri36                           RI36  29
dp: P(50),V1(50)#                      Ri36                             RI36  30
INTEGER IPOSV(50),IVPOS(50),IORD(50)#  Ri                               RI36  31
dp: S#                                                                  RI36  32
dp: RHO#                                                                RI36  33
dp: RHOSQ#                                                              RI36  34
dp: TEMP#                                                               RI36  35
dp: D#                                                                  RI36  36
dp: PPBS#                                                               RI36  37
dp: PPBR#                                                               RI36  38
dp: BSCAL#                                                              RI36  39
dp: R1,R2#                                                              RI36  40
dp: DIF#                                                                RI36  41
dp: SHIFT#                                                              RI36  42
#                                                                       RI36  43
EQUIVALENCE (VEC(1,1),A(1,1))#                                          RI36  44
EQUIVALENCE (P(1),BETA(1)),(V1(1),BETA(1))#                             RI36  45
EQUIVALENCE (IPOSV(1),GAMMA(1)),(IVPOS(1),BETA(1)),(IORD(1),BETASQ(1))# RI36  46
#                                                                       RI36  47
DATA ONE,TWO,HALF,ZERO,CONE,IARRAY/1.,2.,.5,0.,-1.,2850/#               RI36  48
#                                                                       RI36  49
#                                                                       RI36  50
WRITE(LPT2,1010)#                                                       RI36  51
RHOSQ=RHO*RHO#                                                          RI36  52
N=NN#                                                                   RI36  53
IF(N==0)  GOTO 500#                                                     RI36  54
N1=N-1#                                                                 RI36  55
N2=N-2#                                                                 RI36  56
##                                                                      RI36  57
GAMMA(1)=A(1,1)#                                                        RI36  58
IF(N2>0)#                                                               RI36  59
DO NR=1,N2#                                                             RI36  60
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI36  61
BSCAL=A(NR+1,NR)#                                                       RI36  62
S=ZERO#                                                                 RI36  63
DO I=NR,N2;S=S+A(I+2,NR)**2#                                            RI36  64
#                                                                       RI36  65
#     PREPARE FOR POSSIBLE BYPASS OF TRANSFORMATION#                    RI36  66
#                                                                       RI36  67
A(NR+1,NR)=ZERO#                                                        RI36  68
IF (S<=0) GOTO 250#                                                     RI36  69
S=S+BSCAL*BSCAL#                                                        RI36  70
SGN=ONE#                                                                RI36  71
IF (BSCAL<=0) SGN=CONE#                                                 RI36  72
SQRTS=dsqrt:(S)#                                                        RI36  73
D=SGN/(SQRTS+SQRTS)#                                                    RI36  74
TEMP=dsqrt:(HALF+BSCAL*D)#                                              RI36  75
W(NR)=TEMP#                                                             RI36  76
A(NR+1,NR)=TEMP#                                                        RI36  77
D=D/TEMP#                                                               RI36  78
BSCAL=-SGN*SQRTS#                                                       RI36  79
#                                                                       RI36  80
#     D IS FACTOR OF PROPORTIONALITY. NOW COMPUTE AND SAVE W VECTOR.#   RI36  81
#     EXTRA SINGLY SUBSCRIPTED W VECTOR USED FOR SPEED.#                RI36  82
#                                                                       RI36  83
DO I=NR,N2#                                                             RI36  84
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI36  85
TEMP=D*A(I+2,NR)#                                                       RI36  86
W(I+1)=TEMP#                                                            RI36  87
A(I+2,NR)=TEMP#                                                         RI36  88
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI36  89
#                                                                       RI36  90
#     PREMULTIPLY VECTOR W BY MATRIX A TO OBTAIN P VECTOR.#             RI36  91
#     SIMULTANEOUSLY ACCUMULATE DOT PRODUCT WP,(THE SCALAR K)#          RI36  92
#                                                                       RI36  93
WTAW=ZERO#                                                              RI36  94
DO I=NR,N1#                                                             RI36  95
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI36  96
SUM=ZERO#                                                               RI36  97
DO J=NR,I; SUM=SUM+A(I+1,J+1)*W(J)#                                     RI36  98
I1=I+1#                                                                 RI36  99
IF(N1-I1>=0) DO J=I1,N1; SUM=SUM+A(J+1,I+1)*W(J)#                       RI36 100
P(I)=SUM#                                                               RI36 101
WTAW=WTAW+SUM*W(I)#                                                     RI36 102
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI36 103
#                                                                       RI36 104
#     P VECTOR AND SCALAR K NOW STORED. NEXT COMPUTE Q VECTOR#          RI36 105
#                                                                       RI36 106
DO I=NR,N1; V1(I)=P(I)-WTAW*W(I)#                                       RI36 107
#                                                                       RI36 108
#     NOW FORM PAP MATRIX, REQUIRED PART#                               RI36 109
DO J=NR,N1#                                                             RI36 110
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI36 111
QJ=V1(J)#                                                               RI36 112
WJ=W(J)#                                                                RI36 113
DO I=J,N1; A(I+1,J+1)=A(I+1,J+1)-TWO*(W(I)*QJ+WJ*V1(I))#                RI36 114
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI36 115
250 BETA(NR)=BSCAL#                                                     RI36 116
BETASQ(NR)=BSCAL*BSCAL#                                                 RI36 117
GAMMA(NR+1)=A(NR+1,NR+1)#                                               RI36 118
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI36 119
IF (N2>=0)#                                                             RI36 120
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI36 121
BSCAL=A(N,N-1)#                                                         RI36 122
BETA(N-1)=BSCAL#                                                        RI36 123
BETASQ(N-1)=BSCAL*BSCAL#                                                RI36 124
GAMMA(N)=A(N,N)#                                                        RI36 125
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI36 126
BETASQ(N)=ZERO#                                                         RI36 127
DO I=1,N#                                                               RI36 128
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI36 129
DO J=I,N;A(I,J)=ZERO#                                                   RI36 130
A(I,I)=ONE#                                                             RI36 131
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI36 132
#     FORM PRODUCT P1*P2*...*PN-2#                                      RI36 133
K=N#                                                                    RI36 134
540 KP=K#                                                               RI36 135
K=K-1#                                                                  RI36 136
KM=K-1#                                                                 RI36 137
AKKM=A(K,KM)#                                                           RI36 138
DO J=KP,N#                                                              RI36 139
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI36 140
DO I=K,N#                                                               RI36 141
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  RI36 142
SUM=ZERO#                                                               RI36 143
DO L=K,N;SUM=SUM+A(L,KM)*A(L,J)#                                        RI36 144
SUM=SUM+SUM#                                                            RI36 145
W(I)=A(I,J)-SUM*A(I,KM)#                                                RI36 146
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  RI36 147
DO  I=K,N;A(I,J)=W(I)#                                                  RI36 148
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI36 149
DO I=KP,N;A(I,K)=-TWO*A(I,KM)*AKKM#                                     RI36 150
A(K,K)=ONE-TWO*AKKM*AKKM#                                               RI36 151
IF(K-3>=0) GOTO 540#                                                    RI36 152
DO I=2,N;A(I,1)=ZERO#                                                   RI36 153
M=N#                                                                    RI36 154
SUM=ZERO#                                                               RI36 155
GO TO 400#                                                              RI36 156
310 CONTINUE#                                                           RI36 157
SUM=SUM+SHIFT#                                                          RI36 158
COSA=ONE#                                                               RI36 159
G=GAMMA(1)-SHIFT#                                                       RI36 160
PP=G#                                                                   RI36 161
PPBS=PP*PP+BETASQ(1)#                                                   RI36 162
PPBR=dsqrt:(PPBS)#                                                      RI36 163
DO J=1,M#                              Ri36                             RI36 164
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  RI36 165
COSAP=COSA#                                                             RI36 166
IF(PPBS==0)#                                                            RI36 167
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 RI36 168
SINA=ZERO#                                                              RI36 169
SINA2=ZERO#                                                             RI36 170
COSA=ONE#                                                               RI36 171
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 RI36 172
ELSE#                                                                   RI36 173
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 RI36 174
SINA=BETA(J)/PPBR#                     Ri36                             RI36 175
SINA2=BETASQ(J)/PPBS#                                                   RI36 176
COSA=PP/PPBR#                                                           RI36 177
#     POSTMULTIPLY P1*...*PN-2 BY ROTATION MATRIX   (P-TRANSPOSE)#      RI36 178
DO I=1,N#                                                               RI36 179
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 RI36 180
VTEMP=VEC(I,J)#                                                         RI36 181
VJTEMP=VEC(I,J+1)#                                                      RI36 182
VEC(I,J)=COSA*VTEMP+SINA*VJTEMP#                                        RI36 183
VEC(I,J+1)=-SINA*VTEMP+COSA*VJTEMP#                                     RI36 184
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 RI36 185
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 RI36 186
DIA=GAMMA(J+1)-SHIFT#                                                   RI36 187
U=SINA2*(G+DIA)#                                                        RI36 188
GAMMA(J)=G+U#                                                           RI36 189
G=DIA-U#                                                                RI36 190
PP=DIA*COSA-SINA*COSAP*BETA(J)#                                         RI36 191
IF(J-M==0)#                                                             RI36 192
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 RI36 193
BETA(J)=SINA*PP#                                                        RI36 194
BETASQ(J)=SINA2*PP*PP#                                                  RI36 195
BREAK#                                                                  RI36 196
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 RI36 197
PPBS=PP*PP+BETASQ(J+1)#                                                 RI36 198
PPBR=dsqrt:(PPBS)#                                                      RI36 199
BETA(J)=SINA*PPBR#                                                      RI36 200
BETASQ(J)=SINA2*PPBS#                                                   RI36 201
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  RI36 202
GAMMA(M+1)=G#                                                           RI36 203
#                                                                       RI36 204
#     TEST FOR CONVERGENCE OF LAST DIAGONAL ELEMENT#                    RI36 205
#                                                                       RI36 206
385 CONTINUE#                                                           RI36 207
IF(BETASQ(M)-RHOSQ >0) GOTO 410#       R                                RI36 208
EIG(M+1)=GAMMA(M+1)+SUM#                                                RI36 209
#                                                                       RI36 210
400 CONTINUE#                                                           RI36 211
BETA(M)=ZERO#                                                           RI36 212
BETASQ(M)=ZERO#                                                         RI36 213
M=M-1#                                                                  RI36 214
IF(M==0) GOTO 430#                                                      RI36 215
ELSE GOTO 385#                                                          RI36 216
#                                                                       RI36 217
#     TAKE ROOT OF CORNER 2 BY 2 NEAREST TO LOWER DIAGONAL IN VALUE#    RI36 218
#     AS ESTIMATE OF EIGENVALUE TO USE FOR SHIFT#                       RI36 219
#                                                                       RI36 220
410 A2=GAMMA(M+1)#                                                      RI36 221
R2=HALF*A2#                                                             RI36 222
R1=HALF*GAMMA(M)#                                                       RI36 223
R12=R1+R2#                                                              RI36 224
DIF=R1-R2#                                                              RI36 225
TEMP=dsqrt:(DIF*DIF+BETASQ(M))#                                         RI36 226
R1=R12+TEMP#                                                            RI36 227
R2=R12-TEMP#                                                            RI36 228
DIF=dabs:(A2-R1)-dabs:(A2-R2)#                                          RI36 229
IF(DIF>=0) SHIFT=R2#                                                    RI36 230
ELSE       SHIFT =R1#                                                   RI36 231
GO TO 310#                                                              RI36 232
#                                                                       RI36 233
#                                                                       RI36 234
#                                                                       RI36 235
430 EIG(1)=GAMMA(1)+SUM#                                                RI36 236
#  INITIALIZE AUXILIARY TABLES REQUIRED FOR REARRANGING THE VECTORS     RI36 237
DO J=1,N#                                                               RI36 238
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 RI36 239
IPOSV(J)=J#                                                             RI36 240
IVPOS(J)=J#                                                             RI36 241
IORD(J)=J#                                                              RI36 242
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 RI36 243
#                                                                       RI36 244
#     USE A TRANSPOSITION SORT TO ORDER THE EIGENVALUES#                RI36 245
#                                                                       RI36 246
M=N#                                                                    RI36 247
GO TO 470#                                                              RI36 248
450 CONTINUE#                                                           RI36 249
DO J=1,M#                                                               RI36 250
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 RI36 251
IF(EIG(J)-EIG(J+1)<=0)  NEXT#                                           RI36 252
TEMP=EIG(J)#                                                            RI36 253
EIG(J)=EIG(J+1)#                                                        RI36 254
EIG(J+1)=TEMP#                                                          RI36 255
ITEMP=IORD(J)#                                                          RI36 256
IORD(J)=IORD(J+1)#                                                      RI36 257
IORD(J+1)=ITEMP#                                                        RI36 258
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 RI36 259
470 CONTINUE#                                                           RI36 260
M=M-1#                                                                  RI36 261
IF(M!=0) GOTO 450#                                                      RI36 262
IF(N1==0)GOTO 500#                                                      RI36 263
DO  L=1,N1#                                                             RI36 264
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 RI36 265
NV=IORD(L)#                                                             RI36 266
NP=IPOSV(NV)#                                                           RI36 267
IF(NP-L==0) GOTO 490#                                                   RI36 268
LV=IVPOS(L)#                                                            RI36 269
IVPOS(NP)=LV#                                                           RI36 270
IPOSV(LV)=NP#                                                           RI36 271
DO I=1,N#                                                               RI36 272
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 RI36 273
TEMP=VEC(I,L)#                                                          RI36 274
VEC(I,L)=VEC(I,NP)#                                                     RI36 275
VEC(I,NP)=TEMP#                                                         RI36 276
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 RI36 277
490 CONTINUE#                                                           RI36 278
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 RI36 279
500 CONTINUE#                                                           RI36 280
WRITE(LPT2,1011)#                                                       RI36 281
1011 FORMAT(' EXIT EIGNIM ')#                                           RI36 282
1010 FORMAT(' ENTER EIGNIM ')#                                          RI36 283
RETURN#                                                                 RI36 284
END#                                                                    RI36 285
#                                                                       RI36 286
systemheader:(RI40)#                   Enter control lines & bdf        RI40   1
# ################################################################## #  RI40   2
#                                                                    #  RI40   3
#                               RI40                                 #  RI40   4
#                                                                    #  RI40   5
#  THIS PROGRAM CORRECTS FRACTIONAL ATOM COORDS. FOR LIBRATION -     #  RI40   6
#  FIRST SEGMENT IS MAJOR ONE, ALL OTHERS RIDE ON IT.  IE. ALL       #  RI40   7
#  COORDS. ARE FIRST CORRECTED FOR LIBRATION EFFECTS OF THE MAIN     #  RI40   8
#  SEGMENT OF THE MOLECULE. ATOMS OF OTHER SEGMENTS ARE THEN FURTHER #  RI40   9
#  CORRECTED FOR LIBRATION EFFECTS OF THE INDIVIDUAL SMALL SEGMENTS. #  RI40  10
#  ASSUME EACH SEGMENT HAS AN ASSOCIATED JOINT. IE. SEG1..JOINT1 ETC.#  RI40  11
# ####################################################################  RI40  12
SUBROUTINE CORLIB#                                                      RI40  13
include:(AACOMN)#                                                       RI40  14
include:(RICOMN)#                                                       RI40  15
#                                                                       RI40  16
#                                                                       RI40  17
dp: R(3,3),RR(3,3),RR1(3,3),XX(3,42),Y(3,42),Z(3,42)# Ri40              RI40  18
dp: TTO(3,3),P(3,42)#                  Ri40                             RI40  19
INTEGER I,J,K,N,M#                                                      RI40  20
INTEGER M1,M2,M3#                                                       RI40  21
INTEGER N1#                                                             RI40  22
INTEGER K1,K2#                                                          RI40  23
INTEGER NV#                                                             RI40  24
dp: OR(3)#                             R                                RI40  25
dp: L(3,3)#                            R                                RI40  26
dp: B(50)#                             R                                RI40  27
dp: DEL#                                                                RI40  28
#                                                                       RI40  29
900 FORMAT(16I5)#                                                       RI40  30
901 FORMAT(/,/,1X,'NO. OF ATOMS =',I5,/)# Ri                            RI40  31
1000 FORMAT(3F10.5)#                                                    RI40  32
1001 FORMAT(/,' ORIGIN')#              R                                RI40  33
1002 FORMAT(/,' ROLLETT MATRIX')#      R                                RI40  34
1003 FORMAT(/,' ROTATION MATRIX')#     R                                RI40  35
1004 FORMAT(/,' MODIFIED LIBRATION MATRIX')# R                          RI40  36
1005 FORMAT(/,'  ATOM        UNCORRECTED COORDS.',27X,$# Ri             RI40  37
'CORRECTED COORDS.',/)#                                                 RI40  38
1007 FORMAT(/,//,/,' JOINT AND SEGMENT',I2,//)# Ri4                     RI40  39
1010 FORMAT(7X,A5,1X,3F8.4,11X,3F8.5)#                                  RI40  40
1011 FORMAT(80X)#                                                       RI40  41
1020 FORMAT(2X,A5,3X,3F12.6,10X,3F12.6)#                                RI40  42
1021 FORMAT('ATOM',3X,A5,1X,3F8.5,7X,'1.00',3F8.5)#                     RI40  43
1050 FORMAT(/,' LIBRATION TENSOR',3(/3F10.5),/)#                        RI40  44
#                                                                       RI40  45
OR(1)=X1 ;OR(2)=Y1 ; OR(3)=Z1#                                          RI40  46
WRITE(LPT2,901)NN2#                                                     RI40  47
WRITE(LPT2,1001)#                                                       RI40  48
WRITE(LPT2,1000)(OR(I),I=1,3)#                                          RI40  49
WRITE(LPT2,1002)#                                                       RI40  50
WRITE(LPT2,1000)((TO(I,J),J=1,3),I=1,3)#                                RI40  51
WRITE(LPT2,1003)#                                                       RI40  52
WRITE(LPT2,1000)((AR(I,J),J=1,3),I=1,3)#                                RI40  53
DEL=TO(1,1)*TO(2,2)*TO(3,3)#                                            RI40  54
TTO(1,1)=1./TO(1,1)#                                                    RI40  55
TTO(2,2)=1./TO(2,2)#                                                    RI40  56
TTO(3,3)=1./TO(3,3)#                                                    RI40  57
TTO(2,1)=0.#                                                            RI40  58
TTO(3,1)=0.#                                                            RI40  59
TTO(2,3)=0.#                                                            RI40  60
TTO(1,2)=(TO(3,2)*TO(1,3)-TO(1,2)*TO(3,3))/DEL#                         RI40  61
TTO(3,2)=-TO(1,1)*TO(3,2)/DEL#                                          RI40  62
TTO(1,3)=-TO(1,3)*TO(2,2)/DEL#                                          RI40  63
K1=1#                                                                   RI40  64
NV=0#                                                                   RI40  65
#     CYCLE OVER NO. OF JOINTS#                                         RI40  66
DO M=1,NJOINT#                                                          RI40  67
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI40  68
WRITE(LPT2,1007)M#                                                      RI40  69
M1=(M-1)*21+1#                                                          RI40  70
M2=M1+15#                                                               RI40  71
M3=M2+5#                                                                RI40  72
DO N=M1,M3#                                                             RI40  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI40  74
IF(TSL1(N)!=0) NV=NV+1#                                                 RI40  75
IF(N<M2) NEXT#                                                          RI40  76
N1=N-15-((M-1)*21)#                                                     RI40  77
IF(TSL1(N)==0) B(N1)=0.0#                                               RI40  78
ELSE           B(N1)=VAR(NV)#                                           RI40  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI40  80
L(1,1)=B(1)#                                                            RI40  81
L(1,2)=B(2)#                                                            RI40  82
L(1,3)=B(3)#                                                            RI40  83
L(2,1)=B(2)#                                                            RI40  84
L(2,2)=B(4)#                                                            RI40  85
L(2,3)=B(5)#                                                            RI40  86
L(3,1)=B(3)#                                                            RI40  87
L(3,2)=B(5)#                                                            RI40  88
L(3,3)=B(6)#                                                            RI40  89
WRITE(LPT,1050)((L(I,J),J=1,3),I=1,3)#                                  RI40  90
DO I=1,3#                                                               RI40  91
DO J=1,3; R(I,J)=0.#                                                    RI40  92
R(1,1)=1.#                                                              RI40  93
R(2,2)=1.#                                                              RI40  94
R(3,3)=1.#                                                              RI40  95
TRACE=L(1,1)+L(2,2)+L(3,3)#                                             RI40  96
DO I=1,3#                                                               RI40  97
DO J=1,3 ;L(I,J)=(1.+0.5*TRACE)*R(I,J)-0.5*L(I,J)#                      RI40  98
WRITE(LPT2,1004)#                                                       RI40  99
WRITE(LPT2,1000)(( L(I,J),J=1,3),I=1,3)#                                RI40 100
WRITE(LPT2,1001)#                                                       RI40 101
WRITE(LPT2,1000)(O(I,M),I=1,3)#                                         RI40 102
WRITE(LPT2,1003)#                                                       RI40 103
WRITE(LPT2,1000)((Q(M,I,J),J=1,3),I=1,3)#                               RI40 104
IF(M==1)CALL BONAN(L,X,NN2,M,LPT,CDR)#                                  RI40 105
WRITE(LPT2,1005)#                                                       RI40 106
DO I=1,3#                                                               RI40 107
DO J=1,3#                                                               RI40 108
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI40 109
R(J,I)=Q(M,J,I)#                                                        RI40 110
RR(I,J)=Q(M,J,I)#                                                       RI40 111
RR1(I,J)=AR(J,I)#                                                       RI40 112
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI40 113
K2=NX(M)+K1-1#                                                          RI40 114
IF(M==1)K2=NN2#                                                         RI40 115
DO K=K1,K2#                                                             RI40 116
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI40 117
DO  I=1,3#                                                              RI40 118
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI40 119
IF(M==1)Z(I,K)=X(I,K)#                                                  RI40 120
Z(I,K)=Z(I,K)-O(I,M)#                                                   RI40 121
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI40 122
CALL XMATV(R,Z,XX,K)#                                                   RI40 123
CALL XMATV(L,XX,Y,K)#                                                   RI40 124
CALL XMATV(RR,Y,Z,K)#                                                   RI40 125
DO I=1,3 ;Z(I,K)=Z(I,K)+O(I,M)#                                         RI40 126
CALL XMATV(RR1,Z,P,K)#                                                  RI40 127
CALL XMATV(TTO,P,Y,K)#                                                  RI40 128
CALL XMATV(RR1,X,P,K)#                                                  RI40 129
CALL XMATV(TTO,P,XX,K)#                                                 RI40 130
DO I=1,3#                                                               RI40 131
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI40 132
XX(I,K)=XX(I,K)+OR(I)#                                                  RI40 133
Y(I,K)=Y(I,K)+OR(I)#                                                    RI40 134
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI40 135
WRITE(LPT2,1020)AT(K),(XX(I,K),I=1,3),(Y(I,K),I=1,3)#                   RI40 136
WRITE(7,1021)AT(K),(Y(I,K),I=1,3),(SIG(I,K),I=1,3)#                     RI40 137
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI40 138
K1=NX(M)+K1#                                                            RI40 139
DO I=1,3#                                                               RI40 140
DO J=1,3#                                                               RI40 141
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI40 142
L(I,J)=0.#                                                              RI40 143
L(I,I)=1.#                                                              RI40 144
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI40 145
IF(M!=1)CALL BONAN(L,Z,NN2,M,LPT,CDR)#                                  RI40 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI40 147
#                                                                       RI40 148
RETURN#                                                                 RI40 149
END#                                                                    RI40 150
systemheader:(RI42)#                                                    RI42   1
# ################################################################## #  RI42   2
#                                                                    #  RI42   3
#                               RI42                                 #  RI42   4
#                                                                    #  RI42   5
# ################################################################## #  RI42   6
SUBROUTINE XMATV(A,X,Y,K)#                                              RI42   7
dp: A(3,3),X(3,42),Y(3,42)#            Ri42                             RI42   8
#                                                                       RI42   9
DO  I=1,3#                                                              RI42  10
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI42  11
Y(I,K)=0.#                                                              RI42  12
DO  J=1,3;Y(I,K)=Y(I,K)+A(I,J)*X(J,K)#                                  RI42  13
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI42  14
RETURN#                                                                 RI42  15
END#                                                                    RI42  16
systemheader:(RI43)#                                                    RI43   1
# ################################################################## #  RI43   2
#                                                                    #  RI43   3
#                               RI43                                 #  RI43   4
#                                                                    #  RI43   5
#        PROGRAM TO DETERMINE CORRECTIONS TO BONDS AND ANGLES DUE TO    RI43   6
#        LIBRATIONS DETERMINED FROM RIGID BODY CONSIDERATIONS           RI43   7
# ################################################################## #  RI43   8
SUBROUTINE BONAN(L,X,NN,MX,LPT,CDR)#                                    RI43   9
#                                                                       RI43  10
dp: L(3,3),R(3,3),X(3,42),V(3),U(3),VR(3),UR(3)#                        RI43  11
dp: D#                                                                  RI43  12
dp: D1,D2,ANG#                                                          RI43  13
INTEGER Y,MX#                                                           RI43  14
INTEGER NN,LPT,CDR#                                                     RI43  15
#                                                                       RI43  16
11 FORMAT(/,' CORRECTIONS TO BOND LENGTHS AND ANGLES DUE TO RIGID',$#   RI43  17
' BODY MOTION OF MAJOR SEGMENT OF MOLECULE',//)#                        RI43  18
12 FORMAT(/,' CORRECTIONS FOR LIBRATION DUE TO ACCUMULATED MOTION ',$#  RI43  19
'OF',I3,' SEGMENTS OF MOLECULE',//)#                                    RI43  20
21 FORMAT(5X,'ATOM  UNCORRECTED CARTESIAN COORDS.')#                    RI43  21
22 FORMAT(5X,'ATOM  CORRECTED CARTESIAN COORDS.')#                      RI43  22
25   FORMAT(5X,I3,2X,3F10.5)#                                           RI43  23
30   FORMAT(10X,3I5)#                                                   RI43  24
40   FORMAT(3X,'CORRECTED DISTANCE BETWEEN ATOM',I3,' AND',$#           RI43  25
I3,' =',F10.7,' ANGSTROMS')#                                            RI43  26
50 FORMAT(5X,'CORRECTED ANGLE FOR ATOM CONFIG ',3I5,' =',F8.3,' DEG')#  RI43  27
#                                                                       RI43  28
IF(MX==1)#                                                              RI43  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI43  30
WRITE(LPT,11)#                                                          RI43  31
WRITE(LPT,21)#                                                          RI43  32
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI43  33
ELSE#                                                                   RI43  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI43  35
WRITE(LPT,12)MX#                                                        RI43  36
WRITE(LPT,22)#                                                          RI43  37
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI43  38
DO I=1,NN; WRITE(LPT,25)I,(X(J,I),J=1,3)#                               RI43  39
WRITE(LPT,*)" "#                                                        RI43  40
#                                                                       RI43  41
#      READ CARDS 15         BOND AND ANGLE DESCRIPTORS#                RI43  42
#                                                                       RI43  43
REPEAT#                                                                 RI43  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI43  45
READ(CDR,30)Y,M,N#                                                      RI43  46
IF(Y==0) BREAK#                                                         RI43  47
DO I=1,3; V(I)=X(I,Y)-X(I,M)#                                           RI43  48
#                                                                       RI43  49
DO I=1,3#                                                               RI43  50
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI43  51
VR(I)=0.#                                                               RI43  52
DO J=1,3; VR(I)=VR(I)+L(I,J)*V(J)#                                      RI43  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI43  54
D=0.#                                                                   RI43  55
DO I=1,3 ; D=D+VR(I)*VR(I)#                                             RI43  56
D=dsqrt:(dabs:(D))#                                                     RI43  57
#                                                                       RI43  58
IF(N==0) WRITE(LPT,40)Y,M,D#           Bond length calculation          RI43  59
ELSE#                                  Bond angle calculation           RI43  60
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI43  61
DO I=1,3;U(I)=X(I,N)-X(I,M)#                                            RI43  62
DO I=1,3#                                                               RI43  63
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI43  64
UR(I)=0.#                                                               RI43  65
DO J=1,3; UR(I)=UR(I)+L(I,J)*U(J)#                                      RI43  66
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI43  67
D1=0.#                                                                  RI43  68
D2=0.#                                                                  RI43  69
DO I=1,3#                                                               RI43  70
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI43  71
D1=D1+UR(I)*UR(I)#                                                      RI43  72
D2=D2+VR(I)*UR(I)#                                                      RI43  73
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI43  74
ANG=D2/(D*dsqrt:(dabs:(D1)))#                                           RI43  75
IF (dabs:(ANG)>1.0) ANG=0.99999999#                                     RI43  76
ANG=dacos:(ANG)*180./3.141529#                                          RI43  77
WRITE(LPT,50)Y,M,N,ANG#                                                 RI43  78
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI43  79
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI43  80
#                                                                       RI43  81
RETURN#                                                                 RI43  82
END#                                                                    RI43  83
#                                                                       RI43  84
systemheader:(RI50)#                                                    RI50   1
# ################################################################## #  RI50   2
#                                                                    #  RI50   3
#                               RI50                                 #  RI50   4
#                                                                    #  RI50   5
#      THIS PART CALCULATES NEW B(IJ) VALUES WHEN HYDROGEN RIGID        RI50   6
#      BODIES ARE ADDED TO THE SYSTEM                                   RI50   7
#                                                                       RI50   8
# ################################################################## #  RI50   9
SUBROUTINE SEG3#                                                        RI50  10
include:(AACOMN)#                                                       RI50  11
include:(RICOMN)#                                                       RI50  12
#                                                                       RI50  13
INTEGER SN,SM,RS,B(50)#                                                 RI50  14
INTEGER Q1,Q2,Q3,Q4,Q5,Q6,Q7,Q8#                                        RI50  15
dp: OR(3)#                             R                                RI50  16
#                                                                       RI50  17
dp: R(3,3),RR(3,3),P(3,3),TT(126,60)#  Ri50                             RI50  18
EQUIVALENCE (A(1,1),TT(1,1))#                                           RI50  19
##                                                                      RI50  20
#   NH IS NO. OF HYDROGEN ATOMS ADDED TO THE SYSTEM.#                   RI50  21
#   NHV IS NO. OF PARAMETERS ADDED TO THE TSL MATRIX.#                  RI50  22
#   READ CARD 16          NO. OF H-ATOMS AND NEW TSL PARAMETERS#        RI50  23
#                                                                       RI50  24
OR(1)=X1 ;OR(2)=Y1 ; OR(3)=Z1#                                          RI50  25
READ(CDR,1000)NH,NHV#                                                   RI50  26
1000 FORMAT(10X,2I5)#                                                   RI50  27
IF(NH==0)#                                                              RI50  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  RI50  29
FIN=yes:#                                                               RI50  30
GO TO 3000#                                                             RI50  31
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  RI50  32
NN1=NATOM#                                                              RI50  33
NATOM=NN2#                                                              RI50  34
WRITE(LPT,1002)NH,NHV#                                                  RI50  35
1002 FORMAT(/,'  INSERTION OF',I4,' HYDROGEN RIGID BODIES INTO',$# R    RI50  36
' THE SYSTEM',//, ' WITH ',I5,' NEW ELEMENTS IN TSL SYSTEM',///,$#      RI50  37
10X,'ATOM',6X,'FRACTIONAL COORDS',14X,'MOLECULAR COORDS',/)#            RI50  38
Q1=NJOINT+NH#                                                           RI50  39
Q2=NATOM+1#                                                             RI50  40
Q3=NATOM+NH#                                                            RI50  41
DO K=Q2,Q3#                                                             RI50  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  RI50  43
#   READ CARDS 17#                                                      RI50  44
READ(CDR,1001)AT(K),(X(I,K),I=1,3)#                                     RI50  45
1001 FORMAT(10X,A5,3F10.5)#                                             RI50  46
DO I=1,3; X(I,K)=X(I,K)-OR(I)#                                          RI50  47
DO I=1,3#                                                               RI50  48
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  RI50  49
P(I,1)=0.#                                                              RI50  50
DO J=1,3; P(I,1)=P(I,1)+TO(I,J)*X(J,K)#                                 RI50  51
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  RI50  52
DO I=1,3#                                                               RI50  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  RI50  54
P(I,2)=0.#                                                              RI50  55
DO J=1,3; P(I,2)=P(I,2)+AR(I,J)*P(J,1)#                                 RI50  56
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  RI50  57
WRITE(LPT,1003)AT(K),(X(I,K),I=1,3),(P(J,2),J=1,3)#                     RI50  58
1003 FORMAT(10X,A5,3F10.5,',',3F10.5)#                                  RI50  59
DO I=1,3#                                                               RI50  60
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  RI50  61
XO(I,K+NN1-NN2)=P(I,2)#                                                 RI50  62
X(I,K)=P(I,2)#                                                          RI50  63
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  RI50  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  RI50  65
##                                                                      RI50  66
#        INSERTS A MOLECULAR AXIAL SYSTEM OF EACH HYDROGEN#             RI50  67
##                                                                      RI50  68
Q2=NJOINT+1#                                                            RI50  69
DO K=Q2,Q1#                                                             RI50  70
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  RI50  71
#      READ CARDS 18#                                                   RI50  72
READ(CDR,1013)L1,L2,L3,M#                                               RI50  73
1013 FORMAT(10X,4I5)#                                                   RI50  74
DO J=1,3#                                                               RI50  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  RI50  76
P(1,J)=XO(J,L1)#                                                        RI50  77
P(2,J)=XO(J,L2)#                                                        RI50  78
P(3,J)=XO(J,L3)#                                                        RI50  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  RI50  80
CALL MATROT(P,R)#                                                       RI50  81
DO I=1,3#                                                               RI50  82
DO J=1,3; Q(K,I,J)=R(I,J)#                                              RI50  83
WRITE (LPT ,1014) K,((Q(K,I,J),J=1,3),I=1,3)#                           RI50  84
1014 FORMAT('  ROTATION MATRIX FOR JOINT ',I2,/3(/3F10.5))#             RI50  85
DO I=1,3; O(I,K)=XO(I,M)#                                               RI50  86
WRITE (LPT,1016)(O(I,K),I=1,3)#                                         RI50  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  RI50  88
1016 FORMAT(/,' ORIGIN AT ',3F10.5,///)#                                RI50  89
Q2=NN1+NH#                                                              RI50  90
DO I=1,3#                                                               RI50  91
DO J=1,Q2#                                                              RI50  92
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  RI50  93
AFF=X(I,J)#                                                             RI50  94
BFF=XO(I,J)#                                                            RI50  95
X(I,J)=BFF#                                                             RI50  96
XO(I,J)=AFF#                                                            RI50  97
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  RI50  98
##                                                                      RI50  99
#        ENTERS LOGIC ELEMENTS OF NEW TRANSFORMATION MATRIX#            RI50 100
##                                                                      RI50 101
Q2=(NATOM+NH)*3#                                                        RI50 102
Q3=Q1*6#                                                                RI50 103
DO L=1,Q2#                                                              RI50 104
DO M=1,Q3; TT(L,M)=0.#                                                  RI50 105
Q2=NJOINT+1#                                                            RI50 106
Q3=NJOINT+NH#                                                           RI50 107
DO L=Q2,Q3#                                                             RI50 108
DO M=1,NSEG; T1(M,L)=0#                                                 RI50 109
#                                                                       RI50 110
#      READ CARDS 19#                                                   RI50 111
#                                                                       RI50 112
DO L=1,NH; READ(CDR,1019)(T1(NSEG+L,M),M=1,Q1)#                         RI50 113
1019 FORMAT(10X,20I3)#                                                  RI50 114
WRITE (LPT,2001)#                                                       RI50 115
2001 FORMAT(' BLOCK ELEMENTS OF TRANS. MATRIX')#                        RI50 116
Q2=NSEG+NH#                                                             RI50 117
DO L=1,Q2; WRITE(LPT,2002)(T1(L,M) ,M=1,Q1)#                            RI50 118
#                                                                       RI50 119
2002 FORMAT(5X,25I3,////)#                                              RI50 120
SN=0.#                                                                  RI50 121
DO L=1,Q2#                                                              RI50 122
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  RI50 123
DO M=1,Q1#                                                              RI50 124
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 RI50 125
IF(T1(L,M)==0) NEXT#                                                    RI50 126
SM=T1(L,M)#                                                             RI50 127
Q3=SN+1#                                                                RI50 128
Q4=SN+SM#                                                               RI50 129
DO N=Q3,Q4#                                                             RI50 130
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 RI50 131
DO I=1,3#                                                               RI50 132
DO J=1,3; TT((N-1)*3+I,6*(M-1)+J)=Q(M,I,J)#                             RI50 133
#                                                                       RI50 134
DX=XO(1,N)-O(1,M)#                                                      RI50 135
DY=XO(2,N)-O(2,M)#                                                      RI50 136
DZ=XO(3,N)-O(3,M)#                                                      RI50 137
R(1,1)=0.#                                                              RI50 138
R(1,2)=-DZ#                                                             RI50 139
R(1,3)= DY#                                                             RI50 140
R(2,1)= DZ#                                                             RI50 141
R(2,2)=0.#                                                              RI50 142
R(2,3)=-DX#                                                             RI50 143
R(3,1)=-DY#                                                             RI50 144
R(3,2)= DX#                                                             RI50 145
R(3,3)= 0.#                                                             RI50 146
DO I=1,3#                                                               RI50 147
DO J=1,3#                                                               RI50 148
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 RI50 149
SS=0.#                                                                  RI50 150
DO IM=1,3#                                                              RI50 151
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 RI50 152
RR(I,J)=R(I,IM)*Q(M,IM,J)+SS#                                           RI50 153
SS=RR(I,J)#                                                             RI50 154
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 RI50 155
TT((N-1)*3+I,6*(M-1)+J+3)=RR(I,J)#                                      RI50 156
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 RI50 157
#                                                                       RI50 158
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 RI50 159
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 RI50 160
SN=SN+SM#                                                               RI50 161
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  RI50 162
IF(CONTRL(3)==0.)GO TO 667#                                             RI50 163
Q5=3*(NATOM+NH)#                                                        RI50 164
Q7=6*(NJOINT+NH)#                                                       RI50 165
DO I=1,Q5; WRITE(LPT,10)(TT(I,J),J=1,Q7)#                               RI50 166
#                                                                       RI50 167
667  CONTINUE#                                                          RI50 168
WRITE(LPT,2004)#                                                        RI50 169
2004 FORMAT(/)#                                                         RI50 170
##                                                                      RI50 171
#        READS IN AND SETS UP NEW TSL MATRIX ELEMENTS#                  RI50 172
##                                                                      RI50 173
IK=21*NJOINT#                                                           RI50 174
IO=21*NJOINT+1#                                                         RI50 175
Q3=NJOINT+1#                                                            RI50 176
DO MA=Q3,Q1#                                                            RI50 177
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 RI50 178
#      READ CARDS 20#                                                   RI50 179
READ(CDR,555)(TSL1(IK+JJ),JJ=1,21)#                                     RI50 180
555  FORMAT(10X,21I3)#                                                  RI50 181
WRITE(LPT,3) MA,(TSL1(IK+JJ),JJ=1,21)#                                  RI50 182
3 FORMAT(/,' SEGMENT ',I2,' CONTRIBUTION TO THE TSL MATRIX',/,21I4)#    RI50 183
IK=IK+21#                                                               RI50 184
IA=6*(MA-1)#                                                            RI50 185
JA=6*(MA-1)#                                                            RI50 186
Q4=IA+1#                                                                RI50 187
Q5=IA+3#                                                                RI50 188
Q7=JA+3#                                                                RI50 189
DO I=Q4,Q5#                                                             RI50 190
DO J=I,Q7#                                                              RI50 191
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 RI50 192
TSL2(IO)=I*1000+J#                                                      RI50 193
IO=IO+1#                                                                RI50 194
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 RI50 195
#                                                                       RI50 196
JA=JA+3#                                                                RI50 197
Q6=JA+1#                                                                RI50 198
Q7=JA+3#                                                                RI50 199
DO I=Q4,Q5#                                                             RI50 200
DO J=Q6,Q7#                                                             RI50 201
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 RI50 202
TSL2(IO)=I*1000+J#                                                      RI50 203
IO=IO+1#                                                                RI50 204
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 RI50 205
#                                                                       RI50 206
IA=3+IA#                                                                RI50 207
Q4=IA+1#                                                                RI50 208
Q5=IA+3#                                                                RI50 209
DO I=Q4,Q5#                                                             RI50 210
DO J=I,Q7#                                                              RI50 211
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 RI50 212
TSL2(IO)=I*1000+J#                                                      RI50 213
IO=IO+1#                                                                RI50 214
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 RI50 215
#                                                                       RI50 216
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 RI50 217
Q4=NJOINT+1#                                                            RI50 218
DO  I=1 ,Q1#                                                            RI50 219
DO J1=1,21#                                                             RI50 220
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 RI50 221
J=J1+(I-1)*21#                                                          RI50 222
IF(TSL1(J)==0)  NEXT#                                                   RI50 223
B(TSL1(J))=TSL2(J)#                                                     RI50 224
IF(CONTRL(3)==0.) NEXT#                                                 RI50 225
WRITE(LPT,4) TSL1(J),TSL2(J)#                                           RI50 226
4    FORMAT('  B(',I2,') = TSL ',I6,/)#                                 RI50 227
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 RI50 228
##                                                                      RI50 229
#        SETS UP MATRIX OF CONSRTAINTS ON THE TSL ELEMENTS#             RI50 230
##                                                                      RI50 231
Q3=NH+NATOM#                                                            RI50 232
DO IM=1,Q3#                                                             RI50 233
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 RI50 234
MM=3*(IM-1)#                                                            RI50 235
Q4=MM+1#                                                                RI50 236
Q5=MM+3#                                                                RI50 237
DO IA=Q4,Q5#                                                            RI50 238
DO JA=IA,Q5#                                                            RI50 239
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 RI50 240
RS=RS+1#                                                                RI50 241
Q8=NVAR+NHV#                                                            RI50 242
DO K=1,Q8#                                                              RI50 243
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 RI50 244
M=B(K)/1000#                                                            RI50 245
N=B(K)-M*1000#                                                          RI50 246
IF(N!=M)#                                                               RI50 247
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 RI50 248
AN(RS,K)=TT(IA,M)*TT(JA,N)+TT(IA,N)*TT(JA,M)#                           RI50 249
BREAK#                                                                  RI50 250
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 RI50 251
AN(RS,K)=TT(IA,M)*TT(JA,M)#                                             RI50 252
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 RI50 253
IF(CONTRL(3)==0.) NEXT#                                                 RI50 254
WRITE(LPT,10)(AN(RS,II),II=1,Q8)#                                       RI50 255
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 RI50 256
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 RI50 257
IF(ID!=9) GO TO 99#                                                     RI50 258
NVAR=NVAR+1#                                                            RI50 259
I=NVAR+1#                                                               RI50 260
98   I=I-1#                                                             RI50 261
VAR(I)=VAR(I-1)#                                                        RI50 262
IF(I>8) GO TO 98#                                                       RI50 263
VAR(7)=-(VAR(11)+VAR(15))#                                              RI50 264
99   CONTINUE#                                                          RI50 265
10   FORMAT(12F9.5)#                                                    RI50 266
#      READ CARDS 201#                                                  RI50 267
READ(CDR,1007)(VAR(NVAR+I),I=1,NHV)#                                    RI50 268
1007 FORMAT(10X,6F10.5)#                                                RI50 269
WRITE(LPT,1021)(VAR(NVAR+I),I=1,NHV)#                                   RI50 270
1021 FORMAT(//,'  HYDROGEN TSL ELEMENTS INSERTED ',/60(/10F10.5))#      RI50 271
Q4=NVAR+NHV#                                                            RI50 272
Q5=6*Q3#                                                                RI50 273
DO I=1,Q5#                                                              RI50 274
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 RI50 275
V(I)=0.#                                                                RI50 276
DO J=1,Q4; V(I)=V(I)+AN(I,J)*VAR(J)#                                    RI50 277
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 RI50 278
##                                                                      RI50 279
#        OUTPUTS NEW THERMAL PARAMETERS#                                RI50 280
##                                                                      RI50 281
WRITE(LPT,1006)#                                                        RI50 282
DO I=1,Q3; WRITE(LPT,1005)AT(I),(V((I-1)*6+J),J=1,6)#                   RI50 283
1005 FORMAT(10X,A5,6F10.5)#                                             RI50 284
1006 FORMAT(5X,' NEW UIJ S FOR ATOMS USING HYDROGEN RIGID BODIES',$#    RI50 285
//,10X,'ATOM',4X,'U11',7X,'U12',7X,'U13',7X,'U22',7X,'U23',7X,$#        RI50 286
'U33',/)#                                                               RI50 287
WRITE(LPT,1008)#                                                        RI50 288
DO I=1,Q3;  WRITE(LPT,1005)AT(I),(V((I-1)*6+J),J=1,6)#                  RI50 289
1008 FORMAT(/,5X,' NEW BIJ S FOR SYSTEM',//10X,'ATOM',4X,$# Ri          RI50 290
'B11',7X,'B12',7X,'B13',7X,'B22',7X,'B23',7X,'B33',/)#                  RI50 291
3000 CONTINUE#                                                          RI50 292
RETURN#                                                                 RI50 293
END#                                                                    RI50 294
#                                                                       RI50 295
pmacro:([no:])#                        Finish finish finish finish fi   RI50 296
