RIETVD: Structure Refinement from Powder Diffraction Patterns
-------------------------------------------------------------
Tentative Description   (Version 7 Oct 1995)
---------------------

Authors:
-------
S R Hall
Ch Baerlocher
G J Kruger
D Billing

RIETVD is a program for the the refinement of crystal structure parameters from
powder diffraction data. It performs Rietveld refinement on X-ray or neutron
powder diffraction data collected on a diffractometer in step scan mode
It adjusts three types of parameters:
 those specific to the pattern (background, absorption, zeropoint offset)
 those specific to the crystal (cell dimensions, peak shape parameters)
 those specific to the structure (scale and atom parameters)
More than one crystalline phase can be treated simultaneously.
It can also be used for simulating powder patterns by calculation only.
RIETVD is based on the single crystal least-squares program CRYLSQ and employs
the same control lines for many of its options (such as NOREF and CONSTR for 
non-refinement of constraining of parameters).

Calculations performed:
----------------------

The intensity at a step in the powder pattern is calculated as the sum of the 
background and the contributions of all reflections (from all k phases present) 
close enough to add to it.

     Yc = Yb + sum(k) [ S(k) * Lp(k) * m(k) * P(k) * PHI(k) * (Fhkl(k)**2) ]

Yb = background intensity which can be:
     (a) a predetermined value stored with the pattern
     (b) a value calculated from the refinable parameters b(j) in
         Yb = sum(j) [ b(j) * (2theta)**(j-1) ]

S(k) = scale factor for phase k
   = s(k) * ZMV(k)  - according to Hill and Howard
   i.e. already scaled to give the weight fraction of phase k

Lp(k) = Lorentz and polarization factors

m(k) = reflection multiplicity factor

P(k) = preferred orientation (March-Dollase) function

PHI(k) = peak shape function for the calculation of the intensity contributed
     at the step angle 2Tstep by a reflection at the diffraction angle 2Thkl
     i.e. PHI is a function of 2Tdelta = (2Tstep - 2Thkl)
     Two functions are available namely:
     (a) the pseudo-Voigt function mixing Gaussian and Lorentzian parts:
         PHI = xG + (1-x)L
     (b) an experimental function (learned profile) determined during the
         pattern processing in ADDPAT.

In the least squares procedure the function Q is minimized:

     Q = sum(i) [ w(i) * (Yobs(i) - Ycalc(i)**2 ]

where 
     w(i) = weight at each step = 1/Yobs(i)
     and the summation is over all i steps in the pattern


FILE ASSIGNMENTS

Reads atom site and hkl data from the input archive file(s).
Writes atom site data and hkl data to output archive file(s).
Reads measured powder pattern intensities from file PAT.
Writes calculated powder pattern intensities to file POT.
Writes powder pattern plot data to file PPP.
Writes card images to punch file PCH.


CONTROL LINES

LIMITS set powder pattern limits and step size
WAVLEN set wavelength and polarization ratio
ZERO   set zero point corrections
BACKGR specify parameters for background function
PEAKF  define a learned (experimental) peak shape function
PEAKSH enter experimental peak shape values
EXCLTT exclude zones of the powder pattern
NOREF  force zero and background parameters invariant

PHASE  specify id and scale factor of a new phase
PEAK   select peak shape function for this phase
HWIDTH specify halfwidth parameters
PASYMM specify asymmetry parameters
ORIENT specify preferred orientation parameters
SELECT select subset of atoms in bdf
CONSTR constrain a parameter in terms of others
NOREF  force specified parameters invariant


RIETVD   Option                    Code   Action                        Default

         refinement cycles         CY n   refine n cycles                  <n=1>
         calculate pattern only    CP     calculate do not refine            <no>
         2theta zero corrections   RZ     refine zero corrections           <no>
         background parameters     RB     refine background parameters      <no>
         scale factors             NS     do not refine scale factors      <yes>
         cell parameters           RC     refine cell parameters            <no>
         preferred orientation     RP     refine preferred orientation      <no>
         peak profile parameters   RH     refine peak profile parameters    <no>
         overall thermal parameter RU     refine uov's                      <no>
         displacement parameters   NT     do not refine individual tdp's   <yes>
         population occupancies    RO     refine occupancies                <no>
         select step weights type  WT n   unit/file/sigma^-2          <0; n=1>
         peak span in halfwidths   PS n   reflection contribution width <3;n=12>
         list hkl data last cycle  LI n   list n reflections         <no; n=all>
                                   LR q   list hkl if Fdif >q        <no; n=10.>
         list correlation matrix   LC q   list element if correl. >q <no; n=0.6>
         list matrix               LM     list matrix after last cycle      <no>
         list each step            LS     list steps after last cycle       <no>
         convergence termination   TC     terminate if convergence reached  <no>
         dispersion treatment      ND     exclude dispersion           <include>
         skip obs step data        SK n   use only every nth pattern step  <n=1>
         site refinement damping   SD q   damp atom parameters        <1.; q=.5>
         profile data damping      PD q   damp profile & cell params  <1.; q=.5>
         contribute to ls matrix   MR q   include hkl if Fdif <q    <all; n=100>
         punch updated parameters  PC     punch cards after last cycle      <no>


        PATTERN SPECIFIC CONTROL LINES
        ------------------------------
limits
       1   minimum two theta of pattern data processed
       2   maximum two theta of pattern data processed
       3   two theta step size of pattern data processed

wavlen 1 Wavelength alpha1                     <1.540598>
       2 wavelength alpha2                     <0.0>
       3 alpha2/alpha1 intensity ratio         <0.0>
       4 polarisation ratio                    <.915>
       5 radiation type (x, n, s or e)         <xray>
       6 fixed divergence slit (0/1=fix/var)   <fixed>
       7 data lp corrected (0/1=no/yes)        <no>

zero
       1-2  two theta zero correction factors z1 and z2
            in the correction expression 
            2theta(corrected) = 2theta(measured) + z1 + z2*2theta

backgr
       1-5 polynomial coefficients b(1) to b(5) to describe background
           as the expression ybg = b1 + b2*2t + b3*(2t)**2 + .....
           i.e. ybg = sum(j) [ b(j) * (2theta)**(j-1) ]
           (values included in pattern file will be ignored)

peakf
       1   identification number of learned peak shape function to follow  <1>

peaksh
       1   ri     (values for the learned peak shape function) 
       2   fsi
       3   fai
       4   dfsi
       5   dasi

excltt
       1   minimum two theta value to be excluded from refinement: zone 1
       2   maximum two theta value to be excluded from refinement: zone 1
       3-4 min/max two theta for exclusion zone 2
       5-  min/max pairs for up to 10 exclusion zones

noref
       1-  do not refine background or zero parameters entered in the format
           
                 param1 param2 .....

           where paramN can be a sequence of parameters from the list:
                
                 b1 b2 b3 b4 b5 z1 z2


        PHASE SPECIFIC CONTROL LINES (one set per phase)
        ----------------------------

phase
       1   compound ID of phase (to identify archive files)  <current compid>
       2   scale factor for this phase                       <1.0>

peak
       1   one of the following to define the peak shape
            v for pseudo-Voigt (x = variable)
            g for Gaussian     (x=1)
            l for Lorentzian   (x=0)
            n=number of learned peak shape function (defined with peakf) <v>
       2   peak span value (in halfwidths) for this shape function       <12.>
       3   mixing parameter z in the pseudo-Voigt function:              <1.0>
            f = z * gaussian part + (1-z) * lorentzian part

hwidth
       1-3 halfwidth parameters u,v,w used in the expression 
           fwhm = sqrt(u*(tan(theta))**2 + v*tan(theta) + w)

pasymm
       1-2 peak asymmetry correction parameters x and y
           used in the expression peak asymm = x + y/tan(theta)

orient
       1   preferred orientation factor
       2-4 preferred orientation vector (U,V,W)
  
select
       1-  atom label *or* atom type symbol *or* 
           atom sequence (labels separated by a '/')

constr
       1   single constraint equation of the format 

           param1(atomlab1)=const+mult*param2(atomlab2)

           where the const or mult* terms may be omitted.
           
           param1 is the constrained atomic parameter and param2 is the refined
           atomic parameter which are designated by the symbolic names 
           X Y Z U U11 U12 U13 U22 U23 U33 POP (see noref description)

           atomlab1 and atomlab2 are the atom labels or symbols or sequences
           separated by '/'.
         
noref
       1-  argument(s) to fix (do not refine) structural or profile parameters.

           The formats for the arguments are:

           g1 g2 .....          for general parameters
           (p1/p2)(a1/a2) ...   for atomic parameters
           cell c1 c2 ....      for cell parameters
           prof p1 p2 ....      for peak profile parameters

           Individual parameter mnemonics are:

           General parameters:
              SKF    scale factor for this phase
              UOV    overall thermal factor for this phase
              POF    preferred orientation correction factor
           Atomic parameters:
              X Y Z                      coordinates
              U U11 U12 U13 U22 U23 U33  displacement params
              POP                        population (occupancy)
           Profile parameters:
              U V W  halfwidth paramaters
              X Y    peak asymmetry parameters
              MIX    mixing parameter in pseudo-Voigt peak function
           Cell parameters:
              A B C    cell edges
              AL BE GA cell angles

           In the case of prof and cell no param symbol implies all parameters.
           Cell and prof lines should not contain other parameters.
           Refinement of non-positional parameters are switched on or off 
           by the switches on the RIETVD line.

 examples:
 noref skf uov
 noref x(c1/c5) pop(SI)  :fix x for C1 to C5; all SI occpancy
 noref u(H) (u11/u33)(C) :fix all tdf's for H (isotropic) and C (aniso)
 noref pof SI            :fix preferred orientation; all SI parameters
 noref cell al be ga     :fix cell angles
 noref prof v w y mix    :fix halfwidth v w; asymmetry y; mixing param

INPUT EXAMPLES
--------------

RIETVD cy 3 sk 2 
limits 10 60
wavlen  mo
backgr
finish


RIETVD CY 5 NT RC LS PC LI 20
limits 12 22
peakf    1    15.00
setid peaksh
              0.0000+00  0.3872+00  0.0000+00  0.0000+00  0.1237+01
              0.2800+00  0.3540+00  0.3445+00 -0.2021+00  0.1078+01
              0.5600+00  0.2978+00  0.6473+00 -0.2449+00  0.1004+01
              0.8400+00  0.2299+00  0.9225+00 -0.2221+00  0.3801+00
              0.1120+01  0.1681+00  0.1025+01 -0.1645+00 -0.6435+00
              0.1400+01  0.1220+00  0.8644+00 -0.1146+00  0.1825+00
              0.1680+01  0.8966-01  0.9074+00 -0.8591-01  0.2567+00
              0.1960+01  0.6553-01  0.9617+00 -0.6388-01 -0.1298+01
              0.2240+01  0.4771-01  0.6284+00 -0.3644-01 -0.5002-01
              0.2520+01  0.3733-01  0.6080+00 -0.2504-01  0.6074+00
              0.2800+01  0.3023-01  0.7742+00 -0.2165-01  0.9040+00
              0.3080+01  0.2416-01  0.1019+01 -0.1886-01  0.4370+00
              0.3360+01  0.1891-01  0.1130+01 -0.1389-01 -0.1080+01
              0.3640+01  0.1501-01  0.8381+00 -0.9331-02 -0.1643+01
              0.3920+01  0.1238-01  0.3899+00 -0.7159-02 -0.1088+01
              0.4200+01  0.1036-01  0.8595-01 -0.5744-02 -0.4756+00
              0.4480+01  0.8751-02 -0.4640-01 -0.4640-02  0.2116+00
              0.4760+01  0.7446-02  0.8370-02 -0.4141-02  0.4828+00
              0.5040+01  0.6288-02  0.1402+00 -0.3513-02  0.4431+00
              0.5320+01  0.5303-02  0.2621+00 -0.2944-02  0.2029+00
              0.5600+01  0.4479-02  0.3197+00 -0.2362-02  0.3210-01
              0.5880+01  0.3817-02  0.3295+00 -0.1858-02 -0.6470-01
              0.6160+01  0.3295-02  0.3118+00 -0.1485-02 -0.1222+00
              0.6440+01  0.2878-02  0.2779+00 -0.1229-02 -0.1470+00
              0.6720+01  0.2533-02  0.2370+00 -0.1025-02 -0.1502+00
              0.7000+01  0.2246-02  0.1951+00 -0.8608-03 -0.1403+00
              0.7280+01  0.2005-02  0.1559+00 -0.7168-03 -0.1190+00
              0.7560+01  0.1803-02  0.1226+00 -0.6160-03 -0.9912-01
              0.7840+01  0.1631-02  0.9481-01 -0.5330-03 -0.8026-01
              0.8120+01  0.1481-02  0.7232-01 -0.4649-03 -0.6311-01
              0.8400+01  0.1351-02  0.5463-01 -0.4025-03 -0.4756-01
              0.8680+01  0.1238-02  0.4127-01 -0.3530-03 -0.3587-01
              0.8960+01  0.1139-02  0.3119-01 -0.3135-03 -0.2727-01
              0.9240+01  0.1051-02  0.2354-01 -0.2796-03 -0.2069-01
              0.9520+01  0.9731-03  0.1773-01 -0.2492-03 -0.1542-01
              0.9800+01  0.9033-03  0.1340-01 -0.2218-03 -0.1131-01
              0.1008+02  0.8411-03  0.1022-01 -0.2008-03 -0.8605-02
              0.1036+02  0.7849-03  0.7800-02 -0.1817-03 -0.6516-02
              0.1064+02  0.7340-03  0.5970-02 -0.1645-03 -0.4935-02
              0.1092+02  0.6880-03  0.4585-02 -0.1481-03 -0.3647-02
              0.1120+02  0.6464-03  0.3559-02 -0.1354-03 -0.2806-02
              0.1148+02  0.6085-03  0.2771-02 -0.1240-03 -0.2168-02
              0.1176+02  0.5738-03  0.2163-02 -0.1139-03 -0.1673-02
              0.1204+02  0.5419-03  0.1693-02 -0.1041-03 -0.1274-02
              0.1232+02  0.5127-03  0.1335-02 -0.9570-04 -0.9864-03
              0.1260+02  0.4859-03  0.1058-02 -0.8860-04 -0.7779-03
              0.1288+02  0.4611-03  0.8398-03 -0.8214-04 -0.6129-03
              0.1316+02  0.4381-03  0.6679-03 -0.7610-04 -0.4786-03
              0.1344+02  0.4168-03  0.5336-03 -0.7005-04 -0.3689-03
              0.1372+02  0.3971-03  0.4298-03 -0.6806-04 -0.3050-03
              0.1400+02  0.3788-03  0.3470-03  0.0000+00  0.0000+00
setid 
hwidth  0.00688 0.00506 0.02428

RIETVD  cy 3 
limits 12 70
wavlen  1.54050 1.0
pasymm  0.8346 -0.001 
hwidth  0.00415 0.03880 0.0
noref U(Ba/O22)


-------------------------------------------------------------
date: 18-3-94
----
Description of Binary Data File records used in POWDER files:

LRPWAV: MUST STILL BE IMPLEMENTED!!

-------------------------------------------------------------

3  lrppha: (F) Phase count and Names

	packet size - 11 words

Seq	Description

+1	Number of Phases
+2	Name of Phase one
+3	Name of Phase two
..
..
+11     Name of Phase ten

4  lrpwav: (F) Powder Pattern Wavelength 

	packet size - 7 words

Seq	Description

+1	neutron/x-ray = 0/1
+2	wavelength 1 (e.g. K alpha 1)
+3	wavelength 2 (e.g. K alpha 2)
+4      Intensity ratio of second to first w/l
+5      Polarization ratio
+6      Lp corrected (0/1) = no/yes
+7      Divergence slit (fixed/auto)


5  lrplim: (F) Powder Pattern Limits and Stepsize

	packet size - 7 words

Seq	Description

+1	Two theta minimum (observed pattern)
+2	Two theta maximum (observed pattern)
+3	Two theta step size
+4      Minimum observed counts
+5      Maximum observed counts
+6      Minimum calculated counts
+7      Maximum calculated counts


6  lrppat: (F) Powder Pattern Data

	packet size - 7

IDN	Description

1	Two theta angle
2	Observed intensity (counts)
3	Sigma (std dev in total counts)
4       Weight (supplied by user in raw data file)
5	Calculated intensity
6       Background intensity
7       hkl/phase marker (refl position for one/more phases)
        (phase numbers coded in binary)
