/*  
 *  An example of a possible structured record which may be useful 
 *  to enable direct access to all data for new XTAL programs , but
 *  which preserves the integrity of the Binary archive file for old 
 *  programs, which may enable tools to be written to copy structured 
 *  record info into  a program supplied buffer in the format 
 *  existing 3.5 programs expect
 *
 */


/*      define equivalents of the RATMAC inbuilt preprocessor  macros */

#define YES   1
#define NO    0



/* The following is just a hypothetical multithreading construction 
   Idealy the flag should be a MUTEX (mutual exclusion) lock which is
   tested to see if separate concurrent threads are actually reading and/or 
   writing into this particular record 
 */

typedef struct {
      short  flag;          /* flag record as locked or unlocked */
      char   *program;      /* record locked by this program */
      char   *instance;     /* record locked by this instance of program */ 
} Lock;                     /* a record lock  data structure */






/* the following was an attempt to write a record structure for internal 
   representation of all XTAL BDF info information */


typedef  struct {
      short  length;        /* length  of directory record */
      short  *data;         /* pointer to directory record info */
} Directory ;

struct _lrhist{             /* File History    50 *24 chars */
      Lock   lock;          /* is record locked ? */
      short  lrec;          /* logical record number  1*/
      short  cnt;           /* rewrite counts      */
      char  *record[24][50];/* logical record number */
};

struct _lrlabl{             /* Label           n+3*80 chars */
      Lock   lock;          /* is logical record locked */
      short  lrec;          /* logical record number 2 */
      char   create[80];    /* creation time  */
      char   title[80];     /* Title in force at creation time  */
      short  cnt;           /* label  counts      */
struct _lrlabel *labelrec;    /* pointer to linked list of labels  */
};

struct _lrcell{     /* lrcell         n+3*80 chars */
      Lock   lock;          /* is logical record locked */
      float  abc[9];        /*  a,b,c,Cos Al,Cos Bet,Cos Ga,Al,Be,Ga */
      float  sabc[9];       /*  sigma a,b,c,Cos Al,Cos Bet,Cos Ga,Al,Be,Ga */
      float  rabc[9];       /*  Reciprocal cell constants a*,b*,c* ... */
      float  rollet[9];     /*  Rollet matrixi convertinf frac to Orthog Angs*/
      float  rrollet[9];    /*  Reciprocal space Rollet matrix */
      float  metric[9];     /*  metric tensor */
      float  imetric[9];    /*  Inverse metric tensor */
      float  pscale[12];    /*  protein scale matrix 3*4  */
      float  pftoo[12];     /*  protein Frac to orthog angstrom coords  */
};

typedef  struct _symop Symop;  /* linked list */
struct  _symop {            /* stucture for an xtal symop */
      short  rotop[3][4];   /* 12 element represent of seitz matrix */
      Symop  *next;         /* point to next symop */
};

struct _lrsymm{             /* 5 Symmetry Informtn  1+24*12 int symops */
      Lock   lock;          /* is logical record locked */
      short  code;          /* centring type  0 Centro / 1 Noncentro  */
      char   lattice;       /* LAttice type P I R F A B C  */
      short  centre;        /* centring type  0 Centro / 1 Noncentro  */
      short  equivp;        /* total number of symm equiv positions */
      short  numrot0;       /* number of rotation matrices identical 0s */
      short  multip;        /* cell multiplicity */
      short  nummol;        /* number of molecules in cell  */
      short  system;        /* crystal system tri=1 mon=2  ... cub=8  */
      short  uqaxis;        /* unique axis */
      short  numrot;        /* number of centre indep rotation matrices */
      Symop  rotops;        /* pointer to linked list of rotation ops */
      short  numced;        /* number of cedar rotation matrices  */
      Symop  cedops;        /* pointer to linked list of CEDAR symops */
};

struct _lrsgnm{     /* 6 Space Group name 2*40 chars */
      Lock   lock;          /* is logical record locked */
      char  hall[40];       /* Hall space group name  */
      char  mauguin[40];    /* Herman-Mauguin space group name  */
};


typedef struct  {            /* stucture for an xtal symop */
      short  rotop[3][3];   /* 12 element represent of seitz matrix */
} RotMatrix;

typedef struct _twinrec {           /* fixed length - variable packet numbers */
      RotMatrix rotmat;     /*  3 x 3 transformation matrix    */
      float  volume;        /* 0.0  ... 1.0  fractional volume */
      float  volumeSig;     /*  volume sigma                   */
      float  invvolume;     /* 0.0  ... 1.0  fractional volume invert related*/
      float  invvolumeSig;  /*  volume sigma  invert related   */
struct _twinrec *next;      /* pointer to linked list of labels  */
}            *TwinRec;

struct _lrtwin{             /* 7 Twin Laws           13 floats */
      Lock     lock;        /* is logical record locked        */
      TwinRec  data;        /* linked list of twin data        */
}; 


typedef struct _scatrec {   /* variable length - variable packet numbers */
      float      number;    /* numbe rof atoms of type */
      float      weight;    /* atomic weight  */
      float      ANumber;   /* atomic Number  */
      float      eNumber;   /* Neutral atom electron  Number  */
      float      bondRadius;
      float      contactRadius;
      short      interpFlag;
      float      spinQuantum;
      float      neutronScatLength;
      char       *symbol;
      float      Cromer[9];
      float      realDispersion[25];
      float      imagDispersion[25];
      char      *scatSource;
      float      massPercent; 
      float     *scatFactor;
struct _scatrec *next;      /* pointer to linked list of labels  */
}              *ScatRec;

struct _lrscat {            /* 8 Atom-type params */
      Lock     lock;        /* is logical record locked */
      ScatRec *data;        /* linked list of twin data        */
 
}; 


typedef struct _sequrec {   /* variable length - variable packet numbers */
      char    residue[9];   /* 8 character residue code */
struct _sequrec *next;
}             *SequRec;

struct _lrsequ {            /* 9  Protein Atom Sequence Information   */
      Lock     lock;        /* is logical record locked */
      SequRec *data;
}; 


typedef struct _ddefrec {   /* variable length - variable packet numbers */
      char    describe[13]; /* 8 character residue code */
struct  _ddefrec *next;
} *DdefRec;

struct _lrddef {            /* 10 data set names     n*12 chars  */
      Lock   lock;          /* is logical record locked */
      DdefRec *data;        /* pointer to linked list of data */
}; 


struct _lrexpl {            /* 11 experimental parameters var length var pack*/
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrdset {            /* 12 general data per packet var length var pack*/
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrdict {            /* 13 MacroMolecular Residue Information n*20 */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrgraf {            /* 14 Molecular graphing Information  var?? */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lratom {            /* 16 Atom parameters    var length 1/indep atom */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrcons {            /* 17 Refinement constraints v length 1/in atom*/
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrslav {            /* 18 Protein atom link information */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrmole {            /* 19 Molecular information */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 
struct _lrrefl {            /* 20 Reflection info var length  var packs */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 

struct _lrthet {            /* 21 Precise angle data for cel dim  n*6 real */
      Lock      lock;       /* is logical record locked     */
      Directory dir;        /* directory for logical record */
      int       *data;      /* Linked list data record      */
}; 


typedef struct _sflsData {
      short     dataSet;    /* data set number  */
      char      *weighting;
      char      *extinction;
      char      *lsMatrix;
      char      *hydrogens;
      char      *absolute;
      char      *extincEquate;
      float     rFactorAll;
      float     rFactorRcode1;
      float     wrFactorAll;
      float     wrFactorRcode1;
      float     gofAll;
      float     gofRcode1;
      float     maxShiftError;
      float     meanShiftError;
      float     Rogers;
      float     Flack;
      float     extFactor;
      float     extFactorSig;
      short     refinedParam;   /* F=1 ; F2=2; I =3  */
      short     params;
      short     reflections;
      short     restraints;
      short     constraints;
struct _sflsData  *next;
}   *SFLSData;

struct _lrsfls {            /* 30 Structure Factor least squares V leng pack*/
      Lock      lock;       /* is logical record locked */
      Directory dir;        
      SFLSData  data;        
}; 


typedef struct _rFactorRec {
      short  lsqCycles;     /* number  of SFLS cycles  */    
      float  unitWfRFactor; /* R factor  - unit weights  on f      */
      float  sigWfRFactor; /* R factor  - sigma weights  on f     */
      float  sigWf2RFactor; /* R factor  - sigma weights  on f^2   */
      float  goodnessOfFit; /*                                     */
      char   *text;         /* title used during least squares run */
struct _rFactorRec *next;     /* linked list */
}   *RFactorRec;
struct _lrrfac {            /* 31 Refinement history     */
      Lock        lock;     /* is logical record locked  */
      RFactorRec data;     /* Linked List R Factor Data */
}; 

struct _lrdens {            /* 35 Electron Density data   variable  length */
      Lock      lock;       /* is logical record locked */
      Directory dir;        
      float  minDiffDensity;
      float  maxDiffDensity;
}; 



typedef struct _atomref {
        char   *label;
        float   fracCoords[3];  
        short   symopID;
        short   transID;
} Atomref;

typedef struct _bondpak {
      Atomref   atom1;
      Atomref   atom2;
      float     calcLength;
      float     calcLengthSig;
      float     obsLength;
      float     obsLengthSig;
      Symop     atom1sym;
      Symop     atom2sym;
struct _bondpak *next;
}    *Bondpak;

struct _lrbond {            /* 40 bond distances var length var packs */
      Lock      lock;       /* is logical record locked */
      Directory dir;        
      Bondpak   bonds;
}; 

typedef struct _contpak {
      Atomref   atom1;
      Atomref   atom2;
      float     Length;
      float     LengthSig;
      Symop     atom1sym;
      Symop     atom2sym;
struct _contpak *next;
}   *Contpak;
struct _lrcont {            /* 41 contact  distances var length var packs */
      Lock   lock;          /* is logical record locked */
      Directory dir;        
      Contpak   contacts;
}; 

typedef struct _anglepak {
      Atomref   atom1;
      Atomref   atom2;
      Atomref   atom3;
      float     calcAngle;
      float     calcAngleSig;
      float     obsAngle;
      float     obsAngleSig;
      Symop     atom1sym;
      Symop     atom2sym;
      Symop     atom3sym;
struct _anglepak  *next;
} *Anglepak;
struct _lrangl {            /* 42 bond angles   var length var packs */
      Lock      lock;       /* is logical record locked */
      Directory dir;        
      Anglepak  angles;
}; 

typedef struct _dihedpak {
      Atomref   atom1;
      Atomref   atom2;
      Atomref   atom3;
      Atomref   atom4;
      float     calcAngle;
      float     calcAngleSig;
      float     obsAngle;
      float     obsAngleSig;
      Symop     atom1sym;
      Symop     atom2sym;
      Symop     atom3sym;
      Symop     atom4sym;
struct _dihedpak *next;
}    *Dihedpak;

struct _lrdihe {            /* 43 dihedral angles var length var packs */
      Lock      lock;       /* is logical record locked ? */
      Directory dir;        /* directory information      */
      Dihedpak  dihed;      /* packets of dihedral angle information */
}; 

typedef struct _rigidpak {
      Atomref   atom1;
      Atomref   atom2;
      float     calcRigid;
      float     calcRigidSig;
      float     obsRigid;
      float     obsRigidVar;/* variance */
      float     weight;     /* rigid link weight  */ 
      Symop     atom1sym;
      Symop     atom2sym;
struct _rigidpak *next;
} *Rigidpak;
struct _lrrigi {            /* 44 Rigid link restraint info V length V packs */
      Lock      lock;       /* is logical record locked */
      Directory dir;        /* directory information    */
      Rigidpak  data;
}; 

struct _lreof  {            /* end of file record  */
      short    recnum;      /* 65535 */
}; 





struct _xtalRec {
struct    _lrhist  hist;     /* 1  File History    50 *24 chars */
struct    _lrlabl  labl;     /* 2  Label           n+3*80 chars */

struct    _lrcell  cell;     /* 4  Cell constants  12 *9  floats */
struct    _lrsymm  symm;     /* 5  Symmetry Informtn  1+24*12 int symops */
struct    _lrsgnm  sgnm;     /* 6  Space Group name 2*40 chars */
struct    _lrtwin  twin;     /* 7  Twin Laws           13 floats */
struct    _lrscat  scat;     /* 8  Atom-type params */
struct    _lrsequ  sequ;     /* 9  Protein Atom Sequence Information   */
struct    _lrddef  ddef;     /* 10 data set names     n*12 chars  */
struct    _lrexpl  expl;     /* 11 experimental parameters var length var pack*/
struct    _lrdset  dset;     /* 12 general data per packet var length var pack*/
struct    _lrdict  dict;     /* 13 MacroMolecular Residue Information n*20 */
struct    _lrgraf  graf;     /* 14 Molecular graphing Information  var?? */

struct    _lratom  atom;     /* 16 Atom parameters    var length 1/indep atom */
struct    _lrcons  cons;     /* 17 Refinement constraints v length 1/in atom*/
struct    _lrslav  slav;     /* 18 Protein atom link information */
struct    _lrmole  mole;     /* 19 Molecular information */
struct    _lrrefl  refl;     /* 20 Reflection info var length  var packs */
struct    _lrthet  thet;     /* 21 Precise angle data for cel dim  n*6 real */

struct    _lrsfls  sfls;     /* 30 Structure Factor least squares V leng pack*/
struct    _lrrfac  rfac;     /* 31 Refinement history */
struct    _lrdens  dens;     /* 35 Electron Density data   variable  length */

struct    _lrbond  bond;     /* 40 bond distances var length var packs */
struct    _lrcont  cont;     /* 41 contact  distances var length var packs */
struct    _lrangl  angl;     /* 42 bond angles   var length var packs */
struct    _lrdihe  dihe;     /* 43 dihedral angles var length var packs */
struct    _lrrigi  rigi;     /* 44 Rigid link restraint info V length V packs */
struct    _lreof   eof;      /* 65535 */
} ;

/* In a multithreaded graphical interface environment 
   you  could use a common program  call such as 
    (void) ABSORB( XtalRec compidName, Char *controlLineBuff,)

   enabling the user to control multiple structural refienments etc 
   from the one user interface, by virtue of having as many compidName
   pointers to bdf records as data sets being worked on.
 */




/* This record is a fixed length record of pointers to structured 
   records of data items. It isn't very well thought out,
   because it doesn't allow Joe.Programmer to add other bdf logical records
   to the archive without having to recompile all programs to recognise 
   the archive has a different number of records.
   
   Currently to get cell dimension "c" you could simply use:
   compidName->cell.abc[2]


   However, to be extendable, probably it should be a variable length array 
   of pointers to variable record types. This could make extracting info 
   more difficult because to access a particular archive logical record,
   it will be necessary  to parse the  whole    compidName     array 
   looking for the requested logical record, (then again for directory or 
    variable records - parse the record looking for a specific packet item).

   i.e.  for a permanent fixed length record

struct XtalRec {            
      Lock      lock;       
      int       recordType;    /* 1 for lrhist 2 for lrlabl ....*/
      void      *lrec;         /* pointer to actual logical record data   */
}; 
       

        XtalRec     *compidName[];  /* define a (global) variable length array 
                                       of pointers to XtalRecords */
       
        for(i=0;i<LREOF; i++) if (compidName[i]->recordType==LRCELL) break;
        if (compidName[i]->recordType<LREOF)  {
                      /* convert void pointer to _lrcell pointer  */   
             localCellRecPtr = (_lrcell *) compidName[i]->lrec; 
             cdim = localLRecPtr->abc[2];      /* get c dimension */
        }
        else  there is no LRCELL packet


        note that this also required valid definitions of LRCELL and
        LREOF, used here as macro definitions, but maybe better 
        identified from a lookup table somewhere ?? eg xtal_arc.dic

 */


/*      ---------------------The End---------------------- */
