/*---------------------------------------------------------------------
    @(#) siemcvt.c Novemeber 1995

    The siemcvt program is based on a similar rbsync program that was
    written to read the four-circle single-crystal X-ray synchrotron
    diffractometer data stored as a file that represents the physical
    contents of a 8-inch floppy disk used at the National Laboratory
    for High Energy Physics, Tsukuba, Japan.
    The program uses the UNIX low-level interface to read the contents
    of the file specified as the first argument to this program. The
    optional second argument must be a valid UNIX file name. If the a
    second argument is not provided the program's output is sent to the
    standard output. The output's format will be suitable as input for
    XTAL.

    This C program is written in Standard C. The floating point numbers
    in the synchroton data file are in IBM 360/370 representation, and
    the function dgfloat translates them into the standard IEEE format.

    Author:  T.P. Vaalsta
    Date:    November 1995
---------------------------------------------------------------------*/
#define SIEMCVT_C

#include "siemcvt.h"

void main(int argc, char *argv[])
{
int fd,mode,rv;
FILE *fout;

    if(argc > 1) {
	mode = O_RDONLY
#ifdef __unix
	| O_NDELAY;
#else /* MS DOS C version 6.00 */
	| O_BINARY;
#endif /* __unix */
        if((fd = open(argv[1],mode)) == -1) {
            fprintf(stderr, "%s: can't open %s\n", argv[0], argv[1]);
            exit(1);
            }
        if(argc == 2)
            fout = stdout;
        else
            if((fout = fopen(argv[2], "w")) == NULL) {
                fprintf(stderr, "%s: can't open %s\n", argv[0], argv[2]);
                exit(1);
                }
        extract(fd, fout);
        if((rv = close(fd)))
            fprintf(stderr, "%s: error in closing input file\n", progname);
        if(argc > 1)
            if((rv += fclose(fout)))
                fprintf(stderr, "%s: error in closing output file\n", progname);
        if(rv)
            exit(1);
        }
    else
        fprintf(stderr, "Usage: %s <input data file> [<file for output>]\n", argv[0]);
    exit(0);
}
/*---------------------------------------------------------------------
    extract - this function reads the input file containing the physical
    binary pattern of the P3 data collection program. Each reflection
    record has 248 bytes for the two theta:theta collection mode.
    Each reflection record is extracted and transmitted to the output
    stream in the format suitable for input to XTAL 3.4 version.
    The output format is in the P4 data collection format - an ASCII
    printable file.
    ARGUMENTS:
        fd                  file descriptor of the input file
        fout                output stream
---------------------------------------------------------------------*/
static void extract(int fd, FILE *fout)
{
int cont;
int i,j,m,rv;
float *t;
unsigned long scount,pcount;
unsigned short *us,h;
char buffer[512];
static char *fmt[] = {      /* floating point format array for XTAL input */
        "%8.2f",            /* Two theta setting angle */
        "%8.2f",            /* Omega setting angle */
        "%8.2f",            /* Phi setting angle */
        "%8.2f",            /* Chi setting angle */
        "%5.2f",            /* Two theta range */
        "%8.2f",            /* Two theta scan rate */
        "%6i",              /* Left Bacground */
        "%12i",             /* Scan count */
        "%6i",              /* Right Background */
        "%15.1f",           /* Intensity (in one degree per minute scale) */
        "%8.1f",            /* Standard deviation of intensity */
        "%8.2f",            /* Exposure in Hours */
        (char)NULL
        };

    itype = 0;
    pcount = scount = 0L;
    cont = TRUE;
    do {
        switch( itype ) {
            case 0 :
                if((rv = read(fd,cbuf,nbtr)) != nbtr) {
                    if(rv == -1) {
                        fprintf(stderr, "%s: error in reading input file", progname);
                        exit(1);
                        }
                    else if(rv == 0)
                        fprintf(stdout, "%lu bytes processed %lu bytes skipped. Total: %lu\n",
                        248L * pcount, 248L * scount, 248L * (pcount + scount));
                    else
                        fprintf(stdout, "%lu bytes processed %lu bytes skipped. Total: %lu\n",
                        248L * pcount, 248L * scount + (unsigned long)rv,
                        248L * (pcount + scount) + (unsigned long)rv);
                    cont = FALSE;
                    break;
                    }
                un = (unsigned short *)cbuf;
                itype++;
                us = un;
                rn = (float *)(un + 4);
                m = 0;
                pcount++;
                break;
            case 1 :
                for(i=0;i< 4;i++,us++) {
                    h = *us;
                /*    h = chgbytes(*us);*/
                    switch(i) {
                        case 0:
                            fprintf(fout," %6i", (short)h);
                            break;
                        case 1:
                        case 2:
                        case 3:
                            fprintf(fout, "%4i", (short)h);
                            break;
                        default:
                            fprintf(stderr, "Impossible case\n");
                            exit(1);
                            break;
                        }
                    if(ferror(fout)) {
                        fprintf(stderr, "%s: error in writing to output\n", progname);
                        exit(1);
                        }
                    }
                itype++;
                break;
            case 2 :
                un = (unsigned short *)rn;
/*
                for(i = 0, us = (unsigned short *)temp; i < 18; un++,
                us++, i++)
                    *us = *un;
                us = chgbytes(*un);*/
                t = rn;
                for(i = 0; i < 12; i++, t++)
                    if(i >= 6 && i <= 8)
                        fprintf(fout, fmt[i], (short)*t);
                    else
                        fprintf(fout, fmt[i], *t);
                fprintf(fout,"\n");
                if(ferror(fout)) {
                    fprintf(stderr, "%s: error in writing to output\n", progname);
                    exit(1);
                    }
                itype = 0;
                us = (unsigned short *)t;
                i = packit(us, buffer);
                j = 0;
                do {
                     if(i >= 80) {
                         j += fprintf(fout, "%.80s\n", buffer + j);
                         j--;
                         i -= 80;
                         }
                     else {
                         (void)fprintf(fout, "%s\n", buffer + j);
                         break;
                         }
                   } while(i > 0);
                break;
            default :
                cont = FALSE;
                break;
            }
        } while( cont );
}
/*---------------------------------------------------------------------
    chgbytes - change bytes around
    ARGUMENTS:
        a                    the word in which bytes are to be swapped.
    RETURN: the changed word.
---------------------------------------------------------------------*/
static unsigned short chgbytes(unsigned short a)
{
unsigned short x,y;

    x = 0xff00 & a;
    x>>=8;
    y = 0x00ff & a;
    y<<=8;
    return( x | y );
}
/*---------------------------------------------------------------------
    dgfloat - convert dg mv-series (aka IBM 360/370) floating point data
    ARGUMENTS:
        r                    floating point value to be converted
    RETURN: IEEE floating point representation.
----------------------------------------------------------------------*/
static float dgfloat(float r)
{
double e;        /* exponent */
float f;        /* result */
union u_tag {
    float r;
    long l;
    } u;

    u.r = r;
    e = ((u.l & 0x7f000000) >> 24) - 64;    /* should check range */
    f = (u.l & 0x00ffffff) / 16777216.0;    /* / 2 to power of 24 */
    f *= pow(16.0,e);
    return(u.l & 0x80000000 ? -f : f);        /* check sign bit */
}
/*------------------------------------------------------------------------------
  packit  -  This function translates the record integer profiles of a
             diffraction intensity into a packed array of printable characters.
             The packing format is the same as for the P4 data file format -
             see appendix B of the Siemens Xscans Technical Reference.
------------------------------------------------------------------------------*/
static int packit(unsigned short *val, char *buf)
{
static const char *low_order = "0123456789`~@#$%^&*()-_=+|\\[{]};:'\",<.>/?";
static const char *high_order =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
char packing[7] = "zzzzz?";
int hi,lo,i,j;
char *ptr, *p_ptr;

    ptr = buf;
    j = 0;
    for(i = 0; i < 96; i++, val++) {
        p_ptr = &packing[5];
        hi = *val / 42;
        lo = *val % 42;
        *p_ptr = low_order[lo];
        while(hi) {
            p_ptr--;
            lo = hi % 52;
            hi /= 52;
            *p_ptr = high_order[lo];
            }
        j += sprintf(ptr + j, "%s", p_ptr);
        }
    return( j );
}
