systemheader:(AS    )#                                                  AS     1
####################################################################### AS     2
#                                                                     # AS     3
#                          *** ABSCAL ***                             # AS     4
#                                                                     # AS     5
#    THE PROGRAM, ABSCAL, CORRECTS FOR PHI SPIN MEASURED ABSORPTION   # AS     6
#    AND FOR RADIATION  DECAY AS A FUNCTION OF EXPOSURE TIME AND,     # AS     7
#    OPTIONALLY, AS A FUNCTION OF EXPOSURE TIME AND THETA ANGLE.      # AS     8
#    THE PROGRAM REQUIRES INFORMATION ON TAU ABSORPTION OBTAINED BY   # AS     9
#    MEASURING INTENSITIES FOR A SINGLE REFLECTION AT CHI NINETY FOR  # AS    10
#    PHI RANGING FROM 0 TO 359 IN STEPS NO COARSER THAN 20 DEGREES.   # AS    11
#    IN ADDITION IT REQUIRES MONITORING REFLECTIONS TO HAVE BEEN      # AS    12
#    MEASURED PERIODICALLY ALL DURING THE INTENSITY GATHERING PERIOD. # AS    13
#    A FIT IS MADE TO THE MONITOR REFLECTIONS AS A FUNCTION OF TIME   # AS    14
#    ( AND THETA, OPTIONALLY) AND THE RESULTING EQUATION IS USED TO   # AS    15
#    CORRECT ALL THE OBSERVED REFLECTIONS.                            # AS    16
#    THE CORRECTED DATA SET, LESS THE ABSORPTION AND MONITORING       # AS    17
#    REFLECTIONS, IS WRITTEN TO LOGICAL RECORD LRREFL: OF THE BDF.    # AS    18
#    K.D.WATENPAUGH, 27 SEP 85  UPDATED 7/15/86,3/9/86,16/10/86       # AS    19
#                                                                     # AS    20
####################################################################### AS    21
#                                                                       AS    22
include:(XMACRO)#                                                       AS    23
#                                                                       AS    24
macro:(aitken:,[AS32($1,$2,$3,$4,$5,3)])# 4-point interpolator          AS    25
macro:(exchange:,$(#                   Interchange the 4 parts          AS    26
movepakwd:(FNUM(I),TEMP)#              Of the ith and jth               AS    27
movepakwd:(FNUM(J),FNUM(I))#           Entries in the stack             AS    28
movepakwd:(TEMP,FNUM(J))#                                               AS    29
movepakwd:(FNUM(I+1),TEMP)#                                             AS    30
movepakwd:(FNUM(J+1),FNUM(I+1))#                                        AS    31
movepakwd:(TEMP,FNUM(J+1))#                                             AS    32
movepakwd:(FNUM(I+2),TEMP)#                                             AS    33
movepakwd:(FNUM(J+2),FNUM(I+2))#                                        AS    34
movepakwd:(TEMP,FNUM(J+2))#                                             AS    35
movepakwd:(FNUM(I+3),TEMP)#                                             AS    36
movepakwd:(FNUM(J+3),FNUM(I+3))#                                        AS    37
movepakwd:(TEMP,FNUM(J+3))$))#                                          AS    38
macro:(exchangei:,[$(#                 Interchange the 2 parts of       AS    39
TEMP = NUM(I)#                         The ith and jth entries          AS    40
NUM(I) = NUM(J)#                       In the stack.                    AS    41
NUM(J) = TEMP#                                                          AS    42
TEMP = NUM(I+1)#                                                        AS    43
NUM(I+1) = NUM(J+1)#                                                    AS    44
NUM(J+1) = TEMP$)])#                                                    AS    45
macro:(maxcoef:,6)#  ## Max decay coefficients                          AS    46
macro:(maxmon:,30)#  ## Maximum number of monitor reflections           AS    47
macro:(maxtau:,72)#  ## Maximum no. of absorption measurements          AS    48
macro:(maxdk:,5000)# ## Total no. of monitor observations (all refs)    AS    49
macro:(maxplot:,5000)## Maximum area of plots                           AS    50
#                                                                       AS    51
extrude:(ASCOMN)#                                                       AS    52
INTEGER APPLY#     ## Flag to apply corrections and write fileb         AS    53
INTEGER DKFLG#     ## Flag to apply derived decay correction            AS    54
INTEGER DKFLGL#    ## Flag to apply line input decay correction         AS    55
INTEGER EXFRST#    ## First reflection to exclude from calc. decay      AS    56
INTEGER EXLAST#    ## Last refection to exclude from calc. decay        AS    57
INTEGER SCFRST#    ## First reflection to apply scales to               AS    58
INTEGER SCLAST#    ## Last reflection to apply scales to                AS    59
INTEGER DKMRK#     ## Pointer to decay monitoring reflections in qx     AS    60
INTEGER DKSTEP#    ## Number of items per decay monitor reflection      AS    61
INTEGER DKTYPE#    ## Method of decay correction                        AS    62
INTEGER ENDMRK#    ## End of qx array                                   AS    63
INTEGER MKTAU#     ## Pointer to absorption correction factors          AS    64
INTEGER NCOEF#     ## Number of coefficients for decay correction       AS    65
INTEGER PLTMRK#    ## Pointer to plot data array                        AS    66
INTEGER SCLMRK#    ## Pointer for dkscale array                         AS    67
INTEGER TIMEFL#    ## Signal use of exposure time from lrrefl:          AS    68
##        RELATIVE POINTERS OF INFORMATION ON LR REFL:                  AS    69
INTEGER NHKL#      ## Hkl                                               AS    70
INTEGER NPHI#      ## Phi angle                                         AS    71
INTEGER NCHI#      ## Chi angle                                         AS    72
INTEGER NOME#      ## Omega angle                                       AS    73
INTEGER NTTH#      ## Two-theta angle                                   AS    74
INTEGER NEXP#      ## Exposure time in seconds on file                  AS    75
INTEGER NRFT#      ## Reflection type code 0/1/2/3 obs/std/phi spin/b   AS    76
INTEGER NRFN#      ## Reflection number                                 AS    77
INTEGER NIRL#      ## Relative intensity                                AS    78
INTEGER NSIR#      ## Relative sigma (intensity)                        AS    79
INTEGER NDK#       ## Total measurement of all monitor reflections      AS    80
INTEGER NSTAND#    ## Number of monitor reflection                      AS    81
INTEGER NTAU#      ## Number of tau values input for calc. absorption   AS    82
INTEGER PURGE#     ## Flag removal of all void and monitoring refs      AS    83
INTEGER TAUFLG#    ## Flag to apply absorption                          AS    84
INTEGER TAUMRK#    ## Pointer to absorp table in qx array               AS    85
INTEGER TAUMIN#    ## Minimum refl. no. of tau correction refl.         AS    86
INTEGER TAUMAX#    ## Maximum refl. no. of tau correction refl. range   AS    87
REAL ADDREF#           ## Value used to adjust reflection numbers       AS    88
REAL ADDEXP#           ## Value used to adjust exposure times           AS    89
REAL COEF(maxcoef:)#   ## Decay correction coefficient                  AS    90
REAL SHOVER(maxcoef:)# ## Shift over error for test of convergence      AS    91
REAL SGCOEF(maxcoef:)# ## Sigma of coefficients                         AS    92
REAL DKHKL(3,maxdk:)#  ## Hkl of monitor reflections                    AS    93
REAL INTENS(maxtau:)#  ## Intensity of absorp corr reflection at phi    AS    94
REAL PHI(maxtau:)#     ## Phi values of absorption correction reflect   AS    95
REAL TAUHKL(3)#        ## Reflection indices of absorp corr reflectio   AS    96
REAL TIMEPR#           ## Time in seconds per reflection measurement    AS    97
REAL TTLOW,TTHIGH#     ## Limits of two theta to be treated             AS    98
#                                                                       AS    99
COMMON/COMIAS/$#                                                        AS   100
APPLY,DKFLG,DKFLGL,SCFRST,SCLAST,DKMRK,DKSTEP,ENDMRK,NHKL,NPHI,#        AS   101
NCHI,NOME,NTTH,NEXP,NRFT,NRFN,NIRL,NSIR,NSTAND,NTAU,#                   AS   102
SCLMRK,TIMEFL,DKTYPE,MKTAU,NDK,NCOEF,PLTMRK,PURGE,TAUFLG,TAUMRK,#       AS   103
TAUMIN,TAUMAX,EXFRST,EXLAST#                                            AS   104
#                                                                       AS   105
COMMON/COMFAS/ADDREF,ADDEXP,DKHKL,INTENS,PHI,TAUHKL,COEF,#              AS   106
SGCOEF,SHOVER,TIMEPR,TTLOW,TTHIGH#                                      AS   107
endext:#                                                                AS   108
#                                                                       AS   109
systemheader:(AS00)#                   Modify hkl data on bdf           AS00   1
# ################################################################## #  AS00   2
#                                                                    #  AS00   3
#                               AS00                                 #  AS00   4
#                                                                    #  AS00   5
# ################################################################## #  AS00   6
SUBROUTINE AS00#                       K.d.watenpaugh                   AS00   7
include:(AACOMN)#                                                       AS00   8
include:(ASCOMN)#                                                       AS00   9
#                                                                       AS00  10
#    SET IO BUFFERS                                                     AS00  11
#    --------------                                                     AS00  12
#                                                                       AS00  13
IOMARK(1)=QXSTR#                       Start of buffer for file a       AS00  14
IOMARK(2)=IOMARK(1)+bdfbuf:#           Start of buffer for file b       AS00  15
TAUMRK   =IOMARK(2)+bdfbuf:#           Start of absorption              AS00  16
DKMRK = TAUMRK + arith:(maxtau:,*,2)#  Start of decay scale             AS00  17
DKSTEP=4#                              Four items to save               AS00  18
PLTMRK = DKMRK + maxdk:*DKSTEP#        Area for plotting scales         AS00  19
incrqx:(ENDMRK,PLTMRK+maxplot:,AS0001)# A                               AS00  20
#                                                                       AS00  21
# PROCESS INFORMATION FOR ABSORPTION AND DECAY SCALING                  AS00  22
#                                                                       AS00  23
CALL AS11#                             Initialize and read control      AS00  24
IF((TAUFLG==yes:&NTAU==0)\DKFLG==yes:)#                                 AS00  25
CALL AS12#                             Get absorp and decay monitor     AS00  26
#                                        Information from bdf           AS00  27
IF (NTAU > 0) TAUFLG=yes:#             If tau lines were read then      AS00  28
IF (TAUFLG==yes:) CALL AS13#           Produce absorption               AS00  29
IF (TAUFLG==yes:) CALL AS15#           Plot absorption correction       AS00  30
IF (DKFLG==yes:)  CALL AS16#           Calculate the decay              AS00  31
#                                                                       AS00  32
# IF SPECIFIED, APPLY THE GENERATED CORRECTIONS TO THE INTESITY DATA    AS00  33
# AND WRITE AN OUTPUT BINARY DATA FILE                                  AS00  34
#                                                                       AS00  35
IF(APPLY == yes:) CALL AS30#                                            AS00  36
#                                                                       AS00  37
RETURN#                                                                 AS00  38
END#                                                                    AS00  39
systemheader:(AS01)#                   Three coefficient l-s            AS01   1
###################################################################     AS01   2
#                                                                 #     AS01   3
#                            AS01                                 #     AS01   4
#                                                                 #     AS01   5
###################################################################     AS01   6
SUBROUTINE AS01(SC,DER,COEF,THETA,EXPOS,NCOEF,IDER)#                    AS01   7
#                                     # K.d.watenpaugh                  AS01   8
#     CALCULATES A FUNCTION OF THETA AND NREF USING THE NCOEF           AS01   9
#       COEFFICIENTS STORED IN COEF.  IT ALSO RETURNS THE DERIVATIVES   AS01  10
#       IN DER.  THE FUNCTION VALUE IS RETURNED AS SC.                  AS01  11
#     IF IDER EQUALS ZERO, THE DERIVATIVE CALCULATION IS SKIPPED.       AS01  12
#                                                                       AS01  13
REAL DER(maxcoef:)#                                                     AS01  14
REAL COEF(3)#                                                           AS01  15
REAL SC,THETA,EXPOS#                                                    AS01  16
INTEGER NCOEF,IDER#                                                     AS01  17
REAL EX,T#                                                              AS01  18
#                                                                       AS01  19
NCOEF=3#                                                                AS01  20
T=ABS(THETA)#                                                           AS01  21
EX=EXPOS/3600.#                        Use hours instead of seconds     AS01  22
SC=COEF(1)+COEF(2)*EX+COEF(3)*EX**2#                                    AS01  23
IF (IDER!=0)#                                                           AS01  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS01  25
DER(1)=1.0#                                                             AS01  26
DER(2)=EX#                                                              AS01  27
DER(3)=EX**2#                                                           AS01  28
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS01  29
RETURN#                                                                 AS01  30
END#                                                                    AS01  31
systemheader:(AS02)#                   Six coefficient l-s              AS02   1
###################################################################     AS02   2
#                                                                 #     AS02   3
#                            AS02                                 #     AS02   4
#                                                                 #     AS02   5
###################################################################     AS02   6
SUBROUTINE AS02(SC,DER,COEF,THETA,EXPOS,NCOEF,IDER)#                    AS02   7
#                                     # K.d.watenpaugh                  AS02   8
#     CALCULATES A FUNCTION OF THETA AND EXPOS USING THE NCOEF          AS02   9
#       COEFFICIENTS STORED IN COEF.  IT ALSO RETURNS THE DERIVATIVES   AS02  10
#       IN DER.  THE FUNCTION VALUE IS RETURNED AS SC.                  AS02  11
#     IF IDER EQUALS ZERO, THE DERIVATIVE CALCULATION IS SKIPPED.       AS02  12
#                                                                       AS02  13
REAL DER(maxcoef:)#                                                     AS02  14
REAL COEF(maxcoef:)#                                                    AS02  15
REAL SC,THETA,EXPOS#                                                    AS02  16
INTEGER NCOEF,IDER#                                                     AS02  17
REAL EX,T#                                                              AS02  18
#                                                                       AS02  19
NCOEF=6#                                                                AS02  20
T=ABS(THETA)#                                                           AS02  21
EX=EXPOS/3600.#                        Use hours instead of seconds     AS02  22
SC=COEF(1)+COEF(2)*T*EX    $#                                           AS02  23
+COEF(3)*T*T*EX**2         $#                                           AS02  24
+COEF(4)*T                 $#                                           AS02  25
+COEF(5)*T*T*EX            $#                                           AS02  26
+COEF(6)*T*T#                                                           AS02  27
#                                                                       AS02  28
IF (IDER!=0)#                                                           AS02  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS02  30
DER(1)=1.0#                                                             AS02  31
DER(2)=T*EX#                                                            AS02  32
DER(3)=T*T*EX**2#                                                       AS02  33
DER(4)=T#                                                               AS02  34
DER(5)=T*T*EX#                                                          AS02  35
DER(6)=T*T#                                                             AS02  36
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS02  37
RETURN#                                                                 AS02  38
END#                                                                    AS02  39
systemheader:(AS03)#                   Five coefficient l-s             AS03   1
###################################################################     AS03   2
#                                                                 #     AS03   3
#                            AS03                                 #     AS03   4
#                                                                 #     AS03   5
###################################################################     AS03   6
SUBROUTINE AS03(SC,DER,COEF,THETA,EXPOS,NCOEF,IDER)#                    AS03   7
#                                     # J.m.stewart 7 sept 1986         AS03   8
#     CALCULATES A FUNCTION OF THETA AND NREF USING THE NCOEF           AS03   9
#       COEFFICIENTS STORED IN COEF.  IT ALSO RETURNS THE DERIVATIVES   AS03  10
#       IN DER.  THE FUNCTION VALUE IS RETURNED AS SC.                  AS03  11
#     IF IDER EQUALS ZERO, THE DERIVATIVE CALCULATION IS SKIPPED.       AS03  12
#                                                                       AS03  13
REAL DER(maxcoef:)#                                                     AS03  14
REAL COEF(maxcoef:)#                                                    AS03  15
REAL SC,THETA,EXPOS#                                                    AS03  16
INTEGER NCOEF,IDER#                                                     AS03  17
REAL EX,T#                                                              AS03  18
#                                                                       AS03  19
NCOEF=4#                                                                AS03  20
T=ABS(THETA)#                                                           AS03  21
EX=EXPOS/3600.#                        Use hours instead of seconds     AS03  22
SC=COEF(1)+COEF(4)*T*EX    $#                                           AS03  23
+COEF(2)*EX                $#                                           AS03  24
+COEF(3)*T                 $#                                           AS03  25
+COEF(5)*EX**2#                                                         AS03  26
#                                                                       AS03  27
IF (IDER!=0)#                                                           AS03  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS03  29
DER(1)=1.0#                                                             AS03  30
DER(4)=T*EX#                                                            AS03  31
DER(2)=EX#                                                              AS03  32
DER(3)=T#                                                               AS03  33
DER(5)=EX**2#                                                           AS03  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS03  35
RETURN#                                                                 AS03  36
END#                                                                    AS03  37
systemheader:(AS04)#                   Aitken's interpolation           AS04   1
# ################################################################## #  AS04   2
#                                                                    #  AS04   3
#                               AS04                                 #  AS04   4
#                                                                    #  AS04   5
# ################################################################## #  AS04   6
SUBROUTINE AS04(ARG,TAB1,TAB2,RESULT,SIZE,DEGREE)#                      AS04   7
#                                                                       AS04   8
# AITKEN'S METHOD OF EVALUATING THE LAGRANGE INTERPOLATING POLYNOMIAL   AS04   9
#  THIS ALGORITHM IS TAKEN FROM THE COMPUTER JOURNAL, SEP 1966, PG. 211 AS04  10
#                                                                       AS04  11
#                                        Mar 1983  r.a. alden           AS04  12
#                                                                       AS04  13
INTEGER DEGREE#                        The degree of the polynomial     AS04  14
#                                         Evaluation.                   AS04  15
INTEGER EQUAL#                         A flag set = yes: when arg       AS04  16
#                                         Is exactly the same as a      AS04  17
#                                         Point in table 1.             AS04  18
INTEGER FIRST#                         First point used in the          AS04  19
#                                         Current interpolation.        AS04  20
INTEGER I#                             Loop index                       AS04  21
INTEGER J#                             Another loop index               AS04  22
INTEGER POINT#                         Number of points used in the     AS04  23
#                                         Interpolation.                AS04  24
INTEGER SIZE#                          Number of points in the tabl     AS04  25
REAL ARG#                              Value to be interpolated         AS04  26
REAL F(10)#                            Temporarily holds the            AS04  27
#                                         Interpolating points from     AS04  28
#                                         Table 2.                      AS04  29
REAL RESULT#                           The resulting interpolated       AS04  30
#                                         Value.                        AS04  31
REAL TAB1(1)#                          Start of table 1                 AS04  32
REAL TAB2(1)#                          Start of table 2                 AS04  33
REAL Z(10)#                            Temporarily holds the            AS04  34
#                                         Difference between arg and    AS04  35
#                                         The interpolating points in   AS04  36
#                                         Table 1.                      AS04  37
realdata:(SMALL,[.00001])#                                              AS04  38
datastuff:#                                                             AS04  39
#                                                                       AS04  40
EQUAL = no:#                           Default is not equal             AS04  41
IF(ARG<0.)    ARG=ARG+360.#            Ensure in range 0-pi             AS04  42
ELSE IF(ARG>360.) ARG=ARG-360.#        Ensure in range 0-pi             AS04  43
#                                                                       AS04  44
FOR(I=1; I < SIZE; I=I+1)#             Find out where arg falls in      AS04  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS04  46
IF(ABS(ARG-TAB1(I)) < SMALL)#          Table 1.                         AS04  47
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS04  48
EQUAL = yes:#                          It's exactly equal to one of     AS04  49
BREAK#                                 The points.                      AS04  50
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS04  51
IF(ARG < TAB1(I))  BREAK#                                               AS04  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS04  53
#                                                                       AS04  54
IF(EQUAL == yes:)  RESULT = TAB2(I)#   This is the easy way out         AS04  55
#                                                                       AS04  56
ELSE#                                  We'll have to interpolate        AS04  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS04  58
IF(DEGREE > SIZE \ DEGREE > 10)  DEGREE = MIN0(10,SIZE)#                AS04  59
POINT = DEGREE + 1#                    Number of points to be used      AS04  60
#                                         From the tables.              AS04  61
I = I - DEGREE/2 - 1#                  Locate first table point to      AS04  62
FIRST = I#                             Be used.                         AS04  63
IF(I < 1)  FIRST = 1#                  Obviously it can't be before     AS04  64
#                                         The first point or after the  AS04  65
ELSE  IF((I+DEGREE) > SIZE)  FIRST = SIZE - DEGREE# Last.               AS04  66
#                                                                       AS04  67
FOR(I=1; I <= POINT; I=I+1)#           Now loop over the mini-table     AS04  68
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS04  69
Z(I) = ARG - TAB1(FIRST)#              Save the deltas                  AS04  70
F(I) = TAB2(FIRST)#                    And the table 2 equivalences     AS04  71
FIRST = FIRST + 1#                                                      AS04  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS04  73
#                                                                       AS04  74
FOR(I=1; I <= DEGREE; I=I+1)#          Now do the interpolation         AS04  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS04  76
FOR(J=I+1; J <= POINT; J=J+1)#                                          AS04  77
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS04  78
F(J) = F(I) + Z(I)*(F(J)-F(I))/(Z(I)-Z(J))#                             AS04  79
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS04  80
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS04  81
RESULT = F(POINT)#                                                      AS04  82
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS04  83
#                                                                       AS04  84
RETURN#                                                                 AS04  85
END#                                                                    AS04  86
systemheader:(AS11)#                   Initialization line input        AS11   1
###################################################################     AS11   2
#                                                                 #     AS11   3
#                            AS11                                 #     AS11   4
#                                                                 #     AS11   5
###################################################################     AS11   6
SUBROUTINE AS11#                       K.d.watenpaugh, 27 sep 85        AS11   7
#                                                                       AS11   8
include:(AACOMN)#                                                       AS11   9
include:(ASCOMN)#                                                       AS11  10
#                                                                       AS11  11
INTEGER I#                                                              AS11  12
INTEGER FIELD#                                                          AS11  13
INTEGER FIRST#                                                          AS11  14
INTEGER LAST#                                                           AS11  15
INTEGER MATCH#                                                          AS11  16
#                                                                       AS11  17
chardata:(AS01,[END ABSORP TAU DECAY STAND APPLY EXCLUD OFFSET ])#      AS11  18
chardata:(AS02,[NOAB ABSO NODE DECA NOAP APPL PURG NOPU TIME ])#        AS11  19
chardata:(AS11A,YESNO )#                                                AS11  20
chardata:(AS11B, DETERMINE TAU ABSORPTION $#                            AS11  21
CORRECTIONS                   )#                                        AS11  22
chardata:(AS11C, DETERMINE CRYSTAL DECAY CORRECTIONS $#                 AS11  23
TYPE               )#                                                   AS11  24
chardata:(AS11D, APPLY CORRECTIONS TO OUTPUT $#                         AS11  25
INTENSITIES                )#                                           AS11  26
chardata:(AS11E, ACCEPT DECAY COEFFICIENTS FROM LINE $#                 AS11  27
INPUT         YES  )#                                                   AS11  28
chardata:(AS11F, ACCEPT TAU ABSORPTION CORRECTIONS FROM LINE $#         AS11  29
INPUT YES  )#                                                           AS11  30
chardata:(AS11G,[ REFLECTIONS FROM NUMBER       TO NUMBER        $#     AS11  31
ARE TREATED, OTHERS COPIED.])#                                          AS11  32
chardata:(AS11I,[ REFLECTIONS FROM TWO-THETA         $#                 AS11  33
TO TWO-THETA          ARE TREATED, OTHERS COPIED.])#                    AS11  34
chardata:(AS11H,[ PURGE MONITORING REFLECTIONS $#                       AS11  35
& VOIDS FROM OUTPUT       ])#                                           AS11  36
chardata:(AS11J,[ RANGE OF REFLECTIONS TO EXCLUDE FROM DECAY $#         AS11  37
CALCULATION                   ])#                                       AS11  38
chardata:(AS11K,[ ADJUSTMENTS TO REFLECTION NO. AND EXPOSURE $#         AS11  39
TIME                          ])#                                       AS11  40
realdata:(FMT,[300513.,470613.])#                                       AS11  41
realdata:(FMQ,[350622.,570622.])#                                       AS11  42
realdata:(FMTJ,[640613.,700613.])#                                      AS11  43
realdata:(FMTK,[640613.,700622.])#                                      AS11  44
datastuff:#                                                             AS11  45
#                                                                       AS11  46
# SET DEFAULT VALUES AND INITIALIZATION                                 AS11  47
# -------------------------------------                                 AS11  48
#                                                                       AS11  49
APPLY=yes:#                            Update bdf this run              AS11  50
TAUFLG = yes:#                         Apply absorption                 AS11  51
PURGE = yes:#                          Clean lrrefl on output           AS11  52
DKFLG = yes:#                          Apply data derived decay sca     AS11  53
DKFLGL = no:#                          Apply line input decay scale     AS11  54
TIMEFL = yes:#                         Set use of exp time from ref     AS11  55
TIMEPR = 60.0#                         Default seconds/reflection       AS11  56
NTAU = 0#                              Initialize input tau correct     AS11  57
EXFRST = -1000000#                     First and last reflect exclude   AS11  58
EXLAST = -1000000#                     From use in decay calc.          AS11  59
SCFRST = 0#                            First reflect to apply scale     AS11  60
SCLAST = 1000000#                      Last reflect to apply scale      AS11  61
TTLOW=0.0 ; TTHIGH=180.0#              Range of two theta to scale      AS11  62
DKTYPE = 2#                                                             AS11  63
NSTAND = 0#                            Count of monitor reflections     AS11  64
COEF(1) = 1.0#                                                          AS11  65
DO I=2,maxcoef:#                                                        AS11  66
COEF(I) = 0.0#                                                          AS11  67
DO I=1,3#                              Clear absorption reflection      AS11  68
TAUHKL(I)=0.0#                         Indices as signal to as12        AS11  69
TAUMIN = 0#                            Clear absorption reflection      AS11  70
TAUMAX = 0#                            Number range                     AS11  71
ADDREF = 0.0#                          Reflection no. offset            AS11  72
ADDEXP = 0.0#                          Exposure time offset             AS11  73
#                                                                       AS11  74
# DECODE ABSCAL LINE PARAMETERS                                         AS11  75
# ----------------------------                                          AS11  76
#                                                                       AS11  77
FOR(FIELD=1;FIELD>0;FIELD=FIELD+1)#    Loop over input fields until     AS11  78
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS11  79
ifvo:(BUFIN(FIELD)) BREAK#                                              AS11  80
stringlim:(FIELD,FIRST,LAST)#          Get limits of char field         AS11  81
compchar:(CHRIN,FIRST,AS02,1,4,5,NAS02,MATCH)# Find option code         AS11  82
IF(MATCH == 0) iquit:(AS1101)#         Unrecognized code                AS11  83
ELSE IF (MATCH==1) TAUFLG = no:#       Do not calc. absorption corr     AS11  84
ELSE IF (MATCH==2) TAUFLG = yes:#      Do calc. absorption correcti     AS11  85
ELSE IF (MATCH==3) DKFLG  = no:#       Do not calc. decay correct       AS11  86
ELSE IF (MATCH==4)#                                                     AS11  87
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS11  88
DKFLG  = yes:#                         Do calc. decay correction        AS11  89
IF(BUFIN(FIELD+1)>0.5&BUFIN(FIELD+1)<3.5)# Test next field for number   AS11  90
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS11  91
DKTYPE = NINT(BUFIN(FIELD+1))#         Only 1, 2 or 3 allowed           AS11  92
FIELD = FIELD + 1#                     Skip numeric field               AS11  93
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS11  94
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS11  95
ELSE IF (MATCH==5) APPLY  = no:#       Do no apply correct to refle     AS11  96
ELSE IF (MATCH==6) APPLY  = yes:#      Apply corrections to reflect     AS11  97
ELSE IF (MATCH==7) PURGE  = yes:#      Drop void and monitor refs       AS11  98
ELSE IF (MATCH==8) PURGE  = no:#       From lrrefl: of output bdf       AS11  99
ELSE IF(MATCH==9)#                     Time per reflection given?       AS11 100
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS11 101
TIMEFL = no:#                          Set flag to use sequence num     AS11 102
IF(BUFIN(FIELD+1)>=1.0)#               Next field a number?             AS11 103
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS11 104
FIELD = FIELD + 1#                     Skip numeric field               AS11 105
TIMEPR = BUFIN(FIELD)#                 Exposure time per reflection     AS11 106
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS11 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS11 108
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS11 109
#                                                                       AS11 110
#    ENTER CONTROL LINES                                                AS11 111
#    -------------------                                                AS11 112
#                                                                       AS11 113
REPEAT#                                Loop over control lines          AS11 114
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS11 115
readline:(AS01,NAS01)#                 Enter control lines              AS11 116
#                                                                       AS11 117
IF(LINID==1)#                          *********** end line             AS11 118
BREAK#                                 Go start processing bdf          AS11 119
#                                                                       AS11 120
ELSE IF(LINID==2)#                     *********** absorp line          AS11 121
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS11 122
DO I=1,3#                                                               AS11 123
ifnv:(BUFIN(I)) TAUHKL(I) = BUFIN(I)#  Get hkl of reflectio   as11      AS11 124
#                                        To use for absorption          AS11 125
ifnv:(BUFIN(4)) TAUMIN = INT(BUFIN(4))## Get refl no. minimum and       AS11 126
ifnv:(BUFIN(5)) TAUMAX = INT(BUFIN(5))## Maximum for range              AS11 127
IF (TAUMIN!=0 & TAUMAX!=0$#                                             AS11 128
  & TAUMAX <= TAUMIN) iquit:(AS1107)#  Bad refl. range                  AS11 129
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS11 130
#                                                                       AS11 131
ELSE IF(LINID==3)#                     *********** tau line             AS11 132
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS11 133
ifvo:(BUFIN(1)) iquit:(AS1102)#        Missing phi on tau line          AS11 134
ifvo:(BUFIN(2)) iquit:(AS1103)#        Missing intensity on tau lin     AS11 135
IF(NTAU==0)#                           First time only                  AS11 136
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS11 137
TAUFLG=no:#                            Shut off automatic absorptio     AS11 138
PURGE=no:#                             And do not remove monitor refn   AS11 139
writeline:(1,AS11F,NAS11F,3,3)#        Declare intention to use lin     AS11 140
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS11 141
IF(NTAU>=maxtau:) iquit:(AS1106)#      Too many angles for space        AS11 142
NTAU = NTAU + 1#                       Incre number of input absorp     AS11 143
PHI(NTAU) = BUFIN(1)#                                                   AS11 144
INTENS(NTAU) = BUFIN(2)#                                                AS11 145
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS11 146
#                                                                       AS11 147
ELSE IF(LINID==4)#                     *********** decay line           AS11 148
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS11 149
NCOEF = 0#                                                              AS11 150
DKFLG=no: ; DKFLGL=yes:#               Set switches to use line inp     AS11 151
writeline:(1,AS11E,NAS11E,3,3)#        Witness use of line input        AS11 152
DO I=1,maxcoef:#                                                        AS11 153
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS11 154
ifnv:(BUFIN(I))#                                                        AS11 155
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS11 156
NCOEF = NCOEF+1#                                                        AS11 157
COEF(NCOEF) = BUFIN(I)#                Get scale coefficients           AS11 158
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS11 159
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS11 160
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS11 161
ELSE IF(LINID==5)#                     *********** stand line           AS11 162
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS11 163
IF(NSTAND>=maxmon:) iquit:(AS1105)#    Too many monitors for space      AS11 164
NSTAND = NSTAND + 1#                   Read next desired standard       AS11 165
DO I = 1,3#                                                             AS11 166
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AS11 167
ifvo:(BUFIN(I)) iquit:(AS1104)#        Void reflection index            AS11 168
DKHKL(I,NSTAND) = BUFIN(I)#            Get indices of standard          AS11 169
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AS11 170
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS11 171
ELSE IF(LINID==6)#                     *********** apply line           AS11 172
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AS11 173
ifnv:(BUFIN(1)) SCFRST = BUFIN(1)#     Initial reflect to scale         AS11 174
ifnv:(BUFIN(2)) SCLAST = BUFIN(2)#     Last reflect to scale            AS11 175
ifnv:(BUFIN(3)) TTLOW = BUFIN(3)#      Initial reflect to scale         AS11 176
ifnv:(BUFIN(4)) TTHIGH = BUFIN(4)#     Last reflect to scale            AS11 177
ncodefld:(BUFIN,1,AS11G,FMT,2)#        Report range                     AS11 178
writeline:(1,AS11G,NAS11G,3,3)#                                         AS11 179
ncodefld:(BUFIN(3),1,AS11I,FMQ,2)#     Report range                     AS11 180
writeline:(1,AS11I,NAS11I,3,3)#                                         AS11 181
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AS11 182
ELSE IF(LINID==7)#                     *********** exclude line         AS11 183
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AS11 184
ifnv:(BUFIN(1)) EXFRST = BUFIN(1)#     Region to exclude from           AS11 185
ifnv:(BUFIN(2)) EXLAST = BUFIN(2)#     Decay calculation                AS11 186
ncodefld:(BUFIN,1,AS11J,FMTJ,2)#                                        AS11 187
writeline:(1,AS11J,NAS11J,3,3)#                                         AS11 188
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AS11 189
ELSE IF(LINID==8)#                     *********** offset line          AS11 190
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AS11 191
ifnv:(BUFIN(1)) ADDREF = BUFIN(1)#     Value used to adjust refl #'s    AS11 192
ELSE            BUFIN(1) = 0.0#                                         AS11 193
ifnv:(BUFIN(2)) ADDEXP = BUFIN(2)*3600.# Value used to adjust exp tim   AS11 194
ELSE            BUFIN(2) = 0.0#                                         AS11 195
ncodefld:(BUFIN,1,AS11K,FMTK,2)#                                        AS11 196
writeline:(1,AS11K,NAS11K,3,3)#                                         AS11 197
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AS11 198
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS11 199
IF(TAUFLG==yes:) movechar:(AS11A,1,AS11B,52,3,0)#                       AS11 200
ELSE             movechar:(AS11A,4,AS11B,52,2,0)#                       AS11 201
writeline:(1,AS11B,NAS11B,3,3)#                                         AS11 202
IF(DKFLG==yes:) movechar:(AS11A,1,AS11C,52,3,0)#                        AS11 203
ELSE            movechar:(AS11A,4,AS11C,52,2,0)#                        AS11 204
BUFOT(1)=FLOAT(DKTYPE)#                                                 AS11 205
ncodefld:(BUFOT,1,AS11C,430113.,1)#                                     AS11 206
writeline:(1,AS11C,NAS11C,3,3)#                                         AS11 207
IF(APPLY==yes:) movechar:(AS11A,1,AS11D,52,3,0)#                        AS11 208
ELSE            movechar:(AS11A,4,AS11D,52,2,0)#                        AS11 209
writeline:(1,AS11D,NAS11D,3,3)#                                         AS11 210
IF(PURGE==yes:) movechar:(AS11A,1,AS11H,52,3,0)#                        AS11 211
ELSE            movechar:(AS11A,4,AS11H,52,2,0)#                        AS11 212
writeline:(1,AS11H,NAS11H,3,3)#                                         AS11 213
RETURN#                                                                 AS11 214
END#                                                                    AS11 215
systemheader:(AS12)#                   Bdf calibration pass             AS12   1
###################################################################     AS12   2
#                                                                 #     AS12   3
#                            AS12                                 #     AS12   4
#                                                                 #     AS12   5
###################################################################     AS12   6
SUBROUTINE AS12#                       K.d.watenpaugh                   AS12   7
# PROCESS  INFORMATION ON ASBSORPTION AND DECAY FROM SPECIAL            AS12   8
# REFLECTIONS ON THE BDF                                                AS12   9
include:(AACOMN)#                                                       AS12  10
include:(ASCOMN)#                                                       AS12  11
INTEGER IP,NPAK#                                                        AS12  12
INTEGER IREL(12),KEY(4)#               Lr refl wanted items             AS12  13
INTEGER N4DK#                                                           AS12  14
INTEGER RFTYPE#                                                         AS12  15
INTEGER USE#                                                            AS12  16
REAL AH(3)#                            H,k,l indices                    AS12  17
intdata:(IWNT20,[ 1,1005,1006,1007,1008,$#                              AS12  18
1018, 1300, 1301,1014,1027])#                                           AS12  19
realdata:(DTOR,[0.017453292])#         Degrees to radians               AS12  20
chardata:(AS12A, EXPOSURE TIMES ESTIMATED FROM REFLECTION SEQUENCE $#   AS12  21
NUMBER TIMES        SECONDS PER REFLECTION.)#                           AS12  22
datastuff:#                                                             AS12  23
#                                                                       AS12  24
# INITIALIZE                                                            AS12  25
N4DK = DKMRK-DKSTEP#                   Standards storage pointer        AS12  26
#                                        Into qx array                  AS12  27
#                                                                       AS12  28
#    SEARCH DIRECTORY OF LRREFL:                                        AS12  29
#    ---------------------------                                        AS12  30
#                                                                       AS12  31
KEY(1)=8; KEY(2)=10; KEY(3)=0; KEY(4)=0# Set search keys                AS12  32
indexpkt:(1,lrrefl:,NPAK,IP,0,KEY,IWNT20,IREL)# Search lrrefl directo   AS12  33
IF(KEY(4)<=0) iquit:(AS1206)#          Want items missing               AS12  34
#                                                                       AS12  35
# STORE POINTERS TO REFLECTION ITEMS                                    AS12  36
NHKL = IREL(1)#                        Hkl                              AS12  37
NPHI = IREL(2)#                        Phi                              AS12  38
NCHI = IREL(3)#                        Chi                              AS12  39
NOME = IREL(4)#                        Omega                            AS12  40
NTTH = IREL(5)#                        Two theta                        AS12  41
NRFT = IREL(6)#                        Reflection code                  AS12  42
NIRL = IREL(7)#                        Relative intensity               AS12  43
NSIR = IREL(8)#                        Sigma relative intensity         AS12  44
NEXP = IREL(9)#                        Exposure time                    AS12  45
NRFN = IREL(10)#                       Reflection number                AS12  46
#                                                                       AS12  47
#    CHECK ON METHOD OF OBTAINING CRYSTAL EXPOSURE TIMES                AS12  48
#                                                                       AS12  49
IF(DKFLG==yes:)#                       If the decay flag is on          AS12  50
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS12  51
IF(NEXP==0 & NRFN==0) iquit:(AS1207)#  Die if no exposure time info     AS12  52
IF(NEXP==0 & NRFN!=0)#                 Check if no exposure times       AS12  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS12  54
TIMEFL = no:#                          Turn off use of file time        AS12  55
ncodefld:(TIMEPR,1,AS12A,700512.,1)#   Report action                    AS12  56
writeline:(1,AS12A,NAS12A,3,3)#                                         AS12  57
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS12  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS12  59
#                                                                       AS12  60
writeline:(3,0,0,1,3); LINRM=6#        Force first header               AS12  61
#                                                                       AS12  62
#    ENTER LRREFL: BDF RECORDS                                          AS12  63
#    -------------------------                                          AS12  64
#                                                                       AS12  65
NREF = -100000#                                                         AS12  66
IF (TAUMAX!=0 & NRFN <=0) iquit:(AS1208)# Desired reflect number        AS12  67
#                                        Needed is not present          AS12  68
REPEAT#                                Loop over lrrefl records         AS12  69
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS12  70
readwpkt:(1,lrrefl:,NPAK,IP,0)#        Point to ip packet               AS12  71
IF(IP<=0) BREAK#                       Exit reflection loop             AS12  72
ifvo:(QX(IP+NIRL)) NEXT#               Scrub unknown reflections        AS12  73
gethkl:(IP+NHKL,AH)#                   Unpack hkl                       AS12  74
RFTYPE = NINT(QX(IP+NRFT))#            Store reflection code            AS12  75
IF (NRFN>0) NREF = NINT(QX(IP+NRFN) + ADDREF)# Get reflection number    AS12  76
IF (TAUFLG == yes:)#                                                    AS12  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS12  78
IF (TAUMIN!=0 & TAUMAX!=0)#            Test for absorption range        AS12  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS12  80
IF (RFTYPE==2) RFTYPE = 0#             Reset as normal reflection       AS12  81
IF (NREF>=TAUMIN & NREF<=TAUMAX)#      Check on absorption range        AS12  82
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS12  83
RFTYPE = 2#                            Code as absorption               AS12  84
#                                              Reflection               AS12  85
IF(TAUHKL(1)!=0&TAUHKL(2)!=0&TAUHKL(3)!=0)# Check on indices            AS12  86
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS12  87
IF (RFTYPE==2) RFTYPE = 0#             Reset as normal reflection       AS12  88
IF (AH(1)==TAUHKL(1)&AH(2)==TAUHKL(2)&AH(3)==TAUHKL(3))#                AS12  89
RFTYPE = 2#                                                             AS12  90
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS12  91
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS12  92
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS12  93
ELSE IF(TAUHKL(1)!=0&TAUHKL(2)!=0&TAUHKL(3)!=0)# Check on indices       AS12  94
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS12  95
IF (RFTYPE==2) RFTYPE = 0#             Reset as normal reflection       AS12  96
IF (AH(1)==TAUHKL(1)&AH(2)==TAUHKL(2)&AH(3)==TAUHKL(3))#                AS12  97
RFTYPE = 2#                                                             AS12  98
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS12  99
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS12 100
IF (RFTYPE==2)#                        Phi scan reflection              AS12 101
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS12 102
IF (TAUFLG==yes:)#                     Get tau reflect from bdf         AS12 103
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS12 104
IF(TAUHKL(1)==0&TAUHKL(2)==0&TAUHKL(3)==0)# None defined?               AS12 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS12 106
TAUHKL(1)=AH(1);TAUHKL(2)=AH(2);TAUHKL(3)=AH(3)# Accept first one as    AS12 107
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS12 108
IF (AH(1)==TAUHKL(1)&AH(2)==TAUHKL(2)&AH(3)==TAUHKL(3))# Use this       AS12 109
#                                        Phi scan reflection            AS12 110
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS12 111
NTAU = NTAU + 1#                       Incre number of input absorp     AS12 112
IF (NTAU>maxtau:) iquit:(AS1201)#      Too many tau values              AS12 113
PHI(NTAU) =QX(IP+NPHI)#                Save this phi value              AS12 114
INTENS(NTAU) = QX(IP+NIRL)#            Save this intensity              AS12 115
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS12 116
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS12 117
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS12 118
ELSE IF (RFTYPE==1)#                   Decay monitor reflection         AS12 119
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS12 120
IF (DKFLG==yes:)#                      Get decay monitors from bdf      AS12 121
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AS12 122
USE = no:#                                                              AS12 123
gethkl:(IP+NHKL,AH)#                                                    AS12 124
IF(PPRIOR >3)#                                                          AS12 125
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AS12 126
WRITE(6,1001) AH,QX(IP+NIRL),QX(IP+NRFN),QX(IP+NEXP)#                   AS12 127
1001 FORMAT(3F5.0,F8.2,F6.0,F6.2)#                                      AS12 128
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AS12 129
IF (NSTAND==0) USE = yes:#             Use all monitors reflections     AS12 130
ELSE#                                  Check against list               AS12 131
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AS12 132
DO I=1,NSTAND#                                                          AS12 133
IF (DKHKL(1,I)==AH(1) & DKHKL(2,I)==AH(2) $#                            AS12 134
 & DKHKL(3,I)==AH(3))#                 Use this one from bdf            AS12 135
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AS12 136
USE = yes:#                                                             AS12 137
BREAK#                                                                  AS12 138
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AS12 139
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AS12 140
IF (NRFN>0) NREF = QX(IP+NRFN) + INT(ADDREF)#                           AS12 141
USE = yes:#                                                             AS12 142
IF (NREF>=EXFRST&NREF<=EXLAST) USE = no:#                               AS12 143
IF (USE == yes:)#                                                       AS12 144
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AS12 145
NDK = NDK + 1#                         As                               AS12 146
IF (NDK>maxdk:) iquit:(AS1202)#        Too many monitor reflections     AS12 147
N4DK = N4DK+DKSTEP#                                                     AS12 148
movepakwd:(QX(IP+NHKL),QX(N4DK+1))#    Store this hkl                   AS12 149
IF(TIMEFL==no:) QX(N4DK+2)=TIMEPR*QX(IP+NRFN)# Estimate or take         AS12 150
ELSE QX(N4DK+2) = QX(IP+NEXP) + ADDEXP# X-ray exposure in seconds       AS12 151
QX(N4DK+3) = DTOR*QX(IP+NTTH)#         Two-theta                        AS12 152
QX(N4DK+4) = QX(IP+NIRL)#              Relative intensity               AS12 153
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AS12 154
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AS12 155
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS12 156
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS12 157
readwpkt:(1,endrecord:,NPAK,IP,0)#     Finish off input file            AS12 158
IF(TAUFLG==yes: & NTAU<18) iquit:(AS1203)# Protect absorb step size     AS12 159
#                                                                       AS12 160
RETURN#                                                                 AS12 161
END#                                                                    AS12 162
systemheader:(AS13)#                   Build absorption scales          AS13   1
# ################################################################## #  AS13   2
#                                                                    #  AS13   3
#                               AS13                                 #  AS13   4
#                                                                    #  AS13   5
# ################################################################## #  AS13   6
SUBROUTINE AS13#                       K.d.watenpaugh                   AS13   7
# PROCESS A SET OF PHI ANGLES AND CORRESPONDING                         AS13   8
# INTEGRATED INTENSITIES TO BE USED IN GENERATING AN                    AS13   9
# ABSORPTION CORRECTION CURVE.  THIS CURVE CAN THEN BE USED             AS13  10
# TO CORRECT EACH COLLECTED INTENSITY.                                  AS13  11
#                                                                       AS13  12
include:(AACOMN)#                                                       AS13  13
include:(ASCOMN)#                                                       AS13  14
#                                                                       AS13  15
INTEGER DELTA#                         Difference between 1st and       AS13  16
#                                         Last phi angle in the table   AS13  17
INTEGER I#                             Loop index                       AS13  18
INTEGER IJ#                            Points to end of phi table       AS13  19
INTEGER JJ#                            Pointer to un-normalized         AS13  20
#                                         Intensity.                    AS13  21
INTEGER JPHI(arith:(arith:(maxtau:,*,2),+,4))# Table of co              AS13  22
#                                         Angle/intensity data.         AS13  23
INTEGER K#                             Loop index                       AS13  24
INTEGER M#                             Phi table pointer                AS13  25
INTEGER MKPTR#                         Points to specific phi angle     AS13  26
INTEGER N#                             Index into unsorted phi tabl     AS13  27
INTEGER NGAP#                          Number of table gaps greater     AS13  28
#                                         Than 20 degrees.              AS13  29
INTEGER NT#                            Number of angles minus 1         AS13  30
INTEGER TAUPTR#                        Points to specific absorptio     AS13  31
#                                         Correction.                   AS13  32
REAL PHITAU(5)#                        Array for listing purposes       AS13  33
REAL PKMAX#                            Maximum integrated peak coun     AS13  34
#                                                                       AS13  35
chardata:(AS133, THE FOLLOWING ABSORPTION CURVE IS BASED ON $# A        AS13  36
THE               REFLECTION)#                                          AS13  37
chardata:(AS134, [ PHI    NORMALIZED-I ABS CORRECTION   PEAK   $#       AS13  38
ORIGINAL ORDER])#                                                       AS13  39
#                                                                       AS13  40
realdata:(FMT132,[520413.,560413.,600413.])#                            AS13  41
#                                                                       AS13  42
realdata:(FMT133,[80722.,160842.,311042.,450812.,550413.])#             AS13  43
#                                                                       AS13  44
datastuff:#                                                             AS13  45
#                                                                       AS13  46
movechar:(AS133,1,CHROT,1,NAS133,0)#   Now tell user the reflection     AS13  47
ncodefld:(TAUHKL,1,CHROT,FMT132(1),3)# Used for collecting tau          AS13  48
writeline:(1,0,0,1,3)#                 Absorption curve.                AS13  49
#                                                                       AS13  50
# THE PHI VALUES DO NOT HAVE TO BE IN ANY PARTICULAR ORDER.  JUST BE    AS13  51
#  SURE THEY ARE NOT SEPARATED BY MORE THAN 20-DEGREE INCREMENTS WHEN   AS13  52
#  SORTED INTO CONTIGUOUS POINTS.  DO NOT HAVE ANY DUPLICATES MODULO    AS13  53
#  360.                                                                 AS13  54
#                                                                       AS13  55
PKMAX=-99999.0#                        Find maximum intenstiy           AS13  56
DO I=1,NTAU#                                                            AS13  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS13  58
IF(INTENS(I) > PKMAX)  PKMAX = INTENS(I)# Update maximum intensity      AS13  59
N = 2*I - 1#                           Get subscript for storing        AS13  60
#                                         Phi/index pairs.              AS13  61
JPHI(N) = MOD(NINT(100.*PHI(I))+72000,36000)# Integerize angle for so   AS13  62
#                                        Make angle positive            AS13  63
JPHI(N+1) = I#                         Retain the original index        AS13  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS13  65
CALL AS14(JPHI,NTAU)#                  Sort the angles retaining        AS13  66
#                                        The original pointers.         AS13  67
NGAP = 0#                              Initialize gap counter.  no      AS13  68
#                                         Gaps greater than 20          AS13  69
#                                         Degrees allowed.              AS13  70
NT = NTAU - 1#                         Set number of gaps to be         AS13  71
#                                         Checked.                      AS13  72
#                                                                       AS13  73
# NOW ARRANGE THE ANGLES INTO CONTIGUOUS INCREASING VALUES WITH NO      AS13  74
#  GAPS GREATER THAN 21.                                                AS13  75
REPEAT#                                                                 AS13  76
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS13  77
FOR(I=1; I <= NT; I=I+1)#              Start gap-checking loop          AS13  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS13  79
M = I + I - 1#                         First angle pointer              AS13  80
N = M + 2#                             Second angle pointer             AS13  81
IF((JPHI(N)-JPHI(M)) > 2100)  BREAK#                                    AS13  82
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS13  83
IF(I > NT)  BREAK#                     If no gaps found, then           AS13  84
iquit:(AS1303)#                        Otherwise it's dead meat         AS13  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS13  86
#                                                                       AS13  87
IJ = NTAU + NTAU - 1#                  Pointer to last angle            AS13  88
DELTA = JPHI(IJ) - JPHI(1)#            Get difference between 1st       AS13  89
#                                         And last angle.               AS13  90
# IF GREATER THAN 340 DEGREES, MAKE SURE THE CIRCLE IS COMPLETE         AS13  91
IF(DELTA >= 33900)#                                                     AS13  92
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS13  93
#                                                                       AS13  94
# IF NECESSARY, CREATE AN END POINT EQUIVALENT TO THE FIRST POINT       AS13  95
IF(DELTA != 36000)#                                                     AS13  96
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS13  97
NTAU = NTAU + 1#                       Update the angle counter         AS13  98
IJ = IJ + 2#                           And pointer.                     AS13  99
JPHI(IJ) = JPHI(1) + 36000#                                             AS13 100
JPHI(IJ+1) = JPHI(2)#                                                   AS13 101
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS13 102
#                                                                       AS13 103
JPHI(IJ+2) = JPHI(3) + 36000#          Extend the circle by putting     AS13 104
JPHI(IJ+3) = JPHI(4)#                  Points on both ends,             AS13 105
JPHI(IJ+4) = JPHI(IJ-2) - 36000#       So that when complete            AS13 106
JPHI(IJ+5) = JPHI(IJ-1)#               The values will range from       AS13 107
#                                         -10 to 370 degrees.  this is  AS13 108
#                                         To allow the interpolater to  AS13 109
#                                         Work properly.  otherwise, it AS13 110
#                                         Is up to the user to supply   AS13 111
#                                         Enough points at each end of  AS13 112
#                                         The curve.                    AS13 113
NTAU = NTAU + 2#                       Update number of points          AS13 114
#                                                                       AS13 115
CALL AS14(JPHI,NTAU)#                  Sort once again                  AS13 116
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS13 117
ELSE iquit:(AS1301)#                   No full phi spin, quit           AS13 118
#                                                                       AS13 119
writeline:(1,AS134,NAS134,3,3)#        Write heading for curve          AS13 120
MKTAU = TAUMRK + NTAU#                 Pointer in qx to absorption      AS13 121
#                                         Correction factors.           AS13 122
#                                                                       AS13 123
# LOOP OVER ALL THE POINTS STORING AND PRINTING THEM                    AS13 124
FOR(I=1; I <= NTAU; I=I+1)#                                             AS13 125
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS13 126
MKPTR = TAUMRK + I#                                                     AS13 127
K = 2*I#                                                                AS13 128
QX(MKPTR) = .01*FLOAT(JPHI(K-1))#      Store angle                      AS13 129
PHITAU(1) = QX(MKPTR)#                 Also save in print buffer        AS13 130
JJ = JPHI(K)#                          Pick up original index           AS13 131
PHITAU(5)=FLOAT(JJ)#                   And put it to output             AS13 132
PHITAU(2) = INTENS(JJ)/PKMAX#          Normalize intensity for list     AS13 133
TAUPTR = MKTAU + I#                                                     AS13 134
QX(TAUPTR) = 1./PHITAU(2)#             Store absorption correction      AS13 135
PHITAU(3) = QX(TAUPTR)#                Also save for listing            AS13 136
PHITAU(4) = INTENS(JJ)#                Original collected intensity     AS13 137
#                                                                       AS13 138
ncodefld:(PHITAU,1,CHROT,FMT133(1),5)## Encode listing buffer           AS13 139
writeline:(0,AS134,NAS134,1,3)#        Write out line                   AS13 140
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS13 141
#                                                                       AS13 142
FOR(I=2; I <= NTAU; I=I+1)#            If there are any duplicates,     AS13 143
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS13 144
MKPTR = TAUMRK + I#                                                     AS13 145
ifeq:(QX(MKPTR),QX(MKPTR-1))  iquit:(AS1304)#                           AS13 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS13 147
#                                                                       AS13 148
RETURN#                                                                 AS13 149
END#                                                                    AS13 150
systemheader:(AS14)#                   Two word integer quick sort      AS14   1
# ################################################################## #  AS14   2
#                                                                    #  AS14   3
#                               AS14                                 #  AS14   4
#                                                                    #  AS14   5
# ################################################################## #  AS14   6
SUBROUTINE AS14(NUM,NO)#                                                AS14   7
#                                                                       AS14   8
# QUICKSORT OF 'NO' 2-WORD INTEGER VARIABLES                            AS14   9
#  BASED ON CODE FOUND IN SOFTWARE TOOLS.                               AS14  10
#                                                                       AS14  11
#                                        May 1979  r.a. alden           AS14  12
#                                                                       AS14  13
INTEGER I#                             Lower partition index            AS14  14
INTEGER J#                             Upper partition index            AS14  15
INTEGER LV(20)#                        Lower partition points           AS14  16
INTEGER NO#                            Number of 2-word integers to     AS14  17
#                                         Be sorted.                    AS14  18
INTEGER NUM(10)#                       Table of 2-word integers to      AS14  19
#                                         Sorted.                       AS14  20
INTEGER P#                             Stack pointer                    AS14  21
INTEGER PIVPT#                         Current pivot point              AS14  22
INTEGER TEMP#                          Scratch storage during           AS14  23
#                                         Interchange.                  AS14  24
INTEGER UV(20)#                        Upper partition points           AS14  25
#                                                                       AS14  26
LV(1) = 1#                             Set lower partition              AS14  27
UV(1) = 2*NO - 1#                      Set upper partition              AS14  28
P = 1#                                 Stack pointer for simulated      AS14  29
#                                         Recursion.                    AS14  30
WHILE(P > 0)#                                                           AS14  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS14  32
IF(LV(P) >= UV(P))  P = P - 1#         If subset has only one           AS14  33
#                                         Element, pop the stack.       AS14  34
ELSE#                                  Do partitioning                  AS14  35
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS14  36
I = LV(P) - 2#                         The elements are rearranged      AS14  37
J = UV(P)#                             Entirely within the subset       AS14  38
PIVPT = NUM(J)#                        Of num between the upper         AS14  39
#                                         And lower limits              AS14  40
WHILE(I < J)#                                                           AS14  41
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS14  42
FOR(I=I+2; NUM(I) < PIVPT; I=I+2)#     Increase i                       AS14  43
;#                                                                      AS14  44
FOR(J=J-2; J > I; J=J-2)#              Decrease j                       AS14  45
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS14  46
# AT THIS POINT, EITHER I AND J HAVE MET OR WE HAVE AN                  AS14  47
#  OUT-OF-ORDER PAIR.                                                   AS14  48
IF(NUM(J) <= PIVPT)  BREAK#                                             AS14  49
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS14  50
IF(I < J)  exchangei:#                 Exchange out-of-order pair       AS14  51
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS14  52
#                                                                       AS14  53
# I AND J HAVE MET.  MOVE THE PIVOT ELEMENT TO THE "MIDDLE"             AS14  54
#                                                                       AS14  55
J = UV(P)#                                                              AS14  56
exchangei:#                            Exchange num(i) and last pt.     AS14  57
IF(I-LV(P) < UV(P)-I)#                 Stack so short partition don     AS14  58
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS14  59
LV(P+1) = LV(P)#                                                        AS14  60
UV(P+1) = I - 2#                                                        AS14  61
LV(P) = I + 2#                                                          AS14  62
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS14  63
ELSE#                                                                   AS14  64
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS14  65
LV(P+1) = I + 2#                                                        AS14  66
UV(P+1) = UV(P)#                                                        AS14  67
UV(P) = I - 2#                                                          AS14  68
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS14  69
P = P + 1#                             Push onto stack                  AS14  70
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS14  71
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS14  72
RETURN#                                                                 AS14  73
END#                                                                    AS14  74
systemheader:(AS15)#                   Printer plot of absorption       AS15   1
# ################################################################## #  AS15   2
#                                                                    #  AS15   3
#                               AS15                                 #  AS15   4
#                                                                    #  AS15   5
# ################################################################## #  AS15   6
SUBROUTINE AS15#                       Printer plot of absorption       AS15   7
#                                        K. watenpaugh, oct 1985        AS15   8
include:(AACOMN)#                                                       AS15   9
include:(ASCOMN)#                                                       AS15  10
INTEGER MAXPLT#                                                         AS15  11
INTEGER MKPTR#                         Pointer to phi of scales         AS15  12
INTEGER I,J,K#                         Temporary integers               AS15  13
chardata:(AS154,[        :])#                                           AS15  14
chardata:(STAR,[ *+])#                 Blank,asterisk,plus characte     AS15  15
chardata:(AS156,[    0.0 +-----+-----+-----+-----+-----+-----+-$#       AS15  16
----+-----+-----+-----+-----+-----+])#                                  AS15  17
chardata:(AS157,[        0    30    60    90   120   150   180$#        AS15  18
   210   240   270   300   330   360])#                                 AS15  19
chardata:(AS158,[                             ABSORPTION VS.$#          AS15  20
 PHI ANGLE])#                                                           AS15  21
realdata:(ZERO,[0.0])#                                                  AS15  22
datastuff:#                                                             AS15  23
#                                                                       AS15  24
# SET-UP POINTS OF THE ABSORPTION CORRECTION INTO AN ARRAY OF           AS15  25
# 72 COLUMNS OF 50 ROWS.  THEN ENCODE THAT INFORMATION FOR PRINTING     AS15  26
#                                                                       AS15  27
MAXPLT = 72*50#                                                         AS15  28
movereal:(ZERO,1,QX(PLTMRK+1),1,MAXPLT,1)# Zero the array               AS15  29
# PLOT INTERPOLATION OF ABSORPTION INTO THE PLOT ARRAY AS PLUSES        AS15  30
JPHI = 1#                                                               AS15  31
FOR (ANGLE=5.0;ANGLE<=360.0;ANGLE=ANGLE+5.0)#                           AS15  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS15  33
CALL AS04(ANGLE,QX(TAUMRK+1),QX(MKTAU+1),SCALE,NTAU,3)#                 AS15  34
IF (SCALE<1.0) SCALE = 1.0#                                             AS15  35
JSCALE = IFIX((1.0-1.0/SCALE)*50.0)#   Calc which row of plot           AS15  36
MKPTR = JSCALE*72 + JPHI#              Calc which column                AS15  37
QX(PLTMRK+MKPTR+1) = 1.0#                                               AS15  38
JPHI = JPHI + 1#                                                        AS15  39
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS15  40
# PLOT THE OBSERVED VALUES AS ASTERICKS                                 AS15  41
DO I=1,NTAU#                                                            AS15  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS15  43
MKPTR = TAUMRK+I#                                                       AS15  44
JPHI = NINT(QX(MKPTR)*0.2)#            Five degree increments           AS15  45
#                                        To store col.                  AS15  46
MKPTR = MKTAU + I#                                                      AS15  47
JSCALE = NINT((1.0-1.0/QX(MKPTR))*50.0)# 0.02 unit increments           AS15  48
#                                         To store row                  AS15  49
MKPTR = JSCALE*72 + JPHI#                                               AS15  50
QX(PLTMRK+MKPTR+1) = 2.0#                                               AS15  51
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS15  52
# NOW PRINT OUT THE PLOT                                                AS15  53
LINCT = 0#                             Force a new page                 AS15  54
writeline:(2,STAR,0,1,3)#              Space do a couple of lines       AS15  55
#                                                                       AS15  56
JP = 4#                                                                 AS15  57
DO I=1,50#                                                              AS15  58
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS15  59
JP = JP + 1#                                                            AS15  60
movechar:(STAR,1,CHROT,1,81,1)#                                         AS15  61
IF (JP!=5) movechar:(AS154,1,CHROT,1,NAS154,0)# A                       AS15  62
ELSE#                                  Set up tics on left axis         AS15  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS15  64
JP = 0#                                                                 AS15  65
###movechar:(AS155,1,CHROT,1,NAS155,0)#Put tic mark on left             AS15  66
SCALE = 0.02*FLOAT(50-I)#                                               AS15  67
ncodefld:(SCALE,1,CHROT,70612.,1)#                                      AS15  68
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS15  69
K = (I-1)*72#                                                           AS15  70
DO J=1,72#                             Bring plot points out of arr     AS15  71
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS15  72
MKPTR = PLTMRK + K + J#                                                 AS15  73
IF (QX(MKPTR)==1.0) movechar:(STAR,3,CHROT,J+8,1,0)# Put plus           AS15  74
IF (QX(MKPTR)==2.0) movechar:(STAR,2,CHROT,J+8,1,0)# Put asterisk       AS15  75
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS15  76
writeline:(0,0,0,1,3)#                                                  AS15  77
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS15  78
writeline:(0,AS156,NAS156,3,3)#        Print bottom axis                AS15  79
writeline:(0,AS157,NAS157,3,3)#        Print tau angles                 AS15  80
writeline:(0,AS158,NAS158,3,3)#        Print legend                     AS15  81
#                                                                       AS15  82
RETURN#                                                                 AS15  83
END#                                                                    AS15  84
systemheader:(AS16)#                   Monitor decay analysis           AS16   1
# ################################################################## #  AS16   2
#                                                                    #  AS16   3
#                               AS16                                 #  AS16   4
#                                                                    #  AS16   5
# ################################################################## #  AS16   6
SUBROUTINE AS16#                       Process a set of                 AS16   7
#                                        Decay monitors                 AS16   8
#                                        K. watenpaugh, oct 1985        AS16   9
include:(AACOMN)#                                                       AS16  10
include:(ASCOMN)#                                                       AS16  11
INTEGER MONPTR(arith:(maxmon:,+,1))#                                    AS16  12
INTEGER MKPTR#                                                          AS16  13
INTEGER MKPTR2#                                                         AS16  14
INTEGER I,J,K,JC,JD#                                                    AS16  15
INTEGER MEASUR#                                                         AS16  16
INTEGER IOBS#                                                           AS16  17
INTEGER IOBSS#                         Like iobs for pointing           AS16  18
REAL AH(3)#                            Tempoary storage in indices      AS16  19
REAL EXPOS#                                                             AS16  20
REAL SCALC#                                                             AS16  21
REAL FRST#                                                              AS16  22
REAL TTHETA#                                                            AS16  23
chardata:(AS161, FOR INDIVIDUAL MONITORS TYPE 1 PRELIMINARY SCALING)#   AS16  24
chardata:(AS16Z, FOR COMBINED MONITORS FINAL SCALE FACTORS)#            AS16  25
chardata:(AS162,[ MONITOR             IS OBSERVED      TIMES.$#         AS16  26
 TWO THETA =     ])#                                                    AS16  27
chardata:(AS163, [EXPOSURE(HR)  SCALE(OBS)  SCALE(CALC)])#              AS16  28
chardata:(AS16A,[                COEF1       COEF2       COEF3       $# AS16  29
COEF4       COEF5       COEF6])#                                        AS16  30
chardata:(AS16B,[COEFFICIENT])#                                         AS16  31
chardata:(AS16C,[SIGMA(C)])#                                            AS16  32
chardata:(AS16D,[ TOTAL NO. OF REFLECTIONS USED TO DETERMINE TYPE X $#  AS16  33
DECAY COEFFICIENTS IS          ])#                                      AS16  34
chardata:(AS16T, TYPE 3 DETERIORATION IS DEFINED BY)#                   AS16  35
chardata:(AS16S,[SCALE=COEF1+COEF2*EXPOS+COEF3*TTHETA+COEF4*EXPOS*$#    AS16  36
TTHETA+COEF5*EXPOS**2])#                                                AS16  37
chardata:(AS16W, TYPE 2 DETERIORATION IS DEFINED BY)#                   AS16  38
chardata:(AS16V, [SCALE=COEF1+COEF2*EXPOS+COEF3*TTHETA**2*EXPOS**2+])#  AS16  39
chardata:(AS16U, [      COEF4*TTHETA+COEF5*TTHETA**2*EXPOS+COEF6*$#     AS16  40
TTHETA**2])#                                                            AS16  41
chardata:(AS16Y, TYPE 1 DETERIORATION IS DEFINED BY)#                   AS16  42
chardata:(AS16X,[SCALE=COEF1+COEF2*EXPOS+COEF3*EXPOS**2])#              AS16  43
chardata:(AS1, REFLECTION             HAS TOO FEW MEASUREMENTS)#        AS16  44
realdata:(FMTDKB,[241151.,361151.,481151.,601151.,721151.,841151.])#    AS16  45
realdata:(FMT21,[120413.,160413.,200413.])#                             AS16  46
realdata:(FMT22,[370413.])#                                             AS16  47
realdata:(FMT23,[600622.])#                                             AS16  48
realdata:(FMT31,[100622.])#                                             AS16  49
realdata:(FMT32,[220642.])#                                             AS16  50
realdata:(FMTO,[500113.,790613.])#                                      AS16  51
realdata:(FMT1,[140413.,180413.,220413.])#                              AS16  52
realdata:(FMT33,[340642.])#                                             AS16  53
realdata:(RTOD,[57.295779])#                                            AS16  54
datastuff:#                                                             AS16  55
#                                                                       AS16  56
IF (NDK<=0) iquit:(AS1601)#            Shouldn't be in here             AS16  57
#                                        There is no data               AS16  58
# GET SOME SPACE TO STORE MONITORS SCALES                               AS16  59
SCLMRK = ENDMRK#                                                        AS16  60
incrqx:(ENDMRK,SCLMRK+NDK,AS1602)#     Insufficent memory for decay     AS16  61
NSTAND = 0#                                                             AS16  62
HKL = 0.0#                                                              AS16  63
DO I=1,maxmon:#                        Initialize monitor pointers      AS16  64
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS16  65
MONPTR(I) = 0#                                                          AS16  66
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS16  67
CALL AS18(QX(DKMRK+1),NDK)#            Sort same hkl's together         AS16  68
#                                                                       AS16  69
# SET UP POINTERS TO START-1 OF EACH MONITOR                            AS16  70
#                                                                       AS16  71
MKPTR = DKMRK+1#                                                        AS16  72
MKPTR2 = MKPTR + DKSTEP*NDK - 1#                                        AS16  73
DO I=MKPTR,MKPTR2,DKSTEP#                                               AS16  74
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS16  75
compakwd:(QX(I),EQ,HKL,NEXT)#                                           AS16  76
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS16  77
movepakwd:(QX(I),HKL)#                 New monitor, replace current     AS16  78
NSTAND = NSTAND + 1#                   With new hkl                     AS16  79
MONPTR(NSTAND) = I-1#                  Set end of new monitor           AS16  80
IF (NSTAND > maxmon:) iquit:(AS1603)#  Too many monitors                AS16  81
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS16  82
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS16  83
MONPTR(NSTAND+1) = MKPTR2#             End of last monitor array        AS16  84
#                                                                       AS16  85
# SORT EACH MONITOR ON INCREASING EXPOSURE TIME                         AS16  86
#                                                                       AS16  87
DO I=1,NSTAND#                                                          AS16  88
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS16  89
MKPTR = MONPTR(I)+1#                                                    AS16  90
MKPTR2 = MONPTR(I+1)#                                                   AS16  91
DO J=MKPTR,MKPTR2,DKSTEP#              Reverse hkl and exposure         AS16  92
#                                        And integerize exposure        AS16  93
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS16  94
JC = INT(QX(J+1))#                                                      AS16  95
movepakwd:(QX(J),QX(J+1))#                                              AS16  96
intpak:(JC,QX(J),0,32)#                                                 AS16  97
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS16  98
MEASUR = (MKPTR2-MKPTR+1)/DKSTEP#                                       AS16  99
CALL AS18(QX(MKPTR),MEASUR)#           Sort on exposure time            AS16 100
DO J=MKPTR,MKPTR2,DKSTEP#              Reverse back and float           AS16 101
#                                         Exposure time                 AS16 102
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS16 103
intunpak:(QX(J),JC,0,32)#              ????? changed inpak to intunpa   AS16 104
movepakwd:(QX(J+1),QX(J))#                                              AS16 105
QX(J+1) = FLOAT(JC)#                                                    AS16 106
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS16 107
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS16 108
#                                                                       AS16 109
# SET ARRAY OF SCALED MONITORS                                          AS16 110
#                                                                       AS16 111
IOBS = 0#                                                               AS16 112
DO I=1,NSTAND#                                                          AS16 113
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS16 114
MKPTR = MONPTR(I)+1#                                                    AS16 115
MKPTR2 = MONPTR(I+1)#                                                   AS16 116
FRST = QX(MKPTR+3)#                    Intensity of first one           AS16 117
DO K=MKPTR,MKPTR2,DKSTEP#                                               AS16 118
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS16 119
IOBS=IOBS+1#                                                            AS16 120
#                                                                       AS16 121
#   NORMALIZE ALL THE OBSERVATIONS FOR EACH MONITOR SO THAT THE FIRST   AS16 122
#   MEASURED POINT IS 1.0 SCALE FACTOR (NOT DETERIORATION).             AS16 123
#                                                                       AS16 124
QX(SCLMRK+IOBS) = FRST/QX(K+3)#                                         AS16 125
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS16 126
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS16 127
#                                                                       AS16 128
# BEGIN PROCESSING MONITORS TO FIND DECAY COEF                          AS16 129
# TREAT INDIVIDUAL MONITORS GROUPS FIRST                                AS16 130
# THIS SHOULD PUT EACH GROUP  ON PROPER INTERCEPT SCALING               AS16 131
#                                                                       AS16 132
IOBSS=0#                                                                AS16 133
IOBS = 1#                                                               AS16 134
LINRM = 6#                             Be sure space on page            AS16 135
writeline:(1,AS161,NAS161,3,3)#        Announce preliminary scaling     AS16 136
writeline:(1,AS16Y,NAS16Y,3,3)#                                         AS16 137
writeline:(0,AS16X,NAS16X,3,3)#                                         AS16 138
DO I=1,NSTAND#                                                          AS16 139
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS16 140
MKPTR = MONPTR(I)+1#                   Set range of monitor             AS16 141
MKPTR2 = MONPTR(I+1)#                                                   AS16 142
JC = SCLMRK+IOBS#                                                       AS16 143
MEASUR = (MKPTR2-MKPTR+1)/DKSTEP#      Number of times measured         AS16 144
IF (MEASUR <= 4       )#               Are there less measurements      AS16 145
#                                        Than possible variables        AS16 146
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS16 147
movechar:(AS1,1,CHROT,1,NAS1,0)#                                        AS16 148
gethkl:(MKPTR,AH)#                                                      AS16 149
ncodefld:(AH,1,CHROT,FMT1,3)#                                           AS16 150
writeline:(1,0,0,1,3)#                                                  AS16 151
iquit:(AS1604)#                        Insuficient measurements         AS16 152
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS16 153
#                                                                       AS16 154
#   EACH MONITOR REFLECTION IS FIRST TREATED BY TYPE ONE SCALING WHICH  AS16 155
#   IS DEPENDENT ONLY ON EXPOSURE TIME.  THIS IS NOT AN OPTION.         AS16 156
#   AFTER THE PRELIMINARY SCALES FOR EACH MONITOR ARE OBTAINED THE      AS16 157
#   COMBINED SCALE FACTOR, OF DKTYPE, IS FORMED USING ALL DATA FOR ALL  AS16 158
#   MONITOR REFLECTIONS WHICH HAVE ALREADY BEEN SCALED ONCE.            AS16 159
#                                                                       AS16 160
CALL AS17(MKPTR,MKPTR2,QX(JC),1)#      Call for scaling of type 1       AS16 161
JC = JC-1#                                                              AS16 162
DO JD=1,MEASUR#                        Reset the intercept              AS16 163
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS16 164
QX(JC+JD) = QX(JC+JD)/COEF(1)#         Rescale to 1.0 intercept         AS16 165
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS16 166
#                                                                       AS16 167
#   RESCALE THE COEFFICIENTS OF THE SCALE/DECAY EQUATION AS WELL        AS16 168
#                                                                       AS16 169
DO JD=2,NCOEF#                                                          AS16 170
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS16 171
COEF(JD) = COEF(JD)/COEF(1)#           Multiply equation by 1/c1        AS16 172
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS16 173
COEF(1) = 1.0#                         Set intercept, c1, to 1.0        AS16 174
IOBS = IOBS + MEASUR#                                                   AS16 175
#                                                                       AS16 176
# WRITE OUT THE COEFFICIENTS                                            AS16 177
#                                                                       AS16 178
LINRM=8#                                                                AS16 179
movechar:(AS16B,1,CHROT,1,NAS16B,0)#                                    AS16 180
ncodefld:(COEF,1,CHROT,FMTDKB,3)#                                       AS16 181
writeline:(1,AS16A,NAS16A,2,3)#                                         AS16 182
movechar:(AS16C,1,CHROT,1,NAS16C,0)#                                    AS16 183
ncodefld:(SGCOEF,1,CHROT,FMTDKB,3)#                                     AS16 184
writeline:(1,0,0,1,3)#                                                  AS16 185
#                                                                       AS16 186
MKPTR = MONPTR(I)+1#                   Set range of monitor             AS16 187
MKPTR2 = MONPTR(I+1)#                                                   AS16 188
AMEASR = FLOAT((MKPTR2-MKPTR+1)/DKSTEP)# Number of times measured       AS16 189
LINRM = NINT(AMEASR)+4#                Force new page if needed         AS16 190
movechar:(AS162,1,CHROT,1,NAS162,0)#                                    AS16 191
gethkl:(MKPTR,AH)#                                                      AS16 192
TTHETA = RTOD*QX(MKPTR+2)#             Put two theta into degrees       AS16 193
ncodefld:(AH,1,CHROT,FMT21,3)#         Write indices                    AS16 194
ncodefld:(AMEASR,1,CHROT,FMT22,1)#     Times measured                   AS16 195
ncodefld:(TTHETA,1,CHROT,FMT23,1)#     Write two theta in degrees       AS16 196
writeline:(1,0,0,1,3)#                                                  AS16 197
writeline:(1,AS163,NAS163,3,3)#                                         AS16 198
#                                                                       AS16 199
DO J=MKPTR,MKPTR2,DKSTEP#              Loop over measurements           AS16 200
#                                        Of this monitor                AS16 201
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS16 202
IOBSS = IOBSS + 1#                                                      AS16 203
EXPOS  = QX(J+1)#                                                       AS16 204
CALL AS01(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#                         AS16 205
BUFOT(1)=EXPOS/3600.#                  Convert from seconds to hour     AS16 206
ncodefld:(BUFOT,1,CHROT,FMT31,1)#      Exposure in hours                AS16 207
ncodefld:(QX(SCLMRK+IOBSS),1,CHROT,FMT32,1)#                            AS16 208
ncodefld:(SCALC,1,CHROT,FMT33,1)#                                       AS16 209
writeline:(0,0,0,1,3)#                                                  AS16 210
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS16 211
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS16 212
#                                                                       AS16 213
# NOW SCALE ALL THE MONITORS TOGETHER                                   AS16 214
#                                                                       AS16 215
LINRM = 8#                             See that it goes on page         AS16 216
writeline:(1,AS16Z,NAS16Z,3,3)#        Announce combined scaling        AS16 217
IF(DKTYPE==1)#                                                          AS16 218
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AS16 219
writeline:(1,AS16Y,NAS16Y,3,3)#                                         AS16 220
writeline:(0,AS16X,NAS16X,3,3)#                                         AS16 221
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AS16 222
ELSE IF(DKTYPE==2)#                                                     AS16 223
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AS16 224
writeline:(1,AS16W,NAS16W,3,3)#                                         AS16 225
writeline:(0,AS16V,NAS16V,3,3)#                                         AS16 226
writeline:(0,AS16U,NAS16U,3,3)#                                         AS16 227
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AS16 228
ELSE IF(DKTYPE==3)#                                                     AS16 229
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AS16 230
writeline:(1,AS16T,NAS16T,3,3)#                                         AS16 231
writeline:(0,AS16S,NAS16S,3,3)#                                         AS16 232
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AS16 233
IOBS = 1#                                                               AS16 234
MKPTR = MONPTR(1)+1#                   Set range of monitor             AS16 235
MKPTR2 = MONPTR(NSTAND+1)#                                              AS16 236
CALL AS17(MKPTR,MKPTR2,QX(SCLMRK+IOBS),DKTYPE)#                         AS16 237
#                                                                       AS16 238
# WRITE OUT THE COEFFICIENTS                                            AS16 239
#                                                                       AS16 240
I=MIN0(6,NCOEF)#                       Up to six coefficients           AS16 241
movechar:(AS16B,1,CHROT,1,NAS16B,0)#                                    AS16 242
ncodefld:(COEF,1,CHROT,FMTDKB,I)#                                       AS16 243
writeline:(1,AS16A,NAS16A,2,3)#                                         AS16 244
movechar:(AS16C,1,CHROT,1,NAS16C,0)#                                    AS16 245
ncodefld:(SGCOEF,1,CHROT,FMTDKB,I)#                                     AS16 246
writeline:(1,0,0,1,3)#                                                  AS16 247
#                                                                       AS16 248
IOBS = 0#                                                               AS16 249
DO I=1,NSTAND#                                                          AS16 250
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AS16 251
MKPTR = MONPTR(I)+1#                   Set range of monitor             AS16 252
MKPTR2 = MONPTR(I+1)#                                                   AS16 253
AMEASR = FLOAT((MKPTR2-MKPTR+1)/DKSTEP)# Number of times measured       AS16 254
IF(NINT(AMEASR) <= maxcoef:) NEXT#     Are there too few measuremen     AS16 255
LINRM = NINT(AMEASR)+4#                Force new page if needed         AS16 256
movechar:(AS162,1,CHROT,1,NAS162,0)#                                    AS16 257
gethkl:(MKPTR,AH)#                                                      AS16 258
TTHETA = RTOD*QX(MKPTR+2)#             Put two theta into degrees       AS16 259
ncodefld:(AH,1,CHROT,FMT21,3)#         Write indices                    AS16 260
ncodefld:(AMEASR,1,CHROT,FMT22,1)#     Times measured                   AS16 261
ncodefld:(TTHETA,1,CHROT,FMT23,1)#     Write two theta in degrees       AS16 262
writeline:(1,0,0,1,3)#                                                  AS16 263
writeline:(1,AS163,NAS163,3,3)#                                         AS16 264
TTHETA=TTHETA/RTOD#                    Put two theta back to radian     AS16 265
#                                                                       AS16 266
DO J=MKPTR,MKPTR2,DKSTEP#              Loop over measurements           AS16 267
#                                        Of this monitor                AS16 268
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AS16 269
IOBS = IOBS + 1#                                                        AS16 270
EXPOS  = QX(J+1)#                                                       AS16 271
BUFOT(1)=EXPOS/3600.0#                 To hours for print               AS16 272
IF (DKTYPE==1) CALL AS01(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#          AS16 273
ELSE IF (DKTYPE==2) CALL AS02(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#     AS16 274
ELSE IF (DKTYPE==3) CALL AS03(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#     AS16 275
ncodefld:(BUFOT,1,CHROT,FMT31,1)#                                       AS16 276
ncodefld:(QX(SCLMRK+IOBS),1,CHROT,FMT32,1)# A                           AS16 277
ncodefld:(SCALC,1,CHROT,FMT33,1)#                                       AS16 278
writeline:(0,0,0,1,3)#                                                  AS16 279
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AS16 280
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AS16 281
BUFOT(1)=FLOAT(DKTYPE) ; BUFOT(2)=FLOAT(IOBS)# Tell total refl used     AS16 282
ncodefld:(BUFOT,1,AS16D,FMTO,2)#                                        AS16 283
writeline:(1,AS16D,NAS16D,3,3)#                                         AS16 284
RETURN#                                                                 AS16 285
END#                                                                    AS16 286
systemheader:(AS17)#                   Least squares fit of decay       AS17   1
# ################################################################## #  AS17   2
#                                                                    #  AS17   3
#                               AS17                                 #  AS17   4
#                                                                    #  AS17   5
# ################################################################## #  AS17   6
SUBROUTINE AS17(MKPTR,MKPTR2,SCALE,TYPE)#                               AS17   7
#                                        Decay monitors                 AS17   8
#                                        K. watenpaugh, oct 1985        AS17   9
include:(AACOMN)#                                                       AS17  10
include:(ASCOMN)#                                                       AS17  11
INTEGER EXIT#                          Flag convergence                 AS17  12
INTEGER MKPTR#                                                          AS17  13
INTEGER MKPTR2#                                                         AS17  14
INTEGER I,J,K,JC,JD,JE#                                                 AS17  15
INTEGER TYPE#                                                           AS17  16
REAL EXPOS#                                                             AS17  17
REAL SCALE(12)#                                                         AS17  18
REAL SCALC#                                                             AS17  19
DOUBLE PRECISION RMAT(maxcoef:,maxcoef:)#                               AS17  20
DOUBLE PRECISION HMAT(maxcoef:,maxcoef:)#                               AS17  21
DOUBLE PRECISION PMAT(maxcoef:,maxcoef:)#                               AS17  22
DOUBLE PRECISION RAVE#                                                  AS17  23
REAL DELS(maxcoef:)#                                                    AS17  24
REAL DUMS(maxcoef:)#                                                    AS17  25
REAL DER(maxcoef:)#                                                     AS17  26
REAL FRACT#                            Fractional error in coefs        AS17  27
REAL SFCOEF(maxcoef:)#                                                  AS17  28
REAL TTHETA#                                                            AS17  29
chardata:(AS17A,[ CONVERGENCE    COEF1       COEF2       COEF3       $# AS17  30
COEF4       COEF5       COEF6])#                                        AS17  31
chardata:(AS17B,[SHIFT/ERROR])#                                         AS17  32
realdata:(FMSH,[241151.,361151.,481151.,601151.,721151.,841151.])#      AS17  33
datastuff:#                                                             AS17  34
#                                                                       AS17  35
# LOOP UNTIL THE PROCESS CONVERGES OR TEN TIMES.  THIS SHOULD NOT BE    AS17  36
# NECCESSARY,  BUT DO IT ANYWAY.                                        AS17  37
#                                                                       AS17  38
LINRM=2#                                                                AS17  39
DO K=1,10#                                                              AS17  40
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS17  41
#                                                                       AS17  42
# BEGIN PROCESSING MONITORS TO FIND DECAY COEF                          AS17  43
#                                                                       AS17  44
# INITIALIZE MATRIX AND VECTORS                                         AS17  45
#                                                                       AS17  46
SUMDS = 0.0#                           Sum of del**2                    AS17  47
DO JC=1,maxcoef:#                                                       AS17  48
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS17  49
DELS(JC)=0.0#                          Clear the vector                 AS17  50
SHOVER(JC)=0.0#                        Clear the shift over error       AS17  51
DUMS(JC)=0.0#                          Clear dummy vector               AS17  52
DO JD=1,maxcoef:#                                                       AS17  53
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS17  54
RMAT(JC,JD)=DBLE(0.0)#                 Clear the matrix                 AS17  55
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS17  56
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS17  57
I=0#                                                                    AS17  58
DO J=MKPTR,MKPTR2,DKSTEP#              Loop over measurements           AS17  59
#                                        Of this monitor                AS17  60
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS17  61
I=I+1#                                                                  AS17  62
EXPOS = QX(J+1)#                       Exposure time in seconds         AS17  63
TTHETA  = QX(J+2)#                     Two-theta in radians             AS17  64
IF (TYPE==1) CALL AS01(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,1)#            AS17  65
ELSE IF (TYPE==2) CALL AS02(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,1)#       AS17  66
ELSE IF (TYPE==3) CALL AS03(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,1)#       AS17  67
#                                                                       AS17  68
#   CALCULATE THE DIFFERENCE BETWEEN THE OBSERVED AND CALCULATED SCALE  AS17  69
#                                                                       AS17  70
DEL = SCALE(I) - SCALC#                                                 AS17  71
SUMDS = SUMDS + DEL**2#                                                 AS17  72
DO JC = 1, NCOEF#                                                       AS17  73
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS17  74
#                                                                       AS17  75
#   CALULATE THE CONTRIBUTION TO THE VECTOR - DERIVATIVE * DELTA        AS17  76
#                                                                       AS17  77
DELS(JC) = DELS(JC)+DER(JC)*DEL#                                        AS17  78
DO JD = 1, NCOEF#                                                       AS17  79
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS17  80
#                                                                       AS17  81
#   CALCULATE THE MATRIX ELEMENTS.                                      AS17  82
#                                                                       AS17  83
RMAT(JC,JD) = RMAT(JC,JD)+DBLE(DER(JC))*DBLE(DER(JD))#                  AS17  84
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS17  85
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS17  86
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS17  87
#                                                                       AS17  88
# INVERT TO MATRIX AND SCALE PRINTING IF NECESSARY                      AS17  89
#                                                                       AS17  90
RAVE =DBLE( 0.0)#                                                       AS17  91
DO JC=1,NCOEF#                                                          AS17  92
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS17  93
DO JD=1,NCOEF#                                                          AS17  94
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS17  95
HMAT(JC,JD)=RMAT(JC,JD)#               Save the original matrix         AS17  96
PMAT(JC,JD)=DBLE(0.0)#                 Clear the product to be          AS17  97
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS17  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS17  99
CALL AS21(RMAT,DELS,NCOEF,1)#          Dump guts                        AS17 100
CALL AS19(RMAT,RAVE,NCOEF)#            Adjust matrix                    AS17 101
CALL AS21(RMAT,DELS,NCOEF,2)#          Dump guts                        AS17 102
CALL AS20(RMAT,NCOEF,DET)#             Invert matrix                    AS17 103
CALL AS21(RMAT,DELS,NCOEF,3)#          Dump guts                        AS17 104
CALL AS19(RMAT,RAVE,NCOEF)#            Adjust back                      AS17 105
CALL AS21(RMAT,DELS,NCOEF,4)#          Dump guts                        AS17 106
DO JC=1,NCOEF#                                                          AS17 107
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS17 108
DO JD=1,NCOEF#                                                          AS17 109
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS17 110
DO JE=1,NCOEF#                                                          AS17 111
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS17 112
PMAT(JC,JD)=PMAT(JC,JD)+HMAT(JC,JE)*RMAT(JE,JD)# Product of matrices    AS17 113
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS17 114
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS17 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS17 116
CALL AS21(PMAT,DUMS,NCOEF,5)#          Dump guts                        AS17 117
#                                                                       AS17 118
# GET THE SHIFTS                                                        AS17 119
#                                                                       AS17 120
DO JC = 1,NCOEF#                                                        AS17 121
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS17 122
SFCOEF(JC) = 0.0#                                                       AS17 123
DO JD = 1,NCOEF#                                                        AS17 124
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS17 125
SFCOEF(JC)=SNGL(DBLE(SFCOEF(JC))+RMAT(JC,JD)*DBLE(DELS(JD)))#           AS17 126
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS17 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS17 128
#                                                                       AS17 129
# GET NEW COEF                                                          AS17 130
#                                                                       AS17 131
DO JC = 1,NCOEF#                                                        AS17 132
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AS17 133
COEF(JC) = COEF(JC) + SFCOEF(JC)#                                       AS17 134
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AS17 135
#                                                                       AS17 136
# RECALC SUM DELTAS                                                     AS17 137
#                                                                       AS17 138
I = 0#                                                                  AS17 139
DO J=MKPTR,MKPTR2,DKSTEP#              Loop over measurements           AS17 140
#                                        Of this monitor                AS17 141
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AS17 142
I=I+1#                                                                  AS17 143
EXPOS  = QX(J+1)#                      Exposure in seconds              AS17 144
TTHETA = QX(J+2)#                      Two theta in radians             AS17 145
IF (TYPE==1) CALL AS01(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#            AS17 146
ELSE IF (TYPE==2) CALL AS02(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#       AS17 147
ELSE IF (TYPE==3) CALL AS03(SCALC,DER,COEF,TTHETA,EXPOS,NCOEF,0)#       AS17 148
DEL = SCALE(I) - SCALC#                                                 AS17 149
SUMDSN = SUMDSN + DEL**2#                                               AS17 150
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AS17 151
#                                                                       AS17 152
# GET STANDARD DEVIATIONS                                               AS17 153
#                                                                       AS17 154
DO JC = 1,NCOEF#                                                        AS17 155
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AS17 156
SGCOEF(JC) = SQRT(ABS(SNGL(RMAT(JC,JC))*SUMDSN/$# Find the estimated    AS17 157
AMAX1(FLOAT(I-NCOEF),0.001)))#         Standard deviation in coef       AS17 158
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AS17 159
#                                                                       AS17 160
# TEST FOR CONVERGENCE BY MAKING SHIFT OVER ERROR FACTOR FOR ALL        AS17 161
# COEFICIENTS.  QUIT WHEN ALL SHIFT OVER ERRORS ARE LESS THAN 0.5       AS17 162
#                                                                       AS17 163
EXIT = yes:#                           Believe we can leave             AS17 164
FRACT=0.0#                             Calc frational error in coef     AS17 165
DO JC = 1,NCOEF#                                                        AS17 166
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AS17 167
IF(SGCOEF(JC)>1.0E-15)#                                                 AS17 168
SHOVER(JC) = ABS(SFCOEF(JC)/SGCOEF(JC))# Save blather                   AS17 169
ELSE SHOVER(JC)=0.0#                                                    AS17 170
IF(ABS(COEF(JC))>1.0E-15) FRACT=FRACT+SGCOEF(JC)/COEF(JC)#              AS17 171
IF(SHOVER(JC) > 0.1) EXIT = no:#       If big try again                 AS17 172
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AS17 173
ncodefld:(SHOVER,1,CHROT,FMSH,NCOEF)#                                   AS17 174
movechar:(AS17B,1,CHROT,2,NAS17B,0)#                                    AS17 175
writeline:(1,AS17A,NAS17A,1,4)#        Dump shift over error            AS17 176
IF((EXIT == yes:\(FRACT<0.00003))&(K>2)) BREAK# Get out when all smal   AS17 177
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS17 178
RETURN#                                                                 AS17 179
END#                                                                    AS17 180
systemheader:(AS18)#                   Four word quicksort reals        AS18   1
# ################################################################## #  AS18   2
#                                                                    #  AS18   3
#                               AS18                                 #  AS18   4
#                                                                    #  AS18   5
# ################################################################## #  AS18   6
SUBROUTINE AS18(FNUM,NO)#                                               AS18   7
#                                                                       AS18   8
# QUICKSORT OF 'NO' 4-WORD REAL VARIABLES                               AS18   9
#  BASED ON CODE FOUND IN SOFTWARE TOOLS.                               AS18  10
#                                                                       AS18  11
#                                        Jun 1979  r.a. alden           AS18  12
#                                        7 sept 1986 j.m. stewart       AS18  13
INTEGER I#                             Lower partition index            AS18  14
INTEGER J#                             Upper partition index            AS18  15
INTEGER LV(100)#                       Lower partition points           AS18  16
INTEGER NO#                            Number of 4-word reals to        AS18  17
#                                         Be sorted.                    AS18  18
INTEGER P#                             Stack pointer                    AS18  19
INTEGER UV(100)#                       Upper partition points           AS18  20
#                                                                       AS18  21
REAL FNUM(10)#                         Table of 4-word real variabl     AS18  22
#                                         To be sorted.                 AS18  23
REAL PIVPT#                            Current pivot point              AS18  24
REAL TEMP#                             Scratch storage during           AS18  25
#                                         Interchange.                  AS18  26
#                                                                       AS18  27
LV(1) = 1#                             Set lower partition              AS18  28
UV(1) = 4*NO - 3#                      Set upper partition              AS18  29
P = 1#                                 Stack pointer for simulated      AS18  30
#                                         Recursion.                    AS18  31
WHILE(P > 0)#                                                           AS18  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS18  33
IF(LV(P) >= UV(P))  P = P - 1#         If subset has only one           AS18  34
#                                         Element, pop the stack.       AS18  35
ELSE#                                  Do partitioning                  AS18  36
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS18  37
I = LV(P) - 4#                         The elements are rearranged      AS18  38
J = UV(P)#                             Entirely within the subset       AS18  39
movepakwd:(FNUM(J),PIVPT)#             Of num between the upper         AS18  40
#                                         And lower limits.             AS18  41
WHILE(I < J)#                                                           AS18  42
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS18  43
I = I + 4#                                                              AS18  44
REPEAT#                                                                 AS18  45
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS18  46
compakwd:(FNUM(I),GE,PIVPT,BREAK)#     Go next one greater or equal     AS18  47
I = I + 4#                             Move on in stack                 AS18  48
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS18  49
FOR(J=J-4; J > I; J=J-4)#              Decrease j                       AS18  50
#                                                                       AS18  51
# AT THIS POINT, EITHER I AND J HAVE MET OR WE HAVE AN                  AS18  52
#  OUT-OF-ORDER PAIR.                                                   AS18  53
#                                                                       AS18  54
compakwd:(FNUM(J),LE,PIVPT,BREAK)#                                      AS18  55
IF(I < J)  exchange:#                  Exchange out-of-order pair       AS18  56
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS18  57
#                                                                       AS18  58
# I AND J HAVE MET.  MOVE THE PIVOT ELEMENT TO THE 'MIDDLE'             AS18  59
#                                                                       AS18  60
J = UV(P)#                                                              AS18  61
exchange:#                             Exchange fnum(i) and last pt     AS18  62
IF(I-LV(P) < UV(P)-I)#                 Stack so short partition don     AS18  63
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS18  64
LV(P+1) = LV(P)#                                                        AS18  65
UV(P+1) = I - 4#                                                        AS18  66
LV(P) = I + 4#                                                          AS18  67
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS18  68
ELSE#                                                                   AS18  69
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS18  70
LV(P+1) = I + 4#                                                        AS18  71
UV(P+1) = UV(P)#                                                        AS18  72
UV(P) = I - 4#                                                          AS18  73
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS18  74
P = P + 1#                             Push onto stack                  AS18  75
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS18  76
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS18  77
#                                                                       AS18  78
RETURN#                                                                 AS18  79
END#                                                                    AS18  80
systemheader:(AS19)#                   Rescale l-s matrix               AS19   1
# ################################################################## #  AS19   2
#                                                                    #  AS19   3
#                               AS19                                 #  AS19   4
#                                                                    #  AS19   5
# ################################################################## #  AS19   6
SUBROUTINE AS19(RMAT,RAVE,NCOEF)#      K.d.watenpaugh                   AS19   7
#                                                                       AS19   8
#     ADJUSTS THE MATRIX AND VECTOR OF THE LEAST SQUARES PROBLEM        AS19   9
#       TO HAVE MAGNITUDES NEAR ONE FOR THE MATRIX INVERSION            AS19  10
#                                                                       AS19  11
DOUBLE PRECISION RMAT(maxcoef:,maxcoef:)#                               AS19  12
DOUBLE PRECISION AR#                                                    AS19  13
DOUBLE PRECISION RAVE#                                                  AS19  14
DOUBLE PRECISION RMIN,RMAX#                                             AS19  15
INTEGER NCOEF#                                                          AS19  16
#                                                                       AS19  17
#     IF RAVE IS NOT ZERO, GO AND SCALE BY IT                           AS19  18
#     FIND THE SMALLEST AND LARGEST ELEMENTS                            AS19  19
#     ALSO THE AVERAGE                                                  AS19  20
#                                                                       AS19  21
ifeq:(RAVE,0.0)#                                                        AS19  22
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS19  23
RAVE=DBLE(0.0)#                                                         AS19  24
RMIN=DBLE(10E+8)#                                                       AS19  25
RMAX=DBLE(10E-8)#                                                       AS19  26
DO I=1,NCOEF#                                                           AS19  27
DO J=1,NCOEF#                                                           AS19  28
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS19  29
AR = DABS(RMAT(I,J))#                                                   AS19  30
RAVE=RAVE+AR#                                                           AS19  31
IF (AR>RMAX) RMAX=AR#                                                   AS19  32
IF (AR<RMIN) RMIN=AR#                                                   AS19  33
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS19  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS19  35
#                                                                       AS19  36
#      SCALE EVERYTHING BY RAVE                                         AS19  37
#                                                                       AS19  38
DO I=1,NCOEF#                                                           AS19  39
DO J=1,NCOEF#                                                           AS19  40
RMAT(I,J)=RMAT(I,J)/RAVE#                                               AS19  41
#                                                                       AS19  42
RETURN#                                                                 AS19  43
END#                                                                    AS19  44
systemheader:(AS20)#                   Matrix inversion for l-s         AS20   1
# ################################################################## #  AS20   2
#                                                                    #  AS20   3
#                               AS20                                 #  AS20   4
#                                                                    #  AS20   5
# ################################################################## #  AS20   6
SUBROUTINE AS20(ARRAY,NORDER,DET)#                                      AS20   7
#                                                                       AS20   8
#     MATRIX INVERSION ROUTINE TAKEN FROM BEVINGTON.                    AS20   9
#     DATA REDUCTION AND ERROR ANALYSIS FOR THE PHYSICAL SCIENCES       AS20  10
#     PHILIP R. BEVINGTON   MCGRAW-HILL (1969)                          AS20  11
#                                                                       AS20  12
INTEGER IK(maxcoef:)#                                                   AS20  13
INTEGER JK(maxcoef:)#                                                   AS20  14
INTEGER NORDER,I,K,J#                                                   AS20  15
DOUBLE PRECISION ARRAY(maxcoef:,maxcoef:)#                              AS20  16
DOUBLE PRECISION SAVE#                                                  AS20  17
DOUBLE PRECISION AMAX#                                                  AS20  18
REAL DET#                                                               AS20  19
#                                                                       AS20  20
DET=1.0#                                                                AS20  21
DO K=1,NORDER#                                                          AS20  22
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS20  23
AMAX=0.0#                                                               AS20  24
DO I=K,NORDER#                                                          AS20  25
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS20  26
DO J=K,NORDER#                                                          AS20  27
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS20  28
IF (DABS(AMAX)<=DABS(ARRAY(I,J)))#                                      AS20  29
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS20  30
AMAX=ARRAY(I,J)#                                                        AS20  31
IK(K)=I#                                                                AS20  32
JK(K)=J#                                                                AS20  33
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS20  34
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS20  35
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS20  36
IF (AMAX==0.0)#                                                         AS20  37
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS20  38
DET=0.0#                                                                AS20  39
BREAK#                                                                  AS20  40
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS20  41
I=IK(K)#                                                                AS20  42
IF (I<K) NEXT#                                                          AS20  43
ELSE IF (I>K)#                                                          AS20  44
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS20  45
DO J=1,NORDER#                                                          AS20  46
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS20  47
SAVE =ARRAY(K,J)#                                                       AS20  48
ARRAY(K,J)=ARRAY(I,J)#                                                  AS20  49
ARRAY(I,J)=-SAVE#                                                       AS20  50
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS20  51
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS20  52
J=JK(K)#                                                                AS20  53
IF (J<K) NEXT#                                                          AS20  54
ELSE IF (J>K)#                                                          AS20  55
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS20  56
DO I=1,NORDER#                                                          AS20  57
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS20  58
SAVE =ARRAY(I,K)#                                                       AS20  59
ARRAY(I,K)=ARRAY(I,J)#                                                  AS20  60
ARRAY(I,J)=-SAVE#                                                       AS20  61
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS20  62
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS20  63
DO I=1,NORDER#                                                          AS20  64
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS20  65
IF (I!=K)#                                                              AS20  66
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS20  67
ARRAY(I,K)=-ARRAY(I,K)/AMAX#                                            AS20  68
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS20  69
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS20  70
DO I=1,NORDER#                                                          AS20  71
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS20  72
DO J=1,NORDER#                                                          AS20  73
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS20  74
IF (I!=K)#                                                              AS20  75
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AS20  76
IF (J!=K)#                                                              AS20  77
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AS20  78
ARRAY(I,J)=ARRAY(I,J)+ARRAY(I,K)*ARRAY(K,J)#                            AS20  79
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AS20  80
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AS20  81
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS20  82
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS20  83
DO J=1,NORDER#                                                          AS20  84
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AS20  85
IF (J!=K)#                                                              AS20  86
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AS20  87
ARRAY(K,J)=ARRAY(K,J)/AMAX#                                             AS20  88
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AS20  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AS20  90
ARRAY(K,K)=1./AMAX#                                                     AS20  91
DET=SNGL(DBLE(DET)*AMAX)#                                               AS20  92
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS20  93
DO L=1,NORDER#                                                          AS20  94
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AS20  95
K=NORDER-L+1#                                                           AS20  96
J=IK(K)#                                                                AS20  97
IF (J>K)#                                                               AS20  98
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AS20  99
DO I=1,NORDER#                                                          AS20 100
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AS20 101
SAVE =ARRAY(I,K)#                                                       AS20 102
ARRAY(I,K)=-ARRAY(I,J)#                                                 AS20 103
ARRAY(I,J)=SAVE#                                                        AS20 104
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AS20 105
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AS20 106
I=JK(K)#                                                                AS20 107
IF (I>K)#                                                               AS20 108
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AS20 109
DO J=1,NORDER#                                                          AS20 110
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AS20 111
SAVE =ARRAY(K,J)#                                                       AS20 112
ARRAY(K,J)=-ARRAY(I,J)#                                                 AS20 113
ARRAY(I,J)=SAVE#                                                        AS20 114
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AS20 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AS20 116
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AS20 117
RETURN#                                                                 AS20 118
END#                                                                    AS20 119
systemheader:(AS21)#                   Print normal equations           AS21   1
# ################################################################## #  AS21   2
#                                                                    #  AS21   3
#                              AS21                                  #  AS21   4
#                                                                    #  AS21   5
# ################################################################## #  AS21   6
SUBROUTINE AS21(RMAT,DELS,NCOEF,PSIG)# J.m.stewart 6 sept 1986          AS21   7
include:(AACOMN)#                                                       AS21   8
DOUBLE PRECISION RMAT(maxcoef:,maxcoef:)#                               AS21   9
REAL DELS(maxcoef:)#                                                    AS21  10
INTEGER NCOEF,PSIG,I,J#                                                 AS21  11
chardata:(AS21A,   VECTOR       MATRIX OF NORMAL $#                     AS21  12
EQUATIONS                       )#                                      AS21  13
chardata:(AS21B,RAW                  )#                                 AS21  14
chardata:(AS21C,AFTER RESCALING      )#                                 AS21  15
chardata:(AS21D,AFTER INVERSION      )#                                 AS21  16
chardata:(AS21E,AFTER RE-RESCALING   )#                                 AS21  17
chardata:(AS21F,PRODUCT MATRIX       )#                                 AS21  18
realdata:(FMT,[141361.,321361.,461361.,601361.,741361.,881361.,$#       AS21  19
1021361.])#                                                             AS21  20
datastuff:#                                                             AS21  21
#                                                                       AS21  22
IF(PPRIOR  >3)#                        Don't bother unless asked        AS21  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS21  24
IF(PSIG==1)      movechar:(AS21B,1,AS21A,44,NAS21B,0)#                  AS21  25
ELSE IF(PSIG==2) movechar:(AS21C,1,AS21A,44,NAS21C,0)#                  AS21  26
ELSE IF(PSIG==3) movechar:(AS21D,1,AS21A,44,NAS21D,0)#                  AS21  27
ELSE IF(PSIG==4) movechar:(AS21E,1,AS21A,44,NAS21E,0)#                  AS21  28
ELSE             movechar:(AS21F,1,AS21A,44,NAS21F,0)#                  AS21  29
LINRM=2*NCOEF+2#                       Keep it on the page              AS21  30
writeline:(1,AS21A,NAS21A,3,3)#                                         AS21  31
DO I=1,NCOEF#                                                           AS21  32
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS21  33
DO J=1,NCOEF#                                                           AS21  34
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS21  35
BUFOT(J+1)=SNGL(RMAT(I,J))#            Pick up the rows                 AS21  36
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS21  37
BUFOT(1)=DELS(I)#                      Get the vector element           AS21  38
ncodefld:(BUFOT,1,CHROT,FMT,NCOEF+1)#                                   AS21  39
writeline:(1,0,0,1,3)#                                                  AS21  40
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS21  41
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS21  42
RETURN#                                                                 AS21  43
END#                                                                    AS21  44
systemheader:(AS30)#                   Write corrected data to file     AS30   1
# ################################################################## #  AS30   2
#                                                                    #  AS30   3
#                              AS30                                  #  AS30   4
#                                                                    #  AS30   5
# ################################################################## #  AS30   6
SUBROUTINE AS30#                       J.m.stewart 3 sept 1986          AS30   7
include:(AACOMN)#                                                       AS30   8
include:(ASCOMN)#                                                       AS30   9
INTEGER IP,OP#        ## Packet pointers                                AS30  10
INTEGER IREL(21)#     ## Packet offset pointers                         AS30  11
INTEGER KEY(4)#       ## Indexpkt signals                               AS30  12
INTEGER NPAK,OPAK#    ## Lrrefl packet sizes                            AS30  13
INTEGER RFTYPE#       ## Reflection type                                AS30  14
INTEGER TOTAL#        ## Total reflections read                         AS30  15
REAL ABSORB#          ## Phi spin absorption correction (tau)           AS30  16
REAL ABSOR1,ABSOR2#   ## Inlet and outlet components of tau corr.       AS30  17
REAL CHI#             ## Current reflection eulerian chi value          AS30  18
REAL CORR#            ## Correction to phi as a function of chi         AS30  19
REAL DECAY#           ## Correction for loss of intensity as a          AS30  20
#                     ## FUNCTION OF TIME AND, PERHAPS, TWO THETA       AS30  21
REAL EXPOS#           ## Crystal exposure time in seconds               AS30  22
REAL MINDEC,MAXDEC,MINABS,MAXABS# ## Stats for functions                AS30  23
REAL TPHI#            ## Current reflection eulerian angle phi          AS30  24
REAL TTHETA#          ## Twice current reflection diffraction angle     AS30  25
REAL REJECI#          ## Count of deleted void i's                      AS30  26
REAL REJECM#          ## Count of deleted monitors                      AS30  27
REAL ACCEPT#          ## Count of accepted reflections                  AS30  28
REAL UNTREA#          ## Count of untreated reflections                 AS30  29
#                                                                       AS30  30
realdata:(DTOR,[0.017453292])#         Degrees to radians               AS30  31
intdata:(IWNT20,[ 1,1005,1006,1007,1008,$# Items needed for reflectns   AS30  32
1300,1301,1018,1014,1027])#                                             AS30  33
chardata:(AS30A,           REFLECTIONS WITH MISSING INTENSITY PASSED $# AS30  34
UNCHANGED TO OUTPUT FILE.)#                                             AS30  35
chardata:(AS30B,[           MONITORING AND ABSORPTION $#                AS30  36
REFLECTIONS NOT PASSED TO OUTPUT FILE.])#                               AS30  37
chardata:(AS30C,           REFLECTIONS CORRECTED AND PASSED TO $#       AS30  38
OUTPUT FILE. )#                                                         AS30  39
chardata:(AS30D,     MAXIMUM     MINIMUM    ABSORPTION CORRECTION)#     AS30  40
chardata:(AS30E,     MAXIMUM     MINIMUM    DECAY CORRECTION)#          AS30  41
chardata:(AS30F,           REFLECTIONS SPECIFIED IN THE APPLY LINE $#   AS30  42
PASSED UNCHANGED TO OUTPUT FILE.)#                                      AS30  43
datastuff:#                                                             AS30  44
#                                                                       AS30  45
copyfile:(1,2,1,decr:(lrrefl:))#       Copy all records to lrrefl:      AS30  46
#                                                                       AS30  47
#    SEARCH DIRECTORY OF LRREFL:                                        AS30  48
#    ---------------------------                                        AS30  49
#                                                                       AS30  50
KEY(1)=7; KEY(2)=10; KEY(3)=0; KEY(4)=0# Repeat of as12, can't fail     AS30  51
indexpkt:(1,lrrefl:,NPAK,IP,0,KEY,IWNT20,IREL)# Search lrrefl direct    AS30  52
IF(KEY(4)<=0) iquit:(AS1206)#          Wanted items missing             AS30  53
NHKL=IREL(1) ; NPHI=IREL(2) ; NCHI=IREL(3)# Hkl, phi, chi               AS30  54
NOME=IREL(4) ; NTTH=IREL(5) ; NRFT=IREL(8)# Omega, 2 theta, refl code   AS30  55
NIRL=IREL(6) ; NSIR=IREL(7) ; NEXP=IREL(9)# Intens, sig(i), exp time    AS30  56
NRFN=IREL(10)#                         Reflection number                AS30  57
OPAK=KEY(4)#                           Set output pkt size              AS30  58
writepkt:(2,lrrefl:,OPAK,OP)#          Write pkt header                 AS30  59
movereal:(QX,IP+1,QX,OP+1,NPAK,0)#     Move directory                   AS30  60
#                                                                       AS30  61
#    ENTER LRREFL: BDF RECORDS                                          AS30  62
#    -------------------------                                          AS30  63
#                                                                       AS30  64
REJECI=0.0 ; REJECM=0.0 ; ACCEPT=0.0 ; UNTREA=0.0# Clear counts         AS30  65
MINABS=+999999.0 ; MAXABS=-999999.0#   Statistics for absorb            AS30  66
MINDEC=+999999.0 ; MAXDEC=-999999.0#   And decay factors                AS30  67
TOTAL = 0#                             Zero count of reflections        AS30  68
REPEAT#                                Loop over lrrefl records         AS30  69
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AS30  70
readwpkt:(1,lrrefl:,NPAK,IP,0)#        Point to ip packet               AS30  71
IF(IP<=0) BREAK#                       Exit reflection loop             AS30  72
TOTAL=TOTAL+1#                         Count every packet read          AS30  73
CORREC=yes:#                           Assume correction will be do     AS30  74
IF(TOTAL<SCFRST\TOTAL>SCLAST)#         Exclude as directed              AS30  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AS30  76
UNTREA = UNTREA + 1.0#                 Count the unwashed               AS30  77
CORREC=no:#                            And skip to next input           AS30  78
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AS30  79
IF(PURGE==yes:)#                       See if duds are to go out        AS30  80
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AS30  81
RFTYPE=QX(IP+NRFT)#                    Pick up reflection type          AS30  82
ifvo:(QX(IP+NIRL))#                    Skip unknown intensities         AS30  83
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AS30  84
REJECI=REJECI+1.0#                     Count it and pass it through     AS30  85
CORREC=no:#                            By pass correction applicati     AS30  86
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AS30  87
ELSE IF(RFTYPE==1\RFTYPE==2)#          Ditch monitors and absorbers     AS30  88
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AS30  89
REJECM=REJECM+1.0#                                                      AS30  90
NEXT#                                                                   AS30  91
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AS30  92
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AS30  93
writepkt:(2,lrrefl:,OPAK,OP)#          Get output packet pointer        AS30  94
movereal:(QX,IP+1,QX,OP+1,NPAK,0)#     Move the whole works             AS30  95
TTHETA = QX(OP+NTTH)#                  Get two theta in degrees         AS30  96
IF(TTHETA<TTLOW\TTHETA>TTHIGH)#        Exclude as directed              AS30  97
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AS30  98
UNTREA = UNTREA + 1.0#                 Count the unwashed               AS30  99
CORREC=no:#                            And skip to next input           AS30 100
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AS30 101
IF(CORREC==no:) NEXT#                  Corrections to be applied?       AS30 102
#                                                                       AS30 103
#   NOW MODIFY THE INTENSITIES IN THE OUTPUT FILE                       AS30 104
#   ---------------------------------------------                       AS30 105
#                                                                       AS30 106
ACCEPT=ACCEPT+1.0#                     Count reflections treated        AS30 107
#                                                                       AS30 108
#   IF KEYED, APPLY TAU ABSORPTION CORRECTIONS (PHI SPIN DATA)          AS30 109
#   ----------------------------------------------------------          AS30 110
#                                                                       AS30 111
IF(TAUFLG==yes:)#                                                       AS30 112
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AS30 113
CHI = QX(OP+NCHI)#                     Get chi                          AS30 114
TPHI = QX(OP+NPHI)#                    Get phi                          AS30 115
CORR = 0.5*TTHETA*COS(DTOR*CHI)#       Path correction to phi           AS30 116
#                                                                       AS30 117
#   NOW INTERPOLATE A CORRECTION FROM THE PHI SPIN DATA                 AS30 118
#  ----------------------------------------------------                 AS30 119
#                                                                       AS30 120
CALL AS04(TPHI+CORR,QX(TAUMRK+1),QX(MKTAU+1),ABSOR1,NTAU,3)#            AS30 121
CALL AS04(TPHI-CORR,QX(TAUMRK+1),QX(MKTAU+1),ABSOR2,NTAU,3)#            AS30 122
ABSORB = AMAX1(0.000001,0.5*(ABSOR1 + ABSOR2))# Get the average         AS30 123
QX(OP+NIRL) = QX(OP+NIRL)*ABSORB#      Apply the correction to i        AS30 124
QX(OP+NSIR) = QX(OP+NSIR)*ABSORB#      And to sigma(i)                  AS30 125
MINABS=AMIN1(ABSORB,MINABS)#                                            AS30 126
MAXABS=AMAX1(ABSORB,MAXABS)#                                            AS30 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AS30 128
#                                                                       AS30 129
#   SHOULD THE DECAY IN INTENSITY BE COMPENSATED FOR?                   AS30 130
#   -------------------------------------------------                   AS30 131
#                                                                       AS30 132
IF(DKFLG==yes:\DKFLGL==yes:)#          Apply decay?                     AS30 133
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AS30 134
IF(TIMEFL==no:) EXPOS = TIMEPR*QX(OP+NRFN)# Use estimated or use        AS30 135
ELSE EXPOS = QX(OP+NEXP)#              Exposure time in seconds         AS30 136
TTHETA = TTHETA*DTOR#                  Put two theta into radians       AS30 137
#                                                                       AS30 138
#   TEST METHOD OF DECAY COMPENSATION                                   AS30 139
#   ---------------------------------                                   AS30 140
#                                                                       AS30 141
IF(DKTYPE==1)#                         Function one?                    AS30 142
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AS30 143
CALL AS01(DECAY,DUMMY,COEF,TTHETA,EXPOS,NCOEF,0)# Do one                AS30 144
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AS30 145
ELSE IF(DKTYPE==2)#                    Function two?                    AS30 146
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AS30 147
CALL AS02(DECAY,DUMMY,COEF,TTHETA,EXPOS,NCOEF,0)# Do two                AS30 148
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AS30 149
ELSE IF(DKTYPE==3)#                    Function three?                  AS30 150
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AS30 151
CALL AS03(DECAY,DUMMY,COEF,TTHETA,EXPOS,NCOEF,0)# Do three              AS30 152
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AS30 153
#                                                                       AS30 154
#   APPLY THE CORRECTION TO THE REFLECTION INTENSITY                    AS30 155
#   ------------------------------------------------                    AS30 156
#                                                                       AS30 157
DECAY = AMAX1(DECAY,0.000001)#         Protect divide                   AS30 158
QX(OP+NIRL) = QX(OP+NIRL)*DECAY#       Zap intensity back up            AS30 159
QX(OP+NSIR) = QX(OP+NSIR)*DECAY#       Zap sigma(i) back up             AS30 160
MINDEC=AMIN1(DECAY,MINDEC)#                                             AS30 161
MAXDEC=AMAX1(DECAY,MAXDEC)#                                             AS30 162
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AS30 163
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AS30 164
#                                                                       AS30 165
#   FINISH OFF THE BINARY DATA FILE                                     AS30 166
#   ------------------------------                                      AS30 167
#                                                                       AS30 168
copyfile:(1,2,incr:(lrrefl:),endrecord:)# All the rest                  AS30 169
#                                                                       AS30 170
#   SUMMARIZE THE RUN                                                   AS30 171
#   -----------------                                                   AS30 172
#                                                                       AS30 173
LINRM=12#                                                               AS30 174
ncodefld:(REJECI,1,AS30A,100913.,1)#                                    AS30 175
writeline:(1,AS30A,NAS30A,3,3)#                                         AS30 176
ncodefld:(REJECM,1,AS30B,100913.,1)#                                    AS30 177
writeline:(1,AS30B,NAS30B,3,3)#                                         AS30 178
ncodefld:(ACCEPT,1,AS30C,100913.,1)#                                    AS30 179
writeline:(1,AS30C,NAS30C,3,3)#                                         AS30 180
ncodefld:(UNTREA,1,AS30F,100913.,1)#                                    AS30 181
writeline:(1,AS30F,NAS30F,3,3)#                                         AS30 182
IF(TAUFLG==yes:)#                      Absorption?                      AS30 183
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AS30 184
ncodefld:(MAXABS,1,CHROT,121142.,1)#                                    AS30 185
ncodefld:(MINABS,1,CHROT,241142.,1)#                                    AS30 186
writeline:(1,AS30D,NAS30D,2,3)#        Report absorption                AS30 187
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AS30 188
IF(DKFLG==yes:\DKFLGL==yes:)#          Decay?                           AS30 189
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AS30 190
ncodefld:(MAXDEC,1,CHROT,121142.,1)#                                    AS30 191
ncodefld:(MINDEC,1,CHROT,241142.,1)#                                    AS30 192
writeline:(1,AS30E,NAS30E,2,3)#        Report decay                     AS30 193
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AS30 194
#                                                                       AS30 195
RETURN#                                                                 AS30 196
END#                                                                    AS30 197
pmacro:([no:])#                        Delete local macros              AS30 198
