systemheader:(CB    )#                                                  CB     1
####################################################################### CB     2
##      Copyright (C) 2000, The Xtal System, The University of        # CB     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # CB     4
## Distributed under the GNU General Public Licence as published by   # CB     5
## the Free Software Foundation, in the hope that it will be useful   # CB     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # CB     7
## for more details.                                                  # CB     8
##                                                                    # CB     9
##                       **** CBAPA ****                              # CB    10
##                                                                    # CB    11
## Adds Crystal Based Azimuthal Psi Angle to the BDF                  # CB    12
## (Schwarzenbach and Flack J. Appl. Cryst. 1989, 22 p601-605)        # CB    13
## This is useful for running LSABS and LSLS for anisotropic          # CB    14
## extinction and refined absorption corrections.                     # CB    15
##                                                                    # CB    16
## Someone should really check that this works properly!              # CB    17
##                                   Doug du Boulay 1999              # CB    18
##                                                                    # CB    19
####################################################################### CB    20
#                                                                       CB    21
include:(XMACRO)#                                                       CB    22
#                                                                       CB    23
macro:(setflg:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")$2=ABS($3-1))#   CB    24
macro:(setint:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")#                CB    25
$(ifnm:(BUFIN(I+1))$($2=NINT(BUFIN(I+1));I=I+1$);ELSE $2=$4$))#         CB    26
macro:(setval:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")#                CB    27
$(ifnm:(BUFIN(I+1))$($2=BUFIN(I+1);I=I+1$);ELSE $2=$4$))#               CB    28
macro:(tiny:,2.0E-arith:(mxsgfp:,-,2))##                                CB    29
macro:(dsmall:,1.0E-mxsgfp:)#                                           CB    30
#                                                                       CB    31
macro:(invert:,CALL CB02($1,$2))#                                       CB    32
macro:(postm:, CALL CB04($1,$2,$3))#                                    CB    33
macro:(mmult:, CALL CB05($1,$2,$3))#                                    CB    34
macro:(triple:,CALL CB06($1,$2,$3))#                                    CB    35
macro:(norm:,  CALL CB07($1,$2))#                                       CB    36
macro:(vdotp:, CALL CB09($1,$2,$3))#                                    CB    37
#                                                                       CB    38
macro:(ifpos:,IF($1>small:))#                                           CB    39
#                                                                       CB    40
extrude:(CBCOMN)#                                                       CB    41
INTEGER ISET#                          Data set being treated           CB    42
INTEGER ICOS#                          Compute dir. cosines n and d     CB    43
INTEGER IANG#                          save angles to archive
INTEGER LIST#                          Print n reflections              CB    44
INTEGER LIM#                           Number of reflections treated    CB    45
INTEGER MARK1#                                                          CB    46
INTEGER MARKT#                                                          CB    47
INTEGER ISTOPT#                                                         CB    48
REAL BX(3,3)#                          Transf hkl to xtal system        CB    49
REAL ROT(5)#                           Rotation flags                   CB    50
REAL DISTSC#                           Convert to mm                    CB    51
REAL DITES#                            Precision limit                  CB    52
REAL DBTES#                            Double precision test            CB    53
REAL PHI90#                            Phi at chi 90                    CB    54
REAL MT(9)#                            Inverse metric tensor            CB    55
REAL OMIN#                             Min diffractometer omega         CB    56
REAL OMAX#                             Max diffractometer omega         CB    57
REAL RAD#                              Radius of sphere                 CB    58
REAL TC(3,3)#                          Rotation matrix                  CB    59
REAL UINV(3,3)#                        The inverse of the u matrix      CB    60
REAL UB(3,3)#                          Rotation and translation         CB    61
REAL VOL#                              Volume of the crystal            CB    62
REAL WAV#                              Radiation wavelength             CB    63
REAL DBZ#                              Double precision zero            CB    64
#                                                                       CB    65
#                                                                       CB    66
COMMON/COMDCB/$#                                                        CB    67
BX,ROT,DISTSC,DITES,DBTES,PHI90,MT,OMIN,OMAX,RAD,TC,UINV,$#             CB    68
UB,VOL,WAV,DBZ#                                                         CB    69
#                                                                       CB    70
COMMON/COMICB/ISET,ICOS,LIST,LIM,MARK1,MARKT,ISTOPT,IANG#   
endext:#                                                                CB    72
#                                                                       CB    73
systemheader:(CB00)#                                                    CB00   1
#################################################################       CB00   2
#                                                               #       CB00   3
#                             CB00                              #       CB00   4
#                                                               #       CB00   5
#################################################################       CB00   6
SUBROUTINE CB00#                                                        CB00   7
include:(AACOMN)#                                                       CB00   8
include:(CBCOMN)#                                                       CB00   9
#                                                                       CB00  10
CALL CB10#                             Read optional data               CB00  11
CALL CB20#                             Process lrrefl:                  CB00  12
#                                                                       CB00  13
RETURN#                                                                 CB00  14
END#                                                                    CB00  15
systemheader:(CB01)#                                                    CB01   1
###################################################################     CB01   2
#                                                                 #     CB01   3
#                             CB01                                #     CB01   4
#                                                                 #     CB01   5
###################################################################     CB01   6
SUBROUTINE CB01(DH1,DH2,DT)#                                            CB01   7
#                                                                       CB01   8
## COMPUTE NORMALIZED CROSS PRODUCT                                     CB01   9
## XRAY PROGRAM BY H.L. AMMON, SEPTEMBER 1971                           CB01  10
## XTAL PROGRAM BY J.M. STEWART, NOVEMBER 1980                          CB01  11
#                                                                       CB01  12
REAL DH1(3)#                           First vector                     CB01  13
REAL DH2(3)#                           Second vector                    CB01  14
REAL S#                                Normalization factor             CB01  15
REAL DT(3)#                            Cross product                    CB01  16
REAL DB1#                                                               CB01  17
#                                                                       CB01  18
DB1=1.0#                               Cb01                             CB01  19
DT(1)=DH1(2)*DH2(3)-DH1(3)*DH2(2)#     First element                    CB01  20
DT(2)=DH1(3)*DH2(1)-DH1(1)*DH2(3)#     Second element                   CB01  21
DT(3)=DH1(1)*DH2(2)-DH1(2)*DH2(1)#     Third element                    CB01  22
S=DB1/SQRT(DT(1)**2+DT(2)**2+DT(3)**2)# Normalization factor            CB01  23
DT(1)=DT(1)*S ; DT(2)=DT(2)*S ; DT(3)=DT(3)*S# Normalize                CB01  24
#                                                                       CB01  25
RETURN#                                                                 CB01  26
END#                                                                    CB01  27
systemheader:(CB02)#                   Invert a 3x3 matrix              CB02   1
####################################################################### CB02   2
#                                                                     # CB02   3
#                          CB02                                       # CB02   4
#                                                                     # CB02   5
####################################################################### CB02   6
SUBROUTINE CB02(MI,MO)#                                                 CB02   7
#                                                                       CB02   8
REAL MI(3,3),MO(3,3)#                                                   CB02   9
REAL DD#                                                                CB02  10
INTEGER I,J#                                                            CB02  11
#                                                                       CB02  12
MO(1,1)=MI(2,2)*MI(3,3)-MI(2,3)*MI(3,2)# Calculate adjunct              CB02  13
MO(1,2)=MI(1,3)*MI(3,2)-MI(1,2)*MI(3,3)# Calculate adjunct              CB02  14
MO(1,3)=MI(1,2)*MI(2,3)-MI(1,3)*MI(2,2)# Calculate adjunct              CB02  15
MO(2,1)=MI(2,3)*MI(3,1)-MI(2,1)*MI(3,3)# Calculate adjunct              CB02  16
MO(2,2)=MI(1,1)*MI(3,3)-MI(1,3)*MI(3,1)# Calculate adjunct              CB02  17
MO(2,3)=MI(1,3)*MI(2,1)-MI(1,1)*MI(2,3)# Calculate adjunct              CB02  18
MO(3,1)=MI(2,1)*MI(3,2)-MI(3,1)*MI(2,2)# Calculate adjunct              CB02  19
MO(3,2)=MI(1,2)*MI(3,1)-MI(1,1)*MI(3,2)# Calculate adjunct              CB02  20
MO(3,3)=MI(1,1)*MI(2,2)-MI(1,2)*MI(2,1)# Calculate adjunct              CB02  21
DD=MI(1,1)*MO(1,1)+MI(2,1)*MO(1,2)+MI(3,1)*MO(1,3)# Get determinant     CB02  22
DO I=1,3; DO J=1,3; MO(I,J)=MO(I,J)/DD# Store as single                 CB02  23
RETURN#                                                                 CB02  24
END#                                                                    CB02  25
systemheader:(CB04)#                                                    CB04   1
####################################################################    CB04   2
#                                                                  #    CB04   3
#                              CB04                                #    CB04   4
#                                                                  #    CB04   5
####################################################################    CB04   6
SUBROUTINE CB04(A,B,C)#                                                 CB04   7
#                                                                       CB04   8
REAL A(3,3)#                           Matrix                           CB04   9
REAL B(3)#                             Vector                           CB04  10
REAL C(3)#                             New vector c=a*b                 CB04  11
#                                                                       CB04  12
C(1)=A(1,1)*B(1)+A(1,2)*B(2)+A(1,3)*B(3)# First element                 CB04  13
C(2)=A(2,1)*B(1)+A(2,2)*B(2)+A(2,3)*B(3)# Second element                CB04  14
C(3)=A(3,1)*B(1)+A(3,2)*B(2)+A(3,3)*B(3)# Third element                 CB04  15
RETURN#                                                                 CB04  16
END#                                                                    CB04  17
systemheader:(CB05)#                                                    CB05   1
#################################################################       CB05   2
#                                                               #       CB05   3
#                            CB05                               #       CB05   4
#                                                               #       CB05   5
#################################################################       CB05   6
SUBROUTINE CB05(A,B,C)#                                                 CB05   7
#                                                                       CB05   8
## 3X3 MATRIX MULTIPLY                                                  CB05   9
#                                                                       CB05  10
REAL A(3,3)#                           First matrix                     CB05  11
REAL B(3,3)#                           Second matrix                    CB05  12
REAL C(3,3)#                           Product matrix                   CB05  13
INTEGER I#                             Loop index                       CB05  14
INTEGER J#                             Loop index                       CB05  15
INTEGER K#                             Loop index                       CB05  16
#                                                                       CB05  17
DO I=1,3#                              Loop over 3 elements             CB05  18
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  CB05  19
DO J=1,3#                              Loop over 3 elements             CB05  20
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  CB05  21
C(I,J)=0.0#                            Initialize output element        CB05  22
DO K=1,3#                              Sum of element products          CB05  23
C(I,J)=C(I,J)+A(I,K)*B(K,J)#           Product element                  CB05  24
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  CB05  25
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  CB05  26
#                                                                       CB05  27
RETURN#                                                                 CB05  28
END#                                                                    CB05  29
systemheader:(CB06)#                                                    CB06   1
##################################################################      CB06   2
#                                                                #      CB06   3
#                            CB06                                #      CB06   4
#                                                                #      CB06   5
##################################################################      CB06   6
SUBROUTINE CB06(H1,H2,T)#                                               CB06   7
#                                                                       CB06   8
## FORM A RIGHT-HANDED UNIT VECTOR TRIPLE                               CB06   9
## H.L. AMMON, SEPTEMBER 1971                                           CB06  10
## J.M. STEWART, NOVEMBER 1980                                          CB06  11
#                                                                       CB06  12
REAL H1(3)#                            First input vector               CB06  13
REAL H2(3)#                            Second input vector              CB06  14
REAL S#                                Normalization factor             CB06  15
REAL T(3,3)#                           Vector triple (columns)          CB06  16
REAL TEMP(3)#                          Temporary storage                CB06  17
REAL TEMP1(3)#                         Temporary storage                CB06  18
#                                                                       CB06  19
S=1.0/SQRT(H1(1)**2+H1(2)**2+H1(3)**2)# Normalization factor            CB06  20
T(1,1)=H1(1)*S#                        First element vector 1           CB06  21
T(2,1)=H1(2)*S#                        Second                           CB06  22
T(3,1)=H1(3)*S#                        Third                            CB06  23
#                                                                       CB06  24
CALL CB01(H1,H2,TEMP)#                 Vector 3                         CB06  25
T(1,3)=TEMP(1)#                        First element                    CB06  26
T(2,3)=TEMP(2)#                        Second                           CB06  27
T(3,3)=TEMP(3)#                        Third                            CB06  28
#                                                                       CB06  29
CALL CB01(TEMP,H1,TEMP1)#              Vector 2                         CB06  30
T(1,2)=TEMP1(1)#                       First element                    CB06  31
T(2,2)=TEMP1(2)#                       Second                           CB06  32
T(3,2)=TEMP1(3)#                       Third                            CB06  33
#                                                                       CB06  34
RETURN#                                                                 CB06  35
END#                                                                    CB06  36
systemheader:(CB07)#                   Vector normalization             CB07   1
###############################################################         CB07   2
#                                                                       CB07   3
#                             CB07                                      CB07   4
#                                                                       CB07   5
###############################################################         CB07   6
SUBROUTINE CB07(V,B)#                  W. dreissig 10/85                CB07   7
IMPLICIT INTEGER (I-N)#                                                 CB07   8
#                                                                       CB07   9
# VECTOR NORMALIZATION V=V/SQRT(VX**2+VY**2+VZ**2)                      CB07  10
#                                                                       CB07  11
REAL     V(3)#                         Vector (unnormalized,n           CB07  12
REAL     B#                            Value of vector                  CB07  13
INTEGER  I#                            Index                            CB07  14
#                                                                       CB07  15
B=0.#                                                                   CB07  16
DO I=1,3; B=B+V(I)*V(I)#               Add squares                      CB07  17
B=SQRT(B)#                             Value                            CB07  18
DO I=1,3; V(I)=V(I)/B#                 Normalization                    CB07  19
#                                                                       CB07  20
RETURN#                                                                 CB07  21
END#                                                                    CB07  22
systemheader:(CB09)#                   Vector dot product               CB09   1
###############################################################         CB09   2
#                                                                       CB09   3
#                             CB09                                      CB09   4
#                                                                       CB09   5
###############################################################         CB09   6
SUBROUTINE CB09(V1,V2,P)#              W. dreissig 10/85                CB09   7
#                                                                       CB09   8
REAL     V1(3),V2(3),P#                Vectors                          CB09   9
INTEGER  I#                            Index                            CB09  10
#                                                                       CB09  11
P=0.0#                                 Initialize                       CB09  12
DO I=1,3; P=P+V1(I)*V2(I)#             Add component                    CB09  13
#                                                                       CB09  14
RETURN#                                                                 CB09  15
END#                                                                    CB09  16
systemheader:(CB10)#                                                    CB10   1
################################################################        CB10   2
#                                                              #        CB10   3
#                            CB10                              #        CB10   4
#                                                              #        CB10   5
################################################################        CB10   6
SUBROUTINE CB10#                                                        CB10   7
IMPLICIT INTEGER (I-N)#                                                 CB10   8
include:(AACOMN)#                                                       CB10   9
include:(CBCOMN)#                                                       CB10  10
#                                                                       CB10  11
## J.M. STEWART, NOVEMBER 1980                                          CB10  12
## G. DAVENPORT, JANUARY 1982                                           CB10  13
## T. MASLEN, JANUARY 1984                                              CB10  14
## N. SPADACCINI, MARCH 1989 - PSI SCANS INCLUDED                       CB10  15
#                                                                       CB10  16
INTEGER I#                             Loop index                       CB10  17
INTEGER IP#                            Packet pointer                   CB10  18
INTEGER IPP#                           Pointer                          CB10  19
INTEGER IREL(4)#                       Relative pointers                CB10  20
INTEGER IWANT(3)#                      Want list for indexpkt           CB10  21
INTEGER J,K,MARK#                      Loop index                       CB10  22
INTEGER JP#                            Pointer                          CB10  23
INTEGER KEY(4)#                        Keys for indexpkt                CB10  24
INTEGER M#                             Start of string                  CB10  25
INTEGER N#                             End of string                    CB10  26
INTEGER PSIZE#                         Packet size                      CB10  27
INTEGER LINEX#                         Linids expected                  CB10  28
#                                                                       CB10  29
REAL CHI#                              Chi                              CB10  30
REAL PHI#                              Phi                              CB10  31
REAL TCTRAN(3,3)#                      Tc transpose                     CB10  32
REAL TPHI(3,3)#                        Vector triple                    CB10  33
REAL U(3,3)#                           Rotation matrix                  CB10  34
REAL VEC(10)#                          Orref vectors                    CB10  35
REAL ORT(10)#                          Orient input                     CB10  36
REAL CVOL#                                                              CB10  37
#                                                                       CB10  38
#                                                                       CB10  39
chardata:(CB101,"   gaussian analytical  sphericalcylindrical$#         CB10  40
     tables ")#                                                         CB10  41
chardata:(CB102, DIFF ORIENT END )#                                     CB10  42
chardata:(CB103,"AC noyes")#                                            CB10  43
chardata:(CB104," Calculation method                     ")#            CB10  44
chardata:(CB105," Apply correction to                    ")#            CB10  45
chardata:(CB106," Store Tbar on the bdf                  ")#            CB10  46
chardata:(CB107," Limit reflections to                   ")#            CB10  47
chardata:(CB108," List reflections                       ")#            CB10  48
chardata:(CB109," Dataset number                         ")#            CB10  49
chardata:(CB10A," Absorption coef /mm                    ")#            CB10  50
chardata:(CB10C," Internal precision factor              ")#            CB10  51
chardata:(CB10D," Calculate direction cosines         yes")#            CB10  52
chardata:(CB10E," Radiation wavelength                   ")#            CB10  53
chardata:(CB10F,"(input)     (calculated)(default)          ")#         CB10  54
chardata:(CB10G," Atom type   Atomic number   Atoms Cell        MUA")#  CB10  55
chardata:(CB10H," Plot crystal views in                      ")#        CB10  56
chardata:(CB10I,"projection    stereo ")#                               CB10  57
chardata:(CB10J," Rotation senses  theta/chi/phi                   ")#  CB10  58
chardata:(CB10K,"/omega/psi  ")#                                        CB10  59
chardata:(CB10L,"store irelf2rel frel ")#                               CB10  60
chardata:(CB10M," Crystal radius in mm                   ")#            CB10  61
#                                                                       CB10  62
realdata:(FM,[210303.,350403.,521042.])#                                CB10  63
datastuff:#                                                             CB10  64
#                                                                       CB10  65
# SET UP INPUT AND OUTPUT FILE BUFFERS                                  CB10  66
# ------------------------------------                                  CB10  67
#                                                                       CB10  68
IOMARK(1)=QXSTR#                       Start of input buffer            CB10  69
IOMARK(2)=IOMARK(1)+bdfbuf:#           Start of output buffer           CB10  70
incrqx:(MARK,IOMARK(2)+bdfbuf:,CB1001)# Get mem for buffers             CB10  71
#                                                                       CB10  73
# INTERPRET THE ABSORB LINE                                             CB10  74
# -------------------------                                             CB10  75
#                                                                       CB10  76
FOR(I=0;I>=0;I=I+1)#                   Loop over input options          CB10  77
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  CB10  78
stringlim:(I,J,K)#                     Get string limits                CB10  79
#                                                                       CB10  80
setint:(PR,LIST,20,99999)#             Print reflections                CB10  81
setint:(DS,ISET,  1,    1)#            Data set number                  CB10  82
setflg:(DC,ICOS, no:)#                 Output d.c.'s                    CB10  83
setflg:(AN,IANG, no:)#                 save angles to archive
ifvo:(BUFIN(I+1)) BREAK#               Exit when field void             CB10  84
ifnm:(BUFIN(I+1)) iquit:(CB1002,I+1)#  Must be char string              CB10  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  CB10  86
#                                                                       CB10  87
copyfile:(1,2,1,decr:(lrcell:))#       Copy up to lrcell                CB10  88
#                                                                       CB10  89
# OUTPUT FOR USER                                                       CB10  90
# ---------------                                                       CB10  91
#                                                                       CB10  92
IF(ICOS>0) writeline:(0,CB10D,NCB10D,3,3)# Store d.c.'s                 CB10  93
ncodefld:(FLOAT(LIST),1,CB108,400613.,1)# Format list reflns            CB10  94
writeline:(0,CB108,NCB108,3,3)#        Print list limit                 CB10  95
ncodefld:(FLOAT(LIM),1,CB107,400613.,1)# Format process limit           CB10  96
writeline:(0,CB109,NCB109,3,3)#        Print data set number            CB10  97
#NCODEFLD:(SCALE,1,CB10C,400822.,1)#    FORMAT INTERNAL SCALE           CB10  98
#WRITELINE:(0,CB10C,NCB10C,3,3)#        PRINT INTERNAL SCALE            CB10  99
DISTSC=0.1; RAD=RAD*0.1#               Convert to cm                    CB10 100
#                                                                       CB10 101
# GET THE B MATRIX FROM LRCELL:                                         CB10 102
# -----------------------------                                         CB10 103
#                                                                       CB10 104
DO K=1,10#                             Loop over lrcell packets         CB10 105
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  CB10 106
readwpkt:(1,lrcell:,PSIZE,IP,0)#       Point to next lrcell pkt         CB10 107
IF(IP<=0) BREAK#                       Exit after last packet           CB10 108
writepkt:(2,lrcell:,PSIZE,IPP)#                                         CB10 109
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                    CB10 110
IF(K==5)#                              Skip unless trans matrix         CB10 111
DO I=1,3;  DO J=1,3#                   Loop over columns & rows         CB10 112
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  CB10 113
JP=IP+3*I-3+J#                         Point into qx                    CB10 114
BX(J,I)=QX(JP)#                        Store element                    CB10 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  CB10 116
IF(K==6) CVOL=QX(IP+1)#                Get cell volume                  CB10 117
IF(K==8)#                              Test if inv metric tensor        CB10 118
DO I=1,9; MT(I)=QX(IP+I)#              Store metric tensor              CB10 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  CB10 120
#                                                                       CB10 121
copyfile:(1,2,incr:(lrcell:),decr:(lrexpl:))# Copy up to l         cb   CB10 122
#                                                                       CB10 123
# GET LAMBDA FROM LREXPL:                                               CB10 124
# -----------------------                                               CB10 125
#                                                                       CB10 126
WAV=0.71073#                           Default molybednum               CB10 127
IWANT(1)=1; IWANT(2)=2#                Request lambda                   CB10 128
KEY(1)=1; KEY(3)=0; KEY(2)=2; KEY(4)=0# Set search keys                 CB10 129
indexpkt:(1,lrexpl:,PSIZE,IP,2,KEY,IWANT,IREL)#                         CB10 130
IF(KEY(4)<=0)iquit:(CB1003)#           Essential is missing             CB10 131
REPEAT#                                Loop over lrexpl: packets        CB10 132
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  CB10 133
readwpkt:(1,lrexpl:,PSIZE,IP,0)#       Point to next pkt                CB10 134
IF(IP<=0) BREAK#                       Exit after last pkt              CB10 135
writepkt:(2,lrexpl:,PSIZE,IPP)#                                         CB10 136
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                    CB10 137
IPP=IP+IREL(1)#                        Point to data set number         CB10 138
IF(ISET!=NINT(QX(IPP))) NEXT#          Skip if wrong dset               CB10 139
IF(IREL(2)==0) NEXT#                   Skip if lambda not present       CB10 140
WAV=QX(IP+IREL(2))#                    Lambda                           CB10 141
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  CB10 142
ncodefld:(WAV,1,CB10E,400952.,1)#      Format wavelength                CB10 143
writeline:(0,CB10E,NCB10E,3,3)#        Print radiation wavelength       CB10 144
ISET=ISET*1000#                                                         CB10 145
#                                                                       CB10 146
#                                                                       CB10 147
copyfile:(1,2,incr:(lrexpl:),decr:(lrrefl:))# Copy up to lrrefl:        CB10 148
#                                                                       CB10 149
# READ DIFF AND ORIENT LINES                                            CB10 150
# -------------------------                                             CB10 151
#                                                                       CB10 152
LINEX=0#                                                                CB10 153
OMIN=-.837758 ; OMAX=1.01229#          Default omega limits - syntex    CB10 154
PHI90=0.0#                             Default 0.0                      CB10 155
DO J=1,5; ROT(J)=1.0#                  Default clockwise                CB10 156
DO K=1,3#                              Loop over input lines            CB10 157
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  CB10 158
readline:(CB102,NCB102)#               Read control line                CB10 159
#                                                                       CB10 160
# DIFF LINE                                                             CB10 161
# ---------                                                             CB10 162
#                                                                       CB10 163
IF(LINID==1)#                          Diff line                        CB10 164
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  CB10 165
LINEX=LINEX+1#                                                          CB10 166
DO J=1,3#                              Loop over compulsory fields 1-   CB10 167
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  CB10 168
ifnm:(BUFIN(J)) iquit:(CB1004)#        Character field                  CB10 169
stringlim:(J,M,N)#                     Get string limits                CB10 170
compchar:(CHRIN,M,CB103,1,1,1,2,N)#    Identify string                  CB10 171
IF(N==0) iquit:(CB1005)#               Default clockwise                CB10 172
IF(N==1) ROT(J)=-1.0#                  Store rotation element           CB10 173
movechar:(CHRIN,M,CB10J,40+(J-1)*2,1,0)# Store in op header             CB10 174
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  CB10 175
DO J=4,5#                                                               CB10 176
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  CB10 177
ifvo:(BUFIN(J)) NEXT#                  If not there - next              CB10 178
movechar:(CB10K,1+(J-4)*6,CB10J,32+(J-4)*6,6,0)# Title                  CB10 179
stringlim:(J,M,N)#                     Get string limits                CB10 180
compchar:(CHRIN,M,CB103,1,1,1,2,N)#    Identify string                  CB10 181
IF(N==0) iquit:(CB1005)#               Default clockwise                CB10 182
IF(N==1) ROT(J)=-1#                                                     CB10 183
movechar:(CB103,N,CB10J,40+(J-1)*2,1,0)#                                CB10 184
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  CB10 185
writeline:(0,CB10J,NCB10J,3,3)#                                         CB10 186
ifnm:(BUFIN(6)) PHI90=BUFIN(6)*.017453# Phi at chi=90              cb   CB10 187
ELSE            PHI90=0.0#             Default 0.0                      CB10 188
ifnm:(BUFIN(7)) OMIN=BUFIN(7)/57.29578# Omega min in radians       cb   CB10 189
ifnm:(BUFIN(8)) OMAX=BUFIN(8)/57.29578# Omega max in radians       cb   CB10 190
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  CB10 191
#                                                                       CB10 192
# ORIENT LINE                                                           CB10 193
# -----------                                                           CB10 194
#                                                                       CB10 195
ELSE IF(LINID==2)#                     Orient line                      CB10 196
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  CB10 197
LINEX=LINEX+2#                                                          CB10 198
DO I=1,10#                             Loop over fields                 CB10 199
IF(BUFIN(I)<=voidflg:) iquit:(CB1006)# Non-numeric, so quit             CB10 200
DO I=1,10; ORT(I)=BUFIN(I)#            Store as dp                      CB10 201
postm:(BX,ORT,VEC)#                    B x hkl                          CB10 202
postm:(BX,ORT(6),VEC(4))#              B x hkl                          CB10 203
triple:(VEC,VEC(4),TC)#                Form vector triple               CB10 204
#                                                                       CB10 205
CHI=0.01745329*ORT(4)#                 Chi                              CB10 206
PHI=0.01745329*ORT(5)#                 Phi                              CB10 207
VEC(1)=ROT(1)*COS(CHI)*COS(PHI)#       Element 1                        CB10 208
VEC(2)=ROT(1)*ROT(3)*COS(CHI)*SIN(PHI)# Element 2                       CB10 209
VEC(3)=ROT(1)*ROT(2)*SIN(CHI)#         Element 3                        CB10 210
CHI=0.01745329*ORT(9)#                 Chi                              CB10 211
PHI=0.01745329*ORT(10)#                Phi                              CB10 212
VEC(4)=ROT(1)*COS(CHI)*COS(PHI)#       Element 1                        CB10 213
VEC(5)=ROT(1)*ROT(3)*COS(CHI)*SIN(PHI)# Element 2                       CB10 214
VEC(6)=ROT(1)*ROT(2)*SIN(CHI)#         Element 3                        CB10 215
#                                                                       CB10 216
triple:(VEC,VEC(4),TPHI)#              Form vector triple               CB10 217
#                                                                       CB10 218
DO I=1,3; DO J=1,3; TCTRAN(I,J)=TC(J,I)#                                CB10 219
mmult:(TPHI,TCTRAN,U)#                 Matrix multiply                  CB10 220
#                                                                       CB10 221
DO I=1,3; DO J=1,3; UINV(I,J)=U(J,I)#                                   CB10 222
mmult:(U,BX,UB)#                       Matrix multiply                  CB10 223
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  CB10 224
# IGNORE THIRD LINID CASE                                               CB10 225
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  CB10 226
IF(LINEX<3)iquit:(CB1007)#             Compulsory fields missing        CB10 227
#                                                                       CB10 228
RETURN#                                                                 CB10 229
END#                                                                    CB10 230
systemheader:(CB20)#                                                    CB20   1
####################################################################    CB20   2
#                                                                  #    CB20   3
#                             CB20                                 #    CB20   4
#                                                                  #    CB20   5
####################################################################    CB20   6
SUBROUTINE CB20#                       Modified w. dreissig 1986        CB20   7
include:(AACOMN)#                                                       CB20   8
include:(CBCOMN)#                                                       CB20   9
#                                                                       CB20  10
###                                                                     CB20  11
## I/O FOR LRREFL:, CALCULATE ANGLES AND BEAM VECTORS                   CB20  12
## GEORGE DAVENPORT, JANUARY 1982                                       CB20  13
## T. MASLEN, JANUARY 1984                                              CB20  14
## N. SPADACCINI, MARCH 1989 - PSI SCANS INCLUDED                       CB20  15
#                                                                       CB20  16
INTEGER BISECT#                        Bisect/psi yes/no                CB20  17
REAL CHI#                              Chi                              CB20  18
REAL COSCHI#                           Cos(chi)                         CB20  19
REAL COSPHI#                           Cos(phi)                         CB20  20
REAL COSPSI#                           Cos(psi)#                        CB20  21
REAL COSOM#                            Cos(om)                          CB20  22
REAL COST#                             Cos(theta)                       CB20  23
REAL DET#                              Determinant of b-matrix          CB20  24
REAL DXYZ(3)#                          Used to get chi, phi             CB20  25
INTEGER I,J,M,II#                                                       CB20  26
INTEGER INP#                           Pointer into input packet        CB20  27
INTEGER IP#                            Miscellaneous pointer            CB20  28
INTEGER IPP#                           Output packet pointer            CB20  29
INTEGER IRP#                           Pointer to psi                   CB20  30
INTEGER IREL(19)#                      Relative pointers                CB20  31
INTEGER IWANT(19)#                     Want list for indexpacket        CB20  32
INTEGER OUTP#                          Output packet pointer            CB20  33
REAL OM#                               Omega - eularian                 CB20  34
REAL OMEGA#                            Omega - diffractometer           CB20  35
REAL PHI#                              Phi                              CB20  36
REAL PSI#                              Psi                              CB20  37
INTEGER PSIZ#                          Input packet size                CB20  38
INTEGER PSIZO#                         Output packet size               CB20  39
INTEGER COUNT,COUNDN#                  Total count of reflections       CB20  40
INTEGER KEY(4)
REAL R(3,3)#                           Normed matrix                    CB20  41
REAL RDST#                             Normalization factor             CB20  42
REAL SINCHI#                           Sin(chi)                         CB20  43
REAL SINPHI#                           Sin(phi)                         CB20  44
REAL SINPSI#                           Sin(psi)                         CB20  45
REAL SINOM#                            Sin(om)                          CB20  46
REAL SINT#                             Sin(theta)                       CB20  47
REAL T(6)#                             Temporary                        CB20  48
REAL T1,T2#                            Temporary                        CB20  49
REAL TEMP(8)#                          Calling arguments for ab03       CB20  50
REAL TH#                               Theta                            CB20  51
REAL TTHETA#                           Twice theta                      CB20  52
REAL X(3)#                             Miller indices                   CB20  53
REAL CBMAX, CBMIN#                     Min astar                        CB20  54
REAL UB1(3,3)#                         Inverse ub                       CB20  55
REAL U(3),Q1(3),Q(3),PHIM(3,3),CHIM(3,3),OHMM(3,3)# Flack azimuth       CB20  56
REAL FAPSI#                            Crystal based azimuthal angle    CB20  57
REAL XR13,XR23,XR31,XR32,XR33#                                          CB20  58
REAL T3,T4#                                                             CB20  59
#                                                                       CB20  60
chardata:(CB031, ARCHIV END   )#                                        CB20  61
chardata:(CB032," TbarIrelF2rlFrelsigIsgF2sigFstoreN       DIR. COS$#   CB20  62
      D ")#                                                             CB20  63
chardata:(CB033,"   h   k   l 2theta   omega     chi     phi   dPSI   $#CB20  64
cBPSI   ( 0.0,  sin,  cos)                                           ")#CB20  65
chardata:(CB034," Max absorption A*          $#                         CB20  66
Min transmission A       ")#                                            CB20  67
chardata:(CB035," Min absorption A*          $#                         CB20  68
Max transmission A       ")#                                            CB20  69
chardata:(CB036,HKL C  )#                                               CB20  70
realdata:(FM,[40413.,80413.,120413.,190622.,270722.,350722.,430722.,#   CB20  71
500722.,580722.,660742.,730742.,800742.,900742.,9710742.,1040742.,#     CB20  72
1150742.,1220742.,1290742.])#                                           CB20  73
realdata:(FM1,[150213.,190413.,230413.,270413.,370922.,470922.,#        CB20  74
800862.,350742.,430742.,510742.,590742.,670742.,750742.,#               CB20  75
370902.])#                                                              CB20  76
#SAVE CBMAX,CBMIN,COUNDN,COUNT,IPP,IREL#                                 CB20  78
datastuff:#                                                             CB20  79
#                                                                       CB20  80
#                                                                       CB20  81
# INITIALIZE LRREFL: I/O                                                CB20  82
# ----------------------                                                CB20  83
#                                                                       CB20  84
invert:(UB,UB1)#                       Invert matrix                    CB20  85
CBMAX=0. ; CBMIN=100.#                 Initialize astar max and min     CB20  86
COUNT=0 ; COUNDN=0#                    Zero counts                      CB20  87
DO I=1,8; TEMP(I)=0.#                  Zero temp array                  CB20  88
IWANT(1)=1#                            Request hkl                      CB20  89
IWANT(2)=2#                            Request stol                     CB20  90
IWANT(3)=ISET+19#                      Request diff psi angle           CB20  91
IWANT(4)=ISET+11#                      Append flack psi                 CB20  92
KEY(1)=1; KEY(2)=3; KEY(4)=0#          Hkl required; update mode        CB20  93
KEY(3)=4#                              Will add psi regardless if       CB20  94
IF(ICOS>0)#                            Add direction cosines            CB20  96
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  CB20  97
M=KEY(3)#     
DO I=1,6; IWANT(M+I)=ISET+205+I#       Add dir.cosines                  CB20  99
KEY(3)=KEY(3)+6
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  CB20 100
IF(IANG>0)#                            Add direction cosines            CB20  96
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  CB20  97
M=KEY(3)#     
DO I=1,4; IWANT(M+I)=ISET+4+I#         Add angles N005,N006,N007
KEY(3)=KEY(3)+4
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  CB20 100
indexpkt:(1,lrrefl:,PSIZ,IP,2,KEY,IWANT,IREL)# Search directory         CB20 101
IF(IP<=0) iquit:(CB2001)#              Missing lrrefl:                  CB20 102
IF(KEY(4)<0) iquit:(CB2002)#           Missing hkl from bdf             CB20 103
PSIZO=KEY(4)#                          Output packet size               CB20 104
#                                                                       CB20 105
#                                                                       CB20 106
LINRM=3#                                                                CB20 107
J=0#                                   Header pointer increment         CB20 108
IF(ICOS>0)#                                                             CB20 109
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  CB20 110
movechar:(CB032,35,CB033,63+J,23,0)#   Heading for dir.cosine           CB20 111
#         PREPARE FOR LATER COMPUTING OF N,D-VECTORS                    CB20 112
DO I=1,3;DO J=1,3; R(I,J)=BX(I,J)#     Store transformation matrix      CB20 113
DO I=1,3; CALL CB07(R(1,I),DET)#       Normalize components o           CB20 114
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  CB20 115
#                                                                       CB20 116
# CALCULATE ANGLES AND BEAM VECTORS                                     CB20 117
# ---------------------------------                                     CB20 118
#                                                                       CB20 119
REPEAT#                                                                 CB20 120
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  CB20 121
readwpkt:(1,lrrefl:,PSIZ,IP,0)#        Read a packet                    CB20 122
IF(IP<=0) BREAK#                                                        CB20 123
writepkt:(2,lrrefl:,PSIZO,IPP)#        **** writepkt ****               CB20 124
movereal:(QX,IP+1,QX,IPP+1,PSIZ,1)#    Copy old pkt across              CB20 125
BISECT=yes: ; OM=0.0 ; PSI=0.0#        Default - bisecting mode         CB20 126
TEMP(1)=0.0#                           Set flag for no break            CB20 127
TEMP(2)=0.0#                           Set flag to process reflecti     CB20 128
COUNT=COUNT+1#                         Count reflections                CB20 129
COUNDN=COUNDN+1#                       Count processed reflections      CB20 130
INP=IP+IREL(1)#                        Pointer to hkl                   CB20 131
gethkl:(INP,X)#                        Get hkl                          CB20 132
II=IP+IREL(2)#                         Point to stol                    CB20 133
IF(II>IP) SINT=QX(II)*WAV#             Calculate sin(theta)             CB20 134
ELSE#                                  If stol not on bdf               CB20 135
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  CB20 136
SINT=X(1)*(X(1)*MT(1)+X(2)*MT(2)+X(3)*MT(3))$# Calc 4*stol**2           CB20 137
+    X(2)*(X(1)*MT(4)+X(2)*MT(5)+X(3)*MT(6))$# Calc 4*stol**2           CB20 138
+    X(3)*(X(1)*MT(7)+X(2)*MT(8)+X(3)*MT(9))# Calc 4*stol**2            CB20 139
SINT=WAV*SQRT(SINT)/2.#                Calc sine theta                  CB20 140
IF(SINT>1.) iquit:(CB2003)#            Nonsense sine theta              CB20 141
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  CB20 142
#                                                                       CB20 143
IRP=IREL(3)#                           Pointer to psi in degrees        CB20 144
IF(IRP>0)#                                                              CB20 145
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  CB20 146
BISECT=no:#                            Not bisecting mode               CB20 147
PSI=QX(IP+IRP)/57.29577951#            Psi in radians                   CB20 148
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  CB20 149
TH=ASIN(SINT)#                         Theta in radians                 CB20 150
TEMP(4)=SINT/WAV#                      Sin(theta)/wavelength            CB20 151
TTHETA=114.591559*ASIN(SINT)#          Two theta                        CB20 152
COST=SQRT(1.0-SINT**2)#                Calculate cos(theta)             CB20 153
RDST=0.5*ROT(1)*WAV/SINT#              Multiplier                       CB20 154
DXYZ(1)=UB(1,1)*X(1)+UB(1,2)*X(2)+UB(1,3)*X(3)#                         CB20 155
DXYZ(2)=UB(2,1)*X(1)+UB(2,2)*X(2)+UB(2,3)*X(3)#                         CB20 156
DXYZ(3)=UB(3,1)*X(1)+UB(3,2)*X(2)+UB(3,3)*X(3)#                         CB20 157
DXYZ(1)=DXYZ(1)*RDST#                                                   CB20 158
DXYZ(2)=DXYZ(2)*RDST#                                                   CB20 159
DXYZ(3)=DXYZ(3)*RDST#                                                   CB20 160
#                                                                       CB20 161
# CALCULATE CHI, PHI, COSINES AND SINES - BISECTING MODE                CB20 162
# ------------------------------------------------------                CB20 163
#                                                                       CB20 164
CHI=ATAN2((ROT(2)*DXYZ(3)),SQRT(DXYZ(1)**2+DXYZ(2)**2))#                CB20 165
IF(NINT(ABS(57.29577951*CHI))==90) PHI=PHI90#                           CB20 166
ELSE PHI=ATAN2((ROT(3)*DXYZ(2)),DXYZ(1))#                               CB20 167
COSPHI=COS(PHI) ; SINPHI=SIN(PHI)#                                      CB20 168
COSCHI=COS(CHI) ; SINCHI=SIN(CHI)#                                      CB20 169
#                                                                       CB20 170
# CALCULATE SETTING ANGLES - GENERAL MODE                               CB20 171
# ---------------------------------------                               CB20 172
#                                                                       CB20 173
IF(BISECT==no:)#                       Non-bisecting mode               CB20 174
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  CB20 175
SINPSI=SIN(PSI) ; COSPSI=COS(PSI)#     Sin and cos of psi               CB20 176
XR13=ROT(2)*SINCHI#                                                     CB20 177
XR23=ROT(5)*SINPSI*COSCHI#                                              CB20 178
XR31=ROT(3)*ROT(5)*SINPSI*SINPHI-ROT(2)*COSPSI*SINCHI*COSPHI#           CB20 179
XR32=-ROT(5)*SINPSI*COSPHI-ROT(2)*ROT(3)*COSPSI*SINCHI*SINPHI#          CB20 180
XR33=COSPSI*COSCHI#                                                     CB20 181
CHI=ATAN2(SQRT(XR31**2+XR32**2),XR33)#                                  CB20 182
PHI=ATAN2(-ROT(2)*ROT(3)*XR32,-ROT(2)*XR31)#                            CB20 183
OM=ATAN2(-ROT(4)*ROT(2)*XR23,ROT(2)*XR13)#                              CB20 184
#                                                                       CB20 185
# IS OMEGA OUT OF RANGE (-48 < OMEGA < 58) - USE EQUIVALENT SETTING     CB20 186
# -----------------------------------------------------------------     CB20 187
#                                                                       CB20 188
OMEGA=TH+OM#                           Diffractometer angle omega       CB20 189
IF((OMEGA<OMIN)\(OMEGA>OMAX))#         Outside diffractomter limits     CB20 190
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  CB20 191
CHI=-CHI#                              Chi to minus chi                 CB20 192
PHI=3.141592654+PHI#                   Rotate phi through pi            CB20 193
OM =3.141592654+OM#                    Rotate om through pi             CB20 194
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  CB20 195
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  CB20 196
COSPHI=COS(PHI) ; SINPHI=SIN(PHI)#     Store sin and cos of angles      CB20 197
COSCHI=COS(CHI) ; SINCHI=SIN(CHI)#                                      CB20 198
COSOM=COS(OM)   ; SINOM=SIN(OM)#                                        CB20 199
COST=COS(TH)    ; SINT=SIN(TH)#                                         CB20 200
#                                                                       CB20 201
OMEGA=(TH+OM)*57.29577951#             Omega in degrees                 CB20 202
IF(OMEGA-360.>=0.) OMEGA=OMEGA-360.#   Reset to between 0 and 360       CB20 203
TH= TH *57.29577951#                   Theta in degrees                 CB20 204
PHI=PHI*57.29577951#                   Phi in degrees                   CB20 205
CHI=CHI*57.29577951#                   Chi in degrees                   CB20 206
#                                                                       CB20 207
# CALCULATE BEAM VECTORS                                                CB20 208
# ----------------------                                                CB20 209
#                                                                       CB20 210
T1=ROT(1)*SINT*COSOM*COSPHI*COSCHI#    Elements of the primary and      CB20 211
T2=ROT(4)*COST*SINOM*COSCHI*COSPHI#    Diffracted beam vectors          CB20 212
T3=ROT(3)*COST*COSOM*SINPHI#                                            CB20 213
T4=ROT(1)*ROT(4)*ROT(3)*SINT*SINOM*SINPHI#                              CB20 214
T(1)=T1+T2+T3-T4 ; T(4)=T1-T2-T3-T4#                                    CB20 215
#                                                                       CB20 216
T1=ROT(1)*ROT(3)*SINT*COSOM*SINPHI*COSCHI#                              CB20 217
T2=ROT(4)*ROT(3)*COST*SINOM*COSCHI*SINPHI#                              CB20 218
T3=COST*COSOM*COSPHI#                                                   CB20 219
T4=ROT(1)*ROT(4)*SINT*SINOM*COSPHI#                                     CB20 220
T(2)=T1+T2-T3+T4 ; T(5)=T1-T2+T3+T4#                                    CB20 221
#                                                                       CB20 222
T1=ROT(1)*ROT(2)*SINT*COSOM*SINCHI#                                     CB20 223
T2=ROT(4)*ROT(2)*COST*SINOM*SINCHI#                                     CB20 224
T(3)=T1+T2 ; T(6)=T1-T2#                                                CB20 225
#                                                                       CB20 226
postm:(UINV,T,TEMP(3))#                Post multiply                    CB20 227
postm:(UINV,T(4),TEMP(6))#             Post multiply                    CB20 228
#                                                                       CB20 229
#                                                                       CB20 230
# CALCULATE FLACK AZIMUTHAL PSI ANGLE                                   CB20 231
# -----------------------------------                                   CB20 232
#                                                                       CB20 233
IF(NINT(X(1))==NINT(X(2))&NINT(X(1))==NINT(X(3)))# Special h==k==l      CB20 234
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  CB20 235
U(1)= X(1)#                            Define as  h                     CB20 236
U(2)=-X(1)#                            Define as -h                     CB20 237
U(3)= 0.#                              Define as 0                      CB20 238
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  CB20 239
ELSE#                                  General refln reference vector   CB20 240
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  CB20 241
U(1)=X(2)-X(3)#                        Define as k-l                    CB20 242
U(2)=X(3)-X(1)#                        Define as l-h                    CB20 243
U(3)=X(1)-X(2)#                        Define as h-k                    CB20 244
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  CB20 245
Q(1)=UB1(1,1)*U(1)+UB1(2,1)*U(2)+UB1(3,1)*U(3)#                         CB20 246
Q(2)=UB1(1,2)*U(1)+UB1(2,2)*U(2)+UB1(3,2)*U(3)#                         CB20 247
Q(3)=UB1(1,3)*U(1)+UB1(2,3)*U(2)+UB1(3,3)*U(3)#                         CB20 248
norm:(Q,DET)#                          Normalize q= q/||q||             CB20 249
#POSTM:(UB,X,Q1)#                      Q= UB . H                        CB20 250
#NORM:(Q1,XXX)#                        NORMALIZE                        CB20 251
#VDOTP:(Q1,Q,XXX)#                     GET DOT PRODUCT                  CB20 252
#WRITE(6,*)"DOT PROD ",XXX#            TEST DUMP - SHOULD BE ZERO       CB20 253
PHIM(1,1)= COSPHI;PHIM(1,2)=SINPHI;PHIM(1,3)=0.# Phi rotation matrix    CB20 254
PHIM(2,1)=-SINPHI;PHIM(2,2)=COSPHI;PHIM(2,3)=0.# Phi rotation matrix    CB20 255
PHIM(3,1)=    0.0;PHIM(3,2)=   0.0;PHIM(3,3)=1.# Rotate about z axis    CB20 256
postm:(PHIM,Q ,Q1)#                    Q1= phi . q                      CB20 257
CHIM(1,1)= COSCHI;CHIM(1,2)=   0.0;CHIM(1,3)=SINCHI# Phi rotation mat   CB20 258
CHIM(2,1)=    0.0;CHIM(2,2)=   1.0;CHIM(2,3)=0.# Phi rotation matrix    CB20 259
CHIM(3,1)=-SINCHI;CHIM(3,2)=   0.0;CHIM(3,3)=COSCHI# Rotate about z a   CB20 260
postm:(CHIM,Q1,Q )#                    Q=  chi . phi.q                  CB20 261
OHMM(1,1)=  COSOM;OHMM(1,2)= SINOM;OHMM(1,3)=0.# Phi rotation matrix    CB20 262
OHMM(2,1)= -SINOM;OHMM(2,2)= COSOM;OHMM(2,3)=0.# Phi rotation matrix    CB20 263
OHMM(3,1)=    0.0;OHMM(3,2)=   0.0;OHMM(3,3)=1.# Rotate about z axis    CB20 264
postm:(OHMM,Q ,Q1)#                    Q1= ohm . chi .phi . q           CB20 265
FAPSI=ATAN2(Q1(2),Q1(3))#              Q1=(sinp,0,cosp)                 CB20 266
QX(IPP+IREL(4))= FAPSI#                Flack crystal based psi on bdf   CB20 267
#                                                                       CB20 268
# STORE ITEMS AND APPLY CORRECTION                                      CB20 269
# --------------------------------                                      CB20 270
#                                                                       CB20 271
TEMP(1)=MAX(0.0,TEMP(1))#              Protect square root              CB20 272
J=4#
IF(ICOS>0)#                                                             CB20 273
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 CB20 274
OUTP=IPP+IREL(J+1)-1#                  Output address                   CB20 275
J=J+6
CALL CB01(TEMP(3),TEMP(6),T)#          Vector d=[p,s] in xtal-space     CB20 276
CALL CB01(T,TEMP(3),T(4))#             Vector n=[[p,s],p] in xtal-spa   CB20 277
#                    (SEE HAMILTON, ACTA CRYST.A26,P.71)                CB20 278
DO I=1,3#                              Loop over components             CB20 279
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 CB20 280
CALL CB09(R(1,I),T(4),QX(OUTP+I))#     Dot product - vector n           CB20 281
CALL CB09(R(1,I),T   ,QX(OUTP+3+I))#   Vector d in crystal sy           CB20 282
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 CB20 283
movereal:(QX,OUTP+1,BUFOT,13,6,0)#     Store for printing               CB20 284
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 CB20 285
IF(IANG>0)#                            Store hkl diff angles on archive
$(
QX(IPP+IREL(J+1))=PHI#                 save phi
QX(IPP+IREL(J+2))=CHI#                 save chi
QX(IPP+IREL(J+3))=OMEGA#               save omega
QX(IPP+IREL(J+4))=TTHETA#              save twotheta
J=J+4
$)
#                                                                       CB20 286
#                                                                       CB20 287
#                                                                       CB20 288
#                                                                       CB20 289
IF(COUNDN<=LIST)#                                                       CB20 290
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 CB20 291
DO I=1,3; BUFOT(I)=X(I)#               Hkl                              CB20 292
BUFOT(4)=TTHETA#                       Calculated 2theta                CB20 293
BUFOT(5)=OMEGA#                        Calculaed omega                  CB20 294
BUFOT(6)=CHI#                          Calculated chi                   CB20 295
BUFOT(7)=PHI#                          Calculated phi                   CB20 296
BUFOT(8)=  PSI*57.29577951#            Diffractometer based psi         CB20 297
BUFOT(9)=FAPSI*57.29577951#            Flack crystal based psi          CB20 298
BUFOT(10)=Q1(1)#                                                        CB20 299
BUFOT(11)=Q1(2)#                                                        CB20 300
BUFOT(12)=Q1(3)#                                                        CB20 301
ncodefld:(BUFOT,1,CHROT,FM,12)#        Encode values                    CB20 302
IF(ICOS>0) ncodefld:(BUFOT,13,CHROT,FM(13),6)# Encode dir.co            CB20 303
writeline:(0,CB033,NCB033,1,3)#                                         CB20 304
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 CB20 305
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  CB20 306
#                                                                       CB20 307
# COPY REST OF FILE                                                     CB20 308
# -----------------                                                     CB20 309
#                                                                       CB20 310
copyfile:(1,2,incr:(lrrefl:),endrecord:)# Copy rest of file             CB20 311
RETURN#                                                                 CB20 312
END#                                                                    CB20 313
pmacro:([no:])#                        Purge xtal system macros         CB20 314
