/*#                                                                     UNXC   3
flagon:(R)#                            Prevent tidying                          
*/                                                                              
/* The following C routines are callable from FORTRAN and provide               
 * access to Unix CPU time, wallclock time and date utilities.                  
 * The routines emulate the f77 library functions ETIME and FDATE, which        
 * are available on most versions of Unix - except HP (coming in 9.0) an        
 * IBM's AIX...                                                                 
 * The code has been tested and runs on the following:                          
 *   HP 9000 Series 700 under HPUX 8.07                                         
 *   IBM RS/6000 under AIX 3.1                                                  
 *   DEC 5000 Series under Ultrix 4.2                                           
 *   SGI 4D25 under Irix 3.3                                                    
 *   Sun SPARCstation under SunOS 4.1.1 (Note: the value returned by eti        
 *       must be ignored as it is incorrect: the correct value of the CP        
 *       time is given by the sum of the two array elements passed to et        
 *                                                                              
 *       This problem arises because the return value is always promoted        
 *       a float to a double. Different architectures handle this in dif        
 *       ways.                                                                  
 *                                                                              
 * This code should be applicable for most versions of Unix.                    
*/                                                                              
                                                                                
/* Elapsed CPU time routine for FORTRAN etime.                                  
 * This version tries to be correct about getting clock ticks.                  
 * Alternate entries are provided: ETIME and etime_ in case the                 
 * FORTRAN code is compiled with -U or +ppu, or the f77 compiler automat        
 * appends underscores to externals.                                            
*/                                                                              
/* Original etime code by Bob Montgomery, HP */                                 
                                                                                
#include <unistd.h>                                                             
#include <sys/times.h>                                                          
                                                                                
static long ticks = 0;                                                          
                                                                                
float                                                                           
etime(tarray)                                                                   
float *tarray;                                                                  
{                                                                               
   struct tms buffer;                                                           
                                                                                
   (void) times(&buffer);                                                       
   if (ticks == 0) {                                                            
      ticks = sysconf(_SC_CLK_TCK);                                             
   }                                                                            
   tarray[0] = (double)(buffer.tms_utime)/ticks;                                
   tarray[1] = (double)(buffer.tms_stime)/ticks;                                
   return (tarray[0] + tarray[1]);                                              
}                                                                               
                                                                                
float                                                                           
ETIME(tarray)                                                                   
float *tarray;                                                                  
{                                                                               
   return(etime(tarray));                                                       
}                                                                               
                                                                                
float                                                                           
etime_(tarray)                                                                  
float *tarray;                                                                  
{                                                                               
   return(etime(tarray));                                                       
}                                                                               
                                                                                
                                                                                
/* Date and wallclock time routine for FORTRAN fdate.                           
 * Alternate entries are provided: FDATE and fdate_ in case the                 
 * FORTRAN code is compiled with -U or +ppu, or the compiler automatical        
 * appends underscores to externals.                                            
*/                                                                              
/* Mark Favas */                                                                
                                                                                
#include <time.h>                                                               
#include <string.h>                                                             
                                                                                
void                                                                            
fdate(s)                                                                        
char *s;                                                                        
{                                                                               
   char *t;                                                                     
   time_t tloc;                                                                 
   size_t count = 24;                                                           
                                                                                
   (void) time(&tloc);                                                          
   t = ctime(&tloc);                                                            
   (void) strncpy(s,t,count);                                                   
}                                                                               
                                                                                
void                                                                            
fdate_(s)                                                                       
char *s;                                                                        
{                                                                               
   void fdate();                                                                
   fdate(s);                                                                    
}                                                                               
                                                                                
void                                                                            
FDATE(s)                                                                        
char *s;                                                                        
{                                                                               
   void fdate();                                                                
   fdate(s);                                                                    
}                                                                               
