systemheader:(FD    )#                                                  FD     1
####################################################################### FD     2
#                                                                     # FD     3
#                           *** FODIFF ***                            # FD     4
#                                                                     # FD     5
#    FODIFF CALCULATES A FOURIER MAP WHICH IS THE DIFFERENCE          # FD     6
#    BETWEEN TWO OTHER FOURIER MAPS WHICH HAVE BEEN CALCULATED        # FD     7
#    WITH IDENTICAL SPECIFICATIONS FOR GRID SIZE AND LAYOUT.          # FD     8
####################################################################### FD     9
#                                                                       FD    10
include:(XMACRO)#                                                       FD    11
#                                                                       FD    12
systemheader:(FD00)#                                                    FD00   1
# ################################################################## #  FD00   2
#                                                                    #  FD00   3
#                               FD00                                 #  FD00   4
#                                                                    #  FD00   5
# ################################################################## #  FD00   6
#                                         2 electron density maps       FD00   7
SUBROUTINE FD00#                       Calculated over the same         FD00   8
#                                         Volume, grid, increment, etc. FD00   9
#                                        Mar 1982  r.a. alden           FD00  10
include:(AACOMN)#                                                       FD00  11
#                                                                       FD00  12
INTEGER I#                             Loop index                       FD00  13
INTEGER IPT1#                          Input packet pointer - unit      FD00  14
INTEGER IPT2#                          Input packet pointer - unit      FD00  15
INTEGER IPTR1#                         Pointer to current density       FD00  16
#                                         Value - unit a.               FD00  17
INTEGER IPTR2#                         Pointer to current density       FD00  18
#                                         Value - unit b.               FD00  19
INTEGER JPT#                           Output packet pointer            FD00  20
INTEGER JPTR#                          Pointer to location of curre     FD00  21
#                                         Output density value.         FD00  22
INTEGER PSIZE1#                        Unit a packet size               FD00  23
INTEGER PSIZE2#                        Unit b packet size               FD00  24
#                                                                       FD00  25
REAL RHO#                              Current electron density         FD00  26
#                                         Difference.                   FD00  27
REAL RHOMAX#                           Maximum difference density       FD00  28
REAL RHOMIN#                           Minimum difference density       FD00  29
REAL SCALE1#                           Scale to be applied to unit      FD00  30
#                                         Stored density values.        FD00  31
REAL SCALE2#                           Scale to be applied to unit      FD00  32
#                                         Stored density values.        FD00  33
#                                                                       FD00  34
chardata:(FD001, END    )#                                              FD00  35
chardata:(FD002, THE OUTPUT ELECTRON DENSITY MAP HAS VALUES$,$#         FD00  36
 K1*F(A) - K2*F(B)$, WHERE)#                                            FD00  37
chardata:(FD003,F(A) IS THE VALUE READ FROM THE MAP ON UNIT A$,$#       FD00  38
 K1 =)#                                                                 FD00  39
chardata:(FD004,F(B) IS THE VALUE READ FROM THE MAP ON UNIT B$,$#       FD00  40
 K2 =)#                                                                 FD00  41
chardata:(FD005, LOWEST VALUE =         HIGHEST =)#                     FD00  42
realdata:(FMT001,[630722.,230712.,410712.])# Formats                    FD00  43
datastuff:#                                                             FD00  44
#                                                                       FD00  45
FILEXT(IOUNIT(3))='MAD'#               Extension for op diff map        FD00  46
FILEXT(IOUNIT(4))='MA1'#               Extension for ip fourr map 1     FD00  47
FILEXT(IOUNIT(5))='MA2'#               Extension for ip fourr map 2     FD00  48
#                                                                       FD00  49
IOMARK(4) = QXSTR#                     Set up i/o buffers               FD00  50
IOMARK(5) = IOMARK(4) + bdfbuf:#                                        FD00  51
IOMARK(3) = IOMARK(5) + bdfbuf:#                                        FD00  52
incrqx:(I,IOMARK(3)+bdfbuf:+1,FD0001)#                                  FD00  53
#                                                                       FD00  54
copyfile:(4,3,1,1)#                    Copy history to unit c           FD00  55
readwpkt:(5,1,PSIZE2,IPT2,0)#          Read history on unit b           FD00  56
copyfile:(4,3,lrcell:,lrsymm:)#        Copy cell and symop lr's         FD00  57
readwpkt:(4,21,PSIZE1,IPT1,0)#         Read map descriptors             FD00  58
readwpkt:(5,21,PSIZE2,IPT2,0)#                                          FD00  59
#                                                                       FD00  60
# READ IN SCALES (OPTIONAL)                                             FD00  61
#                                                                       FD00  62
ifvo:(BUFIN(1))  SCALE1 = QX(IPT1+24)##                                 FD00  63
ELSE  SCALE1 = BUFIN(1)#                                                FD00  64
ifvo:(BUFIN(2))  SCALE2 = QX(IPT2+24)##                                 FD00  65
ELSE  SCALE2 = BUFIN(2)#                                                FD00  66
IF(SCALE1 <= 0. \ SCALE2 <= 0.)  iquit:(FD0002)# F                      FD00  67
#                                                                       FD00  68
DO I=1,18#                             They must agree prezactly        FD00  69
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  FD00  70
IPTR1 = IPT1 + I#                                                       FD00  71
IPTR2 = IPT2 + I#                                                       FD00  72
ifne:(QX(IPTR1),QX(IPTR2))  iquit:(FD0003)# F                           FD00  73
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  FD00  74
IPTR1 = IPT1 + 21#                     They must also both be simil     FD00  75
IPTR2 = IPT2 + 21#                     Type maps, no mixtures.          FD00  76
ifne:(QX(IPTR1),QX(IPTR2)) iquit:(FD0004)# F                            FD00  77
#                                                                       FD00  78
writepkt:(3,21,PSIZE1,JPT)#            Copy out logical record 21       FD00  79
movereal:(QX,IPT1+1,QX,JPT+1,PSIZE1,1)#                                 FD00  80
#                                                                       FD00  81
# SINCE THE RESULTING STORED MAP WILL HAVE HAD SCALES APPLIED TO THE    FD00  82
#  TWO INPUT MAPS, THE MAP SCALE FACTOR MUST BE SET TO 1.               FD00  83
#                                                                       FD00  84
QX(JPT+24) = 1.#                                                        FD00  85
#                                                                       FD00  86
writeline:(1,FD002,NFD002,3,3)#        Tell user what's happening       FD00  87
movechar:(FD003,1,CHROT,6,NFD003,0)#                                    FD00  88
BUFOT(1) = SCALE1#                                                      FD00  89
ncodefld:(BUFOT,1,CHROT,FMT001(1),1)##                                  FD00  90
writeline:(0,0,0,1,3)#                                                  FD00  91
movechar:(FD004,1,CHROT,6,NFD004,0)#                                    FD00  92
BUFOT(1) = SCALE2#                                                      FD00  93
ncodefld:(BUFOT,1,CHROT,FMT001(1),1)##                                  FD00  94
writeline:(0,0,0,1,3)#                                                  FD00  95
#                                                                       FD00  96
readline:(FD001,NFD001)#               Process the end line             FD00  97
#                                                                       FD00  98
RHOMIN = 1.E20#                        Initialize                       FD00  99
RHOMAX = -RHOMIN#                                                       FD00 100
#                                                                       FD00 101
REPEAT#                                Loop over each map line          FD00 102
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  FD00 103
readwpkt:(4,22,PSIZE1,IPT1,0)#                                          FD00 104
readwpkt:(5,22,PSIZE2,IPT2,0)#                                          FD00 105
IF(IPT1 <= 0)#                         Both must terminate              FD00 106
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  FD00 107
IF(IPT2 > 0)  iquit:(FD0005)#          Simultaneously.                  FD00 108
BREAK#                                                                  FD00 109
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  FD00 110
#                                                                       FD00 111
writepkt:(3,22,PSIZE1,JPT)#            Set aside output buffer          FD00 112
#                                                                       FD00 113
DO I=1,PSIZE1#                         Loop over each point             FD00 114
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  FD00 115
IPTR1 = IPT1 + I#                                                       FD00 116
IPTR2 = IPT2 + I#                                                       FD00 117
JPTR  = JPT  + I#                                                       FD00 118
RHO = SCALE1*QX(IPTR1) - SCALE2*QX(IPTR2)#                              FD00 119
IF(RHO > RHOMAX)  RHOMAX = RHO#        Find current maximum             FD00 120
IF(RHO < RHOMIN)  RHOMIN = RHO#        And minimum.                     FD00 121
QX(JPTR) = RHO#                        Store new value                  FD00 122
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  FD00 123
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  FD00 124
#                                                                       FD00 125
readwpkt:(4,endrecord:,PSIZE1,IPT1,0)# Tidy everything up               FD00 126
readwpkt:(5,endrecord:,PSIZE2,IPT2,0)#                                  FD00 127
writepkt:(3,endrecord:,1,JPT)#                                          FD00 128
#                                                                       FD00 129
# TELL USER ABOUT NEW MINIMUM AND MAXIMUM                               FD00 130
movechar:(FD005,1,CHROT,1,NFD005,0)#                                    FD00 131
BUFOT(1) = RHOMIN#                                                      FD00 132
BUFOT(2) = RHOMAX#                                                      FD00 133
ncodefld:(BUFOT,1,CHROT,FMT001(2),2)##                                  FD00 134
writeline:(1,0,0,1,3)#                                                  FD00 135
#                                                                       FD00 136
RETURN#                                                                 FD00 137
END#                                                                    FD00 138
pmacro:([no:])#                        Purge xtal system macros         FD00 139
