systemheader:(FL    )#                                                  FL     1
####################################################################### FL     2
#                                                                       FL     3
#                            ** FOLOAD **                               FL     4
#                                                                       FL     5
#      THE PROGRAM FOLOAD TAKES IN A FOURIER MAP IN .MAP FORM AND       FL     6
#      ONE IN ASCII FORM OR AS A GENERATED TEST FORM AND                FL     7
#      PUTS OUT A MODIFIED MAP IN THE FOLLOWING WAY.  ALL THE POINTS    FL     8
#      IN EACH MAP ARE SCREENED BY A THRESHOLD CHECK. IF THE ELECTRON   FL     9
#      DENSITY IS BELOW THE MAP THRESHOLD SPECIFIED BY THE USER THE     FL    10
#      POINT IS SET TO THE VALUE SPECIFIED BY THE USER.  THE TWO MAPS   FL    11
#      ARE ADDED TOGETHER WITH EACH POINT WEIGHTED AS SPECIFIED BY      FL    12
#      THE USER. THE RESULTING MAP IS WRITTEN TO OUTPUT AS THE SUM      FL    13
#      OF THE TWO SCREENED, WEIGHTED INPUT MAPS.  THE PURPOSE OF        FL    14
#      THIS PROGRAM IS TO ALLOW THE INTRODUCTION OF FOURIER MAPS        FL    15
#      GENERATED FROM SOURCES OUTSIDE XTAL TO BE PROCESSED BY XTAL      FL    16
#      UTILITIES SUCH AS CONTUR, RFOURR, ETC.                           FL    17
#      THIS LINK WAS WRITTEN AT THE SUGGESTION OF JOHN KONNERT.         FL    18
#                                                                       FL    19
#                                      J. m. stewart  september 1992    FL    20
#                                                                       FL    21
####################################################################### FL    22
#                                                                       FL    23
include:(XMACRO)#                                                       FL    24
#                                                                       FL    25
extrude:(FLCOMN)#                                                       FL    26
INTEGER IN(3)#      ## Grid point increment on input map                FL    27
INTEGER ISUM(3)#    ## Summation order correlation between              FL    28
INTEGER JSUM(3)#    ## Crystal axes and map axes                        FL    29
INTEGER MARKP#      ## Start of storage of el dens map                  FL    30
INTEGER NG(3)#      ## Number of grid points in map                     FL    31
INTEGER NP(3)#      ## Actual number of points in map                   FL    32
INTEGER NPIX#       ## Total pixels in input fourier map                FL    33
INTEGER OG(3)#      ## Grid values of the origin of the map             FL    34
INTEGER PRINTL#     ## Flag to control printing                         FL    35
REAL TH1,TH2#       ## Thresholds for input electron densities          FL    36
REAL WT1,WT2#       ## Weights for input electron densities             FL    37
#                                                                       FL    38
COMMON/COMIFL/$#                                                        FL    39
IN    ,ISUM  ,JSUM  ,MARKP ,$#                                          FL    40
NG    ,NP    ,NPIX  ,$#                                                 FL    41
OG    ,PRINTL,WRDBLK#                                                   FL    42
COMMON/COMFFL/$#                                                        FL    43
TH1,TH2,WT1,WT2#                                                        FL    44
endext:#                                                                FL    45
#                                                                       FL    46
systemheader:(FL00)#                                                    FL00   1
# ################################################################## #  FL00   2
#                                                                       FL00   3
#                               FL00                                 #  FL00   4
#                                                                    #  FL00   5
# ################################################################## #  FL00   6
SUBROUTINE FL00#                       J.m.stewart sept 1992            FL00   7
include:(AACOMN)#                                                       FL00   8
include:(FLCOMN)#                                                       FL00   9
#                                                                       FL00  10
CALL FL10#                             Bdf input and startup            FL00  11
CALL FL70#                             Merge maps                       FL00  12
#                                                                       FL00  13
RETURN#                                                                 FL00  14
END#                                                                    FL00  15
systemheader:(FL10)## bdf input and initialization                      FL10   1
# ################################################################## #  FL10   2
#                                                                    #  FL10   3
#                               FL10                                 #  FL10   4
#                                                                    #  FL10   5
# ################################################################## #  FL10   6
SUBROUTINE FL10#                       J.m.stewart sept 1992            FL10   7
include:(AACOMN)#                                                       FL10   8
include:(FLCOMN)#                                                       FL10   9
#                                                                       FL10  10
chardata:(FL10A, END    )#             Input lines allowed              FL10  11
chardata:(FL10B,PRT TH1 TH2 WT1 WT2 )# Control commands                 FL10  12
chardata:(FL10D,XYZ)#                                                   FL10  13
chardata:(FL10G, MAP TRANSFORMATION    X FIRST Y SEC Z THIRD )#         FL10  14
chardata:(FL10J,YESNO )#                                                FL10  15
chardata:(FL10K, LOAD TWO ELECTRON DENSITY MAPS. ONE FROM ASCII INPUT $#FL10  16
TO PRODUCE AN ELECTRON DENSITY MAP WHICH IS THE WEIGHTED SUM OF BOTH.)# FL10  17
chardata:(FL10L, DIRECTIVES GIVEN OR IMPLIED BY THE FOLOAD LINE...)#    FL10  18
chardata:(FL10N, PRINT        SAMPLES OF MODIFIED MAP PER LAYER)#       FL10  19
chardata:(FL10O, SCRATCH FILE ON H             )#                       FL10  20
chardata:(FL10Q, MAXIMUM ELECTRON DENSITY IN FILE             $#        FL10  21
AND DETERMINED BY SCAN OF MAP           )#                              FL10  22
chardata:(FL10W,FROM MAP ON .MAP FILE )#                                FL10  23
chardata:(FL10Y,TO MODIFIED ON .MPN FILE )#                             FL10  24
chardata:(FL10Z, THIS IS SIMPLY A WARNING IN CASE $#                    FL10  25
FUTURE CALCULATIONS ARE TO BE DONE WITH RFOURR PROGRAM.)#               FL10  26
#                                                                       FL10  27
#------  ALLOWED VALUES OF NX IN FOURIER GRIDS                          FL10  28
#        (RANGE FROM 2 TO 5040) ONLY MXGRID IMPLEMENTED                 FL10  29
realdata:(FLFL,[450931.,850931.])#                                      FL10  30
intdata:(ND,[$#                                                         FL10  31
   2,   3,   4,   5,   6,   7,   8,   9,#                               FL10  32
10,  12,  14,  15,  16,  18,  20,  21,#                                 FL10  33
24,  28,  30,  35,  36,  40,  42,  45,#                                 FL10  34
48,  56,  60,  63,  70,  72,  80,  84,#                                 FL10  35
90, 105, 112, 120, 126, 140, 144, 168,#                                 FL10  36
180, 210, 240, 252, 280, 315, 336, 360,#                                FL10  37
420, 504, 560, 630, 720, 840,1008,1260,#                                FL10  38
1680,2520,5040])#                                                       FL10  39
intdata:(MXGRID,[57])##                                                 FL10  40
realdata:(FLNX,[110413.,170413.,270413.])# Format for grid error        FL10  41
chardata:(FL7A, ONE OR MORE SUMMATION DIRECTIONS ARE NOT IN $#          FL10  42
GRID DIVISIONS HANDLED BY FFT SUBROUTINES.)#                            FL10  43
chardata:(FL7B,[ GIVEN     ,     , AND     .   ACCEPTIBLE.......])#     FL10  44
chardata:(FL7C, THRESHOLD = XXXX.XXX  WEIGHT = XX.XX FOR MAP X )#       FL10  45
chardata:(FL7S, INSUFFICIENT IMMEDIATE ACCESS MEMORY FOR WHOLE MAP.  $# FL10  46
SCRATCH FILE WILL BE WRITTEN ON H)#                                     FL10  47
#                                                                       FL10  48
INTEGER I,J,K,L,N#                     Local indices                    FL10  49
INTEGER PSIZ,IP,IIP,JP#                Bdf reading parameters           FL10  50
INTEGER NDIV1,NDIV2,NDIV3#             Fourier grid divisions           FL10  51
datastuff:#                                                             FL10  52
#                                                                       FL10  53
PRINTL=0#                              Suppress sample point print      FL10  54
WT1=1.0 ; WT2=1.0 ; TH1=0.001 ; TH2=0.001# Set defaults                 FL10  55
#                                                                       FL10  56
FOR(K=1;K>0;K=K+1)#                    Loop over input fields           FL10  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  FL10  58
IF(BUFIN(K)>=voidflg:) BREAK#          Go to first non-alpha field      FL10  59
stringlim:(K,I,J) ; L=MIN0(3,J-K)#     Find characters in input         FL10  60
compchar:(CHRIN,I,FL10B,1,L,4,NFL10B,J)# Search list of options         FL10  61
IF(J==0) iquit:(FL1001)#               It must be valid                 FL10  62
ELSE IF(J==1)#                         Sample point print control       FL10  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  FL10  64
IF(BUFIN(K+1)>0.0)#                                                     FL10  65
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  FL10  66
K=K+1 ; PRINTL=NINT(BUFIN(K))#                                          FL10  67
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  FL10  68
ELSE PRINTL=10#                                                         FL10  69
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  FL10  70
ELSE IF(J==2)#                         Test threshold 1 value           FL10  71
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  FL10  72
IF(BUFIN(K+1)>=-1000.)#                                                 FL10  73
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  FL10  74
K=K+1 ; TH1=BUFIN(K)#                  Accept threshold value           FL10  75
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  FL10  76
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  FL10  77
ELSE IF(J==3)#                         Test threshold 2 value           FL10  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  FL10  79
IF(BUFIN(K+1)>=-1000.)#                                                 FL10  80
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  FL10  81
K=K+1 ; TH2=BUFIN(K)#                  Accept threshold value           FL10  82
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  FL10  83
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  FL10  84
ELSE IF(J==4)#                         Test weight 1 value              FL10  85
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  FL10  86
IF(BUFIN(K+1)>-1000.)#                                                  FL10  87
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  FL10  88
K=K+1 ; WT1=BUFIN(K)#                  Accept weight value              FL10  89
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  FL10  90
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  FL10  91
ELSE IF(J==5)#                         Test weigt 2 value               FL10  92
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 FL10  93
IF(BUFIN(K+1)>-1000.)#                                                  FL10  94
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 FL10  95
K=K+1 ; WT2=BUFIN(K)#                  Accept weight value              FL10  96
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 FL10  97
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 FL10  98
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  FL10  99
#                                                                       FL10 100
# WRITE OUT A SUMMARY OF INSTRUCTIONS                                   FL10 101
# -----------------------------------                                   FL10 102
#                                                                       FL10 103
LINRM=12 ; writeline:(1,FL10K,NFL10K,3,3)# Heading                      FL10 104
writeline:(1,FL10L,NFL10L,3,3)#                                         FL10 105
BUFOT(1)=FLOAT(PRINTL) ; ncodefld:(BUFOT,1,FL10N,130513.,1)#            FL10 106
writeline:(1,FL10N,NFL10N,3,3)#                                         FL10 107
ncodefld:(TH1,1,FL7C,210832.,1) ; ncodefld:(WT1,1,FL7C,370522.,1)#      FL10 108
ncodefld:(1.0,1,FL7C,470113.,1) ; writeline:(1,FL7C,NFL7C,3,3)#         FL10 109
ncodefld:(TH2,1,FL7C,210832.,1) ; ncodefld:(WT2,1,FL7C,370522.,1)#      FL10 110
ncodefld:(2.0,1,FL7C,470113.,1) ; writeline:(1,FL7C,NFL7C,3,3)#         FL10 111
#                                                                       FL10 112
# TAKE IN ANY OTHER INPUT LINES                                         FL10 113
# -----------------------------                                         FL10 114
#                                                                       FL10 115
REPEAT#                                Loop over lines                  FL10 116
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 FL10 117
readline:(FL10A,NFL10A)#                                                FL10 118
IF(LINID==1) BREAK#                    Stop reading when end line       FL10 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 FL10 120
#                                                                       FL10 121
# SET UP I/O BUFFERS FOR THREE FILES                                    FL10 122
# ----------------------------------                                    FL10 123
#                                                                       FL10 124
FILEXT(IOUNIT(3))='MAP'#               Input first density map          FL10 125
FILEXT(IOUNIT(5))='MPN'#               Output new density map           FL10 126
IOMARK(3)=QXSTR#                       Set up electron density in/p1    FL10 127
IOMARK(5)=IOMARK(3)+bdfbuf:#           Set up electron density out/p    FL10 128
incrqx:(MARKP,IOMARK(5)+bdfbuf:,FL1005)# Pointer to stored density      FL10 129
#                                                                       FL10 130
copyfile:(3,5,1,20)#                   Move lead records                FL10 131
#                                                                       FL10 132
# RETRIEVE THE MAP PARAMETERS FROM LR 21 OF FILE .MAP                   FL10 133
# AND START BUILDING THE MERGED ELECTRON DENSITY MAP ON FILE .MPN       FL10 134
# ------------------------------------------------------------          FL10 135
#                                                                       FL10 136
readwpkt:(3,21,PSIZ,IP,0)#                                              FL10 137
writepkt:(5,21,PSIZ,JP) ; movereal:(QX,IP+1,QX,JP+1,PSIZ,0)#            FL10 138
DO K=1,3#                                                               FL10 139
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 FL10 140
IIP=IP+K#                                                               FL10 141
OG(K)=NINT(QX(IIP))#                   Pick up origins                  FL10 142
IN(K)=NINT(QX(IIP+3))#                 Increments along axes            FL10 143
NG(K)=NINT(QX(IIP+6))#                 No. of grid points               FL10 144
NP(K)=NINT(QX(IIP+9))#                 Calculated points                FL10 145
ISUM(K)=NINT(QX(IIP+12))#              Correlations between crystal     FL10 146
JSUM(K)=NINT(QX(IIP+15))#              Axes and calculation axes        FL10 147
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 FL10 148
#                                                                       FL10 149
#----TO DESCRIBE PERMUTATIONS BETWEEN XTAL AXES AND COMPUTATIONAL AXES, FL10 150
#     TWO PERMUTATION VECTORS ARE DEFINED.       THE ISUM'S AND JSUM'S  FL10 151
#                                                                       FL10 152
#----THE ISUM'S GIVE COMPUTATIONAL VARIABLES CORRESPONDING TO THE XTAL  FL10 153
#     VARIABLES.  THE JSUM'S GIVE XTAL VARIABLES CORRESPONDING TO THE   FL10 154
#     COMPUTATIONAL VARIABLES.                                          FL10 155
#                                                                       FL10 156
#----THUS, FOR ALL POSSIBLE PERMUTATIONS                                FL10 157
#        LAYER  DOWN  ACROSS  ISUM1  ISUM2  ISUM3  JSUM1  JSUM2  JSUM3  FL10 158
#          X      Y     Z       1      2      3      1      2      3    FL10 159
#          X      Z     Y       1      3      2      1      3      2    FL10 160
#          Y      X     Z       2      1      3      2      1      3    FL10 161
#          Z      Y     X       3      2      1      3      2      1    FL10 162
#          Z      X     Y       2      3      1      3      1      2    FL10 163
#          Y      Z     X       3      1      2      2      3      1    FL10 164
#                                                                       FL10 165
#                                                                       FL10 166
movechar:(FL10D,JSUM(3),FL10G,24,1,0)# Change to correct axis           FL10 167
movechar:(FL10D,JSUM(2),FL10G,32,1,0)# Change to correct axis           FL10 168
movechar:(FL10D,JSUM(1),FL10G,38,1,0)# Change to correct axis           FL10 169
writeline:(1,FL10G,NFL10G,3,3)#                                         FL10 170
#                                                                       FL10 171
# MAKE SURE GRID SIZES ARE POSSIBLE TO DO                               FL10 172
# ---------------------------------------                               FL10 173
#                                                                       FL10 174
NDIV1=0;NDIV2=0;NDIV3=0#               Set grid size pointers           FL10 175
DO J=1,MXGRID#                         Loop over possible grids         FL10 176
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 FL10 177
IF(NG(1)==ND(J)) NDIV1=J#              Set layer divisions              FL10 178
IF(NG(2)==ND(J)) NDIV2=J#              Set down layer divisions         FL10 179
IF(NG(3)==ND(J)) NDIV3=J#              Set across layer divisions       FL10 180
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 FL10 181
IF((NDIV1==0)\(NDIV2==0)\(NDIV3==0))#  Test that all o.k.               FL10 182
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 FL10 183
BUFOT(1)=FLOAT(NG(1));BUFOT(2)=FLOAT(NG(2));BUFOT(3)=FLOAT(NG(3))#      FL10 184
writeline:(1,FL7A,NFL7A,3,1)#                                           FL10 185
ncodefld:(BUFOT,1,FL7B,FLNX,3)#                                         FL10 186
writeline:(0,FL7B,NFL7B,3,1)#                                           FL10 187
K=0; FOR(J=1;J<=4;J=J+1)#              Dump all possible grids          FL10 188
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 FL10 189
IF(J<4) N=15#                          15 items per line for three li   FL10 190
ELSE N=14#                             14 on the last                   FL10 191
FLT=413.#                              Start building the format        FL10 192
DO L=1,N#                                                               FL10 193
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 FL10 194
K=K+1 ; FLT=FLT+50000. ; BUFOT(1)=FLOAT(ND(K))#                         FL10 195
ncodefld:(BUFOT,1,CHROT,FLT,1)#                                         FL10 196
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 FL10 197
writeline:(0,0,0,1,1)#                                                  FL10 198
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 FL10 199
writeline:(1,FL10Z,NFL10Z,3,3)#        Warn user of possible glitch     FL10 200
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 FL10 201
#                                                                       FL10 202
RETURN#                                                                 FL10 203
END#                                                                    FL10 204
systemheader:(FL70)#                   Map merging                      FL70   1
# ################################################################## #  FL70   2
#                                                                    #  FL70   3
#                               FL70                                 #  FL70   4
#                                                                    #  FL70   5
# ################################################################## #  FL70   6
SUBROUTINE FL70## J.M. Stewart Sept 1992                                FL70   7
include:(AACOMN)##                                                      FL70   8
include:(FLCOMN)#                                                       FL70   9
#                                                                       FL70  10
# THIS ROUTINE SCREENS, WEIGHTS AND MERGES ELECTRON DENSITY MAPS        FL70  11
# RHO(OUT) = AMAX1(TH1,RHO(1))*WT1 + AMAX1(TH2,RHO(2))*WT2              FL70  12
# WHERE "TH1" IS A THRESHOLD, SO ALL VALUES OF RHO1 BELOW               FL70  13
# A VALUE, TH1, ARE SET TO TH1.                                         FL70  14
#--------------------------------------+------------------------------- FL70  15
#                                                                       FL70  16
INTEGER I,J,M#       ## Local indices for loops etc.                    FL70  17
INTEGER PRINTP#      ## Electron density dumping control                FL70  18
INTEGER PSIZ,IP,JP#  ## I/o pointers                                    FL70  19
REAL ED1LIM(3)#      ## El-dens statistics from file c                  FL70  20
REAL ED2LIM(3)#      ## El-dens statistics from file d                  FL70  21
REAL EDNLIM(3)#      ## El-dens statistics from file E                  FL70  22
REAL NEWELD,ELD1,ELD2## Electron densities during processing            FL70  23
REAL GENPT1,GENPT2,GENPT3#             Generated coordinates            FL70  24
REAL GENMX1,GENMX2,GENMX3#             Maxima of generated coordinate   FL70  25
REAL GMOD1,GMOD2,GMOD3#                E. d. moduli along axes          FL70  26
REAL GBASE1,GBASE2,GBASE3#             E. d. base along axes            FL70  27
REAL GTERM2,GTERM3#                    E. d. terms for gener'd pseudo   FL70  28
chardata:(FL7G, PRINT A SAMPLE OF INPUT AND OUTPUT ELECTRON DENSITY)#   FL70  29
chardata:(FL7H,  POINT NO.    INPUT RHO   ASCII RHO    NEW RHO   $#     FL70  30
LAYER    DOWN  ACROSS  )#                                               FL70  31
realdata:(FLT7,[341031.,631031.])#                                      FL70  32
realdata:(FLT8,[421031.,681013.])#                                      FL70  33
realdata:(FLTL,[390852.,520852.])#                                      FL70  34
realdata:(FLTO,[110813.,241041.,361041.,481041.,560613,640613.,$#       FL70  35
720613.])#                                                              FL70  36
realdata:(FLTST,[370931.,570931.,780931.])#                             FL70  37
chardata:(FL7Q,   .MAP  DENSITY MAXIMUM =             $#                FL70  38
MINIMUM =           AVERAGE =           )#                              FL70  39
chardata:(FL7R,   ASCII DENSITY MAXIMUM =             $#                FL70  40
MINIMUM =           AVERAGE =           )#                              FL70  41
chardata:(FL7S,   NEW   DENSITY MAXIMUM =             $#                FL70  42
MINIMUM =           AVERAGE =           )#                              FL70  43
datastuff:#                                                             FL70  44
#                                                                       FL70  45
# INITIALIZE COUNTS, NUMBERS OF POINTS IN SUMMATIONS AND PRINT OUT      FL70  46
# ----------------------------------------------------------------      FL70  47
#                                                                       FL70  48
GENPT1=0.0 ; GENPT2=0.0 ; GENPT3=0.0#  Set coordinates to zero          FL70  49
GENMX1=FLOAT(NP(ISUM(1))) ; GENMX2=FLOAT(NP(ISUM(2)))#                  FL70  50
GENMX3=FLOAT(NP(ISUM(3)))#                                              FL70  51
GMOD1=0.5*GENMX1 ; GMOD2=0.5*GENMX2#                                    FL70  52
GMOD3=0.5*GENMX3#                      Moduli for pattern generation    FL70  53
GBASE1=0.25*GENMX1 ; GBASE2=0.25*GENMX2#                                FL70  54
GBASE3=0.25*GENMX3#                    Floor for pattern generation     FL70  55
WRITE(6,*)' DUMP GENMX1,GENMX2,GENMX3 ',GENMX1,GENMX2,GENMX3#           FL70  56
WRITE(6,*)' DUMP GMOD1,GMOD2,GMOD3 ',GMOD1,GMOD2,GMOD3#                 FL70  57
WRITE(6,*)' DUMP GBASE1,GBASE2,GBASE3 ',GBASE1,GBASE2,GBASE3#           FL70  58
NPIX=0#                                Zero numper of points            FL70  59
ED1LIM(1)=-99999. ; ED1LIM(2)=99999. ; ED1LIM(3)=0.0# El-den stats      FL70  60
ED2LIM(1)=-99999. ; ED2LIM(2)=99999. ; ED2LIM(3)=0.0# El-den stats      FL70  61
EDNLIM(1)=-99999. ; EDNLIM(2)=99999. ; EDNLIM(3)=0.0# El-den stats      FL70  62
PRINTP=PRINTL ; LINRM=6; IF(PRINTL>0)#                                  FL70  63
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  FL70  64
writeline:(1,FL7G,NFL7G,3,3)#                                           FL70  65
writeline:(0,0,0,1,3) ; LINRM=10#      Assure heading on sample o/p     FL70  66
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  FL70  67
#                                                                       FL70  68
# TOP OF LOOP OVER PACKETS (LINES) TO PROCESS ELECTRON DENSITY          FL70  69
# ------------------------------------------------------------          FL70  70
#                                                                       FL70  71
REPEAT#                                Loop over layers                 FL70  72
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  FL70  73
readwpkt:(3,22,PSIZ,IP,0)#             Read next line on file c         FL70  74
IF(IP<=0) BREAK#                       Quit if an end-of-file is hit    FL70  75
writepkt:(5,22,PSIZ,JP)#               Write a packet for holding e.d   FL70  76
I=IP ; M=JP#                           Set pointers to el-density       FL70  77
GENPT2=GENPT2+1.0#                     Increment down (lines) coord     FL70  78
GTERM2=(GBASE2-AMOD(GENPT2,GMOD2))**2# Generate e. d. term              FL70  79
IF(GENPT2>GENMX2)#                     Maximum reached?                 FL70  80
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  FL70  81
GENPT2=1.0 ; GTERM2=(GBASE2-1.0)**2#   Reset for next line              FL70  82
GENPT1=GENPT1+1.0#                                                      FL70  83
PRINTP=PRINTL#                         Sample next layer                FL70  84
IF(GENPT1>GENMX1) BREAK#               Maximum reached?                 FL70  85
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  FL70  86
GENPT3=0.0#                            Reset across coord for next li   FL70  87
DO J=1,PSIZ#                           Loop across line                 FL70  88
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  FL70  89
I=I+1 ; M=M+1#                         Move ahead in line buffer        FL70  90
NPIX=NPIX+1#                           Count up all points              FL70  91
ELD1=QX(I)#                            Take input electron densities    FL70  92
#                                                                       FL70  93
GENPT3=GENPT3+1.0#                     Increment inner axis coordinat   FL70  94
GTERM3=(GBASE3-AMOD(GENPT3,GMOD3))**2# Generate e. d. term              FL70  95
ELD2=GTERM3+GTERM2#                    Form pseudo electron density     FL70  96
ED1LIM(3)=ED1LIM(3)+ELD1#              Build statistics                 FL70  97
ED1LIM(1)=AMAX1(ELD1,ED1LIM(1)) ; ED1LIM(2)=AMIN1(ELD1,ED1LIM(2))#      FL70  98
ED2LIM(3)=ED2LIM(3)+ELD2#              Build statistics                 FL70  99
ED2LIM(1)=AMAX1(ELD2,ED2LIM(1)) ; ED2LIM(2)=AMIN1(ELD2,ED2LIM(2))#      FL70 100
#                                                                       FL70 101
#  FORM THE NEW ELECTRON DENSITY FROM THE INPUT ELECTRON DENSITIES      FL70 102
#  ---------------------------------------------------------------      FL70 103
#                                                                       FL70 104
NEWELD=WT1*AMAX1(TH1,ELD1)+WT2*AMAX1(TH2,ELD2)# Combine densities       FL70 105
#                                                                       FL70 106
EDNLIM(3)=EDNLIM(3)+NEWELD#            Build statistics                 FL70 107
EDNLIM(1)=AMAX1(NEWELD,EDNLIM(1));EDNLIM(2)=AMIN1(NEWELD,EDNLIM(2))#    FL70 108
#                                                                       FL70 109
QX(M)=NEWELD#                          Electron dens to o/p buffer      FL70 110
IF(PRINTP>0)#                          See if dumping a sample          FL70 111
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  FL70 112
BUFOT(1)=FLOAT(NPIX)#                                                   FL70 113
BUFOT(2)=ELD1 ; BUFOT(3)=ELD2#         Get electron densities           FL70 114
BUFOT(4)=NEWELD#                                                        FL70 115
BUFOT(5)=GENPT1 ; BUFOT(6)=GENPT2 ; BUFOT(7)=GENPT3#                    FL70 116
ncodefld:(BUFOT,1,CHROT,FLTO,7) ; writeline:(0,FL7H,NFL7H,1,3)#         FL70 117
PRINTP=PRINTP-1#                       Decrement ouput lines            FL70 118
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  FL70 119
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  FL70 120
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  FL70 121
#                                                                       FL70 122
# AT THE END OF READING THE WHOLE MAP GIVE EL-DENS STATS                FL70 123
# ------------------------------------------------------                FL70 124
#                                                                       FL70 125
ED1LIM(3)=ED1LIM(3)/FLOAT(NPIX)#       Find ave medens el-dens          FL70 126
ncodefld:(ED1LIM,1,FL7Q,FLTST,3) ; writeline:(1,FL7Q,NFL7Q,3,3)#        FL70 127
ED2LIM(3)=ED2LIM(3)/FLOAT(NPIX)#       Find ave difference el-dens      FL70 128
ncodefld:(ED2LIM,1,FL7R,FLTST,3) ; writeline:(1,FL7R,NFL7R,3,3)#        FL70 129
EDNLIM(3)=EDNLIM(3)/FLOAT(NPIX)#       Find ave difference el-dens      FL70 130
ncodefld:(EDNLIM,1,FL7S,FLTST,3) ; writeline:(1,FL7S,NFL7S,3,3)#        FL70 131
#                                                                       FL70 132
#                                                                       FL70 133
# FINISH OFF ALL BDFS AND QUIT                                          FL70 134
# ----------------------------                                          FL70 135
#                                                                       FL70 136
readwpkt:(3,endrecord:,PSIZ,IP,0)#     End off old el-dens file c       FL70 137
writepkt:(5,endrecord:,3,IP)#          End off new el-dens file e       FL70 138
#                                                                       FL70 139
RETURN#                                                                 FL70 140
END#                                                                    FL70 141
#        FOLOAD    *****FL*****                                         FL70 142
#FL1001 THERE IS AN INVALID OPTION CODE ON THE FOLOAD LINE.             FL70 143
#FL1005 INSUFFICIENT STORAGE FOR ELECTRON DENSITY FILE BUFFERS.         FL70 144
#FL1006 THE RFOURR PROGRAM IS RESTRICTED TO CERTAIN GRID DIVISIONS.     FL70 145
#FL1006 THE INPUT ELECTRON DENSITY MAP DOES NOT MEET THE RESTRICTIONS.  FL70 146
pmacro:([no:])#                        Purge xtal system macros         FL70 147
#                                                                       FL70 148
#                                                                       FL70 149
