systemheader:(ME    )#                                                  ME     1
####################################################################### ME     2
#                                                                     # ME     3
#                           *** MEDENS ***                            # ME     4
#                                                                     # ME     5
#    THE PROGRAM MEDENS COMPUTES THE CONSTRAINED EXPONENTIAL DENSITY  # ME     6
#    DISTRIBUTION CORRESPONDING TO A LOWER RESOLUTION INPUT FOURIER   # ME     7
#    MAP.  GIVEN AN INPUT ELECTRON DENSITY MAP, THE DENSITY IS        # ME     8
#    PROCESSED TO GIVE THE EXPONENTIAL DENSITY DISTRIBUTION.  THIS IS # ME     9
#    A FUNCTION OF MAXIMUM ENTROPY AND IS INTENDED TO BE USED FOR THE # ME    10
#    EXTRAPOLATION, INTERPOLATION, AND SMOOTHING OF REFLECTION PHASES # ME    11
#    THE RESULTING PHASES MUST BE RECOVERED BY THE USE OF THE XTAL    # ME    12
#    PROGRAM 'RFOURR' (FB).                                           # ME    13
#    THIS FUNCTION BECOMES ILL-CONDITIONED AND MAY FAIL AS THE        # ME    14
#    MEAN OF THE INPUT ELECTRON DENSITY APPROACHES ZERO FROM THE      # ME    15
#    POSITIVE.                                                        # ME    16
#    THIS XTAL LINK IS MODELED ON THE SUBROUTINE MAXENT, WHICH WAS    # ME    17
#    WRITTEN BY E. PRINCE AT N.B.S, IN APRIL 1987 "TO FIND THE        # ME    18
#    MAXIMUM ENTROPY DENSITY DISTRIBUTION CORRESPONDING TO A LOW      # ME    19
#    RESOLUTION FOURIER MAP".                                         # ME    20
####################################################################### ME    21
#                                                                       ME    22
include:(XMACRO)#                                                       ME    23
#                                                                       ME    24
extrude:(MECOMN)#                                                       ME    25
REAL    ENTRPY#     ## Entropy of the map                               ME    26
INTEGER FILUSE,FILSIG## Signal to use files instead of memory           ME    27
INTEGER FORCE#      ## Force parameters to make map                     ME    28
REAL    F000US#     ## Forced  F(0,0,0)                                 ME    29
INTEGER GRIDCK#     ## Signal check on fft allowed grid intervals       ME    30
INTEGER IN(3)#      ## Grid point increment on input map                ME    31
INTEGER ISUM(3)#    ## Summation order correlation between              ME    32
INTEGER JP#         ## File D offset pointer for record 21              ME    33
INTEGER JSUM(3)#    ## Crystal axes and map axes                        ME    34
REAL    LOGLIM#     ## Factor for constraining Newton-Raphson           ME    35
REAL    LRMNUS#     ## Forced value of ln(rho min)                      ME    36
REAL    LRMXUS#     ## Forced value of ln(rho max)                      ME    37
INTEGER MARKP#      ## Start of storage of el dens map                  ME    38
INTEGER NG(3)#      ## Number of grid points in map                     ME    39
INTEGER NP(3)#      ## Actual number of points in map                   ME    40
INTEGER NPIX#       ## Total pixels in input fourier map                ME    41
INTEGER NX1,NX2,NX3## No. of points in each direction                   ME    42
INTEGER OG(3)#      ## Grid values of the origin of the map             ME    43
INTEGER PRINTL#     ## Flag to control printing                         ME    44
INTEGER SHARP#      ## Signal control of sharpness of map               ME    45
REAL    SHARPF#     ## Factor to control sharpness of map               ME    46
INTEGER STOPP#      ## End of electron density array                    ME    47
REAL    VOL#        ## Unit cell volume                                 ME    48
#                                                                       ME    49
COMMON/COMIME/$#                                                        ME    50
ENTRPY,FILUSE,FILSIG,FORCE ,GRIDCK,IN    ,$#                            ME    51
ISUM  ,JP    ,JSUM  ,MARKP ,$#                                          ME    52
NG    ,NP    ,NPIX  ,NX1   ,NX2   ,NX3   ,$#                            ME    53
OG    ,PRINTL,SHARP ,STOPP#                                             ME    54
COMMON/COMFME/F000US,LOGLIM,LRMNUS,LRMXUS,SHARPF,VOL#                   ME    55
endext:#                                                                ME    56
#                                                                       ME    57
systemheader:(ME00)#                                                    ME00   1
# ################################################################## #  ME00   2
#                                                                       ME00   3
#                               ME00                                 #  ME00   4
#                                                                    #  ME00   5
# ################################################################## #  ME00   6
SUBROUTINE ME00#                       D.m.collins,j.m.stewart may 19   ME00   7
include:(AACOMN)#                                                       ME00   8
include:(MECOMN)#                                                       ME00   9
#                                                                       ME00  10
FILEXT(IOUNIT(3))='MAP'#               Input density map                ME00  11
FILEXT(IOUNIT(4))='MED'#               Output density map               ME00  12
#                                                                       ME00  13
CALL ME10#                             Bdf input and startup            ME00  14
CALL ME70#                             Calculate fft summations         ME00  15
#                                                                       ME00  16
RETURN#                                                                 ME00  17
END#                                                                    ME00  18
systemheader:(ME10)## bdf input and initialization                      ME10   1
# ################################################################## #  ME10   2
#                                                                    #  ME10   3
#                               ME10                                 #  ME10   4
#                                                                    #  ME10   5
# ################################################################## #  ME10   6
SUBROUTINE ME10## J.M.Stewart D.M.Collins May 1987                      ME10   7
include:(AACOMN)#                                                       ME10   8
include:(MECOMN)#                                                       ME10   9
#                                                                       ME10  10
chardata:(ME10A, END    )#             Input lines allowed              ME10  11
chardata:(ME10B,PR  FI  LL  SH  NC  FO  )# Control commands             ME10  12
chardata:(ME10D,XYZ)#                                                   ME10  13
chardata:(ME10G, MAP TRANSFORMATION    X FIRST Y SEC Z THIRD )#         ME10  14
chardata:(ME10J,YESNO )#                                                ME10  15
chardata:(ME10K, PROCESS AN ELECTRON DENSITY MAP TO PRODUCE THE  $#     ME10  16
MAXIMUM ENTROPY DENSITY DISTRIBUTION.)#                                 ME10  17
chardata:(ME10L, DIRECTIVES GIVEN OR IMPLIED BY THE MEDENS LINE...)#    ME10  18
chardata:(ME10N, PRINT        SAMPLES OF MODIFIED MAP PER LAYER)#       ME10  19
chardata:(ME10O, USE FILE C FOR MEMORY         )#                       ME10  20
chardata:(ME10W,FROM FOURR MAP ON FILE C)#                              ME10  21
chardata:(ME10X, CONTROL SHARPNESS OF FINAL MAP BY FACTOR OF        )#  ME10  22
chardata:(ME10Y, CHECK FFT GRID SIZE           )#                       ME10  23
chardata:(ME101,[ FORCE OUTPUT TO HAVE LN(RHO) MIN =           ])#      ME10  24
chardata:(ME102,[ FORCE OUTPUT TO HAVE LN(RHO) MAX =           ])#      ME10  25
chardata:(ME103,[ FORCE OUTPUT TO HAVE F(0,0,0)    =           ])#      ME10  26
#                                                                       ME10  27
#------  ALLOWED VALUES OF NX IN FOURIER GRIDS                          ME10  28
#        (RANGE FROM 2 TO 5040) ONLY MXGRID IMPLEMENTED                 ME10  29
intdata:(ND,[$#                                                         ME10  30
   2,   3,   4,   5,   6,   7,   8,   9,#                               ME10  31
10,  12,  14,  15,  16,  18,  20,  21,#                                 ME10  32
24,  28,  30,  35,  36,  40,  42,  45,#                                 ME10  33
48,  56,  60,  63,  70,  72,  80,  84,#                                 ME10  34
90, 105, 112, 120, 126, 140, 144, 168,#                                 ME10  35
180, 210, 240, 252, 280, 315, 336, 360,#                                ME10  36
420, 504, 560, 630, 720, 840,1008,1260,#                                ME10  37
1680,2520,5040])#                                                       ME10  38
intdata:(MXGRID,[57])##                                                 ME10  39
realdata:(FMNX,[110413.,170413.,270413.])# Format for grid error        ME10  40
chardata:(ME7A, ONE OR MORE SUMMATION DIRECTIONS ARE NOT IN $#          ME10  41
GRID DIVISIONS HANDLED BY FFT SUBROUTINES.)#                            ME10  42
chardata:(ME7B,[ GIVEN     ,     , AND     .   ACCEPTIBLE.......])#     ME10  43
chardata:(ME7C, LOGARITHMIC DAMPING FACTOR FOR NEWTON-RAPHSON $#        ME10  44
REFINEMENT =          )#                                                ME10  45
chardata:(ME7S, FILE C WILL BE USED IN MULTIPLE READS.  $#              ME10  46
IMMEDIATE ACCESS MEMORY TOO LIMITED.)#                                  ME10  47
#                                                                       ME10  48
REAL FMT#                              A format register                ME10  49
INTEGER I,J,K,L,N#                     Local indices                    ME10  50
INTEGER PSIZ,IP,IIP,PSIZJ#             Bdf reading parameters           ME10  51
INTEGER NDIV1,NDIV2,NDIV3#             Fourier grid divisions           ME10  52
#                                                                       ME10  53
datastuff:#                                                             ME10  54
#                                                                       ME10  55
PRINTL=0#                              Suppress refl print              ME10  56
FILSIG=no:#                            No multiple use of file c        ME10  57
LOGLIM=1.0#                            Limit newton-raphson shifts      ME10  58
SHARPF=1.0#                            Set no change in sharpness       ME10  59
SHARP=no:#                             Of output maps. sharpf < 1.0     ME10  60
#                                      Reduces sharpness. 0.5=sqrt      ME10  61
GRIDCK=yes:#                           Check if grid intervals o.k.     ME10  62
FORCE=no:#                             Check if map to be forced        ME10  63
#                                                                       ME10  64
FOR(K=1;BUFIN(K)<voidflg:;K=K+1)#      Loop over input fields           ME10  65
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ME10  66
stringlim:(K,I,J)#                     Find characters in input         ME10  67
compchar:(CHRIN,I,ME10B,1,2,4,NME10B,J)# Search list of options         ME10  68
IF(J==0) iquit:(ME1001)#               It must be valid                 ME10  69
ELSE IF(J==1)#                         Test reflection print control    ME10  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ME10  71
IF(BUFIN(K+1)>0.0)#                                                     ME10  72
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ME10  73
K=K+1 ; PRINTL=NINT(BUFIN(K))#                                          ME10  74
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ME10  75
ELSE PRINTL=10#                                                         ME10  76
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ME10  77
ELSE IF(J==2) FILSIG=yes:#             Force use of file c for memory   ME10  78
ELSE IF(J==3)#                         Test refinement limit control    ME10  79
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ME10  80
IF(BUFIN(K+1)>0.0)#                                                     ME10  81
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ME10  82
K=K+1 ; LOGLIM=AMIN1(BUFIN(K),10.0)#   Accept limit on damping          ME10  83
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ME10  84
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ME10  85
ELSE IF(J==4)#                         Test sharpness limit control     ME10  86
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ME10  87
IF(BUFIN(K+1)>0.0) ; SHARP=yes:#                                        ME10  88
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ME10  89
K=K+1 ; SHARPF=AMAX1(AMIN1(BUFIN(K),2.0),0.2)# Set limit on sharpenin   ME10  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ME10  91
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ME10  92
ELSE IF(J==5) GRIDCK=no:#              Shut off grid interval check     ME10  93
ELSE IF(J==6)#                         Test forcing parameters of map   ME10  94
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ME10  95
IF(BUFIN(K+1)>0.0) ; FORCE=yes:#       Set to do user specified map     ME10  96
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ME10  97
K=K+1 ; LRMNUS=AMAX1(AMIN1(BUFIN(K),2.0),-6.9)# Get ln(rho) minimum     ME10  98
K=K+1 ; LRMXUS=AMAX1(AMIN1(BUFIN(K),15.0),-2.3)# Get ln(rho) maximum    ME10  99
K=K+1 ; F000US=AMAX1(AMIN1(BUFIN(K),2.0E6),6.0)# Get f(0,0,0)           ME10 100
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ME10 101
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ME10 102
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ME10 103
#                                                                       ME10 104
# WRITE OUT A SUMMARY OF INSTRUCTIONS                                   ME10 105
# -----------------------------------                                   ME10 106
#                                                                       ME10 107
LINRM=12 ; writeline:(1,ME10K,NME10K,3,3)# Heading                      ME10 108
writeline:(1,ME10L,NME10L,3,3)#                                         ME10 109
BUFOT(1)=FLOAT(PRINTL) ; ncodefld:(BUFOT,1,ME10N,130513.,1)#            ME10 110
writeline:(1,ME10N,NME10N,3,3)#                                         ME10 111
IF(FILSIG==yes:)#                      User demands file c use          ME10 112
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ME10 113
movechar:(ME10J,1,ME10O,26,3,0) ; writeline:(1,ME10O,NME10O,3,3)#       ME10 114
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ME10 115
IF(GRIDCK==no:) movechar:(ME10J,4,ME10Y,26,3,0)# No grid check          ME10 116
ELSE movechar:(ME10J,1,ME10Y,26,3,0) ; writeline:(1,ME10Y,NME10Y,3,3)#  ME10 117
ncodefld:(LOGLIM,1,ME7C,680832.,1) ; writeline:(1,ME7C,NME7C,3,3)#      ME10 118
ncodefld:(SHARPF,1,ME10X,520832.,1) ; writeline:(1,ME10X,NME10X,3,3)#   ME10 119
IF(FORCE==yes:)#                       User specifies map?              ME10 120
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ME10 121
ncodefld:(LRMNUS,1,ME101,451062.,1) ; writeline:(1,ME101,NME101,3,3)#   ME10 122
ncodefld:(LRMXUS,1,ME102,451062.,1) ; writeline:(1,ME102,NME102,3,3)#   ME10 123
ncodefld:(F000US,1,ME103,451012.,1) ; writeline:(1,ME103,NME103,3,3)#   ME10 124
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ME10 125
#                                                                       ME10 126
# TAKE IN ANY OTHER INPUT LINES                                         ME10 127
# -----------------------------                                         ME10 128
#                                                                       ME10 129
REPEAT#                                Loop over lines                  ME10 130
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ME10 131
readline:(ME10A,NME10A)#                                                ME10 132
IF(LINID==1) BREAK#                    Stop reading when end line       ME10 133
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ME10 134
#                                                                       ME10 135
# SET UP I/O BUFFERS FOR TWO FILES                                      ME10 136
# --------------------------------                                      ME10 137
#                                                                       ME10 138
IOMARK(3)=QXSTR#                       Set up electron density in/p     ME10 139
IOMARK(4)=IOMARK(3)+bdfbuf:#           Set up electron density out/p    ME10 140
incrqx:(MARKP,IOMARK(4)+bdfbuf:,ME1005)# Pointer to stored densi        ME10 141
#                                                                       ME10 142
FOR(K=1;K>0;K=K+1)#                    Loop over lrcell packets         ME10 143
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ME10 144
readwpkt:(1,lrcell:,PSIZ,IP,0)#        Read in cell record              ME10 145
IF(IP<=0) BREAK#                       Exit after last record           ME10 146
IF(K==6)VOL=QX(IP+1)#                  Save real cell volume            ME10 147
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ME10 148
readwpkt:(1,endrecord:,PSIZ,IP,0)#     Finihed with file                ME10 149
#                                                                       ME10 150
copyfile:(3,4,1,20)#                   Move lead records                ME10 151
#                                                                       ME10 152
# RETRIEVE THE MAP PARAMETERS FROM LR 21 OF FILE C                      ME10 153
# AND START BUILDING THE ENTROPY MAXIMIZED MAP ON FILE D                ME10 154
# ------------------------------------------------------                ME10 155
#                                                                       ME10 156
readwpkt:(3,21,PSIZ,IP,0) ; PSIZJ=30#                                   ME10 157
writepkt:(4,21,PSIZJ,JP,0) ; movereal:(QX,IP+1,QX,JP+1,PSIZ,0)#         ME10 158
DO K=1,3#                                                               ME10 159
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ME10 160
IIP=IP+K#                                                               ME10 161
OG(K)=NINT(QX(IIP))#                   Pick up origins                  ME10 162
IN(K)=NINT(QX(IIP+3))#                 Increments along axes            ME10 163
NG(K)=NINT(QX(IIP+6))#                 No. of grid points               ME10 164
NP(K)=NINT(QX(IIP+9))#                 Calculated points                ME10 165
ISUM(K)=NINT(QX(IIP+12))#              Correlations between crystal     ME10 166
JSUM(K)=NINT(QX(IIP+15))#              Axes and calculation axes        ME10 167
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ME10 168
#                                                                       ME10 169
#----TO DESCRIBE PERMUTATIONS BETWEEN XTAL AXES AND COMPUTATIONAL AXES, ME10 170
#     TWO PERMUTATION VECTORS ARE DEFINED.       THE ISUM'S AND JSUM'S  ME10 171
#                                                                       ME10 172
#----THE ISUM'S GIVE COMPUTATIONAL VARIABLES CORRESPONDING TO THE XTAL  ME10 173
#     VARIABLES.  THE JSUM'S GIVE XTAL VARIABLES CORRESPONDING TO THE   ME10 174
#     COMPUTATIONAL VARIABLES.                                          ME10 175
#                                                                       ME10 176
#----THUS, FOR ALL POSSIBLE PERMUTATIONS                                ME10 177
#        LAYER  DOWN  ACROSS  ISUM1  ISUM2  ISUM3  JSUM1  JSUM2  JSUM3  ME10 178
#          X      Y     Z       1      2      3      1      2      3    ME10 179
#          X      Z     Y       1      3      2      1      3      2    ME10 180
#          Y      X     Z       2      1      3      2      1      3    ME10 181
#          Z      Y     X       3      2      1      3      2      1    ME10 182
#          Z      X     Y       2      3      1      3      1      2    ME10 183
#          Y      Z     X       3      1      2      2      3      1    ME10 184
#                                                                       ME10 185
movechar:(ME10D,JSUM(3),ME10G,24,1,0)# Change to correct axis           ME10 186
movechar:(ME10D,JSUM(2),ME10G,32,1,0)# Change to correct axis           ME10 187
movechar:(ME10D,JSUM(1),ME10G,38,1,0)# Change to correct axis           ME10 188
writeline:(1,ME10G,NME10G,3,3)#                                         ME10 189
#                                                                       ME10 190
# MAKE SURE GRID SIZES ARE POSSIBLE TO DO                               ME10 191
# ---------------------------------------                               ME10 192
#                                                                       ME10 193
NX1=NG(1);NX2=NG(2);NX3=NG(3)#         Get points in 3 directions       ME10 194
NDIV1=0;NDIV2=0;NDIV3=0#               Set grid size pointers           ME10 195
NPIX=NX1*NX2*NX3#                      Get total pixels in map          ME10 196
DO J=1,MXGRID#                         Loop over possible grids         ME10 197
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ME10 198
IF(NX1==ND(J)) NDIV1=J#                Set layer divisions              ME10 199
IF(NX2==ND(J)) NDIV2=J#                Set down layer divisions         ME10 200
IF(NX3==ND(J)) NDIV3=J#                Set across layer divisions       ME10 201
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ME10 202
IF((NDIV1==0)\(NDIV2==0)\(NDIV3==0))#  Test that all o.k.               ME10 203
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ME10 204
BUFOT(1)=FLOAT(NX1);BUFOT(2)=FLOAT(NX2);BUFOT(3)=FLOAT(NX3)#            ME10 205
writeline:(1,ME7A,NME7A,3,1)#                                           ME10 206
ncodefld:(BUFOT,1,ME7B,FMNX,3)#                                         ME10 207
writeline:(0,ME7B,NME7B,3,1)#                                           ME10 208
K=0; FOR(J=1;J<=4;J=J+1)#              Dump all possible grids          ME10 209
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 ME10 210
IF(J<4) N=15#                          15 items per line for three li   ME10 211
ELSE N=14#                             14 on the last                   ME10 212
FMT=413.#                              Start building the format        ME10 213
DO L=1,N#                                                               ME10 214
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 ME10 215
K=K+1 ; FMT=FMT+50000. ; BUFOT(1)=FLOAT(ND(K))#                         ME10 216
ncodefld:(BUFOT,1,CHROT,FMT,1)#                                         ME10 217
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 ME10 218
writeline:(0,0,0,1,1)#                                                  ME10 219
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 ME10 220
IF(GRIDCK==yes:) iquit:(ME1006)#       Immposible grid for fft          ME10 221
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ME10 222
#                                                                       ME10 223
# AQUIRE STORAGE AND SET UP USE OF FILE C, IF NEED BE                   ME10 224
# ---------------------------------------------------                   ME10 225
#                                                                       ME10 226
STOPP=MARKP+NPIX ; FILUSE=no:#         Get storage for electron densi   ME10 227
IF(STOPP>QXMAX\FILSIG==yes:)#          Or does user fancy file use      ME10 228
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 ME10 229
FILUSE=yes:#                           Have to use file c               ME10 230
STOPP=MARKP#                           Lower sights on memory           ME10 231
writeline:(0,ME7S,NME7S,3,3)#                                           ME10 232
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 ME10 233
qxmemory:(STOPP)#                      Get needed memory                ME10 234
#                                                                       ME10 235
RETURN#                                                                 ME10 236
END#                                                                    ME10 237
systemheader:(ME70)#                   Maximum entropy density calc     ME70   1
# ################################################################## #  ME70   2
#                                                                    #  ME70   3
#                               ME70                                 #  ME70   4
#                                                                    #  ME70   5
# ################################################################## #  ME70   6
SUBROUTINE ME70#                       D.m.collins j.m.stewart may 87   ME70   7
include:(AACOMN)##                                                      ME70   8
include:(MECOMN)#                                                       ME70   9
#                                                                       ME70  10
# THIS ROUTINE CALCULATES MAXIMUM ENTROPY MODIFIED ELECTRON DENSITY     ME70  11
#                                                                       ME70  12
#--------------------------------------+------------------------------- ME70  13
#     SUBROUTINE TO FIND A CONSTRAINED EXPONENTIAL DENSITY DISTRIBUTION ME70  14
#     CORRESPONDING TO A LOWER RESOLUTION INPUT FOURIER MAP.            ME70  15
#   **WARNING******WARNING******WARNING******WARNING******WARNING****** ME70  16
#   * THIS ROUTINE BECOMES ILL-CONDITIONED AND MAY FAIL AS THE MEAN OF* ME70  17
#   * THE ELECTRON DENSITY APPROACHES ZERO FROM THE POSITIVE.         * ME70  18
#   * E.G. IF E000 OR F000 IS LEFT OUT THE PROCESS WILL CRASH BECAUSE * ME70  19
#   * THE AVERAGE ELECTRON DENSITY WILL BE NEAR ZERO.                 * ME70  20
#   ******************************************************************* ME70  21
#--------------------------------------+------------------------------- ME70  22
#***  TAKEN FROM 'SUBROUTINE MAXENT(...)' WHICH WAS                     ME70  23
#***  WRITTEN BY E. PRINCE, NATIONAL BUREAU OF STANDARDS. APRIL, 1987,  ME70  24
# TO TAKE ADVANTAGE OF THE INCREASE OF RESOLUTION IN THE NEW DENSITY,   ME70  25
# THE GRID ON WHICH THE DENSITY IS SAMPLED MUST BE APPROPRIATELY        ME70  26
# FINE.  THE INPUT DENSITY, ON NORMAL EXIT, IS OVERWRITTEN              ME70  27
# BY AN EXPONENTIAL REPRESENTATION THAT SATISFIES THE TWO               ME70  28
# CONSTRAINTS:                                                          ME70  29
#         1)  THAT THE SUM(RHO(I)) IS CONSTANT, AND                     ME70  30
#         2)  THAT THE EXPECTED VALUE OF SUM(|F|*COS(2*PI*H.R+PHI)) IS  ME70  31
#             CONSTANT; ALTERNATIVELY,                                  ME70  32
#             THAT THE EXPECTATIONS OF [(ORIGINAL DENSITY)**0] AND      ME70  33
#             [(ORIGINAL DENSITY)**1] IN THE NEW REPRESENTATION ARE     ME70  34
#             THE SAME AS IN THE ORIGINAL.                              ME70  35
# THE CAPTURED ORIGINAL DENSITY IS HELD IN THE MARKP ARRAY. THE UPDATED ME70  36
# DENSITY CONVERTED TO THE MODIFIED DENSITY FOR OUTPUT.                 ME70  37
# EPS=MAXIMUM TOLERABLE VALUE OF CONSTRAINT FAILURE, E.G.,              ME70  38
# ABS(SUM(DER(I)**2)-SUM(DER(I)*RHO(I))/SUM(DER(I)**2)                  ME70  39
#--------------------------------------+------------------------------- ME70  40
INTEGER I,J,L## Local indices for loops etc.                            ME70  41
INTEGER IDMC## Newton-Raphson iteration counter                         ME70  42
INTEGER PRINTP## Electron density dumping control                       ME70  43
INTEGER PSIZ,IP## I/o pointers                                          ME70  44
INTEGER X1,X2## Layer, down the layer, across the layer coordinates     ME70  45
#                                                                       ME70  46
REAL LNRHO## Logarithm of the new density                               ME70  47
REAL LNRHMN,LNRHMX## Min & max in rho, an ordered pair sets contrast    ME70  48
REAL AHIST(11),BHIST(11)## History of the values of LNRHMN & LNRHMX     ME70  49
REAL SOED## Original sum for constraint one                             ME70  50
REAL SOED2## Original sum for constraint two                            ME70  51
REAL SSNED2## Subsequent sum for constraint two                         ME70  52
REAL DEDX## Derivative (constr two) by [LNRHMX-LNRHMN]                  ME70  53
REAL DELTA## Fixed, original (RHOMAX-RHOMIN)                            ME70  54
REAL DENLIM(3)## El-dens statistics                                     ME70  55
REAL DMC(11)## Fractional failure of constr two                         ME70  56
REAL EPS## Limit of fractional failure of constraints                   ME70  57
REAL FMT## Word for creating formats                                    ME70  58
REAL MAXELD## Final modified density maximum for MEFFIT                 ME70  59
REAL NEWELD,OLDELD## Electron density during processing                 ME70  60
REAL Q,QP## Temporary variables                                         ME70  61
REAL RESCAL## Renormalization factor                                    ME70  62
REAL REHIST(11)## History of the renormalization factor                 ME70  63
REAL SNED,SNED2,SZNED,SZNED2## Accumulators of convenience              ME70  64
REAL TOL## Larger constraint failure                                    ME70  65
REAL TOLONE## Fractional failure of constr one                          ME70  66
REAL TOLTWO## Fractional failure of constr two                          ME70  67
REAL V## Normalized density variance                                    ME70  68
REAL Z## Density mapped on the interval [0,1]                           ME70  69
chardata:(ME7D, SUM OF ELECTRON DENS =             $#                   ME70  70
SUM OF SQUARED =             )#                                         ME70  71
chardata:(ME7E, CYCLES   CONSTRAINT HISTORY IN TERMS OF $#              ME70  72
FRACTIONAL MISMATCH)#                                                   ME70  73
chardata:(ME7F, VARIANCE OF ELECTRON DENSITY =            $#            ME70  74
NO. OF PIXELS =             )#                                          ME70  75
chardata:(ME7G, PRINT A SAMPLE OF INPUT AND OUTPUT ELECTRON DENSITY)#   ME70  76
chardata:(ME7H,  X1  X2  X3   INPUT RHO  OUTPUT RHO )#                  ME70  77
chardata:(ME7I, THE INPUT MAP IS ILL CONDITIONED SINCE )#               ME70  78
chardata:(ME7J, THE VARIANCE OF THE ELECTRON DENSITY IS $#              ME70  79
OUTSIDE THE RANGE 0.0001 THRU 0.9999.)#                                 ME70  80
chardata:(ME7K,[LN(RHMN)])#                                             ME70  81
chardata:(ME7L,[LN(RHMX)])#                                             ME70  82
chardata:(ME7M,RESCALES)#                                               ME70  83
chardata:(ME7N,[ PRELIMINARY LN(RHMN) AND LN(RHMX) VALUES =           $#ME70  84
AND          ])#                                                        ME70  85
chardata:(ME7Q, ELECTRON DENSITY MAXIMUM =           $#                 ME70  86
MINIMUM =           AVERAGE =           )#                              ME70  87
chardata:(ME7Y, MAXIMUM ELECTRON DENSITY IN OUTPUT =            )#      ME70  88
chardata:(ME7R, THE NORMALIZED ENTROPY OF THE OUTPUT MAP =           )# ME70  89
chardata:(ME7S, RELATIVE TO THE UNIFORM DISTRIBUTION.)#                 ME70  90
chardata:(ME7U,[ F(0,0,0) OF INPUT MAP    F(0,0,0) OF OUTPUT MAP    $#  ME70  91
SCALE FACTOR TO FORCE OUTPUT TO INPUT   SHARPENING FACTOR])#            ME70  92
realdata:(FMTU,[131222.,371222.,641052.,1021042.])#                     ME70  93
realdata:(FMTV,[650642.,770642.])#                                      ME70  94
realdata:(FMT7,[341031.,631031.])#                                      ME70  95
realdata:(FMT8,[421031.,681013.])#                                      ME70  96
realdata:(FMTL,[530852.,660852.])#                                      ME70  97
realdata:(FMTO,[40313.,80313.,120313.,241041.,361041.])#                ME70  98
realdata:(FMTST,[370931.,570931.,770931.])#                             ME70  99
realdata:(VARLOW,[0.0001])#            Lower limit for variance         ME70 100
realdata:(VARHIG,[0.9999])#            Upper limit for variance         ME70 101
datastuff:#                                                             ME70 102
#                                                                       ME70 103
# INITIALIZE COUNTS, NUMBERS OF POINTS IN SUMMATIONS AND PRINT OUT      ME70 104
# ----------------------------------------------------------------      ME70 105
#                                                                       ME70 106
DO J=1,11#                             Clear refinement history datum   ME70 107
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ME70 108
DMC(J)=0.0 ; AHIST(J)=0.0 ; BHIST(J)=0.0 ; REHIST(J)=0.0#               ME70 109
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ME70 110
DENLIM(1)=-99999. ; DENLIM(2)=99999. ; DENLIM(3)=0.0# El-den stats      ME70 111
EPS=0.001#                             Limit of constraint failure      ME70 112
SOED=0.0 ; SOED2=0.0#                  Initialize constraint sums       ME70 113
PRINTP=PRINTL ; IF(PRINTL>0) writeline:(1,ME7G,NME7G,3,3)#              ME70 114
LINRM=6#                                                                ME70 115
#                                                                       ME70 116
# TOP OF LOOPS OVER LINES AND LAYERS TO PROCESS ELECTRON DENSITY        ME70 117
# --------------------------------------------------------------        ME70 118
#                                                                       ME70 119
STOPP=MARKP#                           Set storage of el. dens.         ME70 120
X1=-1#                                 Set to loop over layers          ME70 121
REPEAT#                                Loop over layers                 ME70 122
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ME70 123
X1=X1+1#                               Increment layer count            ME70 124
IF(X1>=NX1) BREAK#                     Get out if all done              ME70 125
X2=-1#                                 Set to loop over pages           ME70 126
REPEAT#                                Loop over pages                  ME70 127
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ME70 128
X2=X2+1#                               Increment page count             ME70 129
IF(X2>=NX2) BREAK#                     Test if whole layer is in        ME70 130
readwpkt:(3,22,PSIZ,IP,0)#             Read next line                   ME70 131
IF(IP<=0) BREAK#                       Quit if all layers done          ME70 132
I=IP#                                  Set pointer for el-density       ME70 133
DO J=1,NX3#                            Loop across line                 ME70 134
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ME70 135
I=I+1 ; OLDELD=QX(I)#                                                   ME70 136
DENLIM(3)=DENLIM(3)+OLDELD#            Build statistics                 ME70 137
DENLIM(1)=AMAX1(OLDELD,DENLIM(1)) ; DENLIM(2)=AMIN1(OLDELD,DENLIM(2))#  ME70 138
#                                                                       ME70 139
# FIND SUMMATIONS TO GIVE CONSTRAINTS ONE AND TWO                       ME70 140
# -----------------------------------------------                       ME70 141
#                                                                       ME70 142
SOED=SOED+OLDELD ; SOED2=SOED2+OLDELD**2# Total old dens and square     ME70 143
IF(FILUSE==yes:) NEXT#                 All memory on file c             ME70 144
STOPP=STOPP+1 ; QX(STOPP)=OLDELD#      Save the original el. dens.      ME70 145
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ME70 146
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ME70 147
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ME70 148
IF(FILUSE==yes:) rewindbdf:(3)#        Rewind file if need be           ME70 149
#                                                                       ME70 150
# AT THE END OF READING THE WHOLE MAP GIVE EL-DENS STATS                ME70 151
# ------------------------------------------------------                ME70 152
#                                                                       ME70 153
DENLIM(3)=DENLIM(3)/FLOAT(NPIX)#       Find ave el-dens                 ME70 154
ncodefld:(DENLIM,1,ME7Q,FMTST,3) ; writeline:(1,ME7Q,NME7Q,3,3)#        ME70 155
#                                                                       ME70 156
# CHECK THE DENSITY FUNCTION FOR SUITABILITY                            ME70 157
# ------------------------------------------                            ME70 158
#                                                                       ME70 159
Q=(SOED**2)/FLOAT(NPIX)#               Q=sum( old d)**2 per pixel       ME70 160
V=(SOED2-Q)/SOED2#                     Soed2=sum( old d**2)             ME70 161
BUFOT(1)=V ; BUFOT(2)=FLOAT(NPIX)#                                      ME70 162
ncodefld:(BUFOT,1,ME7F,FMT8,2) ; writeline:(1,ME7F,NME7F,3,3)#          ME70 163
IF(V<VARLOW\V>VARHIG)#                 Variance must lie in [0,1]       ME70 164
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ME70 165
BUFOT(1)=SOED ; BUFOT(2)=SOED2#        Display sums                     ME70 166
ncodefld:(BUFOT,1,ME7D,FMT7,2) ; writeline:(1,ME7D,NME7D,3,3)#          ME70 167
writeline:(1,ME7I,NME7I,3,3) ; BUFOT(1)=VARLOW ; BUFOT(2)=VARHIG#       ME70 168
ncodefld:(BUFOT,1,ME7J,FMTV,2) ; writeline:(0,ME7J,NME7J,3,3)#          ME70 169
iquit:(ME7001)#                        If variance extreme              ME70 170
#                                      I.e., if dens flat or spiky      ME70 171
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ME70 172
#                                                                       ME70 173
# SET PARAMETERS TO START BUILDING THE EXPONENTIAL REPRESENTATION       ME70 174
# ---------------------------------------------------------------       ME70 175
#                                                                       ME70 176
DELTA=DENLIM(1)-DENLIM(2)#             Original range of density        ME70 177
LNRHMX=ALOG(DENLIM(1))#                Lnrhmx part of contrast          ME70 178
Q=AMAX1(DENLIM(2),0.005*DENLIM(1))#    Limit on init. contrast          ME70 179
LNRHMN=ALOG(Q)#                        A part of contrast               ME70 180
IDMC=0#                                Set iteration counter            ME70 181
#                                                                       ME70 182
# IF THE MAP IS TO BE FORCED TO THE USERS PARAMETERS SKIP CYCLING       ME70 183
# ---------------------------------------------------------------       ME70 184
#                                                                       ME70 185
IF(FORCE==no:)#                        Test user demand                 ME70 186
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ME70 187
BUFOT(1)=LNRHMN ; BUFOT(2)=LNRHMX ; ncodefld:(BUFOT,1,ME7N,FMTL,2)#     ME70 188
writeline:(1,ME7N,NME7N,3,3)#                                           ME70 189
#                                                                       ME70 190
# TOP OF THE NEWTON-RAPHSON LOOP TO HONOR THE CONSTRAINTS               ME70 191
# -------------------------------------------------------               ME70 192
#                                                                       ME70 193
REPEAT#                                Iterate!                         ME70 194
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ME70 195
SNED=0.;SNED2=0.;SZNED=0.;SZNED2=0.#   Initialize                       ME70 196
IDMC=IDMC+1#                           Increment counter                ME70 197
#                                                                       ME70 198
# THE LOGARITHM OF THE DENSITY IN EACH PIXEL IS A LINEAR FUNCTION       ME70 199
# OF LNRHOMN AND LNRHMX.                                                ME70 200
# ---------------------------------------------------------------       ME70 201
#                                                                       ME70 202
IF(FILUSE==no:)#                       If no file c use, go for it      ME70 203
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ME70 204
DO I=MARKP+1,STOPP#                    Loop over all density            ME70 205
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ME70 206
OLDELD=QX(I)#                          Pick up electron density         ME70 207
Z=(OLDELD-DENLIM(2))/DELTA#            Density mapped on [0,1]          ME70 208
LNRHO=Z*LNRHMX+(1.-Z)*LNRHMN#          Logarithm of new density         ME70 209
NEWELD=EXP(LNRHO)#                     The new density                  ME70 210
SNED=SNED+NEWELD ; SNED2=SNED2+NEWELD*NEWELD# Accumulator               ME70 211
SZNED=SZNED+Z*NEWELD ; SZNED2=SZNED2+Z*NEWELD*NEWELD# Accumulator       ME70 212
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ME70 213
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ME70 214
ELSE## Use of file c for memory complicates loop                        ME70 215
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ME70 216
X1=-1#                                 Set to loop over layers          ME70 217
REPEAT#                                Loop over layers                 ME70 218
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ME70 219
X1=X1+1#                               Increment layer count            ME70 220
IF(X1>=NX1) BREAK#                     Get out if all done              ME70 221
X2=-1#                                 Set to loop over pages           ME70 222
REPEAT#                                Loop over pages                  ME70 223
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ME70 224
X2=X2+1#                               Increment page count             ME70 225
IF(X2>=NX2) BREAK#                     Test if whole layer is in        ME70 226
readwpkt:(3,22,PSIZ,IP,0) ; I=IP#      Take in next line from file c    ME70 227
DO J=1,NX3#                            Loop across line                 ME70 228
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ME70 229
I=I+1#                                                                  ME70 230
OLDELD=QX(I)#                          Pick up electron density         ME70 231
Z=(OLDELD-DENLIM(2))/DELTA#            Density mapped on [0,1]          ME70 232
LNRHO=Z*LNRHMX+(1.-Z)*LNRHMN#          Logarithm of new density         ME70 233
NEWELD=EXP(LNRHO)#                     The new density                  ME70 234
SNED=SNED+NEWELD ; SNED2=SNED2+NEWELD*NEWELD# Accumulator               ME70 235
SZNED=SZNED+Z*NEWELD ; SZNED2=SZNED2+Z*NEWELD*NEWELD# Accumulator       ME70 236
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ME70 237
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ME70 238
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ME70 239
rewindbdf:(3)#                         Rewind file c for next pass      ME70 240
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ME70 241
#                                                                       ME70 242
# RENORMALIZE BY MULTIPLYING ALL DENSITIES BY A CONSTANT AND            ME70 243
# USE NEWTON-RAPHSON ALGORITHM TO FIND VALUES OF (LNRHMN,LNRHMX) THAT   ME70 244
# SATISFY THE SECOND CONSTRAINT CONDITION.                              ME70 245
# -----------------------------------------------------------           ME70 246
#                                                                       ME70 247
RESCAL=SOED/SNED#                      Establish the renormalization    ME70 248
REHIST(IDMC)=RESCAL#                   Keep a history of value          ME70 249
SSNED2=SNED2*RESCAL**2#                Constr two in current density    ME70 250
TOLONE=$#                              The constraint one failure       ME70 251
ABS(SNED-SOED)/SOED#                   Test parameter                   ME70 252
TOLTWO=$#                              The constraint two failure       ME70 253
ABS(SSNED2-SOED2)/SOED2#                                                ME70 254
TOL=AMAX1(TOLONE,TOLTWO)#              Use larger constraint failure    ME70 255
DMC(IDMC)=TOL#                         Capture it                       ME70 256
#                                                                       ME70 257
# STOP AFTER EIGHT CYCLES OR WHEN CONSTRAINTS ARE SATISFIED             ME70 258
# ---------------------------------------------------------             ME70 259
#                                                                       ME70 260
IF(IDMC>10\TOL<EPS) BREAK#             Test if cycling done             ME70 261
Q=ALOG(RESCAL)#                        Ln of normalization scale        ME70 262
Q=AMIN1(AMAX1(Q,-LOGLIM),LOGLIM)#      Restrain rescaling               ME70 263
DEDX=2.0*SOED*RESCAL*$#                Derivative of (constr two)       ME70 264
(SZNED2/SNED-(SNED2*SZNED)/SNED**2)#   By [lnrhmx-lnrhmn]               ME70 265
QP=(SOED2-SSNED2)/DEDX#                Calculate shift                  ME70 266
QP=AMIN1(AMAX1(QP,-LOGLIM),LOGLIM)#    Restrain shift                   ME70 267
LNRHMN=LNRHMN+Q#                       Rescale ln(rho minimum)          ME70 268
LNRHMX=LNRHMX+Q+QP#                    Rescale and adjust contrast      ME70 269
AHIST(IDMC)=LNRHMN ; BHIST(IDMC)=LNRHMX# Keep record of how they go     ME70 270
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ME70 271
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ME70 272
ELSE#                                  Force users parameters on map    ME70 273
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ME70 274
LNRHMN=LRMNUS ; LNRHMX=LRMXUS#                                          ME70 275
SOED=F000US*FLOAT(NP(1)*NP(2)*NP(3))/VOL# Set the three parameters      ME70 276
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ME70 277
#                                                                       ME70 278
# CORRECT THE RESCALE FACTOR SO F(0,0,0) WILL BE MAINTAINED             ME70 279
# ---------------------------------------------------------             ME70 280
#                                                                       ME70 281
SNED=0.#                               Initialize                       ME70 282
#                                                                       ME70 283
# THE LOGARITHM OF THE DENSITY IN EACH PIXEL IS A LINEAR FUNCTION       ME70 284
# OF LNRHMN AND LNRHMX MODIFIED BY SHARPF.                              ME70 285
# ---------------------------------------------------------------       ME70 286
#                                                                       ME70 287
IF(FILUSE==no:)#                       If no file c use, go for it      ME70 288
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ME70 289
DO I=MARKP+1,STOPP#                    Loop over all density            ME70 290
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ME70 291
OLDELD=QX(I)#                          Pick up electron density         ME70 292
Z=(OLDELD-DENLIM(2))/DELTA#            Density mapped on [0,1]          ME70 293
LNRHO=SHARPF*(Z*LNRHMX+(1.-Z)*LNRHMN)# Logarithm of new density         ME70 294
NEWELD=EXP(LNRHO)#                     The new density                  ME70 295
SNED=SNED+NEWELD#                      Accumulator                      ME70 296
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ME70 297
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ME70 298
ELSE## Reuse of file c for memory complicates loop                      ME70 299
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 ME70 300
X1=-1#                                 Set to loop over layers          ME70 301
REPEAT#                                Loop over layers                 ME70 302
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 ME70 303
X1=X1+1#                               Increment layer count            ME70 304
IF(X1>=NX1) BREAK#                     Get out if all done              ME70 305
X2=-1#                                 Set to loop over pages           ME70 306
REPEAT#                                Loop over pages                  ME70 307
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 ME70 308
X2=X2+1#                               Increment page count             ME70 309
IF(X2>=NX2) BREAK#                     Test if whole layer is in        ME70 310
readwpkt:(3,22,PSIZ,IP,0) ; I=IP#      Take in next line from file c    ME70 311
DO J=1,NX3#                            Loop across line                 ME70 312
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 ME70 313
I=I+1#                                                                  ME70 314
OLDELD=QX(I)#                          Pick up electron density         ME70 315
Z=(OLDELD-DENLIM(2))/DELTA#            Density mapped on [0,1]          ME70 316
LNRHO=SHARPF*(Z*LNRHMX+(1.-Z)*LNRHMN)# Logarithm of new density         ME70 317
NEWELD=EXP(LNRHO)#                     The new density                  ME70 318
SNED=SNED+NEWELD#                      Accumulator                      ME70 319
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 ME70 320
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 ME70 321
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 ME70 322
rewindbdf:(3)#                         Reposition file c once more      ME70 323
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 ME70 324
IF(FORCE==no:)#                        Summary written only on cyclin   ME70 325
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 ME70 326
#                                                                       ME70 327
# COMPLETION OF CONSTRUCTION OF EXPONENTIAL DENSITY REPRESENTATION      ME70 328
# BY SATISFYING (TOL .LE. EPS) OR FROM EXHAUSTION (IDMC .GE. 11)        ME70 329
# ----------------------------------------------------------------      ME70 330
#                                                                       ME70 331
BUFOT(1)=SOED ; BUFOT(2)=SOED2#        Get sums of el. dens.            ME70 332
ncodefld:(BUFOT,1,ME7D,FMT7,2) ; writeline:(1,ME7D,NME7D,3,3)#          ME70 333
FMT=180852. ; BUFOT(1)=FLOAT(IDMC)#                                     ME70 334
ncodefld:(BUFOT,1,CHROT,70313.,1)#                                      ME70 335
FOR(I=1;I<=IDMC;I=I+1)#                                                 ME70 336
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 ME70 337
ncodefld:(DMC,I,CHROT,FMT,1) ; FMT=FMT+100000.#                         ME70 338
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 ME70 339
writeline:(1,ME7E,NME7E,2,3)#          Display refinement history       ME70 340
FMT=180852. ; movechar:(ME7M,1,CHROT,2,NME7M,0)#                        ME70 341
DO I=1,IDMC#                                                            ME70 342
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 ME70 343
ncodefld:(REHIST,I,CHROT,FMT,1) ; FMT=FMT+100000.#                      ME70 344
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 ME70 345
writeline:(0,0,0,1,3)#                 Display history of rescale       ME70 346
FMT=180852. ; movechar:(ME7K,1,CHROT,2,NME7K,0)#                        ME70 347
DO I=1,IDMC-1#                                                          ME70 348
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 ME70 349
ncodefld:(AHIST,I,CHROT,FMT,1) ; FMT=FMT+100000.#                       ME70 350
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 ME70 351
writeline:(0,0,0,1,3)#                 Display history of a values      ME70 352
FMT=180852. ; movechar:(ME7L,1,CHROT,2,NME7L,0)#                        ME70 353
DO I=1,IDMC-1#                                                          ME70 354
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 ME70 355
ncodefld:(BHIST,I,CHROT,FMT,1) ; FMT=FMT+100000.#                       ME70 356
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 ME70 357
writeline:(0,0,0,1,3)#                 Display history of b values      ME70 358
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 ME70 359
#                                                                       ME70 360
# SEE IF SHARPENING VALUE HAS BEEN SUPPLIED.  IF SO REPORT OUT NEW      ME70 361
# MAXIMUM ELECTRON DENSITY.  IF NO SHARPENING, THEN SCALING HAS         ME70 362
# BEEN COMPLETELY DONE THROUGH LNRHMN AND LNRHMX FACTORS. IF SHARPENING ME70 363
# IS TO BE CARRIED OUT IT IS AS THOUGH LNRHMN & LNRHMX WERE ALTERED SO  ME70 364
# THE SCALE MUST BE ALTERED AND THE SHARPENING APPLIED.                 ME70 365
# ----------------------------------------------------------------      ME70 366
#                                                                       ME70 367
RESCAL=SOED/SNED#                      Establish the renormalization    ME70 368
BUFOT(1)=SOED*VOL/FLOAT(NP(1)*NP(2)*NP(3))# Convert sum of e.d.         ME70 369
BUFOT(2)=RESCAL*SNED*VOL/FLOAT(NP(1)*NP(2)*NP(3))# Back to f(0,0,0)     ME70 370
BUFOT(3)=RESCAL#                       Report out f(0,0,0)s & rescale   ME70 371
BUFOT(4)=SHARPF#                       Sharpener that gave scale        ME70 372
ncodefld:(BUFOT,1,CHROT,FMTU,4) ; writeline:(1,ME7U,NME7U,2,3)#         ME70 373
IF(SHARP==yes:)#                                                        ME70 374
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 ME70 375
MAXELD=RESCAL*EXP(SHARPF*LNRHMX)#      Get sharpend e.d. max            ME70 376
ncodefld:(MAXELD,1,ME7Y,481042.,1) ; writeline:(1,ME7Y,NME7Y,3,3)#      ME70 377
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 ME70 378
ELSE#                                  Protect unsharp scale            ME70 379
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 ME70 380
MAXELD=EXP(LNRHMX)#                    And set true maximum             ME70 381
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 ME70 382
writeline:(0,0,0,1,3) ; LINRM=10#      Assure heading on sample o/p     ME70 383
#                                                                       ME70 384
# NOW PLACE THE MAXIMUM IN THE MODIFIED FUNCTION IN LR 21 FOR USE       ME70 385
# BY THE PROGRAM MEFFIT                                                 ME70 386
# ---------------------------------------------------------------       ME70 387
QX(JP+25)=MAXELD#                      Pointer jp is saved from me10    ME70 388
QX(JP+26)=LNRHMX ; QX(JP+27)=LNRHMN#   Save the scaling for use         ME70 389
QX(JP+28)=SHARPF ; QX(JP+29)=RESCAL#   By other programs in me series   ME70 390
#                                                                       ME70 391
# TOP OF LOOPS OVER LINES AND LAYERS TO OUTPUT NEW ELECTRON DENSITY     ME70 392
# -----------------------------------------------------------------     ME70 393
#                                                                       ME70 394
SNED=0.0 ; ENTRPY=0.0#                 Zero sums for entropy calc       ME70 395
L=MARKP#                               Set fetching of el. dens.        ME70 396
X1=-1#                                 Set to loop over layers          ME70 397
REPEAT#                                Loop over layers                 ME70 398
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 ME70 399
X1=X1+1#                               Increment layer count            ME70 400
IF(X1>=NX1) BREAK#                     Get out if all done              ME70 401
X2=-1 ; PRINTP=PRINTL#                 Set to loop over pages           ME70 402
REPEAT#                                Loop over pages                  ME70 403
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 ME70 404
X2=X2+1#                               Increment page count             ME70 405
IF(X2>=NX2) BREAK#                     Test if whole layer is in        ME70 406
IF(FILUSE==yes:)#                      Check use of file c for memory   ME70 407
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 ME70 408
readwpkt:(3,22,PSIZ,IP,0) ; L=IP#      Take in next line from file c    ME70 409
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 ME70 410
writepkt:(4,22,PSIZ,JP,0)#             Write next line                  ME70 411
I=JP#                                  Set pointer for buffer           ME70 412
IF(PRINTP>0)#                          Check if dumping to be done      ME70 413
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 ME70 414
DO J=1,NX3#                            Loop across line                 ME70 415
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 ME70 416
I=I+1 ; L=L+1#                                                          ME70 417
Z=(QX(L)-DENLIM(2))/DELTA#             Density mapped on [0,1]          ME70 418
LNRHO=(Z*LNRHMX+(1.0-Z)*LNRHMN)#       Log of the new density           ME70 419
SNED=SNED+EXP(LNRHO)#                  Find the sum of rho              ME70 420
ENTRPY=ENTRPY-EXP(LNRHO)*LNRHO#        Find sum rho*ln(rho)             ME70 421
LNRHO=SHARPF*LNRHO#                    Sharpen if need be               ME70 422
QX(I)=RESCAL*EXP(LNRHO)#               Antilog of x gives new dens      ME70 423
IF(PRINTP>0&X2>=(J-1))#                See if dumping a sample          ME70 424
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 ME70 425
BUFOT(1)=FLOAT(X1);BUFOT(2)=FLOAT(X2);BUFOT(3)=FLOAT(J-1)#              ME70 426
BUFOT(4)=QX(L) ; BUFOT(5)=QX(I)#       Get electron densities           ME70 427
ncodefld:(BUFOT,1,CHROT,FMTO,5) ; writeline:(0,ME7H,NME7H,1,3)#         ME70 428
PRINTP=PRINTP-1#                       Decrement ouput lines            ME70 429
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 ME70 430
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 ME70 431
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 ME70 432
ELSE#                                  No dumping give optimizer head   ME70 433
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 ME70 434
DO J=1,NX3#                            Loop across line                 ME70 435
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 ME70 436
I=I+1 ; L=L+1#                                                          ME70 437
Z=(QX(L)-DENLIM(2))/DELTA#             Density mapped on [0,1]          ME70 438
LNRHO=(Z*LNRHMX+(1.0-Z)*LNRHMN)#       Log of the new density           ME70 439
SNED=SNED+EXP(LNRHO)#                  Find the sum of rho              ME70 440
ENTRPY=ENTRPY-EXP(LNRHO)*LNRHO#        Find sum rho*ln(rho)             ME70 441
LNRHO=SHARPF*LNRHO#                    Sharpen if need be               ME70 442
QX(I)=RESCAL*EXP(LNRHO)#               Antilog gives new dens           ME70 443
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 ME70 444
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 ME70 445
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 ME70 446
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 ME70 447
#                                                                       ME70 448
# FINISH OFF ALL BDFS REPORT ENTROPY AND QUIT                           ME70 449
# -------------------------------------------                           ME70 450
#                                                                       ME70 451
readwpkt:(3,endrecord:,PSIZ,IP,0)#     End off old el-dens file         ME70 452
writepkt:(4,endrecord:,PSIZ,IP,0)#     End off new el-dens file         ME70 453
#                                                                       ME70 454
# CALCULATE THE ENROPY OF THE MAP IN A UNITLESS FORM NORMALIZED         ME70 455
# RELATIVE TO THE UNIFORM DISTRIBUTION.                                 ME70 456
# ENTROPY = -(SUM(RHO*LN(RHO)))/SUM(RHO) + LN(SUM(RHO)/NPIX)            ME70 457
# --------------------------------------------------------------        ME70 458
#                                                                       ME70 459
ENTRPY=(ENTRPY/SNED)+(ALOG(SNED/FLOAT(NPIX)))# Calculate the entropy    ME70 460
ncodefld:(ENTRPY,1,ME7R,541041.,1)#    Write out for viewing            ME70 461
writeline:(2,ME7R,NME7R,3,3) ; writeline:(0,ME7S,NME7S,3,3)#            ME70 462
#                                                                       ME70 463
RETURN#                                                                 ME70 464
END#                                                                    ME70 465
pmacro:([no:])#                        Purge xtal system macros         ME70 466
