systemheader:(MF    )#                                                  MF     1
####################################################################### MF     2
##                                                                    # MF     3
##                        ** MEFFIT **                                # MF     4
##                                                                    # MF     5
##  THE PROGRAM MEFFIT COMPUTES A MAXIMUM-ENTROPY ELECTRON DENSITY    # MF     6
##  DISTRIBUTION OF IMPROVED AGREEMENT WITH OBSERVED STRUCTURE        # MF     7
##  MODULI.  GIVEN AN INPUT POSITIVE-DEFINITE ELECTRON DENSITY MAP    # MF     8
##  (FROM MEDENS, FOR EXAMPLE) AND A DIFFERENCE MAP BASED ON          # MF     9
##  CALCULATED STRUCTURE FACTORS FROM ITS REVERSE FOURIER TRANSFORM   # MF    10
##  (PROGRAM RFOURR), THIS PROGRAM FINDS A NEW MAXIMUM-ENTROPY MAP    # MF    11
##  IN BETTER AGREEMENT WITH THE OBSERVATIONS.  THE PROCESS IS        # MF    12
##  POINTWISE MULTIPLICATION OF THE INPUT MAP BY EXP{CONSTANT*        # MF    13
##  DIFFERENCE MAP} WITH SUBSEQUENT RESCALING TO KEEP THE MEAN OF     # MF    14
##  DENSITY CONSTANT.  THE METHOD, USING THE EQUIVALENT LOGARITHMIC   # MF    15
##  FORMULATION, IS DESCRIBED IN:  COLLINS, D. M. & MAHAR, M. C.      # MF    16
##  (1983).  ACTA CRYST. A39, 252-256.                                # MF    17
##                                                                    # MF    18
##                  D. M. COLLINS & J. M. STEWART  FEBRUARY 1988      # MF    19
##                                                                    # MF    20
####################################################################### MF    21
#                                                                       MF    22
include:(XMACRO)#                                                       MF    23
#                                                                       MF    24
extrude:(MFCOMN)#                                                       MF    25
REAL    AFACT#      ## "A" factor from eqn (2.5) p253                   MF    26
REAL    DAMPLM#     ## Exponential damping limit used in MF70           MF    27
INTEGER FILUSE,FILSIG## Signal to use files in place of memory          MF    28
INTEGER IN(3)#      ## Grid point increment on input map                MF    29
INTEGER ISUM(3)#    ## Summation order correlation between              MF    30
INTEGER JSUM(3)#    ## Crystal axes and map axes                        MF    31
INTEGER MARKP#      ## Start of storage of el dens map                  MF    32
INTEGER NG(3)#      ## Number of grid points in map                     MF    33
INTEGER NP(3)#      ## Actual number of points in map                   MF    34
INTEGER NPIX#       ## Total pixels in input fourier map                MF    35
INTEGER NX1,NX2,NX3## No. of points in each direction                   MF    36
INTEGER OG(3)#      ## Grid values of the origin of the map             MF    37
INTEGER PRINTL#     ## Flag to control printing                         MF    38
REAL    RMXDEN#     ## Maximum density in file c input map              MF    39
INTEGER STOPP#      ## End of electron density array                    MF    40
#                                                                       MF    41
COMMON/COMIMF/$#                                                        MF    42
FILSIG,FILUSE,IN    ,ISUM  ,JSUM  ,MARKP ,$#                            MF    43
NG    ,NP    ,NPIX  ,NX1   ,NX2   ,NX3   ,$#                            MF    44
OG    ,PRINTL,STOPP ,WRDBLK#                                            MF    45
COMMON/COMFMF/AFACT,DAMPLM,RMXDEN#                                      MF    46
endext:#                                                                MF    47
#                                                                       MF    48
systemheader:(MF00)#                                                    MF00   1
# ################################################################## #  MF00   2
#                                                                       MF00   3
#                               MF00                                 #  MF00   4
#                                                                    #  MF00   5
# ################################################################## #  MF00   6
SUBROUTINE MF00## D.M.Collins,J.M.Stewart Feb 1988                      MF00   7
include:(AACOMN)#                                                       MF00   8
include:(MFCOMN)#                                                       MF00   9
#                                                                       MF00  10
#                                                                       MF00  11
CALL MF10#                             Bdf input and startup            MF00  12
CALL MF70#                             Calculate fft summations         MF00  13
#                                                                       MF00  14
RETURN#                                                                 MF00  15
END#                                                                    MF00  16
systemheader:(MF10)## bdf input and initialization                      MF10   1
# ################################################################## #  MF10   2
#                                                                    #  MF10   3
#                               MF10                                 #  MF10   4
#                                                                    #  MF10   5
# ################################################################## #  MF10   6
SUBROUTINE MF10## J.M.Stewart D.M.Collins Feb 1988                      MF10   7
include:(AACOMN)#                                                       MF10   8
include:(MFCOMN)#                                                       MF10   9
#                                                                       MF10  10
chardata:(MF10A, END    )#             Input lines allowed              MF10  11
chardata:(MF10B,PR  FI  A   DA  )#     Control commands                 MF10  12
chardata:(MF10D,XYZ)#                                                   MF10  13
chardata:(MF10G, MAP TRANSFORMATION    X FIRST Y SEC Z THIRD )#         MF10  14
chardata:(MF10J,YESNO )#                                                MF10  15
chardata:(MF10K, PROCESS A DIFFERENCE ELECTRON DENSITY MAP AND A $#     MF10  16
MAXIMUM ENTROPY DENSITY DISTRIBUTION TO HOLD FOBS VALUES.)#             MF10  17
chardata:(MF10L, DIRECTIVES GIVEN OR IMPLIED BY THE MEFFIT LINE...)#    MF10  18
chardata:(MF10N, PRINT        SAMPLES OF MODIFIED MAP PER LAYER)#       MF10  19
chardata:(MF10O, SCRATCH FILE ON H             )#                       MF10  20
chardata:(MF10Q, MAXIMUM ELECTRON DENSITY IN FILE             $#        MF10  21
AND DETERMINED BY SCAN OF MAP           )#                              MF10  22
chardata:(MF10W,FROM MAXIMUM ENTROPY MAP ON FILE C)#                    MF10  23
chardata:(MF10X,FROM DIFFERENCE MAP ON FILE D)#                         MF10  24
chardata:(MF10Y,TO MODIFIED MAXIMUM ENTROPY MAP ON FILE E)#             MF10  25
#                                                                       MF10  26
#------  ALLOWED VALUES OF NX IN FOURIER GRIDS                          MF10  27
#        (RANGE FROM 2 TO 5040) ONLY MXGRID IMPLEMENTED                 MF10  28
realdata:(MFFM,[450931.,850931.])#                                      MF10  29
intdata:(ND,[$#                                                         MF10  30
   2,   3,   4,   5,   6,   7,   8,   9,#                               MF10  31
10,  12,  14,  15,  16,  18,  20,  21,#                                 MF10  32
24,  28,  30,  35,  36,  40,  42,  45,#                                 MF10  33
48,  56,  60,  63,  70,  72,  80,  84,#                                 MF10  34
90, 105, 112, 120, 126, 140, 144, 168,#                                 MF10  35
180, 210, 240, 252, 280, 315, 336, 360,#                                MF10  36
420, 504, 560, 630, 720, 840,1008,1260,#                                MF10  37
1680,2520,5040])#                                                       MF10  38
intdata:(MXGRID,[57])##                                                 MF10  39
realdata:(FMNX,[110413.,170413.,270413.])# Format for grid error        MF10  40
chardata:(MF7A, ONE OR MORE SUMMATION DIRECTIONS ARE NOT IN $#          MF10  41
GRID DIVISIONS HANDLED BY FFT SUBROUTINES.)#                            MF10  42
chardata:(MF7B,[ GIVEN     ,     , AND     .   ACCEPTIBLE.......])#     MF10  43
chardata:(MF7C, WEIGHT CONTROL FACTOR FOR $#                            MF10  44
REFINEMENT =          )#                                                MF10  45
chardata:(MF7S, INSUFFICIENT IMMEDIATE ACCESS MEMORY FOR WHOLE MAP.  $# MF10  46
SCRATCH FILE WILL BE WRITTEN ON H)#                                     MF10  47
#                                                                       MF10  48
INTEGER I,J,K,L,N#                     Local indices                    MF10  49
INTEGER PSIZ,PSIZD,IP,IPD,IIP,IIPP,JP# Bdf reading parameters           MF10  50
INTEGER NDIV1,NDIV2,NDIV3#             Fourier grid divisions           MF10  51
#                                                                       MF10  52
datastuff:#                                                             MF10  53
#                                                                       MF10  54
PRINTL=0#                              Suppress refl print              MF10  55
FILSIG=no:#                            No scrath file to be used        MF10  56
AFACT=1.0#                             Set weight control factor        MF10  57
DAMPLM=2.0#                            Limit exp(+\-damplm)             MF10  58
#                                                                       MF10  59
#      ENTER THE MEFFIT OPTIONS                                         MF10  60
#      ------------------------                                         MF10  61
#                                                                       MF10  62
FOR(K=1;BUFIN(K)<voidflg:;K=K+1)#      Loop over input fields           MF10  63
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  MF10  64
stringlim:(K,I,J)#                     Find characters in input         MF10  65
compchar:(CHRIN,I,MF10B,1,2,4,NMF10B,J)# Search list of options         MF10  66
IF(J==0) iquit:(MF1001)#               It must be valid                 MF10  67
ELSE IF(J==1)#                         Test reflection print control    MF10  68
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  MF10  69
IF(BUFIN(K+1)>0.0)#                                                     MF10  70
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  MF10  71
K=K+1 ; PRINTL=NINT(BUFIN(K))#                                          MF10  72
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  MF10  73
ELSE PRINTL=10#                                                         MF10  74
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  MF10  75
ELSE IF(J==2) FILSIG=yes:#             Force use of scratch file        MF10  76
ELSE IF(J==3)#                         Test refinement limit control    MF10  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  MF10  78
IF(BUFIN(K+1)>=0.0&BUFIN(K+1)<=4.0)#                                    MF10  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  MF10  80
K=K+1 ; AFACT=AMIN1(BUFIN(K),4.0)#     Accept limit on weight control   MF10  81
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  MF10  82
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  MF10  83
ELSE IF(J==4)#                         Test exponent damping limit      MF10  84
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  MF10  85
IF(BUFIN(K+1)>=0.0&BUFIN(K+1)<=4.0)#                                    MF10  86
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  MF10  87
K=K+1 ; AFACT=AMIN1(BUFIN(K),4.0)#     Accept limit on damping          MF10  88
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  MF10  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  MF10  90
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  MF10  91
#                                                                       MF10  92
# WRITE OUT A SUMMARY OF INSTRUCTIONS                                   MF10  93
# -----------------------------------                                   MF10  94
#                                                                       MF10  95
LINRM=12 ; writeline:(1,MF10K,NMF10K,3,3)# Heading                      MF10  96
writeline:(1,MF10L,NMF10L,3,3)#                                         MF10  97
BUFOT(1)=FLOAT(PRINTL) ; ncodefld:(BUFOT,1,MF10N,130513.,1)#            MF10  98
writeline:(1,MF10N,NMF10N,3,3)#                                         MF10  99
IF(FILSIG==yes:)#                      User demands file usage          MF10 100
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  MF10 101
movechar:(MF10J,1,MF10O,26,3,0) ; writeline:(1,MF10O,NMF10O,3,3)#       MF10 102
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  MF10 103
ncodefld:(AFACT,1,MF7C,490832.,1) ; writeline:(1,MF7C,NMF7C,3,3)#       MF10 104
#                                                                       MF10 105
# TAKE IN ANY OTHER INPUT LINES                                         MF10 106
# -----------------------------                                         MF10 107
#                                                                       MF10 108
REPEAT#                                Loop over lines                  MF10 109
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  MF10 110
readline:(MF10A,NMF10A)#                                                MF10 111
IF(LINID==1) BREAK#                    Stop reading when end line       MF10 112
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  MF10 113
#                                                                       MF10 114
#  FORM THE FILE 'EXTENTION' NAMES FOR THE VARIOUS BINARY FILES         MF10 115
#  ------------------------------------------------------------         MF10 116
#                                                                       MF10 117
FILEXT(IOUNIT(3))='MED'#               Input medens density             MF10 118
FILEXT(IOUNIT(4))='MAP'#               Input difference map             MF10 119
FILEXT(IOUNIT(5))='MEM'#               Output me density map            MF10 120
FILEXT(IOUNIT(8))='XX1'#               Scratch file                     MF10 121
#                                                                       MF10 122
# SET UP I/O BUFFERS FOR TWO FILES; THE SCRATCH FILE DONE LATER         MF10 123
# -------------------------------------------------------------         MF10 124
#                                                                       MF10 125
IOMARK(3)=QXSTR#                       Set up electron density in/p1    MF10 126
IOMARK(4)=IOMARK(3)+bdfbuf:#           Set up electron density in/p2    MF10 127
IOMARK(5)=IOMARK(4)+bdfbuf:#           Set up electron density out/p    MF10 128
incrqx:(MARKP,IOMARK(5)+bdfbuf:,MF1005)# Pointer to stored densi        MF10 129
#                                                                       MF10 130
copyfile:(3,5,1,20)#                   Move lead records                MF10 131
#                                                                       MF10 132
# RETRIEVE THE MAP PARAMETERS FROM LR 21 OF FILE C & D                  MF10 133
# AND START BUILDING THE ENTROPY MAXIMIZED MAP ON FILE E                MF10 134
# ------------------------------------------------------                MF10 135
#                                                                       MF10 136
readwpkt:(3,21,PSIZ,IP,0)#                                              MF10 137
readwpkt:(4,21,PSIZD,IPD,0)#                                            MF10 138
writepkt:(5,21,PSIZ,JP,0) ; movereal:(QX,IP+1,QX,JP+1,PSIZ,0)#          MF10 139
DO K=1,3#                                                               MF10 140
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 MF10 141
IIP=IP+K ; IIPP=IPD+K#                                                  MF10 142
OG(K)=NINT(QX(IIP))#                   Pick up origins                  MF10 143
IF(NINT(QX(IIP))!=NINT(QX(IIPP))) iquit:(MF1010)# Origins mismatch      MF10 144
IN(K)=NINT(QX(IIP+3))#                 Increments along axes            MF10 145
IF(NINT(QX(IIP+3))!=NINT(QX(IIPP+3))) iquit:(MF1011)# Grid inc mismat   MF10 146
NG(K)=NINT(QX(IIP+6))#                 No. of grid points               MF10 147
IF(NINT(QX(IIP+6))!=NINT(QX(IIPP+6))) iquit:(MF1012)# Grid mismatch     MF10 148
NP(K)=NINT(QX(IIP+9))#                 Calculated points                MF10 149
IF(NINT(QX(IIP+9))!=NINT(QX(IIPP+9))) iquit:(MF1013)# Grid mismatch     MF10 150
ISUM(K)=NINT(QX(IIP+12))#              Correlations between crystal     MF10 151
IF(NINT(QX(IIP+12))!=NINT(QX(IIPP+12))) iquit:(MF1014)# Sum ord misma   MF10 152
JSUM(K)=NINT(QX(IIP+15))#              Axes and calculation axes        MF10 153
IF(NINT(QX(IIP+15))!=NINT(QX(IIPP+15))) iquit:(MF1015)# Sum ord misma   MF10 154
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 MF10 155
IF(QX(IP+25)<0.01) iquit:(MF1016)#     Max density to too small         MF10 156
RMXDEN=1.0/AMAX1(QX(IP+25),1.0)#       Find reciprocal of max density   MF10 157
#                                                                       MF10 158
movechar:(MF10D,JSUM(3),MF10G,24,1,0)# Change to correct axis           MF10 159
movechar:(MF10D,JSUM(2),MF10G,32,1,0)# Change to correct axis           MF10 160
movechar:(MF10D,JSUM(1),MF10G,38,1,0)# Change to correct axis           MF10 161
writeline:(1,MF10G,NMF10G,3,3)#                                         MF10 162
#                                                                       MF10 163
# MAKE SURE GRID SIZES ARE POSSIBLE TO DO                               MF10 164
# ---------------------------------------                               MF10 165
#                                                                       MF10 166
NX1=NG(1);NX2=NG(2);NX3=NG(3)#         Get points in 3 directions       MF10 167
NDIV1=0;NDIV2=0;NDIV3=0#               Set grid size pointers           MF10 168
NPIX=NX1*NX2*NX3#                      Get total pixels in map          MF10 169
DO J=1,MXGRID#                         Loop over possible grids         MF10 170
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 MF10 171
IF(NX1==ND(J)) NDIV1=J#                Set layer divisions              MF10 172
IF(NX2==ND(J)) NDIV2=J#                Set down layer divisions         MF10 173
IF(NX3==ND(J)) NDIV3=J#                Set across layer divisions       MF10 174
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 MF10 175
IF((NDIV1==0)\(NDIV2==0)\(NDIV3==0))#  Test that all o.k.               MF10 176
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 MF10 177
BUFOT(1)=FLOAT(NX1);BUFOT(2)=FLOAT(NX2);BUFOT(3)=FLOAT(NX3)#            MF10 178
writeline:(1,MF7A,NMF7A,3,1)#                                           MF10 179
ncodefld:(BUFOT,1,MF7B,FMNX,3)#                                         MF10 180
writeline:(0,MF7B,NMF7B,3,1)#                                           MF10 181
K=0; FOR(J=1;J<=4;J=J+1)#              Dump all possible grids          MF10 182
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 MF10 183
IF(J<4) N=15#                          15 items per line for three li   MF10 184
ELSE N=14#                             14 on the last                   MF10 185
FMT=413.#                              Start building the format        MF10 186
DO L=1,N#                                                               MF10 187
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 MF10 188
K=K+1 ; FMT=FMT+50000. ; BUFOT(1)=FLOAT(ND(K))#                         MF10 189
ncodefld:(BUFOT,1,CHROT,FMT,1)#                                         MF10 190
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 MF10 191
writeline:(0,0,0,1,1)#                                                  MF10 192
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 MF10 193
iquit:(MF1006)#                        Impossible grid for fft          MF10 194
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 MF10 195
#                                                                       MF10 196
# ACQUIRE STORAGE                                                       MF10 197
# --------------                                                        MF10 198
#                                                                       MF10 199
STOPP=MARKP+NPIX ; FILUSE=no:#         Get storage for electron densi   MF10 200
IF(QXMAX<STOPP\FILSIG==yes:)#          Or does user fancy scratch fil   MF10 201
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 MF10 202
FILUSE=yes:#                           Have to use a scratch file       MF10 203
IOMARK(8)=MARKP ; MARKP=MARKP+bdfbuf:# Get a buffer for file h          MF10 204
STOPP=MARKP#                           Lower sights on memory           MF10 205
writeline:(0,MF7S,NMF7S,3,3)#                                           MF10 206
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 MF10 207
incrqx:(I,STOPP,MF1009)#               Get memory                       MF10 208
RMXDEN=0.0#                            Prepare to find reciprocal       MF10 209
REPEAT#                                Scan map for maximum density     MF10 210
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 MF10 211
readwpkt:(3,22,PSIZ,IP,0)#             Read next line on file c         MF10 212
IF(IP<=0) BREAK#                       Quit if all layers done          MF10 213
I=IP#                                  Set pointer for el-density       MF10 214
DO J=1,PSIZ#                           Loop across line                 MF10 215
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 MF10 216
I=I+1 ; RMXDEN=AMAX1(QX(I),RMXDEN)#                                     MF10 217
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 MF10 218
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 MF10 219
BUFOT(1)=QX(JP+25) ; BUFOT(2)=RMXDEN#  Report electron densities        MF10 220
ncodefld:(BUFOT,1,MF10Q,MFFM,2) ; writeline:(1,MF10Q,NMF10Q,3,3)#       MF10 221
QX(JP+25)=RMXDEN#                      Save maximum density in file e   MF10 222
rewindbdf:(3)#                                                          MF10 223
RMXDEN=1.0/AMAX1(RMXDEN,1.0)#          Get reciprocal of maximum dens   MF10 224
#                                                                       MF10 225
RETURN#                                                                 MF10 226
END#                                                                    MF10 227
systemheader:(MF70)#                   Maximum entropy density calc     MF70   1
# ################################################################## #  MF70   2
#                                                                    #  MF70   3
#                               MF70                                 #  MF70   4
#                                                                    #  MF70   5
# ################################################################## #  MF70   6
SUBROUTINE MF70## D.M.Collins J.M.stewart Feb 88                        MF70   7
include:(AACOMN)##                                                      MF70   8
include:(MFCOMN)#                                                       MF70   9
#                                                                       MF70  10
# THIS ROUTINE CALCULATES MAXIMUM ENTROPY MODIFIED ELECTRON DENSITY     MF70  11
# IN WHICH THE DENSITY HAS BEEN FORCED TO CONFORM TO THE MAGNITUDE      MF70  12
# OF F OBSERVED.                                                        MF70  13
#--------------------------------------+------------------------------- MF70  14
#                                                                       MF70  15
INTEGER I,J,L## Local indices for loops etc.                            MF70  16
INTEGER PRINTP## Electron density dumping control                       MF70  17
INTEGER PSIZ,PSIZD,IP,IPD,JP## I/o pointers                             MF70  18
INTEGER X1,X2## Layer, down the layer, across the layer                 MF70  19
REAL DAMPER#                           Exponential protection           MF70  20
REAL DENLIM(3)## El-dens statistics from file c                         MF70  21
REAL DIFLIM(3)## El-dens statistics from file d                         MF70  22
INTEGER NDAMP## Count of damping failures                               MF70  23
REAL OLDELD,DIFELD## Electron densities during processing               MF70  24
REAL RESCAL## Renormalization factor                                    MF70  25
REAL SAVEDM(25)## Examples of too big exponents                         MF70  26
REAL X1DMP(25),X2DMP(25),X3DMP(25)## With their coordinates             MF70  27
INTEGER USEDAM## Signal to show damper was used                         MF70  28
chardata:(MF7D, SUM OF ELECTRON DENS =             $#                   MF70  29
SUM OF SQUARED =             )#                                         MF70  30
chardata:(MF7E, CYCLES   CONSTRAINT HISTORY IN TERMS OF $#              MF70  31
FRACTIONAL MISMATCH)#                                                   MF70  32
chardata:(MF7F, VARIANCE OF ELECTRON DENSITY =            $#            MF70  33
NO. OF PIXELS =             )#                                          MF70  34
chardata:(MF7G, PRINT A SAMPLE OF INPUT AND OUTPUT ELECTRON DENSITY)#   MF70  35
chardata:(MF7H,  X1  X2  X3   INPUT RHO  OUTPUT RHO )#                  MF70  36
chardata:(MF7I, THE INPUT MAP IS ILL CONDITIONED SINCE )#               MF70  37
chardata:(MF7J, THE VARIANCE OF THE ELECTRON DENSITY IS $#              MF70  38
OUTSIDE THE RANGE 0.01 THRU 0.99.)#                                     MF70  39
chardata:(MF7K,A VALUES)#                                               MF70  40
chardata:(MF7L,B VALUES)#                                               MF70  41
chardata:(MF7M,RESCALES)#                                               MF70  42
chardata:(MF7N, PRELIMINARY A AND B VALUES =           $#               MF70  43
AND          )#                                                         MF70  44
chardata:(MF7Q,   MEDENS DENSITY MAXIMUM =           $#                 MF70  45
MINIMUM =           AVERAGE =           )#                              MF70  46
chardata:(MF7R, DIFFERENCE DENS. MAXIMUM =           $#                 MF70  47
MINIMUM =           AVERAGE =           )#                              MF70  48
chardata:(MF7S, RESCALE FACTOR TO HOLD MEAN MEDENS DENSITY $#           MF70  49
CONSTANT =         )#                                                   MF70  50
realdata:(FMT7,[341031.,631031.])#                                      MF70  51
realdata:(FMT8,[421031.,681013.])#                                      MF70  52
realdata:(FMTL,[390852.,520852.])#                                      MF70  53
realdata:(FMTO,[40313.,80313.,120313.,241041.,361041.])#                MF70  54
realdata:(FMTST,[370931.,570931.,770931.])#                             MF70  55
datastuff:#                                                             MF70  56
#                                                                       MF70  57
# INITIALIZE COUNTS, NUMBERS OF POINTS IN SUMMATIONS AND PRINT OUT      MF70  58
# ----------------------------------------------------------------      MF70  59
#                                                                       MF70  60
DENLIM(1)=-99999. ; DENLIM(2)=99999. ; DENLIM(3)=0.0# El-den stats      MF70  61
RESCAL=0.0#                            Sums for rescale factor          MF70  62
DIFLIM(1)=-99999. ; DIFLIM(2)=99999. ; DIFLIM(3)=0.0# El-den stats      MF70  63
PRINTP=PRINTL ; IF(PRINTL>0) writeline:(1,MF7G,NMF7G,3,3)#              MF70  64
LINRM=6 ; USEDAM=no: ; NDAMP=0#        Signal large exponents           MF70  65
#                                                                       MF70  66
# TOP OF LOOPS OVER LINES AND LAYERS TO PROCESS ELECTRON DENSITY        MF70  67
# --------------------------------------------------------------        MF70  68
#                                                                       MF70  69
STOPP=MARKP#                           Set storage of el. dens.         MF70  70
X1=-1#                                 Set to loop over layers          MF70  71
REPEAT#                                Loop over layers                 MF70  72
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  MF70  73
X1=X1+1#                               Increment layer count            MF70  74
IF(X1>=NX1) BREAK#                     Get out if all done              MF70  75
X2=-1#                                 Set to loop over pages           MF70  76
REPEAT#                                Loop over pages                  MF70  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  MF70  78
X2=X2+1#                               Increment page count             MF70  79
IF(X2>=NX2) BREAK#                     Test if whole layer is in        MF70  80
readwpkt:(3,22,PSIZ,IP,0)#             Read next line on file c         MF70  81
readwpkt:(4,22,PSIZD,IPD,0)#           Read next line on file d         MF70  82
IF(IP<=0\IPD<=0) BREAK#                Quit if all layers done          MF70  83
IF(FILUSE==yes:)#                      Check use of scratch file        MF70  84
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  MF70  85
writepkt:(8,22,PSIZ,JP) ; STOPP=JP#    Write a packet for holding e.d   MF70  86
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  MF70  87
I=IP ; L=IPD#                          Set pointer for el-density       MF70  88
DO J=1,NX3#                            Loop across line                 MF70  89
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  MF70  90
I=I+1 ; L=L+1 ; OLDELD=QX(I) ; DIFELD=QX(L)#                            MF70  91
DENLIM(3)=DENLIM(3)+OLDELD#            Build statistics                 MF70  92
DENLIM(1)=AMAX1(OLDELD,DENLIM(1)) ; DENLIM(2)=AMIN1(OLDELD,DENLIM(2))#  MF70  93
DIFLIM(3)=DIFLIM(3)+DIFELD#            Build statistics                 MF70  94
DIFLIM(1)=AMAX1(DIFELD,DIFLIM(1)) ; DIFLIM(2)=AMIN1(DIFELD,DIFLIM(2))#  MF70  95
STOPP=STOPP+1#                         Increment the storage pointer    MF70  96
#                                                                       MF70  97
# NOW FORM THE MODIFIED DENSITY ACCORDING TO EQUATIONS (2.5) AND (2.6)  MF70  98
# OF THE ACTA CRYST PAPER.  PROTECT AGAINST NUMBERS OUT OF RANGE.       MF70  99
# --------------------------------------------------------------------  MF70 100
#                                                                       MF70 101
WEIGHT=RMXDEN*(AFACT*(-1.0+RMXDEN*OLDELD)**2+1.0)# Form eqn 2.6 p253    MF70 102
DAMPER=WEIGHT*DIFELD#                  Calculate the damping/enhancin   MF70 103
IF(ABS(DAMPER)>DAMPLM)#                Protect against large exponent   MF70 104
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  MF70 105
QX(STOPP)=OLDELD*EXP(SIGN(DAMPLM,DAMPER))# Keep eldens sane             MF70 106
NDAMP=NDAMP+1#                         Count crazy points               MF70 107
USEDMP=yes: ; IF(NDAMP<=25)#           Save some points                 MF70 108
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  MF70 109
SAVEDM(NDAMP)=DAMPER#                                                   MF70 110
X1DMP(NDAMP)=X1 ; X2DMP(NDAMP)=X2 ; X3DMP(NDAMP)=FLOAT(J)#              MF70 111
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  MF70 112
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  MF70 113
ELSE#                                  Scale density in usual way       MF70 114
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  MF70 115
QX(STOPP)=OLDELD*EXP(DAMPER)#          Find the modified electron den   MF70 116
RESCAL=RESCAL+QX(STOPP)#               Sum for rescale                  MF70 117
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  MF70 118
#                                                                       MF70 119
# FIND SUMMATIONS TO GIVE CONSTRAINTS ONE AND TWO                       MF70 120
# -----------------------------------------------                       MF70 121
#                                                                       MF70 122
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  MF70 123
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  MF70 124
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  MF70 125
IF(FILUSE==yes:) writepkt:(8,endrecord:,PSIZ,JP)# End file if need be   MF70 126
#                                                                       MF70 127
# AT THE END OF READING THE WHOLE MAP GIVE EL-DENS STATS                MF70 128
# ------------------------------------------------------                MF70 129
#                                                                       MF70 130
RESCAL=DENLIM(3)/AMAX1(RESCAL,1.0)#    Get rescale factor               MF70 131
ncodefld:(RESCAL,1,MF7S,620732.,1) ; writeline:(1,MF7S,NMF7S,3,3)#      MF70 132
DENLIM(3)=DENLIM(3)/FLOAT(NPIX)#       Find ave medens el-dens          MF70 133
ncodefld:(DENLIM,1,MF7Q,FMTST,3) ; writeline:(1,MF7Q,NMF7Q,3,3)#        MF70 134
DIFLIM(3)=DIFLIM(3)/FLOAT(NPIX)#       Find ave difference el-dens      MF70 135
ncodefld:(DIFLIM,1,MF7R,FMTST,3) ; writeline:(1,MF7R,NMF7R,3,3)#        MF70 136
#                                                                       MF70 137
writeline:(0,0,0,1,3) ; LINRM=10#      Assure heading on sample o/p     MF70 138
#                                                                       MF70 139
# TOP OF LOOPS OVER LINES AND LAYERS TO OUTPUT NEW ELECTRON DENSITY     MF70 140
# -----------------------------------------------------------------     MF70 141
#                                                                       MF70 142
L=MARKP#                               Set fetching of el. dens.        MF70 143
X1=-1#                                 Set to loop over layers          MF70 144
REPEAT#                                Loop over layers                 MF70 145
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  MF70 146
X1=X1+1#                               Increment layer count            MF70 147
IF(X1>=NX1) BREAK#                     Get out if all done              MF70 148
X2=-1 ; PRINTP=PRINTL#                 Set to loop over pages           MF70 149
REPEAT#                                Loop over pages                  MF70 150
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  MF70 151
X2=X2+1#                               Increment page count             MF70 152
IF(X2>=NX2) BREAK#                     Test if whole layer is in        MF70 153
IF(FILUSE==yes:)#                      Check for scratch file activit   MF70 154
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 MF70 155
readwpkt:(8,22,PSIZD,JP,0); L=JP#      Unscaled, but modified e.d.      MF70 156
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 MF70 157
writepkt:(5,22,PSIZ,JP,0)#             Write next line of scaled e.d.   MF70 158
I=JP#                                  Set pointer for buffer           MF70 159
IF(PRINTP>0)#                          Check if dumping to be done      MF70 160
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 MF70 161
DO J=1,NX3#                            Loop across line                 MF70 162
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 MF70 163
I=I+1 ; L=L+1#                                                          MF70 164
QX(I)=RESCAL*QX(L)#                    Scale new density                MF70 165
IF(PRINTP>0&X2>=(J-1))#                See if dumping a sample          MF70 166
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 MF70 167
BUFOT(1)=FLOAT(X1);BUFOT(2)=FLOAT(X2);BUFOT(3)=FLOAT(J-1)#              MF70 168
BUFOT(4)=QX(L) ; BUFOT(5)=QX(I)#       Get electron densities           MF70 169
ncodefld:(BUFOT,1,CHROT,FMTO,5) ; writeline:(0,MF7H,NMF7H,1,3)#         MF70 170
PRINTP=PRINTP-1#                       Decrement ouput lines            MF70 171
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 MF70 172
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 MF70 173
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 MF70 174
ELSE#                                  No dumping give optimizer head   MF70 175
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 MF70 176
DO J=1,NX3#                            Loop across line                 MF70 177
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 MF70 178
I=I+1 ; L=L+1#                                                          MF70 179
QX(I)=RESCAL*QX(L)#                    Rescale to give new density      MF70 180
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 MF70 181
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 MF70 182
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  MF70 183
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  MF70 184
#                                                                       MF70 185
# FINISH OFF ALL BDFS REPORT EL DENS SAMPLE AND QUIT                    MF70 186
# --------------------------------------------------                    MF70 187
#                                                                       MF70 188
readwpkt:(3,endrecord:,PSIZ,IP,0)#     End off old el-dens file c       MF70 189
readwpkt:(4,endrecord:,PSIZ,IP,0)#     End off old el-dens file d       MF70 190
writepkt:(5,endrecord:,PSIZ,IP,0)#     End off new el-dens file e       MF70 191
IF(FILUSE==yes:) readwpkt:(8,endrecord:,PSIZ,IP,0)# End scratch         MF70 192
#                                                                       MF70 193
RETURN#                                                                 MF70 194
END#                                                                    MF70 195
pmacro:([no:])#                        Purge xtal system macros         MF70 196
