systemheader:(SC    )#                                                  SC     1
####################################################################### SC     2
#                                                                     # SC     3
#                           *** SCALE1 ***                            # SC     4
#                                                                     # SC     5
#    THE PROGRAM SCALE1 SCALES SETS OF INTENSITY DATA BY THE METHOD   # SC     6
#     OF MONAHAN, SCHIFFER AND SCHIFFER.                              # SC     7
####################################################################### SC     8
#                                                                       SC     9
include:(XMACRO)#                                                       SC    10
#                                                                       SC    11
macro:(maxshifts:,64)#                 Maximum number of shifts tha     SC    12
#                                         Can be used.                  SC    13
extrude:(SCCOMN)#                                                       SC    14
COMMON/COMISC/ FSQPT,FSQPTR,   GPT,  GPTR, GRPPT,$# Scc                 SC    15
GRPPTR,   HKL, HKLBY,HKLNOW,HKLNXT,ILIST,$#                             SC    16
IMERGE,   KPT,  KPTR,  RMAX,MAXCYC,MAXOBS,  RMIN,$#                     SC    17
MXOBUZ,   NFS,NSHIFT,   OBS, OBSBY,OBSKIP, OBSPT,$#                     SC    18
OBSPTR, OBSSG,OFFSET, REFBY, REFSG,RFBOTH,SHFTIN,$#                     SC    19
TBYOBS,TOBSKP,TOTOBS,TOTREF,TSGOBS#                                     SC    20
#                                                                       SC    21
INTEGER FSQPT#                         Pointer to current f**2          SC    22
INTEGER FSQPTR#                        Pointer to start of input        SC    23
#                                         F**2 storage.                 SC    24
INTEGER GPT#                           Pointer to a current final       SC    25
#                                         Scale factor.                 SC    26
INTEGER GPTR#                          Pointer to start of table of     SC    27
#                                         Refined scale factors.        SC    28
INTEGER GRPPT#                         Pointer to current shift         SC    29
#                                         Number.                       SC    30
INTEGER GRPPTR#                        Pointer to table of shift        SC    31
#                                         Numbers being used in         SC    32
#                                         Refinement.                   SC    33
INTEGER HKL#                           Packed hkl, space group          SC    34
INTEGER HKLBY#                         Packed hkl, bijvoet              SC    35
INTEGER HKLNOW#                        Packed hkl found on lrrefl:      SC    36
INTEGER HKLNXT#                        Packed hkl for next reflecti     SC    37
#                                         On the bdf.                   SC    38
INTEGER ILIST#                         Output from every ilist cycl     SC    39
#                                         Is to be listed.              SC    40
INTEGER IMERGE#                        If zero, merge space group a     SC    41
#                                         Bijvoet observations.         SC    42
INTEGER KPT#                           Pointer to initial scale         SC    43
#                                         Factor of current shift       SC    44
#                                         Number.                       SC    45
INTEGER KPTR#                          Pointer to table of initial      SC    46
#                                         Scale factors to be refined.  SC    47
INTEGER RMAX#                          Do not use reflections with      SC    48
#                                         More than max obs.            SC    49
INTEGER MAXCYC#                        Number of cycles to be done      SC    50
INTEGER MAXOBS#                        Use up to maxobs obs./refl.,     SC    51
#                                         Discard the rest.             SC    52
INTEGER RMIN#                          Do not use refls. with fewer     SC    53
#                                         Than min obs.                 SC    54
INTEGER MXOBUZ#                        Max. number of useable obs.      SC    55
#                                         Found for a refl.             SC    56
INTEGER NFS#                           Total number of useable obs.     SC    57
INTEGER NSHIFT#                        Number of scale groups           SC    58
INTEGER OBS#                           Total obs. for current refl.     SC    59
INTEGER OBSBY#                         Total bijvoet obs. for curre     SC    60
#                                         Refl.                         SC    61
INTEGER OBSKIP#                        Obs. skipped for current ref     SC    62
INTEGER OBSPT#                         Pointer to number of obs.        SC    63
#                                         In current shift number.      SC    64
INTEGER OBSPTR#                        Pointer to table of counts       SC    65
#                                         Per shift.                    SC    66
INTEGER OBSSG#                         Total space group obs. for       SC    67
#                                         Current refl.                 SC    68
INTEGER OFFSET#                        Pointer to scales in lrexpl:     SC    69
INTEGER REFBY#                         Number of refls. with bijvoe     SC    70
#                                         Only obs.                     SC    71
INTEGER REFSG#                         Number of refls. with space      SC    72
#                                         Group only obs.               SC    73
INTEGER RFBOTH#                        Number of refls. with both       SC    74
#                                         Space group and bijvoet obs.  SC    75
INTEGER SHFTIN#                        Number of scale groups on        SC    76
#                                         The bdf.                      SC    77
INTEGER TBYOBS#                        Total number of bijvoet obs.     SC    78
#                                         On the bdf.                   SC    79
INTEGER TOBSKP#                        Total number of obs. skipped     SC    80
#                                         Because of insufficient room. SC    81
INTEGER TOTOBS#                        Total number of obs. on the      SC    82
#                                         Bdf.                          SC    83
INTEGER TOTREF#                        Total number of refls.           SC    84
#                                         Processed.                    SC    85
INTEGER TSGOBS#                        Total number of space group      SC    86
#                                         Obs. on the bdf.              SC    87
#                                                                       SC    88
COMMON/COMFSC/FSQMAX,FSQMIN, SITOL,  SMAX,  SMIN# Scc                   SC    89
#                                                                       SC    90
REAL FSQMAX#                           Do not use obs. with f**2        SC    91
#                                         Greater than fsqmax.          SC    92
REAL FSQMIN#                           Do not use obs. with f**2 le     SC    93
#                                         Than fsqmin.                  SC    94
REAL SITOL#                            Sine(theta)/lambda               SC    95
REAL SMAX#                             Do not use observations          SC    96
REAL SMIN#                             Outside the range smin to        SC    97
#                                         Smax exclusive.               SC    98
endext:#                                                                SC    99
#                                                                       SC   100
systemheader:(SC00)#                                                    SC00   1
# ################################################################## #  SC00   2
#                                                                    #  SC00   3
#                               SC00                                 #  SC00   4
#                                                                    #  SC00   5
# ################################################################## #  SC00   6
SUBROUTINE SC00#                       Scale all observations           SC00   7
#                                        May 1979  r.a. alden           SC00   8
#                                        Revised  aug 1979   r. alden   SC00   9
#                                        Updated  jan 1980  r. alden    SC00  10
#                                         Dec 1982  r. alden            SC00  11
#                                        Revised june 1985,             SC00  12
#                                         K.d.watenpaugh                SC00  13
include:(AACOMN)#                                                       SC00  14
include:(SCCOMN)#                                                       SC00  15
#                                                                       SC00  16
CALL SC10#                             Process input specs and          SC00  17
#                                         Scale constants.              SC00  18
CALL SC20#                             Filter logical record lrrefl     SC00  19
#                                         According to instructions.    SC00  20
CALL SC30#                             Perform the refinement           SC00  21
#                                         Cycles.                       SC00  22
RETURN#                                                                 SC00  23
END#                                                                    SC00  24
systemheader:(SC10)#                                                    SC10   1
# ################################################################## #  SC10   2
#                                                                    #  SC10   3
#                               SC10                                 #  SC10   4
#                                                                    #  SC10   5
# ################################################################## #  SC10   6
SUBROUTINE SC10#                       Read in the scale1 image         SC10   7
#                                        May 1979  r.a. alden           SC10   8
#                                        Revised  aug 1979  r alden     SC10   9
include:(AACOMN)#                                                       SC10  10
include:(SCCOMN)#                                                       SC10  11
#                                                                       SC10  12
INTEGER J#                             Scratch                          SC10  13
chardata:(DONT,WILLWONT)#                                               SC10  14
chardata:(SC011,THERE WILL BE     REFINEMENT CYCLES)#                   SC10  15
chardata:(SC012,A REFLECTION WILL BE USED ONLY IF IT HAS AT $#          SC10  16
LEAST    BUT NOT MORE THAN      OBSERVATIONS)#                          SC10  17
chardata:(SC013,THE MAXIMUM NUMBER OF OBSERVATIONS PER REFLECTION $#    SC10  18
THAT WILL BE USED IS)#                                                  SC10  19
chardata:(SC014,OBSERVATIONS WILL NOT BE USED IF F**2 <=         $#     SC10  20
OR F**2 >=)#                                                            SC10  21
chardata:(SC015,1 OUT OF EVERY    CYCLES WILL BE LISTED)#               SC10  22
chardata:(SC016,BIJVOET AND SPACE GROUP REFLECTIONS      BE MERGED)#    SC10  23
chardata:(SC017,THE MAXIMUM AND MINIMUM SINE(THETA)/LAMBDA LIMITS $#    SC10  24
FOR INCLUSION IN THE REFINEMENT ARE        AND)#                        SC10  25
chardata:(SC018, SCALE  END    )#                                       SC10  26
#                                                                       SC10  27
realdata:(FMT011,[1040642.,930642.])#                                   SC10  28
realdata:(FMT012,[740812.,510812.])#                                    SC10  29
realdata:(FMT013,[530213.,760413.])#                                    SC10  30
realdata:(FMT014,[180313.,760513.,180213.])#                            SC10  31
datastuff:#                                                             SC10  32
#                                                                       SC10  33
IOMARK(1) = QXSTR#                     Point to input buffer            SC10  34
IOMARK(2) = IOMARK(1) + bdfbuf:#       Also to the output buffer        SC10  35
GRPPTR = IOMARK(2) + bdfbuf:#          Start of list of shift numbe     SC10  36
incrqx:(J,GRPPTR+arith:(2,*,maxshifts:),SC1001)# S                      SC10  37
#                                                                       SC10  38
# INITIALLY THE GRPPTR ARRAY ALLOWS FOR MAXSHIFTS: SHIFT NUMBERS AND    SC10  39
#  SCALE FACTORS.  HOWEVER, ONCE THE ACTUAL NUMBER OF SHIFT NUMBERS     SC10  40
#  HAS BEEN DETERMINED, THE ARRAY WILL BE SQUEEZED DOWN TO THE          SC10  41
#  NUMBER OF LOCATIONS ACTUALLY REQUIRED AND THEN 8 TIMES THAT          SC10  42
#  ARE REQUESTED.  THE FINAL SIZE OF THE GRPPTR ARRAY WILL BE           SC10  43
#  8*NSHIFT AND WILL BE COMPOSED OF THE FOLLOWING SUB-ARRAYS:           SC10  44
#  1) LIST OF SHIFT NUMBERS                                             SC10  45
#  2) LIST OF CORRESPONDING INITIAL SCALE FACTORS                       SC10  46
#  3) COMPUTED SCALES                                                   SC10  47
#  4) NUMBER OF OBSERVATIONS CONTRIBUTING TO EACH SHIFT                 SC10  48
#  5) COMPUTED INVERSE SCALE NUMERATOR                                  SC10  49
#  6) COMPUTED INVERSE SCALE DENOMINATOR                                SC10  50
#  7) WEIGHTED SUM OF THE OBSERVATIONS                                  SC10  51
#  8) WEIGHTED SUM OF SQUARES                                           SC10  52
#  THE LATTER 6 ARRAYS WILL BE FILLED IN LATER.                         SC10  53
#                                                                       SC10  54
# READ THE SCALE1 IMAGE.  THE FIELD DEFINITIONS ARE AS FOLLOWS:         SC10  55
#                                                                       SC10  56
#  FIELD                        DEFINITION                      DEFAULT SC10  57
#  -----                        ----------                      ------- SC10  58
#    1          MAXCYC; NUMBER OF CYCLES TO BE DONE               10    SC10  59
#    2          ILIST;  LIST ONE OUT OF EVERY ILIST CYCLES         5    SC10  60
#    3          IMERGE; 0/1 MERGE/DO NOT MERGE BIJVOET PAIRS       0    SC10  61
#    4          RMIN;    ONLY USE REFLECTIONS WITH MORE THAN MIN    2   SC10  62
#    5          RMAX;    AND FEWER THAN MAX OBSERVATIONS.         250   SC10  63
#                       MAX MUST BE .LE. 1500                           SC10  64
#    6          MAXOBS; THE MAX NO. OF OBS. FOR ANY REFLECTION    25    SC10  65
#                       ON THE INPUT BDF.                               SC10  66
#    7          FSQMIN; OBSERVATIONS WITH F**2 .LT. FSQMIN       .01    SC10  67
#    8          FSQMAX; OR .GT. FSQMAX WILL NOT BE USED.      100000.   SC10  68
#    9          SMIN;   OBSERVATIONS WITH SINTH .LT. SMIN         0.    SC10  69
#   10          SMAX;   OR .GT. SMAX WILL NOT BE USED.            1.    SC10  70
#                                                                       SC10  71
IF(BUFIN(1) <= 0.)  BUFIN(1) = 10.#    Number of refinement cycles      SC10  72
MAXCYC = NINT(BUFIN(1))#               (default is 10 cycles)           SC10  73
movechar:(SC011,1,CHROT,2,NSC011,0)#                                    SC10  74
ncodefld:(BUFIN,1,CHROT,FMT014(1),1)##                                  SC10  75
writeline:(1,0,0,1,3)#                 Tell user                        SC10  76
#                                                                       SC10  77
IF(BUFIN(4) <= 0.)  RMIN = 2#          Minimum no. of observations      SC10  78
ELSE                RMIN = NINT(BUFIN(4))#                              SC10  79
IF(BUFIN(5) <= 0.)      RMAX = 250#    Required and maximum             SC10  80
ELSE IF(BUFIN(5)>1500.) RMAX= 1500#    Permitted for inclusion.         SC10  81
ELSE                    RMAX = NINT(BUFIN(5))#                          SC10  82
movechar:(SC012,1,CHROT,2,NSC012,0)#                                    SC10  83
ncodefld:(FLOAT(RMIN),1,CHROT,FMT013(1),1)##                            SC10  84
ncodefld:(FLOAT(RMAX),1,CHROT,FMT013(2),1)##                            SC10  85
writeline:(1,0,0,1,3)#                 Tell user                        SC10  86
#                                                                       SC10  87
IF(BUFIN(6) <= 0.)  BUFIN(6) = 250.#   This sets up the amount of       SC10  88
MAXOBS = NINT(BUFIN(6))#               Buffer space required for        SC10  89
movechar:(SC013,1,CHROT,2,NSC013,0)#   The fsq's and sigmas for         SC10  90
ncodefld:(BUFIN,6,CHROT,FMT014(2),1)#  A reflection.                    SC10  91
writeline:(1,0,0,1,3)#                 Tell user                        SC10  92
#                                                                       SC10  93
SMIN = BUFIN(9)#                       Inclusive resolution limits      SC10  94
SMAX = BUFIN(10)#                                                       SC10  95
IF(SMIN < 0.)  SMIN = 0.#              (defaults are 0. and 1.)         SC10  96
IF(SMAX <= 0.) SMAX = 1.#                                               SC10  97
movechar:(SC017,1,CHROT,2,NSC017,0)#                                    SC10  98
ncodefld:(SMAX,1,CHROT,FMT011(1),2)#                                    SC10  99
writeline:(1,0,0,1,3)#                 Tell user                        SC10 100
#                                                                       SC10 101
FSQMIN = BUFIN(7)#                     Maximum and minimum limits       SC10 102
FSQMAX = BUFIN(8)#                     On f**2.                         SC10 103
IF(FSQMIN < .01)  FSQMIN = .01#        (defaults are .01 and            SC10 104
IF(FSQMAX <= 0.)  FSQMAX = 100000.#    100000.).                        SC10 105
movechar:(SC014,1,CHROT,2,NSC014,0)#                                    SC10 106
ncodefld:(FSQMAX,1,CHROT,FMT012(1),2)##                                 SC10 107
writeline:(1,0,0,1,3)#                 Tell user                        SC10 108
#                                                                       SC10 109
IF(BUFIN(2) <= 0.)  BUFIN(2) = 5.#     Listing instruction              SC10 110
ILIST = NINT(BUFIN(2))#                (default is 5).                  SC10 111
movechar:(SC015,1,CHROT,2,NSC015,0)#                                    SC10 112
ncodefld:(BUFIN,2,CHROT,FMT014(3),1)#                                   SC10 113
writeline:(1,0,0,1,3)#                 Tell user                        SC10 114
#                                                                       SC10 115
ifvo:(BUFIN(3))    BUFIN(3) = 0.#                                       SC10 116
ifne:(BUFIN(3),0.) BUFIN(3) = 1.#      To merge or not to merge         SC10 117
IMERGE = NINT(BUFIN(3))#               (default is to merge).           SC10 118
movechar:(SC016,1,CHROT,2,NSC016,0)#                                    SC10 119
movechar:(DONT,1,CHROT,38,4,0)#                                         SC10 120
IF(IMERGE > 0)  movechar:(DONT,5,CHROT,38,4,0)#                         SC10 121
writeline:(1,0,0,1,3)#                 Tell user                        SC10 122
#                                                                       SC10 123
readline:(SC018,NSC018)#               Read the next image              SC10 124
#                                                                       SC10 125
# SCALE IMAGES AT THIS POINT ARE OPTIONAL.  IF THEY APPEAR, THE SHIFTS  SC10 126
#  FOUND ON THESE IMAGES AND THESE SHIFTS ONLY WILL HAVE THEIR SCALE    SC10 127
#  FACTORS REFINED.  IF THEY ARE NOT PRESENT, THEN THOSE SHIFTS         SC10 128
#  RECORDED IN LOGICAL RECORD LREXPL: WILL HAVE THEIR SCALE FACTORS     SC10 129
#  REFINED USING AS THEIR INITIAL VALUES 1.0                            SC10 130
#                                                                       SC10 131
# PREPARE A LIST OF SHIFT NUMBERS TO BE REFINED AND A CORRESPONDING     SC10 132
#  LIST OF THEIR INITIAL SCALE FACTOR VALUES.                           SC10 133
#                                                                       SC10 134
CALL SC11#                                                              SC10 135
#                                                                       SC10 136
# NOW RE-DIMENSION THE GRPPTR ARRAY TO A SIZE OF 8*NSHIFT               SC10 137
#                                                                       SC10 138
# THE NEXT ARRAY POINTS TO THE BUFFER REQUIRED BY SC21 TO HOLD          SC10 139
#  FSQ, SIGMA, AND THE SHIFT NUMBER FOR EACH OBSERVATION OF A           SC10 140
#  REFLECTION.  IF THEY ARE NOT BEING MERGED, THE SIZE OF THIS          SC10 141
#  ARRAY IS 3*MAXOBS;  IF THEY ARE BEING MERGED, THE SIZE IS 6*MAXOBS.  SC10 142
#                                                                       SC10 143
incrqx:(FSQPTR,GRPPTR+8*NSHIFT,SC1001)# Request the space               SC10 144
#                                                                       SC10 145
RETURN#                                                                 SC10 146
END#                                                                    SC10 147
systemheader:(SC11)#                                                    SC11   1
# ################################################################## #  SC11   2
#                                                                    #  SC11   3
#                               SC11                                 #  SC11   4
#                                                                    #  SC11   5
# ################################################################## #  SC11   6
SUBROUTINE SC11#                       Process either scale images      SC11   7
#                                        R.a. alden  jun 1979           SC11   8
#                                         Updated 12 jan 1980  r. alden SC11   9
#                                        Re-written 29 dec 1982         SC11  10
#                                         R. alden                      SC11  11
include:(AACOMN)#                                                       SC11  12
include:(SCCOMN)#                                                       SC11  13
#                                                                       SC11  14
INTEGER I#                             Loop index                       SC11  15
INTEGER IPT#                           Pointer to current shift no.     SC11  16
INTEGER IPTR#                          Input packet pointer             SC11  17
INTEGER KEY(4)#                        Signals to indexpkt:             SC11  18
INTEGER NEED11(arith:(1,+,maxshifts:))## Lrexpl: want list              SC11  19
INTEGER PSIZE#                         Input packet size                SC11  20
INTEGER PT11(arith:(2,+,maxshifts:))#  Pointers to lrexpl: items        SC11  21
REAL SCALE#                            Initial scale factor             SC11  22
chardata:(SC101, SCALE  END    )#                                       SC11  23
chardata:(SC102, SHIFT NUMBER   INITIAL SCALE FACTOR)#                  SC11  24
realdata:(FMT101,[110413.,281252.])#                                    SC11  25
datastuff:#                                                             SC11  26
#                                                                       SC11  27
GRPPT = 0#                             Set no groups loaded             SC11  28
KPTR = GRPPTR + maxshifts:#            Point to scale factors           SC11  29
NEED11(1) = 100#                       Get count of total scale gr.     SC11  30
DO I=1,maxshifts:#                                                      SC11  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  SC11  32
NEED11(I+1) = I + 100#                                                  SC11  33
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  SC11  34
KEY(1) = 2#                                                             SC11  35
KEY(2) = arith:(1,+,maxshifts:)#                                        SC11  36
KEY(3) = arith:(1,+,maxshifts:)#                                        SC11  37
KEY(4) = 0#                                                             SC11  38
indexpkt:(1,lrexpl:,PSIZE,IPTR,0,KEY,NEED11,PT11)#                      SC11  39
IF(KEY(4) <= 0)  iquit:(SC1101)#       Not there; bad news              SC11  40
readwpkt:(1,lrexpl:,PSIZE,IPTR,0)#     Get the scale info               SC11  41
IPT = IPTR + PT11(1)#                                                   SC11  42
SHFTIN = NINT(QX(IPT))#                Number of scale groups prese     SC11  43
#                                                                       SC11  44
WHILE(LINID == 1)#                     Loop over the scale images       SC11  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  SC11  46
ifvo:(BUFIN(1))  iquit:(SC1102,1)#     Illegal shift number             SC11  47
IF(BUFIN(1) < 1.) iquit:(SC1102,1)#                                     SC11  48
GRPPT = NINT(BUFIN(1))#                Get group number                 SC11  49
IF(GRPPT > SHFTIN)#                    An error, if it's not there      SC11  50
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  SC11  51
iquit:(-SC1103,1)#                                                      SC11  52
NEXT#                                                                   SC11  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  SC11  54
#                                                                       SC11  55
# PICK UP THE SHIFT NUMBER AND ITS SCALE FACTOR                         SC11  56
#                                                                       SC11  57
movereal:(BUFIN,1,QX,GRPPTR+GRPPT,1,0)#                                 SC11  58
movereal:(BUFIN,1,QX,KPTR+GRPPT,1,0)#                                   SC11  59
ifvo:(BUFIN(2))#                       If no scale provided,            SC11  60
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  SC11  61
KPT = IPTR + PT11(GRPPT+1)#            Then get it off of the bdf.      SC11  62
SCALE = QX(KPT)#                                                        SC11  63
ifvo:(SCALE)  SCALE = 1.#              If it's also void, then set      SC11  64
#                                         To default (1.).              SC11  65
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  SC11  66
ELSE  SCALE = BUFIN(2)#                Otherwise, use value on line     SC11  67
#                                                                       SC11  68
IF(SCALE < 0.)  iquit:(-SC1105)#       Must be greater than zero        SC11  69
#                                                                       SC11  70
BUFOT(1) = SCALE#                                                       SC11  71
movereal:(BUFOT,1,QX,KPTR+GRPPT,1,0)## Save the initial bufot fa        SC11  72
#                                                                       SC11  73
readline:(SC101,NSC101)#               Get the next image               SC11  74
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  SC11  75
#                                                                       SC11  76
# IF THERE WERE NO SCALE IMAGES, THEN READ THE SHIFT NUMBERS FROM       SC11  77
#  LOGICAL RECORD LREXPL:.                                              SC11  78
#                                                                       SC11  79
IF(GRPPT == 0)#                                                         SC11  80
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  SC11  81
NSHIFT = SHFTIN#                       Number of shifts                 SC11  82
DO I=1,NSHIFT#                         Loop over all scale groups       SC11  83
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  SC11  84
KPT = IPTR + PT11(I+1)#                Set pointer to scale factor      SC11  85
QX(KPTR+I) = QX(KPT)#                  Move group scale                 SC11  86
ifvo:(QX(KPTR+I)) QX(KPTR+I) = 1.0#    Set void scale to default        SC11  87
QX(GRPPTR+I) = FLOAT(I)#               Save group number                SC11  88
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  SC11  89
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  SC11  90
#                                                                       SC11  91
# WRITE THE TABLE HEADING                                               SC11  92
#                                                                       SC11  93
writeline:(1,SC102,NSC102,3,3)#                                         SC11  94
DO I=1,NSHIFT#                         Loop over input shift number     SC11  95
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  SC11  96
ncodefld:(QX,GRPPTR+I,CHROT,FMT101(1),1)#                               SC11  97
ncodefld:(QX,KPTR+I,CHROT,FMT101(2),1)#                                 SC11  98
writeline:(0,SC102,NSC102,1,3)#                                         SC11  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  SC11 100
IF(QUITF(1^1)!=' ') iquit:(SC1199)#    Any errors so far?               SC11 101
IF(NSHIFT < 2)  iquit:(SC1106)#        There must be at least           SC11 102
KPT = GRPPTR + NSHIFT#                 Scale factor pointer             SC11 103
movereal:(QX,KPTR+1,QX,KPT+1,NSHIFT,0)#                                 SC11 104
KPTR = KPT#                            Save new pointer                 SC11 105
#                                                                       SC11 106
RETURN#                                                                 SC11 107
END#                                                                    SC11 108
systemheader:(SC20)#                                                    SC20   1
# ################################################################## #  SC20   2
#                                                                    #  SC20   3
#                               SC20                                 #  SC20   4
#                                                                    #  SC20   5
# ################################################################## #  SC20   6
SUBROUTINE SC20#                       Be used in the scaling           SC20   7
#                                        Aug 1979  alden & freer        SC20   8
include:(AACOMN)#                                                       SC20   9
include:(SCCOMN)#                                                       SC20  10
#                                                                       SC20  11
INTEGER FSPT#                          Pointer to current saved fsq     SC20  12
INTEGER FSPTR#                         Pointer to start of saved        SC20  13
#                                         Fsq/sigma buffer.             SC20  14
INTEGER I#                             Observation loop index           SC20  15
INTEGER IPT#                           Input packet pointer             SC20  16
INTEGER J#                             Scratch pointer                  SC20  17
INTEGER JUST1O#                        Number of obs. with 1 scale      SC20  18
#                                         Group only.                   SC20  19
INTEGER JUST1R#                        Number of refls. with 1 scal     SC20  20
#                                         Group only.                   SC20  21
INTEGER NN#                            Number of scaled obs. for        SC20  22
#                                         Current reflection.           SC20  23
INTEGER NOBUSE#                        Number of obs. used in scali     SC20  24
INTEGER NRJFO#                         Number of obs. out of fsq ra     SC20  25
INTEGER NRJOBO#                        Number of obs. and refls.        SC20  26
INTEGER NRJOBR#                        Thrown out because too few o     SC20  27
#                                         Too many observations.        SC20  28
INTEGER NRJSTO#                        Number of obs. out of sitol      SC20  29
#                                         Range.                        SC20  30
INTEGER NRJSTR#                        Number of refls. out of sito     SC20  31
#                                         Range.                        SC20  32
INTEGER NRUSE#                         Number of refls. used in         SC20  33
#                                         Scaling.                      SC20  34
INTEGER NSCOBS#                        Total number of scalable obs     SC20  35
INTEGER NSCREF#                        Total number of refls. with      SC20  36
#                                         Scalable observations.        SC20  37
INTEGER NUNREP#                        Number of reflections with n     SC20  38
#                                         Replicates.                   SC20  39
INTEGER PREVSN#                        Shift number of the previous     SC20  40
#                                         Useful observation.           SC20  41
INTEGER PSIZE#                         Input packet size                SC20  42
INTEGER ROOM#                          Number of words needed to        SC20  43
#                                         Store an observation and,     SC20  44
#                                         Possibly, its bijvoet.        SC20  45
INTEGER SHIFT#                         Shift number of the current      SC20  46
#                                         Observation.                  SC20  47
INTEGER USEFUL#                        Flag telling if any useable      SC20  48
#                                         Obs. for current refl.        SC20  49
REAL FSQ#                              F**2                             SC20  50
REAL OBSREF(2)#                        Number of obs. and refls. us     SC20  51
chardata:(SC021,     OVERALL DATA TALLY)#                               SC20  52
chardata:(SC022, OBSERVATIONS     REFLECTIONS)#                         SC20  53
chardata:(SC023,IN BDF)#                                                SC20  54
chardata:(SC024,TOTAL WITH SCALES)#                                     SC20  55
chardata:(SC025,SCALED DATA THAT WERE REJECTED)#                        SC20  56
chardata:(SC026,USED)#                                                  SC20  57
chardata:(SC027, REJECTION TALLY FOR SCALED DATA)#                      SC20  58
chardata:(SC028,UNREPLICATED)#                                          SC20  59
chardata:(SC029,SIN(THETA/LAMBDA) OUT OF RANGE)#                        SC20  60
chardata:(SC02A,F**2 (PRIOR TO SCALING) OUT OF RANGE)#                  SC20  61
chardata:(SC02B,WITH ONLY 1 SCALE GROUP LEFT)#                          SC20  62
chardata:(SC02C, THE MAXIMUM NUMBER OF OBSERVATIONS USED FOR $#         SC20  63
A SINGLE REFLECTION =    )#                                             SC20  64
realdata:(FMT021,[90613.,250613.])#                                     SC20  65
realdata:(FMT022,[700413.])#                                            SC20  66
realdata:(NUL,[0.])#                   Holds zero                       SC20  67
datastuff:#                                                             SC20  68
#                                                                       SC20  69
GPTR = KPTR + NSHIFT#                  Final scales pointer             SC20  70
OBSPTR = GPTR + NSHIFT#                Observations/scale group ptr     SC20  71
#                                                                       SC20  72
IF(IMERGE == 0)  ROOM = 6#             Allow room for bijvoets          SC20  73
ELSE             ROOM = 3#                                              SC20  74
#                                                                       SC20  75
incrqx:(FSPTR,FSQPTR+ROOM*MAXOBS,SC2001)# S                             SC20  76
FSPT = FSPTR - 2#                                                       SC20  77
#                                                                       SC20  78
movereal:(NUL,1,QX,OBSPTR+1,NSHIFT,2)# Clear obs./shift counters        SC20  79
#                                                                       SC20  80
NSCREF = 0#                            Counter - scaled refs.           SC20  81
NSCOBS = 0#                            Counter - scaled obs.            SC20  82
NUNREP = 0#                            Counter - unreplicated refs.     SC20  83
NRJSTO = 0#                            Counter - out of sinth range     SC20  84
NRJSTR = 0#                            Obs and refs.                    SC20  85
NRJOBO = 0#                            Counter - out of observation     SC20  86
NRJOBR = 0#                            No. range obs and refs.          SC20  87
NRJFO  = 0#                            Counter - out of f**2 range      SC20  88
JUST1O = 0#                            Counter - obs w/1scale only      SC20  89
JUST1R = 0#                            Counter - ref w/1scale only      SC20  90
NRUSE  = 0#                            Counter - reflections used       SC20  91
NOBUSE = 0#                            Counter - observations used      SC20  92
NFS    = 0#                            F**2-sigma array counter         SC20  93
MXOBUZ = 0#                            Max useable obs for any ref      SC20  94
SHIFT  = 0#                            Keep garbage out                 SC20  95
#                                                                       SC20  96
CALL SC21(0,MAXOBS,FSQPTR)#            Initialize reflection reader     SC20  97
#                                                                       SC20  98
# LOOP OVER THE REFLECTIONS BRINGING IN ALL THE OBSERVATIONS FOR EACH   SC20  99
#                                                                       SC20 100
FOR(CALL SC21(1,MAXOBS,FSQPTR); SITOL >= 0.;$#                          SC20 101
CALL SC21(1,MAXOBS,FSQPTR))#                                            SC20 102
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  SC20 103
NN = 0#                                Counter - scaled obs.            SC20 104
FSQPT = FSQPTR - 3#                    Initialize next fsq pointer      SC20 105
#                                                                       SC20 106
FOR(I=1; I <= OBS; I=I+1)#             Loop over observations           SC20 107
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  SC20 108
FSQPT = FSQPT + 3#                                                      SC20 109
SHIFT = NINT(QX(FSQPT+3))#             Pick up shift number             SC20 110
#                                                                       SC20 111
IF(SHIFT > NSHIFT)#                    Could not be found, so           SC20 112
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  SC20 113
QX(FSQPT+1) = -QX(FSQPT+1)#            Flag unscaled obs.               SC20 114
NEXT#                                                                   SC20 115
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  SC20 116
NN = NN + 1#                           Count scaled obs. for            SC20 117
#                                         This refl.                    SC20 118
#                                                                       SC20 119
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  SC20 120
IF(NN == 0)  NEXT#                     No scaled obs., get next ref     SC20 121
NSCREF = NSCREF + 1#                   Increment counts                 SC20 122
NSCOBS = NSCOBS + NN#                                                   SC20 123
IF(NN == 1)#                           Was reflection unreplicated?     SC20 124
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  SC20 125
NUNREP = NUNREP + 1#                   Count and skip out               SC20 126
NEXT#                                                                   SC20 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  SC20 128
IF(SITOL < SMIN \ SITOL > SMAX)#       Filter on sinth range            SC20 129
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  SC20 130
NRJSTO = NRJSTO + NN#                  Increment counts                 SC20 131
NRJSTR = NRJSTR + 1#                                                    SC20 132
NEXT#                                                                   SC20 133
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  SC20 134
IF(NN < RMIN \ NN > RMAX)#             Filter on observation count      SC20 135
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  SC20 136
NRJOBO = NRJOBO + NN#                  Increment counts                 SC20 137
NRJOBR = NRJOBR + 1#                                                    SC20 138
NEXT#                                                                   SC20 139
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  SC20 140
NN = 0#                                Filter on f**2 range             SC20 141
PREVSN = 0#                            Make sure more than just one     SC20 142
USEFUL = no:#                          Scale group is left for          SC20 143
FSQPT = FSQPTR - 3#                    This reflection.                 SC20 144
#                                                                       SC20 145
FOR(I=1; I <= OBS; I=I+1)#             Loop over observations again     SC20 146
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  SC20 147
FSQPT = FSQPT + 3#                     Point to next f**2               SC20 148
FSQ = QX(FSQPT+1)#                     And pick it up.                  SC20 149
IF(FSQ < 0.)  NEXT#                    Skip it if not scaled or         SC20 150
IF(FSQ < FSQMIN \ FSQ > FSQMAX)#       Out of range.                    SC20 151
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  SC20 152
NRJFO = NRJFO + 1#                     Count out-of-range obs.          SC20 153
NEXT#                                                                   SC20 154
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  SC20 155
NN = NN + 1#                                                            SC20 156
SHIFT = NINT(QX(FSQPT+3))#             Now get shift ordinal and        SC20 157
IF(NN == 1)#                           Check for more than one          SC20 158
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  SC20 159
PREVSN = SHIFT#                        Scale for this refl., and        SC20 160
QX(FSQPT+1) = -QX(FSQPT+1)#            Set first f**2 negative          SC20 161
#                                         To flag start of new refl.    SC20 162
#                                         In output table.              SC20 163
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  SC20 164
#                                                                       SC20 165
ELSE  IF(SHIFT != PREVSN) USEFUL = yes:# Got a multiply-observed refl   SC20 166
NFS = NFS + 3#                         Increment pointers               SC20 167
FSPT = FSPT + 3#                                                        SC20 168
incrqx:(J,FSPT+3,SC2001)#              Get some more memory             SC20 169
movereal:(QX,FSQPT+1,QX,FSPT,3,1)#     Store f**2, sigma, and ordin     SC20 170
OBSPT = OBSPTR + SHIFT#                                                 SC20 171
QX(OBSPT) = QX(OBSPT) + 1.#            Count obs. for this shift        SC20 172
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  SC20 173
IF(USEFUL == no:)#                     If only 1 useable obs.,          SC20 174
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 SC20 175
NFS = NFS - NN*3#                      Decrement output observation     SC20 176
QX(OBSPT) = QX(OBSPT) - FLOAT(NN)#     Counter and obs./shift count     SC20 177
JUST1O = JUST1O + NN#                  Count observations and           SC20 178
JUST1R = JUST1R + 1#                   Reflection with but 1 scale.     SC20 179
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 SC20 180
ELSE#                                  Multiply observed                SC20 181
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 SC20 182
NRUSE = NRUSE + 1#                     Count refls. and obs.            SC20 183
NOBUSE = NOBUSE + NN#                                                   SC20 184
MXOBUZ = MAX0(MXOBUZ,NN)#              Get max useable obs/refl.        SC20 185
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 SC20 186
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  SC20 187
#                                                                       SC20 188
readwpkt:(1,endrecord:,PSIZE,IPT,0)#   Finish reading input bdf         SC20 189
#                                                                       SC20 190
LINRM = 14#                            Output reflection/observatio     SC20 191
writeline:(1,SC021,NSC021,3,3)#        Counts.                          SC20 192
writeline:(0,SC022,NSC022,3,3)#                                         SC20 193
#                                                                       SC20 194
OBSREF(1) = FLOAT(TOTOBS)#                                              SC20 195
OBSREF(2) = FLOAT(TOTREF)#                                              SC20 196
movechar:(SC023,1,CHROT,31,NSC023,0)#                                   SC20 197
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 198
writeline:(0,0,0,1,3)#                                                  SC20 199
#                                                                       SC20 200
OBSREF(1) = FLOAT(NSCOBS)#                                              SC20 201
OBSREF(2) = FLOAT(NSCREF)#                                              SC20 202
movechar:(SC024,1,CHROT,31,NSC024,0)#                                   SC20 203
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 204
writeline:(0,0,0,1,3)#                                                  SC20 205
#                                                                       SC20 206
OBSREF(1) = FLOAT(NSCOBS-NOBUSE)#                                       SC20 207
OBSREF(2) = FLOAT(NSCREF-NRUSE)#                                        SC20 208
movechar:(SC025,1,CHROT,31,NSC025,0)#                                   SC20 209
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 210
writeline:(0,0,0,1,3)#                                                  SC20 211
#                                                                       SC20 212
OBSREF(1) = FLOAT(NOBUSE)#                                              SC20 213
OBSREF(2) = FLOAT(NRUSE)#                                               SC20 214
movechar:(SC026,1,CHROT,31,NSC026,0)#                                   SC20 215
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 216
writeline:(0,0,0,1,3)#                                                  SC20 217
#                                                                       SC20 218
writeline:(1,SC027,NSC027,3,3)#                                         SC20 219
#                                                                       SC20 220
OBSREF(1) = FLOAT(NUNREP)#                                              SC20 221
OBSREF(2) = OBSREF(1)#                                                  SC20 222
movechar:(SC028,1,CHROT,31,NSC028,0)#                                   SC20 223
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 224
writeline:(0,0,0,1,3)#                                                  SC20 225
#                                                                       SC20 226
OBSREF(1) = FLOAT(NRJSTO)#                                              SC20 227
OBSREF(2) = FLOAT(NRJSTR)#                                              SC20 228
movechar:(SC029,1,CHROT,31,NSC029,0)#                                   SC20 229
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 230
writeline:(0,0,0,1,3)#                                                  SC20 231
#                                                                       SC20 232
OBSREF(1) = FLOAT(NRJFO)#                                               SC20 233
movechar:(SC02A,1,CHROT,31,NSC02A,0)#                                   SC20 234
ncodefld:(OBSREF,1,CHROT,FMT021(1),1)##                                 SC20 235
writeline:(0,0,0,1,3)#                                                  SC20 236
#                                                                       SC20 237
OBSREF(1) = FLOAT(JUST1O)#                                              SC20 238
OBSREF(2) = FLOAT(JUST1R)#                                              SC20 239
movechar:(SC02B,1,CHROT,31,NSC02B,0)#                                   SC20 240
ncodefld:(OBSREF,1,CHROT,FMT021(1),2)##                                 SC20 241
writeline:(0,0,0,1,3)#                                                  SC20 242
#                                                                       SC20 243
OBSREF(1) = FLOAT(MXOBUZ)#                                              SC20 244
ncodefld:(OBSREF,1,SC02C,FMT022,1)#                                     SC20 245
writeline:(1,SC02C,NSC02C,3,3)#                                         SC20 246
#                                                                       SC20 247
# RE-POSITION ARRAY OF FSQ/SIGMA/ORDINALS TO UTILIZE INPUT BUFFER SPACE SC20 248
#                                                                       SC20 249
FSQPTR = FSQPTR + 2*MXOBUZ#            Allow for shift and weight       SC20 250
movereal:(QX,FSPTR+1,QX,FSQPTR+1,NFS,1)# Move the huge array down       SC20 251
qxmemory:(FSQPTR+NFS)#                                                  SC20 252
#                                                                       SC20 253
RETURN#                                                                 SC20 254
END#                                                                    SC20 255
systemheader:(SC21)#                                                    SC21   1
# ################################################################## #  SC21   2
#                                                                    #  SC21   3
#                               SC21                                 #  SC21   4
#                                                                    #  SC21   5
# ################################################################## #  SC21   6
SUBROUTINE SC21(WHICH,ALLOWD,FPTR)#    Routine for reading logical      SC21   7
#                                         Record lrrefl:.               SC21   8
#                                        Mar 1979  r.a. alden           SC21   9
#                                        Updated oct 1979  r. alden     SC21  10
#                                        Revised mar 1983  r.a. alden   SC21  11
include:(AACOMN)#                                                       SC21  12
include:(SCCOMN)#                                                       SC21  13
#                                                                       SC21  14
INTEGER ALLOWD#                        Max. number of obs./refl.        SC21  15
#                                         Allowed.                      SC21  16
INTEGER FPTR#                          Pointer to start of space        SC21  17
#                                         Group fsq storage.            SC21  18
INTEGER IBY#                           The bijvoet flag                 SC21  19
INTEGER JP#                            Pointer to current fsq           SC21  20
INTEGER NOBS#                          Number of observation for        SC21  21
#                                         Current reflection.           SC21  22
INTEGER WHICH#                         Flag denoting which part of      SC21  23
#                                         Routine is being executed.    SC21  24
intdata:(IP,[0])#                      Ptr to start of space            SC21  25
datastuff:#                                                             SC21  26
#                                                                       SC21  27
IF(WHICH == 0)#                                                         SC21  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  SC21  29
IP = FPTR#                             Start of space group fsq's       SC21  30
#                                                                       SC21  31
TOTREF = 0#                            Clear                            SC21  32
REFSG  = 0#                                                             SC21  33
REFBY  = 0#                            Various                          SC21  34
RFBOTH = 0#                                                             SC21  35
TSGOBS = 0#                            Counts                           SC21  36
TBYOBS = 0#                                                             SC21  37
TOTOBS = 0#                                                             SC21  38
TOBSKP = 0#                                                             SC21  39
HKLNXT = 0#                            Clear next hkl                   SC21  40
CALL SC22(1,NOBS,FPTR,OBSKIP,HKLNXT,SITOL,ALLOWD)#                      SC21  41
#                                                                       SC21  42
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  SC21  43
#                                                                       SC21  44
ELSE#                                                                   SC21  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  SC21  46
# UPON RETURN...........                                                SC21  47
#   HKL CONTAINS THE SPACE GROUP INDICES CORRESPONDING TO HKLNOW        SC21  48
#   HKLBY  "      "     BIJVOET     "          "        "    "          SC21  49
#   HKLNOW ARE THE INDICES FOUND ON THE BDF                             SC21  50
#   HKLNXT  "   "     "    FOR THE NEXT REFLECTION TO BE PROCESSED      SC21  51
#                                                                       SC21  52
# THE WAY TO KNOW WHAT IS SITTING IN THE REFLECTION RECORD IS TO        SC21  53
#   CHECK OBSSG AND OBSBY.                                              SC21  54
#                                                                       SC21  55
OBSSG = 0#                             Clear space group obs. count     SC21  56
OBSBY = 0#                             Likewise for bijvoet obs.        SC21  57
#                                                                       SC21  58
# TEST THE NEXT PACKED HKL.  IT IS ALREADY SITTING IN HKLNXT            SC21  59
#                                                                       SC21  60
IF(HKLNXT != 0)#                       An hklnxt of 0 signals the       SC21  61
#                                         End of all reflections.       SC21  62
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  SC21  63
HKLNOW = HKLNXT#                                                        SC21  64
#                                                                       SC21  65
# SINCE A VALID HKL IS PRESENT, READ IN ITS DATA                        SC21  66
CALL SC22(1,NOBS,IP,OBSKIP,HKLNXT,SITOL,ALLOWD)#                        SC21  67
#                                                                       SC21  68
TOTOBS = TOTOBS + OBSKIP + NOBS#       Increment counters               SC21  69
TOBSKP = TOBSKP + OBSKIP#                                               SC21  70
#                                                                       SC21  71
HKL = 0#                                                                SC21  72
intpak:(HKLNOW,HKL,1,30)#              Get rid of any bijvoet bit       SC21  73
HKLBY = HKL+1#                         Put the bijvoet indices into     SC21  74
#                                         Hklby.                        SC21  75
IBY = 0#                                                                SC21  76
intunpak:(HKLNOW,IBY,0,1)#             Pick up the bijvoet bit          SC21  77
IF(IBY == 0)#                          Process all the space group      SC21  78
#                                         Observations.                 SC21  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  SC21  80
OBSSG = NOBS#                          Save the count                   SC21  81
REFSG = REFSG + 1#                     Increment refsg                  SC21  82
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  SC21  83
ELSE#                                  Process all the bijvoet obs.     SC21  84
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  SC21  85
OBSBY = NOBS#                          Save the count                   SC21  86
REFBY = REFBY + 1#                     Increment refby                  SC21  87
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  SC21  88
#                                                                       SC21  89
# IF SPACE GROUP AND BIJVOET OBSERVATIONS ARE TO BE MERGED, THEN        SC21  90
#  THE NEXT REFLECTION MUST NOW BE CHECKED. THE FOLLOWING               SC21  91
#  POSSIBILITIES EXIST:                                                 SC21  92
#  IF THE LAST REFLECTION WAS A SPACE GROUP ONE, THE NEXT ONE MAY       SC21  93
#  BE                                                                   SC21  94
#   1) THE BIJVOET OF THAT,                                             SC21  95
#   2) A NEW SPACE GROUP ONE,                                           SC21  96
#   3) A NEW BIJVOET ONE, OR                                            SC21  97
#   4) NOTHING AT ALL (END OF LOGICAL RECORD).                          SC21  98
#  IF THE LAST REFLECTION WAS A BIJVOET ONE WITH NO CORRESPONDING       SC21  99
#   SPACE GROUP ONE, THE NEXT ONE MAY BE                                SC21 100
#   1) A NEW SPACE GROUP ONE,                                           SC21 101
#   2) A NEW BIJVOET ONE, OR                                            SC21 102
#   3) NOTHING AT ALL, BUT CANNOT BE THE CORRESPONDING SPACE GROUP      SC21 103
#      ONE.                                                             SC21 104
#                                                                       SC21 105
# IF NO MERGING IS TO BE DONE, THEN WE DON'T CARE ABOUT THE NEXT        SC21 106
#  REFLECTION YET.                                                      SC21 107
#                                                                       SC21 108
IF(IMERGE == 0)#                                                        SC21 109
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  SC21 110
#                                                                       SC21 111
# ONCE AGAIN THE NEXT HKL IS PRESENT IN HKLNXT.  ITS VALUE CAN          SC21 112
#  BE CHECKED TO DETERMINE IF IT IS THE BIJVOET OF THE CURRENT          SC21 113
#  REFLECTION.  IF IT IS, THEN THE BIJVOET DATA WILL BE MERGED          SC21 114
#  WITH THAT FOR THE SPACE GROUP, CURRENTLY RESIDING IN THE             SC21 115
#  BUFFER.                                                              SC21 116
#                                                                       SC21 117
IBY = 0#                               Pick up the bijvoet bit          SC21 118
intunpak:(HKLNXT,IBY,0,1)#                                              SC21 119
IF(IBY == 1)#                          Check if it's a bijvoet refl     SC21 120
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  SC21 121
# PICK UP THE BIJVOET PART OF THE HKL FOR COMPARISON                    SC21 122
IF(HKLBY == HKLNXT)#                   By gum, i think we've got        SC21 123
#                                         A live one!                   SC21 124
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  SC21 125
JP = IP + 3*NOBS#                      Point to next storage            SC21 126
#                                         Location of f**2              SC21 127
#                                                                       SC21 128
# READ IN THE DATA FOR THE BIJVOET REFLECTION TO BE MERGED              SC21 129
#                                                                       SC21 130
CALL SC22(1,NOBS,JP,OBSKIP,HKLNXT,SITOL,ALLOWD)#                        SC21 131
#                                                                       SC21 132
TOTOBS = TOTOBS + OBSKIP + NOBS#       Increment counts                 SC21 133
TOBSKP = TOBSKP + OBSKIP#                                               SC21 134
#                                                                       SC21 135
OBSBY = NOBS#                          Save the count                   SC21 136
REFSG = REFSG - 1#                     Decrement space group only       SC21 137
#                                         Counter.                      SC21 138
RFBOTH = RFBOTH + 1#                   Increment rfboth                 SC21 139
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  SC21 140
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  SC21 141
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  SC21 142
#                                                                       SC21 143
OBS = OBSSG + OBSBY#                   Total obs. for this refl.        SC21 144
TOTREF = REFSG + REFBY + RFBOTH#       Increment total refl. counte     SC21 145
TSGOBS = TSGOBS + OBSSG#               Total space group obs.           SC21 146
TBYOBS = TBYOBS + OBSBY#               Total bijvoet obs.               SC21 147
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  SC21 148
ELSE SITOL = -1.#                      Put out a negative sitol to      SC21 149
#                                         Signal the end of lrrefl:.    SC21 150
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  SC21 151
#                                                                       SC21 152
RETURN#                                                                 SC21 153
END#                                                                    SC21 154
systemheader:(SC22)#                                                    SC22   1
# ################################################################## #  SC22   2
#                                                                    #  SC22   3
#                               SC22                                 #  SC22   4
#                                                                    #  SC22   5
# ################################################################## #  SC22   6
SUBROUTINE SC22(IDISC,NOBS,ADDR,OBSKIP,HKLNXT,SITOL,ALLOWD)#            SC22   7
#                                                                       SC22   8
# SUPPLY ALL OBSERVATIONS FOR ONE REFLECTION                            SC22   9
#                                        Mar 1983  r.a. alden           SC22  10
include:(AACOMN)#                                                       SC22  11
#                                                                       SC22  12
INTEGER ADDR#                          Pointer to store fsq, sigma      SC22  13
INTEGER ALLOWD#                        Maximum number of obs./refl.     SC22  14
#                                         Allowed.                      SC22  15
INTEGER FSPTR#                         Pointer to start of f**2         SC22  16
#                                         Storage.                      SC22  17
INTEGER HKLNOW#                        Current hkl                      SC22  18
INTEGER HKLNXT#                        The next hkl                     SC22  19
INTEGER IDISC#                         File being read                  SC22  20
INTEGER IPT#                           Input packet pointer             SC22  21
INTEGER JPT#                           Temporary pointer                SC22  22
INTEGER KPT#                           Temporary pointer                SC22  23
INTEGER NOBS#                          Obs. per refl. counter           SC22  24
INTEGER OBSKIP#                        Obs. skipped for current hkl     SC22  25
INTEGER PT20(6)#                       Flag and pointers to lrrefl:     SC22  26
REAL SITOL#                            Sine(theta)/lambda               SC22  27
intdata:(KEY,[5,5,5,0])#               Signals for indexptk:            SC22  28
intdata:(NEED20,[1,2,1302,1303,1309])# Required idnums for lrrefl:      SC22  29
datastuff:#                                                             SC22  30
#                                                                       SC22  31
IF (HKLNXT == 0)#                      First time into subroutine       SC22  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  SC22  33
#                                                                       SC22  34
KEY(4) = 0#                            Look for lrrefl: directory a     SC22  35
indexpkt:(1,lrrefl:,PSIZE,IPT,0,KEY,NEED20,PT20)# Required idnums.      SC22  36
IF(IPT < 0 \ KEY(4) < 0)  iquit:(SC2201)#                               SC22  37
readwpkt:(1,lrrefl:,PSIZE,IPT,0)#      Read first reflection            SC22  38
IF(IPT == 0)  iquit:(SC2202)#          Something's wrong                SC22  39
#                                                                       SC22  40
intunpak:(QX(IPT+1),HKLNXT,0,30)#      Get packed indices               SC22  41
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  SC22  42
ELSE#                                  Not first time through           SC22  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  SC22  44
OBSKIP = 0#                            Clear obs. skipped counter       SC22  45
NOBS = 0#                              Clear observation counter        SC22  46
#                                                                       SC22  47
HKLNOW = HKLNXT#                       Convert next hkl to current      SC22  48
#                                         Status.                       SC22  49
JPT = IPT + PT20(2)#                   Get pointer to packet now        SC22  50
#                                         Waiting to be processed.      SC22  51
SITOL = QX(JPT)#                       Save sine(theta)/lambda          SC22  52
FSPTR = -2#                            Initialize f**2 pointer          SC22  53
#                                                                       SC22  54
WHILE(HKLNXT == HKLNOW)#               Find all the observations fo     SC22  55
#                                         The current reflection.       SC22  56
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  SC22  57
IF(NOBS < ALLOWD)#                     Don't allow overflow             SC22  58
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  SC22  59
NOBS = NOBS + 1#                       Count obs. processed             SC22  60
FSPTR = FSPTR + 3#                     Point to next f**s               SC22  61
KPT = FSPTR + ADDR - 1#                Storage pointer                  SC22  62
DO I=1,3#                              Transfer f**2, sigma, and        SC22  63
#                                         Shift number.                 SC22  64
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  SC22  65
JPT = IPT + PT20(I+2)#                                                  SC22  66
QX(KPT+I) = QX(JPT)#                                                    SC22  67
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  SC22  68
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  SC22  69
ELSE  OBSKIP = OBSKIP + 1#             Count skipped obs.               SC22  70
readwpkt:(IDISC,lrrefl:,PSIZE,IPT,0)#  Read next observation            SC22  71
HKLNXT = 0#                            Clear packed hkl                 SC22  72
IF(IPT == 0)  BREAK#                   Leave, if end of lrrefl:         SC22  73
intunpak:(QX(IPT+1),HKLNXT,0,30)#      Otherwise save new packed hk     SC22  74
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  SC22  75
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  SC22  76
#                                                                       SC22  77
RETURN#                                                                 SC22  78
END#                                                                    SC22  79
systemheader:(SC30)#                                                    SC30   1
# ################################################################## #  SC30   2
#                                                                    #  SC30   3
#                               SC30                                 #  SC30   4
#                                                                    #  SC30   5
# ################################################################## #  SC30   6
SUBROUTINE SC30#                       Update the scales,               SC30   7
#                                         And put out new scales.       SC30   8
#                                        Aug 1979  freer & alden        SC30   9
include:(AACOMN)#                                                       SC30  10
include:(SCCOMN)#                                                       SC30  11
#                                                                       SC30  12
INTEGER F1PTR#                         Pointer to fsq of 1st obs. o     SC30  13
#                                         A reflection.                 SC30  14
INTEGER GDPT#                          Pointer to sum of wt*y**2 fo     SC30  15
#                                         The current shift.            SC30  16
INTEGER GDPTR#                         Pointer to start of sum of       SC30  17
#                                         Wt*y**2 table.                SC30  18
INTEGER GNPT#                          Pointer to sum of wt*y*fsq f     SC30  19
#                                         The current shift.            SC30  20
INTEGER GNPTR#                         Pointer to start of sum of       SC30  21
#                                         Wt*y*fsq table.               SC30  22
INTEGER I#                             Loop index                       SC30  23
INTEGER IFLIST#                        When zero, list the cycle st     SC30  24
INTEGER IFS#                           Reflection loop index            SC30  25
INTEGER IPT#                           Input packet pointer             SC30  26
INTEGER J#                             Pointer to start of qx worki     SC30  27
#                                        Area.                          SC30  28
INTEGER KEY(4)#                        Indexpkt: keys for lrexpl:       SC30  29
INTEGER JPT#                           Output packet pointer            SC30  30
#                                         Being done.                   SC30  31
INTEGER NCYCLE#                        Cycle loop index                 SC30  32
INTEGER NEED11(maxshifts:)#            Want list for lrexpl:            SC30  33
INTEGER NN#                            Number of obs. for current r     SC30  34
INTEGER PSIZE#                         Bdf packet size                  SC30  35
INTEGER PT11(arith:(1,+,maxshifts:))#  Pointers for lrexpl:             SC30  36
INTEGER SCPT#                          Pointer to current scale         SC30  37
#                                         Factor on the bdf.            SC30  38
INTEGER SHIFT#                         Shift number of current obs.     SC30  39
INTEGER SHPT#                          Pointer to current scale gro     SC30  40
#                                         Ordinal on the bdf.           SC30  41
INTEGER SHPTR#                         Pointer to table of scale        SC30  42
#                                         Group ordinals on the bdf.    SC30  43
INTEGER WSOPT#                         Pointer to sum of wt*fsq**2      SC30  44
#                                         For current scale group.      SC30  45
INTEGER WSOPTR#                        Pointer to start of sum of       SC30  46
#                                         Wt*fsq**2 table.              SC30  47
INTEGER WSRPT#                         Pointer to sum of                SC30  48
#                                         Wt*(fsq-scale*y)**2 for       SC30  49
#                                         Current scale group.          SC30  50
INTEGER WSRPTR#                        Pointer to start of sum of       SC30  51
#                                         Wt*(fsq-scale*y)**2 table.    SC30  52
INTEGER WTPT#                          Pointer to weight for curren     SC30  53
#                                         Observation.                  SC30  54
INTEGER WTPTR#                         Pointer to start of weights      SC30  55
#                                                                       SC30  56
REAL DELTAK#                           Difference between old and n     SC30  57
#                                         Scale factors.                SC30  58
REAL ERRFIT#                           Error of fit                     SC30  59
REAL FREEDM#                           Number of degrees of freedom     SC30  60
REAL FSQ#                              Current f**2                     SC30  61
REAL GCONST#                           Constant to multiply all         SC30  62
#                                         Refined scale factors by to   SC30  63
#                                         Keep the sum of all the       SC30  64
#                                         Factors constant.             SC30  65
REAL KREL#                             Old scale factor for 1st gro     SC30  66
REAL NEWK#                             New scale factor for current     SC30  67
#                                         Group.                        SC30  68
REAL OLDG#                             Old relative scale constant      SC30  69
REAL OLDK#                             Old scale factor                 SC30  70
REAL PVWSSR#                           Weighted sum of squares of       SC30  71
#                                         Resdiduals for previous cycle SC30  72
REAL R#                                Measurement of agreement in      SC30  73
#                                         Percent.                      SC30  74
REAL SCALE#                            Old scale factor for current     SC30  75
#                                         Observation.                  SC30  76
REAL SUMG#                             Sum of all refined relative      SC30  77
#                                         Scale factors (prior to being SC30  78
#                                         Made equal to sumgin).        SC30  79
REAL SUMGIN#                           Sum of all input relative        SC30  80
#                                         Scale factors.                SC30  81
REAL WSSOBT#                           Sum of wt*fsq**2                 SC30  82
REAL WSSRT#                            Sum of wt*(fsq-scale*y)**2       SC30  83
REAL WT#                               1./sigma(f**2)**2 for curren     SC30  84
#                                         Observation.                  SC30  85
REAL Y#                                Mean fsq for current refl.       SC30  86
REAL YD#                               Sum of wt*scale**2 for curre     SC30  87
#                                         Reflection.                   SC30  88
REAL YN#                               Sum of wt*scale*fsq for          SC30  89
#                                         Current reflection.           SC30  90
chardata:(SC031, GROUP     NEW-K     OLD-K    CHANGE       $#           SC30  91
OBS         R     CYCLE   )#                                            SC30  92
chardata:(SC032, OVERALL R      WSSR      WSSO  ERROR OF FIT)#          SC30  93
chardata:(SC033, *****SUM OF SQUARES INCREASED*****  $#                 SC30  94
FROM           TO          )#                                           SC30  95
chardata:(SC034,   FINAL SCALES WITH OVERALL CHANGE  INITIAL TO FINAL)# SC30  96
realdata:(CKOLD,[0.0])#                Constant for recovering old      SC30  97
realdata:(CKNEW,[0.0])#                Constant for recovering new      SC30  98
realdata:(FMT031,[690313.])#                                            SC30  99
realdata:(FMT032,[60513.,161052.,#                                      SC30 100
261052.,361052.,461013.,561022.])##                                     SC30 101
realdata:(FMT033,[100922.,201051.,#                                     SC30 102
301051.,441052.,511051.,641051.])##                                     SC30 103
realdata:(NUL,[0.])#                                                    SC30 104
datastuff:#                                                             SC30 105
#                                                                       SC30 106
GNPTR = OBSPTR + NSHIFT#               Set up various summation         SC30 107
GDPTR = GNPTR + NSHIFT#                Pointers.                        SC30 108
WSOPTR = GDPTR + NSHIFT#                                                SC30 109
WSRPTR = WSOPTR + NSHIFT#                                               SC30 110
#                                                                       SC30 111
SHPTR = WSRPTR + NSHIFT#                                                SC30 112
WTPTR = SHPTR + MXOBUZ#                                                 SC30 113
#                                                                       SC30 114
#INITIALIZATION BLOCK --  INVERT INPUT SCALES, ETC...                   SC30 115
FREEDM = FLOAT(NFS-NSHIFT+1)#          Degrees of freedom               SC30 116
KREL = QX(KPTR+1)#                     Regard k(1) as relative k        SC30 117
SUMGIN = 0.#                           Convert input k's to g's and     SC30 118
FOR(I=1; I <= NSHIFT; I=I+1)#          Get sum of initial g's.          SC30 119
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  SC30 120
GPT = GPTR + I#                                                         SC30 121
KPT = KPTR + I#                                                         SC30 122
QX(GPTR+I) = KREL/QX(KPTR+I)#                                           SC30 123
SUMGIN = SUMGIN + QX(GPTR+I)#                                           SC30 124
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  SC30 125
PVWSSR = 1.E30#                        Set prev sum of squares larg     SC30 126
LASTCY = no:#                          Set last cycle indicator         SC30 127
IFLIST = ILIST - 1#                    Initialize listing control       SC30 128
#                                                                       SC30 129
#--- REFINEMENT OUTER LOOP -- OVER REFINEMENT CYCLES ----               SC30 130
#                                                                       SC30 131
FOR(NCYCLE=1; NCYCLE <= MAXCYC; NCYCLE=NCYCLE+1)#                       SC30 132
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  SC30 133
IF(NCYCLE == MAXCYC)#                                                   SC30 134
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  SC30 135
LASTCY = yes:#                                                          SC30 136
IFLIST = 0#                                                             SC30 137
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  SC30 138
movereal:(NUL,1,QX, GNPTR+1,NSHIFT,2)# Scale numerator                  SC30 139
movereal:(NUL,1,QX, GDPTR+1,NSHIFT,2)# Scale denominator                SC30 140
movereal:(NUL,1,QX,WSOPTR+1,NSHIFT,2)# Wtd. sum of observations         SC30 141
movereal:(NUL,1,QX,WSRPTR+1,NSHIFT,2)# Weighted sum of squares          SC30 142
YN = 0.#                               Initialize mean f**2 num         SC30 143
YD = 0.#                               & denom, obs counter and         SC30 144
NN = 0#                                Pointer to first obs, since      SC30 145
F1PTR = FSQPTR + 1#                    First refl. is special case.     SC30 146
#                                                                       SC30 147
# THIS CYCLE STARTS HERE --  LOOP OVER REFLECTIONS                      SC30 148
#                                                                       SC30 149
FOR(IFS=1; IFS <= NFS; IFS=IFS+3)#                                      SC30 150
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  SC30 151
FSQPT = FSQPTR + IFS#                  Point to next f**2               SC30 152
IF(QX(FSQPT) < 0. & IFS != 1)#         Check for new reflection         SC30 153
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  SC30 154
Y = YN/YD#                             Mean f**2 for prev refl.         SC30 155
FOR(J=1; J <= NN; J=J+1)#              Second pass over prev. refl.     SC30 156
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  SC30 157
SHPT = SHPTR + J#                      To contribute to g sums.         SC30 158
SHIFT = NINT(QX(SHPT))#                Scale group ordinal              SC30 159
FSQPT = F1PTR + 3*(J-1)#                                                SC30 160
FSQ = ABS(QX(FSQPT))#                  F**2 for this observation        SC30 161
GPT = GPTR + SHIFT#                                                     SC30 162
SCALE = QX(GPT)#                       Inverse scale constant           SC30 163
WTPT = WTPTR + J#                                                       SC30 164
WT = QX(WTPT)#                         Weight for this observation      SC30 165
#                                         Contribute to sums            SC30 166
GNPT = GNPTR + SHIFT#                                                   SC30 167
QX(GNPT) = QX(GNPT) + WT*Y*FSQ#                                         SC30 168
GDPT = GDPTR + SHIFT#                                                   SC30 169
QX(GDPT) = QX(GDPT) + WT*Y**2#                                          SC30 170
WSRPT = WSRPTR + SHIFT#                                                 SC30 171
QX(WSRPT) = QX(WSRPT) + WT*(FSQ-SCALE*Y)**2#                            SC30 172
WSOPT = WSOPTR + SHIFT#                                                 SC30 173
QX(WSOPT) = QX(WSOPT) + WT*FSQ**2#                                      SC30 174
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  SC30 175
YN = 0.#                               Initialize for new refl.         SC30 176
YD = 0.#                                                                SC30 177
NN = 0#                                                                 SC30 178
F1PTR = FSQPTR + IFS#                                                   SC30 179
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  SC30 180
NN = NN + 1#                           Count obs. of this refl.         SC30 181
FSQPT = FSQPTR + IFS#                                                   SC30 182
SHIFT = NINT(QX(FSQPT+2))#             Get shift ordinal                SC30 183
SHPT = SHPTR + NN#                                                      SC30 184
QX(SHPT) = FLOAT(SHIFT)#               Store shift for 2nd pass         SC30 185
WT = 1./(QX(FSQPT+1)**2)#              Comput weight and                SC30 186
WTPT = WTPTR + NN#                     Store it for 2nd pass.           SC30 187
QX(WTPT) = WT#                                                          SC30 188
FSQ = ABS(QX(FSQPT))#                  F**2 (neg means 1st obs).        SC30 189
GPT = GPTR + SHIFT#                                                     SC30 190
SCALE = QX(GPT)#                       Inverse scale for this obs.      SC30 191
YN = YN + SCALE*WT*FSQ#                Mean f**2 numerator and          SC30 192
YD = YD + WT*SCALE**2#                 Denominator.                     SC30 193
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  SC30 194
#                                                                       SC30 195
# LOOP OVER REFLECTIONS IS COMPLETED, NOW  COMPUTE AND LIST SCALES...   SC30 196
#                                                                       SC30 197
WSSOBT = 0.#                           Initialize...                    SC30 198
WSSRT = 0.#                                                             SC30 199
SUMG = 0.#                                                              SC30 200
#                                                                       SC30 201
FOR(I=1; I <= NSHIFT; I=I+1)#          Get sum of inverse scales        SC30 202
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  SC30 203
GNPT = GNPTR + I#                                                       SC30 204
GDPT = GDPTR + I#                                                       SC30 205
SUMG = SUMG + QX(GNPT)/QX(GDPT)#                                        SC30 206
WSRPT = WSRPTR + I#                                                     SC30 207
WSSRT = WSSRT + QX(WSRPT)#             And sum of squares.              SC30 208
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  SC30 209
#                                                                       SC30 210
IF(WSSRT > PVWSSR)#                    Force list, the sum of           SC30 211
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  SC30 212
IFLIST = 0#                            Squares increased.               SC30 213
LASTCY = yes:#                                                          SC30 214
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  SC30 215
GCONST = SUMGIN/SUMG#                  Factor to keep sum g constnt     SC30 216
#                                                                       SC30 217
FOR(I=1; I <= NSHIFT; I=I+1)#          Get new scales & list...         SC30 218
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  SC30 219
OBSPT = OBSPTR + I#                                                     SC30 220
IF(QX(OBSPT) <= 0.)  NEXT#             Skip groups with no obs.         SC30 221
GPT = GPTR + I#                                                         SC30 222
OLDG = QX(GPT)#                        Save prev g for list             SC30 223
GNPT = GNPTR + I#                                                       SC30 224
GDPT = GDPTR + I#                                                       SC30 225
QX(GPT) = GCONST*(QX(GNPT)/QX(GDPT))#  New inverse g                    SC30 226
IF(IFLIST != 0)  NEXT#                 Skip out if not listing          SC30 227
IF(I == 1)#                            If this is first scale, prin     SC30 228
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 SC30 229
BUFOT(1) = FLOAT(NCYCLE)#                                               SC30 230
ncodefld:(BUFOT,1,SC031,FMT031,1)#     Heading.                         SC30 231
writeline:(1,0,0,1,3)#                                                  SC30 232
LINRM = 3#                                                              SC30 233
CKOLD = KREL*OLDG#                     G to k conversion                SC30 234
CKNEW = KREL*QX(GPTR+1)#               Constants.                       SC30 235
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 SC30 236
#                                                                       SC30 237
WSOPT = WSOPTR + I#                                                     SC30 238
WSSOBT = WSSOBT + QX(WSOPT)#           Total sum-sq of observations     SC30 239
#                                         Scaling r-factor.             SC30 240
OLDK = CKOLD/OLDG#                     Get k's for listing              SC30 241
NEWK = CKNEW/QX(GPT)#                                                   SC30 242
DELTAK = NEWK - OLDK#                                                   SC30 243
#                                        Last cycle, get g to k-final   SC30 244
IF(LASTCY == yes:)  QX(GPT) = NEWK#                                     SC30 245
WSRPT = WSRPTR + I#                                                     SC30 246
R = 100.*SQRT(QX(WSRPT)/QX(WSOPT))#                                     SC30 247
GRPPT = GRPPTR + I#                                                     SC30 248
#                                                                       SC30 249
BUFOT(1) = QX(GRPPT)#                                                   SC30 250
BUFOT(2) = NEWK#                                                        SC30 251
BUFOT(3) = OLDK#                                                        SC30 252
BUFOT(4) = DELTAK#                                                      SC30 253
BUFOT(5) = QX(OBSPT)#                                                   SC30 254
BUFOT(6) = R#                                                           SC30 255
ncodefld:(BUFOT,1,CHROT,FMT032(1),6)##                                  SC30 256
writeline:(0,SC031,NSC031,1,3)#                                         SC30 257
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  SC30 258
#                                                                       SC30 259
IF(IFLIST == 0)#                       List summary statistics...       SC30 260
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 SC30 261
R = 100.*SQRT(WSSRT/WSSOBT)#                                            SC30 262
ERRFIT = SQRT(WSSRT/FREEDM)#                                            SC30 263
writeline:(1,SC032,NSC032,3,3)#                                         SC30 264
BUFOT(1) = R#                                                           SC30 265
BUFOT(2) = WSSRT#                                                       SC30 266
BUFOT(3) = WSSOBT#                                                      SC30 267
BUFOT(4) = ERRFIT#                                                      SC30 268
ncodefld:(BUFOT,1,CHROT,FMT033(1),4)##                                  SC30 269
writeline:(0,0,0,1,3)#                                                  SC30 270
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 SC30 271
IF(WSSRT > PVWSSR)#                    Write message if sum             SC30 272
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 SC30 273
BUFOT(1) = PVWSSR#                                                      SC30 274
BUFOT(2) = WSSRT#                                                       SC30 275
ncodefld:(BUFOT,1,SC033,FMT033(5),2)#                                   SC30 276
writeline:(1,SC033,NSC033,3,3)#                                         SC30 277
BREAK#                                 Terminate refinement             SC30 278
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 SC30 279
PVWSSR = WSSRT#                                                         SC30 280
IFLIST = IFLIST - 1#                   Listing control                  SC30 281
IF(IFLIST < 0)  IFLIST = ILIST - 1#    Is reset when negative.          SC30 282
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  SC30 283
#                                                                       SC30 284
# REFINEMENT CYCLE LOOP ENDS HERE                                       SC30 285
#                                                                       SC30 286
copyfile:(1,2,1,decr:(lrexpl:))#       Prepare output bdf               SC30 287
DO I=1,NSHIFT#                         Loop over shift numbers          SC30 288
NEED11(I) = 100 +I#                    Making up want list              SC30 289
KEY(1) = NSHIFT ; KEY(2) = NSHIFT#     All should still be there        SC30 290
KEY(3) = NSHIFT ; KEY(4) = 0#          And it is an update              SC30 291
indexpkt:(1,lrexpl:,PSIZE,IPT,0,KEY,NEED11,PT11)# Get directory         SC30 292
IF(KEY(4)<=0) iquit:(SC3001)#          Impossible error.                SC30 293
writepkt:(2,lrexpl:,PSIZE,JPT)#                                         SC30 294
movereal:(QX,IPT+1,QX,JPT+1,PSIZE,1)#                                   SC30 295
readwpkt:(1,lrexpl:,PSIZE,IPT,0)#      Copy only other packet           SC30 296
writepkt:(2,lrexpl:,PSIZE,JPT)#                                         SC30 297
#                                                                       SC30 298
movereal:(QX,IPT+1,QX,JPT+1,PSIZE,1)#  Transfer from input to outpu     SC30 299
#                                                                       SC30 300
writeline:(2,SC034,NSC034,3,3)#        Output & list final scales       SC30 301
writeline:(0,SC031,NSC031,3,3)#                                         SC30 302
#                                                                       SC30 303
FOR(I=1; I <= NSHIFT; I=I+1)#                                           SC30 304
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 SC30 305
ncodefld:(QX,GRPPTR+I,CHROT,FMT032(1),1)# Shift number                  SC30 306
ncodefld:(QX,GPTR+I,CHROT,FMT032(2),1)# Final scale factor              SC30 307
ncodefld:(QX,KPTR+I,CHROT,FMT032(3),1)# Initial scale factor            SC30 308
KPT = KPTR + I#                                                         SC30 309
GPT = GPTR + I#                                                         SC30 310
DELTAK = QX(GPT) - QX(KPT)#            Change from initial cycle        SC30 311
ncodefld:(DELTAK,1,CHROT,FMT032(4),1)##                                 SC30 312
ncodefld:(QX,OBSPTR+I,CHROT,FMT032(5),1)# Number of observations        SC30 313
WSRPT = WSRPTR + I#                                                     SC30 314
WSOPT = WSOPTR + I#                                                     SC30 315
R = 100.*SQRT(QX(WSRPT)/QX(WSOPT))#    R-factor                         SC30 316
ncodefld:(R,1,CHROT,FMT032(6),1)#                                       SC30 317
writeline:(0,SC031,NSC031,1,3)#                                         SC30 318
#                                                                       SC30 319
GRPPT = GRPPTR + I#                                                     SC30 320
SHIFT = NINT(QX(GRPPT))#               Current shift number             SC30 321
#                                                                       SC30 322
SCPT = JPT + PT11(I)#                  Found a match                    SC30 323
QX(SCPT) = QX(GPT)#                    Store away new scale factor      SC30 324
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 SC30 325
#                                                                       SC30 326
copyfile:(1,2,incr:(lrexpl:),endrecord:)# Finish copying input bd       SC30 327
#                                                                       SC30 328
RETURN#                                                                 SC30 329
END#                                                                    SC30 330
pmacro:([no:])#                        Purge xtal system macros         SC30 331
