systemheader:(AX    )#                                                  AX     1
######################################################################  AX     2
##      Copyright (C) 2000, The Xtal System, The University of        # AX     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # AX     4
## Distributed under the GNU General Public Licence as published by   # AX     5
## the Free Software Foundation, in the hope that it will be useful   # AX     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # AX     7
## for more details.                                                  # AX     8
#                                                                    #  AX     9
#                            *** LSLS ***                            #  AX    10
#                                                                    #  AX    11
# LEAST-SQUARE PROGRAM FOLLOWING THE INSTRUCTIONS GIVEN BY "STATIS-  #  AX    12
# TICAL DESCRIPTORS", I.E. REFINEMENT ON INTENSITIES, REFINEMENT ON  #  AX    13
# ALL POSSIBLE INFORMATIONS (CRYSTAL SHAPE, ...), USE OF RESTRAINTS  #  AX    14
# INSTEAD OF CONSTRAINTS, ...                                        #  AX    15
# ------------------------------------------------------------------ #  AX    16
# WRITTEN BY E.BLANC WITH HELP FORM D.SCHWARZENBACH                  #  AX    17
# THE CODE IS BUILD FROM CRYLSQ BY R.OLTHOF & FROM SFLSX BY S.HALL   #  AX    18
# ------------------------------------------------------------------ #  AX    19
# GENERAL INFORMATION ON THE PROGRAM:                                #  AX    20
# AX00: CALL THE DIFFERENT MODULES OF THE PROGRAM (AXX0)             #  AX    21
# AX0X: SUBROUTINES USED AS MACROS                                   #  AX    22
# AX10: CALL THE INPUT PART ON THE PROGRAM                           #  AX    23
#       * AX11: READ LSLS CARD                                       #  AX    24
#       * AX12: READ LRCELL:, LRSCAT: FROM BDF (CELL AND SCATTERING  #  AX    25
#               INFO)                                                #  AX    26
#       * AX13: READ MODIFICATION CARDS (MAXHKL)                     #  AX    27
#       * AX14: READ ATOMS FROM BDF (LRATOM:)                        #  AX    28
#       * AX16: READ LRCONS:                                         #  AX    29
#       * AX17: CHECK KEY ARRAY (FOR SELF-CONSISTENCY) AND PACK MEM  #  AX    30
#       * AX18: READ LIMSFT CARDS                                    #  AX    31
# AX20: CALL THE RESTRAINT AND KEY ARRAY CHECKS                      #  AX    32
#       * AX22: READ RESTRAINT RECORD(S) (LRDIST:)                   #  AX    33
#       * AX23: OUTPUT INITIAL PARAMETERS                            #  AX    34
#       * AX25: SET ADDRESSES IN VECTOR ARRAY AND MATRIX             #  AX    35
# AX30: CONSTRUCT MATRIX                                             #  AX    36
#       * AX31: DIRECTORY OF LRREFL: PROCESSING                      #  AX    37
#       * AX32: COMPUTE |F|**2, I AND DERIVATIVES                    #  AX    38
#       * AX33: COMPUTE DISTANCES                                    #  AX    39
#       * AX34: COMPUTE RESTRAINTS ON XYZ                            #  AX    40
#       * AX35: COMPUTE RIGID BOND RESTRAINTS                        #  AX    41
#       * AX38: PRINT REFLECTION INFORMATION                         #  AX    42
#       * AX39: BECKER & COPPENS EXTINCTION                          #  AX    43
# AX40: TRANSFORM MATRIX (ORIGIN FIXING, TEST & INVERSION)           #  AX    44
#       * AX41: INSERT POLAR LIMSHIFT                                #  AX    45
#       * AX42: TEST MATRIX, INVERSION, BUILD  VARIANCE COVARIANCE   #  AX    46
#               MATRIX                                               #  AX    47
#       * AX43: PRINT SUMMARY (R, WR, ...)                           #  AX    48
#       * AX44: COMPUTE & PRINT NEW GENERAL PARAMETERS               #  AX    49
#       * AX45: COMPUTE & PRINT NEW ATOMIC PARAMETERS                #  AX    50
#       * AX47: PRINT MATRIX                                         #  AX    51
#       * AX48: G0 & G1 FUNCTIONS                                    #  AX    52
#       * AX49: BLOCK INVERSION (FROM LINPACK)                       #  AX    53
# AX50: END OF REFINEMENT AND CLOSE SHOP                             #  AX    54
#       * AX51: PREPARE LISTS XYZ FOR SAVING PARAMETERS              #  AX    55
#       * AX52: SAVE GENERAL PARAMETERS ON BDF                       #  AX    56
#       * AX53: SAVE ATOMIC PARAMETERS ON BDF                        #  AX    57
#       * AX54: WRITE LRREFL:                                        #  AX    58
#       * AX55: SAVE REFINEMENT INFO & RESTRAINTS                    #  AX    59
#       * AX56: WRITE VARIANCE COVARIANCE MATRIX                     #  AX    60
#       * AX59: GET SIGMA OF PARAMETERS                              #  AX    61
# ------------------------------------------------------------------ #  AX    62
# USE OF MEMORY:                                                     #  AX    63
# 1) I/O BUFFERS (MAX 3)                                             #  AX    64
# 2) NON-PARAMETRIC LISTS                                            #  AX    65
#    * MARKA - STOPA: SYMETRY OPERATORS (MATRICES AND VECTORS)       #  AX    66
#    * MARKB - STOPB: SYMETRY OPERATORS LEADING TO HOLOHEDRY         #  AX    67
#    * MARKC - STOPC: SCATTERING FACTOR TABLES                       #  AX    68
# 3) PARAMETRIC LISTS (CONTAINING ALL INFORMATIONS)                  #  AX    69
#    * MARK(1) - STOP(1): CRYSTAL SHAPE INFORMATION                  #  AX    70
#    * MARK(2) - STOP(2): ENANTIOMORPH-POLARITY PARAMTERS            #  AX    71
#    * MARK(3) - STOP(3): SCALES                                     #  AX    72
#    * MARK(4) - STOP(4): EXTINCTION PARAMETERS                      #  AX    73
#    * MARK(5) - STOP(5): OVERALL DISPLACEMENT FACTOR                #  AX    74
#    * MARK(6) - STOP(6): DISPERSION FACTORS                         #  AX    75
#    * MARK(7) - STOP(7): ATOMIC PARAMETERS                          #  AX    76
# 4) CONTROL LISTS                                                   #  AX    77
#    * MARKX - STOPX: NOREF/CONSTRAINT LIST                          #  AX    78
#    * MARKY - STOPY: BODY OF CONSTRAINT LIST                        #  AX    79
#    * MARKZ - STOPZ: LIM.SHIFT LIST                                 #  AX    80
# 5) RESTRAINT LIST: MARKR(I) - STOPR(I): I = 1: DISTANCE            #  AX    81
#                                         I = 2: angle               #  AX    82
#                                         I = 3: torsion angle       #  AX    83
#                                         I = 4: rigid bond          #  AX    84
# 6) VARIABLE LISTS                                                  #  AX    85
#    * MARK1(1)       : CRYSTAL SHAPE                                #  AX    86
#    * MARK1(2)       : ENANTIOMORPH-POLARITY                        #  AX    87
#    * MARK1(3)       : SCALES                                       #  AX    88
#    * MARK1(4)       : EXTINCTION                                   #  AX    89
#    * MARK1(5)       : OVERALL DISPLACEMENT FACTOR                  #  AX    90
#    * MARK1(6)       : DISPERSION                                   #  AX    91
#    * MARK1(7)       : ATOMS                                        #  AX    92
# 7) VECTOR AND MATRIX LISTS                                         #  AX    93
#    * MARK20       : VARIABLE LIST (=MARK10) UNTIL INVERSION, THEN  #  AX    94
#                     CONTAINS SHIFTS                                #  AX    95
#    * MARK21       : DIFFERENCE VECTOR UNTIL INVERSION              #  AX    96
#    * MARK22       : NORMAL EQUATION MATRIX UNTIL INVERSION, THEN   #  AX    97
#                     CONTAINS ITS INVERSE                           #  AX    98
#    * MARK23       : (TAWVOWA)                                      #  AX    99
#    * MARK24       : VARIABLES VARIANCE-COVARIANCES MATRIX          #  AX   100
# ------------------------------------------------------------------ #  AX   101
# ABOUT KEY-ARRAY LISTS                                              #  AX   102
# 1) LIST X: * ADDRESS OF CONSTRAINED OR NOT REFINED PARAMETER       #  AX   103
#            * FLAG TO KNOW IF IT IS CONSTRAINED OR NOR REFINED      #  AX   104
# 2) LIST Y: AT LEAST ONE RECORD:                                    #  AX   105
#            * ADDRESS OF REFERENCE PARAMETER BEFORE AX17, ADDRESS   #  AX   106
#              OF REFERENCE VARIABLE AFTER AX17                      #  AX   107
#            * MULTIPLICATIVE FACTOR OF REFERENCE PARAMETER          #  AX   108
#            LAST RECORD:                                            #  AX   109
#            * FLAG FOR END-OF-CONSTRAINT (0)                        #  AX   110
#            * CONTRAINTE ADITIVE                                    #  AX   111
# 3) LIST Z: * ADDRESS OF SHIFT-LIMITED PARAMETER                    #  AX   112
#            * SHIFT-LIMITING VALUE                                  #  AX   113
# N.B. THE LISTS X & Y ACT ON VECTOR, BUT LIST Z ACT ON MATRIX. THAT #  AX   114
#      IS THE REASON WHY WE MADE DIFFERENT LISTS.                    #  AX   115
# N.B. ALL LISTS ARE SORTED FROM IN ASCENDING ADDRESSES. SO CONSTR.  #  AX   116
#      ON DISTANCES WILL BE STORED BEFORE CONSTRAINTS ON ENTPOL, AND #  AX   117
#      THEN ON SCALE, ...                                            #  AX   118
#                                                                    #  AX   119
######################################################################  AX   120
#                                                                       AX   121
include:(XMACRO)#                                                       AX   122
#                                                                       AX   123
macro:(rad:,0.017453293)#              1 deg in radians                 AX   124
macro:(pi:,3.141592654)#                                                AX   125
macro:(utob:,78.9568352)#              = 8pi**2                         AX   126
#                                                                       AX   127
macro:(mxwd8c:,incr:(arith:(7,/,mxchwd:)))# 8 chars                     AX   128
macro:(mxwd24c:,incr:(arith:(23,/,mxchwd:)))# 24 chars                  AX   129
macro:(mxwd40c:,incr:(arith:(39,/,mxchwd:)))# 40 chars                  AX   130
macro:(lena:,24)#                      Atom name length (in char)       AX   131
macro:(lens:,8)#                       Atom type name length (in char   AX   132
macro:(swap:,$(JQJ=$1;$1=$2;$2=JQJ$))#                                  AX   133
macro:(ifpr:,[IF ((IREL($1) > 0) & $#                                   AX   134
      (ABS(voidflg:-QX(IP+IREL($1))) > 1.0E arith:(mxsgfp:,-,20)))])#   AX   135
#                                                                       AX   136
macro:(sfmax:,16)#                     Max nb of scattering factors     AX   137
macro:(dist:,1)#                       Distance      restraint code     AX   138
macro:(angl:,2)#                       Angle         restraint code     AX   139
macro:(tort:,3)#                       Torsion angle restraint code     AX   140
macro:(bond:,4)#                       Rigid bond    restraint code     AX   141
macro:(noref:,-1)#                     Noref      code in n/c list      AX   142
macro:(constr:,-2)#                    Constraint code in n/c list      AX   143
macro:(limsft:,0)#                     Lim.shift code in b/l list       AX   144
#                                                                       AX   145
#                                      ***** symop list                 AX   146
macro:(stepa:,12)#                     9 for matrix and 3 for vector    AX   147
macro:(incra:,$1=$1+stepa:)#                                            AX   148
#                                                                       AX   149
#                                      ***** symop for holohedry list   AX   150
macro:(stepb:,9)#                      9 for matrix                     AX   151
macro:(incrb:,$1=$1+stepb:)#                                            AX   152
#                                                                       AX   153
macro:(step1:,8)#                      ***** crystal shape list         AX   154
macro:(off1:,7)#                       Offset for dist                  AX   155
macro:(incr1:,$1=$1+step1:)#                                            AX   156
macro:(facem:,QX($1))#                 Miller indices (for output)      AX   157
macro:(faceh:,QX($1+1))#               H                                AX   158
macro:(facek:,QX($1+2))#               K                                AX   159
macro:(facel:,QX($1+3))#               L                                AX   160
macro:(facedo:,QX($1+4))#              Observed distance                AX   161
macro:(faceso:,QX($1+5))#              Esd of observed distance         AX   162
macro:(facew:,QX($1+6))#               Weight                           AX   163
macro:(facedc:,QX($1+7))#              Calculated distance              AX   164
macro:(facesc:,QX($1+8))#              Esd of calculated distance       AX   165
#                                                                       AX   166
macro:(step2:,1)#                      ***** ent-pol list               AX   167
macro:(off2:,1)#                       Offset for entpol                AX   168
macro:(incr2:,$1=$1+step2:)#                                            AX   169
macro:(entpol:,QX(MARK(2)+$1))#                                         AX   170
#                                                                       AX   171
macro:(step3:,2)#                      ***** scale factor list          AX   172
macro:(off3:,1)#                       Offset for scale                 AX   173
macro:(incr3:,$1=$1+step3:)#                                            AX   174
macro:(sclfac:,QX($1+1))#              Scale factor on i                AX   175
macro:(sclgrp:,QX($1+2))#              Scale group number               AX   176
#                                                                       AX   177
macro:(step4:,1)#                      ***** extinction list            AX   178
macro:(off4:,1)#                       Offset for extinction            AX   179
macro:(extinc:,QX(MARK(4)+$1))#        General parameter (addressed b   AX   180
macro:(g:,QX(MARK(4)+1))#              Extinction g value               AX   181
macro:(rho:,QX(MARK(4)+2))#            Extinction rho value             AX   182
macro:(extani1:,QX(MARK(4)+2+$1))#     Anisotropic tensor 1             AX   183
macro:(extani2:,QX(MARK(4)+8+$1))#     Anisotropic tensor 2             AX   184
#                                                                       AX   185
macro:(step5:,1)#                      ***** overall displacement par   AX   186
macro:(off5:,1)#                       Offset for u overall             AX   187
macro:(uovl:,QX(MARK(5)+1))#           Uovl                             AX   188
#                                                                       AX   189
macro:(incr6:,$1=$1+STEP(6))#          ***** dispersion list            AX   190
macro:(redisp:,QX($1+mxwd8c:+1))#      Real part                        AX   191
macro:(imdisp:,QX($1+mxwd8c:+2))#      Imaginary part                   AX   192
macro:(nattyp:,QX($1+1))#              Atom type name                   AX   193
#                                                                       AX   194
#                                      ***** atom list                  AX   195
macro:(step7:,arith:(14,+,mxwd24c:))#  14 words + atom name             AX   196
macro:(incr7:,$1=$1+step7:)#                                            AX   197
macro:(atom:,QX($1))#                  General parameter                AX   198
macro:(atomxyz:,QX($1))#               Position                         AX   199
macro:(atomx:,QX($1+1))#               X                                AX   200
macro:(atomy:,QX($1+2))#               Y                                AX   201
macro:(atomz:,QX($1+3))#               Z                                AX   202
macro:(atomdis:,QX($1+3))#             Displacement parameters          AX   203
macro:(atomiso:,QX($1+4))#             Isotropic displacement paramet   AX   204
macro:(atomu11:,QX($1+4))#             U11                              AX   205
macro:(atomu22:,QX($1+5))#             U22                              AX   206
macro:(atomu33:,QX($1+6))#             U33                              AX   207
macro:(atomu12:,QX($1+7))#             U12                              AX   208
macro:(atomu13:,QX($1+8))#             U13                              AX   209
macro:(atomu23:,QX($1+9))#             U23                              AX   210
macro:(atompop:,QX($1+10))#            Atom population parameter        AX   211
macro:(atomnsf:,QX($1+11))#            Neutron scattering length        AX   212
macro:(atomsft:,QX($1+12))#            Scattering factor type           AX   213
macro:(atomtft:,QX($1+13))#            Temp. factor type                AX   214
macro:(atommul:,QX($1+14))#            Atom multiplicity                AX   215
macro:(atomn:,QX($1+15))#              Atom name                        AX   216
#                                                                       AX   217
macro:(stepx:,2)#                      ***** n/c list                   AX   218
macro:(incrx:,$1=$1+stepx:)#                                            AX   219
macro:(ncaddr:,QX($1+1))#              Address of parameter             AX   220
macro:(nctype:,QX($1+2))#              1: constrained, 2: not refined   AX   221
#                                                                       AX   222
macro:(stepy:,2)#                      ***** body of constraint         AX   223
macro:(incry:,$1=$1+stepy:)#                                            AX   224
macro:(caddr:,QX($1+1))#               Address of reference (0 for la   AX   225
macro:(cfact:,QX($1+2))#               Multiplicative factor            AX   226
#                                                                       AX   227
#                                      ***** tools for nc list          AX   228
#                                                                       AX   229
#      FINDNC:(START_SEARCH, RETURN_PTR, SUBJECT_ADDRESS)               AX   230
macro:(findnc:,$(#                     Find subject_param_address in    AX   231
[  DO MX = $1, STOPX-stepx:, stepx:]#  Loop over remaining part of li   AX   232
IF (NINT(ncaddr:(MX))>=$3)BREAK#       Exit if param nb > subj          AX   233
$2=MX$))#                              Return nc pointer                AX   234
#                                      Findc:(markx, marky, lx, ly, sub AX   235
macro:(findc:,$(#                      Find                             AX   236
MY=$2#                                                                  AX   237
[  DO MX = $1, STOPX-stepx:, stepx:]$(#                                 AX   238
IF (NINT(ncaddr:(MX))>=$5)BREAK#                                        AX   239
IF (NINT(nctype:(MX))!=noref:)$(#                                       AX   240
WHILE (caddr:(MY)>small:)incry:(MY)#                                    AX   241
incry:(MY)$)$)#                                                         AX   242
$3=MX;$4=MY$))#                                                         AX   243
#                                      Putnc:(lx, subj, noref:)         AX   244
macro:(putnc:,CALL AX04($1,$2,0.0,$3))#                                 AX   245
#                                      Ifnc:(lx, l)                     AX   246
macro:(ifnc:,IF (($1<STOPX)&(NINT(ncaddr:($1))==$2)))#                  AX   247
#                                                                       AX   248
macro:(stepz:,2)#                      ***** b/l list                   AX   249
macro:(incrz:,$1=$1+stepz:)#                                            AX   250
macro:(bladdr:,QX($1+1))#              Address of parameter             AX   251
macro:(blfact:,QX($1+2))#              Limshift value                   AX   252
#                                                                       AX   253
#                                      ***** tools for bl list          AX   254
#                                      Findbl:(markz, lz, subj)         AX   255
macro:(findbl:,$(#                                                      AX   256
[  DO MZ = $1, STOPZ-stepz:, stepz:]#                                   AX   257
IF (NINT(bladdr:(MZ))>=$3)BREAK#                                        AX   258
$2=MZ$))#                                                               AX   259
#                                      Putbl:(lz, subj, lim_val)        AX   260
macro:(putbl:,CALL AX04($1,$2,$3,limsft:))#                             AX   261
#                                      Ifbl:(lz, l)                     AX   262
macro:(ifbl:,IF (($1<STOPZ)&(NINT(bladdr:($1))==$2)))#                  AX   263
#                                                                       AX   264
#                                      ***** restraints lists           AX   265
macro:(recl:,13)#                      13 words/record                  AX   266
macro:(incrr:,$1=$1+$2*recl:+9)#                                        AX   267
macro:(resobs:,QX($1+1))#                                               AX   268
macro:(resd:,QX($1+4))#                                                 AX   269
macro:(resw:,QX($1+5))#                                                 AX   270
macro:(rescal:,QX($1+6))#                                               AX   271
macro:(rescsd:,QX($1+9))#                                               AX   272
macro:(resp:,QX($1+($2-1)*recl:+10))#                                   AX   273
macro:(resm:,[ifelse:($4,,QX($1+($2-1)*recl:+10+$3),$#                  AX   274
                              QX($1+($2-1)*recl:+3*($4-1)+$3+10))])#    AX   275
#                                                                       AX   276
macro:(incrt:,$1=$1+STEPT)#            Warning: stept is not a macro    AX   277
macro:(temp:,QX($1))#                  General usage                    AX   278
macro:(tfact:,QX($1+2))#               Used to build card constraint    AX   279
macro:(taddr:,QX($1+1))#               Used to build card constraint    AX   280
#                                                                       AX   281
extrude:(AXCOMN)#                                                       AX   282
#                                                                       AX   283
# FLAGS                                                                 AX   284
# -----                                                                 AX   285
#                                                                       AX   286
REAL CORBND#  ## Correlation bound                                      AX   287
INTEGER EXTTYP#    ## Extinction type (>0 gaussian;  <0 lorentzian)     AX   288
#                  ## 1=iso 1; 2=iso 2; 3=gen iso; 4=ani 1; 5=ani 2; 6= AX   289
INTEGER ICENT#     ## yes:=centric cell; no:=acentric cell              AX   290
INTEGER INEU#      ## Flag for data type: 1= neut; 2=elect; 3=x-ray ^   AX   291
INTEGER INTREF#    ## Intensity or |F|**2 refinement                    AX   292
INTEGER INTRES#    ## Intensites & restraints refinement                AX   293
INTEGER IWT#       ## Weighting scheme; 0:w=1; 1:w=1/sigma; 2:w=bdf     AX   294
INTEGER LIST#      ## List reflections: 0=no; n=last; -n=every cycle    AX   295
INTEGER LISTMA#    ## List matrix: 0=no; >0=each; <0=last; 1=inv; 2=d   AX   296
INTEGER LISTCO#    ## List var-covar matrix: no:, yes:                  AX   297
INTEGER LISTRE#    ## List restraints (as list reflections)             AX   298
INTEGER MAXPAR#    ## Max nb of param                                   AX   299
INTEGER NCYCLE#    ## Number of refinement cycles                       AX   300
INTEGER PNCHCD#    ## Punch atom parameters on last cycle: no:/yes:     AX   301
INTEGER REFPOP#    ## Refine pop.par.: 0=no; 1=yes                      AX   302
INTEGER REFTMP#    ## Refinement flag of temp.fact.:                    AX   303
INTEGER RFLMLT#    ## Reflection multiplicity                           AX   304
INTEGER SAVEMA#    ## Save var-covar matrix on last cycle yes:/no:      AX   305
INTEGER TERM#      ## Terminate refinement on this cycle                AX   306
REAL RFLIST#    ## Reflections with weighted delta(f2/i) greater        AX   307
#                  ## THAN RFLIST ARE LISTED                            AX   308
REAL CUTSIG#    ## Less-than bound                                      AX   309
REAL REJECT#    ## Rejection bound                                      AX   310
REAL SIGMAG#    ## Sigma g                                              AX   311
REAL SIGMAR#    ## Sigma r                                              AX   312
#                                                                       AX   313
# BDF CONTROL                                                           AX   314
# -----------                                                           AX   315
#                                                                       AX   316
INTEGER IP#        ## Start of input  packet pointer                    AX   317
INTEGER IPACK#     ## Input packet length                               AX   318
INTEGER IPP#       ## Start of output packet pointer                    AX   319
INTEGER IREL(256)# ## Relative pointers for io packet                   AX   320
INTEGER IWNT(255)# ## To specify wanted items in directory packets      AX   321
INTEGER SIGNAL(4)# ## Group codes for pointpkt:                         AX   322
#                                                                       AX   323
# LIST CONTROL                                                          AX   324
# ------------                                                          AX   325
#                                                                       AX   326
INTEGER MARKA,STOPA#                   List of symop                    AX   327
INTEGER MARKB,STOPB#                   List of symetry transformation   AX   328
INTEGER MARKC,STOPC#                   List of scattering factors       AX   329
INTEGER MARK(7),STOP(7),STEP(7),NB(7),TST(7)# List of all input         AX   330
INTEGER OFFSET(6),REFINE(6)#                                            AX   331
REAL LBND(6),HBND(6)#                  Number of/flag for refinement    AX   332
INTEGER MARK1(7),STOP1(7)#             List of all variables            AX   333
INTEGER MARKX,STOPX#                   List of noref/constraint         AX   334
INTEGER MARKY,STOPY#                   List of constraints packets      AX   335
INTEGER MARKZ,STOPZ#                   List of limshift                 AX   336
INTEGER MARKR(4),STOPR(4)#             List of restraints               AX   337
#                                                                       AX   338
INTEGER MARK20,STOP20#                 Variable list                    AX   339
INTEGER MARK21,STOP21#                 Normal-equation vector           AX   340
INTEGER MARK22,STOP22#                 Normal-equation matrix           AX   341
INTEGER MARK23,STOP23#                 Esd matrix                       AX   342
INTEGER MARK24,STOP24#                 Var-covar matrix                 AX   343
INTEGER MARKP,STOPP,STEPP#             End of reserved space for deri   AX   344
INTEGER MARKT,STOPT#                   Temoporary list                  AX   345
#                                                                       AX   346
INTEGER STEPC#     ## Number of scattering factor intervaax             AX   347
INTEGER STEPR(4)#  ## Increment of restraint list (restraint incr)      AX   348
#                                                                       AX   349
# OTHER VARIABLES                                                       AX   350
# ---------------                                                       AX   351
#                                                                       AX   352
REAL B(3,3),BINV(3,3)#                 Orientation matrix and inverse   AX   353
REAL CELL(12)#  ## Conversion factors; U-Uij & 2pi**2(a*i)(a*j)         AX   354
REAL CORDIS#    ## Correlation between face distances                   AX   355
REAL DAMP#      ## Damping factor for shifts                            AX   356
INTEGER DATSET#    ## Data set number                                   AX   357
REAL DDPAR#     ## Correction to derivatives due to extinction          AX   358
REAL DSFTB#     ## Sine theta over lambda interval for s.f.             AX   359
REAL EDW#       ## Expectation value for weighted difference            AX   360
REAL GOOF#      ## Goodness-of-fit                                      AX   361
INTEGER IFRI#      ## Signal-counter for Friedel pair in twinning ref   AX   362
INTEGER IMAG#      ## Signal-counter for complex s.f. calc.             AX   363
REAL HMAX,KMAX,LMAX#                   Upper bounds                     AX   364
REAL LAMBDA#    ## Radiation wavelength                                 AX   365
REAL MAXSOE#    ## Shift/error max                                      AX   366
REAL SOEAV#   ## Mean of shift/error                                    AX   367
REAL MT(3,3),MTINV(3,3)#   ## Metric tensor and inverse                 AX   368
INTEGER NSKIP#     ## Include each nskip-th reflection in the calc.     AX   369
INTEGER NSYM#      ## Number of symm. operators                         AX   370
REAL POLCON(9)# ## Pojector used to compute polar axes                  AX   371
REAL R#         ## R-value after refinement                             AX   372
REAL RATIO#     ## Monochromator polarization ratio (c = 1/2)           AX   373
REAL SMIN,SMAX# ## Lower and upper bounds                               AX   374
REAL TBAR#      ## Average path length from absorb in mm                AX   375
REAL TRACE#     ## Trace of polcon                                      AX   376
INTEGER VARTOT#    ## Total number of variables, to be refined          AX   377
REAL VOL#       ## Volume of unit cell in direct space                  AX   378
REAL YEXT#      ## Extinction                                           AX   379
REAL XABS,SUXABS## Flack XABS parameter and su                          AX   380
#                                                                       AX   381
COMMON/COMFAX/B,BINV,CELL,CORDIS,CORBND,DAMP,DSFTB,DDPAR,#              AX   382
EDW,GOOF,HMAX,KMAX,LMAX,LAMBDA,MAXSOE,SOEAV,MT,MTINV,#                  AX   383
POLCON,R,RATIO,RFLIST,SMIN,SMAX,TBAR,TRACE,VOL,YEXT,CUTSIG,#            AX   384
REJECT,LBND,HBND,SIGMAG,SIGMAR,XABS,SUXABS#                             AX   385
#                                                                       AX   386
COMMON/COMIX1/DATSET,EXTTYP,ICENT,IFRI,IMAG,INEU,INTREF,INTRES,IWT,#    AX   387
LIST,LISTMA,LISTCO,LISTRE,MAXPAR,NCYCLE,NSKIP,#                         AX   388
NSYM,PNCHCD,REFPOP,REFTMP,RFLMLT,SAVEMA,TERM,VARTOT,#                   AX   389
IP,IPP,IPACK,IREL,IWNT,SIGNAL#                                          AX   390
#                                                                       AX   391
COMMON/COMIX2/MARKA,MARKB,MARKC,MARK,MARK1,STOP,STOP1,NB,TST,#          AX   392
MARKX,MARKY,MARKZ,MARKR,MARK20,MARK21,MARK22,MARK23,MARK24,MARKP,#      AX   393
MARKT,STEP,OFFSET,REFINE,STOPA,STOPB,STOPC,STOPX,STOPY,STOPZ,#          AX   394
STOPR,STOP20,STOP21,STOP22,STOP23,STOP24,STOPP,STEPR,STEPC,STEPP,#      AX   395
STOPT#                                                                  AX   396
endext:#                                                                AX   397
#                                                                       AX   398
systemheader:(AX00)#                   Least squares refinement         AX00   1
####################################################################    AX00   2
#                                                                  #    AX00   3
#                                AX00                              #    AX00   4
#                                                                  #    AX00   5
####################################################################    AX00   6
SUBROUTINE AX00#                                                        AX00   7
include:(AACOMN)#                                                       AX00   8
include:(AXCOMN)#                                                       AX00   9
#                                                                       AX00  10
INTEGER ICYCLE#                        Cycle nb                         AX00  11
#                                                                       AX00  12
chardata:(AX001,[" New value out of bounds- refinement stopped"])#      AX00  13
datastuff:#                                                             AX00  14
#                                                                       AX00  15
CALL AX10#                             Input from cards and from bdf    AX00  16
CALL AX20#                             Output initial var, var count,   AX00  17
DO ICYCLE=0,NCYCLE#                    Refinement loop                  AX00  18
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX00  19
EDW=0.0#                               Clear stat descriptors           AX00  20
CALL AX30(ICYCLE)#                     Build matrix                     AX00  21
CALL AX40(ICYCLE)#                     Invert matrix                    AX00  22
IF (TERM>0)BREAK#                      Terminate refinement             AX00  23
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX00  24
IF (TERM==0)CALL AX50#                 Final output & close shop        AX00  25
ELSE#                                  Refinement stopped               AX00  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX00  27
writeline:(2,AX001,NAX001,3,3)#        Print warning                    AX00  28
readwpkt:(1,endrecord:,IPACK,IP,0)#    Close input bdf                  AX00  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX00  30
RETURN#                                                                 AX00  31
END#                                                                    AX00  32
#                                                                       AX00  33
systemheader:(AX01)#                   Extract param/attyp/atom         AX01   1
####################################################################    AX01   2
#                                                                  #    AX01   3
#                                AX01                              #    AX01   4
#                                                                  #    AX01   5
####################################################################    AX01   6
SUBROUTINE AX01(I,TYPE,NUM,EOF,SPAN)#                                   AX01   7
include:(AACOMN)#                                                       AX01   8
include:(AXCOMN)#                                                       AX01   9
#                                                                       AX01  10
INTEGER EOF#                           End-of-field flag                AX01  11
INTEGER I,J,K#                         Char counter                     AX01  12
INTEGER ILEN#                          String length                    AX01  13
INTEGER L6,L7#                         Atom type & atom pointers        AX01  14
INTEGER MATCH#                         String comparision               AX01  15
string:(NAME1,lena:);string:(NAME2,lena:)#                              AX01  16
INTEGER NUM#                           Returned number of the param/a   AX01  17
INTEGER SPAN#                          Spanning flag                    AX01  18
INTEGER TYPE#                          Type of char string encountere   AX01  19
#                                                                       AX01  20
chardata:(AX011,[ /()0123456789])#                                      AX01  21
chardata:(AX012,[X  Y  Z  U  U11U22U33U12U13U23POP$#                    AX01  22
SHPTWISKFEXTUOVDSPRE IM ])#                                             AX01  23
chardata:(AX013,"yu")#                 Lowercase                        AX01  24
chardata:(AX014,[" WARNING- Parameters y & u may be confused with $#    AX01  25
elements Y & U- lowercase assumed for parameters"])#                    AX01  26
datastuff:#                                                             AX01  27
#                                                                       AX01  28
EOF=no:;SPAN=no:;TYPE=0#               Set initial values               AX01  29
compchar:(CHRIN,I,AX011,1,1,1,NAX011,MATCH)# Compare current char       AX01  30
IF (MATCH==3)I=I+1#                    Take next if bracket             AX01  31
K=I#                                   Set current char as initial ch   AX01  32
compchar:(CHRIN,K,AX011,1,1,1,NAX011,MATCH)# Compare char               AX01  33
IF ((MATCH>0)&(MATCH<=4))iquit:(AX0102)# Exit if delimiter              AX01  34
IF (MATCH>4)#                          Number field                     AX01  35
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX01  36
TYPE=4;NUM=0#                          Set type                         AX01  37
REPEAT#                                Loop over unsigned integer       AX01  38
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX01  39
NUM=10*NUM+MATCH-5;K=K+1#              Increment digit                  AX01  40
compchar:(CHRIN,K,AX011,1,1,1,NAX011,MATCH)# Compare with next          AX01  41
IF (MATCH<=4)BREAK#                    Exit on non-num                  AX01  42
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX01  43
IF (MATCH==0)iquit:(AX0103)#           Exit of end-of-field             AX01  44
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX01  45
ELSE REPEAT#                           Loop over alpha chars            AX01  46
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX01  47
REPEAT#                                Loop over alpha chars            AX01  48
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX01  49
compchar:(CHRIN,K,AX011,1,1,1,NAX011,MATCH)# Compare with char          AX01  50
IF ((MATCH>0)&(MATCH<=4))BREAK#        Exit on delimiter                AX01  51
K=K+1#                                 Take next char                   AX01  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX01  53
TYPE=1#                                Default: param field             AX01  54
ILEN=MIN(K-I,3)#                       Set max length                   AX01  55
compchar:(CHRIN,I,AX012,1,ILEN,3,NAX012,NUM)# Compare with parameters   AX01  56
IF (NUM!=0)#                           May be a parameter               AX01  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX01  58
IF (ILEN==1)#                          Only one char has been tested    AX01  59
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX01  60
IF ((NUM==1)\(NUM==3))BREAK#           X or z                           AX01  61
IF ((NUM==2)\(NUM==4))#                Y or u                           AX01  62
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX01  63
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over atom types             AX01  64
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX01  65
movertoc:(nattyp:(L6),1,NAME2,1,lens:,0)# Extract atom type name        AX01  66
upcase:(NAME2,NAME2,lens:)#            Covert to uppercase              AX01  67
compchar:(CHRIN,I,NAME2,1,1,lens:,lens:,J)# Compare with list           AX01  68
IF (J!=0)BREAK#                        Exit if found                    AX01  69
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX01  70
IF (J!=0)#                             Yttrium or uranium in list       AX01  71
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX01  72
writeline:(1,AX014,NAX014,3,3)#        Print warning                    AX01  73
compchar:(CHRIP,I,AX013,1,1,1,NAX013,NUM)# Compare with y and u         AX01  74
IF (NUM!=0)#                           Coordinate y                     AX01  75
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX01  76
NUM=2*NUM;BREAK#                       Exit with coordinate y or u      AX01  77
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX01  78
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX01  79
ELSE BREAK#                            Not in list, no confusion        AX01  80
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX01  81
NUM=0#                                                                  AX01  82
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX01  83
ELSE BREAK#                            Exit if known                    AX01  84
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX01  85
TYPE=2#                                Default: attype name             AX01  86
ILEN=MIN(K-I,lens:)#                   Set max length                   AX01  87
movechar:(BLANK,1,NAME1,1,lens:,2)#    Clear name1                      AX01  88
movechar:(CHRIP,I,NAME1,1,ILEN,0)#     Copy sf name                     AX01  89
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Compare with atom types          AX01  90
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX01  91
movertoc:(nattyp:(L6),1,NAME2,1,lens:,0)# Extract atom type name        AX01  92
compchar:(NAME1,1,NAME2,1,lens:,lens:,lens:,NUM)# Compare with list     AX01  93
IF (NUM!=0)BREAK#                      Exit if found                    AX01  94
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX01  95
IF (L6<STOP(6))#                       Atom type found in list          AX01  96
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX01  97
NUM=(L6-MARK(6))/STEP(6)+1#            Extract atom type num            AX01  98
BREAK#                                 Exit on found                    AX01  99
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX01 100
TYPE=3#                                Default: atom type name          AX01 101
ILEN=MIN(K-I,lena:)#                   Set max length                   AX01 102
movechar:(BLANK,1,NAME1,1,lena:,2)#    Clear name1                      AX01 103
movechar:(CHRIP,I,NAME1,1,ILEN,0)#     Copy sf name                     AX01 104
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Compare with atom names          AX01 105
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX01 106
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Extract atom name from list    AX01 107
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,NUM)# Compare with card     AX01 108
IF (NUM!=0)BREAK#                      Exit if found                    AX01 109
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX01 110
IF (L7<STOP(7))#                       Found atom in list               AX01 111
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX01 112
NUM=(L7-MARK(7))/STEP(7)+1#            Extract atom num                 AX01 113
BREAK#                                 Exit on found                    AX01 114
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX01 115
IF (MATCH==1)iquit:(AX0101)#           End-of-field & anything found    AX01 116
K=K+1#                                 Set to next char                 AX01 117
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX01 118
IF ((MATCH==2)\(MATCH==4))#            Test for "/" and ")"             AX01 119
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX01 120
IF (MATCH==2)SPAN=yes:#                Turn on spanning after "/"       AX01 121
K=K+1;compchar:(CHRIN,K,AX011,1,1,1,NAX011,MATCH)# Take next char       AX01 122
IF ((SPAN==yes:)&(MATCH==1))iquit:(AX0104)# No end-of-field after spa   AX01 123
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX01 124
IF (MATCH==1)EOF=yes:#                                                  AX01 125
I=K#                                                                    AX01 126
RETURN#                                                                 AX01 127
END#                                                                    AX01 128
#                                                                       AX01 129
systemheader:(AX02)#                   Extract param/attyp/atom         AX02   1
####################################################################    AX02   2
#                                                                  #    AX02   3
#                                AX02                              #    AX02   4
#                                                                  #    AX02   5
####################################################################    AX02   6
SUBROUTINE AX02(I,IMARK,ISTOP,ISTEP,P1,P2,S1,S2)#                       AX02   7
include:(AACOMN)#                                                       AX02   8
include:(AXCOMN)#                                                       AX02   9
#                                                                       AX02  10
INTEGER EOF#                           End-of-field flag                AX02  11
INTEGER I#                             Char counter                     AX02  12
INTEGER J#                             Temp usage                       AX02  13
INTEGER MATCH#                         String comp.                     AX02  14
INTEGER IMARK,ISTOP,ISTEP#             Return addresses                 AX02  15
INTEGER NB1(4),NB2(4)#                                                  AX02  16
INTEGER P1,P2#                         Returned parameters              AX02  17
INTEGER S1,S2#                         Returned nattyp                  AX02  18
INTEGER TYPE1,TYPE2#                   Type of param/attyp/atom         AX02  19
INTEGER SPAN#                          Spanning flag                    AX02  20
#                                                                       AX02  21
intdata:(U,4)#                                                          AX02  22
intdata:(POP,11)#                                                       AX02  23
intdata:(GEN,12)#                                                       AX02  24
intdata:(SHP,12)#                                                       AX02  25
intdata:(POL,13)#                                                       AX02  26
intdata:(SKF,14)#                                                       AX02  27
intdata:(EXT,15)#                                                       AX02  28
intdata:(UOV,16)#                                                       AX02  29
intdata:(DSP,17)#                                                       AX02  30
intdata:(RE,18)#                                                        AX02  31
intdata:(IM,19)#                                                        AX02  32
datastuff:#                                                             AX02  33
#                                                                       AX02  34
S1=0;S2=0#                                                              AX02  35
REPEAT#                                                                 AX02  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX02  37
compchar:(CHRIN,I,BLANK,1,1,1,1,MATCH)#                                 AX02  38
IF (MATCH==0)BREAK#                                                     AX02  39
I=I+1#                                                                  AX02  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX02  41
DO J=1,4;NB1(J)=0#                     Clear first pos                  AX02  42
REPEAT#                                                                 AX02  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX02  44
CALL AX01(I,TYPE1,J,EOF,SPAN);NB1(TYPE1)=J#                             AX02  45
IF (SPAN==yes:)#                                                        AX02  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX02  47
CALL AX01(I,TYPE2,J,EOF,SPAN);NB2(TYPE2)=J#                             AX02  48
IF (SPAN==yes:)iquit:(AX0201)#                                          AX02  49
IF (TYPE1!=TYPE2)iquit:(AX0202)#                                        AX02  50
IF (NB2(TYPE1)<NB1(TYPE1))swap:(NB1(TYPE1),NB2(TYPE1))#                 AX02  51
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX02  52
ELSE NB2(TYPE1)=NB1(TYPE1)#                                             AX02  53
IF (EOF==yes:)BREAK#                                                    AX02  54
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX02  55
IF ((NB1(2)!=0)&(NB1(3)!=0))iquit:(AX0203)# Mixing atom & atom type     AX02  56
IF ((NB1(1)>=GEN)&(NB1(1)<DSP))#       General params excl. dsp, re,    AX02  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX02  58
IF (NB2(1)!=NB1(1))iquit:(AX0204)#     Spanning different classes       AX02  59
IF ((NB1(2)!=0)\(NB1(3)!=0))iquit:(AX0205)# Mixing gen & atom/atom ty   AX02  60
J=NB1(1)-SHP+1#                        Class list number                AX02  61
IF (NB1(4)==0)#                        No number specification          AX02  62
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX02  63
NB1(4)=1;NB2(4)=NB(J)#                 Sepecify all items in class      AX02  64
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX02  65
IMARK=MARK(J)+(NB1(4)-1)*STEP(J);ISTOP=MARK(J)+(NB2(4)-1)*STEP(J)# Se   AX02  66
ISTEP=STEP(J);P1=0;P2=0#               Clear parameters                 AX02  67
IF (NB2(4)>NB(J))iquit:(AX0209)#       Out of bounds                    AX02  68
IF ((NB1(1)==SHP)&(REFINE(1)==0))iquit:(AX0208)# Turn on refinement f   AX02  69
IF ((NB1(1)==POL)&(REFINE(2)<2))iquit:(AX0208)#                         AX02  70
IF ((NB1(1)==EXT)&(REFINE(4)<2))iquit:(AX0208)#                         AX02  71
IF ((NB1(1)==UOV)&(NB1(2)+NB1(3)>0)&(NB2(4)>1))iquit:(AX0210)#          AX02  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX02  73
ELSE IF (NB1(1)>=GEN)#                 Dsp, re, im params               AX02  74
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX02  75
IF (NB1(4)>0)iquit:(AX0206)#           Don't mix with numbers           AX02  76
IF ((NB1(1)==DSP)&(NB1(2)>DSP))iquit:(AX0207)# Don't mix dsp with re    AX02  77
IF (NB1(1)==DSP)#                      Span re and im                   AX02  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX02  79
NB1(1)=RE;NB2(1)=IM#                                                    AX02  80
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX02  81
IMARK=MARK(6)+(NB1(2)-1)*STEP(6);ISTOP=MARK(6)+(NB2(2)-1)*STEP(6)# Se   AX02  82
ISTEP=STEP(6);P1=NB1(1)-RE+1;P2=NB2(1)-RE+1# Set offsets                AX02  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX02  84
ELSE#                                  Atomic parameters                AX02  85
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX02  86
P1=NB1(1);P2=NB2(1)#                   Set parameters bounds            AX02  87
IF ((P1>=POP)&(P2>P1))iquit:(AX0212)#  No span with pop                 AX02  88
IF ((P1==POP)&(REFPOP==no:))iquit:(AX0213)# Turn on ref first           AX02  89
IF (NB1(2)==0)#                        No nattyp as atom name           AX02  90
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX02  91
S1=1;S2=NB(6)#                                                          AX02  92
IF (NB1(3)==0)#                        No atom name                     AX02  93
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX02  94
IMARK=MARK(7);ISTOP=STOP(7)-STEP(7)#                                    AX02  95
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX02  96
ELSE#                                                                   AX02  97
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX02  98
IMARK=MARK(7)+(NB1(3)-1)*STEP(7);ISTOP=MARK(7)+(NB2(3)-1)*STEP(7)#      AX02  99
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX02 100
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX02 101
ELSE#                                  Attyp as atom name               AX02 102
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX02 103
S1=NB1(2);S2=NB2(2)#                                                    AX02 104
IMARK=MARK(7);ISTOP=STOP(7)-STEP(7)#                                    AX02 105
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX02 106
IF (P1>U)P1=P1-1#                                                       AX02 107
IF (P2>U)P2=P2-1#                                                       AX02 108
ISTEP=STEP(7)#                                                          AX02 109
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX02 110
RETURN#                                                                 AX02 111
END#                                                                    AX02 112
#                                                                       AX02 113
systemheader:(AX03)#                                                    AX03   1
####################################################################    AX03   2
#                                                                  #    AX03   3
#                                AX03                              #    AX03   4
#                                                                  #    AX03   5
####################################################################    AX03   6
SUBROUTINE AX03(FIELD,FMUL,L)#                                          AX03   7
#                                                                       AX03   8
include:(AACOMN)#                                                       AX03   9
include:(AXCOMN)#                                                       AX03  10
#                                                                       AX03  11
INTEGER EOF#                           Local end-of-field marker        AX03  12
INTEGER FIELD#                         Field counter                    AX03  13
INTEGER I,J#                           Loop index                       AX03  14
INTEGER L#                             Returned param address           AX03  15
INTEGER L7#                            Atom pointer                     AX03  16
INTEGER NUM(4)#                        Parameter description            AX03  17
REAL FMUL#                             Returned mult fact (default 1.   AX03  18
INTEGER SPAN#                          Local spanning flag              AX03  19
INTEGER TFT#                           Local temp fact type             AX03  20
INTEGER TYPE#                          Type of string(/1:param/2:atom   AX03  21
#                                                                       AX03  22
intdata:(U,4)#                                                          AX03  23
intdata:(POP,11)#                                                       AX03  24
intdata:(GEN,12)#                                                       AX03  25
intdata:(SHP,12)#                                                       AX03  26
intdata:(POL,13)#                                                       AX03  27
intdata:(SKF,14)#                                                       AX03  28
intdata:(EXT,15)#                                                       AX03  29
intdata:(UOV,16)#                                                       AX03  30
intdata:(DSP,17)#                                                       AX03  31
datastuff:#                                                             AX03  32
#                                                                       AX03  33
DO J=1,4;NUM(J)=0#                     Clear first pos                  AX03  34
ifnm:(BUFIN(FIELD))#                                                    AX03  35
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX03  36
FMUL=BUFIN(FIELD);FIELD=FIELD+1#                                        AX03  37
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX03  38
ELSE FMUL=1.0#                                                          AX03  39
stringlim:(FIELD,I,J);FIELD=FIELD+1#                                    AX03  40
CALL AX01(I,TYPE,J,EOF,SPAN);NUM(TYPE)=J#                               AX03  41
IF (EOF==yes:)iquit:(AX0302)#          Not all atom or class            AX03  42
IF (TYPE==4)iquit:(AX0306)#            Not number as first part         AX03  43
CALL AX01(I,TYPE,J,EOF,SPAN);NUM(TYPE)=J#                               AX03  44
IF ((SPAN==yes:)\(EOF==no:))iquit:(AX0301)# No span in constraint       AX03  45
IF ((NUM(2)!=0)&(NUM(3)!=0))iquit:(AX0303)# Not atom type & atom        AX03  46
IF ((NUM(1)>=GEN)&(NUM(1)<DSP))#       General params excl. dsp, re,    AX03  47
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX03  48
IF ((NUM(2)!=0)\(NUM(3)!=0))iquit:(AX0305)# No atom type or atom with   AX03  49
J=NUM(1)-SHP+1#                        Get type nb                      AX03  50
IF (NUM(4)==0)iquit:(AX0304)#          No constr on all params of one   AX03  51
L=MARK(J)+(NUM(4)-1)*STEP(J)#          Get address                      AX03  52
IF (L>=STOP(J))iquit:(AX0309)#         Out of bounds                    AX03  53
IF ((NUM(1)==SHP)&(REFINE(1)==0))iquit:(AX0308)# Turn on ref first      AX03  54
IF ((NUM(1)==POL)&(REFINE(2)<2))iquit:(AX0308)#                         AX03  55
IF ((NUM(1)==EXT)&(REFINE(4)<2))iquit:(AX0308)#                         AX03  56
IF ((NUM(1)==UOV)&(NUM(2)+NUM(3)+NUM(4)>0))iquit:(AX0310)#              AX03  57
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX03  58
ELSE IF (NUM(1)>=GEN)#                 Dsp, re, im params               AX03  59
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX03  60
IF (NUM(1)==DSP)iquit:(AX0307)#        Not dsp in constr                AX03  61
IF (NUM(4)>0)iquit:(AX0306)#           No number as atom type           AX03  62
L=MARK(6)+(NUM(2)-1)*STEP(6)+(NUM(1)-DSP)# Address of param             AX03  63
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX03  64
ELSE#                                  Atomic parameters                AX03  65
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX03  66
L=NUM(1)#                              Get param nb                     AX03  67
IF ((L==POP)&(REFPOP==no:))iquit:(AX0313)#                              AX03  68
IF (NUM(2)==0)#                        No nattyp as atom name           AX03  69
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX03  70
IF (NUM(3)==0)iquit:(AX0311)#          No atom name                     AX03  71
L7=MARK(7)+(NUM(3)-1)*STEP(7)#         Get param address                AX03  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX03  73
ELSE#                                  Attyp as atom name               AX03  74
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over all atoms              AX03  75
IF (NINT(atomsft:(L7))==NUM(2))BREAK#  Exit on first of this type       AX03  76
TFT=NINT(atomtft:(L7))#                Extract displ fact type          AX03  77
IF ((L>U)&(L<POP)&(TFT<2))iquit:(AX0312)# Temp fact type don't coinci   AX03  78
IF ((L==U)&(TFT!=1))iquit:(AX0312)#    Temp fact type don't coincide    AX03  79
IF (L>U)L=L-1#                         U & u11 occupy the same locati   AX03  80
L=L+L7#                                                                 AX03  81
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX03  82
RETURN#                                                                 AX03  83
END#                                                                    AX03  84
#                                                                       AX03  85
systemheader:(AX04)#                                                    AX04   1
####################################################################    AX04   2
#                                                                  #    AX04   3
#                                AX04                              #    AX04   4
#                                                                  #    AX04   5
####################################################################    AX04   6
SUBROUTINE AX04(L,SUBJ,VAL,CODE)#                                       AX04   7
#                                                                       AX04   8
include:(AACOMN)#                                                       AX04   9
include:(AXCOMN)#                                                       AX04  10
#                                                                       AX04  11
INTEGER II#                                                             AX04  12
INTEGER L#                             Position in list                 AX04  13
INTEGER SUBJ#                          Subject parameter address        AX04  14
INTEGER CODE#                          Limsft/noref/constr              AX04  15
REAL VAL#                              Value of limsft                  AX04  16
#                                                                       AX04  17
IF (CODE==limsft:)#                                                     AX04  18
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX04  19
incrqx:(II,STOPZ+stepz:,AX0401)#                                        AX04  20
movereal:(QX,L+1,QX,L+1+stepz:,STOPZ-L,1)#                              AX04  21
bladdr:(L)=REAL(SUBJ);blfact:(L)=VAL#                                   AX04  22
incrz:(STOPZ);incrz:(L)#                                                AX04  23
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX04  24
ELSE#                                                                   AX04  25
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX04  26
incrqx:(II,STOPY+stepx:,AX0402)#                                        AX04  27
movereal:(QX,L+1,QX,L+stepx:+1,STOPY-L,1)#                              AX04  28
ncaddr:(L)=REAL(SUBJ);nctype:(L)=REAL(CODE)#                            AX04  29
incrx:(STOPX);MARKY=STOPX;incrx:(STOPY)#                                AX04  30
incrx:(L)#                                                              AX04  31
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX04  32
RETURN#                                                                 AX04  33
END#                                                                    AX04  34
#                                                                       AX04  35
systemheader:(AX05)#                                                    AX05   1
####################################################################    AX05   2
#                                                                  #    AX05   3
#                                AX05                              #    AX05   4
#                                                                  #    AX05   5
####################################################################    AX05   6
SUBROUTINE AX05(L,CODE)#                                                AX05   7
#                                                                       AX05   8
include:(AACOMN)#                                                       AX05   9
include:(AXCOMN)#                                                       AX05  10
#                                                                       AX05  11
INTEGER CODE#                          Noref = no:,  ref = yes:         AX05  12
INTEGER L#                                                              AX05  13
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX05  14
#                                                                       AX05  15
IF (CODE==yes:)#                       Ref                              AX05  16
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX05  17
findc:(MARKX,MARKY,LX,LY,L)#           Get pointers                     AX05  18
ifnc:(LX,L)#                           Remove subj param                AX05  19
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX05  20
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX05  21
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX05  22
MY=LY;WHILE (caddr:(MY)>small:)incry:(MY);incry:(MY)# Next constraint   AX05  23
movereal:(QX,MY+1,QX,LY+1,STOPY-MY,1)# Krunch list                      AX05  24
STOPY=STOPY-(MY-LY)#                   Update end-of-list marker        AX05  25
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX05  26
movereal:(QX,LX+stepx:+1,QX,LX+1,STOPY-LX-stepx:,1)# Krunch list        AX05  27
STOPX=STOPX-stepx:;MARKY=MARKY-stepx:;STOPY=STOPY-stepx:# Update mark   AX05  28
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX05  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX05  30
ELSE#                                  Noref                            AX05  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX05  32
findnc:(MARKX,LX,L)#                   Get pointers                     AX05  33
ifnc:(LX,L)iquit:(AX0501)#             Already constrained              AX05  34
putnc:(LX,L,noref:)#                   Put invariant param              AX05  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX05  36
RETURN#                                                                 AX05  37
END#                                                                    AX05  38
#                                                                       AX05  39
systemheader:(AX06)#                   Search item in directory         AX06   1
######################################################################  AX06   2
#                                                                       AX06   3
# SEARCH DIRECTORY ROUTINE:                                             AX06   4
# -------------------------------------------------------------------   AX06   5
# THIS ROUTINE EXTRACT THE OFFSET POINTER OF AN ITEM IN A DIRECTORY     AX06   6
# PACKET.                                                               AX06   7
# IT IS USED TO WRITE AN ITEM ON THE BDF ONLY IF IT IS MISSING BEFORE   AX06   8
# READING THE FILE.                                                     AX06   9
# FOR INSTANCE: WE WANT TO WRITE THE WAVELENGTH ON THE BDF ONLY IF      AX06  10
# THE BDF DOESN'T CONTAIN LAMBDA. THEN WE HAVE TO PUT THE LAMBDA IDN    AX06  11
# IN THE UPDATE PACKET. BUT IN THIS CASE, THE IREL POINTER FOR LAMBDA   AX06  12
# WILL ALWAYS BE POSITIVE. SO WE CAN'T KNOW IF THE WAVELENGTH WAS       AX06  13
# ALREADY ON BDF OR NOT. THE ONLY WAY TO KNOW IT IS TO SEEK IN THE      AX06  14
# INPUT DIRECTORY PACKET.                                               AX06  15
#                                                                       AX06  16
######################################################################  AX06  17
SUBROUTINE AX06(IP0,PSIZE0,IDN,PTR)#                                    AX06  18
#                                                                       AX06  19
include:(AACOMN)#                                                       AX06  20
include:(AXCOMN)#                                                       AX06  21
#                                                                       AX06  22
REAL AIDN#                             Real idn value                   AX06  23
INTEGER I#                             Loop index                       AX06  24
INTEGER IDN#                           Integer idn value                AX06  25
INTEGER PTR#                           Returned offset pointer in inp   AX06  26
INTEGER IP0,PSIZE0#                    Set to ip and psize              AX06  27
#                                                                       AX06  28
AIDN=REAL(IDN)#                        Real value                       AX06  29
PTR=1#                                 Reset offset pointer             AX06  30
DO I=IP0+1,IP0+PSIZE0#                 Loop over directory              AX06  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX06  32
ifeq:(QX(I),AIDN)BREAK#                Exit if match                    AX06  33
PTR=PTR+1#                             Increment offset pointer         AX06  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX06  35
IF(PTR>PSIZE0)PTR=0#                   Match not found                  AX06  36
RETURN#                                                                 AX06  37
END#                                                                    AX06  38
#                                                                       AX06  39
systemheader:(AX09)#                   Print bdf constraint             AX09   1
#################################################################       AX09   2
#                                                               #       AX09   3
#                               AX09                            #       AX09   4
#                                                               #       AX09   5
#################################################################       AX09   6
SUBROUTINE AX09(NREF)#                                                  AX09   7
#                                                                       AX09   8
include:(AACOMN)#                                                       AX09   9
include:(AXCOMN)#                                                       AX09  10
#                                                                       AX09  11
INTEGER COL#                           Column pointer in chrot          AX09  12
REAL FACT#                             Multiplicative factor            AX09  13
INTEGER I,J#                           Loop index and parameter nb      AX09  14
INTEGER NREF#                          Max number of reference parame   AX09  15
#                                                                       AX09  16
chardata:(AX096,[X  Y  Z  U11U22U33U12U13U23POP])#                      AX09  17
chardata:(AX097,[+-()=*])#                                              AX09  18
intdata:(RANGE,arith:(14,+,lena:))#                                     AX09  19
realdata:(FMT,[90842.,]arith:(24,+,lena:)0842.[,0.0])#                  AX09  20
datastuff:#                                                             AX09  21
#                                                                       AX09  22
LINRM=1#                                                                AX09  23
BUFOT(1)=QX(IP+IREL(3));BUFOT(2)=QX(IP+IREL(4))# Save constants         AX09  24
movechar:(AX097,6,CHROT,10,1,0)#       Move *                           AX09  25
J=NINT(QX(IP+IREL(2)))#                Subject parameter                AX09  26
movechar:(AX096,3*(J-1)+1,CHROT,11,3,0)# Move subject parameter         AX09  27
movechar:(AX097,3,CHROT,14,1,0)#       Move (                           AX09  28
movertoc:(QX(IP+IREL(1)),1,CHROT,15,lena:,0)# Move subject atom name    AX09  29
movechar:(AX097,4,CHROT,15+lena:,2,0)# Move )=                          AX09  30
ncodefld:(BUFOT,1,CHROT,FMT,2)#        Move subj mult fact & cst        AX09  31
COL=25+lena:#                          Update column counter            AX09  32
I=0#                                   Reset reference counter          AX09  33
REPEAT#                                Loop over reference part of co   AX09  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX09  35
IF(I>=NREF)BREAK#                      Exit if end-of-constraint        AX09  36
FACT=QX(IP+IREL(5+3*I+3))#             Get multiplicative factor        AX09  37
ifeq:(FACT,0.0)BREAK#                  Exit if 0                        AX09  38
IF(FACT<0.0)movechar:(AX097,2,CHROT,COL,1,0)# Move sign                 AX09  39
ELSE movechar:(AX097,1,CHROT,COL,1,0)#                                  AX09  40
FMT(3)=(COL+7)*10000.+742.#            Get format                       AX09  41
BUFOT(1)=ABS(FACT)#                    Print mult fact                  AX09  42
movechar:(AX097,6,CHROT,COL+8,1,0)#    Move *                           AX09  43
J=NINT(QX(IP+IREL(5+3*I+2)))#          Get reference param              AX09  44
movechar:(AX096,3*(J-1)+1,CHROT,COL+9,3,0)# Move ref param              AX09  45
movechar:(AX097,3,CHROT,COL+12,1,0)#   Move (                           AX09  46
movertoc:(QX(IP+IREL(5+3*I+1)),1,CHROT,COL+13,lena:,0)# Move ref atom   AX09  47
COL=COL+RANGE#                         Update column                    AX09  48
I=I+1#                                 Increment packet counter         AX09  49
movechar:(AX097,4,CHROT,COL-1,1,0)#    Move )                           AX09  50
ncodefld:(BUFOT,1,CHROT,FMT(3),1)#     Code numbers                     AX09  51
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX09  52
writeline:(0,0,0,1,3)#                 Print constraint                 AX09  53
RETURN#                                                                 AX09  54
END#                                                                    AX09  55
#                                                                       AX09  56
systemheader:(AX10)#                   Input cards and parameters       AX10   1
####################################################################    AX10   2
#                                                                  #    AX10   3
#                                AX10                              #    AX10   4
#                                                                  #    AX10   5
####################################################################    AX10   6
SUBROUTINE AX10#                                                        AX10   7
#                                                                       AX10   8
include:(AACOMN)#                                                       AX10   9
include:(AXCOMN)#                                                       AX10  10
#                                                                       AX10  11
CALL AX11#              ## LSLS                                         AX10  12
CALL AX12#              ## lrcell:, lrsymm:, lrscat:, lrexpl:, lrdset   AX10  13
CALL AX13#              ## MAXHKL                                       AX10  14
CALL AX14#              ## lratom:                                      AX10  15
CALL AX15#              ## CONSTR, NOREF, REF                           AX10  16
CALL AX16#              ## lrcons:                                      AX10  17
CALL AX17#              ## Self-consistency of constraint list          AX10  18
CALL AX18#              ## LIMSFT cards                                 AX10  19
RETURN#                                                                 AX10  20
END#                                                                    AX10  21
#                                                                       AX10  22
systemheader:(AX11)#                   Lsls line                        AX11   1
####################################################################    AX11   2
#                                                                  #    AX11   3
#                               AX11                               #    AX11   4
#                                                                  #    AX11   5
####################################################################    AX11   6
SUBROUTINE AX11#                                                        AX11   7
include:(AACOMN)#                                                       AX11   8
include:(AXCOMN)#                                                       AX11   9
#                                                                       AX11  10
INTEGER FIELD,IFIRST,ILAST#            String limits of field           AX11  11
INTEGER I#                             General usage                    AX11  12
INTEGER MATCH#                         Used in compchar:                AX11  13
#                                                                       AX11  14
chardata:(AX000," LSLS Control Parameters")#                            AX11  15
chardata:(AX001," Data Set                             1")#             AX11  16
chardata:(AX002," Number of Cycles                     1")#             AX11  17
chardata:(AX027," Refinement Type                     IN")#             AX11  18
chardata:(AX005," Temp.Fact.Type                      MX")#             AX11  19
chardata:(AX006," Weighting Scheme                    WS")#             AX11  20
chardata:(AX008," Refine Scale Factor                 US")#             AX11  21
chardata:(AX009," Refine Crystal Dim.                 No")#             AX11  22
chardata:(AX010," Refine Pop. Parameter               No")#             AX11  23
chardata:(AX061," Dispersion                       Apply")#             AX11  24
chardata:(AX012," Extinction                    Not Used")#             AX11  25
chardata:(AX013," Twinning Parameters           Not Used")#             AX11  26
chardata:(AX014," Skip Reflection                     No")#             AX11  27
chardata:(AX015," Damping Factor                        ")#             AX11  28
chardata:(AX026," Correlation between Distances         ")#             AX11  29
chardata:(AX016," Save Matrix                    $#                     AX11  30
No                              ")#                                     AX11  31
chardata:(AX017," List Normal-Equations Matrix   $#                     AX11  32
No                                                                  ")# AX11  33
chardata:(AX018," List Var-Cov Matrix            $#                     AX11  34
No                                                                  ")# AX11  35
chardata:(AX020," List Reflections               $#                     AX11  36
No            ")#                                                       AX11  37
chardata:(AX021," Punch Parameter Cards          $#                     AX11  38
No            ")#                                                       AX11  39
chardata:(AX022," List Poor Reflections          $#                     AX11  40
If w*Delta[>]                     ")#                                   AX11  41
chardata:(AX023," List Correlations              $#                     AX11  42
No            ")#                                                       AX11  43
chardata:(AX024," Refinement on           Restraint only")#             AX11  44
chardata:(AX028," List Restraints                $#                     AX11  45
No            ")#                                                       AX11  46
chardata:(AX031," If Crystal not Convex then     $#                     AX11  47
Terminate Run ")#                                                       AX11  48
chardata:(AX032,[" If Twin Ratio <0 or >1 then   $#                     AX11  49
 Print Message "])#                                                     AX11  50
chardata:(AX033," If Scale on (I) negative then  $#                     AX11  51
Terminate Run ")#                                                       AX11  52
chardata:(AX035," If U overall negative then     $#                     AX11  53
Terminate Run ")#                                                       AX11  54
chardata:(AX036," If Isotropic Extinc param then $#                     AX11  55
Terminate Run ")#                                                       AX11  56
chardata:(AX037," If non-positive TF then        $#                     AX11  57
Terminate Run ")#                                                       AX11  58
chardata:(AX051,"Yes Not Used   Apply and Refine")#                     AX11  59
chardata:(AX053,["Direct & Inverted, Last Cycle Every Cycle"])#         AX11  60
chardata:(AX054,"If |cor|[>]X.XXX")#                                    AX11  61
chardata:(AX056,"Reset Value   Print Message Terminate Run ")#          AX11  62
chardata:(AX041,[" Do not use refl in matrix      $#                    AX11  63
If I/sigma(I)    <                "])#                                  AX11  64
chardata:(AX042,[" Do not use refl in matrix      $#                    AX11  65
If w*(Io-Ic)     >                "])#                                  AX11  66
chardata:(CODE,[DS CY OV IS AN MX WU WS WW US MS RS PP ND AD RD $#      AX11  67
NX AX RX NP AP RP SK FU SM LM LC LL LE PL CD PC LP CS RJ RO SR SP SS $# AX11  68
ER EP ES KR KP KS OR OP OS TR TP TS XR XP XS F2 IN TL TE FM ])#         AX11  69
realdata:(FMT,[390313.,390642.,460532.,510842., 590842.])#              AX11  70
datastuff:#                                                             AX11  71
#                                                                       AX11  72
# SET DEFAULT VALUES                                                    AX11  73
# ------------------                                                    AX11  74
#                                                                       AX11  75
DATSET=1#                              Default data set number          AX11  76
NCYCLE=1#                              Default cycle number             AX11  77
INTREF=yes:#                           Refinement on intensities        AX11  78
REFTMP=3#                              Mixed temp refinement            AX11  79
IWT=1#                                 Weight: 1/sigma                  AX11  80
REFPOP=no:#                            Do not refine pop param          AX11  81
NSKIP=1#                               Do not skip reflections          AX11  82
DAMP=1.0#                              Damping factor                   AX11  83
CORDIS=0.0#                            No correlation                   AX11  84
SAVEMA=no:#                            Do not save matrix               AX11  85
LISTMA=0#                              Do not list matrix               AX11  86
LISTCO=no:#                            Do not list var-covar matrix     AX11  87
LIST=0#                                Do not list reflections          AX11  88
LISTRE=0#                              Do not list restraints           AX11  89
PNCHCD=no:#                            No punch cards                   AX11  90
INTRES=yes:#                           Refine intensities               AX11  91
CORBND=1.0#                            Print cov if corr > corbnd       AX11  92
RFLIST =large:#                        Poor reflection bound            AX11  93
TERM=no:#                              Do not end refinement            AX11  94
CUTSIG=-large:#                        No less-than                     AX11  95
REJECT=large:#                         Use all reflections              AX11  96
REFINE(1)=0;NB(1)=0;TST(1)=3;LBND(1)=0.0;HBND(1)=large:# Distances      AX11  97
REFINE(2)=0;NB(2)=1;TST(2)=2;LBND(2)=0.0;HBND(2)=1.0# Ent-pol           AX11  98
REFINE(3)=1;NB(3)=1;TST(3)=3;LBND(3)=0.0;HBND(3)=large:# Scale          AX11  99
REFINE(4)=0;NB(4)=14;TST(4)=3;LBND(4)=0.0;HBND(4)=large:# Extinction    AX11 100
REFINE(5)=2;NB(5)=1;TST(5)=3;LBND(5)=0.0;HBND(5)=large:# U overall      AX11 101
REFINE(6)=1;NB(6)=0;TST(6)=0;LBND(6)=0.0;HBND(6)=0.0# Dispersion        AX11 102
NB(7)=0;TST(7)=3#                      Atoms                            AX11 103
#                                                                       AX11 104
DDPAR=1.0#                             Extinction derivative fudge      AX11 105
#                                                                       AX11 106
# EXTRACT VALUES FROM LSLS CARD                                         AX11 107
# -----------------------------                                         AX11 108
#                                                                       AX11 109
FIELD=0#                               First code field                 AX11 110
REPEAT#                                Loop over code fields            AX11 111
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX11 112
FIELD=FIELD+1#                                                          AX11 113
IF(BUFIN(FIELD)>=voidflg:)BREAK#       Exit on last field               AX11 114
stringlim:(FIELD,IFIRST,ILAST)#        Extract code limits              AX11 115
compchar:(CHRIN,IFIRST,CODE,1,2,3,NCODE,MATCH)# Compare with codes      AX11 116
IF (MATCH==0)iquit:(AX1199)#           Unrecogized input code           AX11 117
IF (MATCH==1)#                         Data set                         AX11 118
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX11 119
FIELD=FIELD+1#                         Take next field                  AX11 120
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if non numeric          AX11 121
DATSET=NINT(BUFIN(FIELD))#             Save dataset number              AX11 122
IF (DATSET<=0)iquit:(AX1101)#          Exit if non positive data set    AX11 123
ncodefld:(BUFIN(FIELD),1,AX001,FMT(1),1,1)# Code data set number        AX11 124
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX11 125
IF (MATCH==2)#                         Cycles                           AX11 126
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX11 127
FIELD=FIELD+1#                         Take next field                  AX11 128
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if non numeric          AX11 129
NCYCLE=NINT(BUFIN(FIELD))#             Save field number                AX11 130
IF (NCYCLE<0)iquit:(AX1102)#           Exit if non positive             AX11 131
ncodefld:(BUFIN(FIELD),1,AX002,FMT(1),1,1)# Code cycles                 AX11 132
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX11 133
IF ((MATCH>=3)&(MATCH<=6))#            Displacement factor type         AX11 134
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX11 135
REFTMP=MATCH-3#                        0:ovl, 1:iso, 2:ani, 3:mxd       AX11 136
movechar:(CODE,(MATCH-1)*3+1,AX005,38,2,0)# Move code                   AX11 137
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX11 138
IF ((MATCH>=7)&(MATCH<=9))#            Weighting scheme                 AX11 139
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX11 140
IWT=MATCH-7#                           0:unit, 1:1/sig**2, 2:form bdf   AX11 141
movechar:(CODE,(MATCH-1)*3+1,AX006,38,2,0)# Move code                   AX11 142
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX11 143
IF ((MATCH>=10)&(MATCH<=11))#          Scaling                          AX11 144
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX11 145
REFINE(3)=MATCH-9#                     1:together, 2:separate           AX11 146
movechar:(CODE,(MATCH-1)*3+1,AX008,38,2,0)# Move code                   AX11 147
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX11 148
IF (MATCH==12)#                        Refine crystal shape             AX11 149
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX11 150
REFINE(1)=2#                           0:do not, 2:do                   AX11 151
movechar:(AX051,(REFINE(1)-2)*3+1,AX009,37,3,0)# Move "yes"             AX11 152
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX11 153
IF (MATCH==13)#                        Population parameter             AX11 154
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX11 155
REFPOP=yes:#                           0:do not, 1:do                   AX11 156
movechar:(AX051,(REFPOP-1)*3+1,AX010,37,3,0)# Move "yes"                AX11 157
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX11 158
IF ((MATCH>=14)&(MATCH<=16))#          Dispersion                       AX11 159
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX11 160
REFINE(6)=MATCH-14#                    0:not used, 1:apply, 2:refine    AX11 161
IF (REFINE(6)==0)movechar:(AX051,5,AX061,32,8,0)# Not used              AX11 162
IF (REFINE(6)==1)movechar:(AX051,13,AX061,32,8,0)# Not used             AX11 163
IF (REFINE(6)==2)movechar:(AX051,16,AX061,24,16,0)# Not used            AX11 164
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX11 165
IF ((MATCH>=17)&(MATCH<=19))#          Extinction                       AX11 166
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX11 167
REFINE(4)=MATCH-17#                    0:not used, 1:apply, 2:refine    AX11 168
IF (REFINE(4)==0)movechar:(AX051,5,AX012,32,8,0)# Not used              AX11 169
IF (REFINE(4)==1)movechar:(AX051,13,AX012,32,8,0)# Not used             AX11 170
IF (REFINE(4)==2)movechar:(AX051,16,AX012,24,16,0)# Not used            AX11 171
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX11 172
IF ((MATCH>=20)&(MATCH<=22))#          Enantiomorph-polarity            AX11 173
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX11 174
REFINE(2)=MATCH-20#                    0:not used, 1:apply, 2:refine    AX11 175
IF (REFINE(2)==0)movechar:(AX051,5,AX013,32,8,0)# Not used              AX11 176
IF (REFINE(2)==1)movechar:(AX051,13,AX013,32,8,0)# Not used             AX11 177
IF (REFINE(2)==2)movechar:(AX051,16,AX013,24,16,0)# Not used            AX11 178
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX11 179
IF (MATCH==23)#                        Skip reflections                 AX11 180
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX11 181
FIELD=FIELD+1#                         Take next field                  AX11 182
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if non numeric          AX11 183
NSKIP=NINT(BUFIN(FIELD))#              Save skipping range              AX11 184
IF (NSKIP<1)iquit:(AX1114)#            Exit if non positive             AX11 185
ncodefld:(BUFIN(FIELD),1,AX014,FMT(1),1,1)# Code skip                   AX11 186
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX11 187
IF (MATCH==24)#                        Damping factor                   AX11 188
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX11 189
FIELD=FIELD+1#                         Take next field                  AX11 190
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if non numeric          AX11 191
DAMP=BUFIN(FIELD)#                     Save damping factor              AX11 192
IF (DAMP<=0.0)iquit:(AX1115)#          Exit if not positive             AX11 193
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX11 194
IF (MATCH==25)#                        Save matrix                      AX11 195
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX11 196
SAVEMA=yes:#                           Set flag                         AX11 197
movechar:(AX051,1,AX016,33,3,0)#       Move "yes"                       AX11 198
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX11 199
IF (MATCH==26)#                        List matrix                      AX11 200
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX11 201
FIELD=FIELD+1#                         Take next field                  AX11 202
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 203
LISTMA=NINT(BUFIN(FIELD))#             List matrix scheme               AX11 204
IF (IABS(LISTMA)>2)iquit:(AX1117)#     Unsupported scheme               AX11 205
I=33#                                  Set column                       AX11 206
IF (IABS(LISTMA)==2)#                  Direct & inverted                AX11 207
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX11 208
movechar:(AX053,1,AX017,I,19,0)#       Move "direct & inverted"         AX11 209
I=I+19#                                                                 AX11 210
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX11 211
ELSE#                                                                   AX11 212
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX11 213
movechar:(AX053,10,AX017,I,10,0)#      Move "inverted"                  AX11 214
I=I+10#                                                                 AX11 215
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX11 216
IF (LISTMA>0)movechar:(AX053,31,AX017,I,11,0)# Every                    AX11 217
ELSE movechar:(AX053,20,AX017,I,10,0)# Last                             AX11 218
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX11 219
IF (MATCH==27)#                        List var-covar matrix            AX11 220
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX11 221
LISTCO=yes:#                           List matrix scheme               AX11 222
movechar:(AX053,1,AX018,33,19,0)#      Move "direct & inverted"         AX11 223
movechar:(AX053,20,AX018,52,10,0)#     Last                             AX11 224
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX11 225
IF ((MATCH>=28)&(MATCH<=29))#          List reflections                 AX11 226
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX11 227
LIST=100000#                           Large number of reflections to   AX11 228
ifnm:(BUFIN(FIELD+1))#                 Numerical field                  AX11 229
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX11 230
FIELD=FIELD+1#                         Next field                       AX11 231
LIST=NINT(BUFIN(FIELD))#               Extract number of reflections    AX11 232
IF (LIST<=0)iquit:(AX1128)#            Only positive number             AX11 233
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX11 234
IF (MATCH==29)LIST=-LIST#              Every cycle                      AX11 235
IF (LIST>0)movechar:(AX053,20,AX020,33,10,0)# Last                      AX11 236
ELSE movechar:(AX053,31,AX020,33,11,0)# Every                           AX11 237
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX11 238
IF (MATCH==30)#                        Punch cards                      AX11 239
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX11 240
PNCHCD=yes:#                           Set flag                         AX11 241
movechar:(AX053,20,AX021,33,10,0)#     Move "last"                      AX11 242
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX11 243
IF (MATCH==31)#                        Correlation between dist obs     AX11 244
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX11 245
FIELD=FIELD+1#                         Take next field                  AX11 246
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 247
CORDIS=BUFIN(FIELD)#                   Save correlation                 AX11 248
IF (ABS(CORDIS)>1.0)iquit:(AX1126)#    Exit if greater than 1           AX11 249
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX11 250
IF (MATCH==32)#                        Print large correlation bound    AX11 251
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX11 252
FIELD=FIELD+1#                         Take next field                  AX11 253
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 254
CORBND=BUFIN(FIELD)#                   Save bound                       AX11 255
IF ((CORBND>1.0)\(CORBND<0.0))iquit:(AX1123)# Exit if <0 or >1          AX11 256
movechar:(AX054,1,AX023,33,9,0)#       Move bound                       AX11 257
ncodefld:(CORBND,1,AX023,FMT(3),1)#    Code bound                       AX11 258
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX11 259
IF (MATCH==33)#                        List poor reflections            AX11 260
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX11 261
FIELD=FIELD+1#                         Take next field                  AX11 262
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 263
RFLIST=BUFIN(FIELD)#                   Save bound                       AX11 264
IF (RFLIST<0.0)iquit:(AX1130)#         Exit if negative                 AX11 265
ncodefld:(RFLIST,1,AX022,FMT(4),1)#    Code bound                       AX11 266
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX11 267
IF (MATCH==34)#                        Less-than bound                  AX11 268
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX11 269
FIELD=FIELD+1#                         Take next field                  AX11 270
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 271
CUTSIG=BUFIN(FIELD)#                   Save bound                       AX11 272
ncodefld:(CUTSIG,1,AX041,FMT(5),1)#    Code bound                       AX11 273
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX11 274
IF (MATCH==35)#                        Rejection bound                  AX11 275
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX11 276
FIELD=FIELD+1#                         Take next field                  AX11 277
IF (BUFIN(FIELD)<=voidflg:)iquit:(AX1100)# Exit if not numeric          AX11 278
REJECT=BUFIN(FIELD)#                   Save bound                       AX11 279
ncodefld:(REJECT,1,AX042,FMT(5),1)#    Code bound                       AX11 280
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX11 281
IF (MATCH==36)INTRES=no:#              Refinement on restraints only    AX11 282
IF ((MATCH>=37)&(MATCH<=39))#          Not pos distances                AX11 283
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX11 284
TST(1)=MATCH-36#                       1:reset, 2:print, 3:stop         AX11 285
IF (TST(1)==1)movechar:(AX056,1,AX031,33,14,0)# Reset                   AX11 286
IF (TST(1)==2)movechar:(AX056,15,AX031,33,14,0)# Print                  AX11 287
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX11 288
IF ((MATCH>=40)&(MATCH<=42))#          Not pos entpol                   AX11 289
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX11 290
TST(2)=MATCH-39#                       1:reset, 2:print, 3:stop         AX11 291
IF (TST(2)==3)movechar:(AX056,1,AX032,33,14,0)# Stop                    AX11 292
IF (TST(2)==2)movechar:(AX056,29,AX032,33,14,0)# Print                  AX11 293
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX11 294
IF ((MATCH>=43)&(MATCH<=45))#          Not pos scales                   AX11 295
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX11 296
TST(3)=MATCH-40#                       1:reset, 2:print, 3:stop         AX11 297
IF (TST(3)==1)movechar:(AX056,1,AX032,33,14,0)# Reset                   AX11 298
IF (TST(3)==2)movechar:(AX056,15,AX032,33,14,0)# Print                  AX11 299
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX11 300
IF ((MATCH>=46)&(MATCH<=48))#          Not pos u overall                AX11 301
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX11 302
TST(5)=MATCH-45#                       1:reset, 2:print, 3:stop         AX11 303
IF (TST(5)==1)movechar:(AX056,1,AX032,33,14,0)# Reset                   AX11 304
IF (TST(5)==2)movechar:(AX056,15,AX032,33,14,0)# Print                  AX11 305
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX11 306
IF ((MATCH>=49)&(MATCH<=51))#          Non-pos-def displacement facto   AX11 307
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX11 308
TST(7)=MATCH-48#                       1:reset, 2:print, 3:stop         AX11 309
IF (TST(7)==1)movechar:(AX056,1,AX037,33,14,0)# Reset                   AX11 310
IF (TST(7)==2)movechar:(AX056,15,AX037,33,14,0)# Print                  AX11 311
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX11 312
IF ((MATCH>=52)&(MATCH<=54))#          Extinction < 0                   AX11 313
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX11 314
TST(4)=MATCH-51#                       1:reset, 2:print, 3:stop         AX11 315
IF (TST(4)==1)movechar:(AX056,1,AX036,33,14,0)# Reset                   AX11 316
IF (TST(4)==2)movechar:(AX056,15,AX036,33,14,0)# Print                  AX11 317
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX11 318
IF ((MATCH>=55)&(MATCH<=56))#          Refinement type                  AX11 319
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX11 320
INTREF=MATCH-55#                       Intef=no:(0) or intref=yes:(1)   AX11 321
movechar:(CODE,(MATCH-1)*3+1,AX027,38,2,0)# Move code                   AX11 322
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX11 323
IF ((MATCH>=57)&(MATCH<=58))#          List restraints                  AX11 324
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX11 325
LISTRE=100000#                         Large number of reflections to   AX11 326
ifnm:(BUFIN(FIELD+1))#                 Numerical field                  AX11 327
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX11 328
FIELD=FIELD+1#                         Next field                       AX11 329
LISTRE=NINT(BUFIN(FIELD))#             Extract number of reflections    AX11 330
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX11 331
IF (MATCH==58)LISTRE=-LISTRE#          Every cycle                      AX11 332
IF (LISTRE>0)movechar:(AX053,20,AX028,33,10,0)# Last                    AX11 333
ELSE         movechar:(AX053,31,AX028,33,11,0)# Every                   AX11 334
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX11 335
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX11 336
#                                                                       AX11 337
#    UPDATE CONTROL PARAMETERS                                          AX11 338
#    -------------------------                                          AX11 339
#                                                                       AX11 340
IF ((INTREF==no:)&(REFINE(1)>0))iquit:(AX1127)# Intensities mandatory   AX11 341
IF ((INTRES==no:)&(REFINE(1)>0))iquit:(AX1131)# Intensities mandatory   AX11 342
IF ((INTRES==no:)&(REFPOP==yes:))iquit:(AX1133)# I or |f|**2 to refin   AX11 343
IF ((ABS(CORDIS)>small:)&(REFINE(1)==0))iquit:(AX1132)# Distance ref    AX11 344
IF ((LISTCO==yes:)&(IWT==1)&(CORDIS==0.0))iquit:(AX1118)#               AX11 345
ncodefld:(DAMP,1,AX015,FMT(2),1)#      Code damping                     AX11 346
ncodefld:(CORDIS,1,AX026,FMT(2),1)#    Code correlation between dista   AX11 347
#                                                                       AX11 348
#    PRINT CONTROL PARAMETERS                                           AX11 349
#    ------------------------                                           AX11 350
#                                                                       AX11 351
LINRM=24#                              Reserve 24 lines                 AX11 352
IF (REFINE(1)>0)LINRM=LINRM+1#         Reset/print/term shape           AX11 353
IF (REFINE(2)>0)LINRM=LINRM+1#         Reset/print/term ent-pol         AX11 354
IF (IWT!=1)LINRM=LINRM+2#              2 lines more                     AX11 355
IF (RFLIST<large:)LINRM=LINRM+1#       One line more                    AX11 356
IF (CUTSIG>-large:)LINRM=LINRM+1#      One line more                    AX11 357
IF (REJECT<large:)LINRM=LINRM+1#       One line more                    AX11 358
IF (INTRES==no:)LINRM=LINRM+1#         One line more                    AX11 359
movechar:('-',1,CHROT,2,NAX000-1,1)#   Store underline                  AX11 360
writeline:(1,AX000,NAX000,2,3)#        Print title                      AX11 361
writeline:(0,AX001,NAX001,3,3)#        Print data set                   AX11 362
writeline:(0,AX002,NAX002,3,3)#        Print cycles                     AX11 363
writeline:(0,AX027,NAX027,3,3)#        Print refinement type            AX11 364
writeline:(0,AX005,NAX005,3,3)#        Print temp fact type             AX11 365
writeline:(0,AX006,NAX006,3,3)#        Print weighting scheme           AX11 366
writeline:(0,AX008,NAX008,3,3)#        Print scale fact ref type        AX11 367
writeline:(0,AX009,NAX009,3,3)#        Print shape refinement type      AX11 368
writeline:(0,AX010,NAX010,3,3)#        Print pop param                  AX11 369
writeline:(0,AX061,NAX061,3,3)#        Print apply disp                 AX11 370
writeline:(0,AX012,NAX012,3,3)#        Print apply ext                  AX11 371
writeline:(0,AX013,NAX013,3,3)#        Print apply entpol               AX11 372
writeline:(0,AX014,NAX014,3,3)#        Print skip range                 AX11 373
writeline:(0,AX015,NAX015,3,3)#        Print damping factor             AX11 374
IF (INTRES==no:)writeline:(0,AX024,NAX024,3,3)# Print restraints only   AX11 375
IF (REFINE(1)!=no:)writeline:(0,AX026,NAX026,3,3)# Print correlatio     AX11 376
writeline:(1,AX016,NAX016,3,3)#        Print save matrix                AX11 377
writeline:(0,AX017,NAX017,3,3)#        Print list matrix                AX11 378
IF ((IWT!=1)\(CORDIS>0))writeline:(0,AX018,NAX018,3,3)# Print save va   AX11 379
writeline:(0,AX020,NAX020,3,3)#        Print list refl                  AX11 380
writeline:(0,AX021,NAX021,3,3)#        Print punch cards                AX11 381
IF (REFTMP>0)writeline:(0,AX037,NAX037,3,3)# Print tf test              AX11 382
writeline:(0,AX035,NAX035,3,3)#        Print uov test                   AX11 383
writeline:(0,AX033,NAX033,3,3)#        Print scale test                 AX11 384
IF (REFINE(4)>1)writeline:(0,AX036,NAX036,3,3)# Print extinc test       AX11 385
IF (REFINE(2)>1)writeline:(0,AX032,NAX032,3,3)# Print entpol test       AX11 386
IF (REFINE(1)>0)writeline:(0,AX031,NAX031,3,3)# Print dist test         AX11 387
writeline:(0,AX023,NAX023,3,3)#        Print correlation bound          AX11 388
IF (CUTSIG>-large:)writeline:(0,AX041,NAX041,3,3)# Print cutoff         AX11 389
IF (REJECT<large:)writeline:(0,AX042,NAX042,3,3)# Print rejection       AX11 390
RETURN#                                                                 AX11 391
END#                                                                    AX11 392
#                                                                       AX11 393
systemheader:(AX12)#                   Read bdf until lrdset:           AX12   1
####################################################################    AX12   2
#                                                                  #    AX12   3
#                               AX12                               #    AX12   4
#                                                                  #    AX12   5
####################################################################    AX12   6
SUBROUTINE AX12#                                                        AX12   7
#                                                                       AX12   8
include:(AACOMN)#                                                       AX12   9
include:(AXCOMN)#                                                       AX12  10
#                                                                       AX12  11
REAL RCELL(6)#                         Reciprocal cell constants        AX12  12
REAL DET#                              Determinant of b matrix          AX12  13
INTEGER I,J,K,M#                       Loops and pointers               AX12  14
INTEGER LA,LB,LC,L1,L3,L6#             List pointers                    AX12  15
INTEGER RANGE#                         Number of words to move          AX12  16
REAL LATCON(6)#                                                         AX12  17
#REAL RHO#                              MONOCHROMATOR DIHEDRAL ANGLE    AX12  18
#                                                                       AX12  19
chardata:(AX121,["WARNING- Polarity not allowed with centric groups, $# AX12  20
turned off"])#                                                          AX12  21
datastuff:#                                                             AX12  22
#                                                                       AX12  23
# SET LIST VARIABLES                                                    AX12  24
# ------------------                                                    AX12  25
#                                                                       AX12  26
STEP(1)=step1:;STEP(2)=step2:;STEP(3)=step3:#                           AX12  27
STEP(4)=step4:;STEP(5)=step5:;STEP(6)=mxwd8c:#                          AX12  28
IF (REFINE(6)>0)STEP(6)=STEP(6)+2#                                      AX12  29
STEP(7)=step7:#                                                         AX12  30
OFFSET(1)=off1:;OFFSET(2)=off2:;OFFSET(3)=off3:#                        AX12  31
OFFSET(4)=off4:;OFFSET(5)=off5:#                                        AX12  32
#                                                                       AX12  33
# SET UP AND REQUEST FILE BUFFERS                                       AX12  34
# -------------------------------                                       AX12  35
#                                                                       AX12  36
IOMARK(1)=QXSTR#                       Buffer for file 1                AX12  37
IOMARK(2)=IOMARK(1)+bdfbuf:#           Buffer for file 2                AX12  38
incrqx:(MARKA,IOMARK(2)+bdfbuf:,AX1201)# Request memory                 AX12  39
readwpkt:(1,lrlabl:,IPACK,IP,0)#       Enter bdf history                AX12  40
#                                                                       AX12  41
# CELL INFORMATION                                                      AX12  42
# ----------------                                                      AX12  43
#                                                                       AX12  44
DO I=1,8#                              Loop over lrcell: packets        AX12  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX12  46
readwpkt:(1,lrcell:,IPACK,IP,0)#       *lr 4* cell constants            AX12  47
IF(IP<=0)iquit:(AX1202)#               Lrcell: is missing               AX12  48
IF (I==1)#                             If first packet                  AX12  49
DO J=1,3#                              Lattice constants                AX12  50
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX12  51
LATCON(J)=QX(IP+J)#                    Axis length                      AX12  52
LATCON(J+3)=QX(IP+6+J)*360.0#          Angles in degrees                AX12  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX12  54
IF(I==3)movereal:(QX,IP+1,RCELL,1,6,0)# Reciprocal cell constants       AX12  55
IF(I==6)#                              Retrieve unit cell volume        AX12  56
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX12  57
VOL=QX(IP+1)#                          Get cell volume                  AX12  58
IF(VOL<=0.)iquit:(AX1203)#             Non-positive                     AX12  59
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX12  60
IF (I==7)#                             Metric tensor                    AX12  61
DO J=1,3;DO M=1,3;MT(J,M)=QX(IP+(J-1)*3+M)# Save metric tensor          AX12  62
IF (I==8)#                             Inverse metric tensor            AX12  63
DO J=1,3;DO M=1,3;MTINV(J,M)=QX(IP+(J-1)*3+M)# Save inverse             AX12  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX12  65
#                                                                       AX12  66
CELL(1)=1.0;CELL(4)=RCELL(6)#          U -> uij                         AX12  67
CELL(2)=1.0;CELL(5)=RCELL(5)#                                           AX12  68
CELL(3)=1.0;CELL(6)=RCELL(4)#                                           AX12  69
CELL(7)=0.25*utob:*RCELL(1)*RCELL(1)#  Temp fact                        AX12  70
CELL(8)=0.25*utob:*RCELL(2)*RCELL(2)#                                   AX12  71
CELL(9)=0.25*utob:*RCELL(3)*RCELL(3)#                                   AX12  72
CELL(10)=0.5*utob:*RCELL(1)*RCELL(2)#                                   AX12  73
CELL(11)=0.5*utob:*RCELL(1)*RCELL(3)#                                   AX12  74
CELL(12)=0.5*utob:*RCELL(2)*RCELL(3)#                                   AX12  75
#                                      ***** build b and b**-1          AX12  76
B(1,2)=0.0;B(1,3)=0.0;B(2,3)=0.0#                                       AX12  77
B(3,3)=SQRT(MT(3,3))#                                                   AX12  78
B(3,1)=MT(3,1)/B(3,3);B(3,2)=MT(3,2)/B(3,3)#                            AX12  79
B(2,2)=SQRT(MT(2,2)-B(3,2)**2)#                                         AX12  80
B(2,1)=(MT(2,1)-B(3,1)*B(3,2))/B(2,2)#                                  AX12  81
B(1,1)=SQRT(MT(1,1)-B(2,1)**2-B(3,1)**2)#                               AX12  82
#                                                                       AX12  83
DET=B(1,1)*B(2,2)*B(3,3)+B(1,2)*B(2,3)*B(3,1)$#                         AX12  84
+B(1,3)*B(3,2)*B(2,1)-B(1,3)*B(2,2)*B(3,1)$#                            AX12  85
-B(1,2)*B(2,1)*B(3,3)-B(1,1)*B(2,3)*B(3,2)#                             AX12  86
BINV(1,1)=(B(2,2)*B(3,3)-B(2,3)*B(3,2))/DET#                            AX12  87
BINV(2,1)=(B(3,1)*B(2,3)-B(2,1)*B(3,3))/DET#                            AX12  88
BINV(3,1)=(B(2,1)*B(3,2)-B(2,2)*B(3,1))/DET#                            AX12  89
BINV(1,2)=(B(1,3)*B(3,2)-B(1,2)*B(3,3))/DET#                            AX12  90
BINV(2,2)=(B(1,1)*B(3,3)-B(1,3)*B(3,1))/DET#                            AX12  91
BINV(3,2)=(B(3,1)*B(1,2)-B(1,1)*B(3,2))/DET#                            AX12  92
BINV(1,3)=(B(1,2)*B(2,3)-B(2,2)*B(1,3))/DET#                            AX12  93
BINV(2,3)=(B(1,3)*B(2,1)-B(1,1)*B(2,3))/DET#                            AX12  94
BINV(3,3)=(B(2,2)*B(1,1)-B(2,1)*B(1,2))/DET#                            AX12  95
#                                                                       AX12  96
# SET UP ALL LIST POINTERS FOR STORAGE                                  AX12  97
# ------------------------------------                                  AX12  98
#                                                                       AX12  99
MARKB=MARKA+24*stepa:;MARKC=MARKB+24*stepb:# Set pointers               AX12 100
STEPC=200;MARK(1)=MARKC+sfmax:*STEPC#                                   AX12 101
IF (REFINE(1)>0)MARK(2)=MARK(1)+20*STEP(1)+1# Reserve mem for dist      AX12 102
ELSE            MARK(2)=MARK(1)+1#                                      AX12 103
IF (REFINE(2)>0)MARK(3)=MARK(2)+48*STEP(2)+1# Reserve mem for twins     AX12 104
ELSE            MARK(3)=MARK(2)+1#                                      AX12 105
MARK(4)=MARK(3)+64*STEP(3)+1#          Reserve mem for scales           AX12 106
IF (REFINE(4)>0)MARK(5)=MARK(4)+14+1#  Reserve mem for extinction       AX12 107
ELSE            MARK(5)=MARK(4)+1#                                      AX12 108
MARK(6)=MARK(5)+1+1#                   Reserve mem for u overall        AX12 109
incrqx:(MARK(7),MARK(6)+sfmax:*STEP(6)+1,AX1205)# Not enough mem        AX12 110
STOPA=MARKA;STOPB=MARKB;STOPC=MARKC#   Set end-of-lists                 AX12 111
STOP(1)=MARK(1);STOP(2)=MARK(2);STOP(3)=MARK(3)+STEP(3)#                AX12 112
STOP(4)=MARK(4);STOP(5)=MARK(5)+STEP(5);STOP(6)=MARK(6)#                AX12 113
#                                                                       AX12 114
# SYMMETRY INFORMATION                                                  AX12 115
# --------------------                                                  AX12 116
#                                                                       AX12 117
readwpkt:(1,lrsymm:,IPACK,IP,0)#       *lr 5* symmetry informatio       AX12 118
IF(IP<=0)iquit:(AX1204)#               Lrsymm: is missing               AX12 119
ICENT=1-NINT(QX(IP+2))#                Icent (yes:)/(no:) (sym)/(asym   AX12 120
NSYM=NINT(QX(IP+4))#                   Number of symop                  AX12 121
RFLMLT=NINT(QX(IP+6))#                 Extract refln multiplicity       AX12 122
LA=MARKA;STOPA =MARKA+NSYM*stepa:#     Set end-of-list pointer          AX12 123
DO I=1,NSYM#                           Loop over symop                  AX12 124
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX12 125
readwpkt:(1,lrsymm:,IPACK,IP,0)#       Get one packet                   AX12 126
IF(IP<=0)iquit:(AX1206)#               Lrsymm ended too soon            AX12 127
movereal:(QX,IP+1,QX,LA+1,12,1)#       Copy symop                       AX12 128
incra:(LA)#                            Update pointer                   AX12 129
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX12 130
#                                                                       AX12 131
# TWINNING INFORMATION                                                  AX12 132
# --------------------                                                  AX12 133
#                                                                       AX12 134
XABS=voidflg:;SUXABS=voidflg:#         Xabs and su assumed void         AX12 135
LB=MARKB#                              Default: one one twinning matr   AX12 136
QX(LB+1)=1.0;QX(LB+2)=0.0;QX(LB+3)=0.0# Column of identity matrix       AX12 137
QX(LB+4)=0.0;QX(LB+5)=1.0;QX(LB+6)=0.0#                                 AX12 138
QX(LB+7)=0.0;QX(LB+8)=0.0;QX(LB+9)=1.0#                                 AX12 139
STOPB=MARKB+stepb:#                    Just identity matrix             AX12 140
IF (REFINE(2)>0)#                      Twinning laws applied            AX12 141
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX12 142
NB(2)=0#                               Identity matrix on file          AX12 143
REPEAT#                                                                 AX12 144
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX12 145
readwpkt:(1,lrtwin:,IPACK,IP,0)#       Get one packet                   AX12 146
IF (IP<=0)BREAK#                       Exit if last                     AX12 147
movereal:(QX,IP+1,QX,LB+1,stepb:,0)#   Copy matrix                      AX12 148
NB(2)=NB(2)+1;incrb:(LB)#              Increment twinop pointer         AX12 149
entpol:(NB(2))=QX(IP+10)#                                               AX12 150
IF (ICENT==yes:) NEXT#                 If centro then skip this bit     AX12 151
entpol:(NB(2)+24)=QX(IP+12)#           Save enantiomorphic fraction     AX12 152
IF(NB(2)==1)$(XABS=QX(IP+12);SUXABS=QX(IP+13)$)# Save first non-cent    AX12 153
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX12 154
IF (NB(2)==0)#                         Lrtwin absent                    AX12 155
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX12 156
NB(2)=1;entpol:(1)=1.0;entpol:(25)=0.0# One matrix                      AX12 157
incrb:(LB)#                            Count identity matrix            AX12 158
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX12 159
ELSEIF(NB(2)>1)$(XABS=voidflg:;SUXABS=voidflg:$)# Xabs is meaningless   AX12 160
IF (ICENT==no:)#                       2 twins for each matrix          AX12 161
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX12 162
movereal:(QX,MARK(2)+25+1,QX,MARK(2)+NB(2)+1,NB(2),0)# Krunch list      AX12 163
NB(2)=2*NB(2)#                         Count all twins                  AX12 164
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX12 165
STOP(2)=MARK(2)+NB(2);STOPB=LB#        Set end-of-list markers          AX12 166
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX12 167
ELSE STOP(2)=MARK(2)#                  Set end-of-list pointer          AX12 168
#                                                                       AX12 169
# SCATTERING FACTORS, DISPERSION                                        AX12 170
# ------------------------------                                        AX12 171
#                                                                       AX12 172
IWNT(1)=11#                            Scatt.fact.name                  AX12 173
IWNT(2)=20+DATSET#                     Real part of disp sf             AX12 174
IWNT(3)=60+DATSET#                     Imag part of disp sf             AX12 175
IWNT(4)=7#                             Check for sf in lrrefl:          AX12 176
IWNT(5)=10#                            Get neutron sf                   AX12 177
IWNT(6)=100#                           Get xray sf tables               AX12 178
SIGNAL(1)=1#                           Item 1 mandatory                 AX12 179
SIGNAL(2)=6#                           Items 2-6 optionals              AX12 180
SIGNAL(3)=0#                           No additions or deletions        AX12 181
SIGNAL(4)=0#                           Non-archive mode                 AX12 182
indexpkt:(1,lrscat:,IPACK,IP,0,SIGNAL,IWNT,IREL)# Get directory         AX12 183
IF(IP<=0)iquit:(AX1208)#               Lrscat: is missing               AX12 184
#                                                                       AX12 185
# COUNT ENTRIES IN SCATTERING FACTOR TABLES                             AX12 186
# -----------------------------------------                             AX12 187
#                                                                       AX12 188
STEPC=0#                               Count entries                    AX12 189
M=IP+IREL(6)#                          Point to first entry             AX12 190
IF(M>IP)#                              Tables present                   AX12 191
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX12 192
STEPC=1#                               0.00 present                     AX12 193
DSFTB=(QX(M+1)-QX(M))*0.01#            Delta stol                       AX12 194
REPEAT#                                Loop over directory              AX12 195
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX12 196
STEPC=STEPC+1#                         Count entry                      AX12 197
IF (STEPC>IPACK)BREAK#                 Out of packet                    AX12 198
M=M+1#                                 Next possible entry              AX12 199
IF((QX(M)<99.5)\(QX(M)>299.5))BREAK#   Not stol entry                   AX12 200
ifne:((QX(M)-QX(M-1))*0.01,DSFTB)iquit:(AX1210)# Partition not regula   AX12 201
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX12 202
STEPC=STEPC-1#                         Update counter                   AX12 203
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX12 204
ELSE IF (IREL(5)>0)STEPC=1#            Neutron sf present               AX12 205
#                                                                       AX12 206
# GET SF NAMES, DISPERSIONS, AND TABLES FROM LRSCAT:                    AX12 207
# --------------------------------------------------                    AX12 208
#                                                                       AX12 209
LC=MARKC;L6=MARK(6);NB(6)=0#           Count atom types                 AX12 210
REPEAT#                                Loop over lrscat: packets        AX12 211
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX12 212
readwpkt:(1,lrscat:,IPACK,IP,0)#       Read a packet                    AX12 213
IF(IP<=0)BREAK#                        End of logical record            AX12 214
NB(6)=NB(6)+1#                         Count atom type                  AX12 215
movereal:(QX,IP+IREL(1),nattyp:(L6),1,mxwd8c:,0)# Store sf name         AX12 216
IF(REFINE(6)>0)#                       Test if disp applied             AX12 217
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX12 218
redisp:(L6)=0.0;imdisp:(L6)=0.0#       Default: no disp                 AX12 219
M=IP+IREL(2);IF(M>IP)redisp:(L6)=QX(M)# Store real disp if there        AX12 220
M=IP+IREL(3);IF(M>IP)imdisp:(L6)=QX(M)# Store imag disp if there        AX12 221
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX12 222
incr6:(L6)#                            Increase pointer                 AX12 223
IF (STEPC==0)NEXT#                     No sf tables                     AX12 224
IF (STEPC==1)QX(LC+1)=QX(IP+IREL(5))#  Store neutron sf                 AX12 225
ELSE movereal:(QX,IP+IREL(6),QX,LC+1,STEPC,1)# Move the table           AX12 226
LC=LC+STEPC#                           Increment s.f. storage           AX12 227
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX12 228
STOP(6)=L6;STOPC=LC#                   Set end-of-list pointers         AX12 229
#                                                                       AX12 230
# GET SCALE FACTORS FROM LREXPL:                                        AX12 231
# ------------------------------                                        AX12 232
#                                                                       AX12 233
LAMBDA=0.71069#                        Default molykalpha               AX12 234
INEU=3#                                Conventional x-ray               AX12 235
sclgrp:(MARK(3))=1.0;sclfac:(MARK(3))=1.0# Default scale                AX12 236
IWNT(1)=1#                             Set data set idnum               AX12 237
IWNT(2)=2#                             Lambda from bdf                  AX12 238
IWNT(3)=94#                            Radiation type                   AX12 239
IWNT(4)=52#                            Polarization ratio               AX12 240
IWNT(5)=55#                            Monochromator dihedral angle     AX12 241
DO I=1,64;IWNT(I+5)=100+I#             Request each scale group         AX12 242
IF (REFINE(1)>0)#                      Crystal shape to be refined      AX12 243
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX12 244
IWNT(70)=300#                          Face number                      AX12 245
DO I=1,20#                             Loop over all possible faces (   AX12 246
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX12 247
J=6*(I-1)+70;M=300+10*(I-1)#           Set pointers                     AX12 248
DO K=1,4;IWNT(J+K)=M+K#                Hkl and observed dist            AX12 249
IWNT(J+5)=M+8#                         Esd of observed dist             AX12 250
IWNT(J+6)=M+9#                         Weight                           AX12 251
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX12 252
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX12 253
SIGNAL(1)=0;SIGNAL(2)=210#             No mandatory items               AX12 254
SIGNAL(3)=0;SIGNAL(4)=0#               No additions or deletions        AX12 255
indexpkt:(1,lrexpl:,IPACK,IP,0,SIGNAL,IWNT,IREL)# Search directory      AX12 256
IF(IP<=0)iquit:(AX1213)#               Lrexpl: not there                AX12 257
REPEAT#                                Loop over lrexpl pkts            AX12 258
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX12 259
readwpkt:(1,lrexpl:,IPACK,IP,0)#       Read lrexpl: packet              AX12 260
IF(IP<=0)iquit:(AX1214)#               Lr ended too soon                AX12 261
IF(IREL(1)>0)#                         Test if datasets stored          AX12 262
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX12 263
I=IP+IREL(1)#                          Point to data set                AX12 264
IF(DATSET!=NINT(QX(I)))NEXT#           Skip incorrect datasets          AX12 265
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX12 266
IF(REFINE(1)>0)#                       Crystal shape to be refined      AX12 267
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX12 268
ifpr:(70)NB(1)=NINT(QX(IP+IREL(70)))#  Get number of faces              AX12 269
ELSE iquit:(AX1215)#                   No faces on bdf, impossible to   AX12 270
IF (NB(1)<=0)iquit:(AX1216)#           No faces on bdf, impossible to   AX12 271
STOP(1)=MARK(1)+NB(1)*STEP(1)#         Set end-of-list marker           AX12 272
L1=MARK(1)#                            Set distance pointer             AX12 273
DO I=1,NB(1)#                          Loop over faces                  AX12 274
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX12 275
J=70+(I-1)*6#                          Set irel pointer                 AX12 276
DO K=1,5#                              Mandatory: hkl, do, so           AX12 277
IF (IREL(K+J)<=0)iquit:(AX1217)#       Exit if not present              AX12 278
ELSE facem:(L1+K)=QX(IP+IREL(K+J))#    Store                            AX12 279
IF (IREL(J+6)<=0)facew:(L1)=1.0/faceso:(L1)**2#                         AX12 280
ELSE facew:(L1)=QX(IP+IREL(J+6))#                                       AX12 281
facedc:(L1)=facedo:(L1)#                                                AX12 282
incr1:(L1)#                            Update pointer                   AX12 283
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX12 284
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX12 285
L3=MARK(3)#                            Pointer into mark(3)             AX12 286
DO I=1,64#                             Loop over scale groups           AX12 287
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX12 288
M=IP+IREL(I+5);IF(M<=IP)NEXT#          Point to scale factor            AX12 289
sclfac:(L3)=1.0/QX(M)**2#              Store scale factor               AX12 290
sclgrp:(L3)=REAL(I)#                   Store scale group number         AX12 291
incr3:(L3)#                            Increment pointer                AX12 292
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX12 293
STOP(3)=L3;NB(3)=(STOP(3)-MARK(3))/STEP(3)# Set end-of-list marker      AX12 294
#                                      1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX12 295
M=IP+IREL(2)#                          Pnter to wavelength              AX12 296
IF(M>IP)LAMBDA=QX(M)#                  Lambda present                   AX12 297
ifpr:(3)INEU=NINT(QX(IP+IREL(3)))#     Radiation type                   AX12 298
IF (REFINE(4)>0)#                      Extinction                       AX12 299
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX12 300
ifpr:(4)RATIO=QX(IP+IREL(4))#          Get polarization ratio           AX12 301
#IFPR:(5)RHO=QX(IP+IREL(5))#            GET DIHEDRAL ANGLE              AX12 302
#RHO=COS(RHO*0.01745329)**2#            COSINE**2 OF DIHEDRAL ANGLE     AX12 303
#RATIO=(RHO+(1-RHO)*RATIO)/(1-RHO+RHO*RATIO)# MODIFY FOR DIHEDRAL ANGLE AX12 304
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX12 305
BREAK#                                 Exit packet loop                 AX12 306
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX12 307
IF (INEU==2)iquit:(AX1211)#            Electrons not implemented        AX12 308
#                                                                       AX12 309
# EXTINCTION AND MAXHKL FROM LRDSET:                                    AX12 310
# ----------------------------------                                    AX12 311
#                                                                       AX12 312
IWNT(1)=2;IWNT(2)=1#                   Request uov & data set           AX12 313
DO I=1,5;IWNT(2+I)=10+I#               Request maxhkl, min-max stol     AX12 314
I=7#                                   Set directory pointer            AX12 315
IF (REFINE(4)>0)DO J=1,NB(4)+3#        Extinction required              AX12 316
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX12 317
I=I+1;IWNT(I)=100+J#                   Get all extinction parameters    AX12 318
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX12 319
SIGNAL(1)=0;SIGNAL(2)=I#               No mandatory items               AX12 320
SIGNAL(3)=0;SIGNAL(4)=0#               No additions or deletions        AX12 321
indexpkt:(1,lrdset:,IPACK,IP,0,SIGNAL,IWNT,IREL)#                       AX12 322
IF(IP<=0)iquit:(AX1218)#               Lr missing                       AX12 323
#                                                                       AX12 324
uovl:=0.03#                            Initialize overall u             AX12 325
HMAX=999.#                             Initialize max /h/               AX12 326
KMAX=999.#                             Initialize max /k/               AX12 327
LMAX=999.#                             Initialize max /l/               AX12 328
SMIN=0.0001#                           Initialize min st/l              AX12 329
SMAX=10.0#                             Initialize max st/l              AX12 330
EXTTYP=1#                              Default zach ext=iso1/gauss      AX12 331
TBAR=0.#                               Set absurd default for later ^   AX12 332
IF (REFINE(4)>0)DO I=1,NB(4);extinc:(I)=0.0# Default extinction param   AX12 333
REPEAT#                                Loop over packets                AX12 334
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX12 335
readwpkt:(1,lrdset:,IPACK,IP,0)#       Read lrdset: packet              AX12 336
IF(IP<=0)BREAK#                        Lr ended                         AX12 337
ifpr:(2)#                              Data set on bdf                  AX12 338
IF (NINT(QX(IP+IREL(2)))!=DATSET)NEXT#                                  AX12 339
ifpr:(1)uovl:=QX(IP+IREL(1))#          Uov on bdf                       AX12 340
IF(IREL(3)>0)HMAX=QX(IP+IREL(3))#      Store hmax                       AX12 341
IF(IREL(4)>0)KMAX=QX(IP+IREL(4))#      Store kmax                       AX12 342
IF(IREL(5)>0)LMAX=QX(IP+IREL(5))#      Store lmax                       AX12 343
IF(IREL(6)>0)SMIN=QX(IP+IREL(6))#      Store sin(theta)/lambda min      AX12 344
IF(IREL(7)>0)SMAX=QX(IP+IREL(7))#      Store sin(theta)/lambda max      AX12 345
#                                                                       AX12 346
I=7#                                                                    AX12 347
IF (REFINE(4)>0)#                                                       AX12 348
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX12 349
STOP(4)=MARK(4)+14#                    Reserve mem for params           AX12 350
I=I+1#                                 Increment dir pointer            AX12 351
ifpr:(I)IF(QX(IP+IREL(I))>.1)EXTTYP=NINT(QX(IP+IREL(I)))# Type          AX12 352
I=I+1#                                 Increment dir pointer            AX12 353
ifpr:(I)IF(QX(IP+IREL(I))>.1)EXTTYP=-EXTTYP# Distr lorentz              AX12 354
DO J=1,NB(4);ifpr:(I+J)extinc:(J)=QX(IP+IREL(I+J))# Store extinc para   AX12 355
I=I+NB(4)+1#                           Increment dir pointer            AX12 356
ifpr:(I)TBAR=QX(IP+IREL(I))#           Store mean tbar in mm            AX12 357
#RHO: = RHO:*LAMBDA#                                                    AX12 358
rho:=rho:*LAMBDA/1000.#                Transform to r/10000 in mm       AX12 359
g:=g:/10000.#                          Scale g: to g/10000  in mm       AX12 360
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX12 361
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX12 362
#                                                                       AX12 363
# COMPACT ALL LISTS                                                     AX12 364
# -----------------                                                     AX12 365
#                                                                       AX12 366
RANGE=MARKB-STOPA#                     Copy list b after list a         AX12 367
movereal:(QX,MARKB+1,QX,STOPA+1,STOPB-MARKB,1)#                         AX12 368
MARKB=MARKB-RANGE;STOPB=STOPB-RANGE#                                    AX12 369
RANGE=MARKC-STOPB#                     Copy list c after list b         AX12 370
movereal:(QX,MARKC+1,QX,STOPB+1,STOPC-MARKC,1)#                         AX12 371
MARKC=MARKC-RANGE;STOPC=STOPC-RANGE#                                    AX12 372
RANGE=MARK(1)-STOPC#                   Copy list 1 after list c         AX12 373
movereal:(QX,MARK(1)+1,QX,STOPC+1,STOP(1)-MARK(1),1)#                   AX12 374
MARK(1)=MARK(1)-RANGE;STOP(1)=STOP(1)-RANGE#                            AX12 375
RANGE=MARK(2)-STOP(1)-1#               Copy list 2 after list 1         AX12 376
movereal:(QX,MARK(2)+1,QX,STOP(1)+2,STOP(2)-MARK(2),1)#                 AX12 377
MARK(2)=MARK(2)-RANGE;STOP(2)=STOP(2)-RANGE#                            AX12 378
RANGE=MARK(3)-STOP(2)-1#               Copy list 3 after list 2         AX12 379
movereal:(QX,MARK(3)+1,QX,STOP(2)+2,STOP(3)-MARK(3),1)#                 AX12 380
MARK(3)=MARK(3)-RANGE;STOP(3)=STOP(3)-RANGE#                            AX12 381
RANGE=MARK(4)-STOP(3)-1#               Copy list 4 after list 3         AX12 382
movereal:(QX,MARK(4)+1,QX,STOP(3)+2,STOP(4)-MARK(4),1)#                 AX12 383
MARK(4)=MARK(4)-RANGE;STOP(4)=STOP(4)-RANGE#                            AX12 384
RANGE=MARK(5)-STOP(4)-1#               Copy list 5 after list 4         AX12 385
movereal:(QX,MARK(5)+1,QX,STOP(4)+2,STOP(5)-MARK(5),1)#                 AX12 386
MARK(5)=MARK(5)-RANGE;STOP(5)=STOP(5)-RANGE#                            AX12 387
RANGE=MARK(6)-STOP(5)-1#               Copy list 6 after list 5         AX12 388
movereal:(QX,MARK(6)+1,QX,STOP(5)+2,STOP(6)-MARK(6),1)#                 AX12 389
MARK(6)=MARK(6)-RANGE;STOP(6)=STOP(6)-RANGE#                            AX12 390
RETURN#                                                                 AX12 391
END#                                                                    AX12 392
#                                                                       AX12 393
systemheader:(AX13)#                   Parameter modification cards     AX13   1
####################################################################    AX13   2
#                                                                  #    AX13   3
#                               AX13                               #    AX13   4
#                                                                  #    AX13   5
####################################################################    AX13   6
SUBROUTINE AX13#                                                        AX13   7
include:(AACOMN)#                                                       AX13   8
include:(AXCOMN)#                                                       AX13   9
#                                                                       AX13  10
INTEGER I#                             Loop index                       AX13  11
INTEGER LB#                            Twin list pointer                AX13  12
INTEGER STEPT#                         Twinop input packet length       AX13  13
#                                                                       AX13  14
chardata:(AX131,MAXHKL TWINOP CONSTR NOREF REF LIMSFT END )#            AX13  15
chardata:(AX132," Max /h/ /k/ /l/               ")#                     AX13  16
chardata:(AX133," Min/Max sin[(]t/l[)]             ")#                  AX13  17
realdata:(FMT,[210313.,250313.,290313.,230642.,300642.])#               AX13  18
datastuff:#                                                             AX13  19
#                                                                       AX13  20
STEPT=(2-ICENT)+9#                     Matrix plus one ore two words    AX13  21
REPEAT#                                Loop over cards                  AX13  22
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX13  23
readline:(AX131,NAX131)#               Read card                        AX13  24
IF (LINID>2)BREAK#                     Not param modif card             AX13  25
IF (LINID==1)#                                                          AX13  26
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX13  27
#                                                                       AX13  28
# READ MAXHKL LINE                                                      AX13  29
# ----------------                                                      AX13  30
#                                                                       AX13  31
ifnv:(BUFIN(1))HMAX=BUFIN(1)#                                           AX13  32
ncodefld:(HMAX,1,AX132,FMT(1),1)#                                       AX13  33
ifnv:(BUFIN(2))KMAX=BUFIN(2)#                                           AX13  34
ncodefld:(KMAX,1,AX132,FMT(2),1)#                                       AX13  35
ifnv:(BUFIN(3))LMAX=BUFIN(3)#                                           AX13  36
ncodefld:(LMAX,1,AX132,FMT(3),1)#                                       AX13  37
writeline:(0,AX132,NAX132,3,1)#        Write the line                   AX13  38
ifnv:(BUFIN(4))SMIN=BUFIN(4)#                                           AX13  39
ncodefld:(SMIN,1,AX133,FMT(4),1)#      Encode max-min stol              AX13  40
ifnv:(BUFIN(5))SMAX=BUFIN(5)#                                           AX13  41
ncodefld:(SMAX,1,AX133,FMT(5),1)#      Encode max-min stol              AX13  42
writeline:(0,AX133,NAX133,3,1)#        Write the line                   AX13  43
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX13  44
IF (LINID==2)#                                                          AX13  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX13  46
#                                                                       AX13  47
# READ TWINOP LINE                                                      AX13  48
# ----------------                                                      AX13  49
#                                                                       AX13  50
IF ((BUFIN(11)<0.0)\(BUFIN(11)>1.0))iquit:(AX1303)# Outside bounds      AX13  51
IF ((ICENT==no:)&((BUFIN(12)<0.0)\(BUFIN(12)>1.0)))iquit:(AX1303)#      AX13  52
IF (BUFIN(10)<=0.0)iquit:(AX1304)#     Twinop number is missing         AX13  53
IF (NINT(BUFIN(10))>NB(2)/(2-ICENT))#  Inserted twinop                  AX13  54
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX13  55
IF (NINT(BUFIN(10))>1+NB(2)/(2-ICENT))iquit:(AX1302)# Silly serial n    AX13  56
incrqx:(I,STOP(6)+STEPT,AX1301)#       Get mem                          AX13  57
movereal:(QX,MARK(3)+1,QX,MARK(3)+STEPT+1,STOP(6)-MARK(3),1)# Shift l   AX13  58
DO I=3,6#                              Update markers                   AX13  59
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX13  60
MARK(I)=MARK(I)+STEPT;STOP(I)=STOP(I)+STEPT#                            AX13  61
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX13  62
IF (ICENT==no:)#                                                        AX13  63
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX13  64
I=NB(2)/2#                                                              AX13  65
movereal:(QX,MARK(2)+1+I,QX,MARK(2)+1+I+incr:(stepb:),I,1)# Shift inv   AX13  66
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX13  67
ELSE I=NB(2)#                                                           AX13  68
movereal:(QX,MARKC+1,QX,MARKC+stepb:+1,MARK(2)+I-MARKC,1)# Shift list   AX13  69
incrb:(STOPB);incrb:(MARKC);incrb:(STOPC)# Update markers               AX13  70
incrb:(MARK(1));incrb:(STOP(1));incrb:(MARK(2))#                        AX13  71
NB(2)=NB(2)+(2-ICENT);STOP(2)=STOP(2)+STEPT#                            AX13  72
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX13  73
I=NINT(BUFIN(10));LB=MARKB+(I-1)*stepb:# Twinop serial number           AX13  74
ifnv:(BUFIN(11))entpol:(I)=BUFIN(11)#  Store ratio                      AX13  75
IF (ICENT==no:)ifnv:(BUFIN(12))entpol:(I+NB(2)/2)=BUFIN(12)# Store ra   AX13  76
DO I=1,stepb:;ifnv:(BUFIN(I))QX(LB+I)=BUFIN(I)# Store twinop            AX13  77
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX13  78
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX13  79
#                                                                       AX13  80
RETURN#                                                                 AX13  81
END#                                                                    AX13  82
#                                                                       AX13  83
systemheader:(AX14)#                   Input atoms                      AX14   1
####################################################################    AX14   2
#                                                                  #    AX14   3
#                               AX14                               #    AX14   4
#                                                                  #    AX14   5
####################################################################    AX14   6
SUBROUTINE AX14#                                                        AX14   7
include:(AACOMN)#                                                       AX14   8
include:(AXCOMN)#                                                       AX14   9
#                                                                       AX14  10
INTEGER I#         ## General indices                                   AX14  11
INTEGER IFF#       ## Form factor pkt pointer                           AX14  12
INTEGER ITF#       ## Temp factor type pointer                          AX14  13
INTEGER L7#        ## Atom pointer                                      AX14  14
INTEGER NAM#       ## Name pointer in lratom pkt                        AX14  15
#                                                                       AX14  16
# READ LRATOM: DIRECTORY                                                AX14  17
# ----------------------                                                AX14  18
#                                                                       AX14  19
IWNT(1)=14#                            Request atom name                AX14  20
IWNT(2)=22#                            Request sf pointer               AX14  21
DO I=1,10;IWNT(I+2)=I;I=I+2#           Xyz & u & uij                    AX14  22
IWNT(I)=11;I=I+1#                      Pop param                        AX14  23
IWNT(I)=21;I=I+1#                      Atom multiplicity                AX14  24
IWNT(I)=23;I=I+1#                      Temp factor type                 AX14  25
IWNT(I)=17#                            Data set pointer                 AX14  26
SIGNAL(1)=5;SIGNAL(2)=I#               Items 1-5 mandatory              AX14  27
SIGNAL(3)=0;SIGNAL(4)=0#               No additions or deletions        AX14  28
indexpkt:(1,lratom:,IPACK,IP,0,SIGNAL,IWNT,IREL)# Search directory      AX14  29
IF(IP<=0)iquit:(AX1404)#               Lratom: not there                AX14  30
IF(SIGNAL(4)<=0)iquit:(AX1405)#        Mandatory item absent            AX14  31
#                                                                       AX14  32
NAM=IREL(1);IFF=IREL(2);ITF=IREL(15)#  Store much used pointers         AX14  33
NB(7)=0#                               Count atoms                      AX14  34
#                                                                       AX14  35
#     LOOP OVER THE ATOMS                                               AX14  36
#     ===================                                               AX14  37
#                                                                       AX14  38
MARK(7)=STOP(6)+1;STOP(7)=MARK(7);L7=MARK(7)# Set atom pointer          AX14  39
REPEAT#                                Loop over atom packets           AX14  40
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX14  41
readwpkt:(1,lratom:,IPACK,IP,0)#       Read an atom packet              AX14  42
IF(IP<=0)BREAK#                        End of logical record            AX14  43
I=IP+IREL(16)#                         Point to dataset number          AX14  44
IF(I>IP)#                              If data set present              AX14  45
IF(DATSET!=NINT(QX(I)))NEXT#           If wrong dataset skip            AX14  46
#                                                                       AX14  47
# STORE ATOM                                                            AX14  48
# ----------                                                            AX14  49
#                                                                       AX14  50
incrqx:(STOP(7),L7+STEP(7),AX1406)#    Get mem                          AX14  51
NB(7)=NB(7)+1#                         Count atoms                      AX14  52
I=IP+IFF;atomsft:(L7)=QX(I)#           Get form factor type             AX14  53
#                                                                       AX14  54
movereal:(QX,IP+NAM,atomn:(L7),1,mxwd24c:,0)# Store atom name           AX14  55
ifpr:(14)atommul:(L7)=QX(IP+IREL(14))# Store site multiplicity          AX14  56
ELSE     atommul:(L7)=1.#              Store default mult. as 1.        AX14  57
ifpr:(13)atompop:(L7)=QX(IP+IREL(13))# Store population param           AX14  58
ELSE     atompop:(L7)=1.#                                               AX14  59
#                                                                       AX14  60
DO I=1,3; atomxyz:(L7+I)=QX(IP+IREL(I+2))# Store coords                 AX14  61
#                                                                       AX14  62
IF ((REFTMP!=3)&(NINT(QX(IP+ITF))!=REFTMP))# Modify displ               AX14  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX14  64
IF (REFTMP==1)#                        All iso                          AX14  65
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX14  66
IF (NINT(QX(IP+ITF))==0)atomiso:(L7)=uovl:# Overall                     AX14  67
ELSE atomiso:(L7)=(QX(IP+IREL(7))+QX(IP+IREL(8))+QX(IP+IREL(9)))/3.0#   AX14  68
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX14  69
IF (REFTMP==2)#                        All aniso                        AX14  70
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX14  71
IF (NINT(QX(IP+ITF))==0)DO I=1,6;atomdis:(L7+I)=uovl:*CELL(I)#          AX14  72
ELSE                    DO I=1,6;atomdis:(L7+I)=QX(IP+IREL(6))*CELL(I)# AX14  73
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX14  74
atomtft:(L7)=REAL(REFTMP)#             Store new displ fact type        AX14  75
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX14  76
ELSE#                                  No modification                  AX14  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX14  78
IF (NINT(QX(IP+ITF))==1)atomiso:(L7)=QX(IP+IREL(6))#                    AX14  79
IF (NINT(QX(IP+ITF))==2)DO I=1,6;atomdis:(L7+I)=QX(IP+IREL(6+I))#       AX14  80
atomtft:(L7)=QX(IP+ITF)#               Store bdf displ fact type        AX14  81
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX14  82
incr7:(L7)#                            Increment atom pointer           AX14  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX14  84
MAXPAR=9#                              Default: xyz + umixed            AX14  85
IF (REFTMP==1)MAXPAR=4#                Xyz + uiso                       AX14  86
IF (REFTMP==0)MAXPAR=3#                Xyz + uoverall                   AX14  87
IF (REFPOP==yes:)MAXPAR=10#            Pop = 10th param                 AX14  88
#                                                                       AX14  89
MARKX=STOP(7);STOPX=MARKX;MARKY=STOPX;STOPY=MARKY# Empty lists          AX14  90
RETURN#                                                                 AX14  91
END#                                                                    AX14  92
#                                                                       AX14  93
systemheader:(AX15)#                   Constraint images                AX15   1
###################################################################     AX15   2
#                                                                 #     AX15   3
#                            AX15                                 #     AX15   4
#                                                                 #     AX15   5
###################################################################     AX15   6
SUBROUTINE AX15#                                                        AX15   7
include:(AACOMN)#                                                       AX15   8
include:(AXCOMN)#                                                       AX15   9
#                                                                       AX15  10
REAL CSTM,CSTA#                        Constants used in constraints    AX15  11
INTEGER FIELD,IFIRST,ILAST#            Field string delimiters          AX15  12
INTEGER I,L#                           Loop indexes                     AX15  13
INTEGER II#                            Used in incrqx:                  AX15  14
INTEGER L3,LT#                         List pointers                    AX15  15
INTEGER LX,MX,LY,MY#                   Constraint list pointers         AX15  16
INTEGER KEY#                           General purpose match flag       AX15  17
INTEGER MXP#                           Max nb of atomic param           AX15  18
REAL NUM#                              Multiplicative constant in con   AX15  19
INTEGER IMARK,ISTOP,ISTEP#             Used in ax02                     AX15  20
INTEGER S1,S2,P1,P2#                   Match for ref/noref cards        AX15  21
INTEGER SFT#                           Atom scattering factor type      AX15  22
INTEGER SUBJ#                          Address of subject param         AX15  23
INTEGER TFT#                           Displ fact type                  AX15  24
INTEGER RANGE#                         Range to move                    AX15  25
#                                                                       AX15  26
chardata:(AX151,MAXHKL TWINOP CONSTR NOREF REF LIMSFT END )#            AX15  27
chardata:(AX152,[-])#                                                   AX15  28
chardata:(AX153,[=*+])#                                                 AX15  29
intdata:(STEPT,stepy:)#                                                 AX15  30
datastuff:#                                                             AX15  31
#                                                                       AX15  32
# SET NC AND CONS BUFFERS                                               AX15  33
# -----------------------                                               AX15  34
#                                                                       AX15  35
IF (REFINE(2)>1)#                      Set constraint for sum(entpol)   AX15  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX15  37
findc:(MARKX,MARKY,LX,LY,MARK(2))#     Find location in nc list         AX15  38
putnc:(LX,MARK(2),constr:)#            Put last mero address            AX15  39
incrqx:(STOPY,STOPY+NB(2)*stepy:,AX1501)# Get mem for constraint        AX15  40
LY=MARKY#                              Set pointer                      AX15  41
DO I=1,NB(2)-1#                        Loop over all except last mero   AX15  42
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX15  43
cfact:(LY)=-1.0;caddr:(LY)=REAL(MARK(2)+I)# Store address               AX15  44
incry:(LY)#                            Increment constraint pointer     AX15  45
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX15  46
cfact:(LY)=1.0;caddr:(LY)=0.0#         Set sum to 1.0                   AX15  47
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX15  48
IF (REFINE(3)==1)#                     Set constraints for unit scale   AX15  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX15  50
findc:(MARKX,MARKY,LX,LY,MARK(3))#     Find location in nc list         AX15  51
DO L3=MARK(3)+STEP(3),STOP(3)-STEP(3),STEP(3)# Loop over scale fact (   AX15  52
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX15  53
putnc:(LX,L3,constr:)#                 Put scale fact address           AX15  54
LY=STOPY#                              Set ptr to end of list           AX15  55
incrqx:(STOPY,STOPY+2*stepy:,AX1501)#  Get mem for constraint           AX15  56
cfact:(LY)=sclfac:(L3)/sclfac:(MARK(3));caddr:(LY)=REAL(MARK(3))#       AX15  57
cfact:(LY+stepy:)=0.0;caddr:(LY+stepy:)=0.0#                            AX15  58
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX15  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX15  60
REFINE(3)=2#                           Set scale refinement             AX15  61
IF (REFINE(4)==2)#                     Extinction to be refined         AX15  62
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX15  63
findnc:(MARKX,LX,MARK(4))#             Find location in nc list         AX15  64
IF ((IABS(EXTTYP)!=1)&(IABS(EXTTYP)!=3))# Iso i or gen iso              AX15  65
ifnc:(LX,MARK(4))incrx:(LX)#           Already norefed                  AX15  66
ELSE putnc:(LX,MARK(4),noref:)#                                         AX15  67
IF ((IABS(EXTTYP)!=2)&(IABS(EXTTYP)!=3)&(IABS(EXTTYP)!=6))# Iso ii      AX15  68
ifnc:(LX,MARK(4)+1)incrx:(LX)#         Already norefed                  AX15  69
ELSE putnc:(LX,MARK(4)+1,noref:)#                                       AX15  70
IF ((IABS(EXTTYP)!=4)&(IABS(EXTTYP)!=6))DO I=1,6# Ani i or gen ani      AX15  71
ifnc:(LX,MARK(4)+1+I)incrx:(LX)#       Already norefed                  AX15  72
ELSE putnc:(LX,MARK(4)+1+I,noref:)#                                     AX15  73
IF (IABS(EXTTYP)!=5)DO I=1,6#          Ani ii                           AX15  74
ifnc:(LX,MARK(4)+7+I)incrx:(LX)#       Already norefed                  AX15  75
ELSE putnc:(LX,MARK(4)+7+I,noref:)#                                     AX15  76
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX15  77
REPEAT#                                Loop over constr lines           AX15  78
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX15  79
IF((LINID>5)\(LINID<3))BREAK#          Constr image                     AX15  80
#                                                                       AX15  81
# CONSTR CARDS                                                          AX15  82
# ------------                                                          AX15  83
#                                                                       AX15  84
IF(LINID==3)#                          Constr card                      AX15  85
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX15  86
I=1#                                                                    AX15  87
REPEAT#                                Loop over input line             AX15  88
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX15  89
compchar:(CHRIN,I,AX152,1,1,1,NAX152,KEY)# Insert " " before "-"        AX15  90
IF (KEY!=0)#                           - encountered                    AX15  91
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX15  92
DO L=CHRMAX-1,I,-1#                    Shift left                       AX15  93
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX15  94
movechar:(CHRIN,L,CHRIN,L+1,1,0)#      Shift left 1 char                AX15  95
movechar:(CHRIP,L,CHRIP,L+1,1,0)#      Shift left 1 char                AX15  96
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX15  97
movechar:(BLANK,1,CHRIN,I,1,0)#        Insert blank                     AX15  98
movechar:(BLANK,1,CHRIP,I,1,0)#        Insert blank                     AX15  99
I=I+2#                                 Skip blank                       AX15 100
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX15 101
compchar:(CHRIN,I,AX153,1,1,1,NAX153,KEY)# Replace "+*=" by " "         AX15 102
IF (KEY!=0)#                           "+*=" found                      AX15 103
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX15 104
movechar:(BLANK,1,CHRIN,I,1,0)#        Replace by " "                   AX15 105
movechar:(BLANK,1,CHRIP,I,1,0)#        Replace by " "                   AX15 106
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX15 107
I=I+1;IF (I>CHRMAX)BREAK#              Exit if last char                AX15 108
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX15 109
dcodefld:#                             Decode field again               AX15 110
FIELD=1#                               Set field counter                AX15 111
CALL AX03(FIELD,CSTM,SUBJ)#            Get subject atom param           AX15 112
findc:(MARKX,MARKY,LX,LY,SUBJ)#        Find location in nc list         AX15 113
ifnc:(LX,SUBJ)iquit:(AX1511)#          Already constrained              AX15 114
putnc:(LX,SUBJ,constr:)#               Store in nc list                 AX15 115
DO I=1,7;IF ((SUBJ>=MARK(I))&(SUBJ<=STOP(I)))BREAK# Find subj bounds    AX15 116
IMARK=MARK(I);ISTOP=STOP(I)#           Find bounds                      AX15 117
CSTA=BUFIN(FIELD)/CSTM;FIELD=FIELD+1#  Get additive constant            AX15 118
MARKT=STOPY;LT=MARKT;STOPT=MARKT#      Set temp list makers             AX15 119
REPEAT#                                Loop over reference fields       AX15 120
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX15 121
ifvo:(BUFIN(FIELD))BREAK#              Exit on first void field         AX15 122
CALL AX03(FIELD,NUM,L)#                Get reference params             AX15 123
IF ((L<IMARK)\(L>ISTOP))iquit:(AX1500)# Not same class                  AX15 124
incrqx:(STOPT,STOPT+STEPT,AX1512)#     Get mem                          AX15 125
taddr:(LT)=REAL(L);tfact:(LT)=NUM/CSTM# Store address & mult fact       AX15 126
incrt:(LT)#                            Increment pointer                AX15 127
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX15 128
incrx:(LY)#                            Update ptr ly modified by putn   AX15 129
RANGE=STOPT-MARKT+stepy:#              Lenght of constraint body        AX15 130
incrqx:(II,STOPT+RANGE,AX1512)#        Get mem                          AX15 131
movereal:(QX,LY+1,QX,LY+RANGE+1,STOPT-LY,1)# Make space for temp list   AX15 132
MARKT=MARKT+RANGE;STOPT=STOPT+RANGE#   Update list pointers             AX15 133
movereal:(QX,MARKT+1,QX,LY+1,STOPT-MARKT,1)# Move temp list             AX15 134
LY=LY+(STOPT-MARKT)#                   Update constr list pointer       AX15 135
caddr:(LY)=0.0#                        End-of-constraint mark           AX15 136
cfact:(LY)=CSTA/CSTM#                  Normalized constant              AX15 137
STOPY=STOPY+RANGE#                                                      AX15 138
incry:(LY)#                            Ly points to next constraint     AX15 139
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX15 140
#                                                                       AX15 141
#   REF AND NOREF                                                       AX15 142
#   =============                                                       AX15 143
#                                                                       AX15 144
ELSE IF((LINID==4)\(LINID==5))#        Ref and noref cards              AX15 145
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX15 146
FOR (FIELD=1;FIELD<=mxfdim:;FIELD=FIELD+1)# Loop over fields            AX15 147
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX15 148
IF(BUFIN(FIELD)>=voidflg:)BREAK#       Exit if numeric                  AX15 149
stringlim:(FIELD,IFIRST,ILAST)#        Extract param field              AX15 150
CALL AX02(IFIRST,IMARK,ISTOP,ISTEP,P1,P2,S1,S2)#                        AX15 151
#                                                                       AX15 152
# TREAT GENERAL PARAMETERS FIRST                                        AX15 153
# ------------------------------                                        AX15 154
#                                                                       AX15 155
IF (IMARK==MARK(5))#                   U overall                        AX15 156
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX15 157
IF (LINID==5)iquit:(AX1515)#           No ref card for uov              AX15 158
REFINE(5)=no:#                                                          AX15 159
NEXT#                                                                   AX15 160
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX15 161
IF (ISTOP<=STOP(4))DO L=IMARK,ISTOP,ISTEP;$# From shp to ext            AX15 162
CALL AX05(L,LINID-4)#                                                   AX15 163
IF ((IMARK>=MARK(6))&(ISTOP<=STOP(6)))$# Dispersion                     AX15 164
DO L=IMARK,ISTOP,ISTEP;DO I=P1,P2;CALL AX05(L+I,LINID-4)#               AX15 165
#                                                                       AX15 166
# TREAT ATOMIC PARAMETERS                                               AX15 167
# -----------------------                                               AX15 168
#                                                                       AX15 169
IF (IMARK>=MARK(7))#                   Atomic param                     AX15 170
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX15 171
DO L=IMARK,ISTOP,ISTEP#                Loop over sel atoms              AX15 172
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX15 173
TFT=NINT(atomtft:(L))#                 Get displ fact type              AX15 174
SFT=NINT(atomsft:(L))#                 Get scat fact type               AX15 175
IF ((S1>0)&((SFT<S1)\(SFT>S2)))NEXT#   Skip if not in bounds            AX15 176
IF (P1==0)#                            Ref/noref whole atom             AX15 177
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX15 178
MXP=3#                                 Default: refine only xyz         AX15 179
IF (REFPOP==yes:)MXP=MAX0(MXP,10)#                                      AX15 180
IF (TFT==2)MXP=MAX0(MXP,9)#                                             AX15 181
ELSE IF (TFT==1)MXP=MAX0(MXP,4)#                                        AX15 182
DO I=1,MXP;CALL AX05(L+I,yes:)#        Remove all constraints           AX15 183
IF (LINID==4)CALL AX05(L,no:)#         Store noref                      AX15 184
ELSE CALL AX05(L,yes:)#                Remove noref whole atom          AX15 185
NEXT#                                  Take next constraint             AX15 186
$)findnc:(MARKX,LX,L)#                 Get pointer                      AX15 187
ifnc:(LX,L)iquit:(AX1515)#             Turn on atom refinement first    AX15 188
IF (REFTMP!=3)#                        Cannot change displ fact type    AX15 189
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX15 190
IF ((P1==4)&(P2==9)&(TFT!=2))iquit:(AX1520)# Not aniso                  AX15 191
IF ((P1==4)&(P2==4)&(TFT!=1))iquit:(AX1521)# Not iso                    AX15 192
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX15 193
ELSE IF (LINID==5)#                    Change displ param type          AX15 194
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX15 195
DO I=P1,P2;CALL AX05(L+I,yes:)#        Remove all constraints           AX15 196
IF ((P1==4)&(P2==9))#                  Change to aniso                  AX15 197
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX15 198
atomtft:(L)=2.0#                       Set displ fact type              AX15 199
IF (TFT==0)DO I=6,1,-1;atomdis:(L+I)=uovl:*CELL(I)#                     AX15 200
IF (TFT==1)DO I=6,1,-1;atomdis:(L+I)=atomiso:(L)*CELL(I)#               AX15 201
NEXT#                                  Take next constraint             AX15 202
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX15 203
IF ((P1==4)&(P2==4))#                  Change to iso                    AX15 204
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX15 205
atomtft:(L)=1.0#                       Set displ fact type              AX15 206
IF (TFT==0)atomiso:(L)=uovl:#                                           AX15 207
IF (TFT==2)atomiso:(L)=(atomu11:(L)+atomu22:(L)+atomu33:(L))/3.0#       AX15 208
NEXT#                                  Take next constraint             AX15 209
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX15 210
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX15 211
DO I=P1,P2;CALL AX05(L+I,LINID-4)#     Loop over selcted parameters     AX15 212
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX15 213
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX15 214
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX15 215
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX15 216
readline:(AX151,NAX151)#               Read next card                   AX15 217
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX15 218
IF (LINID<6)iquit:(AX1531)#            Card misplaced                   AX15 219
RETURN#                                                                 AX15 220
END#                                                                    AX15 221
#                                                                       AX15 222
systemheader:(AX16)#                   Read bdf noref and constraints   AX16   1
####################################################################    AX16   2
#                                                                  #    AX16   3
#                               AX16                               #    AX16   4
#                                                                  #    AX16   5
####################################################################    AX16   6
SUBROUTINE AX16#                                                        AX16   7
#                                                                       AX16   8
include:(AACOMN)#                                                       AX16   9
include:(AXCOMN)#                                                       AX16  10
#                                                                       AX16  11
INTEGER I,J,K#                         General loop or pointer indexe   AX16  12
INTEGER IPOL#                          Polar constraint flag            AX16  13
INTEGER ISUB#                          Subj param address (1<=isub<=n   AX16  14
INTEGER L7#                            Atom pointer                     AX16  15
INTEGER LX,MX,LY,MY#                   Constraint list pointers         AX16  16
INTEGER MATCH#                         General purpose match flag       AX16  17
string:(NAME1,lena:)#                                                   AX16  18
string:(NAME2,lena:)#                  To compare atom names            AX16  19
INTEGER NREF#                          Max number of reference parame   AX16  20
INTEGER PTR#                           Pointer in directory packet      AX16  21
INTEGER SUBJ#                          Address of subject parameter     AX16  22
INTEGER TFT#                           Temp factor type of current at   AX16  23
INTEGER TYPE#                          Type of constraint               AX16  24
#                                                                       AX16  25
chardata:(AX161," All constraints on anisotropic displacement $#        AX16  26
parameters will not be used")#                                          AX16  27
chardata:(AX162," WARNING- Origin fix constraints on BDF are not used$# AX16  28
 for refinement")#                                                      AX16  29
chardata:(AX163,[" Subject atom not $#                                  AX16  30
selected, BDF constraint ignored"])#                                    AX16  31
chardata:(AX165,[" Parameter already norefed or constrained, $#         AX16  32
BDF constraint ignored"])#                                              AX16  33
chardata:(AX166,[" Reference atom(s) not $#                             AX16  34
selected, constraint ignored"])#                                        AX16  35
chardata:(AX167," Turn off refinement on general parameter type $#      AX16  36
from BDF constraint ignored")#                                          AX16  37
chardata:(AX168," Constraint on general parameters from BDF $#          AX16  38
ignored")#                                                              AX16  39
datastuff:#                                                             AX16  40
#                                                                       AX16  41
IPOL=yes:#                             Print polar constraint message   AX16  42
#                                                                       AX16  43
# INPUT CONSTRAINTS FROM LRCONS:                                        AX16  44
# ------------------------------                                        AX16  45
#                                                                       AX16  46
IWNT(1)=1;IWNT(2)=2;IWNT(3)=3#         Subj. atom name, param., mult    AX16  47
IWNT(4)=4;IWNT(5)=5#                   Constant and constraint type     AX16  48
SIGNAL(1)=5;SIGNAL(2)=0;SIGNAL(3)=0;SIGNAL(4)=0# Mandatory: subject &   AX16  49
indexpkt:(1,lrcons:,IPACK,IP,0,SIGNAL,IWNT,IREL)# Search directory      AX16  50
IF(IP>0)#                              Lrcons: present on bdf           AX16  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX16  52
IF(REFTMP<2)writeline:(1,AX161,NAX161,3,3)# Print ani message           AX16  53
IF(SIGNAL(4)<0)iquit:(AX1602)#         Missing mandatory items          AX16  54
NREF=(IPACK-(mxwd24c:+4))/(mxwd24c:+2)# Max nb of ref param             AX16  55
DO K=1,NREF#                           Loop over ref packets            AX16  56
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX16  57
I=10*K;J=5+3*(K-1)#                    Set pointers                     AX16  58
CALL AX06(IP,IPACK,I+1,PTR)#           Seek atom name in directory pa   AX16  59
IREL(J+1)=PTR#                         Store in irel                    AX16  60
CALL AX06(IP,IPACK,I+2,PTR)#           Seek parameter nb in directory   AX16  61
IREL(J+2)=PTR#                         Store in irel                    AX16  62
CALL AX06(IP,IPACK,I+3,PTR)#           Seek mult factor in directory    AX16  63
IREL(J+3)=PTR#                         Store in irel                    AX16  64
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX16  65
#                                                                       AX16  66
# LOOP OVER LRCONS PACKETS                                              AX16  67
# ========================                                              AX16  68
#                                                                       AX16  69
REPEAT#                                Search all the packets           AX16  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX16  71
readwpkt:(1,lrcons:,IPACK,IP,0)#       Read one packet                  AX16  72
IF(IP<=0)BREAK#                        Exit if last                     AX16  73
TYPE=NINT(QX(IP+IREL(5)))#             Type of constraint               AX16  74
ISUB=NINT(QX(IP+IREL(2)))#             Subject parameter number         AX16  75
IF(TYPE==4)#                           Use restraints for polar axis    AX16  76
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX16  77
IF (IPOL==yes:)writeline:(1,AX162,NAX162,3,3)# Print message            AX16  78
IPOL=no:#                              Print message only once          AX16  79
NEXT#                                  Ignore constraint                AX16  80
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX16  81
IF (TYPE==1)TYPE=constr:#              Reset type                       AX16  82
IF (TYPE==3)TYPE=noref:#                                                AX16  83
#                                                                       AX16  84
# TRANSLATE ATOM NAME INTO ATOM NUMBER IN MEMORY                        AX16  85
# ----------------------------------------------                        AX16  86
#                                                                       AX16  87
movertoc:(QX(IP+IREL(1)),1,NAME1,1,lena:,0)# Copy subj atom name        AX16  88
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX16  89
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX16  90
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Copy atom name                 AX16  91
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,MATCH)# Test if the same    AX16  92
IF(MATCH!=0)BREAK#                     Exit if match                    AX16  93
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX16  94
IF(L7>=STOP(7))#                       No match found                   AX16  95
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX16  96
writeline:(1,AX163,NAX163,3,3);CALL AX09(NREF)# Print warning           AX16  97
NEXT#                                  Take next constraint             AX16  98
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX16  99
#                                                                       AX16 100
# IGNORE D.P. CONSTRAINTS WHEN ALL ATOMS ISOTROPIC                      AX16 101
# ------------------------------------------------                      AX16 102
#                                                                       AX16 103
TFT=NINT(atomtft:(L7))#                Extract temp fact of subj atom   AX16 104
IF((ISUB>3)&((TFT<2)\(REFTMP<2)))NEXT# Displ types don't match          AX16 105
#                                                                       AX16 106
# STORE NOREF (INVARIANT) SIGNALS IN MARKX ARRAY                        AX16 107
# ----------------------------------------------                        AX16 108
#                                                                       AX16 109
SUBJ=L7+ISUB#                          Subject address (atom + param)   AX16 110
findc:(MARKX,MARKY,LX,LY,SUBJ)#        Find location in nc list         AX16 111
ifnc:(LX,SUBJ)#                        Already in list                  AX16 112
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX16 113
writeline:(1,AX165,NAX165,3,3);CALL AX09(NREF)# Print warning           AX16 114
NEXT#                                  Take next constraint             AX16 115
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX16 116
putnc:(LX,SUBJ,TYPE)#                  Put in list                      AX16 117
#                                                                       AX16 118
# STORE CONSTRAINT RELATIONSHIPS IN MARKX ARRAY                         AX16 119
# ---------------------------------------------                         AX16 120
#                                                                       AX16 121
IF (TYPE==constr:)#                    Constraint                       AX16 122
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX16 123
incrx:(LY)#                            Update constraint pointer        AX16 124
DO I=1,NREF#                           Loop over reference packets      AX16 125
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX16 126
ifeq:(QX(IP+IREL(5+(I-1)*3+3)),0.0)BREAK# Mult fact = 0                 AX16 127
movertoc:(QX(IP+IREL(5+(I-1)*3+1)),1,NAME1,1,lena:,0)# Copy name        AX16 128
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX16 129
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX16 130
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Copy atom name from list       AX16 131
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,MATCH)# Compare             AX16 132
IF(MATCH!=0)BREAK#                     Exit if match                    AX16 133
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX16 134
IF(L7>=STOP(7))BREAK#                  Exit if no match                 AX16 135
incrqx:(STOPY,STOPY+stepy:,AX1601)#    Get mem                          AX16 136
movereal:(QX,LY+1,QX,LY+stepy:+1,STOPY-stepy:-LY,1)# Move range         AX16 137
caddr:(LY)=REAL(L7)+QX(IP+IREL(5+(I-1)*3+2))# Store ref address         AX16 138
cfact:(LY)=QX(IP+IREL(5+(I-1)*3+3))/QX(IP+IREL(3))# Store normed fact   AX16 139
incry:(LY)#                            Take next packet                 AX16 140
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX16 141
IF(L7>=STOP(7))#                       No match                         AX16 142
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX16 143
writeline:(1,AX166,NAX166,3,3);CALL AX09(NREF)# Print warning           AX16 144
NEXT#                                  Take next constraint             AX16 145
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX16 146
incrqx:(STOPY,STOPY+stepy:,AX1601)#    Get mem                          AX16 147
movereal:(QX,LY+1,QX,LY+stepy:+1,STOPY-stepy:-LY,1)# Move range         AX16 148
caddr:(LY)=0.0#                        End-of-constraint marker         AX16 149
cfact:(LY)=QX(IP+IREL(4))/QX(IP+IREL(3))# Normalized constant           AX16 150
incry:(LY)#                            Ly points to next constraint     AX16 151
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX16 152
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX16 153
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX16 154
#                                                                       AX16 155
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX16 156
IF (NINT(atomtft:(L7))==0)BREAK#                                        AX16 157
IF (L7>=STOP(7))REFINE(5)=0#           Turn off u overall refinement    AX16 158
TST(5)=TST(7)#                         Same test as for atomic displ    AX16 159
RETURN#                                                                 AX16 160
END#                                                                    AX16 161
#                                                                       AX16 162
systemheader:(AX17)#                   Test lists for self-consistenc   AX17   1
####################################################################    AX17   2
#                                                                  #    AX17   3
#                               AX17                               #    AX17   4
#                                                                  #    AX17   5
####################################################################    AX17   6
SUBROUTINE AX17#                                                        AX17   7
#                                                                       AX17   8
include:(AACOMN)#                                                       AX17   9
include:(AXCOMN)#                                                       AX17  10
#                                                                       AX17  11
REAL CALC#                             Calculated value of constraint   AX17  12
REAL FACT#                             Multiplicative factor            AX17  13
INTEGER I#                             Loop index                       AX17  14
INTEGER L7#                            Atom pointer                     AX17  15
INTEGER L1X,L1Y,M1Y,N1Y#               Constraint 1 list pointers       AX17  16
INTEGER L2X,L2Y,M2Y,N2Y#               Constraint 2 list pointers       AX17  17
INTEGER MXP#                           Max nb of par for each atom      AX17  18
INTEGER OBJ1,SUBJ1,OFF1#               Object, subject & offset of li   AX17  19
INTEGER OBJ2,SUBJ2,OFF2#               Object, subject & offset of li   AX17  20
INTEGER RANGE#                         Number of fp numbers to move     AX17  21
INTEGER TFT#                           Displ fact type                  AX17  22
#                                                                       AX17  23
# SELF-CONSTISTENCY OF CONSTRAINT LIST                                  AX17  24
# ------------------------------------                                  AX17  25
#                                                                       AX17  26
L1Y=MARKY#                             Set constraint pointer           AX17  27
FOR (L1X=MARKX;L1X<STOPX;incrx:(L1X))# Loop over all constraints        AX17  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX17  29
SUBJ1=NINT(ncaddr:(L1X))#              Get subject 1 pointer            AX17  30
DO I=1,7;IF ((SUBJ1>=MARK(I))&(SUBJ1<=STOP(I)))BREAK# Find class        AX17  31
IF (I<5)OFF1=OFFSET(I)#                General parameter                AX17  32
ELSE OFF1=0#                           No offset                        AX17  33
L2Y=L1Y;N1Y=0#                         Set constraint 2 pointer & end   AX17  34
IF ((SUBJ1>=MARK(7))&(MOD(SUBJ1-MARK(7),STEP(7))==0))# Noref whole at   AX17  35
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX17  36
L7=SUBJ1#                              Set pointer to not refined ato   AX17  37
TFT=NINT(atomtft:(L7))#                Set displ type                   AX17  38
L2X=L1X+stepx:#                        Next constraint                  AX17  39
MXP=MAXPAR#                            Set to max nb of paramters       AX17  40
IF ((MXP<=9)&(TFT<2))#                                                  AX17  41
IF (TFT==0)MXP=3#                                                       AX17  42
ELSE MXP=4#                                                             AX17  43
DO I=1,MXP#                            Loop over parameters             AX17  44
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX17  45
IF ((TFT==0)&(I>3)&(I<10))NEXT#        U overall                        AX17  46
IF ((TFT==1)&(I>4)&(I<10))NEXT#        U iso                            AX17  47
ifnc:(L2X,L7+I)#                       Param twice not refined          AX17  48
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX17  49
IF (NINT(nctype:(L2X))!=noref:)#       Constraint                       AX17  50
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX17  51
M1Y=L1Y#                                                                AX17  52
WHILE (caddr:(M1Y)>small:)incry:(M1Y)# Loop over constraint             AX17  53
movereal:(QX,M1Y+stepy:+1,QX,L1Y+1,STOPY-M1Y-stepy:,1)# Kill constr     AX17  54
STOPY=STOPY-(M1Y+stepy:-L1Y)#          Update marker                    AX17  55
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX17  56
movereal:(QX,L2X+stepx:+1,QX,L2X+1,STOPY-L2X-stepx:,1)# Krunch          AX17  57
STOPX=STOPX-stepx:;L1Y=L1Y-stepy:#     Update markers                   AX17  58
MARKY=MARKY-stepy:;STOPY=STOPY-stepx:# Update markers                   AX17  59
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX17  60
FOR (M1Y=MARKY;M1Y<STOPY;incry:(M1Y))# Loop over constraints            AX17  61
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX17  62
FACT=0.0#                              Reset                            AX17  63
WHILE (caddr:(M1Y)>small:)#            Loop over one constraint         AX17  64
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX17  65
IF (NINT(caddr:(M1Y))==L7+I)#          Not ref param appears as refer   AX17  66
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX17  67
FACT=FACT+cfact:(M1Y)*QX(L7+I)#                                         AX17  68
movereal:(QX,M1Y+stepy:+1,QX,M1Y+1,STOPY-M1Y-stepy:,1)# Krunch          AX17  69
STOPY=STOPY-stepy:#                    Update markers                   AX17  70
IF (M1Y<L1Y)L1Y=L1Y-stepy:#            Update constraint                AX17  71
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX17  72
ELSE incry:(M1Y)#                      Next packet                      AX17  73
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX17  74
ifne:(FACT,0.0)cfact:(M1Y)=cfact:(M1Y)-FACT# Update constant            AX17  75
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX17  76
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX17  77
NEXT#                                  Take next constraint             AX17  78
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX17  79
IF (NINT(nctype:(L1X))==constr:)#      Non-degenerated constraint       AX17  80
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX17  81
N1Y=L1Y#                               Zero length                      AX17  82
WHILE (caddr:(N1Y)>small:)incry:(N1Y)# Find end of constraint           AX17  83
L2Y=N1Y+stepy:#                        Start of next one                AX17  84
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX17  85
FOR (L2X=L1X+stepx:;L2X<STOPX;incrx:(L2X))# Loop over following const   AX17  86
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX17  87
SUBJ2=NINT(ncaddr:(L2X))#              Get subject 2 pointer            AX17  88
DO I=1,7;IF ((SUBJ2>=MARK(I))&(SUBJ2<=STOP(I)))BREAK# Find class        AX17  89
IF (I<5)OFF2=OFFSET(I)#                General parameter                AX17  90
ELSE OFF2=0#                           No offset                        AX17  91
IF ((SUBJ2>=MARK(7))&(MOD(SUBJ2-MARK(7),STEP(7))==0))# Noref whole at   AX17  92
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX17  93
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX17  94
IF (NINT(nctype:(L2X))==constr:)#      Non-degenerated constraint       AX17  95
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX17  96
N2Y=L2Y#                               Zero length                      AX17  97
WHILE (caddr:(N2Y)>small:)incry:(N2Y)# Find end of constraint           AX17  98
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX17  99
IF (NINT(nctype:(L1X))==constr:)#      Non-degenerated 1st constraint   AX17 100
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX17 101
FOR (M1Y=L1Y;M1Y<N1Y;incry:(M1Y))#     Loop over body of 1st constr     AX17 102
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX17 103
OBJ1=NINT(caddr:(M1Y))#                Object 1                         AX17 104
IF (SUBJ2==OBJ1)#                      Object 1 is constrained in con   AX17 105
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX17 106
FACT=cfact:(M1Y)#                      Save mult factor                 AX17 107
movereal:(QX,M1Y+stepy:+1,QX,M1Y+1,STOPY-M1Y-stepy:,1)# Krunch          AX17 108
N1Y=N1Y-stepy:;STOPY=STOPY-stepy:#     Update markers                   AX17 109
L2Y=L2Y-stepy:;N2Y=N2Y-stepy:#                                          AX17 110
IF (NINT(nctype:(L2X))==constr:)#      Copy body of constr 2 into con   AX17 111
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX17 112
RANGE=N2Y-L2Y#                         Prepare to insert constr 2       AX17 113
incrqx:(STOPY,STOPY+RANGE,AX1701)#     Get memory                       AX17 114
movereal:(QX,M1Y+1,QX,M1Y+RANGE+1,STOPY-M1Y,1)# Shift list              AX17 115
N1Y=N1Y+RANGE;L2Y=L2Y+RANGE;N2Y=N2Y+RANGE# Update pointers              AX17 116
movereal:(QX,L2Y+1,QX,M1Y+1,RANGE,0)#  Copy constr 2                    AX17 117
DO I=1,RANGE/stepy:#                   Loop over inserted part          AX17 118
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX17 119
cfact:(M1Y)=FACT*cfact:(M1Y)#          Rescale mult factors             AX17 120
incry:(M1Y)#                           Take next packet                 AX17 121
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX17 122
cfact:(N1Y)=cfact:(N1Y)+FACT*cfact:(N2Y)# Update constant               AX17 123
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX17 124
ELSE#                                  Only update the constant         AX17 125
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX17 126
cfact:(N1Y)=cfact:(N1Y)+FACT*QX(SUBJ2+OFF2)# Update constant            AX17 127
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX17 128
M1Y=M1Y-stepy:#                        Reset pointer to previous valu   AX17 129
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX17 130
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX17 131
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX17 132
#   ***** END OF REPLACEMENT OF CONSTR 2 INTO 1                         AX17 133
IF (NINT(nctype:(L2X))==constr:)#      Non-degenerated 1st constraint   AX17 134
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX17 135
FOR (M2Y=L2Y;M2Y<N2Y;incry:(M2Y))#     Loop over body of 2nd constr     AX17 136
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX17 137
OBJ2=NINT(caddr:(M2Y))#                Object 2                         AX17 138
IF (SUBJ1==OBJ2)#                      Object 2 is constrained in con   AX17 139
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX17 140
FACT=cfact:(M2Y)#                      Save mult factor                 AX17 141
movereal:(QX,M2Y+stepy:+1,QX,M2Y+1,STOPY-M2Y-stepy:,1)# Krunch          AX17 142
N2Y=N2Y-stepy:;STOPY=STOPY-stepy:#     Update markers                   AX17 143
IF (NINT(nctype:(L1X))==constr:)#      Copy body of constr 1 into con   AX17 144
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX17 145
RANGE=N1Y-L1Y#                         Prepare to insert constr 2       AX17 146
incrqx:(STOPY,STOPY+RANGE,AX1701)#     Get memory                       AX17 147
movereal:(QX,M2Y+1,QX,M2Y+RANGE+1,STOPY-M2Y,1)# Shift list              AX17 148
N2Y=N2Y+RANGE#                         Update pointer                   AX17 149
movereal:(QX,L1Y+1,QX,M2Y+1,RANGE,0)#  Copy constr 2                    AX17 150
DO I=1,RANGE/stepy:#                   Loop over inserted part          AX17 151
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX17 152
cfact:(M2Y)=FACT*cfact:(M2Y)#          Rescale mult factors             AX17 153
incry:(M2Y)#                           Take next packet                 AX17 154
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX17 155
cfact:(N2Y)=cfact:(N2Y)+FACT*cfact:(N1Y)# Update constant               AX17 156
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX17 157
ELSE#                                  Only update the constant         AX17 158
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX17 159
cfact:(N2Y)=cfact:(N2Y)+FACT*QX(SUBJ1+OFF1)# Update constant            AX17 160
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX17 161
M2Y=M2Y-stepy:#                        Reset pointer to previous valu   AX17 162
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX17 163
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX17 164
L2Y=N2Y+stepy:#                        Set end-of-constraint            AX17 165
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX17 166
#   ***** END OF REPLACEMENT OF CONSTR 1 INTO 2                         AX17 167
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX17 168
#                                                                       AX17 169
# TEST ONE CONSTRAINT ALONE AND COMPUTE IT                              AX17 170
# ----------------------------------------                              AX17 171
#                                                                       AX17 172
IF (NINT(nctype:(L1X))==constr:)#      Non-degenerated constr 1         AX17 173
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX17 174
M1Y=L1Y#                               Set ptr to start-of-constr       AX17 175
WHILE (caddr:(M1Y)>small:)#            Loop over constraint             AX17 176
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX17 177
OBJ1=NINT(caddr:(M1Y))#                Get object 1                     AX17 178
N1Y=M1Y+stepy:#                        Next packet                      AX17 179
WHILE (caddr:(N1Y)>small:)#            Loop over rest of constraint     AX17 180
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX17 181
OBJ2=NINT(caddr:(N1Y))#                Get object                       AX17 182
IF (OBJ1==OBJ2)#                       Same object appears twice        AX17 183
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX17 184
FACT=cfact:(N1Y)#                      Save mult factor                 AX17 185
movereal:(QX,N1Y+stepy:+1,QX,N1Y+1,STOPY-N1Y-stepy:,1)# Krunch          AX17 186
N1Y=N1Y-stepy:;STOPY=STOPY-stepy:#     Update pointers                  AX17 187
cfact:(M1Y)=cfact:(M1Y)+FACT#          Update mult factor               AX17 188
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX17 189
incry:(N1Y)#                           Take next packet                 AX17 190
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX17 191
IF (OBJ1==SUBJ1)#                      Subject of constr appears as o   AX17 192
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX17 193
FACT=1.0-cfact:(M1Y)#                  Save object mult factor          AX17 194
ifeq:(FACT,0.0)iquit:(AX1702)#         Object mult fact is 0            AX17 195
movereal:(QX,M1Y+stepy:+1,QX,M1Y+1,STOPY-M1Y-stepy:,1)# Krunch          AX17 196
M1Y=M1Y-stepy:;STOPY=STOPY-stepy:#     Update pointers                  AX17 197
FOR (N1Y=L1Y;caddr:(N1Y)>small:;incry:(N1Y))# Rescale mult fact         AX17 198
cfact:(N1Y)=cfact:(N1Y)/FACT#                                           AX17 199
cfact:(N1Y)=cfact:(N1Y)/FACT#          Rescale constant                 AX17 200
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX17 201
incry:(M1Y)#                           Take next packet                 AX17 202
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX17 203
CALC=0.0#                              Clear constraint calc value      AX17 204
WHILE (caddr:(L1Y)>small:)#            Loop over constraint             AX17 205
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX17 206
OBJ2=NINT(caddr:(L1Y))#                Set object                       AX17 207
DO I=1,7;IF ((OBJ2>=MARK(I))&(OBJ2<=STOP(I)))BREAK# Find class          AX17 208
IF (I<5)OFF2=OFFSET(I)#                General parameter                AX17 209
ELSE OFF2=0#                           No offset                        AX17 210
CALC=CALC+cfact:(L1Y)*QX(OBJ2+OFF2)#   Perform sum                      AX17 211
incry:(L1Y)#                           Take next object                 AX17 212
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX17 213
QX(SUBJ1+OFF1)=CALC+cfact:(L1Y)#       With constant                    AX17 214
incry:(L1Y)#                           Take next constant               AX17 215
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX17 216
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX17 217
RETURN#                                                                 AX17 218
END#                                                                    AX17 219
#                                                                       AX17 220
systemheader:(AX18)#                   Origin-fix setting & limshift    AX18   1
#################################################################       AX18   2
#                                                               #       AX18   3
#                               AX18                            #       AX18   4
#                                                               #       AX18   5
#################################################################       AX18   6
SUBROUTINE AX18#                                                        AX18   7
include:(AACOMN)#                                                       AX18   8
include:(AXCOMN)#                                                       AX18   9
#                                                                       AX18  10
INTEGER FIELD,IFIRST,ILAST#            String delimiters                AX18  11
INTEGER I,J#                           Loop index                       AX18  12
INTEGER IMARK,ISTOP,ISTEP#             Markers in lists                 AX18  13
INTEGER L#                             Address of limsft param          AX18  14
INTEGER LA#                            Symop pointer                    AX18  15
INTEGER LX,LY,LZ,MX,MY,MZ#             Constr & limsft list pointers    AX18  16
INTEGER P1,P2,S1,S2#                   Matches                          AX18  17
REAL NORM#                             Vector norm                      AX18  18
INTEGER SUBJ#                          Subject address                  AX18  19
INTEGER TFT#                           Displ fact type                  AX18  20
REAL U(3),W(3)#                        Vector used in flack-schwaba a   AX18  21
#                                                                       AX18  22
chardata:(AX181,[" WARNING- Polar structure, automatic origin fixing"])#AX18  23
chardata:(AX182," WARNING- LIMSFT card on X[(]XXXXXXXXXXXXXXXXXXXXXXXX$#AX18  24
[)] will overwrite automatic origin fixing")#                           AX18  25
chardata:(AX183,MAXHKL TWINOP CONSTR NOREF REF LIMSFT END )#            AX18  26
chardata:(AX186," WARNING- LIMSFT card will release contraint and $#    AX18  27
turn on refinement")#                                                   AX18  28
intdata:(GEN,14)#                                                       AX18  29
datastuff:#                                                             AX18  30
#                                                                       AX18  31
MARKZ=STOPY;STOPZ=MARKZ#               Set bl list markers              AX18  32
TRACE=0.0;DO I=1,9;POLCON(I)=0.0#      Clear polcon & its trace         AX18  33
#                                                                       AX18  34
# READ LIMSFT CARDS                                                     AX18  35
# =================                                                     AX18  36
#                                                                       AX18  37
WHILE (LINID==6)#                      Loop over these cards            AX18  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX18  39
DO FIELD=1,mxfdim:,2#                  Loop over fields                 AX18  40
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX18  41
IF(BUFIN(FIELD)>=voidflg:)BREAK#       Exit if numeric                  AX18  42
IF (BUFIN(FIELD+1)<=voidflg:)iquit:(AX1803)# Error in limsft card       AX18  43
IF (BUFIN(FIELD+1)<0.0)iquit:(AX1804)# No neg values for lim shift      AX18  44
stringlim:(FIELD,IFIRST,ILAST)#        Extract string limits            AX18  45
CALL AX02(IFIRST,IMARK,ISTOP,ISTEP,P1,P2,S1,S2)# Syntax analysis        AX18  46
IF (P2>P1)iquit:(AX1801)#              No param span                    AX18  47
IF ((P1==0)&(IMARK>=MARK(6))&(ISTOP<=STOP(6)))iquit:(AX1802)# Paramet   AX18  48
IF ((P1==0)&(IMARK>=MARK(7))&(ISTOP<=STOP(7)))iquit:(AX1802)# Paramet   AX18  49
IF (((IMARK<MARK(7))\(ISTOP>STOP(7)))&((ISTOP-IMARK)/ISTEP>1))$#        AX18  50
iquit:(AX1801)#                        No inside-class spanning         AX18  51
IF (IMARK>=MARK(7))#                   Atomic param                     AX18  52
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX18  53
IF (ISTOP>IMARK)#                      Atom name is atom type name      AX18  54
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX18  55
DO IMARK=MARK(7),STOP(7)-STEP(7),STEP(7)#                               AX18  56
IF (NINT(atomsft:(IMARK))==S1)BREAK#   Exit on  first match             AX18  57
ISTOP=IMARK#                                                            AX18  58
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX18  59
TFT=NINT(atomtft:(IMARK))#             Get displ fact type              AX18  60
IF (P1<4)#                             Positional limsft card           AX18  61
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX18  62
ifne:(POLCON(4*P1-3),0.0)#             Diagonal el = 0 ?                AX18  63
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX18  64
movertoc:(atomn:(IMARK),1,AX182,28,lena:,0)# Atom name                  AX18  65
writeline:(0,AX182,NAX182,3,3)#        Print warning                    AX18  66
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX18  67
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX18  68
ELSE IF (P1<10)#                       Displ param                      AX18  69
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX18  70
IF (TFT==0)iquit:(AX1805)#             U overall                        AX18  71
IF ((P1>4)&(TFT==1))iquit:(AX1805)#    U iso                            AX18  72
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX18  73
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX18  74
L=IMARK+P1;findbl:(MARKZ,LZ,L)#                                         AX18  75
ifbl:(LZ,L)iquit:(AX1815)#             Already shift-limited            AX18  76
putbl:(LZ,L,BUFIN(FIELD+1))#           Insert limsft                    AX18  77
findc:(MARKX,MARKY,LX,LY,L)#                                            AX18  78
ifnc:(LX,L)#                           Already constrained or norefed   AX18  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX18  80
writeline:(1,AX186,NAX186,3,3)#        Print warning                    AX18  81
MY=LY#                                 Start of constraint              AX18  82
WHILE (caddr:(MY)>small:)incry:(MY)#   Loop over constraint             AX18  83
incry:(MY)#                            Start of next constraint         AX18  84
movereal:(QX,MY+1,QX,LY+1,STOPZ-MY,1)# Krunch constraint                AX18  85
STOPZ=STOPZ-(MY-LY);MARKZ=MARKZ-(MY-LY)#                                AX18  86
STOPY=STOPY-(MY-LY)#                   Update markers                   AX18  87
movereal:(QX,LX+stepx:+1,QX,LX+1,STOPZ-LX-stepx:,1)# Krunch nc list     AX18  88
STOPZ=STOPZ-stepx:;MARKZ=MARKZ-stepx:#                                  AX18  89
STOPY=STOPY-stepx:;MARKY=MARKY-stepy:#                                  AX18  90
STOPX=STOPX-stepx:#                    Update markers                   AX18  91
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX18  92
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX18  93
readline:(AX183,NAX183)#                                                AX18  94
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX18  95
IF (LINID<7)iquit:(AX1820)#            Card after limsft                AX18  96
#                                                                       AX18  97
# FIND POLAR AXIS                                                       AX18  98
# ---------------                                                       AX18  99
#                                                                       AX18 100
IF (ICENT==no:)#                       Origin may be unfixed            AX18 101
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX18 102
SUBJ=1;NORM=0.0#                       Clear trace & norm               AX18 103
DO LA=MARKA,STOPA-stepa:,stepa:#       Build projector                  AX18 104
DO I=1,9;POLCON(I)=POLCON(I)+QX(LA+I)/REAL(NSYM)#                       AX18 105
writeline:(0,AX181,NAX181,3,3)#        Print message                    AX18 106
findc:(MARKX,MARKY,LX,LY,MARK(7))#     Set pointers                     AX18 107
WHILE (LX<STOPX)#                      Loop over nc list                AX18 108
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX18 109
SUBJ=MOD(NINT(ncaddr:(LX))-MARK(7),STEP(7))# Parameter nb               AX18 110
IF (SUBJ==0)BREAK#                     One atom not refined             AX18 111
IF (SUBJ>3)#                           Non-positionnal parameters       AX18 112
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX18 113
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX18 114
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX18 115
WHILE(caddr:(LY)>small:)incry:(LY)#    Take next constraint             AX18 116
incry:(LY)#                                                             AX18 117
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX18 118
incrx:(LX);NEXT#                       Take next item in nc list        AX18 119
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX18 120
U(1)=0.0;U(2)=0.0;U(3)=0.0#            Clear u vector                   AX18 121
U(SUBJ)=1.0#                           Assume fact(subj param) = 1.0    AX18 122
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX18 123
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX18 124
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX18 125
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX18 126
SUBJ=MOD(NINT(caddr:(LY))-MARK(7),STEP(7))# Param nb of ref atom        AX18 127
U(SUBJ)=U(SUBJ)+cfact:(LY)#            Form u vector                    AX18 128
incry:(LY)#                            Increment constraint pointer     AX18 129
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX18 130
incry:(LY)#                            Skip constant                    AX18 131
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX18 132
NORM=SQRT(U(1)*U(1)+U(2)*U(2)+U(3)*U(3))# Take norm                     AX18 133
DO I=1,3#                              W = polcon*u/norm(u)             AX18 134
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX18 135
W(I)=0.0#                              Clear coordinate                 AX18 136
DO J=1,3;W(I)=W(I)+POLCON(3*(I-1)+J)*U(J)# Sum                          AX18 137
ifne:(NORM,0.0)W(I)=W(I)/NORM#         Normalize                        AX18 138
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX18 139
NORM=SQRT(W(1)*W(1)+W(2)*W(2)+W(3)*W(3))# Take norm                     AX18 140
IF (ABS(NORM-1.0)<0.01)#               |w| = 1                          AX18 141
DO I=1,3;DO J=1,3#                     Update polcon                    AX18 142
POLCON(3*(I-1)+J)=POLCON(3*(I-1)+J)-W(I)*W(J)#                          AX18 143
incrx:(LX)#                                                             AX18 144
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX18 145
IF (SUBJ!=0)DO I=1,3;TRACE=TRACE+POLCON(3*(I-1)+I)# Trace of polcon     AX18 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX18 147
#                                                                       AX18 148
RETURN#                                                                 AX18 149
END#                                                                    AX18 150
#                                                                       AX18 151
systemheader:(AX20)#                   Test everything before refinem   AX20   1
#################################################################       AX20   2
#                                                               #       AX20   3
#                               AX20                            #       AX20   4
#                                                               #       AX20   5
#################################################################       AX20   6
SUBROUTINE AX20#                                                        AX20   7
include:(AACOMN)#                                                       AX20   8
include:(AXCOMN)#                                                       AX20   9
#                                                                       AX20  10
CALL AX22#                             Read restraints                  AX20  11
CALL AX23#                             Output initial parameters        AX20  12
CALL AX25#                             Var count and pointers setting   AX20  13
RETURN#                                                                 AX20  14
END#                                                                    AX20  15
#                                                                       AX20  16
systemheader:(AX22)#                   Read lrdist:, lrangl:, lrdihe:   AX22   1
#################################################################       AX22   2
#                                                               #       AX22   3
#                               AX22                            #       AX22   4
#                                                               #       AX22   5
#################################################################       AX22   6
SUBROUTINE AX22#                                                        AX22   7
include:(AACOMN)#                                                       AX22   8
include:(AXCOMN)#                                                       AX22   9
#                                                                       AX22  10
INTEGER I,J,K#                         Loop indices                     AX22  11
INTEGER MATCH#                         Flag to compare strings          AX22  12
INTEGER L1,L7#                         List pointers                    AX22  13
INTEGER LR#                                                             AX22  14
INTEGER LX,LY,MX,MY#                   Constraint list pointer          AX22  15
INTEGER M7#                            Auxiliary atom pointer           AX22  16
INTEGER MXP#                           Atomic nb of params              AX22  17
string:(NAME1,lena:)#                  Atom names                       AX22  18
string:(NAME2,lena:)#                                                   AX22  19
INTEGER RANGE#                         Number of moved words            AX22  20
INTEGER TFT#                           Displacement factor type         AX22  21
INTEGER TYPE#                          Type of the restraint            AX22  22
#                                                                       AX22  23
chardata:(AX221," WARNING- Atom XXXXXXXXXXXXXXXXXXXXXXXX unknown[,] $#  AX22  24
restraint skipped")#                                                    AX22  25
chardata:(AX222,"Distance      Angle        $#                          AX22  26
Torsion Angle Rigid Bond    void")#                                     AX22  27
chardata:(AX223," Restraint Type                                   $#   AX22  28
                        Obs   E.s.d  Weight    Calc   E.s.d")#          AX22  29
chardata:(AX224,[" WARNING- Displacement type of atom xxxxxxxxxxxx$#    AX22  30
xxxxxxxxxxxx overall, restraint skipped"])#                             AX22  31
chardata:(AX225,[" WARNING- Displacement type of atoms xxxxxxxxxxxx$#   AX22  32
xxxxxxxxxxxx & xxxxxxxxxxxxxxxxxxxxxxxx isotropic, restraint skipped"])#AX22  33
intdata:(LRR,[40, 42, 43, 44])#        Restraint lr                     AX22  34
intdata:(PAR,[2, 3, 4, 2])#            Number of parameter of restrai   AX22  35
realdata:(FMT,[770752., 850742., 930742., 1010742., 1090742.])#         AX22  36
datastuff:#                                                             AX22  37
#                                                                       AX22  38
# TEST FOR NOT REFINED FACES                                            AX22  39
# --------------------------                                            AX22  40
#                                                                       AX22  41
IF (REFINE(1)>0)#                      Shape refined                    AX22  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX22  43
LX=MARKX#                              Set pointer                      AX22  44
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over faces                  AX22  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX22  46
ifnc:(LX,L1)#                          Face not refined                 AX22  47
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX22  48
IF (NINT(nctype:(LX))==noref:)facew:(L1)=0.0# Clear weight              AX22  49
incrx:(LX)#                            Take next item in nc list        AX22  50
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX22  51
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX22  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX22  53
MARKR(1)=STOPZ;STOPR(1)=MARKR(1);LR=MARKR(1)# Set restraint pointers    AX22  54
DO TYPE=1,4#                           Loop over restraint types        AX22  55
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX22  56
IF (TYPE>1)#                           Set pointers                     AX22  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX22  58
MARKR(TYPE)=STOPR(TYPE-1);STOPR(TYPE)=MARKR(TYPE);LR=MARKR(TYPE)#       AX22  59
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX22  60
#                                                                       AX22  61
# READ LOGICAL RECORDS DIRECTORIES                                      AX22  62
# --------------------------------                                      AX22  63
#                                                                       AX22  64
I=1#                                   First item in iwant/irel array   AX22  65
DO K=10,10*PAR(TYPE),10#               Loop over atoms                  AX22  66
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX22  67
IWNT(I)=K+1;I=I+1#                     Atom name                        AX22  68
DO J=1,12#                             Matrix & vector                  AX22  69
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX22  70
IWNT(I)=10*K+J;I=I+1#                  Store matrix & vector nb         AX22  71
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX22  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX22  73
IWNT(I)=93;I=I+1#                      Store observed value             AX22  74
IWNT(I)=94;I=I+1#                      Store esd of obs                 AX22  75
IWNT(I)=95;I=I+1#                      Store weight                     AX22  76
SIGNAL(1)=I-1#                         End of mandatory inputs          AX22  77
IWNT(I)=91;I=I+1#                      Store calc                       AX22  78
IWNT(I)=92#                            Esd of calc                      AX22  79
SIGNAL(2)=I#                           End of optionnal input           AX22  80
SIGNAL(3)=0;SIGNAL(4)=0#               Update mode                      AX22  81
indexpkt:(1,LRR(TYPE),IPACK,IP,0,SIGNAL,IWNT,IREL)# Read directory      AX22  82
IF (IP>0)#                             Lr: present on bdf               AX22  83
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX22  84
IF (SIGNAL(4)<0)#                      Missing mandatory items          AX22  85
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX22  86
IF (TYPE==1)iquit:(AX2201)#            Distance                         AX22  87
IF (TYPE==2)iquit:(AX2202)#            Angle                            AX22  88
IF (TYPE==3)iquit:(AX2203)#            Torsion                          AX22  89
IF (TYPE==4)iquit:(AX2204)#            Bond                             AX22  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX22  91
movechar:(AX222,1+(TYPE-1)*14,AX223,17,14,0)#                           AX22  92
writeline:(1,AX223,NAX223,3,4);LINRM=0# Print title                     AX22  93
REPEAT#                                                                 AX22  94
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX22  95
#                                                                       AX22  96
# READ ONE RESTRAINT                                                    AX22  97
# ------------------                                                    AX22  98
#                                                                       AX22  99
readwpkt:(1,LRR(TYPE),IPACK,IP,0)#     Read one packet                  AX22 100
IF (IP<=0)BREAK#                       Exit if last                     AX22 101
incrqx:(STOPR(TYPE),STOPR(TYPE)+PAR(TYPE)*recl:+9,AX2205)# Get memory   AX22 102
I=0#                                   Reset directory pointer          AX22 103
DO K=1,PAR(TYPE)#                      Loop over atoms                  AX22 104
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX22 105
#                                                                       AX22 106
# CHECK ATOM NAME                                                       AX22 107
# ---------------                                                       AX22 108
#                                                                       AX22 109
movertoc:(QX(IP+IREL(I+1)),1,NAME1,1,lena:,0)# Move name                AX22 110
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Test atom names                  AX22 111
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX22 112
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Move atom name                 AX22 113
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,MATCH)# Compare             AX22 114
IF (MATCH!=0)BREAK#                    Exit if match                    AX22 115
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX22 116
IF (L7>=STOP(7))#                      Name not found in atom list      AX22 117
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX22 118
movechar:(NAME1,1,AX221,16,lena:,0)#   Move to warning                  AX22 119
writeline:(0,AX221,NAX221,3,3)#        Print warning                    AX22 120
STOPR(TYPE)=LR#                        Discard this restraint           AX22 121
BREAK#                                 Take next restraint              AX22 122
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX22 123
IF (TYPE==4)#                          Rigid bond                       AX22 124
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX22 125
TFT=NINT(atomtft:(L7))#                Extract temp fact type           AX22 126
IF (TFT==0)#                           U overall                        AX22 127
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX22 128
movechar:(NAME1,1,AX224,37,lena:,0)#   Move atom name                   AX22 129
writeline:(1,AX224,NAX224,3,3)#        Print warning                    AX22 130
BREAK#                                 Take next restraint              AX22 131
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX22 132
IF ((TFT==1)&(I==2))#                  2nd atom isotropic               AX22 133
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX22 134
M7=NINT(resp:(LR,1))#                  1st atom pointer                 AX22 135
IF (NINT(atomtft:(M7))<2)#             Both isotropic                   AX22 136
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX22 137
movertoc:(atomn:(M7),1,AX225,38,lena:,0)# Move 1st atom name            AX22 138
movechar:(NAME1,1,AX225,65,lena:,0)#   Move 2nd atom name               AX22 139
writeline:(1,AX225,NAX225,3,3)#        Print warning                    AX22 140
BREAK#                                 Take next restraint              AX22 141
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX22 142
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX22 143
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX22 144
movechar:(NAME1,1,CHROT,2+(K-1)*17,15,0)# Copy name                     AX22 145
resp:(LR,K)=REAL(L7)#                  Save atom pointer                AX22 146
DO J=1,12;resm:(LR,K,J)=QX(IP+IREL(I+1+J))# Store matrix & vector       AX22 147
I=I+13#                                Update directory pointer         AX22 148
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX22 149
IF (K<=PAR(TYPE))NEXT#                 Take next restraint              AX22 150
resobs:(LR)=QX(IP+IREL(I+1))#          Observed value                   AX22 151
BUFOT(1)=resobs:(LR)#                                                   AX22 152
resd:(LR)=QX(IP+IREL(I+2))#            Esd of obs                       AX22 153
resw:(LR)=QX(IP+IREL(I+3))#            Weight                           AX22 154
BUFOT(1)=resobs:(LR);BUFOT(2)=resd:(LR);BUFOT(3)=resw:(LR)#             AX22 155
ncodefld:(BUFOT,1,CHROT,FMT,3)#                                         AX22 156
ifpr:(I+4)#                            Calc value of restraint          AX22 157
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX22 158
rescal:(LR)=QX(IP+IREL(I+4))#          Observed value                   AX22 159
ncodefld:(rescal:(LR),1,CHROT,FMT(4),1)#                                AX22 160
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX22 161
ELSE movechar:(AX222,56,CHROT,NINT(FMT(4))/10000-3,4,0)# Move void      AX22 162
ifpr:(I+5)#                            Esd of calc                      AX22 163
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX22 164
rescsd:(LR)=QX(IP+IREL(I+5))#          Get esd                          AX22 165
ncodefld:(rescsd:(LR),1,CHROT,FMT(5),1)#                                AX22 166
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX22 167
ELSE movechar:(AX222,56,CHROT,NINT(FMT(5))/10000-3,4,0)# Move void      AX22 168
writeline:(0,AX223,NAX223,1,4)#        Print restraint                  AX22 169
I=I+5#                                 Increment directory pointer      AX22 170
incrr:(LR,PAR(TYPE))#                  Increment restraint pointer      AX22 171
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX22 172
STOPR(TYPE)=LR#                        Reset end-of-list value          AX22 173
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX22 174
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX22 175
#                                                                       AX22 176
# PREPARE CONSTRAINT LISTS FOR NO INTENSITIES                           AX22 177
# -------------------------------------------                           AX22 178
#                                                                       AX22 179
IF (INTRES==no:)#                      No intensities                   AX22 180
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX22 181
IF (STOPR(4)==MARKR(1))iquit:(AX2206)# No intensities, no restraints    AX22 182
DO I=1,6;IF (REFINE(I)==2)REFINE(I)=1# Turn off general params refine   AX22 183
findc:(MARKX,MARKY,LX,LY,MARK(7))#     Find pointers                    AX22 184
movereal:(QX,LX+1,QX,MARKX+1,STOPX-LX,1)# Krunch noref/constr list      AX22 185
STOPX=STOPX-(LX-MARKX)#                Reset end-of-list marker         AX22 186
movereal:(QX,LY+1,QX,STOPX+1,STOPY-LY,1)# Krunch constr list            AX22 187
MARKY=STOPX;STOPY=MARKY+(STOPY-LY)#    Reset end-of-list marker         AX22 188
movereal:(QX,MARKZ+1,QX,STOPY+1,STOPR(4)-MARKZ,1)# Move lists           AX22 189
RANGE=MARKZ-STOPY#                     Number of krunched words         AX22 190
MARKZ=MARKZ-RANGE;STOPZ=STOPZ-RANGE#   Update markers                   AX22 191
DO I=1,4#                              Loop over restraint markers      AX22 192
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX22 193
MARKR(I)=MARKR(I)-RANGE;STOPR(I)=STOPR(I)-RANGE# Update                 AX22 194
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX22 195
IF (MARKR(4)==STOPR(4))#               No displ factors                 AX22 196
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX22 197
findc:(MARKX,MARKY,LX,LY,MARK(7))#     Get pointers                     AX22 198
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX22 199
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX22 200
ifnc:(LX,L7)#                          Atom not refined                 AX22 201
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX22 202
incrx:(LX);NEXT#                       Skip atom                        AX22 203
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX22 204
TFT=NINT(atomtft:(L7))#                Get displ fact type              AX22 205
MXP=MAXPAR#                            Get atomic parma nb              AX22 206
IF ((MXP<=9)&(TFT<2))#                                                  AX22 207
IF (TFT==0)MXP=3#                                                       AX22 208
ELSE MXP=4#                                                             AX22 209
DO I=1,MXP#                            Loop over parameters             AX22 210
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX22 211
IF ((TFT==0)&(I>3)&(I<10))NEXT#                                         AX22 212
IF ((TFT==1)&(I>4)&(I<10))NEXT#                                         AX22 213
IF ((REFPOP==no:)&(I==10))NEXT#                                         AX22 214
IF ((I<4)\(I>9))#                      Do not kill constraint           AX22 215
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX22 216
ifnc:(LX,L7+I)#                        Param is constrained             AX22 217
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX22 218
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX22 219
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX22 220
WHILE (caddr:(LY)>small:)incry:(LY)#   Skip constraint                  AX22 221
incry:(LY)#                                                             AX22 222
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX22 223
incrx:(LX)#                            Next constraint                  AX22 224
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX22 225
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX22 226
ELSE#                                  Keep displ param invariant       AX22 227
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX22 228
ifnc:(LX,L7+I)#                        Already constrained              AX22 229
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX22 230
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX22 231
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX22 232
MY=LY#                                 Set aux pointer                  AX22 233
WHILE (caddr:(MY)>small:)incry:(MY)#   Skip constraint                  AX22 234
incry:(MY)#                            Point to next constraint         AX22 235
movereal:(QX,MY+1,QX,LY+1,STOPR(4)-MY,1)# Krunch lists                  AX22 236
RANGE=MY-LY#                           Number of krunched words         AX22 237
STOPY=STOPY-RANGE#                     Set markers                      AX22 238
MARKZ=MARKZ-RANGE;STOPZ=STOPZ-RANGE#                                    AX22 239
DO J=1,4#                              Update restraint makers          AX22 240
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX22 241
MARKR(J)=MARKR(J)-RANGE;STOPR(J)=STOPR(J)-RANGE# Update                 AX22 242
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX22 243
nctype:(LX)=REAL(noref:)#              Fixed                            AX22 244
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX22 245
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX22 246
ELSE#                                  Param is variable                AX22 247
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX22 248
incrqx:(J,STOPR(4)+stepx:,AX2207)#     Get mem                          AX22 249
movereal:(QX,LX+1,QX,LX+stepx:+1,STOPR(4),1)# Shift arrays              AX22 250
ncaddr:(LX)=REAL(L7+I);nctype:(LX)=REAL(noref:)# Fix param              AX22 251
incrx:(STOPX);MARKY=STOPX;incrx:(STOPY)# Adjust pointers                AX22 252
MARKZ=STOPY;incrz:(STOPZ)#                                              AX22 253
DO J=1,4#                                                               AX22 254
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX22 255
incrx:(MARKR(J));incrx:(STOPR(J))#                                      AX22 256
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX22 257
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX22 258
incrx:(LX)#                                                             AX22 259
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX22 260
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX22 261
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX22 262
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX22 263
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX22 264
RETURN#                                                                 AX22 265
END#                                                                    AX22 266
#                                                                       AX22 267
systemheader:(AX23)#                   Output of initial param          AX23   1
#################################################################       AX23   2
#                                                               #       AX23   3
#                               AX23                            #       AX23   4
#                                                               #       AX23   5
#################################################################       AX23   6
SUBROUTINE AX23#                                                        AX23   7
include:(AACOMN)#                                                       AX23   8
include:(AXCOMN)#                                                       AX23   9
#                                                                       AX23  10
INTEGER I,J,M,N#                       Loop indexes                     AX23  11
INTEGER L1,L2,M2,L3,L6,L7#             List pointers                    AX23  12
INTEGER LX,MX,LY,LZ,MZ#                Constraint & limshift list poi   AX23  13
INTEGER MXP#                           Local number of atomic paramte   AX23  14
INTEGER NOREFA#                        Whole atom noref flag            AX23  15
INTEGER TFT#                           Number of tfs                    AX23  16
#                                                                       AX23  17
chardata:(AX231," Initial Parameters [(I=]Invariant C[=]Constrained $#  AX23  18
L[=]shift-Limited[)]")#                                                 AX23  19
chardata:(AX232," Crystal Faces    h   k   l   Distance       ESD$#     AX23  20
      Weight  Calc Dist")#                                              AX23  21
chardata:(AX233," Scale Factors")#                                      AX23  22
chardata:(AX234," Uov*100  xx.xxxx ")#                                  AX23  23
chardata:(AX235," Dispersion  f prime            f second")#            AX23  24
chardata:(AX236,[" Twinning Parameters                Centrosymmetry"])#AX23  25
chardata:(AX230,[" WARNING- U & Uij are multiplied by 100"])#           AX23  26
chardata:(AX237," Atom                        x/a      y/b      $#      AX23  27
z/c   U/U11     U22     U33     U12     U13     U23     Pop")#          AX23  28
chardata:(AX238," ExtinctionGaussianLorentzianAnisotropicIsotropic$#    AX23  29
Type12GeneralTensor")#                                                  AX23  30
chardata:(AX239," g (/10000)   [= XXXXX.XXXX                        ]$# AX23  31
 r mm (/10000)[ =XXXXX.XXXX                        ]")#                 AX23  32
realdata:(FM0,[190313.,230313.,270313.,380962., 490962., 600952., $#    AX23  33
710962.])#                                                              AX23  34
realdata:(FM1,[100642., 200642., 300642., 400642., 500642., 600642.])#  AX23  35
realdata:(FM2,[100642., 400642.])#                                      AX23  36
realdata:(FM3,[261042., 180632., 340632., 500632., 660632., 820632., $# AX23  37
980632.])#                                                              AX23  38
realdata:(FM4,[170742.])#                                               AX23  39
realdata:(FM5,[200842., 400842.])#                                      AX23  40
realdata:(FM6,[320752.,410752.,500752.,580642.,$#                       AX23  41
660642.,740642.,820642.,900642.,980642.,1060642.])#                     AX23  42
datastuff:#                                                             AX23  43
#                                                                       AX23  44
writeline:(2,AX231,NAX231,3,3)#        Print title                      AX23  45
LX=MARKX;LY=MARKY;LZ=MARKZ#            Set pointers                     AX23  46
#                                                                       AX23  47
# WRITE CRYSTAL SHAPE                                                   AX23  48
# -------------------                                                   AX23  49
#                                                                       AX23  50
IF (REFINE(1)>0)#                      Crystal shape refined            AX23  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX23  52
writeline:(1,0,0,3,3)#                 Print blank line                 AX23  53
LINRM=2#                               Reserve 2 lines                  AX23  54
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over distances              AX23  55
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX23  56
ifnc:(LX,L1)#                                                           AX23  57
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX23  58
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23  59
ELSE N=34#                                                              AX23  60
incrx:(LX)#                                                             AX23  61
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX23  62
ELSE ifbl:(LZ,L1)#                                                      AX23  63
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX23  64
N=48;incrz:(LZ)#                                                        AX23  65
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX23  66
ELSE N=1#                                                               AX23  67
J=NINT(FM0(7))/10000+1#                Status column                    AX23  68
movechar:(AX231,N,CHROT,J,1,0)#        Move status                      AX23  69
ncodefld:(facem:(L1),2,CHROT,FM0,7)#   Encode hkl, dist, esd, wght, c   AX23  70
writeline:(0,AX232,NAX232,1,3)#        Print line                       AX23  71
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX23  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX23  73
#                                                                       AX23  74
# WRITE ENTPOL PARAMETERS                                               AX23  75
# -----------------------                                               AX23  76
#                                                                       AX23  77
IF (REFINE(2)>0)#                      Ent-pol refined                  AX23  78
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX23  79
writeline:(1,0,0,3,3)#                 Print blank line                 AX23  80
LINRM=2#                               Reserve 2 lines                  AX23  81
M=NB(2)/(2-ICENT);L2=MARK(2)#          Get pointer                      AX23  82
DO I=1,M#                              Loop over twins                  AX23  83
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX23  84
findnc:(MARKX,LX,L2);findbl:(MARKZ,LZ,L2)# Find ent-pol in noref/cons   AX23  85
ifnc:(LX,L2)#                                                           AX23  86
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX23  87
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23  88
ELSE N=34#                                                              AX23  89
incrx:(LX)#                                                             AX23  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX23  91
ELSE ifbl:(LZ,L2)#                                                      AX23  92
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX23  93
N=48;incrz:(LZ)#                                                        AX23  94
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX23  95
ELSE N=1#                                                               AX23  96
IF (REFINE(2)==1)N=22#                 Ent-pol not refined              AX23  97
ncodefld:(entpol:(I),1,CHROT,FM2(1),1)# Encode twin                     AX23  98
J=NINT(FM2(1))/10000+1#                Get status column                AX23  99
movechar:(AX231,N,CHROT,J,1,0)#        Move status                      AX23 100
IF (ICENT==no:)#                       Not centrosymmetric group        AX23 101
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX23 102
M2=L2+NB(2)/2#                         Get centrosymmetric pointer      AX23 103
findnc:(MARKX,LX,M2);findbl:(MARKZ,LZ,M2)# Get constraint pointer       AX23 104
ifnc:(LX,M2)#                          Constraint or not refined        AX23 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX23 106
IF (NINT(nctype:(LX))==noref:)N=22#    Not refined                      AX23 107
ELSE                          N=34#    Constrained                      AX23 108
incrx:(LX)#                                                             AX23 109
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX23 110
ELSE ifbl:(LZ,M2)#                                                      AX23 111
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX23 112
N=48;incrz:(LZ)#                                                        AX23 113
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX23 114
ELSE N=1#                                                               AX23 115
IF (REFINE(2)==1)N=22#                 Twin not refined                 AX23 116
ncodefld:(entpol:(I+NB(2)/2),1,CHROT,FM2(2),1)# Encode  twin            AX23 117
J=NINT(FM2(2))/10000+1#                Get status column                AX23 118
movechar:(AX231,N,CHROT,J,1,0)#        Move status                      AX23 119
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX23 120
writeline:(0,AX236,J-1,1,0)#           Print line                       AX23 121
incr2:(L2)#                            Next packet                      AX23 122
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX23 123
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX23 124
#                                                                       AX23 125
# WRITE SCALE FACTORS                                                   AX23 126
# -------------------                                                   AX23 127
#                                                                       AX23 128
writeline:(1,0,0,3,3)#                 Print blank line                 AX23 129
LINRM=2#                               Reserve 2 lines                  AX23 130
I=1#                                                                    AX23 131
DO L3=MARK(3),STOP(3)-STEP(3),STEP(3)# Loop over scales                 AX23 132
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX23 133
ifnc:(LX,L3)#                                                           AX23 134
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX23 135
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 136
ELSE                          N=34#                                     AX23 137
incrx:(LX)#                                                             AX23 138
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX23 139
ELSE ifbl:(LZ,L3)#                                                      AX23 140
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX23 141
N=48;incrz:(LZ)#                                                        AX23 142
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX23 143
ELSE N=1#                                                               AX23 144
BUFOT(1)=1.0/SQRT(sclfac:(L3))#        Prepare for output               AX23 145
ncodefld:(BUFOT,1,CHROT,FM1(1+MOD(I-1,6)),1)# Encode fields             AX23 146
J=NINT(FM1(1+MOD(I-1,6)))/10000+1#     Get status column                AX23 147
movechar:(AX231,N,CHROT,J,1,0)#        Move status                      AX23 148
IF (MOD(I,6)==0)writeline:(0,AX233,NAX233,1,3)# Print line after 6      AX23 149
I=I+1#                                 Increment scale counter          AX23 150
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX23 151
IF (MARK(3)+STEP(3)==STOP(3))writeline:(0,AX233,NAX233-1,1,3)#          AX23 152
ELSE writeline:(0,AX233,NAX233,1,3)#                                    AX23 153
#                                                                       AX23 154
# WRITE EXTINCTION PARAMETERS                                           AX23 155
# ---------------------------                                           AX23 156
#                                                                       AX23 157
IF (REFINE(4)>0)#                      Refine extinction                AX23 158
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX23 159
movechar:(AX238,1,CHROT,1,11,0)#       Move title                       AX23 160
IF (EXTTYP>0)movechar:(AX238,12,CHROT,20,8,0)# Move gauss               AX23 161
ELSE         movechar:(AX238,20,CHROT,20,10,0)# Move lorentz            AX23 162
IF (IABS(EXTTYP)<=3)#                  Isotropic stuff                  AX23 163
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX23 164
movechar:(AX238,41,CHROT,35,9,0)#      Move iso                         AX23 165
IF (IABS(EXTTYP)!=3)#                  Type 1 & 2                       AX23 166
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX23 167
movechar:(AX238,50,CHROT,50,4,0)#      Move type                        AX23 168
movechar:(AX238,53+IABS(EXTTYP),CHROT,55,1,0)# Move 0 or 1              AX23 169
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX23 170
ELSE movechar:(AX238,56,CHROT,50,7,0)# Move general                     AX23 171
writeline:(1,0,0,1,3)#                 Print first line                 AX23 172
IF (IABS(EXTTYP)!=2)#                  Type 1 or gen                    AX23 173
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX23 174
ifnc:(LX,MARK(4))#                     Test if noref/constr             AX23 175
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX23 176
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 177
ELSE                          N=34#                                     AX23 178
incrx:(LX)#                                                             AX23 179
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX23 180
ELSE ifbl:(LZ,MARK(4))#                                                 AX23 181
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX23 182
N=48;incrz:(LZ)#                                                        AX23 183
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX23 184
ELSE                          N=1#                                      AX23 185
IF (REFINE(4)==1)             N=22#    Not refined                      AX23 186
movechar:(AX239,1,CHROT,1,50,0)#       Copy line                        AX23 187
movechar:(AX231,N,CHROT,27,1,0)#       Copy status                      AX23 188
ncodefld:(g:,1,CHROT,FM3,1)#           Print g                          AX23 189
writeline:(0,0,0,1,3)#                 Print line                       AX23 190
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX23 191
IF (IABS(EXTTYP)!=1)#                  Type 2 or gen                    AX23 192
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX23 193
ifnc:(LX,MARK(4)+1)#                                                    AX23 194
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX23 195
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 196
ELSE                          N=34#                                     AX23 197
incrx:(LX)#                                                             AX23 198
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX23 199
ELSE ifbl:(LZ,MARK(4)+1)#                                               AX23 200
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX23 201
N=48;incrz:(LZ)#                                                        AX23 202
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX23 203
ELSE                          N=1#                                      AX23 204
IF (REFINE(4)==1)             N=22#    Extinction not refined           AX23 205
movechar:(AX239,51,CHROT,1,50,0)#      Copy line                        AX23 206
movechar:(AX231,N,CHROT,27,1,0)#       Copy status                      AX23 207
ncodefld:(rho:,1,CHROT,FM3,1)#         Print r in mm                    AX23 208
writeline:(0,0,0,1,3)#                 Print line                       AX23 209
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX23 210
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX23 211
ELSE#                                  Anisotropic stuff                AX23 212
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX23 213
movechar:(AX238,30,CHROT,35,11,0)#     Move ani                         AX23 214
IF (IABS(EXTTYP)!=6)#                  Type 1 & 2                       AX23 215
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX23 216
movechar:(AX238,50,CHROT,50,4,0)#      Move type                        AX23 217
movechar:(AX238,50+IABS(EXTTYP),CHROT,55,1,0)# Move 0 or 1              AX23 218
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX23 219
ELSE movechar:(AX238,56,CHROT,50,7,0)# Move general                     AX23 220
writeline:(0,0,0,1,3)#                 Print line                       AX23 221
IF (IABS(EXTTYP)!=5)#                  Type 1 or gen                    AX23 222
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX23 223
movechar:(AX238,63,CHROT,2,6,0)#       Move type                        AX23 224
movechar:(AX238,54,CHROT,9,1,0)#       Move 1                           AX23 225
DO J=1,6#                                                               AX23 226
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX23 227
ifnc:(LX,MARK(4)+1+J)#                                                  AX23 228
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX23 229
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 230
ELSE                          N=34#                                     AX23 231
incrx:(LX)#                                                             AX23 232
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX23 233
ELSE ifbl:(LZ,MARK(4)+1+J)#                                             AX23 234
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX23 235
N=48;incrz:(LZ)#                                                        AX23 236
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX23 237
ELSE N=1#                                                               AX23 238
IF (REFINE(4)==1)N=22#                 Extinction not refined           AX23 239
M=NINT(FM3(J+1))/10000+1#              Get column                       AX23 240
movechar:(AX231,N,CHROT,M,1,0)#        Copy status                      AX23 241
ncodefld:(extani1:(J),1,CHROT,FM3(J+1),1)#                              AX23 242
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX23 243
writeline:(0,0,0,1,3)#                 Print line                       AX23 244
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX23 245
IF (IABS(EXTTYP)!=4)#                  Type 2 or gen                    AX23 246
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX23 247
movechar:(AX238,63,CHROT,2,6,0)#       Move type                        AX23 248
movechar:(AX238,55,CHROT,9,1,0)#       Move 1                           AX23 249
DO J=1,6#                                                               AX23 250
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX23 251
ifnc:(LX,MARK(4)+7+J)#                                                  AX23 252
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX23 253
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 254
ELSE                          N=34#                                     AX23 255
incrx:(LX)#                                                             AX23 256
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX23 257
ELSE ifbl:(LZ,MARK(4)+7+J)#                                             AX23 258
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX23 259
N=48;incrz:(LZ)#                                                        AX23 260
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX23 261
ELSE N=1#                                                               AX23 262
IF (REFINE(4)==1)N=22#                 Extinction not refined           AX23 263
M=NINT(FM3(J+1))/10000+1#              Get column                       AX23 264
movechar:(AX231,N,CHROT,M,1,0)#        Copy status                      AX23 265
ncodefld:(extani2:(J),1,CHROT,FM3(J+1),1)#                              AX23 266
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX23 267
writeline:(0,0,0,1,3)#                 Print line                       AX23 268
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX23 269
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX23 270
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX23 271
#                                                                       AX23 272
# WRITE OVERALL TEMPERATURE FACTORS                                     AX23 273
# ---------------------------------                                     AX23 274
#                                                                       AX23 275
IF (REFINE(5)==0)N=22#                 Overall displ fact refined       AX23 276
ELSE N=1#                                                               AX23 277
ifbl:(LZ,MARK(5)+1)#                   Uov shift-limited                AX23 278
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX23 279
N=48;incrz:(LZ)#                                                        AX23 280
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX23 281
movechar:(AX231,N,AX234,18,1,0)#                                        AX23 282
ncodefld:(100.0*uovl:,1,AX234,FM4,1)#  Encode u & esd                   AX23 283
uovl:=utob:*uovl:#                                                      AX23 284
writeline:(1,AX234,NAX234,3,3)#        Print line                       AX23 285
#                                                                       AX23 286
# WRITE DISPERSION PARAMETERS                                           AX23 287
# ---------------------------                                           AX23 288
#                                                                       AX23 289
findnc:(LX,LX,MARK(6))#                                                 AX23 290
IF (REFINE(6)>0)#                      Dispersion refined               AX23 291
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX23 292
writeline:(1,0,0,3,3)#                 Blank line                       AX23 293
LINRM=2#                               Reserve 2 lines                  AX23 294
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over dispersion             AX23 295
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX23 296
movertoc:(nattyp:(L6),1,CHROT,2,lens:,0)# Insert ff name                AX23 297
ncodefld:(redisp:(L6),1,CHROT,FM5,2)#  Encode re & im                   AX23 298
DO I=1,2#                              Loop over re/im                  AX23 299
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 37 AX23 300
ifnc:(LX,L6+I)#                        Param not variable               AX23 301
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 38 AX23 302
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 303
ELSE N=34#                                                              AX23 304
incrx:(LX)#                                                             AX23 305
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 38 AX23 306
ELSE ifbl:(LZ,L6+I)#                                                    AX23 307
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 39 AX23 308
N=48;incrz:(LZ)#                                                        AX23 309
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 39 AX23 310
ELSE N=1#                                                               AX23 311
IF (REFINE(6)==1)N=22#                                                  AX23 312
J=NINT(FM5(I))/10000+1#                Get destination column           AX23 313
movechar:(AX231,N,CHROT,J,1,0)#        Code                             AX23 314
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 37 AX23 315
writeline:(0,AX235,NAX235,1,3)#        Print line                       AX23 316
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX23 317
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX23 318
#                                                                       AX23 319
# WRITE ATOM PARAMETERS                                                 AX23 320
# ---------------------                                                 AX23 321
#                                                                       AX23 322
writeline:(1,AX230,NAX230,3,3)#        Print warning                    AX23 323
writeline:(1,AX237,NAX237,3,3)#        Print title                      AX23 324
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX23 325
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 40 AX23 326
ifnc:(LX,L7)#                          Atom norefed                     AX23 327
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 41 AX23 328
incrx:(LX);NOREFA=yes:#                                                 AX23 329
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 41 AX23 330
ELSE NOREFA=no:#                                                        AX23 331
TFT=NINT(atomtft:(L7))#                Get displ fact type              AX23 332
movertoc:(atomn:(L7),1,CHROT,2,lena:,0)# Move atom name                 AX23 333
MXP=MAXPAR#                            Set to max nb of paramters       AX23 334
IF ((MXP<=9)&(TFT<2))#                                                  AX23 335
IF (TFT==0)MXP=3#                                                       AX23 336
ELSE MXP=4#                                                             AX23 337
DO J=1,MXP#                            Loop over parameters             AX23 338
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 42 AX23 339
IF ((TFT==0)&(J>3)&(J<10))NEXT#        U overall                        AX23 340
IF ((TFT==1)&(J>4)&(J<10))NEXT#        U iso                            AX23 341
ifnc:(LX,L7+J)#                                                         AX23 342
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 43 AX23 343
IF (NINT(nctype:(LX))==noref:)N=22#                                     AX23 344
ELSE N=34#                                                              AX23 345
incrx:(LX)#                                                             AX23 346
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 43 AX23 347
ELSE ifbl:(LZ,L7+J)#                   Var shift-limited                AX23 348
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 44 AX23 349
N=48;incrz:(LZ)#                                                        AX23 350
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 44 AX23 351
ELSE IF (NOREFA==yes:)N=22#            Noref whole atom                 AX23 352
ELSE N=1#                                                               AX23 353
IF ((REFPOP==no:)&(J==10))N=22#        Pp not refined                   AX23 354
I=NINT(FM6(J))/10000+1#                Status column                    AX23 355
movechar:(AX231,N,CHROT,I,1,0)#        Code status                      AX23 356
IF ((TFT==0)&(J>3)&(J<10))NEXT#                                         AX23 357
IF ((TFT==1)&(J>4)&(J<10))NEXT#                                         AX23 358
IF ((J>3)&(J<10))#                     Transform to b & betas           AX23 359
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 45 AX23 360
ncodefld:(100.0*atomxyz:(L7+J),1,CHROT,FM6(J),1)# Encode jth field      AX23 361
BUFOT(J)=100.0*atomxyz:(L7+J)#         Print as u or uij                AX23 362
IF (TFT==1)  atomxyz:(L7+J)=atomxyz:(L7+J)*utob:# To b                  AX23 363
ELSE IF (J<7)atomxyz:(L7+J)=atomxyz:(L7+J)*CELL(J+3)# To betaij         AX23 364
ELSE         atomxyz:(L7+J)=0.5*atomxyz:(L7+J)*CELL(J+3)#               AX23 365
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 45 AX23 366
ELSE ncodefld:(atomxyz:(L7+J),1,CHROT,FM6(J),1)# Encode jth field       AX23 367
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 AX23 368
IF (REFPOP==no:)#                      Population not refined           AX23 369
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 46 AX23 370
ncodefld:(atompop:(L7),1,CHROT,FM6(10),1)# Encode pop                   AX23 371
movechar:(AX231,22,CHROT,NINT(FM6(10))/10000+1,1,0)# Code status        AX23 372
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 46 AX23 373
M=NINT(FM6(MAXPAR))/10000#             Find end-of-title                AX23 374
writeline:(0,AX237,M+1,1,3)#           Print final atomic params        AX23 375
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 40 AX23 376
RETURN#                                                                 AX23 377
END#                                                                    AX23 378
#                                                                       AX23 379
systemheader:(AX25)#                   Variables counter                AX25   1
####################################################################    AX25   2
#                                                                  #    AX25   3
#                               AX25                               #    AX25   4
#                                                                  #    AX25   5
####################################################################    AX25   6
SUBROUTINE AX25#                                                        AX25   7
#                                                                       AX25   8
include:(AACOMN)#                                                       AX25   9
include:(AXCOMN)#                                                       AX25  10
#                                                                       AX25  11
INTEGER I#                             General loop index               AX25  12
INTEGER L#                             General params pointer           AX25  13
INTEGER L6,L7#                         List pointers                    AX25  14
INTEGER LX,LY,LZ#                      Constraint & limshift list poi   AX25  15
INTEGER L20#                           Variable pointer                 AX25  16
INTEGER MXP#                           Local number of atomic paramet   AX25  17
INTEGER RANGE#                         Number of words to reserve for   AX25  18
INTEGER TFT#                           Atomic temp fact type            AX25  19
#                                                                       AX25  20
# REPLACE CONSTRAINT FLAG BY ITS ADDRESS IN Y LIST                      AX25  21
# ------------------------------------------------                      AX25  22
#                                                                       AX25  23
LY=MARKY#                              Set constr pointer               AX25  24
DO LX=MARKX,STOPX-stepx:,stepx:#       Loop over nc list                AX25  25
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX25  26
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX25  27
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX25  28
nctype:(LX)=REAL(LY)#                  Replace by address               AX25  29
WHILE (caddr:(LY)>small:)incry:(LY)#   Take next constraint             AX25  30
incry:(LY)#                            Skip constant                    AX25  31
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX25  32
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX25  33
#                                                                       AX25  34
# RESERVE MEM SPACE FOR GENERAL VARIABLES                               AX25  35
# ---------------------------------------                               AX25  36
#                                                                       AX25  37
IF ((ICENT==yes:)&(REFINE(6)==0))IMAG=1#                                AX25  38
ELSE IMAG=2#                                                            AX25  39
IF ((ICENT==yes:)\(REFINE(2)==0))IFRI=1#                                AX25  40
ELSE IFRI=2#                                                            AX25  41
LX=MARKX;LY=MARKY;LZ=MARKZ#            Set list pointers                AX25  42
MARK20=STOPR(4)+1;MARK1(1)=MARK20;L20=MARK20# Start of variable list    AX25  43
DO I=1,5#                              Loop over general parameters     AX25  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX25  45
IF (REFINE(I)>1)#                      Refined class                    AX25  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX25  47
DO L=MARK(I),STOP(I)-STEP(I),STEP(I)#  Loop over one class              AX25  48
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX25  49
ifnc:(LX,L)incrx:(LX)#                 Parameter not variable           AX25  50
ELSE#                                  Parameter is variable            AX25  51
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX25  52
DO LY=MARKY,STOPY-stepy:,stepy:#       Loop over body constraints       AX25  53
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX25  54
IF (NINT(caddr:(LY))==L)caddr:(LY)=REAL(L20)# Replace address           AX25  55
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX25  56
ifbl:(LZ,L)#                           Variable is limited              AX25  57
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX25  58
bladdr:(LZ)=REAL(L20)#                 Replace address                  AX25  59
incrz:(LZ)#                            Take next limshift               AX25  60
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX25  61
L20=L20+1#                             Increment variable pointer       AX25  62
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX25  63
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX25  64
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX25  65
MARK1(I+1)=L20#                        Set vector list marker           AX25  66
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX25  67
IF (REFINE(6)>1)#                                                       AX25  68
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX25  69
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)#                                  AX25  70
DO I=1,2#                              Loop over re, im                 AX25  71
ifnc:(LX,L6+I)incrx:(LX)#                                               AX25  72
ELSE#                                                                   AX25  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX25  74
DO LY=MARKY,STOPY-stepy:,stepy:#                                        AX25  75
IF (NINT(caddr:(LY))==L6+I)caddr:(LY)=REAL(L20)#                        AX25  76
ifbl:(LZ,L6+I)#                                                         AX25  77
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX25  78
bladdr:(LZ)=REAL(L20)#                                                  AX25  79
incrz:(LZ)#                                                             AX25  80
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX25  81
L20=L20+1#                                                              AX25  82
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX25  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX25  84
MARK1(7)=L20#                                                           AX25  85
#                                                                       AX25  86
# REPLACE REF PARAM ADDRESS BY DERIVATIVE ADDRESS                       AX25  87
# -----------------------------------------------                       AX25  88
#                                                                       AX25  89
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX25  90
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX25  91
ifnc:(LX,L7)#                          Atom norefed                     AX25  92
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX25  93
incrx:(LX)#                            Get next n/c/b                   AX25  94
NEXT#                                  Take next atom                   AX25  95
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX25  96
TFT=NINT(atomtft:(L7))#                Temp fact of atom                AX25  97
MXP=MAXPAR#                            Set to max nb of paramters       AX25  98
IF ((MXP<=9)&(TFT<2))#                                                  AX25  99
IF (TFT==0)MXP=3#                                                       AX25 100
ELSE MXP=4#                                                             AX25 101
DO I=1,MXP#                            Loop over param                  AX25 102
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX25 103
IF((TFT==0)&(I>3)&(I<10))NEXT#         Uov and uij params               AX25 104
IF((TFT==1)&(I>4)&(I<10))NEXT#         U and uij params                 AX25 105
IF((REFPOP==no:)&(I==10))NEXT#         Pop param                        AX25 106
ifnc:(LX,L7+I)#                        Noref or constr param            AX25 107
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX25 108
incrx:(LX)#                            Take next n/c/b                  AX25 109
NEXT#                                  Take next param                  AX25 110
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX25 111
DO LY=MARKY,STOPY-stepy:,stepy:#       Replace var address by ...       AX25 112
IF(NINT(caddr:(LY))==L7+I)caddr:(LY)=REAL(L20)# Derivative address      AX25 113
ifbl:(LZ,L7+I)#                        Shift limited                    AX25 114
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX25 115
bladdr:(LZ)=REAL(L20)#                 Replace limsft address           AX25 116
incrz:(LZ)#                            Take next b/l                    AX25 117
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX25 118
L20=L20+1#                             Increment var pointer            AX25 119
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX25 120
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX25 121
STOP20=L20-1;VARTOT=L20-MARK20;RANGE=(VARTOT*(VARTOT+1))/2# End of va   AX25 122
MARKP=MARK1(5);STEPP=L20-MARKP#        P list                           AX25 123
IF (REFINE(2)>0)I=IMAG*NB(2)#          1 or 2 lists for each twin       AX25 124
ELSE            I=IMAG#                                                 AX25 125
STOPP=MARKP+I*STEPP-1#                 End-of-mero/imaginary list       AX25 126
MARK21=STOPP+1#                        Start of difference list         AX25 127
STOP21=MARK21+VARTOT-1#                Diff vector                      AX25 128
MARK22=STOP21+1#                       Beginning of matrix list         AX25 129
IF ((IWT==1)&(ABS(CORDIS)<=small:))#   1/sigma**2 weighting scheme      AX25 130
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX25 131
incrqx:(STOP22,MARK22+RANGE-1,AX2501)# Get mem                          AX25 132
STOP23=STOP22;STOP24=STOP23#           Set end of mem marker            AX25 133
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX25 134
ELSE#                                  Other weighting shemes           AX25 135
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX25 136
STOP22=MARK22+RANGE-1#                 End of normal matrix             AX25 137
MARK23=STOP22+1#                       Beginning of other matrix        AX25 138
STOP23=MARK23+RANGE-1#                 End of other matrix              AX25 139
MARK24=STOP23+1#                       Beginning of covariance matrix   AX25 140
incrqx:(STOP24,MARK24+RANGE-1,AX2501)# Get mem for refinement           AX25 141
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX25 142
ifne:(TRACE,0.0)#                      Reserve mem for polar limshift   AX25 143
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX25 144
MARKT=STOP24+1#                        Start of list                    AX25 145
incrqx:(STOPT,MARKT+NB(7),AX2501)#     Get mem                          AX25 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX25 147
#                                                                       AX25 148
RETURN#                                                                 AX25 149
END#                                                                    AX25 150
#                                                                       AX25 151
systemheader:(AX30)#                   Refinement control               AX30   1
####################################################################    AX30   2
#                                                                  #    AX30   3
#                               AX30                               #    AX30   4
#                                                                  #    AX30   5
####################################################################    AX30   6
SUBROUTINE AX30(ICYCLE)#                                                AX30   7
#                                                                       AX30   8
include:(AACOMN)#                                                       AX30   9
include:(AXCOMN)#                                                       AX30  10
#                                                                       AX30  11
INTEGER ICYCLE#                        Current cycle nb                 AX30  12
#                                                                       AX30  13
CALL AX31(ICYCLE)#                     Get lrrefl: directory            AX30  14
IF ((IP<=0)&(INTRES==yes:))iquit:(AX3001)# Lrrefl: not there            AX30  15
movereal:(0.0,1,QX,MARK20,STOP24-MARK20+1,2)# Clear mem space           AX30  16
movereal:(0.0,1,BUFIN,1,31,2)#         Clear agreement factors          AX30  17
IF (IP>0)CALL AX32(ICYCLE)#            Observations                     AX30  18
BUFIN(18)=0.0#                         Clear restraint counter          AX30  19
IF (STOPR(3)>MARKR(1))CALL AX34(ICYCLE)# Restraints on xyz              AX30  20
IF (STOPR(4)>MARKR(4))CALL AX35(ICYCLE)# Rigid bond restraints          AX30  21
IF ((ABS(CORDIS)>small:)&(IWT==1))$#   1/s**2, but non-diag vo          AX30  22
movereal:(QX,MARK22,QX,MARK23,MARK23-MARK22,0)# Copy matrix             AX30  23
IF (REFINE(1)>0)CALL AX33(ICYCLE)#     Refinement on distances          AX30  24
RETURN#                                                                 AX30  25
END#                                                                    AX30  26
#                                                                       AX30  27
systemheader:(AX31)#                   Read directory of lr 20          AX31   1
####################################################################    AX31   2
#                                                                  #    AX31   3
#                               AX31                               #    AX31   4
#                                                                  #    AX31   5
####################################################################    AX31   6
SUBROUTINE AX31(ICYCLE)#                                                AX31   7
include:(AACOMN)#                                                       AX31   8
include:(AXCOMN)#                                                       AX31   9
#                                                                       AX31  10
INTEGER I,M#                                                            AX31  11
INTEGER ICYCLE#                        Current cycle                    AX31  12
#                                                                       AX31  13
chardata:(AX311," WARNING- Default values of Tbar will be used in the$# AX31  14
 extinction calculation")#                                              AX31  15
datastuff:#                                                             AX31  16
#                                                                       AX31  17
rewindbdf:(1)#                         Rewind input file                AX31  18
M=DATSET*1000#                         Data set bias for idnums         AX31  19
IWNT(1)=1;IWNT(2)=2;IWNT(3)=3#         Indices,sin(t/l),mlt,equiv.      AX31  20
IF (INTREF==yes:)IWNT(4)=M+300#        Relative i                       AX31  21
ELSE             IWNT(4)=M+302#        |f|**2                           AX31  22
IWNT(5)=IWNT(4)+1#                     Sigma i or f**2                  AX31  23
IWNT(6)=M+900#                         Weight from bdf                  AX31  24
IWNT(7)=M+204#                         1/lp for refinement on i         AX31  25
IWNT(8)=M+201#                         Absorption for ref on i          AX31  26
IWNT(9)=M+11#                          Angle psi (for ani extinction)   AX31  27
IWNT(10)=M+308#                        Rcode                            AX31  28
IWNT(11)=M+309#                        Scale factor group               AX31  29
IWNT(12)=M+200#                        Tbar for extinction              AX31  30
IWNT(13)=M+304#                        F                                AX31  31
IWNT(14)=M+305#                        Sigma f                          AX31  32
DO I=1,NB(1);IWNT(I+15)=M+210+I#       Derivatives w.r.to face dist     AX31  33
DO I=1,NB(6);IWNT(I+15+NB(1))=500+I#   Form factor idnums               AX31  34
SIGNAL(1)=5;SIGNAL(2)=NB(6)+NB(1)+15#  Set search limits                AX31  35
SIGNAL(3)=SIGNAL(2);SIGNAL(4)=0#       Set search limit & archiv        AX31  36
indexpkt:(1,lrrefl:,IPACK,IP,0,SIGNAL,IWNT,IREL)# Point to refl dir     AX31  37
IF (IP<=0)RETURN#                      Lrrefl is not there              AX31  38
IF(SIGNAL(4)<0)iquit:(AX3102)#         Item missing.......162005        AX31  39
#                                                                       AX31  40
#   CHECK-OUT REQUIRED POINTERS                                         AX31  41
#   ---------------------------                                         AX31  42
#                                                                       AX31  43
DO I=1,NB(1);IF (IREL(15+I)<=0)iquit:(AX3103)# Dt/d(dl) missing         AX31  44
IF ((IABS(EXTTYP)>3)&(IREL(9)==0))iquit:(AX3106)#                       AX31  45
IF((REFINE(4)>0)&(IREL(12)==0))#       No tbar for extinction           AX31  46
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX31  47
writeline:(1,AX311,NAX311,3,3)#        Write warning- defaults          AX31  48
IF(TBAR<small:)#                       Null default                     AX31  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX31  50
IF(INEU>2)TBAR=0.3#                    Default for xray data in mm      AX31  51
ELSE      TBAR=1.5#                    Default for neutron data in mm   AX31  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX31  53
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX31  54
IF (INTREF==no:)#                      Refinement on |f|**2             AX31  55
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX31  56
IREL(8)=0;IREL(7)=0#                   Neither absorption nor lp corr   AX31  57
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX31  58
#                                                                       AX31  59
RETURN#                                                                 AX31  60
END#                                                                    AX31  61
systemheader:(AX32)#                   Calculation of sf and deriv      AX32   1
####################################################################    AX32   2
#                                                                  #    AX32   3
#                               AX32                               #    AX32   4
#                                                                  #    AX32   5
####################################################################    AX32   6
SUBROUTINE AX32(ICYCLE)#                                                AX32   7
include:(AACOMN)#                                                       AX32   8
include:(AXCOMN)#                                                       AX32   9
#                                                                       AX32  10
REAL TEMP#                             Temp variable                    AX32  11
REAL AA(2),BB(2),C,S,T#                Structure factor components      AX32  12
REAL ABSO# ## Absorption & mean path                                    AX32  13
REAL AC(48),BC(48),FF#                 Total structure factor compone   AX32  14
REAL AH,AK,AL#  ## Equivalent h,k,l                                     AX32  15
INTEGER CODE#   ## Status code of refelction (1=obs, 2=less-than, 3=e   AX32  16
REAL DA(2),DB(2)## Real & imag dispersion components                    AX32  17
REAL DELA(11),DELB(11)#                Parameters derivative            AX32  18
REAL EQV(480)#  ## Space for generated reflections                      AX32  19
REAL FAC#       ## Multiplication factor                                AX32  20
REAL FAC1,FAC2# ## Real & imag scattering components                    AX32  21
REAL FO#        ## F rel ("observation")                                AX32  22
REAL FMUL#      ## Atom multiplicative factor                           AX32  23
INTEGER HOLO#   ## Twin number                                          AX32  24
REAL HS(3),H(3)#                       Reflection indices               AX32  25
REAL OBS,CALC,DIF#                     Observed and calculated i        AX32  26
INTEGER I,J,L,M,MM#                    General indices                  AX32  27
INTEGER LA,LB#                         Symop & twinop list pointers     AX32  28
INTEGER L1,L3,M3,L6,L7#                List pointers                    AX32  29
INTEGER LX,MX,LY,MY#                   Constraint list pointers         AX32  30
INTEGER L20,L21,L22,L23#               Vector & normal matrix pointer   AX32  31
INTEGER L16#    ## Dispersion variables pointer                         AX32  32
INTEGER ICYCLE# ## Current cycle number                                 AX32  33
INTEGER IREF#   ## Refinement flag                                      AX32  34
INTEGER IREJ#   ## Rejection flag                                       AX32  35
INTEGER ISKIP#  ## Count of skipped reflections                         AX32  36
REAL LORZ#      ## Lorentz-polarization                                 AX32  37
INTEGER MXP#    ## Local number of atomic parameters                    AX32  38
INTEGER MEQV#   ## Length of symeq table                                AX32  39
INTEGER NSG#    ## Scale group number                                   AX32  40
INTEGER NX,NY# ## Auxiliary pointers                                    AX32  41
INTEGER REF#                           Reference param address          AX32  42
REAL SIGMA#     ## Sigma of observation                                 AX32  43
REAL SCATF(sfmax:)#                    Current reflection scatt.facts   AX32  44
INTEGER SFT#    ## Scattering factor type                               AX32  45
INTEGER STEPI#  ## Used to go to next row in matrix                     AX32  46
INTEGER TFT#    ## Number of temp. factors                              AX32  47
REAL STOL#      ## Sin(theta/lambda)                                    AX32  48
REAL STL2#      ## Sin(theta/lambda) sqare                              AX32  49
REAL SUMCOS#    ## Cos summation of fc                                  AX32  50
REAL SUMSIN#    ## Sin summation of fc                                  AX32  51
REAL WEIGHT,WFF,SQRW## Weight on observed value, on F & square root     AX32  52
#                                                                       AX32  53
realdata:(P2,6.2831853)#                                                AX32  54
chardata:(AX321," WARNING- Reflection with unknown scale factor $#      AX32  55
number skipped")#                                                       AX32  56
datastuff:#                                                             AX32  57
#                                                                       AX32  58
# SET RELATIVE POINTERS, INITIALIZATION                                 AX32  59
# -------------------------------------                                 AX32  60
#                                                                       AX32  61
movereal:(0.0,1,EQV,1,480,2)#          Zero equiv hkl array             AX32  62
writeline:(0,0,0,1,3)#                 Add one blank line               AX32  63
LINRM=5#                               Reserve page space               AX32  64
MEQV=10*NSYM;YEXT=1.0;BUFIN(20)=voidflg:#                               AX32  65
findnc:(MARKX,NX,MARK(6))#             Find beg of dsp constr           AX32  66
IF (REFINE(2)==0)HOLO=0#               No twins                         AX32  67
ELSE HOLO=(STOPB-MARKB)/stepb:-1#      Count all matrices               AX32  68
#                                                                       AX32  69
#     LOOP OVER THE REFLECTIONS                                         AX32  70
#     =========================                                         AX32  71
#                                                                       AX32  72
ISKIP=0#                               Reset skip counter               AX32  73
REPEAT#                                Reflection loop                  AX32  74
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX32  75
readwpkt:(1,lrrefl:,IPACK,IP,0)#       Read reflection packet           AX32  76
IF(IP<=0)BREAK#                        End of lrrefl:                   AX32  77
#                                      *********** test skip limit      AX32  78
IREJ=no:#                              Reset refln reject flag          AX32  79
ISKIP=ISKIP+1#                         Count reflection                 AX32  80
IF(ISKIP!=NSKIP)IREJ=yes:#             Skip this reflection             AX32  81
ELSE ISKIP=0#                          Reset skip count                 AX32  82
#                                      ********** miller indices hkl    AX32  83
gethkl:(IP+IREL(1),HS)#                Extract hkl                      AX32  84
IF(ABS(HS(1))>HMAX)IREJ=yes:#          H out of bounds                  AX32  85
IF(ABS(HS(2))>KMAX)IREJ=yes:#          K out of bounds                  AX32  86
IF(ABS(HS(3))>LMAX)IREJ=yes:#          L out of bounds                  AX32  87
#                                      *********** test st/l limit      AX32  88
STOL=QX(IP+IREL(2))#                   Get st/l                         AX32  89
IF((STOL<SMIN-small:)\(STOL>SMAX+small:))IREJ=yes:# Stol outside rang   AX32  90
#                                      *********** reject refl          AX32  91
IF(IREJ==yes:)NEXT#                    Test if refln rejected           AX32  92
BUFIN(1)=BUFIN(1)+1.0#                 Increment reflection counter     AX32  93
#                                                                       AX32  94
# NOW OBESRVATIONS, SCALES AND WEIGHTS (LOAD ONLY)                      AX32  95
# ------------------------------------------------                      AX32  96
#                                      *********** extract obs & esd    AX32  97
OBS=QX(IP+IREL(4));SIGMA=QX(IP+IREL(5))#                                AX32  98
#                                      *********** extract mult fact    AX32  99
ifpr:(7)LORZ=QX(IP+IREL(7))#           Get lorentz-polarization from    AX32 100
ELSE    LORZ=1.0#                      No lorentz-polarization          AX32 101
ifpr:(8)ABSO=QX(IP+IREL(8))#           Get absorption from bdf          AX32 102
ELSE    ABSO=1.0#                      No absorption                    AX32 103
ifpr:(12)TBAR=QX(IP+IREL(12))#         Get mean path from bdf           AX32 104
#                                      *********** test status          AX32 105
ifpr:(10)CODE=NINT(QX(IP+IREL(10)))#                                    AX32 106
ELSE     CODE=1#                       Observed                         AX32 107
#                                      *********** extract scales       AX32 108
ifpr:(11)NSG=MAX(1,NINT(QX(IP+IREL(11))))# Get scale group number       AX32 109
ELSE     NSG=1#                        Default scale group              AX32 110
DO L3=MARK(3),STOP(3)-STEP(3),STEP(3)# Loop over scale groups           AX32 111
IF(NSG==NINT(sclgrp:(L3)))BREAK#       Exit if match found              AX32 112
IF(L3>=STOP(3))#                       No scale group found             AX32 113
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX32 114
writeline:(0,AX321,NAX321,3,3)#        Print warning                    AX32 115
NEXT#                                  Ignore reflection                AX32 116
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX32 117
STL2=STOL*STOL#                        Calc st/l **2                    AX32 118
IF (REFINE(1)>0)#                      Refine shape                     AX32 119
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX32 120
ABSO=1.0/QX(IP+IREL(8));I=1#           Transmission                     AX32 121
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over faces                  AX32 122
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX32 123
ABSO=ABSO+(facedc:(L1)-facedo:(L1))*QX(IP+IREL(I+15))# New transmissi   AX32 124
I=I+1#                                                                  AX32 125
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX32 126
ABSO=1.0/ABSO#                                                          AX32 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX32 128
FAC=1.0/(ABSO*LORZ)#                   1/lp                             AX32 129
#                                      ********** |fo|                  AX32 130
FO=SQRT(ABS(OBS*LORZ*ABSO)/sclfac:(L3))# |fo| in [e-] with sign         AX32 131
IF (OBS<0.0)FO=-FO#                                                     AX32 132
#                                      ********** weights               AX32 133
IF(SIGMA<small:)   SIGMA=100.#         Prevent silly sigmas             AX32 134
IF (IWT==0)        WEIGHT=1.0#         Unit weights                     AX32 135
IF (IWT==1)        WEIGHT=1.0/SIGMA**2# 1/sigma**2 weights              AX32 136
IF (IWT==2)ifpr:(6)WEIGHT=QX(IP+IREL(6))**2# Weights from bdf           AX32 137
ELSE               WEIGHT=1.0/SIGMA**2#                                 AX32 138
IF (OBS>0.0)$#                         Positive observed intensity      AX32 139
WFF=SQRT(FAC*sclfac:(L3)/(SQRT(OBS**2+1.0/(2.0*WEIGHT))-OBS+small:))#   AX32 140
ELSE WFF=SQRT(FAC*sclfac:(L3)*SQRT(2.0*WEIGHT))#                        AX32 141
SQRW=SQRT(AMAX1(WEIGHT,0.0))#          Square root of obs weight        AX32 142
#                                                                       AX32 143
IF (OBS/SIGMA<CUTSIG)CODE=6#           Set code                         AX32 144
IF (WEIGHT==0.0)CODE=7#                Not in matrix                    AX32 145
IF (CODE==5)#                          Systematically absent            AX32 146
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX32 147
FF=0.0;movereal:(0.0,1,QX,MARK20,STOP20-MARK20+1,2)# Compute f & deri   AX32 148
DO I=1,8#                              Clear as & bs                    AX32 149
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX32 150
AC(I)=0.0;BC(I)=0.0#                                                    AX32 151
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX32 152
GO TO 200#                             Skip tedious part                AX32 153
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX32 154
#                                                                       AX32 155
# NOW FORM FACTORS                                                      AX32 156
# ----------------                                                      AX32 157
#                                      ********** extract form fac      AX32 158
DO I=1,NB(6)#                          Loop over form factors           AX32 159
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX32 160
IF(IREL(I+15+NB(1))>0)M=IP+IREL(I+15+NB(1))# Get sf from bdf            AX32 161
ELSE IF(INEU>2)M=NINT(STOL/DSFTB)+MARKC+1+(I-1)*STEPC# Get sf from ta   AX32 162
ELSE           M=MARKC+I#              Get neutron scat length          AX32 163
SCATF(I)=QX(M)#                        Sf for this reflection           AX32 164
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX32 165
#                                                                       AX32 166
# LOOP OVER TWINNED                                                     AX32 167
# -----------------                                                     AX32 168
#                                                                       AX32 169
movereal:(0.0,1,QX,MARK20,STOPP+1-MARK20,2)# Clear all derivatives      AX32 170
LB=MARKB#                              Set twinning matrix pointer      AX32 171
DO M=0,HOLO#                           Loop over twinned                AX32 172
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX32 173
MM=HOLO+1+M#                           Friedel pair twin pointer        AX32 174
H(1)=QX(LB+1)*HS(1)+QX(LB+2)*HS(2)+QX(LB+3)*HS(3)# Build new h          AX32 175
H(2)=QX(LB+4)*HS(1)+QX(LB+5)*HS(2)+QX(LB+6)*HS(3)#                      AX32 176
H(3)=QX(LB+7)*HS(1)+QX(LB+8)*HS(2)+QX(LB+9)*HS(3)#                      AX32 177
incrb:(LB)#                            Take next matrix                 AX32 178
AC(M+1)=0.0;BC(M+1)=0.0#               Clear ff components              AX32 179
AC(MM+1)=0.0;BC(MM+1)=0.0#             Clear friedel pair               AX32 180
IF (REFINE(2)>0)#                      Twinning operations              AX32 181
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX32 182
IF (ICENT==no:)#                       Skip                             AX32 183
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX32 184
IF ((ABS(entpol:(M+1))<small:)&(ABS(entpol:(MM+1))<small:))NEXT#        AX32 185
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX32 186
ELSE ifeq:(entpol:(M+1),0.0)NEXT#                                       AX32 187
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX32 188
#SKIP fractional index lattices.
IF($#                                  Check bad hkl indices
ABS(ANINT(H(1))-H(1))>0.02\#           non integral h'
ABS(ANINT(H(2))-H(2))>0.02\#           or non integral k'
ABS(ANINT(H(3))-H(3))>0.02) NEXT#      or non integral l'
#write(6,*)HS(1),HS(2),HS(3),M,H(1),H(2),H(3)
#                                                                       AX32 189
# NOW GENERATE HKL EQUIVALENT                                           AX32 190
# ---------------------------                                           AX32 191
#                                      ********* gen equiv hkl          AX32 192
J=0#                                   Set equiv hkl store point        AX32 193
DO LA=MARKA,STOPA-stepa:,stepa:#       Loop over symm matrices          AX32 194
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX32 195
AH=H(1)*QX(LA+1)+H(2)*QX(LA+2)+H(3)*QX(LA+3)# Equivalent h              AX32 196
AK=H(1)*QX(LA+4)+H(2)*QX(LA+5)+H(3)*QX(LA+6)# Equivalent k              AX32 197
AL=H(1)*QX(LA+7)+H(2)*QX(LA+8)+H(3)*QX(LA+9)# Equivalent l              AX32 198
EQV(J+1)=P2*AH;EQV(J+2)=P2*AK;EQV(J+3)=P2*AL# Store hkl indices         AX32 199
EQV(J+4)=AH*AH;EQV(J+5)=AK*AK;EQV(J+6)=AL*AL# Store new diag els        AX32 200
EQV(J+7)=2.0*AH*AK;EQV(J+8)=2.0*AH*AL;EQV(J+9)=2.0*AK*AL# Store new h   AX32 201
EQV(J+10)=P2*(H(1)*QX(LA+10)+H(2)*QX(LA+11)+H(3)*QX(LA+12))# Phase sh   AX32 202
J=J+10#                                Incr parameter array             AX32 203
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX32 204
#                                    ***********SET ATOM POINTERS       AX32 205
findc:(MARKX,MARKY,LX,LY,MARK(7))#     Set ncb and constr ptrs          AX32 206
#                                                                       AX32 207
#     LOOP OVER ALL ATOMS                                               AX32 208
#     ===================                                               AX32 209
#                                                                       AX32 210
L20=MARK1(7)#                          Set var ptr on atomic params     AX32 211
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)#                                  AX32 212
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX32 213
SFT=NINT(atomsft:(L7));L6=MARK(6)+(SFT-1)*STEP(6)# Extract scatf type   AX32 214
TFT=NINT(atomtft:(L7))#                Extract tempf type               AX32 215
IREF=INTRES#                           No matrix from intensities       AX32 216
IF ((CODE==3)\(CODE>4))IREF=no:#       No derivatives                   AX32 217
ifnc:(LX,L7)#                          Noref all atomic params          AX32 218
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX32 219
IREF=no:#                              Put refine off                   AX32 220
incrx:(LX)#                            Get next n/c                     AX32 221
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX32 222
MXP=MAXPAR#                            Set to max nb of paramters       AX32 223
IF ((MXP<=9)&(TFT<2))#                                                  AX32 224
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX32 225
IF (TFT==0)  MXP=3#                                                     AX32 226
ELSE         MXP=4#                                                     AX32 227
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX32 228
SUMCOS=0.;SUMSIN=0.#                   Zero fc components               AX32 229
IF(ICENT==no:)movereal:(0.,1,DELB,1,MXP,2)# Zero imag derivative sums   AX32 230
movereal:(0.,1,DELA,1,MXP,2)#          Zero real derivative compone     AX32 231
#                                                                       AX32 232
#     LOOP OVER EQUIVALENT HKL                                          AX32 233
#     ------------------------                                          AX32 234
#                                                                       AX32 235
FOR(J=0;J<MEQV;J=J+10)#                ********** equiv indices         AX32 236
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX32 237
TEMP=AMOD(EQV(J+1)*atomx:(L7)$#                                         AX32 238
         +EQV(J+2)*atomy:(L7)$#                                         AX32 239
         +EQV(J+3)*atomz:(L7)$#                                         AX32 240
         +EQV(J+10),P2)#               H*x/a+k*y/b+l*z/c + phi shift    AX32 241
C=COS(TEMP);      S=SIN(TEMP)#         Sin and cos (hx+ky+lz+t)         AX32 242
IF(TFT>1)#                             Anisotropic temp.fact.           AX32 243
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX32 244
T=EXP(-EQV(J+4)*atomu11:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX32 245
      -EQV(J+5)*atomu22:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX32 246
      -EQV(J+6)*atomu33:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX32 247
      -EQV(J+7)*atomu12:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX32 248
      -EQV(J+8)*atomu13:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX32 249
      -EQV(J+9)*atomu23:(L7))#         Sum h(i)*h(j)*u(i,j)             AX32 250
S=T*S;        C=T*C#                   Sin & cos contributions          AX32 251
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX32 252
IF (ICENT==no:)$#                                                       AX32 253
SUMSIN=SUMSIN+S#                       Sum components                   AX32 254
SUMCOS=SUMCOS+C#                       Sum components                   AX32 255
#                                                                       AX32 256
IF(IREF==no:)NEXT#                     *********** sum derivative       AX32 257
DO I=1,3#                              Loop over x,y,z parameters       AX32 258
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX32 259
IF(ICENT==no:)DELB(I)=DELB(I)+EQV(J+I)*C# Sum real components           AX32 260
DELA(I)=DELA(I)-EQV(J+I)*S#            Sum imag components              AX32 261
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX32 262
IF (TFT>1)#                            Ani displ factor                 AX32 263
DO I=4,9#                              Loop over equiv bij              AX32 264
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX32 265
IF(ICENT==no:)DELB(I)=DELB(I)-EQV(I+J)*S# Sum imag components           AX32 266
DELA(I)=DELA(I)-EQV(I+J)*C#            Sum real components              AX32 267
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX32 268
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX32 269
#                                                                       AX32 270
#    APPLY TF AND SITE MULT AND POPULATION PARAMETER                    AX32 271
#    -----------------------------------------------                    AX32 272
#                                                                       AX32 273
FMUL=REAL(RFLMLT)*atommul:(L7)*atompop:(L7)# Set atom multiplicity      AX32 274
IF (TFT==0)FMUL=FMUL*EXP(-STL2*uovl:)# Apply overall displ fact         AX32 275
IF (TFT==1)FMUL=FMUL*EXP(-STL2*atomiso:(L7))# Apply iso displ           AX32 276
#                                                                       AX32 277
#   SUM STRUCTURE FACTOR COMPONENTS                                     AX32 278
#   -------------------------------                                     AX32 279
#                                                                       AX32 280
FAC1=SCATF(SFT);FAC2=0.0#              Scattering factor without disp   AX32 281
IF (REFINE(6)>0)#                      Apply dispersion                 AX32 282
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX32 283
L6=MARK(6)+(SFT-1)*STEP(6)#            Get disp pointer                 AX32 284
FAC1=FAC1+redisp:(L6);FAC2=imdisp:(L6)# Include disp                    AX32 285
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX32 286
AA(1)=FMUL*(FAC1*SUMCOS-FAC2*SUMSIN)#  Real part                        AX32 287
BB(1)=FMUL*(FAC1*SUMSIN+FAC2*SUMCOS)#  Im part                          AX32 288
IF (IFRI==2)#                          Apply ent-pol                    AX32 289
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX32 290
AA(2)=FMUL*(FAC1*SUMCOS+FAC2*SUMSIN)#  Re part of friedel               AX32 291
BB(2)=FMUL*(-FAC1*SUMSIN+FAC2*SUMCOS)# Im part of friedel               AX32 292
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX32 293
#                                                                       AX32 294
#     ACCUMULATE DERIVATIVES                                            AX32 295
#     ----------------------                                            AX32 296
#                                                                       AX32 297
IF (IREF==no:)GO TO 100#               Skip derivative part             AX32 298
IF ((REFINE(5)==2)&(TFT==0))#          Overall displ fact derivative    AX32 299
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX32 300
QX(MARKP+M*IMAG*STEPP)=QX(MARKP+M*IMAG*STEPP)-STL2*AA(1)#               AX32 301
IF (IMAG==2)$#                         Imaginary part                   AX32 302
QX(MARKP+(M*IMAG+1)*STEPP)=QX(MARKP+(M*IMAG+1)*STEPP)-STL2*BB(1)#       AX32 303
IF (IFRI==2)#                          Friedel pair                     AX32 304
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX32 305
QX(MARKP+MM*IMAG*STEPP)=QX(MARKP+MM*IMAG*STEPP)-STL2*AA(2)#             AX32 306
QX(MARKP+(MM*IMAG+1)*STEPP)=QX(MARKP+(MM*IMAG+1)*STEPP)-STL2*BB(2)#     AX32 307
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX32 308
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX32 309
#                                      ***** dispersion                 AX32 310
IF(REFINE(6)>1)#                       Test if dispersion refined       AX32 311
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX32 312
DA(1)=FMUL*SUMCOS;DA(2)=DA(1)#         Calc cos component               AX32 313
DB(1)=FMUL*SUMSIN;DB(2)=-DB(1)#        Calc sin component               AX32 314
findnc:(NX,L,L6+1)#                    Re norefed/constrained           AX32 315
DO I=1,2#                              Re/im loop                       AX32 316
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX32 317
ifnc:(L,L6+I)#                         Test if norefed/constrained      AX32 318
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX32 319
IF (NINT(nctype:(L))!=noref:)#         Constrained                      AX32 320
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX32 321
NY=NINT(nctype:(L))#                   Address of constraint            AX32 322
WHILE (caddr:(NY)>small:)#             Loop over constraint             AX32 323
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX32 324
REF=NINT(caddr:(NY))+IMAG*M*STEPP#     Ref param address                AX32 325
QX(REF)=QX(REF)+cfact:(NY)*DA(1)#      Update derivatives               AX32 326
QX(REF+STEPP)=QX(REF+STEPP)+cfact:(NY)*DB(1)#                           AX32 327
IF (IFRI==2)#                          Friedel pair                     AX32 328
$(#                                    8>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX32 329
REF=NINT(caddr:(LY))+IMAG*MM*STEPP#    Friedel ref param address        AX32 330
QX(REF)=QX(REF)+cfact:(NY)*DA(2)#                                       AX32 331
QX(REF+STEPP)=QX(REF+STEPP)+cfact:(NY)*DB(2)#                           AX32 332
$)#                                    8<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX32 333
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX32 334
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX32 335
incrx:(L)#                             Ready for im                     AX32 336
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX32 337
ELSE#                                  Not subject                      AX32 338
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX32 339
L16=MARK1(6)+((L6-MARK(6))/STEP(6))*2+I-1-(L-NX)+IMAG*M*STEPP#          AX32 340
QX(L16)=QX(L16)+DA(1)#                 Deriv of a w.r. to f'            AX32 341
QX(L16+STEPP)=QX(L16+STEPP)+DB(1)#     Deriv of b w.r. to f'            AX32 342
IF (IFRI==2)#                          Friedel pair                     AX32 343
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX32 344
L16=MARK1(6)+((L6-MARK(6))/STEP(6))*2+I-1-(L-NX)+IMAG*MM*STEPP#         AX32 345
QX(L16)=QX(L16)+DA(2)#                                                  AX32 346
QX(L16+STEPP)=QX(L16+STEPP)+DB(2)#                                      AX32 347
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX32 348
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX32 349
DA(1)=-FMUL*SUMSIN;DA(2)=-DA(1)#                                        AX32 350
DB(1)=FMUL*SUMCOS;DB(2)=DB(1)#                                          AX32 351
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX32 352
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX32 353
#                                      ***** deriv of atomic params     AX32 354
DO I=1,MXP#                            Loop over all parameters         AX32 355
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX32 356
IF(I <10)#                             Not pop nor neu param            AX32 357
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX32 358
IF ((I>3)&(TFT==0))NEXT#               No displ params                  AX32 359
IF ((I>4)&(TFT==1))NEXT#               No ani displ params              AX32 360
IF((I==4)&(TFT==1))#                   Iso temp fact                    AX32 361
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX32 362
DA(1)=-AA(1)*STL2;DB(1)=-BB(1)*STL2#   Imag of u iso                    AX32 363
DA(2)=-AA(2)*STL2;DB(2)=-BB(2)*STL2#                                    AX32 364
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX32 365
ELSE#                                  Xyz or aniso temp fact           AX32 366
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX32 367
DA(1)=FMUL*(FAC1*DELA(I)-FAC2*DELB(I))# Real of pos                     AX32 368
DB(1)=FMUL*(FAC1*DELB(I)+FAC2*DELA(I))# Imag of pos                     AX32 369
DA(2)=FMUL*(FAC1*DELA(I)+FAC2*DELB(I))#                                 AX32 370
DB(2)=FMUL*(-FAC1*DELB(I)+FAC2*DELA(I))#                                AX32 371
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX32 372
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX32 373
ELSE#                                  Pop or neu param                 AX32 374
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX32 375
DA(1)=AA(1)/atompop:(L7);DB(1)=BB(1)/atompop:(L7)# Imag of pop          AX32 376
DA(2)=AA(2)/atompop:(L7);DB(2)=BB(2)/atompop:(L7)#                      AX32 377
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX32 378
#                                     ***** WE HAVE DERIVATIVE IN DA &  AX32 379
ifnc:(LX,L7+I)#                        Test if param n/c/b              AX32 380
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX32 381
IF(NINT(nctype:(LX))!=noref:)#         Param constrained                AX32 382
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 37 AX32 383
LY=NINT(nctype:(LX))#                  Get constraint address           AX32 384
WHILE(NINT(caddr:(LY))!=0)#            Loop over ref params             AX32 385
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 38 AX32 386
REF=NINT(caddr:(LY))+IMAG*M*STEPP#                                      AX32 387
QX(REF)=QX(REF)+DA(1)*cfact:(LY)#                                       AX32 388
IF (IMAG==2)QX(REF+STEPP)=QX(REF+STEPP)+DB(1)*cfact:(LY)#               AX32 389
IF (IFRI==2)#                                                           AX32 390
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 39 AX32 391
REF=NINT(caddr:(LY))+IMAG*MM*STEPP#                                     AX32 392
QX(REF)=QX(REF)+DA(2)*cfact:(LY)#                                       AX32 393
QX(REF+STEPP)=QX(REF+STEPP)+DB(2)*cfact:(LY)#                           AX32 394
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 39 AX32 395
incry:(LY)#                            Point to next ref var            AX32 396
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 38 AX32 397
incry:(LY)#                            Point to next constraint         AX32 398
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 37 AX32 399
incrx:(LX)#                            Next n/c                         AX32 400
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX32 401
ELSE#                                  Parameter is variable            AX32 402
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 40 AX32 403
QX(L20+IMAG*M*STEPP)=QX(L20+IMAG*M*STEPP)+DA(1)#                        AX32 404
IF (IMAG==2)$#                         Imaginary part                   AX32 405
QX(L20+(IMAG*M+1)*STEPP)=QX(L20+(IMAG*M+1)*STEPP)+DB(1)#                AX32 406
IF (IFRI==2)#                          Friedel pair                     AX32 407
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 41 AX32 408
QX(L20+IMAG*MM*STEPP)=QX(L20+IMAG*MM*STEPP)+DA(2)#                      AX32 409
QX(L20+(IMAG*MM+1)*STEPP)=QX(L20+(IMAG*MM+1)*STEPP)+DB(2)#              AX32 410
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 41 AX32 411
L20=L20+1#                             Next variable                    AX32 412
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 40 AX32 413
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX32 414
#                                      ***** end of derivative loop     AX32 415
100CONTINUE#                                                            AX32 416
AC(M+1)=AC(M+1)+AA(1);BC(M+1)=BC(M+1)+BB(1)# Sum total sf components    AX32 417
IF (IMAG==1)BC(M+1)=0.0#               Clear b component                AX32 418
IF (IFRI==2)#                          Friedel pair                     AX32 419
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 42 AX32 420
AC(MM+1)=AC(MM+1)+AA(2);BC(MM+1)=BC(MM+1)+BB(2)#                        AX32 421
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 AX32 422
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX32 423
#                                      ***** end of atom loop           AX32 424
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX32 425
#                                      ***** end of twin loop           AX32 426
FF=0.0#                                Clear ff for twinning            AX32 427
IF (REFINE(2)>0)#                      Twinning                         AX32 428
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 43 AX32 429
DO M=1,NB(2);FF=FF+entpol:(M)*(AC(M)*AC(M)+BC(M)*BC(M))#                AX32 430
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 43 AX32 431
ELSE         FF=AC(1)*AC(1)+BC(1)*BC(1)#                                AX32 432
ifpr:(9)T=QX(IP+IREL(9))#              Get psi for ani ext              AX32 433
IF (REFINE(4)>0)$#                     Compute extinction               AX32 434
CALL AX39(HS,FF,STOL,T,sclfac:(L3),FAC,SQRW)#                           AX32 435
CALC=2.0*FAC*sclfac:(L3)*DDPAR*SQRW#   Fudge fact for derivatives       AX32 436
IF (IREF==no:)GOTO 200#                Skip derivatives                 AX32 437
IF (REFINE(2)==0)#                     No twinning                      AX32 438
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 44 AX32 439
T=CALC*AC(1);S=CALC*BC(1)#                                              AX32 440
IF (IMAG==1)DO L20=MARK1(5),STOP20;QX(L20)=T*QX(L20)#                   AX32 441
ELSE        DO L20=MARK1(5),STOP20#    Imaginary present                AX32 442
QX(L20)=T*QX(L20)+S*QX(L20+STEPP)#                                      AX32 443
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 44 AX32 444
ELSE#                                  Twinning                         AX32 445
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 45 AX32 446
IF (IMAG==1)#                          No b                             AX32 447
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 46 AX32 448
T=entpol:(1)*CALC*AC(1)#                                                AX32 449
DO L20=MARK1(5),STOP20;QX(L20)=T*QX(L20)#                               AX32 450
DO M=1,NB(2)-1#                        Loop over following merohedrie   AX32 451
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 47 AX32 452
T=entpol:(M+1)*CALC*AC(M+1)#                                            AX32 453
DO L20=MARK1(5),STOP20#                                                 AX32 454
QX(L20)=QX(L20)+T*QX(L20+M*STEPP)#                                      AX32 455
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 47 AX32 456
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 46 AX32 457
ELSE#                                  B not 0                          AX32 458
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 48 AX32 459
T=entpol:(1)*CALC*AC(1);S=entpol:(1)*CALC*BC(1)#                        AX32 460
DO L20=MARK1(5),STOP20;QX(L20)=T*QX(L20)+S*QX(L20+STEPP)#               AX32 461
DO M=1,NB(2)-1#                        Loop over following merohedrie   AX32 462
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 49 AX32 463
T=entpol:(M+1)*CALC*AC(M+1);S=entpol:(M+1)*CALC*BC(M+1)#                AX32 464
DO L20=MARK1(5),STOP20#                                                 AX32 465
QX(L20)=QX(L20)+T*QX(L20+2*M*STEPP)+S*QX(L20+(2*M+1)*STEPP)#            AX32 466
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 49 AX32 467
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 48 AX32 468
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 45 AX32 469
200CONTINUE#                                                            AX32 470
CALC=FAC*FF*sclfac:(L3)#               ***** calculated intensity       AX32 471
FF=SQRT(AMAX1(0.0,FF))#                Form fcal corrected disp/ext     AX32 472
#                                                                       AX32 473
#    SUM STATISTICAL QUANTITIES                                         AX32 474
#    --------------------------                                         AX32 475
#                                                                       AX32 476
IF (SQRW*ABS(OBS-CALC)>REJECT)CODE=8#  Set to rejected                  AX32 477
DIF=OBS-CALC#                          Form difference                  AX32 478
IF ((CODE==2)\(CODE==6))#              Less-thans                       AX32 479
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 50 AX32 480
BUFIN(26)=BUFIN(26)+1.0#               Count less-than                  AX32 481
BUFIN(14)=BUFIN(14)+ABS(FO)/2.0#       As in crylsq                     AX32 482
IF (FF>FO)#                            Calc > obs                       AX32 483
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 51 AX32 484
BUFIN(27)=BUFIN(27)+1.0#               Count less-than greater          AX32 485
BUFIN(13)=BUFIN(13)+ABS(FO-FF)#                                         AX32 486
IF (CODE==2)#                          Only status = 2 can build matr   AX32 487
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 52 AX32 488
BUFIN(15)=BUFIN(15)+WEIGHT*DIF**2;EDW=EDW+WEIGHT*SIGMA**2#              AX32 489
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 52 AX32 490
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 51 AX32 491
ELSE IF (CODE==2)CODE=-2#                                               AX32 492
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 50 AX32 493
IF ((CODE==1)\(CODE==4)\(CODE==5))#    Observed                         AX32 494
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 53 AX32 495
BUFIN(4)=BUFIN(4)+ABS(FO-FF)#          Sum |delta f|                    AX32 496
BUFIN(5)=BUFIN(5)+ABS(FO)#             Sum |f obs|                      AX32 497
BUFIN(7)=BUFIN(7)+(WFF*(FO-FF))**2#    Sum (w delta f) squared          AX32 498
BUFIN(8)=BUFIN(8)+(WFF*FO)**2#         Sum (w fobs) squared             AX32 499
BUFIN(10)=BUFIN(10)+ABS(DIF)#          Sum |delta y|                    AX32 500
BUFIN(11)=BUFIN(11)+ABS(OBS)#          Sum |y obs|                      AX32 501
ifnv:(BUFIN(20))BUFIN(21)=BUFIN(21)+((FO-FF)-BUFIN(20))**2# Durbin-wa   AX32 502
BUFIN(20)=FO-FF;#                                                       AX32 503
BUFIN(22)=BUFIN(22)+BUFIN(20)**2#      Ax32 488                         AX32 504
BUFIN(15)=BUFIN(15)+WEIGHT*DIF**2;#                                     AX32 505
EDW=EDW+WEIGHT*SIGMA**2#                                                AX32 506
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 53 AX32 507
IF ((NINT(BUFIN(1))<=IABS(LIST))\(SQRW*ABS(DIF)>RFLIST))# Test only f   AX32 508
CALL AX38(HS,STOL,ABSO,LORZ,AC,BC,CALC,WEIGHT,FO,WFF,CODE,ICYCLE)#      AX32 509
IF ((INTRES==no:)\(CODE==3)\(CODE>=5)\(CODE<0))NEXT#                    AX32 510
IF(PPRIOR>3)#                          Dump structure factors           AX32 511
WRITE(6,'(3F6.0,6f10.5)') HS,STOL,YEXT,TBAR,FO,FF,FO-FF##               AX32 512
#                                                                       AX32 513
#   FORM DERIVATIVES                                                    AX32 514
#   ----------------                                                    AX32 515
#                                                                       AX32 516
BUFIN(2)=BUFIN(2)+1.0#                 Increment reflections in matri   AX32 517
LX=MARKX;LY=MARKY;L20=MARK20;I=0#      Reset pointers                   AX32 518
IF (REFINE(1)>0)#                      Refine shape                     AX32 519
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 54 AX32 520
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over faces                  AX32 521
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 55 AX32 522
I=I+1#                                 Increment face counter           AX32 523
DA(1)=QX(IP+IREL(15+I))*SQRW*CALC*ABSO# Correct intensity               AX32 524
ifnc:(LX,L1)#                          Not constrained                  AX32 525
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 56 AX32 526
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX32 527
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 57 AX32 528
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX32 529
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 58 AX32 530
REF=NINT(caddr:(LY))#                  Get reference param address      AX32 531
QX(REF)=QX(REF)+cfact:(LY)*DA(1)#      Update ref par der               AX32 532
incry:(LY)#                            Next packet                      AX32 533
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 58 AX32 534
incry:(LY)#                            Next constraint                  AX32 535
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 57 AX32 536
incrx:(LX)#                            Next nc                          AX32 537
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 56 AX32 538
ELSE#                                  Not refined or constrained       AX32 539
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 59 AX32 540
QX(L20)=QX(L20)+DA(1);L20=L20+1#       Store der & take next var        AX32 541
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 59 AX32 542
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 55 AX32 543
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 54 AX32 544
IF (REFINE(2)==2)#                     Refine ent-pol                   AX32 545
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 60 AX32 546
DO M=1,NB(2)#                          Loop over merohedries            AX32 547
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 61 AX32 548
DA(1)=FAC*sclfac:(L3)*(AC(M)*AC(M)+BC(M)*BC(M))*SQRW# Get derivative    AX32 549
ifnc:(LX,MARK(2)+M-1)#                 Param is noref/constr            AX32 550
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 62 AX32 551
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX32 552
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 63 AX32 553
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX32 554
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 64 AX32 555
REF=NINT(caddr:(LY))#                  Get ref param address            AX32 556
QX(REF)=QX(REF)+cfact:(LY)*DA(1)#      Update ref param der             AX32 557
incry:(LY)#                            Next packet                      AX32 558
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 64 AX32 559
incry:(LY)#                            Next constraint                  AX32 560
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 63 AX32 561
incrx:(LX)#                            Next nc                          AX32 562
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 62 AX32 563
ELSE#                                  Not refined or constrained       AX32 564
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 65 AX32 565
QX(L20)=QX(L20)+DA(1);L20=L20+1#       Store deriv & take next var      AX32 566
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 65 AX32 567
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 61 AX32 568
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 60 AX32 569
DA(1)=SQRW*CALC/sclfac:(L3)#           Get scale derivative             AX32 570
L20=MARK1(3);findnc:(LX,LX,MARK(3))#   Find nc address                  AX32 571
DO M3=MARK(3),L3-STEP(3),STEP(3)#      Loop over prev scales            AX32 572
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 66 AX32 573
ifnc:(LX,M3)incrx:(LX)#                Update constr pointer            AX32 574
ELSE L20=L20+1#                        Update var pointer               AX32 575
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 66 AX32 576
ifnc:(LX,L3)#                          Param is variable                AX32 577
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 67 AX32 578
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX32 579
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 68 AX32 580
LY=NINT(nctype:(LX))#                  Get constraint address           AX32 581
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX32 582
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 69 AX32 583
REF=NINT(caddr:(LY))#                  Get ref param address            AX32 584
QX(REF)=QX(REF)+cfact:(LY)*DA(1)#      Update ref param de              AX32 585
incry:(LY)#                            Take next packet                 AX32 586
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 69 AX32 587
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 68 AX32 588
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 67 AX32 589
ELSE#                                  Not refined or constrained       AX32 590
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 70 AX32 591
QX(L20)=QX(L20)+DA(1);L20=L20+1#       Store der & take next var        AX32 592
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 70 AX32 593
#                                                                       AX32 594
#    BUILD DERIVATIVE MATRICES                                          AX32 595
#    -------------------------                                          AX32 596
#                                                                       AX32 597
L21=MARK21;L22=MARK22;STEPI=VARTOT#    Matrix pointers                  AX32 598
IF ((NCYCLE==ICYCLE)&(IWT!=1))#                                         AX32 599
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 71 AX32 600
L23=MARK23#                                                             AX32 601
DO L20=MARK20,STOP20#                  Loop over variables              AX32 602
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 72 AX32 603
IF (QX(L20)!=0.0)#                     Perform summation                AX32 604
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 73 AX32 605
DO L=L20,STOP20#                                                        AX32 606
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 74 AX32 607
QX(L22)=QX(L22)+QX(L)*QX(L20)#         Increase matrix                  AX32 608
QX(L23)=QX(L23)+WEIGHT*SIGMA**2*QX(L)*QX(L20)# Sum other matrix         AX32 609
L22=L22+1;L23=L23+1#                   Increment pointers               AX32 610
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 74 AX32 611
QX(L21)=QX(L21)+QX(L20)*SQRW*DIF#      Form vector                      AX32 612
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 73 AX32 613
ELSE#                                  Go to next row                   AX32 614
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 75 AX32 615
L22=L22+STEPI;L23=L23+STEPI#           Take next diag element           AX32 616
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 75 AX32 617
L21=L21+1;STEPI=STEPI-1#               Increment vector ptr             AX32 618
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 72 AX32 619
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 71 AX32 620
ELSE DO L20=MARK20,STOP20#             Loop over variables              AX32 621
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 76 AX32 622
IF (QX(L20)!=0.0)#                     Perform summation                AX32 623
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 77 AX32 624
DO L=L20,STOP20#                                                        AX32 625
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 78 AX32 626
QX(L22)=QX(L22)+QX(L)*QX(L20)#         Increase matrix                  AX32 627
L22=L22+1#                             Increment pointer                AX32 628
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 78 AX32 629
QX(L21)=QX(L21)+QX(L20)*SQRW*DIF#      Form vector                      AX32 630
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 77 AX32 631
ELSE L22=L22+STEPI#                    Take next disg element           AX32 632
L21=L21+1;STEPI=STEPI-1#               Increment vector ptr             AX32 633
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 76 AX32 634
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX32 635
RETURN#                                                                 AX32 636
END#                                                                    AX32 637
#                                                                       AX32 638
systemheader:(AX33)#                   Compute restraints derivatives   AX33   1
####################################################################    AX33   2
#                                                                  #    AX33   3
#                               AX33                               #    AX33   4
#                                                                  #    AX33   5
####################################################################    AX33   6
SUBROUTINE AX33(ICYCLE)#                                                AX33   7
include:(AACOMN)#                                                       AX33   8
include:(AXCOMN)#                                                       AX33   9
#                                                                       AX33  10
INTEGER I,J#                           Pointers in deriv lists          AX33  11
INTEGER ICYCLE#                                                         AX33  12
INTEGER IJ#                            Off-diagonal matrix pointer      AX33  13
INTEGER L1,M1#                         Face list pointers               AX33  14
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX33  15
INTEGER L20,L21,L22,L23#               Matrix & vector pointers         AX33  16
INTEGER M20#                           Used as 2nd der vector pointer   AX33  17
INTEGER REF#                           Address of reference parameter   AX33  18
INTEGER STEPI#                         Go to next row                   AX33  19
INTEGER VARBLK#                        Nb of variables refined          AX33  20
REAL W1,SQRW1,W2,SQRW2#                Weight & square root             AX33  21
REAL X#                                Temp var                         AX33  22
#                                                                       AX33  23
VARBLK=STOP20-MARK20+1#                Var nb                           AX33  24
L20=MARK20;LX=MARKX;LY=MARKY#          Set constarint list pointers     AX33  25
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over obs k                  AX33  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX33  27
IF ((NINT(ncaddr:(LX))==L1)&(NINT(nctype:(LX))==noref:))# Not refined   AX33  28
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX33  29
incrx:(LX);NEXT#                       Skip this observation            AX33  30
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX33  31
IF (IWT==0)W1=1.0#                     Unit weights                     AX33  32
IF (IWT==1)W1=1.0/faceso:(L1)**2#      1/sigma**2                       AX33  33
IF (IWT==2)W1=facew:(L1)#              Bdf weights                      AX33  34
SQRW1=SQRT(W1)#                                                         AX33  35
BUFIN(30)=BUFIN(30)+W1*(facedo:(L1)-facedc:(L1))**2#                    AX33  36
BUFIN(31)=BUFIN(31)+W1*facedo:(L1)**2#                                  AX33  37
BUFIN(17)=BUFIN(17)+(facedo:(L1)-facedc:(L1))**2*W1#                    AX33  38
EDW=EDW+W1*faceso:(L1)**2#                                              AX33  39
#                                                                       AX33  40
# CONSTRUCT D(DK)/D(DI)                                                 AX33  41
# ---------------------                                                 AX33  42
#                                                                       AX33  43
movereal:(0.0,1,QX,MARK20,MARK1(2)-MARK20,2)# Clear face deriv          AX33  44
ifnc:(LX,L1)#                          Obs not trivial function         AX33  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX33  46
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX33  47
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX33  48
REF=NINT(caddr:(LY))#                  Get reference address            AX33  49
QX(REF)=cfact:(LY)#                    Derivative of linear function    AX33  50
incry:(LY)#                                                             AX33  51
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX33  52
incry:(LY);incrx:(LX)#                 Next items                       AX33  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX33  54
ELSE#                                  Variable                         AX33  55
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX33  56
QX(L20)=1.0;L20=L20+1#                                                  AX33  57
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX33  58
#                                                                       AX33  59
# ADD TO NORMAL MATRIX                                                  AX33  60
# --------------------                                                  AX33  61
#                                                                       AX33  62
L21=MARK21;L22=MARK22;STEPI=VARBLK#    Get pointers                     AX33  63
DO I=MARK20,MARK1(2)-1#                Loop over i                      AX33  64
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX33  65
QX(L22)=QX(L22)+W1*QX(I)**2#           Add to dial el                   AX33  66
QX(L21)=QX(L21)+W1*(facedo:(L1)-facedc:(L1))*QX(I)#                     AX33  67
IJ=L22#                                                                 AX33  68
L21=L21+1;L22=L22+STEPI;STEPI=STEPI-1# Next diag element                AX33  69
IF (ABS(QX(I))==0.0)NEXT#              Skip mult by 0                   AX33  70
DO J=I+1,MARK1(2)-1#                   Loop over j                      AX33  71
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX33  72
IJ=IJ+1#                               Next element                     AX33  73
QX(IJ)=QX(IJ)+W1*QX(I)*QX(J)#          Sum off-diag elements            AX33  74
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX33  75
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX33  76
#                                                                       AX33  77
# CONSTRUCT D(DK')/D(DJ)                                                AX33  78
# ----------------------                                                AX33  79
#                                                                       AX33  80
IF (((IWT!=1)\(ABS(CORDIS)>small:))&(ICYCLE==NCYCLE))# Variance-covar   AX33  81
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX33  82
MX=MARKX;MY=MARKY;M20=1#                                                AX33  83
DO M1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over k'                     AX33  84
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX33  85
movereal:(0.0,1,BUFOT,1,MARK1(2)-MARK1(1),2)# Clear deriv               AX33  86
ifnc:(MX,M1)#                          Obs not trivial function         AX33  87
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX33  88
IF (NINT(nctype:(MX))==noref:)#                                         AX33  89
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX33  90
incrx:(MX);NEXT#                       Skip constant                    AX33  91
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX33  92
ELSE#                                  Constraint                       AX33  93
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX33  94
WHILE (caddr:(MY)>small:)#             Loop over constraint             AX33  95
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX33  96
REF=NINT(caddr:(MY))-MARK20+1#         Get reference address            AX33  97
BUFOT(REF)=cfact:(MY)#                 Derivative of linear function    AX33  98
incry:(MY)#                                                             AX33  99
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX33 100
incry:(MY);incrx:(MX)#                 Next items                       AX33 101
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX33 102
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX33 103
ELSE#                                  Variable                         AX33 104
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX33 105
BUFOT(M20)=1.0;M20=M20+1#                                               AX33 106
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX33 107
IF (IWT==0)W2=1.0#                     Unit weights                     AX33 108
IF (IWT==1)W2=1.0/faceso:(M1)**2#      1/sigma**2                       AX33 109
IF (IWT==2)W2=facew:(M1)#              Weights from bdf                 AX33 110
SQRW2=SQRT(W2)#                                                         AX33 111
#                                                                       AX33 112
# ADD TO VARIANCE-COVARIANCE MATRIX                                     AX33 113
# ---------------------------------                                     AX33 114
#                                                                       AX33 115
L23=MARK23;STEPI=VARBLK#               Get matrix pointer               AX33 116
IF (L1==M1)X=(W1*faceso:(L1))**2#                                       AX33 117
ELSE X=W1*W2*faceso:(L1)*faceso:(M1)*CORDIS#                            AX33 118
IF (X==0.0)NEXT#                       Skip mult by 0                   AX33 119
DO I=MARK20,MARK1(2)-1#                                                 AX33 120
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX33 121
IF (L1==M1)QX(L23)=QX(L23)+X*QX(I)**2# Add to diag el                   AX33 122
IJ=L23#                                                                 AX33 123
L23=L23+STEPI;STEPI=STEPI-1#           Next diag element                AX33 124
IF (ABS(QX(I))==0.0)NEXT#              Skip mult by 0                   AX33 125
DO J=I-MARK20+2,MARK1(2)-MARK20#       Loop over derive                 AX33 126
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX33 127
IJ=IJ+1#                               Next element                     AX33 128
QX(IJ)=QX(IJ)+X*QX(I)*BUFOT(J)#                                         AX33 129
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX33 130
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX33 131
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX33 132
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX33 133
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX33 134
RETURN#                                                                 AX33 135
END#                                                                    AX33 136
#                                                                       AX33 137
systemheader:(AX34)#                   Compute restraints derivatives   AX34   1
####################################################################    AX34   2
#                                                                  #    AX34   3
#                               AX34                               #    AX34   4
#                                                                  #    AX34   5
####################################################################    AX34   6
SUBROUTINE AX34(ICYCLE)#                                                AX34   7
include:(AACOMN)#                                                       AX34   8
include:(AXCOMN)#                                                       AX34   9
#                                                                       AX34  10
REAL CALC,DIF,C0,C1,C2#                Calculated value of the restra   AX34  11
REAL DER(3),DP(3)#                     Restraint derivatives            AX34  12
INTEGER ICYCLE#                        Current cycle nb                 AX34  13
INTEGER I,J,K,L#                       Loop indices                     AX34  14
INTEGER L7#                            Atom pointer                     AX34  15
INTEGER LR#                            Restraint pointer                AX34  16
INTEGER LX,MX,LY#                      Constraint list pointers         AX34  17
INTEGER L20,L21,L22,L23#               Matrix & vector pointers         AX34  18
INTEGER MXP#                           Local number of atomic paramet   AX34  19
INTEGER M7,M22,M23#                    Auxiliary pointers               AX34  20
INTEGER NO#                            Restraint counter                AX34  21
INTEGER REF#                           Address of reference parameter   AX34  22
INTEGER STEPI#                         Row length                       AX34  23
INTEGER TFT#                           Temp fact type                   AX34  24
INTEGER TYPE#                          Type of restraint                AX34  25
REAL EPS(3,3,3)#                       Totally antisymmetric tensor     AX34  26
REAL WEIGHT,SQRW#                      Restraint weight & square root   AX34  27
REAL X(4,3)#                           Coordinates of atoms             AX34  28
REAL Y(3,4)#                           Distance between two atoms       AX34  29
REAL Z(2,4)#                           Distances 1-3 and 2-4            AX34  30
REAL N(2,4)#                           Cross product between distance   AX34  31
#                                                                       AX34  32
intdata:(NPAR,[2, 3, 4])#                                               AX34  33
chardata:(AX341,[" WARNING- Atoms XXXXXXXXXXXXXXXXXXXXXXXX and $#       AX34  34
XXXXXXXXXXXXXXXXXXXXXXXX occupy the same place, restraint ignored"])#   AX34  35
chardata:(AX342,[" WARNING- Atoms XXXXXXXXXXXXXXXXXXXXXXXX, $#          AX34  36
XXXXXXXXXXXXXXXXXXXXXXXX and XXXXXXXXXXXXXXXXXXXXXXXX are $#            AX34  37
aligned, restraint ignored"])#                                          AX34  38
chardata:(AX340,["Dist Bond Dihe Rigid"])#                              AX34  39
chardata:(AX343,[" Type    Atoms                               $#       AX34  40
                 Obs      Calc     w dif    E.s.d."])#                  AX34  41
realdata:(FMT,[650842., 750842., 850842., 950842.])#                    AX34  42
datastuff:#                                                             AX34  43
#                                                                       AX34  44
DO I=1,3;DO J=1,3;DO K=1,3;EPS(I,J,K)=0.0# Clear eps                    AX34  45
EPS(1,2,3)=1.0;EPS(2,3,1)=1.0;EPS(3,1,2)=1.0# Pos permut                AX34  46
EPS(1,3,2)=-1.0;EPS(3,2,1)=-1.0;EPS(2,1,3)=-1.0# Neg permut             AX34  47
L22=MARK22+((MARK1(7)-MARK20)*(2*VARTOT-MARK1(7)+MARK20+1))/2# 1st at   AX34  48
L23=MARK23+L22-MARK22#                 Idem for list23                  AX34  49
DO TYPE=1,3#                           Loop over restraint type         AX34  50
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX34  51
NO=0#                                  Clear counter                    AX34  52
FOR (LR=MARKR(TYPE);LR<STOPR(TYPE);incrr:(LR,NPAR(TYPE)))# Loop over    AX34  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX34  54
NO=NO+1#                               Increment restraint counter      AX34  55
#                                                                       AX34  56
# CALCULATE RESTRAINT VALUE AND GET SIGNIFICANT VECTORS                 AX34  57
# -----------------------------------------------------                 AX34  58
#                                                                       AX34  59
DO I=1,NPAR(TYPE)#                     Loop over restraint packets      AX34  60
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX34  61
L7=NINT(resp:(LR,I))#                  Get atom ptr                     AX34  62
IF ((NO<=IABS(LISTRE))&((LISTRE<0)\(ICYCLE==NCYCLE)))# Is this restra   AX34  63
movertoc:(atomn:(L7),1,CHROT,10+(I-1)*10,8,0)# Atom name                AX34  64
DO J=1,3#                              Build postion from root atom     AX34  65
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX34  66
X(I,J)=resm:(LR,I,J+9)#                Translation (cell)               AX34  67
DO K=1,3;X(I,J)=X(I,J)+resm:(LR,I,J,K)*atomxyz:(L7+K)# Rot              AX34  68
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX34  69
IF (I==1)NEXT#                         Exit if first                    AX34  70
DO J=1,3;Y(I-1,J)=X(I,J)-X(I-1,J)#     Build difference vectors         AX34  71
Y(I-1,4)=0.0#                          Get norms                        AX34  72
DO J=1,3;DO K=1,3#                     Find norm                        AX34  73
Y(I-1,4)=Y(I-1,4)+Y(I-1,J)*MT(J,K)*Y(I-1,K)#                            AX34  74
IF (ABS(Y(I-1,4))<small:)BREAK#        Distance = 0                     AX34  75
IF (I==2)NEXT#                         Exit if 2nd packet               AX34  76
DO J=1,3#                              Cross product                    AX34  77
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX34  78
N(I-2,J)=0.0#                          Clear component                  AX34  79
DO K=1,3;DO L=1,3#                                                      AX34  80
N(I-2,J)=N(I-2,J)+EPS(J,K,L)*Y(I-2,K)*Y(I-1,L)#                         AX34  81
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX34  82
N(I-2,4)=0.0#                          Get norms                        AX34  83
DO J=1,3;DO K=1,3;$#                   Loop over coordinates            AX34  84
N(I-2,4)=N(I-2,4)+N(I-2,J)*MTINV(J,K)*N(I-2,K)#                         AX34  85
IF (N(I-2,4)==0.0)BREAK#               3 atoms are aligned              AX34  86
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX34  87
#                                                                       AX34  88
# TREAT SAME LOCATIONS AND ALIGNEMENT                                   AX34  89
# -----------------------------------                                   AX34  90
#                                                                       AX34  91
IF (I<=NPAR(TYPE))#                    Configuration error, ignore re   AX34  92
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX34  93
ifeq:(Y(I-1,4),0.0)#                   Distance = 0                     AX34  94
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX34  95
DO J=I-1,I#                            Loop over packets                AX34  96
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX34  97
L7=NINT(resp:(LR,J))#                  Extract atom pointer             AX34  98
IF (J==I-1)K=17#                       Column to put atom name          AX34  99
ELSE K=46#                                                              AX34 100
movertoc:(atomn:(L7),1,AX341,K,lena:,0)# Move atom name                 AX34 101
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX34 102
writeline:(0,AX341,NAX341,3,3)#        Print warning                    AX34 103
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX34 104
ELSE#                                  3 atoms aligned                  AX34 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX34 106
DO J=I-2,I#                            Loop over packets                AX34 107
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX34 108
L7=NINT(resp:(LR,J))#                  Extract atom pointer             AX34 109
IF (J==I-2)K=17#                       Column to put atom name          AX34 110
ELSE IF (J==I-1)K=43#                                                   AX34 111
ELSE K=72#                                                              AX34 112
movertoc:(atomn:(L7),1,AX342,K,lena:,0)# Move atom name                 AX34 113
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX34 114
writeline:(0,AX342,NAX342,3,3)#        Print warning                    AX34 115
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX34 116
NEXT#                                  Ignore restraint                 AX34 117
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX34 118
#                                                                       AX34 119
# COMPUTE RESTRAINTS VALUE & PREPARE DERIVATIVES                        AX34 120
# ----------------------------------------------                        AX34 121
#                                                                       AX34 122
IF (TYPE==dist:)#                      Distance                         AX34 123
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX34 124
CALC=SQRT(Y(1,4))#                     Get distance                     AX34 125
DO I=1,3#                              Loop over coordinates            AX34 126
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX34 127
X(1,I)=0.0#                            Clear value                      AX34 128
DO J=1,3;X(1,I)=X(1,I)-MT(I,J)*Y(1,J)# Derivative w.r. 1st              AX34 129
X(1,I)=X(1,I)/CALC#                                                     AX34 130
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX34 131
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX34 132
IF (TYPE==angl:)#                      Bond angle                       AX34 133
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX34 134
C1=0.0#                                Clear angle                      AX34 135
DO I=1,3#                              Loop over coordinates            AX34 136
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX34 137
X(1,I)=0.0;X(2,I)=0.0#                 Clear temp derivatives           AX34 138
DO J=1,3#                              Loop over coordinates            AX34 139
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX34 140
C1=C1+Y(1,I)*MT(I,J)*Y(2,J)#           Dot product                      AX34 141
X(1,I)=X(1,I)-MT(I,J)*Y(1,J)#          Gradient w.r. 1st                AX34 142
X(2,I)=X(2,I)+MT(I,J)*Y(2,J)#          Gradient w.r. 3rd                AX34 143
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX34 144
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX34 145
C0=SQRT(Y(1,4)*Y(2,4))#                Denominator                      AX34 146
CALC=-C1/C0#                           Cosine of angle                  AX34 147
ifeq:(ABS(CALC),1.0)NEXT#              Impossible to compute derivati   AX34 148
CALC=ACOS(CALC)/rad:#                  Angle in degrees                 AX34 149
C1=C1/C0;C2=1.0/(rad:*SQRT(1.0-C1**2))# Cos derivative                  AX34 150
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX34 151
IF (TYPE==tort:)#                      Torsion angle                    AX34 152
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX34 153
C1=0.0;C2=0.0#                         Clear cos & sin of angle         AX34 154
DO I=1,3#                              Dot products                     AX34 155
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX34 156
DO J=1,3;C1=C1+N(1,I)*MTINV(I,J)*N(2,J)# Dot between normals            AX34 157
C2=C2+Y(3,I)*N(1,I)#                   Volume (triple product)          AX34 158
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX34 159
C2=C2*SQRT(Y(2,4))/VOL#                Same denom as cos                AX34 160
CALC=ATAN2(C2,C1)/rad:#                Angle in degrees                 AX34 161
IF (CALC<0.0)CALC=CALC+360.0#          Always positive                  AX34 162
IF (ABS(C1)<ABS(C2))C0=REAL(yes:)#     Derivative via cos               AX34 163
ELSE C0=REAL(no:)#                     Derivative via sin               AX34 164
IF (NINT(C0)==no:)#                    Sine                             AX34 165
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX34 166
C0=SQRT(Y(2,4))/VOL#                   Temp usage                       AX34 167
DO I=1,3#                              Make m*y2 & y1^y3                AX34 168
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX34 169
Z(1,I)=0.0;Z(2,I)=0.0#                 Clear                            AX34 170
DO J=1,3#                                                               AX34 171
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX34 172
Z(1,I)=Z(1,I)+MT(I,J)*Y(2,J)#                                           AX34 173
DO K=1,3;Z(2,I)=Z(2,I)+EPS(I,J,K)*Y(1,J)*Y(3,K)#                        AX34 174
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX34 175
X(1,I)=-C0*N(2,I)#                                                      AX34 176
X(2,I)=-C2*Z(1,I)/Y(2,4)+(N(2,I)+Z(2,I))*C0#                            AX34 177
X(3,I)=C2*Z(1,I)/Y(2,4)-(N(1,I)+Z(2,I))*C0#                             AX34 178
X(4,I)=C0*N(1,I)#                                                       AX34 179
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX34 180
C0=REAL(no:)#                          Reset flag                       AX34 181
C1=C2/SQRT(N(1,4)*N(2,4));C2=1.0/(rad:*SQRT(1.0-C1**2))# Set mult       AX34 182
IF ((CALC>90.0)&(CALC<270.0))C2=-C2#   Reverse sign                     AX34 183
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX34 184
DO I=1,3#                              Make m**-1*n1 & m**-1*n2         AX34 185
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX34 186
Z(1,I)=0.0;Z(2,I)=0.0#                 Clear                            AX34 187
DO J=1,3#                                                               AX34 188
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX34 189
Z(1,I)=Z(1,I)+MTINV(I,J)*N(1,J)#                                        AX34 190
Z(2,I)=Z(2,I)+MTINV(I,J)*N(2,J)#                                        AX34 191
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX34 192
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX34 193
IF (NINT(C0)==yes:)#                   Cosine                           AX34 194
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX34 195
DO I=1,3#                                                               AX34 196
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX34 197
X(1,I)=0.0;X(2,I)=0.0;X(3,I)=0.0;X(4,I)=0.0# Clear                      AX34 198
DO J=1,3;DO K=1,3#                     Cross product                    AX34 199
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX34 200
X(1,I)=X(1,I)-EPS(I,J,K)*Y(2,J)*Z(2,K)#                                 AX34 201
X(2,I)=X(2,I)+EPS(I,J,K)*((Y(2,J)+Y(1,J))*Z(2,K)-$#                     AX34 202
Y(3,J)*Z(1,K))#                                                         AX34 203
X(3,I)=X(3,I)+EPS(I,J,K)*((Y(2,J)+Y(3,J))*Z(1,K)-$#                     AX34 204
Y(1,J)*Z(2,K))#                                                         AX34 205
X(4,I)=X(4,I)-EPS(I,J,K)*Y(2,J)*Z(1,K)#                                 AX34 206
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX34 207
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX34 208
C1=C1/SQRT(N(1,4)*N(2,4));C2=-1.0/(rad:*SQRT(1.0-C1**2))# Set mult      AX34 209
IF (CALC>180.0)C2=-C2#                 Reverse sign                     AX34 210
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX34 211
DO I=1,3#                              Divide by square of norm         AX34 212
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX34 213
N(1,I)=C1*Z(1,I)/N(1,4);N(2,I)=C1*Z(2,I)/N(2,4)#                        AX34 214
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX34 215
C0=1.0/SQRT(N(1,4)*N(2,4))#                                             AX34 216
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX34 217
IF (TYPE!=3)DIF=resobs:(LR)-CALC#      Form difference                  AX34 218
ELSE IF (ABS(resobs:(LR)-CALC)<=180.0)DIF=resobs:(LR)-CALC#             AX34 219
ELSE $#                                                                 AX34 220
DIF=resobs:(LR)-CALC+360.0*(CALC-resobs:(LR))/ABS(resobs:(LR)-CALC)#    AX34 221
IF (IWT==0)WEIGHT=1.0#                                                  AX34 222
IF (IWT==1)WEIGHT=1.0/resd:(LR)**2#    1/sigma**2                       AX34 223
IF (IWT==2)WEIGHT=resw:(LR)#           Weight from bdf                  AX34 224
SQRW=SQRT(WEIGHT)#                     Take square root                 AX34 225
IF ((NO<=IABS(LISTRE))&((LISTRE<0)\(ICYCLE==NCYCLE)))# Is this restra   AX34 226
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX34 227
IF (LR==MARKR(TYPE))#                  First of type                    AX34 228
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX34 229
writeline:(1,0,0,3,3)#                 Print one blank line             AX34 230
LINRM=3#                               Reserve 3 lines                  AX34 231
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX34 232
movechar:(AX340,5*(TYPE-1)+1,CHROT,2,5,0)# Move type                    AX34 233
ncodefld:(resobs:(LR),1,CHROT,FMT(1),1)# Encode obs                     AX34 234
ncodefld:(CALC,1,CHROT,FMT(2),1)#      Encode calc                      AX34 235
ncodefld:(SQRW*DIF,1,CHROT,FMT(3),1)#  Encode weighted dif              AX34 236
ncodefld:(resd:(LR),1,CHROT,FMT(4),1)# Encode esd                       AX34 237
writeline:(0,AX343,NAX343,1,3)#        Print line                       AX34 238
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX34 239
BUFIN(24)=BUFIN(24)+(ABS(DIF)*SQRW)**2#                                 AX34 240
BUFIN(25)=BUFIN(25)+(resobs:(LR)*SQRW)**2#                              AX34 241
BUFIN(16)=BUFIN(16)+DIF**2*WEIGHT#                                      AX34 242
#                                                                       AX34 243
# COMPUTE DERIVATIVES                                                   AX34 244
# -------------------                                                   AX34 245
#                                                                       AX34 246
movereal:(0.0,1,QX,MARK1(7),STOP20-MARK1(7)+1,2)# Clear vector          AX34 247
DO I=1,NPAR(TYPE)#                     Loop over restraint packets      AX34 248
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX34 249
L7=NINT(resp:(LR,I))#                  Get current atom ptrs            AX34 250
IF (TYPE==dist:)#                      Distance derivatives             AX34 251
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX34 252
IF (I==1)DO J=1,3;DP(J)=X(1,J)#                                         AX34 253
ELSE DO J=1,3;DP(J)=-X(1,J)#                                            AX34 254
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX34 255
IF (TYPE==angl:)#                      Bond angle                       AX34 256
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX34 257
IF (I==1)DO J=1,3;DP(J)=-X(2,J)/C0-C1*X(1,J)/Y(1,4)#                    AX34 258
IF (I==3)DO J=1,3;DP(J)=-X(1,J)/C0-C1*X(2,J)/Y(2,4)#                    AX34 259
IF (I==2)DO J=1,3;$#                                                    AX34 260
DP(J)=(X(1,J)+X(2,J))/C0+C1*(X(1,J)/Y(1,4)+X(2,J)/Y(2,4))#              AX34 261
DO J=1,3;DP(J)=DP(J)*C2#               Correct for angle                AX34 262
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX34 263
IF (TYPE==tort:)#                      Torsion angle                    AX34 264
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX34 265
DO J=1,3#                              Loop over coord                  AX34 266
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX34 267
DP(J)=X(I,J)*C0#                       Restore preparation              AX34 268
IF (I==1)DO K=1,3;DO L=1,3#            Cross product                    AX34 269
DP(J)=DP(J)+EPS(J,K,L)*Y(2,K)*N(1,L)#                                   AX34 270
IF (I==2)DO K=1,3;DO L=1,3#            Cross product                    AX34 271
DP(J)=DP(J)-EPS(J,K,L)*$#                                               AX34 272
((Y(2,K)+Y(1,K))*N(1,L)-Y(3,K)*N(2,L))#                                 AX34 273
IF (I==3)DO K=1,3;DO L=1,3#            Cross product                    AX34 274
DP(J)=DP(J)-EPS(J,K,L)*$#                                               AX34 275
((Y(2,K)+Y(3,K))*N(2,L)-Y(1,K)*N(1,L))#                                 AX34 276
IF (I==4)DO K=1,3;DO L=1,3#            Cross product                    AX34 277
DP(J)=DP(J)+EPS(J,K,L)*Y(2,K)*N(2,L)#                                   AX34 278
DP(J)=C2*DP(J)#                        Derivative of angle in degrees   AX34 279
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX34 280
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX34 281
DO J=1,3#                              Get derivatives of root atom     AX34 282
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX34 283
DER(J)=0.0#                            Clear der                        AX34 284
DO K=1,3;DER(J)=DER(J)+resm:(LR,I,K,J)*DP(K)# Rotation                  AX34 285
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX34 286
#                                                                       AX34 287
# LOOP TO FIND ATOM VARIABLE ADDRESS                                    AX34 288
# ----------------------------------                                    AX34 289
#                                                                       AX34 290
findnc:(MARKX,LX,MARK(7))#             Set ptrs to atomic nc            AX34 291
L20=MARK1(7)#                          Set to beg of atomic variables   AX34 292
DO M7=MARK(7),L7-STEP(7),STEP(7)#      Get l20 ptr & clear var list     AX34 293
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX34 294
TFT=NINT(atomtft:(M7))#                Get displ fact type              AX34 295
ifnc:(LX,M7)#                          Noref whole atom                 AX34 296
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX34 297
incrx:(LX)#                            Take next nc ptr                 AX34 298
NEXT#                                  Take next atom                   AX34 299
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX34 300
MXP=MAXPAR#                            Set to max nb of paramters       AX34 301
IF ((MXP<=9)&(TFT<2))#                                                  AX34 302
IF (TFT==0)MXP=3#                                                       AX34 303
ELSE MXP=4#                                                             AX34 304
DO J=1,MXP#                            Loop over params                 AX34 305
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 37 AX34 306
IF ((TFT==0)&(J>3)&(J<10))NEXT#        Overall                          AX34 307
IF ((TFT==1)&(J>4)&(J<10))NEXT#        Isotropic                        AX34 308
ifnc:(LX,M7+J)#                        Norefed or constrained           AX34 309
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 38 AX34 310
incrx:(LX)#                            Take next nc                     AX34 311
NEXT#                                  Take next param                  AX34 312
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 38 AX34 313
L20=L20+1#                             Take next variable               AX34 314
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 37 AX34 315
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX34 316
#                                                                       AX34 317
# STORE DERIVATIVES                                                     AX34 318
# -----------------                                                     AX34 319
#                                                                       AX34 320
ifnc:(LX,L7)#                          Atom not refined                 AX34 321
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 39 AX34 322
incrx:(LX);NEXT#                       Skip not refined atom            AX34 323
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 39 AX34 324
DO J=1,3#                              Loop over coordinates            AX34 325
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 40 AX34 326
ifnc:(LX,L7+J)#                        Param norefed or constrained     AX34 327
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 41 AX34 328
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX34 329
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 42 AX34 330
LY=NINT(nctype:(LX))#                  Get constraint address           AX34 331
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX34 332
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 43 AX34 333
REF=NINT(caddr:(LY))#                  Get reference param address      AX34 334
QX(REF)=QX(REF)+cfact:(LY)*DER(J)#     Update derivative                AX34 335
incry:(LY)#                            Take next packet                 AX34 336
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 43 AX34 337
incry:(LY)#                            Take next constraint             AX34 338
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 AX34 339
incrx:(LX)#                            Take next nc ptr                 AX34 340
NEXT#                                  Take next param                  AX34 341
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 41 AX34 342
QX(L20)=QX(L20)+DER(J)#                Sum derivative                   AX34 343
L20=L20+1#                             Take next variable               AX34 344
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 40 AX34 345
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX34 346
#                                                                       AX34 347
# BUILD MATRIX                                                          AX34 348
# ------------                                                          AX34 349
#                                                                       AX34 350
L21=MARK1(7)-MARK20+MARK21;M22=L22#    Set matrix pointers              AX34 351
STEPI=STOP20-MARK1(7)+1#               Row length                       AX34 352
IF ((IWT!=1)&(NCYCLE==ICYCLE))#        Build variance-covariance matr   AX34 353
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 44 AX34 354
M23=L23#                               Set covariance matrix pointer    AX34 355
DO L20=MARK1(7),STOP20#                Restraints only for atomic var   AX34 356
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 45 AX34 357
L=L20#                                 Point to diag el                 AX34 358
IF (QX(L20)!=0.0)#                     Not strictly equal 0             AX34 359
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 46 AX34 360
DO K=1,STEPI#                          Loop over one row                AX34 361
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 47 AX34 362
QX(M22)=QX(M22)+WEIGHT*QX(L20)*QX(L)#  Sum normal-eq matrix             AX34 363
QX(M23)=QX(M23)+(WEIGHT*resd:(LR))**2*QX(L20)*QX(L)# Var.               AX34 364
M22=M22+1;M23=M23+1;L=L+1#             Increment pointers               AX34 365
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 47 AX34 366
QX(L21)=QX(L21)+WEIGHT*QX(L20)*DIF#    Sum diff vector                  AX34 367
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 46 AX34 368
ELSE#                                  No sum                           AX34 369
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 48 AX34 370
M22=M22+STEPI;M23=M23+STEPI#           Next diag element, next row      AX34 371
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 48 AX34 372
L21=L21+1;STEPI=STEPI-1#               Increment pointer                AX34 373
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 45 AX34 374
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 44 AX34 375
ELSE#                                  Only normal matrix               AX34 376
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 49 AX34 377
DO L20=MARK1(7),STOP20#                Restraints only for atomic var   AX34 378
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 50 AX34 379
L=L20#                                 Point to diag el                 AX34 380
IF (QX(L20)!=0.0)#                     Not strictly equal 0             AX34 381
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 51 AX34 382
DO K=1,STEPI#                          Loop over one row                AX34 383
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 52 AX34 384
QX(M22)=QX(M22)+WEIGHT*QX(L20)*QX(L)#  Sum normal-eq matrix             AX34 385
M22=M22+1;L=L+1#                       Increment pointers               AX34 386
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 52 AX34 387
QX(L21)=QX(L21)+WEIGHT*QX(L20)*DIF#    Sum diff vector                  AX34 388
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 51 AX34 389
ELSE M22=M22+STEPI#                    Next diag element                AX34 390
L21=L21+1;STEPI=STEPI-1#               Increment pointer                AX34 391
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 50 AX34 392
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 49 AX34 393
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX34 394
BUFIN(18)=BUFIN(18)+REAL(NO)#          Update stat quantities           AX34 395
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX34 396
RETURN#                                                                 AX34 397
END#                                                                    AX34 398
#                                                                       AX34 399
systemheader:(AX35)#                   Compute restraints derivatives   AX35   1
####################################################################    AX35   2
#                                                                  #    AX35   3
#                               AX35                               #    AX35   4
#                                                                  #    AX35   5
####################################################################    AX35   6
SUBROUTINE AX35(ICYCLE)#                                                AX35   7
include:(AACOMN)#                                                       AX35   8
include:(AXCOMN)#                                                       AX35   9
#                                                                       AX35  10
REAL CALC#                             Calculated value of the restra   AX35  11
REAL DER(9),DP(9)#                     Restraint derivatives            AX35  12
INTEGER ICYCLE#                        Current cycle nb                 AX35  13
INTEGER I,J,K,L,M#                     Loop indices                     AX35  14
INTEGER L7#                            Atom pointer                     AX35  15
INTEGER LR#                            Restraint pointer                AX35  16
INTEGER LX,MX,LY#                      Constraint list pointers         AX35  17
INTEGER L20,L21,L22,L23#               Matrix & vector pointers         AX35  18
REAL MAT(3,3,3),U(2)#                  Symetry equivalents aniso tens   AX35  19
INTEGER MXP#                           Local number of atomic paramet   AX35  20
INTEGER M7,M22,M23#                    Auxiliary atom pointer           AX35  21
INTEGER NO#                            Restraint counter                AX35  22
INTEGER REF#                           Address of reference parameter   AX35  23
INTEGER STEPI#                         Row length                       AX35  24
INTEGER TFT(2)#                        Temp fact type                   AX35  25
REAL WEIGHT,SQRW#                      Weight & square root             AX35  26
REAL X(2,3)#                           Coordinates of atoms             AX35  27
REAL Y(4)#                             Distance between two atoms       AX35  28
#                                                                       AX35  29
intdata:(NPAR,[2])#                                                     AX35  30
chardata:(AX351,[" WARNING- Atoms XXXXXXXXXXXXXXXXXXXXXXXX and $#       AX35  31
XXXXXXXXXXXXXXXXXXXXXXXX occupy the same place, restraint ignored"])#   AX35  32
chardata:(AX352,["Rigid"])#                                             AX35  33
chardata:(AX353,[" Type    Atoms                               $#       AX35  34
                 Obs      Calc     w dif    E.s.d."])#                  AX35  35
realdata:(FMT,[650842., 750842., 850842., 950842.])#                    AX35  36
datastuff:#                                                             AX35  37
#                                                                       AX35  38
L22=MARK22+((MARK1(7)-MARK20)*(2*VARTOT-MARK1(7)+MARK20+1))/2# 1st at   AX35  39
L23=MARK23+L22-MARK22#                 Idem for list23                  AX35  40
NO=0#                                  Clear counter                    AX35  41
FOR (LR=MARKR(4);LR<STOPR(4);incrr:(LR,NPAR))# Loop over restraints     AX35  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX35  43
NO=NO+1#                               Increment restraint counter      AX35  44
#                                                                       AX35  45
# CALCULATE RESTRAINT VALUE AND GET SIGNIFICANT VECTORS                 AX35  46
# -----------------------------------------------------                 AX35  47
#                                                                       AX35  48
DO I=1,2#                              Loop over restraint packets      AX35  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX35  50
L7=NINT(resp:(LR,I))#                  Get atom ptr                     AX35  51
IF ((NO<=IABS(LISTRE))&((LISTRE<0)\(ICYCLE==NCYCLE)))# Is this restra   AX35  52
movertoc:(atomn:(L7),1,CHROT,10+(I-1)*10,8,0)# Atom name                AX35  53
TFT(I)=NINT(atomtft:(L7))#             Get temp fact                    AX35  54
DO J=1,3#                              Transform coordinates            AX35  55
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX35  56
X(I,J)=resm:(LR,I,J+9)#                Translation                      AX35  57
DO K=1,3;X(I,J)=X(I,J)+resm:(LR,I,J,K)*atomxyz:(L7+K)#                  AX35  58
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX35  59
IF (TFT(I)==2)#                        Anisotropic atom                 AX35  60
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX35  61
DO J=1,3;MAT(I,J,J)=atomdis:(L7+J)#    Diag elements                    AX35  62
MAT(I,1,2)=atomu12:(L7);MAT(I,2,1)=MAT(I,1,2)#                          AX35  63
MAT(I,1,3)=atomu13:(L7);MAT(I,3,1)=MAT(I,1,3)#                          AX35  64
MAT(I,2,3)=atomu23:(L7);MAT(I,3,2)=MAT(I,2,3)#                          AX35  65
DO J=1,3;DO K=1,3#                     Symmetry related atom            AX35  66
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX35  67
MAT(3,J,K)=0.0#                        Clear element                    AX35  68
DO L=1,3;DO M=1,3#                     Multiplication                   AX35  69
MAT(3,J,K)=MAT(3,J,K)+MAT(I,L,M)*$#                                     AX35  70
resm:(LR,I,J,L)*resm:(LR,I,K,M)#                                        AX35  71
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX35  72
DO J=1,3;DO K=1,3#                     Reciprocal of tensor             AX35  73
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX35  74
MAT(I,J,K)=0.0#                        Clear element                    AX35  75
DO L=1,3;DO M=1,3#                     Multiplication                   AX35  76
MAT(I,J,K)=MAT(I,J,K)+MAT(3,L,M)*MT(J,L)*MT(K,M)#                       AX35  77
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX35  78
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX35  79
ELSE#                                  Atom iso                         AX35  80
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX35  81
U(I)=atomiso:(L7)/utob:#               Local isotropic depl fact        AX35  82
DO J=1,3;DO K=1,3;MAT(I,J,K)=0.0#                                       AX35  83
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX35  84
IF (I==1)NEXT#                         Exit if first                    AX35  85
DO J=1,3;Y(J)=X(I,J)-X(I-1,J)#         Build difference vectors         AX35  86
Y(4)=0.0#                              Get norms                        AX35  87
DO J=1,3;DO K=1,3;Y(4)=Y(4)+Y(J)*MT(J,K)*Y(K)# Norm                     AX35  88
ifeq:(Y(4),0.0)#                       Distance = 0                     AX35  89
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX35  90
movertoc:(atomn:(L7),1,AX351,46,lena:,0)# Encode 2nd atom name          AX35  91
L7=NINT(resp:(LR,1))#                  Get 1st atom pointer             AX35  92
movertoc:(atomn:(L7),1,AX351,17,lena:,0)#                               AX35  93
writeline:(1,AX351,NAX351,3,3)#        Print warning                    AX35  94
BREAK#                                 Exit on error                    AX35  95
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX35  96
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX35  97
IF (I<=2)NEXT#                         Skip restraint if not finished   AX35  98
#                                                                       AX35  99
# RIGID BOND CALCULATION                                                AX35 100
# ----------------------                                                AX35 101
#                                                                       AX35 102
DO J=1,3;DO K=1,3;MAT(3,J,K)=MAT(2,J,K)-MAT(1,J,K)#                     AX35 103
DO I=1,2;IF (TFT(I)==2)#               Mean square depl of ani atom     AX35 104
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX35 105
U(I)=0.0#                              Clear                            AX35 106
DO J=1,3;DO K=1,3;U(I)=U(I)+Y(J)*MAT(I,J,K)*Y(K)#                       AX35 107
U(I)=4.0*U(I)/(utob:*Y(4))#                                             AX35 108
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX35 109
CALC=U(2)-U(1)#                        Form difference                  AX35 110
IF (IWT==0)WEIGHT=1.0#                 Unit weights                     AX35 111
IF (IWT==1)WEIGHT=1.0/resd:(LR)**2#    1/sigma**2                       AX35 112
IF (IWT==2)WEIGHT=resw:(LR)#           Weight from bdf                  AX35 113
SQRW=SQRT(WEIGHT)#                                                      AX35 114
IF ((NO<=IABS(LISTRE))&((LISTRE<0)\(ICYCLE==NCYCLE)))# Is this restra   AX35 115
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX35 116
IF (LR==MARKR(4))#                     First of type                    AX35 117
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX35 118
writeline:(1,0,0,3,3)#                 Print one blank line             AX35 119
LINRM=3#                               Reserve 3 lines                  AX35 120
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX35 121
movechar:(AX352,1,CHROT,2,NAX352,0)#   Move rigid bond                  AX35 122
ncodefld:(resobs:(LR),1,CHROT,FMT(1),1)# Encode obs                     AX35 123
ncodefld:(CALC,1,CHROT,FMT(2),1)#      Encode calc                      AX35 124
ncodefld:(SQRW*(resobs:(LR)-CALC),1,CHROT,FMT(3),1)# Encode weighted    AX35 125
ncodefld:(resd:(LR),1,CHROT,FMT(4),1)# Encode esd                       AX35 126
writeline:(0,AX353,NAX353,1,3)#        Print line                       AX35 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX35 128
BUFIN(24)=BUFIN(24)+(ABS(resobs:(LR)-CALC)*SQRW)**2#                    AX35 129
BUFIN(25)=BUFIN(25)+(resobs:(LR)*SQRW)**2#                              AX35 130
BUFIN(16)=BUFIN(16)+(resobs:(LR)-CALC)**2*WEIGHT#                       AX35 131
#                                                                       AX35 132
# RIGID BOND DERIVATIVE                                                 AX35 133
# ---------------------                                                 AX35 134
#                                                                       AX35 135
movereal:(0.0,1,QX,MARK1(7),STOP20-MARK1(7)+1,2)# Clear vector          AX35 136
DO J=1,3;DO K=1,3#                     Tensor to matrix                 AX35 137
MAT(3,J,K)=8.0*MAT(3,J,K)/utob:-2.0*CALC*MT(J,K)#                       AX35 138
DO J=1,3#                              Spatial partial derivatives &    AX35 139
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX35 140
DP(J)=0.0#                             Clear component                  AX35 141
DO K=1,3;DP(J)=DP(J)+MAT(3,J,K)*Y(K)#                                   AX35 142
X(1,J)=0.0#                            Clear component                  AX35 143
DO K=1,3;X(1,J)=X(1,J)+MT(J,K)*Y(K)#                                    AX35 144
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX35 145
DO J=1,3;DO K=1,3;MAT(3,J,K)=4.0*X(1,J)*X(1,K)/utob:# Build partial a   AX35 146
#                                                                       AX35 147
# COMPLETE DERIVATIVE                                                   AX35 148
# -------------------                                                   AX35 149
#                                                                       AX35 150
DO I=1,2#                              Loop over atoms                  AX35 151
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX35 152
DO J=1,3#                              Get derivatives of root atom     AX35 153
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX35 154
DER(J)=0.0#                            Clear der                        AX35 155
DO K=1,3;DER(J)=DER(J)+resm:(LR,I,K,J)*DP(K)# Rotation                  AX35 156
IF (I==1)DER(J)=-DER(J)/Y(4)#          Take care of sign                AX35 157
ELSE DER(J)=DER(J)/Y(4)#                                                AX35 158
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX35 159
IF (TFT(I)==2)#                        Anisotropic                      AX35 160
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX35 161
DO J=1,3;DO K=1,3#                                                      AX35 162
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX35 163
MAT(I,J,K)=0.0#                                                         AX35 164
DO L=1,3;DO M=1,3;MAT(I,J,K)=MAT(I,J,K)+$#                              AX35 165
resm:(LR,I,L,J)*resm:(LR,I,M,K)*MAT(3,L,M)#                             AX35 166
IF (I==1)MAT(I,J,K)=-MAT(I,J,K)/Y(4)#                                   AX35 167
ELSE MAT(I,J,K)=MAT(I,J,K)/Y(4)#                                        AX35 168
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX35 169
DO J=4,6;DER(J)=MAT(I,J-3,J-3)#        Transform to uij                 AX35 170
DER(7)=2.0*MAT(I,1,2)#                                                  AX35 171
DER(8)=2.0*MAT(I,1,3)#                                                  AX35 172
DER(9)=2.0*MAT(I,2,3)#                                                  AX35 173
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX35 174
ELSE IF (I==1)DER(4)=-1.0/utob:#       No coordinates tranformation     AX35 175
ELSE DER(4)=1.0/utob:#                                                  AX35 176
#                                                                       AX35 177
# LOOP TO FIND ATOM VARIABLE ADDRESS                                    AX35 178
# ----------------------------------                                    AX35 179
#                                                                       AX35 180
findnc:(MARKX,LX,MARK(7))#             Set ptrs to atomic nc            AX35 181
L20=MARK1(7)#                          Set to beg of atomic variables   AX35 182
L7=NINT(resp:(LR,I))#                  Get atom pointer                 AX35 183
DO M7=MARK(7),L7-STEP(7),STEP(7)#      Get l20 ptr & clear var list     AX35 184
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX35 185
K=NINT(atomtft:(M7))#                  Get displ fact type              AX35 186
ifnc:(LX,M7)#                          Noref whole atom                 AX35 187
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX35 188
incrx:(LX)#                            Take next nc ptr                 AX35 189
NEXT#                                  Take next atom                   AX35 190
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX35 191
MXP=MAXPAR#                            Set to max nb of paramters       AX35 192
IF ((MXP<=9)&(K<2))#                                                    AX35 193
IF (K==0)MXP=3#                                                         AX35 194
ELSE MXP=4#                                                             AX35 195
DO J=1,MXP#                            Loop over params                 AX35 196
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX35 197
IF ((K==0)&(J>3)&(J<10))NEXT#          Overall                          AX35 198
IF ((K==1)&(J>4)&(J<10))NEXT#          Isotropic                        AX35 199
IF ((REFPOP==no:)&(J==10))NEXT#        Population                       AX35 200
ifnc:(LX,M7+J)incrx:(LX)#              Norefed or constrained           AX35 201
ELSE L20=L20+1#                        Take next variable               AX35 202
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX35 203
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX35 204
#                                                                       AX35 205
# STORE DERIVATIVES                                                     AX35 206
# -----------------                                                     AX35 207
#                                                                       AX35 208
ifnc:(LX,L7)#                          Atom not refined                 AX35 209
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX35 210
incrx:(LX);NEXT#                                                        AX35 211
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX35 212
IF (TFT(I)==2)MXP=9#                                                    AX35 213
ELSE MXP=4#                                                             AX35 214
DO J=1,MXP#                            Loop over params                 AX35 215
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX35 216
ifnc:(LX,L7+J)#                        Param norefed or constrained     AX35 217
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX35 218
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX35 219
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX35 220
LY=NINT(nctype:(LX))#                  Get constraint address           AX35 221
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX35 222
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX35 223
REF=NINT(caddr:(LY))#                  Get reference param address      AX35 224
QX(REF)=QX(REF)+cfact:(LY)*DER(J)#     Update derivative                AX35 225
incry:(LY)#                            Take next packet                 AX35 226
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX35 227
incry:(LY)#                            Take next constraint             AX35 228
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX35 229
incrx:(LX)#                            Take next nc ptr                 AX35 230
NEXT#                                  Take next param                  AX35 231
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX35 232
QX(L20)=QX(L20)+DER(J)#                Sum derivative                   AX35 233
L20=L20+1#                             Take next variable               AX35 234
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX35 235
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX35 236
#                                                                       AX35 237
# BUILD MATRIX                                                          AX35 238
# ------------                                                          AX35 239
#                                                                       AX35 240
L21=MARK1(7)-MARK20+MARK21;M22=L22#    Set matrix pointers              AX35 241
STEPI=STOP20-MARK1(7)+1#               Row length                       AX35 242
IF ((IWT!=1)&(NCYCLE==ICYCLE))#        Build variance-covariance matr   AX35 243
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX35 244
M23=L23#                               Set covariance matrix pointer    AX35 245
DO L20=MARK1(7),STOP20#                Restraints only for atomic var   AX35 246
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX35 247
L=L20#                                 Point to diag el                 AX35 248
IF (QX(L20)!=0.0)#                     Not strictly equal 0             AX35 249
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX35 250
DO K=1,STEPI#                          Loop over one row                AX35 251
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX35 252
QX(M22)=QX(M22)+WEIGHT*QX(L20)*QX(L)#  Sum normal-eq matrix             AX35 253
QX(M23)=QX(M23)+(WEIGHT*resd:(LR))**2*QX(L20)*QX(L)# Var.               AX35 254
M22=M22+1;M23=M23+1;L=L+1#             Increment pointers               AX35 255
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX35 256
QX(L21)=QX(L21)+WEIGHT*QX(L20)*(resobs:(LR)-CALC)# Sum diff vector      AX35 257
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX35 258
ELSE#                                  No sum                           AX35 259
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX35 260
M22=M22+STEPI;M23=M23+STEPI#           Next diag element, next row      AX35 261
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX35 262
L21=L21+1;STEPI=STEPI-1#               Increment pointer                AX35 263
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX35 264
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX35 265
ELSE#                                  Only normal matrix               AX35 266
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX35 267
DO L20=MARK1(7),STOP20#                Restraints only for atomic var   AX35 268
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX35 269
L=L20#                                 Point to diag el                 AX35 270
IF (QX(L20)!=0.0)#                     Not strictly equal 0             AX35 271
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX35 272
DO K=1,STEPI#                          Loop over one row                AX35 273
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX35 274
QX(M22)=QX(M22)+WEIGHT*QX(L20)*QX(L)#  Sum normal-eq matrix             AX35 275
M22=M22+1;L=L+1#                       Increment pointers               AX35 276
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX35 277
QX(L21)=QX(L21)+WEIGHT*QX(L20)*(resobs:(LR)-CALC)# Sum diff vector      AX35 278
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX35 279
ELSE M22=M22+STEPI#                    Next diag element                AX35 280
L21=L21+1;STEPI=STEPI-1#               Increment pointer                AX35 281
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX35 282
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX35 283
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX35 284
BUFIN(18)=BUFIN(18)+REAL(NO)#          Restraint counter                AX35 285
RETURN#                                                                 AX35 286
END#                                                                    AX35 287
#                                                                       AX35 288
systemheader:(AX38)#                   Print reflection                 AX38   1
####################################################################    AX38   2
#                                                                  #    AX38   3
#                               AX38                               #    AX38   4
#                                                                  #    AX38   5
####################################################################    AX38   6
SUBROUTINE AX38(H,STOL,ABSO,LORZ,AC,BC,CALC,WEIGHT,FO,$#                AX38   7
WFF,STATUS,ICYCLE)#                                                     AX38   8
include:(AACOMN)#                                                       AX38   9
include:(AXCOMN)#                                                       AX38  10
#                                                                       AX38  11
REAL ABSO#                             Absorption & mean path           AX38  12
REAL AC(8),BC(8)#                      Acal et bcal (entpol)            AX38  13
REAL FO#                               "observed" |f|                   AX38  14
REAL H(3)#                             Miller indices                   AX38  15
INTEGER I#                             Loop index                       AX38  16
INTEGER ICYCLE#                        Current cycle                    AX38  17
REAL LORZ#                             Lorentz-polarization             AX38  18
INTEGER MERO#                          Merohedries counter              AX38  19
REAL CALC#                             Calcuated values                 AX38  20
INTEGER STATUS#                        Status code                      AX38  21
REAL STOL#                             Sin(theta)/lambda                AX38  22
REAL WEIGHT#                           Weight of obs                    AX38  23
REAL WFF#                              Weight of f                      AX38  24
#                                                                       AX38  25
chardata:(AX381,["   h   k   l  STOL CY ST      Obs    Calc  $#         AX38  26
Sigma w(O-C)   F Obs  F Calc   w*dF       A      B   $#                 AX38  27
Yext Abs. Path 1/LP"])#                                                 AX38  28
chardata:(AX382," UEFALWR*")#                                           AX38  29
realdata:(FMT,[40413., 80413., 120413., 180632., 210313., 240303., $#   AX38  30
330913., 410813., 480713., 550713., 630822., 710822., 780722., $#       AX38  31
860822., 930722., 1000732., 1050522., 1100532., 1150522.])#             AX38  32
datastuff:#                                                             AX38  33
#                                                                       AX38  34
BUFOT(6)=0.0;BUFOT(10)=SQRT(WEIGHT)*(QX(IP+IREL(4))-CALC)#              AX38  35
IF ((ABS(BUFOT(10))>RFLIST)\(LIST<0)\(ICYCLE==NCYCLE))# List reflecti   AX38  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX38  37
DO I=1,3;BUFOT(I)=H(I)#                Hkl                              AX38  38
BUFOT(4)=STOL;BUFOT(5)=REAL(ICYCLE);BUFOT(8)=CALC#                      AX38  39
BUFOT(7)=QX(IP+IREL(4));BUFOT(9)=QX(IP+IREL(5))#                        AX38  40
BUFOT(12)=0.0;BUFOT(14)=0.0;BUFOT(15)=0.0#                              AX38  41
BUFOT(16)=SQRT(AMAX1(YEXT,0.0))#                                        AX38  42
IF (REFINE(2)>0)#                                                       AX38  43
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX38  44
DO MERO=1,NB(2);BUFOT(12)=BUFOT(12)+entpol:(MERO)*$#                    AX38  45
(AC(MERO)*AC(MERO)+BC(MERO)*BC(MERO))#                                  AX38  46
BUFOT(12)=SQRT(BUFOT(12)*YEXT)#                                         AX38  47
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX38  48
ELSE#                                                                   AX38  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX38  50
BUFOT(14)=AC(1);BUFOT(15)=BC(1)#                                        AX38  51
BUFOT(12)=SQRT(AMAX1((BUFOT(14)**2+BUFOT(15)**2)*YEXT,0.0))#            AX38  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX38  53
BUFOT(11)=FO;BUFOT(13)=WFF*(BUFOT(11)-BUFOT(12))#                       AX38  54
BUFOT(17)=ABSO;BUFOT(18)=TBAR;BUFOT(19)=LORZ#                           AX38  55
ncodefld:(BUFOT,1,CHROT,FMT,19)#                                        AX38  56
movechar:(AX382,IABS(STATUS),CHROT,23,1,0)# Status                      AX38  57
IF ((STATUS>5)\(STATUS<0))movechar:(AX382,9,CHROT,24,1,0)# Move "*"     AX38  58
writeline:(0,AX381,NAX381,1,3)#        Output line                      AX38  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX38  60
RETURN#                                                                 AX38  61
END#                                                                    AX38  62
#                                                                       AX38  63
macro:(yan:,1.0/SQRT(1.0+$3*$4+$1*$4**2/(1.0+$2*$4)))#                  AX38  64
macro:(dyan:,-0.5*yan:($1,$2,$3,$4)**2*$#                               AX38  65
($3+$1*$4*(2.0+$2*$4)/(1.0+$2*$4)**2))#                                 AX38  66
#                                                                       AX38  67
systemheader:(AX39)#                   Extinction derivatives           AX39   1
####################################################################    AX39   2
#                                                                  #    AX39   3
#                               AX39                               #    AX39   4
#                                                                  #    AX39   5
####################################################################    AX39   6
SUBROUTINE AX39(H,FF,STOL,PSI,SKF,FAC,SQRW)#                            AX39   7
include:(AACOMN)#                                                       AX39   8
include:(AXCOMN)#                                                       AX39   9
#                                                                       AX39  10
REAL FF#                               (a+ib)*(a-ib)                    AX39  11
REAL AP,BP,CP,AS,BS,CS#                See becker and coppens papers    AX39  12
REAL ASQ#                              Square of classical electron r   AX39  13
REAL COS2T,COS2T2,SIN2T#               Cos 2theta, (cos 2theta)**2, s   AX39  14
REAL COST,SINT,COSP,SINP#              Theta and psi                    AX39  15
REAL H(3)#                             Reflection indices, diffracted   AX39  16
REAL FAC#                              Fudge factor for derivative      AX39  17
REAL G,RHO,PSI,DPSIDG,DPSIDR#                                           AX39  18
REAL GAMMAP,GAMMAS#                                                     AX39  19
INTEGER GAUSS#                         Distribution                     AX39  20
INTEGER I,J#                           Loop indexes                     AX39  21
INTEGER LX,LY,MX#                      List pointers                    AX39  22
INTEGER L20#                           Vector pointer                   AX39  23
REAL NORMX,NORMY#                      Vector norms                     AX39  24
REAL PZPER,PZPAR#                      Polarization                     AX39  25
REAL U(3)#                             Miller indices of u-vector (fl   AX39  26
INTEGER REF#                           Constraint address               AX39  27
REAL STOL#                             Sine(theta)/lambda               AX39  28
REAL SKF,SF4#                          Scale factor and scaled struct   AX39  29
REAL SQRW#                             Square root of weight (for der   AX39  30
REAL V(6),VV(6),VZV#                   Vector and tensor                AX39  31
REAL X(3),Y(3),Z(3)#                   X: h/|h| in orth base            AX39  32
#                                      Y: r/|r| in orth base            AX39  33
#                                      Z: x^y   in orth base            AX39  34
REAL VEXT(6)#                          Temporary vector                 AX39  35
REAL XIP,XIS#                                                           AX39  36
REAL XPPER,YPPER,ZPPER,XSPER,YSPER,ZSPER#                               AX39  37
REAL XPPAR,YPPAR,ZPPAR,XSPAR,YSPAR,ZSPAR#                               AX39  38
REAL YPER,YPAR#                                                         AX39  39
#                                                                       AX39  40
IF (EXTTYP>0)GAUSS=yes:#               Gaussian distibution             AX39  41
ELSE         GAUSS=no:#                Lorentzian distribution          AX39  42
EXTTYP=IABS(EXTTYP)#                   Extinction type                  AX39  43
COS2T=1.0-2.0*(STOL*LAMBDA)**2#        Cos(2*theta)                     AX39  44
COS2T2=COS2T*COS2T#                    (cos(2*theta))**2                AX39  45
SIN2T=SQRT(1.0-COS2T2)#                Sin(2*theta)                     AX39  46
#                                                                       AX39  47
# VECTORS D=S0^S & N=S0^D (SEE COPPENS)                                 AX39  48
# -------------------------------------                                 AX39  49
#                                                                       AX39  50
IF (EXTTYP>3)#                         Anisotropic stuff                AX39  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX39  52
IF ((ABS(H(1)-H(2))<small:)&(ABS(H(2)-H(3))<small:))# Flack & schwa     AX39  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX39  54
U(1)=H(1);U(2)=-H(1);U(3)=0.0#                                          AX39  55
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX39  56
ELSE#                                                                   AX39  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX39  58
U(1)=H(2)-H(3);U(2)=H(3)-H(1);U(3)=H(1)-H(2)#                           AX39  59
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX39  60
PSI=PSI/57.295779513#                  Azimuthal angle in rad           AX39  61
SINT=STOL*LAMBDA;COST=SQRT(1.0-SINT**2)# Sin & cos of theta             AX39  62
COSP=COS(PSI);SINP=SIN(PSI)#           Sin & cos of psi                 AX39  63
NORMX=0.0;NORMY=0.0#                   Clear norms                      AX39  64
DO I=1,3#                              Loop over coordinates            AX39  65
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX39  66
X(I)=0.0;Y(I)=0.0#                     Clear x & y                      AX39  67
DO J=1,3#                              Loop over coordinates            AX39  68
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX39  69
X(I)=X(I)+B(I,J)*H(J)#                 H in orthonormal base            AX39  70
Y(I)=Y(I)+U(J)*BINV(J,I)#              U in orthonormal base            AX39  71
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX39  72
NORMX=NORMX+X(I)*X(I)#                 Sum norm                         AX39  73
NORMY=NORMY+Y(I)*Y(I)#                 Sum norm                         AX39  74
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX39  75
NORMX=SQRT(NORMX);NORMY=SQRT(NORMY)#   Compute norms                    AX39  76
DO I=1,3#                              Normalize vectors                AX39  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX39  78
X(I)=X(I)/NORMX;Y(I)=Y(I)/NORMY#                                        AX39  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX39  80
Z(1)=X(2)*Y(3)-X(3)*Y(2)#              Cross product                    AX39  81
Z(2)=X(3)*Y(1)-X(1)*Y(3)#                                               AX39  82
Z(3)=X(1)*Y(2)-X(2)*Y(1)#                                               AX39  83
NORMX=0.0;DO I=1,3;NORMX=NORMX+Z(I)*Z(I);NORMX=SQRT(NORMX)#             AX39  84
DO I=1,3;Z(I)=Z(I)/NORMX#              Normalize cross product          AX39  85
IF ((EXTTYP==4)\(EXTTYP==6))#          Aniso type i or gen              AX39  86
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX39  87
DO I=1,3;V(I)=COSP*Y(I)-SINP*Z(I)#     V normal to diffr plane          AX39  88
DO I=1,6;VEXT(I)=extani1:(I)#                                           AX39  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX39  90
IF (EXTTYP==5)#                        Aniso type ii                    AX39  91
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX39  92
DO I=1,3;V(I)=SINT*X(I)-COST*(SINP*Y(I)+COSP*Z(I))# V // diffr beam     AX39  93
DO I=1,6;VEXT(I)=extani2:(I)#                                           AX39  94
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX39  95
DO I=1,3;VV(I)=V(I)**2#                Loop over coordinates            AX39  96
VV(4)=2.0*V(1)*V(2);VV(5)=2.0*V(1)*V(3);VV(6)=2.0*V(2)*V(3)#            AX39  97
VZV=0.0;DO I=1,3;VZV=VZV+VEXT(I)*VV(I)+VEXT(I+3)*VV(I+3)#               AX39  98
VZV=1.0/SQRT(VZV)#                     Following thornley & nelmes      AX39  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX39 100
#                                                                       AX39 101
# POLARIZATION COMPONENTS                                               AX39 102
# -----------------------                                               AX39 103
#                                                                       AX39 104
IF (INEU==1)#                          Neutron ray                      AX39 105
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX39 106
PZPAR=0.0;PZPER=1.0#                   Perpendiculary polarized         AX39 107
ASQ=1.0#                               A = 10**-12 cm                   AX39 108
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX39 109
ELSE#                                  Xray                             AX39 110
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX39 111
PZPAR=(1.-RATIO)*COS2T2;   PZPER=RATIO#                                 AX39 112
PZPAR=PZPAR/(PZPAR+PZPER); PZPER=1.-PZPAR#                              AX39 113
#W=1.0/(RATIO+COS2T2)#                  CONSTRUCT POLARIZATIONS         AX39 114
#PZPAR=COS2T2*W;PZPER=RATIO*W#          PARALLEL AND PERP POLARIZATION  AX39 115
ASQ=0.07940786#                        A = classical radius of electr   AX39 116
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX39 117
#                                                                       AX39 118
# AP, BP, CP, AS, BS, CS, ... FROM BECKER AND COPPENS                   AX39 119
# ---------------------------------------------------                   AX39 120
#                                                                       AX39 121
GAMMAP=1.5*(LAMBDA**2/VOL)**2*ASQ#                                      AX39 122
GAMMAS=1000.0*TBAR*(LAMBDA**3/VOL**2)*ASQ# Warning- tbar in mm          AX39 123
AP=0.20+0.45*COS2T#                                                     AX39 124
BP=0.22-0.12*(0.5-COS2T)**2#                                            AX39 125
CP=2.00#                                                                AX39 126
IF ((EXTTYP==2)\(EXTTYP==5))#          Type ii iso or aniso             AX39 127
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX39 128
AS=AP;BS=BP;CS=CP#                                                      AX39 129
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX39 130
ELSE#                                  Type i or general, iso or anis   AX39 131
IF (GAUSS==yes:)#                      Gaussian distribution            AX39 132
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX39 133
AS=0.58+0.480*COS2T+0.24*COS2T2#                                        AX39 134
BS=0.02-0.025*COS2T#                                                    AX39 135
CS=2.12#                                                                AX39 136
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX39 137
ELSE#                                  Lorentzian distribution          AX39 138
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX39 139
AS=0.025+0.285*COS2T#                                                   AX39 140
BS=0.150-0.200*(0.75-COS2T)**2;IF (COS2T<0.0)BS=-0.45*COS2T#            AX39 141
CS=2.000#                                                               AX39 142
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX39 143
IF(EXTTYP==1)#                         For zachariasen case             AX39 144
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX39 145
AS=0.;   CS=2.#                                                         AX39 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX39 147
#                                                                       AX39 148
# COMPUTE G, RHO (=R/LAMBDA), PSI, DPSI/DG, DPSI/DR                     AX39 149
# -------------------------------------------------                     AX39 150
#                                                                       AX39 151
IF (MOD(EXTTYP-1,3)==0)#               Iso and aniso type i             AX39 152
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX39 153
IF (EXTTYP<3)G=g:#                     Local g iso                      AX39 154
ELSE         G=VZV#                    Local g ani                      AX39 155
RHO=0.0#                               Local rho                        AX39 156
PSI=G/SIN2T#                                                            AX39 157
DPSIDG=1.0/SIN2T#                                                       AX39 158
DPSIDR=0.0#                                                             AX39 159
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX39 160
IF (MOD(EXTTYP-1,3)==1)#               Iso and aniso type ii            AX39 161
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX39 162
G=0.0#                                 Local g                          AX39 163
IF (EXTTYP<3)RHO=rho:/LAMBDA#          Local rho iso                    AX39 164
ELSE         RHO=VZV/LAMBDA#           Local rho ani                    AX39 165
PSI=RHO#                                                                AX39 166
DPSIDG=0.0#                                                             AX39 167
DPSIDR=1.0/LAMBDA#                                                      AX39 168
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX39 169
IF (MOD(EXTTYP-1,3)==2)#               General iso and aniso            AX39 170
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX39 171
IF (EXTTYP<3)#                         General isotropic                AX39 172
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX39 173
G=g:;RHO=rho:/LAMBDA#                  Local g & rho                    AX39 174
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX39 175
ELSE#                                  General anisotropic              AX39 176
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX39 177
G=VZV;RHO=rho:/LAMBDA#                 Local g & rho                    AX39 178
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX39 179
IF (GAUSS==yes:)#                      Gaussian distribution            AX39 180
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX39 181
PSI=(RHO*G)/SQRT(G**2+(RHO*SIN2T)**2)#                                  AX39 182
DPSIDG=SIN2T**2*(PSI/G)**3#                                             AX39 183
DPSIDR=(PSI/RHO)**3/LAMBDA#                                             AX39 184
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX39 185
ELSE#                                  Lorentzian distribution          AX39 186
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX39 187
PSI=(RHO*G)/(G+RHO*SIN2T)#                                              AX39 188
DPSIDG=SIN2T*(PSI/G)**2#                                                AX39 189
DPSIDR=(PSI/RHO)**2/LAMBDA#                                             AX39 190
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX39 191
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX39 192
#                                                                       AX39 193
# COMPUTE XI (I & II)                                                   AX39 194
# -------------------                                                   AX39 195
#                                                                       AX39 196
XIP=0.0#                                                                AX39 197
IF (MOD(EXTTYP,3)==0)XIP=GAMMAP*FF*RHO**2# Primary extinction present   AX39 198
XIS=GAMMAS*FF*PSI#                     Secondary extinction             AX39 199
XPPER=XIP#                                                              AX39 200
YPPER=yan:(AP,BP,CP,XPPER)#                                             AX39 201
ZPPER=0.0#                                                              AX39 202
IF (MOD(EXTTYP,3)==0)ZPPER=dyan:(AP,BP,CP,XPPER)# Primary extinction    AX39 203
XSPER=XIS*YPPER#                                                        AX39 204
YSPER=yan:(AS,BS,CS,XSPER)#                                             AX39 205
ZSPER=dyan:(AS,BS,CS,XSPER)#                                            AX39 206
IF (INEU==1)#                                                           AX39 207
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX39 208
YEXT=YPPER*YSPER#                                                       AX39 209
IF (REFINE(4)>1)#                                                       AX39 210
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX39 211
DDPAR=YEXT*(1.0+XPPER*ZPPER)*(1.0+XSPER*ZSPER)#                         AX39 212
V(1)=YEXT*YPPER*ZSPER#                                                  AX39 213
V(2)=2.0*YEXT*GAMMAP*RHO*ZPPER*(1.0+ZSPER*YPPER*XIS)+$#                 AX39 214
V(1)*GAMMAS*DPSIDR#                                                     AX39 215
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX39 216
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX39 217
ELSE#                                  Xray                             AX39 218
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX39 219
XPPAR=COS2T2*XIP#                                                       AX39 220
YPPAR=yan:(AP,BP,CP,XPPAR)#                                             AX39 221
ZPPAR=0.0#                                                              AX39 222
IF (MOD(EXTTYP,3)==0)ZPPAR=dyan:(AP,BP,CP,XPPAR)#                       AX39 223
XSPAR=COS2T2*XIS*YPPAR#                                                 AX39 224
YSPAR=yan:(AS,BS,CS,XSPAR)#                                             AX39 225
ZSPAR=dyan:(AS,BS,CS,XSPAR)#                                            AX39 226
YPER=PZPER*YPPER*YSPER#                                                 AX39 227
YPAR=PZPAR*YPPAR*YSPAR#                                                 AX39 228
YEXT=YPER+YPAR#                                                         AX39 229
DDPAR=YPER*(1.0+XPPER*ZPPER)*(1.0+XSPER*ZSPER)$#                        AX39 230
+YPAR*(1.0+XPPAR*ZPPAR)*(1.0+XSPAR*ZSPAR)#                              AX39 231
IF (REFINE(4)>1)#                                                       AX39 232
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX39 233
V(1)=GAMMAS*(YPER*YPPER*ZSPER+YPAR*YPPAR*ZSPAR*COS2T2)*DPSIDG#          AX39 234
V(2)=YPER*(2.0*GAMMAP*RHO*ZPPER*(1.0+XSPER*ZSPER)/LAMBDA$#              AX39 235
+GAMMAS*YPPER*ZSPER*DPSIDR)$#                                           AX39 236
+YPAR*(2.0*GAMMAP*RHO*ZPPAR*(1.0+XSPAR*ZSPAR)/LAMBDA$#                  AX39 237
+GAMMAS*YPPAR*ZSPAR*DPSIDR)*COS2T2#                                     AX39 238
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX39 239
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX39 240
#                                                                       AX39 241
# TOTAL CONTRIBUTION AND DERIVATIVES                                    AX39 242
# ----------------------------------                                    AX39 243
#                                                                       AX39 244
IF (REFINE(4)==2)#                     Refine extinction                AX39 245
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX39 246
movereal:(0.0,1,BUFOT,1,14,2)#         Clear derivative list            AX39 247
SF4=FAC*SKF*FF**2#                     Scaled |f|**4                    AX39 248
#                                                                       AX39 249
# STORE EXTINCTION DERIVATIVES IN PARAMETER LIST                        AX39 250
# ----------------------------------------------                        AX39 251
#                                                                       AX39 252
IF (EXTTYP<4)#                         Isotropic extinction             AX39 253
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX39 254
IF (MOD(EXTTYP,2)==1)BUFOT(1)=SF4*V(1)# Type i  and gen                 AX39 255
IF (EXTTYP>1)        BUFOT(2)=SF4*V(2)# Type ii and gen                 AX39 256
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX39 257
ELSE#                                  Anisotropic extinction           AX39 258
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX39 259
V(3)=-(VZV**3)/2.0#                                                     AX39 260
IF (MOD(EXTTYP,2)==0)DO I=1,6;BUFOT(I+2)=SF4*V(1)*VV(I)*V(3)# I  and    AX39 261
IF (EXTTYP==5)DO I=1,6;BUFOT(I+8)=SF4*V(2)*VV(I)*V(3)*LAMBDA**2# Ii     AX39 262
IF (EXTTYP==6)BUFOT(2)=SF4*V(2)#                                        AX39 263
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX39 264
findnc:(MARKX,LX,MARK(4));L20=MARK1(4)# Set extinction ptrs             AX39 265
DO I=1,14#                             Loop over extinction params      AX39 266
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX39 267
ifnc:(LX,MARK(4)+I-1)#                 Constrained or not refined       AX39 268
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX39 269
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX39 270
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX39 271
LY=NINT(nctype:(LX))#                  Get constraint address           AX39 272
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX39 273
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX39 274
REF=NINT(caddr:(LY))#                  Get reference address paramete   AX39 275
QX(REF)=QX(REF)+BUFOT(I)*cfact:(LY)#                                    AX39 276
incry:(LY)#                            Next packet                      AX39 277
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX39 278
incry:(LY)#                            Next constraint                  AX39 279
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX39 280
incrx:(LX)#                            Next item in nc list             AX39 281
NEXT#                                  Take next param                  AX39 282
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX39 283
QX(L20)=BUFOT(I)*SQRW;L20=L20+1#       Store derivative                 AX39 284
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX39 285
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX39 286
IF (GAUSS==no:)EXTTYP=-EXTTYP#         Reset extinction type            AX39 287
FF=FF*YEXT#                            Correct for extinction           AX39 288
RETURN#                                                                 AX39 289
END#                                                                    AX39 290
#                                                                       AX39 291
xmacro:([yan:])#                                                        AX39 292
xmacro:([dyan:])#                                                       AX39 293
#                                                                       AX39 294
systemheader:(AX40)#                   Matrix operations                AX40   1
####################################################################    AX40   2
#                                                                  #    AX40   3
#                               AX40                               #    AX40   4
#                                                                  #    AX40   5
####################################################################    AX40   6
SUBROUTINE AX40(ICYCLE)#                                                AX40   7
include:(AACOMN)#                                                       AX40   8
include:(AXCOMN)#                                                       AX40   9
#                                                                       AX40  10
INTEGER ICYCLE#                        Current cycle nb                 AX40  11
#                                                                       AX40  12
CALL AX41(ICYCLE)#                     Limshift & origin fixing         AX40  13
CALL AX42(ICYCLE)#                     Positiveness & inversion         AX40  14
CALL AX43(ICYCLE)#                     Print summary                    AX40  15
IF (ICYCLE==NCYCLE)RETURN#             End of refinement                AX40  16
CALL AX44#                             Shift constraints on general p   AX40  17
CALL AX45#                             Print new parameters             AX40  18
RETURN#                                                                 AX40  19
END#                                                                    AX40  20
#                                                                       AX40  21
systemheader:(AX41)#                   Do limshift                      AX41   1
####################################################################    AX41   2
#                                                                  #    AX41   3
#                               AX41                               #    AX41   4
#                                                                  #    AX41   5
####################################################################    AX41   6
SUBROUTINE AX41(ICYCLE)#                                                AX41   7
include:(AACOMN)#                                                       AX41   8
include:(AXCOMN)#                                                       AX41   9
#                                                                       AX41  10
REAL BN,Q,BMAX,SUM#                    Used in origin fixing            AX41  11
REAL BOUND#                            Limit value for choosing solut   AX41  12
REAL G0,DG0,G1,DG1#                    Values of g0, g1 & their deriv   AX41  13
INTEGER ICYCLE#                                                         AX41  14
INTEGER I,J,K,L#                       Loop indexes                     AX41  15
INTEGER II,JJ,KK#                      Auxiliary diagonal pointers      AX41  16
INTEGER IJ#                            Non-diag matrix pointer          AX41  17
INTEGER MXP,MXP2#                      Local number of atomic paramet   AX41  18
INTEGER L20,L22,M20,LT#                Auxiliary matrix pointer         AX41  19
INTEGER L7,M7#                         Auxiliary atom list ptr          AX41  20
INTEGER LX,MX,LZ,MZ#                   Constr & limsft list pointers    AX41  21
INTEGER N#                             Address of maximal constant      AX41  22
INTEGER RANGE#                         Ecart entre certaines listes     AX41  23
REAL S#                                Solution of system               AX41  24
INTEGER STEPI,STEPJ,STEPK#             Go to next row                   AX41  25
INTEGER TFT,TFT2#                      Temp factor type                 AX41  26
#                                                                       AX41  27
chardata:(AX411,[" WARNING- Impossible to find optimal restraint"])#    AX41  28
chardata:(AX412,[" Origin fixing coefficients, one per atom"])#         AX41  29
chardata:(AX418,[" WARNING- Variable xxxx causes the matrix to be $#    AX41  30
degenerate, forced invariant"])#                                        AX41  31
realdata:(FMT,[111051., 221051., 331051., 441051., 551051., 661051., $# AX41  32
771051., 881051., 991051., 1101051.])#                                  AX41  33
realdata:(FORM,[101031., 211031., 321031., 431031., 541031., 651031., $#AX41  34
761031., 871031., 981031., 1091031.])#                                  AX41  35
realdata:(FM1,[171051., 321051., 471051., 621051., 771051., 921051., $# AX41  36
1071051.])#                                                             AX41  37
datastuff:#                                                             AX41  38
#                                                                       AX41  39
# LIM SHIFT OF ATOMIC VAR & GET B LIST                                  AX41  40
# ------------------------------------                                  AX41  41
#                                                                       AX41  42
IF (TRACE>small:)#                     Test for origin fixing           AX41  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX41  44
BMAX=0.0;SUM=0.0#                      Clear                            AX41  45
findnc:(MARKX,LX,MARK(7))#             Set nc list pointer              AX41  46
L20=MARK1(7);LT=MARKT#                 Set atom pointer in var list     AX41  47
STEPI=VARTOT-(L20-MARK20)#             Set row length                   AX41  48
L22=MARK22+((L20-MARK20)*(2*VARTOT-(L20-MARK20)+1))/2# Pointer on mat   AX41  49
movereal:(0.0,1,QX,MARK20,STOP20-MARK20+1,2)# Clear vector              AX41  50
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atomic variables       AX41  51
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX41  52
ifnc:(LX,L7)#                          Atom not refiend                 AX41  53
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX41  54
incrx:(LX);NEXT#                       Take next atom                   AX41  55
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX41  56
TFT=NINT(atomtft:(L7))#                Extract atom temp fact           AX41  57
MXP=MAXPAR#                            Set to max nb of paramters       AX41  58
IF ((MXP<=9)&(TFT<2))#                                                  AX41  59
IF (TFT==0)MXP=3#                                                       AX41  60
ELSE MXP=4#                                                             AX41  61
Q=0.0;BN=0.0#                          Clear factors                    AX41  62
DO I=1,MXP#                            Loop over parameters             AX41  63
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX41  64
IF ((TFT==0)&(I>3)&(I<10))NEXT#        Uov                              AX41  65
IF ((TFT==1)&(I>4)&(I<10))NEXT#        Uiso                             AX41  66
IF ((REFPOP==no:)&(I==10))NEXT#        Population                       AX41  67
ifnc:(LX,L7+I)#                        Parameter constrained            AX41  68
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX41  69
incrx:(LX);NEXT#                       Take next parameter              AX41  70
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX41  71
IF (I<=3)#                             Positionnal param                AX41  72
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX41  73
Q=Q+POLCON(4*I-3);BN=BN+POLCON(4*I-3)*QX(L22)#                          AX41  74
MX=LX;K=0#                             Set pointer & matrix offset      AX41  75
DO J=I+1,3#                            Loop over non diag part          AX41  76
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX41  77
ifnc:(MX,L7+J)#                        Parameter constrained            AX41  78
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX41  79
incrx:(MX);NEXT#                       Take next param                  AX41  80
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX41  81
K=K+1;BN=BN+2.0*POLCON(3*(I-1)+J)*QX(L22+K)#                            AX41  82
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX41  83
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX41  84
L20=L20+1;L22=L22+STEPI;STEPI=STEPI-1# Increment matrix pointers        AX41  85
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX41  86
QX(LT)=BN/Q#                           Get new item                     AX41  87
IF (QX(LT)>BMAX)N=LT#                  Get max of list                  AX41  88
BMAX=AMAX1(QX(LT),BMAX);SUM=SUM+QX(LT)# Sum of constants                AX41  89
LT=LT+1#                               Take next atom                   AX41  90
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX41  91
STOPT=LT-1#                            Set end-of-list marker           AX41  92
#                                                                       AX41  93
# SOLVE SYSTEM                                                          AX41  94
# ------------                                                          AX41  95
#                                                                       AX41  96
RANGE=STOPT-MARKT+1;BOUND=REAL(RANGE-2)/REAL(RANGE)# Get nb of refine   AX41  97
DO LT=MARKT,STOPT;QX(LT)=QX(LT)/BMAX#  Normalize list                   AX41  98
S=1.0+small:#                          Initial value                    AX41  99
CALL AX48(S,G0,DG0,G1,DG1)#            Get values for functions in bm   AX41 100
BMAX=SQRT(BMAX)#                       Get square root for normalizat   AX41 101
IF (G0>BOUND+small:)#                  No sol for g0                    AX41 102
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX41 103
IF (SUM-2.0*BMAX**2<small:)#           No solution                      AX41 104
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX41 105
writeline:(1,AX411,NAX411,3,3)#        Print warning                    AX41 106
BN=0.0;DO LT=MARKT,STOPT;IF (LT!=N)BN=AMAX1(BN,QX(LT))# Find 2nd        AX41 107
S=250000.0*(1.0-BN)#                   Approximate sol                  AX41 108
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX41 109
ELSE WHILE (ABS(G1-BOUND)>small:)#     Solution for g1                  AX41 110
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX41 111
S=S+(BOUND-G1)/DG1;CALL AX48(S,G0,DG0,G1,DG1)# Next iteration           AX41 112
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX41 113
DO LT=MARKT,STOPT#                                                      AX41 114
IF (LT!=N)QX(LT)=BMAX*(SQRT(S)-SQRT(S-QX(LT)))#                         AX41 115
ELSE QX(N)=BMAX*(SQRT(S)+SQRT(S-QX(N)))#                                AX41 116
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX41 117
ELSE#                                  Sol exists for g0                AX41 118
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX41 119
WHILE (ABS(G0-BOUND)>small:)#                                           AX41 120
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX41 121
S=S+(BOUND-G0)/DG0;CALL AX48(S,G0,DG0,G1,DG1)# Next iteration           AX41 122
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX41 123
DO LT=MARKT,STOPT;QX(LT)=BMAX*(SQRT(S)-SQRT(S-QX(LT)))# Get sol         AX41 124
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX41 125
writeline:(1,AX412,NAX412,3,3)#        Print header                     AX41 126
LT=MARKT#                              Encode 10 values per line        AX41 127
REPEAT#                                Loop over solution list          AX41 128
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX41 129
I=MIN0(STOPT-LT+1,10)#                 Nb of values to encode           AX41 130
ncodefld:(QX,LT,CHROT,FMT,I)#          Encode solutions                 AX41 131
writeline:(0,AX412,NAX412,1,3)#        Print line                       AX41 132
LT=LT+10;IF (LT>STOPT)BREAK#                                            AX41 133
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX41 134
#                                                                       AX41 135
# UPDATE ARRAY ELEMENTS                                                 AX41 136
# ---------------------                                                 AX41 137
#                                                                       AX41 138
findnc:(MARKX,LX,MARK(7));findbl:(MARKZ,LZ,MARK1(7))# Set list pointe   AX41 139
LT=MARKT;L20=MARK1(7);STEPI=VARTOT-L20+MARK20# Set variable & atom po   AX41 140
L22=MARK22+((L20-MARK20)*(2*VARTOT-L20+MARK20+1))/2# Pointer on matri   AX41 141
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX41 142
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX41 143
TFT=NINT(atomtft:(L7))#                Get displ fact type              AX41 144
ifnc:(LX,L7)#                          Atom not refined                 AX41 145
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX41 146
incrx:(LX);NEXT#                       Take next atom                   AX41 147
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX41 148
MXP=MAXPAR#                            Set to max nb of paramters       AX41 149
IF ((MXP<=9)&(TFT<2))#                                                  AX41 150
IF (TFT==0)MXP=3#                                                       AX41 151
ELSE MXP=4#                                                             AX41 152
DO I=1,MXP#                            Loop over params                 AX41 153
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX41 154
IF ((TFT==0)&(I>3)&(I<10))NEXT#        Uov                              AX41 155
IF ((TFT==1)&(I>4)&(I<10))NEXT#        Uiso                             AX41 156
IF ((REFPOP==no:)&(I==10))NEXT#        Population                       AX41 157
ifnc:(LX,L7+I)#                        Parameter constrained            AX41 158
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX41 159
incrx:(LX);NEXT#                       Take next parameter              AX41 160
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX41 161
ifbl:(LZ,L20)incrz:(LZ)#               This variable is already limit   AX41 162
ELSE IF (I<=3)#                        Positional variable not user-s   AX41 163
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX41 164
QX(L22)=QX(L22)+QX(LT)**2*POLCON(4*I-3)# Update diag el                 AX41 165
MX=LX;MZ=LZ;M20=L20;K=0#               Get pointer                      AX41 166
DO J=I+1,MXP#                          Loop over remaining pos param    AX41 167
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX41 168
IF ((TFT==0)&(J>3)&(J<10))NEXT#        Uov                              AX41 169
IF ((TFT==1)&(J>4)&(J<10))NEXT#        Uiso                             AX41 170
IF ((REFPOP==no:)&(J==10))NEXT#        Population                       AX41 171
ifnc:(MX,L7+J)#                        Constrained                      AX41 172
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX41 173
incrx:(MX);NEXT#                       Take next atom                   AX41 174
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX41 175
K=K+1#                                 Count this variable              AX41 176
ifbl:(MZ,M20)incrz:(M20)#              Already shift-limited            AX41 177
ELSE IF (J<=3)QX(L22+K)=QX(L22+K)+QX(LT)**2*POLCON(3*(I-1)+J)# Same a   AX41 178
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX41 179
L=LT+1#                                Set 2nd atom in list             AX41 180
DO M7=L7+STEP(7),STOP(7)-STEP(7),STEP(7)# Loop over 2nd atom            AX41 181
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX41 182
ifnc:(MX,M7)#                          Atom not refined                 AX41 183
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX41 184
incrx:(MX);NEXT#                       Take next atom                   AX41 185
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX41 186
TFT2=NINT(atomtft:(M7))#                                                AX41 187
MXP2=MAXPAR#                           Set to max nb of paramters       AX41 188
IF ((MXP2<=9)&(TFT2<2))#                                                AX41 189
IF (TFT2==0)MXP2=3#                                                     AX41 190
ELSE MXP2=4#                                                            AX41 191
DO J=1,MXP2#                           Loop over params                 AX41 192
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX41 193
IF ((TFT2==0)&(J>3)&(J<10))NEXT#       Uov                              AX41 194
IF ((TFT2==1)&(J>4)&(J<10))NEXT#       Uiso                             AX41 195
IF ((REFPOP==no:)&(J==10))NEXT#        Population                       AX41 196
ifnc:(MX,M7+J)#                        Constrained                      AX41 197
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX41 198
incrx:(MX);NEXT#                       Take next atom                   AX41 199
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX41 200
K=K+1#                                 Count this variable              AX41 201
ifbl:(MZ,M20)incrz:(M20)#              Already shift-limited            AX41 202
ELSE IF (J<=3)QX(L22+K)=QX(L22+K)+QX(LT)*QX(L)*POLCON(3*(I-1)+J)# Sam   AX41 203
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX41 204
L=L+1#                                                                  AX41 205
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX41 206
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX41 207
L20=L20+1;L22=L22+STEPI;STEPI=STEPI-1#                                  AX41 208
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX41 209
LT=LT+1#                                                                AX41 210
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX41 211
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX41 212
#                                                                       AX41 213
# LIM SHIFT OF ALL VARIABLES                                            AX41 214
# --------------------------                                            AX41 215
#                                                                       AX41 216
LZ=MARKZ;L22=MARK22;STEPI=VARTOT#      Set matrix pointers              AX41 217
DO L20=MARK20,STOP20#                  Loop over all variables          AX41 218
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX41 219
ifbl:(LZ,L20)#                         Variable is shift-limited        AX41 220
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX41 221
IF (blfact:(LZ)>1.0)QX(L22)=QX(L22)+blfact:(LZ)# Increment diagonal e   AX41 222
ELSE QX(L22)=QX(L22)*(1.0+blfact:(LZ))# Levenberg-markart               AX41 223
incrz:(LZ)#                            Take next limshift               AX41 224
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX41 225
L22=L22+STEPI;STEPI=STEPI-1#           Increment pointers               AX41 226
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX41 227
#                                                                       AX41 228
# TEST FOR POSITIVESS OF CORRELATIONS                                   AX41 229
# -----------------------------------                                   AX41 230
#                                                                       AX41 231
II=MARK22;STEPI=VARTOT#                Set ptrs                         AX41 232
IF ((IWT!=1)&(ICYCLE==NCYCLE))RANGE=MARK23-MARK22#                      AX41 233
ELSE RANGE=0#                                                           AX41 234
DO I=1,VARTOT#                         Loop over vars                   AX41 235
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX41 236
IF (QX(II)<=0.0)#                      Matrix singular                  AX41 237
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX41 238
ncodefld:(REAL(I),1,AX418,230413.,1)#  Code var nb                      AX41 239
writeline:(0,AX418,NAX418,3,3)#        Print warning                    AX41 240
IJ=MARK22+I-1;STEPJ=VARTOT-1#                                           AX41 241
DO J=1,I-1#                            Loop over ith column             AX41 242
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX41 243
QX(IJ)=0.0;QX(IJ+RANGE)=0.0#           Clear element                    AX41 244
IJ=IJ+STEPJ;STEPJ=STEPJ-1#             Update pointers                  AX41 245
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX41 246
QX(II)=1.0;QX(MARK21+I-1)=0.0;IJ=II#   Set diag element                 AX41 247
IF (RANGE>0)QX(II+RANGE)=0.0#          Clear variance                   AX41 248
DO J=I+1,VARTOT#                       Loop over ith row                AX41 249
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX41 250
IJ=IJ+1#                               Update pointer                   AX41 251
QX(IJ)=0.0;QX(IJ+RANGE)=0.0#           Clear element                    AX41 252
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX41 253
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX41 254
ELSE#                                  Further test                     AX41 255
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX41 256
JJ=II;STEPJ=STEPI;IJ=II#               Set ptrs to ith diag el          AX41 257
DO J=I+1,VARTOT#                       Loop over remaining vars         AX41 258
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX41 259
JJ=JJ+STEPJ;STEPJ=STEPJ-1;IJ=IJ+1#     Jj=next diag el, ij=row i,col    AX41 260
IF (QX(JJ)<=0.0)NEXT#                  Test only positive diag elemen   AX41 261
IF (QX(II)*QX(JJ)<1.01*QX(IJ)*QX(IJ))# Test for ill-defined matrix      AX41 262
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX41 263
ncodefld:(REAL(I),1,AX418,230413.,1)#  Code var nb                      AX41 264
writeline:(0,AX418,NAX418,3,3)#        Print warning                    AX41 265
QX(MARK21+I-1)=0.0#                    Reset diff vector                AX41 266
QX(II)=1.0#                            Reset diag el.                   AX41 267
IF (RANGE>0)QX(II+RANGE)=0.0#          Clear esd                        AX41 268
DO K=I+1,VARTOT#                       Clear end of row                 AX41 269
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX41 270
QX(II+K-I)=0.0;QX(II+RANGE+K-I)=0.0#   Clear covariances                AX41 271
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX41 272
KK=II;STEPK=STEPI#                     Set ptrs                         AX41 273
DO K=I-1,1,-1#                         Clear start-of-column            AX41 274
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX41 275
KK=KK-STEPK;STEPK=STEPK+1#             Set pointers                     AX41 276
QX(KK)=0.0;QX(KK+RANGE)=0.0#           Clear el.                        AX41 277
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX41 278
BREAK#                                 Test next var                    AX41 279
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX41 280
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX41 281
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX41 282
II=II+STEPI;STEPI=STEPI-1#             Next diag el                     AX41 283
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX41 284
RETURN#                                                                 AX41 285
END#                                                                    AX41 286
#                                                                       AX41 287
systemheader:(AX42)#                   Call inversion                   AX42   1
####################################################################    AX42   2
#                                                                  #    AX42   3
#                               AX42                               #    AX42   4
#                                                                  #    AX42   5
####################################################################    AX42   6
SUBROUTINE AX42(ICYCLE)#                                                AX42   7
include:(AACOMN)#                                                       AX42   8
include:(AXCOMN)#                                                       AX42   9
#                                                                       AX42  10
INTEGER ICYCLE#                        Current cycle number             AX42  11
INTEGER I,J,K,L#                       Loop indices                     AX42  12
INTEGER II,JJ,IJ,IK,JK,IL,JL#          Matrix pointers                  AX42  13
INTEGER L20,L23,L24#                   Matrix pointers                  AX42  14
INTEGER N#                                                              AX42  15
REAL S#                                Correlation                      AX42  16
REAL T#                                General purpose                  AX42  17
INTEGER RANGE#                         Offset betwen mem locations      AX42  18
INTEGER STEPI,STEPJ#                   Row lengths                      AX42  19
#                                                                       AX42  20
chardata:(AX422," Correlations between variables")#                     AX42  21
chardata:(AX426,[-:])#                                                  AX42  22
realdata:(FMT,[40313., 80313., 160742., 220313., 260313., 340742., $#   AX42  23
400313., 440313., 520742., 580313., 620313., 700742., 760313., 800313.,#AX42  24
880742., 940313., 980313., 1060742.])#                                  AX42  25
datastuff:#                                                             AX42  26
#                                                                       AX42  27
IF ((IABS(LISTMA)==2)&((ICYCLE==NCYCLE)\(LISTMA>0)))# Print direct      AX42  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX42  29
CALL AX47(ICYCLE,MARK22,VARTOT,-1)#    Normal-eq                        AX42  30
CALL AX47(ICYCLE,MARK21,1,-2)#         Vector                           AX42  31
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX42  32
IF ((LISTCO==yes:)&(ICYCLE==NCYCLE))CALL AX47(ICYCLE,MARK23,VARTOT,-3)# AX42  33
#                                                                       AX42  34
# INVERSION                                                             AX42  35
# ---------                                                             AX42  36
#                                                                       AX42  37
IF (VARTOT==1)#                        Only one var                     AX42  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX42  39
QX(MARK22)=1.0/QX(MARK22);QX(MARK20)=QX(MARK21)*QX(MARK22)# Inverse "   AX42  40
IF ((IWT!=1)&(ICYCLE==NCYCLE))QX(MARK21)=QX(MARK23)*QX(MARK22)**2# Ge   AX42  41
ELSE QX(MARK21)=QX(MARK22)#            Esd with 1/sigma**2 weighting    AX42  42
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX42  43
ELSE#                                  Many variables                   AX42  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX42  45
CALL AX49(MARK22,VARTOT)#              Invert this block                AX42  46
#                                                                       AX42  47
# VECTOR MULTIPLICATION                                                 AX42  48
# ---------------------                                                 AX42  49
#                                                                       AX42  50
L20=MARK20#                            Set pointer                      AX42  51
DO I=1,VARTOT#                         Loop over variables              AX42  52
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX42  53
QX(L20)=0.0#                           Clear rslt                       AX42  54
IJ=MARK22+I-1;STEPI=VARTOT#            Set pointers                     AX42  55
DO J=1,VARTOT#                         Loop over delta_obs vector       AX42  56
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX42  57
QX(L20)=QX(L20)+QX(IJ)*QX(MARK21+J-1)# Perform multiplication           AX42  58
IF (I>J)#                              Next matrix element              AX42  59
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX42  60
STEPI=STEPI-1;IJ=IJ+STEPI#             Down to next row                 AX42  61
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX42  62
ELSE IJ=IJ+1#                          Right to next column             AX42  63
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX42  64
L20=L20+1#                             Increment vector element         AX42  65
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX42  66
IF ((LISTMA!=0)&((ICYCLE==NCYCLE)\(LISTMA>0)))# Print inverse           AX42  67
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX42  68
CALL AX47(ICYCLE,MARK22,VARTOT,1)#     Normal-eq                        AX42  69
CALL AX47(ICYCLE,MARK20,1,2)#          Vector                           AX42  70
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX42  71
#                                                                       AX42  72
# MATRIX MULTIPLICATION (DIAGONAL ELEMENTS [VARIANCES])                 AX42  73
# -----------------------------------------------------                 AX42  74
#                                                                       AX42  75
IF (((IWT!=1)\(ABS(CORDIS)>small:))&(ICYCLE==NCYCLE))# Not usual weig   AX42  76
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX42  77
RANGE=MARK23-MARK22;L24=MARK24;IJ=MARK22# Set matrix pointers           AX42  78
DO I=1,VARTOT;DO J=I,VARTOT#           Loop over all elements           AX42  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX42  80
IF (I==J)EDW=EDW-QX(IJ)*QX(IJ+RANGE)#  Diag element                     AX42  81
ELSE EDW=EDW-2.0*QX(IJ)*QX(IJ+RANGE)#                                   AX42  82
QX(L24)=0.0#                           Clear element                    AX42  83
IJ=IJ+1#                               Take next element                AX42  84
IK=MARK22+I-1;JK=MARK22+J-1#           Set pointers                     AX42  85
L23=MARK23#                                                             AX42  86
DO K=1,VARTOT#                                                          AX42  87
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX42  88
S=QX(IK);T=QX(JK)#                     Save elements                    AX42  89
QX(L24)=QX(L24)+S*T*QX(L23)#           Diag element                     AX42  90
L23=L23+1#                                                              AX42  91
IF (K<I)IK=IK+VARTOT-K#                                                 AX42  92
ELSE IK=IK+1#                                                           AX42  93
IF (K<J)JK=JK+VARTOT-K#                                                 AX42  94
ELSE JK=JK+1#                                                           AX42  95
IL=IK;JL=JK#                           Off-diagonal                     AX42  96
DO L=K+1,VARTOT#                                                        AX42  97
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX42  98
QX(L24)=QX(L24)+(S*QX(JL)+T*QX(IL))*QX(L23)#                            AX42  99
L23=L23+1#                                                              AX42 100
IF (L<I)IL=IL+VARTOT-L#                                                 AX42 101
ELSE IL=IL+1#                                                           AX42 102
IF (L<J)JL=JL+VARTOT-L#                                                 AX42 103
ELSE JL=JL+1#                                                           AX42 104
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX42 105
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX42 106
L24=L24+1#                             Take next element                AX42 107
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX42 108
movereal:(QX,MARK24,QX,MARK22,MARK23-MARK22,0)# Copy to list22          AX42 109
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX42 110
ELSE#                                  Weight = 1/sigma**2              AX42 111
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX42 112
EDW=(BUFIN(2)+BUFIN(18)+REAL(MARK1(2)-MARK1(1)))-REAL(VARTOT)# E(dw)    AX42 113
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX42 114
IF (ICYCLE==NCYCLE)#                   Only last cycle                  AX42 115
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX42 116
N=0;IJ=MARK22;II=MARK22;STEPI=VARTOT#                                   AX42 117
DO I=1,VARTOT-1#                       Loop over 1st variable           AX42 118
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX42 119
JJ=II+STEPI;STEPJ=STEPI-1#             Get next diag el                 AX42 120
DO J=I+1,VARTOT#                       Loop over 2nd variable           AX42 121
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX42 122
IJ=IJ+1#                               Increment covariance pointer     AX42 123
S=QX(IJ)/SQRT(QX(II)*QX(JJ))#          Correlation                      AX42 124
IF (ABS(S)>CORBND)#                    Print if greater than bound      AX42 125
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX42 126
IF (N==0)#                             No correlation printed yet       AX42 127
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX42 128
writeline:(1,AX422,NAX422,3,3)#        Print title                      AX42 129
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX42 130
K=MOD(N,6)#                            6 correlations per line          AX42 131
ncodefld:(REAL(I),1,CHROT,FMT(3*K+1),1)# Encode ij                      AX42 132
ncodefld:(REAL(J),1,CHROT,FMT(3*K+2),1)#                                AX42 133
ncodefld:(S,1,CHROT,FMT(3*K+3),1)#     Encode corr                      AX42 134
movechar:(AX426,1,CHROT,NINT(FMT(3*K+1))/10000+1,1,0)# Move delimiter   AX42 135
movechar:(AX426,2,CHROT,NINT(FMT(3*K+2))/10000+1,1,0)#                  AX42 136
IF (K==5)#                             Print line                       AX42 137
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX42 138
writeline:(0,AX422,NAX422,1,3)#        Print line                       AX42 139
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX42 140
N=N+1#                                 Count entry                      AX42 141
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX42 142
JJ=JJ+STEPJ;STEPJ=STEPJ-1#             Take next diag el                AX42 143
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX42 144
II=II+STEPI;STEPI=STEPI-1;IJ=IJ+1#     Take next diag el                AX42 145
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX42 146
IF (N>0)writeline:(0,AX422,NAX422,1,3)# Print last correlation line     AX42 147
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX42 148
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX42 149
IF ((LISTCO==yes:)&(ICYCLE==NCYCLE))CALL AX47(ICYCLE,MARK22,VARTOT,3)#  AX42 150
RETURN#                                                                 AX42 151
END#                                                                    AX42 152
#                                                                       AX42 153
systemheader:(AX43)#                   Print agreement factors          AX43   1
####################################################################    AX43   2
#                                                                  #    AX43   3
#                               AX43                               #    AX43   4
#                                                                  #    AX43   5
####################################################################    AX43   6
SUBROUTINE AX43(ICYCLE)#                                                AX43   7
include:(AACOMN)#                                                       AX43   8
include:(AXCOMN)#                                                       AX43   9
#                                                                       AX43  10
INTEGER ICYCLE#                                                         AX43  11
#                                                                       AX43  12
chardata:(AX000," Final Agreement Factors Before Cycle   ")#            AX43  13
chardata:(AX001," Goodness of Fit            XX.XXXX")#                 AX43  14
chardata:(AX011,[" Sum(Weight(Obs-Calc)**2)                  $#         AX43  15
        "])#                                                            AX43  16
chardata:(AX012,[" Sum(Weight(Robs-Rcalc)**2)                $#         AX43  17
        "])#                                                            AX43  18
chardata:(AX013,[" Sum(Weight(Dobs-Dcalc)**2)                $#         AX43  19
        "])#                                                            AX43  20
chardata:(AX014,[" Number of Observations             (NOBS)       $#   AX43  21
  "])#                                                                  AX43  22
chardata:(AX015,[" Number of Contributing Reflections (NREF)       $#   AX43  23
  "])#                                                                  AX43  24
chardata:(AX016,[" Number of Restraints               (NRES)       $#   AX43  25
  "])#                                                                  AX43  26
chardata:(AX017,[" Number of Variables                (NV)         $#   AX43  27
  "])#                                                                  AX43  28
chardata:(AX018,[" Number of Less-Than                (NLT)        $#   AX43  29
  "])#                                                                  AX43  30
chardata:(AX020,[" Durbin-Watson d Statistic                       $#   AX43  31
  "])#                                                                  AX43  32
chardata:(AX031," R factors                                        $#   AX43  33
R                Numerator     Denominator")#                           AX43  34
chardata:(AX032," of Contributing Reflections")#                        AX43  35
chardata:(AX033," of Weighted Contributing Reflections")#               AX43  36
chardata:(AX034," of Observations")#                                    AX43  37
chardata:(AX035," of Less-than")#                                       AX43  38
chardata:(AX037," of Weighted Restraints")#                             AX43  39
chardata:(AX039," of Weighted Distances")#                              AX43  40
realdata:(FMT,[511281., 510713., 510742., 761312., 921312., 400213.])#  AX43  41
datastuff:#                                                             AX43  42
#                                                                       AX43  43
IF (ICYCLE==NCYCLE)#                   Last cycle                       AX43  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX43  45
IF (EDW>0.0)GOOF=SQRT((BUFIN(15)+BUFIN(16)+BUFIN(17))/EDW)# Compute s   AX43  46
writeline:(2,AX000,23,3,3)#            Print title                      AX43  47
ncodefld:(GOOF,1,AX001,350742.,1)#     Encode s                         AX43  48
writeline:(1,AX001,NAX001,3,3)#        Print goodness-of-fit            AX43  49
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX43  50
ELSE#                                                                   AX43  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX43  52
movechar:(AX000,7,CHROT,1,31,0)#                                        AX43  53
ncodefld:(REAL(ICYCLE),1,CHROT,FMT(6),1)#                               AX43  54
writeline:(2,0,0,1,3)#                                                  AX43  55
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX43  56
BUFIN(7)=SQRT(AMAX1(0.0,BUFIN(7)))#                                     AX43  57
BUFIN(8)=SQRT(AMAX1(0.0,BUFIN(8)))#                                     AX43  58
BUFIN(30)=SQRT(AMAX1(0.0,BUFIN(30)))#                                   AX43  59
BUFIN(31)=SQRT(AMAX1(0.0,BUFIN(31)))#                                   AX43  60
BUFIN(24)=SQRT(AMAX1(0.0,BUFIN(24)))#                                   AX43  61
BUFIN(25)=SQRT(AMAX1(0.0,BUFIN(25)))#                                   AX43  62
IF (BUFIN(5)>0.0)BUFIN(3)=BUFIN(4)/BUFIN(5)#                            AX43  63
IF (BUFIN(8)>0.0)BUFIN(6)=BUFIN(7)/BUFIN(8)#                            AX43  64
IF (BUFIN(11)>0.0)BUFIN(9)=BUFIN(10)/BUFIN(11)#                         AX43  65
IF (BUFIN(14)>0.0)BUFIN(12)=BUFIN(13)/BUFIN(14)#                        AX43  66
IF (BUFIN(25)>0.0)BUFIN(23)=BUFIN(24)/BUFIN(25)#                        AX43  67
IF (BUFIN(31)>0.0)BUFIN(29)=BUFIN(30)/BUFIN(31)#                        AX43  68
IF (BUFIN(22)>0.0)BUFIN(20)=BUFIN(21)/BUFIN(22)#                        AX43  69
BUFIN(19)=REAL(STOP20-MARK20+1)#                                        AX43  70
ncodefld:(BUFIN(15),1,AX011,FMT(1),1);writeline:(1,AX011,NAX011,3,3)#   AX43  71
ncodefld:(BUFIN(16),1,AX012,FMT(1),1);writeline:(0,AX012,NAX012,3,3)#   AX43  72
IF (REFINE(1)>0)#                      Refine shape                     AX43  73
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX43  74
ncodefld:(BUFIN(17),1,AX013,FMT(1),1);writeline:(0,AX013,NAX013,3,3)#   AX43  75
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX43  76
ncodefld:(BUFIN(1),1,AX014,FMT(2),1);writeline:(1,AX014,NAX014,3,3)#    AX43  77
ncodefld:(BUFIN(2),1,AX015,FMT(2),1);writeline:(0,AX015,NAX015,3,3)#    AX43  78
ncodefld:(BUFIN(18),1,AX016,FMT(2),1);writeline:(0,AX016,NAX016,3,3)#   AX43  79
ncodefld:(BUFIN(19),1,AX017,FMT(2),1);writeline:(0,AX017,NAX017,3,3)#   AX43  80
ncodefld:(BUFIN(26),1,AX018,FMT(2),1);writeline:(0,AX018,NAX018,3,3)#   AX43  81
ncodefld:(BUFIN(20),1,AX020,FMT(3),1);writeline:(0,AX020,NAX020,3,3)#   AX43  82
writeline:(1,AX031,NAX031,3,3)#                                         AX43  83
movechar:(AX032,1,CHROT,1,NAX032,0)#                                    AX43  84
ncodefld:(BUFIN(3),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#    AX43  85
movechar:(AX033,1,CHROT,1,NAX033,0)#                                    AX43  86
ncodefld:(BUFIN(6),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#    AX43  87
movechar:(AX034,1,CHROT,1,NAX034,0)#                                    AX43  88
ncodefld:(BUFIN(9),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#    AX43  89
movechar:(AX035,1,CHROT,1,NAX035,0)#                                    AX43  90
ncodefld:(BUFIN(12),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#   AX43  91
movechar:(AX037,1,CHROT,1,NAX037,0)#                                    AX43  92
ncodefld:(BUFIN(23),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#   AX43  93
IF (REFINE(1)>0)#                      Refine shape                     AX43  94
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX43  95
movechar:(AX039,1,CHROT,1,NAX039,0)#                                    AX43  96
ncodefld:(BUFIN(29),1,CHROT,FMT(3),3);writeline:(0,AX031,NAX031,1,3)#   AX43  97
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX43  98
RETURN#                                                                 AX43  99
END#                                                                    AX43 100
#                                                                       AX43 101
systemheader:(AX44)#                   Sum and decrepencies (i)         AX44   1
####################################################################    AX44   2
#                                                                  #    AX44   3
#                               AX44                               #    AX44   4
#                                                                  #    AX44   5
####################################################################    AX44   6
SUBROUTINE AX44#                                                        AX44   7
include:(AACOMN)#                                                       AX44   8
include:(AXCOMN)#                                                       AX44   9
#                                                                       AX44  10
REAL CALC#                             Constraint value & rescale fac   AX44  11
INTEGER I#                             Loop index                       AX44  12
INTEGER L,M#                           Parameter pointers               AX44  13
INTEGER L6#                            Atom type pointer                AX44  14
INTEGER LX,LY,MY#                      Constraint list pointers         AX44  15
INTEGER REF#                           Reference param address          AX44  16
#                                                                       AX44  17
chardata:(AX442,"       Face   Ent-PolScale  Extinc.U ovl  ")#          AX44  18
chardata:(AX441,[" WARNING- Constraint parameter xxxxxxx xx $#          AX44  19
outside limits"])#                                                      AX44  20
datastuff:#                                                             AX44  21
#                                                                       AX44  22
# COMPUTE NEW CONSTRAINT VALUES ON GENERAL PARAMETERS                   AX44  23
# ---------------------------------------------------                   AX44  24
#                                                                       AX44  25
LX=MARKX;LY=MARKY#                     Set constraint list pointers     AX44  26
DO I=1,5#                              Loop over first 5 classes        AX44  27
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX44  28
IF (REFINE(I)==2)#                     Refined class                    AX44  29
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX44  30
DO L=MARK(I),STOP(I)-STEP(I),STEP(I)#  Loop over all params in class    AX44  31
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX44  32
ifnc:(LX,L)#                           Param is not refined or constr   AX44  33
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX44  34
IF (NINT(nctype:(LX))!=noref:)#        Param is constraint              AX44  35
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX44  36
LY=NINT(nctype:(LX))#                  Get constraint address           AX44  37
M=L+OFFSET(I);MY=LY#                   Extract actual param address     AX44  38
CALC=QX(M)#                            Old value                        AX44  39
WHILE (caddr:(MY)>small:)#             Loop over constraint             AX44  40
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX44  41
REF=NINT(caddr:(MY))#                  Get reference param address      AX44  42
CALC=CALC+cfact:(MY)*QX(REF)*DAMP#     Sum partial shifts               AX44  43
incry:(MY)#                            Next reference                   AX44  44
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX44  45
QX(M)=CALC#                            Store new constraint value       AX44  46
IF ((TST(I)>0)&((CALC<=LBND(I))\(CALC>=HBND(I))))# Outside bounds       AX44  47
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX44  48
IF (TST(I)==1)#                        Reset value                      AX44  49
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX44  50
IF (CALC<=LBND(I))#                    Low bound                        AX44  51
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX44  52
CALC=(CALC-QX(M))/(LBND(I)+small:-QX(M))# Damping factor                AX44  53
QX(M)=LBND(I)+small:#                  New value                        AX44  54
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX44  55
ELSE#                                  High bound                       AX44  56
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX44  57
CALC=(CALC-QX(M))/(HBND(I)-small:-QX(M))# Damping factor                AX44  58
QX(M)=HBND(I)-small:#                  New value                        AX44  59
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX44  60
MY=LY#                                 Return to start of constraint    AX44  61
WHILE (caddr:(MY)>small:)#             Loop over constraint             AX44  62
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX44  63
REF=NINT(caddr:(MY))#                  Get reference address            AX44  64
QX(REF)=CALC*QX(REF)#                  Rescale shifts                   AX44  65
incry:(MY)#                                                             AX44  66
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX44  67
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX44  68
IF (TST(I)==2)#                        Print warning                    AX44  69
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX44  70
movechar:(AX442,7*I+1,AX441,32,7,0)#   Move class name                  AX44  71
ncodefld:(REAL((L-MARK(I))/STEP(I)+1),1,AX441,410213.,1)# Encode        AX44  72
writeline:(1,AX441,NAX441,3,3)#        Print warning                    AX44  73
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX44  74
IF (TST(I)==3)TERM=yes:#               Exit                             AX44  75
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX44  76
LY=MY+stepy:#                          Skip constant part               AX44  77
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX44  78
incrx:(LX)#                            Next constraint                  AX44  79
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX44  80
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX44  81
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX44  82
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX44  83
#                                                                       AX44  84
# SHIFT DISPERSION CONSTRAINTS                                          AX44  85
# ----------------------------                                          AX44  86
#                                                                       AX44  87
IF (REFINE(6)==2)#                     Dispersion refined               AX44  88
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX44  89
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over atom types             AX44  90
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX44  91
DO I=1,2#                              Loop over re & im                AX44  92
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX44  93
ifnc:(LX,L6+I)#                        Item constraint                  AX44  94
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX44  95
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX44  96
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX44  97
LY=NINT(nctype:(LX))#                  Get constraint address           AX44  98
M=L6+I+mxwd8c:#                        Get subject address              AX44  99
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX44 100
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX44 101
REF=NINT(caddr:(LY))#                  Get reference pointer            AX44 102
QX(M)=QX(M)+cfact:(LY)*QX(REF)*DAMP#   Shift reference                  AX44 103
incry:(LY)#                            Next reference                   AX44 104
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX44 105
incry:(LY)#                            Next constraint                  AX44 106
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX44 107
incrx:(LX)#                            Take next constraint             AX44 108
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX44 109
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX44 110
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX44 111
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX44 112
RETURN#                                                                 AX44 113
END#                                                                    AX44 114
#                                                                       AX44 115
systemheader:(AX45)#                   Sum and decrepencies (ii)        AX45   1
####################################################################    AX45   2
#                                                                  #    AX45   3
#                               AX45                               #    AX45   4
#                                                                  #    AX45   5
####################################################################    AX45   6
SUBROUTINE AX45#                                                        AX45   7
include:(AACOMN)#                                                       AX45   8
include:(AXCOMN)#                                                       AX45   9
#                                                                       AX45  10
INTEGER REF#                           Reference param address          AX45  11
REAL DET#                              Det of ani temp fact             AX45  12
REAL EIGEN(3)#                         Displacement tensor eigenvalue   AX45  13
INTEGER I,J,K,I1,I2#                   Loop indices                     AX45  14
INTEGER L,M#                           Parameter pointers               AX45  15
INTEGER L6,L7#                         Atom type & atom pointers        AX45  16
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX45  17
INTEGERL20,M20#                        Auxiliary list pointers          AX45  18
INTEGER MXP#                           Local number of atomic paramet   AX45  19
REAL MINOR(3)#                         Diagonal minors of ani temp fa   AX45  20
INTEGER TFT#                           Temp fact type                   AX45  21
REAL U(3,3)#                           Uij tensor                       AX45  22
REAL V(3,3)#                           Passage matrix made of eigenve   AX45  23
REAL Q,P,S,T#                          Used in diagonalization of dis   AX45  24
#                                                                       AX45  25
chardata:(AX455," WARNING- Displacement tensor of atom XXXXXXXX$#       AX45  26
XXXXXXXXXXXXXXXX goes non definite positive")#                          AX45  27
chardata:(AX451,"                           Old                 $#      AX45  28
New               Shift")#                                              AX45  29
chardata:(AX454,[X  Y  Z  U  U11U22U33U12U13U23POPRE IM ])#             AX45  30
chardata:(AX453,"ConstrainedNot Refined")#                              AX45  31
chardata:(AX452,"       Face   Twin   Scale  Extinc.U ovl  ")#          AX45  32
chardata:(AX456,[" WARNING- xxxxxxx xx outside limits"])#               AX45  33
realdata:(FMT,[301272., 501272., 701272.])#                             AX45  34
datastuff:#                                                             AX45  35
#                                                                       AX45  36
# PRINT GENERAL PARAMETERS                                              AX45  37
# ------------------------                                              AX45  38
#                                                                       AX45  39
LX=MARKX;LY=MARKY;L20=MARK20#          Set pointers                     AX45  40
DO I=1,5#                              Loop over classes                AX45  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX45  42
IF (REFINE(I)==2)#                     Refined class                    AX45  43
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX45  44
writeline:(1,AX451,NAX451,3,3)#        Print title                      AX45  45
DO L=MARK(I),STOP(I)-STEP(I),STEP(I)#  Loop over all params             AX45  46
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX45  47
J=(L-MARK(I))/STEP(I)+1#               Get nb                           AX45  48
IF (I!=5)ncodefld:(REAL(J),1,CHROT,120213.,1)# Encode number            AX45  49
IF (L==MARK(I))movechar:(AX452,7*I+1,CHROT,2,7,0)# Move class name      AX45  50
ELSE movechar:(AX452,1,CHROT,2,12,1)#  Move blanks                      AX45  51
M=L+OFFSET(I)#                         Get actual parameter address     AX45  52
BUFOT(1)=QX(M)#                        Get old value                    AX45  53
J=3#                                   Default: 3 fields to print       AX45  54
ifnc:(LX,L)#                           Constrained param                AX45  55
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX45  56
J=1#                                   One field to print               AX45  57
movechar:(AX453,1,CHROT,60,11,0)#      Move constrained                 AX45  58
incrx:(LX)#                            Next constraint                  AX45  59
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX45  60
ELSE#                                  Parameter is variable            AX45  61
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX45  62
BUFOT(2)=QX(M)+DAMP*QX(L20)#           New value                        AX45  63
IF (TST(I)>0)#                         New value has to be tested       AX45  64
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX45  65
IF ((BUFOT(2)<=LBND(I))\(BUFOT(2)>=HBND(I)))# Outside limits            AX45  66
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX45  67
movechar:(AX452,7*I+1,AX456,11,7,0)#   Move class name                  AX45  68
ncodefld:(REAL(1+(L-MARK(I))/STEP(I)),1,AX456,200213.,1)# Encode nb     AX45  69
writeline:(1,AX456,NAX456,3,3)#        Print warning                    AX45  70
IF (TST(I)==3)TERM=yes:#               Set termination flag             AX45  71
IF (TST(I)==1)#                        Reset value                      AX45  72
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX45  73
IF (BUFOT(2)<=LBND(I))BUFOT(2)=LBND(I)+small:#                          AX45  74
ELSE BUFOT(2)=HBND(I)-small:#                                           AX45  75
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX45  76
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX45  77
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX45  78
QX(M)=BUFOT(2)#                        New value (ev modified)          AX45  79
L20=L20+1#                             Pointer on next variable         AX45  80
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX45  81
IF (I==5)#                             U overall                        AX45  82
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX45  83
DO M=1,3;BUFOT(M)=BUFOT(M)/utob:#      Transform to u                   AX45  84
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX45  85
BUFOT(3)=BUFOT(2)-BUFOT(1)#            Get difference                   AX45  86
ncodefld:(BUFOT,1,CHROT,FMT,J)#        Encode j value(s)                AX45  87
IF ((I!=4)\(J!=1))writeline:(0,AX451,NAX451,1,3)# Print line            AX45  88
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX45  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX45  90
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX45  91
#                                                                       AX45  92
# PRINT DISPERSION PARAMETERS                                           AX45  93
# ---------------------------                                           AX45  94
#                                                                       AX45  95
IF (REFINE(6)==2)#                     Refined class                    AX45  96
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX45  97
writeline:(1,AX451,NAX451,3,3)#        Print title                      AX45  98
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over all params             AX45  99
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX45 100
DO I=1,2#                              Loop over re & im                AX45 101
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX45 102
movechar:(AX454,3*(I+10)+1,CHROT,2,3,0)# Move re or im                  AX45 103
movertoc:(nattyp:(L6),1,CHROT,5,lens:,0)# Move atom type name           AX45 104
BUFOT(1)=QX(L6+I+mxwd8c:)#             Get old value                    AX45 105
J=3#                                   Default: 3 fields to print       AX45 106
ifnc:(LX,L6+I)#                        Constrained param                AX45 107
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX45 108
J=1#                                   One field to print               AX45 109
movechar:(AX453,1,CHROT,60,11,0)#      Move constrained                 AX45 110
incrx:(LX)#                            Next constraint                  AX45 111
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX45 112
ELSE#                                  Parameter is variable            AX45 113
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX45 114
QX(L6+I+mxwd8c:)=QX(L6+I+mxwd8c:)+DAMP*QX(L20)# Get new value           AX45 115
BUFOT(2)=QX(L6+I+mxwd8c:)#             New value to print               AX45 116
L20=L20+1#                             Pointer on next variable         AX45 117
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX45 118
BUFOT(3)=BUFOT(2)-BUFOT(1)#            Get difference                   AX45 119
ncodefld:(BUFOT,1,CHROT,FMT,J)#        Encode j value(s)                AX45 120
writeline:(0,AX451,NAX451,1,3)#        Print line                       AX45 121
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX45 122
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX45 123
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX45 124
#                                                                       AX45 125
# SHIFT ATOMIC PARAMETERS                                               AX45 126
# -----------------------                                               AX45 127
#                                                                       AX45 128
findnc:(MARKX,LX,MARK(7))#             Set nc list pointer              AX45 129
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over all params             AX45 130
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX45 131
ifnc:(LX,L7)#                                                           AX45 132
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX45 133
incrx:(LX);NEXT#                       Noref whole atom                 AX45 134
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX45 135
TFT=NINT(atomtft:(L7))#                Temp fact                        AX45 136
MXP=MAXPAR#                            Set to max nb of paramters       AX45 137
IF ((MXP<=9)&(TFT<2))#                                                  AX45 138
IF (TFT==0)MXP=3#                                                       AX45 139
ELSE MXP=4#                                                             AX45 140
movertoc:(atomn:(L7),1,AX451,2,lena:,0)# Move atom name                 AX45 141
writeline:(1,AX451,NAX451,3,3)#        Print title                      AX45 142
DO I=1,MXP#                            Loop over atomic parameters      AX45 143
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX45 144
IF ((TFT==0)&(I>3)&(I<10))NEXT#        No temp factors for this atom    AX45 145
IF ((TFT==1)&(I>4)&(I<10))NEXT#        No aniso temp factor             AX45 146
IF ((REFPOP==no:)&(I==10))NEXT#        Pop not refined                  AX45 147
IF ((I>4)\((I==4)&(TFT>1)))J=I+1#      Parameter counter                AX45 148
ELSE J=I#                                                               AX45 149
movechar:(AX454,3*(J-1)+1,CHROT,2,3,0)# Move param name                 AX45 150
IF ((I>3)&(I<10))#                                                      AX45 151
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX45 152
IF (TFT==1)  BUFOT(1)=100.0*QX(L7+I)/utob:# Print 100*u                 AX45 153
ELSE IF (I<7)BUFOT(1)=100.0*QX(L7+I)/CELL(3+I)# Print 100*uij           AX45 154
ELSE         BUFOT(1)=200.0*QX(L7+I)/CELL(3+I)#                         AX45 155
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX45 156
ELSE BUFOT(1)=QX(L7+I)#                                                 AX45 157
IF ((TFT==2)&(I==4))#                  Save pointers to reset temp fa   AX45 158
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX45 159
MX=LX;MY=LY;M20=L20#                                                    AX45 160
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX45 161
J=3#                                   Default: 3 fields to print       AX45 162
ifnc:(LX,L7+I)#                        Constrained param                AX45 163
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX45 164
IF (NINT(nctype:(LX))!=noref:)#        Constraint                       AX45 165
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX45 166
LY=NINT(nctype:(LX))#                  Get constraint address           AX45 167
WHILE (caddr:(LY)>small:)#             Loop over the constraint         AX45 168
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX45 169
REF=NINT(caddr:(LY))#                  Get reference param address      AX45 170
QX(L7+I)=QX(L7+I)+DAMP*cfact:(LY)*QX(REF)#                              AX45 171
incry:(LY)#                            Next reference param             AX45 172
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX45 173
incry:(LY)#                            Pointer on next constraint       AX45 174
IF ((I>3)&(I<10))#                     Print u & uij                    AX45 175
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX45 176
IF (TFT==1)  BUFOT(2)=100.0*QX(L7+I)/utob:# Transform to u              AX45 177
ELSE IF (I<7)BUFOT(2)=100.0*QX(L7+I)/CELL(3+I)# Print 100*uij           AX45 178
ELSE         BUFOT(2)=200.0*QX(L7+I)/CELL(3+I)#                         AX45 179
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX45 180
ELSE BUFOT(2)=QX(L7+I)#                                                 AX45 181
movechar:(AX453,1,CHROT,60,11,0)#      Move constrained                 AX45 182
J=2#                                                                    AX45 183
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX45 184
ELSE#                                                                   AX45 185
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX45 186
movechar:(AX453,12,CHROT,60,11,0)#     Move not refined                 AX45 187
J=1#                                                                    AX45 188
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX45 189
incrx:(LX)#                            Next constraint                  AX45 190
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX45 191
ELSE#                                  Parameter is variable            AX45 192
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX45 193
QX(L7+I)=QX(L7+I)+DAMP*QX(L20)#        Get new value                    AX45 194
IF ((I>3)&(I<10))#                     Print u & uij                    AX45 195
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX45 196
IF (TFT==1)  BUFOT(2)=100.0*QX(L7+I)/utob:# Transform to u              AX45 197
ELSE IF (I<7)BUFOT(2)=100.0*QX(L7+I)/CELL(3+I)# Print 100*uij           AX45 198
ELSE         BUFOT(2)=200.0*QX(L7+I)/CELL(3+I)#                         AX45 199
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX45 200
ELSE BUFOT(2)=QX(L7+I)#                                                 AX45 201
L20=L20+1#                             Pointer on next variable         AX45 202
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX45 203
BUFOT(3)=BUFOT(2)-BUFOT(1)#            Get difference                   AX45 204
ncodefld:(BUFOT,1,CHROT,FMT,J)#        Encode j value(s)                AX45 205
writeline:(0,AX451,NAX451,1,3)#        Print line                       AX45 206
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX45 207
movechar:(BLANK,1,AX451,2,lena:,1)#    Clear atom name                  AX45 208
IF ((TFT==1)&(atomiso:(L7)<=0.0))#     U iso not positive               AX45 209
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX45 210
movertoc:(atomn:(L7),1,AX455,39,lena:,0)# Print warning                 AX45 211
writeline:(0,AX455,NAX455,3,3)#                                         AX45 212
IF (TST(7)==1)atomiso:(L7)=small:#     Reset to almost 0                AX45 213
IF (TST(7)==3)TERM=yes:#               Stop                             AX45 214
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX45 215
IF (TFT==2)#                           Test positiveness of ani temp    AX45 216
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX45 217
U(1,1)=atomu11:(L7);U(2,2)=atomu22:(L7);U(3,3)=atomu33:(L7)#            AX45 218
U(1,2)=atomu12:(L7);U(1,3)=atomu13:(L7);U(2,3)=atomu23:(L7)#            AX45 219
U(2,1)=U(1,2);U(3,1)=U(1,3);U(3,2)=U(2,3)#                              AX45 220
DET=atomu11:(L7)*atomu22:(L7)*atomu33:(L7)+$#                           AX45 221
2.0*atomu12:(L7)*atomu23:(L7)*atomu13:(L7)-$#                           AX45 222
atomu11:(L7)*atomu23:(L7)**2-$#                                         AX45 223
atomu22:(L7)*atomu13:(L7)**2-$#                                         AX45 224
atomu33:(L7)*atomu12:(L7)**2#                                           AX45 225
T=0.0;S=0.0#                           Clear trace & minors             AX45 226
DO I=1,3#                              Loop over diag els               AX45 227
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX45 228
T=T+U(I,I)#                            Trace                            AX45 229
I1=1+MOD(I,3);I2=6-I-I1#                                                AX45 230
MINOR(I)=U(I1,I1)*U(I2,I2)-U(I1,I2)**2#                                 AX45 231
S=S+MINOR(I)#                                                           AX45 232
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX45 233
Q=(T**2-3.0*S)/9.0#                                                     AX45 234
P=(-2.0*T**3+9.0*T*S-27.0*DET)/54.0#                                    AX45 235
IF (P*P>=Q**3)#                        Cosine close to +/- 1            AX45 236
IF (P>0.0)S=0.0#                                                        AX45 237
ELSE S=pi:#                                                             AX45 238
ELSE S=ACOS(P/SQRT(Q**3))#                                              AX45 239
Q=-2.0*SQRT(AMAX1(Q,0.0))#                                              AX45 240
EIGEN(1)=Q*COS(S/3.0)+T/3.0;EIGEN(2)=Q*COS((S+2.0*pi:)/3.0)+T/3.0#      AX45 241
EIGEN(3)=Q*COS((S+4.0*pi:)/3.0)+T/3.0#                                  AX45 242
IF (AMIN1(EIGEN(1),EIGEN(2),EIGEN(3))<=0.0)# Not pos-def uij            AX45 243
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX45 244
movertoc:(atomn:(L7),1,AX455,39,lena:,0)# Print warning                 AX45 245
writeline:(0,AX455,NAX455,3,3)#                                         AX45 246
IF (TST(7)==1)#                        Reset uij                        AX45 247
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX45 248
ifeq:(Q,0.0)DO I=1,3;atomdis:(L7+I)=small:# Tensor is scalar            AX45 249
ELSE#                                  Tensor not scalar                AX45 250
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX45 251
DO I=1,2#                              Loop over first eigenvalues      AX45 252
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX45 253
ifeq:(EIGEN(I),EIGEN(3))#              Degenerate eigenvalue ?          AX45 254
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX45 255
DET=0.0#                               Clear 1*1 determinant            AX45 256
DO J=1,3;IF (ABS(U(J,J)-EIGEN(I))>ABS(DET))# Find greatest minor        AX45 257
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX45 258
DET=U(J,J)-EIGEN(I);K=J;I1=1+MOD(J,3);I2=6-J-I1#                        AX45 259
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX45 260
V(I,I1)=1.0;V(I,I2)=1.0#               Eigenvector of degenerate eige   AX45 261
V(I,K)=-(U(K,I1)+U(K,I2))/DET#                                          AX45 262
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX45 263
ELSE#                                  Eigenvalue not degenerate        AX45 264
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX45 265
DET=0.0#                               Clear 2*2 determinant            AX45 266
DO J=1,3#                              Find greatest minor              AX45 267
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 37 AX45 268
P=EIGEN(I)**2-(T-U(J,J))*EIGEN(I)+MINOR(J)#                             AX45 269
IF (ABS(P)>ABS(DET))#                                                   AX45 270
$(#                                    8>>>>>>>>>>>>>>>>>>>>>>>>>>>> 38 AX45 271
DET=P;K=J;I1=1+MOD(J,3);I2=6-J-I1#                                      AX45 272
$)#                                    8<<<<<<<<<<<<<<<<<<<<<<<<<<<< 38 AX45 273
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 37 AX45 274
V(I,I1)=((EIGEN(I)-U(I2,I2))*U(I1,K)+U(I1,I2)*U(I2,K))/DET#             AX45 275
V(I,I2)=((EIGEN(I)-U(I1,I1))*U(I2,K)+U(I1,I2)*U(I1,K))/DET#             AX45 276
V(I,K)=1.0#                            Eigenvector of not degenerate    AX45 277
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX45 278
S=SQRT(V(I,I1)**2+V(I,I2)**2+V(I,K)**2)# Normalized eigenvector         AX45 279
V(I,1)=V(I,1)/S;V(I,2)=V(I,2)/S;V(I,3)=V(I,3)/S#                        AX45 280
IF (EIGEN(I)<small:)EIGEN(I)=small:#   Reset eigenvalue                 AX45 281
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX45 282
V(3,1)=V(1,2)*V(2,3)-V(1,3)*V(2,2)#    Cross product                    AX45 283
V(3,2)=V(1,3)*V(2,1)-V(1,1)*V(2,3)#                                     AX45 284
V(3,3)=V(1,1)*V(2,2)-V(1,2)*V(2,1)#                                     AX45 285
IF (EIGEN(3)<small:)EIGEN(3)=small:#   Reset eigenvalue                 AX45 286
DO I=1,3;DO J=1,3#                     Build new tensor                 AX45 287
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 39 AX45 288
U(I,J)=0.0#                                                             AX45 289
DO K=1,3;U(I,J)=U(I,J)+V(K,I)*V(K,J)*EIGEN(K)#                          AX45 290
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 39 AX45 291
atomu11:(L7)=U(1,1);atomu22:(L7)=U(2,2);atomu33:(L7)=U(3,3)#            AX45 292
atomu12:(L7)=U(1,2);atomu13:(L7)=U(1,3);atomu23:(L7)=U(2,3)#            AX45 293
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX45 294
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX45 295
IF (TST(7)==3)TERM=yes:#               Set termination flag             AX45 296
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX45 297
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX45 298
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX45 299
RETURN#                                                                 AX45 300
END#                                                                    AX45 301
#                                                                       AX45 302
systemheader:(AX47)#                   Print matrices                   AX47   1
####################################################################    AX47   2
#                                                                  #    AX47   3
#                               AX47                               #    AX47   4
#                                                                  #    AX47   5
####################################################################    AX47   6
SUBROUTINE AX47(ICYCLE,IMARK,ROW,INV)#                                  AX47   7
include:(AACOMN)#                                                       AX47   8
include:(AXCOMN)#                                                       AX47   9
#                                                                       AX47  10
INTEGER ICYCLE#                        Local cycle nb                   AX47  11
INTEGER I,L,II,IJ#                                                      AX47  12
INTEGER IMARK#                         Matrix pointer                   AX47  13
INTEGER INV#                           Direct/inverse flag              AX47  14
INTEGER ROW#                           Number of rows in matrix         AX47  15
#                                                                       AX47  16
chardata:(AX471,[" Cycle xx, Direct  Variance-Covariance Matrix"])#     AX47  17
chardata:(AX472,[" Cycle xx, Inverse                           "])#     AX47  18
chardata:(AX473,["Normal-Equation Matrix    Vector                    $#AX47  19
Variance-Covariance Matrix"])#                                          AX47  20
chardata:(AX474," Row XXXX")#                                           AX47  21
realdata:(FORM,[101051., 211051., 321051., 431051., 541051., 651051., $#AX47  22
761051., 871051., 981051., 1091051.])#                                  AX47  23
datastuff:#                                                             AX47  24
#                                                                       AX47  25
IF (INV<0)#                                                             AX47  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX47  27
IF (IABS(INV)==1)movechar:(AX473,1,AX471,20,26,0)#                      AX47  28
IF (IABS(INV)==2)movechar:(AX473,27,AX471,20,26,0)#                     AX47  29
IF (IABS(INV)==3)movechar:(AX473,53,AX471,20,26,0)#                     AX47  30
ncodefld:(REAL(ICYCLE),1,AX471,90213.,1)# Code cycle nb                 AX47  31
writeline:(1,AX471,NAX471,3,3)#        Print title                      AX47  32
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX47  33
ELSE#                                                                   AX47  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX47  35
IF (INV==1)movechar:(AX473,1,AX472,20,26,0)#                            AX47  36
IF (INV==2)movechar:(AX473,27,AX472,20,26,0)#                           AX47  37
IF (INV==3)movechar:(AX473,53,AX472,20,26,0)#                           AX47  38
ncodefld:(REAL(ICYCLE),1,AX472,90213.,1)# Code cycle nb                 AX47  39
writeline:(1,AX472,NAX472,3,3)#        Print title                      AX47  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX47  41
IJ=IMARK#                                                               AX47  42
DO II=1,ROW#                           Loop over rows                   AX47  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX47  44
I=VARTOT+1-II#                         Nb of items to print             AX47  45
IF (IABS(INV)!=2)LINRM=2#              Matrix                           AX47  46
REPEAT#                                Loop over printing lines         AX47  47
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX47  48
L=MIN0(10,I)#                          Nb of items to print             AX47  49
ncodefld:(QX,IJ,CHROT,FORM,L)#         Encode max 12 numbers            AX47  50
IF (IABS(INV)!=2)#                                                      AX47  51
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX47  52
ncodefld:(REAL(II),1,AX474,90413.,1)#  Encode row                       AX47  53
writeline:(0,AX474,NAX474,1,3)#        Print row                        AX47  54
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX47  55
ELSE writeline:(0,0,0,1,3)#            Print vector                     AX47  56
I=I-L;IJ=IJ+L;IF (I<=0)BREAK#          Exit on last                     AX47  57
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX47  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX47  59
RETURN#                                                                 AX47  60
END#                                                                    AX47  61
#                                                                       AX47  62
#                                                                       AX47  63
systemheader:(AX48)#                   G0 & g1 functions                AX48   1
####################################################################    AX48   2
#                                                                  #    AX48   3
#                               AX48                               #    AX48   4
#                                                                  #    AX48   5
####################################################################    AX48   6
SUBROUTINE AX48(X,G0,DG0,G1,DG1)#                                       AX48   7
include:(AACOMN)#                                                       AX48   8
include:(AXCOMN)#                                                       AX48   9
#                                                                       AX48  10
INTEGER LT#                            Temporary list                   AX48  11
INTEGER N#                             Number of items in list          AX48  12
REAL X#                                Input variable                   AX48  13
REAL G0,DG0,G1,DG1#                    Returned variables               AX48  14
#                                                                       AX48  15
G0=0.0;DG0=0.0;G1=0.0;DG1=0.0#         Default values                   AX48  16
IF (X<=1.0)RETURN#                                                      AX48  17
N=STOPT-MARKT+1#                       Count item                       AX48  18
DO LT=MARKT,STOPT#                     Loop over list                   AX48  19
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX48  20
G0=G0+SQRT(1.0-QX(LT)/X)#                                               AX48  21
DG0=DG0+QX(LT)/(2.0*X**2*SQRT(1.0-QX(LT)/X))#                           AX48  22
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX48  23
G1=G0-2.0*SQRT(1.0-1.0/X)#                                              AX48  24
DG1=DG0-1.0/(X**2*SQRT(1.0-1.0/X))#                                     AX48  25
G0=G0/REAL(N);DG0=DG0/REAL(N)#                                          AX48  26
G1=G1/REAL(N);DG1=DG1/REAL(N)#                                          AX48  27
RETURN#                                                                 AX48  28
END#                                                                    AX48  29
#                                                                       AX48  30
systemheader:(AX49)#                   Matrix inversion                 AX49   1
####################################################################    AX49   2
#                                                                  #    AX49   3
#                               AX49                               #    AX49   4
#                                                                  #    AX49   5
####################################################################    AX49   6
SUBROUTINE AX49(IMARK,N)#                                               AX49   7
include:(AACOMN)#                                                       AX49   8
include:(AXCOMN)#                                                       AX49   9
#                                                                       AX49  10
INTEGER BLKLEN#                        Total length of block            AX49  11
INTEGER I,J,K#                         Loop index                       AX49  12
INTEGER JJ,KK,JK#                      Diag & non-diag ptrs             AX49  13
INTEGER IMARK#                         Start of matrix storage          AX49  14
INTEGER N#                             Nb of var in block               AX49  15
REAL U,T,S#                            Temp usage                       AX49  16
#                                                                       AX49  17
chardata:(AX491,[" Matrix ill-defined, accuracy of inverse very low"])# AX49  18
chardata:(AX492,[" Matrix not positive-defined, diag element reset $#   AX49  19
to 0"])#                                                                AX49  20
datastuff:#                                                             AX49  21
#                                                                       AX49  22
BLKLEN=(N*(N+1))/2#                    Block length                     AX49  23
#                                                                       AX49  24
# NORM OF DIRECT MATRIX                                                 AX49  25
# ---------------------                                                 AX49  26
#                                                                       AX49  27
JK=IMARK#                                                               AX49  28
DO J=MARK20,STOP20#                                                     AX49  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX49  30
QX(J)=ABS(QX(JK));JK=JK+1#                                              AX49  31
DO K=MARK20,J-1#                                                        AX49  32
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX49  33
QX(J)=QX(J)+ABS(QX(JK));QX(K)=QX(K)+ABS(QX(JK))#                        AX49  34
JK=JK+1#                                                                AX49  35
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX49  36
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX49  37
S=0.0;DO J=MARK20,STOP20;S=AMAX1(S,QX(J))#                              AX49  38
#                                                                       AX49  39
# TRIANGULARISATION                                                     AX49  40
# -----------------                                                     AX49  41
#                                                                       AX49  42
JJ=IMARK+BLKLEN-1#                     Point to last el                 AX49  43
DO J=N,1,-1#                           Loop over rows                   AX49  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX49  45
S=0.0#                                 Clear sum                        AX49  46
JK=JJ+N-J#                             Point to last el of row k        AX49  47
KK=IMARK+BLKLEN-1#                     Point to last el of block        AX49  48
DO K=N,J+1,-1#                         Loop over columns                AX49  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX49  50
U=0.0#                                 Clear sum                        AX49  51
DO I=N,K+1,-1;U=U+QX(KK+I-K)*QX(JJ+I-J)# Row(k).row(j)                  AX49  52
T=QX(JK)-U#                                                             AX49  53
T=T/QX(KK)#                                                             AX49  54
KK=KK-(N-K)-2#                         Point to next diag el            AX49  55
QX(JK)=T#                                                               AX49  56
JK=JK-1#                               Point to next el on same row     AX49  57
S=S+T*T#                                                                AX49  58
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX49  59
S=QX(JJ)-S#                                                             AX49  60
IF (S<=0.0)iquit:(AX4901)#             Inversion impossible             AX49  61
QX(JJ)=SQRT(S)#                                                         AX49  62
JJ=JJ-(N-J)-2#                         Take next diag el                AX49  63
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX49  64
#                                                                       AX49  65
# TRIANGULAR MATRIX INVERSION                                           AX49  66
# ---------------------------                                           AX49  67
#                                                                       AX49  68
JJ=IMARK+BLKLEN-1#                     Point to last el of block        AX49  69
DO J=N,1,-1#                           Loop over rows                   AX49  70
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX49  71
QX(JJ)=1.0/QX(JJ)#                     Invert diag el                   AX49  72
T=-QX(JJ)#                                                              AX49  73
DO I=N,J+1,-1;QX(JJ+I-J)=QX(JJ+I-J)*T# Row(j) = row(j)/el(jj)           AX49  74
KK=JJ#                                 Point to diag el                 AX49  75
JK=JJ#                                 Point to diag el                 AX49  76
DO K=J-1,1,-1#                         Loop over columns                AX49  77
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX49  78
JK=JK-(N-K)#                           Point to upper row               AX49  79
KK=KK-(N-K)-1#                         Next diag el                     AX49  80
T=QX(JK)#                                                               AX49  81
QX(JK)=0.0#                                                             AX49  82
DO I=N,J,-1;QX(KK+I-K)=QX(KK+I-K)+T*QX(JJ+I-J)# Row(k) = row(k)+el(kk   AX49  83
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX49  84
JJ=JJ-(N-J)-2#                         Next diag el                     AX49  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX49  86
#                                                                       AX49  87
# INVERSE RECONSTRUCTION                                                AX49  88
# ----------------------                                                AX49  89
#                                                                       AX49  90
JJ=IMARK+BLKLEN-1#                     Point to last el of block        AX49  91
DO J=N,1,-1#                           Loop over rows                   AX49  92
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX49  93
KK=IMARK+BLKLEN-1#                     Point to last block el           AX49  94
JK=JJ+(N-J)#                           Point to last el of row j        AX49  95
DO K=N,J+1,-1#                         Loop over columns                AX49  96
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX49  97
T=QX(JK)#                                                               AX49  98
DO I=N,K,-1;QX(KK+I-K)=QX(KK+I-K)+T*QX(JJ+I-J)#                         AX49  99
KK=KK-(N-K)-2#                         Take next diag el                AX49 100
JK=JK-1#                               Take next el on row j            AX49 101
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX49 102
T=QX(JJ)#                                                               AX49 103
DO I=N,J,-1;QX(JJ+I-J)=QX(JJ+I-J)*T#                                    AX49 104
JJ=JJ-(N-J)-2#                         Take next diag el                AX49 105
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX49 106
#                                                                       AX49 107
# NORM OF INVERSE MATRIX                                                AX49 108
# ----------------------                                                AX49 109
#                                                                       AX49 110
JK=IMARK#                                                               AX49 111
DO J=MARK20,STOP20#                                                     AX49 112
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX49 113
QX(J)=ABS(QX(JK));JK=JK+1#                                              AX49 114
DO K=MARK20,J-1#                                                        AX49 115
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX49 116
QX(J)=QX(J)+ABS(QX(JK));QX(K)=QX(K)+ABS(QX(JK))#                        AX49 117
JK=JK+1#                                                                AX49 118
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX49 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX49 120
T=0.0;DO J=MARK20,STOP20;T=AMAX1(T,QX(J))#                              AX49 121
#                                                                       AX49 122
# GET PRECISION FACTOR                                                  AX49 123
# --------------------                                                  AX49 124
#                                                                       AX49 125
S=1.0/(T*S)#                                                            AX49 126
IF (1.0==1.0+S)writeline:(1,AX491,NAX491,3,3)# Print warning            AX49 127
RETURN#                                                                 AX49 128
END#                                                                    AX49 129
#                                                                       AX49 130
systemheader:(AX50)#                   Write bdf & close shop           AX50   1
####################################################################    AX50   2
#                                                                  #    AX50   3
#                               AX50                               #    AX50   4
#                                                                  #    AX50   5
####################################################################    AX50   6
SUBROUTINE AX50#                                                        AX50   7
include:(AACOMN)#                                                       AX50   8
include:(AXCOMN)#                                                       AX50   9
#                                                                       AX50  10
CALL AX51#                             Replace address in nc list       AX50  11
CALL AX52#                             Write refined params             AX50  12
CALL AX53#                             Write atomic parameters          AX50  13
IF ((REFINE(2)==0)\(NB(2)==1))CALL AX54# Write lrrefl:                  AX50  14
ELSE copyfile:(1,2,incr:(lratom:),decr:(lrsfls:))#                      AX50  15
CALL AX55#                             Write lrsfls: & restraints       AX50  16
IF (SAVEMA==yes:)CALL AX56#            Write variance-covariance matr   AX50  17
RETURN#                                                                 AX50  18
END#                                                                    AX50  19
#                                                                       AX50  20
systemheader:(AX51)#                   Transform constraint list        AX51   1
####################################################################    AX51   2
#                                                                  #    AX51   3
#                               AX51                               #    AX51   4
#                                                                  #    AX51   5
####################################################################    AX51   6
SUBROUTINE AX51#                                                        AX51   7
include:(AACOMN)#                                                       AX51   8
include:(AXCOMN)#                                                       AX51   9
#                                                                       AX51  10
INTEGER I#                             Loop indexes                     AX51  11
INTEGER L#                             Parameter address                AX51  12
INTEGER L21#                           Vector pointer                   AX51  13
INTEGER L6,L7#                         Atom type & atom pointers        AX51  14
INTEGER LX#                            Constraint list pointer          AX51  15
INTEGER MXP#                           Local number of atomic paramet   AX51  16
INTEGER TFT#                           Displ fact type                  AX51  17
#                                                                       AX51  18
# PUT ADDRESS OF VARIABLE IN MARK21 LIST                                AX51  19
# --------------------------------------                                AX51  20
#                                                                       AX51  21
L21=MARK21;LX=MARKX#                   Set pointers                     AX51  22
DO I=1,5#                              Loop over classes                AX51  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX51  24
IF (REFINE(I)>1)#                      Class to be refiend              AX51  25
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX51  26
DO L=MARK(I),STOP(I)-STEP(I),STEP(I)#  Loop over one class              AX51  27
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX51  28
ifnc:(LX,L)incrx:(LX)#                 Constrained                      AX51  29
ELSE#                                                                   AX51  30
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX51  31
QX(L21)=REAL(L);L21=L21+1#             Save var address                 AX51  32
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX51  33
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX51  34
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX51  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX51  36
IF (REFINE(6)>1)#                      Dispersion                       AX51  37
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX51  38
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over atom types             AX51  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX51  40
DO I=1,2#                              Loop over re, im                 AX51  41
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX51  42
ifnc:(LX,L6+I)incrx:(LX)#              Constrained                      AX51  43
ELSE#                                                                   AX51  44
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX51  45
QX(L21)=REAL(L6+I);L21=L21+1#          Save address                     AX51  46
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX51  47
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX51  48
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX51  49
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX51  50
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atom types             AX51  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX51  52
ifnc:(LX,L7)#                          Atom not refined                 AX51  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX51  54
incrx:(LX);NEXT#                       Take next atom                   AX51  55
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX51  56
TFT=NINT(atomtft:(L7))#                Extract temp fact type           AX51  57
MXP=MAXPAR#                            Set to max nb of paramters       AX51  58
IF ((MXP<=9)&(TFT<2))#                                                  AX51  59
IF (TFT==0)MXP=3#                                                       AX51  60
ELSE MXP=4#                                                             AX51  61
DO I=1,MXP#                            Loop over atomic parameters      AX51  62
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX51  63
IF ((TFT==0)&(I>3)&(I<10))NEXT#        Uov                              AX51  64
IF ((TFT==1)&(I>4)&(I<10))NEXT#        U iso                            AX51  65
IF ((REFPOP==no:)&(I==10))NEXT#        Pop                              AX51  66
ifnc:(LX,L7+I)incrx:(LX)#              Constrained                      AX51  67
ELSE#                                                                   AX51  68
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX51  69
QX(L21)=REAL(L7+I);L21=L21+1#          Save address                     AX51  70
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX51  71
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX51  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX51  73
MAXSOE=0.0;SOEAV=0.0#                  Clear shift/error values         AX51  74
RETURN#                                                                 AX51  75
END#                                                                    AX51  76
#                                                                       AX51  77
systemheader:(AX52)#                   Write output bdf                 AX52   1
####################################################################    AX52   2
#                                                                  #    AX52   3
#                               AX52                               #    AX52   4
#                                                                  #    AX52   5
####################################################################    AX52   6
SUBROUTINE AX52#                                                        AX52   7
include:(AACOMN)#                                                       AX52   8
include:(AXCOMN)#                                                       AX52   9
#                                                                       AX52  10
INTEGER I,J,K,M,N#                     Loop index                       AX52  11
INTEGER LB,LC,L1,L2,M2,L3,L6#          List pointers                    AX52  12
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX52  13
INTEGER OPACK#                         Output packet length             AX52  14
INTEGER REFF#                          F refined flag (neutrons)        AX52  15
REAL SIGMA#                            Pour l'instant                   AX52  16
#                                                                       AX52  17
chardata:(AX521," Final Parameters [(I=]Invariant $#                    AX52  18
C[=]Constrained[)]")#                                                   AX52  19
chardata:(AX522," Crystal Faces    h   k   l   Obs Dist        ESD$#    AX52  20
     Weight  Calc Dist   Calc ESD")#                                    AX52  21
chardata:(AX523," Scale Factors")#                                      AX52  22
chardata:(AX524," Uov*100  xx.xxxx  xx.xxxx")#                          AX52  23
chardata:(AX525," Dispersion  f prime             f second")#           AX52  24
chardata:(AX526,[" Twinning Parameters                Centrosymmetry"])#AX52  25
chardata:(AX528," ExtinctionGaussianLorentzianAnisotropicIsotropic$#    AX52  26
Type12GeneralTensor")#                                                  AX52  27
chardata:(AX529," g (/10000)   [= XXXXX.XXXX (XXXXX.XXXX)           ]$# AX52  28
 r mm (/10000)[ =XXXXX.XXXX (XXXXX.XXXX)           ]")#                 AX52  29
chardata:(AX52A,["SCALE  XX.XXXX  XX"])#                                AX52  30
realdata:(FM0,[190313.,230313.,270313.,380962., 490962., 600952., $#    AX52  31
710962., 820962.])#                                                     AX52  32
realdata:(FM1,[100642., 180642., 280642., 360642., 460642., 540642., $# AX52  33
640642., 720642., 820642., 900642., 1000642., 1080642.])#               AX52  34
realdata:(FM2,[100642., 200642., 400642., 500642.])#                    AX52  35
realdata:(FM3,[261042., 381042., 180632., 260632., 340632., 420632., $# AX52  36
500632., 580632., 660632., 740632., 820632., 900632., 980632., $#       AX52  37
1060632.])#                                                             AX52  38
realdata:(FM4,[170742., 260742.])#                                      AX52  39
realdata:(FM5,[200842., 280642., 400842., 480642.])#                    AX52  40
datastuff:#                                                             AX52  41
#                                                                       AX52  42
# COPY FILE UP TO LRCELL:                                               AX52  43
# -----------------------                                               AX52  44
#                                                                       AX52  45
rewindbdf:(1)#                         Beginning of files               AX52  46
copyfile:(1,2,1,decr:(lrcell:))#       Do nothing until lrcell:         AX52  47
I=0#                                   Set pointer                      AX52  48
REPEAT#                                Loop over lrcell: packets        AX52  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX52  50
readwpkt:(1,lrcell:,IPACK,IP,0)#       Read one packet                  AX52  51
IF (IP<=0)BREAK#                       Exit if last                     AX52  52
I=I+1#                                 Increment packet counter         AX52  53
writepkt:(2,lrcell:,IPACK,IPP)#        Write this packet                AX52  54
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Move words                       AX52  55
IF (I!=6)NEXT#                         No change exept packet 6         AX52  56
QX(IPP+4)=QX(IPP+4)+REAL(NCYCLE)#      Save cycle nb                    AX52  57
QX(IPP+5)=GOOF#                        Save goodness-of-fit             AX52  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX52  59
copyfile:(1,2,incr:(lrcell:),decr:(lrtwin:))# Do nothing until lrtwin   AX52  60
writeline:(1,AX521,NAX521,3,3)#        Print 2 blank lines              AX52  61
#                                                                       AX52  62
# PRINT GENERAL PARAMETERS IN BDF ORDER                                 AX52  63
# -------------------------------------                                 AX52  64
#                                                                       AX52  65
IF (REFINE(2)>1)#                      Twinning                         AX52  66
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX52  67
writeline:(1,0,0,3,3)#                 Print blank line                 AX52  68
LINRM=2#                               Reserve 2 lines                  AX52  69
IF (ICENT==no:)NB(2)=NB(2)/2#          Count matrices                   AX52  70
OPACK=13;LB=MARKB;L2=MARK(2)#          Set pointers                     AX52  71
DO I=1,NB(2)#                          Loop over twinning matrices      AX52  72
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX52  73
writepkt:(2,lrtwin:,OPACK,IPP)#        Write packet                     AX52  74
movereal:(QX,LB+1,QX,IPP+1,OPACK,0)#   Copy packets                     AX52  75
CALL AX59(L2,SIGMA)#                   Get esd                          AX52  76
QX(IPP+10)=entpol:(I);QX(IPP+11)=SIGMA# Save merohedry                  AX52  77
findnc:(MARKX,LX,L2)#                  Get constraint pointer           AX52  78
ifnc:(LX,L2)#                          Constraint or not refined        AX52  79
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX52  80
IF (NINT(nctype:(LX))==noref:)N=20#    Not refined                      AX52  81
ELSE N=32#                             Constraint                       AX52  82
incrx:(LX)#                                                             AX52  83
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX52  84
ELSE N=1#                                                               AX52  85
BUFOT(1)=entpol:(I);BUFOT(2)=SIGMA#    Prepare for output               AX52  86
K=NINT(FM2(1))/10000+1#                Get status column                AX52  87
movechar:(AX521,N,CHROT,K,1,0)#        Move status                      AX52  88
IF (ICENT==no:)#                       Not centrosymmetric group        AX52  89
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX52  90
M2=L2+NB(2)#                                                            AX52  91
CALL AX59(M2,SIGMA)#                   Get esd                          AX52  92
QX(IPP+12)=entpol:(NB(2)+I);QX(IPP+13)=SIGMA# Save merohedry            AX52  93
findnc:(MARKX,LX,M2)#                  Get constraint pointer           AX52  94
ifnc:(LX,M2)#                          Constraint or not refined        AX52  95
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX52  96
IF (NINT(nctype:(LX))==noref:)N=20#    Not refined                      AX52  97
ELSE N=32#                             Constraint                       AX52  98
incrx:(LX)#                                                             AX52  99
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX52 100
ELSE N=1#                                                               AX52 101
BUFOT(3)=entpol:(NB(2)+I);BUFOT(4)=SIGMA# Prepare for output            AX52 102
IF(I==1&NB(2)==1)$(XABS=BUFOT(3);SUXABS=SIGMA$)# Save xabs first frag   AX52 103
K=NINT(FM2(3))/10000+1#                Get status column                AX52 104
movechar:(AX521,N,CHROT,K,1,0)#        Move status                      AX52 105
J=NAX526;K=4#                          4 fields to print                AX52 106
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX52 107
ELSE#                                                                   AX52 108
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX52 109
QX(IPP+12)=0.0;QX(IPP+13)=0.0#         Clear centrosymmetric part       AX52 110
J=20;K=2#                              2 fields to print                AX52 111
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX52 112
ncodefld:(BUFOT,1,CHROT,FM2,K)#        Encode fields                    AX52 113
writeline:(0,AX526,J,1,3)#             Print line                       AX52 114
incrb:(LB);incr2:(L2)#                 Next merohedry                   AX52 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX52 116
copyfile:(1,2,incr:(lrtwin:),decr:(lrscat:))# Copy bdf                  AX52 117
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX52 118
ELSE copyfile:(1,2,lrtwin:,decr:(lrscat:))# Copy bdf                    AX52 119
IF (REFINE(6)>1)#                      Dispersion refined               AX52 120
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX52 121
IWNT(1)=20+DATSET;IWNT(2)=60+DATSET#   Real % imag scat fact            AX52 122
SIGNAL(1)=0;SIGNAL(2)=0;SIGNAL(3)=2;SIGNAL(4)=0# No mandatory           AX52 123
indexpkt:(1,lrscat:,IPACK,IP,2,SIGNAL,IWNT,IREL)# Get directory         AX52 124
IF (IP<=0)iquit:(AX5201)#              Lrscat: is missing               AX52 125
writeline:(1,AX525,NAX525,3,3)#        Print title                      AX52 126
findnc:(MARKX,LX,MARK(6))#             Set pointers for dispersion      AX52 127
L6=MARK(6)#                            Set pointer                      AX52 128
IF (INEU==1)REFF=yes:#                 F can be refined                 AX52 129
ELSE        REFF=no:#                  F cannot be refined              AX52 130
REPEAT#                                Loop over lrscat: packets        AX52 131
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX52 132
readwpkt:(1,lrscat:,IPACK,IP,0)#       Read one packet                  AX52 133
IF (IP<=0)BREAK#                       Exit if last                     AX52 134
IF (L6>=STOP(6))iquit:(AX5203)#        Too many scattering factors      AX52 135
writepkt:(2,lrscat:,IPACK,IPP)#        Write this packet                AX52 136
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Move words                       AX52 137
movertoc:(nattyp:(L6),1,CHROT,2,lens:,0)# Insert ff name                AX52 138
DO I=1,2#                              Loop over re/im                  AX52 139
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX52 140
QX(IPP+IREL(I))=QX(L6+I+mxwd8c:)#      Save re/im part                  AX52 141
CALL AX59(L6+I,SIGMA)#                 Get esd                          AX52 142
ifnc:(LX,L6+I)#                        Param not variable               AX52 143
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX52 144
IF ((I==2)&(ABS(QX(L6+I+mxwd8c:))>small:))REFF=no:# Im not 0            AX52 145
IF (NINT(nctype:(LX))==noref:)N=20#    Not refined                      AX52 146
ELSE N=32#                             Constrained                      AX52 147
incrx:(LX)#                                                             AX52 148
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX52 149
ELSE#                                  Variable                         AX52 150
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX52 151
N=1#                                   No marker                        AX52 152
IF (I==2)REFF=no:#                     Im was free                      AX52 153
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX52 154
J=NINT(FM5(2*I-1))/10000+1#            Get destination column           AX52 155
movechar:(AX521,N,CHROT,J,1,0)#        Code                             AX52 156
BUFOT(2*I-1)=QX(L6+I+mxwd8c:);BUFOT(2*I)=SIGMA#                         AX52 157
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX52 158
ncodefld:(BUFOT,1,CHROT,FM5,4)#        Format dispersion values         AX52 159
writeline:(0,AX525,NAX525,1,3)#        Print line                       AX52 160
incr6:(L6)#                            Take next atom type              AX52 161
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX52 162
copyfile:(1,2,incr:(lrscat:),decr:(lrexpl:))# Copy up to lrexpl:        AX52 163
IF (REFF==yes:)#                       F was refined instead of f'      AX52 164
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX52 165
LC=MARKC#                              Set scat length pointer          AX52 166
DO L6=MARK(6),STOP(6)-STEP(6),STEP(6)# Loop over sf types               AX52 167
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX52 168
QX(LC+1)=QX(LC+1)+redisp:(L6)#         Replace f by f+f'                AX52 169
redisp:(L6)=0.0#                       Clear f'                         AX52 170
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX52 171
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX52 172
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX52 173
ELSE copyfile:(1,2,lrscat:,decr:(lrexpl:))# Copy up to lrexpl:          AX52 174
#                                                                       AX52 175
IWNT(1)=1;I=2#                         Dataset nb                       AX52 176
DO L3=MARK(3),STOP(3)-STEP(3),STEP(3)# Loop over scales                 AX52 177
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX52 178
IWNT(I)=100+NINT(sclgrp:(L3))#         Scale group                      AX52 179
I=I+1#                                 Next item                        AX52 180
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX52 181
SIGNAL(1)=0;SIGNAL(2)=1;SIGNAL(3)=I-1;SIGNAL(4)=0#                      AX52 182
indexpkt:(1,lrexpl:,IPACK,IP,2,SIGNAL,IWNT,IREL)# Get directory         AX52 183
IF (IP<=0)iquit:(AX5204)#              Lrexpl:is missing                AX52 184
REPEAT#                                Loop over lrexpl: packets        AX52 185
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX52 186
readwpkt:(1,lrexpl:,IPACK,IP,0)#       Read one packet                  AX52 187
IF (IP<=0)BREAK#                       Exit if last                     AX52 188
writepkt:(2,lrexpl:,IPACK,IPP)#        Write this packet                AX52 189
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Move words                       AX52 190
IF (IREL(1)!=0)#                       Get dataset nb                   AX52 191
IF (NINT(QX(IPP+IREL(1)))!=DATSET)NEXT# Not the right one               AX52 192
I=1#                                   Set counter                      AX52 193
findnc:(MARKX,LX,MARK(3))#             Find pointers                    AX52 194
writeline:(1,0,0,3,3)#                 Print blank line                 AX52 195
LINRM=2#                               Reserve 2 lines                  AX52 196
DO L3=MARK(3),STOP(3)-STEP(3),STEP(3)# Loop over scales                 AX52 197
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX52 198
IF (IREL(I+1)<=0)iquit:(AX5205)#       Something happened in scales     AX52 199
QX(IPP+IREL(I+1))=1.0/SQRT(AMAX1(small:,sclfac:(L3)))# Save scale       AX52 200
IF (PNCHCD>0)#                         Puch scale cards                 AX52 201
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX52 202
ncodefld:(QX(IPP+IREL(I+1)),1,AX52A,140751.,1)# Encode scale            AX52 203
ncodefld:(sclgrp:(L3),1,AX52A,180213.,1)# Encode group                  AX52 204
writepch:(AX52A,NAX52A)#               Print line                       AX52 205
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX52 206
ifnc:(LX,L3)#                                                           AX52 207
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX52 208
IF (NINT(nctype:(LX))==noref:)N=20#                                     AX52 209
ELSE N=32#                                                              AX52 210
incrx:(LX)#                                                             AX52 211
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX52 212
ELSE N=1#                                                               AX52 213
CALL AX59(L3,SIGMA)#                   Get esd of scale                 AX52 214
SIGMA=SIGMA/(2.0*SQRT(AMAX1(small:,sclfac:(L3)))**3)#                   AX52 215
BUFOT(1)=QX(IPP+IREL(I+1));BUFOT(2)=SIGMA# Prepare for output           AX52 216
ncodefld:(BUFOT,1,CHROT,FM1(1+2*MOD(I-1,6)),2)# Encode fields           AX52 217
J=NINT(FM1(1+2*MOD(I-1,6)))/10000+1#   Get status column                AX52 218
movechar:(AX521,N,CHROT,J,1,0)#        Move status                      AX52 219
IF (MOD(I,6)==0)writeline:(0,AX523,NAX523,1,3)# Print line after 6      AX52 220
I=I+1#                                 Increment irel ptr               AX52 221
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX52 222
IF (MARK(3)+STEP(3)==STOP(3))writeline:(0,AX523,NAX523-1,1,3)#          AX52 223
ELSE IF (MOD(I-1,6)!=0)writeline:(0,AX523,NAX523,1,3)#                  AX52 224
IF (REFINE(1)>0)#                      Crystal shape refined            AX52 225
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AX52 226
writeline:(1,AX522,NAX522,3,3)#        Print blank line                 AX52 227
LX=MARKX;LY=MARKY#                     Get pointers                     AX52 228
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over distances              AX52 229
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AX52 230
ifnc:(LX,L1)#                                                           AX52 231
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AX52 232
IF (NINT(nctype:(LX))==noref:)N=20#                                     AX52 233
ELSE N=32#                                                              AX52 234
incrx:(LX)#                                                             AX52 235
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AX52 236
ELSE N=1#                                                               AX52 237
J=NINT(FM0(7))/10000+1#                Status column                    AX52 238
movechar:(AX521,N,CHROT,J,1,0)#        Move status                      AX52 239
CALL AX59(L1,SIGMA)#                   Get esd of dist                  AX52 240
ncodefld:(facem:(L1),2,CHROT,FM0,7)#   Encode hkl, dist, esd, wght, c   AX52 241
ncodefld:(SIGMA,1,CHROT,FM0(8),1)#     Encode esd(calc)                 AX52 242
writeline:(0,AX522,NAX522,1,3)#        Print line                       AX52 243
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AX52 244
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AX52 245
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX52 246
copyfile:(1,2,incr:(lrexpl:),decr:(lrdset:))# Copy up to lrdset         AX52 247
uovl:=uovl:/utob:#                     Back to u                        AX52 248
IF ((REFINE(2)>1)\(REFINE(5)==2)\(REFINE(4)>1))# Overall displ fact o   AX52 249
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AX52 250
I=1;IWNT(I)=1#                         Dataset                          AX52 251
IF (REFINE(5)==2)#                     Overall displ fact refined       AX52 252
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AX52 253
I=I+1;IWNT(I)=2#                       Uov                              AX52 254
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AX52 255
IF (REFINE(4)>1)#                      Refine extinction                AX52 256
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AX52 257
movechar:(AX528,1,CHROT,1,11,0)#       Move title                       AX52 258
IF (EXTTYP>0)movechar:(AX528,12,CHROT,20,8,0)# Move gauss               AX52 259
ELSE         movechar:(AX528,20,CHROT,20,10,0)# Move lorentz            AX52 260
IF (IABS(EXTTYP)<=3)#                  Isotropic stuff                  AX52 261
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AX52 262
movechar:(AX528,41,CHROT,35,9,0)#      Move iso                         AX52 263
IF (IABS(EXTTYP)!=3)#                  Type 1 & 2                       AX52 264
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AX52 265
movechar:(AX528,50,CHROT,50,4,0)#      Move type                        AX52 266
movechar:(AX528,53+EXTTYP,CHROT,55,1,0)# Move 0 or 1                    AX52 267
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AX52 268
ELSE movechar:(AX528,56,CHROT,50,7,0)# Move general                     AX52 269
writeline:(1,0,0,1,3)#                 Print line                       AX52 270
IF (IABS(EXTTYP)!=2)#                  Type 1 or gen                    AX52 271
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 AX52 272
I=I+1;IWNT(I)=103#                     Get g                            AX52 273
ifnc:(LX,MARK(4))#                     Test if noref/constr             AX52 274
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 AX52 275
IF (NINT(nctype:(LX))==noref:)N=20#    Not refined                      AX52 276
ELSE                          N=32#    Constrained                      AX52 277
incrx:(LX)#                                                             AX52 278
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 AX52 279
ELSE                          N=1#                                      AX52 280
movechar:(AX529,1,CHROT,1,50,0)#       Copy line                        AX52 281
movechar:(AX521,N,CHROT,27,1,0)#       Copy status                      AX52 282
CALL AX59(MARK(4),SIGMAG)#             Get esd of g                     AX52 283
BUFOT(1)=g:;BUFOT(2)=SIGMAG#           Store g ans sigma g              AX52 284
ncodefld:(BUFOT,1,CHROT,FM3,2)#                                         AX52 285
writeline:(0,0,0,1,3)#                 Print line                       AX52 286
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 AX52 287
IF (IABS(EXTTYP)!=1)#                  Type 2 or gen                    AX52 288
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 AX52 289
I=I+1;IWNT(I)=104#                                                      AX52 290
ifnc:(LX,MARK(4)+1)#                                                    AX52 291
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 AX52 292
IF (NINT(nctype:(LX))==noref:)N=20#                                     AX52 293
ELSE                          N=32#                                     AX52 294
incrx:(LX)#                                                             AX52 295
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 AX52 296
ELSE                          N=1#                                      AX52 297
movechar:(AX529,51,CHROT,1,50,0)#      Copy line                        AX52 298
movechar:(AX521,N,CHROT,27,1,0)#       Copy status                      AX52 299
CALL AX59(MARK(4)+1,SIGMAR)#           Get esd of r                     AX52 300
####BUFOT(1)=rho:/LAMBDA;BUFOT(2)=SIGMA/LAMBDA# Back to rho             AX52 301
BUFOT(1)=rho:;BUFOT(2)=SIGMAR#         Store r and sigma r              AX52 302
ncodefld:(BUFOT,1,CHROT,FM3,2)#                                         AX52 303
writeline:(0,0,0,1,3)#                 Print line                       AX52 304
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 AX52 305
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AX52 306
ELSE#                                  Anisotropic stuff                AX52 307
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 AX52 308
movechar:(AX528,30,CHROT,35,11,0)#     Move ani                         AX52 309
IF (IABS(EXTTYP)!=6)#                  Type 1 & 2                       AX52 310
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 AX52 311
movechar:(AX528,50,CHROT,50,4,0)#      Move type                        AX52 312
movechar:(AX528,50+IABS(EXTTYP),CHROT,55,1,0)# Move 0 or 1              AX52 313
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 AX52 314
ELSE movechar:(AX528,56,CHROT,50,7,0)# Move general                     AX52 315
writeline:(0,0,0,1,3)#                 Print line                       AX52 316
IF (IABS(EXTTYP)!=5)#                  Type 1 or gen                    AX52 317
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 AX52 318
movechar:(AX528,63,CHROT,2,6,0)#       Move type                        AX52 319
movechar:(AX528,54,CHROT,9,1,0)#       Move 1                           AX52 320
DO J=1,6#                                                               AX52 321
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 36 AX52 322
I=I+1;IWNT(I)=104+J#                                                    AX52 323
ifnc:(LX,MARK(4)+1+J)#                                                  AX52 324
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 37 AX52 325
IF (NINT(nctype:(LX))==noref:)N=20#                                     AX52 326
ELSE                          N=32#                                     AX52 327
incrx:(LX)#                                                             AX52 328
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 37 AX52 329
ELSE                          N=1#                                      AX52 330
M=NINT(FM3(2*J+1))/10000+1#            Get column                       AX52 331
movechar:(AX521,N,CHROT,M,1,0)#        Copy status                      AX52 332
CALL AX59(MARK(4)+1+J,SIGMA)#          Get esd                          AX52 333
BUFOT(1)=extani1:(J);BUFOT(2)=SIGMA#                                    AX52 334
ncodefld:(BUFOT,1,CHROT,FM3(2*J+1),2)#                                  AX52 335
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 36 AX52 336
writeline:(0,0,0,1,3)#                 Print line                       AX52 337
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 AX52 338
IF (IABS(EXTTYP)!=4)#                  Type 2 or gen                    AX52 339
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 38 AX52 340
movechar:(AX528,63,CHROT,2,6,0)#       Move type                        AX52 341
movechar:(AX528,55,CHROT,9,1,0)#       Move 1                           AX52 342
DO J=1,6#                                                               AX52 343
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 39 AX52 344
I=I+1;IWNT(I)=110+J#                                                    AX52 345
ifnc:(LX,MARK(4)+7+J)#                                                  AX52 346
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 40 AX52 347
IF (NINT(nctype:(LX))==noref:)N=20#                                     AX52 348
ELSE                          N=32#                                     AX52 349
incrx:(LX)#                                                             AX52 350
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 40 AX52 351
ELSE                          N=1#                                      AX52 352
M=NINT(FM3(2*J+1))/10000+1#            Get column                       AX52 353
movechar:(AX521,N,CHROT,M,1,0)#        Copy status                      AX52 354
CALL AX59(MARK(4)+7+J,SIGMA)#          Get esd                          AX52 355
BUFOT(1)=extani2:(J);BUFOT(2)=SIGMA#                                    AX52 356
ncodefld:(BUFOT,1,CHROT,FM3(2*J+1),2)#                                  AX52 357
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 39 AX52 358
writeline:(0,0,0,1,3)#                 Print line                       AX52 359
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 38 AX52 360
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 AX52 361
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AX52 362
SIGNAL(1)=0;SIGNAL(2)=1;SIGNAL(3)=I;SIGNAL(4)=0# No mandatory           AX52 363
indexpkt:(1,lrdset:,IPACK,IP,2,SIGNAL,IWNT,IREL)# Get directory         AX52 364
IF (IP<=0)iquit:(AX5207)#              Lrdset: is missing               AX52 365
REPEAT#                                Loop over lrdset: packets        AX52 366
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 41 AX52 367
readwpkt:(1,lrdset:,IPACK,IP,0)#       Read one packet                  AX52 368
IF (IP<=0)BREAK#                       Exit if last                     AX52 369
writepkt:(2,lrdset:,IPACK,IPP)#        Write this packet                AX52 370
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Mover words                      AX52 371
IF (IREL(1)!=0)#                       Data set nb present              AX52 372
IF (NINT(QX(IPP+IREL(1)))!=DATSET)NEXT# Skip other data sets            AX52 373
I=1#                                   Set directory pointer            AX52 374
IF (REFINE(5)==2)#                     Overall displ fact refined       AX52 375
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 42 AX52 376
I=I+1#                                 Increment dir ptr                AX52 377
findc:(MARKX,MARKY,LX,LY,MARK(5))#     Get pointers                     AX52 378
CALL AX59(MARK(5),SIGMA)#              Get esd                          AX52 379
QX(IPP+IREL(2))=uovl:#                 Save                             AX52 380
BUFOT(1)=100.0*uovl:;BUFOT(2)=100.0*SIGMA/utob:# Prepare output         AX52 381
ncodefld:(BUFOT,1,AX524,FM4,2)#        Encode u & esd                   AX52 382
writeline:(1,AX524,NAX524,3,3)#        Print line                       AX52 383
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 AX52 384
IF (REFINE(4)>1)#                      Extinction refined               AX52 385
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 43 AX52 386
IF (IABS(EXTTYP)<=3)#                  Isotropic stuff                  AX52 387
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 44 AX52 388
IF (IABS(EXTTYP)!=2)#                  1 or gen                         AX52 389
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 45 AX52 390
I=I+1;QX(IPP+IREL(I))=10000.*g:#       Save g                           AX52 391
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 45 AX52 392
ELSE#                                  2                                AX52 393
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 46 AX52 394
I=I+1;QX(IPP+IREL(I))=1000.*rho:/LAMBDA# Save rho                       AX52 395
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 46 AX52 396
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 44 AX52 397
ELSE#                                  Aniso stuff                      AX52 398
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 47 AX52 399
IF (IABS(EXTTYP)!=5)#                  1 or gen                         AX52 400
DO J=1,6#                                                               AX52 401
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 48 AX52 402
I=I+1;QX(IPP+IREL(I))=extani1:(J)#     Save tensor 1                    AX52 403
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 48 AX52 404
IF (IABS(EXTTYP)!=4)#                  2 or gen                         AX52 405
DO J=1,6#                                                               AX52 406
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 49 AX52 407
I=I+1;QX(IPP+IREL(I))=extani2:(J)#     Save tensor 2                    AX52 408
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 49 AX52 409
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 47 AX52 410
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 43 AX52 411
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 41 AX52 412
copyfile:(1,2,incr:(lrdset:),decr:(lratom:))# Copy up to lratom:        AX52 413
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AX52 414
ELSE copyfile:(1,2,lrdset:,decr:(lratom:))# Copy up to lratom:          AX52 415
RETURN#                                                                 AX52 416
END#                                                                    AX52 417
#                                                                       AX52 418
systemheader:(AX53)#                   Write output bdf                 AX53   1
####################################################################    AX53   2
#                                                                  #    AX53   3
#                               AX53                               #    AX53   4
#                                                                  #    AX53   5
####################################################################    AX53   6
SUBROUTINE AX53#                                                        AX53   7
include:(AACOMN)#                                                       AX53   8
include:(AXCOMN)#                                                       AX53   9
#                                                                       AX53  10
INTEGER I,J#                           Loop indices                     AX53  11
INTEGER L7#                            Atom pointer                     AX53  12
INTEGER LX,LY,MX#                      Constraint list pointers         AX53  13
INTEGER MATCH#                         Used in string comparision       AX53  14
INTEGER M,N#                           Used to print                    AX53  15
REAL SIGMA#                            Esd of var                       AX53  16
INTEGER TFT#                           Temp fact type                   AX53  17
string:(COPY,117)#                     Copy of chrot                    AX53  18
string:(NAME1,lena:);string:(NAME2,lena:)# Atom names                   AX53  19
#                                                                       AX53  20
chardata:(AX531,"remark Atomic Parameters after Final Cycle")#          AX53  21
chardata:(AX532,[ATOM XXXXXXXXXXXXXXXXXXXXXXXX$#                        AX53  22
                                                   ])#                  AX53  23
chardata:(AX533,[UIJ  XXXXXXXXXXXXXXXXXXXXXXXX$#                        AX53  24
                                                   ])#                  AX53  25
chardata:(AX534,[IC ])#                                                 AX53  26
chardata:(AX535,[" Mean shift/error xxxx.xxxxx   Max shift/error $#     AX53  27
xxxx.xxxxx"])#                                                          AX53  28
chardata:(AX530,[" WARNING- U & Uij are multiplied by 100"])#           AX53  29
chardata:(AX53A,[" Atom                        x/a      y/b      $#     AX53  30
z/c   U/U11     U22     U33     U12     U13     U23     Pop"])#         AX53  31
realdata:(FM6,[320752.,410752.,500752.,580642.,$#                       AX53  32
660642.,740642.,820642.,900642.,980642.,1060642.])#                     AX53  33
realdata:(FANI,[370752., 450752., 530752., 620852., 710852., 800852.])# AX53  34
realdata:(FPCH,[370752., 450752., 530752., 610752., 680642., 760642.])# AX53  35
datastuff:#                                                             AX53  36
#                                                                       AX53  37
# READ LRATOM: DIRECTORY                                                AX53  38
# ----------------------                                                AX53  39
#                                                                       AX53  40
IF (PNCHCD==yes:)writepch:(AX531,NAX531)# Write remark card             AX53  41
I=0#                                   Reset directory pointer          AX53  42
I=I+1;IWNT(I)=14#                      Atom name                        AX53  43
I=I+1;IWNT(I)=17#                      Data set                         AX53  44
I=I+1;IWNT(I)=23#                      Atom displ fact type             AX53  45
DO J=1,10#                             Loop over xyz, u & uij           AX53  46
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX53  47
I=I+1;IWNT(I)=J#                       Coordinate                       AX53  48
I=I+1;IWNT(I)=100+J#                   Sigma of coordinate              AX53  49
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX53  50
IF (REFPOP==yes:)#                     Pop param refined                AX53  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX53  52
I=I+1;IWNT(I)=11#                      Pop                              AX53  53
I=I+1;IWNT(I)=111#                     Esd of pop                       AX53  54
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX53  55
SIGNAL(1)=3;SIGNAL(2)=3#               Name, tft & data set mandatory   AX53  56
SIGNAL(3)=I;SIGNAL(4)=0#               Update mode                      AX53  57
indexpkt:(1,lratom:,IPACK,IP,2,SIGNAL,IWNT,IREL)# Get directory         AX53  58
IF (IP<=0)iquit:(AX5308)#              Lratom: is missing               AX53  59
IF (SIGNAL(4)<0)iquit:(AX5309)#        Mandatory item is missing        AX53  60
#                                                                       AX53  61
# READ LRATOM:                                                          AX53  62
# ------------                                                          AX53  63
#                                                                       AX53  64
writeline:(1,AX530,NAX530,3,3)#        Print warning                    AX53  65
REPEAT#                                Loop over lratom: packets        AX53  66
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX53  67
readwpkt:(1,lratom:,IPACK,IP,0)#       Read one packet                  AX53  68
IF (IP<=0)BREAK#                       Exit if last                     AX53  69
writepkt:(2,lratom:,IPACK,IPP)#        Write it                         AX53  70
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Copy packet                      AX53  71
I=2#                                   Set pointer                      AX53  72
IF (IREL(I)!=0)#                       Test dataset nb                  AX53  73
IF (NINT(QX(IPP+IREL(I)))!=DATSET)NEXT# Wrong dataset                   AX53  74
movertoc:(QX(IPP+IREL(1)),1,NAME1,1,lena:,0)# Get bdf atom name         AX53  75
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX53  76
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX53  77
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Move name                      AX53  78
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,MATCH)# Compare names       AX53  79
IF (MATCH!=0)BREAK#                    Exit if found                    AX53  80
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX53  81
IF (L7>=STOP(7))iquit:(AX5399)#        Something wrong                  AX53  82
#                                                                       AX53  83
# TESTS & PRINT FOR USER                                                AX53  84
# ----------------------                                                AX53  85
#                                                                       AX53  86
TFT=NINT(atomtft:(L7))#                Get displ fact type              AX53  87
findnc:(MARKX,LX,L7)#                  Get constraint pointers          AX53  88
ifnc:(LX,L7)#                          Atom norefed                     AX53  89
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX53  90
IF(TFT==1)atomiso:(L7)=atomiso:(L7)/utob:# Restore u                    AX53  91
IF(TFT==2)DO J=1,3#                    Restore uij                      AX53  92
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX53  93
atomdis:(L7+J)=atomdis:(L7+J)/CELL(J+6)#                                AX53  94
atomdis:(L7+J+3)=2.0*atomdis:(L7+J+3)/CELL(J+9)#                        AX53  95
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX53  96
incrx:(LX);NEXT#                       Take next atom                   AX53  97
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX53  98
movertoc:(atomn:(L7),1,CHROT,2,lena:,0)# Move atom name                 AX53  99
movechar:(BLANK,1,COPY,1,NINT(FM6(10))/10000+1,1)# Clear copy           AX53 100
QX(IPP+IREL(3))=REAL(TFT)#             Save new displ fact type         AX53 101
I=2#                                   Set directory pointer            AX53 102
DO J=1,MAXPAR#                         Loop over parameters             AX53 103
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX53 104
I=I+2#                                 Increment pointer                AX53 105
IF ((J==4)&(TFT!=1))I=I+2#             Skip u location                  AX53 106
IF ((TFT==0)&(J>3)&(J<10))NEXT#        U overall                        AX53 107
IF ((TFT==1)&(J>4)&(J<10))NEXT#        U iso                            AX53 108
IF ((REFPOP==no:)&(J==10))NEXT#        Pp not refined                   AX53 109
CALL AX59(L7+J,SIGMA)#                 Compute sigma                    AX53 110
IF ((J>3)&(J<10))#                                                      AX53 111
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX53 112
IF (TFT==1)#                           Iso                              AX53 113
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX53 114
atomxyz:(L7+J)=atomxyz:(L7+J)/utob:;SIGMA=SIGMA/utob:# Restore u        AX53 115
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX53 116
ELSE#                                  Ani                              AX53 117
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX53 118
IF (J<7)#                                                               AX53 119
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX53 120
atomxyz:(L7+J)=atomxyz:(L7+J)/CELL(3+J);SIGMA=SIGMA/CELL(3+J)# Uij      AX53 121
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX53 122
ELSE#                                                                   AX53 123
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX53 124
atomxyz:(L7+J)=2.0*atomxyz:(L7+J)/CELL(3+J);SIGMA=2.0*SIGMA/CELL(3+J)#  AX53 125
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX53 126
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX53 127
BUFOT(J)=100.0*atomxyz:(L7+J);BUFOT(J+20)=100.0*SIGMA# Print paramete   AX53 128
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX53 129
ELSE#                                                                   AX53 130
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX53 131
BUFOT(J)=atomxyz:(L7+J);BUFOT(J+20)=SIGMA# Print parameter & esd        AX53 132
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX53 133
QX(IPP+IREL(I))=atomxyz:(L7+J)#        Save param                       AX53 134
QX(IPP+IREL(I+1))=SIGMA#               Save esd                         AX53 135
IF ((J==4)&(TFT==1))I=I+2#             Skip u11 location                AX53 136
ifnc:(LX,L7+J)#                                                         AX53 137
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX53 138
IF (NINT(nctype:(LX))==noref:)N=1#                                      AX53 139
ELSE#                                  Parameter constrained            AX53 140
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX53 141
N=2#                                   C                                AX53 142
WHILE (caddr:(LY)>small:)incry:(LY)#                                    AX53 143
incry:(LY)#                                                             AX53 144
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX53 145
incrx:(LX)#                                                             AX53 146
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX53 147
ELSE N=3#                                                               AX53 148
M=NINT(FM6(J))/10000+1#                Status column                    AX53 149
movechar:(AX534,N,CHROT,M,1,0)#        Code status                      AX53 150
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX53 151
#                                                                       AX53 152
# STORE AND PUCH CARDS                                                  AX53 153
# --------------------                                                  AX53 154
#                                                                       AX53 155
IF (PNCHCD==yes:)#                     Output punch cards               AX53 156
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX53 157
ncodefld:(atomx:(L7),1,AX532,FPCH,3)#  Encode xyz                       AX53 158
IF (TFT==2)#                                                            AX53 159
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX53 160
ncodefld:(atomu11:(L7),1,AX533,FANI,6)# Encode uij                      AX53 161
ncodefld:((atomu11:(L7)+atomu22:(L7)+atomu33:(L7))/3.0,$# Transform u   AX53 162
1,AX532,FPCH(4),1)#                    To u & encode                    AX53 163
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX53 164
ELSE IF (TFT==1)ncodefld:(atomiso:(L7),1,AX532,FPCH(4),1)# Encode u     AX53 165
ELSE ncodefld:(uovl:,1,AX532,FPCH(4),1)# Encode uov                     AX53 166
ncodefld:(atompop:(L7),1,AX532,FPCH(5),1)# Encode pop                   AX53 167
movechar:(NAME1,1,AX532,6,lena:,0)#    Move atom name                   AX53 168
writepch:(AX532,NAX532)#               Output card                      AX53 169
IF (TFT==2)#                           Print also uijs                  AX53 170
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX53 171
movechar:(NAME1,1,AX533,6,lena:,0)#    Move atom name                   AX53 172
writepch:(AX533,NAX533)#               Output uij card                  AX53 173
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX53 174
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX53 175
M=3#                                   Default: xyz printed only        AX53 176
IF (TFT==1)M=4#                        Print also u iso                 AX53 177
IF (TFT==2)M=9#                        Print also uij                   AX53 178
ncodefld:(BUFOT,1,CHROT,FM6,M)#        Encode first m fields            AX53 179
ncodefld:(BUFOT,21,COPY,FM6,M)#        Encode first m sigmas            AX53 180
ncodefld:(atompop:(L7),1,CHROT,FM6(10),1)# Encode pop                   AX53 181
IF (REFPOP==no:)#                      Do not refine pop                AX53 182
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX53 183
BUFOT(30)=0.0#                         No sigma                         AX53 184
movechar:(AX534,1,CHROT,NINT(FM6(10))/10000+1,1,0)# Status i            AX53 185
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX53 186
ncodefld:(BUFOT(30),1,COPY,FM6(10),1)# Encode sigma                     AX53 187
writeline:(1,AX53A,NAX53A,2,3)#        Print final atomic params        AX53 188
movechar:(COPY,1,CHROT,1,NAX53A+1,0)#  Copy esd to chrot                AX53 189
writeline:(0,AX53A,NAX53A,1,3)#        Print esds                       AX53 190
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX53 191
#                                                                       AX53 192
# PRINT MEAN & MAX VALUES OF SHIFT/ERROR                                AX53 193
# --------------------------------------                                AX53 194
#                                                                       AX53 195
SOEAV=SOEAV/REAL(STOP20-MARK20+1)#     Get mean value                   AX53 196
ncodefld:(SOEAV,1,AX535,281052.,1)#    Encode mean value                AX53 197
ncodefld:(MAXSOE,1,AX535,571052.,1)#   Encode max value                 AX53 198
writeline:(1,AX535,NAX535,3,3)#        Print line                       AX53 199
RETURN#                                                                 AX53 200
END#                                                                    AX53 201
#                                                                       AX53 202
systemheader:(AX54)#                   Write lrrefl:                    AX54   1
####################################################################    AX54   2
#                                                                  #    AX54   3
#                               AX54                               #    AX54   4
#                                                                  #    AX54   5
####################################################################    AX54   6
SUBROUTINE AX54#                                                        AX54   7
include:(AACOMN)#                                                       AX54   8
include:(AXCOMN)#                                                       AX54   9
#                                                                       AX54  10
REAL TEMP,TEMPFO,TEMPDF#               Temp variable                    AX54  11
REAL C,S,T#                            Structure factor components      AX54  12
REAL FN,FNX,PH#                        Structure factor componenets     AX54  13
REAL AN,BN,AX,BX,FF#                   Total structure factor compone   AX54  14
REAL AH,AK,AL#                         Equivalent h,k,l                 AX54  15
REAL DA,DB,DAA,DBA#                    Dispersion corrections           AX54  16
REAL EQV(480)#  ## Space for generated reflections                      AX54  17
REAL FAC#       ## Multiplication factor                                AX54  18
REAL FAC1,FAC2# ## Real & imag scattering components                    AX54  19
REAL FO#        ## F rel ("observation")                                AX54  20
REAL FMUL#      ## Atom multiplicative factor                           AX54  21
REAL H(3)#                             Reflection indices               AX54  22
REAL OBS#                              Observed and calculated i        AX54  23
INTEGER I,J,K,L,M#                     General indices                  AX54  24
INTEGER LA#                            Symop & twinop list pointers     AX54  25
INTEGER L1,L3,L6,L7#                   List pointers                    AX54  26
INTEGER MEQV#   ## Length of symeq table                                AX54  27
INTEGER NSG#    ## Scale group number                                   AX54  28
INTEGER OPACK#  ## Output packet length                                 AX54  29
REAL PHI#                              Reflection phase                 AX54  30
REAL SCATF(sfmax:)#                    Current reflection scatt.facts   AX54  31
INTEGER SFT#    ## Scattering factor type                               AX54  32
INTEGER TFT#    ## Number of temp. factors                              AX54  33
REAL STOL#      ## Sin(theta/lambda)                                    AX54  34
REAL STL2#      ## Sin(theta/lambda) sqare                              AX54  35
REAL SUMCOS#    ## Cos summation of fc                                  AX54  36
REAL SUMSIN#    ## Sin summation of fc                                  AX54  37
#                                                                       AX54  38
realdata:(P2,6.28319)#                                                  AX54  39
chardata:(AX541," WARNING- Reflection with unknown scale factor $#      AX54  40
number skipped")#                                                       AX54  41
datastuff:#                                                             AX54  42
#                                                                       AX54  43
copyfile:(1,2,incr:(lratom:),decr:(lrrefl:))# Copy up to lrrefl:        AX54  44
M=DATSET*1000#                         Data set bias for idnums         AX54  45
IWNT(1)=1;IWNT(2)=2;IWNT(3)=3#         Indices,sin(t/l),mlt,equiv.      AX54  46
IF (INTREF==yes:)IWNT(4)=M+300#        Relative i                       AX54  47
ELSE             IWNT(4)=M+302#        |f|**2                           AX54  48
IWNT(5)=M+201#                         Absorption                       AX54  49
IWNT(6)=M+204#                         1/lp                             AX54  50
IWNT(7)=M+11#                          Azimuthal angle for ani extinc   AX54  51
IWNT(8)=M+308#                         Rcode                            AX54  52
IWNT(9)=M+309#                         Scale factor group               AX54  53
IWNT(10)=M+200#                        Tbar                             AX54  54
DO I=1,NB(1);IWNT(I+10)=M+210+I#       Crystal shape derivatives        AX54  55
DO I=1,NB(6);IWNT(I+10+NB(1))=500+I#   Form factor idnums               AX54  56
J=10+NB(1)+NB(6)#                      Pointer for additional values^   AX54  57
IWNT(J+1) =+M+700#                     Phase                            AX54  58
IWNT(J+2) =+M+701#                     |fo| without dispersion          AX54  59
IWNT(J+3) =+M+800#                     |fc| without disp or ext         AX54  60
IWNT(J+4) =+M+801#                     A without disp or exp            AX54  61
IWNT(J+5) =+M+802#                     B without disp or exp            AX54  62
IWNT(J+6) =+M+820#                     |fc| with disp and ext           AX54  63
IWNT(J+7) =-M-702#                     Remove scaled n701               AX54  64
IWNT(J+8) =-M-703#                     Remove scaled sigma n701         AX54  65
IWNT(J+9) =-M-803#                     Remove a disp only               AX54  66
IWNT(J+10)=-M-804#                     Remove b disp only               AX54  67
IWNT(J+11)=-M-805#                     Remove a n801 with disp; no ex   AX54  68
IWNT(J+12)=-M-806#                     Remove b n802 with disp; no ex   AX54  69
IWNT(J+13)=-M-202#                     Remove extinction correction ^   AX54  70
SIGNAL(1)=4;           SIGNAL(2)=J#    Set search limits                AX54  71
SIGNAL(3)=SIGNAL(2)+13;SIGNAL(4)=1#    Set search limit & archive       AX54  72
indexpkt:(1,lrrefl:,IPACK,IP,2,SIGNAL,IWNT,IREL)# Point to refl dir     AX54  73
IF(IP<0)RETURN#                        Lrrefl is not there              AX54  74
IF(SIGNAL(4)<0)iquit:(AX5402)#         Item missing                     AX54  75
OPACK=SIGNAL(4)#                       Output packet length             AX54  76
IF (INTREF==no:)#                                                       AX54  77
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX54  78
IREL(5)=0;IREL(6)=0#                   No correction for |f|**2         AX54  79
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX54  80
#                                                                       AX54  81
# SET RELATIVE POINTERS, INITIALIZATION                                 AX54  82
# -------------------------------------                                 AX54  83
#                                                                       AX54  84
movereal:(0.0,1,EQV,1,480,2)#          Zero equiv hkl array             AX54  85
writeline:(0,0,0,1,3)#                 Add one blank line               AX54  86
LINRM=5#                               Reserve page space               AX54  87
MEQV=10*NSYM#                                                           AX54  88
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX54  89
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX54  90
TFT=NINT(atomtft:(L7))#                Get atom type                    AX54  91
IF (TFT==1)atomiso:(L7)=utob:*atomiso:(L7)# To b                        AX54  92
IF (TFT==2)#                           To beta                          AX54  93
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX54  94
DO I=1,3;atomdis:(L7+I)=CELL(I+6)*atomdis:(L7+I)#                       AX54  95
DO I=4,6;atomdis:(L7+I)=0.5*CELL(I+6)*atomdis:(L7+I)#                   AX54  96
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX54  97
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX54  98
uovl:=uovl:*utob:#                                                      AX54  99
#                                                                       AX54 100
#     LOOP OVER THE REFLECTIONS                                         AX54 101
#     =========================                                         AX54 102
#                                                                       AX54 103
REPEAT#                                Reflection loop                  AX54 104
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX54 105
readwpkt:(1,lrrefl:,IPACK,IP,0)#       Read reflection packet           AX54 106
IF(IP<=0)BREAK#                        End of lrrefl:                   AX54 107
writepkt:(2,lrrefl:,OPACK,IPP)#        Write packet                     AX54 108
K=IPP+1;   L=SIGNAL(2)+1#              Set initial pointers             AX54 109
DO I=1,IPACK#                          Loop over input packet           AX54 110
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX54 111
J=IP+I#                                Point to output items            AX54 112
movepakwd:(QX(J),QX(K))#               Move i/p to o/p              ^   AX54 113
IF(IREL(L)==I) L=L+1#                  Delete item & incr               AX54 114
ELSE           K=K+1#                  Tranfer item & incr              AX54 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX54 116
movereal:(voidflg:,1,QX,K,6,2)#        Void addon area in packet        AX54 117
#                                                                       AX54 118
gethkl:(IP+IREL(1),H)#                 Extract hkl                      AX54 119
STOL=QX(IP+IREL(2))#                   Get st/l                         AX54 120
#                                      *********** extract corrections  AX54 121
FAC=1.0#                                                                AX54 122
ifpr:(5)FAC=1.0/QX(IP+IREL(5))#        Transmission                     AX54 123
IF (REFINE(1)>0)#                      Refine shape                     AX54 124
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX54 125
I=1#                                   Transmission                     AX54 126
DO L1=MARK(1),STOP(1)-STEP(1),STEP(1)# Loop over faces                  AX54 127
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX54 128
FAC=FAC+(facedc:(L1)-facedo:(L1))*QX(IP+IREL(I+10))# New transmission   AX54 129
I=I+1#                                                                  AX54 130
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX54 131
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX54 132
ifpr:(6)FAC=FAC/QX(IP+IREL(6))#        Lp correction                    AX54 133
#                                      *********** extract scales       AX54 134
ifpr:(9)NSG=MAX(1,NINT(QX(IP+IREL(9))))# Get scale group number         AX54 135
ELSE    NSG=1#                         Default scale group              AX54 136
DO L3=MARK(3),STOP(3)-STEP(3),STEP(3)# Loop over scale groups           AX54 137
IF(NSG==NINT(sclgrp:(L3)))BREAK#       Exit if match found              AX54 138
IF(L3>=STOP(3))#                       No scale group found             AX54 139
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX54 140
writeline:(0,AX541,NAX541,3,3)#        Print warning                    AX54 141
NEXT#                                  Ignore reflection                AX54 142
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX54 143
STL2=STOL*STOL#                        Calc st/l **2                    AX54 144
OBS=QX(IP+IREL(4))#                    ********** |fo|                  AX54 145
FO=SQRT(ABS(OBS)/FAC)#                 |fo| in rel with sign            AX54 146
IF (OBS<0.0)FO=-FO#                                                     AX54 147
#                                                                       AX54 148
# NOW FORM FACTORS                                                      AX54 149
# ----------------                                                      AX54 150
#                                      ********** extract form fac      AX54 151
DO I=1,NB(6)#                          Loop over form factors           AX54 152
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX54 153
IF(IREL(I+10+NB(1))>0)M=IP+IREL(I+10+NB(1))# Get sf from bdf            AX54 154
ELSE IF(INEU>2)      M=NINT(STOL/DSFTB)+MARKC+1+(I-1)*STEPC# Table      AX54 155
ELSE                 M=MARKC+I#        Get neutron scat length          AX54 156
SCATF(I)=QX(M)#                        Sf for this reflection           AX54 157
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX54 158
#                                                                       AX54 159
# NOW GENERATE HKL EQUIVALENT                                           AX54 160
# ---------------------------                                           AX54 161
#                                      ********* gen equiv hkl          AX54 162
I=0#                                   Set equiv hkl store point        AX54 163
DO LA=MARKA,STOPA-stepa:,stepa:#       Loop over symm matrices          AX54 164
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX54 165
AH=H(1)*QX(LA+1)+H(2)*QX(LA+2)+H(3)*QX(LA+3)# Equivalent h              AX54 166
AK=H(1)*QX(LA+4)+H(2)*QX(LA+5)+H(3)*QX(LA+6)# Equivalent k              AX54 167
AL=H(1)*QX(LA+7)+H(2)*QX(LA+8)+H(3)*QX(LA+9)# Equivalent l              AX54 168
EQV(I+1)=P2*AH;EQV(I+2)=P2*AK;EQV(I+3)=P2*AL# Store hkl indices         AX54 169
EQV(I+4)=AH*AH;EQV(I+5)=AK*AK;EQV(I+6)=AL*AL# Store new diag els        AX54 170
EQV(I+7)=2.0*AH*AK;EQV(I+8)=2.0*AH*AL;EQV(I+9)=2.0*AK*AL# Store new h   AX54 171
EQV(I+10)=P2*(H(1)*QX(LA+10)+H(2)*QX(LA+11)+H(3)*QX(LA+12))# Phase sh   AX54 172
I=I+10#                                Incr parameter array             AX54 173
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX54 174
#                                    ***********SET ATOM POINTERS       AX54 175
AN=0.0;BN=0.0#                         No dispersion                    AX54 176
DA=0.0;DB=0.0;DAA=0.0;DBA=0.0#         Dispersion corrections           AX54 177
#                                                                       AX54 178
#     LOOP OVER ALL ATOMS                                               AX54 179
#     ===================                                               AX54 180
#                                                                       AX54 181
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)#                                  AX54 182
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX54 183
SFT=NINT(atomsft:(L7));L6=MARK(6)+(SFT-1)*STEP(6)# Extract scatf type   AX54 184
TFT=NINT(atomtft:(L7))#                Extract tempf type               AX54 185
SUMCOS=0.;SUMSIN=0.#                   Zero fc components               AX54 186
#                                                                       AX54 187
#   LOOP OVER EQUIVALENT HKL                                            AX54 188
#   ------------------------                                            AX54 189
#                                                                       AX54 190
FOR(I=0;I<MEQV;I=I+10)#                ********** equiv indices         AX54 191
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX54 192
TEMP=AMOD(EQV(I+1)*atomx:(L7)$#                                         AX54 193
         +EQV(I+2)*atomy:(L7)$#                                         AX54 194
         +EQV(I+3)*atomz:(L7)$#                                         AX54 195
         +EQV(I+10),P2)#               H*x/a+k*y/b+l*z/c + phi shift    AX54 196
C=COS(TEMP);    S=SIN(TEMP)#           Sin and cos (hx+ky+lz+t)         AX54 197
IF(TFT>1)#                             Anisotropic temp.fact.           AX54 198
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX54 199
T=EXP(-EQV(I+4)*atomu11:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX54 200
      -EQV(I+5)*atomu22:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX54 201
      -EQV(I+6)*atomu33:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX54 202
      -EQV(I+7)*atomu12:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX54 203
      -EQV(I+8)*atomu13:(L7)$#         Sum h(i)*h(j)*u(i,j)             AX54 204
      -EQV(I+9)*atomu23:(L7))#         Sum h(i)*h(j)*u(i,j)             AX54 205
S=T*S;      C=T*C#                     Sin & cos contributions          AX54 206
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX54 207
SUMCOS=SUMCOS+C;IF (ICENT==no:)SUMSIN=SUMSIN+S# Sum components          AX54 208
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX54 209
#                                                                       AX54 210
#  APPLY TF AND SITE MULT AND POPULATION PARAMETER                      AX54 211
#  -----------------------------------------------                      AX54 212
#                                                                       AX54 213
FMUL=REAL(RFLMLT)*atommul:(L7)*atompop:(L7)# Set atom multiplicity      AX54 214
IF (TFT==0)FMUL=FMUL*EXP(-STL2*uovl:)# Apply overall displ fact         AX54 215
IF (TFT==1)FMUL=FMUL*EXP(-STL2*atomiso:(L7))# Apply iso displ           AX54 216
#                                                                       AX54 217
# SUM STRUCTURE FACTOR COMPONENTS                                       AX54 218
# -------------------------------                                       AX54 219
#                                                                       AX54 220
FAC1=SCATF(SFT);FAC2=0.0#              Scattering factor without disp   AX54 221
AN=AN+FMUL*FAC1*SUMCOS;IF (ICENT==no:)BN=BN+FMUL*FAC1*SUMSIN#           AX54 222
IF (REFINE(6)>0)#                      Apply dispersion                 AX54 223
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX54 224
L6=MARK(6)+(SFT-1)*STEP(6)#            Get disp pointer                 AX54 225
DA=DA+FMUL*(redisp:(L6)*SUMCOS-imdisp:(L6)*SUMSIN)# Real part           AX54 226
DB=DB+FMUL*(redisp:(L6)*SUMSIN+imdisp:(L6)*SUMCOS)# Im part             AX54 227
DAA=DAA+FMUL*(redisp:(L6)*SUMCOS+imdisp:(L6)*SUMSIN)# Anti real part    AX54 228
DBA=DBA+FMUL*(-redisp:(L6)*SUMSIN+imdisp:(L6)*SUMCOS)# Anti im part     AX54 229
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX54 230
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX54 231
IF (REFINE(2)>0)#                      Twin enantiomorph                AX54 232
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX54 233
AX=entpol:(1)*DA+entpol:(2)*DAA#                                        AX54 234
BX=entpol:(1)*DB+entpol:(2)*DBA#                                        AX54 235
T=entpol:(1)*(DA**2+DB**2)+entpol:(2)*(DAA**2+DBA**2)-(AX**2+BX**2)#    AX54 236
FF=entpol:(1)*((AN+DA)**2+(BN+DB)**2)+$# Get disp f**2                  AX54 237
entpol:(2)*((AN+DAA)**2+(-BN+DBA)**2)#                                  AX54 238
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX54 239
ELSE#                                                                   AX54 240
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX54 241
AX=DA;BX=DB;T=0.0;FF=(AN+DA)**2+(BN+DB)**2# No twin refinements         AX54 242
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX54 243
IF ((AN**2+BN**2)<small:)     PHI=0.0# Too small - assume phi=0         AX54 244
ELSE IF(AN>0.&ABS(BN)<0.0005) PHI=0.0# Phi is zero                      AX54 245
ELSE IF(AN<0.&ABS(BN)<0.0005) PHI=0.5# Phi is pi                        AX54 246
ELSE IF(BN>0.&ABS(AN)<0.0005) PHI=.25# Phi is pi/2                      AX54 247
ELSE IF(BN<0.&ABS(AN)<0.0005) PHI=.75# Phi is 3pi/2                     AX54 248
ELSE                          PHI=ATAN2(BN,AN)/P2# General phase        AX54 249
IF (REFINE(4)>0)#                      Extinction correction            AX54 250
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX54 251
I=REFINE(4);REFINE(4)=1#               No derivatives                   AX54 252
ifpr:(7)S=QX(IP+IREL(7))#              Get psi for ani extinction       AX54 253
ifpr:(10)TBAR=QX(IP+IREL(10))#         Get tbar                         AX54 254
CALL AX39(H,FF,STOL,S,sclfac:(L3),0.0,0.0)# Compute extinction          AX54 255
FNX=FO/SQRT(sclfac:(L3)); FO=FO/SQRT(YEXT)# Extinction correction       AX54 256
REFINE(4)=I#                           Restore extinction refinement    AX54 257
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX54 258
FN=SQRT(MAX(0.,AN*AN+BN*BN))#          No disp fc                       AX54 259
FO=FO/SQRT(sclfac:(L3))#               |fo| in e-                       AX54 260
IF (REFINE(6)>0)#                      Dispersion correction            AX54 261
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX54 262
PH=PHI*P2#                             Convert phase to radians         AX54 263
T=SQRT(MAX((FO**2-(AX*SIN(PH)-BX*COS(PH))**2-T),0.0))#                  AX54 264
S=-(AX*COS(PH)+BX*SIN(PH))#            From bernardinelli & flack   ^   AX54 265
IF(ABS(S+T-FN)<ABS(S-T-FN)) FO=S+T#                                     AX54 266
ELSE                        FO=S-T#                                     AX54 267
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX54 268
QX(K  )=PHI#                           Phase in cycles                  AX54 269
QX(K+1)=FO*SQRT(sclfac:(L3))#          |fo| corrected for ext & disp    AX54 270
QX(K+2)=FN#                            No dispersion |fc|               AX54 271
QX(K+3)=AN#                            No dispersion a                  AX54 272
QX(K+4)=BN#                            No dispersion b                  AX54 273
QX(K+5)=SQRT(FF)#                      |fc| corrected for disp & ext    AX54 274
IF(PPRIOR>3)#                          Dump structure factors           AX54 275
WRITE(6,'(3f6.0,6f10.5)') H,STOL,YEXT,TBAR,FNX,QX(K+5),FNX-QX(K+5)##    AX54 276
TEMPFO=TEMPFO+FNX; TEMPDF=TEMPDF+ABS(FNX-QX(K+5))##                     AX54 277
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX54 278
IF(PPRIOR>3)#                          Dump structure factors           AX54 279
WRITE(6,*) TEMPFO,TEMPDF,TEMPDF/TEMPFO##                                AX54 280
copyfile:(1,2,incr:(lrrefl:),decr:(lrsfls:))#                           AX54 281
RETURN#                                                                 AX54 282
END#                                                                    AX54 283
#                                                                       AX54 284
systemheader:(AX55)#                   Write lrsfls: & restraints       AX55   1
####################################################################    AX55   2
#                                                                  #    AX55   3
#                               AX55                               #    AX55   4
#                                                                  #    AX55   5
####################################################################    AX55   6
SUBROUTINE AX55#                                                        AX55   7
include:(AACOMN)#                                                       AX55   8
include:(AXCOMN)#                                                       AX55   9
#                                                                       AX55  10
INTEGER I,J,K#                         Loop index                       AX55  11
INTEGER L7#                            Atom pointer                     AX55  12
INTEGER LR#                            Restraint pointer                AX55  13
INTEGER LX#                            Constraint list pointer          AX55  14
INTEGER MATCH#                         Flag to compare strings          AX55  15
string:(NAME1,lena:)#                  Atom names                       AX55  16
string:(NAME2,lena:)#                                                   AX55  17
INTEGER NCONS#                         Number of constraints            AX55  18
INTEGER NRES#                          Number of restraints             AX55  19
INTEGER OPACK#                         Output packet size               AX55  20
INTEGER TYPE#                          Type of restraint                AX55  21
string:(S40,40)#                       40 chars string                  AX55  22
#                                                                       AX55  23
chardata:(AX551,["Distances"])#        Refinement process               AX55  24
chardata:(AX552,["Unit/Minimal variance/User"])# Bdf weights            AX55  25
chardata:(AX553,["None/Applied/Refined/ Isotropic /Anisotropic$#        AX55  26
Type 1 /Type 2 /General/Lorentzian/Gaussian  "])# Extinction            AX55  27
chardata:(AX555,"[Full matrix]")#                                       AX55  28
chardata:(AX556,"[None/Flack_s method-Applied/Refined]")#               AX55  29
chardata:(AX557,"Distance      Angle        $#                          AX55  30
Torsion Angle Rigid Bond    void")#                                     AX55  31
chardata:(AX558," Restraint Type                                   $#   AX55  32
                        Obs   E.s.d  Weight    Calc   E.s.d")#          AX55  33
intdata:(LRR,[40,42,43,44])#  ## lrbond: lrangl lrdihe: lrrigi:         AX55  34
intdata:(PAR,[2, 3, 4, 2])#            Number of parameter of restrai   AX55  35
realdata:(FMT,[770752., 850742., 930742., 1010742., 1090742.])#         AX55  36
datastuff:#                                                             AX55  37
#                                                                       AX55  38
# WRITE LRSFLS: FROM SCRATCH                                            AX55  39
# --------------------------                                            AX55  40
#                                                                       AX55  41
IPACK=1;IWNT(IPACK)=1#                 Data set number                  AX55  42
DO J=1,mxwd40c:#                       Refinement process               AX55  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX55  44
IPACK=IPACK+1;IWNT(IPACK)=11#                                           AX55  45
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX55  46
DO J=1,mxwd40c:#                       Weighting scheme                 AX55  47
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX55  48
IPACK=IPACK+1;IWNT(IPACK)=12#                                           AX55  49
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX55  50
DO J=1,mxwd40c:#                       Extinction process               AX55  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX55  52
IPACK=IPACK+1;IWNT(IPACK)=13#                                           AX55  53
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX55  54
DO J=1,mxwd40c:#                       Matrix description               AX55  55
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX55  56
IPACK=IPACK+1;IWNT(IPACK)=14#                                           AX55  57
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX55  58
DO J=1,mxwd40c:#                       Treatment of hydrogens           AX55  59
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX55  60
IPACK=IPACK+1;IWNT(IPACK)=15#                                           AX55  61
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX55  62
DO J=1,mxwd40c:#                       Treatment of absolute structur   AX55  63
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX55  64
IPACK=IPACK+1;IWNT(IPACK)=16#                                           AX55  65
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX55  66
IPACK=IPACK+1;IWNT(IPACK)=21#          R-value on f diff                AX55  67
IPACK=IPACK+1;IWNT(IPACK)=27#          Goodness of fit                  AX55  68
IPACK=IPACK+1;IWNT(IPACK)=29#          Max shift/err                    AX55  69
IPACK=IPACK+1;IWNT(IPACK)=30#          Mean shift/err                   AX55  70
IPACK=IPACK+1;IWNT(IPACK)=31#          Rogers abs conf measure nu       AX55  71
IPACK=IPACK+1;IWNT(IPACK)=32#          Flacks abs conf measure x    ^   AX55  72
IPACK=IPACK+1;IWNT(IPACK)=33#          Refined extinction coeff     ^   AX55  73
IPACK=IPACK+1;IWNT(IPACK)=34#          Sigma of extinction coeff        AX55  74
IPACK=IPACK+1;IWNT(IPACK)=35#          Rogers abs conf measure sigma^   AX55  75
IPACK=IPACK+1;IWNT(IPACK)=36#          Flacks abs conf measure sigma^   AX55  76
IPACK=IPACK+1;IWNT(IPACK)=41#          Struct fact ref (i)              AX55  77
IPACK=IPACK+1;IWNT(IPACK)=42#          Number of params refined         AX55  78
IPACK=IPACK+1;IWNT(IPACK)=43#          Number of reflections            AX55  79
IPACK=IPACK+1;IWNT(IPACK)=44#          Number of restraints             AX55  80
IPACK=IPACK+1;IWNT(IPACK)=45#          Number of constraints            AX55  81
writepkt:(2,lrsfls:,IPACK,IPP)#                                         AX55  82
DO I=1,IPACK;QX(IPP+I)=REAL(IWNT(I))#                                   AX55  83
writepkt:(2,lrsfls:,IPACK,IPP)#                                         AX55  84
QX(IPP+1)=REAL(DATSET);I=2#            Store dataset number             AX55  85
movechar:(BLANK,1,S40,1,40,2)#         Clear s40                        AX55  86
IF (REFINE(1)>0)movechar:(AX551,1,S40,1,9,0)# Move "distances"          AX55  87
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store ref process       AX55  88
movechar:(BLANK,1,S40,1,40,2)#         Clear s40                        AX55  89
IF (IWT==0)movechar:(AX552,1,S40,1,4,0)# Move "unit"                    AX55  90
IF (IWT==1)movechar:(AX552,6,S40,1,16,0)# Move "minimal variance"       AX55  91
IF (IWT==2)movechar:(AX552,22,S40,1,4,0)# Move "user"                   AX55  92
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store weighting descr   AX55  93
movechar:(BLANK,1,S40,1,40,2)#         Clear s40                        AX55  94
IF (REFINE(5)==0)movechar:(AX553,1,S40,1,4,0)# Move "none"              AX55  95
ELSE#                                  Extinction present               AX55  96
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX55  97
IF (REFINE(5)==1)movechar:(AX553,6,S40,1,8,0)# Move "applied"           AX55  98
ELSE movechar:(AX553,14,S40,1,8,0)#    Move "refined"                   AX55  99
IF (IABS(EXTTYP)<=3)movechar:(AX553,22,S40,9,12,0)# Move "isotropic"    AX55 100
ELSE movechar:(AX553,32,S40,9,12,0)#   Move "anisotropic"               AX55 101
K=46+MOD(IABS(EXTTYP)-1,3)*8#          Extinction type                  AX55 102
movechar:(AX553,K,S40,8,21,0)#         Move extinction type             AX55 103
IF (EXTTYP<0)movechar:(AX553,70,S40,29,10,0)# Move "lorentzian"         AX55 104
ELSE movechar:(AX553,80,S40,29,10,0)#  Move "gaussian"                  AX55 105
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX55 106
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store extinc process    AX55 107
movechar:(BLANK,1,S40,1,40,2)#         Clear blank                      AX55 108
movechar:(AX555,1,S40,1,11,0)#         Move "full matrix"               AX55 109
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store matrix decr       AX55 110
movechar:(BLANK,1,S40,1,40,2)#         Clear s40                        AX55 111
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store hydrogen treatm   AX55 112
movechar:(BLANK,1,S40,1,40,2)#         Clear blank                      AX55 113
IF (REFINE(2)==0)movechar:(AX556,1,S40,1,4,0)# Move "none"              AX55 114
ELSE#                                  Abs structure present            AX55 115
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX55 116
movechar:(AX556,6,S40,1,14,0)#         Move "flack's method"            AX55 117
IF (REFINE(2)==1)movechar:(AX556,20,S40,15,7,0)# Move "applied"         AX55 118
ELSE movechar:(AX556,28,S40,15,7,0)#   Move "refined"                   AX55 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX55 120
movector:(S40,1,QX(IPP+I),1,40,0);I=I+mxwd40c:# Store abs struct        AX55 121
QX(IPP+I)=R;I=I+1#                     Store r-value                    AX55 122
QX(IPP+I)=GOOF;I=I+1#                  Store goodness of fit            AX55 123
QX(IPP+I)=MAXSOE;I=I+1#                Store max shift/err              AX55 124
QX(IPP+I)=SOEAV;I=I+1#                 Store mean shift/err             AX55 125
QX(IPP+I)=voidflg:; I=I+1#             No rogers measure                AX55 126
QX(IPP+I)=XABS; I=I+1#                 Flacks abs conf measure ?????    AX55 127
QX(IPP+I)=voidflg:; I=I+1#             Refined extinction value         AX55 128
QX(IPP+I)=voidflg:; I=I+1#             Sigma of extinction value        AX55 129
IF (REFINE(4)>1)#                      Extinction refined               AX55 130
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX55 131
IF (IABS(EXTTYP)<=3)#                  Isotropic stuff                  AX55 132
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX55 133
IF (IABS(EXTTYP)!=2)#                  1 or gen                         AX55 134
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX55 135
QX(IPP+I-2)=10000.*g:#                 Save g                           AX55 136
QX(IPP+I-1)=10000.*SIGMAG#             Save sigma g                     AX55 137
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX55 138
ELSE#                                  2                                AX55 139
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX55 140
QX(IPP+I-2)=1000.*rho:/LAMBDA#         Save rho                         AX55 141
QX(IPP+I-1)=1000.*SIGMAR/LAMBDA#       Save sigma rho                   AX55 142
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX55 143
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX55 144
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX55 145
QX(IPP+I)=voidflg:; I=I+1#             No rogers sigma                  AX55 146
QX(IPP+I)=SUXABS; I=I+1#               Flacks abs conf sigma            AX55 147
QX(IPP+I)=REAL(INTREF+1);I=I+1#        Refinement on i                  AX55 148
QX(IPP+I)=REAL(VARTOT);I=I+1#          Parameter refined                AX55 149
QX(IPP+I)=BUFIN(2);I=I+1#              Reflections in matrix            AX55 150
NRES=0#                                Reset restraint counter          AX55 151
DO TYPE=1,4#                           Loop over restraint types        AX55 152
FOR (LR=MARKR(TYPE);LR<STOPR(TYPE);incrr:(LR,PAR(TYPE)))# Loop over r   AX55 153
NRES=NRES+1#                           Increment restraint counter      AX55 154
QX(IPP+I)=REAL(NRES);I=I+1#            Restraints applied               AX55 155
NCONS=0#                               Reset constraint counter         AX55 156
DO LX=MARKX,STOPX-stepx:,stepx:#       Loop over nc list                AX55 157
IF (NINT(nctype:(LX))!=noref:)NCONS=NCONS+1#                            AX55 158
QX(IPP+I)=REAL(NCONS);I=I+1#           Constraints applied              AX55 159
#                                                                       AX55 160
# WRITE CALCULATED VALUES OF RESTRAINTS                                 AX55 161
# -------------------------------------                                 AX55 162
#                                                                       AX55 163
LR=MARKR(1)#                           Set restraint pointer            AX55 164
DO TYPE=1,4#                           Loop over restraint types        AX55 165
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AX55 166
IF (MARKR(TYPE)>=STOPR(TYPE))NEXT#     Exit if none of this type        AX55 167
#                                                                       AX55 168
# READ LOGICAL RECORDS DIRECTORIES                                      AX55 169
# --------------------------------                                      AX55 170
#                                                                       AX55 171
I=1#                                   First item in iwant/irel array   AX55 172
DO K=10,10*PAR(TYPE),10#               Loop over atoms                  AX55 173
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AX55 174
IWNT(I)=K+1;I=I+1#                     Atom name                        AX55 175
DO J=1,12#                             Store symop                      AX55 176
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AX55 177
IWNT(I)=10*K+J;I=I+1#                  Symop                            AX55 178
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AX55 179
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AX55 180
SIGNAL(1)=I-1#                         End of mandatory inputs          AX55 181
DO K=10,10*PAR(TYPE),10#               Loop over atoms                  AX55 182
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AX55 183
DO J=1,3#                              Loop over coordinates            AX55 184
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AX55 185
IWNT(I)=K+J+1;I=I+1#                   Store coords                     AX55 186
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AX55 187
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AX55 188
IWNT(I)=91;I=I+1#                      Store calc                       AX55 189
IWNT(I)=92#                            Esd of calc                      AX55 190
SIGNAL(2)=I#                           End of optionnal input           AX55 191
SIGNAL(3)=0;SIGNAL(4)=0#               Update mode                      AX55 192
indexpkt:(1,LRR(TYPE),IPACK,IP,2,SIGNAL,IWNT,IREL)# Read directory      AX55 193
OPACK=SIGNAL(4)#                       Save output packet length        AX55 194
IF (OPACK<0)#                          Missing mandatory items          AX55 195
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AX55 196
IF (TYPE==1)iquit:(AX5501)#            Distance                         AX55 197
IF (TYPE==2)iquit:(AX5502)#            Angle                            AX55 198
IF (TYPE==3)iquit:(AX5503)#            Torsion                          AX55 199
IF (TYPE==4)iquit:(AX5504)#            Rigid link restraint             AX55 200
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AX55 201
movechar:(AX557,1+(TYPE-1)*13,AX558,17,13,0)# Mover restrain type       AX55 202
writeline:(1,AX558,NAX558,3,4)#        Print title                      AX55 203
REPEAT#                                                                 AX55 204
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AX55 205
#                                                                       AX55 206
# READ ONE RESTRAINT                                                    AX55 207
# ------------------                                                    AX55 208
#                                                                       AX55 209
readwpkt:(1,LRR(TYPE),IPACK,IP,0)#     Read one packet                  AX55 210
IF (IP<=0)BREAK#                       Exit if last                     AX55 211
writepkt:(2,LRR(TYPE),OPACK,IPP)#      Write packet                     AX55 212
movereal:(QX,IP+1,QX,IPP+1,IPACK,0)#   Copy values                      AX55 213
I=PAR(TYPE)*16#                        Skip atomic part                 AX55 214
QX(IPP+IREL(I+1))=voidflg:;QX(IPP+IREL(I+2))=voidflg:# Clear calc & e   AX55 215
I=0#                                   Reset directory pointer          AX55 216
DO K=1,PAR(TYPE)#                      Loop over atoms                  AX55 217
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AX55 218
#                                                                       AX55 219
# CHECK ATOM NAMES                                                      AX55 220
# ----------------                                                      AX55 221
#                                                                       AX55 222
movertoc:(QX(IP+IREL(I+1)),1,NAME1,1,lena:,0)# Move name                AX55 223
L7=NINT(resp:(LR,K))#                  Get atom pointer                 AX55 224
DO J=1,3;QX(IPP+IREL(13*PAR(TYPE)+3*(K-1)+J))=atomxyz:(L7+J)# New coo   AX55 225
movertoc:(atomn:(L7),1,NAME2,1,lena:,0)# Move restraint name            AX55 226
compchar:(NAME1,1,NAME2,1,lena:,lena:,lena:,MATCH)# Compare names       AX55 227
IF (MATCH==0)BREAK#                    Not same atom                    AX55 228
DO J=1,12;ifne:(QX(IP+IREL(I+1+J)),resm:(LR,K,J))BREAK# Compare symop   AX55 229
IF (J<=12)BREAK#                       Not same symop                   AX55 230
I=I+13#                                Increment dir pointer            AX55 231
movechar:(NAME1,1,CHROT,2+(K-1)*17,15,0)#                               AX55 232
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AX55 233
IF (K<=PAR(TYPE))NEXT#                 Not same restraint               AX55 234
#                                                                       AX55 235
# UPDATE CALC VALUE AND PRINT                                           AX55 236
# ---------------------------                                           AX55 237
#                                                                       AX55 238
BUFOT(1)=resobs:(LR);BUFOT(2)=resd:(LR);BUFOT(3)=resw:(LR)#             AX55 239
I=PAR(TYPE)*16#                        Point to general part            AX55 240
QX(IPP+IREL(I+1))=rescal:(LR);QX(IPP+IREL(I+2))=rescsd:(LR)# Update c   AX55 241
BUFOT(4)=rescal:(LR);BUFOT(5)=rescsd:(LR)#                              AX55 242
ncodefld:(BUFOT,1,CHROT,FMT,5)#                                         AX55 243
writeline:(0,AX558,NAX558,1,4)#        Print restraint                  AX55 244
incrr:(LR,PAR(TYPE))#                  Increment restraint pointer      AX55 245
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AX55 246
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AX55 247
copyfile:(1,2,LRR(4)+1,endrecord:)#    Close file                       AX55 248
RETURN#                                                                 AX55 249
END#                                                                    AX55 250
#                                                                       AX55 251
systemheader:(AX56)#                   Write correlation matrix         AX56   1
####################################################################    AX56   2
#                                                                  #    AX56   3
#                               AX56                               #    AX56   4
#                                                                  #    AX56   5
####################################################################    AX56   6
SUBROUTINE AX56#                                                        AX56   7
include:(AACOMN)#                                                       AX56   8
include:(AXCOMN)#                                                       AX56   9
#                                                                       AX56  10
INTEGER I,J,L#                         Loop index                       AX56  11
INTEGER L7#                            Atom pointer                     AX56  12
INTEGER L22,II,JJ#                     Matrix pointers                  AX56  13
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX56  14
INTEGER LENATM,LENCST#                 Atom & constraints buffer leng   AX56  15
INTEGER LT#                            Temp list to copy atoms          AX56  16
INTEGER OPACK#                         Output packet length             AX56  17
INTEGER PARAM#                         Subject param number             AX56  18
INTEGER PTR#                           Offset pointer for atom list     AX56  19
INTEGER RANGE#                         Copied range length              AX56  20
INTEGER SFT,TFT#                       Scatt & displ fact types         AX56  21
#                                                                       AX56  22
FILEXT(IOUNIT(4))='cmx'#               Extension                        AX56  23
IOMARK(4)=QXSTR#                       Buffer for file 4                AX56  24
OPACK=8+arith:(15,/,mxchwd:)#          Packet size                      AX56  25
writepkt:(4,2,OPACK,IPP)#              Write 1st packet                 AX56  26
movector:(TITLE,10,QX(IPP+1),1,6,0)#   Move compid                      AX56  27
movector:(TITLE,17,QX(IPP+1),9,8,0)#   Move date                        AX56  28
I=incr:(arith:(15,/,mxchwd:))#         Offset for numerical info        AX56  29
QX(IPP+I+1)=1.0#                       Number of blocks                 AX56  30
QX(IPP+I+2)=MARK1(7)-MARK1(1)#         Number of non-atomic variables   AX56  31
QX(IPP+I+3)=0.0#                       Offset for atom list             AX56  32
LENATM=0#                              Clear atom counter               AX56  33
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX56  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX56  35
TFT=NINT(atomtft:(L7));IF (TFT==2)TFT=6# Nb of displ params             AX56  36
LENATM=LENATM+TFT+mxwd24c:+2+3+2#                                       AX56  37
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX56  38
QX(IPP+I+4)=REAL(LENATM)#              Atom list length                 AX56  39
QX(IPP+I+5)=3.0#                       Size of block information        AX56  40
QX(IPP+I+6)=0.0#                       Size of user block information   AX56  41
L=0#                                   Reset noref counter              AX56  42
DO LX=MARKX,STOPX-stepx:,stepx:#       Loop over constraint list        AX56  43
IF (NINT(nctype:(LX))==noref:)L=L+1#   Count noref                      AX56  44
J=(STOPX-MARKX)/stepx:-L#              Count constraints                AX56  45
LENCST=L+4*J+(STOPY-MARKY-2*J)#        Size of constraint information   AX56  46
QX(IPP+I+7)=REAL(LENCST)#              Save constraint info size        AX56  47
#                                                                       AX56  48
# SAVE ATOMS IN REGFE FORMAT                                            AX56  49
# --------------------------                                            AX56  50
#                                                                       AX56  51
MARKT=STOP22+1;PTR=incr:(mxwd24c:)#    Set pointers                     AX56  52
incrqx:(STOPT,MARKT+STEP(7),AX5601)#   Get memory                       AX56  53
RANGE=0;OPACK=MIN0(bdfbuf:-4,LENATM)#  Max length for atom list         AX56  54
writepkt:(4,3,OPACK,IPP)#              Prepare first packet             AX56  55
DO L7=MARK(7),STOP(7)-STEP(7),STEP(7)# Loop over atoms                  AX56  56
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX56  57
LT=MARKT+incr:(mxwd24c:)#              Set pointer                      AX56  58
TFT=NINT(atomtft:(L7));IF (TFT==2)TFT=6# Get nb of displ params         AX56  59
SFT=NINT(atomsft:(L7))#                Get atom type ptr                AX56  60
movereal:(atomn:(L7),1,QX(MARKT+1),1,mxwd24c:,0)# Move atom name        AX56  61
PTR=PTR+mxwd24c:+2+3+TFT+2#            Pointer to next atom             AX56  62
QX(LT)=REAL(PTR)#                      Save pointer                     AX56  63
intpak:(SFT,QX(LT+1),0,5)#             Save sft                         AX56  64
intpak:(TFT,QX(LT+1),5,3)#             Save tft                         AX56  65
QX(LT+2)=atomx:(L7);QX(LT+3)=atomy:(L7);QX(LT+4)=atomz:(L7)# Save coo   AX56  66
L=1#                                   Set displ param counter          AX56  67
WHILE (L<=TFT)#                                                         AX56  68
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX56  69
QX(LT+4+L)=atomdis:(L7+L)#             Save displ params                AX56  70
L=L+1#                                 Increment counter                AX56  71
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX56  72
QX(LT+4+L)=atompop:(L7);QX(LT+5+L)=atommul:(L7)# Save pop & mult        AX56  73
STOPT=LT+5+L#                          End-of-list pointer              AX56  74
IF (RANGE+(STOPT-MARKT)>OPACK)#        Must be written on 2 packets     AX56  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX56  76
movereal:(QX,MARKT+1,QX,IPP+1,OPACK-RANGE,0)# Finish packet             AX56  77
writepkt:(4,3,OPACK,IPP)#              Write packet                     AX56  78
LT=MARKT+(OPACK-RANGE);RANGE=0#        Clear range                      AX56  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX56  80
ELSE LT=MARKT#                                                          AX56  81
movereal:(QX,LT+1,QX,IPP+1,STOPT-LT,0)# Copy atom list                  AX56  82
IPP=IPP+(STOPT-MARKT)#                 Update ouput pointer             AX56  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX56  84
#                                                                       AX56  85
# SAVE BLOCK INFORMATION                                                AX56  86
# ----------------------                                                AX56  87
#                                                                       AX56  88
writepkt:(4,4,3,IPP)#                  Write 3 words                    AX56  89
QX(IPP+1)=1.0;QX(IPP+2)=1.0;QX(IPP+3)=REAL(VARTOT)#                     AX56  90
#                                                                       AX56  91
# SAVE CONSTRAINT INFO IN REGFE FORMAT                                  AX56  92
# ------------------------------------                                  AX56  93
#                                                                       AX56  94
RANGE=0;OPACK=MIN0(LENCST,bdfbuf:-4)#  Set packet length                AX56  95
writepkt:(4,6,OPACK,IPP)#              Prepare first packet             AX56  96
findc:(MARKX,MARKY,LX,LY,MARK(7))#     Set pointers                     AX56  97
DO LX=MX,STOPX-stepx:,stepx:#          Loop over atomic constraints     AX56  98
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX56  99
L7=MARK(7)+STEP(7)*((NINT(ncaddr:(L7))-MARK(7))/STEP(7))# Atom addres   AX56 100
L=L7-MARK(7)+mxwd24c:#                 Atom pointer in regfe format     AX56 101
PARAM=NINT(ncaddr:(LX))-L7#            Parameter address                AX56 102
IF (NINT(nctype:(LX))==noref:)#        Not refined param                AX56 103
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AX56 104
IF (PARAM==0)QX(MARKT+1)=-L#           Noref all atom                   AX56 105
ELSE QX(MARKT+1)=-(L+1+PARAM)#         Noref just one parameter         AX56 106
LT=MARKT+1#                            Only one word to move            AX56 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AX56 108
ELSE#                                  Constraint                       AX56 109
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AX56 110
QX(MARKT+1)=L+1+PARAM#                 Save subject param address       AX56 111
QX(MARKT+2)=-1.0#                      Normal constraint                AX56 112
LT=MARKT+4#                            Set pointer                      AX56 113
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX56 114
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AX56 115
QX(LT+1)=caddr:(LY)-REAL(MARK20-1)#    Number of variable               AX56 116
QX(LT+2)=cfact:(LY)#                   Multiplicative factor            AX56 117
incry:(LY);incry:(LT)#                 Take next packet                 AX56 118
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AX56 119
QX(MARKT+3)=REAL((LY-NINT(nctype:(LX)))/stepy:);QX(MARKT+4)=2.0# Nb o   AX56 120
incry:(LY)#                            Next constraint                  AX56 121
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AX56 122
STOPT=LT#                              End-of-list pointer              AX56 123
IF (RANGE+(STOPT-MARKT)>OPACK)#        Must be written on 2 packets     AX56 124
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AX56 125
movereal:(QX,MARKT+1,QX,IPP+1,OPACK-RANGE,0)# Finish packet             AX56 126
writepkt:(4,6,OPACK,IPP)#              Write packet                     AX56 127
LT=MARKT+(OPACK-RANGE);RANGE=0#        Clear range                      AX56 128
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AX56 129
ELSE LT=MARKT#                                                          AX56 130
movereal:(QX,LT+1,QX,IPP+1,STOPT-LT,0)# Copy constraint list            AX56 131
IPP=IPP+(STOPT-MARKT)#                 Update ouput pointer             AX56 132
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX56 133
#                                                                       AX56 134
# SAVE MATRIX                                                           AX56 135
# -----------                                                           AX56 136
#                                                                       AX56 137
RANGE=(VARTOT*(VARTOT+1))/2#           Total length                     AX56 138
OPACK=MIN0(RANGE+1,bdfbuf:-4)#         Packet length                    AX56 139
writepkt:(4,7,OPACK,IPP)#              Write 1st packet                 AX56 140
QX(IPP+1)=REAL(VARTOT)#                Write number of variables        AX56 141
L=2;L22=MARK22;II=MARK22#              Set pointers                     AX56 142
DO I=1,VARTOT#                         Loop over rows                   AX56 143
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AX56 144
JJ=II#                                 Set to diag element              AX56 145
DO J=I,VARTOT#                         Loop over columns                AX56 146
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AX56 147
QX(IPP+L)=QX(L22)/SQRT(QX(II)*QX(JJ))# Correlation                      AX56 148
JJ=JJ+VARTOT+1-J;L22=L22+1;L=L+1#      Next element                     AX56 149
IF (L>OPACK)#                          Next packet                      AX56 150
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AX56 151
writepkt:(4,7,OPACK,IPP)#              Take next packet                 AX56 152
L=1#                                   Reset output pakcet pointer      AX56 153
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AX56 154
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AX56 155
II=II+VARTOT+1-I#                      Next diag element                AX56 156
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AX56 157
writepkt:(4,endrecord:,3,IPP)#         Close file                       AX56 158
RETURN#                                                                 AX56 159
END#                                                                    AX56 160
#                                                                       AX56 161
systemheader:(AX59)#                   Compute e.s.d.                   AX59   1
####################################################################    AX59   2
#                                                                  #    AX59   3
#                               AX59                               #    AX59   4
#                                                                  #    AX59   5
####################################################################    AX59   6
SUBROUTINE AX59(L,SIGMA)#                                               AX59   7
include:(AACOMN)#                                                       AX59   8
include:(AXCOMN)#                                                       AX59   9
#                                                                       AX59  10
INTEGER IJ#                            Covariance pointer               AX59  11
INTEGER J,K#                           Variable nb                      AX59  12
INTEGER L#                             Addresse of parameter            AX59  13
INTEGER LX,LY,MX,MY#                   Constraint list pointers         AX59  14
INTEGER L21#                           Vector pointer                   AX59  15
INTEGER L22,M22#                       Auxiliary diag pointer           AX59  16
REAL SIGMA#                            Esd of parameter                 AX59  17
REAL SHIFT#                            Displacement                     AX59  18
INTEGER VARBLK#                        Number of variables              AX59  19
#                                                                       AX59  20
SIGMA=0.0#                             Clear sigma                      AX59  21
VARBLK=STOP20-MARK20+1#                Number of variables              AX59  22
findnc:(MARKX,LX,L)#                   Get pointer                      AX59  23
ifnc:(LX,L)#                           Parameter constrained or not r   AX59  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AX59  25
IF (NINT(nctype:(LX))!=noref:)#        Constrained                      AX59  26
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AX59  27
LY=NINT(nctype:(LX))#                  Get constraint address           AX59  28
WHILE (caddr:(LY)>small:)#             Loop over constraint             AX59  29
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AX59  30
J=NINT(caddr:(LY))-MARK20#             Get ref param address            AX59  31
L22=MARK22+(J*(2*VARBLK-J+1))/2#       Diag el                          AX59  32
SIGMA=SIGMA+cfact:(LY)**2*QX(L22)#     Diag el                          AX59  33
FOR (MY=LY+stepy:;caddr:(MY)>small:;incry:(MY))#                        AX59  34
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AX59  35
K=NINT(caddr:(MY))-MARK20#             Get other ref addr               AX59  36
M22=MARK22+(K*(2*VARBLK-K+1))/2#       Diag el                          AX59  37
IF (K>J)IJ=L22+K-J#                                                     AX59  38
ELSE IJ=M22+J-K#                                                        AX59  39
SIGMA=SIGMA+2.0*cfact:(LY)*cfact:(MY)*QX(IJ)# Sum covariance            AX59  40
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AX59  41
incry:(LY)#                            Take next reference param        AX59  42
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AX59  43
SIGMA=SQRT(AMAX1(0.0,SIGMA))#          Get esd                          AX59  44
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AX59  45
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AX59  46
ELSE#                                                                   AX59  47
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AX59  48
DO L21=MARK21,STOP21;IF (NINT(QX(L21))==L)BREAK# Get parameter pointe   AX59  49
J=L21-MARK21#                          Variable nb                      AX59  50
L22=MARK22+(J*(2*VARBLK-J+1))/2#       Diag el                          AX59  51
SIGMA=SQRT(AMAX1(0.0,QX(L22)))#        Get esd                          AX59  52
SHIFT=QX(MARK20+J)#                    Shift of var                     AX59  53
SOEAV=SOEAV+ABS(SHIFT/SIGMA)#                                           AX59  54
MAXSOE=AMAX1(MAXSOE,ABS(SHIFT/SIGMA))#                                  AX59  55
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AX59  56
RETURN#                                                                 AX59  57
END#                                                                    AX59  58
pmacro:([no:])#                                                         AX59  59
