systemheader:(AZ    )#                                                  AZ     1
####################################################################### AZ     2
##      Copyright (C) 2000, The Xtal System, The University of        # AZ     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # AZ     4
## Distributed under the GNU General Public Licence as published by   # AZ     5
## the Free Software Foundation, in the hope that it will be useful   # AZ     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # AZ     7
## for more details.                                                  # AZ     8
#                                                                     # AZ     9
# ***** LSABS:  ABSORPTION PROGRAM (VERSION 1.0)                      # AZ    10
# ***** WRITTEN BY: E.BLANC WITH HELP FROM D.SCHWARZENBACH            # AZ    11
# ***** MANY PARTS OF THIS PROGRAM ARE TAKEN FROM OR INSPIRED BY THE  # AZ    12
#       PROGRAM LSABS WRITTEN BY G.DAVENPORT AND J.STEWART            # AZ    13
#                                                                     # AZ    14
# ------------------------------------------------------------------- # AZ    15
# WHAT DOES LSABS DO ?                                                # AZ    16
# FOR EACH REFLECTION, IT COMPUTES TRANSMITTANCE FACTOR, MEAN PATH    # AZ    17
# AND THE FIRST DERIVATIVES OF TRANSMITTANCE FACTOR WITH RESPECT TO   # AZ    18
# DISTANCES TO FACES. AS IT IS DIFFRACTOMETER INDEPENDANT, THE USER   # AZ    19
# NEED NOT TO INTRODUCE ANYTHING ABOUT DIFFRACTOMETER GEOMETRY, BUT   # AZ    20
# THE AZIMUTHAL ANGLE PSI.                                            # AZ    21
# MANDATORY INPUTS: * CELL CONSTANTS                                  # AZ    22
#                   * WAVELENGTH                                      # AZ    23
#                   * FOR EACH RELECTION: MILLER INDICES AND PSI      # AZ    24
# IT CAN BE USED EITHER WITH A BDF OR DIRECTLY FROM CARDS.            # AZ    25
# ------------------------------------------------------------------- # AZ    26
# GENERAL ALGORITHM                                                   # AZ    27
# AS THE PROGRAM IS DIFFRACTOMETER INDEPENDANT, IT CAN'T USE THE      # AZ    28
# ORIENTATION MATRIX IN THE ALGORITHM. THAT IS WHY THE MAIN BASE IN   # AZ    29
# WHICH ALL CALCULATIONS TAKE PLACE IS AN ORTHONORMAL BASE CONSTRUC-  # AZ    30
# TED ON THE NATURAL CRYSTAL BASE (A*, B*, C*) AND DEPENDANT ON THE   # AZ    31
# CRYSTAL MACROSCOPIC FORM.                                           # AZ    32
# GAUSSIAN INTEGRATION NEED SURFACE INTEGRATION ON EACH CRYSTAL FACE. # AZ    33
# ANALYTICAL INTEGRATION ALGORITHM IS DIFFERENT OF DE MEULENAER &     # AZ    34
# TOMPA'S. THE CRYSTAL IS CONSIDERED AS A SET OF EDGES INSTEAD OF A   # AZ    35
# SET OF VERTICES FOR BUILDING HOWELL'S POLYHEDRON.                   # AZ    36
# ------------------------------------------------------------------- # AZ    37
# GENERAL SUMMARY OF THE PROGRAM:                                     # AZ    38
# - AZ00: MAIN PROGRAM, LSABS LINE ANALYSIS, FOLLOWING CARDS ANALY-   # AZ    39
#         SYS                                                         # AZ    40
# - AZ0X: GENERAL PURPOSE MATRIX AND VECTOR OPERATIONS                # AZ    41
# - AZ10: BUILDS VERTICES, EDGES AND GRID (IF GAUSSIAN INTEGRATION)   # AZ    42
# - AZ20: MAIN REFLECTION SUBROUTINE (CALLED FOR EACH REFLECTION)     # AZ    43
# - AZ3X: BDF INPUT/OUTPUT SUBROUTINES                                # AZ    44
# - AZ4X: OTHER SUBROUTINES                                           # AZ    45
# ------------------------------------------------------------------- # AZ    46
# QX VECTOR ARRAY USAGE                                               # AZ    47
# 1) THREE INPUT/OUTPUT BUFFERS (ONE FOR SCRATCH)                     # AZ    48
# 2) FACES BUFFER (FROM MARKF TO ISTOPF)                              # AZ    49
#    * MILLER INDICES (SOON REPLACED BY INDICES IN B BASE)            # AZ    50
#    * DISTANCE FROM CENTER TO FACE [IN MM]                           # AZ    51
#    * DISTANCE ESD & WEIGHT (TEMPORARELY STORED FOR I/O PURPOSES)    # AZ    52
#    * SURFACE IN [MM**2]                                             # AZ    53
#    * NUMBER OF VERTICES BOUNDING THE FACE                           # AZ    54
#    * DERIVATIVE OF T W.R.T DIATANCE OF FACE (USED FOR EACH REFLEC-  # AZ    55
#      TION)                                                          # AZ    56
#    * CENTER OF FACE COORDINATES                                     # AZ    57
# 3) VERTICES BUFFER (FROM MARKV TO ISTOPV)                           # AZ    58
#    * VERTEX COORDINATES IN B (ORTHONORMAL) BASE                     # AZ    59
#    * NUMBER OF FACES INTO WHICH THE VERTEX LIES                     # AZ    60
#    * FACES POINTERS                                                 # AZ    61
# 4) EDGES BUFFER (FROM MARKE TO ISTOPE)                              # AZ    62
#    * EXTREMITIES OF EDGE COORDINATES                                # AZ    63
#    * FACE POINTERS                                                  # AZ    64
#    WARNING: AN EDGE IS ALWAYS POSITIVELY ORIENTED W.R.T THE FACE 1, # AZ    65
#             I.E.:                                                   # AZ    66
#             [(X2-X0)^(X1-X0)].N1 > 0                                # AZ    67
#             WHERE X2 IS THE 2ND VERTEX & X1 IS THE FIRST VERTEX,    # AZ    68
#             X0 IS THE CENTER OF THE FACE 1 & N1 THE NORMAL TO THE   # AZ    69
#             FACE 1                                                  # AZ    70
# 5+) GRID BUFFER (FORM MARKG TO ISTOPG)                              # AZ    71
#    * POINT COORDINATES IN B (ORTHONORMAL) BASE                      # AZ    72
#    * VOLUME ELEMENT                                                 # AZ    73
# 6+) SURFACE GRID BUFFER (FROM MARKS TO ISTOPS)                      # AZ    74
#    * POINT COORDINATES IN B (ORTHONORMAL) BASE                      # AZ    75
#    * VOLUME ELEMENT                                                 # AZ    76
#    * FACE POINTER                                                   # AZ    77
# 7) NOABS BUFFER (FROM MARKN TO ISTOPN)                              # AZ    78
#    * MIN AND MAX NUMBERS OF REFLECTIONS TO BE SKIPPED (BDF ONLY)    # AZ    79
# 8+) CUTTING PLANE LIST (FROM MARKC TO ISTOPC; MARKD TO ISTOPD)      # AZ    80
#    * NORMAL TO PLANE                                                # AZ    81
#    * DISTANCE FROM PLANE TO ORIGIN                                  # AZ    82
#    * POINTERS TO THE FACES BUILDING THE EDGE FORM WHICH THE CUTTING # AZ    83
#      PLANE IS ISSUED                                                # AZ    84
# 9+) HOWELLS POLYHEDRON EDGES (FORMAT IDENTICAL TO EDGE LIST E)      # AZ    85
#                                                                     # AZ    86
# + LISTS USED ONLY FOR ONE INTEGRATION METHOD                        # AZ    87
#                                                                     # AZ    88
####################################################################### AZ    89
#                                                                       AZ    90
# WARNING: PROGRAM WRITTEN FOR XTAL 3.0 PACKAGE                         AZ    91
# ---------------------------------------------                         AZ    92
#                                                                       AZ    93
include:(XMACRO)#                                                       AZ    94
#                                                                       AZ    95
# GENERAL PURPOSE MACROS                                                AZ    96
# ----------------------                                                AZ    97
#                                                                       AZ    98
macro:(rad:,0.017453293)#              1 deg in radians                 AZ    99
macro:(pi:,3.141592654)#                                                AZ   100
macro:(lens:,8)#                       Atom type name length in char    AZ   101
macro:(mxwd40c:,incr:(arith:(39,/,mxchwd:)))#                           AZ   102
#                                                                       AZ   103
# TYPE DEFINITION MACROS                                                AZ   104
# ----------------------                                                AZ   105
#                                                                       AZ   106
ifsel:(CRAY,UNXCRA)#                   Double precision machines        AZ   107
macro:(dp:,REAL)#                                                       AZ   108
macro:(dble:,$1)#                                                       AZ   109
macro:(sngl:,$1)#                                                       AZ   110
macro:(dcos:,COS($1))#                                                  AZ   111
macro:(dexp:,EXP($1))#                                                  AZ   112
macro:(dabs:,ABS($1))#                                                  AZ   113
macro:(dsqrt:,SQRT($1))#                                                AZ   114
macro:(dfloat:,REAL($1))#                                               AZ   115
macro:(dsmall:,small:)#                Same value, but double precisi   AZ   116
ifsel:#                                Single precision machines        AZ   117
macro:(dp:,DOUBLE PRECISION)#                                           AZ   118
macro:(dble:,DBLE($1))#                                                 AZ   119
macro:(sngl:,SNGL($1))#                                                 AZ   120
macro:(dcos:,DCOS($1))#                                                 AZ   121
macro:(dexp:,DEXP($1))#                                                 AZ   122
macro:(dabs:,DABS($1))#                                                 AZ   123
macro:(dsqrt:,DSQRT($1))#                                               AZ   124
macro:(dfloat:,DBLE($1))#                                               AZ   125
macro:(dsmall:,1.0D-7)#                Same value, but double precisi   AZ   126
endsel:#                                                                AZ   127
macro:(vector:,dp: $1(3))#                                              AZ   128
macro:(matrix:,dp: $1(3,3))#                                            AZ   129
#                                                                       AZ   130
# FUNCTIONS AND PROCEDURES MACROS                                       AZ   131
# -------------------------------                                       AZ   132
#                                                                       AZ   133
macro:(cross:,CALL AZ01($1,$2,$3))#    Cross product                    AZ   134
macro:(dot:,($1(1)*$2(1)+$1(2)*$2(2)+$1(3)*$2(3)))# Dot product         AZ   135
macro:(invert:,CALL AZ02($1,$2,$3))#   3*3 matrix inversion (det in $   AZ   136
macro:(postm:,CALL AZ03($1,$2,$3))#    $1(matrix) * $2(vector) = $3(v   AZ   137
macro:(transp:,CALL AZ04($1,$2))#      3*3 matrix transposition         AZ   138
macro:(norm:,CALL AZ05($1,$2,$3))#     Vector normalisation             AZ   139
macro:(mmult:,CALL AZ06($1,$2,$3))#    3*3 matrix product               AZ   140
#                                                                       AZ   141
macro:(findidn:,CALL AZ07($1,$2,$3,$4))# Search directory packet        AZ   142
#                                                                       AZ   143
# INDEXATION MACROS                                                     AZ   144
# -----------------                                                     AZ   145
#                                                                       AZ   146
macro:(nstepn:,2)#                     Increment: 2 words/record        AZ   147
macro:(incrn:,$1=$1+nstepn:)#                                           AZ   148
macro:(noabs:,[ifelse:($1,,NN,$1)])#                                    AZ   149
macro:(minn:,[QX(noabs:($1)+1)])#      Lowest  reflection number to s   AZ   150
macro:(maxn:,[QX(noabs:($1)+2)])#      Highest reflection number to s   AZ   151
#                                                                       AZ   152
macro:(nstepf:,13)#                    Increment: 13 words/record       AZ   153
macro:(incrf:,$1=$1+nstepf:)#                                           AZ   154
macro:(getf:,MARKF+($1-1)*nstepf:)#                                     AZ   155
macro:(face:,[ifelse:($1,,FF,$1)])#                                     AZ   156
macro:(facehm:,[QX(face:($1)+1)])#     Miller indices of face           AZ   157
macro:(facekm:,[QX(face:($1)+2)])#                                      AZ   158
macro:(facelm:,[QX(face:($1)+3)])#                                      AZ   159
macro:(faceh:,[QX(face:($1)+4)])#      Coordinates of face in orthono   AZ   160
macro:(facek:,[QX(face:($1)+5)])#                                       AZ   161
macro:(facel:,[QX(face:($1)+6)])#                                       AZ   162
macro:(faced:,[QX(face:($1)+7)])#      Distance from face to origin     AZ   163
macro:(facer:,QX(MARKF+7))#            Radius of sphere                 AZ   164
macro:(facesd:,[QX(face:($1)+9)])#     Esd of distance                  AZ   165
macro:(facew:,[QX(face:($1)+8)])#      Weight                           AZ   166
macro:(faces:,[QX(face:($1)+8)])#      Surface                          AZ   167
macro:(facen:,[QX(face:($1)+10)])#     Number of vertices around face   AZ   168
macro:(facecx:,[QX(face:($1)+11)])#    Coordinates of center of mass    AZ   169
macro:(facecy:,[QX(face:($1)+12)])#                                     AZ   170
macro:(facecz:,[QX(face:($1)+13)])#                                     AZ   171
macro:(faceq:,[QX(face:($1)+1)])#      First derivative of a w.r. to    AZ   172
macro:(faceai:,[QX(face:($1)+2)])#     Cosine of angle between face a   AZ   173
macro:(faceao:,[QX(face:($1)+3)])#     Cosine of angle between face a   AZ   174
#                                                                       AZ   175
macro:(nstepg:,4)#                     Increment: 4 words/record        AZ   176
macro:(incrg:,$1=$1+nstepg:)#                                           AZ   177
macro:(getg:,MARKG+nstepg:*($3-1+NPTZ*($2-1+NPTY*($1-1))))# Gets addr   AZ   178
macro:(grid:,[ifelse:($1,,GG,$1)])#                                     AZ   179
macro:(gridx:,[QX(grid:($1)+1)])#      Coordinates of grid point        AZ   180
macro:(gridy:,[QX(grid:($1)+2)])#                                       AZ   181
macro:(gridz:,[QX(grid:($1)+3)])#                                       AZ   182
macro:(gridv:,[QX(grid:($1)+4)])#      Grid volume element              AZ   183
#                                                                       AZ   184
macro:(nsteps:,5)#                     Increment: 5 words/record        AZ   185
macro:(incrs:,$1=$1+nsteps:)#                                           AZ   186
macro:(gets:,MARKS+nsteps:*((($1-1)*NPTS+($2-1))*NPTS+$3-1))# Gets su   AZ   187
macro:(surf:,[ifelse:($1,,SS,$1)])#                                     AZ   188
macro:(surfx:,[QX(surf:($1)+1)])#      Surf grid pt coordinates         AZ   189
macro:(surfy:,[QX(surf:($1)+2)])#                                       AZ   190
macro:(surfz:,[QX(surf:($1)+3)])#                                       AZ   191
macro:(surfs:,[QX(surf:($1)+4)])#      Surf grid pt weight element      AZ   192
macro:(surff:,[QX(surf:($1)+5)])#      Face address                     AZ   193
#                                                                       AZ   194
macro:(incrv:,$1=$1+NINT(vertl:($1))+4)#                                AZ   195
macro:(vertex:,[ifelse:($1,,VV,$1)])#                                   AZ   196
macro:(vertx:,[QX(vertex:($1)+1)])#    Coordinates of vertex            AZ   197
macro:(verty:,[QX(vertex:($1)+2)])#                                     AZ   198
macro:(vertz:,[QX(vertex:($1)+3)])#                                     AZ   199
macro:(vertl:,[QX(vertex:($1)+4)])#    Length of face list              AZ   200
macro:(vertf:,[QX(vertex:($1)+4)])#    Face addresses (list of)         AZ   201
#                                                                       AZ   202
macro:(nstepe:,9)#                     Increment: 9 words/record        AZ   203
macro:(incre:,$1=$1+nstepe:)#                                           AZ   204
macro:(edge:,[ifelse:($1,,EE,$1)])#                                     AZ   205
macro:(edgex1:,[QX(edge:($1)+1)])#     Extremite 1 de l'arete           AZ   206
macro:(edgey1:,[QX(edge:($1)+2)])#                                      AZ   207
macro:(edgez1:,[QX(edge:($1)+3)])#                                      AZ   208
macro:(edgex2:,[QX(edge:($1)+4)])#     Extremite 2 de l'arete           AZ   209
macro:(edgey2:,[QX(edge:($1)+5)])#                                      AZ   210
macro:(edgez2:,[QX(edge:($1)+6)])#                                      AZ   211
macro:(edgef1:,[QX(edge:($1)+7)])#     Face addresses                   AZ   212
macro:(edgef2:,[QX(edge:($1)+8)])#                                      AZ   213
macro:(edgefl:,[QX(edge:($1)+9)])#     Flag to know if edge exits (us   AZ   214
#                                                                       AZ   215
macro:(nstepc:,10)#                    Increment: 10 words/record       AZ   216
macro:(incrc:,$1=$1+nstepc:)#                                           AZ   217
macro:(cut:,[ifelse:($1,,CC,$1)])#                                      AZ   218
macro:(cutx:,[QX(cut:($1)+1)])#        Cutting plane normal             AZ   219
macro:(cuty:,[QX(cut:($1)+2)])#                                         AZ   220
macro:(cutz:,[QX(cut:($1)+3)])#                                         AZ   221
macro:(cutd:,[QX(cut:($1)+4)])#        Distance from cutting plane to   AZ   222
macro:(cutf:,[QX(cut:($1)+5)])#        Oriented face address            AZ   223
macro:(cutf2:,[QX(cut:($1)+6)])#       Second face address              AZ   224
macro:(cutcx:,[QX(cut:($1)+7)])#       Coordinates of center of mass    AZ   225
macro:(cutcy:,[QX(cut:($1)+8)])#                                        AZ   226
macro:(cutcz:,[QX(cut:($1)+9)])#                                        AZ   227
macro:(cutn:,[QX(cut:($1)+10)])#       Number of vertices around face   AZ   228
#                                                                       AZ   229
systemheader:(AZCM,*)#                                                  AZCM   1
####################################################################### AZCM   2
#                                                                     # AZCM   3
#                         COMMON VARIABLES                            # AZCM   4
#                                                                     # AZCM   5
####################################################################### AZCM   6
extrude:(AZCOMN)#                                                       AZCM   7
#                                                                       AZCM   8
matrix:(M)#                            Metric tensor                    AZCM   9
matrix:(MINV)#                         Inverse metric tensor            AZCM  10
matrix:(B)#                            B matrix                         AZCM  11
matrix:(BINV)#                         Inverse b matrix                 AZCM  12
#                                                                       AZCM  13
vector:(NC)#                           Face normal vector in the crys   AZCM  14
vector:(PC)#                           Primary    beam    in the crys   AZCM  15
vector:(DC)#                           Diffracted beam    in the crys   AZCM  16
vector:(HM)#                           Scattering vector  (miller ind   AZCM  17
vector:(RM)#                           R-vector           (miller ind   AZCM  18
vector:(CM)#                           Center of mass                   AZCM  19
vector:(AXE)#                          Miller indices of main grid ve   AZCM  20
#                                                                       AZCM  21
dp: DBZ,DB1,DB2,DB3#                   Double precision values          AZCM  22
dp: A#                                 Absorption                       AZCM  23
#                                                                       AZCM  24
REAL LATCON(6)#                        Lattice constants                AZCM  25
REAL LAMBDA#                           Wavelength                       AZCM  26
REAL MU#                               Linear absorption coefficient    AZCM  27
REAL VOL,VOL2#                         Crystal volume cm**3             AZCM  28
REAL XMIN,XMAX#                        X crystal length                 AZCM  29
REAL COSP,SINP#                        Cosine & sine psi                AZCM  30
REAL COST,SINT#                        Cosine & sine theta              AZCM  31
REAL TBAR#                             Mean path in the crystal (= da   AZCM  32
REAL ABMIN,ABMAX,MEANAB#               Min, max and average of astar    AZCM  33
REAL TMIN,TMAX,MEANT#                  Min, max and average of tbar     AZCM  34
REAL INT#                              Intensity                        AZCM  35
REAL SIGMA#                            Sigma(intensity)                 AZCM  36
#                                                                       AZCM  37
INTEGER NREFL#                         Reflection counter               AZCM  38
INTEGER NPROC#                         Proceeded reflection counter     AZCM  39
INTEGER NFACES#                        Number of faces                  AZCM  40
INTEGER NVERT#                         Number of vertices               AZCM  41
INTEGER NEDGE#                         Number of edges                  AZCM  42
INTEGER NPTX,NPTY,NPTZ#                Number of grid division on eac   AZCM  43
INTEGER NPTS#                          Number of surface grid divisio   AZCM  44
INTEGER NADRX,NADRY,NADRZ#             Gauss weights addresses          AZCM  45
INTEGER NADRS#                         Gauss surface weights address    AZCM  46
INTEGER ISET#                          Dataset number                   AZCM  47
#                                                                       AZCM  48
# FLAGS                                                                 AZCM  49
# -----                                                                 AZCM  50
#                                                                       AZCM  51
INTEGER STORE#                         Store absorption flag            AZCM  52
INTEGER GAUSS#                         Method of calculus flag          AZCM  53
INTEGER LIST#                          List reflections flag            AZCM  54
INTEGER CALCT#                         Flag to know if sin(theta)/lam   AZCM  55
#                                                                       AZCM  56
# BUFFERS POINTERS (MARK FOR THE BEGINNING, ISTOP FOR THE END)          AZCM  57
# ------------------------------------------------------------          AZCM  58
#                                                                       AZCM  59
INTEGER MARKN#                         Beginning of noabs adress of t   AZCM  60
INTEGER ISTOPN#                        End noabs address of the qx-ar   AZCM  61
INTEGER MARKF#                         Beginning face adress of the q   AZCM  62
INTEGER ISTOPF#                        End face adress of the qx-arra   AZCM  63
INTEGER MARKG#                         Beginning grid point address i   AZCM  64
INTEGER ISTOPG#                        End grid point adress in the q   AZCM  65
INTEGER MARKV#                         Beginning vertices address of    AZCM  66
INTEGER ISTOPV#                        End vertices address of the qx   AZCM  67
INTEGER MARKE#                         Beginning edges address of the   AZCM  68
INTEGER ISTOPE#                        End edges address                AZCM  69
INTEGER MARKS#                         Beginning surface grid of the    AZCM  70
INTEGER ISTOPS#                        End surface grid address of th   AZCM  71
INTEGER MARKT#                         Beginning of howell's vertices   AZCM  72
INTEGER ISTOPT#                        End of howell's vertices         AZCM  73
INTEGER MARKC,MARKD#                   Beginning of howell's faces      AZCM  74
INTEGER ISTOPC,ISTOPD#                 End of howell's faces            AZCM  75
#                                                                       AZCM  76
# INPUT/OUTPUT VARIABLES                                                AZCM  77
# ----------------------                                                AZCM  78
#                                                                       AZCM  79
INTEGER IP,IPP,PSIZE,PSIZO#            Input/output pointers            AZCM  80
INTEGER IWANT(200),IREL(202),KEY(4)#   Indexpkt: pointers               AZCM  81
#                                                                       AZCM  82
# COMMONS                                                               AZCM  83
# -------                                                               AZCM  84
#                                                                       AZCM  85
COMMON/COMDAZ/$#                                                        AZCM  86
M,MINV,B,BINV,AXE,NC,PC,DC,HM,RM,CM,$#                                  AZCM  87
A,DBZ,DB1,DB2,DB3#                                                      AZCM  88
COMMON/COMFAZ/$#                                                        AZCM  89
LATCON,LAMBDA,MU,VOL,VOL2,COSP,SINP,COST,SINT,XMIN,XMAX,$#              AZCM  90
TBAR,TMIN,TMAX,MEANT,ABMIN,ABMAX,MEANAB,INT,SIGMA#                      AZCM  91
#                                                                       AZCM  92
COMMON/COMIAZ/$#                                                        AZCM  93
ISET,STORE,LIST,GAUSS,$#                                                AZCM  94
NFACES,MARKF,ISTOPF,NVERT,MARKV,ISTOPV,$#                               AZCM  95
MARKG,ISTOPG,MARKS,ISTOPS,NEDGE,MARKE,ISTOPE,$#                         AZCM  96
MARKT,ISTOPT,MARKN,ISTOPN,$#                                            AZCM  97
MARKC,ISTOPC,MARKD,ISTOPD,$#                                            AZCM  98
NPTX,NPTY,NPTZ,NPTS,NADRX,NADRY,NADRZ,NADRS,$#                          AZCM  99
IP,IPP,PSIZE,PSIZO,IWANT,IREL,KEY,$#                                    AZCM 100
CALCT,NREFL,NPROC#                                                      AZCM 101
endext:#                                                                AZCM 102
#                                                                       AZCM 103
systemheader:(AZ00)#                                                    AZ00   1
####################################################################### AZ00   2
#                                                                     # AZ00   3
# MAIN PROGRAM: ANALYSIS OF INPUT CARDS                               # AZ00   4
# ------------------------------------------------------------------- # AZ00   5
# LSABS LINE:                                                         # AZ00   6
# - FIELD X  : MU                                                     # AZ00   7
# - FIELD X+1: ABSORPTION COEFFICIENT                                 # AZ00   8
# - FIELD X  : DSET                                                   # AZ00   9
# - FIELD X+1: DATA SET NUMBER                                        # AZ00  10
# - FIELD X  : <ANAL>(ITICAL) OR GAUSS METHOD OF INTEGERATION         # AZ00  11
# - FIELD X  : <STORE>        OR APPLY CORRECTION TO INTENSITY        # AZ00  12
# - FIELD X  : <NOLIST>       OR LIST  TO LIST REFLECTIONS            # AZ00  13
# ------------------------------------------------------------------- # AZ00  14
# CONTINUATION LINES:                                                 # AZ00  15
# - FACE:                                                             # AZ00  16
#   OPTIONNAL, IF PRESENT, AT LEAST 4 AND AT MOST 20                  # AZ00  17
#   * FIELD 1 -> 3: MILLER INDICES                                    # AZ00  18
#   * FIELD 4     : DISTANCE FORM FACE TO CRYSTAL CENTER IN [MM]      # AZ00  19
#   * FIELD 5     : ESD OF DISTANCE                                   # AZ00  20
#   * FIELD 6     : WEIGHT OF DISTANCE                                # AZ00  21
# - SPHERE:                                                           # AZ00  22
#   OPTIONNAL, NOT COMPATIBLE WITH FACE CARDS, NOT COMPATIBLE WITH    # AZ00  23
#   GAUSSIAN INTEGRATION                                              # AZ00  24
#   * FIELD 1     : RADIUS OF SPHERE                                  # AZ00  25
#   * FIELD 2     : ESD OF RADIUS                                     # AZ00  26
#   * FIELD 3     : WEIGHT OF RADIUS                                  # AZ00  27
# - SHAPE:                                                            # AZ00  28
#   OPTIONNAL, NOT COMPATIBLE WITH ANALYTICAL INTEGRATION, DEFAULT    # AZ00  29
#   VALUE: GENERAL                                                    # AZ00  30
#   * FIELD 1     : PLATE OR NEEDLE                                   # AZ00  31
#   * FIELD 2 -> 4: MILLER INDICES                                    # AZ00  32
# - GRID:                                                             # AZ00  33
#   OPTIONNAL, NOT COMPATIBLE WITH ANALYTICAL INTEGRATION, DEFAULT    # AZ00  34
#   VALUE: 16 POINTS ON EACH AXIS                                     # AZ00  35
#   * FIELD 1 -> 3: NUMBER OF GRID POINTS ON AXIS 1 TO 3              # AZ00  36
# - SGRID:                                                            # AZ00  37
#   (IDEM)                                                            # AZ00  38
#   * FIELD 1     : NUMBER OF SURFACE GRID POINTS (N*N)               # AZ00  39
# - NOABS:                                                            # AZ00  40
#   OPTIONNAL, MAY BE USED MORE THAN ONCE                             # AZ00  41
#   * FIELD 1     : NUMBER OF FIRST REFLECTION TO BE SKIPPED          # AZ00  42
#   * FIELD 2     : NUMBER OF LAST  REFLECTION TO BE SKIPPED (IF PRE- # AZ00  43
#                   SENT, ELSE LAST IS SET TO FIRST)                  # AZ00  44
#                                                                     # AZ00  45
####################################################################### AZ00  46
SUBROUTINE AZ00#                                                        AZ00  47
#                                                                       AZ00  48
include:(AACOMN)#                                                       AZ00  49
include:(AZCOMN)#                                                       AZ00  50
#                                                                       AZ00  51
dp: DET#                               Determinant                      AZ00  52
dp: DTEMP#                             General purpose floating point   AZ00  53
INTEGER SHAPE#                         Shape of the crystal (= 0: gen   AZ00  54
INTEGER KEY1#                          General purpose syntax analysi   AZ00  55
INTEGER MORE#                          Is there more reflections to b   AZ00  56
INTEGER I,J,K,ILEN#                    General purpose loop indexes     AZ00  57
INTEGER NN#                            Main noabs pointer               AZ00  58
INTEGER FF#                            Main faces pointer               AZ00  59
#                                                                       AZ00  60
intdata:(NBPTS,[ 6,8,10,12,14,16,24,32])#                               AZ00  61
intdata:(ADRESS,[60,0,8,18,30,44,66,90])#                               AZ00  62
chardata:(AZ000,' LSABS Control Parameters-')#                          AZ00  63
chardata:(AZ001,DSET   MU     STORE  APPLY  LIST   NOLIST ANAL   $#     AZ00  64
GAUSS  PLATE  NEEDLE NONE   )#                                          AZ00  65
chardata:(AZ002, FACE SPHERE SHAPE GRID SGRID NOABS END)#               AZ00  66
chardata:(AZ003," Dataset                             1")#              AZ00  67
chardata:(AZ004," Absorption calculation                   ")#          AZ00  68
chardata:(AZ005," Store absorption and derivatives   No")#              AZ00  69
chardata:(AZ007," Crystal shape                 General")#              AZ00  70
chardata:(AZ008," Main axis                                          ")#AZ00  71
chardata:(AZ009," Number of divisions on axis                   ")#     AZ00  72
chardata:(AZ00A,"Yes  Plate Needle (Main) (Reciprocal) (Direct)")#      AZ00  73
chardata:(AZ00B," Reflection absorption summary")#                      AZ00  74
chardata:(AZ00C,"   Tbar min     Tbar max    Mean Tbar")#               AZ00  75
chardata:(AZ00D,"    A*  min      A*  max    Mean   A*")#               AZ00  76
chardata:(AZ00E," Sphere radius  xx.xxxxxx   Esd  xx.xxxxxx")#          AZ00  77
realdata:(FM1,[300313.,340313.,380313.])#                               AZ00  78
realdata:(FM2,[111052.,241052.,371052.])#                               AZ00  79
realdata:(FM3,[250962.,420962.,620942.])#                               AZ00  80
datastuff:#                                                             AZ00  81
#                                                                       AZ00  82
# INITIALIZATION: GET MEMORY                                            AZ00  83
# --------------------------                                            AZ00  84
#                                                                       AZ00  85
IOMARK(1)=QXSTR#                       Input buffer                     AZ00  86
IOMARK(2)=IOMARK(1)+bdfbuf:#           Output buffer                    AZ00  87
IOMARK(3)=IOMARK(2)+bdfbuf:#           Output buffer scratch            AZ00  88
incrqx:(MARKF,IOMARK(3)+bdfbuf:,AZ0001)# Request memory buffers         AZ00  89
FILEXT(IOUNIT(3))='XX1'#               Extension for scratch            AZ00  90
#                                                                       AZ00  91
# INITIALIZATION: SET DEFAULT VALUES                                    AZ00  92
# ----------------------------------                                    AZ00  93
#                                                                       AZ00  94
DBZ=dble:(0.0);DB1=dble:(1.0);DB2=dble:(2.0);DB3=dble:(3.0)#            AZ00  95
NREFL=0#                               Reflection nb 0                  AZ00  96
NPROC=0#                               No reflection proceeded          AZ00  97
ISET=1#                                Data set 1                       AZ00  98
GAUSS=no:#                             Analytical method                AZ00  99
STORE=yes:#                            Store absorption                 AZ00 100
LIST=no:#                              Don't list reflections           AZ00 101
MU=-1.0#                               Mu on bdf or to be calculated    AZ00 102
SHAPE=0#                               General shape                    AZ00 103
VOL=0.0#                               Clear volume                     AZ00 104
NPTX=16;NADRX=44#                      16 grid points on x direction    AZ00 105
NPTY=16;NADRY=44#                      16 grid points on y direction    AZ00 106
NPTZ=16;NADRZ=44#                      16 grid points on z direction    AZ00 107
NPTS=16;NADRS=44#                      256 surface grid points          AZ00 108
DO I=1,3;AXE(I)=DBZ#                   General crystal shape            AZ00 109
TMIN=large:;TMAX=0.0;MEANT=0.0#        Mean path statistics             AZ00 110
ABMIN=large:;ABMAX=0.0;MEANAB=0.0#     Absorption statistics            AZ00 111
#                                                                       AZ00 112
# LSABS LINE ANALYSIS                                                   AZ00 113
# -------------------                                                   AZ00 114
#                                                                       AZ00 115
FOR(I=1;BUFIN(I)<voidflg:;I=I+1)#      Loop over fields                 AZ00 116
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ00 117
stringlim:(I,J,K);ILEN=MIN(6,K-J)#     Extract field limits             AZ00 118
compchar:(CHRIN,J,AZ001,1,ILEN,7,NAZ001,KEY1)#                          AZ00 119
IF(KEY1==0) iquit:(AZ0003)#            Unrecognized keyword             AZ00 120
IF(KEY1==1)#                           Dataset                          AZ00 121
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ00 122
IF(BUFIN(I+1)<=voidflg:) NEXT#         Skip if no number                AZ00 123
ISET=NINT(BUFIN(I+1));I=I+1#           Data set number                  AZ00 124
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ00 125
IF(KEY1==2)#                           Mu                               AZ00 126
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ00 127
IF(BUFIN(I+1)<=voidflg:) NEXT#         Skip if no number                AZ00 128
MU=BUFIN(I+1);I=I+1#                   Store mu                         AZ00 129
IF(MU<-small:)iquit:(AZ0002)#          Negative forbidden               AZ00 130
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ00 131
IF(KEY1==4)STORE=no:#                  Do not store                     AZ00 132
IF(KEY1==5)#                           Test for list option             AZ00 133
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ00 134
LIST=9999999#                          List all reflections             AZ00 135
IF(BUFIN(I+1)<=voidflg:) NEXT#         Skip if no number                AZ00 136
LIST=NINT(BUFIN(I+1));I=I+1#           Set list limit                   AZ00 137
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ00 138
IF(KEY1==8) GAUSS=yes:#                Gaussian method                  AZ00 139
IF(KEY1==11)GAUSS=2#                   No absorption calc. just mu      AZ00 140
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ00 141
copyfile:(1,3,lrhist:,decr:(lrcell:))# Just copy until lrcell:          AZ00 142
#                                                                       AZ00 143
# GET FACE, SHAPE, GRID, SGRID AND CELL LINES                           AZ00 144
# -------------------------------------------                           AZ00 145
#                                                                       AZ00 146
ISTOPF=MARKF#                          Set face pointers                AZ00 147
FF=MARKF#                                                               AZ00 148
NFACES=0#                              No face yet                      AZ00 149
REPEAT#                                Loop over input cards            AZ00 150
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ00 151
readline:(AZ002,NAZ002)#               Read one card                    AZ00 152
IF(LINID==1)#                          ********** face line *******     AZ00 153
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ00 154
incrqx:(ISTOPF,ISTOPF+nstepf:,AZ0005)# Increase mem for one face        AZ00 155
facehm:(FF)=BUFIN(1)#                  Store miller indices             AZ00 156
facekm:(FF)=BUFIN(2)#                                                   AZ00 157
facelm:(FF)=BUFIN(3)#                                                   AZ00 158
faced:(FF)=BUFIN(4)#                   Store distance to face           AZ00 159
ifnm:(BUFIN(5))facesd:(FF)=BUFIN(5)#   Store esd of distance            AZ00 160
ELSE           facesd:(FF)=BUFIN(4)/20.# Assume 5% esd                  AZ00 161
ifnm:(BUFIN(6))facew:(FF)=BUFIN(6)#    Store weight                     AZ00 162
ELSE           facew:(FF)=1./facesd:(FF)**2# Assume 1/esd**2            AZ00 163
incrf:(FF);NFACES=NFACES+1#            Increment face pointer and cou   AZ00 164
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ00 165
ELSE IF(LINID==2)#                     ******** sphere line ********    AZ00 166
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AZ00 167
IF(GAUSS !=no:)iquit:(AZ0018)#         Only analytical with sphere      AZ00 168
IF(FF>MARKF)iquit:(AZ0019)#            More than one sphere line        AZ00 169
incrqx:(ISTOPF,ISTOPF+nstepf:,AZ0005)# Increase mem for one face        AZ00 170
facer:     =BUFIN(1)#                  Store radius                     AZ00 171
ifnm:(BUFIN(2))facesd:(FF)=BUFIN(2)#   Store esd of radius              AZ00 172
ELSE           facesd:(FF)=BUFIN(1)/20.# Assume 5% of radius            AZ00 173
ifnm:(BUFIN(3))facew:(FF)=BUFIN(3)#    Store weight                     AZ00 174
ELSE           facew:(FF)=1./facesd:(FF)**2# Assume 1/esd**2            AZ00 175
incrf:(FF);NFACES=1#                   Increment face pointer           AZ00 176
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AZ00 177
ELSE IF(LINID==3)#                     ********* shape line ********    AZ00 178
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AZ00 179
IF(GAUSS !=yes:)iquit:(AZ0006)#        Shape line incompatible with a   AZ00 180
IF(BUFIN(1)>=voidflg:)iquit:(AZ0007)#  Syntax error in shape card       AZ00 181
stringlim:(1,J,K);ILEN=MIN(6,K-J)#     Extract field limits             AZ00 182
compchar:(CHRIN,J,AZ001,1,ILEN,7,NAZ001,KEY1)#                          AZ00 183
IF(KEY1<9)iquit:(AZ0008)#              Unrecognized keyword             AZ00 184
IF(KEY1==9)SHAPE=-1#                   Plate shaped                     AZ00 185
ELSE SHAPE=1#                          Pin   shaped                     AZ00 186
DO I=1,3;AXE(I)=dble:(BUFIN(I+1))#     Main shape direction             AZ00 187
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AZ00 188
ELSE IF(LINID==4)#                     ********** grid line *********   AZ00 189
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AZ00 190
IF(GAUSS !=yes:)iquit:(AZ0009)#        Grid line incompatible with an   AZ00 191
DO I=1,8#                              Loop over all possible grids     AZ00 192
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AZ00 193
IF(NINT(BUFIN(1))==NBPTS(I))#          Find x division and address      AZ00 194
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AZ00 195
NPTX=NBPTS(I)#                                                          AZ00 196
NADRX=ADRESS(I)#                                                        AZ00 197
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AZ00 198
IF(NINT(BUFIN(2))==NBPTS(I))#          Find y division and address      AZ00 199
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AZ00 200
NPTY=NBPTS(I)#                                                          AZ00 201
NADRY=ADRESS(I)#                                                        AZ00 202
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AZ00 203
IF(NINT(BUFIN(3))==NBPTS(I))#          Find z division and address      AZ00 204
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AZ00 205
NPTZ=NBPTS(I)#                                                          AZ00 206
NADRZ=ADRESS(I)#                                                        AZ00 207
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AZ00 208
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AZ00 209
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AZ00 210
ELSE IF(LINID==5)#                     ********** sgrid line ********   AZ00 211
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AZ00 212
IF(GAUSS !=yes:)iquit:(AZ0009)#        Grid and sgrid lines don't all   AZ00 213
DO I=1,8#                                                               AZ00 214
IF(NINT(BUFIN(1))==NBPTS(I))#          Find surf division and address   AZ00 215
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AZ00 216
NPTS=NBPTS(I)#                                                          AZ00 217
NADRS=ADRESS(I)#                                                        AZ00 218
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AZ00 219
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AZ00 220
ELSE BREAK#                            Other card read                  AZ00 221
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ00 222
IF((NFACES<4)&(NFACES>1))iquit:(AZ0011)# Less than four faces           AZ00 223
IF(NFACES>20)iquit:(AZ0012)#           More than twenty faces           AZ00 224
#                                                                       AZ00 225
# GETS EVERYTHING FROM FILE IF BDF PROVIDED                             AZ00 226
# -----------------------------------------                             AZ00 227
#                                                                       AZ00 228
movechar:(AZ000,NAZ000,CHROT,2,NAZ000-2,2)# Insert underline            AZ00 229
writeline:(1,AZ000,NAZ000-1,2,3)#      Print bondla header              AZ00 230
IF(GAUSS==0)movechar:('Analytical',1,AZ004,32,10,0)# Store mode         AZ00 231
IF(GAUSS==1)movechar:('Gaussian  ',1,AZ004,32,10,0)# Store mode         AZ00 232
IF(GAUSS==2)movechar:('None      ',1,AZ004,32,10,0)# Store mode         AZ00 233
writeline:(0,AZ004,NAZ004,3,3)#        Print calculation mode           AZ00 234
CALL AZ33#                             Get mu, lambda, lattice csts,    AZ00 235
#                                                                       AZ00 236
# WRITE EVERYTHING FOR USER                                             AZ00 237
# -------------------------                                             AZ00 238
#                                                                       AZ00 239
ncodefld:(REAL(ISET),1,AZ003,380413.,1)#                                AZ00 240
writeline:(0,AZ003,NAZ003,3,3)#        Output dataset number            AZ00 241
ISET=ISET*1000#                        Dataset for lrrefl:              AZ00 242
IF(STORE==yes:)movechar:(AZ00A,1,AZ005,36,3,0)#                         AZ00 243
writeline:(0,AZ005,NAZ005,3,3)#        Output storage or store correc   AZ00 244
#                                                                       AZ00 245
# WRITE EVERYTHING FOR USER (CONTINUED)                                 AZ00 246
# -------------------------------------                                 AZ00 247
#                                                                       AZ00 248
IF(GAUSS==yes:)#                                                        AZ00 249
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AZ00 250
IF(SHAPE<0)#                           Crystal shape                    AZ00 251
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AZ00 252
movechar:(AZ00A,4,AZ007,32,7,0)#       Plate                            AZ00 253
movechar:(AZ00A,26,AZ008,40,12,0)#     (reciprocal)                     AZ00 254
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AZ00 255
IF(SHAPE>0)#                                                            AZ00 256
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AZ00 257
movechar:(AZ00A,11,AZ007,32,7,0)#      Pin                              AZ00 258
movechar:(AZ00A,39,AZ008,40,8,0)#      (direct)                         AZ00 259
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AZ00 260
writeline:(0,AZ007,NAZ007,3,3)#        Principal shape axis if presen   AZ00 261
DO I=1,3;BUFOT(I)=sngl:(AXE(I))#                                        AZ00 262
ncodefld:(BUFOT,1,AZ008,FM1,3)#                                         AZ00 263
IF(SHAPE !=0)writeline:(0,AZ008,NAZ008,3,3)# Do write it                AZ00 264
ncodefld:(REAL(1),1,AZ009,300113.,1)#  X grid points                    AZ00 265
ncodefld:(REAL(NPTX),1,AZ009,380413.,1)#                                AZ00 266
writeline:(0,AZ009,NAZ009,3,3)#                                         AZ00 267
ncodefld:(REAL(2),1,AZ009,300113.,1)#  Y grid points                    AZ00 268
ncodefld:(REAL(NPTY),1,AZ009,380413.,1)#                                AZ00 269
writeline:(0,AZ009,NAZ009,3,3)#                                         AZ00 270
ncodefld:(REAL(3),1,AZ009,300113.,1)#  Z grid points                    AZ00 271
ncodefld:(REAL(NPTZ),1,AZ009,380413.,1)#                                AZ00 272
IF(SHAPE !=0)movechar:(AZ00A,19,AZ009,40,6,0)# Axis grid points         AZ00 273
writeline:(0,AZ009,NAZ009,3,3)#                                         AZ00 274
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AZ00 275
IF(GAUSS==2)CALL AZ31(no:,SHAPE)#      Close lrrefl: logical record     AZ00 276
ELSE#                                                                   AZ00 277
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AZ00 278
#                                                                       AZ00 279
# NOW WE HAVE: LATTICE CONSTANTS, METRIC TENSOR, LAMBDA, MU             AZ00 280
# ---------------------------------------------------------             AZ00 281
#                                                                       AZ00 282
invert:(M,MINV,DET)#                   Get metric tensor in reciproca   AZ00 283
IF(dabs:(DET)<dsmall:)iquit:(AZ0013)#  Very unreasonable lattice cons   AZ00 284
CALL AZ41(SHAPE)#                      Computes b and prints m and b    AZ00 285
#                                                                       AZ00 286
# NOW WE HAVE: LAMBDA, MU, M AND B (READY TO START !!)                  AZ00 287
# ----------------------------------------------------                  AZ00 288
#                                                                       AZ00 289
IF(NFACES>1)#                          Polyhedral crystal               AZ00 290
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AZ00 291
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    For each face...                 AZ00 292
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AZ00 293
NC(1)=dble:(facehm:(FF))#                                               AZ00 294
NC(2)=dble:(facekm:(FF))#                                               AZ00 295
NC(3)=dble:(facelm:(FF))#              Get miller indices and           AZ00 296
postm:(B,NC,NC);norm:(NC,NC,DTEMP)#    Transform them in b base         AZ00 297
faceh:(FF)=sngl:(NC(1))#                                                AZ00 298
facek:(FF)=sngl:(NC(2))#                                                AZ00 299
facel:(FF)=sngl:(NC(3))#                                                AZ00 300
facecx:(FF)=0.0;facecy:(FF)=0.0;facecz:(FF)=0.0#                        AZ00 301
facen:(FF)=0.0#                                                         AZ00 302
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AZ00 303
CALL AZ10#                             Gets vertices and grid or edge   AZ00 304
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AZ00 305
ELSE#                                  Spherical crystal                AZ00 306
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AZ00 307
CALL AZ16#                             Set interpolation                AZ00 308
ncodefld:(facer:,1,AZ00E,FM3(1),1)#    Encode radius                    AZ00 309
ncodefld:(facesd:(MARKF),1,AZ00E,FM3(2),1)# Encode radius esd           AZ00 310
writeline:(1,AZ00E,NAZ00E,3,3)#        Print line                       AZ00 311
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AZ00 312
#                                                                       AZ00 313
# PROCEED THROUGH REFLECTIONS                                           AZ00 314
# ---------------------------                                           AZ00 315
#                                                                       AZ00 316
IF(GAUSS==yes:)MARKN=ISTOPS#                                            AZ00 317
ELSE MARKN=ISTOPE#                                                      AZ00 318
ISTOPN=MARKN#                                                           AZ00 319
NN=MARKN#                                                               AZ00 320
REPEAT#                                Loop over noabs cards            AZ00 321
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AZ00 322
IF(LINID==6)#                          Noabs line                       AZ00 323
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AZ00 324
incrqx:(ISTOPN,ISTOPN+nstepn:,AZ0014)# Not enough memory for noabs bu   AZ00 325
IF(BUFIN(1)>voidflg:)#                                                  AZ00 326
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AZ00 327
minn:(NN)=BUFIN(1)#                    Min refl nb to skip              AZ00 328
IF(BUFIN(2)>voidflg:)maxn:(NN)=BUFIN(2)# Max refl nb to skip            AZ00 329
ELSE maxn:(NN)=BUFIN(1)#               Max set to min if not present    AZ00 330
incrn:(NN)#                            Increment pointer                AZ00 331
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AZ00 332
ELSE iquit:(AZ0015)#                   No reflection number             AZ00 333
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AZ00 334
ELSE IF(LINID==7)BREAK#                End line                         AZ00 335
ELSE iquit:(AZ0016)#                   Forbidden input line             AZ00 336
readline:(AZ002,NAZ002)#               Read next card                   AZ00 337
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AZ00 338
CALL AZ31(yes:,SHAPE)#                 Open lrrefl: logical record      AZ00 339
IF(LIST>0)#                                                             AZ00 340
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AZ00 341
LINRM=8;writeline:(2,BLANK,1,3,3)#     3 blank lines                    AZ00 342
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AZ00 343
REPEAT#                                Loop over all reflections        AZ00 344
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AZ00 345
CALL AZ32(MORE)#                       Read one reflection              AZ00 346
IF(MORE==no:)BREAK#                    Exit if last                     AZ00 347
CALL AZ20#                             Treat reflection                 AZ00 348
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AZ00 349
CALL AZ31(no:,SHAPE)#                  Close lrrefl: logical record     AZ00 350
#                                                                       AZ00 351
# PRINT ABSORPTION AND TBAR STATISTICS                                  AZ00 352
# ------------------------------------                                  AZ00 353
#                                                                       AZ00 354
LINRM=7#                                                                AZ00 355
writeline:(2,AZ00B,NAZ00B,3,3)#        Write summary header             AZ00 356
BUFOT(1)=ABMIN;BUFOT(2)=ABMAX;BUFOT(3)=MEANAB/REAL(NPROC)#              AZ00 357
ncodefld:(BUFOT,1,CHROT,FM2,3)#                                         AZ00 358
writeline:(1,AZ00D,NAZ00D,2,3)#                                         AZ00 359
BUFOT(1)=TMIN;BUFOT(2)=TMAX;BUFOT(3)=MEANT/REAL(NPROC)#                 AZ00 360
ncodefld:(BUFOT,1,CHROT,FM2,3)#                                         AZ00 361
writeline:(3,AZ00C,NAZ00C,2,3)#                                         AZ00 362
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AZ00 363
#                                                                       AZ00 364
RETURN#                                                                 AZ00 365
END#                                                                    AZ00 366
#                                                                       AZ00 367
# MATRIX AND VECTOR SUBROUTINES                                         AZ00 368
# -----------------------------                                         AZ00 369
#                                                                       AZ00 370
systemheader:(AZ01)#                                                    AZ01   1
####################################################################### AZ01   2
#                                                                     # AZ01   3
# CROSS PRODUCT: V3 = V1 ^ V2;  V3 MAY BE EQUAL TO V1 OR V2           # AZ01   4
#                                                                     # AZ01   5
####################################################################### AZ01   6
SUBROUTINE AZ01(V1,V2,V3)#                                              AZ01   7
#                                                                       AZ01   8
vector:(V1);vector:(V2);vector:(V3);vector:(V4)#                        AZ01   9
INTEGER I#                                                              AZ01  10
#                                                                       AZ01  11
V4(1)=V1(2)*V2(3)-V1(3)*V2(2)#                                          AZ01  12
V4(2)=V1(3)*V2(1)-V1(1)*V2(3)#                                          AZ01  13
V4(3)=V1(1)*V2(2)-V1(2)*V2(1)#                                          AZ01  14
DO I=1,3;V3(I)=V4(I)#                                                   AZ01  15
RETURN#                                                                 AZ01  16
END#                                                                    AZ01  17
#                                                                       AZ01  18
#                                                                       AZ01  19
systemheader:(AZ02)#                                                    AZ02   1
####################################################################### AZ02   2
#                                                                     # AZ02   3
# INVERSION DE MATRICES 3 * 3 [M2 MAY BE EQUAL TO M1]                 # AZ02   4
#                                                                     # AZ02   5
####################################################################### AZ02   6
SUBROUTINE AZ02(M1,M2,DET)#                                             AZ02   7
#                                                                       AZ02   8
INTEGER I,J#                                                            AZ02   9
matrix:(M1);matrix:(M2);matrix:(M3)#                                    AZ02  10
dp: DET#                                                                AZ02  11
#                                                                       AZ02  12
dbledata:(DBZ,0.0D0)#                                                   AZ02  13
datastuff:#                                                             AZ02  14
#                                                                       AZ02  15
DET=M1(1,1)*M1(2,2)*M1(3,3)+M1(1,2)*M1(2,3)*M1(3,1)$#                   AZ02  16
+M1(1,3)*M1(3,2)*M1(2,1)-M1(1,3)*M1(2,2)*M1(3,1)$#                      AZ02  17
-M1(1,2)*M1(2,1)*M1(3,3)-M1(1,1)*M1(2,3)*M1(3,2)#                       AZ02  18
IF(DET==DBZ)#                                                           AZ02  19
DO I=1,3#                                                               AZ02  20
DO J=1,3#                                                               AZ02  21
M2(I,J)=DBZ#                                                            AZ02  22
ELSE#                                                                   AZ02  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ02  24
M3(1,1)=(M1(2,2)*M1(3,3)-M1(2,3)*M1(3,2))/ DET#                         AZ02  25
M3(2,1)=(M1(3,1)*M1(2,3)-M1(2,1)*M1(3,3))/ DET#                         AZ02  26
M3(3,1)=(M1(2,1)*M1(3,2)-M1(2,2)*M1(3,1))/ DET#                         AZ02  27
M3(1,2)=(M1(1,3)*M1(3,2)-M1(1,2)*M1(3,3))/ DET#                         AZ02  28
M3(2,2)=(M1(1,1)*M1(3,3)-M1(1,3)*M1(3,1))/ DET#                         AZ02  29
M3(3,2)=(M1(3,1)*M1(1,2)-M1(1,1)*M1(3,2))/ DET#                         AZ02  30
M3(1,3)=(M1(1,2)*M1(2,3)-M1(2,2)*M1(1,3))/ DET#                         AZ02  31
M3(2,3)=(M1(1,3)*M1(2,1)-M1(1,1)*M1(2,3))/ DET#                         AZ02  32
M3(3,3)=(M1(2,2)*M1(1,1)-M1(2,1)*M1(1,2))/ DET#                         AZ02  33
DO I=1,3#                                                               AZ02  34
DO J=1,3#                                                               AZ02  35
M2(I,J)=M3(I,J)#                                                        AZ02  36
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ02  37
RETURN#                                                                 AZ02  38
END#                                                                    AZ02  39
#                                                                       AZ02  40
#                                                                       AZ02  41
systemheader:(AZ03)#                                                    AZ03   1
####################################################################### AZ03   2
#                                                                     # AZ03   3
# MATRIX*VECTOR=VECTOR [VECTOR(INPUT) MAY BE EQUAL TO VECTOR(OUTPUT)] # AZ03   4
#                                                                     # AZ03   5
####################################################################### AZ03   6
SUBROUTINE AZ03(M1,T2,T3)#                                              AZ03   7
#                                                                       AZ03   8
matrix:(M1)#                                                            AZ03   9
vector:(T2);vector:(T3);vector:(T4)#                                    AZ03  10
INTEGER I,J#                                                            AZ03  11
#                                                                       AZ03  12
dbledata:(DBZ,0.0D0)#                                                   AZ03  13
datastuff:#                                                             AZ03  14
#                                                                       AZ03  15
DO I=1,3#                                                               AZ03  16
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ03  17
T4(I)=DBZ#                                                              AZ03  18
DO J=1,3#                                                               AZ03  19
T4(I)=T4(I)+M1(I,J)*T2(J)#                                              AZ03  20
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ03  21
DO I=1,3;T3(I)=T4(I)#                                                   AZ03  22
RETURN#                                                                 AZ03  23
END#                                                                    AZ03  24
#                                                                       AZ03  25
#                                                                       AZ03  26
systemheader:(AZ04)#                                                    AZ04   1
####################################################################### AZ04   2
#                                                                     # AZ04   3
# TRANSPOSE(MATRIX) [MATRIX(INPUT) MAY BE EQUAL TO MATRIX(OUTPUT)]    # AZ04   4
#                                                                     # AZ04   5
####################################################################### AZ04   6
SUBROUTINE AZ04(M1,M2)#                                                 AZ04   7
#                                                                       AZ04   8
matrix:(M1);matrix:(M2);matrix:(M3)#                                    AZ04   9
INTEGER I,J#                                                            AZ04  10
#                                                                       AZ04  11
DO I=1,3#                                                               AZ04  12
DO J=1,3#                                                               AZ04  13
M3(J,I)=M1(I,J)#                                                        AZ04  14
DO I=1,3;DO J=1,3;M2(I,J)=M3(I,J)#                                      AZ04  15
RETURN#                                                                 AZ04  16
END#                                                                    AZ04  17
#                                                                       AZ04  18
#                                                                       AZ04  19
systemheader:(AZ05)#                                                    AZ05   1
####################################################################### AZ05   2
#                                                                     # AZ05   3
# R = NORME(V1);  V2 = V1/R;  V2 MAY BE EQUAL TO V1                   # AZ05   4
#                                                                     # AZ05   5
####################################################################### AZ05   6
SUBROUTINE AZ05(V1,V2,R)#                                               AZ05   7
#                                                                       AZ05   8
vector:(V1);vector:(V2)#                                                AZ05   9
dp: R#                                                                  AZ05  10
INTEGER I#                                                              AZ05  11
#                                                                       AZ05  12
dbledata:(DBZ,0.0D0)#                                                   AZ05  13
datastuff:#                                                             AZ05  14
#                                                                       AZ05  15
R=DBZ#                                                                  AZ05  16
DO I=1,3;R=R+V1(I)**2#                                                  AZ05  17
R=dsqrt:(R)#                                                            AZ05  18
IF(R==0.0)#                                                             AZ05  19
DO I=1,3;V2(I)=DBZ#                                                     AZ05  20
ELSE#                                                                   AZ05  21
DO I=1,3;V2(I)=V1(I)/R#                                                 AZ05  22
RETURN#                                                                 AZ05  23
END#                                                                    AZ05  24
#                                                                       AZ05  25
#                                                                       AZ05  26
systemheader:(AZ06)#                                                    AZ06   1
####################################################################### AZ06   2
#                                                                     # AZ06   3
# MATRIX * MATRIX = MATRIX [M3 MAY BE EQUAL TO M1 OR M2]              # AZ06   4
#                                                                     # AZ06   5
######################################################################  AZ06   6
SUBROUTINE AZ06(M1,M2,M3)#                                              AZ06   7
#                                                                       AZ06   8
matrix:(M1);matrix:(M2);matrix:(M3);matrix:(M4)#                        AZ06   9
INTEGER I,J,K#                                                          AZ06  10
#                                                                       AZ06  11
dbledata:(DBZ,0.0D0)#                                                   AZ06  12
datastuff:#                                                             AZ06  13
#                                                                       AZ06  14
DO I=1,3#                                                               AZ06  15
DO J=1,3#                                                               AZ06  16
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ06  17
M4(I,J)=DBZ#                                                            AZ06  18
DO K=1,3#                                                               AZ06  19
M4(I,J)=M4(I,J)+M1(I,K)*M2(K,J)#                                        AZ06  20
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ06  21
DO I=1,3#                                                               AZ06  22
DO J=1,3#                                                               AZ06  23
M3(I,J)=M4(I,J)#                                                        AZ06  24
RETURN#                                                                 AZ06  25
END#                                                                    AZ06  26
#                                                                       AZ06  27
#                                                                       AZ06  28
systemheader:(AZ07)#                                                    AZ07   1
####################################################################### AZ07   2
#                                                                     # AZ07   3
# SEARCH DIRECTORY ROUTINE:                                           # AZ07   4
# ------------------------------------------------------------------- # AZ07   5
# THIS ROUTINE EXTRACT THE OFFSET POINTER OF AN ITEM IN A DIRECTORY   # AZ07   6
# PACKET.                                                             # AZ07   7
# IT IS USED TO WRITE AN ITEM ON THE BDF ONLY IF IT IS MISSING BEFORE # AZ07   8
# READING THE FILE.                                                   # AZ07   9
# FOR INSTANCE: WE WANT TO WRITE THE WAVELENGTH ON THE BDF ONLY IF    # AZ07  10
# THE BDF DOESN'T CONTAIN LAMBDA. THEN WE HAVE TO PUT THE LAMBDA IDN  # AZ07  11
# IN THE UPDATE PACKET. BUT IN THIS CASE, THE IREL POINTER FOR LAMBDA # AZ07  12
# WILL ALWAYS BE POSITIVE. SO WE CAN'T KNOW IF THE WAVELENGTH WAS     # AZ07  13
# ALREADY ON BDF OR NOT. THE ONLY WAY TO KNOW IT IS TO SEEK IN THE    # AZ07  14
# INPUT DIRECTORY PACKET.                                             # AZ07  15
#                                                                     # AZ07  16
####################################################################### AZ07  17
SUBROUTINE AZ07(IP0,PSIZE0,IDN,PTR)#                                    AZ07  18
#                                                                       AZ07  19
include:(AACOMN)#                                                       AZ07  20
include:(AZCOMN)#                                                       AZ07  21
#                                                                       AZ07  22
REAL AIDN#                             Real idn value                   AZ07  23
INTEGER I#                             Loop index                       AZ07  24
INTEGER IDN#                           Integer idn value                AZ07  25
INTEGER PTR#                           Returned offset pointer in inp   AZ07  26
INTEGER IP0,PSIZE0#                    Set to ip and psize              AZ07  27
#                                                                       AZ07  28
AIDN=REAL(IDN)#                        Real value                       AZ07  29
PTR=1#                                 Reset offset pointer             AZ07  30
DO I=IP0+1,IP0+PSIZE0#                                                  AZ07  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ07  32
ifeq:(QX(I),AIDN)BREAK#                Exit if match                    AZ07  33
PTR=PTR+1#                             Increment offset pointer         AZ07  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ07  35
IF(PTR>PSIZE0)PTR=0#                   Match not found                  AZ07  36
RETURN#                                                                 AZ07  37
END#                                                                    AZ07  38
#                                                                       AZ07  39
#                                                                       AZ07  40
systemheader:(AZ10)#                                                    AZ10   1
####################################################################### AZ10   2
#                                                                     # AZ10   3
# MAIN SUBROUTINE TO BUILD CRYSTAL VERTICES, CRYSTAL EDGES AND GRID.  # AZ10   4
# CALLED ONLY BY MAIN ROUTINE AZ00                                    # AZ10   5
# ------------------------------------------------------------------- # AZ10   6
# SUBROUTINES GROUP AZ1X:                                             # AZ10   7
# - AZ11: FIND AND STORE ALL CRYSTAL VERTICES, FINDS XMIN AND XMAX    # AZ10   8
#         AND PRINT THE VERTEX REPORT.                                # AZ10   9
# - AZ12: FIND AND STORE ALL CRYSTAL EDGES. FOR EACH CRYSTAL FACE,    # AZ10  10
#         FIND AND STORE CENTER OF MASS AND NUMBER OF VERTICES AROUND # AZ10  11
#         FACE.                                                       # AZ10  12
# - AZ13: PRINT FACE, VERTEX AND EDGE REPORTS AND COMPUTE FACE SUR-   # AZ10  13
#         FACE.                                                       # AZ10  14
# - AZ15: USED ONLY FOR GAUSSIAN INTEGRATION:                         # AZ10  15
#         BUILD GRID AND CALCULATE THE CRYSTAL VOLUME                 # AZ10  16
#                                                                     # AZ10  17
####################################################################### AZ10  18
SUBROUTINE AZ10#                                                        AZ10  19
#                                                                       AZ10  20
include:(AACOMN)#                                                       AZ10  21
include:(AZCOMN)#                                                       AZ10  22
#                                                                       AZ10  23
# GETS VERTICES                                                         AZ10  24
# -------------                                                         AZ10  25
#                                                                       AZ10  26
CALL AZ11#                             Returns xmin and xmax & builds   AZ10  27
CALL AZ12#                             Build edges                      AZ10  28
CALL AZ13#                             Print report                     AZ10  29
IF(GAUSS==yes:)#                                                        AZ10  30
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ10  31
CALL AZ14#                             Finds best integration           AZ10  32
CALL AZ15#                             Compute grid                     AZ10  33
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ10  34
RETURN#                                                                 AZ10  35
END#                                                                    AZ10  36
#                                                                       AZ10  37
#                                                                       AZ10  38
systemheader:(AZ11)#                                                    AZ11   1
####################################################################### AZ11   2
#                                                                     # AZ11   3
# VERTICES BUILDER (CALLED ONCE BY AZ10):                             # AZ11   4
# ------------------------------------------------------------------- # AZ11   5
# FINDS INTERSECTIONS BETWEEN ALL THE PLANES AND STORES ONLY REAL     # AZ11   6
# VERTICES, I.E. WHICH ARE NOT OUSIDE THE CRYSTAL.                    # AZ11   7
# FINDS EXTREMITIES OF CRYSTAL ON X AXIS.                             # AZ11   8
# FIND CENTER OF MASS OF FACES.                                       # AZ11   9
# LES SOMMETS SONT STOCKES DE LA MANIERE SUIVANTE:                    # AZ11  10
# * 1 -> 3 : COORDONNEES XYZ                                          # AZ11  11
# * 4      : NOMBRE DE FACES DONT LE SOMMET EST ISSU (>2)             # AZ11  12
# * 5 -> N : ADRESSES DES FACES                                       # AZ11  13
# ------------------------------------------------------------------- # AZ11  14
# ALGORITHM USED TO FIND A VERTEX:                                    # AZ11  15
#                                                                     # AZ11  16
#                          { (H1, K1, L1) AND D3                      # AZ11  17
# 1) TAKE 3 FACES GIVEN BY { (H2, K2, L2) AND D2                      # AZ11  18
#                          { (H3, K3, L3) AND D3                      # AZ11  19
#                                                                     # AZ11  20
#    (X)                                      H1 X + K1 Y + L1 Z = D1 # AZ11  21
# 2) (Y) IS THE INTERSECTION OF THE PLANES IF H2 X + K2 Y + L2 Z = D2 # AZ11  22
#    (Z)                                      H3 X + K3 Y + L3 Z = D3 # AZ11  23
#                                                                     # AZ11  24
#                  (H1 K1 L1)                   (D1)                  # AZ11  25
# 3) DEFINE PASS = (H2 K2 L2) MATRIX AND DIST = (D2) VECTOR. THEN     # AZ11  26
#                  (H3 K3 L3)                   (D3)                  # AZ11  27
#                                                                     # AZ11  28
#                                              (x)                    # AZ11  29
#    VERTEX = (PASS)**-1 DIST    WITH VERTEX = (Y)                    # AZ11  30
#                                              (z)                    # AZ11  31
# 4) VERTEX IS A REAL CRYSTAL CORNER IF IT IS NOT OUTSIDE THE CRYSTAL # AZ11  32
#                                                                     # AZ11  33
####################################################################### AZ11  34
SUBROUTINE AZ11#                                                        AZ11  35
#                                                                       AZ11  36
include:(AACOMN)#                                                       AZ11  37
include:(AZCOMN)#                                                       AZ11  38
#                                                                       AZ11  39
INTEGER FF#                            Main face pointer                AZ11  40
INTEGER VV#                            Main vertices pointer            AZ11  41
matrix:(PASS)#                                                          AZ11  42
matrix:(MTEMP)#                        Inverse of pass                  AZ11  43
vector:(DIST)#                                                          AZ11  44
vector:(POINT)#                                                         AZ11  45
vector:(VERTEX)#                       Solution of 3)                   AZ11  46
dp: DET#                               Determinant                      AZ11  47
dp: DTEMP#                                                              AZ11  48
INTEGER F(3)#                          Face pointers                    AZ11  49
INTEGER I,J#                           Loop indexes                     AZ11  50
INTEGER ILEN#                          Number of faces vertex           AZ11  51
INTEGER NUVERT#                        New vertex flag                  AZ11  52
INTEGER VV1#                           Other vertices pointers          AZ11  53
#                                                                       AZ11  54
# FIND XMIN XMAX AND CALCULATE SURFACES                                 AZ11  55
# -------------------------------------                                 AZ11  56
#                                                                       AZ11  57
CM(1)=DBZ;CM(2)=DBZ;CM(3)=DBZ#         Clear crystal center of mass     AZ11  58
XMIN=large:#                                                            AZ11  59
XMAX=-large:#                                                           AZ11  60
NVERT=0#                               No vertices found yet            AZ11  61
MARKV=ISTOPF#                          Set vertex pointers              AZ11  62
ISTOPV=MARKV#                                                           AZ11  63
VV=MARKV#                                                               AZ11  64
FOR(F(1)=MARKF;F(1)<ISTOPF-2*nstepf:;incrf:(F(1)))# First loop over f   AZ11  65
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ11  66
PASS(1,1)=dble:(faceh:(F(1)))#                                          AZ11  67
PASS(1,2)=dble:(facek:(F(1)))#                                          AZ11  68
PASS(1,3)=dble:(facel:(F(1)))#                                          AZ11  69
DIST(1)=dble:(faced:(F(1)))#                                            AZ11  70
FOR(F(2)=F(1)+nstepf:;F(2)<ISTOPF-nstepf:;incrf:(F(2)))# First loop o   AZ11  71
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ11  72
PASS(2,1)=dble:(faceh:(F(2)))#                                          AZ11  73
PASS(2,2)=dble:(facek:(F(2)))#                                          AZ11  74
PASS(2,3)=dble:(facel:(F(2)))#                                          AZ11  75
DIST(2)=dble:(faced:(F(2)))#                                            AZ11  76
FOR(F(3)=F(2)+nstepf:;F(3)<ISTOPF;incrf:(F(3)))# First loop over face   AZ11  77
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ11  78
PASS(3,1)=dble:(faceh:(F(3)))#                                          AZ11  79
PASS(3,2)=dble:(facek:(F(3)))#                                          AZ11  80
PASS(3,3)=dble:(facel:(F(3)))#                                          AZ11  81
DIST(3)=dble:(faced:(F(3)))#                                            AZ11  82
invert:(PASS,MTEMP,DET)#                                                AZ11  83
IF(dabs:(DET)<dsmall:)NEXT#            Faces don't intersect            AZ11  84
postm:(MTEMP,DIST,VERTEX)#             Find sol of 3)                   AZ11  85
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Test if not outside the crysta   AZ11  86
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ11  87
NC(1)=dble:(faceh:(FF))#                                                AZ11  88
NC(2)=dble:(facek:(FF))#                                                AZ11  89
NC(3)=dble:(facel:(FF))#                                                AZ11  90
IF((dble:(faced:(FF))-dot:(NC,VERTEX))<-dsmall:)BREAK# Should be smal   AZ11  91
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ11  92
IF(FF<ISTOPF)NEXT#                     Intersection outside crystal     AZ11  93
IF(XMIN>sngl:(VERTEX(1)))XMIN=sngl:(VERTEX(1))# Search for x bounds     AZ11  94
IF(XMAX<sngl:(VERTEX(1)))XMAX=sngl:(VERTEX(1))#                         AZ11  95
NUVERT=yes:#                           Assume we found a vertex not i   AZ11  96
FOR(VV1=MARKV;VV1<VV;incrv:(VV1))#     Test if prev vertex              AZ11  97
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ11  98
POINT(1)=VERTEX(1)-dble:(vertx:(VV1))# Distance between vertices        AZ11  99
POINT(2)=VERTEX(2)-dble:(verty:(VV1))#                                  AZ11 100
POINT(3)=VERTEX(3)-dble:(vertz:(VV1))#                                  AZ11 101
norm:(POINT,POINT,DTEMP)#                                               AZ11 102
IF(DTEMP<dsmall:)#                     Two points are very close        AZ11 103
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ11 104
NUVERT=no:#                            Vertex already exists            AZ11 105
ILEN=NINT(vertl:(VV1))#                Number of faces in list          AZ11 106
DO I=1,3#                              Loop over faces                  AZ11 107
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AZ11 108
DO J=1,ILEN#                           Loop over face list              AZ11 109
IF(NINT(vertf:(VV1+J))==F(I))BREAK#                                     AZ11 110
IF(J>ILEN)#                            Not found in list                AZ11 111
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AZ11 112
VV=VV+1;incrqx:(ISTOPV,ISTOPV+1,AZ1101)# Get mem                        AZ11 113
movereal:(QX,VV1+4+J,QX,VV1+5+J,ISTOPV-(VV1+4+J),1)# Shift mem          AZ11 114
vertl:(VV1)=vertl:(VV1)+1.0#           Increment list counter           AZ11 115
vertf:(VV1+J)=REAL(F(I))#              Store face pointer               AZ11 116
facen:(F(I))=facen:(F(I))+1.0#         Update face counter              AZ11 117
facecx:(F(I))=facecx:(F(I))+vertx:(VV1)# Store vertex on face           AZ11 118
facecy:(F(I))=facecy:(F(I))+verty:(VV1)#                                AZ11 119
facecz:(F(I))=facecz:(F(I))+vertz:(VV1)#                                AZ11 120
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AZ11 121
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AZ11 122
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ11 123
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ11 124
IF(NUVERT==yes:)#                      New vertex (not in list)         AZ11 125
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AZ11 126
NVERT=NVERT+1#                         Increase vertex counter          AZ11 127
incrqx:(ISTOPV,ISTOPV+7,AZ1101)#       Get mem for another vertex       AZ11 128
vertx:(VV)=sngl:(VERTEX(1))#           Store x                          AZ11 129
verty:(VV)=sngl:(VERTEX(2))#           Store y                          AZ11 130
vertz:(VV)=sngl:(VERTEX(3))#           Store z                          AZ11 131
vertl:(VV)=3.0#                        3 faces vertex                   AZ11 132
DO I=1,3#                                                               AZ11 133
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AZ11 134
vertf:(VV+I)=REAL(F(I))#               Store face 1 pointer             AZ11 135
facen:(F(I))=facen:(F(I))+1.0#                                          AZ11 136
facecx:(F(I))=facecx:(F(I))+sngl:(VERTEX(1))#                           AZ11 137
facecy:(F(I))=facecy:(F(I))+sngl:(VERTEX(2))#                           AZ11 138
facecz:(F(I))=facecz:(F(I))+sngl:(VERTEX(3))#                           AZ11 139
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AZ11 140
incrv:(VV)#                            Increment vertex pointer         AZ11 141
CM(1)=CM(1)+VERTEX(1)#                 Increase center of mass          AZ11 142
CM(2)=CM(2)+VERTEX(2)#                                                  AZ11 143
CM(3)=CM(3)+VERTEX(3)#                                                  AZ11 144
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AZ11 145
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ11 146
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ11 147
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ11 148
IF((XMIN==large:)\(XMAX==-large:))iquit:(AZ1102)# Crystal not bounded   AZ11 149
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Find center of faces             AZ11 150
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AZ11 151
IF(NINT(facen:(FF))<3)iquit:(AZ1103)#  Face not bounded                 AZ11 152
facecx:(FF)=facecx:(FF)/facen:(FF)#                                     AZ11 153
facecy:(FF)=facecy:(FF)/facen:(FF)#                                     AZ11 154
facecz:(FF)=facecz:(FF)/facen:(FF)#                                     AZ11 155
faces:(FF)=0.0#                        Clear surface                    AZ11 156
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AZ11 157
CM(1)=CM(1)/dfloat:(NVERT)#            Crystal's center of mass         AZ11 158
CM(2)=CM(2)/dfloat:(NVERT)#                                             AZ11 159
CM(3)=CM(3)/dfloat:(NVERT)#                                             AZ11 160
RETURN#                                                                 AZ11 161
END#                                                                    AZ11 162
#                                                                       AZ11 163
#                                                                       AZ11 164
systemheader:(AZ12)#                                                    AZ12   1
####################################################################### AZ12   2
#                                                                     # AZ12   3
# EDGES BUILDER (CALLED ONCE BY AZ10):                                # AZ12   4
# ------------------------------------------------------------------- # AZ12   5
# THE EDGES ARE STORED IN QX-ARRAY IN THE FOLLOWING WAY:              # AZ12   6
#   WORDS 1 -> 3: V(1)                                                # AZ12   7
#   WORDS 4 -> 6: V(2)                                                # AZ12   8
#   WORDS 7 -> 8: ADDRESSES OF FACES 1 AND 2                          # AZ12   9
#   WORD       9: FLAG TO KNOW IF EDGE EXISTS                         # AZ12  10
#                 THIS WORD IS NOT USED IN BUILDING CRYSTAL EDGES,    # AZ12  11
#                 IT WILL BE USED TO CONSTRUCT HOWELL'S POLYHEDRON  #   AZ12  12
#                 EDGES (IN ANALYTICAL INTEGRATION METHOD)            # AZ12  13
# ------------------------------------------------------------------- # AZ12  14
# ALGORITHM USED:                                                     # AZ12  15
#                                                                     # AZ12  16
# 1) TAKE 2 VERTICES AND CHECK IF THEY HAVE TWO FACES IN COMMON       # AZ12  17
# 2) IF THEY HAVE, CHECK ORIENTATION [(V1-V0)^(V2-V0)].N1 > 0         # AZ12  18
#    IF IT IS < 0, SWAP FACE 1 AND FACE 2                             # AZ12  19
# 3) STORE EVERYTHING                                                 # AZ12  20
# ------------------------------------------------------------------- # AZ12  21
# EN PLUS DE LA CONSTRUCTION DES ARETES, CETTE SUBROUTINE CALCULE LE  # AZ12  22
# VOLUME DU CRISTAL AINSI QUE L'AIRE DES FACES. EN OUTRE, ELLE TESTE  # AZ12  23
# LE CRISTAL PAR EULER (FACE+SOMMETS-ARETES=2 POUR UN POLYEDRE CON-   # AZ12  24
# VEXE                                                                # AZ12  25
#                                                                     # AZ12  26
####################################################################### AZ12  27
SUBROUTINE AZ12#                                                        AZ12  28
#                                                                       AZ12  29
include:(AACOMN)#                                                       AZ12  30
include:(AZCOMN)#                                                       AZ12  31
#                                                                       AZ12  32
INTEGER VV#                            Main vertices pointer            AZ12  33
INTEGER EE#                            Main edges pointer               AZ12  34
vector:(DIST1);vector:(DIST2)#         Distance from center of face t   AZ12  35
dp: DTEMP#                                                              AZ12  36
INTEGER I,J,K#                         Loop indexes and counters        AZ12  37
INTEGER ILEN1,ILEN2#                   Length of vertex face list       AZ12  38
INTEGER VV1,VV2,VVA,VVB#               Pointers in vertex list          AZ12  39
INTEGER F(2)#                          Face addresses of edge           AZ12  40
#                                                                       AZ12  41
MARKE=ISTOPV#                          Set edge pointers                AZ12  42
incrqx:(ISTOPE,MARKE+(NFACES+NVERT-2)*nstepe:,AZ1201)# Mem for edge l   AZ12  43
NEDGE=0#                               Edges counter                    AZ12  44
VV1=MARKV;EE=MARKE#                    Set pointers                     AZ12  45
FOR(VV=1;VV<NVERT;VV=VV+1)#            Loop over vertices               AZ12  46
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ12  47
ILEN1=NINT(vertl:(VV1))#               Length of face list              AZ12  48
FOR(VV2=VV1+4+ILEN1;VV2<ISTOPV;incrv:(VV2))# Loop over remaining vert   AZ12  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ12  50
ILEN2=NINT(vertl:(VV2))#               Length of face list              AZ12  51
K=0#                                   Set match flag                   AZ12  52
DO I=1,ILEN1;DO J=1,ILEN2#             Loop over both lists             AZ12  53
ifeq:(vertf:(VV1+I),vertf:(VV2+J))#    Test faces in common             AZ12  54
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ12  55
IF(K>1)iquit:(AZ1202)#                 More than 2 faces in common      AZ12  56
K=K+1#                                 Vertices vv1 & vv2 have this f   AZ12  57
F(K)=NINT(vertf:(VV1+I))#              Store face address               AZ12  58
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ12  59
IF(K==2)#                              Edge found (2 faces in common)   AZ12  60
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ12  61
NEDGE=NEDGE+1#                         Increment edge counter           AZ12  62
DO I=1,2#                              Loop over common faces           AZ12  63
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ12  64
DIST1(1)=dble:(vertx:(VV1)-facecx:(F(I)))# V1-v0                        AZ12  65
DIST1(2)=dble:(verty:(VV1)-facecy:(F(I)))#                              AZ12  66
DIST1(3)=dble:(vertz:(VV1)-facecz:(F(I)))#                              AZ12  67
DIST2(1)=dble:(vertx:(VV2)-facecx:(F(I)))# V2-v0                        AZ12  68
DIST2(2)=dble:(verty:(VV2)-facecy:(F(I)))#                              AZ12  69
DIST2(3)=dble:(vertz:(VV2)-facecz:(F(I)))#                              AZ12  70
cross:(DIST1,DIST2,DIST1);norm:(DIST1,DIST2,DTEMP)# Surface of parall   AZ12  71
faces:(F(I))=faces:(F(I))+sngl:(DTEMP)/2.0# Increase face aera          AZ12  72
DIST2(1)=dble:(facecx:(F(I)))-CM(1)#   Add tetrahedron to volume        AZ12  73
DIST2(2)=dble:(facecy:(F(I)))-CM(2)#                                    AZ12  74
DIST2(3)=dble:(facecz:(F(I)))-CM(3)#                                    AZ12  75
VOL=VOL+ABS(sngl:(dot:(DIST1,DIST2)))/6.0# Increase crystal volume      AZ12  76
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ12  77
NC(1)=dble:(faceh:(F(2)))#             Normal to face 2                 AZ12  78
NC(2)=dble:(facek:(F(2)))#                                              AZ12  79
NC(3)=dble:(facel:(F(2)))#                                              AZ12  80
DTEMP=dot:(NC,DIST1)#                  Perform triple product           AZ12  81
IF(DTEMP<DBZ)#                         Test orientation with face 2     AZ12  82
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ12  83
VVA=VV1;VVB=VV2#                       No swap                          AZ12  84
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ12  85
ELSE#                                                                   AZ12  86
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AZ12  87
VVB=VV1;VVA=VV2#                       Swap                             AZ12  88
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AZ12  89
edgef1:(EE)=REAL(F(1));edgef2:(EE)=REAL(F(2))# Store face addresses     AZ12  90
edgex1:(EE)=vertx:(VVA);edgex2:(EE)=vertx:(VVB)# Store vertices         AZ12  91
edgey1:(EE)=verty:(VVA);edgey2:(EE)=verty:(VVB)#                        AZ12  92
edgez1:(EE)=vertz:(VVA);edgez2:(EE)=vertz:(VVB)#                        AZ12  93
edgefl:(EE)=2.0#                       Store flag (2 vertices found)    AZ12  94
incre:(EE)#                            Next edge                        AZ12  95
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ12  96
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ12  97
incrv:(VV1)#                           Next vertex                      AZ12  98
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ12  99
IF((NFACES+NVERT-NEDGE)!=2)iquit:(AZ1203)# Euler's law not fulfilled    AZ12 100
RETURN#                                                                 AZ12 101
END#                                                                    AZ12 102
#                                                                       AZ12 103
#                                                                       AZ12 104
systemheader:(AZ13)#                                                    AZ13   1
####################################################################### AZ13   2
#                                                                     # AZ13   3
# PRINT REPORT                                                        # AZ13   4
#                                                                     # AZ13   5
####################################################################### AZ13   6
SUBROUTINE AZ13#                                                        AZ13   7
#                                                                       AZ13   8
include:(AACOMN)#                                                       AZ13   9
include:(AZCOMN)#                                                       AZ13  10
#                                                                       AZ13  11
INTEGER FF#                            Main face pointer                AZ13  12
INTEGER VV#                            Main vertices pointer            AZ13  13
INTEGER EE#                            Main edges pointer               AZ13  14
dp: DIST1,DIST2#                                                        AZ13  15
INTEGER FF1,FF2#                       Face addresses (in edge list)    AZ13  16
INTEGER I#                             Loop index                       AZ13  17
INTEGER ILEN#                          Length of face list              AZ13  18
INTEGER VV1,VV2,VVA,VVB#               Vertex addresses (in edge list   AZ13  19
vector:(X1);vector:(X2)#                                                AZ13  20
#                                                                       AZ13  21
chardata:(AZ131," List of edges")#                                      AZ13  22
chardata:(AZ132," From Vertex  To Vertex         x        y        $#   AZ13  23
z     u    v    w     Faces")#                                          AZ13  24
chardata:(AZ133,[" List of faces : Equations of faces (hkl) as $#       AZ13  25
ax+by+cz=d, d in millimeters"])#                                        AZ13  26
chardata:(AZ134," Miller Indices          a          b          $#      AZ13  27
c          d         Area")#                                            AZ13  28
chardata:(AZ135," Crystal Volume                        mm**3")#        AZ13  29
chardata:(AZ136," List of vertices")#                                   AZ13  30
chardata:(AZ137,"        x          y          z    Face Number")#      AZ13  31
realdata:(FMT1,[120313.,230313.,330742.,420742.,510742.,$#              AZ13  32
570313.,620313.,670313.,730313.,770313.])#                              AZ13  33
realdata:(FMT2,[50313.,100313.,150313.,260952.,370952.,480952.,$#       AZ13  34
590952.,720952.])#                                                      AZ13  35
realdata:(FMT3,[90842.,200842.,310842.,370313.,410313.,450313.,$#       AZ13  36
490313.,530313.,570313.,610313.,650313.,670313.])#                      AZ13  37
datastuff:#                                                             AZ13  38
#                                                                       AZ13  39
# PRINT FACE REPORT                                                     AZ13  40
# -----------------                                                     AZ13  41
#                                                                       AZ13  42
LINRM=NFACES+3#                                                         AZ13  43
writeline:(3,AZ133,NAZ133,3,3)#        Face report header               AZ13  44
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#                                     AZ13  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ13  46
ncodefld:(QX,FF+1,CHROT,FMT2,8)#       Code parameters                  AZ13  47
writeline:(0,AZ134,NAZ134,1,3)#        Print face report                AZ13  48
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ13  49
#                                                                       AZ13  50
# PRINT VERTEX REPORT                                                   AZ13  51
# -------------------                                                   AZ13  52
#                                                                       AZ13  53
LINRM=NVERT+3#                                                          AZ13  54
writeline:(1,AZ136,NAZ136,3,3)#        Vertex report header             AZ13  55
FOR(VV=MARKV;VV<ISTOPV;incrv:(VV))#                                     AZ13  56
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ13  57
ILEN=MIN0(NINT(vertl:(VV)),9)#         Max 9 face in list               AZ13  58
DO I=1,ILEN;BUFOT(I+3)=REAL((NINT(vertf:(VV+I))-MARKF)/nstepf:+1)#      AZ13  59
BUFOT(1)=vertx:(VV)#                                                    AZ13  60
BUFOT(2)=verty:(VV)#                                                    AZ13  61
BUFOT(3)=vertz:(VV)#                                                    AZ13  62
ncodefld:(BUFOT,1,CHROT,FMT3,ILEN+3)#                                   AZ13  63
writeline:(0,AZ137,NAZ137,1,3)#                                         AZ13  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ13  65
#                                                                       AZ13  66
# PRINT EDGE REPORT                                                     AZ13  67
# -----------------                                                     AZ13  68
#                                                                       AZ13  69
LINRM=NEDGE+3#                                                          AZ13  70
writeline:(1,AZ131,NAZ131,3,3)#        Edge report header               AZ13  71
DO EE=MARKE,ISTOPE-nstepe:,nstepe:#    Loop over all edges              AZ13  72
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ13  73
I=1#                                                                    AZ13  74
FOR(VV=MARKV;VV<ISTOPV;incrv:(VV))#    Find vertex number in list       AZ13  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ13  76
X1(1)=dble:(vertx:(VV)-edgex1:(EE))#   Distance between vertex and ed   AZ13  77
X1(2)=dble:(verty:(VV)-edgey1:(EE))#                                    AZ13  78
X1(3)=dble:(vertz:(VV)-edgez1:(EE))#                                    AZ13  79
X2(1)=dble:(vertx:(VV)-edgex2:(EE))#   Distance between vertex and ed   AZ13  80
X2(2)=dble:(verty:(VV)-edgey2:(EE))#                                    AZ13  81
X2(3)=dble:(vertz:(VV)-edgez2:(EE))#                                    AZ13  82
norm:(X1,X1,DIST1);norm:(X2,X2,DIST2)# Find norms                       AZ13  83
IF(DIST1<dsmall:)#                     Same point                       AZ13  84
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ13  85
VVA=VV;VV1=I#                          Store pointers                   AZ13  86
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ13  87
IF(DIST2<dsmall:)#                     Same point                       AZ13  88
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ13  89
VVB=VV;VV2=I#                          Store pointers                   AZ13  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ13  91
I=I+1#                                 Increment vertex counter         AZ13  92
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ13  93
FF1=NINT(edgef1:(EE));FF2=NINT(edgef2:(EE))# Face addresses             AZ13  94
BUFOT(1)=REAL(VV1);BUFOT(2)=REAL(VV2)# Number of extremities            AZ13  95
BUFOT(3)=edgex2:(EE)-edgex1:(EE)#      Direction of edge                AZ13  96
BUFOT(4)=edgey2:(EE)-edgey1:(EE)#                                       AZ13  97
BUFOT(5)=edgez2:(EE)-edgez1:(EE)#                                       AZ13  98
BUFOT(6)=facekm:(FF1)*facelm:(FF2)-facelm:(FF1)*facekm:(FF2)# Miller    AZ13  99
BUFOT(7)=facelm:(FF1)*facehm:(FF2)-facehm:(FF1)*facelm:(FF2)#           AZ13 100
BUFOT(8)=facehm:(FF1)*facekm:(FF2)-facekm:(FF1)*facehm:(FF2)#           AZ13 101
BUFOT(9)=REAL((FF1-MARKF)/nstepf:+1)#  Face addresses                   AZ13 102
BUFOT(10)=REAL((FF2-MARKF)/nstepf:+1)#                                  AZ13 103
ncodefld:(BUFOT,1,CHROT,FMT1,10)#      Code parameters                  AZ13 104
writeline:(0,AZ132,NAZ132,1,3)#        Print them                       AZ13 105
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ13 106
ncodefld:(VOL,1,AZ135,381061.,1)#      Code crystal volume              AZ13 107
writeline:(1,AZ135,NAZ135,3,3)#        Print it                         AZ13 108
RETURN#                                                                 AZ13 109
END#                                                                    AZ13 110
#                                                                       AZ13 111
#                                                                       AZ13 112
systemheader:(AZ14)#                                                    AZ14   1
####################################################################### AZ14   2
#                                                                     # AZ14   3
# BEST ORIENTATION FOR BULK & SURFACE GRIDS                           # AZ14   4
# ------------------------------------------------------------------- # AZ14   5
# 1) GET SURFACE INERTIA TENSOR                                       # AZ14   6
# 2) GET MAIN AXIS OF SURFACE GRID                                    # AZ14   7
# 3)                                                                  # AZ14   8
#                                                                     # AZ14   9
####################################################################### AZ14  10
SUBROUTINE AZ14#                                                        AZ14  11
#                                                                       AZ14  12
include:(AACOMN)#                                                       AZ14  13
include:(AZCOMN)#                                                       AZ14  14
#                                                                       AZ14  15
INTEGER FF#                            Main faces pointer               AZ14  16
#                                                                       AZ14  17
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#                                     AZ14  18
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ14  19
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ14  20
RETURN#                                                                 AZ14  21
END#                                                                    AZ14  22
#                                                                       AZ14  23
systemheader:(AZ15)#                                                    AZ15   1
####################################################################### AZ15   2
#                                                                     # AZ15   3
# CONSTRUCTION DU GRID:                                               # AZ15   4
# *  CONSTRUIRE LES ARRAYS DE WEIGHTS (2ND PART = 1ST PART) ET PIVOTS # AZ15   5
#    ET LES PIVOTS (2ND PART = 1.0 - 1ST PART)                        # AZ15   6
# *  BOUCLE DE XMIN A XMAX: POUR CHAQUE X, ON CHERCHE YMIN ET YMAX,   # AZ15   7
#    ON DIVISE LE SEGMENT SELON GAUSS, ON BOUCLE SUR CHAQUE (X, Y),   # AZ15   8
#    ON CHERCHE ZMIN ET ZMAX ET ON DIVISE LE SEGMENT SELON GAUSS.     # AZ15   9
# *  ON A MAINTENANT CONSTRUIT LE GRILLAGE AVEC TOUS LES POINTS XYZ   # AZ15  10
#                                                                     # AZ15  11
####################################################################### AZ15  12
SUBROUTINE AZ15#                                                        AZ15  13
#                                                                       AZ15  14
include:(AACOMN)#                                                       AZ15  15
include:(AZCOMN)#                                                       AZ15  16
#                                                                       AZ15  17
INTEGER FF#                            Main faces pointer               AZ15  18
INTEGER GG#                            Main grid pointer                AZ15  19
INTEGER VV#                            Main vertices pointer            AZ15  20
INTEGER SS#                            Main surface grid pointer        AZ15  21
matrix:(PASS)#                         Matrix to give intersection of   AZ15  22
matrix:(MTEMP)#                        Inverse of pass                  AZ15  23
vector:(POINT)#                        Grid point                       AZ15  24
vector:(DIST)#                                                          AZ15  25
vector:(VERTEX)#                       Bounding point                   AZ15  26
REAL YMIN,YMAX,ZMIN,ZMAX#              Bounds of crystal                AZ15  27
REAL DELX,DELY,DELZ#                   Max dimensions of crystal        AZ15  28
REAL DX,DY,DZ#                         Increment between two grid poi   AZ15  29
dp: DET#                               Determinant                      AZ15  30
REAL WEIGHT(122),PIVOTS(122)#          Weigths and pivots               AZ15  31
INTEGER FF1,FF2,FF3#                   Face pointers                    AZ15  32
INTEGER I,J,K#                         Loop over grid points            AZ15  33
INTEGER ILEN#                          Number of entries in vertex fa   AZ15  34
INTEGER GRID#                          Grid address                     AZ15  35
#                                                                       AZ15  36
realdata:(P8,[$#                                                        AZ15  37
0.019855072,.101666761,.237233795,.408282679])#                         AZ15  38
realdata:(P10,[$#                                                       AZ15  39
0.013046736,.067468317,.160295216,.283302303,.425562831])#              AZ15  40
realdata:(P12,[$#                                                       AZ15  41
0.009219683,.047941372,.115048663,.206341023,.316084251,.437333296])#   AZ15  42
realdata:(P14,[$#                                                       AZ15  43
0.006858096,.035782558,.086399342,.156353548,.242375682,.340443816,#    AZ15  44
0.445972526])#                                                          AZ15  45
realdata:(P16,[$#                                                       AZ15  46
0.005299533,.027712488,.067184399,.122297796,.191061878,.270991611,#    AZ15  47
0.359198225,.452493745])#                                               AZ15  48
realdata:(P6,[$#                                                        AZ15  49
0.033765243,.169395307,.380690407])#                                    AZ15  50
realdata:(P24,[$#                                                       AZ15  51
0.002406390,.012635722,.030862724,.056792236,.089999007,.129937904,#    AZ15  52
0.175953174,.227289264,.283103246,.342478660,.404440566,.467971553])#   AZ15  53
realdata:(P32,[$#                                                       AZ15  54
0.001368069,.007194244,.017618872,.032546962,.051839422,.075316193,#    AZ15  55
0.102758102,.133908940,.168477867,.206142121,.246550045,.289324362,#    AZ15  56
0.334065699,.380356319,.427764019,.475846167])#                         AZ15  57
realdata:(W8,[$#                                                        AZ15  58
0.050614268,.111190517,.156853323,.181341892])#                         AZ15  59
realdata:(W10,[$#                                                       AZ15  60
0.033335672,.074725675,.109543181,.134633360,.147762112])#              AZ15  61
realdata:(W12,[$#                                                       AZ15  62
0.023587668,.053469663,.080039164,.101583713,.116746268,.124573523])#   AZ15  63
realdata:(W14,[$#                                                       AZ15  64
0.017559730,.040079044,.060759285,.078601584,.092769199,.102599232,#    AZ15  65
0.107631927])#                                                          AZ15  66
realdata:(W16,[$#                                                       AZ15  67
0.013576230,.031126762,.047579256,.062314486,.074797994,.084578260,#    AZ15  68
0.091301708,.094725305])#                                               AZ15  69
realdata:(W6,[$#                                                        AZ15  70
0.085662246,.180380787,.233956967])#                                    AZ15  71
realdata:(W24,[$#                                                       AZ15  72
0.006170615,.014265694,.022138719,.029649292,.036673241,.043095081,#    AZ15  73
0.048809326,.053722135,.057752834,.060835236,.062918728,.063969098])#   AZ15  74
realdata:(W32,[$#                                                       AZ15  75
0.003509305,.008137197,.012696033,.017136932,.021417949,.025499030,#    AZ15  76
0.029342047,.032911111,.036172897,.039096948,.041655962,.043826047,#    AZ15  77
0.045586939,.046922200,.047819360,.048270044])#                         AZ15  78
datastuff:#                                                             AZ15  79
#                                                                       AZ15  80
# INITIALIZATION                                                        AZ15  81
# --------------                                                        AZ15  82
#                                                                       AZ15  83
DO I=1,4#                                                               AZ15  84
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ15  85
WEIGHT(I)=W8(I);WEIGHT(9-I)=W8(I)#                                      AZ15  86
PIVOTS(I)=P8(I);PIVOTS(9-I)=1.0-P8(I)#                                  AZ15  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ15  88
DO I=1,5#                                                               AZ15  89
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ15  90
WEIGHT(8+I)=W10(I);WEIGHT(19-I)=W10(I)#                                 AZ15  91
PIVOTS(8+I)=P10(I);PIVOTS(19-I)=1.0-P10(I)#                             AZ15  92
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ15  93
DO I=1,6#                                                               AZ15  94
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ15  95
WEIGHT(18+I)=W12(I);WEIGHT(31-I)=W12(I)#                                AZ15  96
PIVOTS(18+I)=P12(I);PIVOTS(31-I)=1.0-P12(I)#                            AZ15  97
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ15  98
DO I=1,7#                                                               AZ15  99
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ15 100
WEIGHT(30+I)=W14(I);WEIGHT(45-I)=W14(I)#                                AZ15 101
PIVOTS(30+I)=P14(I);PIVOTS(45-I)=1.0-P14(I)#                            AZ15 102
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ15 103
DO I=1,8#                                                               AZ15 104
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ15 105
WEIGHT(44+I)=W16(I);WEIGHT(61-I)=W16(I)#                                AZ15 106
PIVOTS(44+I)=P16(I);PIVOTS(61-I)=1.0-P16(I)#                            AZ15 107
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ15 108
DO I=1,3#                                                               AZ15 109
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ15 110
WEIGHT(60+I)=W6(I);WEIGHT(67-I)=W6(I)#                                  AZ15 111
PIVOTS(60+I)=P6(I);PIVOTS(67-I)=1.0-P6(I)#                              AZ15 112
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ15 113
DO I=1,12#                                                              AZ15 114
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AZ15 115
WEIGHT(66+I)=W24(I);WEIGHT(91-I)=W24(I)#                                AZ15 116
PIVOTS(66+I)=P24(I);PIVOTS(91-I)=1.0-P24(I)#                            AZ15 117
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AZ15 118
DO I=1,16#                                                              AZ15 119
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AZ15 120
WEIGHT(90+I)=W32(I);WEIGHT(123-I)=W32(I)#                               AZ15 121
PIVOTS(90+I)=P32(I);PIVOTS(123-I)=1.0-P32(I)#                           AZ15 122
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AZ15 123
MARKG=ISTOPV#                          Set grid pointers                AZ15 124
GG=MARKG#                                                               AZ15 125
incrqx:(ISTOPG,MARKG+NPTX*NPTY*NPTZ*nstepg:,AZ1501)# Get memory         AZ15 126
#                                                                       AZ15 127
# GRID CONSTRUCTION: FIND POINTS AND VOLUME ELEMENTS                    AZ15 128
# --------------------------------------------------                    AZ15 129
#                                                                       AZ15 130
PASS(1,1)=DB1#                         Intersection of faces matrix     AZ15 131
PASS(1,2)=DBZ;PASS(1,3)=DBZ#           2 * 2 matrix                     AZ15 132
PASS(2,1)=DBZ;PASS(3,1)=DBZ#                                            AZ15 133
DELX=XMAX-XMIN#                        Crystal max dimension on x       AZ15 134
DO I=1,NPTX#                           Loop over all x-axis divisions   AZ15 135
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AZ15 136
GRID=NADRX+I#                          Grid address                     AZ15 137
POINT(1)=dble:(XMIN+DELX*PIVOTS(GRID))# X coordinate of grid point      AZ15 138
DX=WEIGHT(GRID)*DELX#                  Weight of grid point             AZ15 139
YMIN=large:;YMAX=-large:#              Default crystal dimension on y   AZ15 140
DIST(1)=POINT(1)#                                                       AZ15 141
#                                                                       AZ15 142
# LOOKS FOR EXTREMA VALUES ON Y AXIS, X GIVEN                           AZ15 143
# -------------------------------------------                           AZ15 144
#                                                                       AZ15 145
DO FF2=MARKF,ISTOPF-2*nstepf:,nstepf:# Loop over faces                  AZ15 146
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AZ15 147
PASS(2,2)=dble:(facek:(FF2));PASS(2,3)=dble:(facel:(FF2))# Constructi   AZ15 148
DIST(2)=dble:(faced:(FF2))-POINT(1)*dble:(faceh:(FF2))#                 AZ15 149
DO FF3=FF2+nstepf:,ISTOPF-nstepf:,nstepf:# Loop over faces              AZ15 150
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AZ15 151
PASS(3,2)=dble:(facek:(FF3));PASS(3,3)=dble:(facel:(FF3))# Matrix con   AZ15 152
invert:(PASS,MTEMP,DET)#               Find intersection, x given       AZ15 153
IF(dabs:(DET)>dsmall:)#                Faces intersect                  AZ15 154
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AZ15 155
DIST(3)=dble:(faced:(FF3))-POINT(1)*dble:(faceh:(FF3))#                 AZ15 156
postm:(MTEMP,DIST,VERTEX)#             Find solution                    AZ15 157
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Test if solution not outside c   AZ15 158
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AZ15 159
NC(1)=dble:(faceh:(FF))#               Normal to face                   AZ15 160
NC(2)=dble:(facek:(FF))#                                                AZ15 161
NC(3)=dble:(facel:(FF))#                                                AZ15 162
IF((dble:(faced:(FF))-dot:(NC,VERTEX))<-dsmall:)BREAK# Exit if outsid   AZ15 163
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AZ15 164
IF(FF==ISTOPF)#                        Point not outside the crystal    AZ15 165
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AZ15 166
IF(sngl:(VERTEX(2))<YMIN)YMIN=sngl:(VERTEX(2))# Set y-bounds            AZ15 167
IF(sngl:(VERTEX(2))>YMAX)YMAX=sngl:(VERTEX(2))#                         AZ15 168
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AZ15 169
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AZ15 170
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AZ15 171
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AZ15 172
DELY=YMAX-YMIN#                        Y crystal dimension, x given     AZ15 173
DO J=1,NPTY#                           Loop over y divisions            AZ15 174
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AZ15 175
GRID=NADRY+J#                          Grid address                     AZ15 176
POINT(2)=dble:(YMIN+DELY*PIVOTS(GRID))# Y coordinate of grid point      AZ15 177
DY=WEIGHT(GRID)*DELY#                  Weight of point                  AZ15 178
ZMIN=large:;ZMAX=-large:#              Default crystal dimension, x &   AZ15 179
DIST(2)=POINT(2)#                                                       AZ15 180
#                                                                       AZ15 181
# LOOKS FOR EXTREMA ON Z AXIS, X AND Y GIVEN                            AZ15 182
# ------------------------------------------                            AZ15 183
#                                                                       AZ15 184
DO FF3=MARKF,ISTOPF-nstepf:,nstepf:#   Loop over all faces              AZ15 185
IF(ABS(facel:(FF3))>small:)#           If face intersect plane (x, y)   AZ15 186
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AZ15 187
DIST(3)=(dble:(faced:(FF3))-dble:(faceh:(FF3))*POINT(1)-$#              AZ15 188
dble:(facek:(FF3))*POINT(2))/ dble:(facel:(FF3))#                       AZ15 189
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Test if point not outside crys   AZ15 190
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AZ15 191
NC(1)=dble:(faceh:(FF))#               Normal to face                   AZ15 192
NC(2)=dble:(facek:(FF))#                                                AZ15 193
NC(3)=dble:(facel:(FF))#                                                AZ15 194
IF((dble:(faced:(FF))-dot:(NC,DIST))<-dsmall:)BREAK# Exit if outside    AZ15 195
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AZ15 196
IF(FF==ISTOPF)#                        Point not outside the crystal    AZ15 197
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AZ15 198
IF(sngl:(DIST(3))<ZMIN)ZMIN=sngl:(DIST(3))# Set z-bounds                AZ15 199
IF(sngl:(DIST(3))>ZMAX)ZMAX=sngl:(DIST(3))#                             AZ15 200
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AZ15 201
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AZ15 202
DELZ=ZMAX-ZMIN#                        Crystal z dimension, (x, y) gi   AZ15 203
DO K=1,NPTZ#                           Loop over all grid points        AZ15 204
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AZ15 205
GRID=NADRZ+K#                          Grid address                     AZ15 206
POINT(3)=dble:(ZMIN+DELZ*PIVOTS(GRID))# Z-coordinate of grid point      AZ15 207
DZ=WEIGHT(GRID)*DELZ#                  Weight                           AZ15 208
gridx:(GG)=sngl:(POINT(1))#            Store grid point                 AZ15 209
gridy:(GG)=sngl:(POINT(2))#                                             AZ15 210
gridz:(GG)=sngl:(POINT(3))#                                             AZ15 211
gridv:(GG)=DX*DY*DZ#                   Store weight                     AZ15 212
VOL2=VOL2+gridv:(GG)#                  Increase volume                  AZ15 213
incrg:(GG)#                            Increment grid pointer           AZ15 214
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AZ15 215
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AZ15 216
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AZ15 217
#                                                                       AZ15 218
# CONSTRUCT SURFACE GRID                                                AZ15 219
# ----------------------                                                AZ15 220
#                                                                       AZ15 221
MARKS=ISTOPG#                          Set surface grid pointers        AZ15 222
SS=MARKS#                                                               AZ15 223
incrqx:(ISTOPS,MARKS+nsteps:*NPTS*NPTS*NFACES,AZ1502)# Get memory       AZ15 224
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over all faces              AZ15 225
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AZ15 226
faces:(FF)=0.0#                                                         AZ15 227
ifne:(ABS(facel:(FF)),1.0)#            Face in general position         AZ15 228
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AZ15 229
PASS(3,1)=dble:(faceh:(FF))#           Builds passage matrix            AZ15 230
PASS(3,2)=dble:(facek:(FF))#                                            AZ15 231
PASS(3,3)=dble:(facel:(FF))#                                            AZ15 232
PASS(1,3)=DBZ#                                                          AZ15 233
PASS(2,3)=-dsqrt:(1-PASS(3,3)**2)#                                      AZ15 234
PASS(1,1)=-PASS(3,2)/PASS(2,3)#                                         AZ15 235
PASS(1,2)=PASS(3,1)/PASS(2,3)#                                          AZ15 236
PASS(2,1)=-PASS(1,2)*PASS(3,3)#                                         AZ15 237
PASS(2,2)=PASS(1,1)*PASS(3,3)#                                          AZ15 238
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AZ15 239
ELSE IF(facel:(FF)>0.0)#               Vector (0 0 1)                   AZ15 240
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AZ15 241
PASS(1,1)=DB1;PASS(2,2)=DB1;PASS(3,3)=DB1# Build identity               AZ15 242
PASS(1,2)=DBZ;PASS(1,3)=DBZ;PASS(2,3)=DBZ#                              AZ15 243
PASS(2,1)=DBZ;PASS(3,1)=DBZ;PASS(3,2)=DBZ#                              AZ15 244
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AZ15 245
ELSE#                                  Vector (0 0 -1)                  AZ15 246
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AZ15 247
PASS(1,1)=DBZ;PASS(1,2)=DB1;PASS(1,3)=DBZ# Build matrix                 AZ15 248
PASS(2,1)=DB1;PASS(2,2)=DBZ;PASS(2,3)=DBZ#                              AZ15 249
PASS(3,1)=DBZ;PASS(3,2)=DBZ;PASS(3,3)=-DB1#                             AZ15 250
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AZ15 251
transp:(PASS,MTEMP)#                   Invert passage matrix            AZ15 252
XMIN=large:;XMAX=-large:#              Default x bounds                 AZ15 253
#                                                                       AZ15 254
# GETS EXTREMAS ON X AXE                                                AZ15 255
# ----------------------                                                AZ15 256
#                                                                       AZ15 257
FOR(VV=MARKV;VV<ISTOPV;incrv:(VV))#    Loop over all vertices           AZ15 258
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AZ15 259
ILEN=NINT(vertl:(VV))#                                                  AZ15 260
DO I=1,ILEN;IF(NINT(vertf:(VV+I))==FF)BREAK#                            AZ15 261
IF(I>ILEN)NEXT#                                                         AZ15 262
VERTEX(1)=dble:(vertx:(VV))#                                            AZ15 263
VERTEX(2)=dble:(verty:(VV))#                                            AZ15 264
VERTEX(3)=dble:(vertz:(VV))#           Store vertex                     AZ15 265
postm:(PASS,VERTEX,VERTEX)#            Change of base                   AZ15 266
IF(sngl:(VERTEX(1))<XMIN)XMIN=sngl:(VERTEX(1))# Set x-bounds            AZ15 267
IF(sngl:(VERTEX(1))>XMAX)XMAX=sngl:(VERTEX(1))#                         AZ15 268
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AZ15 269
DELX=XMAX-XMIN#                        Face ff x dimension              AZ15 270
#                                                                       AZ15 271
# FINDS EXTREMAS ON Y AXE, X GIVEN                                      AZ15 272
# --------------------------------                                      AZ15 273
#                                                                       AZ15 274
DO I=1,NPTS#                           Loop over x divisions            AZ15 275
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AZ15 276
GRID=NADRS+I#                          Grid address                     AZ15 277
VERTEX(1)=dble:(XMIN+DELX*PIVOTS(GRID))# Surface grid point x coordin   AZ15 278
DX=WEIGHT(GRID)*DELX#                  Weight                           AZ15 279
YMIN=large:;YMAX=-large:#              Default crystal y dimension      AZ15 280
DO FF1=MARKF,ISTOPF-nstepf:,nstepf:#   Loop over all faces              AZ15 281
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 AZ15 282
IF(FF1==FF)NEXT#                       Skip current face                AZ15 283
NC(1)=dble:(faceh:(FF1))#                                               AZ15 284
NC(2)=dble:(facek:(FF1))#                                               AZ15 285
NC(3)=dble:(facel:(FF1))#              Normal to face                   AZ15 286
postm:(PASS,NC,NC)#                    Change base                      AZ15 287
IF(dabs:(NC(2))<dsmall:)NEXT#          Faces doesn't intersect          AZ15 288
VERTEX(2)=(dble:(faced:(FF1))-NC(1)*VERTEX(1)-NC(3)*VERTEX(3))/NC(2)#   AZ15 289
DO FF2=MARKF,ISTOPF-nstepf:,nstepf:#   Check if inside                  AZ15 290
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 AZ15 291
IF((FF2==FF)\(FF2==FF1))NEXT#                                           AZ15 292
NC(1)=dble:(faceh:(FF2))#                                               AZ15 293
NC(2)=dble:(facek:(FF2))#                                               AZ15 294
NC(3)=dble:(facel:(FF2))#                                               AZ15 295
postm:(PASS,NC,NC)#                                                     AZ15 296
IF((dble:(faced:(FF2))-dot:(NC,VERTEX))<-dsmall:)BREAK#                 AZ15 297
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 AZ15 298
IF(FF2<ISTOPF)NEXT#                    Outside crystal                  AZ15 299
IF(sngl:(VERTEX(2))<YMIN)YMIN=sngl:(VERTEX(2))# Set y bounds            AZ15 300
IF(sngl:(VERTEX(2))>YMAX)YMAX=sngl:(VERTEX(2))#                         AZ15 301
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 AZ15 302
DELY=YMAX-YMIN#                        Crystal y dimension, x given     AZ15 303
#                                                                       AZ15 304
# STORES SURFACE GRID POINTS                                            AZ15 305
# --------------------------                                            AZ15 306
#                                                                       AZ15 307
DO J=1,NPTS#                           Loop over y divisions            AZ15 308
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 AZ15 309
GRID=NADRS+J#                          Grid address                     AZ15 310
VERTEX(2)=dble:(YMIN+DELY*PIVOTS(GRID))# Y surface grid point coordin   AZ15 311
DY=WEIGHT(GRID)*DELY#                  Weight                           AZ15 312
postm:(MTEMP,VERTEX,POINT)#            Come back to normal base         AZ15 313
surfx:(SS)=sngl:(POINT(1))#                                             AZ15 314
surfy:(SS)=sngl:(POINT(2))#                                             AZ15 315
surfz:(SS)=sngl:(POINT(3))#            Store point                      AZ15 316
surfs:(SS)=DX*DY#                      Store weight                     AZ15 317
faces:(FF)=faces:(FF)+surfs:(SS)#      Increase surface                 AZ15 318
surff:(SS)=REAL(FF)#                   Store face pointer               AZ15 319
incrs:(SS)#                            Increment pointer                AZ15 320
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 AZ15 321
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AZ15 322
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AZ15 323
RETURN#                                                                 AZ15 324
END#                                                                    AZ15 325
#                                                                       AZ15 326
#                                                                       AZ15 327
systemheader:(AZ16)#                   Analytical integration of a sp   AZ16   1
####################################################################### AZ16   2
#                                                                     # AZ16   3
# TABLES FOR SPHERICAL ABSORPTION AND MEAN PATH                       # AZ16   4
#                                                                     # AZ16   5
####################################################################### AZ16   6
SUBROUTINE AZ16#                                                        AZ16   7
#                                                                       AZ16   8
include:(AACOMN)#                                                       AZ16   9
include:(AZCOMN)#                                                       AZ16  10
#                                                                       AZ16  11
INTEGER I,J,K#                         Loop indices                     AZ16  12
INTEGER TT#                            Main interpolation pointer       AZ16  13
REAL B1,B2#                            Interpolation constants          AZ16  14
REAL DMR0(190),DMR1(190),DMR2(190),DMR3(190),DMR4(190)#                 AZ16  15
REAL DMR5(190),DMR6(190),DMR7(190),DMR8(190),DMR9(209)#                 AZ16  16
#                                                                       AZ16  17
realdata:(MUR0,[$#                                                      AZ16  18
1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,$#                                        AZ16  19
1.,1.,1.,1.,1.,1.,1.,1.,1.,$#                                           AZ16  20
1.1609,1.1609,1.1609,1.1607,1.1606,1.1603,1.1600,1.1597,1.1593,1.1589,$#AZ16  21
1.1586,1.1582,1.1579,1.1575,1.1572,1.1570,1.1568,1.1567,1.1567,$#       AZ16  22
1.3457,1.3456,1.3452,1.3447,1.3439,1.3428,1.3415,1.3400,1.3383,1.3366,$#AZ16  23
1.3348,1.3331,1.3313,1.3297,1.3282,1.3271,1.3262,1.3256,1.3254,$#       AZ16  24
1.5574,1.5571,1.5561,1.5546,1.5525,1.5497,1.5463,1.5426,1.5383,1.5339,$#AZ16  25
1.5293,1.5248,1.5204,1.5162,1.5126,1.5096,1.5074,1.5059,1.5055,$#       AZ16  26
1.7994,1.7988,1.7968,1.7935,1.7891,1.7833,1.7765,1.7689,1.7604,1.7515,$#AZ16  27
1.7425,1.7335,1.7249,1.7169,1.7099,1.7041,1.6997,1.6970,1.6961,$#       AZ16  28
2.0755,2.0743,2.0706,2.0647,2.0565,2.0462,2.0340,2.0204,2.0056,1.9901,$#AZ16  29
1.9745,1.9592,1.9445,1.9311,1.9194,1.9097,1.9024,1.8979,1.8964,$#       AZ16  30
2.3897,2.3877,2.3816,2.3715,2.3578,2.3406,2.3206,2.2984,2.2746,2.2500,$#AZ16  31
2.2255,2.2015,2.1789,2.1583,2.1403,2.1256,2.1145,2.1076,2.1053,$#       AZ16  32
2.7467,2.7434,2.7336,2.7177,2.6959,2.6691,2.6382,2.6042,2.5683,2.5316,$#AZ16  33
2.4952,2.4602,2.4274,2.3977,2.3719,2.3508,2.3351,2.3253,2.3220,$#       AZ16  34
3.1511,3.1461,3.1312,3.1069,3.0740,3.0339,2.9882,2.9386,2.8869,2.8347,$#AZ16  35
2.7835,2.7346,2.6892,2.6484,2.6133,2.5845,2.5632,2.5499,2.5454,$#       AZ16  36
3.6082,3.6009,3.5789,3.5431,3.4952,3.4374,3.3723,3.3026,3.2308,3.1592,$#AZ16  37
3.0898,3.0241,2.9637,2.9098,2.8634,2.8258,2.7979,2.7805,2.7747])#       AZ16  38
realdata:(MUR1,[$#                                                      AZ16  39
4.1237,4.1131,4.0815,4.0304,3.9625,3.8816,3.7917,3.6966,3.6001,3.5048,$#AZ16  40
3.4135,3.3280,3.2499,3.1807,3.1216,3.0738,3.0383,3.0163,3.0090,$#       AZ16  41
4.7035,4.6886,4.6442,4.5729,4.4790,4.3686,4.2474,4.1211,3.9945,3.8710,$#AZ16  42
3.7540,3.6455,3.5470,3.4605,3.3870,3.3276,3.2838,3.2566,3.2474,$#       AZ16  43
5.3542,5.3335,5.2722,5.1747,5.0476,4.9001,4.7404,4.5761,4.4137,4.2571,$#AZ16  44
4.1104,3.9756,3.8542,3.7483,3.6586,3.5866,3.5334,3.5005,3.4894,$#       AZ16  45
6.082,6.054,5.9710,5.8399,5.6710,5.4776,5.2711,5.0617,4.8573,4.6625,$#  AZ16  46
4.4819,4.3175,4.1706,4.0432,3.9360,3.8500,3.7868,3.7477,3.7344,$#       AZ16  47
6.895,6.857,6.746,6.573,6.352,6.102,5.8400,5.5774,5.3244,5.0862,$#      AZ16  48
4.8676,4.6703,4.4955,4.3447,4.2183,4.1174,4.0432,3.9974,3.9819,$#       AZ16  49
7.801,7.750,7.604,7.377,7.092,6.775,6.447,6.123,5.8143,5.5273,$#        AZ16  50
5.2666,5.0333,4.8281,4.6520,4.5052,4.3883,4.3024,4.2495,4.2315,$#       AZ16  51
8.806,8.740,8.549,8.256,7.894,7.497,7.092,6.697,6.326,5.9849,$#         AZ16  52
5.6780,5.4057,5.1678,4.9647,4.7961,4.6622,4.5641,4.5036,4.4830,$#       AZ16  53
9.920,9.834,9.587,9.214,8.759,8.268,7.774,7.299,6.859,6.458,$#          AZ16  54
6.101,5.7867,5.5140,5.2823,5.0907,4.9390,4.8279,4.7595,4.7361,$#        AZ16  55
11.151,11.040,10.725,10.254,9.689,9.088,8.492,7.928,7.411,6.946,$#      AZ16  56
6.535,6.176,5.8662,5.6045,5.3888,5.2184,5.0936,5.0170,4.9908,$#         AZ16  57
12.507,12.366,11.967,11.380,10.685,9.957,9.246,8.583,7.982,7.447,$#     AZ16  58
6.978,6.572,6.224,5.9308,5.6900,5.5001,5.3613,5.2760,5.2468])#          AZ16  59
realdata:(MUR2,[$#                                                      AZ16  60
13.998,13.819,13.320,12.593,11.746,10.873,10.034,9.262,8.570,7.961,$#   AZ16  61
7.431,6.975,6.587,6.261,5.9942,5.7842,5.6307,5.5365,5.5041,$#           AZ16  62
15.632,15.408,14.788,13.895,12.874,11.837,10.855,9.964,9.175,8.486,$#   AZ16  63
7.893,7.385,6.955,6.595,6.301,6.070,5.9017,5.7982,5.7627,$#             AZ16  64
17.419,17.141,16.376,15.290,14.067,12.847,11.708,10.688,9.795,9.023,$#  AZ16  65
8.362,7.800,7.327,6.932,6.610,6.358,6.174,6.061,6.022,$#                AZ16  66
19.369,19.025,18.089,16.778,15.327,13.902,12.592,11.433,10.429,9.569,$# AZ16  67
8.839,8.220,7.702,7.272,6.922,6.648,6.448,6.325,6.282,$#                AZ16  68
21.489,21.069,19.931,18.361,16.652,15.000,13.504,12.198,11.077,10.125,$#AZ16  69
9.322,8.645,8.081,7.614,7.235,6.938,6.722,6.589,6.543,$#                AZ16  70
23.791,23.280,21.907,20.040,18.041,16.142,14.445,12.982,11.738,10.690,$#AZ16  71
9.810,9.074,8.462,7.957,7.548,7.229,6.996,6.853,6.803,$#                AZ16  72
26.30,25.69,24.03,21.82,19.50,17.33,15.42,13.78,12.41,11.26,$#          AZ16  73
10.30,9.507,8.846,8.302,7.863,7.520,7.271,7.117,7.064,$#                AZ16  74
28.99,28.26,26.28,23.69,21.01,18.55,16.41,14.60,13.09,11.84,$#          AZ16  75
10.80,9.944,9.233,8.650,8.180,7.814,7.547,7.382,7.326,$#                AZ16  76
31.90,31.02,28.67,25.65,22.58,19.80,17.42,15.43,13.79,12.43,$#          AZ16  77
11.31,10.38,9.622,9.000,8.498,8.108,7.824,7.648,7.588,$#                AZ16  78
35.02,33.98,31.21,27.71,24.21,21.10,18.46,16.28,14.49,13.02,$#          AZ16  79
11.82,10.83,10.01,9.351,8.818,8.403,8.102,7.915,7.850])#                AZ16  80
realdata:(MUR3,[$#                                                      AZ16  81
38.38,37.14,33.90,29.86,25.90,22.42,19.52,17.14,15.20,13.62,$#          AZ16  82
12.33,11.27,10.41,9.704,9.139,8.699,8.379,8.181,8.113,$#                AZ16  83
41.97,40.51,36.73,32.10,27.64,23.78,20.59,18.01,15.92,14.23,$#          AZ16  84
12.85,11.72,10.80,10.06,9.460,8.996,8.658,8.448,8.376,$#                AZ16  85
45.81,44.09,39.71,34.43,29.43,25.17,21.69,18.89,16.65,14.83,$#          AZ16  86
13.37,12.17,11.20,10.41,9.783,9.293,8.937,8.716,8.639,$#                AZ16  87
49.91,47.90,42.84,36.85,31.27,26.58,22.80,19.79,17.38,15.45,$#          AZ16  88
13.89,12.63,11.60,10.77,10.11,9.591,9.216,8.983,8.902,$#                AZ16  89
54.27,51.93,46.12,39.35,33.16,28.03,23.93,20.69,18.12,16.07,$#          AZ16  90
14.42,13.08,12.00,11.13,10.43,9.890,9.496,9.251,9.166,$#                AZ16  91
58.91,56.19,49.54,41.93,35.09,29.50,25.07,21.60,18.86,16.69,$#          AZ16  92
14.95,13.54,12.40,11.49,10.76,10.19,9.776,9.519,9.430,$#                AZ16  93
63.83,60.69,53.12,44.60,37.07,30.99,26.23,22.52,19.62,17.31,$#          AZ16  94
15.48,14.00,12.81,11.85,11.08,10.49,10.06,9.787,9.693,$#                AZ16  95
69.05,65.43,56.84,47.35,39.09,32.51,27.40,23.45,20.37,17.94,$#          AZ16  96
16.01,14.46,13.21,12.21,11.41,10.79,10.34,10.06,9.958,$#                AZ16  97
74.56,70.42,60.71,50.17,41.15,34.04,28.59,24.39,21.13,18.58,$#          AZ16  98
16.55,14.92,13.62,12.57,11.73,11.09,10.62,10.32,10.22,$#                AZ16  99
80.38,75.65,64.72,53.06,43.24,35.60,29.78,25.34,21.90,19.21,$#          AZ16 100
17.08,15.39,14.02,12.93,12.06,11.39,10.90,10.59,10.49])#                AZ16 101
realdata:(MUR4,[$#                                                      AZ16 102
86.52,81.14,68.87,56.03,45.37,37.18,30.99,26.29,22.67,19.85,$#          AZ16 103
17.63,15.85,14.43,13.29,12.39,11.69,11.18,10.86,10.75,$#                AZ16 104
92.99,86.89,73.17,59.07,47.54,38.77,32.21,27.24,23.44,20.49,$#          AZ16 105
18.17,16.32,14.84,13.66,12.72,11.99,11.46,11.13,11.02,$#                AZ16 106
99.79,92.89,77.61,62.17,49.74,40.39,33.43,28.21,24.22,21.14,$#          AZ16 107
18.71,16.79,15.25,14.02,13.05,12.29,11.74,11.40,11.28,$#                AZ16 108
106.9,99.15,82.19,65.34,51.97,42.02,34.67,29.18,25.00,21.78,$#          AZ16 109
19.26,17.26,15.66,14.39,13.38,12.60,12.03,11.67,11.54,$#                AZ16 110
114.4,105.7,86.90,68.57,54.23,43.66,35.91,30.15,25.79,22.43,$#          AZ16 111
19.80,17.73,16.07,14.75,13.71,12.90,12.31,11.94,11.81,$#                AZ16 112
122.3,112.5,91.74,71.86,56.52,45.32,37.17,31.13,26.58,23.08,$#          AZ16 113
20.35,18.20,16.48,15.12,14.04,13.20,12.59,12.21,12.07,$#                AZ16 114
130.5,119.5,96.72,75.21,58.84,47.00,38.43,32.12,27.37,23.73,$#          AZ16 115
20.90,18.67,16.90,15.48,14.37,13.50,12.88,12.48,12.34,$#                AZ16 116
139.0,126.9,101.8,78.61,61.18,48.69,39.70,33.11,28.16,24.39,$#          AZ16 117
21.45,19.14,17.31,15.85,14.70,13.81,13.16,12.75,12.60,$#                AZ16 118
148.0,134.5,107.1,82.07,63.55,50.39,40.98,34.10,28.96,25.04,$#          AZ16 119
22.00,19.62,17.72,16.22,15.03,14.11,13.44,13.02,12.87,$#                AZ16 120
157.4,142.4,112.4,85.58,65.94,52.10,42.26,35.10,29.76,25.70,$#          AZ16 121
22.56,20.09,18.14,16.59,15.36,14.42,13.72,13.29,13.14])#                AZ16 122
realdata:(MUR5,[$#                                                      AZ16 123
167.1,150.5,117.9,89.15,68.36,53.83,43.55,36.10,30.56,26.36,$#          AZ16 124
23.11,20.57,18.55,16.95,15.69,14.72,14.01,13.56,13.40,$#                AZ16 125
177.3,159.0,123.5,92.76,70.80,55.57,44.85,37.11,31.37,27.02,$#          AZ16 126
23.67,21.04,18.97,17.32,16.02,15.02,14.29,13.83,13.67,$#                AZ16 127
187.9,167.7,129.2,96.42,73.26,57.31,46.15,38.12,32.17,27.68,$#          AZ16 128
24.22,21.52,19.38,17.69,16.36,15.33,14.58,14.10,13.93,$#                AZ16 129
198.8,176.7,135.0,100.1,75.74,59.07,47.46,39.13,32.98,28.35,$#          AZ16 130
24.78,22.00,19.80,18.06,16.69,15.63,14.86,14.37,14.20,$#                AZ16 131
210.3,186.0,141.0,103.9,78.24,60.84,48.77,40.14,33.80,29.01,$#          AZ16 132
25.34,22.47,20.22,18.43,17.02,15.94,15.14,14.64,14.46,$#                AZ16 133
222.1,195.5,147.0,107.7,80.76,62.62,50.09,41.16,34.61,29.68,$#          AZ16 134
25.90,22.95,20.63,18.80,17.36,16.24,15.43,14.91,14.73,$#                AZ16 135
234.4,205.4,153.2,111.5,83.30,64.41,51.42,42.19,35.42,30.35,$#          AZ16 136
26.46,23.43,21.05,19.17,17.69,16.55,15.71,15.18,14.99,$#                AZ16 137
247.1,215.5,159.5,115.4,85.85,66.20,52.75,43.21,36.24,31.02,$#          AZ16 138
27.02,23.91,21.47,19.54,18.02,16.85,16.00,15.45,15.26,$#                AZ16 139
260.3,225.9,165.8,119.3,88.43,68.01,54.08,44.24,37.06,31.69,$#          AZ16 140
27.58,24.39,21.89,19.91,18.36,17.16,16.28,15.72,15.52,$#                AZ16 141
274.0,236.6,172.3,123.2,91.02,69.82,55.42,45.27,37.88,32.36,$#          AZ16 142
28.14,24.87,22.31,20.28,18.69,17.46,16.57,15.99,15.79])#                AZ16 143
realdata:(MUR6,[$#                                                      AZ16 144
288.2,247.6,178.9,127.2,93.62,71.64,56.76,46.30,38.70,33.03,$#          AZ16 145
28.71,25.36,22.73,20.65,19.03,17.77,16.85,16.27,16.06,$#                AZ16 146
302.8,258.8,185.6,131.3,96.24,73.47,58.11,47.34,39.52,33.70,$#          AZ16 147
29.27,25.84,23.15,21.03,19.36,18.08,17.14,16.54,16.32,$#                AZ16 148
317.9,270.4,192.3,135.3,98.88,75.31,59.46,48.38,40.35,34.38,$#          AZ16 149
29.84,26.32,23.57,21.40,19.69,18.38,17.42,16.81,16.59,$#                AZ16 150
333.5,282.2,199.2,139.4,101.5,77.15,60.82,49.42,41.17,35.05,$#          AZ16 151
30.40,26.80,23.99,21.77,20.03,18.69,17.71,17.08,16.85,$#                AZ16 152
349.6,294.3,206.1,143.5,104.2,79.00,62.18,50.46,42.00,35.73,$#          AZ16 153
30.97,27.28,24.41,22.14,20.36,18.99,17.99,17.35,17.12,$#                AZ16 154
366.2,306.7,213.2,147.7,106.9,80.86,63.54,51.50,42.83,36.41,$#          AZ16 155
31.53,27.77,24.83,22.51,20.70,19.30,18.28,17.62,17.38,$#                AZ16 156
383.4,319.4,220.3,151.9,109.6,82.73,64.91,52.55,43.66,37.08,$#          AZ16 157
32.10,28.25,25.25,22.89,21.03,19.61,18.56,17.89,17.65,$#                AZ16 158
401.1,332.3,227.5,156.1,112.3,84.60,66.28,53.60,44.49,37.76,$#          AZ16 159
32.67,28.74,25.67,23.26,21.37,19.91,18.85,18.16,17.92,$#                AZ16 160
419.3,345.5,234.8,160.3,115.0,86.47,67.65,54.65,45.32,38.44,$#          AZ16 161
33.23,29.22,26.09,23.63,21.70,20.22,19.13,18.43,18.18,$#                AZ16 162
438.1,359.1,242.2,164.6,117.7,88.35,69.03,55.70,46.16,39.12,$#          AZ16 163
33.80,29.71,26.51,24.00,22.04,20.53,19.42,18.71,18.45])#                AZ16 164
realdata:(MUR7,[$#                                                      AZ16 165
457.4,372.8,249.7,168.9,120.5,90.24,70.41,56.76,46.99,39.80,$#          AZ16 166
34.37,30.19,26.93,24.38,22.38,20.83,19.70,18.98,18.71,$#                AZ16 167
477.3,386.9,257.2,173.3,123.2,92.14,71.79,57.81,47.83,40.48,$#          AZ16 168
34.94,30.68,27.36,24.75,22.71,21.14,19.99,19.25,18.98,$#                AZ16 169
497.7,401.3,264.9,177.6,126.0,94.03,73.17,58.87,48.66,41.16,$#          AZ16 170
35.51,31.16,27.78,25.12,23.05,21.45,20.27,19.52,19.25,$#                AZ16 171
518.7,415.9,272.6,182.0,128.8,95.94,74.56,59.93,49.50,41.84,$#          AZ16 172
36.08,31.65,28.20,25.50,23.38,21.75,20.56,19.79,19.51,$#                AZ16 173
540.3,430.8,280.4,186.4,131.5,97.85,75.95,60.99,50.34,42.53,$#          AZ16 174
36.65,32.13,28.62,25.87,23.72,22.06,20.84,20.06,19.78,$#                AZ16 175
562.5,446.0,288.2,190.8,134.3,99.76,77.35,62.05,51.18,43.21,$#          AZ16 176
37.22,32.62,29.05,26.25,24.06,22.37,21.13,20.34,20.04,$#                AZ16 177
585.3,461.4,296.1,195.3,137.1,101.7,78.74,63.11,52.02,43.89,$#          AZ16 178
37.79,33.11,29.47,26.62,24.39,22.68,21.42,20.61,20.31,$#                AZ16 179
608.7,477.1,304.1,199.8,140.0,103.6,80.14,64.18,52.86,44.58,$#          AZ16 180
38.36,33.59,29.89,26.99,24.73,22.98,21.70,20.88,20.58,$#                AZ16 181
632.8,493.1,312.2,204.3,142.8,105.5,81.54,65.24,53.70,45.26,$#          AZ16 182
38.93,34.08,30.32,27.37,25.06,23.29,21.99,21.15,20.84,$#                AZ16 183
657.4,509.4,320.3,208.8,145.6,107.5,82.94,66.31,54.54,45.95,$#          AZ16 184
39.50,34.57,30.74,27.74,25.40,23.60,22.27,21.42,21.11])#                AZ16 185
realdata:(MUR8,[$#                                                      AZ16 186
682.7,525.9,328.5,213.4,148.5,109.4,84.35,67.38,55.39,46.64,$#          AZ16 187
40.08,35.06,31.16,28.12,25.74,23.91,22.56,21.69,21.38,$#                AZ16 188
708.6,542.7,336.8,217.9,151.3,111.3,85.76,68.45,56.23,47.32,$#          AZ16 189
40.65,35.54,31.59,28.49,26.07,24.21,22.85,21.97,21.64,$#                AZ16 190
735.2,559.8,345.1,222.5,154.2,113.3,87.17,69.52,57.08,48.01,$#          AZ16 191
41.22,36.03,32.01,28.87,26.41,24.52,23.13,22.24,21.91,$#                AZ16 192
762.4,577.1,353.5,227.1,157.0,115.2,88.58,70.59,57.92,48.70,$#          AZ16 193
41.79,36.52,32.44,29.24,26.75,24.83,23.42,22.51,22.17,$#                AZ16 194
790.3,594.7,361.9,231.8,159.9,117.2,89.99,71.67,58.77,49.38,$#          AZ16 195
42.37,37.01,32.86,29.62,27.08,25.14,23.71,22.78,22.44,$#                AZ16 196
818.8,612.5,370.4,236.4,162.8,119.1,91.41,72.74,59.62,50.07,$#          AZ16 197
42.94,37.50,33.28,29.99,27.42,25.45,23.99,23.05,22.71,$#                AZ16 198
848.1,630.6,379.0,241.1,165.7,121.1,92.82,73.82,60.46,50.76,$#          AZ16 199
43.51,37.99,33.71,30.37,27.76,25.75,24.28,23.32,22.97,$#                AZ16 200
878.0,649.0,387.6,245.8,168.6,123.1,94.24,74.90,61.31,51.45,$#          AZ16 201
44.09,38.48,34.13,30.74,28.10,26.06,24.56,23.60,23.24,$#                AZ16 202
908.6,667.6,396.3,250.5,171.5,125.0,95.66,75.97,62.16,52.14,$#          AZ16 203
44.66,38.96,34.56,31.12,28.43,26.37,24.85,23.87,23.50,$#                AZ16 204
940.0,686.4,405.1,255.2,174.4,127.0,97.09,77.05,63.01,52.83,$#          AZ16 205
45.24,39.45,34.98,31.49,28.77,26.68,25.14,24.14,23.77])#                AZ16 206
realdata:(MUR9,[$#                                                      AZ16 207
972.0,705.5,413.9,259.9,177.3,129.0,98.51,78.13,63.86,53.52,$#          AZ16 208
45.81,39.94,35.41,31.87,29.11,26.99,25.42,24.41,24.04,$#                AZ16 209
1005.,724.9,422.7,264.7,180.3,130.9,99.94,79.21,64.71,54.21,$#          AZ16 210
46.39,40.43,35.83,32.24,29.44,27.29,25.71,24.68,24.30,$#                AZ16 211
1038.,744.5,431.6,269.5,183.2,132.9,101.4,80.29,65.56,54.90,$#          AZ16 212
46.96,40.92,36.26,32.62,29.78,27.60,26.00,24.96,24.57,$#                AZ16 213
1072.,764.4,440.6,274.3,186.1,134.9,102.8,81.38,66.41,55.59,$#          AZ16 214
47.54,41.41,36.68,32.99,30.12,27.91,26.28,25.23,24.84,$#                AZ16 215
1107.,784.5,449.6,279.1,189.1,136.9,104.2,82.46,67.26,56.28,$#          AZ16 216
48.11,41.90,37.11,33.37,30.46,28.22,26.57,25.50,25.10,$#                AZ16 217
1143.,804.8,458.6,283.9,192.0,138.9,105.7,83.54,68.12,56.97,$#          AZ16 218
48.69,42.39,37.53,33.75,30.79,28.53,26.86,25.77,25.37,$#                AZ16 219
1180.,825.4,467.7,288.7,195.0,140.9,107.1,84.63,68.97,57.67,$#          AZ16 220
49.27,42.88,37.96,34.12,31.13,28.83,27.14,26.04,25.63,$#                AZ16 221
1217.,846.2,476.9,293.6,198.0,142.9,108.5,85.71,69.82,58.36,$#          AZ16 222
49.84,43.37,38.38,34.50,31.47,29.14,27.43,26.32,25.90,$#                AZ16 223
1255.,867.3,486.1,298.4,200.9,144.9,110.0,86.80,70.68,59.05,$#          AZ16 224
50.42,43.86,38.81,34.87,31.81,29.45,27.72,26.59,26.17,$#                AZ16 225
1294.,888.5,495.3,303.3,203.9,146.9,111.4,87.89,71.53,59.74,$#          AZ16 226
51.00,44.35,39.24,35.25,32.15,29.76,28.00,26.86,26.43,$#                AZ16 227
1333.,910.1,504.6,308.2,206.9,148.9,112.8,88.97,72.38,60.44,$#          AZ16 228
51.57,44.85,39.66,35.63,32.48,30.07,28.29,27.13,26.70])#                AZ16 229
intdata:(STEP,19)#                                                      AZ16 230
datastuff:#                                                             AZ16 231
#                                                                       AZ16 232
# BUILD INTERPOLATED TABLE FOR DERIVATIVES                              AZ16 233
# ----------------------------------------                              AZ16 234
#                                                                       AZ16 235
DO I=1,19#                                                              AZ16 236
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ16 237
DMR0(I)=1.5#                           Theta = 0                        AZ16 238
DMR0(I+STEP)=10.0*$#                                                    AZ16 239
(-17.0*MUR0(I)+9.0*(MUR0(I+STEP)+MUR0(I+2*STEP))-MUR0(I+3*STEP)-0.90)/$#AZ16 240
(18.0*MUR0(I+STEP))#                                                    AZ16 241
DO J=2,7;DMR0(I+J*STEP)=10.0*$#                                         AZ16 242
(MUR0(I+(J-2)*STEP)-8.0*MUR0(I+(J-1)*STEP)+$#                           AZ16 243
8.0*MUR0(I+(J+1)*STEP)-MUR0(I+(J+2)*STEP))/(12.0*MUR0(I+J*STEP))#       AZ16 244
DMR0(I+8*STEP)=-10.0*$#                                                 AZ16 245
(MUR1(I)-8.0*MUR0(I+9*STEP)+$#                                          AZ16 246
8.0*MUR0(I+7*STEP)-MUR0(I+6*STEP))/(12.0*MUR0(I+8*STEP))#               AZ16 247
DMR0(I+9*STEP)=-10.0*$#                                                 AZ16 248
(MUR1(I+STEP)-8.0*MUR1(I)+$#                                            AZ16 249
8.0*MUR0(I+8*STEP)-MUR0(I+7*STEP))/(12.0*MUR0(I+9*STEP))#               AZ16 250
DMR1(I)=-10.0*$#                                                        AZ16 251
(MUR1(I+2*STEP)-8.0*MUR1(I+STEP)+$#                                     AZ16 252
8.0*MUR0(I+9*STEP)-MUR0(I+8*STEP))/(12.0*MUR1(I))#                      AZ16 253
DMR1(I+STEP)=-10.0*$#                                                   AZ16 254
(MUR1(I+3*STEP)-8.0*MUR1(I+2*STEP)+$#                                   AZ16 255
8.0*MUR1(I)-MUR0(I+9*STEP))/(12.0*MUR1(I+STEP))#                        AZ16 256
DO J=2,7;DMR1(I+J*STEP)=10.0*$#                                         AZ16 257
(MUR1(I+(J-2)*STEP)-8.0*MUR1(I+(J-1)*STEP)+$#                           AZ16 258
8.0*MUR1(I+(J+1)*STEP)-MUR1(I+(J+2)*STEP))/(12.0*MUR1(I+J*STEP))#       AZ16 259
DMR1(I+8*STEP)=-10.0*$#                                                 AZ16 260
(MUR2(I)-8.0*MUR1(I+9*STEP)+$#                                          AZ16 261
8.0*MUR1(I+7*STEP)-MUR1(I+6*STEP))/(12.0*MUR1(I+8*STEP))#               AZ16 262
DMR1(I+9*STEP)=-10.0*$#                                                 AZ16 263
(MUR2(I+STEP)-8.0*MUR2(I)+$#                                            AZ16 264
8.0*MUR1(I+8*STEP)-MUR1(I+7*STEP))/(12.0*MUR1(I+9*STEP))#               AZ16 265
DMR2(I)=-10.0*$#                                                        AZ16 266
(MUR2(I+2*STEP)-8.0*MUR2(I+STEP)+$#                                     AZ16 267
8.0*MUR1(I+9*STEP)-MUR1(I+8*STEP))/(12.0*MUR2(I))#                      AZ16 268
DMR2(I+STEP)=-10.0*$#                                                   AZ16 269
(MUR2(I+3*STEP)-8.0*MUR2(I+2*STEP)+$#                                   AZ16 270
8.0*MUR2(I)-MUR1(I+9*STEP))/(12.0*MUR2(I+STEP))#                        AZ16 271
DO J=2,7;DMR2(I+J*STEP)=10.0*$#                                         AZ16 272
(MUR2(I+(J-2)*STEP)-8.0*MUR2(I+(J-1)*STEP)+$#                           AZ16 273
8.0*MUR2(I+(J+1)*STEP)-MUR2(I+(J+2)*STEP))/(12.0*MUR2(I+J*STEP))#       AZ16 274
DMR2(I+8*STEP)=-10.0*$#                                                 AZ16 275
(MUR3(I)-8.0*MUR2(I+9*STEP)+$#                                          AZ16 276
8.0*MUR2(I+7*STEP)-MUR2(I+6*STEP))/(12.0*MUR2(I+8*STEP))#               AZ16 277
DMR2(I+9*STEP)=-10.0*$#                                                 AZ16 278
(MUR3(I+STEP)-8.0*MUR3(I)+$#                                            AZ16 279
8.0*MUR2(I+8*STEP)-MUR2(I+7*STEP))/(12.0*MUR2(I+9*STEP))#               AZ16 280
DMR3(I)=-10.0*$#                                                        AZ16 281
(MUR3(I+2*STEP)-8.0*MUR3(I+STEP)+$#                                     AZ16 282
8.0*MUR2(I+9*STEP)-MUR2(I+8*STEP))/(12.0*MUR3(I))#                      AZ16 283
DMR3(I+STEP)=-10.0*$#                                                   AZ16 284
(MUR3(I+3*STEP)-8.0*MUR3(I+2*STEP)+$#                                   AZ16 285
8.0*MUR3(I)-MUR2(I+9*STEP))/(12.0*MUR3(I+STEP))#                        AZ16 286
DO J=2,7;DMR3(I+J*STEP)=10.0*$#                                         AZ16 287
(MUR3(I+(J-2)*STEP)-8.0*MUR3(I+(J-1)*STEP)+$#                           AZ16 288
8.0*MUR3(I+(J+1)*STEP)-MUR3(I+(J+2)*STEP))/(12.0*MUR3(I+J*STEP))#       AZ16 289
DMR3(I+8*STEP)=-10.0*$#                                                 AZ16 290
(MUR4(I)-8.0*MUR3(I+9*STEP)+$#                                          AZ16 291
8.0*MUR3(I+7*STEP)-MUR3(I+6*STEP))/(12.0*MUR3(I+8*STEP))#               AZ16 292
DMR3(I+9*STEP)=-10.0*$#                                                 AZ16 293
(MUR4(I+STEP)-8.0*MUR4(I)+$#                                            AZ16 294
8.0*MUR3(I+8*STEP)-MUR3(I+7*STEP))/(12.0*MUR3(I+9*STEP))#               AZ16 295
DMR4(I)=-10.0*$#                                                        AZ16 296
(MUR4(I+2*STEP)-8.0*MUR4(I+STEP)+$#                                     AZ16 297
8.0*MUR3(I+9*STEP)-MUR3(I+8*STEP))/(12.0*MUR4(I))#                      AZ16 298
DMR4(I+STEP)=-10.0*$#                                                   AZ16 299
(MUR4(I+3*STEP)-8.0*MUR4(I+2*STEP)+$#                                   AZ16 300
8.0*MUR4(I)-MUR3(I+9*STEP))/(12.0*MUR4(I+STEP))#                        AZ16 301
DO J=2,7;DMR4(I+J*STEP)=10.0*$#                                         AZ16 302
(MUR4(I+(J-2)*STEP)-8.0*MUR4(I+(J-1)*STEP)+$#                           AZ16 303
8.0*MUR4(I+(J+1)*STEP)-MUR4(I+(J+2)*STEP))/(12.0*MUR4(I+J*STEP))#       AZ16 304
DMR4(I+8*STEP)=-10.0*$#                                                 AZ16 305
(MUR5(I)-8.0*MUR4(I+9*STEP)+$#                                          AZ16 306
8.0*MUR4(I+7*STEP)-MUR4(I+6*STEP))/(12.0*MUR4(I+8*STEP))#               AZ16 307
DMR4(I+9*STEP)=-10.0*$#                                                 AZ16 308
(MUR5(I+STEP)-8.0*MUR5(I)+$#                                            AZ16 309
8.0*MUR4(I+8*STEP)-MUR4(I+7*STEP))/(12.0*MUR4(I+9*STEP))#               AZ16 310
DMR5(I)=-10.0*$#                                                        AZ16 311
(MUR5(I+2*STEP)-8.0*MUR5(I+STEP)+$#                                     AZ16 312
8.0*MUR4(I+9*STEP)-MUR4(I+8*STEP))/(12.0*MUR5(I))#                      AZ16 313
DMR5(I+STEP)=-10.0*$#                                                   AZ16 314
(MUR5(I+3*STEP)-8.0*MUR5(I+2*STEP)+$#                                   AZ16 315
8.0*MUR5(I)-MUR4(I+9*STEP))/(12.0*MUR5(I+STEP))#                        AZ16 316
DO J=2,7;DMR5(I+J*STEP)=10.0*$#                                         AZ16 317
(MUR5(I+(J-2)*STEP)-8.0*MUR5(I+(J-1)*STEP)+$#                           AZ16 318
8.0*MUR5(I+(J+1)*STEP)-MUR5(I+(J+2)*STEP))/(12.0*MUR5(I+J*STEP))#       AZ16 319
DMR5(I+8*STEP)=-10.0*$#                                                 AZ16 320
(MUR6(I)-8.0*MUR5(I+9*STEP)+$#                                          AZ16 321
8.0*MUR5(I+7*STEP)-MUR5(I+6*STEP))/(12.0*MUR5(I+8*STEP))#               AZ16 322
DMR5(I+9*STEP)=-10.0*$#                                                 AZ16 323
(MUR6(I+STEP)-8.0*MUR6(I)+$#                                            AZ16 324
8.0*MUR5(I+8*STEP)-MUR5(I+7*STEP))/(12.0*MUR5(I+9*STEP))#               AZ16 325
DMR6(I)=-10.0*$#                                                        AZ16 326
(MUR6(I+2*STEP)-8.0*MUR6(I+STEP)+$#                                     AZ16 327
8.0*MUR5(I+9*STEP)-MUR5(I+8*STEP))/(12.0*MUR6(I))#                      AZ16 328
DMR6(I+STEP)=-10.0*$#                                                   AZ16 329
(MUR6(I+3*STEP)-8.0*MUR6(I+2*STEP)+$#                                   AZ16 330
8.0*MUR6(I)-MUR5(I+9*STEP))/(12.0*MUR6(I+STEP))#                        AZ16 331
DO J=2,7;DMR6(I+J*STEP)=10.0*$#                                         AZ16 332
(MUR6(I+(J-2)*STEP)-8.0*MUR6(I+(J-1)*STEP)+$#                           AZ16 333
8.0*MUR6(I+(J+1)*STEP)-MUR6(I+(J+2)*STEP))/(12.0*MUR6(I+J*STEP))#       AZ16 334
DMR6(I+8*STEP)=-10.0*$#                                                 AZ16 335
(MUR7(I)-8.0*MUR6(I+9*STEP)+$#                                          AZ16 336
8.0*MUR6(I+7*STEP)-MUR6(I+6*STEP))/(12.0*MUR6(I+8*STEP))#               AZ16 337
DMR6(I+9*STEP)=-10.0*$#                                                 AZ16 338
(MUR7(I+STEP)-8.0*MUR7(I)+$#                                            AZ16 339
8.0*MUR6(I+8*STEP)-MUR6(I+7*STEP))/(12.0*MUR6(I+9*STEP))#               AZ16 340
DMR7(I)=-10.0*$#                                                        AZ16 341
(MUR7(I+2*STEP)-8.0*MUR7(I+STEP)+$#                                     AZ16 342
8.0*MUR6(I+9*STEP)-MUR6(I+8*STEP))/(12.0*MUR7(I))#                      AZ16 343
DMR7(I+STEP)=-10.0*$#                                                   AZ16 344
(MUR7(I+3*STEP)-8.0*MUR7(I+2*STEP)+$#                                   AZ16 345
8.0*MUR7(I)-MUR6(I+9*STEP))/(12.0*MUR7(I+STEP))#                        AZ16 346
DO J=2,7;DMR7(I+J*STEP)=10.0*$#                                         AZ16 347
(MUR7(I+(J-2)*STEP)-8.0*MUR7(I+(J-1)*STEP)+$#                           AZ16 348
8.0*MUR7(I+(J+1)*STEP)-MUR7(I+(J+2)*STEP))/(12.0*MUR7(I+J*STEP))#       AZ16 349
DMR7(I+8*STEP)=-10.0*$#                                                 AZ16 350
(MUR8(I)-8.0*MUR7(I+9*STEP)+$#                                          AZ16 351
8.0*MUR7(I+7*STEP)-MUR7(I+6*STEP))/(12.0*MUR7(I+8*STEP))#               AZ16 352
DMR7(I+9*STEP)=-10.0*$#                                                 AZ16 353
(MUR8(I+STEP)-8.0*MUR8(I)+$#                                            AZ16 354
8.0*MUR7(I+8*STEP)-MUR7(I+7*STEP))/(12.0*MUR7(I+9*STEP))#               AZ16 355
DMR8(I)=-10.0*$#                                                        AZ16 356
(MUR8(I+2*STEP)-8.0*MUR8(I+STEP)+$#                                     AZ16 357
8.0*MUR7(I+9*STEP)-MUR7(I+8*STEP))/(12.0*MUR8(I))#                      AZ16 358
DMR8(I+STEP)=-10.0*$#                                                   AZ16 359
(MUR8(I+3*STEP)-8.0*MUR8(I+2*STEP)+$#                                   AZ16 360
8.0*MUR8(I)-MUR7(I+9*STEP))/(12.0*MUR8(I+STEP))#                        AZ16 361
DO J=2,7;DMR8(I+J*STEP)=10.0*$#                                         AZ16 362
(MUR8(I+(J-2)*STEP)-8.0*MUR8(I+(J-1)*STEP)+$#                           AZ16 363
8.0*MUR8(I+(J+1)*STEP)-MUR8(I+(J+2)*STEP))/(12.0*MUR8(I+J*STEP))#       AZ16 364
DMR8(I+8*STEP)=-10.0*$#                                                 AZ16 365
(MUR9(I)-8.0*MUR8(I+9*STEP)+$#                                          AZ16 366
8.0*MUR8(I+7*STEP)-MUR8(I+6*STEP))/(12.0*MUR8(I+8*STEP))#               AZ16 367
DMR8(I+9*STEP)=-10.0*$#                                                 AZ16 368
(MUR9(I+STEP)-8.0*MUR9(I)+$#                                            AZ16 369
8.0*MUR8(I+8*STEP)-MUR8(I+7*STEP))/(12.0*MUR8(I+9*STEP))#               AZ16 370
DMR9(I)=-10.0*$#                                                        AZ16 371
(MUR9(I+2*STEP)-8.0*MUR9(I+STEP)+$#                                     AZ16 372
8.0*MUR8(I+9*STEP)-MUR8(I+8*STEP))/(12.0*MUR9(I))#                      AZ16 373
DMR9(I+STEP)=-10.0*$#                                                   AZ16 374
(MUR9(I+3*STEP)-8.0*MUR9(I+2*STEP)+$#                                   AZ16 375
8.0*MUR9(I)-MUR8(I+9*STEP))/(12.0*MUR9(I+STEP))#                        AZ16 376
DO J=2,8;DMR9(I+J*STEP)=10.0*$#                                         AZ16 377
(MUR9(I+(J-2)*STEP)-8.0*MUR9(I+(J-1)*STEP)+$#                           AZ16 378
8.0*MUR9(I+(J+1)*STEP)-MUR9(I+(J+2)*STEP))/(12.0*MUR9(I+J*STEP))#       AZ16 379
DMR9(I+9*STEP)=5.0*(MUR9(I+10*STEP)-MUR9(I+8*STEP))/MUR9(I+9*STEP)#     AZ16 380
DMR9(I+10*STEP)=10.0*(MUR9(I+10*STEP)-MUR9(I+9*STEP))/MUR9(I+10*STEP)#  AZ16 381
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ16 382
#                                                                       AZ16 383
# EXTRACT POINTERS FOR DATA                                             AZ16 384
# -------------------------                                             AZ16 385
#                                                                       AZ16 386
MARKT=ISTOPF#                          Set sphere list                  AZ16 387
IF(MU*facer:>=10.0)iquit:(AZ1601)#     Must be <= 10                    AZ16 388
incrqx:(ISTOPT,MARKT+2*STEP,AZ1602)#   Get mem for t & dt/d(mu*r)       AZ16 389
TT=MARKT+STEP#                         Set for dt/d(mu*r)               AZ16 390
I=NINT(10.0*MU*facer:-0.5);J=MOD(I,10)# Set pointers                    AZ16 391
B2=10.0*MU*facer:-REAL(I);B1=1.0-B2#   Set mult factors (ponderation)   AZ16 392
IF(I<10)#                                                               AZ16 393
DO K=1,STEP#                                                            AZ16 394
IF(J==9)#                                                               AZ16 395
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ16 396
QX(MARKT+K)=B1*MUR0(J*STEP+K)+B2*MUR1(K)#                               AZ16 397
QX(TT+K)=B1*DMR0(J*STEP+K)+B2*DMR1(K)#                                  AZ16 398
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ16 399
ELSE#                                                                   AZ16 400
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ16 401
QX(MARKT+K)=B1*MUR0(J*STEP+K)+B2*MUR0((J+1)*STEP+K)#                    AZ16 402
QX(TT+K)=B1*DMR0(J*STEP+K)+B2*DMR0((J+1)*STEP+K)#                       AZ16 403
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ16 404
ELSE IF(I<20)#                                                          AZ16 405
DO K=1,STEP#                                                            AZ16 406
IF(J==9)#                                                               AZ16 407
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ16 408
QX(MARKT+K)=B1*MUR1(J*STEP+K)+B2*MUR2(K)#                               AZ16 409
QX(TT+K)=B1*DMR1(J*STEP+K)+B2*DMR2(K)#                                  AZ16 410
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ16 411
ELSE#                                                                   AZ16 412
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AZ16 413
QX(MARKT+K)=B1*MUR1(J*STEP+K)+B2*MUR1((J+1)*STEP+K)#                    AZ16 414
QX(TT+K)=B1*DMR1(J*STEP+K)+B2*DMR1((J+1)*STEP+K)#                       AZ16 415
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AZ16 416
ELSE IF(I<30)#                                                          AZ16 417
DO K=1,STEP#                                                            AZ16 418
IF(J==9)#                                                               AZ16 419
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AZ16 420
QX(MARKT+K)=B1*MUR2(J*STEP+K)+B2*MUR3(K)#                               AZ16 421
QX(TT+K)=B1*DMR2(J*STEP+K)+B2*DMR3(K)#                                  AZ16 422
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AZ16 423
ELSE#                                                                   AZ16 424
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AZ16 425
QX(MARKT+K)=B1*MUR2(J*STEP+K)+B2*MUR2((J+1)*STEP+K)#                    AZ16 426
QX(TT+K)=B1*DMR2(J*STEP+K)+B2*DMR2((J+1)*STEP+K)#                       AZ16 427
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AZ16 428
ELSE IF(I<40)#                                                          AZ16 429
DO K=1,STEP#                                                            AZ16 430
IF(J==9)#                                                               AZ16 431
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AZ16 432
QX(MARKT+K)=B1*MUR3(J*STEP+K)+B2*MUR4(K)#                               AZ16 433
QX(TT+K)=B1*DMR3(J*STEP+K)+B2*DMR4(K)#                                  AZ16 434
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AZ16 435
ELSE#                                                                   AZ16 436
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AZ16 437
QX(MARKT+K)=B1*MUR3(J*STEP+K)+B2*MUR3((J+1)*STEP+K)#                    AZ16 438
QX(TT+K)=B1*DMR3(J*STEP+K)+B2*DMR3((J+1)*STEP+K)#                       AZ16 439
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AZ16 440
ELSE IF(I<50)#                                                          AZ16 441
DO K=1,STEP#                                                            AZ16 442
IF(J==9)#                                                               AZ16 443
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AZ16 444
QX(MARKT+K)=B1*MUR4(J*STEP+K)+B2*MUR5(K)#                               AZ16 445
QX(TT+K)=B1*DMR4(J*STEP+K)+B2*DMR5(K)#                                  AZ16 446
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AZ16 447
ELSE#                                                                   AZ16 448
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AZ16 449
QX(MARKT+K)=B1*MUR4(J*STEP+K)+B2*MUR4((J+1)*STEP+K)#                    AZ16 450
QX(TT+K)=B1*DMR4(J*STEP+K)+B2*DMR4((J+1)*STEP+K)#                       AZ16 451
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AZ16 452
ELSE IF(I<60)#                                                          AZ16 453
DO K=1,STEP#                                                            AZ16 454
IF(J==9)#                                                               AZ16 455
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AZ16 456
QX(MARKT+K)=B1*MUR5(J*STEP+K)+B2*MUR6(K)#                               AZ16 457
QX(TT+K)=B1*DMR5(J*STEP+K)+B2*DMR6(K)#                                  AZ16 458
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AZ16 459
ELSE#                                                                   AZ16 460
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AZ16 461
QX(MARKT+K)=B1*MUR5(J*STEP+K)+B2*MUR5((J+1)*STEP+K)#                    AZ16 462
QX(TT+K)=B1*DMR5(J*STEP+K)+B2*DMR5((J+1)*STEP+K)#                       AZ16 463
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AZ16 464
ELSE IF(I<70)#                                                          AZ16 465
DO K=1,STEP#                                                            AZ16 466
IF(J==9)#                                                               AZ16 467
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AZ16 468
QX(MARKT+K)=B1*MUR6(J*STEP+K)+B2*MUR7(K)#                               AZ16 469
QX(TT+K)=B1*DMR6(J*STEP+K)+B2*DMR7(K)#                                  AZ16 470
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AZ16 471
ELSE#                                                                   AZ16 472
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AZ16 473
QX(MARKT+K)=B1*MUR6(J*STEP+K)+B2*MUR6((J+1)*STEP+K)#                    AZ16 474
QX(TT+K)=B1*DMR6(J*STEP+K)+B2*DMR6((J+1)*STEP+K)#                       AZ16 475
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AZ16 476
ELSE IF(I<80)#                                                          AZ16 477
DO K=1,STEP#                                                            AZ16 478
IF(J==9)#                                                               AZ16 479
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AZ16 480
QX(MARKT+K)=B1*MUR7(J*STEP+K)+B2*MUR8(K)#                               AZ16 481
QX(TT+K)=B1*DMR7(J*STEP+K)+B2*DMR8(K)#                                  AZ16 482
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AZ16 483
ELSE#                                                                   AZ16 484
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AZ16 485
QX(MARKT+K)=B1*MUR7(J*STEP+K)+B2*MUR7((J+1)*STEP+K)#                    AZ16 486
QX(TT+K)=B1*DMR7(J*STEP+K)+B2*DMR7((J+1)*STEP+K)#                       AZ16 487
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AZ16 488
ELSE IF(I<90)#                                                          AZ16 489
DO K=1,STEP#                                                            AZ16 490
IF(J==9)#                                                               AZ16 491
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AZ16 492
QX(MARKT+K)=B1*MUR8(J*STEP+K)+B2*MUR9(K)#                               AZ16 493
QX(TT+K)=B1*DMR8(J*STEP+K)+B2*DMR9(K)#                                  AZ16 494
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AZ16 495
ELSE#                                                                   AZ16 496
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AZ16 497
QX(MARKT+K)=B1*MUR8(J*STEP+K)+B2*MUR8((J+1)*STEP+K)#                    AZ16 498
QX(TT+K)=B1*DMR8(J*STEP+K)+B2*DMR8((J+1)*STEP+K)#                       AZ16 499
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AZ16 500
ELSE#                                                                   AZ16 501
DO K=1,STEP#                                                            AZ16 502
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AZ16 503
QX(MARKT+K)=B1*MUR9(J*STEP+K)+B2*MUR9((J+1)*STEP+K)#                    AZ16 504
QX(TT+K)=B1*DMR9(J*STEP+K)+B2*DMR9((J+1)*STEP+K)#                       AZ16 505
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AZ16 506
ISTOPE=ISTOPT#                         Set list markers                 AZ16 507
RETURN#                                                                 AZ16 508
END#                                                                    AZ16 509
#                                                                       AZ16 510
#                                                                       AZ16 511
systemheader:(AZ20)#                                                    AZ20   1
####################################################################### AZ20   2
#                                                                     # AZ20   3
# CALLS REFLECTIONS TREATEMENT SUBROUTINES (CALLED FOR EACH REFL.)    # AZ20   4
# TESTS IF REFLECTION IS FLAGGED BY USER TO BE SKIPPED                # AZ20   5
# ------------------------------------------------------------------- # AZ20   6
# * AZ21: FINDS ORIENTATION OF BEAMS                                  # AZ20   7
# * AZ22: COMPUTES T, TBAR AND DT/D(DL): GAUSSIAN INTEGRATION ROUTINE # AZ20   8
# * AZ23: STORE EVERYTHING ON BDF AND PRINTS REFLECTION REPORT        # AZ20   9
# * AZ24: MAIN ANALYTICAL INTEGRATION SUBROUTINE                      # AZ20  10
#                                                                     # AZ20  11
####################################################################### AZ20  12
SUBROUTINE AZ20#                                                        AZ20  13
#                                                                       AZ20  14
include:(AACOMN)#                                                       AZ20  15
include:(AZCOMN)#                                                       AZ20  16
#                                                                       AZ20  17
INTEGER NN#                            Main noabs pointer               AZ20  18
INTEGER I#                             Loop index                       AZ20  19
REAL THETA#                            Theta angle in degrees           AZ20  20
REAL B1,B2#                            Weights for absorption of sphe   AZ20  21
INTEGER SKIP#                          Must the reflection be proceed   AZ20  22
#                                                                       AZ20  23
NREFL=NREFL+1#                         Increment reflection read coun   AZ20  24
SKIP=no:#                              Default: treat reflection        AZ20  25
IF(ISTOPN>MARKN)#                      Test if reflection must be ski   AZ20  26
DO NN=MARKN,ISTOPN-nstepn:,nstepn:#    Loop over noabs buffer           AZ20  27
IF((NREFL>=NINT(minn:(NN)))& $#                                         AZ20  28
(NREFL<=NINT(maxn:(NN))))SKIP=yes:#    Skip reflection                  AZ20  29
IF(SKIP==no:)#                         Observed reflection              AZ20  30
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AZ20  31
NPROC=NPROC+1#                         Increment proceeded refection    AZ20  32
CALL AZ21#                             Get p & d vectors in the cryst   AZ20  33
IF(NFACES==1)#                         Spherical absorption             AZ20  34
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AZ20  35
THETA=ATAN2(SINT,COST)/rad:#           Get theta in degrees             AZ20  36
I=NINT(THETA/5.0-0.5)+1#               Get pointer                      AZ20  37
B2=THETA/5.0-REAL(I-1)#                Fraction of delta                AZ20  38
B1=1.0-B2#                             Get weights                      AZ20  39
A=dble:(B1*QX(MARKT+I)+B2*QX(MARKT+I+1))# Get mean value                AZ20  40
TBAR=facer:*(B1*QX(MARKT+19+I)+B2*QX(MARKT+19+I+1))#                    AZ20  41
faceq:(MARKF)=MU*(B1*QX(MARKT+19+I)+B2*QX(MARKT+19+I+1))/sngl:(A)#      AZ20  42
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AZ20  43
ELSE IF(GAUSS==yes:)CALL AZ22#         Calculate a, da/dmu, da/d(dl)    AZ20  44
ELSE CALL AZ24#                        Analytical method                AZ20  45
CALL AZ23#                             Store and print everything       AZ20  46
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AZ20  47
ELSE#                                  Clear bdf buffer                 AZ20  48
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AZ20  49
IF(STORE==yes:)#                                                        AZ20  50
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AZ20  51
DO I=1,NFACES;QX(IPP+IREL(9+I))=voidflg:# Da/d(dl) = 0.0                AZ20  52
QX(IPP+IREL(7))=voidflg:;QX(IPP+IREL(8))=voidflg:#                      AZ20  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AZ20  54
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AZ20  55
RETURN#                                                                 AZ20  56
END#                                                                    AZ20  57
#                                                                       AZ20  58
#                                                                       AZ20  59
systemheader:(AZ21)#                                                    AZ21   1
####################################################################### AZ21   2
#                                                                     # AZ21   3
# COMPUTES NORMALIZED INCIDENT BEAM (PC) AND DIFFRACTED BEAM (DC) IN  # AZ21   4
# CRYSTAL ORTHONORMAL BASE. FORMULAES:                                # AZ21   5
# ------------------------------------------------------------------- # AZ21   6
# TRAVAILLONS DANS UNE BASE ORTHONORMALE (U X R) OU:                  # AZ21   7
#      H    (0)         S0          S         (0)                     # AZ21   8
# X = --- = (1),   P = ----,   D = ---,   R = (0)                     # AZ21   9
#     |H|   (0)        |S0|        |S|        (1)                     # AZ21  10
# ------------------------------------------------------------------- # AZ21  11
# DEFINITIONS:                                                        # AZ21  12
# PSI = ANGLE ENTRE D^P ET R (DANS CET ORDRE => 0 < PSI < 2*PI)       # AZ21  13
#      S^S0     H^S0     D^P     X^P                                  # AZ21  14
# V = ------ = ------ = ----- = -----                                 # AZ21  15
#     |S^S0|   |H^S0|   |D^P|   |X^P|                                 # AZ21  16
# ------------------------------------------------------------------- # AZ21  17
#     (COS(PSI+PI/2))   (-SIN(PSI))                                   # AZ21  18
# V = (      0      ) = (    0    ) CAR PSI = ANGLE ENTRE V ET R      # AZ21  19
#     (SIN(PSI+PI/2))   ( COS(PSI))                                   # AZ21  20
#                                                                     # AZ21  21
#      D+P           |E1  -SIN(PSI)  0|   (-COS(PSI))                 # AZ21  22
# =>  ----- = V^X  = |E2      0      1| = (    0    )                 # AZ21  23
#     |D+P|          |E3   COS(PSI)  0|   (-SIN(PSI))                 # AZ21  24
#                                                                     # AZ21  25
#                                     D+P                             # AZ21  26
# =>  P = -SIN(THETA)*X + COS(THETA)*-----   ET                       # AZ21  27
#                                    |D+P|                            # AZ21  28
#                                     D+P                             # AZ21  29
# =>  D =  SIN(THETA)*X + COS(THETA)*-----                            # AZ21  30
#                                    |D+P|                            # AZ21  31
#                                                                     # AZ21  32
#         (- COS(THETA)*COS(PSI))           (- COS(THETA)*COS(PSI))   # AZ21  33
# =>  P = (    - SIN(THETA)     )       D = (      SIN(THETA)     )   # AZ21  34
#         (- COS(THETA)*SIN(PSI))           (- COS(THETA)*SIN(PSI))   # AZ21  35
#                                                                     # AZ21  36
####################################################################### AZ21  37
SUBROUTINE AZ21#                                                        AZ21  38
#                                                                       AZ21  39
include:(AACOMN)#                                                       AZ21  40
include:(AZCOMN)#                                                       AZ21  41
#                                                                       AZ21  42
matrix:(MTEMP)#                        Matrice de passage               AZ21  43
vector:(PT)#                           Normalized incident   beam       AZ21  44
vector:(DT)#                           Normalized diffracted beam       AZ21  45
vector:(R)#                            Normalized r-vector                      
vector:(X)#                            Normalised scattering vector             
vector:(U)#                            Third base vector                        
dp: DTEMP#                             General usage                            
INTEGER I#                             Loop index                               
#                                                                               
IF(dabs:(dot:(HM,RM))>dsmall:)iquit:(AZ2101)# R-vector isn't perpendi           
postm:(B,HM,X);norm:(X,X,DTEMP)#       Construct base                           
transp:(BINV,MTEMP)#                   Transpose binv to go in rec. s           
postm:(MTEMP,RM,R);norm:(R,R,DTEMP)#   R in crystal base                        
cross:(X,R,U)#                         X, r, u form a right-handed ba           
DO I=1,3#                              Construct passage matrix                 
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
MTEMP(I,1)=U(I);MTEMP(I,2)=X(I);MTEMP(I,3)=R(I)#                                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
DT(1)=-dble:(COSP*COST)#               Diffracted beam in current bas           
DT(2)=dble:(SINT)#                                                              
DT(3)=-dble:(SINP*COST)#                                                        
PT(1)=DT(1)#                           Incident beam in current base            
PT(2)=-DT(2)#                                                                   
PT(3)=DT(3)#                                                                    
postm:(MTEMP,DT,DC);postm:(MTEMP,PT,PC)# Back to crystal base                   
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ22)#                                                            
#######################################################################         
#                                                                     #         
# A    = (SOMME      EXP(-MU(T1+T2)) * DVOL) / VOL                    #         
#         GRID POINTS                                                 #         
# TBAR = (SOMME      (T1+T2) * EXP(-MU(T1+T1)) * DVOL) / VOL          #         
#         GRID POINTS                                                 #         
# QL   =  SOMME  QI + SOMME   QO                                      #         
#         INFACES     OUTFACES                                        #         
# SL   =  SOMME              EXP(-MU(T1+T2)) * DS                     #         
#         SURFACE GRID POINTS                                         #         
# COMPUTES DA/D(DL) FROM QL BY THE EQUATION:                          #         
#    DA       1                                                       #         
#   ----- = - - * [A*SL + MU*QL - SL] [FLACK & SCHWARZENBACH]         #         
#   D(DL)     V                                                       #         
# ------------------------------------------------------------------- #         
# DA     = EXP(-MU(T1+T2)) * DVOL [STORED IN GRIDV:(GG)]              #         
# COS(I) = COS(ANGLE BETWEEN INCIDENT   BEAM AND NORMAL TO FACE I)    #         
# COS(O) = COS(ANGLE BETWEEN DIFFRACTED BEAM AND NORMAL TO FACE I)    #         
# QI = SOMME                                               DA/COS(I)  #         
#      GRID POINTS CAUGHT BY INCIDENT   BEAM THROUGH FACE I           #         
# QO = SOMME                                               DA/COS(O)  #         
#      GRID POINTS CAUGHT BY DIFFRACTED BEAM THROUGH FACE O           #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ22#                                                                
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER GG#                            Main grid pointer                        
INTEGER SS#                            Main surface grid pointer                
vector:(POINT)#                        Spatial coordonates of a grid            
REAL T1,T2#                            Incident and diffracted path l           
REAL DA#                               Element d'transmittance                  
REAL D#                                Distances from point to faces            
INTEGER I,J,K#                         General purpose loop indexes             
INTEGER FFI,FFO,FF1#                   Pointers in the face array               
#                                                                               
#                                                                               
# INITIALIZATIONS: STORE COSINES OF ANGLE BETWEEN FACE AND BEAMS                
# --------------------------------------------------------------                
#                                                                               
A=DBZ;TBAR=0.0#                        Transmittance and mean path se           
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over all faces                      
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
faceq:(FF)=0.0#                        Derivative set to 0                      
NC(1)=dble:(faceh:(FF))#                                                        
NC(2)=dble:(facek:(FF))#                                                        
NC(3)=dble:(facel:(FF))#               Normal to face                           
faceai:(FF)=-sngl:(dot:(NC,PC))#                                                
faceao:(FF)=sngl:(dot:(NC,DC))#        Cosines between face and beams           
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
#                                                                               
# LOOP OVER ALL BULK GRID POINTS                                                
# ------------------------------                                                
#                                                                               
DO I=1,NPTX#                           Loop over x divisions                    
DO J=1,NPTY#                           Loop over y divisions                    
DO K=1,NPTZ#                           Loop over z divisions                    
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
GG=getg:(I,J,K)#                       Get grid point address                   
POINT(1)=gridx:(GG)#                   Get grid point coordinates               
POINT(2)=gridy:(GG)#                                                            
POINT(3)=gridz:(GG)#                                                            
T1=large:;T2=large:#                   Set optical paths to inifinity           
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over all faces                      
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
NC(1)=dble:(faceh:(FF))#               Normal to face                           
NC(2)=dble:(facek:(FF))#                                                        
NC(3)=dble:(facel:(FF))#                                                        
D=faced:(FF)-sngl:(dot:(NC,POINT))#    Distance from point to face              
#                                                                               
# FOR EACH FACE, FINDS THE MIN DISTANCE FROM POINT TO FACE                      
# --------------------------------------------------------                      
#                                                                               
IF(((D/faceai:(FF))<T1)&(faceai:(FF)>small:))# Choose min optical pat           
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
T1=D/faceai:(FF)#                      Optical path                             
FFI=FF#                                Pointer to input face                    
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
IF(((D/faceao:(FF))<T2)&(faceao:(FF)>small:))# Choose min optical pat           
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
T2=D/faceao:(FF)#                      Optical path                             
FFO=FF#                                Pointer to output face                   
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
DA=gridv:(GG)*EXP(-(T1+T2)*MU)#        Contribution to transmittance            
A=A+dble:(DA)#                         Accumulate transmittance                 
TBAR=TBAR-(T1+T2)*DA#                  Accumulate mean path                     
faceq:(FFI)=faceq:(FFI)-MU*DA/faceai:(FFI)# Bulk contr. to derivative           
faceq:(FFO)=faceq:(FFO)-MU*DA/faceao:(FFO)# Bulk contr. to derivative           
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
#                                                                               
# LOOP OVER ALL SURFACE GRID POINTS                                             
# ---------------------------------                                             
#                                                                               
FF=MARKF#                              Set face pointer                         
DO I=1,NFACES#                         Loop over all faces                      
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
DO J=1,NPTS#                           Loop over surface grid points            
DO K=1,NPTS#                                                                    
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
SS=gets:(I,J,K)#                       Get surface grid point                   
POINT(1)=dble:(surfx:(SS))#                                                     
POINT(2)=dble:(surfy:(SS))#                                                     
POINT(3)=dble:(surfz:(SS))#            Get coordinates of grid point            
T1=large:;T2=large:#                   Set optical path length to inf           
DO FF1=MARKF,ISTOPF-nstepf:,nstepf:#   Loop over all faces                      
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
NC(1)=dble:(faceh:(FF1))#                                                       
NC(2)=dble:(facek:(FF1))#                                                       
NC(3)=dble:(facel:(FF1))#              Normal to face                           
D=faced:(FF1)-sngl:(dot:(NC,POINT))#                                            
IF(D<-small:)iquit:(AZ2201)#           Point outside crystal                    
IF((D/faceai:(FF1)<T1)&(faceai:(FF1)>small:))# Choose min optical pat           
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
T1=ABS(D/faceai:(FF1))#                Optical path                             
FFI=FF1#                               Input face pointer                       
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
IF((D/faceao:(FF1)<T2)&(faceao:(FF1)>small:))# Choose min optical pat           
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
T2=ABS(D/faceao:(FF1))#                Optical path                             
FFO=FF1#                               Output face pointer                      
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
DA=EXP(-MU*(T1+T2))*surfs:(SS)#        Contribution to transmittance            
faceq:(FF)=faceq:(FF)+DA#              Accumulate derivative                    
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
incrf:(FF)#                            Next face                                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
#                                                                               
# COMPUTES DERIVATIVES                                                          
# --------------------                                                          
#                                                                               
TBAR=-TBAR/sngl:(A);A=A/dble:(VOL)#    Normalize transmittance and me           
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Normalize and correct derivati           
faceq:(FF)=(faceq:(FF)-faces:(FF)*sngl:(A))/ VOL#                               
A=DB1/A#                               Transmittance to absorption              
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ23)#                                                            
#######################################################################         
#                                                                     #         
# * STORES A, TBAR, DA/D(DL) ON BDF                                   #         
# * WRITES REFL. NUMBER, MILLER INDICES, PSI, 1/A, TBAR, CORRECTED I  #         
#   AND E.S.D. (IF STORE = YES:), MILLER INDICES OF R-VECTOR AND      #         
#   THETA FOR USER                                                    #         
# * OUTPUTS REFL. NUMBER, MILLER INDICES, CORRECTED I AND E.S.D. (SET #         
#   TO 0 IF STORE = NO:), PSI AND MILLER INDICES OF R-VECTOR TO       #         
#   PUNCH FILE                                                        #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ23#                                                                
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER I#                                                                      
REAL SUM#                              Used to compute esd of correct           
#                                                                               
chardata:(AZ231," Refl.     h    k    l       psi    Absorp. Mean Pat$#         
h    Intensity   Sigma(I)   Theta")#                                            
realdata:(FORM,[60513.,120313.,170313.,220313.,320722.,430852.,$#               
530752.,661002.,770802.,850522.])#                                              
datastuff:#                                                                     
#                                                                               
IF(STORE==no:)#                        Compute esd ?                            
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
SIGMA=QX(IPP+IREL(4))#                 Get esd of uncorrected                   
SUM=0.0;FF=MARKF#                      Set pointer & clear esd                  
DO I=1,NFACES#                         Loop over faces                          
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
SUM=SUM+(sngl:(A)*faceq:(FF)*facesd:(FF))**2# Uncorrelated faces                
incrf:(FF)#                            Increment pointer                        
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
SUM=(sngl:(A)*INT)**2*SUM+(sngl:(A)*SIGMA)**2# Esd in intensity                 
SUM=SQRT(AMAX1(0.0,SUM))#              Esd                                      
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
#                                                                               
# STORE A, TBAR AND QL ON BDF                                                   
# ---------------------------                                                   
#                                                                               
QX(IPP+IREL(3))=INT;QX(IPP+IREL(4))=SIGMA# Store modified i & esd               
IF(STORE==yes:)#                       Store correction ?                       
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
QX(IPP+IREL(7))=TBAR#                  Store mean path                          
QX(IPP+IREL(8))=sngl:(A)#              Store absorption                         
FF=MARKF#                              Set face pointer                         
DO I=1,NFACES#                         Loop over all faces                      
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
QX(IPP+IREL(8+I))=faceq:(FF)#          Store derivative                         
incrf:(FF)#                            Increment pointer                        
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
ELSE#                                  Apply correction                         
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
QX(IPP+IREL(3))=sngl:(A)*INT#          Store corrected intensity                
QX(IPP+IREL(4))=SUM#                   Store corrected sigma                    
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
IF(NREFL<=LIST)#                       List proceeded reflections               
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
#                                                                               
# WRITE EVERYTHING FOR USER                                                     
# -------------------------                                                     
#                                                                               
BUFOT(1)=REAL(NREFL)#                  Reflection serial nb                     
DO I=1,3;BUFOT(I+1)=sngl:(HM(I))#      Miller indices                           
BUFOT(5)=ATAN2(SINP,COSP)/ rad:#       Psi in degrees                           
IF(BUFOT(5)<0.0)BUFOT(5)=BUFOT(5)+360.0# Always positive                        
BUFOT(6)=sngl:(A);BUFOT(7)=TBAR#       Absorption and mean path                 
BUFOT(8)=INT;BUFOT(9)=SIGMA#           Intensity & esd                          
BUFOT(10)=ATAN2(SINT,COST)/ rad:#      Theta in degrees                         
ncodefld:(BUFOT,1,CHROT,FORM,10)#      Encode everything                        
writeline:(0,AZ231,NAZ231,1,3)#        ...and print it                          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
IF(TBAR<TMIN)TMIN=TBAR#                Set mean path statistics                 
IF(TBAR>TMAX)TMAX=TBAR#                                                         
MEANT=MEANT+TBAR#                                                               
IF(sngl:(A)<ABMIN)ABMIN=sngl:(A)#                                               
IF(sngl:(A)>ABMAX)ABMAX=sngl:(A)#                                               
MEANAB=MEANAB+sngl:(A)#                                                         
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ24)#                                                            
#######################################################################         
#                                                                     #         
# ANALITICAL METHOD (USED FOR EACH REFLECTION)                        #         
# --------------------------------------------                        #         
# 1) DIVIDES CRYSTAL INTO POLYHEDRA                                   #         
# 2) DIVIDES EACH POLYHEDRON INTO TETRAHEDRA                          #         
# 3) COMPUTES TRANSMITTANCE AND DERIVATIVES FOR EACH TETRAHEDRA       #         
# 4) MERGES ALL                                                       #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ24#                                                                
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER TT#                            Main howell's vertices point             
dp: DIST,DTEMP#                                                                 
vector:(X)#                                                                     
INTEGER F(2)#                          Face pointer                             
INTEGER FFP,FFD#                       Input and output face pointers           
INTEGER CC#                            Main cutting plane pointer               
INTEGER I#                             Loop index                               
INTEGER NFACE2#                        Howell's polyhedron face count           
INTEGER RANGE#                         Number of words to move                  
#                                                                               
A=DBZ;TBAR=0.0#                        Clear transmittance and mean p           
#                                                                               
# MARK FACES AS LIT BY PRIMARY OR/AND DIFFRACTED BEAMS                          
# ----------------------------------------------------                          
#                                                                               
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over all faces                      
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
faceq:(FF)=0.0#                        Clear derivatives                        
NC(1)=dble:(faceh:(FF))#               Normal to face                           
NC(2)=dble:(facek:(FF))#                                                        
NC(3)=dble:(facel:(FF))#                                                        
faceai:(FF)=-sngl:(dot:(NC,PC))#       Input  face cosine                       
faceao:(FF)=sngl:(dot:(NC,DC))#        Output face cosine                       
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
#                                                                               
# BUILD CUTTING PLANES                                                          
# --------------------                                                          
#                                                                               
MARKC=ISTOPN#                          Set cutting planes pointer               
CALL AZ25(MARKC,ISTOPC,yes:)#          Build input beam cut planes              
MARKD=ISTOPC#                          Set cut planes pointer                   
CALL AZ25(MARKD,ISTOPD,no:)#           Build output beam cut planes             
#                                                                               
# LOOP OVER VOLUMES VIJ                                                         
# ---------------------                                                         
#                                                                               
DO FFP=MARKF,ISTOPF-nstepf:,nstepf:#   Loop over faces                          
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
IF(faceai:(FFP)<1.0D-4)NEXT#           Skip hidden faces                        
DO FFD=MARKF,ISTOPF-nstepf:,nstepf:#   Loop over faces                          
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
IF(faceao:(FFD)<1.0D-4)NEXT#           Skip if not lit by diffracted            
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over all faces                      
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
facecx:(FF)=0.0;facecy:(FF)=0.0;facecz:(FF)=0.0# Clear center of face           
facen:(FF)=0.0#                        Clear number of vertices aroun           
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
RANGE=ISTOPE-MARKE#                    Number of words to move                  
MARKT=ISTOPD#                          Set howell's polyhedron vertic           
incrqx:(ISTOPT,MARKT+RANGE,AZ2401)#    Get memory                               
movereal:(QX,MARKE+1,QX,MARKT+1,RANGE,0)# Copy edge buffer to temp              
DO CC=MARKC,ISTOPC-nstepc:,nstepc:#    Cut crystal by incident beam             
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
cutcx:(CC)=0.0;cutcy:(CC)=0.0;cutcz:(CC)=0.0;cutn:(CC)=0.0# Clear cf            
FF=NINT(cutf:(CC))#                    Get main address                         
IF(FF !=FFP)NEXT#                      This face is not in-face                 
NC(1)=dble:(cutx:(CC))#                Normal to cutting plane                  
NC(2)=dble:(cuty:(CC))#                                                         
NC(3)=dble:(cutz:(CC))#                                                         
DIST=dble:(cutd:(CC))#                 Distance to origin                       
CALL AZ26(DIST,CC)#                    Cut crystal by this plane                
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
DO CC=MARKD,ISTOPD-nstepc:,nstepc:#    Cut crystal by diffracted beam           
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
cutcx:(CC)=0.0;cutcy:(CC)=0.0;cutcz:(CC)=0.0;cutn:(CC)=0.0#                     
FF=NINT(cutf:(CC))#                                                             
IF(FF !=FFD)NEXT#                                                               
NC(1)=dble:(cutx:(CC))#                                                         
NC(2)=dble:(cuty:(CC))#                                                         
NC(3)=dble:(cutz:(CC))#                                                         
DIST=dble:(cutd:(CC))#                                                          
CALL AZ26(DIST,CC)#                                                             
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
IF(ISTOPT<=MARKT)NEXT#                 No vij volume                            
CM(1)=DBZ;CM(2)=DBZ;CM(3)=DBZ;NEDGE=0;NFACE2=0# Clear cm                        
DO TT=MARKT,ISTOPT-nstepe:,nstepe:#    Loop over howell's edges                 
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
IF(NINT(edgefl:(TT))==0)NEXT#          Does not belong                          
NEDGE=NEDGE+1#                         Count edge                               
F(1)=NINT(edgef1:(TT));F(2)=NINT(edgef2:(TT))# Addresses of faces               
X(1)=dble:(edgex1:(TT)+edgex2:(TT))#   Center of edge                           
X(2)=dble:(edgey1:(TT)+edgey2:(TT))#                                            
X(3)=dble:(edgez1:(TT)+edgez2:(TT))#                                            
DO I=1,2#                              Loop over faces whose edge bel           
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
IF(F(I)<ISTOPF)#                       Crystal face                             
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
facecx:(F(I))=facecx:(F(I))+sngl:(X(1))# Increase center of face                
facecy:(F(I))=facecy:(F(I))+sngl:(X(2))#                                        
facecz:(F(I))=facecz:(F(I))+sngl:(X(3))#                                        
facen:(F(I))=facen:(F(I))+2.0#         Count both extremities                   
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
ELSE#                                  Cutting plane                            
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
cutcx:(F(I))=cutcx:(F(I))+sngl:(X(1))#                                          
cutcy:(F(I))=cutcy:(F(I))+sngl:(X(2))#                                          
cutcz:(F(I))=cutcz:(F(I))+sngl:(X(3))#                                          
cutn:(F(I))=cutn:(F(I))+2.0#                                                    
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Loop over faces                          
IF(NINT(facen:(FF))!=0)#               Belong to vij                            
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11         
NFACE2=NFACE2+1#                       Count face                               
facecx:(FF)=facecx:(FF)/facen:(FF);CM(1)=CM(1)+dble:(facecx:(FF))# Ge           
facecy:(FF)=facecy:(FF)/facen:(FF);CM(2)=CM(2)+dble:(facecy:(FF))#              
facecz:(FF)=facecz:(FF)/facen:(FF);CM(3)=CM(3)+dble:(facecz:(FF))#              
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11         
DO CC=MARKC,ISTOPD-nstepc:,nstepc:#    Loop over all cutting planes             
IF(NINT(cutn:(CC))!=0)#                Belong to vij                            
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12         
NFACE2=NFACE2+1#                       Count face                               
cutcx:(CC)=cutcx:(CC)/cutn:(CC);CM(1)=CM(1)+dble:(cutcx:(CC))#                  
cutcy:(CC)=cutcy:(CC)/cutn:(CC);CM(2)=CM(2)+dble:(cutcy:(CC))#                  
cutcz:(CC)=cutcz:(CC)/cutn:(CC);CM(3)=CM(3)+dble:(cutcz:(CC))#                  
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12         
NVERT=NEDGE+2-NFACE2#                  Get vertex number by euler's l           
IF(NFACE2<4)NEXT#                      Howell's polyhedron not closed           
CM(1)=CM(1)/dfloat:(NFACE2)#           Get center of mass                       
CM(2)=CM(2)/dfloat:(NFACE2)#                                                    
CM(3)=CM(3)/dfloat:(NFACE2)#                                                    
#                                                                               
# BUILDS TETRAHEDRON                                                            
# ------------------                                                            
#                                                                               
NC(1)=dble:((faceh:(FFP)/faceai:(FFP)+faceh:(FFD)/faceao:(FFD))*MU)#            
NC(2)=dble:((facek:(FFP)/faceai:(FFP)+facek:(FFD)/faceao:(FFD))*MU)#            
NC(3)=dble:((facel:(FFP)/faceai:(FFP)+facel:(FFD)/faceao:(FFD))*MU)#            
DTEMP=dot:(NC,CM)-$#                                                            
dble:(MU*(faced:(FFP)/faceai:(FFP)+faced:(FFD)/faceao:(FFD)))#                  
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Crystal faces                            
IF(NINT(facen:(FF))!=0)CALL AZ27(DTEMP,FF,FFP,FFD)#                             
DO CC=MARKC,ISTOPD-nstepc:,nstepc:#    Cutting planes                           
IF(NINT(cutn:(CC))!=0)CALL AZ27(DTEMP,CC,FFP,FFD)#                              
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
#                                                                               
# COMPUTES FRIST DERIVATIVES                                                    
# --------------------------                                                    
#                                                                               
TBAR=TBAR/sngl:(A);A=A/dble:(VOL)#     Normalization                            
DO FF=MARKF,ISTOPF-nstepf:,nstepf:#    Derivatives normalization                
faceq:(FF)=(faceq:(FF)-faces:(FF)*sngl:(A))/VOL#                                
A=DB1/A#                               Transmittance to absorption              
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ25)#                                                            
#######################################################################         
#                                                                     #         
# BUILDS CUTTING PLANES                                               #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ25(MARKX,ISTOPX,INP)#                                              
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER EE#                            Main edges pointer                       
vector:(BEAM)#                         Incident or diffracted beam              
vector:(X);vector:(V);vector:(U)#                                               
dp: DTEMP#                                                                      
INTEGER FF1,FF2#                                                                
INTEGER INP#                                                                    
INTEGER MARKX,XX,ISTOPX#                                                        
#                                                                               
XX=MARKX#                              Set cut plane pointers                   
ISTOPX=MARKX#                                                                   
IF(INP==yes:)#                         Incident beam ?                          
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
BEAM(1)=-PC(1);BEAM(2)=-PC(2);BEAM(3)=-PC(3)# Reverse incident beam             
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
ELSE#                                  Diffracted beam                          
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
BEAM(1)=DC(1);BEAM(2)=DC(2);BEAM(3)=DC(3)# Diffracted beam                      
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
DO EE=MARKE,ISTOPE-nstepe:,nstepe:#                                             
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
FF1=NINT(edgef1:(EE));FF2=NINT(edgef2:(EE))# Get face pointer                   
IF((INP==yes:)&((faceai:(FF1)<1.0D-4)\(faceai:(FF2)<1.0D-4)))NEXT#              
IF((INP==no:)&((faceao:(FF1)<1.0D-4)\(faceao:(FF2)<1.0D-4)))NEXT#               
V(1)=dble:(edgex2:(EE)-edgex1:(EE));U(1)=dble:(edgex1:(EE))#                    
V(2)=dble:(edgey2:(EE)-edgey1:(EE));U(2)=dble:(edgey1:(EE))#                    
V(3)=dble:(edgez2:(EE)-edgez1:(EE));U(3)=dble:(edgez1:(EE))#                    
cross:(V,BEAM,X)#                      X est la normale au plan qui c           
norm:(X,X,DTEMP)#                      Normalize x                              
incrqx:(ISTOPX,XX+2*nstepc:,AZ2501)#   Insufficient space to store cu           
cutx:(XX)=sngl:(X(1));cutx:(XX+nstepc:)=-cutx:(XX)#                             
cuty:(XX)=sngl:(X(2));cuty:(XX+nstepc:)=-cuty:(XX)#                             
cutz:(XX)=sngl:(X(3));cutz:(XX+nstepc:)=-cutz:(XX)# Store normal                
cutd:(XX)=sngl:(dot:(X,U));cutd:(XX+nstepc:)=-cutd:(XX)# Store distan           
cutf:(XX)=edgef1:(EE);cutf:(XX+nstepc:)=edgef2:(EE)#                            
cutf2:(XX)=edgef2:(EE);cutf2:(XX+nstepc:)=edgef1:(EE)# Store face poi           
XX=ISTOPX#                             Next cutting plane                       
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ26)#                                                            
#######################################################################         
#                                                                     #         
# FIND HOWELL'S POLYHEDRON EDGES                                      #         
# ------------------------------------------------------------------- #         
# ALGORITHM USED:                                                     #         
# 1) LA SUBROUTINE RECOIT LA LISTE DES ARETES ET LE COUPLE NORMALE-   #         
#    DISTANCE DU PLAN DE COUPE.                                       #         
# 2) POUR CHAQUE ARETE DE LA LISTE, ON TESTE SI ELLE COUPE ON NON LE  #         
#    PLAN.                                                            #         
# 3) SI ELLE COUPE, 4 CAS PEUVENT SE PRESENTER:                       #         
#    A. ELLE EST COMPELETEMENT DANS LE PLAN DE COUPE                  #         
#    B. ELLE A UNE DE SES EXTREMITES DANS LE PLAN, L'AUTRE ETANT HORS #         
#       DU POLYHEDRE. DANS CE CAS ON EFFACE L'ARETE.                  #         
#    C. ELLE A UNE DE SES EXTREMITES DANS LE PLAN, L'AUTRE ETANT DANS #         
#       LE POLYHEDRE.                                                 #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ26(DIST,CC)#                                                       
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER TT#                            Main howell's vertices pointer           
vector:(X1);vector:(X2);vector:(Y)#                                             
vector:(Y1);vector:(Y2)#                                                        
dp: DIST,PERP#                                                                  
dp: D1,D2,DTEMP#                                                                
INTEGER CC#                                                                     
INTEGER F(2)#                                                                   
INTEGER FF1,FF2,FF3,FF4#                                                        
INTEGER I#                             Loop index                               
INTEGER TT1,TT2#                                                                
#                                                                               
TT2=ISTOPT#                            Set mark for start of new foun           
FF3=NINT(cutf:(CC));FF4=NINT(cutf2:(CC))# Get face addresses                    
FOR(TT=MARKT;TT<TT2;incre:(TT))#       Loop over old edges                      
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
IF(NINT(edgefl:(TT))==0)NEXT#          Edge already deleted                     
F(1)=NINT(edgef1:(TT));F(2)=NINT(edgef2:(TT))# Get face addresses               
X1(1)=dble:(edgex1:(TT));X2(1)=dble:(edgex2:(TT))# Get edge extremiti           
X1(2)=dble:(edgey1:(TT));X2(2)=dble:(edgey2:(TT))#                              
X1(3)=dble:(edgez1:(TT));X2(3)=dble:(edgez2:(TT))#                              
D1=DIST-dot:(NC,X1);D2=DIST-dot:(NC,X2)# Distance to plane                      
IF((D1>dsmall:)&(D2>dsmall:))NEXT#     Edge inside                              
IF((D1<-dsmall:)&(D2<-dsmall:))#       Edge outside plane                       
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
edgefl:(TT)=0.0#                       Delete it                                
NEXT#                                                                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
#                                                                               
# THE EDGE IS INSIDE THE PLANE                                                  
# ----------------------------                                                  
#                                                                               
IF((dabs:(D1)<dsmall:)&(dabs:(D2)<dsmall:))# Edge inside plane                  
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
IF(F(1)<ISTOPF)#                       Crystal face                             
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
Y1(1)=dble:(faceh:(F(1)))#             Normal                                   
Y1(2)=dble:(facek:(F(1)))#                                                      
Y1(3)=dble:(facel:(F(1)))#                                                      
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
ELSE#                                  Cutting plane                            
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
Y1(1)=dble:(cutx:(F(1)))#              Normal                                   
Y1(2)=dble:(cuty:(F(1)))#                                                       
Y1(3)=dble:(cutz:(F(1)))#                                                       
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
IF(F(2)<ISTOPF)#                       Crystal face                             
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
Y2(1)=dble:(faceh:(F(2)))#             Normal                                   
Y2(2)=dble:(facek:(F(2)))#                                                      
Y2(3)=dble:(facel:(F(2)))#                                                      
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
ELSE#                                  Cutting plane                            
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
Y2(1)=dble:(cutx:(F(2)))#              Normal                                   
Y2(2)=dble:(cuty:(F(2)))#                                                       
Y2(3)=dble:(cutz:(F(2)))#                                                       
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
D1=dot:(NC,Y1);D2=dot:(NC,Y2);DTEMP=dot:(Y1,Y2)# Angles between norma           
IF(DTEMP<=D1*D2)#                                                               
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
IF(DTEMP*D1>=D2)#                                                               
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
ISTOPT=MARKT;TT2=ISTOPT#                                                        
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
ELSE#                                                                           
DO TT1=TT2,ISTOPT-nstepe:,nstepe:#                                              
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
FF1=NINT(edgef1:(TT1));FF2=NINT(edgef2:(TT1))#                                  
IF((FF1==CC)\(FF2==CC))edgefl:(TT1)=0.0#                                        
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
BREAK#                                                                          
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
IF(D1<D2)I=1#                                                                   
ELSE I=2#                                                                       
DO TT1=TT2,ISTOPT-nstepe:,nstepe:#                                              
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11         
IF(NINT(edgefl:(TT1))==0)NEXT#                                                  
FF1=NINT(edgef1:(TT1));FF2=NINT(edgef2:(TT1))#                                  
IF(((FF1==CC)&(FF2==F(I)))\((FF1==F(I))&(FF2==CC)))#                            
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12         
edgefl:(TT1)=0.0;BREAK#                                                         
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12         
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11         
IF(I==1)edgef2:(TT)=REAL(CC)#                                                   
ELSE edgef1:(TT)=REAL(CC)#                                                      
NEXT#                                                                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
#                                                                               
# ONE OF THE EXTREMITIES BELONGS TO PLANE                                       
# ---------------------------------------                                       
#                                                                               
IF(dabs:(D1)<dsmall:)#                 X1 is in the cutting plane               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13         
IF(D2>dsmall:)#                        The edge is to be kept                   
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14         
CALL AZ29(X1,CC,F(1),TT2);CALL AZ29(X1,CC,F(2),TT2)# Build new edges            
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14         
ELSE edgefl:(TT)=0.0#                  Delete edge                              
NEXT#                                                                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13         
IF(dabs:(D2)<dsmall:)#                 X2 is in the cutting plane               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15         
IF(D1>dsmall:)#                        The edge is to be kept                   
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16         
CALL AZ29(X2,CC,F(1),TT2);CALL AZ29(X2,CC,F(2),TT2)# Build new edges            
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16         
ELSE edgefl:(TT)=0.0#                  Delete edge                              
NEXT#                                                                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15         
#                                                                               
# THE PLANE CUTS THE EDGE                                                       
# -----------------------                                                       
#                                                                               
DO I=1,3;Y(I)=X2(I)-X1(I)#             Direction of edge                        
PERP=D1/dot:(NC,Y)#                    Find mult factor                         
DO I=1,3;Y(I)=X1(I)+PERP*Y(I)#         Find new extremity                       
IF(D1<DBZ)#                            Extremity 1 to be replaced               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17         
edgex1:(TT)=sngl:(Y(1))#               Store vertex                             
edgey1:(TT)=sngl:(Y(2))#                                                        
edgez1:(TT)=sngl:(Y(3))#                                                        
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17         
ELSE#                                  Extremity 2 to be replaced               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18         
edgex2:(TT)=sngl:(Y(1))#               Store vertex                             
edgey2:(TT)=sngl:(Y(2))#                                                        
edgez2:(TT)=sngl:(Y(3))#                                                        
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18         
CALL AZ29(Y,CC,F(1),TT2);CALL AZ29(Y,CC,F(2),TT2)# Build new edges              
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
DO TT=TT2,ISTOPT-nstepe:,nstepe:#                                               
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19         
IF(NINT(edgefl:(TT))==0)NEXT#                                                   
IF(NINT(edgefl:(TT))==1)edgefl:(TT)=0.0#                                        
F(1)=NINT(edgef1:(TT));F(2)=NINT(edgef2:(TT))#                                  
DO TT1=MARKT,TT2-nstepe:,nstepe:#                                               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20         
IF(NINT(edgefl:(TT))==0)NEXT#                                                   
FF1=NINT(edgef1:(TT1));FF2=NINT(edgef2:(TT1))#                                  
IF(((FF1==F(1))&(FF2==F(2)))\((FF1==F(2))&(FF2==F(1))))#                        
BREAK#                                                                          
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20         
IF(TT1<TT2)edgefl:(TT)=0.0#                                                     
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19         
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ27)#                                                            
#######################################################################         
#                                                                     #         
# FOR EACH FACE, DIVIDES THE POLYHEDRON INTO TETRAHEDRA               #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ27(DTEMP,FF,FFP,FFD)#                                              
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER TT#                            Main howell's vertives pointer           
vector:(CF)#                           Center of face                           
dp: DIST,DTEMP#                                                                 
INTEGER FFP,FFD,FF1,FF2#               Current inp and out faces                
#                                                                               
IF(FF<ISTOPF)#                         Crystal face                             
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
CF(1)=dble:(facecx:(FF))-CM(1)#        Translation                              
CF(2)=dble:(facecy:(FF))-CM(2)#                                                 
CF(3)=dble:(facecz:(FF))-CM(3)#                                                 
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
ELSE#                                  Cutting plane                            
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
CF(1)=dble:(cutcx:(FF))-CM(1)#         Translation                              
CF(2)=dble:(cutcy:(FF))-CM(2)#                                                  
CF(3)=dble:(cutcz:(FF))-CM(3)#                                                  
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
DIST=dot:(CF,NC)#                                                               
DO TT=MARKT,ISTOPT-nstepe:,nstepe:#    Loop over edges                          
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
IF(NINT(edgefl:(TT))==0)NEXT#          Deleted edge                             
FF1=NINT(edgef1:(TT));FF2=NINT(edgef2:(TT))# Get face addresses                 
IF((FF1 !=FF)&(FF2 !=FF))NEXT#         Edge don't belong to face                
CALLAZ28(CF,DTEMP,DIST,TT,FF,FFP,FFD)# Compute t, tbar, dt/d(dl)                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ28)#                                                            
#######################################################################         
#                                                                     #         
# TRANSMITTANCE, MEAN PATH AND DERIVATIVES                            #         
# ------------------------------------------------------------------- #         
# COMPUTES ALL OF THESE FOR A TETRAHEDRON BASED IN ORIGIN.            #         
# FORMULAS:                                                           #         
#     1 - EXP(X)    1 - EXP(Y)    1 - EXP(Z)                          #         
# T = ----------- + ----------- + -----------                         #         
#     X(X-Y)(Z-X)   Y(Y-Z)(X-Y)   Z(Z-X)(Y-Z)                         #         
#                                                                     #         
#                          EXP(X)       EXP(Y)       EXP(Z)           #         
# INTEGRALE DE SURFACE = ---------- + ---------- + ----------         #         
#                        (X-Y)(Z-X)   (Y-Z)(X-Y)   (Z-X)(Y-Z)         #         
#                                                                     #         
# DANS LES CAS ON LES ARGUMENTS SONT PROCHES OU PROCHES DE 0, ON UTI- #         
# LISE 2 TYPES DE D.L.:                                               #         
#                                                                     #         
# 1 - EXP(X)                                                          #         
# ---------- = -(1+X/2) [+O(X^2)] SI X < 10^-7 => ERREUR < 10^-14     #         
#     X                                                               #         
#                                                                     #         
#    F(X)         F(X)         F(X)                                   #         
# ---------- + ---------- + ---------- = -(1/6F''(X)+.... SI X=Y=Z    #         
# (X-Y)(Z-X)   (Y-Z)(X-Y)   (Z-X)(Y-Z)                                #         
#                                                                     #         
# AVEC F(X) = EXP(X) OU (1-EXP(X))/X SUIVANT L'INTEGRALE A RESOUDRE   #         
#                                                                     #         
# IL EST IMPORTANT DE NOTER QUE X <= Y <= Z ET D(I) = X(J) - X(K)     #         
# (PREMUTATION CYCLIQUE)                                              #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ28(CF,DTEMP,DIST,TT,FF,FFP,FFD)#                                   
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER TT#                            Main howell's vertices pointer           
vector:(V1);vector:(V2)#               Edge extremities                         
vector:(X0)#                           (exp(x(i))-1)/x(i)                       
vector:(D)#                            D(i) = x(j)-x(k) [permutation            
vector:(X)#                            X(i) = v(i) . (normal)                   
vector:(CF)#                           Center of face                           
dp: DELA,DELT#                         Increase of transmittance and            
dp: DTEMP#                             Distance from center of mass t           
dp: DET#                               Final multiplicative constant            
dp: DIST#                              (center of face) . (normal)              
dp: DELV,SURF#                         Volume and surface (111) of te           
dp: X1,X2,X3,X4,X5,X6#                 Dl of (1-exp(y))/y (orders 1 t           
dp: K,KP,Y#                            Used to get derivatives                  
INTEGER FFP,FFD#                       Input and output face pointers           
INTEGER I,J#                           Loop indexes                             
#                                                                               
dbledata:(FACT0,[1.0D0,2.0D0,6.0D0,2.4D1,1.2D2,7.2D2,5.04D3,$#                  
4.032D4,3.6288D5,3.6288D6,3.99168D7,4.790016D8,6.2270208D9])#                   
dbledata:(DB4,4.0D0)#                                                           
dbledata:(DB5,5.0D0)#                                                           
dbledata:(DB6,6.0D0)#                                                           
datastuff:#                                                                     
#                                                                               
V1(1)=dble:(edgex1:(TT))-CM(1);V2(1)=dble:(edgex2:(TT))-CM(1)# Transl           
V1(2)=dble:(edgey1:(TT))-CM(2);V2(2)=dble:(edgey2:(TT))-CM(2)#                  
V1(3)=dble:(edgez1:(TT))-CM(3);V2(3)=dble:(edgez2:(TT))-CM(3)#                  
cross:(V1,V2,X0);DELV=dabs:(dot:(X0,CF))# Triple product (delv = 6*vo           
DET=dexp:(DTEMP)*DELV#                 Get mult constant                        
IF(DET==DBZ)RETURN#                    Constant too small                       
X(1)=dot:(V1,NC);X(2)=dot:(V2,NC);X(3)=DIST#                                    
DO I=1,2;DO J=I+1,3#                   Order x                                  
IF(X(I)>X(J))#                                                                  
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
DELV=X(I);X(I)=X(J);X(J)=DELV#                                                  
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
D(1)=X(2)-X(3);D(2)=X(3)-X(1);D(3)=X(1)-X(2)# Differences                       
DO I=1,3#                              Compute (1-exp(x))/x                     
IF(dabs:(X(I))<5.0D-2)#                                                         
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
X0(I)=dexp:(X(I));Y=X0(I)#                                                      
DO J=1,6#                                                                       
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
Y=-Y*X(I);X0(I)=X0(I)+Y/FACT0(J+1)#                                             
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
ELSE X0(I)=(dexp:(X(I))-DB1)/X(I)#                                              
IF(dabs:(X(2))<5.0D-2)#                Compute derivatives of x(2) if           
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
Y=dexp:(X(2))#                                                                  
X1=DBZ;X2=DBZ;X3=DBZ;X4=DBZ;X5=DBZ;X6=DBZ#                                      
DO I=1,7#                                                                       
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
X1=X1+Y/FACT0(I+1);X2=X2+Y/FACT0(I+2);X3=X3+Y/FACT0(I+3)#                       
X4=X4+Y/FACT0(I+4);X5=X5+Y/FACT0(I+5);X6=X6+Y/FACT0(I+6)#                       
Y=-X(2)*Y#                                                                      
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
ELSE#                                  Derivatives of x(2) (exact for           
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
K=dexp:(X(2));X1=-(X0(2)-K)/X(2)#                                               
X2=-(X1-K/FACT0(2))/X(2);X3=-(X2-K/FACT0(3))/X(2)#                              
X4=-(X3-K/FACT0(4))/X(2);X5=-(X4-K/FACT0(5))/X(2)#                              
X6=-(X5-K/FACT0(6))/X(2)#                                                       
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
IF(dabs:(D(2))<1.0D-3)#                X(1) = x(2) = x(3)                       
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
K=D(3)-D(1);KP=D(3)**2+D(1)**2;Y=KP-D(1)*D(3)#                                  
DELA=X2+X3*K+X4*Y+X5*K*KP+X6*(K**2*Y+(D(1)*D(3))**2)#                           
DELT=dexp:(X(2))*(DB1/FACT0(2)+K/FACT0(3)+Y/FACT0(4)+$#                         
K*KP/FACT0(5)+(K**2*Y+(D(1)*D(3))**2)/FACT0(6))#                                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
ELSE#                                                                           
IF(dabs:(D(1))<1.0D-3)#                X(2) = x(3)                              
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
DELA=-(X0(1)-X0(2))/(D(2)*D(3))-$#                                              
(-X1+D(1)*X2-D(1)**2*X3+D(1)**3*X4-D(1)**4*X5+D(1)**5*X6)/D(2)#                 
DELT=-(dexp:(X(1))-dexp:(X(2)))/(D(2)*D(3))-dexp:(X(2))*$#                      
(-DB1+D(1)/FACT0(2)-D(1)**2/FACT0(3)+D(1)**3/FACT0(4)-$#                        
D(1)**4/FACT0(5)+D(1)**5/FACT0(6))/D(2)#                                        
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
ELSE IF(dabs:(D(3))<1.0D-3)#           X(1) = x(2)                              
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
DELA=-(X0(3)-X0(2))/(D(1)*D(2))-$#                                              
(X1+D(3)*X2+D(3)**2*X3+D(3)**3*X4+D(3)**4*X5+D(3)**5*X6)/D(2)#                  
DELT=-(dexp:(X(3))-dexp:(X(2)))/(D(1)*D(2))-dexp:(X(2))*$#                      
(DB1+D(3)/FACT0(2)+D(3)**2/FACT0(3)+D(3)**3/FACT0(4)+$#                         
D(3)**4/FACT0(5)+D(3)**5/FACT0(6))/D(2)#                                        
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
ELSE#                                  Nothing special                          
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
DELA=-X0(1)/(D(2)*D(3))-X0(2)/(D(3)*D(1))-X0(3)/(D(1)*D(2))#                    
DELT=-(D(1)*dexp:(X(1))+D(2)*dexp:(X(2))+D(3)*dexp:(X(3)))/ $#                  
(D(1)*D(2)*D(3))#                                                               
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
IF(DELA<DBZ)DELA=DBZ#                                                           
A=A+DET*DELA#                          Increase tranmittance                    
TBAR=TBAR+sngl:(DET*((DB3-DTEMP)*DELA-DELT))/MU# Increase mean path             
IF(FF<ISTOPF)#                         Crystal face (could be varied)           
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11         
V1(1)=V1(1)-CF(1);V1(2)=V1(2)-CF(2);V1(3)=V1(3)-CF(3)# Translation in           
V2(1)=V2(1)-CF(1);V2(2)=V2(2)-CF(2);V2(3)=V2(3)-CF(3)#                          
cross:(V1,V2,X0);norm:(X0,X0,SURF)#    Get surface                              
faceq:(FF)=faceq:(FF)+sngl:(dexp:(DTEMP)*SURF*DELT)#                            
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11         
faceq:(FFP)=faceq:(FFP)-MU*sngl:(DET*DELA)/faceai:(FFP)#                        
faceq:(FFD)=faceq:(FFD)-MU*sngl:(DET*DELA)/faceao:(FFD)#                        
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ29)#                   Insert a new edge in list                
#######################################################################         
#                                                                     #         
# INSERTION OF A NEW EDGE IN LIST                                     #         
# ------------------------------------------------------------------- #         
# ALGORITHM USED:                                                     #         
# 1) TEST SI CETTE ARETE APPARAIT DEJA DANS LA LISTE, OU SI ELLE EST  #         
#    DE LONGUEUR NULLE. DANS CE CAS, ELLE EST IGNOREE.                #         
#    PAR CONTRE, SI ELLE APPARAIT DANS LA LISTE AVEC SEULEMENT UNE    #         
#    EXTREMITE CONNUE, ALORS ON STOCKE L'AUTRE EXTREMITE ET ON TESTE  #         
#    L'ORIENTATION.                                                   #         
# 2) SI C'EST VRAIMENT UNE NOUVELLE ARETE, ON STOCKE LES ADRESSES DES #         
#    PLANS QUI FORMENT L'ARETE ET LES COORDONNEES DU POINT EN QUES-   #         
#    TION.                                                            #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ29(Y2,CC,FF,TT2)#                                                  
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Main faces pointer                       
INTEGER TT#                            Main howell's vertices pointer           
vector:(X);vector:(Y1);vector:(Y2)#    Vertices                                 
dp: DTEMP#                             Distance between vertices                
INTEGER CC#                            Address of cutting plane                 
INTEGER FF1,FF2#                       Face addresses                           
INTEGER I#                             Loop index                               
INTEGER K#                             New edge flag                            
INTEGER TT2,TT3#                       Polyhedrons edges pointers               
#                                                                               
K=0#                                   Default: new edge                        
DO TT=TT2,ISTOPT-nstepe:,nstepe:#      Loop over edges found by same            
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
IF(NINT(edgefl:(TT))!=1)NEXT#          Complete or deleted edge                 
FF1=NINT(edgef1:(TT));FF2=NINT(edgef2:(TT))# Face addresses                     
IF((FF1==FF)\(FF2==FF))#               They have both face in common            
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
Y1(1)=dble:(edgex1:(TT))#              Get already known vertex                 
Y1(2)=dble:(edgey1:(TT))#                                                       
Y1(3)=dble:(edgez1:(TT))#                                                       
DO I=1,3;X(I)=Y2(I)-Y1(I)#             Distance with new found vertex           
norm:(X,X,DTEMP)#                                                               
IF(DTEMP<dsmall:)#                     Same point                               
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
edgefl:(TT)=0.0#                       Delete edge                              
BREAK#                                 Exit loop                                
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
edgex2:(TT)=sngl:(Y2(1))#              Store new found vertex                   
edgey2:(TT)=sngl:(Y2(2))#                                                       
edgez2:(TT)=sngl:(Y2(3))#                                                       
edgefl:(TT)=2.0#                       Update flag                              
IF(FF<ISTOPF)#                         Crystal face                             
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
Y1(1)=dble:(faceh:(FF))#               Get normal                               
Y1(2)=dble:(facek:(FF))#                                                        
Y1(3)=dble:(facel:(FF))#                                                        
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
ELSE#                                  Cutting plane                            
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
Y1(1)=dble:(cutx:(FF))#                Get normal                               
Y1(2)=dble:(cuty:(FF))#                                                         
Y1(3)=dble:(cutz:(FF))#                                                         
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
cross:(Y1,NC,Y1)#                      Test orientation                         
IF(dot:(X,Y1)<DBZ)#                                                             
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
edgef1:(TT)=REAL(CC);edgef2:(TT)=REAL(FF)# Save face addresses                  
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
ELSE#                                                                           
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
edgef1:(TT)=REAL(FF);edgef2:(TT)=REAL(CC)#                                      
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
K=1#                                   Edge completed                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
IF(K==0)#                              New edge                                 
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
TT3=ISTOPT#                            Save end of list                         
incrqx:(ISTOPT,TT3+nstepe:,AZ2901)#    Get memory                               
edgex1:(TT3)=sngl:(Y2(1));edgex2:(TT3)=0.0# Store vertex                        
edgey1:(TT3)=sngl:(Y2(2));edgey2:(TT3)=0.0#                                     
edgez1:(TT3)=sngl:(Y2(3));edgez2:(TT3)=0.0#                                     
edgefl:(TT3)=1.0#                      Set flag                                 
edgef1:(TT3)=REAL(CC);edgef2:(TT3)=REAL(FF)# Store face addresses               
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ31)#                                                            
#######################################################################         
#                                                                     #         
# OPENS OR CLOSES THE LRREFL: LOGICAL RECORD: (MODE = YES: OR NO:)    #         
# TESTS IF MANDATORY INPUTS ARE ON BDF.                               #         
# MANDATORY INPUT : - MILLER INDICES OF REFLECTION                    #         
#                   - AZIMUTHAL ANGLE                                 #         
# OPTIONAL INPUT  : - INTENSITY AND E.S.D.                            #         
#                   - SIN(THETA)/LAMBDA                               #         
#                   - MILLER INDICES OF R-VECTOR                      #         
# MANDATORY OUTPUT: - TRANSMISSION FACTOR A                           #         
#                   - MEAN PATH TBAR                                  #         
#                   - FIRST DERIVATIVES WITH RESPECT TO DISTANCES     #         
# OPTIONAL OUTPUT : - MODIFIED INTENSITY AND E.S.D.                   #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ31(MODE,SHAPE)#                                                    
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER I#                             Loop index                               
INTEGER MODE,SHAPE#                    Open/close mode, crystal shape           
string:(DESCR,40)#                                                              
#                                                                               
chardata:(AZ311," sin(theta)/lambda taken from file")#                          
chardata:(AZ312," sin(theta)/lambda calculated")#                               
chardata:(AZ313," warning: The Friedel mate is present on BDF, but $#           
its absorption correction will not be computed")#                               
chardata:(AZ314,[ANALYTICAL INTEGRATION GAUSS GRID XX*XX*XX $#                  
(XX*XX) AUTOMATIC   DR])#                                                       
chardata:(AZ315,"No absorption correction applied")#                            
realdata:(FMT,[130213.,160213.,190213.,230213.,260213.,$#                       
320413.,360413.,400413.])#                                                      
datastuff:#                                                                     
#                                                                               
IF(MODE==yes:)#                                                                 
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
KEY(1)=4;KEY(2)=6;KEY(3)=0;KEY(4)=0#                                            
IWANT(1)=1#                            Needed: hkl (packed form)                
IWANT(2)=ISET+11#                      Psi                                      
IWANT(3)=ISET+300#                     Instensity                               
IWANT(4)=ISET+301#                     E.s.d.                                   
IWANT(5)=2#                            Sin(theta)/lambda                        
IWANT(6)=ISET+310#                     Instensity of friedel mate               
IF(STORE==yes:)#                                                                
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
IWANT(7)=ISET+200#                     Tbar                                     
IWANT(8)=ISET+201#                     A                                        
DO I=1,NFACES;IWANT(I+8)=ISET+210+I#   Da/d(dl)                                 
KEY(3)=NFACES+8#                                                                
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
indexpkt:(1,lrrefl:,PSIZE,IP,3,KEY,IWANT,IREL)#                                 
IF(IP<=0)iquit:(AZ3101)#               Missing lrrefl: form bdf                 
IF(KEY(4)<0)iquit:(AZ3102)#            Missing miller indices or psi            
CALCT=yes:#                            Default: calculate theta                 
IF(IREL(5)!=0)#                        Theta present on file so use i           
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
CALCT=no:#                             Set flag                                 
writeline:(1,AZ311,NAZ311,3,3)#        Print theta on bdf                       
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
ELSE writeline:(1,AZ312,NAZ312,3,3)#   Print theta calculated                   
IF(IREL(6)>0)writeline:(1,AZ313,NAZ313,3,3)# Print warning                      
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
ELSE#                                  Close bdf                                
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
ISET=ISET/1000#                        Reset data set number                    
movechar:(BLANK,1,DESCR,1,40,2)#       Clear descr                              
IF(GAUSS==no:)movechar:(AZ314,1,DESCR,1,23,0)# Move anlytical                   
ELSE IF(GAUSS==yes:)#                  Could be 2 if no absorption ap           
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
movechar:(AZ314,24,DESCR,1,40,0)#      Move gaussian                            
BUFOT(1)=REAL(NPTX);BUFOT(2)=REAL(NPTY);BUFOT(3)=REAL(NPTZ)#                    
BUFOT(4)=REAL(NPTS);BUFOT(5)=REAL(NPTS);I=5#                                    
IF(SHAPE !=0)#                                                                  
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
BUFOT(6)=sngl:(AXE(1));BUFOT(7)=sngl:(AXE(2));BUFOT(8)=sngl:(AXE(3))#           
I=8#                                                                            
IF(SHAPE>0)movechar:(AZ314,64,DESCR,28,1,0)# Move d                             
ELSE movechar:(AZ314,65,DESCR,28,1,0)# Move r                                   
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
ncodefld:(BUFOT,1,DESCR,FMT,I)#        Encode numbers                           
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
ELSE#                                  No absorptin correction                  
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
movechar:(AZ315,1,DESCR,1,NAZ315,0)#   Move no correct                          
ABMIN=voidflg:;ABMAX=voidflg:#         Void min/max a*                          
MEANT=0.03;NPROC=1#                    Fix mean tbar                            
copyfile:(1,3,lrrefl:,lrrefl:)#        Copy lrrefl:                             
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
copyfile:(1,3,incr:(lrrefl:),endrecord:)# Rewind scratch file                   
copyfile:(3,2,1,decr:(lrexpl:))#       Copy up to lrexpl:                       
IWANT(1)=1;IWANT(2)=526;IWANT(3)=527#  Dset, min a*, max a*                     
DO I=1,mxwd40c:;IWANT(I+3)=513#        Loop over abs decritpion                 
KEY(1)=0;KEY(2)=0;KEY(3)=mxwd40c:+3;KEY(4)=0# Update mode                       
indexpkt:(3,lrexpl:,PSIZE,IP,2,KEY,IWANT,IREL)#                                 
IF(IP<=0)iquit:(AZ3103)#               Missing lrexpl:                          
PSIZO=KEY(4)#                          Output packet length                     
REPEAT#                                                                         
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
readwpkt:(3,lrexpl:,PSIZE,IP,0)#       Get one packet                           
IF(IP<=0)BREAK#                        Exit on last packet                      
writepkt:(2,lrexpl:,PSIZO,IPP)#        Write packet                             
movereal:(QX,IP+1,QX,IPP+1,PSIZE,0)#   Copy packet                              
IF((IREL(1)>0)&(NINT(QX(IP+IREL(1)))!=ISET))NEXT# Not this dataset              
QX(IPP+IREL(1))=REAL(ISET)#            First data set                           
QX(IPP+IREL(2))=1./ABMIN;QX(IPP+IREL(3))=1./ABMAX#                              
movector:(DESCR,1,QX(IPP+IREL(4)),1,40,0)# Copy decription                      
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
IWANT(1)=1;IWANT(2)=117#               Dset & mean tbar                         
KEY(1)=0;KEY(2)=0;KEY(3)=2;KEY(4)=0#   Update mode                              
indexpkt:(3,lrdset:,PSIZE,IP,2,KEY,IWANT,IREL)#                                 
IF(IP<=0)#                             Missing lrdset:                          
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
PSIZO=2#                               Dset & tbar                              
writepkt:(2,lrdset:,PSIZO,IPP)#        Write dir packet                         
QX(IPP+1)=1.0;QX(IPP+2)=117.0#         Idn of dset & tbar                       
writepkt:(2,lrdset:,PSIZO,IPP)#        Write data pakcet                        
QX(IPP+1)=REAL(ISET);QX(IPP+2)=MEANT/REAL(NPROC)#                               
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
ELSE#                                                                           
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
PSIZO=KEY(4)#                          Output packet length                     
REPEAT#                                                                         
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11         
readwpkt:(3,lrdset:,PSIZE,IP,0)#       Get one packet                           
IF(IP<=0)BREAK#                        Exit on last packet                      
writepkt:(2,lrdset:,PSIZO,IPP)#        Write pakcet                             
movereal:(QX,IP+1,QX,IPP+1,PSIZE,0)#   Copy packet                              
IF((IREL(1)>0)&(NINT(QX(IP+IREL(1)))!=ISET))NEXT# Not this dataset              
QX(IPP+IREL(1))=REAL(ISET)#            First data set                           
QX(IPP+IREL(2))=MEANT/REAL(NPROC)#     Mean tbar                                
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11         
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
copyfile:(3,2,incr:(lrdset:),endrecord:)#                                       
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ32)#                                                            
#######################################################################         
#                                                                     #         
# READS ONE REFLECTION FROM BDF.                                      #         
# GETS: - MILLER INDICES                                              #         
#       - PSI AZIMUTHAL ANGLE                                         #         
#       - SIN(THETA)/LAMBDA IF PROVIDED ON BDF, ELSE COMPUTE IT       #         
#       - MILLER INDICES OF R-VECTOR IF PROVIDED ON BDF, ELSE COMPUTE #         
#         IT WITH FLACK AND SCHWARZENBACH ALGORITHM                   #         
#       - INTENSITY AND ESD                                           #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ32(MORE)#                                                          
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
vector:(HC)#                                                                    
REAL H(3)#                                                                      
REAL FTEMP#                                                                     
dp: DTEMP#                                                                      
INTEGER I#                             Loop index                               
INTEGER MORE#                          Flag for last reflection                 
#                                                                               
MORE=yes:#                             Other reflections                        
readwpkt:(1,lrrefl:,PSIZE,IP,0)#       Read one reflection                      
IF(IP<=0)MORE=no:#                     Last one read                            
ELSE#                                  Not the last                             
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
writepkt:(3,lrrefl:,PSIZO,IPP)#        Set output pointers                      
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#   Copy reflection                          
gethkl:(IPP+IREL(1),H)#                Miller indices of reflection             
DO I=1,3;HM(I)=dble:(H(I))#                                                     
IF(STORE==yes:)#                       Store derivatives                        
DO I=1,NFACES;QX(IPP+IREL(8+I))=voidflg:# Clear derivatives                     
SINP=SIN(QX(IPP+IREL(2))*rad:);COSP=COS(QX(IPP+IREL(2))*rad:)# Psi              
INT=QX(IPP+IREL(3));SIGMA=QX(IPP+IREL(4))# Get i from bdf                       
IF(CALCT==yes:)#                       Compute sin(theta)/lambda                
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
postm:(B,HM,HC);norm:(HC,HC,DTEMP);FTEMP=DTEMP/2.0#                             
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
ELSE FTEMP=QX(IPP+IREL(5))#            Get sin(theta)/lambda from fil           
SINT=LAMBDA*FTEMP;COST=SQRT(1.0-SINT**2)# Sin & cos(theta)                      
RM(1)=HM(2)-HM(3)#                                                              
RM(2)=HM(3)-HM(1)#                                                              
RM(3)=HM(1)-HM(2)#                                                              
IF((RM(1)**2+RM(2)**2+RM(3)**2)<dsmall:)#                                       
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
RM(1)=HM(1)#                                                                    
RM(2)=-HM(1)#                                                                   
RM(3)=DBZ#                                                                      
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ33)#                                                            
#######################################################################         
#                                                                     #         
# BDF INPUT : - LATTICE CONSTANTS & CELL VOLUME                       #         
#             - IF MU NOT PROVIDED, ATOMS NAME, ATOMIC NUMBER, NUMBER #         
#               OF ATOMS OF THIS TYPE IN THE UNIT CELL                #         
#             - DATASET NUMBER & LAMBDA (IF NOT PRESENT ON BDF AND    #         
#               NOT GIVEN BY USER THE EXIT)                           #         
# BDF OUTPUT: - MU (PROVIDED BY USER OR CALCULATED FROM ATOMIC SCAT-  #         
#               TERING FACTORS)                                       #         
#             - NUMBER OF FACES                                       #         
#             - MILLER INDICES OF FACES                               #         
#             - DISTANCE FROM FACE TO ORIGIN                          #         
# COMPUTED  : - MU (IF NOT PROVIDED BY USER)                          #         
# PRINTED   : - LAMBDA                                                #         
#             - MU                                                    #         
#             - IF MU CALCULATED, CONTRIBUTION OF EACH TYPE OF ATOMS  #         
# ------------------------------------------------------------------- #         
# MU IS WRITTEN ON BDF, EVEN IF THERE IS ALREADY ONE VALUE.           #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ33#                                                                
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
INTEGER FF#                            Mqin faces pointer                       
REAL CVOL#                             Cell volume                              
INTEGER I,J#                           Loop indexes                             
INTEGER ATOM#                          Element pointer                          
INTEGER CALCMU#                        To know is it is possible to c           
INTEGER ISFACE,SPHERE#                 Flag to know if there are face           
INTEGER WAVE#                          Flag to know if wavelength on            
string:(ACH,arith:(16,*,8))#                                                    
#                                                                               
#  MU(A) FOR CU (CM**2)  INT TAB VOL C P193-198.                                
#                                                                               
realdata:(TCU,#                                                                 
[.0655,.194,.576,1.66,4.15,8.99,17.3,30.4,49.8,76.8,#                           
114.,161.,222.,297.,388.,497.,624.,772.,940.,1130.,#                            
1350.,1590.,1850.,2130.,2460.,2800.,3140.,476.,547.,629.,#                      
719.,819.,929.,1050.,1180.,1320.,1480.,1650.,1830.,2030.,#                      
2230.,2460.,2700.,2950.,3230.,3520.,3820.,4150.,4500.,4860.,#                   
5250.,5650.,6070.,6520.,7000.,7500.,8030.,8570.,9120.,9680.,#                   
10200.,10800.,11000.,10500.,8470.,9770.,3470.,3670.,3930.,4100.,#               
4500.,4600.,4850.,5130.,5720.,5800.,6240.,6340.,6690.,6680.,#                   
7540.,7980.,8430.,8810.,8650.,9720.,10200.,10200.,14300.,11800.,#               
10600.,11200.])#                                                                
#                                                                               
#  MU(A) FOR MO (CM**2)  INT TAB VOL C P193-198.                                
#                                                                               
realdata:(TMO,#                                                                 
[.0624,.134,.228,.383,.661,1.15,1.96,3.25,5.15,7.86,#                           
11.6,16.5,22.9,31.,41.,53.2,67.8,85.1,105.,129.,#                               
156.,186.,220.,258.,302.,349.,401.,457.,518.,586.,#                             
660.,738.,822.,911.,1000.,1100.,1210.,1320.,1430.,247.,#                        
273.,300.,332.,364.,399.,436.,476.,518.,563.,611.,#                             
662.,716.,773.,834.,898.,965.,1040.,1110.,1190.,1270.,#                         
1350.,1440.,1540.,1630.,1740.,1840.,1950.,2070.,2190.,2310.,#                   
2440.,2580.,2720.,2860.,3010.,3160.,3310.,3480.,3650.,3820.,#                   
4010.,4190.,4380.,4580.,4070.,3980.,3220.,3300.,5400.,3700.,#                   
3870.,4030.])#                                                                  
#                                                                               
#  MU(A) FOR AG (CM**2)  INT TAB VOL C P193-198.                                
#                                                                               
realdata:(TAG,#                                                                 
[.0614,.128,.206,.313,.479,.745,1.17,1.82,2.77,4.12,#                           
5.96,8.42,11.6,15.6,20.6,26.7,34.1,42.9,53.2,65.2,#                             
78.9,94.7,112.,133.,155.,180.,207.,238.,271.,307.,#                             
346.,387.,433.,482.,535.,592.,652.,715.,780.,847.,#                             
922.,1150.,1070.,192.,210.,230.,251.,273.,297.,323.,#                           
350.,378.,409.,441.,475.,511.,549.,588.,630.,674.,#                             
720.,768.,819.,872.,927.,985.,1040.,1110.,1170.,1240.,#                         
1310.,1390.,1460.,1540.,1620.,1710.,1800.,1890.,1990.,2090.,#                   
2190.,2290.,2400.,2510.,2620.,2730.,2850.,2980.,3110.,3230.,#                   
3420.,3500.])#                                                                  
realdata:(WV,[1.5418,0.7107,0.5608])#                                           
realdata:(FORM,[210303.,350403.,521042.])#                                      
chardata:(AZ331," Lambda                                A")#                    
chardata:(AZ332," Mu                                    /mm")#                  
chardata:(AZ333,"(File) (User) (Calculated)")#                                  
chardata:(AZ334," Atom type  Atomic number   Atoms cell     MUA ")#             
datastuff:#                                                                     
#                                                                               
# GET METRIC TENSOR M FROM LRCELL:                                              
# --------------------------------                                              
#                                                                               
DO I=1,10#                             Loop over lrcell: packets                
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
readwpkt:(1,lrcell:,PSIZE,IP,0)#       Read one packet                          
IF(IP<=0)BREAK#                        Exit if last one                         
writepkt:(3,lrcell:,PSIZE,IPP)#        Copy it                                  
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                            
IF(I==1)#                              If first packet                          
DO J=1,3#                              Lattice constants                        
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
LATCON(J)=QX(IPP+J)#                   Axis length                              
LATCON(J+3)=QX(IPP+6+J)*360.0#         Angles in degrees                        
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
IF(I==6)CVOL=QX(IPP+1)#                Cell volume                              
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
DO I=1,3#                              Build metric tensor from latti           
DO J=1,3#                                                                       
IF(I==J)M(I,J)=dble:(LATCON(I)**2)#                                             
ELSE M(I,J)=dble:(LATCON(I)*LATCON(J)*COS(LATCON(9-I-J)*rad:))#                 
#                                                                               
# GET ATOM TYPES FROM LRSCAT: IF NO MU GIVEN                                    
# ------------------------------------------                                    
#                                                                               
IF(MU<0.0)#                            Mu from bdf                              
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
CALCMU=yes:#                           Maybe compute mu from chemical           
copyfile:(1,3,incr:(lrcell:),decr:(lrscat:))# Copy bdf                          
IWANT(1)=1#                            Needed: number of atom of this           
IWANT(2)=3#                            Atomic number                            
IWANT(3)=11#                           Scattering factor name                   
KEY(1)=3;KEY(2)=0;KEY(3)=0;KEY(4)=0#   Update mode                              
indexpkt:(1,lrscat:,PSIZE,IP,3,KEY,IWANT,IREL)# Get directory                   
IF(IP<=0)CALCMU=no:#                   Impossible to compute mu (no c           
IF(KEY(4)<0)CALCMU=no:#                Impossible to compute mu (miss           
ATOM=1;J=1#                            Read atom types                          
IF(CALCMU==yes:)#                                                               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
movechar:(AZ333,15,AZ332,5,12,0)#      Move (calculated)                        
REPEAT#                                Loop over atom types                     
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
readwpkt:(1,lrscat:,PSIZE,IP,0)#       Read one atom type                       
IF(IP<=0)BREAK#                        Exit if last                             
writepkt:(3,lrscat:,PSIZE,IPP)#        Copy it                                  
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                            
IF(CALCMU==yes:)#                      Still possible to compute mu             
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6          
BUFOT(ATOM+1)=QX(IPP+IREL(1))#         Store number of atom in the ce           
BUFOT(ATOM)=QX(IPP+IREL(2))#           Store atomic number                      
ATOM=ATOM+3#                           Increment atom type pointer              
movertoc:(QX(IPP+IREL(3)),1,ACH,J,lens:,0)# Get name                            
J=J+lens:#                             8 characters name                        
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6          
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
ELSE movechar:(AZ333,1,AZ332,5,6,0)#   Move (file)                              
IF(ATOM==1)CALCMU=no:#                 Impossible to compute mu, no a           
copyfile:(1,3,incr:(lrscat:),decr:(lrexpl:))# Copy up to lrexpl:                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
ELSE#                                  Mu introduced by user                    
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7          
movechar:(AZ333,8,AZ332,5,6,0)#        Mu from user                             
copyfile:(1,3,incr:(lrcell:),decr:(lrexpl:))# Copy up to lrexpl:                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7          
#                                                                               
# GET DATA FROM LREXPL:                                                         
# -----------------------                                                       
#                                                                               
IWANT(1)=1#                            Dataset                                  
IWANT(2)=2#                            Lambda                                   
IWANT(3)=10#                           Mu                                       
IWANT(4)=300#                          Number of faces                          
WAVE=yes:#                             Default: wavelength on bdf               
KEY(1)=0;KEY(2)=0;KEY(3)=4;KEY(4)=0#   Update mode                              
IF(NFACES==0)#                         No face cards read                       
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8          
ISFACE=no:#                            Face input from bdf                      
indexpkt:(1,lrexpl:,PSIZE,IP,3,KEY,IWANT,IREL)#                                 
IF(IP<=0)iquit:(AZ3301)#               Missing lrexpl: record(s)                
findidn:(IP,PSIZE,541,J)#              Look for sphere                          
IF(J !=0)#                             Sphere                                   
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9          
IREL(5)=J#                             Radius                                   
findidn:(IP,PSIZE,542,J)#                                                       
IF(J==0)iquit:(AZ3310)#                Missing esd                              
IREL(6)=J#                             Esd                                      
findidn:(IP,PSIZE,543,J)#              Weight                                   
IF(J==0)iquit:(AZ3310)#                Missing weight                           
IREL(7)=J#                             Weight                                   
NFACES=1;SPHERE=yes:#                  Set face to one                          
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9          
ELSE#                                  Polyhedron                               
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10         
findidn:(IP,PSIZE,300,J)#              Look for number of face                  
IF(J==0)iquit:(AZ3302)#                No face on bdf nor on cards              
REPEAT#                                Loop over faces idn                      
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11         
findidn:(IP,PSIZE,301+10*(NFACES),J)#  Test if h present                        
IF(J==0)BREAK#                         Missing h                                
IREL(6*NFACES+5)=J#                                                             
findidn:(IP,PSIZE,302+10*(NFACES),J)#  Test if k present                        
IF(J==0)BREAK#                         Missing k                                
IREL(6*NFACES+6)=J#                                                             
findidn:(IP,PSIZE,303+10*(NFACES),J)#  Test if l present                        
IF(J==0)BREAK#                         Missing l                                
IREL(6*NFACES+7)=J#                                                             
findidn:(IP,PSIZE,304+10*(NFACES),J)#  Test if d present                        
IF(J==0)BREAK#                         Missing d                                
IREL(6*NFACES+8)=J#                                                             
findidn:(IP,PSIZE,304+10*(NFACES),J)#  Test if esd present                      
IF(J==0)BREAK#                         Missing esd                              
IREL(6*NFACES+9)=J#                                                             
findidn:(IP,PSIZE,304+10*(NFACES),J)#  Test if weight present                   
IF(J==0)BREAK#                         Missing weight                           
IREL(6*NFACES+10)=J#                                                            
NFACES=NFACES+1#                                                                
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11         
IF(NFACES<4)iquit:(AZ3303)#            Less than four faces on bdf              
SPHERE=no:#                            Spherical shape flag                     
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10         
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8          
ELSE#                                  Face input from cards                    
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12         
ISFACE=yes:#                           Face input from cards                    
IF(NFACES>1)#                          Not spherical crystal                    
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13         
DO I=1,NFACES#                         Loop over all faces                      
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14         
IWANT(6*I-1)=300+10*(I-1)+1#           Store h                                  
IWANT(6*I)=300+10*(I-1)+2#             Store k                                  
IWANT(6*I+1)=300+10*(I-1)+3#           Store l                                  
IWANT(6*I+2)=300+10*(I-1)+4#           Store d                                  
IWANT(6*I+3)=300+10*(I-1)+8#           Store esd of d                           
IWANT(6*I+4)=300+10*(I-1)+9#           Store weight of d                        
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14         
KEY(3)=KEY(3)+6*NFACES#                Insert faces                             
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13         
ELSE#                                                                           
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15         
IWANT(5)=541;IWANT(6)=542;IWANT(7)=543#                                         
KEY(3)=KEY(3)+3#                       Sphere                                   
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15         
indexpkt:(1,lrexpl:,PSIZE,IP,3,KEY,IWANT,IREL)#                                 
IF(IP<=0)iquit:(AZ3301)#               Missing lrexpl: record(s)                
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12         
IF(IREL(2)<=0)iquit:(AZ3304)#          Missing lambda                           
findidn:(IP,PSIZE,10,J)#               Look for mu                              
IF((MU<0.0)&(J==0)&(CALCMU==no:))iquit:(AZ3305)#                                
PSIZO=KEY(4)#                          Output packet size                       
REPEAT#                                Loop over lrexpl: packets                
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16         
readwpkt:(1,lrexpl:,PSIZE,IP,0)#       Read one packet                          
IF(IP<=0)BREAK#                        Exit if last                             
writepkt:(3,lrexpl:,PSIZO,IPP)#        Copy it                                  
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                            
IF((IREL(1)>0)&(ISET !=NINT(QX(IPP+IREL(1)))))NEXT#                             
LAMBDA=QX(IP+IREL(2))#                 Get wavelength                           
IF(ISFACE==no:)#                       Faces on bdf                             
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17         
FF=MARKF#                              Set face pointer                         
incrqx:(ISTOPF,MARKF+NFACES*nstepf:,AZ3307)# Get memory                         
IF(SPHERE==yes:)#                                                               
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18         
facer:=QX(IP+IREL(5))#                 Radius                                   
facesd:(MARKF)=QX(IP+IREL(6))#         Esd of radius                            
facew:(MARKF)=QX(IP+IREL(7))#          Weight of radius                         
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18         
ELSE#                                                                           
DO I=1,NFACES#                         Loop over faces                          
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19         
facehm:(FF)=QX(IP+IREL(6*I-1))#        Input miller indices from bdf            
facekm:(FF)=QX(IP+IREL(6*I))#                                                   
facelm:(FF)=QX(IP+IREL(6*I+1))#                                                 
faced:(FF)=QX(IP+IREL(6*I+2))#         Input distance to face from bd           
facesd:(FF)=QX(IP+IREL(6*I+3))#        Input esd of dist                        
facew:(FF)=QX(IP+IREL(6*I+4))#         Input weight of dist                     
incrf:(FF)#                            Increment face pointer                   
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19         
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17         
#                                                                               
# GET MU FROM LREXPL:                                                           
# -------------------                                                           
#                                                                               
IF(MU<0.0)#                            If mu to be calculated                   
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20         
IF(CALCMU==no:)MU=QX(IP+IREL(3))#      Get mu from bdf                          
ELSE#                                  Compute mu                               
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21         
MU=0.0#                                Clear mu                                 
DO I=1,ATOM-3,3#                       Loop over atom type read                 
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22         
J=NINT(BUFOT(I))#                      Extract the atomiv number                
IF(ABS(LAMBDA-WV(1))<=0.01)#           Cu wavelength                            
BUFOT(I+2)=TCU(J)#                     Extract mu                               
ELSE IF(ABS(LAMBDA-WV(2))<=0.01)#      Mo wavelength                            
BUFOT(I+2)=TMO(J)#                     Extract mu                               
ELSE IF(ABS(LAMBDA-WV(3))<=0.01)#      Ag wavelength                            
BUFOT(I+2)=TAG(J)#                     Extract mu                               
ELSE iquit:(AZ3308)#                   Inmpossible to compute mu                
IF(BUFOT(I+2)<=small:)iquit:(AZ3309)#  Atomic scattering factor is mi           
MU=MU+BUFOT(I+1)*BUFOT(I+2)#           Increase mu                              
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22         
MU=MU/CVOL#                            Mu in mm                                 
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21         
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20         
ncodefld:(LAMBDA,1,AZ331,380842.,1)#   Lambda                                   
writeline:(0,AZ331,NAZ331,3,3)#                                                 
IF(MU<0.0)iquit:(AZ3306)#              No absorption coeff                      
ncodefld:(MU,1,AZ332,380842.,1)#       Mu                                       
writeline:(0,AZ332,NAZ332,3,3)#                                                 
IF(CALCMU==yes:)#                      Print atomic contribution                
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23         
LINRM=(ATOM/3)+1;J=1#                  Reserve lines                            
DO I=1,ATOM-3,3#                       Loop over atoms                          
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24         
movechar:(ACH,J,CHROT,5,lens:,0)#      Move atom name                           
J=J+lens:#                             Increment atom name pointer              
ncodefld:(BUFOT,I,CHROT,FORM,3,0)#     Code absorption                          
writeline:(0,AZ334,NAZ334,1,3)#        Print                                    
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24         
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23         
#                                                                               
# PUT MU AND FACE INFORMATIONS ON LREXPL:                                       
# ---------------------------------------                                       
#                                                                               
QX(IPP+IREL(2))=LAMBDA#                Store wavelength                         
QX(IPP+IREL(3))=MU#                    Store abs coeff                          
QX(IPP+IREL(4))=REAL(NFACES)#          Store face numbers                       
FF=MARKF#                              Set face pointer                         
IF(NFACES==1)#                         Sphere                                   
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25         
QX(IPP+IREL(5))=facer:#                                                         
QX(IPP+IREL(6))=facesd:(MARKF)#                                                 
QX(IPP+IREL(7))=facew:(MARKF)#         Store weight                             
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25         
ELSE#                                                                           
DO I=1,NFACES#                         Loop over faces                          
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26         
QX(IPP+IREL(6*I-1))=facehm:(FF)#       Store miller indices                     
QX(IPP+IREL(6*I))=facekm:(FF)#                                                  
QX(IPP+IREL(6*I+1))=facelm:(FF)#                                                
QX(IPP+IREL(6*I+2))=faced:(FF)#        Store distance in mm                     
QX(IPP+IREL(6*I+3))=facesd:(FF)#       Store e.s.d. of dist                     
QX(IPP+IREL(6*I+4))=facew:(FF)#        Store weight                             
incrf:(FF)#                            Increment face pointer                   
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26         
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16         
copyfile:(1,3,incr:(lrexpl:),decr:(lrrefl:))# Copy up to lrrefl:                
RETURN#                                                                         
END#                                                                            
#                                                                               
#                                                                               
systemheader:(AZ41)#                                                            
#######################################################################         
#                                                                     #         
# COMPUTES B MATRIX BY DIFFERENT WAYS:                                #         
# - SHAPE =      0: GENERAL SHAPE => B TRIANGULAIRE INFERIEURE        #         
# - SHAPE = -N < 0: PLATE   SHAPE => RECIPROCAL AXE 3 // 3RD VECTOR   #         
#                                    OF BASE B                        #         
# - SHAPE =  N > 0: PIN     SHAPE => DIRECT     AXE 3 // 3RD VECTOR   #         
#                                    OF BASE B                        #         
#                                                                     #         
#######################################################################         
SUBROUTINE AZ41(SHAPE)#                                                         
#                                                                               
include:(AACOMN)#                                                               
include:(AZCOMN)#                                                               
#                                                                               
matrix:(PASS)#                                                                  
dp: DET,DTEMP#                                                                  
INTEGER I,J#                                                                    
INTEGER SHAPE#                                                                  
#                                                                               
chardata:(AZ411," Lattice constants")#                                          
chardata:(AZ412," B matrix                            Metric tensor")#          
realdata:(FM1,[100962.,200962.,300962.,470942.,570942.,670942.])#               
realdata:(FM2,[100962.,200962.,300962.,470952.,570952.,670952.])#               
datastuff:#                                                                     
#                                                                               
LINRM=3#                                                                        
ncodefld:(LATCON,1,CHROT,FM1,6)#       Code lattice constants                   
writeline:(1,AZ411,NAZ411,1,3)#        Print them                               
B(1,2)=DBZ;B(1,3)=DBZ;B(2,3)=DBZ#      Build b                                  
B(3,3)=dsqrt:(MINV(3,3))#                                                       
B(3,1)=MINV(3,1)/B(3,3);B(3,2)=MINV(3,2)/B(3,3)#                                
B(2,2)=dsqrt:(MINV(2,2)-B(3,2)**2)#                                             
B(2,1)=-dsqrt:(MINV(1,1)-B(3,1)**2)*M(1,2)/dsqrt:(M(1,1)*M(2,2))#               
B(1,1)=DB1/dsqrt:(M(1,1))#                                                      
invert:(B,BINV,DET)#                   Invert it                                
IF(SHAPE !=0)#                         No general shape, so b must be           
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1          
IF(SHAPE>0)#                           Direct space change (pin shape           
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2          
transp:(BINV,PASS);postm:(PASS,AXE,AXE)# Axis in rec. space                     
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2          
ELSE postm:(B,AXE,AXE)#                Axis in rec. space (b base)              
norm:(AXE,AXE,DTEMP)#                  Normalize                                
IF(DTEMP<dsmall:)iquit:(AZ4101)#       Axe ill-defined                          
IF((dabs:(AXE(3))-DB1)<dsmall:)#       Axis not in z direction                  
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3          
DO I=1,3;PASS(3,I)=AXE(I)#             Construct passage matrix                 
PASS(1,3)=DBZ;PASS(2,3)=-dsqrt:(1-AXE(3)*AXE(3))#                               
PASS(1,1)=-AXE(2)/PASS(2,3);PASS(1,2)=AXE(1)/PASS(2,3)#                         
PASS(2,1)=-AXE(3)*PASS(1,2);PASS(2,2)=AXE(3)*PASS(1,1)#                         
mmult:(PASS,B,B)#                      Change base b                            
transp:(PASS,PASS);mmult:(BINV,PASS,BINV)# Change base binv                     
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3          
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1          
#                                                                               
# PRINT M AND B FOR USER                                                        
# ----------------------                                                        
#                                                                               
writeline:(1,0,0,3,3)#                                                          
LINRM=4#                               4 lines reserved                         
DO I=1,3#                              Print header & matrices                  
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4          
DO J=1,3#                                                                       
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5          
BUFOT(J)=sngl:(B(I,J));BUFOT(J+3)=sngl:(M(I,J))#                                
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5          
ncodefld:(BUFOT,1,CHROT,FM2,6)#        Encode everything                        
writeline:(0,AZ412,NAZ412,1,3)#        ... and print                            
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4          
RETURN#                                                                         
END#                                                                            
#                                                                               
pmacro:([no:])#                        Purge xtal macros                        
