systemheader:(ML    )#                                                  ML     1
####################################################################### ML     2
##      Copyright (C) 2000, The Xtal System, The University of        # ML     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # ML     4
## Distributed under the GNU General Public Licence as published by   # ML     5
## the Free Software Foundation, in the hope that it will be useful   # ML     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # ML     7
## for more details.                                                  # ML     8
##                                                                    # ML     9
##                        *** MODEL ***                               # ML    10
##                                                                    # ML    11
##  MODEL reads peak and atom sites from the PEKPIK file and forms    # ML    12
##  a connected atomic model based on the inter-site distances (to    # ML    13
##  form clusters) and angles (to form sub-clusters). A proposed      # ML    14
##  chemical connectivity model may also be input and MODEL matches   # ML    15
##  this with the sites. The process is critically dependent on the   # ML    16
##  most appropriate distance and angle limits being specified.       # ML    17
##                                                                    # ML    18
####################################################################### ML    19
#                                                                       ML    20
include:(XMACRO)#                                                       ML    21
#                                                                       ML    22
macro:(dumpsite:,CALL ML09($1,$2,$3,$4,$5))#                            ML    23
macro:(cullsite:,CALL ML07($1))#                                        ML    24
#                                                                       ML    25
macro:(peaklist:,[ifelse:($1,,IP,$1)])# Define site list pointer        ML    26
macro:(aname:,[QX(peaklist:($1)+ 1)])# Define 4 char site name          ML    27
macro:(atrad:,[QX(peaklist:($1)+ 2)])# Define atom radius               ML    28
macro:(peakx:,[QX(peaklist:($1)+ 3)])# Define site x coord              ML    29
macro:(peaky:,[QX(peaklist:($1)+ 4)])# Define site y coord              ML    30
macro:(peakz:,[QX(peaklist:($1)+ 5)])# Define site z coord              ML    31
macro:(peakh:,[QX(peaklist:($1)+ 6)])# Define site height               ML    32
macro:(clust:,[QX(peaklist:($1)+ 7)])# Define site cluster group        ML    33
macro:(numbr:,[QX(peaklist:($1)+ 8)])# Define site number               ML    34
macro:(bond1:,[QX(peaklist:($1)+ 9)])# Define bond start addr.          ML    35
macro:(bond2:,[QX(peaklist:($1)+10)])# Define bond stop  addr.          ML    36
macro:(start:,[QX(peaklist:($1)+11)])# Define site start flag           ML    37
macro:(ringf:,[QX(peaklist:($1)+12)])# Define ring indicator flag       ML    38
macro:(group:,[QX(peaklist:($1)+13)])# Define site subclust group       ML    39
macro:(canon:,[QX(peaklist:($1)+14)])# Define canonical descriptor      ML    40
macro:(nbond:,[QX(peaklist:($1)+15)])# Define site bond count           ML    41
macro:(order:,[QX(peaklist:($1)+16)])# Define site merit                ML    42
macro:(match:,[QX(peaklist:($1)+17)])# Define matching atom site ad     ML    43
macro:(ipbias:,17)#                    Set site list offset             ML    44
#                                                                       ML    45
macro:(bondlist:,[ifelse:($1,,IB,$1)])# Define bond list pointer        ML    46
macro:(peak1:,[QX(bondlist:($1)+ 1)])# Define bond site addr. 1         ML    47
macro:(peak2:,[QX(bondlist:($1)+ 2)])# Define bond site addr. 2         ML    48
macro:(bondl:,[QX(bondlist:($1)+ 3)])# Define bond length 1 to 2        ML    49
#                                                                       ML    50
macro:(sitelist:,[ifelse:($1,,IS,$1)])# Define site list pointer        ML    51
macro:(site:,[QX(sitelist:($1)+ 1)])#  Define site site addr.           ML    52
macro:(sumc:,[QX(sitelist:($1)+ 2)])#  Define sum of canonical desc     ML    53
macro:(bonf:,[QX(sitelist:($1)+ 3)])#  Define first bond site addr.     ML    54
macro:(bonl:,[QX(sitelist:($1)+ 4)])#  Define last bond site addr.      ML    55
macro:(ring:,[QX(sitelist:($1)+ 5)])#  Define ring indicator flag       ML    56
#                                                                       ML    57
macro:(sortlist:,[ifelse:($1,,IA,$1)])# Define sort list pointer        ML    58
macro:(star:,[QX(sortlist:($1)+ 1)])#  Define start site addr.          ML    59
macro:(frst:,[QX(sortlist:($1)+ 2)])#  Define first-1 bond site         ML    60
macro:(last:,[QX(sortlist:($1)+ 3)])#  Define last  bond site addr.     ML    61
macro:(link:,[QX(sortlist:($1)+ 4)])#  Define matched parent addr.      ML    62
macro:(cand:,[QX(sortlist:($1)+ 5)])#  Define canonical discriptor      ML    63
#                                                                       ML    64
#                                                                       ML    65
extrude:(MLCOMN)#                                                       ML    66
INTEGER BEST1(20)#          ## Fragment best matching subcluster i      ML    67
INTEGER IEXT#               ## Signal to extend input atom sites        ML    68
INTEGER IMF#                ## Signal better cluster or fragment matc   ML    69
INTEGER IND(3)#             ## Index eigenvectors for ls, ms and othe   ML    70
INTEGER ISTART#             ## Pointer to starting site for interp      ML    71
INTEGER LTYP#               ## Lattice type indicator                   ML    72
INTEGER MARK1,STEP1,STOP1#  ## Section of qx with symops                ML    73
INTEGER MARK2,STEP2,STOP2#  ## Section of qx with site information      ML    74
INTEGER             STOPI#  ## End of the input atom list               ML    75
INTEGER MARK3,STEP3,STOP3#  ## Section of qx with input atom(fragment   ML    76
INTEGER MARK4,STEP4,STOP4#  ## Utility section of qx                    ML    77
INTEGER MARKP,STEPP,STOPP#  ## Site section of qx for assignment to a   ML    78
INTEGER MARKF,STEPF,STOPF#  ## Atom section of qx for assignment to p   ML    79
INTEGER MARKB,STEPB,STOPB#  ## Section of qx with bond array            ML    80
INTEGER MARKT,STEPT,STOPT#  ## Section of qx with special intracluste   ML    81
INTEGER MAXBON#             ## Maximum number of bonds allowed to ato   ML    82
INTEGER MODW#               ## Subcluster search flag                   ML    83
INTEGER MTERP#              ## Max no. of interpretations per cluster   ML    84
INTEGER MAXGRP#             ## Max.no. of atoms or sites in frag/subc   ML    85
INTEGER MSUB#               ## No. of subclusters in curr. cluster      ML    86
INTEGER NATOM#              ## Mo. of expected atom sites               ML    87
INTEGER NBOND#              ## Total number of intersite bonds (redun   ML    88
INTEGER NCLUS#              ## Number of cluster under consideration    ML    89
INTEGER NFRAG#              ## Number of discrete fragments entered     ML    90
INTEGER NINFR(20)#          ## Number of atoms in fragment              ML    91
INTEGER NSITE#              ## Number of atom sites from lratom:        ML    92
INTEGER NSITES#             ## Total number of input sites              ML    93
INTEGER NPKCL(32)#          ## Number of sites in each cluster          ML    94
INTEGER NPROJ#              ## Total proj to be plot/cluster            ML    95
INTEGER NROT#               ## Number of rotation matrices              ML    96
INTEGER NSUB(20)#           ## No. of sites in each subcluster          ML    97
INTEGER NSYM#               ## Total number of space group symops       ML    98
INTEGER NTERP#              ## Count of interps. done for current clu   ML    99
INTEGER NTWIN#              ## Number of special intracluster connect   ML   100
INTEGER RINGF#              ## Number of atoms in rings                 ML   101
INTEGER RINGP#              ## Number of sites in rings                 ML   102
INTEGER SHUFFL#             ## Signal that cluster was shuffled         ML   103
INTEGER TCLUS#              ## Total number of clusters                 ML   104
REAL ANGMAX,ANGMIN#         ## Max and min angles for subclusters       ML   105
REAL RADMAX,RADMIN#         ## Max and min site radii for clusters      ML   106
REAL BMAT(3,3)#             ## Matrix of normal equ for ls plane        ML   107
REAL PRTMAX#                ## Square of maximum distance for listed    ML   108
REAL TOPFOM#                ## Fig. of merit for best interp or match   ML   109
REAL FOMF#                  ## Figure of merit for frag interp.         ML   110
REAL FOMP#                  ## Figure of merit for site interp.         ML   111
REAL G1,G2,G3,G4,G5,G6#     ## Coefficients of metric tensor            ML   112
REAL T1,T2,T3,T5,T6,T9#     ## Coefficients of othogonal matrix         ML   113
REAL MERIT#                 ## Figure of merit for current atom-site    ML   114
REAL SEPMIN#                ## Square of resolution                     ML   115
REAL TRANS(3)#              ## Translation used in ml16                 ML   116
REAL VECT(3,3)#             ## Eigenvectors for least squares           ML   117
#                                                                       ML   118
COMMON/COMIML/BEST1,IMF,IND,ISTART,LTYP,#                               ML   119
MARK1,STEP1,STOP1,MARK2,STEP2,STOP2,STOPI,#                             ML   120
MARK3,STEP3,STOP3,MARK4,STEP4,STOP4,#                                   ML   121
MARKP,STEPP,STOPP,MARKF,STEPF,STOPF,#                                   ML   122
MARKB,STEPB,STOPB,MARKT,STEPT,STOPT,MAXGRP,#                            ML   123
MTERP,MSUB,NATOM,NCLUS,NBOND,NFRAG,NINFR,#                              ML   124
NSITE,MAXBON,MODW,NROT,NSITES,NPKCL,NPROJ,NSUB,#                        ML   125
NTERP,NSYM,RINGF,RINGP,NTWIN,SHUFFL,TCLUS,IEXT#                         ML   126
#                                                                       ML   127
COMMON/COMFML/ANGMAX,ANGMIN,BMAT,RADMAX,RADMIN,#                        ML   128
PRTMAX,FOMF,FOMP,G1,G2,G3,G4,G5,G6,T1,T2,T3,T5,T6,T9,#                  ML   129
MERIT,SEPMIN,TOPFOM,TRANS,VECT#                                         ML   130
endext:#                                                                ML   131
#                                                                       ML   132
systemheader:(ML00)#                   Find atom model from sites       ML00   1
####################################################################### ML00   2
#                                                                       ML00   3
#                                ML00                                   ML00   4
#                                                                       ML00   5
####################################################################### ML00   6
SUBROUTINE ML00#                       R.m. doherty sept 81             ML00   7
include:(AACOMN)#                      S.r. hall    sept 82             ML00   8
include:(MLCOMN)#                                                       ML00   9
#                                                                       ML00  10
#   QX LIST MARK1+1 TO STOP1    SYMMETRY MATRICES                       ML00  11
#           MARK2+1    STOPI/2  INPUT AND ADDED ATOM SITE LIST          ML00  12
#           MARK3+1    STOP3    FRAGMENT SITES FROM CONECT LINES        ML00  13
#           MARKB+1    STOPB    BONDED SITE SITES PLUS DISTANCE         ML00  14
#           MARKT+1    STOPT    SITES RELATED SYMOP                     ML00  15
#           MARK4+1    STOP4    SCRATCH LIST OF SITES                   ML00  16
#           MARKP+1    STOPP    SORTED SITE SITES FOR MATCH TO LISTF    ML00  17
#           MARKF+1    STOPF    SORTED FRAG SITES FOR MATCH TO LISTP    ML00  18
#                                                                       ML00  19
#                                                                       ML00  20
FILEXT(IOUNIT(5))='PEK'#               Ext for ip peak file             ML00  21
#                                                                       ML00  22
CALL ML10#                             Read =limits= line and bdf       ML00  23
FOR(NCLUS=1; NCLUS<=TCLUS; NCLUS=NCLUS+1)# Loop over site clusters      ML00  24
FOR(NTERP=1; NTERP<=MTERP; NTERP=NTERP+1)# Loop over interpretation     ML00  25
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML00  26
CALL ML20#                             Start interpretation run         ML00  27
IF(ISTART==0) BREAK#                   Skip if no starter found         ML00  28
IF     (MODW==1) CALL ML30#            Connected subclust search        ML00  29
ELSE IF(MODW==2) CALL ML35#            Weighted  subclust search        ML00  30
CALL ML37#                             Calculate figure of merit        ML00  31
IF(NCLUS>1&NTERP>1&FOMP<TOPFOM/2.) NEXT# Skip groups with poor fom      ML00  32
CALL ML40#                             Match sites to input model       ML00  33
CALL ML50#                             List cluster & group definit     ML00  34
CALL ML60#                             Plot projections of cluster      ML00  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML00  36
CALL ML70#                             List coords & bondla values      ML00  37
#                                                                       ML00  38
RETURN#                                                                 ML00  39
END#                                                                    ML00  40
systemheader:(ML07)#                   Disable site pointer & bonds     ML07   1
####################################################################### ML07   2
#                                                                     # ML07   3
#                               ML07                                  # ML07   4
#                                                                     # ML07   5
####################################################################### ML07   6
SUBROUTINE ML07(IP)#                                                    ML07   7
include:(AACOMN)#                                                       ML07   8
include:(MLCOMN)#                                                       ML07   9
#                                                                       ML07  10
# THIS SUBROUTINE CAUSES SITE N TO BE ELIMINATED FROM CONSIDERATION IN  ML07  11
# THE INTERPRETATION BY (A) ZEROING THE COUNT OF BONDS TO THE SITE AND  ML07  12
# (B) FLAGGING ALL THE CONNECTIONS TO OTHER SITES IN MARKB.             ML07  13
#                                                                       ML07  14
INTEGER IB,JB,KB,IP,JP,LB#             Loop indices                     ML07  15
#                                                                       ML07  16
nbond:=0.;    group:=0.#               Zero number of bonds             ML07  17
JB=NINT(bond2:)#                       Get end of bond list             ML07  18
FOR(IB=NINT(bond1:);IB<JB;IB=IB+STEPB)# Loop over connect bonds         ML07  19
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML07  20
IF(peak1:<0.5) NEXT#                   Skip if already disabled         ML07  21
peak1:=-peak1:#                        Disable this bond                ML07  22
JP=NINT(peak2:)#                       Get pointer to sitelist          ML07  23
nbond:(JP)=nbond:(JP)-1.#              Decrement bond count             ML07  24
IF(nbond:(JP)<.5) group:(JP)=0.#       Not in group if no bonds         ML07  25
LB=NINT(bond2:(JP))#                   Get end of bond list             ML07  26
FOR(KB=NINT(bond1:(JP));KB<LB;KB=KB+STEPB)# Loop over connect bonds     ML07  27
IF(NINT(peak2:(KB))==IP)#              Find connected site1             ML07  28
peak1:(KB)=-ABS(peak1:(KB))#           Disable this bond too            ML07  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML07  30
#                                                                       ML07  31
RETURN#                                                                 ML07  32
END#                                                                    ML07  33
systemheader:(ML09)#                   Test dump of search lists        ML09   1
###################################################################     ML09   2
#                                                                       ML09   3
#                           ML09                                        ML09   4
#                                                                       ML09   5
####################################################################    ML09   6
SUBROUTINE ML09(NAME,NP,NF,NB,NT)#                                      ML09   7
include:(AACOMN)#                                                       ML09   8
include:(MLCOMN)#                                                       ML09   9
#                                                                       ML09  10
INTEGER I,J,K,L,ISTOP#                                                  ML09  11
INTEGER NP,NF,NB,NT#                                                    ML09  12
CHARACTER*4 NAME#                                                       ML09  13
chardata:(ML1,   N  LIST     X      Y      Z    WT CL NS   B1   $#      ML09  14
B2 ST RF GR CA NB OR MA   P1   P2   P3   P4   P5   P6   P7   P8)#       ML09  15
chardata:(ML2,        P1        P2        BL        P1        P2$#      ML09  16
        BL        P1        P2        BL        P1        P2        BL)#ML09  17
chardata:(ML3,   N  LIST  ATOM    $#                                    ML09  18
GR CA NB OR MA   P1   P2   P3   P4   P5   P6   P7   P8)#                ML09  19
chardata:(ML4,           )#                                             ML09  20
realdata:(FMT1,[40313.,100513.,160532.,230532.,300532.,360622.,#        ML09  21
390313.,420313.,470513.,520513.,550313.,580313.,610313.,640313.,#       ML09  22
670313.,700313.,730313.,780503.,830503.,880503.,930503.,980503.,#       ML09  23
1030503.,1080503.,1130503.,1180503.,1230503.,1180503.])#                ML09  24
realdata:(FMT2,[100813.,200813.,300832.,400813.,500813.,600832.,#       ML09  25
700813.,800813.,900832.,1000813.,1100813.,1200832.])#                   ML09  26
realdata:(FMT3,[40313.,100513.,220313.,250313.,280313.,#                ML09  27
310313.,340313.,390503.,440503.,490503.,540503.,590503.,6400503.,$#     ML09  28
690503.,740503.,790503.,840503.,890503.,940503.])#                      ML09  29
datastuff:#                                                             ML09  30
#                                                                       ML09  31
IF(PPRIOR>=5)#                         Test for dump priority           ML09  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML09  33
#                                                                       ML09  34
#    DUMP SITE LIST                                                     ML09  35
#    --------------                                                     ML09  36
#                                                                       ML09  37
IF(NP>0)#                              Test if site dump requested      ML09  38
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML09  39
movechar:(NAME,1,ML4,2,4,0)#           Store subroutine name            ML09  40
writeline:(1,ML4,NML4,2,3)#            Print subroutne name             ML09  41
J=1;  LINRM=4  ;K=ipbias:+MAXBON-2#                                     ML09  42
ISTOP=MIN0(STOP2,NP*STEP2+MARK2)#                                       ML09  43
FOR(I=MARK2;I<ISTOP;I=I+STEP2)#                                         ML09  44
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML09  45
BUFOT(1)=FLOAT(J)#                                                      ML09  46
BUFOT(2)=FLOAT(I)#                                                      ML09  47
movereal:(QX,I+3,BUFOT,3,K,0)#                                          ML09  48
ncodefld:(BUFOT,1,CHROT,FMT1,K+2)#                                      ML09  49
writeline:(0,ML1,NML1,1,3)#                                             ML09  50
J=J+1#                                                                  ML09  51
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML09  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML09  53
#                                                                       ML09  54
#    DUMP FRAGMENT LIST                                                 ML09  55
#    ------------------                                                 ML09  56
#                                                                       ML09  57
IF(NF>0)#                              Test if frag dump requested      ML09  58
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML09  59
writeline:(0,0,0,1,3)#                                                  ML09  60
J=1;  LINRM=4  ;K=5+MAXBON#                                             ML09  61
ISTOP=MIN0(STOP3,NF*STEP3+MARK3)#                                       ML09  62
FOR(I=MARK3;I<ISTOP;I=I+STEP3)#                                         ML09  63
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML09  64
BUFOT(1)=FLOAT(J)#                                                      ML09  65
BUFOT(2)=FLOAT(I)#                                                      ML09  66
L=I+K+1#                                                                ML09  67
movertoc:(QX(L),1,CHROT,13,6,0)#                                        ML09  68
movereal:(QX,I+1,BUFOT,3,K,0)#                                          ML09  69
ncodefld:(BUFOT,1,CHROT,FMT3,K+2)#                                      ML09  70
writeline:(0,ML3,NML3,1,3)#                                             ML09  71
J=J+1#                                                                  ML09  72
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML09  73
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML09  74
#                                                                       ML09  75
#    DUMP BOND LIST                                                     ML09  76
#    --------------                                                     ML09  77
#                                                                       ML09  78
IF(NB>0)#                              Test if bond dump requested      ML09  79
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML09  80
writeline:(0,0,0,1,3)#                                                  ML09  81
J=1;  LINRM=4#                                                          ML09  82
ISTOP=MIN0(STOPB,NB*STEPB+MARKB)#                                       ML09  83
FOR(I=MARKB;I<ISTOP;I=I+STEPB)#                                         ML09  84
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML09  85
ncodefld:(QX,I+1,CHROT,FMT2(J),3)#                                      ML09  86
J=J+3#                                                                  ML09  87
IF(J<12) NEXT#                                                          ML09  88
writeline:(0,ML2,NML2,1,3)#                                             ML09  89
J=1#                                                                    ML09  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML09  91
IF(J>1) writeline:(0,ML2,NML2,1,3)#                                     ML09  92
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML09  93
#                                                                       ML09  94
#   DUMP TWIN SITE LIST                                                 ML09  95
#   -------------------                                                 ML09  96
#                                                                       ML09  97
IF(NT>0)#                              Test if twin dump requested      ML09  98
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML09  99
writeline:(0,0,0,1,3)#                                                  ML09 100
J=1;  LINRM=4#                                                          ML09 101
ISTOP=MIN0(STOPT,NT*STEPT+MARKT)#                                       ML09 102
FOR(I=MARKT;I<ISTOP;I=I+STEPT)#                                         ML09 103
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML09 104
ncodefld:(QX,I+1,CHROT,FMT2(J),3)#                                      ML09 105
J=J+3#                                                                  ML09 106
IF(J<12) NEXT#                                                          ML09 107
writeline:(0,ML2,NML2,1,3)#                                             ML09 108
J=1#                                                                    ML09 109
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML09 110
IF(J>1) writeline:(0,ML2,NML2,1,3)#                                     ML09 111
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML09 112
writeline:(0,0,0,1,3)#                                                  ML09 113
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML09 114
#                                                                       ML09 115
RETURN#                                                                 ML09 116
END#                                                                    ML09 117
systemheader:(ML10)#                   Process =limit= and bdf          ML10   1
####################################################################### ML10   2
#                                                                       ML10   3
#                                ML10                                   ML10   4
#                                                                       ML10   5
####################################################################### ML10   6
SUBROUTINE ML10#                                                        ML10   7
include:(AACOMN)#                                                       ML10   8
include:(MLCOMN)#                                                       ML10   9
#                                                                       ML10  10
INTEGER I,J,K,L,N#                     General indices                  ML10  11
INTEGER MOLZ,NATYP#                                                     ML10  12
INTEGER MATCH#                         Count of peak-atom overlaps      ML10  13
INTEGER NREAD,ICNT#                    No. of sites to be entered       ML10  14
INTEGER IP,PSIZ#                       Pkt pointer & size               ML10  15
INTEGER IREL(4),IWNT(3),KEY(4)#                                         ML10  16
REAL ATRAD(16)#                        Atom radii                       ML10  17
REAL RADINC#                                                            ML10  18
#                                                                       ML10  19
string:(CHR,4)#                                                         ML10  20
string:(NAME,24)#                                                       ML10  21
string:(ATNAM,128)#                                                     ML10  22
chardata:(ML20," Input Atom and Peak Sites")#                           ML10  23
chardata:(ML21," Site     x/a      y/b      z/c     Weight$#            ML10  24
    Radius")#                                                           ML10  25
chardata:(ML22,NONE CONN MEAN ATOM NOAT SEXT NOSE )#                    ML10  26
chardata:(ML33," Sites connected within LENGTH limits are $#            ML10  27
grouped in CLUSTERS.")#                                                 ML10  28
chardata:(ML34," Sites connected within ANGLE  limits are $#            ML10  29
grouped in SUBCLUSTERS.")#                                              ML10  30
chardata:(ML36," Type of subcluster search applied        ")#           ML10  31
chardata:(ML37,"noneconnmean")#                                         ML10  32
chardata:(ML38," noyes")#                                               ML10  33
chardata:(ML40, LIMITS CONECT END )#                                    ML10  34
chardata:(ML41," Model Control Parameters-")#                           ML10  35
chardata:(ML51," Peak radius limits min/max        A  -.-- A")#         ML10  36
chardata:(ML43," Bond angle  limits min/max                 ")#         ML10  37
chardata:(ML45," Maximum atom and peak sites input        ")#           ML10  38
chardata:(ML46," Maximum bonds allowed per site           ")#           ML10  39
chardata:(ML47," Symmetry generated sites included        ")#           ML10  40
chardata:(ML48," Bond radii applied to atom sites         ")#           ML10  41
realdata:(FMT,[510822.,140952.,230952.,320952.,420622.])#               ML10  42
realdata:(TR11,[.5,.6666667,0.,0.,.5,.5])#                              ML10  43
realdata:(TR12,[.5,.3333333,.5,.5,0.,.5])#                              ML10  44
realdata:(TR13,[.5,.3333333,.5,.5,.5,0.])#                              ML10  45
realdata:(TR21,[.3333333,.5])#                                          ML10  46
realdata:(TR22,[.6666667,0.])#                                          ML10  47
realdata:(TR23,[.6666667,.5])#                                          ML10  48
realdata:(DTOR,0.0174533)#                                              ML10  49
datastuff:#                                                             ML10  50
#                                                                       ML10  51
#   SET DEFAULT PARAMETERS                                              ML10  52
#   ----------------------                                              ML10  53
#                                                                       ML10  54
MODW=1#                                Default connected subcl search   ML10  55
IEXT=-1#                               Unset site extension flag        ML10  56
NPROJ=0#                               No. of projections plotted       ML10  57
MAXBON=4#                              Maximum atomic coordination      ML10  58
PRTMAX=2.40#                           Default dmax for listing         ML10  59
RADMAX=1.0#                            Max allowed bond distance        ML10  60
RADMIN=0.5#                            Min allowed bond distance        ML10  61
RADINC=0.1#                            Default bdf radius increment     ML10  62
ANGMAX=145#                            Max. allowed bond angle          ML10  63
ANGMIN=85#                             Min. allowed bond angle          ML10  64
MTERP=5#                               Max. no. of interpretations      ML10  65
NREAD=99999#                           Set upper limit sites            ML10  66
MAXGRP=0#                              Max no. of atoms or sites        ML10  67
NFRAG=0#                               Counts of input fragments        ML10  68
NSITE=1000000#                         Number of bdf atom sites         ML10  69
RINGF=0; RINGP=0#                      Reset ring atom counts           ML10  70
DO I=1,3; IND(I)=1#                    Set indices for proj planes      ML10  71
#                                                                       ML10  72
#    PROCESS THE MODEL LINE                                             ML10  73
#    ----------------------                                             ML10  74
#                                                                       ML10  75
DO I=1,10#                             Loop over input fields           ML10  76
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML10  77
ifvo:(BUFIN(I)) BREAK#                 Exit on void                     ML10  78
stringlim:(I,J,K)#                     Get string limits                ML10  79
compchar:(CHRIN,J,ML22,1,4,5,NML22,K)# Match up the strings             ML10  80
IF(K==0) iquit:(ML1005)#               Illegal signal                   ML10  81
IF(K<4)         MODW=K-1#              Switch weighted search on        ML10  82
ELSE IF(K==5)   NSITE=0#               Do not include bdf atoms         ML10  83
ELSE IF(K==6)   IEXT=yes:#             Extend input site list           ML10  84
ELSE IF(K==7)   IEXT=no:#              Do not extend site list          ML10  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML10  86
#                                                                       ML10  87
#   ENTER =LIMITS= LINE                                                 ML10  88
#   -------------------                                                 ML10  89
#                                                                       ML10  90
REPEAT#                                Loop over input lines            ML10  91
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML10  92
readline:(ML40,NML40)#                 Read limits or conect line       ML10  93
#                                                                       ML10  94
IF(LINID==1)#                          Test for limits line             ML10  95
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML10  96
IF(BUFIN(1)>0.5) MAXBON=NINT(BUFIN(1))# Max atomic coordination         ML10  97
IF(BUFIN(2)>-.1) RADMIN=BUFIN(2)#      Min bond dist. for cluster       ML10  98
IF(BUFIN(3)>-.1) RADMAX=BUFIN(3)#      Max bond dist. for cluster       ML10  99
IF(BUFIN(4)>-.1) ANGMIN=BUFIN(4)#      Min angle of bonded atom         ML10 100
IF(BUFIN(5)>-.1) ANGMAX=BUFIN(5)#      Max angle of bonded atom         ML10 101
IF(BUFIN(6)>0.5) NREAD=NINT(BUFIN(6))# Max sites to be processed        ML10 102
IF(BUFIN(7)>-.1) PRTMAX=BUFIN(7)#      Max dist to be printed           ML10 103
IF(BUFIN(8)>-.1) RADINC=BUFIN(8)#      Radius incr to bdf values        ML10 104
IF(BUFIN(9)>0.5) MTERP=NINT(BUFIN(9))# Max no. interpretations          ML10 105
IF(BUFIN(10)>0.5) NPROJ=NINT(BUFIN(10))# Number printed projections     ML10 106
NEXT#                                  Read the next line               ML10 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML10 108
IF(LINID==2) NFRAG=1#                  Conect line entered              ML10 109
BREAK#                                 Exit for conect or end           ML10 110
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML10 111
NPROJ =MIN0(NPROJ,3)#                  Max of 3 projections             ML10 112
MAXBON=MIN0(MAXBON,12)#                Max of 12 coordination           ML10 113
ATNAM=' '#                             Clear atom type names            ML10 114
DO I=1,16; ATRAD(I)=RADMAX#            Reset atom type radius           ML10 115
#                                                                       ML10 116
#  ENTER PEKPIK FILE                                                    ML10 117
#  -----------------                                                    ML10 118
#                                                                       ML10 119
IOMARK(5)=QXSTR#                       Mark start of file e buffer      ML10 120
MARK1=IOMARK(5)+bdfbuf:#               Mark start of symops             ML10 121
incrqx:(I,MARK1,ML1001)#               Expand qx........ml1001          ML10 122
#                                                                       ML10 123
DO I=1,7#                              ******* cell data                ML10 124
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML10 125
readwpkt:(5,4,PSIZ,IP,0)#              Enter cell packet                ML10 126
IF(IP<=0) iquit:(ML1002)#              Cell info must be there          ML10 127
IF(I!=4) NEXT#                         Get orthogonal matrix pakt       ML10 128
T1=QX(IP+1)#                           A sinb sing*                     ML10 129
T2=QX(IP+2)#                           -a sinb cosg*                    ML10 130
T3=QX(IP+3)#                           A cosb                           ML10 131
T5=QX(IP+5)#                           B sina                           ML10 132
T6=QX(IP+6)#                           B cosa                           ML10 133
T9=QX(IP+9)#                           C                                ML10 134
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML10 135
G1=QX(IP+1)#                           A**2                             ML10 136
G2=QX(IP+5)#                           B**2                             ML10 137
G3=QX(IP+9)#                           C**2                             ML10 138
G4=QX(IP+2)*2.#                        2 a b cosg                       ML10 139
G5=QX(IP+3)*2.#                        2 a c cosb                       ML10 140
G6=QX(IP+6)*2.#                        2 b c cosa                       ML10 141
#                                                                       ML10 142
#                                      ********* symmetry data          ML10 143
readwpkt:(5,5,PSIZ,IP,0)#              Pick up lattice and symmetry     ML10 144
LTYP=NINT(QX(IP+1))#                   Lattice type                     ML10 145
NSYM=NINT(QX(IP+3))#                   Total number symmetry matrices   ML10 146
NROT=NINT(QX(IP+4))#                   Number of rotation matrices      ML10 147
IF(QX(IP+8)>0.)MOLZ=MAX(1,NINT(QX(IP+3)/QX(IP+8)))# Moles/asymm         ML10 148
ELSE           MOLZ=1#                 Default moles/asym unit is one   ML10 149
IF(IEXT<0&MOLZ>1) IEXT=yes:#           Flag to extend site list         ML10 150
ELSE IF(IEXT<0)   IEXT=no:#            Flag to not extend site list     ML10 151
ICNT=0#                                Set centro flag as centric       ML10 152
IF(LTYP>7) LTYP=LTYP-7#                Remove indicator of centrici     ML10 153
ELSE       ICNT=1#                     Set centro falg as acentric      ML10 154
incrqx:(STOP1,MARK1+12*NSYM,ML1003)#   Expand qx........ml1003          ML10 155
STEP1=12#                              Symop matrix increment           ML10 156
FOR(I=MARK1;I<STOP1;I=I+STEP1)#        Loop over symop matrices         ML10 157
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML10 158
readwpkt:(5,5,PSIZ,IP,0)#              Point to next symop packet       ML10 159
IF(IP<=0) BREAK#                       Finished when ip=0               ML10 160
movereal:(QX,IP+1,QX,I+1,PSIZ,1)#      Store symop in table 1           ML10 161
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML10 162
K=I;  MARK2=STOP1#                     Set matrix & site mark           ML10 163
IF(ICNT==0)#                           Test if centrosymmetric          ML10 164
FOR(J=MARK1;J<I;J=J+1)#                Loop over symm matrices          ML10 165
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML10 166
K=K+1#                                 Incr storage pointer             ML10 167
QX(K)=-QX(J+1)#                        Store inverse element            ML10 168
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML10 169
N=K#                                   Store temp end marker            ML10 170
IF(LTYP>1)#                            Test if sg centered              ML10 171
FOR(J=MARK1;J<N;J=J+STEP1)#            Loop over symm matrices          ML10 172
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML10 173
movereal:(QX,J+1,QX,K+1,9,1)#          Transfer rot elements            ML10 174
QX(K+10)=QX(J+10)+TR11(LTYP-1)#        Transfer t1                      ML10 175
QX(K+11)=QX(J+11)+TR12(LTYP-1)#        Transfer t2                      ML10 176
QX(K+12)=QX(J+12)+TR13(LTYP-1)#        Transfer t3                      ML10 177
K=K+STEP1#                             Incr storage pointer             ML10 178
IF((LTYP==2)\(LTYP>4)) NEXT#           Skip if not r or f               ML10 179
movereal:(QX,J+1,QX,K+1,9,1)#          Transfer rot elements            ML10 180
QX(K+10)=QX(J+10)+TR21(LTYP-2)#        Transfer t1                      ML10 181
QX(K+11)=QX(J+11)+TR22(LTYP-2)#        Transfer t2                      ML10 182
QX(K+12)=QX(J+12)+TR23(LTYP-2)#        Transfer t3                      ML10 183
K=K+STEP1#                             Incr storage pointer             ML10 184
IF(LTYP!=4) NEXT#                      Skip if not f-centre             ML10 185
movereal:(QX,J+1,QX,K+1,9,1)#          Transfer rot elements            ML10 186
QX(K+10)=QX(J+10)+0.5#                 Transfer t1                      ML10 187
QX(K+11)=QX(J+11)+0.5#                 Transfer t2                      ML10 188
QX(K+12)=QX(J+12)+0.0#                 Transfer t3                      ML10 189
K=K+STEP1#                             Incr storage pointer             ML10 190
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML10 191
FOR(J=MARK1;J<STOP1;J=J+STEP1)#        Loop over symops                 ML10 192
DO I=10,12; QX(I+J)=MOD(5.+QX(I+J),1.)# Store only positive shifts      ML10 193
#                                                                       ML10 194
#    GET ATOMIC RADII FOR ATOM SITES                                    ML10 195
#    -------------------------------                                    ML10 196
#                                                                       ML10 197
NATYP=0;   IWNT(1)=11;   IWNT(2)=5#    Request atom type name & radiu   ML10 198
KEY(1)=0; KEY(2)=2; KEY(3)=0; KEY(4)=0# Set the search keys             ML10 199
indexpkt:(5,lrscat:,PSIZ,IP,0,KEY,IWNT,IREL)# Test for wanted items     ML10 200
IF(IP*IREL(1)*IREL(2)>0)#              Provided name & radius present   ML10 201
REPEAT#                                Loop over atom types             ML10 202
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML10 203
readwpkt:(5,lrscat:,PSIZ,IP,0)#        Read next lrscatt packet         ML10 204
IF(IP<=0) BREAK#                       Exit after last packet           ML10 205
NATYP=NATYP+1#                         Count atom types                 ML10 206
movertoc:(QX(IP+IREL(1)),1,ATNAM,NATYP*8-7,8,0)# Store atom type name   ML10 207
ATRAD(NATYP)=QX(IP+IREL(2))+RADINC#    Store atom type radius           ML10 208
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML10 209
#                                                                       ML10 210
#                                      ********* sites information      ML10 211
readwpkt:(5,21,PSIZ,IP,0)#             Read first packet                ML10 212
NSITE=MIN0(NSITE,NINT(QX(IP+1)))#      Get number of bdf atoms          ML10 213
NSITES=MIN0(NREAD,NSITE+NINT(QX(IP+2)))# How many sites entered         ML10 214
SEPMIN=QX(IP+3)*QX(IP+3)#              Square the resolution            ML10 215
NATOM=MIN0(NSITES,NINT(QX(IP+4))/2)#   Expected atom sites              ML10 216
STEP2=ipbias:+MAXBON#                  Set the site-list items          ML10 217
incrqx:(STOP2,STOP1+NSITES*STEP2,ML1004)# Expand qx.......ml1004        ML10 218
IF(IEXT==yes:)#                        If sites to be extended          ML10 219
incrqx:(MARK3,STOP2+MOLZ*NSITES*STEP2,ML1004)# Expand qx.......ml100    ML10 220
ELSE MARK3=STOP2#                      No site extension                ML10 221
#                                                                       ML10 222
#   PRINT CONTROL SUMMARY                                               ML10 223
#   ---------------------                                               ML10 224
#                                                                       ML10 225
movechar:(ML41,NML41,CHROT,2,NML41-2,2)# Underline control header       ML10 226
writeline:(1,ML41,NML41-1,2,3)#        Print control header             ML10 227
ncodefld:(FLOAT(NSITES),1,ML45,420513.,1)# Format number of sites       ML10 228
writeline:(0,ML45,NML45,3,3)#          Print number of sites            ML10 229
ncodefld:(FLOAT(MAXBON),1,ML46,420413.,1)# Format max bonds per atom    ML10 230
writeline:(0,ML46,NML46,3,3)#          Print max bond limit             ML10 231
ncodefld:(RADMIN,1,ML51,340422.,1)#    Format min bond length           ML10 232
ncodefld:(RADMAX,1,ML51,420422.,1)#    Format max bond length           ML10 233
writeline:(0,ML51,NML51,3,3)#          Print length limits              ML10 234
ncodefld:(ANGMIN,1,ML43,340413.,1)#    Format min angle                 ML10 235
ncodefld:(ANGMAX,1,ML43,420413.,1)#    Format max angle                 ML10 236
IF(MODW>0)writeline:(0,ML43,NML43,3,3)# Print angle limits              ML10 237
movechar:(ML37,MODW*4+1,ML36,39,4,0)#  Insert subscluster search        ML10 238
writeline:(0,ML36,NML36,3,3)#          Print weighted search            ML10 239
movechar:(ML38,IEXT*3+1,ML47,40,3,0)#  Insert yes symm extended         ML10 240
writeline:(0,ML47,NML47,2,3)#          Print symm extended              ML10 241
IF(NSITE*NATYP>0)#                     If atom sites input              ML10 242
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML10 243
movechar:(ML48,1,CHROT,1,NML48,0)#     Insert atom radii header         ML10 244
DO I=1,NATYP#                          Loop over atom types             ML10 245
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML10 246
movechar:(ATNAM,I*8-7,CHROT,35,4,0)#   Insert atom type name            ML10 247
ncodefld:(ATRAD(I),1,CHROT,420422.,1)# Format atom type radius          ML10 248
writeline:(0,0,0,1,3)#                 Print atom radius                ML10 249
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML10 250
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML10 251
#                                                                       ML10 252
# EXTRACT ATOM AND PEAK SITES FROM FILE PEK                             ML10 253
# -----------------------------------------                             ML10 254
#                                                                       ML10 255
movechar:(ML40,NML40,CHROT,2,NML20-1,2)# Insert heading underline       ML10 256
writeline:(1,ML20,NML20,2,3)#          Print bdf atoms header           ML10 257
LINRM=6#                               Set bdf atom sites               ML10 258
MATCH=0#                               No peak - atom matches           ML10 259
FOR(I=MARK2;I<STOP2;I=I+STEP2)#        Loop over site table             ML10 260
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML10 261
N=(I-MARK2)/STEP2+1#                   Calc site/site number            ML10 262
numbr:(I)=FLOAT(N)#                    Store site number                ML10 263
IF(N<=NSITE)#                          Test if bdf atoms active         ML10 264
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML10 265
readwpkt:(5,21,PSIZ,IP,0)#             Point to bdf atom packet         ML10 266
movereal:(QX,IP+1,peakx:(I),1,4,1)#    Transfer x,y,z,ph to table       ML10 267
movereal:(QX(IP+5),1,aname:(I),1,1,0)# Transfer atom name (4 char)      ML10 268
movertoc:(QX(IP+5),1,NAME,1,24,0)#     Store atom label                 ML10 269
label:(NAME,J,K,ATNAM,L)#              Process atom label               ML10 270
atrad:(I)=ATRAD(L)#                    Store atomic radii               ML10 271
movechar:(NAME,1,CHROT,2,8,0)#         Store atom label                 ML10 272
ncodefld:(atrad:(I),1,CHROT,FMT,5)#    Format atom x,y,z,w,radius       ML10 273
writeline:(0,ML21,NML21,1,3)#          Print atom coords                ML10 274
NEXT#                                  Skip to next read                ML10 275
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML10 276
readwpkt:(5,22,PSIZ,IP,0)#             Read site packet                 ML10 277
IF(IP<=0) BREAK#                       Exit after last pkt              ML10 278
movereal:(QX,IP+1,peakx:(I),1,4,1)#    Transfer site to table           ML10 279
atrad:(I)=RADMAX#                      Set peak site radius             ML10 280
IF(peakh:(I)>900000.)#                 Skip if site not overlapped      ML10 281
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML10 282
IF(NSITE>0)#                           Skip overlapped sites            ML10 283
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ML10 284
MATCH=MATCH+1#                         Count number of matches          ML10 285
I=I-STEP2;     NEXT#                   Skip overlapped sites            ML10 286
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ML10 287
ELSE peakh:(I)=peakh:(I)-1000000.#     Remove overlap flag              ML10 288
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML10 289
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML10 290
STOP2=I; NSITES=(I-MARK2)/STEP2#       Reset end of site table          ML10 291
#                                                                       ML10 292
FOR(I=MARK2+NSITE*STEP2;I<STOP2;I=I+STEP2)# Loop over peaks in table    ML10 293
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ML10 294
N=(I-NSITE*STEP2-MARK2)/STEP2+1#       Calc peak locatn number          ML10 295
ncodefld:(FLOAT(N+NSITE),1,CHR,40413.,1)# Place number in name (4 cha   ML10 296
movector:(CHR,1,aname:(I),1,4,0)#      Place in atom list               ML10 297
ncodefld:(FLOAT(N+MATCH),1,CHROT,50413.,1)# Format site/site number     ML10 298
ncodefld:(atrad:(I),1,CHROT,FMT,5)#    Format atom x,y,z,ph,radius      ML10 299
writeline:(0,ML21,NML21,1,3)#          Print atom coords                ML10 300
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ML10 301
readwpkt:(5,endrecord:,I,I,0)#         Finish off file e                ML10 302
writeline:(1,ML33,NML33,3,3)#          Print cluster message            ML10 303
writeline:(0,ML34,NML34,3,3)#          Print subcluster message         ML10 304
#                                                                       ML10 305
#  SQUARE SEARCH LIMITS                                                 ML10 306
#  --------------------                                                 ML10 307
#                                                                       ML10 308
RADMAX=4.*RADMAX**2#                   Square search limit              ML10 309
RADMIN=4.*RADMIN**2#                   Square search limit              ML10 310
ANGMAX=ANGMAX*DTOR#                    Convert to radians               ML10 311
ANGMIN=ANGMIN*DTOR#                    Convert to radians               ML10 312
PRTMAX=PRTMAX*PRTMAX#                  Square search limit              ML10 313
#                                                                       ML10 314
CALL ML12#                             Enter any frag sites             ML10 315
CALL ML13#                             Group sites into clusters        ML10 316
#                                                                       ML10 317
RETURN#                                                                 ML10 318
END#                                                                    ML10 319
systemheader:(ML12)#                   Process conect lines             ML12   1
####################################################################### ML12   2
#                                                                       ML12   3
#                                ML12                                   ML12   4
#                                                                       ML12   5
####################################################################### ML12   6
SUBROUTINE ML12#                                                        ML12   7
include:(AACOMN)#                                                       ML12   8
include:(MLCOMN)#                                                       ML12   9
#                                                                       ML12  10
# 1 FRAGMENT NUMBER                     FROM CONECT LINE                ML12  11
# 2 CANONICAL DESCRIPTION               SET IN ML41 (CALLED BY ML40)    ML12  12
# 3 NUMBER OF BONDS                     SET IN ML12                     ML12  13
# 4 RANK IN FRAGMENT                    SET IN ML41                     ML12  14
# 5 SITE TO WHICH THE ATOM CORRESPONDS  SET IN ML42 (CALLED BY ML40)    ML12  15
# 6-11 NUMBERS OF ATOMS BONDED TO IT    FROM CONECT LINES               ML12  16
# 12- ATOM NAME (WORDS TO HOLD 6 CHAR)  FROM CONECT LINES               ML12  17
#                                                                       ML12  18
string:(ANAME,6)#                      Atom name array                  ML12  19
string:(BNAME,6)#                      Atom name array                  ML12  20
INTEGER I,J,K,L,M,N#                   General indices                  ML12  21
INTEGER MN,MM,MK,MS#                   Loop indices                     ML12  22
INTEGER IP,NMBIAS,IPBIAS,JP#           Offsets for frag list            ML12  23
#                                                                       ML12  24
chardata:(SEMI,;-)#                                                     ML12  25
chardata:(ML23, CONECT END )#                                           ML12  26
chardata:(ML24," Atoms Input as Proposed Model")#                       ML12  27
chardata:(ML26," Atom    Group   connected to....")#                    ML12  28
datastuff:#                                                             ML12  29
#                                                                       ML12  30
#   PREPARE TO STORE MODEL STRUCTURE AS FRAGMENTS                       ML12  31
#   ---------------------------------------------                       ML12  32
#                                                                       ML12  33
STEP3=7+MAXBON#                        Set fragment list step           ML12  34
NMBIAS=6+MAXBON#                       Point to atom name               ML12  35
IPBIAS=5-ipbias:#                      Bias macro list elements         ML12  36
STOP3=MARK3#                           Set end of fragment list         ML12  37
DO J=1,20;  NINFR(J)=0#                Zero count of atoms in each      ML12  38
#                                                                       ML12  39
#   ENTER ANY =CONECT= LINES                                            ML12  40
#   ------------------------                                            ML12  41
#                                                                       ML12  42
IF(NFRAG>0)#                           Test if frgmt line in buffer     ML12  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML12  44
REPEAT#                                Loop over all =conect= input     ML12  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML12  46
MN=0;   MS=0#                          Set central atom flag            ML12  47
DO M=1,mxfdim:#                        Loop over all fields             ML12  48
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML12  49
ifvo:(BUFIN(M)) BREAK#                 Exit if void field               ML12  50
IF(BUFIN(M)>voidflg:)#                 Extract frag number              ML12  51
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML12  52
NFRAG=NINT(BUFIN(M));  NEXT#           Extract frag number              ML12  53
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML12  54
IF(MS>0) MN=0#                         Set central atom flag            ML12  55
MS=0#                                  Reset semicolon flag             ML12  56
stringlim:(M,J,K)#                     Get limits of string             ML12  57
compchar:(CHRIN,J,SEMI,1,1,1,1,L)#     Test if semicolon                ML12  58
IF(L==1)#                              If semicolon  found              ML12  59
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML12  60
MN=0;  NEXT#                           Central atom to follow           ML12  61
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML12  62
compchar:(CHRIN,K-1,SEMI,1,1,1,1,L)#   Test for semicolon               ML12  63
IF(L==1)#                              If semicolon is found            ML12  64
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML12  65
MS=1;  K=K-1#                          Set semicolon flag & remove      ML12  66
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML12  67
K=MIN0(6,K-J)#                         Allow 6 chars at most for at     ML12  68
movechar:(BLANK,1,ANAME,1,6,1)#        Blank out atom name              ML12  69
movechar:(CHRIN,J,ANAME,1,K,0)#        Pick up atom name                ML12  70
FOR(MK=MARK3;MK<STOP3;MK=MK+STEP3)#    Loop over fragment list          ML12  71
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML12  72
JP=MK+NMBIAS#                          Point to atom name               ML12  73
movertoc:(QX(JP),1,BNAME,1,6,0)#       Extract atom name                ML12  74
compchar:(ANAME,1,BNAME,1,6,6,6,N)#    Match atom names                 ML12  75
IF(N>0) BREAK#                         Exit if match found              ML12  76
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML12  77
IP=MK+IPBIAS#                          Offset pointer for list macr     ML12  78
IF(MK==STOP3)#                         Test if new atom name            ML12  79
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML12  80
NINFR(NFRAG)=NINFR(NFRAG)+1#           Count atoms in fragment          ML12  81
incrqx:(STOP3,MK+STEP3,ML1202)#        Expand qx.....ml1202             ML12  82
group:=FLOAT(NFRAG)#                   Store fragment number            ML12  83
JP=MK+NMBIAS#                          Point to atom name               ML12  84
movector:(ANAME,1,QX(JP),1,6,0)#       Store new atom name              ML12  85
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML12  86
IF(MN==0) MN=MK#                       If central atom save addr        ML12  87
ELSE#                                  If bonded atom                   ML12  88
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML12  89
JP=MN+IPBIAS#                          Point to connect site            ML12  90
DO I=1,2#                              Flip over central/bonded ato     ML12  91
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML12  92
FOR(J=1;J<=MAXBON;J=J+1)#              Loop over store bond atoms       ML12  93
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML12  94
K=MK+J+5#                              Point to bonded atoms list       ML12  95
IF(NINT(QX(K))==JP) BREAK#             Exit if already stored           ML12  96
IF(QX(K)>.5) NEXT#                     Skip if occupied                 ML12  97
QX(K)=FLOAT(JP)#                       Store connected atom addr        ML12  98
nbond:=nbond:+1.#                      Count bonds to  atom             ML12  99
BREAK#                                 Exit                             ML12 100
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML12 101
IF(J>MAXBON) iquit:(ML1203)#           Ip error ........ml1203          ML12 102
IP=MN+IPBIAS; JP=MK+IPBIAS; MK=MN#     Reset for central atom           ML12 103
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML12 104
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML12 105
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML12 106
readline:(ML23,NML23)#                 Enter next input line            ML12 107
IF(LINID!=1) BREAK#                    Exit with end line               ML12 108
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML12 109
#                                                                       ML12 110
#   LIST STORE LIST OF FRAGMENTS                                        ML12 111
#   ----------------------------                                        ML12 112
#                                                                       ML12 113
LINRM=5#                               Force fragment header            ML12 114
movechar:(SEMI,2,CHROT,2,NML24-1,2)#   Insert header underline          ML12 115
writeline:(1,ML24,NML24,1,3)#          Print frag header                ML12 116
LINRM=4#                               Force first header               ML12 117
FOR(MN=MARK3;MN<STOP3;MN=MN+STEP3)#    Loop over conect atoms           ML12 118
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML12 119
IP=MN+IPBIAS;  JP=MN+NMBIAS#           Point to start & atom name       ML12 120
movertoc:(QX(JP),1,CHROT,2,6,0)#       Put atom name in buffer          ML12 121
ncodefld:(group:,1,CHROT,110413.,1)#   Format fragment number           ML12 122
K=NINT(nbond:)#                        Number of bonded atoms           ML12 123
DO J=1,K#                              Loop over connected atoms        ML12 124
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML12 125
L=MN+J+5#                              Point to bonded atoms list       ML12 126
MM=NINT(QX(L))-IPBIAS+NMBIAS#          Extract atom address             ML12 127
movertoc:(QX(MM),1,CHROT,J*7+11,6,0)#  Put atom name in op buffer       ML12 128
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML12 129
writeline:(0,ML26,NML26,1,3)#          Print frag contents              ML12 130
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML12 131
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML12 132
#                                                                       ML12 133
#   MARK START OF CONNECTION & TWIN ARRAYS                              ML12 134
#   --------------------------------------                              ML12 135
#                                                                       ML12 136
MARKB=STOP3#                           Start of bond list               ML12 137
STEPB=3; STEPT=3#                      Set steps bond & twin list       ML12 138
STOPB=MARKB+2*NSITES*STEPB*(MAXBON+2)# End of bond list                 ML12 139
MARKT=STOPB#                           Start of twin list               ML12 140
incrqx:(STOPT,MARKT,ML1204)#           Expand qx.......ml1204           ML12 141
FOR(K=1;K<=NFRAG;K=K+1)#               Loop over input fragments        ML12 142
MAXGRP=MAX0(MAXGRP,NINFR(K))#          Find max atoms in any frag       ML12 143
#                                                                       ML12 144
RETURN#                                                                 ML12 145
END#                                                                    ML12 146
systemheader:(ML13)#                   Group sites into clusters        ML13   1
####################################################################### ML13   2
#                                                                       ML13   3
#                                ML13                                   ML13   4
#                                                                       ML13   5
####################################################################### ML13   6
SUBROUTINE ML13#                                                        ML13   7
include:(AACOMN)#                                                       ML13   8
include:(MLCOMN)#                                                       ML13   9
#                                                                       ML13  10
INTEGER I,J,K,L,N#                                                      ML13  11
INTEGER IP,IB,JA,JP,KA,KP,IT,FNEW,NC#                                   ML13  12
REAL A,B,C,P,Q,R,X,Y,Z,U,V,W#                                           ML13  13
REAL SX,SY,SZ,CX,CY,CZ,CN#                                              ML13  14
REAL TM(3),DD,DIST,RAD,R2MAX#                                           ML13  15
#                                                                       ML13  16
#  GROUP SITES INTO CONNECTED CLUSTERS ABOUT BODY CENTRE                ML13  17
#  ----------------------------------------------------                 ML13  18
#                                                                       ML13  19
NCLUS=0;     STOPI=STOP2#              Initialize no. of clusters       ML13  20
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13  21
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML13  22
ifne:(clust:,0.) NEXT#                 Skip if cluster assigned         ML13  23
NCLUS=MIN0(NCLUS+1,32)#                Incr cluster counter             ML13  24
clust:=FLOAT(NCLUS)#                   Mark site with this cluster      ML13  25
N=1;   DIST=9999999.;   FNEW=yes:#     Zero cluster cnt & reset flg     ML13  26
X=peakx:;  Y=peaky:;  Z=peakz:#        Store site coords local          ML13  27
#                                                                       ML13  28
FOR(K=MARK1;K<STOP1;K=K+STEP1)#        Loop over all symops             ML13  29
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML13  30
U=X*QX(K+1)+Y*QX(K+4)+Z*QX(K+7)+QX(K+10)# Generate equivalent x         ML13  31
V=X*QX(K+2)+Y*QX(K+5)+Z*QX(K+8)+QX(K+11)# Generate equivalent y         ML13  32
W=X*QX(K+3)+Y*QX(K+6)+Z*QX(K+9)+QX(K+12)# Generate equivalent z         ML13  33
U=MOD(99.+U,1.)-0.5#                   Ensure x' close to x             ML13  34
V=MOD(99.+V,1.)-0.5#                   Ensure y' close to y             ML13  35
W=MOD(99.+W,1.)-0.5#                   Ensure z' close to z             ML13  36
DD=G1*U*U+G2*V*V+G3*W*W+G4*U*V+G5*U*W+G6*V*W# Distance squared          ML13  37
IF(DD>=DIST) NEXT#                     Skip if outside range            ML13  38
P=U+.5; Q=V+.5; R=W+.5; DIST=DD-0.01#  Store the closest coords         ML13  39
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML13  40
peakx:=P; peaky:=Q; peakz:=R#          Keep site closest to 0.          ML13  41
#                                                                       ML13  42
WHILE(FNEW==yes:)#                     Loop if new connects found       ML13  43
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML13  44
FNEW=no:#                              Assume no connects found         ML13  45
FOR(JP=MARK2;JP<STOP2;JP=JP+STEP2)#    Loop over all sites              ML13  46
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML13  47
ifne:(clust:(JP),clust:) NEXT#         Skip unless same cluster         ML13  48
RAD=atrad:(JP)#                        Get atom radius                  ML13  49
P=peakx:(JP); Q=peaky:(JP); R=peakz:(JP)# Get site coords to local      ML13  50
#                                                                       ML13  51
FOR(KP=MARK2;KP<STOP2;KP=KP+STEP2)#    Loop over all sites              ML13  52
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML13  53
ifne:(clust:(KP),0.) NEXT#             Skip if connected                ML13  54
R2MAX=(RAD+atrad:(KP))**2#             Get radius max squared           ML13  55
X=peakx:(KP); Y=peaky:(KP); Z=peakz:(KP)# Get site coords to local      ML13  56
#                                                                       ML13  57
FOR(K=MARK1;K<STOP1;K=K+STEP1)#        Loop over all symops             ML13  58
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML13  59
U=X*QX(K+1)+Y*QX(K+4)+Z*QX(K+7)+QX(K+10)# Generate equivalent x         ML13  60
V=X*QX(K+2)+Y*QX(K+5)+Z*QX(K+8)+QX(K+11)# Generate equivalent y         ML13  61
W=X*QX(K+3)+Y*QX(K+6)+Z*QX(K+9)+QX(K+12)# Generate equivalent z         ML13  62
U=MOD(99.5+U-P,1.)-0.5#                Ensure x' close to x             ML13  63
V=MOD(99.5+V-Q,1.)-0.5#                Ensure y' close to y             ML13  64
W=MOD(99.5+W-R,1.)-0.5#                Ensure z' close to z             ML13  65
DD=G1*U*U+G2*V*V+G3*W*W+G4*U*V+G5*U*W+G6*V*W# Distance squared          ML13  66
IF(DD<R2MAX) GOTO 100#                 Exit if connected                ML13  67
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML13  68
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML13  69
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML13  70
NEXT#                                  Atom not connected; get next     ML13  71
100 CONTINUE#                          Atom connected; bonded or twin   ML13  72
IF(DD<SEPMIN)#                         Are sites sym related            ML13  73
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML13  74
IT=STOPT#                              Point to twin list               ML13  75
incrqx:(STOPT,STOPT+STEPT,ML1301)#     Incr qx.........ml1301           ML13  76
peak1:(IT)=FLOAT(MIN0(JP,KP))#         Site 1 addr. in twin list        ML13  77
peak2:(IT)=FLOAT(MAX0(JP,KP))#         Site 2 addr. in twin list        ML13  78
bondl:(IT)=SQRT(DD)#                   Store distance in twin list      ML13  79
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML13  80
N=N+1;    FNEW=yes:#                   Count atoms in this cluster      ML13  81
peakx:(KP)=U+P#                        Calculate connected site x"      ML13  82
peaky:(KP)=V+Q#                        Calculate connected site y"      ML13  83
peakz:(KP)=W+R#                        Calculate connected site z"      ML13  84
clust:(KP)=clust:#                     Flag as connected                ML13  85
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML13  86
NPKCL(NCLUS)=N#                        Store sites in cluster           ML13  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML13  88
#                                                                       ML13  89
#    EXPAND INPUT ATOM LIST TO INCLUDE ALL CONNECTIONS                  ML13  90
#    -------------------------------------------------                  ML13  91
#                                                                       ML13  92
FOR(FNEW=IEXT;FNEW==yes:;)#            Loop until no connections        ML13  93
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML13  94
FNEW=no:#                              Assume no connections found      ML13  95
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13  96
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML13  97
RAD=atrad:#                            Get atom radius                  ML13  98
FOR(J=MARK1;J<STOP1;J=J+STEP1)#        Loop over symops                 ML13  99
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML13 100
U=peakx:*QX(J+1)+peaky:*QX(J+4)+peakz:*QX(J+7)+QX(J+10)# New x'         ML13 101
V=peakx:*QX(J+2)+peaky:*QX(J+5)+peakz:*QX(J+8)+QX(J+11)# New y'         ML13 102
W=peakx:*QX(J+3)+peaky:*QX(J+6)+peakz:*QX(J+9)+QX(J+12)# New z'         ML13 103
#                                                                       ML13 104
FOR(JA=MARK2;JA<STOP2;JA=JA+STEP2)#    Loop thro. all stored atoms      ML13 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML13 106
ifne:(clust:,clust:(JA))  NEXT#        Skip if different cluster        ML13 107
IF(IP==JA&J==MARK2) NEXT#              Avoid identical overlap          ML13 108
P=MOD(99.5+U-peakx:(JA),1.0)-0.5#      Get closest bonded atom          ML13 109
Q=MOD(99.5+V-peaky:(JA),1.0)-0.5#      Get closest bonded atom          ML13 110
R=MOD(99.5+W-peakz:(JA),1.0)-0.5#      Get closest bonded atom          ML13 111
DD=G1*P*P+G2*Q*Q+G3*R*R+G4*P*Q+G5*P*R+G6*Q*R# Get distance squared      ML13 112
IF(DD<.01\DD>(RAD+atrad:(JA))**2) NEXT# Skip if not bonded              ML13 113
P=P+peakx:(JA); Q=Q+peaky:(JA); R=R+peakz:(JA)# Store new coords        ML13 114
#                                                                       ML13 115
FOR(KA=MARK2;KA<STOP2;KA=KA+STEP2)#    Atom pos. already stored ?       ML13 116
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML13 117
ifne:(clust:,clust:(KA))   NEXT#       Skip if different cluster        ML13 118
IF(ABS(MOD(99.5+P-peakx:(KA),1.)-.5)>.001) NEXT# Skip if x coord diff   ML13 119
IF(ABS(MOD(99.5+Q-peaky:(KA),1.)-.5)>.001) NEXT# Skip if y coord diff   ML13 120
IF(ABS(MOD(99.5+R-peakz:(KA),1.)-.5)<.001) BREAK# Break if pos. equiv   ML13 121
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML13 122
IF(KA<STOP2) NEXT#                     Pos. not found-new atom          ML13 123
I=STOP2;       FNEW=yes:#              Save current end of list         ML13 124
peakx:(I)=P; peaky:(I)=Q; peakz:(I)=R# Store new coords                 ML13 125
clust:(I)=clust:#                      Store cluster                    ML13 126
numbr:(I)=numbr:#                      Store site number                ML13 127
peakh:(I)=peakh:#                      Store peakheight                 ML13 128
movereal:(aname:,1,aname:(I),1,1,0)#   New atom name                    ML13 129
STOP2=STOP2+STEP2#                     Incr expanded list pointer       ML13 130
IF(STOP2==MARK3) GOTO 200#             Exit if list full                ML13 131
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML13 132
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML13 133
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML13 134
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML13 135
200 CONTINUE#                          Exit for site list overflow      ML13 136
#                                                                       ML13 137
#   PACK CENTRES OF CLUSTERS CLOSE TOGETHER                             ML13 138
#   ---------------------------------------                             ML13 139
#                                                                       ML13 140
DO NC=1,NCLUS#                         Loop over clusters               ML13 141
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML13 142
CN=0.;  SX=0.;  SY=0.;  SZ=0.#         Zero centre sums                 ML13 143
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13 144
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ML13 145
IF(NINT(clust:)!=NC) NEXT#             Skip if not in this cluster      ML13 146
CN=CN+1.#                              Count contributions              ML13 147
SX=SX+peakx:#                          Sum x coords                     ML13 148
SY=SY+peaky:#                          Sum y coords                     ML13 149
SZ=SZ+peakz:#                          Sum z coords                     ML13 150
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ML13 151
IF(CN>0.5)#                            If there are contributors        ML13 152
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ML13 153
SX=SX/CN;  SY=SY/CN;  SZ=SZ/CN#        Store centre of cluster          ML13 154
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ML13 155
#                                                                       ML13 156
IF(NC==1)#                             For cluster 1                    ML13 157
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ML13 158
CX=SX;  CY=SY;  CZ=SZ;  NEXT#          Save centre first cluster        ML13 159
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ML13 160
DIST=99999999.#                        Set initial vector               ML13 161
FOR(K=MARK1;K<STOP1;K=K+STEP1)#        Loop over all symops             ML13 162
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 ML13 163
X=SX*QX(K+1)+SY*QX(K+4)+SZ*QX(K+7)+QX(K+10)# Generate equivalent x      ML13 164
Y=SX*QX(K+2)+SY*QX(K+5)+SZ*QX(K+8)+QX(K+11)# Generate equivalent y      ML13 165
Z=SX*QX(K+3)+SY*QX(K+6)+SZ*QX(K+9)+QX(K+12)# Generate equivalent z      ML13 166
P=X-CX;  Q=Y-CY;  R=Z-CZ#              Get diff with centre 1           ML13 167
U=MOD(99.5+P,1.)-0.5#                  Ensure x' close to centre 1      ML13 168
V=MOD(99.5+Q,1.)-0.5#                  Ensure y' close to centre 1      ML13 169
W=MOD(99.5+R,1.)-0.5#                  Ensure z' close to centre 1      ML13 170
DD=G1*U*U+G2*V*V+G3*W*W+G4*U*V+G5*U*W+G6*V*W# Distance squared          ML13 171
IF(DD>=DIST) NEXT#                     Skip if not smallest             ML13 172
A=P-U; B=Q-V; C=R-W; J=K; DIST=DD-0.01# Closest to centre 1             ML13 173
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 ML13 174
#                                                                       ML13 175
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13 176
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 ML13 177
IF(NINT(clust:)!=NC) NEXT#             Skip if not this cluster         ML13 178
X=peakx:*QX(J+1)+peaky:*QX(J+4)+peakz:*QX(J+7)+QX(J+10)# Generate  x    ML13 179
Y=peakx:*QX(J+2)+peaky:*QX(J+5)+peakz:*QX(J+8)+QX(J+11)# Generate  y    ML13 180
Z=peakx:*QX(J+3)+peaky:*QX(J+6)+peakz:*QX(J+9)+QX(J+12)# Generate  z    ML13 181
peakx:=X-A#                            Move x to new centre             ML13 182
peaky:=Y-B#                            Move y to new centre             ML13 183
peakz:=Z-C#                            Move z to new centre             ML13 184
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 ML13 185
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML13 186
#                                                                       ML13 187
#    ENSURE CENTRE OF COMBINED CLUSTERS INSIDE CELL                     ML13 188
#    ----------------------------------------------                     ML13 189
#                                                                       ML13 190
DO I=1,3; TM(I)=0.;    CN=0.#          Set centre sums to zero          ML13 191
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13 192
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 ML13 193
CN=CN+1.#                              Count contributions              ML13 194
DO I=1,3; TM(I)=TM(I)+QX(IP+I+2)#      Sum all coords                   ML13 195
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 ML13 196
IF(CN>0.5)#                            If there are contributors        ML13 197
DO I=1,3#                              Loop over coords                 ML13 198
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 ML13 199
TM(I)=TM(I)/CN#                        Get average coord                ML13 200
IF     (TM(I)<0.) TM(I)=+1.#           Centre out of cell               ML13 201
ELSE IF(TM(I)>1.) TM(I)=-1.#           Centre out of cell               ML13 202
ELSE              TM(I)= 0.#           Centre in cell                   ML13 203
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 ML13 204
#                                                                       ML13 205
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13 206
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 ML13 207
DO I=1,3; QX(IP+I+2)=QX(IP+I+2)+TM(I)# Move to new centre               ML13 208
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 ML13 209
#                                                                       ML13 210
#   COUNT ALL CONNECTED SITES                                           ML13 211
#   -------------------------                                           ML13 212
#                                                                       ML13 213
IB=MARKB#                              Start of bond table              ML13 214
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML13 215
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 ML13 216
X=peakx:; Y=peaky:; Z=peakz:; RAD=atrad:# Store site coords local       ML13 217
#                                                                       ML13 218
FOR(JP=MARK2;JP<STOP2;JP=JP+STEP2)#    Loop over all sites              ML13 219
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 ML13 220
ifne:(clust:,clust:(JP)) NEXT#         Skip if not the same cluster     ML13 221
IF(IP==JP) NEXT#                       Skip identical sites             ML13 222
U=X-peakx:(JP);V=Y-peaky:(JP);W=Z-peakz:(JP)# Get displacements         ML13 223
DD=SQRT(G1*U*U+G2*V*V+G3*W*W+G4*U*V+G5*U*W+G6*V*W)# Distance squared    ML13 224
IF(DD>RAD+atrad:(JP)) NEXT#            Skip unconnected sites           ML13 225
I=(IP-MARK2)/STEP2;  J=(JP-MARK2)/STEP2# Extract list serial no.s       ML13 226
peak1:=FLOAT(IP)#                      Store site address 1             ML13 227
intpak:(J,peak2:, 0,mxbtwd:)#          Store site number 2              ML13 228
intpak:(I,peak2:,16,     16)#          Store sort number                ML13 229
bondl:=DD#                             Store intersite distance         ML13 230
IB=IB+STEPB#                           Incr bond list pointer           ML13 231
IF(IB>STOPB) iquit:(ML1302)#           Overflow.......ml1302            ML13 232
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 ML13 233
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 ML13 234
STOPB=IB#                              Set end of bond list             ML13 235
NBOND=(STOPB-MARKB)/STEPB#             Store total bonds                ML13 236
NTWIN=(STOPT-MARKT)/STEPT#             Store total twin sites           ML13 237
#                                                                       ML13 238
#  SORT CONNECT ATOMS LIST IN INCREASING SITE NUMBER                    ML13 239
#  -------------------------------------------------                    ML13 240
#                                                                       ML13 241
FOR(N=NBOND/2;N>0;N=N/2)#              This is a shell sort             ML13 242
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 ML13 243
L=STEPB*N#                             Get n step                       ML13 244
FOR(I=MARKB+L+STEPB;I<STOPB;I=I+STEPB)# Forward list loop               ML13 245
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 ML13 246
FOR(J=I-L;J>MARKB;J=J-STEPB)#          Backward list loop               ML13 247
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 ML13 248
IB=J+L#                                Incr spacer                      ML13 249
compakwd:(peak2:(J),LE,peak2:,BREAK)#  Compare packed site no.s         ML13 250
movereal:(QX,IB+1,TM,   1,3,0)#        Save first bond                  ML13 251
movereal:(QX,J +1,QX,IB+1,3,0)#        Transfer second bond             ML13 252
movereal:(TM,   1,QX,J +1,3,0)#        Transfer first bond              ML13 253
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 ML13 254
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 ML13 255
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 ML13 256
#                                                                       ML13 257
#   COUNT CONNECTS TO EACH SITE                                         ML13 258
#   ---------------------------                                         ML13 259
#                                                                       ML13 260
IP=MARK2;  A=FLOAT(STEPB)#             Point to start of site list      ML13 261
FOR(IB=MARKB;IB<STOPB;IB=IB+STEPB)#    Loop over bond list              ML13 262
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 ML13 263
intunpak:(peak2:,I,0,16)#              Extract site number              ML13 264
peak2:=FLOAT(MARK2+I*STEP2)#           Store as site address            ML13 265
N=NINT(peak1:)#                        Get site address                 ML13 266
FOR(;IP<=N;IP=IP+STEP2)#               Loop over site list              ML13 267
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 ML13 268
bond1:=FLOAT(IB)#                      Store bond list pointer          ML13 269
bond2:=FLOAT(IB)#                      Store bondlist stop              ML13 270
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 ML13 271
nbond:(N)=nbond:(N)+1.#                Count bonds to site              ML13 272
bond2:(N)=bond2:(N)+A#                 Incr bondlist stop addr.         ML13 273
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 ML13 274
FOR(;IP<STOP2;IP=IP+STEP2)#            Loop over remaining sites        ML13 275
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 ML13 276
bond1:=FLOAT(STOPB)#                   Store bond list start            ML13 277
bond2:=FLOAT(STOPB)#                   Store bond list end              ML13 278
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 ML13 279
TCLUS=NCLUS#                           Store total clusters             ML13 280
MARK4=STOPT#                           Mark beginning of mark4          ML13 281
dumpsite:('ML13',100,100,1000,100)#    End of ml13                      ML13 282
#                                                                       ML13 283
RETURN#                                                                 ML13 284
END#                                                                    ML13 285
systemheader:(ML20)#                   Select site to start search      ML20   1
####################################################################### ML20   2
#                                                                       ML20   3
#                                ML20                                   ML20   4
#                                                                       ML20   5
####################################################################### ML20   6
SUBROUTINE ML20#                                                        ML20   7
include:(AACOMN)#                                                       ML20   8
include:(MLCOMN)#                                                       ML20   9
#                                                                       ML20  10
INTEGER IP,IB#                                                          ML20  11
#                                                                       ML20  12
#   CHECK THE NUMBER OF BOND CONNECTIONS                                ML20  13
#   ------------------------------------                                ML20  14
#                                                                       ML20  15
ISTART=0#                              Set initial site number          ML20  16
IF(NPKCL(NCLUS)>2)#                    Look at clusters >2              ML20  17
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML20  18
IF(NTERP==1) TOPFOM=0.1#               Reset cluster fom max            ML20  19
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over site list              ML20  20
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML20  21
movereal:(0.,1,ringf:,1,MAXBON+6,2)#   Zero site parameters             ML20  22
IF(NINT(clust:)==NCLUS)nbond:=(bond2:-bond1:)/3.# Number of bonds       ML20  23
IF(NTERP==1) start:=0.#                Reset starter flag               ML20  24
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML20  25
FOR(IB=MARKB;IB<STOPB;IB=IB+STEPB)#    Loop over bond list              ML20  26
peak1:=ABS(peak1:)#                    Reset bond cancel flag           ML20  27
#                                                                       ML20  28
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Look for first site in nclus     ML20  29
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML20  30
IF(NINT(clust:)!=NCLUS) NEXT#          Skip until match found           ML20  31
IF((start:>.5)\(nbond:<.5)) NEXT#      Skip if started or void          ML20  32
ISTART=IP#                             Get site list start addr.        ML20  33
start:=1.0#                            Mark this as used for starti     ML20  34
BREAK#                                                                  ML20  35
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML20  36
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML20  37
#                                                                       ML20  38
RETURN#                                                                 ML20  39
END#                                                                    ML20  40
systemheader:(ML30)#                   Unweighted subcluster search     ML30   1
####################################################################### ML30   2
#                                                                       ML30   3
#                                ML30                                   ML30   4
#                                                                       ML30   5
####################################################################### ML30   6
SUBROUTINE ML30#                                                        ML30   7
include:(AACOMN)#                                                       ML30   8
include:(MLCOMN)#                                                       ML30   9
#                                                                       ML30  10
INTEGER IB,JB,IP,JP,KP#                Loop indices                     ML30  11
INTEGER IB1,IB2,IB3#                   Loop limits                      ML30  12
INTEGER M,IS,LP#                                                        ML30  13
REAL COSA,CMAX,CMIN#                   Cosine & limits                  ML30  14
REAL BMIN#                                                              ML30  15
REAL DD,DX,DY,DZ#                      Bond distance & dif xyz          ML30  16
#                                                                       ML30  17
#   PREPARE TO APPLY LIMITS                                             ML30  18
#   -----------------------                                             ML30  19
#                                                                       ML30  20
IF(ANGMAX>3.) CMAX=-2.#                Allow for max of 180 deg         ML30  21
ELSE          CMAX=COS(ANGMAX)#        Find cos of max allowed angl     ML30  22
CMIN=COS(ANGMIN)#                      And min allowed angle            ML30  23
BMIN=SQRT(RADMIN)#                     Get bond minimum                 ML30  24
MSUB=0#                                Zero subcluster limits           ML30  25
#                                                                       ML30  26
#   LOOP OVER SITELIST FROM START                                       ML30  27
#   -----------------------------                                       ML30  28
#                                                                       ML30  29
FOR(IS=ISTART;IS<STOP2;IS=IS+STEP2)#   Loop over site list              ML30  30
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML30  31
IF((NINT(clust:(IS))!=NCLUS)$#         If site not in cluster           ML30  32
\  (ABS(group:(IS))>small:)$#          If subcluster unassigned         ML30  33
\  (nbond:(IS)<.5))            NEXT#   Site is not connected            ML30  34
MSUB=MSUB+1#                           Incr subcluster count            ML30  35
group:(IS)=FLOAT(MSUB)#                Store subcluster number          ML30  36
NSUB(MSUB)=1#                          Increment count of sites in      ML30  37
IP=IS#                                 Set the central site i           ML30  38
#                                                                       ML30  39
#   LOOP UNTIL ALL SUBCLUSTER TESTS COMPLETE                            ML30  40
#   ----------------------------------------                            ML30  41
#                                                                       ML30  42
WHILE(IP<STOP2)#                       Loop until tests complete        ML30  43
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML30  44
IB1=NINT(bond1:)#                      Point to start connect bonds     ML30  45
IB2=NINT(bond2:)#                      Point to end connect bonds       ML30  46
IB3=IB2-STEPB#                         Point to last bond               ML30  47
#                                                                       ML30  48
#    TEST BONDLENGTHS OF CONNECTED SITES                                ML30  49
#    -----------------------------------                                ML30  50
#                                                                       ML30  51
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Loop over bond list              ML30  52
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML30  53
IF((peak1:<.5)\(bondl:>BMIN)) NEXT#    Skip if not too close            ML30  54
KP=NINT(peak2:)#                       Point to sitelist                ML30  55
cullsite:(KP)#                         Disable site and bonds           ML30  56
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML30  57
#                                                                       ML30  58
#  TEST BOND ANGLES TO SITES IN SAME SUBCLUSTER                         ML30  59
#  --------------------------------------------                         ML30  60
#                                                                       ML30  61
IF(nbond:>1.1)#                        Test connect bonds exceed 1      ML30  62
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML30  63
M=0#                                   Set flag for central site        ML30  64
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Primary  bond list loop          ML30  65
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML30  66
IF(peak1:<.5) NEXT#                    Skip if bond disabled            ML30  67
LP=NINT(peak2:)#                       Point to site list               ML30  68
IF(NINT(ABS(group:(LP)))!=MSUB) NEXT#  Skip bond if diff subcluster     ML30  69
#                                                                       ML30  70
FOR(JB=IB1;JB<IB2;JB=JB+STEPB)#        Secondary bond list loop         ML30  71
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML30  72
IF(peak1:(JB)<.5) NEXT#                Skip disabled bonds              ML30  73
JP=NINT(peak2:(JB))#                   Point to site list               ML30  74
IF(JP==LP) NEXT#                       Skip if same site                ML30  75
DX=peakx:(JP)-peakx:(LP)#              Get x difference                 ML30  76
DY=peaky:(JP)-peaky:(LP)#              Get y difference                 ML30  77
DZ=peakz:(JP)-peakz:(LP)#              Get z difference                 ML30  78
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML30  79
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML30  80
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML30  81
COSA=(bondl:(JB)**2+bondl:**2-DD)$#    Calculate cosine angle           ML30  82
/(2.*bondl:(JB)*bondl:)#               Calculate cosine angle           ML30  83
IF((COSA>=CMAX)&(COSA<=CMIN)) NEXT#    Skip if angles within limits     ML30  84
IF(NINT(ABS(group:(JP)))!=MSUB)cullsite:(JP)# Disable connect site j    ML30  85
ELSE#                                  This is same subcluster          ML30  86
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML30  87
NSUB(MSUB)=NSUB(MSUB)-1#               Decrement subcluster count       ML30  88
IF(group:(JP)>.5) cullsite:(JP)#       If positive disable j            ML30  89
ELSE#                                  Else disable central site        ML30  90
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML30  91
cullsite:(IP);  M=1#                   Disable i and flag               ML30  92
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML30  93
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML30  94
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML30  95
IF(M==1) GO TO 100#                    Exit if central site disable     ML30  96
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML30  97
#                                                                       ML30  98
#  TEST ANGLES WITH SITES J & L NOT IN SUBCLUSTER                       ML30  99
#  ---------------------------------------------                        ML30 100
#                                                                       ML30 101
FOR(IB=IB1;IB<IB3;IB=IB+STEPB)#        Primary  bondlist loop           ML30 102
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML30 103
IF(peak1:<.5) NEXT#                    Skip disabled bonds              ML30 104
LP=NINT(peak2:)#                       Get sitelist pointer l           ML30 105
IF(NINT(ABS(group:(LP)))==MSUB) NEXT#  Skip bonds to sites in msub      ML30 106
group:(LP)=FLOAT(MSUB)#                Set at current subcluster        ML30 107
NSUB(MSUB)=NSUB(MSUB)+1#               Increment count of sites         ML30 108
#                                                                       ML30 109
FOR(JB=IB1+STEPB;JB<IB2;JB=JB+STEPB)#  Secondary bond list loop         ML30 110
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML30 111
IF(peak1:(JB)<.5) NEXT#                Skip if bond disabled            ML30 112
JP=NINT(peak2:(JB))#                   Get sitelist pointer j           ML30 113
ifne:(group:(JP),0.) NEXT#             Skip if already assigned         ML30 114
DX=peakx:(JP)-peakx:(LP)#              Get x difference                 ML30 115
DY=peaky:(JP)-peaky:(LP)#              Get y difference                 ML30 116
DZ=peakz:(JP)-peakz:(LP)#              Get z difference                 ML30 117
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML30 118
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML30 119
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML30 120
COSA=(bondl:(JB)**2+bondl:**2-DD)$#    Calculate cosine angle           ML30 121
/(2.*bondl:(JB)*bondl:)#               Calculate cosine angle           ML30 122
IF((COSA<CMAX)\(COSA>CMIN)) cullsite:(JP)# Disable site j               ML30 123
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML30 124
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML30 125
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML30 126
#                                                                       ML30 127
#   TEST LAST CONNECTED SITE FOR CLUSTER                                ML30 128
#   ------------------------------------                                ML30 129
#                                                                       ML30 130
IF(peak1:(IB3)>.5)#                    Test if last site disabled       ML30 131
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML30 132
JP=NINT(peak2:(IB3))#                  Get sitelist pointer j           ML30 133
ifeq:(group:(JP),0.)#                  Skip if already assigned         ML30 134
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML30 135
group:(JP)=FLOAT(MSUB)#                Assign site to subcluster        ML30 136
NSUB(MSUB)=NSUB(MSUB)+1#               Increment count of sites         ML30 137
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML30 138
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML30 139
group:=-ABS(group:)#                   Flag site as processed           ML30 140
#                                                                       ML30 141
#  FIND NEXT SITE IN THIS SUBCLUSTER                                    ML30 142
#  ---------------------------------                                    ML30 143
#                                                                       ML30 144
100 CONTINUE#                          Branch point for retry           ML30 145
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over entire site list       ML30 146
IF(NINT(group:)==MSUB) BREAK#          Exit at first match              ML30 147
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML30 148
#                                                                       ML30 149
#   ACCEPT SUBCLUSTER IF MORE THAN ONE SITE                             ML30 150
#   ---------------------------------------                             ML30 151
#                                                                       ML30 152
IF(NSUB(MSUB)>1) NEXT#                 Skip to next site if ok          ML30 153
group:(IS)=0.#                         Cancel subcluster number         ML30 154
MSUB=MSUB-1#                           Decrement subcluster count       ML30 155
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML30 156
#                                                                       ML30 157
RETURN#                                                                 ML30 158
END#                                                                    ML30 159
systemheader:(ML35)#                   Weighted subcluster search       ML35   1
####################################################################### ML35   2
#                                                                       ML35   3
#                                ML35                                   ML35   4
#                                                                       ML35   5
####################################################################### ML35   6
SUBROUTINE ML35#                                                        ML35   7
include:(AACOMN)#                                                       ML35   8
include:(MLCOMN)#                                                       ML35   9
#                                                                       ML35  10
INTEGER IB,JB,IP,JP,KP#                Loop indices                     ML35  11
INTEGER IB1,IB2,IB3#                   Loop limits                      ML35  12
INTEGER M,IS,LP,NB,KB#                                                  ML35  13
REAL COSA,CMAX,CMIN,BMIN#              Cosine & limits                  ML35  14
REAL DD,DX,DY,DZ,DS,DM,CS,CM#          Bond distance & dif xyz          ML35  15
REAL A,BB,WW,WT#                                                        ML35  16
#                                                                       ML35  17
#   PREPARE TO APPLY LIMITS                                             ML35  18
#   -----------------------                                             ML35  19
#                                                                       ML35  20
IF(ANGMAX>3.) CMAX=-2.#                Allow for max of 180 deg         ML35  21
ELSE          CMAX=COS(ANGMAX)#        Find cos of max allowed angl     ML35  22
CMIN=COS(ANGMIN)#                      And min allowed angle            ML35  23
BMIN=SQRT(RADMIN)#                     Get bond minimum                 ML35  24
MSUB=0#                                Zero subcluster limits           ML35  25
DS=(SQRT(RADMAX)-BMIN)/2.;   DM=BMIN+DS# Calc bond span & mean          ML35  26
CS=(CMAX-CMIN)/2.;       CM=CMIN+CS#   Calc cos**2 span & mean          ML35  27
#                                                                       ML35  28
#   LOOP OVER SITELIST FROM START                                       ML35  29
#   -----------------------------                                       ML35  30
#                                                                       ML35  31
FOR(IS=ISTART;IS<STOP2;IS=IS+STEP2)#   Loop over site list              ML35  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML35  33
IF((NINT(clust:(IS))!=NCLUS)$#         If site not in cluster           ML35  34
\  (ABS(group:(IS))>small:)$#          If subcluster unassigned         ML35  35
\  (nbond:(IS)<.5))            NEXT#   Site is not connected            ML35  36
MSUB=MSUB+1#                           Incr subcluster count            ML35  37
group:(IS)=FLOAT(MSUB)#                Store subcluster number          ML35  38
NSUB(MSUB)=1#                          Increment count of sites in      ML35  39
IP=IS#                                 Set the central site i           ML35  40
#                                                                       ML35  41
#   LOOP UNTIL ALL SUBCLUSTER TESTS COMPLETE                            ML35  42
#   ----------------------------------------                            ML35  43
#                                                                       ML35  44
WHILE(IP<STOP2)#                       Loop until tests complete        ML35  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML35  46
IB1=NINT(bond1:)#                      Point to start connect bonds     ML35  47
IB2=NINT(bond2:)#                      Point to end connect bonds       ML35  48
IB3=IB2-STEPB#                         Point to last bond               ML35  49
#                                                                       ML35  50
#    TEST BONDLENGTHS OF CONNECTED SITES                                ML35  51
#    -----------------------------------                                ML35  52
#                                                                       ML35  53
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Loop over bond list              ML35  54
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML35  55
IF((peak1:<.5)\(bondl:>BMIN)) NEXT#    Skip if not too close            ML35  56
KP=NINT(peak2:)#                       Point to sitelist                ML35  57
cullsite:(KP)#                         Disable site and bonds           ML35  58
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML35  59
#                                                                       ML35  60
#  TEST BOND ANGLES TO SITES IN SAME SUBCLUSTER                         ML35  61
#  --------------------------------------------                         ML35  62
#                                                                       ML35  63
IF(nbond:>1.1)#                        Test connect bons exceeds 1      ML35  64
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML35  65
M=0#                                   Set flag for central site        ML35  66
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Primary  bond list loop          ML35  67
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML35  68
IF(peak1:<.5) NEXT#                    Skip if bond disabled            ML35  69
LP=NINT(peak2:)#                       Point to site list               ML35  70
IF(NINT(ABS(group:(LP)))!=MSUB) NEXT#  Skip bond if diff subcluster     ML35  71
#                                                                       ML35  72
FOR(JB=IB1;JB<IB2;JB=JB+STEPB)#        Secondary bond list loop         ML35  73
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML35  74
IF(peak1:(JB)<.5) NEXT#                Skip disabled bonds              ML35  75
JP=NINT(peak2:(JB))#                   Point to site list               ML35  76
IF(JP==LP) NEXT#                       Skip if same site                ML35  77
DX=peakx:(JP)-peakx:(LP)#              Get x difference                 ML35  78
DY=peaky:(JP)-peaky:(LP)#              Get y difference                 ML35  79
DZ=peakz:(JP)-peakz:(LP)#              Get z difference                 ML35  80
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML35  81
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML35  82
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML35  83
COSA=(bondl:(JB)**2+bondl:**2-DD)$#    Calculate cosine angle           ML35  84
/(2.*bondl:(JB)*bondl:)#               Calculate cosine angle           ML35  85
IF((COSA>=CMAX)&(COSA<=CMIN)) NEXT#    Skip if angles within limits     ML35  86
ifeq:(group:(JP),0.) cullsite:(JP)#    Reject connected site            ML35  87
ELSE IF(NINT(ABS(group:(JP)))==MSUB)#  If in same subcluster            ML35  88
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML35  89
NSUB(MSUB)=NSUB(MSUB)-1#               Decrement subcluster count       ML35  90
IF(group:(JP)>.5) cullsite:(JP)#       If positive disable j            ML35  91
ELSE#                                  Else disable central site        ML35  92
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML35  93
cullsite:(IP);  M=1#                   Disable i and flag               ML35  94
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML35  95
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML35  96
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML35  97
IF(M==1) GO TO 100#                    Exit if central site disable     ML35  98
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML35  99
#                                                                       ML35 100
#  TEST ANGLES WITH SITES J & L NOT IN SUBCLUSTER                       ML35 101
#  ---------------------------------------------                        ML35 102
#                                                                       ML35 103
WT=0.;    NB=0#                        Set initial weight & pointer     ML35 104
FOR(KB=IB1;KB<IB2;KB=KB+STEPB)#        Loop over all bonds              ML35 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML35 106
IF(peak1:(KB)<.5) NEXT#                Skip rejected bonds              ML35 107
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Loop over all bonds              ML35 108
IF(peak1:<.5) bondl:=-ABS(bondl:)#     Switch off reject bonds          ML35 109
ELSE          bondl:= ABS(bondl:)#     Switch on  other  bonds          ML35 110
WW=0.;   IB=IB1#                       Reset weight & pointer           ML35 111
LP=NINT(peak2:(KB))#                   Get site 2 address               ML35 112
BB=bondl:(KB)#                         Get first bond                   ML35 113
REPEAT#                                Loop until all lp tested         ML35 114
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML35 115
A=1.-ABS((BB-DM)/DS)#                  Form bond dist weight            ML35 116
WW=WW+A#                               Add distance weight              ML35 117
FOR(JB=IB1;JB<IB2;JB=JB+STEPB)#        Secondary bond list loop         ML35 118
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML35 119
IF(bondl:(JB)<0.) NEXT#                Skip if bond disabled            ML35 120
JP=NINT(peak2:(JB))#                   Get sitelist pointer j           ML35 121
ifne:(group:(JP),0.) NEXT#             Skip if already assigned         ML35 122
DX=peakx:(JP)-peakx:(LP)#              Get x difference                 ML35 123
DY=peaky:(JP)-peaky:(LP)#              Get y difference                 ML35 124
DZ=peakz:(JP)-peakz:(LP)#              Get z difference                 ML35 125
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML35 126
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML35 127
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML35 128
COSA=(bondl:(JB)**2+BB**2-DD)$#        Calculate cosine angle           ML35 129
/(2.*bondl:(JB)*BB)#                   Calculate cosine angle           ML35 130
IF((COSA<CMAX)\(COSA>CMIN)) bondl:(JB)=-bondl:(JB)# Temp reject bond    ML35 131
ELSE WW=WW+1.-ABS((COSA-CM)/CS)#       Sum bond & angle weights         ML35 132
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML35 133
FOR(;IB<IB2;IB=IB+STEPB)#              Loop along bond list             ML35 134
IF(IB!=KB&bondl:>0.) BREAK#            Get next lp site                 ML35 135
IF(IB==IB2) BREAK#                     Exit after last                  ML35 136
LP=NINT(peak2:)#                       Get next lp site                 ML35 137
BB=bondl:#                             Get bond distance                ML35 138
IB=IB+STEPB#                           Reset bond pointer               ML35 139
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML35 140
IF(WW<=WT) NEXT#                       Skip if new wt smaller           ML35 141
WT=WW;   NB=KB#                        Store best connects              ML35 142
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML35 143
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Loop over all bonds              ML35 144
bondl:=ABS(bondl:)#                    Reset bond lengths               ML35 145
#                                                                       ML35 146
IF(NB>0)#                              If new lp site found             ML35 147
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML35 148
IB=IB1#                                Set start lp pointer             ML35 149
LP=NINT(peak2:(NB))#                   Set first lp site                ML35 150
BB=bondl:(NB)#                         Get first bond                   ML35 151
REPEAT#                                Loop for all lp's                ML35 152
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML35 153
ifeq:(group:(LP),0.)#                  Skip assigned sites              ML35 154
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ML35 155
group:(LP)=FLOAT(MSUB)#                Set at current subcluster        ML35 156
NSUB(MSUB)=NSUB(MSUB)+1#               Increment count of sites         ML35 157
FOR(JB=IB1;JB<IB2;JB=JB+STEPB)#        Secondary bond list loop         ML35 158
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ML35 159
IF(JB==NB\peak1:(JB)<.5) NEXT#         Skip if bond disabled            ML35 160
JP=NINT(peak2:(JB))#                   Get sitelist pointer j           ML35 161
ifne:(group:(JP),0.) NEXT#             Skip if already assigned         ML35 162
DX=peakx:(JP)-peakx:(LP)#              Get x difference                 ML35 163
DY=peaky:(JP)-peaky:(LP)#              Get y difference                 ML35 164
DZ=peakz:(JP)-peakz:(LP)#              Get z difference                 ML35 165
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML35 166
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML35 167
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML35 168
COSA=(bondl:(JB)**2+BB**2-DD)$#        Calculate cosine angle           ML35 169
/(2.*bondl:(JB)*BB)#                   Calculate cosine angle           ML35 170
IF((COSA<CMAX)\(COSA>CMIN)) cullsite:(JP)# Disable site j               ML35 171
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ML35 172
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ML35 173
FOR(;IB<IB2;IB=IB+STEPB)#              Loop along bonds                 ML35 174
IF(IB!=NB&peak1:>.5) BREAK#            Exit if next lp                  ML35 175
IF(IB==IB2) BREAK#                     Exit after last lp               ML35 176
LP=NINT(peak2:)#                       Get site address                 ML35 177
BB=bondl:#                             Get bond distance                ML35 178
IB=IB+STEPB#                           Incr bond pointer                ML35 179
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML35 180
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML35 181
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML35 182
#                                                                       ML35 183
#   TEST LAST CONNECTED SITE FOR CLUSTER                                ML35 184
#   ------------------------------------                                ML35 185
#                                                                       ML35 186
IF(peak1:(IB3)>.5)#                    Test if last site disabled       ML35 187
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ML35 188
JP=NINT(peak2:(IB3))#                  Get sitelist pointer j           ML35 189
ifeq:(group:(JP),0.)#                  Skip if already assigeqd         ML35 190
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 ML35 191
group:(JP)=FLOAT(MSUB)#                Assign site to subcluster        ML35 192
NSUB(MSUB)=NSUB(MSUB)+1#               Increment count of sites         ML35 193
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 ML35 194
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ML35 195
group:=-ABS(group:)#                   Flag site as processed           ML35 196
#                                                                       ML35 197
#  FIND NEXT SITE IN THIS SUBCLUSTER                                    ML35 198
#  ---------------------------------                                    ML35 199
#                                                                       ML35 200
100 CONTINUE#                          Branch point for retry           ML35 201
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over entire site list       ML35 202
IF(NINT(group:)==MSUB) BREAK#          Exit at first match              ML35 203
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML35 204
#                                                                       ML35 205
#   ACCEPT SUBCLUSTER IF MORE THAN ONE SITE                             ML35 206
#   ---------------------------------------                             ML35 207
#                                                                       ML35 208
IF(NSUB(MSUB)>1) NEXT#                 Skip to next site if ok          ML35 209
group:(IS)=0.#                         Cancel subcluster number         ML35 210
MSUB=MSUB-1#                           Decrement subcluster count       ML35 211
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML35 212
#                                                                       ML35 213
RETURN#                                                                 ML35 214
END#                                                                    ML35 215
systemheader:(ML37)#                   Calculate figure-of-merit        ML37   1
####################################################################### ML37   2
#                                                                       ML37   3
#                                ML37                                   ML37   4
#                                                                       ML37   5
####################################################################### ML37   6
SUBROUTINE ML37#                                                        ML37   7
include:(AACOMN)#                                                       ML37   8
include:(MLCOMN)#                                                       ML37   9
INTEGER I,IB,IB2,IP,JP,KP#                                              ML37  10
REAL A,B,DD#                                                            ML37  11
#                                                                       ML37  12
#  CALCULATE FIGURE OF MERIT FOR EACH SUBCLUSTER                        ML37  13
#  ---------------------------------------------                        ML37  14
#                                                                       ML37  15
IF(MODW==0)#                           If no subcluster search          ML37  16
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML37  17
NSUB(1)=0;   MSUB=1#                   Set subcluster count             ML37  18
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML37  19
FOMP=0.0;   A=0.0;    B=0.0#           Zero fomp calculation            ML37  20
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over all sites              ML37  21
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML37  22
A=A+peakh:#                            Sum all site heights             ML37  23
IF(NINT(clust:)!=NCLUS) NEXT#          Skip sites not in cluster        ML37  24
IF(MODW==0)#                           If no subcluster search          ML37  25
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML37  26
NSUB(1)=NSUB(1)+1#                     Incr subcluster count            ML37  27
group:=1.#                             Flag site as subcluster 1        ML37  28
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML37  29
IB2=NINT(bond2:)#                      Get end of bond sequence         ML37  30
JP=IP+ipbias:;   KP=JP+MAXBON#         Get sitelist bond bias           ML37  31
FOR(IB=NINT(bond1:);IB<IB2;IB=IB+STEPB)# Loop over bond list            ML37  32
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML37  33
IF(JP==KP) peak1:=-ABS(peak1:)#        Suppress excess sites            ML37  34
IF(peak1:<.5) NEXT#                    Skip disabled sites              ML37  35
JP=JP+1#                               Incr sitelist pointer            ML37  36
QX(JP)=peak2:#                         Store bonded site addr.          ML37  37
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML37  38
nbond:=FLOAT(JP-IP-ipbias:)#           Store num accepted bonds         ML37  39
IF(nbond:<0.5) group:=0.#              Zero group if no bonds           ML37  40
group:=ABS(group:)#                    Activate site subcluster         ML37  41
I=NINT(group:);  IF(I==0) NEXT#        Skip null groups                 ML37  42
DD=FLOAT(NSUB(I))#                     Get number of sites in group     ML37  43
B=B+1.#                                Count active sites in group      ML37  44
IF(DD>2.1) FOMP=FOMP+peakh:*ALOG10(DD)# Sum site height * population    ML37  45
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML37  46
IF(A>0.) FOMP=FOMP/A#                  Normalize figure of merit        ML37  47
IF(B<2.5) FOMP=0.#                     Cancel cluster if less than      ML37  48
#                                                                       ML37  49
DO IP=1,MSUB; MAXGRP=MAX0(NSUB(IP),MAXGRP)# Loop over subclusters       ML37  50
dumpsite:('ML37',100,100,1000,1000)#   End of ml37                      ML37  51
#                                                                       ML37  52
RETURN#                                                                 ML37  53
END#                                                                    ML37  54
systemheader:(ML40)#                   Match input model & sites        ML40   1
####################################################################### ML40   2
#                                                                       ML40   3
#                                ML40                                   ML40   4
#                                                                       ML40   5
####################################################################### ML40   6
SUBROUTINE ML40#                                                        ML40   7
include:(AACOMN)#                                                       ML40   8
include:(MLCOMN)#                                                       ML40   9
#                                                                       ML40  10
INTEGER J,K,IP,JP,IS,IA#               General indices                  ML40  11
INTEGER NPK,NFR,ISUB,MARK#                                              ML40  12
REAL FMAX,PMAX,FSUM,PSUM#                                               ML40  13
REAL MBEST,SITE#                                                        ML40  14
#                                                                       ML40  15
#   ASSIGN MEMORY LIST4,5,6                                             ML40  16
#   -----------------------                                             ML40  17
#                                                                       ML40  18
STEP4=5;  STEPP=8;  STEPF=8#           Set list packet sizes            ML40  19
STOP4=MARK4+STEP4*MAXGRP#              Set end of list 4                ML40  20
MARKP=STOP4#                           Set start of list 5              ML40  21
STOPP=MARKP+STEPP*MAXGRP#              Set end of list 5                ML40  22
MARKF=STOPP#                           Set start of list 6              ML40  23
NATOM=MAX0(10,NATOM)#                  Protest expected sites           ML40  24
MARK=MARK2+NATOM*STEP2#                End of expected atom sites       ML40  25
SITE=peakh:(MARK-STEP2)#               Last site of expected atoms      ML40  26
incrqx:(STOPF,MARKF+STEPF*MAXGRP,ML4001)# Expand qx........ml4001       ML40  27
dumpsite:('ML40',100,100,1000,100)#    Start of ml40                    ML40  28
#                                                                       ML40  29
#   CALC CANON DESCRIPTION FOR SITE CLUSTER                             ML40  30
#   ---------------------------------------                             ML40  31
#                                                                       ML40  32
FOR(ISUB=1;ISUB<=MSUB;ISUB=ISUB+1)#    Loop over all subclusters        ML40  33
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML40  34
BEST1(ISUB)=0#                         Zero seq. no. of best match      ML40  35
NPK=NSUB(ISUB)#                        Get number of sites/group        ML40  36
IF(NPK<5) NEXT#                        Ignore small clusters            ML40  37
IS=MARK4#                              Set start of list 4              ML40  38
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over site list              ML40  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML40  40
IF((NINT(clust:)!=NCLUS)\(group:!=ISUB)) NEXT# Skip if not cluster      ML40  41
IF(IP<MARK) canon:=1.#                 Start canonical descriptor       ML40  42
ELSE        canon:=peakh:/SITE#        Reduce unexpected atom sites     ML40  43
match:=canon:#                         Save canon descript. twice       ML40  44
site:=FLOAT(IP)#                       Store site address               ML40  45
sumc:=0.#                              Zero canon contrib.              ML40  46
bonf:=FLOAT(IP+ipbias:)#               Point to start bond sites        ML40  47
bonl:=bonf:+nbond:#                    Point to end bond sites          ML40  48
ring:=nbond:#                          Set ring flag indicator          ML40  49
IS=IS+STEP4#                           Incr list 4 pointer              ML40  50
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML40  51
STOP4=IS#                              Mark end of site list            ML40  52
CALL ML41(NPK,1)#                      Sort sites on can descr          ML40  53
#                                                                       ML40  54
#   STORE SORTED SITES IN LISTP                                         ML40  55
#   ---------------------------                                         ML40  56
#                                                                       ML40  57
PSUM=0.#                               Zero sum site can desc.          ML40  58
PMAX=FLOAT(MAX0(6,RINGP))#             Set max ring connects            ML40  59
STOPP=MARKP+NPK*STEPP#                 Set current limit of list        ML40  60
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over list 4 sites           ML40  61
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML40  62
IP=NINT(site:)#                        Point to sitelist addr.          ML40  63
ringf:=ring:#                          Store ring flag in sitelist      ML40  64
IF(order:<PMAX) PSUM=PSUM+canon:#      Sum canonical dessum=ptors       ML40  65
IA=NINT(order:)*STEPP+MARKP#           Point to listp                   ML40  66
star:=site:#                           Store site address               ML40  67
frst:=FLOAT(IP+ipbias:)#               Place before start of bonds      ML40  68
last:=frst:+nbond:#                    Pointer to last bond             ML40  69
link:=0.#                              Seq. no. of atom matches         ML40  70
cand:=canon:#                          Canonical description            ML40  71
QX(IA+6)=0.#                           Best atom match                  ML40  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML40  73
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2) match:=0.# Reset match flag          ML40  74
#                                                                       ML40  75
#   CALC CANON DESCR. FOR FRAG SITES FOR COMPARISON WITH SITES          ML40  76
#   ----------------------------------------------------------          ML40  77
#                                                                       ML40  78
MBEST=1;MERIT=1#                       Set merit parameters             ML40  79
JP=MARK3+5-ipbias:#                    Start frag list with offset      ML40  80
FOR(K=1;K<=NFRAG;K=K+1)#               Loop over input fragments        ML40  81
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML40  82
NFR=NINFR(K)#                          Get number of sites              ML40  83
IS=MARK4 ; IP=JP#                      Start site list                  ML40  84
DO J=1,NFR#                            Loop over frag sites             ML40  85
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML40  86
canon:=1.;    match:=1.#               Flag can. desc. in frag list     ML40  87
site:=FLOAT(IP)#                       Store parent list addr           ML40  88
sumc:=0.#                              Zero canonical desc.             ML40  89
bonf:=FLOAT(IP+ipbias:)#               Point to start bond sites        ML40  90
bonl:=bonf:+nbond:#                    Point to end bond sites          ML40  91
ring:=nbond:#                          Store ring flag indicator        ML40  92
IS=IS+STEP4#                           Incr site list pointer           ML40  93
IP=IP+STEP3#                           Incr frag list pointer           ML40  94
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML40  95
STOP4=IS ; JP=IP#                      Mark end of site list            ML40  96
CALL ML41(NFR,2)#                      Sort sites on can descr.         ML40  97
#                                                                       ML40  98
#    SORT FRAG SITES AND STORE IN LISTF                                 ML40  99
#    -----------------------------------                                ML40 100
#                                                                       ML40 101
FSUM=0.#                               Zero sum frag can desc.          ML40 102
FMAX=FLOAT(MAX0(6,RINGF))#             Set max ring connects            ML40 103
STOPF=MARKF+NFR*STEPF#                 Mark end of sort frag list       ML40 104
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over frag sites             ML40 105
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML40 106
IP=NINT(site:)#                        Point to frag list addr          ML40 107
IF(NINT(group:)!=K) NEXT#              Accept only current frag         ML40 108
IF(order:<FMAX) FSUM=FSUM+canon:#      Sum canon flags                  ML40 109
IA=NINT(order:)*STEPF+MARKF#           Point to listf                   ML40 110
star:=site:#                           Store addr of parent listm       ML40 111
frst:=FLOAT(IP+ipbias:)#               Point to start bond sites        ML40 112
last:=frst:+nbond:#                    Pointer to last bond             ML40 113
link:=0.#                              Set match list flag              ML40 114
cand:=canon:#                          Canonical description            ML40 115
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML40 116
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2) match:=0.# Reset match flag          ML40 117
#                                                                       ML40 118
#   MATCH SITE AND FRAG SITES IN LISTS 5 AND 6                          ML40 119
#   ------------------------------------------                          ML40 120
#                                                                       ML40 121
CALL ML42(NPK,NFR,PSUM,FSUM)#          Match site & frag lists          ML40 122
IF(MERIT<=MBEST) NEXT#                 Skip if no better match          ML40 123
MBEST=MERIT+1.#                        Store current best fit           ML40 124
FOR(IA=MARKP;IA<STOPP;IA=IA+STEPP)#    Loop over assign list            ML40 125
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML40 126
IP=NINT(star:)#                        Get site list address            ML40 127
match:=QX(IA+8)#                       Store frag address in site l     ML40 128
QX(IA+8)=0.#                           Reset assign address             ML40 129
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML40 130
BEST1(ISUB)=K#                         Store frag number                ML40 131
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML40 132
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML40 133
#                                                                       ML40 134
RETURN#                                                                 ML40 135
END#                                                                    ML40 136
systemheader:(ML41)#                   Build canonical description      ML41   1
####################################################################### ML41   2
#                                                                       ML41   3
#                                ML41                                   ML41   4
#                                                                       ML41   5
####################################################################### ML41   6
SUBROUTINE ML41(MSITE,KEY)#                                             ML41   7
include:(AACOMN)#                                                       ML41   8
include:(MLCOMN)#                                                       ML41   9
#                                                                       ML41  10
INTEGER I,J,K,M,N#                     General indices                  ML41  11
INTEGER IS,JS,IB,JB,IP#                Loop indices                     ML41  12
INTEGER RING#                          Count sites in ring              ML41  13
INTEGER KEY#                           Flag sites (1) or frag sites     ML41  14
INTEGER MSITE#                         Number of sites in subcl.        ML41  15
INTEGER CHANGE#                        Ring change flag                 ML41  16
REAL CAN(6)#                           Store can. desc. for bonds       ML41  17
REAL CMAX,A#                           Max canonical descriptor         ML41  18
REAL SUMCN#                            Canonical sum                    ML41  19
REAL REL(6)#                           Sort order of site address       ML41  20
#                                                                       ML41  21
#   SET INITAL PARAMETERS                                               ML41  22
#   ----------------------                                              ML41  23
#                                                                       ML41  24
CMAX=0.#                               Set max canonical discriptor     ML41  25
RING=MSITE#                            Set at number of sites           ML41  26
CHANGE=yes:#                           Set initial ring change flag     ML41  27
#                                                                       ML41  28
# CONSIDER THE CONTRIBUTION OF EVERY SITE TO EVERY OTHER SITE (EVEN     ML41  29
# FOR THOSE WHICH ARE  NOT DIRECTLY BONDED). HENCE, TWO NESTED LOOPS    ML41  30
# OVER ALL SITES. THE INNERMOST LOOPS TAKE CARE OF TRANSMITTING THE     ML41  31
# CONTRIBUTIONS VIA THE BONDED ATOMS                                    ML41  32
#                                                                       ML41  33
DO N=1,MSITE#                          Loop over all sites              ML41  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML41  35
SUMCN=0.#                              Reset canon sum                  ML41  36
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Sum canonical contributions      ML41  37
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML41  38
IB=NINT(bonf:); JB=NINT(bonl:)#        Get addr of last bond site       ML41  39
FOR(;IB<JB;IB=IB+1)#                   Loop over bonded sites           ML41  40
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML41  41
IP=NINT(QX(IB+1))#                     Get bonded site address          ML41  42
sumc:=sumc:+canon:#                    Sum site canonical value         ML41  43
SUMCN=SUMCN+canon:#                    Sum total values                 ML41  44
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML41  45
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML41  46
#                                                                       ML41  47
SUMCN=SUMCN/FLOAT((STOP4-MARK4)/STEP4)# Rescale summation               ML41  48
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Store canonical sums             ML41  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML41  50
IF(N==MSITE) CMAX=AMAX1(CMAX,sumc:)#   Get max for normalization        ML41  51
ELSE#                                  Else store sum as canon          ML41  52
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML41  53
IP=NINT(site:)#                        Get site address                 ML41  54
canon:=sumc:*match:/SUMCN#             Store weighted canonical sum     ML41  55
sumc:=0.#                              Zero canonical sum               ML41  56
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML41  57
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML41  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML41  59
#                                                                       ML41  60
#   NORMALIZE CANONICAL DESCRIPTOR                                      ML41  61
#   ------------------------------                                      ML41  62
#                                                                       ML41  63
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over all sites              ML41  64
sumc:=((sumc:/CMAX)**2)*100.+100.#     Normalize canonical descript     ML41  65
#                                                                       ML41  66
#   IDENTIFY SITES IN CLOSED RINGS                                      ML41  67
#   ------------------------------                                      ML41  68
#                                                                       ML41  69
WHILE(CHANGE==yes:)#                   Loop while no change to ring     ML41  70
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML41  71
CHANGE=no:#                            Switch off change flag           ML41  72
FOR(JS=MARK4;JS<STOP4;JS=JS+STEP4)#    Loop over all sites              ML41  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML41  74
IF(NINT(ring:(JS))!=1) NEXT#           Skip if bonds ne 1               ML41  75
RING=RING-1#                           Decrement ring count             ML41  76
A=site:(JS)#                           Store parent list addr           ML41  77
sumc:(JS)=sumc:(JS)-100.#              Reduce canonical descr.          ML41  78
ring:(JS)=0.#                          Switch off site ring flag        ML41  79
#                                                                       ML41  80
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over all sites              ML41  81
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML41  82
IF(NINT(ring:)<1) NEXT#                Skip if bonds lt 2               ML41  83
JB=NINT(bonl:)#                        Get last bonded site addr.       ML41  84
FOR(IB=NINT(bonf:);IB<JB;IB=IB+1)#     Loop over bonded sites           ML41  85
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML41  86
ifne:(A,QX(IB+1)) NEXT#                Skip until sites match           ML41  87
ring:=ring:-1.#                        Reduce connected bonds           ML41  88
GOTO 100#                              Exit once match found            ML41  89
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML41  90
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML41  91
NEXT#                                  Skip if not found                ML41  92
100 CHANGE=yes:#                       Breakpoint for site match        ML41  93
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML41  94
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML41  95
#                                                                       ML41  96
#   SORT SITES IN DECREASING CANONICAL ORDER                            ML41  97
#   ----------------------------------------                            ML41  98
#                                                                       ML41  99
DO N=1,MSITE#                          Loop over all sites              ML41 100
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML41 101
JS=MARK4#                              Set initial value for sort       ML41 102
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over site list              ML41 103
IF(sumc:>sumc:(JS)) JS=IS#             Save addr of largest can. de     ML41 104
IP=NINT(site:(JS))#                    Get addr. to site/frag list      ML41 105
order:=FLOAT(N-1)#                     Store order of sort              ML41 106
canon:=AMAX1(sumc:(JS),1.)#            Store canonical descriptor       ML41 107
sumc:(JS)=0.#                          Reset canonical descr.           ML41 108
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML41 109
#                                                                       ML41 110
#  ORDER BONDED SITES IN DECREASING ORDER FOR EACH SITE                 ML41 111
#  ----------------------------------------------------                 ML41 112
#                                                                       ML41 113
FOR(IS=MARK4;IS<STOP4;IS=IS+STEP4)#    Loop over site list              ML41 114
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML41 115
IF(NINT(ring:)<2) NEXT#                Ignore single bond sites         ML41 116
IB=NINT(bonf:)#                        Get addr. of first bonded si     ML41 117
JB=NINT(bonl:)#                        Get addr. of last bonded sit     ML41 118
#                                                                       ML41 119
FOR(N=IB;N<JB;N=N+1)#                  Loop over bonded sites           ML41 120
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML41 121
IP=NINT(QX(N+1))#                      Point to site/frag list          ML41 122
M=N-IB+1#                              Get array index                  ML41 123
CAN(M)=canon:#                         Store canonical discription      ML41 124
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML41 125
#                                                                       ML41 126
DO I=1,M#                              Loop over bonds                  ML41 127
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML41 128
K=1#                                   Initialize sort index            ML41 129
DO J=1,M#                              Loop over bonds                  ML41 130
IF(CAN(J)>CAN(K)) K=J#                 Save index of largest can.       ML41 131
J=IB+K#                                Point to bond site               ML41 132
REL(I)=QX(J)#                          Store site address               ML41 133
CAN(K)=0.#                             Reset canonical value            ML41 134
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML41 135
movereal:(REL,1,QX,IB+1,M,0)#          Move in reordered bonds          ML41 136
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML41 137
IF(KEY==1) RINGP=RING#                 Store sites in ring              ML41 138
ELSE       RINGF=RING#                 Store sites in ring              ML41 139
#                                                                       ML41 140
RETURN#                                                                 ML41 141
END#                                                                    ML41 142
systemheader:(ML42)#                   Match sites and input sites      ML42   1
####################################################################### ML42   2
#                                                                       ML42   3
#                                ML42                                   ML42   4
#                                                                       ML42   5
####################################################################### ML42   6
SUBROUTINE ML42(IPK,IFR,PSUM,FSUM)#                                     ML42   7
include:(AACOMN)#                                                       ML42   8
include:(MLCOMN)#                                                       ML42   9
#                                                                       ML42  10
INTEGER IFR,IPK#    ## Number of atoms, sites under consideration       ML42  11
INTEGER LOOP#       ## Number of times a match was attempted            ML42  12
INTEGER MFIT#       ## Max number of atoms & sites to be matched        ML42  13
INTEGER NASG#       ## No. of assigned atoms before redoing list        ML42  14
INTEGER IA,JA,LA,MA,NA#                                                 ML42  15
INTEGER IB,JB,KB,NB#                                                    ML42  16
INTEGER IP,LP,NP,KP#                                                    ML42  17
INTEGER LB1,LB2,KB1,KB2,MB1,MB2#                                        ML42  18
INTEGER RESET,BACKUP,LX,NFIT#                                           ML42  19
REAL SUMCD#         ## Sum of cd's of atoms already matched             ML42  20
REAL PSUM,FSUM#     ## Site and frag site canon descr.                  ML42  21
intdata:(JP,0);  intdata:(MP,0);  intdata:(KA,0)#                       ML42  22
datastuff:#                                                             ML42  23
#                                                                       ML42  24
#    PREPARE TO MATCH SITES                                             ML42  25
#    ----------------------                                             ML42  26
#                                                                       ML42  27
MERIT=0.#                              Initialize sum can descr         ML42  28
MFIT=MIN0(IFR,IPK)#                    Get max sites to be fitted       ML42  29
LX=MARKF;   LA=LX#                     Start listf pointer              ML42  30
RESET=yes:;   BACKUP=no:#              Force start of search            ML42  31
NASG=0#                                Zero matched sites               ML42  32
dumpsite:('ML42',100,100,100,100)#     Start of ml42                    ML42  33
#                                                                       ML42  34
#   LOOP UNTIL MATCH FOUND OR FAILS                                     ML42  35
#   -------------------------------                                     ML42  36
#                                                                       ML42  37
DO LOOP=1,10000#                       Loop for a long time             ML42  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML42  39
IF(RESET==yes:)#                       Test if search restarted         ML42  40
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML42  41
BACKUP=no:;   RESET=no:#               Start in forward search mode     ML42  42
NFIT=1#                                Reset number of sites matche     ML42  43
link:(LX   )=star:(MARKP)#             Store match for top site         ML42  44
link:(MARKP)=star:(LX   )#             Store match for top site         ML42  45
QX(LX+6)=frst:(MARKP)#                 Store bond pointer for y         ML42  46
QX(LX+7)=frst:(LX   )#                 Store bond pointer for x         ML42  47
LX=LX+STEPF#                           Incr starting site               ML42  48
IF(LX==STOPF) GO TO 100#               Search is finished               ML42  49
LA=LX#                                 Start listf pointer              ML42  50
LP=NINT(star:(LA))#                    Get parent list pointer          ML42  51
SUMCD=canon:(LP)#                      Get site l canon descr.          ML42  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML42  53
#                                                                       ML42  54
#   SET PARAMETERS FOR SITE 'L' IN LISTF                                ML42  55
#   ------------------------------------                                ML42  56
#                                                                       ML42  57
LB1=NINT(frst:(LA))#                   Get first bonded site            ML42  58
LB2=NINT(last:(LA))#                   Get last  bonded site            ML42  59
LP= NINT(star:(LA))#                   Get parent list pointer          ML42  60
JB=LB1#                                Set initial 'j' site             ML42  61
IF(BACKUP==yes:) JB=NINT(QX(LA+7))#    If search in backup mode         ML42  62
#                                                                       ML42  63
#   LOOP OVER 'J' SITES BONDED TO 'L' IN LISTF                          ML42  64
#   --------------------------------------------                        ML42  65
#                                                                       ML42  66
FOR(;JB<LB2;JB=JB+1)#                  Loop over bonded sites           ML42  67
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML42  68
JP=NINT(QX(JB+1))#                     Point to parent listf of 'j'     ML42  69
JA=NINT(order:(JP))*STEPF+MARKF#       Point to sorted listf of 'j'     ML42  70
MP=NINT(link:(JA))#                    Point to parent listp of 'm'     ML42  71
IF(MP==0) NEXT#                        Skip if 'j' not matched          ML42  72
#                                                                       ML42  73
#   SITE 'J' IS MATCHED TO SITE 'M' IN LISTP                            ML42  74
#   -----------------------------------------                           ML42  75
#                                                                       ML42  76
MA=NINT(order:(MP))*STEPP+MARKP#       Point to sorted listp of 'm'     ML42  77
MB1=NINT(frst:(MA))#                   Start of sites bonded to 'm'     ML42  78
MB2=NINT(last:(MA))#                   End   of sites bonded to 'm'     ML42  79
KB=MB1#                                Set initial 'k' site             ML42  80
IF(BACKUP==yes:)#                      Test if in backup mode           ML42  81
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML42  82
BACKUP=no:; KB=NINT(QX(LA+6))+1#       Reset backup & restart           ML42  83
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML42  84
#                                                                       ML42  85
#   LOOP OVER 'K' SITES BONDED TO 'M' IN LISTP                          ML42  86
#   -------------------------------------------                         ML42  87
#                                                                       ML42  88
FOR(;KB<MB2;KB=KB+1)#                  Loop over sites bonded m         ML42  89
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML42  90
KP=NINT(QX(KB+1))#                     Point to parent listp of 'k'     ML42  91
KA=NINT(order:(KP))*STEPP+MARKP#       Point to sorted listp of 'k'     ML42  92
IF(link:(KA)>.5) NEXT#                 Skip if already matched          ML42  93
KB1=NINT(frst:(KA))#                   Start of sites bonded to 'k'     ML42  94
KB2=NINT(last:(KA))#                   End   of sites bonded to 'k'     ML42  95
#                                                                       ML42  96
#   SITE 'L' IN LISTF SHOULD BE EQUIVALENT TO SITE 'K' IN LISTP         ML42  97
#   --------------------------------------------------------------      ML42  98
#                     N1   <->    I1                                    ML42  99
#  TRY TO MATCH      /              \                                   ML42 100
#  SITES BONDED     L-N2   <->    I2-K                                  ML42 101
#  TO 'K' AND 'L'    \              /                                   ML42 102
#                     N3   <->    I3                                    ML42 103
#                                                                       ML42 104
FOR(NB=LB1;NB<LB2;NB=NB+1)#            Loop over sites bonded 'l'       ML42 105
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML42 106
NP=NINT(QX(NB+1))#                     Point to parent listf of 'n'     ML42 107
NA=NINT(order:(NP))*STEPF+MARKF#       Point to sorted listf of 'n'     ML42 108
IF(link:(NA)<.5) NEXT#                 Skip if unmatched                ML42 109
FOR(IB=KB1;IB<KB2;IB=IB+1)#            Loop over sites bonded 'k'       ML42 110
IF(ABS(link:(NA)-QX(IB+1))<.5) BREAK#  Exit sites match                 ML42 111
IF(IB==KB2) BREAK#                     Exit if match not found          ML42 112
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML42 113
IF(NB==LB2) BREAK#                     Exit if all sites match          ML42 114
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML42 115
IF(KB<MB2) BREAK#                      Exit if match is found           ML42 116
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML42 117
#                                                                       ML42 118
#   IF SITE 'L' NOT MATCHED RESET PARAMETERS                            ML42 119
#   ----------------------------------------                            ML42 120
#                                                                       ML42 121
IF((JB==LB2)&(link:(LA)>.5))#          Test if site 'l' well matche     ML42 122
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML42 123
NFIT=NFIT-1#                           Decr sites fitted                ML42 124
KP=NINT(link:(LA))#                    Point to parent listp            ML42 125
KA=NINT(order:(KP))*STEPP+MARKP#       Point to sorted listp            ML42 126
link:(KA)=0.#                          Reset link to listf              ML42 127
link:(LA)=0.#                          Reset link to listp              ML42 128
SUMCD=SUMCD-canon:(LP)#                Reduce canon description         ML42 129
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML42 130
#                                                                       ML42 131
#    HAS A NEW MATCH BEEN FOUND                                         ML42 132
#    --------------------------                                         ML42 133
#                                                                       ML42 134
IF(JB<LB2)#                            Test if all sites match          ML42 135
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML42 136
IF(link:(LA)<.5)#                      Test if new site matched         ML42 137
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML42 138
NFIT=NFIT+1#                           Count new match                  ML42 139
SUMCD=SUMCD+canon:(LP)#                Incr total canon description     ML42 140
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML42 141
link:(LA)=FLOAT(KP)#                   Store addr of parent 'k'         ML42 142
link:(KA)=FLOAT(LP)#                   Store addr of parent 'l'         ML42 143
QX(LA+6)=FLOAT(KB)#                    Store pointer to bond sites      ML42 144
QX(LA+7)=FLOAT(JB)#                    Store pointer to bond sites      ML42 145
#                                                                       ML42 146
#   TEST IF CURRENT CANON DESCR BEST YET                                ML42 147
#   ------------------------------------                                ML42 148
#                                                                       ML42 149
IF(SUMCD>MERIT)#                       Test sum can descr > max         ML42 150
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML42 151
MERIT=SUMCD+1.#                        Store new can descr max          ML42 152
FOR(IA=MARKP;IA<STOPP;IA=IA+STEPP)#    Loop over sorted site list       ML42 153
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML42 154
IF(PSUM>FSUM) QX(IA+8)=link:#          Store parent if site list re     ML42 155
ELSE          QX(IA+8)=0.#             Reset if frag is ref list        ML42 156
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML42 157
IF(PSUM<FSUM)#                         Test if frag list is ref         ML42 158
FOR(IA=MARKF;IA<STOPF;IA=IA+STEPF)#    Loop over sorted frag list       ML42 159
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML42 160
IF(link:<.5) NEXT#                     Skip unassigned sites            ML42 161
IP=NINT(link:)#                        Point to parent frag list        ML42 162
JA=NINT(order:)*STEPP+MARKP#           Point to sorted list of matc     ML42 163
QX(JA+8)=star:#                        Store parent site list point     ML42 164
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML42 165
IF(NFIT>=MFIT) GO TO 100#              Finish if max match found        ML42 166
LA=MARKF-STEPF#                        Reset start of next search       ML42 167
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML42 168
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML42 169
#                                                                       ML42 170
#   FIND NEXT UNMATCHED SITE                                            ML42 171
#   ------------------------                                            ML42 172
#                                                                       ML42 173
FOR(LA=LA+STEPF;LA<STOPF;LA=LA+STEPF)# Loop over sorted list            ML42 174
IF(link:(LA)<.5) BREAK#                Exit if unmatched site found     ML42 175
IF((LA==STOPF)\(cand:(LA)+SUMCD<MERIT))# Test if site suitable          ML42 176
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML42 177
IF(NFIT>NASG)#                         Test if matched sites > assi     ML42 178
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ML42 179
NASG=NFIT#                             Store new max fitted             ML42 180
FOR(LA=MARKF;LA<STOPF;LA=LA+STEPF)#    Scan sorted list from start      ML42 181
IF(link:(LA)<.5) BREAK#                Exit at unmatched site           ML42 182
IF(LA==STOPF) BACKUP=yes:#             No unmatched sites so back s     ML42 183
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ML42 184
ELSE         BACKUP=yes:#              Fits < prev max so back scan     ML42 185
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML42 186
#                                                                       ML42 187
#   TEST PREVIOUS MATCHES SHOULD BE REASSIGNED                          ML42 188
#   -------------------------------------------                         ML42 189
IF(BACKUP==yes:)#                      Test site 'l' moved back         ML42 190
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ML42 191
FOR(LA=LA-STEPF;LA>=MARKF;LA=LA-STEPF)# Reverse loop over sort list     ML42 192
IF(link:(LA)>.5) BREAK#                Exit at first matched site       ML42 193
IF((LA<MARKF)\(NFIT==0))#              Pushed back to far- restart      ML42 194
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ML42 195
RESET=yes:;  NEXT#                     Start search with new order      ML42 196
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ML42 197
KP=NINT(link:(LA))#                    Point to parent list of 'k'      ML42 198
KA=NINT(order:(KP))*STEPP+MARKP#       Point to sorted list of 'k'      ML42 199
link:(KA)=0.#                          Reset matched pointer            ML42 200
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ML42 201
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML42 202
100 CONTINUE#                          Exit branch point                ML42 203
#                                                                       ML42 204
RETURN#                                                                 ML42 205
END#                                                                    ML42 206
systemheader:(ML50)#                   List cluster & group values      ML50   1
####################################################################### ML50   2
#                                                                       ML50   3
#                                ML50                                   ML50   4
#                                                                       ML50   5
####################################################################### ML50   6
SUBROUTINE ML50#                                                        ML50   7
include:(AACOMN)#                                                       ML50   8
include:(MLCOMN)#                                                       ML50   9
#                                                                       ML50  10
INTEGER I,J,K,L,N#                     General indices                  ML50  11
INTEGER IP,JP#                         Loop indices                     ML50  12
REAL A,B#                                                               ML50  13
chardata:(ML401," Interpretation   of CLUSTER   ")#                     ML50  14
chardata:(ML402," Number of Subclusters         ")#                     ML50  15
chardata:(ML403," Atom/Site  SUBCL   Connected Sites (- if sym$#        ML50  16
m site)     Assigned  Group")#                                          ML50  17
chardata:(ML404," FOM based on connects        ")#                      ML50  18
chardata:(ML405,*-)#                                                    ML50  19
chardata:(ML406," FOM based on fragment         ")#                     ML50  20
realdata:(FM,[90313.,150213.,220403.,260403.,300403.,340403.,#          ML50  21
380403.,420403.,460403.,500403.,540403.,580403.,620403.,660403.])#      ML50  22
datastuff:#                                                             ML50  23
#                                                                       ML50  24
#   CALCULATE FRAGMENT FOM                                              ML50  25
#   ----------------------                                              ML50  26
#                                                                       ML50  27
FOMF=0.;  A=0.#                        Reset fragment fom               ML50  28
IF(NFRAG>0)#                           Test if frag data input          ML50  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML50  30
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over site list              ML50  31
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML50  32
A=A+peakh:#                            Sum all site heights             ML50  33
IF(match:<.5) NEXT#                    Skip if no match with site       ML50  34
I=NINT(group:);   IF(I==0) NEXT#       Get subcluster number            ML50  35
B=FLOAT(NSUB(I))#                      Get sites in subclust. group     ML50  36
FOMF=FOMF+2.*peakh:*ALOG10(B)#         Sum fom (2*site*subcl)           ML50  37
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML50  38
FOMF=FOMF/(A+1.)#                      Normalize fom                    ML50  39
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML50  40
#                                                                       ML50  41
#  TEST SITE & FRAG FOM VALUES                                          ML50  42
#  ---------------------------                                          ML50  43
#                                                                       ML50  44
IF((FOMP>TOPFOM)\(FOMF>TOPFOM))#       Test if improved interpret t     ML50  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML50  46
TOPFOM=AMAX1(FOMP,FOMF)#               Update top fom value sofar       ML50  47
#                                                                       ML50  48
LINRM=10#                              Force new cluster header         ML50  49
movechar:(ML405,2,CHROT,2,NML401-1,2)# Underline cluster header         ML50  50
ncodefld:(FLOAT(NTERP),1,ML401,170213.,1)# Format interpretation        ML50  51
ncodefld:(FLOAT(NCLUS),1,ML401,310313.,1)# Format cluster number        ML50  52
writeline:(1,ML401,NML401,1,3)#        Print cluster header             ML50  53
ncodefld:(FOMP,1,ML404,280622.,1)#     Format figure of merit           ML50  54
writeline:(0,ML404,NML404,3,3)#        Print figure of merit            ML50  55
ncodefld:(FOMF,1,ML406,280622.,1)#     Format figure of merit           ML50  56
IF(NFRAG>0)writeline:(0,ML406,NML406,3,3)# Print figure of merit        ML50  57
ncodefld:(FLOAT(MSUB),1,ML402,280413.,1)# Format number subclusters     ML50  58
writeline:(0,ML402,NML402,2,3)#        Print number of subclusters      ML50  59
#                                                                       ML50  60
LINRM=4; L=MAXBON+2;   N=0#            Set max list items               ML50  61
FOR(IP=MARK2;IP<STOPI;IP=IP+STEP2)#    Loop over site list              ML50  62
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML50  63
N=N+1#                                 Count site number                ML50  64
IF(NINT(clust:)!=NCLUS) NEXT#          Accept only current clusters     ML50  65
BUFOT(1)=FLOAT(N)#                     Store subcluster group numbe     ML50  66
BUFOT(2)=group:#                       Store subcluster group numbe     ML50  67
DO I=1,MAXBON#                         Loop over possible connects      ML50  68
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML50  69
J=IP+I+ipbias:;  K=NINT(QX(J))#        Get connect site address         ML50  70
IF(K>=STOPI) BUFOT(I+2)=-numbr:(K)#    Store symm generated site        ML50  71
ELSE IF(K>0) BUFOT(I+2)=numbr:(K)#     Store site seq. number           ML50  72
ELSE         BUFOT(I+2)=0.#            Else nullify                     ML50  73
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML50  74
ncodefld:(BUFOT,1,CHROT,FM,L)#         Format site information          ML50  75
movertoc:(aname:,1,CHROT,2,4,0)#       Store atom name                  ML50  76
IF(CHROT(2^2)==' ') CHROT(2^5)=' '#    Blank out site number            ML50  77
IF(match:>.5)#                         Test if matched to fragment      ML50  78
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML50  79
JP=NINT(match:);  J=JP+ipbias:+1+MAXBON# Point to frag site & name      ML50  80
movertoc:(QX(J),1,CHROT,60,6,0)#       Store site name in buff          ML50  81
ncodefld:(group:(JP),1,CHROT,700513.,1)# Format frag number             ML50  82
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML50  83
writeline:(0,ML403,NML403,1,3)#        Print site information           ML50  84
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML50  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML50  86
ELSE ISTART=0#                         Mark as bad interpretation       ML50  87
#                                                                       ML50  88
RETURN#                                                                 ML50  89
END#                                                                    ML50  90
systemheader:(ML60)#                   Plot projections of cluster      ML60   1
####################################################################### ML60   2
#                                                                       ML60   3
#                                ML60                                   ML60   4
#                                                                       ML60   5
####################################################################### ML60   6
SUBROUTINE ML60#                                                        ML60   7
include:(AACOMN)#                                                       ML60   8
include:(MLCOMN)#                                                       ML60   9
#                                                                       ML60  10
INTEGER I,J,K,L,M,N#                   General indices                  ML60  11
INTEGER IXACRS#                        Is x across for first projec     ML60  12
INTEGER LDOWN#                         Lines to skip down page          ML60  13
INTEGER M4,M3,IP,JK,JG,JS#                                              ML60  14
INTEGER MAXPL,STEP,ICODE#                                               ML60  15
INTEGER MJ,MK,MMM#                                                      ML60  16
INTEGER NACROS#                        Loop index                       ML60  17
INTEGER NNCODE#                        No. of sites encoded             ML60  18
INTEGER NPLOT#                         No. of projections plotted o     ML60  19
INTEGER IX,IY#                         Index for x and y axes on pr     ML60  20
REAL SETIN#                            Starting column for print li     ML60  21
REAL T(7)#                             Temp array for sorting           ML60  22
INTEGER LDONE#                         No. of lines done on current     ML60  23
INTEGER LSKIP#                         How far down page to next pe     ML60  24
REAL COL#                              Format for output line           ML60  25
REAL DSCALE#                           Scale in lines per inch          ML60  26
REAL FILLED(12)#                       Allow 12 sites on one line       ML60  27
REAL SUM(3)#                           Sum of x,y, and z coordinate     ML60  28
REAL CMIN(3),CMAX(3)#                  Minimum and maximum x,y,z co     ML60  29
REAL ANUM#                             Number of atoms in ls plane      ML60  30
REAL WIDTH#                            Width of printer page in col     ML60  31
REAL SCMAX#                            Max. possible scale              ML60  32
REAL SCL#                              Scale to be reported to user     ML60  33
#                                                                       ML60  34
string:(TEMP,4)#                                                        ML60  35
chardata:(MLA1,"plane projection of site cluster     SCALE=     $#      ML60  36
cm/A  CODES= *ring  +bond  .lone ")#                                    ML60  37
chardata:(MLA2,"Least squares   Orthogonal most squares")#              ML60  38
chardata:(MLA3,"all Sites    ")#                                        ML60  39
chardata:(MLA4,'*+.()')#                                                ML60  40
realdata:(SCALE1,7.87042);  realdata:(SCALE2,9.84252)#                  ML60  41
datastuff:#                                                             ML60  42
#                                                                       ML60  43
#      PLOT CLUSTER (PLUS ALL SITES ON FIRST PASS)                      ML60  44
#      -------------------------------------------                      ML60  45
#                                                                       ML60  46
FOR(MMM=1;MMM<3;MMM=MMM+1)#            Provide for full site plot &     ML60  47
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML60  48
IF(ISTART==0&MMM==1) NEXT#             Skip poor cluster plot           ML60  49
IF((MMM==2)&(NCLUS>1\NTERP>1)) NEXT#   Plot all atoms only once         ML60  50
#                                                                       ML60  51
#    INITIALIZE SUMMATION VECTORS & MATRICES                            ML60  52
#    ---------------------------------------                            ML60  53
#                                                                       ML60  54
DO J=1,3#                              Some initializations             ML60  55
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML60  56
SUM(J)=0.0#                            Zero coordinate sums             ML60  57
CMAX(J)=-10000.#                       Make max.coord. very large,      ML60  58
CMIN(J)= 10000.#                       Make min.coord. very large,      ML60  59
DO I=1,3#                              Loop over coordinates            ML60  60
BMAT(I,J)=0.0#                         Zero least squares b matrix      ML60  61
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML60  62
ANUM=0.#                               Zero count of contrib. sites     ML60  63
#                                                                       ML60  64
#---------------------------------------------------------------------  ML60  65
# APPLY THE TRANSFORMATION TO THE ORTHOGONAL SYSTEM TO ALL ATOMS IN THE ML60  66
# CLUSTER UNDER CONSIDERATION.  CONCURRENTLY, BUILD UP THE SUMS OF THE  ML60  67
# X,Y AND Z COORDINATES FOR LEAST SQUARES, AND BUILD THE B MATRIX TO    ML60  68
#           SUM(X**2)       SUM(X*Y)       SUM(X*Z)                     ML60  69
#           SUM(X*Y)        SUM(Y**2)      SUM(Y*Z)                     ML60  70
#           SUM(X*Z)        SUM(Y*Z)       SUM(Z**2)                    ML60  71
#---------------------------------------------------------------------  ML60  72
STEP4=7;  M4=MARK4#                    Set up transformed coord lis     ML60  73
FOR(IP=MARK2;IP<STOP2;IP=IP+STEP2)#    Loop over site list              ML60  74
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML60  75
IF((MMM==1)&                          $# For cluster plot               ML60  76
((NINT(clust:)!=NCLUS)\(nbond:<.5)))NEXT# Skip if not current           ML60  77
incrqx:(I,M4+STEP4,ML6001)#            Get more storage....ml6001.      ML60  78
QX(M4+1)=peakx:*T1#                    Transform  x                     ML60  79
QX(M4+2)=peakx:*T2+peaky:*T5#          Transform  y                     ML60  80
QX(M4+3)=peakx:*T3+peaky:*T6+peakz:*T9# Transform z                     ML60  81
QX(M4+4)=FLOAT(IP)#                    Store site list address          ML60  82
IF(RINGP<5 \ ringf:>.5)#               Include only sites in rings      ML60  83
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML60  84
ANUM=ANUM+1.#                          Increment count of contribut     ML60  85
DO J=1,3#                              Add contribution to sums,b       ML60  86
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML60  87
MJ=M4+J#                               Point to coordinates             ML60  88
SUM(J)=SUM(J)+QX(MJ)#                  Accumulate ls sums               ML60  89
DO K=1,3#                              Loop over coordinates            ML60  90
BMAT(J,K)=BMAT(J,K)+QX(M4+K)*QX(MJ)#   Accumulate cross-products        ML60  91
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML60  92
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML60  93
M4=I#                                  Incr storage list                ML60  94
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML60  95
STOP4=M4#                              Mark end of sites to transfo     ML60  96
#                                                                       ML60  97
#---------------------------------------------------------------------  ML60  98
# NOW ADJUST THE ELEMENTS OF B SO THEY ARE THE COEFFICIENTS IN EQUATION ML60  99
# TO FIND THE LS PLANE. THE EQUATION OF THE PLANE IS A*X+B*Y+C*Z+D=0    ML60 100
# THE REQUIREMENT THAT THE SUM OF THE SQUARES OF THE DIFFERENCES BE     ML60 101
# MINIMIZED GIVES FOUR EQUATIONS IN FOUR UNKNOWNS (A,B,C,D). BY ELIMIN- ML60 102
# ATING D, ONE OBTAINS THE 3X3 B MATRIX WITH ELEMENTS                   ML60 103
#                                                                       ML60 104
# SUM(X**2)-(SUM(X)**2)/N SUM(X*Y)-(SUM(X)*SUM(Y))/N SUM(X*Z)-(SUM(X)*  ML60 105
#                                                              Sum(z))/ ML60 106
# SUM(X*Y)-(SUM(X)*SUM(Y))/N  SUM(Y**2)-(SUM(Y)**2)/N SUM(Y*Z)-(SUM(Y)* ML60 107
#                                                              Sum(z))/ ML60 108
# SUM(X*Z)-(SUM(X)*SUM(Z))/N SUM(Y*Z)-(SUM(Y)*SUM(Z))/N SUM(Z**2)-(SUM  ML60 109
#                                                              (z)**2)/ ML60 110
# WHERE N IS THE NUMBER OF SITES IN THE CLUSTER                         ML60 111
#                                                                       ML60 112
# DIAGONALIZE B AND RETURN V (THE TRANSFORMATION MATRIX TO LEAST SQUARE ML60 113
# COORDINATE SYSTEM) AND INDICES FOR THE DIRECTIONS OF THREE POSSIBLE   ML60 114
# PROJECTIONS                                                           ML60 115
#---------------------------------------------------------------------- ML60 116
#                                                                       ML60 117
IF(ANUM>0.5)#                          If there ls sites                ML60 118
DO J=1,3; DO K=1,3#                    Loop over coordinates            ML60 119
BMAT(J,K)=BMAT(J,K)-SUM(J)*SUM(K)/ANUM# Accumulate ls sums for ml61     ML60 120
#                                                                       ML60 121
CALL ML61#                             Diag bmat & return vectors &     ML60 122
#                                                                       ML60 123
# CONVERT COORDINATES TO AXES GIVEN BY EIGENVECTORS                     ML60 124
# -------------------------------------------------                     ML60 125
#                                                                       ML60 126
FOR(I=MARK4;I<STOP4;I=I+STEP4)#        Loop over transformed coords     ML60 127
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML60 128
DO J=1,3#                              Loop over x,y,z                  ML60 129
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML60 130
L=IND(J);    MJ=I+J+4#                 Point to new coordinates         ML60 131
QX(MJ)=QX(I+1)*VECT(1,L)+QX(I+2)*VECT(2,L)+QX(I+3)*VECT(3,L)#           ML60 132
CMAX(J)=AMAX1(CMAX(J),QX(MJ))#         Save max new coordinates         ML60 133
CMIN(J)=AMIN1(CMIN(J),QX(MJ))#         Save min new coordinates         ML60 134
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML60 135
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML60 136
#                                                                       ML60 137
#    SET PROJECTION AXES                                                ML60 138
#    -------------------                                                ML60 139
#                                                                       ML60 140
NPLOT=0#                               Zero count of projections pl     ML60 141
IX=2;  IY=1#                           X across ; y down                ML60 142
IXACRS=yes:#                           Signal x to go across page       ML60 143
WIDTH=prtcmax:-6#                      Get max plot width               ML60 144
IF((WIDTH/(CMAX(1)-CMIN(1)))<SCALE1)#  Test if x fits across page       ML60 145
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML60 146
IX=1;  IY=2#                           X down;  y across                ML60 147
IXACRS=no:#                            X does not go across page        ML60 148
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML60 149
SCMAX=AMAX1(CMAX(IY)-CMIN(IY),CMAX(IY+1)-CMIN(IY+1))# Find max span     ML60 150
#                                                                       ML60 151
#    SET THE PLOTTING SCALE                                             ML60 152
#    ----------------------                                             ML60 153
#                                                                       ML60 154
SCMAX=AMIN1((WIDTH/SCMAX),SCALE2)#     Scale points per angstrom        ML60 155
SCL=SCMAX*2.54/FLOAT(prtcpin:)#        Scale angstroms per cm           ML60 156
MAXPL=NPROJ#                           Pick up no. of projections t     ML60 157
IF(MAXPL==0)#                          Default-check for sphere or      ML60 158
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML60 159
I=IND(3);   J=IND(2)#                  Move pointers for axes           ML60 160
IF(BMAT(J,J)>(2.*BMAT(I,I))) MAXPL=1#  Not spherical/cylindrical        ML60 161
ELSE                         MAXPL=2#  Do another view                  ML60 162
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML60 163
#                                                                       ML60 164
#                                                                       ML60 165
#     LOOP OVER PROJECTIONS                                             ML60 166
#     =====================                                             ML60 167
#                                                                       ML60 168
FOR(NPLOT=1;NPLOT<=MAXPL;NPLOT=NPLOT+1)# Loop over projections          ML60 169
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML60 170
#                                                                       ML60 171
#   SORT SITE COORDS ALONG PROJECTION                                   ML60 172
#   ---------------------------------                                   ML60 173
#                                                                       ML60 174
N=(STOP4-MARK4)/STEP4#                 Number in coord list             ML60 175
FOR(N=N/2;N>0;N=N/2)#                  Loop over halved blocks          ML60 176
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML60 177
STEP=STEP4*N#                          Get sort increment               ML60 178
FOR(I=MARK4+STEP+4;I<STOP4;I=I+STEP4)# Forward loop over coord list     ML60 179
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML60 180
FOR(J=I-STEP;J>MARK4;J=J-STEP)#        Back loop over coord list        ML60 181
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML60 182
JK=J+IX#                               Set proj coord offset            ML60 183
JG=JK+STEP#                            Set forward pointer              ML60 184
IF(QX(JK)>=QX(JG)) BREAK#              Already in order                 ML60 185
JS=J-3#                                Adjust pointer to start of t     ML60 186
movereal:(QX,JS,T,1,STEP4,0)#          Interchange out-of-order ele     ML60 187
JG=JS+STEP#                            Make jg point to beginning o     ML60 188
movereal:(QX,JG,QX,JS,STEP4,1)#        Exchange elements                ML60 189
movereal:(T,1,QX,JG,STEP4,0)#          Exchange elements                ML60 190
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML60 191
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML60 192
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML60 193
#                                                                       ML60 194
#    PRINT PROJECTION HEADING                                           ML60 195
#    ------------------------                                           ML60 196
#                                                                       ML60 197
LINCT=0#                               Start at the top of a page       ML60 198
movechar:(MLA1,1,CHROT,16,NMLA1,0)#    Pick up heading                  ML60 199
movechar:(MLA2,NPLOT*13-12,CHROT,2,13,0)# Add projection type           ML60 200
IF(MMM==2) movechar:(MLA3,1,CHROT,36,12,0)# Overstore header            ML60 201
ELSE ncodefld:(FLOAT(NCLUS),1,CHROT,490213.,1)# Format cluster numbe    ML60 202
ncodefld:(SCL,1,CHROT,620522.,1)#      Format plot scale                ML60 203
writeline:(0,0,0,1,3)#                 Print header                     ML60 204
LINCT=1000#                            Stop interpage title & ff        ML60 205
writeline:(2,0,0,1,3)#                 Space down a few lines           ML60 206
#                                                                       ML60 207
#    PRINT PROJECTED SITES  (X-DOWN Y-ACROSS)                           ML60 208
#    ----------------------------------------                           ML60 209
#                                                                       ML60 210
LDONE=0#                               Zero count of lines done         ML60 211
SETIN=3.#                              Set left page margin             ML60 212
DSCALE=SCMAX*FLOAT(prtlpin:)/FLOAT(prtcpin:)# Scale in lines/angs       ML60 213
LDOWN=0#                               Zero no. of skipped lines        ML60 214
NNCODE=0#                              Zero count of encold fields      ML60 215
#                                                                       ML60 216
FOR(M4=MARK4+4;M4<STOP4;M4=M4+STEP4)#  Loop over sorted coords          ML60 217
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 ML60 218
MJ=M4+IX#                              Get pointer to atom coordina     ML60 219
LSKIP=NINT(DSCALE*(CMAX(IX)-QX(MJ)))#  No. lines to skip                ML60 220
LSKIP=MAX0(0,LSKIP-LDONE)#             Get line skip value              ML60 221
LDONE=LDONE+LSKIP#                     Increment line counter           ML60 222
IF((LSKIP>0)\(NNCODE==12))#            Move one to next line            ML60 223
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 ML60 224
writeline:(LDOWN,0,0,1,3)#             Op blank lines                   ML60 225
NNCODE=0#                              Reset encoded fields             ML60 226
LDOWN=MAX0(LSKIP-1,0)#                 Reset lines to be skipped        ML60 227
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 ML60 228
MK=M4+IY#                              Get coordinate across page       ML60 229
COL=SCMAX*(QX(MK)-CMIN(IY))+SETIN#     Get number of columns to mov     ML60 230
#                                                                       ML60 231
#  CHECK ENCODED FIELDS FOR OVERLAP                                     ML60 232
#  --------------------------------                                     ML60 233
#                                                                       ML60 234
FOR(NACROS=1;NACROS<=NNCODE;NACROS=NACROS+1)# Loop sites on line        ML60 235
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 ML60 236
IF(ABS(COL+3.-FILLED(NACROS))>11.) NEXT# There's room                   ML60 237
writeline:(LDOWN,0,0,1,3)#             Write line if a perceived co     ML60 238
NNCODE=0#                              Reset count of encoded field     ML60 239
LDOWN=0#                               No need to skip more lines       ML60 240
BREAK#                                                                  ML60 241
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 ML60 242
NNCODE=NNCODE+1#                       Increment count of encoded f     ML60 243
FILLED(NNCODE)=COL+3.#                 Mark center of field             ML60 244
ICODE=INT(COL)#                        Get y-coord in columns           ML60 245
#                                                                       ML60 246
#   PRINT SITE NUMBER (& ATOM NAME IF MATCHED)                          ML60 247
#   ------------------------------------------                          ML60 248
#                                                                       ML60 249
IP=NINT(QX(M4))#                       Extract site address             ML60 250
IF     (ringf:>.5) I=1#                Point to * for ring sites        ML60 251
ELSE IF(nbond:>.5) I=2#                Point to + for bond sites        ML60 252
ELSE               I=3#                Point to . for unbonded          ML60 253
movechar:(MLA4,I,CHROT,ICODE,1,0)#     Insert site marker               ML60 254
N=(IP-MARK2)/STEP2+1#                  Get site seq number              ML60 255
movertoc:(aname:,1,TEMP,1,4,0)#        Get atom name                    ML60 256
IF(TEMP(1^1)!=' ')#                    Test if atom name stored         ML60 257
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 ML60 258
FOR(I=4;I>0;I=I-1) IF(TEMP(I^I)!=' ')BREAK# Test for blanks             ML60 259
CHROT(ICODE+1^ICODE+I)=TEMP(1^I)#      Insert non-blank characters      ML60 260
IF(IP>=STOPI)#                         Test if extended site            ML60 261
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 ML60 262
I=I+1; CHROT(ICODE+I^ICODE+I)='"'#     Flag symm gen codes              ML60 263
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 ML60 264
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 ML60 265
ELSE#                                  Otherwise use site number        ML60 266
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 ML60 267
IF     (N<10 ) I=1#                    Number occupies 1 char           ML60 268
ELSE IF(N<100) I=2#                    Number occupies 2 chars          ML60 269
ELSE           I=3#                    Number occupies 3 chars          ML60 270
CHROT(ICODE+1^ICODE+I)=TEMP(5-I^4)#    Insert sequence number           ML60 271
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 ML60 272
IF(match:<.5) NEXT#                    Is site matched to frag          ML60 273
M3=NINT(match:)+MAXBON+ipbias:+1#      Point to name in frag list       ML60 274
ICODE=ICODE+I+1#                       Fix col starting point           ML60 275
movechar:(MLA4,4,CHROT,ICODE,1,0)#     Left parenthesis                 ML60 276
movertoc:(QX(M3),1,CHROT,ICODE+1,6,0)# Move the atom name               ML60 277
compchar:(BLANK,1,CHROT,ICODE+2,1,1,5,M)# Find first blank after n      ML60 278
IF(M==0) M=6#                          Set name length to 6             ML60 279
movechar:(MLA4,5,CHROT,ICODE+M+1,1,0)# Put in right parenthesis         ML60 280
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 ML60 281
IF(NNCODE>0) writeline:(LDOWN,0,0,1,3)# Write any partial lines         ML60 282
#                                                                       ML60 283
IF(IXACRS==yes:)#                      If x was across for first pl     ML60 284
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 ML60 285
IF(NPLOT==1) IX=3#                     Across z down for 3rd, if ne     ML60 286
ELSE         IY=2#                                                      ML60 287
NEXT#                                  Skip to next plot                ML60 288
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 ML60 289
IF(NPLOT==1) IY=3#                     Z across and y down for 3rd.     ML60 290
ELSE         IX=2#                                                      ML60 291
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML60 292
LINCT=0#                                                                ML60 293
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML60 294
#                                                                       ML60 295
RETURN#                                                                 ML60 296
END#                                                                    ML60 297
systemheader:(ML61)#                   Diagonalize matrix to vector     ML61   1
####################################################################### ML61   2
#                                                                       ML61   3
#                                ML61                                   ML61   4
#                                                                       ML61   5
####################################################################### ML61   6
SUBROUTINE ML61#                                                        ML61   7
include:(AACOMN)#                                                       ML61   8
include:(MLCOMN)#                                                       ML61   9
#                                                                       ML61  10
INTEGER I,J,IP1,K,M#                                                    ML61  11
INTEGER KNT#                           Count of small off-diagonal      ML61  12
INTEGER IND1,IND3#                     Index axes for ls and ms pla     ML61  13
REAL DIFF#                             Difference between diagonal      ML61  14
REAL TPHI#                             Tangent(phi)                     ML61  15
REAL THALF#                            Tangent(phi/2) #                 ML61  16
REAL SPHI#                             Sine(phi)                        ML61  17
REAL MLHI#                             Cosine(phi)                      ML61  18
REAL BIK,BKI,VKI#                      Temp. place for elements of      ML61  19
REAL DET#                              Determinant of v                 ML61  20
realdata:(SMALL,0.000001)#                                              ML61  21
datastuff:#                                                             ML61  22
#                                                                       ML61  23
#--------------------------------------------------------------------   ML61  24
# B IS THE MATRIX OF COEFFICIENTS FOR USE IN FINDING THE LEAST SQUARES  ML61  25
# PLANE OF THE CLUSTER.  THIS SUBROUTINE DIAGONALIZES THE MATRIX AND    ML61  26
# RETURNS V, A MATRIX WHICH GIVES THE TRANSFORMATION FROM THE ORTHOGONA ML61  27
# ANGSTROM COORDINATES TO THE LS COORDINATE SYSTEM.  IT ALSO RETURNS    ML61  28
# INDEX, WHICH TELLS THE AXES THAT ARE PERPENDICULAR TO THE LEAST       ML61  29
# SQUARES AND MOST SQUARES PLANES                                       ML61  30
# THIS CORRESPONDS TO THE SUBROUTINE EIGEN IN MULTAN80.                 ML61  31
#                                                                       ML61  32
# THE B MATRIX IS DIAGONALIZED BY A SUCCESSIVE APPROXIMATIONS ALGORITHM ML61  33
# THE ORTHOGONAL COORDINATE SYSTEM IS ROTATED BY AN ANGLE PHI, FIRST IN ML61  34
# THE XY PLANE, THEN IN THE XZ AND YZ PLANES. PHI IS DETERMINED BY THE  ML61  35
# RATIO OF THE OFF-DIAGONAL TERM TO THE DIFFERENCE IN DIAGONAL TERMS    ML61  36
#                                                                       ML61  37
#                 TANGENT(PHI)= BMAT(I,J)/(BMAT(I,I)-BMAT(J,J))         ML61  38
#                                                                       ML61  39
# THE ROTATION IS LIMITED TO 45 DEGREES ON EACH ITERATION               ML61  40
# WHEN ALL THE OFF-DIAGONAL TERMS ARE VERY SMALL, THE DIAGONALIZATION   ML61  41
# IS COMPLETE.                                                          ML61  42
#---------------------------------------------------------------------  ML61  43
#                                                                       ML61  44
DO I=1,3; DO J=1,3#                                                     ML61  45
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML61  46
VECT(I,J)=0.0#                         Zero the i'th row                ML61  47
VECT(I,I)=1.0#                         Make diagonal terms 1            ML61  48
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML61  49
#                                                                       ML61  50
#    LOOP THRU ITERATIONS                                               ML61  51
#    --------------------                                               ML61  52
#                                                                       ML61  53
KNT=0;   IND1=1;   IND3=1#             Zero small off-diag count        ML61  54
FOR(M=1;M<50&KNT<3;M=M+1)#             Loop over iterations             ML61  55
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML61  56
KNT=0;   IND1=1;   IND3=1#             Zero small off-diag count        ML61  57
DO I=1,2#                                                               ML61  58
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML61  59
IP1=I+1#                                                                ML61  60
DO J=IP1,3#                                                             ML61  61
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML61  62
IF(ABS(BMAT(I,J))<SMALL*BMAT(IND1,IND1)) KNT=KNT+1#                     ML61  63
DIFF=BMAT(I,I)-BMAT(J,J)#                                               ML61  64
IF(DIFF<SMALL) TPHI=1.#                                                 ML61  65
ELSE IF(ABS(BMAT(I,J))>=ABS(DIFF))#    Don't let tan(phi) be more t     ML61  66
TPHI=SIGN(1.0,BMAT(I,J)*DIFF)#         Preserve the sense of rotati     ML61  67
ELSE TPHI=BMAT(I,J)/DIFF#                                               ML61  68
IF(TPHI<SMALL) TPHI=0.#                Avoid later underflow            ML61  69
#                                                                       ML61  70
#------------------------------------------------------------------     ML61  71
# TO GET THE SIN AND COS NEEDED FOR THE ROTATION, GO THROUGH TAN(PHI/2) ML61  72
# IT CAN BE SHOWN THAT                                                  ML61  73
#                       TAN(PHI/2)=TAN(PHI)/(1+SEC(PHI))                ML61  74
#                                                                       ML61  75
# IN THE VICINITY OF 45 DEGREES, SEC(PHI) IS APPROXIMATELY 1, SO 2*SEC  ML61  76
# CAN BE USED TO APPROXIMATE SEC(PHI)+1.                                ML61  77
#                                                                       ML61  78
#                       TAN (PHI/2)=TAN(PHI)/(2*SEC(PHI))               ML61  79
#                                                                       ML61  80
# SINCE SEC(PHI) IS ABOUT 1, SO IS SEC(PHI)**2, WHICH IS 1+TAN(PHI)**2. ML61  81
#---------------------------------------------------------------------  ML61  82
#                                                                       ML61  83
THALF=TPHI/(2.0+2.0*TPHI**2)#                                           ML61  84
SPHI=2.0*THALF/(1.0+THALF**2)#                                          ML61  85
MLHI=1.-THALF*SPHI#                                                     ML61  86
DO K=1,3#                              Transform i'th and j'th rows     ML61  87
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML61  88
BIK=BMAT(I,K)#                         Save before overstoring bmat     ML61  89
BMAT(I,K)=MLHI*BIK+SPHI*BMAT(J,K)#                                      ML61  90
BMAT(J,K)=MLHI*BMAT(J,K)-SPHI*BIK#                                      ML61  91
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML61  92
DO K=1,3#                              Transform i'th and j'th colu     ML61  93
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML61  94
BKI=BMAT(K,I)#                                                          ML61  95
BMAT(K,I)=MLHI*BKI+SPHI*BMAT(K,J)#                                      ML61  96
BMAT(K,J)=MLHI*BMAT(K,J)-SPHI*BKI#                                      ML61  97
VKI=VECT(K,I)#                                                          ML61  98
VECT(K,I)=MLHI*VKI+SPHI*VECT(K,J)#                                      ML61  99
VECT(K,J)=MLHI*VECT(K,J)-SPHI*VKI#                                      ML61 100
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML61 101
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML61 102
#                                                                       ML61 103
#   FIND LARGEST & SMALLEST DIAGONAL TERMS                              ML61 104
#  ---------------------------------------                              ML61 105
#                                                                       ML61 106
IF(BMAT(IP1,IP1)>BMAT(IND1,IND1)) IND1=IP1# Largest                     ML61 107
IF(BMAT(IND3,IND3)>BMAT(IP1,IP1)) IND3=IP1# Smallest                    ML61 108
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML61 109
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML61 110
#                                                                       ML61 111
# SAVE INDICES FOR MS AND LS PLANES. IND(2) IS WHATEVER IS LEFT         ML61 112
#                                                                       ML61 113
IND(1)=IND1#                                                            ML61 114
IND(3)=IND3#                                                            ML61 115
IND(2)=6-IND1-IND3#                    Remember 1+2+3=6                 ML61 116
DET=VECT(1,1)*(VECT(2,2)*VECT(3,3)-VECT(2,3)*VECT(3,2))$#               ML61 117
+   VECT(1,2)*(VECT(2,3)*VECT(3,1)-VECT(2,1)*VECT(3,3))$#               ML61 118
+   VECT(1,3)*(VECT(2,1)*VECT(3,2)-VECT(2,2)*VECT(3,1))#                ML61 119
#                                                                       ML61 120
#   NEG DET(V) FOR RHS - CHANGE TO LHS                                  ML61 121
#                                                                       ML61 122
IF(DET<0.)#                                                             ML61 123
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML61 124
DO I=1,3#                                                               ML61 125
VECT(I,3)=-VECT(I,3)#                                                   ML61 126
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML61 127
#                                                                       ML61 128
RETURN#                                                                 ML61 129
END#                                                                    ML61 130
systemheader:(ML70)#                   Bond lengths and angles          ML70   1
####################################################################### ML70   2
#                                                                       ML70   3
#                                ML70                                   ML70   4
#                                                                       ML70   5
####################################################################### ML70   6
SUBROUTINE ML70#                                                        ML70   7
include:(AACOMN)#                                                       ML70   8
include:(MLCOMN)#                                                       ML70   9
#                                                                       ML70  10
INTEGER I,K,N,IB1,IB2#                 General indices                  ML70  11
INTEGER IP,JP,KP,IB,JB#                Loop list indices                ML70  12
REAL TESTBL#                                                            ML70  13
REAL CODE,DD,DX,DY,DZ#                 Dsqd & output format             ML70  14
REAL U,V,W,A,B,C,X,Y,Z#                Coordinates in transf.           ML70  15
REAL COSA#                             Cosine of bond angle             ML70  16
#                                                                       ML70  17
string:(PUNCH,80)#                                                      ML70  18
chardata:(DASH,'-."')#                                                  ML70  19
chardata:(ML601," Atom/Site  Nclus       x/a      y/b      z/c    $#    ML70  20
Weight    Assigned")#                                                   ML70  21
chardata:(ML602," Number of sites in each cluster")#                    ML70  22
chardata:(ML603," Summary of Site Coords and Connections")#             ML70  23
chardata:(ML605," Bond Lengths for sites closer than        A")#        ML70  24
chardata:(ML606,"   Sites  Distance      Sites  Distance    $#          ML70  25
  Sites  Distance      Sites  Distance ")#                              ML70  26
chardata:(ML607," Bond Angles for sites closer than        A")#         ML70  27
chardata:(ML608,"     Sites    Angle       Sites    Angle      $#       ML70  28
 Sites    Angle       Sites    Angle")#                                 ML70  29
chardata:(ML609," Site Distances for all asymmetric units")#            ML70  30
chardata:(ML610,"   Sites   Dist  symop  xs    ys    zs   $#            ML70  31
   Sites   Dist  symop  xs    ys    zs")#                               ML70  32
chardata:(ML611," Sites connected across Special Positions")#           ML70  33
realdata:(F0,[280852.,370852.,460852.,540522.,160313.])#                ML70  34
realdata:(F1,[40313.,80313.,160622.,250313.,#                           ML70  35
290313.,370622.,460313.,500313.,580622.,670313.,710313.,790622.])#      ML70  36
realdata:(F2,[40313.,80313.,120313.,190612.,#                           ML70  37
250313.,290313.,330313.,400612.,460313.,500313.,540313.,610612.,#       ML70  38
670313.,710313.,750313.,820612.])#                                      ML70  39
realdata:(F3,[40313.,80313.,150622.,200213.,260632.,320632.,#           ML70  40
380632.,450313.,490313.,560622.,610213.,670632.,730632.,790632.])#      ML70  41
realdata:(RTOD,57.29583)#                                               ML70  42
datastuff:#                                                             ML70  43
#                                                                       ML70  44
#   PRINT SUMMARY HEADER                                                ML70  45
#   --------------------                                                ML70  46
#                                                                       ML70  47
LINRM=10#                                                               ML70  48
movechar:(DASH,1,CHROT,2,NML603-1,2)#  Underline header                 ML70  49
writeline:(1,ML603,NML603,1,3)#        Print header                     ML70  50
writeline:(0,0,0,1,3)#                 Force space                      ML70  51
LINRM=4#                                                                ML70  52
#                                                                       ML70  53
#   LIST ALL SITES PROCESSED                                            ML70  54
#   ------------------------                                            ML70  55
#                                                                       ML70  56
N=0#                                   Reset site count                 ML70  57
FOR(IP=MARK2;IP<STOPI;IP=IP+STEP2)#    Loop over site list              ML70  58
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  ML70  59
N=N+1#                                 Count each site                  ML70  60
movertoc:(aname:,1,CHROT,2,4,0)#       Store site name                  ML70  61
IF(CHROT(2^2)==' ') CHROT(2^5)=' '#    Blank out site number            ML70  62
ncodefld:(FLOAT(N),1,CHROT,90313.,1)#  Format site number               ML70  63
ncodefld:(peakx:,1,CHROT,F0,5)#        Height,x,y,z,cluster no.         ML70  64
PUNCH='ATOM'#                          Reset punch buffer               ML70  65
movertoc:(aname:,1,PUNCH,6,4,0)#       Store site name for punch        ML70  66
ncodefld:(peakx:,1,PUNCH,F0,3)#        Format x,y,z for punch           ML70  67
IF(match:>.5)#                         If site assigned to input        ML70  68
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  ML70  69
I=NINT(match:)+MAXBON+ipbias:+1#       Point to input atom name         ML70  70
movertoc:(QX(I),1,CHROT,60,6,0)#       Insert atom name to op           ML70  71
movertoc:(QX(I),1,PUNCH, 6,6,0)#       Insert atom name to op           ML70  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  ML70  73
writeline:(0,ML601,NML601,1,3)#        Write it out                     ML70  74
writepch:(PUNCH,55)#                   Punch atom line                  ML70  75
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  ML70  76
#                                                                       ML70  77
#    PRINT CLUSTER SUMMARY                                              ML70  78
#    ---------------------                                              ML70  79
#                                                                       ML70  80
LINRM=4#                               Force header                     ML70  81
movechar:(DASH,1,CHROT,2,NML602-1,2)#  Underline header                 ML70  82
writeline:(1,ML602,NML602,1,3)#        Tell sites in each cluster       ML70  83
movechar:(ML602,26,CHROT,2,7,0)#       Put cluster in buffer            ML70  84
FOR(A=1.;A<FLOAT(TCLUS+1);A=A+1.)#     Loop over clusters               ML70  85
ncodefld:(A,1,CHROT,A*40000.+80313.,1)# Format cluster number           ML70  86
writeline:(0,0,0,1,3)#                 Print line                       ML70  87
movechar:(ML602,12,CHROT,2,5,0)#       Insert sites name                ML70  88
CODE=120313.#                          Set initial format               ML70  89
DO I=1,TCLUS#                          Loop over all clusters           ML70  90
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  ML70  91
BUFOT(1)=FLOAT(NPKCL(I))#              Float count of sites in clus     ML70  92
ncodefld:(BUFOT,1,CHROT,CODE,1)#       Format next site count           ML70  93
CODE=CODE+40000.#                      Move over for next one           ML70  94
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  ML70  95
writeline:(0,0,0,1,3)#                 Print population                 ML70  96
#                                                                       ML70  97
#  LIST SITE DISTANCES IN ASYM UNIT                                     ML70  98
#  --------------------------------                                     ML70  99
#                                                                       ML70 100
LINRM=8#                                                                ML70 101
BUFOT(1)=SQRT(PRTMAX)#                 Store print distance max         ML70 102
ncodefld:(BUFOT,1,ML605,420422.,1)#    Format print distance            ML70 103
movechar:(DASH,1,CHROT,2,NML605-1,2)#  Insert underline                 ML70 104
writeline:(1,ML605,NML605,1,3)#        Print bond header                ML70 105
N=1;  LINRM=4#                         Zero count of dist in output     ML70 106
FOR(IP=MARK2;IP<STOPI;IP=IP+STEP2)#    Loop over site list              ML70 107
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  ML70 108
FOR(JP=MARK2;JP<STOP2;JP=JP+STEP2)#    Loop over site list              ML70 109
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  ML70 110
IF(IP==JP) NEXT#                       Skip if same site                ML70 111
DX=peakx:(JP)-peakx:#                  Get x difference                 ML70 112
DY=peaky:(JP)-peaky:#                  Get y difference                 ML70 113
DZ=peakz:(JP)-peakz:#                  Get z difference                 ML70 114
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML70 115
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML70 116
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML70 117
IF(DD>PRTMAX) NEXT#                    Skip if outside print limit      ML70 118
BUFOT(1)=numbr:#                       Store site number                ML70 119
BUFOT(2)=numbr:(JP)#                   Store site number                ML70 120
BUFOT(3)=SQRT(DD)#                     Store bond distance              ML70 121
movechar:(DASH,1,CHROT,N*7-2,1,0)#     Insert dashes between sites      ML70 122
IF(JP>=STOPI)#                         For symm gen sites only          ML70 123
movechar:(DASH,3,CHROT,N*7+2,1,0)#     Insert " after symm site         ML70 124
ncodefld:(BUFOT,1,CHROT,F1(N),3)#      Format sites and bl              ML70 125
N=N+3#                                 Incr po pointer                  ML70 126
IF(N<12) NEXT#                         Skip until buff full             ML70 127
writeline:(0,ML606,NML606,1,3)#        Print buffer                     ML70 128
N=1#                                   Reset counter                    ML70 129
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  ML70 130
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  ML70 131
IF(N>1) writeline:(0,ML606,NML606,1,3)# Write out last line             ML70 132
#                                                                       ML70 133
#   LIST BOND ANGLES WITHIN BOND MAX                                    ML70 134
#   --------------------------------                                    ML70 135
#                                                                       ML70 136
LINRM=8#                                                                ML70 137
BUFOT(1)=SQRT(RADMAX)#                 Encode max. bond distance        ML70 138
TESTBL=BUFOT(1)#                       Save rejection test              ML70 139
ncodefld:(BUFOT,1,ML607,410422.,1)#    Format bond limit                ML70 140
movechar:(DASH,1,CHROT,2,NML607-1,2)#  Underline header                 ML70 141
writeline:(1,ML607,NML607,1,3)#        Print bond limit header          ML70 142
N=1;  LINRM=4#                         Set op counter                   ML70 143
FOR(IP=MARK2;IP<STOPI;IP=IP+STEP2)#    Loop over site list              ML70 144
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  ML70 145
IB1=NINT(bond1:); IB2=NINT(bond2:)#    Get addr. of bond sequence       ML70 146
FOR(IB=IB1;IB<IB2;IB=IB+STEPB)#        Loop over bondlist sequence      ML70 147
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  ML70 148
IF(bondl:>TESTBL) NEXT#                Skip if bond too long            ML70 149
JP=NINT(peak2:)#                       Get connected site addr          ML70 150
FOR(JB=IB+STEPB;JB<IB2;JB=JB+STEPB)#   Loop over bond sequence          ML70 151
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  ML70 152
IF(bondl:(JB)>TESTBL) NEXT#            Forget ones with long bonds      ML70 153
KP=NINT(peak2:(JB))#                   Get connected site addr          ML70 154
DX=peakx:(JP)-peakx:(KP)#              Get x difference                 ML70 155
DY=peaky:(JP)-peaky:(KP)#              Get y difference                 ML70 156
DZ=peakz:(JP)-peakz:(KP)#              Get z difference                 ML70 157
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML70 158
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML70 159
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML70 160
#IF(DD>PRTMAX) NEXT#                    # SKIP IF TOO FAR AWAY          ML70 161
COSA=(bondl:(JB)**2+bondl:**2-DD)$#    Calculate cosine angle           ML70 162
/(2.*bondl:(JB)*bondl:)#               Calculate cosine angle           ML70 163
COSA=SIGN(AMIN1(ABS(COSA),1.),COSA)#   Ensure cosine in range           ML70 164
BUFOT(1)=numbr:(JP)#                   Store site number                ML70 165
BUFOT(2)=numbr:#                       Store site number                ML70 166
BUFOT(3)=numbr:(KP)#                   Store site number                ML70 167
BUFOT(4)=ACOS(COSA)*RTOD#              Store angle in degrees           ML70 168
movechar:(DASH,1,CHROT,((N-1)/4)*21+4,8,2)# Put dashes between sites    ML70 169
ncodefld:(BUFOT,1,CHROT,F2(N),4)#      Encode buffer                    ML70 170
N=N+4#                                 Count op items                   ML70 171
IF(N<16) NEXT#                         Skip if buffer not full          ML70 172
writeline:(0,ML608,NML608,1,3)#        Print op buffer                  ML70 173
N=1#                                   Reset for next line              ML70 174
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  ML70 175
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  ML70 176
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  ML70 177
IF(N>1) writeline:(0,ML608,NML608,1,3)# Write out last line of angle    ML70 178
#                                                                       ML70 179
#    BOND DISTANCES TO ALL ASYMMETRIC UNITS                             ML70 180
#    --------------------------------------                             ML70 181
#                                                                       ML70 182
LINRM=10#                              Force header                     ML70 183
movechar:(DASH,1,CHROT,2,NML609-1,2)#  Insert underline                 ML70 184
writeline:(1,ML609,NML609,1,3)#        Write heading for next table     ML70 185
N=1;  LINRM=4#                         Set op item counter              ML70 186
FOR(IP=MARK2;IP<STOPI;IP=IP+STEP2)#    Loop over site list              ML70 187
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  ML70 188
A=peakx:; B=peaky:; C=peakz:#          Get site coords to local         ML70 189
#                                                                       ML70 190
FOR(JP=MARK2;JP<STOPI;JP=JP+STEP2)#    Loop over sites list             ML70 191
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 ML70 192
X=peakx:(JP); Y=peaky:(JP); Z=peakz:(JP)# Store site coords local       ML70 193
#                                                                       ML70 194
FOR(K=MARK1;K<STOP1;K=K+STEP1)#        Loop over all symops             ML70 195
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 ML70 196
U=X*QX(K+1)+Y*QX(K+4)+Z*QX(K+7)+QX(K+10)# Generate equivalent x         ML70 197
V=X*QX(K+2)+Y*QX(K+5)+Z*QX(K+8)+QX(K+11)# Generate equivalent y         ML70 198
W=X*QX(K+3)+Y*QX(K+6)+Z*QX(K+9)+QX(K+12)# Generate equivalent z         ML70 199
DX=MOD(99.5+U-A,1.)-0.5#               Get closest distance             ML70 200
DY=MOD(99.5+V-B,1.)-0.5#               Get closest distance             ML70 201
DZ=MOD(99.5+W-C,1.)-0.5#               Get closest distance             ML70 202
DD=G1*DX**2+G6*DY*DZ$#                 Calculate distance squared       ML70 203
+  G2*DY**2+G5*DX*DZ$#                 Calculate distance squared       ML70 204
+  G3*DZ**2+G4*DX*DY#                  Calculate distance squared       ML70 205
IF((DD<SEPMIN)\(DD>RADMAX)) NEXT#      Skip if same or too far          ML70 206
BUFOT(1)=numbr:#                       Store site number                ML70 207
BUFOT(2)=numbr:(JP)#                   Store site number                ML70 208
BUFOT(3)=SQRT(DD)#                     Store bond distance              ML70 209
BUFOT(4)=FLOAT((K- MARK1)/STEP1+1)#    Store symop number               ML70 210
BUFOT(5)=A+DX#                         Store transformed coord          ML70 211
BUFOT(6)=B+DY#                         Store transformed coord          ML70 212
BUFOT(7)=C+DZ#                         Store transformed coord          ML70 213
movechar:(DASH,1,CHROT,(N-1)*6+4,4,2)# Put dashes between sites         ML70 214
ncodefld:(BUFOT,1,CHROT,F3(N),7)#      Format op items                  ML70 215
N=N+7#                                 Incr op item counter             ML70 216
IF(N<14) NEXT#                         Skip if buffer not full          ML70 217
writeline:(0,ML610,NML610,1,3)#        Print op buffer                  ML70 218
N=1#                                   Reset for next line              ML70 219
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 ML70 220
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 ML70 221
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  ML70 222
IF(N>1) writeline:(0,ML610,NML610,1,3)# Write last line if needed       ML70 223
#                                                                       ML70 224
#    PRINT TWINNED SITE INFORMATION                                     ML70 225
#    ------------------------------                                     ML70 226
#                                                                       ML70 227
IF(NTWIN>0)#                           Do twin sites exist              ML70 228
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 ML70 229
LINRM=8#                               Force header line                ML70 230
movechar:(DASH,1,CHROT,2,NML611-1,2)#  Insert underline                 ML70 231
writeline:(1,ML611,NML611,1,3)#        Print header                     ML70 232
N=1;  LINRM=4#                         Zero count of pairs encoded      ML70 233
FOR(IB=MARKT;IB<STOPT;IB=IB+STEPT)#    Loop over twinned site list      ML70 234
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 ML70 235
BUFOT(1)=FLOAT((NINT(peak1:)-MARK2)/STEP2+1)# Store site number         ML70 236
BUFOT(2)=FLOAT((NINT(peak2:)-MARK2)/STEP2+1)# Store site number         ML70 237
BUFOT(3)=bondl:#                       Store cluster number             ML70 238
movechar:(DASH,1,CHROT,(N-1)*7+4,4,2)# Insert dashes between sites      ML70 239
ncodefld:(BUFOT,1,CHROT,F1(N),3)#      Format op items                  ML70 240
N=N+3#                                 Count op items                   ML70 241
IF(N<12) NEXT#                         Skip if buffer not full          ML70 242
writeline:(0,ML606,NML606,1,3)#        Print op buffer                  ML70 243
N=1#                                   Reset op item counter            ML70 244
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 ML70 245
IF(N>1) writeline:(0,ML606,NML606,1,3)# Print last line if active       ML70 246
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 ML70 247
#                                                                       ML70 248
RETURN#                                                                 ML70 249
END#                                                                    ML70 250
pmacro:([no:])#                        Purge xtal system macros         ML70 251
