systemheader:(PT    )#                                                  PT     1
####################################################################    PT     2
#                                                                  #    PT     3
#                            ***PLOT***                            #    PT     4
#                                                                  #    PT     5
#  THE PLOT PROGRAM ACTS AS AN INTERFACE BETWEEN ORTEP AND THE     #    PT     6
#  OTHER XTAL PROGRAMS WHICH PRODUCE PLOTS AND THE LOCAL           #    PT     7
#  PLOTTING DEVICES.  PLOT IS VERY SHOP SPECIFIC AND WILL          #    PT     8
#  REQUIRE CAREFUL EXAMINATION AND SOME MODIFICATION TO WORK       #    PT     9
#  IN ANY GIVEN COMPUTER-PLOTTER ENVIRONMENT.  THERE ARE           #    PT    10
#  PROVISIONS FOR MAKING PLOTS ON THE LINE PRINTER AT              #    PT    11
#  LOW RESOLUTION, ON DOT MATRIX PRINTERS AT ANY POSSIBLE          #    PT    12
#  RESOLUTION, AND SETTING CALLS FOR LOCAL ANALOG PLOTTER          #    PT    13
#  SHOP SPECIFIC SUBROUTINES.                                      #    PT    14
#                                                                  #    PT    15
####################################################################    PT    16
#                                                                  #    PT    17
include:(XMACRO)#                                                       PT    18
#                                                                       PT    19
#  THE PT PROGRAM EXISTS BECAUSE OF THE FACT THAT THERE IS NO STANDARD  PT    20
#  WAY TO MAKE PLOTS.  ALL THE OTHER XTAL SYSTEM PROGRAMS WHICH         PT    21
#  CREATE PLOTS DO SO BY WRITING THE PLOT DIRECTIONS TO A AUX BDF.      PT    22
#  THIS BDF MAT THEN BE READ BY THE PROGRAM PT AND CONVERTED TO         PT    23
#  ACTUALLY MAKE THE PLOTS.  PT SERVES AS AN INTERFACE BETWEEN ALL      PT    24
#  XTAL PROGRAMS AND THE LOCAL PLOTTER(S).  THIS MEANS THAT PT IS       PT    25
#  VERY MACHINE SPECIFIC.  BEFORE PT CAN BE COMPILED AND RUN IT MUST    PT    26
#  BE SET UP TO CONFORM TO LOCAL COMPUTER FACILITY CONDITIONS.          PT    27
#                                                                       PT    28
#  THE LINES WHICH MUST BE                                              PT    29
#  EDITED ARE MARKED WITH  $MASP  .   THERE ARE COMMENTS NEAR THESE     PT    30
#  LINES WITH ADDITIONAL DETAILS OF THE PURPOSE OF THE LOCAL PLOTTER    PT    31
#  CALLS.  THE PROGRAM IS SET UP FOR CONNECTING TO A NUMBER OF DIFFEREN PT    32
#  PLOTTING DEVICES.  THE SETTING OF INTEGER VARIABLE 'NDEV' DIRECTS    PT    33
#  WHICH DEVICE WILL MAKE THE PLOT.                                     PT    34
#                                                                       PT    35
#  NDEV=1#  USE THE LOCAL LINE PRINTER AS A PLOTTER (LOW RESOLUTION)    PT    36
#  NDEV=2#  USE HIGH RESOLUTION DOT PRINTER AS PLOTTER                  PT    37
#        #  THIS GENERATES A WHOLE NEW SET OF MACHINE SPECIFIC PROBS.   PT    38
#        # FOR EXAMPLE    PRINTRONIX    XEROX-DIABLO   HP-LASER+        PT    39
#        #      VDPI:        72            48          100,200,300      PT    40
#        #      HDPI:        60           120          100,200,300      PT    41
#        # IN ADDITION SOME SORT OF ESCAPE SEQUENCE WILL BE NEEDED      PT    42
#        # TO SHIFT FROM CONVENTIONAL PRINTING TO HIGH RESOLUTION       PT    43
#        # PRINTING.  THE ESCAPE SEQUENCE IS SET UP IN PT11.            PT    44
#  NDEV=3#  GENERATE A CALCOMP PLOTTER-DRIVER TAPE DIRECTLY             PT    45
#        #  THIS REQUIRES THE SETTING OF THE STARTPLOT:, OUTPLOT:,      PT    46
#        #  AND STOPPLOT: MACROS FOR THE LOCAL TAPE DRIVE.              PT    47
#  NDEV=4#  VDU SCREEN FOR PLOTS   CURRENTLY IMPLEMENTED FOR            PT    48
#        #  TEKTRONIX 4010 SERIES COLOUR TERMINALS                      PT    49
#  NDEV=5#  USE A LOCAL PLOTTING DEVICE BY INTERFACING THROUGH          PT    50
#        #  LOCAL LIBRARY SUBROUTINES.  CURRENTLY IMPLEMENTED FOR       PT    51
#        #  TEKTRONIX 4662 INTERACTIVE DIGITAL PLOTTER.                 PT    52
#  NDEV=6#  EXACTLY LIKE NDEV=5 BUT ALLOWS FOR AN ALTERNATIVE DEVICE    PT    53
#        #  AT THE SAME INSTALLATION.                                   PT    54
#                                                                       PT    55
#                                                                       PT    56
#################### MACROS FOR DEVICE RESOLUTIONS ##################   PT    57
#                                                                       PT    58
#   THESE MACROS ARE THE DEFAULT VALUES OF RESOLUTION FOR THEIR         PT    59
#   RESPECTIVE DEVICES. THE VALUES ARE THE SMALLER OF THE RESOL-        PT    60
#   UTIONS IN THE X AND Y DIRECTIONS AND ARE IN UNITS OF (UGH!)         PT    61
#   INCHES.                                                             PT    62
#                                                                       PT    63
macro:(rsv1:,0.1000)#                  'char' printer resolution        PT    64
macro:(rsv2:,0.0139)#                  'dot' pixel resolution           PT    65
macro:(rsv3:,0.0005)#                  Calcomp plotter                  PT    66
macro:(rsv4:,0.0196)#                  Graphics vdu                     PT    67
macro:(rsv5:,0.005)#                   Local plotting device 1          PT    68
macro:(rsv6:,0.0005)#                  Local plotting device 2          PT    69
#                                                                       PT    70
#   RESOLUTIONS FOR PLOTTING DEVICES (DEVICES 3,5 & 6) ARE              PT    71
#   ALSO DEPENDENT ON PEN THICKNESS USED IN ACTUALLY DRAW-              PT    72
#   ING SUBJECTS.                                                       PT    73
#   [RSV1:] AND [RSV2:] ARE IMPLEMENTED FOR PRINTRONIX PRINTER.         PT    74
#   [RSV4:] IS IMPLEMENTED FOR TEKTRONIX 4105 GRAPHICS TERMINAL.        PT    75
#   [RSV5:] IS IMPLEMENTED FOR TEKTRONIX 4662 INTERACTIVE               PT    76
#           DIGITAL PLOTTER.                                            PT    77
#                                                                       PT    78
####################################################################    PT    79
#                                                                       PT    80
############### MACROS FOR DOT MATRIX PRINTER OUTPUT #############      PT    81
#                                                                       PT    82
#                                                                       PT    83
macro:(vdpi:,100)#                     Dots per inch vertically         PT    84
macro:(hdpi:,100)#                     Dots per inch horizontally       PT    85
#                                                                       PT    86
#                                                                       PT    87
# ADAPTING THE LOCAL DOT MATRIX PRINTER MAY BE DONE IN TWO              PT    88
# DIFFERENT WAYS----- LOCAL MACROS OR CONTROL LINES.                    PT    89
#                                                                       PT    90
#                                                                       PT    91
#  ***** LOCAL MACRO APPROACH                                           PT    92
#                                                                       PT    93
#  THESE MACROS MAY BE LEFT NULL IF THE USER IS WILLING TO SUPPLY       PT    94
#  THE APPROPRIATE CONTROL CHARACTERS BY MEANS OF INPUT CONTROL         PT    95
#  LINES.  IF NOT THEY MAY BE SET AND USED TO FORCE THE CONTROL         PT    96
#  BITS ACCORDING TO THE LOCAL DOT MATRIX PRINTER CONTROLS.             PT    97
#                                                                       PT    98
ifsel:(PER)#      ## FOR PRINTRONIX ATTACHED TO A PE3240                PT    99
macro:(localdot0:,CONTINUE)#   ## not used in pt75                      PT   100
macro:(localdot1:,## For special word initialization in pt11            PT   101
IF(NDEV==2)intpak:(1077952576,QX(I),0,32))## Set bit 6 of each byte     PT   102
macro:(localdot2:,## For special bit storage in pt61                    PT   103
IF(NDEV==2) $(#    ## For hi res printer only                           PT   104
IF     (KBIT< 6) KBIT= 5-KBIT# ## Store in byte 0 (left just.)          PT   105
ELSE IF(KBIT<12) KBIT=19-KBIT# ## Store in byte 1                       PT   106
ELSE IF(KBIT<18) KBIT=33-KBIT# ## Store in byte 2                       PT   107
ELSE             KBIT=47-KBIT$))#      Store in byte 3                  PT   108
macro:(localdot3:,#            ## Initialize dot output in pt71         PT   109
intpak:(83886080,CONTRO,0,32))## Set 05 in byte 3                       PT   110
macro:(localdot4:,#            ## Print dot output in pt71              PT   111
NWRD=MARK+NBIT/MXBTAV+1#       ## Convert bit string to words           PT   112
NBYT=MOD(NBIT/6,4)+1#          ## Get byte within word                  PT   113
movertoc:(CONTRO,1,CHROT,1,1,0)## First byte is 05                      PT   114
movertoc:(QX(NWRD),NBYT,CHROT,2,ALLCHR,0)# Put bytes in buff            PT   115
movertoc:(CONTRO,3,CHROT,ALLCHR+2,131-ALLCHR,1)# Zero fill              PT   116
writeline:(0,0,0,1,3); NEXT)#  ## Send line to device                   PT   117
macro:(localdot5:,CONTINUE)#   ## close off 'dot' file                  PT   118
#                                                                       PT   119
ifsel:(UNX)#   ## FOR HP-LASERJET+ ATTACHED TO UNIX SYSTEM              PT   120
macro:(localdot0:,TOTPIX=(LIMIT/LINES)*32)# Ensure line mult of 32      PT   121
macro:(localdot1:,CONTINUE)#    ## Not used in PT11 and PT75            PT   122
macro:(localdot2:,CONTINUE)#    ## Not used in PT61                     PT   123
macro:(localdot3:,[$(#          ## Initialize dot op in PT71            PT   124
PIXEL=TOTPIX#                   ## Override pixels per line             PT   125
INT1=TOTPIX/8+1#                ## calc characters per line             PT   126
INT2=1159408244#                ## E esc * t                            PT   127
INT3=808464466+(HZPIX/100)*16777216+$# <n n n> r                        PT   128
(MOD(HZPIX,100)/10)*65536+MOD(HZPIX,10)*256# <n n n> r                  PT   129
INT4=TOTPIX/32-1#               ## set line length-1 in words           PT   130
opendac:(ioda1:,3,INT1+7,' ')#  ## Open file for pixel output           PT   131
writedac:(ioda1:,,1,[(0,II=1,INT4),27,INT2,INT3])# Escresc*t<nnn>r      PT   132
INT2=455762480+INT1/100#        ## set line clause part1                PT   133
INT3=808474368+(MOD(INT1,100)/10)*16777216+MOD(INT1,10)*65536#          PT   134
JJ=1$)])#                       ## start record counter                 PT   135
macro:(localdot4:,[$(#          ## Output dot op in PT71                PT   136
NWRD=MARK+NBIT/32+1;  JJ=JJ+1#  ## Point to word at start of line       PT   137
writedac:(ioda1:,,JJ,[INT2,INT3,(QX(II),II=NWRD,NWRD+INT4)])# Op li     PT   138
IF(JJ>1) NEXT$)])#              ## skip PT71 outputs                    PT   139
macro:(localdot5:,closedac:(ioda1:))#  Close off 'dot' file             PT   140
#                                                                       PT   141
ifsel:#                                                                 PT   142
macro:(localdot0:,CONTINUE)## Unused if 'dot' lpr set on input          PT   143
macro:(localdot1:,CONTINUE)## Unused if 'dot' lpr set on input          PT   144
macro:(localdot2:,CONTINUE)## Unused if 'dot' lpr set on input          PT   145
macro:(localdot3:,CONTINUE)## Unused if 'dot' lpr set on input          PT   146
macro:(localdot4:,CONTINUE)## Unused if 'dot' lpr set on input          PT   147
macro:(localdot5:,CONTINUE)## Unused if 'dot' lpr set on input          PT   148
endsel:#                                                                PT   149
#                                                                       PT   150
#                                                                       PT   151
# ******* CONTROL LINE APPROACH                                         PT   152
#                                                                       PT   153
#  IN ORDER TO CHANGE DECIMAL NUMBERS SUPPLIED BY THE USER              PT   154
#  INTO NON-PRINTING CONTROL CHARACTERS IT IS NECESSARY TO DO           PT   155
#  BIT MANIPULATION INTO REAL WORDS.  THE POSITION OF THE LOW           PT   156
#  ORDER BIT OF A CHARACTER IN A 'STUFFED' REAL WORD IS A FUNCTION      PT   157
#  OF THE WAY SUBROUTINE AA89 OF THE NUCLEUS HAS BEEN IMPLIMENTED.      PT   158
#  THIS DEPENDS UPON THE WORD SIZE OF THE TARGET COMPUTING MACHINE AND  PT   159
#  THE LOW ORDER BIT OF A CHARACTER STUFFED IN THE LEFT MOST            PT   160
#  BYTE OF A REAL.  ON A 32 BIT PER WORD 8 BIT PER CHARACTER            PT   161
#  MACHINE BIT 24 OR 0 WILL PROBABLY BE CORRECT.  ON A 36 BIT PER WORD  PT   162
#  9 BIT PER 8 BIT CHARACTER MACHINE, WITH CHARACTERS STUFFED L TO R    PT   163
#  IN THE WORDS, 27 WILL PROBABLY BE CORRECT.                           PT   164
#                                                                       PT   165
macro:(lowordbit:,27)#                 $masp                            PT   166
#                                                                       PT   167
#  STATEMENT(S) FOR TRANSMITTING CHARACTER STRINGS TO THE HIGH          PT   168
#  RESOLUTION PRINTER AS A PLOT.  FOR MANY MACHINES AND PRINTING        PT   169
#  DEVICES THIS MACRO WILL BE THE SAME AS OR SIMILAR TO THE XTAL        PT   170
#  SYSTEM MACRO LINEOUT: ( WHICH REQUIRES LINEFMT: ).                   PT   171
#                                                                       PT   172
#***** DIRECT ACCESS OUTPUT ON PERKIN ELMER                             PT   173
ifsel:(PER)#                                                            PT   174
macro:(sendtodevice:,CALLAA83(ioprt:,Y'28',$3,KIK,0,$2))#               PT   175
#                                                                       PT   176
#***** UNIVAC 1100 TO XEROX-DIABLO TERMINAL OR PRINTRONIX PRINTER.      PT   177
ifsel:(SPE)#                                                            PT   178
macro:(sendtodevice:,$(KIK=($3-1)/mxchwd:+1;CALLAA84(ioprt:,$2,KIK)$))# PT   179
#                                                                       PT   180
#***** STANDARD LINE OUTPUT                                             PT   181
ifsel:#                                                                 PT   182
macro:(sendtodevice:,WRITE(ioprt:,'(A)')$2(1^$3))#                      PT   183
endsel:#                                                                PT   184
#                                                                       PT   185
#                                                                       PT   186
################# MACROS FOR CALCOMP PLOTTER OUTPUT #################   PT   187
#                                                                       PT   188
#  THE FOLLOWING MACROS ARE DESIGNED TO ALLOW FOR THE WRITING OF        PT   189
#  A CALCOMP PLOT TAPE DIRECTLY ON A LOCAL TAPE DRIVE.                  PT   190
#  THE OBJECTIVE IS TO BYPASS ALL LOCAL PLOTTER SUBROUTINE USAGE.       PT   191
#  IN THE COMMENTS WHICH FOLLOW THE NECESSARY STATEMENTS FOR            PT   192
#  A PERKIN-ELMER AND A DEC VAX ARE SHOWN.  THE MACROS ARE SET FOR      PT   193
#  AVOIDING THE WHOLE METHOD.  THE PROCESS IS AVOIDED BY                PT   194
#  USING NDEV=5 OR NDEV=6 AND STAYING AWAY FROM NDEV=3.                 PT   195
#  PLOTTING WILL THEN BE ACCOMPLISHED BY                                PT   196
#  SETTING 'PROPER' LOCAL SUBROUTINE CALLS WHICH WILL BRING IN          PT   197
#  THE LOCAL PLOTTER LIBRARY SUBROUTINES TO CREATE                      PT   198
#  THE PLOT(S) ON DEVICE 5 OR 6.                                        PT   199
#                                                                       PT   200
#                                                                       PT   201
#    MACROS TO INITIALIZE PLOTTER OUTPUT FILE                           PT   202
#    -----------------------------------------                          PT   203
#                                                                       PT   204
ifsel:(CALCOM)#                                                         PT   205
macro:(plotc:,CALLPLOTC($1,$2,$3))## Std calcomp reset origin command   PT   206
ifsel:#                                                                 PT   207
macro:(plotc:,CONTINUE)#                                                PT   208
endsel:#                                                                PT   209
#                                                                       PT   210
ifsel:(CALCOM)#                                                         PT   211
macro:(pageup:,CALLPAGEUP)## Reset origin for ndev=6                    PT   212
ifsel:#                                                                 PT   213
macro:(pageup:,CONTINUE)#                                               PT   214
endsel:#                                                                PT   215
#                                                                       PT   216
ifsel:(VMS)#                                                            PT   217
macro:(startplot:,$## Start vax 780 plotter file                        PT   218
[RETFLG=SYS$$ASSIGN(TAPE,CHANEL,,)#                                     PT   219
IF(RETFLG.NE.%LOC(SS$$$_NORMAL)) $(#                                    PT   220
WRITE(*,110)TAPE,'SYS$$ASSIGN',RETFLG#                                  PT   221
110 FORMAT(' ERROR - ',A,' ON ',A,'. CODE = ',Z9.8,'X. ')#              PT   222
IF(RETFLG.EQ.%LOC(SS$$$_NOSUCHDEV))WRITE(*,115)TAPE#                    PT   223
115 FORMAT(' LOGICAL NAME ',A,' NOT ASSIGNED.')#                        PT   224
iquit:(PT1101)#                        P                                PT   225
$)])#                                                                   PT   226
#                                                                       PT   227
ifsel:#                                                                 PT   228
macro:(startplot:,CONTINUE)## Null plotter tape starter                 PT   229
endsel:#                                                                PT   230
#                                                                       PT   231
#                                                                       PT   232
# MACRO TO WRITE ONE RECORD OF A CALCOMP PLOTTER FILE                   PT   233
#  (AN INDEXED FILE OF RECORD LENGTH 480 BYTES)                         PT   234
#                                                                       PT   235
ifsel:(PER)#                                                            PT   236
macro:(outplot:,[$## Perkin elmer output plot file                      PT   237
CALL SYSIO(IPBLK,Y'29',0,IBUFF(1),480,0)#                               PT   238
CALL IOERR(IPBLK,ISTAT)#                                                PT   239
IF(ISTAT.NE.0) iquit:(PT2201)])#       P                                PT   240
#                                                                       PT   241
ifsel:(VMS)#                                                            PT   242
macro:(outplot:,$## Output vax 780 plot file                            PT   243
[RETFLG=SYS$$QIOW(,%VAL(CHANEL),IO$$$_WRITEBLK,#                        PT   244
STATUS,,,IBUFF,%VAL(480),,,,)#                                          PT   245
IF(RETFLG.NE.%LOC(SS$$$_NORMAL)) $(#                                    PT   246
WRITE(*,*)' ERROR - ',TAPE,' ON WRITEBLK. CODE= ',RETFLG#               PT   247
iquit:(PT0201) $)#                     P                                PT   248
IF(STATUS.NE.%LOC(SS$$$_NORMAL)) $(#                                    PT   249
IF(STATUS.NE.%LOC(SS$$$_WRITLCK))#                                      PT   250
WRITE(*,*)' OUTPUT TAPE IS LOCKED '#                                    PT   251
ELSE WRITE(*,*)' ERROR - ',TAPE,' ON WRITEBLK. CODE= ',RETFLG#          PT   252
iquit:(PT0201) $)])#                   P                                PT   253
#                                                                       PT   254
ifsel:#                                                                 PT   255
macro:(outplot:,CONTINUE)## Null output plot tape                       PT   256
endsel:#                                                                PT   257
#                                                                       PT   258
#                                                                       PT   259
# MACRO TO FINISH OFF PLOTTER FILE                                      PT   260
#                                                                       PT   261
ifsel:(UNKNOWN)#                                                        PT   262
macro:(enplot:,CALLENPLOT)## End plot for ndev=6                        PT   263
ifsel:#                                                                 PT   264
macro:(enplot:,CONTINUE)#                                               PT   265
endsel:#                                                                PT   266
#                                                                       PT   267
ifsel:(VMS)#                                                            PT   268
macro:(stopplot:,$## End vax 780 plot file                              PT   269
[WRITE(*,*)' END FILE PLOT TAPE.'#                                      PT   270
RETFLG=SYS$$QIOW(,%VAL(CHANEL),IOW$$$_WRITEOF,STATUS,,,,,,,,)#          PT   271
IF(RETFLG.NE.%LOC(SS$$$_NORMAL)) $(#                                    PT   272
WRITE(*,110)TAPE,' WRITEOF ',RETFLG#                                    PT   273
110 FORMAT(' ERROR - ',A,' ON ',A,'. CODE = ',Z9.8,'X. ')#              PT   274
iquit:(PT0201) $)#                     P                                PT   275
IF(STATUS.NE.%LOC(SS$$$_NORMAL)) $(#                                    PT   276
WRITE(*,110)TAPE,' WRITEOF ',STATUS#                                    PT   277
iquit:(PT0201) $)])#                   P                                PT   278
#                                                                       PT   279
ifsel:#                                                                 PT   280
macro:(stopplot:,CONTINUE)## Null macro for ending plot file            PT   281
endsel:#                                                                PT   282
#                                                                       PT   283
#                                                                       PT   284
#                                                                       PT   285
#################### GRAPHICS OUTPUT BUFFER SIZE ####################   PT   286
#                                                                       PT   287
macro:(ibufsiz:,1000)## For buffer BUFF used by VDU and LOC1 below      PT   288
#                                                                       PT   289
################## MACROS FOR VDU GRAPHICS DEVICE ##################    PT   290
#                                                                       PT   291
macro:(pixpi:,432.53)#                 Addressable points per inch      PT   292
#                                                                       PT   293
macro:(vduset:,[CALL PT63($1,$2,$3,$4)])# Set tek start                 PT   294
macro:(vdusta:,[CALL PT07(-1,0, 0,0,0)])# Tek start 'tek' mode          PT   295
macro:(vdudrw:,[CALL PT07(1,$1,$2,1,0)])# Tek draw                      PT   296
macro:(vdumov:,[CALL PT07(2,$1,$2,1,0)])# Tek move                      PT   297
macro:(vduind:,[CALL PT07(3,$1, 0,0,1)])# Tek index                     PT   298
macro:(vdubeg:,[CALL PT07(4,$1,$2,1,0)])# Tek begin panel               PT   299
macro:(vduend:,[CALL PT07(5, 0, 0,0,0)])# Tek end panel                 PT   300
macro:(vdufil:,[CALL PT07(6,$1, 0,0,1)])# Tek fill pattern              PT   301
#                                                                       PT   302
macro:(vduout:,[$($#                   Tektronix (4105) vdu             PT   303
IF($2<0)$(movereal:(0.,1,BUFF,1,ibufsiz:,2)# Zero plot buffer           PT   304
opendac:(ioda1:,3,arith:(mxchwd:,*,ibufsiz:),' ')# Open plot file       PT   305
IREC=0$)#                              Reset dac record number          PT   306
ELSE IF(NWRD+$1>$2)$(IREC=IREC+1#      Check buffer is full yet         PT   307
movereal:(0.,1,BUFF,NWRD+1,ibufsiz:-NWRD,2)# Zero plot buffer           PT   308
writedac:(ioda1:,,IREC,BUFF);NWRD=0$)# Op tek word string               PT   309
IF($2==0) closedac:(ioda1:)#           Close the plot file              PT   310
NWRD=NWRD+$1$)])#                      Update tek buffer pointer        PT   311
#                                                                       PT   312
#                                                                       PT   313
##################### MACROS FOR LOCAL DEVICE 1 ####################    PT   314
#                                                                       PT   315
#********* LOCAL ONE DEVICE AS TEK 4662 PLOTTER                         PT   316
#                                                                       PT   317
macro:(adupi:,273.00)#                 Addressable dev. units/inch      PT   318
macro:(pensl:,[CALL PT07(7,$1,0,no:,yes:)])# Plotter pen select         PT   319
macro:(plt1mov:,[CALL PT07(8,$1,$2,yes:,no:)])# Plotter move            PT   320
macro:(plt1drw:,[CALL PT07(-1,$1,$2,yes:,no:)])# Plotter draw           PT   321
#                                                                       PT   322
macro:(plots:,[$($#                    Initialise local device 1 (l     PT   323
plt1out:(0,-1);ploton:;ploton:$)])#    Turn device on twice             PT   324
#                                                                       PT   325
macro:(plot:,[$($#                     Execute pen command or finis     PT   326
IF($3.EQ.2\$3.EQ.3)$(#                 If move or draw command          PT   327
N=NINT($1); M=NINT($2)#                Convert x,y integers             PT   328
CALL PT08(N,M,INDX,$3)$)#              Call loc1 processor              PT   329
ELSE IF($3.EQ.+999)$(#                 If finish plot                   PT   330
plt1finish:$)$)])#                     Finished with local dev. 1       PT   331
#                                                                       PT   332
macro:(plt1finish:,[$($#               Finished with local dev. 1       PT   333
plt1out:(4,ibufsiz:)#                  Increment and check buffer       PT   334
intpak:(457261648,BUFF(NWRD-3),0,mxbtwd:)# Load pen select comnd        PT   335
intpak:(12314,BUFF(NWRD-2),0,mxbtwd:)# Load ascii 0 and term. char      PT   336
intpak:(490040170,BUFF(NWRD-1),0,mxbtwd:)# Load xy destination          PT   337
intpak:(1063190528,BUFF(NWRD),0,mxbtwd:)# Load xy destination           PT   338
plotoff:#                              Turn plotter logically off       PT   339
plt1out:(0,0)$)])#                     Flush graphics buffer            PT   340
#                                                                       PT   341
macro:(ploton:,[$($#                   Turn local plotter (loc1) lo     PT   342
plt1out:(1,ibufsiz:)#                  Check free buffer space          PT   343
intpak:(457262362,BUFF(NWRD),0,mxbtwd:)$)])# Load "plotter on" cmnd.    PT   344
#                                                                       PT   345
macro:(plotoff:,[$($#                  Turn local plotter (loc1) lo     PT   346
plt1out:(1,ibufsiz:)#                  Check free buffer space          PT   347
intpak:(457262618,BUFF(NWRD),0,mxbtwd:)$)])# Load "plotter off" cmnd.   PT   348
#                                                                       PT   349
macro:(plt1out:,[$($#                  Tektronix (4662) plotter         PT   350
IF($2<0)$(movereal:(0.,1,BUFF,1,ibufsiz:,2)# Zero plot buffer           PT   351
opendac:(ioda1:,3,arith:(mxchwd:,*,ibufsiz:),' ')# Open plot file       PT   352
IREC=0$)#                              Reset dac record number          PT   353
ELSE IF(NWRD+$1>$2)$(IREC=IREC+1#      Check buffer is full yet         PT   354
movereal:(0.,1,BUFF,NWRD+1,ibufsiz:-NWRD,2)# Zero plot buffer           PT   355
writedac:(ioda1:,,IREC,BUFF);NWRD=0$)# Op tek word string               PT   356
IF($2==0) closedac:(ioda1:)#           Close the plot file              PT   357
NWRD=NWRD+$1$)])#                      Update tek buffer pointer        PT   358
#                                                                       PT   359
#                                                                       PT   360
####################################################################    PT   361
#                                                                       PT   362
extrude:(PTCOMN)#                                                       PT   363
INTEGER BCOL#                          Bond colour index                PT   364
INTEGER CCOL#                          Character colour index           PT   365
INTEGER COLR(16)#                      Colour index array               PT   366
INTEGER CONTUR#                        Contour flag                     PT   367
INTEGER FILL#                          Fill bonds/atoms flag            PT   368
INTEGER FTPEN#                         First pen select flag            PT   369
INTEGER NCOL#                          Number colour index              PT   370
INTEGER NEW#                           New subject flag                 PT   371
INTEGER NWRD#                          Op buffer word count             PT   372
INTEGER IPBLK(7)#                      For macro tekout $masp           PT   373
INTEGER CHARPI#                        Characters per inch on devic     PT   374
INTEGER CHRCHR#                        Characters per character         PT   375
INTEGER HZPIX#                         Horizontal pixels on device      PT   376
INTEGER IBRDR#                         Flag for drawing border          PT   377
INTEGER ICENTR#                        Flag for auto centering          PT   378
INTEGER IP#                            Packet pointer                   PT   379
INTEGER IP1#                           Mark1 array pointer              PT   380
INTEGER ISCALE#                        Flag for auto scaling            PT   381
INTEGER ISTOP#                         End of qx used                   PT   382
INTEGER LASTX#                         Last x coord of pen              PT   383
INTEGER LASTY#                         Last y coord of pen              PT   384
INTEGER LINEPI#                        Lines per inch on print devi     PT   385
INTEGER LINES#                         Number of vertical pixels        PT   386
INTEGER TOTPIX#                        Number of horiz pixels           PT   387
INTEGER IREC#                          Record number of op file         PT   388
INTEGER MARK#                          Start of plot storage            PT   389
INTEGER MARK1#                         Instruction buffer               PT   390
INTEGER MPLOT#                                                          PT   391
INTEGER MXBTAV#                        Available bits per word          PT   392
INTEGER NOCOL#                         Indicates no colour changes      PT   393
INTEGER NPROL1#                        No. ch in ltor prologue          PT   394
INTEGER SPROL1#                        Starting character               PT   395
INTEGER NPROL2#                        No. ch in rtol prologue          PT   396
INTEGER SPROL2#                        Starting character               PT   397
INTEGER NCLEAR#                        No. of clearing characters       PT   398
INTEGER SCLEAR#                        Starting character               PT   399
INTEGER NEPIL#                         No. ch in final line             PT   400
INTEGER SEPIL#                         Starting character               PT   401
INTEGER NEPIL1#                        No. ch in eeol epilogue          PT   402
INTEGER SEPIL1#                        Starting character               PT   403
INTEGER NEPIL2#                        No. ch in ebuf epilogue          PT   404
INTEGER SEPIL2#                        Starting character               PT   405
INTEGER NPLOTC#                        No. of chars in plot char        PT   406
INTEGER SPLOTC#                        Starting character               PT   407
INTEGER NPRLIM#                        No. ch in prelim line            PT   408
INTEGER SPRLIM#                        Prelim characters                PT   409
INTEGER BUFMAX#                        Maximum characters chrot         PT   410
INTEGER STARBT#                        Starting bit in character        PT   411
INTEGER NUMBIT#                        No. of settable bits in ch.      PT   412
INTEGER DIRBIT#                        Direction of bits in char.       PT   413
INTEGER FRAME#                         Frame count                      PT   414
INTEGER BIDIR#                         Signal printer bidirectional     PT   415
INTEGER NDEV#                          Device number                    PT   416
INTEGER NDOWN#                         Count pen down movements         PT   417
INTEGER NPLOT#                         Plot number                      PT   418
INTEGER NSTEP1#                        Step for mark1 array             PT   419
INTEGER NUP#                           Count pen up movements           PT   420
INTEGER FIRSTT#                        Signal first call of pt11        PT   421
INTEGER VRPIX#                         Verticle pixels on device        PT   422
REAL BUFF(ibufsiz:)#                   Buffer for tektronics op         PT   423
REAL OMAG#                             Magnification factor             PT   424
REAL AHIGH#                            Plot height                      PT   425
REAL AWIDE#                            Plot width                       PT   426
REAL RESL#                             Resolution                       PT   427
REAL SCALX,SCALY#                      Scale to output frame            PT   428
REAL TEM1,TEM2,TEM3,TEM4#              Corners of border                PT   429
REAL XCEN,YCEN#                        X,y coord of center              PT   430
REAL XOFF,YOFF#                        X,y output offset                PT   431
#                                                                       PT   432
string:(DEV2CH,70)#                    Control chars for ndev=2         PT   433
#                                                                       PT   434
INTEGER RETFLG#                        $masp                            PT   435
INTEGER CHANEL#                        $masp                            PT   436
INTEGER STATUS#                        $masp                            PT   437
string:(TAPE,6)#                       $masp                            PT   438
#                                                                       PT   439
COMMON/COMFPT/$#                                                        PT   440
OMAG,TEM1,TEM2,TEM3,TEM4,XCEN,YCEN,$#                                   PT   441
AHIGH,AWIDE,RESL,SCALX,SCALY,XOFF,YOFF,BUFF#                            PT   442
#                                                                       PT   443
COMMON/COMIPT/$#                                                        PT   444
CHARPI,HZPIX,IBRDR,ICENTR,IP,IP1,ISCALE,ISTOP,#                         PT   445
LASTX,LASTY,LINEPI,LINES,TOTPIX,MARK,MARK1,#                            PT   446
MPLOT,MXBTAV,NDEV,NDOWN,NPLOT,NSTEP1,NUP,FIRSTT,VRPIX,#                 PT   447
IREC,RETFLG,CHANEL,STATUS,BIDIR,#                                       PT   448
NPROL1,NPROL2,NCLEAR,NEPIL1,NEPIL2,STARBT,NUMBIT,DIRBIT,FRAME,#         PT   449
SPROL1,SPROL2,SEPIL1,SEPIL2,SCLEAR,CHRCHR,NPLOTC,SPLOTC,#               PT   450
NPRLIM,SPRLIM,BUFMAX,NEPIL,SEPIL,#                                      PT   451
BCOL,CCOL,NCOL,COLR,FILL,FTPEN,#                                        PT   452
NEW,NWRD,IBUF,IPBLK,NOCOL,CONTUR#                                       PT   453
#                                                                       PT   454
COMMON/COMCPT/TAPE,DEV2CH#                                              PT   455
endext:#                                                                PT   456
#                                                                       PT   457
systemheader:(PT00)#                   **** plot ****                   PT00   1
####################################################################    PT00   2
#                                                                  #    PT00   3
#                                PT00                              #    PT00   4
#                                                                  #    PT00   5
####################################################################    PT00   6
SUBROUTINE PT00#                       J.m.stewart & j.r.holden         PT00   7
include:(AACOMN)#                      S.r.hall & a.imerito             PT00   8
include:(PTCOMN)#                                                       PT00   9
#                                                                       PT00  10
#----------------------------------------------------------------       PT00  11
#           EXPECTED PACKET FORMATS                                     PT00  12
#                                                                       PT00  13
#    FUNCTION CODE IS STORED IN WORD QX(IP+7)                           PT00  14
# FUNCTION CODE VALUE      MEANING ASCRIBED TO VALUE                    PT00  15
#        1                  START OF PLOT                               PT00  16
#        2                  FRAME ADVANCE SIGNAL                        PT00  17
#        3                  PEN MOVEMENT                                PT00  18
#        4                  DRAW NUMBER                                 PT00  19
#        5                  DRAW CHARACTER STRING                       PT00  20
#        6                  AUTO CENTER AND SCALE                       PT00  21
#       10                  END OF PLOT                                 PT00  22
#                                                                       PT00  23
# FOR QX(IP+7)=3           (PEN MOVEMENT COMMAND)                       PT00  24
#     QX(IP+1)      DX      PEN MOVEMENT                                PT00  25
#     QX(IP+2)      DY        VECTOR                                    PT00  26
#     QX(IP+3)      X       PEN                                         PT00  27
#     QX(IP+4)      Y       POSITION                                    PT00  28
#     QX(IP+5)      COL     SUBJECT (COLOR) INDEX                       PT00  29
#     QX(IP+6)      NPEN    PEN UP/DOWN FLAG                            PT00  30
#                                                                       PT00  31
# FOR QX(IP+7)=4           (DRAW NUMBER)                                PT00  32
#     QX(IP+1)      X       START OF CHARACTER                          PT00  33
#     QX(IP+2)      Y         (NUMBER) STRING                           PT00  34
#     QX(IP+3)      HT      HEIGHT OF SYMBOLS IN INCHES                 PT00  35
#     QX(IP+4)     THE NUMBER                                           PT00  36
#     QX(IP+5)      TH      ANGLE TO X-AXIS IN DEGREES                  PT00  37
#     QX(IP+6)         INDICATES NUMBER OF DIGITS AFTER DECIMAL PT.     PT00  38
#                                                                       PT00  39
# FOR QX(IP+7)=5           (DRAW CHARACTER)                             PT00  40
#     QX(IP+1)      X       START OF CHARACTER                          PT00  41
#     QX(IP+2)      Y         STRING                                    PT00  42
#     QX(IP+3)      HT      HEIGHT OF SYMBOLS IN INCHES                 PT00  43
#     QX(IP+4)      TH      ANGLE TO X-AXIS IN DEGREES                  PT00  44
#     QX(IP+5)      NS      NUMBER OF CHARACTERS IN STRING (<73)        PT00  45
#     QX(IP+6)      NBLA    DRAW OR SKIP BLANKS                         PT00  46
#                                                                       PT00  47
# FOR QX(IP+7)=6           (AUTO CENTER AND SCALE)                      PT00  48
#     QX(IP+1)              FRAME WIDTH                                 PT00  49
#     QX(IP+2)              FRAME HEIGHT                                PT00  50
#     QX(IP+3)              STEREO FLAG (1=NO; 2=YES)                   PT00  51
#     QX(IP+4)              INTERNAL SCALE                              PT00  52
#     QX(IP+5)              MAXIMUM X-SPAN OF SUBJECT                   PT00  53
#     QX(IP+6)              MAXIMUM Y-SPAN OF SUBJECT                   PT00  54
#-----------------------------------------------------------------      PT00  55
#                                                                       PT00  56
INTEGER IPLOT#                         Count of plots on file           PT00  57
INTEGER NF#                            Plot command                     PT00  58
INTEGER NPEN#                          Pen up/down flag                 PT00  59
INTEGER PSIZE#                         Packet size                      PT00  60
INTEGER I#                             Loop index                       PT00  61
#                                                                       PT00  62
chardata:(PT000,        PEN UP CALLS AND        PEN DOWN CALLS)#        PT00  63
datastuff:#                                                             PT00  64
#                                                                       PT00  65
#   GET PLOT FILE EXTENSION CODE                                        PT00  66
#   ----------------------------                                        PT00  67
#                                                                       PT00  68
FILEXT(IOUNIT(5))='ORT'#               Default plot ip file             PT00  69
FILEXT(ioda1:)   ='PLT'#               Default plot op file             PT00  70
ifnv:(BUFIN(1))#                       Get plot input file              PT00  71
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT00  72
IF(BUFIN(1)>voidflg:) iquit:(PT0002)#  Numeric file ext illegal         PT00  73
stringlim:(1,M,N)#                     Get file extension limits        PT00  74
FILEXT(IOUNIT(5))=CHRIN(M^M+2)#        Store file extension             PT00  75
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT00  76
IPLOT=0#                               Count of plots on file           PT00  77
FIRSTT=yes:#                           Mark first time                  PT00  78
#                                                                       PT00  79
# LOOP OVER STARTP LINES                                                PT00  80
# ----------------------                                                PT00  81
#                                                                       PT00  82
REPEAT#                                Loop over startp lines           PT00  83
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT00  84
CALL PT11#                             User input                       PT00  85
IF(LINID==2) BREAK#                    No more plots                    PT00  86
CALL PT75#                             Output and memory init.          PT00  87
IF(IPLOT>=NPLOT)#                      Plot number out of order         PT00  88
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT00  89
rewindbdf:(5)#                         Rewind plot file                 PT00  90
IPLOT=0#                               Zero the count                   PT00  91
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT00  92
#                                                                       PT00  93
# FIND THE START OF THE COMMANDS ON THE BDF FOR THE DESIRED PLOT        PT00  94
# --------------------------------------------------------------        PT00  95
#                                                                       PT00  96
WHILE(IPLOT<NPLOT)#                    Find the desired plot            PT00  97
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT00  98
REPEAT#                                Loop over packets                PT00  99
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT00 100
readwpkt:(5,23,PSIZE,IP,0)#            Read a packet                    PT00 101
IF(IP<=0) GOTO 999#                    End of plot file                 PT00 102
NF=NINT(QX(IP+7))#                     Plot function                    PT00 103
IF(NF!=1) NEXT#                        1 signals start of plot          PT00 104
IPLOT=IPLOT+1#                         Found a plot, count it           PT00 105
BREAK#                                 Go check it out                  PT00 106
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT00 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT00 108
CALL PT21#                             Initialize plotter               PT00 109
FRAME=0#                               Zero count of frames process     PT00 110
#                                                                       PT00 111
#   LOOP OVER COMMANDS ON FILE E                                        PT00 112
#   ----------------------------                                        PT00 113
#                                                                       PT00 114
NUP=0;    NDOWN=0#                     Zero pen up/down count           PT00 115
REPEAT#                                Loop over plot commands          PT00 116
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT00 117
readwpkt:(5,23,PSIZE,IP,0)#            Read a plot command              PT00 118
IF(IP<=0) BREAK#                       End of logical record            PT00 119
NF=NINT(QX(IP+7))#                     Plot function                    PT00 120
IF(NF==1) iquit:(PT0001)#              Unexpected end of plot           PT00 121
#                                                                       PT00 122
#   PROCESS FRAME ADVANCE                                               PT00 123
#   ---------------------                                               PT00 124
#                                                                       PT00 125
ELSE IF(NF==2)#                        Frame advance signal             PT00 126
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT00 127
IF(FRAME>0)#                           Printer plots = print now        PT00 128
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT00 129
CALL PT61;   IP1=ISTOP#                Process stored commands          PT00 130
IF((NDEV<=2)\(NDEV==4))#               Special for dot type plters      PT00 131
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT00 132
CALL PT71#                             Print hi/lo resol plots          PT00 133
FOR(I=MARK+1;I<=ISTOP;I=I+1)#          Loop over map storage            PT00 134
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PT00 135
intpak:(0,QX(I),0,mxbtwd:)#            Zero all bits                    PT00 136
localdot1:#                            Start printer dot words          PT00 137
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PT00 138
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT00 139
ELSE CALL PT31#                        Advance the frame                PT00 140
IF(IBRDR==yes:)#                       Is there a border....            PT00 141
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PT00 142
CALL PT06(TEM3,TEM4,0.,0.,3,1)#        Redraw border as in              PT00 143
CALL PT06(TEM3,TEM2,0.,-1000.,2,1)#    Pt51                             PT00 144
CALL PT06(TEM1,TEM2,0.,-1000.,2,1)#                                     PT00 145
CALL PT06(TEM1,TEM4,0.,-1000.,2,1)#                                     PT00 146
CALL PT06(TEM3,TEM4,0.,-1000.,2,1)#    Complete the loop                PT00 147
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PT00 148
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT00 149
FRAME=FRAME+1#                         Count each frame                 PT00 150
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT00 151
#                                                                       PT00 152
#   PROCESS PEN MOVE COMMANDS                                           PT00 153
#   -------------------------                                           PT00 154
#                                                                       PT00 155
ELSE IF(NF==3)#                        Pen movement command             PT00 156
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PT00 157
NPEN=NINT(QX(IP+6))#                   Pen up/down flag                 PT00 158
X=QX(IP+1)+QX(IP+3)#                   Get x coordinate                 PT00 159
Y=QX(IP+2)+QX(IP+4)#                   Get y coordinate                 PT00 160
CALL PT06(X,Y,0.,QX(IP+5),NPEN,1)#     Store move/draw command          PT00 161
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PT00 162
#                                                                       PT00 163
#   PROCESS DRAW CHARACTER COMMAND                                      PT00 164
#   ------------------------------                                      PT00 165
#                                                                       PT00 166
ELSE IF((NF==4)\(NF==5)) CALL PT41#    Draw character commands          PT00 167
#                                                                       PT00 168
#   PROCESS AUTO CENTRE/SCALE                                           PT00 169
#   -------------------------                                           PT00 170
#                                                                       PT00 171
ELSE IF(NF==6) CALL PT51#              Auto center and scale            PT00 172
#                                                                       PT00 173
#    PROCESS END PLOT COMMAND                                           PT00 174
#    ------------------------                                           PT00 175
#                                                                       PT00 176
ELSE IF(NF==10)#                       Test for end of plot             PT00 177
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PT00 178
CALL PT61;  IP1=ISTOP#                 Process stored commands          PT00 179
IF(NDEV>=3) CALL PT81#                 End plot sequence                PT00 180
ELSE        CALL PT71#                 Print lo/hi res plot             PT00 181
BREAK#                                 Exit plot loop                   PT00 182
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PT00 183
#                                                                       PT00 184
#   TEST IF STORED COMMAND BUFFER FULL                                  PT00 185
#   ----------------------------------                                  PT00 186
#                                                                       PT00 187
IF(IP1>MARK1)#                         Is instr buffer full             PT00 188
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PT00 189
CALL PT61#                             Process instructions             PT00 190
IP1=ISTOP#                             Reset buffer end                 PT00 191
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PT00 192
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT00 193
#                                                                       PT00 194
#    PRINT PLOT SUMMARY                                                 PT00 195
#    ------------------                                                 PT00 196
#                                                                       PT00 197
movechar:(PT000,1,CHROT,1,NPT000,0)#   Plot summary                     PT00 198
ncodefld:(FLOAT(NUP),1,CHROT,70613.,1)# Encode pen up calls             PT00 199
ncodefld:(FLOAT(NDOWN),1,CHROT,310613.,1)# Encode pen down calls        PT00 200
writeline:(0,0,0,1,3)#                                                  PT00 201
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT00 202
999 CONTINUE#                                                           PT00 203
IF(NDEV==2) localdot5:#                Close off dot file if used       PT00 204
readwpkt:(5,endrecord:,PSIZE,IP,0)#    Close the plot command file      PT00 205
RETURN#                                                                 PT00 206
END#                                                                    PT00 207
systemheader:(PT01)#                   Calcomp plot                     PT01   1
####################################################################    PT01   2
#                                                                  #    PT01   3
#                               PT01                               #    PT01   4
#                                                                  #    PT01   5
####################################################################    PT01   6
SUBROUTINE PT01(XPAGE,YPAGE,IPEN)#                                      PT01   7
#                                                                       PT01   8
#  THIS SUBROUTINE FORMS CHARACTER STRINGS TO BE WRITTEN DIRECTLY       PT01   9
#  ON A TAPE WHICH WILL RUN A CALCOMP PLOTTER. AS SUCH IT IS            PT01  10
#  TOTALLY MACHINE DEPENDENT.  THE ONLY PART THAT IS SINGLED OUT        PT01  11
#  HOWEVER ARE THE ACTUAL TAPE WRITES WHICH ARE DONE IN PT02.           PT01  12
#  THESE WRITES ARE ISOLATED IN MACROS.  PT01 AND PT02                  PT01  13
#  CARRY OUT THE CALL PLOT() OR CALL PLOTC() FUNCTIONS WITHOUT          PT01  14
#  BRINGING IN THE WHOLE CALCOMP LIBRARY AT LOAD TIME.                  PT01  15
#  THEIR PURPOSE IS TO AVOID SPACE PENALITIES.                          PT01  16
#                                        George davenport 1983          PT01  17
#                                                                       PT01  18
INTEGER IS(3)#                                                          PT01  19
realdata:(FACT,1.); realdata:(STEPS,800.); realdata:(STEPS1,800.)#      PT01  20
intdata:(IBLK,1);   intdata:(ICFLG,0);     intdata:(IFFLG,1)#           PT01  21
intdata:(MAX,500);  intdata:(MAXDX,32768); intdata:(MDX,0)#             PT01  22
intdata:(IOLDX,0);  intdata:(IOLDY,0);     intdata:(IOPEN,0)#           PT01  23
intdata:(MDY,0);    intdata:(ISFLG,0);     intdata:(IPNUM,1)#           PT01  24
intdata:(NA,1);     intdata:(NT,0);        intdata:(NB,1)#              PT01  25
intdata:(NS,1);     intdata:(IMFLG,0);     intdata:(NR,1)#              PT01  26
intdata:(ISY,1);    intdata:(ISX,0)#                                    PT01  27
datastuff:#                                                             PT01  28
#                                                                       PT01  29
IS(1)=2**(mxbtch:-1)-1#                                                 PT01  30
IS(2)=32767#                                                            PT01  31
IS(3)=131071#                                                           PT01  32
INEWP=IPEN#                                                             PT01  33
ICOD1=16#                                                               PT01  34
ICOD=2#                                                                 PT01  35
REPEAT#                                                                 PT01  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT01  37
IF(INEWP==IOPEN) BREAK#                                                 PT01  38
IF((INEWP==0)\(INEWP==1)) BREAK#                                        PT01  39
IF(INEWP<0)#                                                            PT01  40
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT01  41
ISFLG=-1#                                                               PT01  42
INEWP=-INEWP#                                                           PT01  43
NEXT#                                                                   PT01  44
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT01  45
ELSE IF(INEWP>2)#                                                       PT01  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT01  47
IF(INEWP>999) GOTO 1000#                                                PT01  48
ELSE IF(INEWP==999)#                                                    PT01  49
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT01  50
ICFLG=-1#                                                               PT01  51
IBLK=9998#                                                              PT01  52
ISFLG=-1#                                                               PT01  53
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT01  54
IF(IOPEN==3) BREAK#                                                     PT01  55
INEWP=3#                                                                PT01  56
ICOD=3#                                                                 PT01  57
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT01  58
CALL PT02(ICOD,1,0)#                                                    PT01  59
IOPEN=INEWP#                                                            PT01  60
BREAK#                                                                  PT01  61
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT01  62
IX=XPAGE*STEPS+SIGN(0.5,XPAGE)#                                         PT01  63
IDX=IX-IOLDX#                                                           PT01  64
IADX=IABS(IDX)#                                                         PT01  65
IOLDX=IX#                                                               PT01  66
IY=YPAGE*STEPS+SIGN(0.5,YPAGE)#                                         PT01  67
IDY=IY-IOLDY#                                                           PT01  68
IADY=IABS(IDY)#                                                         PT01  69
IOLDY=IY#                                                               PT01  70
IF(IADX<IADY) NDIV=IADY/MAXDX#                                          PT01  71
ELSE          NDIV=IADX/MAXDX#                                          PT01  72
IF(NDIV>0)#                                                             PT01  73
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT01  74
NA=NDIV#                                                                PT01  75
NT=NA+NA#                                                               PT01  76
NB=NA#                                                                  PT01  77
NDIV=NB+1#                                                              PT01  78
ISX=ISIGN(1,IDX)#                                                       PT01  79
ISY=ISIGN(1,IDY)#                                                       PT01  80
IDX=IDX/NDIV#                                                           PT01  81
IDY=IDY/NDIV#                                                           PT01  82
NR=2*(IADX-IABS(IDX)*NDIV)#                                             PT01  83
NS=2*(IADY-IABS(IDY)*NDIV)#                                             PT01  84
IMFLG=0#                                                                PT01  85
IADX=IABS(IDX+MDX)#                                                     PT01  86
IADY=IABS(IDY+MDY)#                                                     PT01  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT01  88
#                                                                       PT01  89
100 CONTINUE#                                                           PT01  90
REPEAT#                                                                 PT01  91
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT01  92
IXCNT=0#                                                                PT01  93
IYCNT=0#                                                                PT01  94
IF(IADX>0)#                                                             PT01  95
REPEAT#                                                                 PT01  96
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT01  97
IXCNT=IXCNT+1#                                                          PT01  98
IF(IADX>IS(IXCNT)) NEXT#                                                PT01  99
IF(IADX==IS(IXCNT)) IMFLG=1#                                            PT01 100
BREAK#                                                                  PT01 101
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT01 102
IF(IADY>0)#                                                             PT01 103
REPEAT#                                                                 PT01 104
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT01 105
IYCNT=IYCNT+1#                                                          PT01 106
IF(IADY>IS(IYCNT)) NEXT#                                                PT01 107
IF(IADY==IS(IYCNT)) IMFLG=1#                                            PT01 108
BREAK#                                                                  PT01 109
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT01 110
200 CONTINUE#                                                           PT01 111
REPEAT#                                                                 PT01 112
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT01 113
ICOD=IXCNT+IYCNT+1#                                                     PT01 114
CALL PT02(ICOD,ICOD,-1)#                                                PT01 115
ICOD=4*IXCNT+IYCNT+ICOD1#                                               PT01 116
IF((ICOD!=16)\(ISFLG!=0))#                                              PT01 117
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PT01 118
CALL PT02(ICOD,1,0)#                                                    PT01 119
CALL PT02(IDX+MDX,IXCNT,0)#                                             PT01 120
CALL PT02(IDY+MDY,IYCNT,0)#                                             PT01 121
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PT01 122
MDX=0#                                                                  PT01 123
MDY=0#                                                                  PT01 124
NDIV=NDIV-1#                                                            PT01 125
IF(NDIV<=0)#                                                            PT01 126
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PT01 127
IF((ISFLG>0)&(IPEN<1000)) ISFLG=0#                                      PT01 128
ELSE IF(ISFLG<0)#                                                       PT01 129
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PT01 130
CALL PT02(MAX,MAX,-1)#                                                  PT01 131
ISFLG=0#                                                                PT01 132
IOLDX=0#                                                                PT01 133
IOLDY=0#                                                                PT01 134
IOPEN=0#                                                                PT01 135
IBLK=IBLK+1#                                                            PT01 136
CALL PT02(1,1,0)#                                                       PT01 137
CALL PT02(IBLK,3,0)#                                                    PT01 138
CALL PT02(1024+IPNUM,2,0)#                                              PT01 139
IFFLG=-1#                                                               PT01 140
IF(ICFLG<0)#                                                            PT01 141
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PT01 142
CALL PT02(MAX,MAX,-1)#                                                  PT01 143
ICFLG=0#                                                                PT01 144
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PT01 145
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PT01 146
GOTO 999#                                                               PT01 147
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PT01 148
NA=NA-NR#                                                               PT01 149
NB=NB-NS#                                                               PT01 150
IF(NA<0)#                                                               PT01 151
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PT01 152
MDX=ISX#                                                                PT01 153
NA=NA+NT#                                                               PT01 154
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PT01 155
IF(NB<0)#                                                               PT01 156
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PT01 157
MDY=ISY#                                                                PT01 158
NB=NB+NT#                                                               PT01 159
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PT01 160
IF(IMFLG>0)#                                                            PT01 161
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PT01 162
IADX=IABS(IDX+MDX)#                                                     PT01 163
IADY=IABS(IDY+MDY)#                                                     PT01 164
BREAK#                                 *****************                PT01 165
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PT01 166
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT01 167
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT01 168
#                                                                       PT01 169
1000 CONTINUE#                                                          PT01 170
IF(IPEN<1001)#                                                          PT01 171
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PT01 172
FACT=XPAGE#                                                             PT01 173
STEPS=STEPS1*FACT#                                                      PT01 174
IFFLG=1#                                                                PT01 175
GOTO 999#                                                               PT01 176
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PT01 177
ELSE IF(IPEN==1001)#                                                    PT01 178
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PT01 179
XPAGE=FLOAT(IOLDX)/STEPS#                                               PT01 180
YPAGE=FLOAT(IOLDY)/STEPS#                                               PT01 181
GOTO 999#                                                               PT01 182
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PT01 183
ELSE IF(IPEN==1002)#                                                    PT01 184
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PT01 185
XPAGE=FACT#                                                             PT01 186
GOTO 999#                                                               PT01 187
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PT01 188
ELSE IF((IPEN==1003)\(IPEN==1004))#                                     PT01 189
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PT01 190
IF((IFFLG!=0)\(IPEN!=1004))#                                            PT01 191
REPEAT#                                                                 PT01 192
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PT01 193
IDX=XPAGE*STEPS+SIGN(0.5,XPAGE)#                                        PT01 194
IDY=YPAGE*STEPS+SIGN(0.5,YPAGE)#                                        PT01 195
XPAGE=FLOAT(IDX)/STEPS#                                                 PT01 196
YPAGE=FLOAT(IDY)/STEPS#                                                 PT01 197
IADX=IABS(IDX)#                                                         PT01 198
IADY=IABS(IDY)#                                                         PT01 199
NDIV=1#                                                                 PT01 200
IFFLG=0#                                                                PT01 201
ICOD1=32#                                                               PT01 202
ISFLG=1#                                                                PT01 203
IF((3*MAX0(IADX,IADY))<=MAXDX) GOTO 100#                                PT01 204
IF(IFFLG==0) BREAK#                                                     PT01 205
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PT01 206
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PT01 207
ELSE IF(IPEN>=1005)#                                                    PT01 208
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PT01 209
IOPEN=0#                                                                PT01 210
IPNUM=IFIX(XPAGE)#                                                      PT01 211
GOTO 999#                                                               PT01 212
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PT01 213
IDX=0#                                                                  PT01 214
IDY=0#                                                                  PT01 215
IADX=0#                                                                 PT01 216
IADY=0#                                                                 PT01 217
IXCNT=0#                                                                PT01 218
IYCNT=0#                                                                PT01 219
ICOD1=32#                                                               PT01 220
ISFLG=1#                                                                PT01 221
NDIV=1#                                                                 PT01 222
GOTO 200#                                                               PT01 223
999 CONTINUE#                                                           PT01 224
RETURN#                                                                 PT01 225
END#                                                                    PT01 226
systemheader:(PT02)#                   Calcomp buff                     PT02   1
####################################################################    PT02   2
#                                                                  #    PT02   3
#                                 PT02                             #    PT02   4
#                                                                  #    PT02   5
####################################################################    PT02   6
SUBROUTINE PT02(LOC,NCNT,ICNT)#                                         PT02   7
include:(AACOMN)#                                                       PT02   8
include:(PTCOMN)#                                                       PT02   9
#                                                                       PT02  10
#  THIS SUBROUTINE, CALLED BY PT01 AND PT21,                            PT02  11
#  WRITES CALCOMP DRIVER INSTRUCTIONS                                   PT02  12
#  TO A MAGNETIC TAPE. ALSO SEE THE MACROS OF PT FOR                    PT02  13
#  STARTPLOT:, STOPPLOT:, AND OUTPLOT:.                                 PT02  14
#                                    GEORGE DAVENPORT 1983              PT02  15
#                                                                       PT02  16
INTEGER IBUFF(120),LOC(1)#                                              PT02  17
intdata:(ISH,[16777216,65536,256,1])#                                   PT02  18
intdata:(ISH1,[268435456,2097152,16384,128,1])#                         PT02  19
intdata:(ILOC2,0);   intdata:(JMAX,120);   intdata:(JSH,4)#             PT02  20
intdata:(J,2);       intdata:(K,0);        intdata:(NK,0)#              PT02  21
intdata:(NCW,4);     intdata:(NCWP,5);     intdata:(NCWP2,6)#           PT02  22
intdata:(IPF,0);     intdata:(ILOC,0)#                                  PT02  23
datastuff:#                                                             PT02  24
#                                                                       PT02  25
JCNT=IABS(NCNT)#                                                        PT02  26
IBUFF(1)=521738527#                                                     PT02  27
IF(ICNT<(-2)) GOTO 999#                                                 PT02  28
IF(ICNT==(-2))#                                                         PT02  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT02  30
LTAPE=LOC(1)#                                                           PT02  31
GOTO 999#                                                               PT02  32
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT02  33
IF(ICNT==(-1))#                                                         PT02  34
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT02  35
IF(((JMAX-J)*NCW+JSH)>JCNT) GOTO 999#                                   PT02  36
JCNT=0#                                                                 PT02  37
ILOC1=15#                                                               PT02  38
GOTO 200#                                                               PT02  39
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT02  40
IF(ICNT==0)#                                                            PT02  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT02  42
IF(JCNT==0) GOTO 999#                                                   PT02  43
K=NCWP-JCNT#                                                            PT02  44
IPF=0#                                                                  PT02  45
ILOC=LOC(1)#                                                            PT02  46
GOTO 100#                                                               PT02  47
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT02  48
IF(ICNT>0)#                                                             PT02  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT02  50
K=NCWP#                                                                 PT02  51
ILOC1=JCNT#                                                             PT02  52
JCNT=JCNT+1#                                                            PT02  53
IF(ILOC1==0)#                                                           PT02  54
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT02  55
JCNT=2#                                                                 PT02  56
NK=MOD(LOC(1),128)#                                                     PT02  57
ILOC1=1#                                                                PT02  58
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT02  59
GOTO 200#                                                               PT02  60
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT02  61
#                                                                       PT02  62
100 CONTINUE#                                                           PT02  63
IF(IPF!=0)#                                                             PT02  64
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT02  65
ILOC1=ILOC/ISH1(K)#                                                     PT02  66
ILOC=ILOC-ILOC1*ISH1(K)#                                                PT02  67
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT02  68
ELSE#                                                                   PT02  69
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT02  70
ILOC1=ILOC/ISH(K)#                                                      PT02  71
ILOC=ILOC-ILOC1*ISH(K)#                                                 PT02  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT02  73
IF(ILOC<0) ILOC1=ILOC1-1#                                               PT02  74
IF(IPF!=0)#                                                             PT02  75
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT02  76
IF(ILOC1<0) ILOC1=ILOC1+128#                                            PT02  77
ILOC1=MOD(ILOC1,128)#                                                   PT02  78
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT02  79
ELSE#                                                                   PT02  80
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT02  81
IF(ILOC1<0) ILOC2=ILOC2+1#                                              PT02  82
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT02  83
#                                                                       PT02  84
200 CONTINUE#                                                           PT02  85
ILOC2=ILOC2*256+ILOC1#                                                  PT02  86
JSH=JSH-1#                                                              PT02  87
IF(JSH<1)#                                                              PT02  88
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PT02  89
IBUFF(J)=ILOC2#                                                         PT02  90
J=J+1#                                                                  PT02  91
ILOC2=0#                                                                PT02  92
JSH=NCW#                                                                PT02  93
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PT02  94
ELSE IF(JSH==1)#                                                        PT02  95
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PT02  96
IF(J==JMAX)#                                                            PT02  97
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PT02  98
ILOC2=ILOC2*256+15#                                                     PT02  99
IBUFF(J)=ILOC2#                                                         PT02 100
#                                                                       PT02 101
#  THE FOLLOWING MACRO OF THE PT PROGRAM MUST BE SET TO                 PT02 102
#  MAKE A DIRECT WRITE OF THE CALCOMP DRIVER INFORMATION TO             PT02 103
#  A MAGNETIC TAPE TO BE USED ON THE CALCOMP PLOTTER.                   PT02 104
#  THIS IS A PLOY TO AVOID USING LIBRARY UTILITY ROUTINES.              PT02 105
#  IT MAY BE IGNORED BY SETTING THE MACROS TO 'CONTINUE'                PT02 106
#  AND SETTING THE NDEV=5 OR 6 CALLS.  SEE THE PT MACROS.               PT02 107
#  -----------------------------------------------------                PT02 108
#                                                                       PT02 109
outplot:#                              $masp                            PT02 110
J=2#                                                                    PT02 111
ILOC2=0#                                                                PT02 112
JSH=NCW#                                                                PT02 113
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PT02 114
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PT02 115
300 CONTINUE#                                                           PT02 116
JCNT=JCNT-1#                                                            PT02 117
IF(JCNT==0) GOTO 999#                                                   PT02 118
IF(JCNT<0)#                                                             PT02 119
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PT02 120
IF(JSH!=NCW) GOTO 200#                                                  PT02 121
JCNT=1#                                                                 PT02 122
IF(J<=2) GOTO 999#                                                      PT02 123
outplot:#                              $masp                            PT02 124
J=2#                                                                    PT02 125
ILOC2=0#                                                                PT02 126
JSH=NCW#                                                                PT02 127
GOTO 300#                                                               PT02 128
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PT02 129
K=K+1#                                                                  PT02 130
IF(IPF!=0) KKK=NCWP2#                                                   PT02 131
ELSE       KKK=NCWP#                                                    PT02 132
IF(K<KKK) GOTO 100#                                                     PT02 133
IF(K>KKK)#                                                              PT02 134
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PT02 135
IPF=0#                                                                  PT02 136
IF(NCNT<0) NK=NCWP#                                                     PT02 137
ELSE IF(NCNT>0) NK=1#                                                   PT02 138
ELSE#                                                                   PT02 139
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PT02 140
ILOC1=NK#                                                               PT02 141
GOTO 200#                                                               PT02 142
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PT02 143
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PT02 144
K=NK#                                                                   PT02 145
IPF=IPF+1#                                                              PT02 146
ILOC=LOC(IPF)#                                                          PT02 147
IF(NCNT>0) ILOC=ILOC/2#                                                 PT02 148
GOTO 100#                                                               PT02 149
#                                                                       PT02 150
999 CONTINUE#                                                           PT02 151
RETURN#                                                                 PT02 152
END#                                                                    PT02 153
systemheader:(PT05)#                   Draw a character symbol          PT05   1
####################################################################    PT05   2
#                                                                  #    PT05   3
#                           PT05 -- PLOTSY                         #    PT05   4
#                                                                  #    PT05   5
####################################################################    PT05   6
SUBROUTINE PT05(N,NPLUM,DELX,DELY,X,Y,LCAR,CIND,SPFLAG)#                PT05   7
include:(AACOMN)#                                                       PT05   8
include:(PTCOMN)#                                                       PT05   9
macro:(mvchr:,movereal:($1,$2*20+1,NTABCH,1,20,0))#                     PT05  10
#                                      J. hester apr 1990               PT05  11
REAL    DELX#       ## STEPSIZE IN X-DIRECTION                          PT05  12
REAL    DELY#       ## STEPSIZE IN Y-DIRECTION                          PT05  13
REAL    CIND#       ## SUBJECT INDEX                                    PT05  14
REAL    DX#         ## X RELATIVE TO STRING ORIGIN                      PT05  15
REAL    DXC#        ## X RELATIVE TO CHAR/NUMB ORIGIN                   PT05  16
REAL    DY#         ## Y RELATIVE TO STRING ORIGIN                      PT05  17
REAL    DYC#        ## Y RELATIVE TO CHAR/NUMB ORIGIN                   PT05  18
REALOFFSTX#         ## offset in x direction                            PT05  19
REALOFFSTY#         ## offset in y direction                            PT05  20
INTEGER SPFLAG#     ## FLAG FOR SUPER/SUBSCRIPT                         PT05  21
INTEGER LCAR#       ## CHAR/NUMB NUMBER - 1                             PT05  22
INTEGER N#          ## Character position in list                       PT05  23
INTEGER NTABCH(20)# ## ARRAY TO HOLD CURRENT CHAR VECTORS               PT05  24
INTEGER II#         ## LOOP VARIABLE                                    PT05  25
INTEGER IVEC#       ## ENCODED VECTOR                                   PT05  26
INTEGER NPLUM#      ## 2/3 FOR PEN DOWN/UP (initially)                  PT05  27
INTEGER PEN#        ## 2/3 FOR PEN DOWN/UP                              PT05  28
INTEGER IX,IY#      ## TRANS X,Y GRID COORDINATES OF PLOT VECTOR        PT05  29
REALXV,YV#          ## TRUE  GRID COORDINATES                           PT05  30
REAL    X#          ## X OF STRING ORIGIN                               PT05  31
REAL    Y#          ## Y OF STRING ORIGIN                               PT05  32
#                                                                       PT05  33
## TABCH? TABLE REFERS TO CHARACTERS                                    PT05  34
#                                                                       PT05  35
intdata:(TABCH1,[$#                    Upper case abcdefghi             PT05  36
7,65,73,75,69,27,144,48,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05  37
11,72,75,69,62,54,51,154,48,34,26,23, 0,0,0,0,0,0,0,0,#                 PT05  38
8,169,75,73,65,30,24,26,34,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  39
6,72,75,69,34,26,23,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  40
6,127,23,72,76,151,55,0,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05  41
4,72,76,151,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05  42
12,169,75,73,65,30,24,26,34,41,47,146,48,0,0,0,0,0,0,0,#                PT05  43
5,72,176,27,151,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  44
6,124,26,125,74,173,75,0,0,0,0,0,0,0,0,0,0,0,0,0])#                     PT05  45
#                                                                       PT05  46
intdata:(TABCH2,[$#                    Upper case jklmnopqr             PT05  47
7,130,24,25,33,75,172,76,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  48
5,72,144,76,152,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  49
3,172,23,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05  50
4,72,53,76,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05  51
3,72,27,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                            PT05  52
9,130,65,73,75,69,34,26,24,30,0,0,0,0,0,0,0,0,0,0,#                     PT05  53
6,72,75,69,62,54,51,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  54
11,130,65,73,75,69,34,26,24,30,139,27,0,0,0,0,0,0,0,0,#                 PT05  55
8,72,75,69,62,54,51,153,27,0,0,0,0,0,0,0,0,0,0,0])#                     PT05  56
#                                                                       PT05  57
intdata:(TABCH3,[$#                    Upper case   stuvwxyza           PT05  58
 12,169,75,73,65,58,52,47,41,34,26,24,30,0,0,0,0,0,0,0,#                PT05  59
4,172,76,174,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  60
6,172,30,24,26,34,76,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  61
3,172,25,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05  62
5,172,24,46,26,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  63
4,172,27,176,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  64
5,172,53,76,153,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  65
4,172,76,23,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05  66
11,133,25,24,30,44,52,53,47,154,26,27,0,0,0,0,0,0,0,0])#                PT05  67
#                                                                       PT05  68
intdata:(TABCH4,[$#                    Lower case bcdefghij             PT05  69
 9,72,144,52,53,47,33,25,24,30,0,0,0,0,0,0,0,0,0,0,#                    PT05  70
8,147,53,52,44,30,24,25,33,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  71
10,133,25,24,30,44,52,53,47,175,26,0,0,0,0,0,0,0,0,0,#                  PT05  72
10,137,40,47,53,52,44,30,24,25,33,0,0,0,0,0,0,0,0,0,#                   PT05  73
7,169,75,74,66,24,151,53,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  74
13,133,25,24,30,44,52,53,47,154,12,4,3,9,0,0,0,0,0,0,#                  PT05  75
8,72,144,52,53,47,40,33,26,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  76
7,123,25,124,52,51,159,59,0,0,0,0,0,0,0,0,0,0,0,0,#                     PT05  77
8,161,61,153,54,12,4,3,9,0,0,0,0,0,0,0,0,0,0,0])#                       PT05  78
#                                                                       PT05  79
intdata:(TABCH5,[$#                    Lower case  k l m n o p q r s    PT05  80
 6,72,137,38,26,138,54,0,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  81
5,172,73,24,123,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  82
9,51,144,52,46,25,146,54,48,27,0,0,0,0,0,0,0,0,0,0,#                    PT05  83
6,51,144,52,53,47,26,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  84
9,130,44,52,53,47,33,25,24,30,0,0,0,0,0,0,0,0,0,0,#                     PT05  85
10,151,2,144,52,53,47,33,25,24,30,0,0,0,0,0,0,0,0,0,#                   PT05  86
11,130,44,52,53,47,33,25,24,30,154,5,0,0,0,0,0,0,0,0,#                  PT05  87
5,51,144,52,53,47,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  88
10,130,24,26,34,40,38,44,52,54,48,0,0,0,0,0,0,0,0,0])#                  PT05  89
#                                                                       PT05  90
intdata:(TABCH6,[$#                    Lower case t u v w x y z 0 1     PT05  91
 6,173,31,25,26,151,54,0,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05  92
8,151,30,24,25,33,154,26,27,0,0,0,0,0,0,0,0,0,0,0,#                     PT05  93
6,151,37,24,25,40,54,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05  94
9,151,30,24,32,46,132,26,34,55,0,0,0,0,0,0,0,0,0,0,#                    PT05  95
4,151,26,154,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05  96
11,151,30,24,25,33,54,133,12,4,3,9,0,0,0,0,0,0,0,0,#                    PT05  97
4,151,54,23,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05  98
11,130,65,73,75,69,34,26,24,30,123,76,0,0,0,0,0,0,0,0,#                 PT05  99
5,166,74,25,124,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0])#                       PT05 100
#                                                                       PT05 101
intdata:(TABCH7,[$#                    2 3 4 5 6 7 8 9 +                PT05 102
 8,165,73,75,69,62,30,23,27,0,0,0,0,0,0,0,0,0,0,0,#                     PT05 103
13,165,73,75,69,62,54,52,154,48,34,26,24,30,0,0,0,0,0,0,#               PT05 104
4,126,75,37,41,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 105
9,130,24,26,34,48,54,51,72,76,0,0,0,0,0,0,0,0,0,0,#                     PT05 106
12,169,75,73,65,30,24,26,34,48,54,52,44,0,0,0,0,0,0,0,#                 PT05 107
5,165,72,76,69,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 108
16,152,44,30,24,26,34,48,54,52,58,65,73,75,69,62,54,0,0,0,#             PT05 109
12,130,24,26,34,69,75,73,65,51,45,47,55,0,0,0,0,0,0,0,#                 PT05 110
4,160,32,144,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])#                        PT05 111
#                                                                       PT05 112
intdata:(TABCH8,[$#                    - * / ( ) = , . %                PT05 113
 2,144,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05 114
8,160,32,144,48,158,34,130,62,0,0,0,0,0,0,0,0,0,0,0,#                   PT05 115
1,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                              PT05 116
4,173,65,30,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 117
4,172,66,31,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 118
4,144,48,130,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 119
6,30,31,24,23,124,16,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 120
4,30,31,24,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05 121
12,165,72,73,66,65,126,33,34,27,26,123,76,0,0,0,0,0,0,0])#              PT05 122
#                                                                       PT05 123
intdata:(TABCH9,[$#                    '  ^ ~ upper case (alpha) etc    PT05 124
4,172,58,73,72,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 125
3,158,74,62,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05 126
4,144,52,46,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 127
7,65,73,75,69,27,144,48,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05 128
11,72,75,69,62,54,51,154,48,34,26,23,0,0,0,0,0,0,0,0,#                  PT05 129
4,172,27,176,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 130
3,74,27,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                            PT05 131
6,127,23,72,76,151,55,0,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05 132
15,131,37,58,66,68,62,41,33,31,124,26,125,74,173,75,0,0,0,0])#          PT05 133
#                                                                       PT05 134
##(epsilon) (eta) (iota) (ascii j) (kappa) (lambda) (mu)                PT05 135
##(nu) (omicron)                                                        PT05 136
intdata:(TABC10,[$#                                                     PT05 137
5,172,76,69,173,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 138
5,72,176,27,151,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 139
6,124,26,125,74,173,75,0,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05 140
7,130,24,25,33,75,172,76,0,0,0,0,0,0,0,0,0,0,0,0,#                      PT05 141
5,72,144,76,152,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 142
2,74,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                             PT05 143
4,72,53,76,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                           PT05 144
3,72,27,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                            PT05 145
9,130,65,73,75,69,34,26,24,30,0,0,0,0,0,0,0,0,0,0])#                    PT05 146
#                                                                       PT05 147
#(PI) (THETA) (RHO) (SIGMA) (TAU) (UPSILON)                             PT05 148
#(...ANGSTROM...) (OMEGA) (XI)                                          PT05 149
intdata:(TABC11,[$#                                                     PT05 150
8,124,73,165,72,76,69,175,26,0,0,0,0,0,0,0,0,0,0,0,#                    PT05 151
15,124,30,65,73,75,69,34,26,24,159,45,152,54,161,47,0,0,0,0,#           PT05 152
6,72,75,69,62,54,51,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 153
7,134,27,23,53,72,76,69,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05 154
4,172,76,174,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 155
5,172,53,76,153,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 156
11,44,74,48,27,148,44,174,80,88,82,74,0,0,0,0,0,0,0,0,#                 PT05 157
12,130,23,24,37,58,73,75,62,41,26,27,34,0,0,0,0,0,0,0,#                 PT05 158
10,130,23,27,34,152,54,165,72,76,69,0,0,0,0,0,0,0,0,0])#                PT05 159
#                                                                       PT05 160
#(PSI) (ZETA) LOWER CASE (ALPHA) (BETA) (CHI) (DELTA) (EPSILON) (PHI)   PT05 161
#          (GAMMA)                                                      PT05 162
intdata:(TABC12,[$#                                                     PT05 163
12,172,51,45,47,55,76,124,26,125,74,173,75,0,0,0,0,0,0,0,#              PT05 164
4,172,76,23,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                          PT05 165
10,155,33,25,24,30,44,52,53,47,27,0,0,0,0,0,0,0,0,0,#                   PT05 166
13,109,58,66,67,61,53,52,153,47,33,25,24,30,0,0,0,0,0,0,#               PT05 167
6,158,66,26,27,123,69,0,0,0,0,0,0,0,0,0,0,0,0,0,#                       PT05 168
13,153,52,44,30,24,25,33,47,53,58,65,67,61,0,0,0,0,0,0,#                PT05 169
11,133,25,24,30,38,39,138,44,52,53,47, 0,0,0,0,0,0,0,0,#                PT05 170
11,124,30,44,52,54,48,34,26,24,167,11, 0,0,0,0,0,0,0,0,#                PT05 171
8,151,44,26,19,11,25,41,55, 0,0,0,0,0,0,0,0,0,0,0])#                    PT05 172
#                                                                       PT05 173
#LOWER CASE (ETA) (IOTA) (ASCII J) (KAPPA) (LAMBDA)                     PT05 174
#           (MU)                                                        PT05 175
#  (NU) (OMICRON) (PI)                                                  PT05 176
intdata:(TABC13,[$#                                                     PT05 177
6,51,144,52,53,47,12,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 178
5,124,26,125,53,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 179
8,161,61,153,54,12,4,3,9,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 180
6,51,147,53,37,25,26,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 181
6,53,165,66,53,33,27,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 182
10,109,51,130,24,25,33,54,133,26,27,0,0,0,0,0,0,0,0,0,#                 PT05 183
5,151,24,33,48,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,#                         PT05 184
9,124,30,44,52,53,47,33,25,24,0,0,0,0,0,0,0,0,0,0,#                     PT05 185
8,124,52,144,51,55,48,154,26,0,0,0,0,0,0,0,0,0,0,0])#                   PT05 186
#                                                                       PT05 187
# LOWER CASE (THETA) (RHO) (SIGMA) (TAU)                                PT05 188
#          (UPSILON) (ASCII V) (OMEGA) (XI) (PSI)                       PT05 189
intdata:(TABC14,[$#                                                     PT05 190
11,124,37,51,66,68,55,41,26,24,144,48, 0,0,0,0,0,0,0,0,#                PT05 191
9,109,44,52,53,47,33,25,24,30,0,0,0,0,0,0,0,0,0,0,#                     PT05 192
9,153,47,33,25,24,30,44,52,55,0,0,0,0,0,0,0,0,0,0,#                     PT05 193
8,144,51,55,48,153,25,26,34,0,0,0,0,0,0,0,0,0,0,0,#                     PT05 194
6,151,30,24,25,33,54,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 195
6,151,37,24,25,40,54,0,0,0,0,0,0,0,0,0,0,0,0,0,#                        PT05 196
11,152,44,30,24,32,39,132,26,34,48,54, 0,0,0,0,0,0,0,0,#                PT05 197
15,158,60,68,67,51,45,46,145,37,30,24,25,19,11,10,0,0,0,0,#             PT05 198
8,151,30,24,26,34,55,167,11,0,0,0,0,0,0,0,0,0,0,0])#                    PT05 199
#                                                                       PT05 200
# LOWER CASE (ZETA)                                                     PT05 201
intdata:(TABC15,[$#                                                     PT05 202
12,158,60,68,67,59,37,30,24,25,19,11,10,0,0,0,0,0,0,0])#                PT05 203
datastuff:#                                                             PT05 204
#                                                                       PT05 205
OFFSTX = -SPFLAG*3*DELX#               Calculate offset                 PT05 206
OFFSTY = SPFLAG*3*DELY#                Calculate offset                 PT05 207
IF(N>127) N = N - 75#                  If n too large, make normal      PT05 208
#                                                                       PT05 209
#  FIND ORIGIN FOR THIS CHARACTER                                       PT05 210
#                                                                       PT05 211
DXC=FLOAT(LCAR)*6.*DELY+OFFSTX#        X - origin of char/numb          PT05 212
DYC=FLOAT(LCAR)*6.*DELX+OFFSTY#        Y - origin of char/numb          PT05 213
CALL PT06(X+DXC,Y+DYC,0.,CIND,NPLUM,1)# First pen move                  PT05 214
#                                                                       PT05 215
# FIND CORRECT TABLE                                                    PT05 216
# ---------------------                                                 PT05 217
#                                                                       PT05 218
NA = INT((N-1)/9)+1#                   Number of array                  PT05 219
ND = N - (NA-1)*9 -1#                  Position in array                PT05 220
IF(NA==1) mvchr:(TABCH1,ND)#           Table 1                          PT05 221
ELSEIF(NA==2) mvchr:(TABCH2,ND)#       Table 2                          PT05 222
ELSEIF(NA==3) mvchr:(TABCH3,ND)#       Table 3                          PT05 223
ELSEIF(NA==4) mvchr:(TABCH4,ND)#       Table 4                          PT05 224
ELSEIF(NA==5) mvchr:(TABCH5,ND)#       Table 5                          PT05 225
ELSEIF(NA==6) mvchr:(TABCH6,ND)#       Table 6                          PT05 226
ELSEIF(NA==7) mvchr:(TABCH7,ND)#       Table 7                          PT05 227
ELSEIF(NA==8) mvchr:(TABCH8,ND)#       Table 8                          PT05 228
ELSEIF(NA==9) mvchr:(TABCH9,ND)#       Table 9                          PT05 229
ELSEIF(NA==10) mvchr:(TABC10,ND)#      Table 10                         PT05 230
ELSEIF(NA==11) mvchr:(TABC11,ND)#      Table 11                         PT05 231
ELSEIF(NA==12) mvchr:(TABC12,ND)#      Table 12                         PT05 232
ELSEIF(NA==13) mvchr:(TABC13,ND)#      Table 13                         PT05 233
ELSEIF(NA==14) mvchr:(TABC14,ND)#      Table 14                         PT05 234
ELSEIF(NA==15) mvchr:(TABC15,ND)#      Table 15                         PT05 235
#                                                                       PT05 236
FOR(II= 2;II<2 + NTABCH(1);II=II+1)#   Loop over vectors                PT05 237
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT05 238
IF(NTABCH(II) > 100)#                                                   PT05 239
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT05 240
PEN  = 3#                              Is a move; pen up                PT05 241
IVEC = NTABCH(II)-100#                 Find move destination            PT05 242
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT05 243
ELSE#                                                                   PT05 244
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT05 245
PEN  = 2#                              Pen down for drawing             PT05 246
IVEC = NTABCH(II)#                     Copy vector                      PT05 247
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT05 248
IY = INT(IVEC/7)#                      Find y position                  PT05 249
IX = IVEC-7*IY#                        Find x position                  PT05 250
XV = FLOAT(IX-2)#                      Translate to grid                PT05 251
YV = FLOAT(IY-3)#                      Y grid coord                     PT05 252
DX = XV*DELY-YV*DELX + DXC#            Total character movement         PT05 253
DY = XV*DELX+YV*DELY + DYC#            Total character movement         PT05 254
CALL PT06(X+DX,Y+DY,0.,CIND,PEN  ,1)#  Call draw                        PT05 255
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT05 256
#                                                                       PT05 257
RETURN#                                                                 PT05 258
END#                                                                    PT05 259
systemheader:(PT06)#                   Store a pen movement             PT06   1
####################################################################    PT06   2
#                                                                  #    PT06   3
#                               PT06                               #    PT06   4
#                                                                  #    PT06   5
####################################################################    PT06   6
SUBROUTINE PT06(X,Y,SPARE,CIND,NPEN,IFLAG)#                             PT06   7
include:(AACOMN)#                                                       PT06   8
include:(PTCOMN)#                                                       PT06   9
#                                                                       PT06  10
INTEGER NPEN#                          Move/draw flag                   PT06  11
INTEGER IFLAG#                         Flag for calcomp                 PT06  12
REAL X, Y#                             X & y coords. (inches)           PT06  13
REAL SPARE#                            Unused parameter                 PT06  14
REAL DX, DY#                           Change in x or y                 PT06  15
realdata:(XLST,-9999.);  realdata:(YLST,-9999.)#                        PT06  16
datastuff:#                                                             PT06  17
#                                                                       PT06  18
DX=ABS(X-XLST)#                        Change in x dirn.                PT06  19
DY=ABS(Y-YLST)#                        Change in y dirn.                PT06  20
IF((DX+DY>=RESL)\((NPEN==3)&(NINT(CIND)==0)))#                          PT06  21
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT06  22
QX(IP1+1)=X#                           X-coord                          PT06  23
QX(IP1+2)=Y#                           Y-coord                          PT06  24
QX(IP1+3)=SPARE#                       Not used yet                     PT06  25
QX(IP1+4)=CIND#                        Subject index                    PT06  26
QX(IP1+5)=FLOAT(NPEN)#                 Pen up/down                      PT06  27
QX(IP1+6)=FLOAT(IFLAG)#                Flag for calcomp routines        PT06  28
IP1=IP1+NSTEP1#                        Add to pointer                   PT06  29
XLST=X#                                Remember x position              PT06  30
YLST=Y#                                Remember y position              PT06  31
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT06  32
RETURN#                                                                 PT06  33
END#                                                                    PT06  34
systemheader:(PT07)#                   Execute tektronix commands       PT07   1
####################################################################    PT07   2
#                                                                  #    PT07   3
#                              PT07                                #    PT07   4
#                                                                  #    PT07   5
####################################################################    PT07   6
SUBROUTINE  PT07(IND,X,Y,J1,J2)#       S.r. hall and a. imerito         PT07   7
include:(AACOMN)#                                                       PT07   8
include:(PTCOMN)#                                                       PT07   9
#                                                                       PT07  10
# $MASP THIS ROUTINE HAS BEEN WRITTEN SPECIFICALLY FOR THE              PT07  11
#       TEKTRONIX 4105 COLOUR TERMINAL AND 4662 INTERACTIVE             PT07  12
#       DIGITAL PLOTTER. IT WILL PROBABLY RUN ON THE 4107               PT07  13
#       AND 4010 SERIES TERMINALS. OTHER DEVICES WILL RE-               PT07  14
#       QUIRE DIFFERENT COMMAND STRINGS AND ENCODING OF                 PT07  15
#       BITS BUT WILL PROBABLY USE A SIMILAR PROCEDURE TO               PT07  16
#       THAT USED HERE. NOTE THAT THIS ROUTINE, AND PT62                PT07  17
#       AND PT63, USE THE VDU IN TERMINAL COMMAND MODE,                 PT07  18
#       NOT PIXEL ADDRESSABLE MODE. THE 4662 IS USED IN A               PT07  19
#       MODE WHICH IS SIMILAR TO VECTOR MODE ON THE TEK-                PT07  20
#       TRONIX VDUS.                                                    PT07  21
#                                                                       PT07  22
INTEGER X,Y#                           Coords or colour index           PT07  23
INTEGER IND#                           Command index                    PT07  24
INTEGER J1,J2#                         Command flags                    PT07  25
INTEGER PKINT#                         Packed integer                   PT07  26
intdata:(ICOM,[$#                                                       PT07  27
457983744,#                            Draw x,y                         PT07  28
457983488,#                            Move x,y                         PT07  29
458050560,#                            Set index                        PT07  30
457986048,#                            Begin panel                      PT07  31
457983232,#                            End panel                        PT07  32
458051584,#                            Set fill pattern                 PT07  33
453001808,#                            Plotter pen select               PT07  34
29])#                                  Plotter move                     PT07  35
datastuff:#                                                             PT07  36
#                                                                       PT07  37
#   MOVE/DRAW TO POINT X,Y                                              PT07  38
#   ----------------------                                              PT07  39
#                                                                       PT07  40
IF(J1==yes:)#                          Encode coords                    PT07  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT07  42
IF(IND==-1)#                           Draw cmnd for local plotter      PT07  43
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT07  44
plt1out:(2,ibufsiz:)#                  Increment buffer and test        PT07  45
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT07  46
ELSE#                                                                   PT07  47
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT07  48
IF(NDEV==4)      vduout:(3,ibufsiz:)#  Incr opt buf and test (vdu)      PT07  49
ELSE IF(NDEV==5) plt1out:(3,ibufsiz:)# Incr opt buf and test (loc1)     PT07  50
intpak:(ICOM(IND),BUFF(NWRD-2),0,mxbtwd:)# Insert command code          PT07  51
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT07  52
I=X/128+543186976#                     Extract hi-x + skeleton          PT07  53
intpak:(I,BUFF(NWRD-1), 0,mxbtwd:)#    Insert hi-x + skeleton           PT07  54
I=MOD((Y/4),32)#                       Extract lo-y                     PT07  55
intpak:(I,BUFF(NWRD-1), 8,5)#          Insert lo-y                      PT07  56
I=4*(MOD(Y,4))+MOD(X,4)#               Extract extra x and y            PT07  57
intpak:(I,BUFF(NWRD-1),16,4)#          Insert extra x and y             PT07  58
I=Y/128#                               Extract hi-y                     PT07  59
intpak:(I,BUFF(NWRD-1),24,5)#          Insert hi-y                      PT07  60
I=(MOD((X/4),32)+64)*16777216#         Extract lo-x                     PT07  61
intpak:(I,BUFF(NWRD), 0,mxbtwd:)#      Insert lo-x                      PT07  62
IF(IND==4) intpak:(49,BUFF(NWRD),0,8)# Encode boundary draw             PT07  63
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT07  64
#                                                                       PT07  65
#   SET COLOUR INDEX                                                    PT07  66
#   ----------------                                                    PT07  67
#                                                                       PT07  68
ELSE IF(J2==yes:)#                     Encode colour                    PT07  69
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT07  70
IF(NDEV==4)      vduout:(2,ibufsiz:)#  Incr opt buf and test (vdu)      PT07  71
ELSE IF(NDEV==5) plt1out:(2,ibufsiz:)# Incr opt buf and test (loc1)     PT07  72
intpak:(ICOM(IND),BUFF(NWRD-1),0,mxbtwd:)# Insert command code          PT07  73
IF(IND==7)#                            If pen select                    PT07  74
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT07  75
intpak:(65,BUFF(NWRD-1),16,8)#         Load plotter address char.       PT07  76
PKINT=256*X+12314#                     Ascii plus termin. char.         PT07  77
intpak:(PKINT,BUFF(NWRD),0,mxbtwd:)#   Load into buffer                 PT07  78
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT07  79
ELSE#                                                                   PT07  80
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT07  81
INT=IABS(X)#                           Absolute value of integer        PT07  82
IF(INT>65535) iquit:(PT0701)#          Test absolute value of int       PT07  83
IF(X>=0) I=MOD(INT,16)+4210736#        Extract lo-x + skeleton          PT07  84
ELSE     I=MOD(INT,16)+4210720#        Extract lo-x + skeleton          PT07  85
intpak:(I,BUFF(NWRD), 0,mxbtwd:)#      Insert lo-x + skeleton           PT07  86
I=MOD((INT/16),64)#                    Extract mid-x                    PT07  87
intpak:(I,BUFF(NWRD), 8,6)#            Insert mid-x                     PT07  88
I=INT/1024#                            Extract hi-x                     PT07  89
intpak:(I,BUFF(NWRD),16,6)#            Insert hi-x                      PT07  90
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT07  91
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT07  92
#                                                                       PT07  93
#    INSERT COMMAND CODE                                                PT07  94
#    -------------------                                                PT07  95
#                                                                       PT07  96
ELSE#                                  Else command word only           PT07  97
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT07  98
IF(IND==-1)#                           Start vdu color terminal         PT07  99
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT07 100
vduout:(0,-1)#                         Open the vdu op file             PT07 101
vduout:(1,ibufsiz:)#                   Reserve one word in op buffer    PT07 102
intpak:(455418160,BUFF(1),0,mxbtwd:)#  Set vdu as 'tek' mode            PT07 103
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT07 104
ELSE#                                  Regular command                  PT07 105
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT07 106
IF(NDEV==4)      vduout:(1,ibufsiz:)#  Incr opt buf and test (vdu)      PT07 107
ELSE IF(NDEV==5) plt1out:(1,ibufsiz:)# Incr opt buf and test (loc1)     PT07 108
intpak:(ICOM(IND),BUFF(NWRD),0,mxbtwd:)# Insert command code            PT07 109
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT07 110
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT07 111
#                                                                       PT07 112
RETURN#                                                                 PT07 113
END#                                                                    PT07 114
systemheader:(PT08)#                   Plotter command processor        PT08   1
####################################################################    PT08   2
#                                                                  #    PT08   3
#                               PT08                               #    PT08   4
#                                                                  #    PT08   5
####################################################################    PT08   6
SUBROUTINE PT08(XPIX,YPIX,INDX,NPEN)#                                   PT08   7
include:(AACOMN)#                                                       PT08   8
include:(PTCOMN)#                                                       PT08   9
#                                                                       PT08  10
# $MASP THIS ROUTINE HAS BEEN WRITTEN SPECIFICALLY FOR THE              PT08  11
#       TEKTRONIX 4662 INTERACTIVE DIGITAL PLOTTER. THE                 PT08  12
#       X,Y COORDINATE ENCODING IS THE SAME AS TEKTRONIX                PT08  13
#       4010 SERIES GRAPHICS TERMINALS. THE 4662 EXECUTES               PT08  14
#       GRAPHIC COMMANDS IN WHAT WOULD BE CALLED VECTOR                 PT08  15
#       MODE FOR THE TEKTRONIX GRAPHICS VDUS.                           PT08  16
#                                                                       PT08  17
INTEGER ACOL#                          Current atom colour              PT08  18
INTEGER INDX#                          Subject type index               PT08  19
INTEGER NPEN#                          Pen up or down                   PT08  20
INTEGER XPIX#                          X-coord in pixels                PT08  21
INTEGER YPIX#                          Y-coord in pixels                PT08  22
#                                                                       PT08  23
IF(NPEN==2)#                           If plotter draw command          PT08  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT08  25
IF(NOCOL==no:)#                        If no colour changes             PT08  26
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT08  27
IF(NEW==yes:)#                         If new subject                   PT08  28
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT08  29
I=MIN0(IABS(INDX),16)#                 Get pointer to colr. index       PT08  30
IF(I==0) ACOL=BCOL#                    Guard against no index           PT08  31
ELSE     ACOL=MOD(IABS(COLR(I)),8)#    Get atom colr. index             PT08  32
IF     (INDX==1000) pensl:(BCOL)#      Set bond colour                  PT08  33
ELSE IF(INDX>0) pensl:(ACOL)#          Set envel colour                 PT08  34
ELSE IF(INDX>-1000) pensl:(ACOL)#      Set elips colour                 PT08  35
ELSE IF(INDX>-1001) pensl:(BCOL)#      Set bord colour                  PT08  36
ELSE IF(INDX>-2001) pensl:(CCOL)#      Set char colour                  PT08  37
ELSE IF(INDX>-3001) pensl:(NCOL)#      Set numb colour                  PT08  38
NEW=no:#                               Set subject flag                 PT08  39
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT08  40
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT08  41
ELSE IF(FTPEN==yes:)#                  If first pen select              PT08  42
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT08  43
pensl:(BCOL)#                          Select colr for everything       PT08  44
FTPEN=no:#                             Set 1st pen flag                 PT08  45
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT08  46
plt1drw:(XPIX,YPIX)#                   Execute plotter draw command     PT08  47
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT08  48
ELSE IF(NPEN==3)#                      If plotter move command          PT08  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT08  50
IF(INDX==0) NEW=yes:#                  If break mark                    PT08  51
plt1mov:(XPIX,YPIX)#                   Execute plotter move command     PT08  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT08  53
#                                                                       PT08  54
RETURN#                                                                 PT08  55
END#                                                                    PT08  56
systemheader:(PT11)#                   Input control info               PT11   1
####################################################################    PT11   2
#                                                                  #    PT11   3
#                                PT11                              #    PT11   4
#                                                                  #    PT11   5
####################################################################    PT11   6
SUBROUTINE PT11#                                                        PT11   7
include:(AACOMN)#                                                       PT11   8
include:(PTCOMN)#                                                       PT11   9
#                                                                       PT11  10
#  THIS ROUTINE SETS UP THE COMMON STORAGE AND READS THE PLOTTING       PT11  11
#  INSTRUCTIONS FROM PSPECS, SETCAR, AND STARTP INPUT LINES.            PT11  12
#  A MAJOR PORTION OF THE ROUTINE IS CONCERNED WITH SETTING UP          PT11  13
#  CONTROL CHARACTER STRINGS FOR DOT MATRIX PRINTERS.                   PT11  14
#                                                                       PT11  15
INTEGER KEY#                           Signal for character checkin     PT11  16
INTEGER M,N#                           Input line column pointers       PT11  17
INTEGER INPCHR#                        Pointer into dev2ch array        PT11  18
INTEGER INPTT#                         Characters on one setcar lin     PT11  19
INTEGER II,I,J#                        Local index                      PT11  20
INTEGER INTCHR#                        A character in integer form      PT11  21
REAL RV(6)#                            Dev. resolutions (default)       PT11  22
REAL RELCHR(1)#                        A character in real form         PT11  23
string:(REALCH,1)#                     A real character from intege     PT11  24
#                                                                       PT11  25
chardata:(PT110, STARTP END    PSPECS SETCAR COLOR )#                   PT11  26
chardata:(PT111,CHAR DOT  PLOT VDU  LOC1 LOC2 )#                        PT11  27
chardata:(PT11B,PLOTAP)#                                                PT11  28
chardata:(PT11C,BIDMON)#               Signal printer mode              PT11  29
chardata:(PT11D,LTRRTL)#               Signal carriage motion           PT11  30
chardata:(PT11E,PLTRPRTLEEOLEBUFCLERPLOTPRELFINL)# Signal control       PT11  31
chardata:(PT11F, SUMMARY OF DOT PRINTER CHARACTERISTICS...)#            PT11  32
chardata:(PT11T, UNPRINTABLE DOT PRINTER CONTROL CHARACTERS AS $#       PT11  33
NUMBERS)#                                                               PT11  34
chardata:(PT11U,   PURPOSE                DECIMAL  OCTAL  BINARY)#      PT11  35
chardata:(PT12A,PROLOGUE L TO R LINEPROLOGUE R TO L LINE$#              PT11  36
EPILOGUE END OF LINEEPILOGUE PARTIAL BUF)#                              PT11  37
chardata:(PT12B,LINE CLEAR CHARACTERPOINT PLOT CHARACTER$#              PT11  38
PRELIMINARY CONTROL FINAL CONTROL LINE  )#                              PT11  39
chardata:(PT12D,FILL  NOFIL BOND  CHAR  NUMB  CONT  NOCOL )#            PT11  40
realdata:(WIDE,[36.,10.,12.,9.4,15.,12.])# Widths by device $masp       PT11  41
realdata:(HIGH,[36.,10.,12.,7.1,10.,12.])# Heights by device masp$      PT11  42
intdata:(MAXCHR,70); intdata:(TOTCHR,0)# Set total chars in dev2cr      PT11  43
realdata:(FMT,[310333.,390435.,500884.])#                               PT11  44
datastuff:#                                                             PT11  45
#                                                                       PT11  46
#  SET UP DEFAULTS FOR PLOTTING AND PLOTTER CHARACTERISTICS             PT11  47
#  --------------------------------------------------------             PT11  48
#                                                                       PT11  49
IF(FIRSTT==yes:)#                      But only on first call           PT11  50
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT11  51
RV(1)=rsv1:; RV(2)=rsv2:; RV(3)=rsv3:# Set resolutions $masp            PT11  52
RV(4)=rsv4:; RV(5)=rsv5:; RV(6)=rsv6:# Set resolutions $masp            PT11  53
IBRDR=yes:#                            Flag border  drawn               PT11  54
ISCALE=no:#                            No automatic scaling             PT11  55
ICENTR=no:#                            No automatic centering           PT11  56
NDEV=1#                                Default line printer             PT11  57
LINEPI=prtlpin:#                       The lines/inch on printer        PT11  58
CHARPI=prtcpin:#                       Characters/inch across pr.       PT11  59
BUFMAX=prtcmax:#                       Max chrs in chrot array          PT11  60
INPCHR=0#                    ## $masp  # set input control chars        PT11  61
TOTCHR=0#                    ## $masp  # set total of inpchr            PT11  62
NUMBIT=mxbtch:#                        Set bits per character           PT11  63
DIRBIT=+1#                   ## $masp  # set bit scan as r to l         PT11  64
STARBT=0#                    ## $masp  # set start bit to rightmos      PT11  65
CHRCHR=1#                    ## $masp  # set one char per char          PT11  66
BIDIR=no:#                   ## $masp  # set dev as monodirectional     PT11  67
VRPIX=vdpi:#                           Vertical pixels on raster        PT11  68
HZPIX=hdpi:#                           Horiz. pixels on raster          PT11  69
AHIGH=HIGH(1)#               ## $masp  # height for ndev=1              PT11  70
AWIDE=WIDE(1)#               ## $masp  # width for ndev=1               PT11  71
OMAG=1.0#                              No magnification                 PT11  72
LINES=0 ; TOTPIX =0#                   Set for analog plotters          PT11  73
NCLEAR=0 ;NPLOTC=0 ;NPROL1=0; NPROL2=0#                                 PT11  74
NEPIL1=0; NEPIL2=0; NEPIL=0; NPRLIM=0#                                  PT11  75
BCOL=1#                                Set bond colour                  PT11  76
CCOL=5#                                Set character colour             PT11  77
NCOL=6#                                Set number colour                PT11  78
CONTUR=no:#                            Indicate plot type               PT11  79
FILL=yes:#                             Set fill pattern on              PT11  80
NWRD=0#                                Start buffer word pointer        PT11  81
NEW=yes:#                              New subject flag                 PT11  82
FTPEN=yes:#                            First pen select                 PT11  83
NOCOL=no:#                             Set different colours flag       PT11  84
DO I=1,16; COLR(I)=MOD(I-1,6)+2#       Default colours                  PT11  85
#                                                                       PT11  86
movechar:(PT11B,1,TAPE,1,NPT11B,0)#    Move tape name                   PT11  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT11  88
#                                                                       PT11  89
#   ENTER ALL CONTROL LINES                                             PT11  90
#   -----------------------                                             PT11  91
#                                                                       PT11  92
REPEAT#                                Loop over input lines            PT11  93
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT11  94
readline:(PT110,NPT110)#               Read next input line             PT11  95
#                                                                       PT11  96
IF(LINID==2)#                          If end encountered               PT11  97
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT11  98
IF(FIRSTT==yes:) iquit:(PT1101)#       Startp line missing              PT11  99
GO TO 901#                             End line found                   PT11 100
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT11 101
#                                                                       PT11 102
#   PROCESS =STARTP= LINE                                               PT11 103
#   ----------------------                                              PT11 104
#                                                                       PT11 105
ELSE IF(LINID==1)#                     *********** startp line          PT11 106
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT11 107
FIRSTT=no:#                            Flag first startp line           PT11 108
IF(BUFIN(1)<voidflg:)#                 Character field                  PT11 109
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT11 110
stringlim:(1,M,N)#                     Get string limits                PT11 111
compchar:(CHRIN,M,PT111,1,4,5,NPT111,KEY)# Identify                     PT11 112
IF(KEY>0) NDEV=KEY#                    Device                           PT11 113
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT11 114
IF(BUFIN(2)>0.0) NPLOT=NINT(BUFIN(2))# Plot number                      PT11 115
ELSE             NPLOT=1#              Default 1st plot                 PT11 116
IF(BUFIN(3)>0.0) AWIDE=BUFIN(3)#       Plot width                       PT11 117
ELSE             AWIDE=WIDE(NDEV)#                                      PT11 118
IF(BUFIN(4)>0.0) AHIGH=BUFIN(4)#       Plot height                      PT11 119
ELSE             AHIGH=HIGH(NDEV)#                                      PT11 120
IF(BUFIN(5)>0.0) OMAG=BUFIN(5)#        Magnification                    PT11 121
ELSE             ISCALE=yes:#          Default no magnification         PT11 122
ifnv:(BUFIN(6))  XCEN=BUFIN(6)#        Plot center                      PT11 123
ELSE             ICENTR=yes:#          Default auto center              PT11 124
ifnv:(BUFIN(7))  YCEN=BUFIN(7)#        Plot center                      PT11 125
ELSE             ICENTR=yes:#          Default auto center              PT11 126
ifnv:(BUFIN(8)) IBRDR=no:#             Supress the border               PT11 127
IF(BUFIN(9)>voidflg:)#                                                  PT11 128
RESL=ABS(BUFIN(9))/OMAG#               Get resolution for display       PT11 129
ELSE RESL=RV(NDEV)/OMAG#               Get default resolution           PT11 130
GO TO 901#                                                              PT11 131
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT11 132
#                                                                       PT11 133
#   PROCESS =PSPECS= LINE                                               PT11 134
#   ---------------------                                               PT11 135
#                                                                       PT11 136
ELSE IF(LINID==3)#                     ************ pspecs line         PT11 137
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT11 138
LINRM=30 ; writeline:(0,PT11F,NPT11F,3,3)# Head the echo                PT11 139
IF(BUFIN(1)>0.0) HZPIX=NINT(BUFIN(1))# Number of horizontal pixels      PT11 140
IF(BUFIN(2)>0.0) VRPIX=NINT(BUFIN(2))# Number of vertical pixels        PT11 141
IF((BUFIN(3)>0.0)&(BUFIN(3)<=8.0))NUMBIT=NINT(BUFIN(3))#                PT11 142
IF(BUFIN(4)<voidflg:)#                 Read the plot writeup            PT11 143
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT11 144
stringlim:(4,M,N)#                     Get location of input chars      PT11 145
compchar:(CHRIN,M,PT11D,1,3,3,NPT11D,KEY)# Scan list                    PT11 146
IF(KEY==1)DIRBIT=-1#                   Bits go l to r                   PT11 147
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT11 148
IF((BUFIN(5)>=0.0)&(BUFIN(5)<12.0))STARBT=NINT(BUFIN(5))#               PT11 149
IF((BUFIN(6)>=1.0)&(BUFIN(6)<=2.0))CHRCHR=NINT(BUFIN(6))#               PT11 150
IF(BUFIN(7)<voidflg:)#                 Gotta be characters              PT11 151
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT11 152
stringlim:(7,M,N)#                     Get loc of chrs in input         PT11 153
compchar:(CHRIN,M,PT11C,1,3,3,NPT11C,KEY)# Check list                   PT11 154
IF(KEY==1)BIDIR=yes:#                  Set the back and forth mode      PT11 155
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT11 156
IF(BUFIN(8)>0.0)BUFMAX=NINT(BUFIN(8))# Set buf size                     PT11 157
IF(((STARBT+DIRBIT*NUMBIT)<-1)\((STARBT+DIRBIT*NUMBIT)>mxbtch:))#       PT11 158
iquit:(PT1109)#                        Bits will go outside chars       PT11 159
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT11 160
#                                                                       PT11 161
#    PROCESS =SETCAR= LINE                                              PT11 162
#    ---------------------                                              PT11 163
#                                                                       PT11 164
ELSE IF(LINID==4)#                     ************ setcar line         PT11 165
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT11 166
IF(BUFIN(1)>=voidflg:) iquit:(PT1104,1.0)# Characters or die            PT11 167
stringlim:(1,M,N)#                                                      PT11 168
compchar:(CHRIN,M,PT11E,1,4,4,NPT11E,KEY)# Scan possibles               PT11 169
IF(KEY<=0) iquit:(PT1104,1.0)#         Die on invalid                   PT11 170
INPTT=0#                               Zero input ch count              PT11 171
writeline:(0,PT11T,NPT11T,3,3)#        Head the echo                    PT11 172
writeline:(0,PT11U,NPT11U,3,3)#                                         PT11 173
FOR(II=2;II<=mxfdim:;II=II+1)#         Loop over input field            PT11 174
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PT11 175
ifvo:(BUFIN(II)) BREAK#                Quit on first void field         PT11 176
IF((BUFIN(II)>=0.0)&(BUFIN(II)<256.0))# Gotta fit 8 bit bytes           PT11 177
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PT11 178
INPCHR=INPCHR+1#                       Count up input characters        PT11 179
INPTT=INPTT+1#                         Count for this line              PT11 180
IF(INPCHR>MAXCHR) iquit:(PT1105)#      Too many characters              PT11 181
INTCHR=NINT(BUFIN(II))#                Change to an integer             PT11 182
intpak:(INTCHR,RELCHR(1),lowordbit:,mxbtch:)# ## Masp## pack it in a    PT11 183
movertoc:(RELCHR,1,REALCH,1,1,1)#      Move to chr array                PT11 184
movechar:(REALCH,1,DEV2CH,INPCHR,1,1)# Save it for printer              PT11 185
BUFOT(1)=BUFIN(II) ; BUFOT(2)=BUFIN(II) ; BUFOT(3)=BUFIN(II)#           PT11 186
ncodefld:(BUFOT,1,CHROT,FMT,3)#        Encode as dec,oct,hex            PT11 187
N=(KEY-1)*20+1#                        Find purpose                     PT11 188
IF(N<80)movechar:(PT12A,N,CHROT,3,20,0)# Tell user in words             PT11 189
ELSE#                                                                   PT11 190
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PT11 191
N=N-80 ; movechar:(PT12B,N,CHROT,3,20,0)# Other words                   PT11 192
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PT11 193
writeline:(0,0,0,1,3)#                                                  PT11 194
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PT11 195
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PT11 196
IF(KEY==1)#                            Prologue left to right           PT11 197
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PT11 198
NPROL1=INPTT ; SPROL1=TOTCHR+1#        Set controls                     PT11 199
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PT11 200
ELSE IF(KEY==2)#                                                        PT11 201
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PT11 202
NPROL2=INPTT ; SPROL2=TOTCHR+1#        Prologue right to left           PT11 203
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PT11 204
ELSE IF(KEY==3)#                                                        PT11 205
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PT11 206
NEPIL1=INPTT ; SEPIL1=TOTCHR+1#        Epilogue end of line             PT11 207
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PT11 208
ELSE IF(KEY==4)#                                                        PT11 209
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PT11 210
NEPIL2=INPTT ; SEPIL2=TOTCHR+1#        Epilogue end of buffer           PT11 211
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PT11 212
ELSE IF(KEY==5)#                                                        PT11 213
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PT11 214
NCLEAR=INPTT ; SCLEAR=TOTCHR+1#        Line clearing characters         PT11 215
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PT11 216
ELSE IF(KEY==6)#                       Line plotting characters         PT11 217
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PT11 218
NPLOTC=INPTT ; SPLOTC=TOTCHR+1#        Set plot character(s)            PT11 219
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PT11 220
ELSE IF(KEY==7)#                       Line plotting characters         PT11 221
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PT11 222
NPRLIM=INPTT ; SPRLIM=TOTCHR+1#        Set prelim character(s)          PT11 223
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PT11 224
ELSE IF(KEY==8)#                       Line plotting characters         PT11 225
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PT11 226
NEPIL=INPTT ; SEPIL=TOTCHR+1#          Set prelim character(s)          PT11 227
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PT11 228
TOTCHR=TOTCHR+INPTT#                   Save how far it goes             PT11 229
IF(INPTT<=0) iquit:(PT1106,2.0)#       But don't tolerate junk          PT11 230
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT11 231
#                                                                       PT11 232
#    PROCESS =COLOR= LINE                                               PT11 233
#    --------------------                                               PT11 234
#                                                                       PT11 235
ELSE IF(LINID==5)#                     *********** color line           PT11 236
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PT11 237
FOR(J=1;J>0;J=J+1)#                    Loop over input fields           PT11 238
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PT11 239
ifvo:(BUFIN(J)) BREAK#                 Exit on first void               PT11 240
IF(BUFIN(J)>voidflg:) iquit:(PT1111)#  Illegal ip order...341110        PT11 241
stringlim:(J,M,N)#                     Extract char string              PT11 242
compchar:(CHRIN,M,PT12D,1,3,6,NPT12D,KEY)# Identify string              PT11 243
IF(KEY==0) iquit:(PT1111)#             Not recognized...pt1111          PT11 244
IF((KEY==1)\(KEY==2)\(KEY==6))#        If "fill","nofill","cont"        PT11 245
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 PT11 246
IF(KEY==2\KEY==6\NDEV==5)#             If nofill, contour or plotte     PT11 247
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 PT11 248
FILL=no:#                              Flag as nofill                   PT11 249
IF(KEY==6) CONTUR=yes:#                If contour then flag as such     PT11 250
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 PT11 251
DO I=1,16#                             Loop over additional fields      PT11 252
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 PT11 253
IF(BUFIN(J+1)<=voidflg:) BREAK#        Exit if alpha or void            PT11 254
J=J+1#                                 Incr field counter               PT11 255
COLR(I)=NINT(BUFIN(J))#                Get colour index                 PT11 256
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 PT11 257
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 PT11 258
ELSE IF(KEY==3&BUFIN(J+1)>voidflg:)#   Char string = "bond"             PT11 259
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 PT11 260
BCOL=NINT(BUFIN(J+1))#                 Store bond colour                PT11 261
J=J+1#                                 Incr field counter               PT11 262
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 PT11 263
ELSE IF(KEY==4&BUFIN(J+1)>voidflg:)#   Char string = "char"             PT11 264
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 PT11 265
CCOL=NINT(BUFIN(J+1))#                 Store character colour           PT11 266
J=J+1#                                 Incr field counter               PT11 267
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 PT11 268
ELSE IF(KEY==5&BUFIN(J+1)>voidflg:)#   Char string = "numb"             PT11 269
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 PT11 270
NCOL=NINT(BUFIN(J+1))#                 Store number colour              PT11 271
J=J+1#                                 Incr field counter               PT11 272
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 PT11 273
ELSE IF(KEY==7)#                       Char string = "nocol"            PT11 274
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 PT11 275
NOCOL=yes:#                            Flag no colour changes           PT11 276
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 PT11 277
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PT11 278
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PT11 279
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT11 280
#                                                                       PT11 281
901 CONTINUE#                          Come from end line               PT11 282
RETURN#                                                                 PT11 283
END#                                                                    PT11 284
systemheader:(PT21)#                   Initialize plotter               PT21   1
####################################################################    PT21   2
#                                                                  #    PT21   3
#                                PT21                              #    PT21   4
#                                                                  #    PT21   5
####################################################################    PT21   6
SUBROUTINE PT21#                                                        PT21   7
include:(PTCOMN)#                                                       PT21   8
include:(AACOMN)#                                                       PT21   9
#                                                                       PT21  10
# WRITE INITIAL RECORD(S) OF EXTERNAL PLOTTER FILE                      PT21  11
# ---------------------------------------                               PT21  12
#                                                                       PT21  13
IF(NDEV==3)#                           Calcomp tape at lowest level     PT21  14
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT21  15
CALL PT02(3,1,-2)#                                                      PT21  16
CALL PT02(3312,3,0)#                                                    PT21  17
CALL PT02(9900,3,0)#                                                    PT21  18
CALL PT02(1000,1000,-1)#                                                PT21  19
CALL PT02(1,1,0)#                                                       PT21  20
CALL PT02(1,3,0)#                                                       PT21  21
CALL PT02(1025,2,0)#                                                    PT21  22
CALL PT06(0.0,0.0,0.0,0.0,-3,-1)#                                       PT21  23
CALL PT06(0.0,0.5,0.0,0.0,-3,-1)#                                       PT21  24
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT21  25
#                                                                       PT21  26
ELSE IF(NDEV==4)#                      Vdu colour terminal              PT21  27
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT21  28
vdusta:#                               Initialize the colour vdu        PT21  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT21  30
#                                                                       PT21  31
ELSE IF(NDEV==5)#                      Local device number one          PT21  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT21  33
plots:(0,0,0)#                ## $masp  # start plot file               PT21  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT21  35
#                                                                       PT21  36
ELSE IF(NDEV==6)#                      Set up for local device two      PT21  37
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT21  38
#PLOTC:(0.,-11.,-3)#        ## $MASP     # PUT PEN AT LIMITS            PT21  39
#PLOTC:(0.,.5,-3)#          ## $MASP     # PUT PEN AT ORIGIN            PT21  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT21  41
#                                                                       PT21  42
RETURN#                                                                 PT21  43
END#                                                                    PT21  44
systemheader:(PT31)#                   Advance frame                    PT31   1
####################################################################    PT31   2
#                                                                  #    PT31   3
#                                PT31                              #    PT31   4
#                                                                  #    PT31   5
####################################################################    PT31   6
SUBROUTINE PT31#                                                        PT31   7
include:(PTCOMN)#                                                       PT31   8
include:(AACOMN)#                                                       PT31   9
#                                                                       PT31  10
IF(FRAME>0)#                           Treat first frame only           PT31  11
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT31  12
#                                                                       PT31  13
IF(NDEV==3)#                           Calcomp directly to plotter      PT31  14
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT31  15
CALL PT06(0.0,0.0,0.0,0.0,-3,-1)#                                       PT31  16
CALL PT06(24.0,0.0,0.0,0.0,-3,-1)#                                      PT31  17
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT31  18
#                                                                       PT31  19
ELSE IF(NDEV==5)#                      Local plot device one            PT31  20
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT31  21
plot:(0.,0.,-999)#         ## $masp    # set frame advance              PT31  22
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT31  23
#                                                                       PT31  24
ELSE IF(NDEV==6)#                      Local plot device two            PT31  25
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT31  26
pageup:#                  ## $masp       # redefine plot origin         PT31  27
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT31  28
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT31  29
#                                                                       PT31  30
RETURN#                                                                 PT31  31
END#                                                                    PT31  32
systemheader:(PT41)#                   Draw characters                  PT41   1
####################################################################    PT41   2
#                                                                  #    PT41   3
#                               PT41                               #    PT41   4
#                                                                  #    PT41   5
####################################################################    PT41   6
SUBROUTINE PT41#                                                        PT41   7
include:(AACOMN)#                                                       PT41   8
include:(PTCOMN)#                                                       PT41   9
#                                                                       PT41  10
#  THIS SUBROUTINE PRODUCES THE VECTOR MOVES REQUIRED TO PRODUCE        PT41  11
#  PLOTTED SYMBOLS.  IT IS USED IN PT TO SUPPLANT THE USE OF            PT41  12
#  LIBRARY PLOTTER CALLS.  IT ALSO ALLOWS PT TO TREAT THE               PT41  13
#  PLOTTING OF ANY SIZE CHARACTER ON DOT MATRIX PRINTERS.               PT41  14
#                                                                       PT41  15
## (X,Y) COORDINATES OF LEFT CORNER OF CHARACTER STRING **OR**          PT41  16
##       COORDINATES OF CENTER OF SPECIAL SYMBOL                        PT41  17
## HT    HEIGHT OF SYMBOLS IN INCHES                                    PT41  18
## NCAR  STRING OF UP TO 72 CHARACTERS                                  PT41  19
## TH    ANGLE TO X-AXIS, IN DEGREES                                    PT41  20
## NS>0  NUMBER OF CHARACTERS IN STRING                                 PT41  21
##    -1 TO -15 NUMBER OF SPECIAL SYMBOL (INITIAL PEN UP MOVE)          PT41  22
##   -16 TO -30 NUMBER OF SPECIAL SYMBOL (INITIAL PEN DOWN MOVE)        PT41  23
## NBLA  0/1 DRAW BLANKS/SKIP BLANKS                                    PT41  24
#                                                                       PT41  25
## X AND Y ARE THE COORDINATES IN A GRID 7 BY 7                         PT41  26
#                                                                       PT41  27
REAL    DELX,DELY#                     Stepsize in x,y                  PT41  28
REAL    FMT#                           Format                           PT41  29
REAL    HT#                            Char/numb heigth                 PT41  30
INTEGER II#                                                             PT41  31
INTEGER GREEK,SPFLAG#                  Flags for greek/offset lett      PT41  32
INTEGER KEY#                           Compchar key                     PT41  33
INTEGER LCAR#                          Count of output characters       PT41  34
INTEGER NBLA#                          0/1 draw blank/skip blanks       PT41  35
string:(NCAR,72)#                      Character string to be plotted   PT41  36
INTEGER NF#                            Function code                    PT41  37
INTEGER NPLUM#                         Pen up/down at beginning         PT41  38
INTEGER NS#                            Number of char in string         PT41  39
REAL    CIND#                          Subject index                    PT41  40
REAL    TH#                            Angle to x-axis in degrees       PT41  41
REAL    X,Y#                           Start of char/numb string        PT41  42
REAL    QQQ(2)#                                                         PT41  43
#                                                                       PT41  44
chardata:(PT410,"ABCDEFGHIJKLMNOPQRSTUVWXYZ$#                           PT41  45
abcdefghijklmnopqrstuvwxyz")## ****WARNING:DO NOT ALTER ORDER OF PT41   PT41  46
chardata:(PT411,'0123456789+-*/()=,.%^~ ')# Numbers and special symbo   PT41  47
datastuff:#                                                             PT41  48
#                                                                       PT41  49
#                                                                       PT41  50
#   PROCESS NUMB (4) & CHAR (5) DATA                                    PT41  51
#   --------------------------------                                    PT41  52
#                                                                       PT41  53
NF=NINT(QX(IP+7))#                     Extract function code            PT41  54
HT=QX(IP+3)#                           Extract char height              PT41  55
X=QX(IP+1);    Y=QX(IP+2)#             Start of char string             PT41  56
IF(X+XOFF<HT) X=.5-XOFF#               Ensure coord inside frame        PT41  57
IF(Y+YOFF<HT) Y=.5-YOFF#               Ensure coord inside frame        PT41  58
#                                                                       PT41  59
IF(NF==4)#                             Process number string            PT41  60
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT41  61
CIND=-3000.#                           Set colour index                 PT41  62
TH=QX(IP+5)#                           Theta angle of string            PT41  63
NS=10#                                 Set number code                  PT41  64
NBLA=0#                                Number of blanks                 PT41  65
ifeq:(QX(IP+6),0.) FMT=101013.#        Format as integer number         PT41  66
ELSE FMT=101002.+10.*QX(IP+6)#         Format as real number            PT41  67
ncodefld:(QX,IP+4,NCAR,FMT,1)#                                          PT41  68
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT41  69
#                                                                       PT41  70
ELSE IF(NF==5)#                        Process character string         PT41  71
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT41  72
CIND=-2000.#                           Set colour index                 PT41  73
TH=QX(IP+4)#                           Theta angle of string            PT41  74
NS=NINT(QX(IP+5))#                     Number of characters in string   PT41  75
IF(NS>72) iquit:(PT4101)#              String is too long               PT41  76
NBLA=NINT(QX(IP+6))#                   Blank flag                       PT41  77
FOR(II=1;II<=NS;II=II+arith:(7,*,mxchwd:))# Loop over char string       PT41  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT41  79
JK=MIN0(NS-II+1,arith:(7,*,mxchwd:))#  Number of chars to transfer      PT41  80
readwpkt:(5,23,PSIZE,IP,0)#            Read a packet of characters      PT41  81
movertoc:(QX(IP+1),1,NCAR,II,JK,0)#    Transfer characters              PT41  82
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT41  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT41  84
QQQ(1)=X;     QQQ(2)=Y#                Store x,y for pt06               PT41  85
CALL PT06(QQQ(1),QQQ(2),0.,0.,3,1)#    Force breakpoint                 PT41  86
DELY=HT*COS(TH*.017453)/7.#                                             PT41  87
DELX=HT*SIN(TH*.017453)/7.#                                             PT41  88
#                                                                       PT41  89
#    PROCESS STANDARD CHARACTER                                         PT41  90
#     --------------------------                                        PT41  91
#                                                                       PT41  92
LCAR=0#                                Reset character counter          PT41  93
DO I=1,NS#                             Loop characters in string        PT41  94
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT41  95
KEY = INDEX(PT410,NCAR(I^I))#          Check standard characters        PT41  96
IF(KEY==0) KEY = INDEX(PT411,NCAR(I^I))+52# Must be number or symbol    PT41  97
#                                                                       PT41  98
##Set special character flags                                           PT41  99
# ---------------------------                                           PT41 100
#                                                                       PT41 101
IF(KEY==72 & GREEK  != yes:)#          % flag for greek character       PT41 102
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT41 103
GREEK = yes:#                          Set greeks flag                  PT41 104
NEXT#                                  Find next character              PT41 105
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT41 106
IF((KEY==73\KEY==74)&SPFLAG==0)#       Now flag subscript               PT41 107
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT41 108
SPFLAG = 2*(73.5 - KEY)#               -1/+1 for super/subscript        PT41 109
NEXT#                                  Get next character               PT41 110
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT41 111
IF(KEY!=75)#                           Not a space                      PT41 112
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT41 113
IF(GREEK==yes:)#                       If is greek                      PT41 114
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT41 115
KEY = KEY+74#                          Print greek character            PT41 116
GREEK = no:#                           Reinitialise flag                PT41 117
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT41 118
NPLUM = 3#                             Move to next character           PT41 119
CALL PT05(KEY,NPLUM,DELX,DELY,X,Y,LCAR,CIND,SPFLAG)# Call plotsy        PT41 120
SPFLAG = 0#                            Reinitialise spflag              PT41 121
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT41 122
ELSE IF(NBLA!=0) LCAR=LCAR-1#          Skip blanks in output            PT41 123
LCAR=LCAR+1#                           Add up one output char           PT41 124
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT41 125
#                                                                       PT41 126
#                                                                       PT41 127
RETURN#                                                                 PT41 128
END#                                                                    PT41 129
systemheader:(PT51)#                   Auto center and scale            PT51   1
####################################################################    PT51   2
#                                                                  #    PT51   3
#                                PT51                              #    PT51   4
#                                                                  #    PT51   5
####################################################################    PT51   6
SUBROUTINE PT51#                                                        PT51   7
include:(AACOMN)#                      George davenport 1983            PT51   8
include:(PTCOMN)#                      Stewart & holden 1983            PT51   9
#                                                                       PT51  10
# INCOMING PACKET CONTAINS:                                             PT51  11
# IP+1 -- FRAME WIDTH                                                   PT51  12
# IP+2 -- FRAME HEIGHT                                                  PT51  13
# IP+3 -- STEREO FLAG (1=NO, 2=YES)                                     PT51  14
# IP+4 -- SCALE OF INPUT X,Y                                            PT51  15
# IP+5 -- MAXIMUM X-SPAN                                                PT51  16
# IP+6 -- MAXIMUM Y-SPAN                                                PT51  17
# IP+7 -- 6                                                             PT51  18
#                                                                       PT51  19
IF(ICENTR==yes:)#                      Auto centering necessary         PT51  20
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT51  21
XCEN=QX(IP+1)*0.5#                     Get x-coord of center            PT51  22
YCEN=QX(IP+2)*0.5#                     Get y-coord of center            PT51  23
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT51  24
IF(ISCALE==yes:)#                      Auto scaling necessary           PT51  25
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT51  26
OMAG=AMIN1(AWIDE/QX(IP+5),AHIGH/QX(IP+6))# Scale output plot            PT51  27
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT51  28
#                                                                       PT51  29
#    SET SCALING TERMS                                                  PT51  30
#    -----------------                                                  PT51  31
#                                                                       PT51  32
IF(NDEV==1)#                           For lo res printer               PT51  33
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT51  34
SCALX=OMAG*FLOAT(CHARPI)#              Convert to char/inch             PT51  35
SCALY=OMAG*FLOAT(LINEPI)#              Convert to line/inch             PT51  36
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT51  37
ELSE IF(NDEV==2)#                      For hi res printer               PT51  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT51  39
SCALX=OMAG*FLOAT(HZPIX)#               Convert to horiz pixels          PT51  40
SCALY=OMAG*FLOAT(VRPIX)#               Convert to vertl pixels          PT51  41
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT51  42
ELSE IF(NDEV==3)#                      For calcomp plotter              PT51  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT51  44
SCALX=OMAG;    SCALY=OMAG#             Convert for calcomp              PT51  45
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT51  46
ELSE IF(NDEV==4)#                      For vdu graphics                 PT51  47
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT51  48
SCALX=OMAG*pixpi:#          ## $masp   # convert to horiz pixels        PT51  49
SCALY=OMAG*pixpi:#          ## $masp   # convert to vert  pixels        PT51  50
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT51  51
ELSE IF(NDEV==5)#                      For local plotter                PT51  52
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT51  53
SCALX=OMAG*adupi:#                     Convert to horiz. adus           PT51  54
SCALY=OMAG*adupi:#                     Convert to vert. adus            PT51  55
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT51  56
XOFF=0.5*AWIDE/OMAG-XCEN#              Calx x offset                    PT51  57
YOFF=0.5*AHIGH/OMAG-YCEN#              Calc y offset                    PT51  58
#                                                                       PT51  59
# DRAW BORDER, BUT NOT FOR CALCOMP                                      PT51  60
# --------------------------------                                      PT51  61
#                                                                       PT51  62
IF((IBRDR==yes:)&(NDEV!=3))#           Border not yet drawn             PT51  63
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT51  64
TEM1=+XOFF+2.*XCEN#                    Right boundary                   PT51  65
TEM2=+YOFF+2.*YCEN#                    Upper boundary                   PT51  66
TEM3=-XOFF#                            Left  boundary                   PT51  67
TEM4=-YOFF#                            Lower boundary                   PT51  68
CALL PT06(TEM3,TEM4,0.,0.,3,1)#        Pen up to sw corner              PT51  69
CALL PT06(TEM3,TEM2,0.,-1000.,2,1)#    Pen down to nw corner            PT51  70
CALL PT06(TEM1,TEM2,0.,-1000.,2,1)#    Pen down to ne corner            PT51  71
CALL PT06(TEM1,TEM4,0.,-1000.,2,1)#    Pen down to se corner            PT51  72
CALL PT06(TEM3,TEM4,0.,-1000.,2,1)#    Pen down to sw corner            PT51  73
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT51  74
ELSE#                                  Zero the corners                 PT51  75
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT51  76
TEM1=0.;TEM2=0.;TEM3=0.;TEM4=0.#       For pt00 border setting          PT51  77
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT51  78
RETURN#                                                                 PT51  79
END#                                                                    PT51  80
systemheader:(PT61)#                   Pen up/down moves                PT61   1
####################################################################    PT61   2
#                                                                  #    PT61   3
#                               PT61                               #    PT61   4
#                                                                  #    PT61   5
####################################################################    PT61   6
SUBROUTINE PT61#                                                        PT61   7
include:(AACOMN)#                                                       PT61   8
include:(PTCOMN)#                                                       PT61   9
#                                                                       PT61  10
# PEN MOVES STORED IN MARK1 ARRAY ARE NOW EXECUTED                      PT61  11
# FORMAT:                                                               PT61  12
# IP+1 -- X-COORD + DX                                                  PT61  13
# IP+2 -- Y-COORD + DY                                                  PT61  14
# IP+3 -- NOT USED                                                      PT61  15
# IP+4 -- COLOUR CODE                                                   PT61  16
# IP+5 -- 2 FOR PEN DOWN MOVE TO (X+DX,Y+DY)                            PT61  17
#         3 FOR PEN UP MOVE TO (X+DX,Y+DY)                              PT61  18
# IP+6 -- -1 FOR SEND ARGUMENTS DIRECTLY TO CALCOMP ROUTINE             PT61  19
#          1 FOR COORDINATES TO BE CALCULATED                           PT61  20
#                                                                       PT61  21
INTEGER TOTP,LINE#                                                      PT61  22
#                                                                       PT61  23
LINE=LINES-1;   TOTP=TOTPIX-1#         Set max lengths                  PT61  24
#                                                                       PT61  25
#    PROCESS STORED PLOT COMMANDS                                       PT61  26
#    ----------------------------                                       PT61  27
#                                                                       PT61  28
FOR(I=ISTOP;I<IP1;I=I+NSTEP1)#         Loop over pen moves              PT61  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT61  30
X=SCALX*(QX(I+1)+XOFF)#                Get scaled x                     PT61  31
Y=SCALY*(QX(I+2)+YOFF)#                Get scaled y                     PT61  32
INDX=NINT(QX(I+4))#                    Get colour index                 PT61  33
NPEN=NINT(QX(I+5))#                    Pen up or down                   PT61  34
IFLG=NINT(QX(I+6))#                    Flag direct to calcomp           PT61  35
IF(NPEN==2) NDOWN=NDOWN+1#             Count pendown calls              PT61  36
ELSE        NUP=NUP+1#                 Count penup calls                PT61  37
#                                                                       PT61  38
IF(NDEV<3)#                            For printer devices              PT61  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT61  40
NEWX=MIN0(IABS(INT(X)),TOTP)#          Convert to horiz int             PT61  41
NOWY=MIN0(IABS(LINE-INT(Y)),LINE)#     Convert to vertl int             PT61  42
IF(NPEN==2)#                           Only for pen down movement       PT61  43
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT61  44
MOVEX=NEWX-LASTX#                                                       PT61  45
MOVEY=NOWY-LASTY#                                                       PT61  46
IF(MOVEX==0)     LAGX= 0#                                               PT61  47
ELSE IF(MOVEX<0) LAGX=-IABS(MOVEY)/2#                                   PT61  48
ELSE             LAGX= IABS(MOVEY)/2#                                   PT61  49
IF(MOVEY==0)     LAGY= 0#                                               PT61  50
ELSE IF(MOVEY<0) LAGY=-IABS(MOVEX)/2#                                   PT61  51
ELSE             LAGY= IABS(MOVEX)/2#                                   PT61  52
MULT=MAX0(IABS(MOVEX),IABS(MOVEY))#                                     PT61  53
IF(MULT>0)#                                                             PT61  54
FOR(J=0;J<=MULT;J=J+1)#                                                 PT61  55
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT61  56
MODX=LASTX+(J*MOVEX+LAGX)/MULT#                                         PT61  57
MODY=LASTY+(J*MOVEY+LAGY)/MULT#                                         PT61  58
NBIT=MODX+TOTPIX*MODY#                 Bit position within qx           PT61  59
NWRD=MARK+NBIT/MXBTAV+1#               Word position within qx          PT61  60
KBIT=(NWRD-MARK)*MXBTAV-NBIT-1#        Bit position within word         PT61  61
localdot2:#                            Control bit packing order        PT61  62
intpak:(1,QX(NWRD),KBIT,1)#            Set the bit                      PT61  63
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT61  64
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT61  65
LASTX=NEWX#                            Save current x-coordinate        PT61  66
LASTY=NOWY#                            Save current y-coordinate        PT61  67
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT61  68
#                                                                       PT61  69
ELSE IF(NDEV==3)#                      Direct calcomp writes            PT61  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT61  71
IF(IFLG<0) CALL PT01(QX(I+1),QX(I+2),NPEN)# Direct calcomp call         PT61  72
ELSE       CALL PT01(X,Y,NPEN)#        Calcomp call                     PT61  73
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT61  74
#                                                                       PT61  75
ELSE IF(NDEV==4)#                      Graphics vdu                     PT61  76
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT61  77
N=NINT(X);   M=NINT(Y)#                Convert x,y integers             PT61  78
CALL PT62(N,M,INDX,NPEN)#              Call graphics processor          PT61  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT61  80
#                                                                       PT61  81
ELSE IF(NDEV==5)#                      Local device one                 PT61  82
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT61  83
IF(IFLG<0) plot:(QX(I+1),QX(I+2),NPEN)# Direct device one call          PT61  84
ELSE       plot:(X,Y,NPEN)#            Device one call                  PT61  85
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT61  86
#                                                                       PT61  87
ELSE IF(NDEV==6)#                      Local device two                 PT61  88
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT61  89
IF(IFLG<0) plotc:(QX(I+1),QX(I+2),NPEN)# Direct device 2 call           PT61  90
ELSE       plotc:(X,Y,NPEN)#           Device 2 call                    PT61  91
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT61  92
#                                                                       PT61  93
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT61  94
#                                                                       PT61  95
RETURN#                                                                 PT61  96
END#                                                                    PT61  97
systemheader:(PT62)#                   Vdu command processor            PT62   1
####################################################################    PT62   2
#                                                                  #    PT62   3
#                              PT62                                #    PT62   4
#                                                                  #    PT62   5
####################################################################    PT62   6
SUBROUTINE  PT62(XPIX,YPIX,INDX,NPEN)# S.r. hall and a. imerito         PT62   7
include:(AACOMN)#                                                       PT62   8
include:(PTCOMN)#                                                       PT62   9
#                                                                       PT62  10
# $MASP THIS ROUTINE HAS BEEN WRITTEN SPECIFICALLY FOR THE              PT62  11
#       TEKTRONIX 4105 COLOR TERMINAL BUT WILL PROBABLY RUN             PT62  12
#       ON THE 4107, AND POSSIBLY THE 4110 SERIES??   OTHER             PT62  13
#       COLOR TERMINALS WILL REQUIRE A DIFFERENT APPROACH BUT           PT62  14
#       WILL PROBABLY USE A SIMILAR PROCEDURE. NOTE THAT THIS           PT62  15
#       ROUTINE, AND PT64, USE THE VDU IN TERMINAL COMMAND              PT62  16
#       MODE, NOT PIXEL ADDRESSABLE MODE.                               PT62  17
#                                                                       PT62  18
INTEGER INDX#                          Subject type index               PT62  19
INTEGER NPEN#                          Pen up or down                   PT62  20
INTEGER XPIX#                          X-coord. in pixels               PT62  21
INTEGER YPIX#                          Y coord. in pixels               PT62  22
INTEGER ACOL#                          Current atom colour              PT62  23
#                                                                       PT62  24
#   PROCESS DRAW COMMAND                                                PT62  25
#   --------------------                                                PT62  26
#                                                                       PT62  27
IF(NPEN==2)#                           If draw command                  PT62  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT62  29
IF(NEW==yes:)#                         If new subject                   PT62  30
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT62  31
I=MIN0(IABS(INDX),16)#                 Get pointer to colr. index       PT62  32
IF (I==0) ACOL=BCOL#                   Guard against no index           PT62  33
ELSE      ACOL=COLR(I)#                Get atom colour index            PT62  34
IF     (INDX==1000) vduset:(FILL,1,BCOL,-BCOL)# Set bond colour         PT62  35
ELSE IF(INDX>    0) vduset:(FILL,1,BCOL,-ACOL)# Set envel colour        PT62  36
ELSE IF(INDX>-1000) vduset:(FILL,0,   0,-ACOL)# Set elips colour        PT62  37
ELSE IF(INDX>-1001) vduset:(   0,0,   0,-BCOL)# Set bord  colour        PT62  38
ELSE IF(INDX>-2001) vduset:(   0,0,   0,-CCOL)# Set char  colour        PT62  39
ELSE IF(INDX>-3001) vduset:(   0,0,   0,-NCOL)# Set numbr colour        PT62  40
NEW=no:#                               Reset new subject flag           PT62  41
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT62  42
vdudrw:(XPIX,YPIX)#                    Execute draw command             PT62  43
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT62  44
#                                                                       PT62  45
#    EXECUTE MOVE COMMAND                                               PT62  46
#    --------------------                                               PT62  47
#                                                                       PT62  48
ELSE IF(NPEN==3)#                      If move command                  PT62  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT62  50
IF(INDX==0)#                           If break mark                    PT62  51
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT62  52
LASTX=XPIX#                            Remember last x                  PT62  53
LASTY=YPIX#                            Remember last y                  PT62  54
NEW=yes:#                              Set flag for new subject         PT62  55
vduend:#                               Execute endpanel                 PT62  56
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT62  57
vdumov:(XPIX,YPIX)#                    Execute move command             PT62  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT62  59
#                                                                       PT62  60
RETURN#                                                                 PT62  61
END#                                                                    PT62  62
systemheader:(PT63)#                   Reset colour parameters          PT63   1
####################################################################    PT63   2
#                                                                  #    PT63   3
#                               PT63                               #    PT63   4
#                                                                  #    PT63   5
####################################################################    PT63   6
SUBROUTINE  PT63 (FLL,FKEY,BNCOL,FLCOL)# S.r.hall & a.imerito 1984      PT63   7
include:(AACOMN)#                                                       PT63   8
include:(PTCOMN)#                                                       PT63   9
#                                                                       PT63  10
# $MASP THIS ROUTINE HAS BEEN WRITTEN SPECIFICALLY FOR THE              PT63  11
#       TEKTRONIX 4105 COLOR TERMINAL BUT WILL PROBABLY RUN             PT63  12
#       ON THE 4107, AND POSSIBLY THE 4110 SERIES??   OTHER             PT63  13
#       COLOR TERMINALS WILL REQUIRE A DIFFERENT APPROACH BUT           PT63  14
#       WILL PROBABLY USE A SIMILAR PROCEDURE. NOTE THAT THIS           PT63  15
#       ROUTINE, AND PT64, USE THE VDU IN TERMINAL COMMAND              PT63  16
#       MODE, NOT PIXEL ADDRESSABLE MODE.                               PT63  17
#                                                                       PT63  18
INTEGER XPIX#                          X-coord in pixels                PT63  19
INTEGER YPIX#                          Y-coord in pixels                PT63  20
INTEGER FLL#                           Flag for fillpattern  (yes:      PT63  21
INTEGER FKEY#                          Flag for boundary  (yes: or      PT63  22
INTEGER BNCOL#                         Colour for boundary              PT63  23
INTEGER FLCOL#                         Colour for fillpattern           PT63  24
#                                                                       PT63  25
IF(FLL==yes:)#                         Ifpanel is to be filled...       PT63  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT63  27
vduind:(BNCOL)#                        Establish boundary colour        PT63  28
IF(FKEY==yes:)#                        Is fill allowed                  PT63  29
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT63  30
XPIX=LASTX;  YPIX=LASTY#               Store coords as local            PT63  31
vdufil:(FLCOL)#                        Establish fill colour            PT63  32
vdubeg:(XPIX,YPIX)#                    Begin panel with boundary        PT63  33
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT63  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT63  35
ELSE vduind:(IABS(FLCOL))#             Establish line colour            PT63  36
#                                                                       PT63  37
RETURN#                                                                 PT63  38
END#                                                                    PT63  39
systemheader:(PT71)#                   Draw the plot                    PT71   1
####################################################################    PT71   2
#                                                                  #    PT71   3
#                               PT71                               #    PT71   4
#                                                                  #    PT71   5
####################################################################    PT71   6
SUBROUTINE PT71#                                                        PT71   7
include:(AACOMN)#                                                       PT71   8
include:(PTCOMN)#                                                       PT71   9
#                                                                       PT71  10
#  THIS ROUTINE IS DESIGNED TO CAUSE PLOTS TO BE FORMED ON A            PT71  11
#  LINE PRINTER.  IN ITS MOST PRIMITIVE USE IT SIMPLY TREATS            PT71  12
#  THE CHARACTERS ACROSS BY LINES DOWN AS A COARSE PIXEL GRID.          PT71  13
#  THIS IS THE NDEV=1 MODE OF PRINTER PLOTTING.  IF THE USER            PT71  14
#  HAS SUPPLIED THE APPROPRIATE 'ESCAPE SEQUENCE' IN THE                PT71  15
#  FORM OF PSPESC AND SETCAR LINES THE ROUTINE WILL STUFF BITS          PT71  16
#  AND LOAD THE OUTPUT BUFFERS IN A WAY THAT WILL DRIVE A DOT           PT71  17
#  MATRIX PRINTER.  THIS NDEV=2 MODE OF PRINTING CAN GIVE MUCH          PT71  18
#  HIGHER RESOLUTION PLOTS.  THE ROUTINE HAS BEEN TESTED                PT71  19
#  ON UNIVAC WITH BOTH A XEROX-DIABLO TERMINAL AND A PRINTRONIX         PT71  20
#  PRINTER AND ON A VAX WITH A PRINTRONIX PRINTER IT WAS FIRST          PT71  21
#  SET UP IN PERTH ON A PERKIN-ELMER WITH A PRINTRONIX PRINTER.         PT71  22
#     IN ALL CASES THE ROUTINE WILL BREAK PLOTS TOO LARGE TO            PT71  23
#  FIT ON ONE PRINTER SHEET INTO MULTIPLE SHEETS.  THIS                 PT71  24
#  MAKES POSSIBLE THE GENERATION OF HIGH ENOUGH RESOLUTION              PT71  25
#  DRAWINGS EVEN WITH REGULAR LINE PRINTERS.  IT MIGHT ALSO             PT71  26
#  BE POSSIBLE TO DO RATHER GEOMETRIC WALLPAPERING.                     PT71  27
#                                                                       PT71  28
#                                        George davenport               PT71  29
#                                        J.m. stewart and j.r. holden   PT71  30
#                                        November 1983                  PT71  31
INTEGER PIXEL,PIXELO#                  Pixels per printer line          PT71  32
INTEGER PUTCHR#                        Chr position for first pixel     PT71  33
REAL HOLDCH(4)#                        Character buffer of sorts        PT71  34
INTEGER MOVE#                          Line chr scan stepper            PT71  35
INTEGER LEFTOR#                        Current print direction          PT71  36
INTEGER IGET#                          1 if bit is set,0 if not         PT71  37
INTEGER J,K,L#                         Loop indices                     PT71  38
INTEGER JBIT#                          Bit position within word         PT71  39
INTEGER M#                             Place in chrot                   PT71  40
INTEGER NBIT#                          Bit position in qx               PT71  41
INTEGER MWRD#                          Word position in qx              PT71  42
INTEGER SPR#                           Start of prologue chars          PT71  43
INTEGER NPR#                           No. prolgue chars                PT71  44
INTEGER MXNEP#                         Longest epilogue                 PT71  45
INTEGER NC#                            Buffer out index                 PT71  46
INTEGER NTIMES#                        Number of buffer writes          PT71  47
INTEGER LSTPER#                        Size of last buffer out          PT71  48
INTEGER CHRPER#                        Characters per buffer out        PT71  49
INTEGER ALLCHR#                        Characters per output line       PT71  50
INTEGER STARBF#                        Starting chr in chrot            PT71  51
INTEGER STAROU#                        Starting chr in output           PT71  52
INTEGER SENDBF#                        Number of chrs to chrot          PT71  53
INTEGER ENDBF#                         Begining of epilogue in bfot     PT71  54
string:(OUTPUT,1800)#                  $masp                            PT71  55
#                                                                       PT71  56
chardata:(PT710, LRT+B*)#                                               PT71  57
intdata:(PACKUP,0);   intdata:(BITPOS,0)#                               PT71  58
intdata:(STARPO,0);   intdata:(MAXPXL,1800)# $masp                      PT71  59
datastuff:#                                                             PT71  60
#                                                                       PT71  61
#  SET UP BIT POSITION RESTORATION FOR NDEV=2 NUMBIT!=1                 PT71  62
#  ----------------------------------------------------                 PT71  63
#                                                                       PT71  64
STARPO=STARBT-DIRBIT+lowordbit:#       ## $masp##                       PT71  65
#                                                                       PT71  66
MXNEP=MAX0(NEPIL1,NEPIL2)#             Set longest epilogue             PT71  67
#                                                                       PT71  68
# AVAILABLE CHARACTERS PER LINE OF OUTPUT DEVICE PAGE                   PT71  69
# ALLOW THREE SPACES FOR POSSIBLE CONTROL CHARACTERS                    PT71  70
# ---------------------------------------------------                   PT71  71
#                                                                       PT71  72
IF(NDEV==1) PIXEL=prtcmax:-3#          Line printer, lores              PT71  73
#                                                                       PT71  74
#    SETUP THE DOT PRINTER                                              PT71  75
#    ---------------------                                              PT71  76
#                                                                       PT71  77
ELSE IF(NDEV==2)#                      Dot matrix printer, hires        PT71  78
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT71  79
LEFTOR=yes:#                           Start out left to right          PT71  80
PIXEL=NUMBIT*(BUFMAX-MAX0(NPROL1,NPROL2)-MAX0(NEPIL1,NEPIL2))#          PT71  81
IF(BUFMAX>MAXPXL)iquit:(PT7101)#       Die if output buffer exceede     PT71  82
IF(NCLEAR>0)#                          Test for clear characters        PT71  83
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT71  84
J=MAXPXL/NCLEAR#                       Calculate how many               PT71  85
FOR(K=1;K<=J;K=K+NCLEAR)#                                               PT71  86
movechar:(DEV2CH,SCLEAR,OUTPUT,K,NCLEAR,0)# Clear buffer                PT71  87
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT71  88
localdot3:#                            Set up dot op control            PT71  89
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT71  90
#                                                                       PT71  91
# LOOP OVER PAGES ACROSS                                                PT71  92
# ----------------------                                                PT71  93
#                                                                       PT71  94
FOR(J=1;J<=TOTPIX;J=J+PIXEL)#          Loop over pages across           PT71  95
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT71  96
IF(LINES/VRPIX>LINCT)nextpage:#        Force a new page                 PT71  97
LEFTOR=yes:#                           Make ndev=2 go l to r first      PT71  98
IF((NDEV==2)&(NPRLIM>0))#              Test if a start up required      PT71  99
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT71 100
movechar:(DEV2CH,SPRLIM,CHROT,1,NPRLIM,0)# Copy preliminaries           PT71 101
sendtodevice:(1,CHROT,NPRLIM)#         Trigger device                   PT71 102
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT71 103
LINCT=100000#                          Do away with form feed           PT71 104
PIXELO=MIN0(PIXEL,TOTPIX-J+1)#         One page may be shorter          PT71 105
ALLCHR=(PIXELO-1)/NUMBIT+1#            Even out characters              PT71 106
#                                                                       PT71 107
# LOOP OVER ALL LINES                                                   PT71 108
# ===================                                                   PT71 109
#                                                                       PT71 110
DO K=1,LINES#                          Loop over all lines on page      PT71 111
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT71 112
NBIT=(K-1)*TOTPIX+J#                   Starting bit                     PT71 113
#                                                                       PT71 114
# PUT OUT ONE LINE ON CHAR PRINTER                                      PT71 115
# --------------------------------                                      PT71 116
#                                                                       PT71 117
IF(NDEV==1)#                           Line printer, lores              PT71 118
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT71 119
movechar:(BLANK,1,CHROT,1,INT(prtcmax:),1)# Blank output line           PT71 120
FOR(L=NBIT;L<NBIT+PIXELO;L=L+1)#       Loop over pixels on line         PT71 121
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PT71 122
MWRD=MARK+(L-1)/MXBTAV+1#              Word containing nbit             PT71 123
JBIT=MXBTAV-MOD(L-1,MXBTAV)-1#         Bit position within word         PT71 124
intunpak:(QX(MWRD),IGET,JBIT,1)#       Check if bit is set              PT71 125
IF(IGET==1)#                           One implies print point          PT71 126
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PT71 127
M=L-NBIT+2#                            Calc output byte position        PT71 128
movechar:(PT710,7,CHROT,M,1,0)#        Move the plot character to o     PT71 129
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PT71 130
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PT71 131
writeline:(0,0,0,1,3)#                 Print low resolution plot li     PT71 132
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT71 133
#                                                                       PT71 134
#                                                                       PT71 135
#  THE SECTION WHICH FOLLOWS IS DESIGNED TO TAKE THE PIXELS STORED IN   PT71 136
#  THE QX ARRAY FROM MARK TO ISTOP AS PACKED BITS AND TRANSLATE THEM    PT71 137
#  INTO PRINTER LINES.  THESE LINES MAY BE ONE CHARACTER PER PIXEL      PT71 138
#  OR SEVERAL PIXELS PER CHARACTER DEPENDING ON THE LOCAL               PT71 139
#  PLOTTING PRINTER.  THE CONTROL PARAMETERS WERE READ IN IN PT11.      PT71 140
#                                                                       PT71 141
ELSE IF(NDEV==2)#                      Dot matrix, hires                PT71 142
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PT71 143
localdot4:#                            Local dot op if required         PT71 144
IF(LEFTOR==yes:)#                      Check print sense of directi     PT71 145
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PT71 146
NPR=NPROL1#                            Set length prologue l to r       PT71 147
SPR=SPROL1#                            Start of prolog l to r           PT71 148
MOVE=CHRCHR#                           Set positive l to r move         PT71 149
PUTCHR=1-MOVE#                         Prep char pointer                PT71 150
IF(BIDIR==yes:)LEFTOR=no:#             Flip-flop direction if neede     PT71 151
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PT71 152
ELSE#                                  Prologue for r to l              PT71 153
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PT71 154
NPR=NPROL2#                            Point to first plot char         PT71 155
SPR=SPROL2#                            Set start of r to l prologue     PT71 156
MOVE=-CHRCHR#                          Set char scan r to l             PT71 157
PUTCHR=ALLCHR+CHRCHR#                  Prep char pointer                PT71 158
IF(BIDIR==yes:)LEFTOR=yes:#            Flip-flop direction if neede     PT71 159
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PT71 160
#                                                                       PT71 161
#  SET UP FOR POSSIBILITY OF PACKED PIXEL BITS IN CHARACTERS            PT71 162
#  -----------------------------------------------------------          PT71 163
#                                                                       PT71 164
IF(NUMBIT!=1)#                         But only if need be              PT71 165
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PT71 166
PACKUP=0#                              Set bits packed so far           PT71 167
BITPOS=STARPO#                         Set before first bit             PT71 168
movector:(DEV2CH,SCLEAR,HOLDCH,1,NCLEAR,0)# Clear char buffer           PT71 169
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PT71 170
#                                                                       PT71 171
#                                                                       PT71 172
#  NOW THE LOOP OVER ALL THE PIXELS IN ONE LINE IS CARRIED OUT.         PT71 173
#  ------------------------------------------------------------         PT71 174
#                                                                       PT71 175
FOR(L=NBIT;L<NBIT+PIXELO;L=L+1)#                                        PT71 176
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PT71 177
MWRD=MARK+(L-1)/MXBTAV+1#              Point to word with pixel         PT71 178
JBIT=MXBTAV-MOD(L-1,MXBTAV)-1#         Point to pixel bit in word       PT71 179
intunpak:(QX(MWRD),IGET,JBIT,1)#       Pull out pixel bit               PT71 180
#                                                                       PT71 181
#  IF THERE IS JUST ONE CHRCHR CHARACTERS PER PIXEL THE JOB IS EASY     PT71 182
#  AND IS AS FOLLOWS......                                              PT71 183
#  -----------------------------------------------------------------    PT71 184
#                                                                       PT71 185
IF(NUMBIT==1)#                         Test for one char/pixel          PT71 186
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PT71 187
#                                                                       PT71 188
#  THE FOLOWING STATEMENT FINDS THE OUTPUT BUFFER PIXEL POSITION        PT71 189
#  AS 0,1,2,3,........,PIXEL-3,PIXEL-2,PIXEL-1  OR                      PT71 190
#  AS PIXEL-1,PIXEL-2,PIXEL-3,........,3,2,1,0  DEPENDING UPON          PT71 191
#  WHICH WAY THE PRINTER HEAD IS SET TO MOVE.                           PT71 192
#  -----------------------------------------------------------------    PT71 193
#                                                                       PT71 194
PUTCHR=PUTCHR+MOVE#                    Move across output buffer        PT71 195
IF(IGET==1)movechar:(DEV2CH,SPLOTC,OUTPUT,PUTCHR,CHRCHR,0)# Move plot   PT71 196
ELSE movechar:(DEV2CH,SCLEAR,OUTPUT,PUTCHR,CHRCHR,0)# Clear cha         PT71 197
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PT71 198
#                                                                       PT71 199
#  IN THE CASE WHERE EACH CHARACTER IN THE OUTPUT CAN CONTAIN MORE      PT71 200
#  THAN ONE PIXEL A CALCULATION MUST BE MADE AND THE APPROPRIATE        PT71 201
#  BITS SET IN THE OUTPUT CHARACTER.                                    PT71 202
#      THIS SECTION OF CODE IS DESIGNED TO COPE WITH LINE PRINTERS      PT71 203
#  WHICH HAVE A DOT MATRIX CONFIGURATION.  THESE PRINTERS RECIEVE       PT71 204
#  A STREAM OF CHARACTERS IN WHICH CERTAIN ESCAPE CHARACTERS AND        PT71 205
#  ESCAPE BITS ARE SET.  THIS ALLOWS PRINTING AT THE TINY DOT LEVEL.    PT71 206
#  EACH CHARACTER IN THE OUTPUT STREAM BEYOND THE ESCAPE CHARACTER      PT71 207
#  HAS BITS SET WHICH CAUSE THE HORIZONTAL PIXELS TO BE STRUCK.         PT71 208
#  THE CODE IS COMPLICATED BY THE FACT THAT THE NUMBER OF BITS          PT71 209
#  PER CHARACTER AND THE DIRECTION OF THE BITS IN THE CHARACTER ARE     PT71 210
#  MACHINE AND IMPLIMENTATION DEPENDENT.   EVEN WORSE,  THE MACHINE     PT71 211
#  UPON WHICH PT IS IMPLIMENTED MAY HAVE A DIFFERENT INTERNAL CODE      PT71 212
#  THAN THAT USED BY THE TARGET DOT MATRIX PRINTER.                     PT71 213
#  -----------------------------------------------------------------    PT71 214
#                                                                       PT71 215
ELSE#                                  More pixels per character        PT71 216
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PT71 217
IF(PACKUP>=NUMBIT)#                    Check if char filled             PT71 218
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PT71 219
PUTCHR=PUTCHR+MOVE#                    Move to next character           PT71 220
movertoc:(HOLDCH,1,OUTPUT,PUTCHR,CHRCHR,0)# From temp buf to outpt      PT71 221
movector:(DEV2CH,SCLEAR,HOLDCH,1,NCLEAR,0)# Clear char buffer           PT71 222
BITPOS=STARPO#                         Reset start of bits              PT71 223
PACKUP=0#                              Reset bit packing                PT71 224
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PT71 225
#                                                                       PT71 226
#  PLACE THE NEXT PIXEL BIT IN THE CHARACTER BUFFER WORD                PT71 227
#  -----------------------------------------------------                PT71 228
#                                                                       PT71 229
PACKUP=PACKUP+1#                       Count up bits packed             PT71 230
BITPOS=BITPOS+DIRBIT#                  Scan over bits                   PT71 231
intpak:(IGET,HOLDCH(1),BITPOS,1)# ## $masp## pack in pixel bit          PT71 232
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PT71 233
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PT71 234
#                                                                       PT71 235
#  NOW THE HIGH RESOLUTION LINE IS FORMED.                              PT71 236
#  SO IT CAN BE SENT TO THE HIGH RESOLUTION PRINT DEVICE.               PT71 237
#  ---------------------------------------------------------            PT71 238
#                                                                       PT71 239
IF(NUMBIT!=1)#                                                          PT71 240
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PT71 241
PUTCHR=PUTCHR+MOVE#                    Advance to last char             PT71 242
movertoc:(HOLDCH,1,OUTPUT,PUTCHR,CHRCHR,0)# Final char                  PT71 243
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PT71 244
#                                                                       PT71 245
#  AT THIS POINT THE OUTPUT CHARACTER ARRAY IS LOADED UP WITH THE       PT71 246
#  CHARACTERS TO BE DELIVERED TO THE LINE OUTPUT DEVICE.                PT71 247
#  UNFORTUNATELY OUTPUT MAY BE MUCH LONGER THAN CHROT!!                 PT71 248
#  THIS MEANS THAT THE CHARACTERS WILL HAVE TO BE PUT OUT IN            PT71 249
#  BUNDLES NO LONGER THAN CHROT.                                        PT71 250
#  EACH BUNDLE WILL HAVE TO BE PRECEDED BY THE CORRECT PROLOGUE         PT71 251
#  SO THAT THE DEVICE WILL STAY 'SHIFTED' INTO ITS PLOT MODE            PT71 252
#  AND THE FINAL BUNDEL WILL HAVE TO HAVE THE EPILOGUE TACKED ON        PT71 253
#  SO THAT THE NEXT LINE FEED WILL BE SENT.  THAT THIS MAY              PT71 254
#  NOT BE GENERAL FOR EVERY COMPUTER COMMUNICATING WITH EVERY           PT71 255
#  PRINTING DEVICE EVER BUILT HAS NOT ESCAPED OUR THINKING.             PT71 256
#  THAT IT WORKS WELL FOR A FEW IS A KNOWN FACT.                        PT71 257
#  --------------------------------------------------------------       PT71 258
#                                                                       PT71 259
movechar:(DEV2CH,SPR,CHROT,1,NPR,0)#   Put approp. prol.                PT71 260
CHRPER=prtcmax:-NPR-MXNEP#             Find room in chrot               PT71 261
NTIMES=((ALLCHR-1)/CHRPER)+1#          Calc no. of writes               PT71 262
LSTPER=MOD(ALLCHR,CHRPER)#             Chars per last write             PT71 263
STAROU=1#                              Start chr in output              PT71 264
STARBF=NPR+1#                          Start chr in chrot               PT71 265
#                                                                       PT71 266
#  NOW PUMP OUT THE CHARACTERS                                          PT71 267
#  ---------------------------                                          PT71 268
#                                                                       PT71 269
FOR(NC=1;NC<=NTIMES;NC=NC+1)#                                           PT71 270
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PT71 271
IF((NC==NTIMES)&(LSTPER>0))CHRPER=LSTPER# Last buffer is specia         PT71 272
movechar:(OUTPUT,STAROU,CHROT,STARBF,CHRPER,0)# Move bundle             PT71 273
STAROU=STAROU+CHRPER#                  Move pointer to outp             PT71 274
SENDBF=CHRPER+NPR#                     Calc no. to write                PT71 275
ENDBF=STARBF+CHRPER#                   Find start of epilogue           PT71 276
IF(NC==NTIMES)#                        Last buffer is speci             PT71 277
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PT71 278
movechar:(DEV2CH,SEPIL1,CHROT,ENDBF,NEPIL1,0)# Move epilogue            PT71 279
SENDBF=SENDBF+NEPIL1#                  Count addition                   PT71 280
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PT71 281
ELSE#                                  Intermediate buffer              PT71 282
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PT71 283
movechar:(DEV2CH,SEPIL2,CHROT,ENDBF,NEPIL2,0)# Move epilogue            PT71 284
SENDBF=SENDBF+NEPIL2#                                                   PT71 285
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PT71 286
sendtodevice:(1,CHROT,SENDBF)#         $masp    # write buffer          PT71 287
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PT71 288
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PT71 289
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT71 290
#                                                                       PT71 291
#  IT MAY BE NECESSARY TO SEND ONE FINAL SERIES OF CHARACTERS TO        PT71 292
#  THE OUTPUT DEVICE IN ORDER TO SHIFT IT BACK TO BEING A LINE          PT71 293
#  PRINTER.  IF SO DO SO.                                               PT71 294
#  --------------------------------------------------------------       PT71 295
#                                                                       PT71 296
IF((NDEV==2)&(NEPIL>0))#                                                PT71 297
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PT71 298
movechar:(DEV2CH,SEPIL,CHROT,1,NEPIL,0)# Move the final dope            PT71 299
sendtodevice:(1,CHROT,NEPIL)#          $masp   # and blast device       PT71 300
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PT71 301
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT71 302
###LINCT=0#                                                             PT71 303
RETURN#                                                                 PT71 304
END#                                                                    PT71 305
systemheader:(PT75)#                   Output user info and init. d     PT75   1
####################################################################    PT75   2
#                                                                  #    PT75   3
#                                PT75                              #    PT75   4
#                                                                  #    PT75   5
####################################################################    PT75   6
SUBROUTINE PT75#                                                        PT75   7
include:(AACOMN)#                                                       PT75   8
include:(PTCOMN)#                                                       PT75   9
#                                                                       PT75  10
#       THIS ROUTINE OUTPUTS CONTROL PARAMETERS (AND OTHER              PT75  11
#       INFORMATION RELEVANT TO THE USER) BY ECHOING IN-                PT75  12
#       FORMATION READ IN BY ROUTINE PT11.                              PT75  13
#       PT75 ALSO DOES MEMORY INITIALISATIONS.                          PT75  14
#                                                                       PT75  15
INTEGER II,I#                          Local index                      PT75  16
chardata:(PT111,CHAR DOT  PLOT VDU  LOC1 LOC2 )#                        PT75  17
chardata:(PT112, PLOT MODE)#                                            PT75  18
chardata:(PT11D,LTRRTL)#                                                PT75  19
chardata:(PT11G, HORIZONTAL PIXELS PER INCH      )#                     PT75  20
chardata:(PT11H, VERTICAL PIXELS PER INCH        )#                     PT75  21
chardata:(PT11I, PIXELS PER CHARACTER            )#                     PT75  22
chardata:(PT11J, STARTING BIT IN CHARACTER)#                            PT75  23
chardata:(PT11K,[ AS 7,6,5,4,3,2,1, OR 0          ])#                   PT75  24
chardata:(PT11L, DIRECTION OF BIT PACKING        )#                     PT75  25
chardata:(PT11M, NUMBER OF COMPUTER)#                                   PT75  26
chardata:(PT11N, CHARACTERS REQUIRED FOR)#                              PT75  27
chardata:(PT11P, EACH PRINTER CHARACTER          )#                     PT75  28
chardata:(PT11Q, PRINTER MOTION                 DIRECTIONAL)#           PT75  29
chardata:(PT11R, MAXIMUM PRINTED CHARACTERS)#                           PT75  30
chardata:(PT11S, ON A SINGLE PRINTED LINE        )#                     PT75  31
chardata:(PT12C,MONO  BI)#                                              PT75  32
intdata:(LIMIT,0)#                                                      PT75  33
datastuff:#                                                             PT75  34
#                                                                       PT75  35
IF(NDEV==2)#                           Check for dot printer            PT75  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT75  37
BUFOT(1)=FLOAT(HZPIX);ncodefld:(BUFOT,1,PT11G,320413.,1)#               PT75  38
writeline:(0,PT11G,NPT11G,3,3)#                                         PT75  39
BUFOT(1)=FLOAT(VRPIX);ncodefld:(BUFOT,1,PT11H,320413.,1)#               PT75  40
writeline:(0,PT11H,NPT11H,3,3)#                                         PT75  41
BUFOT(1)=FLOAT(NUMBIT);ncodefld:(BUFOT,1,PT11I,320413.,1)#              PT75  42
writeline:(0,PT11I,NPT11I,3,3)#                                         PT75  43
writeline:(0,PT11J,NPT11J,3,3)#                                         PT75  44
BUFOT(1)=FLOAT(STARBT);ncodefld:(BUFOT,1,PT11K,320413.,1)#              PT75  45
writeline:(0,PT11K,NPT11K,3,3)#                                         PT75  46
IF(DIRBIT<0)movechar:(PT11D,1,PT11L,30,3,0)#                            PT75  47
ELSE movechar:(PT11D,4,PT11L,30,3,0)#                                   PT75  48
writeline:(0,PT11L,NPT11L,3,3)#                                         PT75  49
writeline:(0,PT11M,NPT11M,3,3)#                                         PT75  50
writeline:(0,PT11N,NPT11N,3,3)#                                         PT75  51
BUFOT(1)=FLOAT(CHRCHR);ncodefld:(BUFOT,1,PT11P,320413.,1)#              PT75  52
writeline:(0,PT11P,NPT11P,3,3)#                                         PT75  53
IF(BIDIR==no:) movechar:(PT12C,1,PT11Q,29,4,0)#                         PT75  54
ELSE movechar:(PT12C,5,PT11Q,29,4,0)#                                   PT75  55
writeline:(0,PT11Q,NPT11Q,3,3)#                                         PT75  56
writeline:(0,PT11R,NPT11R,3,3)#                                         PT75  57
BUFOT(1)=FLOAT(BUFMAX);ncodefld:(BUFOT,1,PT11S,320413.,1)#              PT75  58
writeline:(0,PT11S,NPT11S,3,3)#                                         PT75  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT75  60
movechar:(PT112,1,CHROT,1,NPT112,0)#   Heading                          PT75  61
II=5*(NDEV-1)#                         Point to plot mode               PT75  62
movechar:(PT111,II+1,CHROT,27,4,0)#    Plot mode                        PT75  63
writeline:(1,0,0,1,3)#                                                  PT75  64
#                                                                       PT75  65
IOMARK(5)=QXSTR#                       Start of file e buffer           PT75  66
MARK=IOMARK(5)+bdfbuf:#                Start of bit map                 PT75  67
#                                                                       PT75  68
CALL PT76#                             ******* additional messages      PT75  69
#                                                                       PT75  70
#                                                                       PT75  71
# MEMORY INITIALIZATIONS                                                PT75  72
# ----------------------                                                PT75  73
#                                                                       PT75  74
MXBTAV=32#                             Available bits per word.         PT75  75
#                                                                       PT75  76
IF(NDEV==1)#                           Line printer in 'char' mode      PT75  77
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT75  78
LINES=NINT(AHIGH)*LINEPI#              Vertical pixels                  PT75  79
TOTPIX=NINT(AWIDE)*CHARPI#             Horiz pixels                     PT75  80
LIMIT=(TOTPIX/MXBTAV+1)*LINES#         Map array in words               PT75  81
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT75  82
#                                                                       PT75  83
ELSE IF(NDEV==2)#                      High resolution printer 'dot'    PT75  84
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT75  85
LINES=NINT(AHIGH)*VRPIX#               Vertical pixels                  PT75  86
TOTPIX=NINT(AWIDE)*HZPIX#              Horiz pixels                     PT75  87
MXBTAV=NUMBIT*mxchwd:#                 Calc bits per word               PT75  88
LIMIT=(TOTPIX/MXBTAV+1)*LINES#         Map array in words               PT75  89
localdot0:#                            Set local dot requirements       PT75  90
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT75  91
#                                                                       PT75  92
IF(NDEV==3)#                           For calcomp plotter              PT75  93
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT75  94
startplot:#                ## $masp     # start plot file               PT75  95
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT75  96
#                                                                       PT75  97
incrqx:(ISTOP,MARK+LIMIT,PT7503)#      Start pen movement buffer        PT75  98
I=MIN(QXMAX,ISTOP+10000)#              Ask for big buffer               PT75  99
qxmemory:(I)#                          See if its there...              PT75 100
MARK1=I-60; NSTEP1=6;  IP1=ISTOP#      Set upper limit                  PT75 101
#                                                                       PT75 102
FOR(I=MARK+1;I<=ISTOP;I=I+1)#          Loop over bit map                PT75 103
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT75 104
intpak:(0,QX(I),0,mxbtwd:)#            Zero bits                        PT75 105
localdot1:#                            Initialize dot words             PT75 106
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT75 107
RETURN#                                                                 PT75 108
END#                                                                    PT75 109
systemheader:(PT76)#                   Output user info and init. d     PT76   1
####################################################################    PT76   2
#                                                                  #    PT76   3
#                                PT76                              #    PT76   4
#                                                                  #    PT76   5
####################################################################    PT76   6
SUBROUTINE PT76#                                                        PT76   7
include:(AACOMN)#                                                       PT76   8
include:(PTCOMN)#                                                       PT76   9
#                                                                       PT76  10
#       THIS ROUTINE OUTPUTS CONTROL PARAMETERS (AND OTHER              PT76  11
#       INFORMATION RELEVANT TO THE USER) BY ECHOING IN-                PT76  12
#       FORMATION READ IN BY ROUTINE PT11.                              PT76  13
#       PT75 ALSO DOES MEMORY INITIALISATIONS.                          PT76  14
#       PT76 IS CALLED BY PT75.                                         PT76  15
#                                                                       PT76  16
INTEGER I#                             Local index                      PT76  17
chardata:(PT113, PLOT NUMBER)#                                          PT76  18
chardata:(PT114, HEIGHT OF PLOT IN IN.            )#                    PT76  19
chardata:(PT115, WIDTH OF PLOT IN IN.            )#                     PT76  20
chardata:(PT116,[ PLOT CENTER   (      ,      )])#                      PT76  21
chardata:(PT117,[  (COORDS FROM PREVIOUS PROGRAM)])#                    PT76  22
chardata:(PT118, MAGNIFICATION SCALE              )#                    PT76  23
chardata:(PT119, AUTOMATIC CENTERING)#                                  PT76  24
chardata:(PT11A, AUTOMATIC SCALING)#                                    PT76  25
chardata:(PT12E,[ FILL ATOMS + BONDS       YES ])#                      PT76  26
chardata:(PT12F,YES NO )#                                               PT76  27
chardata:(PT12G, BOND COLOR INDEX              )#                       PT76  28
chardata:(PT12H, ATOM COLOR INDICES)#                                   PT76  29
chardata:(PT12I, NUMB COLOR INDEX              )#                       PT76  30
chardata:(PT12J, CONTOUR COLOR INDICES     $#                           PT76  31
NEG.=      ZERO=      POS.=      BORD=      SITE=      )#               PT76  32
chardata:(PT12K, CHAR COLOR INDEX              )#                       PT76  33
chardata:(PT12L, RESOLUTION IN IN.             )#                       PT76  34
chardata:(PT12N, THIS IS A ONE COLOUR OUTPUT)#                          PT76  35
realdata:(FM0,[300313.,330313.,360313.,390313.,#                        PT76  36
420313.,450313.,480313.,510313.,#                                       PT76  37
540313.,570313.,600313.,630313.,#                                       PT76  38
660313.,690313.,720313.,750313.])#                                      PT76  39
realdata:(FM1,[340313.,450313.,560313.,670313.,780313.,890313.])#       PT76  40
datastuff:#                                                             PT76  41
#                                                                       PT76  42
IF(NDEV==4\NDEV==5)#                   If vdu or local plotter          PT76  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT76  44
DO I=1,16; BUFOT(I)=FLOAT(COLR(I))#    Float atom colour indices        PT76  45
IF(CONTUR==no:)#                       If true, set output for orte     PT76  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT76  47
IF(FILL==no:) movechar:(PT12F,4,PT12E,27,3,0)# Indicate no fill         PT76  48
writeline:(0,PT12E,NPT12E,3,3)#        Output fill/no fill message      PT76  49
movechar:(PT12H,1,CHROT,1,NPT12H,0)#   Load atom colour message         PT76  50
ncodefld:(BUFOT,1,CHROT,FM0,16)#       Load atom colours                PT76  51
writeline:(0,0,0,1,3)#                 Print atom colours               PT76  52
ncodefld:(FLOAT(BCOL),1,PT12G,300313.,1)# Load bond colour              PT76  53
writeline:(0,PT12G,NPT12G,3,3)#        Output bond colour message       PT76  54
ncodefld:(FLOAT(CCOL),1,PT12K,300313.,1)# Load char colour              PT76  55
writeline:(0,PT12K,NPT12K,3,3)#        Output char colour message       PT76  56
ncodefld:(FLOAT(NCOL),1,PT12I,300313.,1)# Load numb colour              PT76  57
writeline:(0,PT12I,NPT12I,3,3)#        Output numb colour message       PT76  58
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT76  59
ELSE#                                  Output for contour plot          PT76  60
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT76  61
ncodefld:(BUFOT,1,PT12J,FM1,5)#        Format contour indices           PT76  62
writeline:(0,PT12J,NPT12J,3,3)#        Output contour message           PT76  63
ncodefld:(FLOAT(CCOL),1,PT12K,300313.,1)# Load char colour              PT76  64
writeline:(0,PT12K,NPT12K,3,3)#        Output char colour message       PT76  65
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT76  66
IF(NOCOL==yes:)#                                                        PT76  67
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT76  68
movechar:(PT12N,1,CHROT,1,NPT12N,0)#   Load one colour message          PT76  69
writeline:(0,0,0,1,3)#                 Output the message               PT76  70
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT76  71
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT76  72
#                                                                       PT76  73
movechar:(PT113,1,CHROT,1,NPT113,0)#   Heading                          PT76  74
ncodefld:(FLOAT(NPLOT),1,CHROT,300313.,1)# Encode plot number           PT76  75
writeline:(0,0,0,1,3)#                                                  PT76  76
#                                                                       PT76  77
ncodefld:(AHIGH,1,PT114,300422.,1)#    Encode plot height               PT76  78
writeline:(0,PT114,NPT114,3,3)#        Print plot height                PT76  79
ncodefld:(AWIDE,1,PT115,300422.,1)#    Encode plot width                PT76  80
writeline:(0,PT115,NPT115,3,3)#        Print width                      PT76  81
ncodefld:(OMAG,1,PT118,300622.,1)#     Encode magnification             PT76  82
writeline:(0,PT118,NPT118,3,3)#        Print magnification              PT76  83
ncodefld:(RESL,1,PT12L,300642.,1)#     Load in resolution               PT76  84
writeline:(0,PT12L,NPT12L,3,3)#        Print resolution                 PT76  85
IF(ICENTR==yes:)#                      Auto centering                   PT76  86
writeline:(0,PT119,NPT119,3,3)#        Heading                          PT76  87
ELSE#                                  Manual centering                 PT76  88
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PT76  89
movechar:(PT116,1,CHROT,1,NPT116,0)#   Heading                          PT76  90
ncodefld:(XCEN,1,CHROT,220622.,1)#     Encode x-coord                   PT76  91
ncodefld:(YCEN,1,CHROT,290622.,1)#     Encode y-coord                   PT76  92
writeline:(0,0,0,1,3)#                                                  PT76  93
writeline:(0,PT117,NPT117,3,3)#        2nd line of heading              PT76  94
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PT76  95
IF(ISCALE==yes:)#                      Automatic scaling                PT76  96
writeline:(0,PT11A,NPT11A,3,3)#        Heading                          PT76  97
ELSE#                                  Manual scaling                   PT76  98
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PT76  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PT76 100
RETURN#                                                                 PT76 101
END#                                                                    PT76 102
systemheader:(PT81)#                   End plot on digital plotter      PT81   1
####################################################################    PT81   2
#                                                                  #    PT81   3
#                                 PT81                             #    PT81   4
#                                                                  #    PT81   5
####################################################################    PT81   6
SUBROUTINE PT81#                                                        PT81   7
include:(PTCOMN)#                                                       PT81   8
include:(AACOMN)#                                                       PT81   9
#                                                                       PT81  10
IF(NDEV==3)#                           Direct tape write for calcom     PT81  11
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PT81  12
CALL PT01(0.,0.,3)#                    Return pen to origin             PT81  13
CALL PT01(0.,0.,999)#                  End plot command                 PT81  14
stopplot:#                             $masp                            PT81  15
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PT81  16
#                                                                       PT81  17
ELSE IF(NDEV==4)#                      Graphics vdu                     PT81  18
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PT81  19
vduout:(0,0)#                          Finish off graphics buffer       PT81  20
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PT81  21
#                                                                       PT81  22
ELSE IF(NDEV==5)#                      Local device one                 PT81  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PT81  24
plot:(0.,0.,+999)#                     $masp                            PT81  25
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PT81  26
#                                                                       PT81  27
ELSE IF(NDEV==6)#                      Local device two                 PT81  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PT81  29
enplot:#                               $masp                            PT81  30
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PT81  31
#                                                                       PT81  32
RETURN#                                                                 PT81  33
END#                                                                    PT81  34
pmacro:([no:])#                        Purge xtal system macros         PT81  35
